/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Crypto Library .h file
 *
 * Filename: trcrylib.h
 * Author: Jin Zhang
 * Date Created: 08/01/2001
 * $Source: include/trcrylib.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:33:18CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifndef _TRCRYLIB_H_
#define _TRCRYLIB_H_


#ifdef __cplusplus
extern "C" {
#endif


/* 32Bit integer is our maximum*/
#define TM_DH_THIRTY_TWO_BIT 
/* the maximum signed integer our machine can handle */
#define TM_INTEGER_MAX                   TM_UL(0x7fffffff)
/*
 * Some 32-bit compilers may not be able to handle 64-bit longs.
 * For such compilers, define TM_NO_LONG64 ins TRSYSTEM.H,
 * thereby allowing the 16-bit macros to be used since they only
 * depend on 32-bit longs.
 */
#ifdef TM_NO_LONG64
#undef TM_DH_THIRTY_TWO_BIT
#define TM_DH_SIXTEEN_BIT
#endif /* TM_NO_LONG64 */

#define TM_AHALGORITHM_NULL_PTR          (ttAhAlgorithmPtr)0
#define TM_ESPALGORITHM_NULL_PTR         (ttEspAlgorithmPtr)0
#define TM_DH_GENERATOR_2                2
#define TM_DH_GENERATOR_5                5

#define TM_ISAKMP_RESPONDER              (tt8Bit)0
#define TM_ISAKMP_INITIATOR              (tt8Bit)1

/* Macros used to determine how to interpret encryption and hashing macros */
#define TM_CRYPTO_MODULE_IKE             (tt8Bit)1

/* 
 *  Diffie-Hellman macroes 
 */

#define TM_OAKLEY_PRIME_MODP512 \
    "FFFFFFFF FFFFFFFF C90FDAA2 2168C234" \
    "C4C6628B 80DC1CD1 29024E08 8A67CC74" \
    "CD3A431B 302B0A6D F25F1437 625E7EC6" \
    "F44C42E9 A63A3620 FFFFFFFF FFFFFFFF"

#define TM_OAKLEY_PRIME_MODP768 \
    "FFFFFFFF FFFFFFFF C90FDAA2 2168C234 C4C6628B 80DC1CD1" \
    "29024E08 8A67CC74 020BBEA6 3B139B22 514A0879 8E3404DD" \
    "EF9519B3 CD3A431B 302B0A6D F25F1437 4FE1356D 6D51C245" \
    "E485B576 625E7EC6 F44C42E9 A63A3620 FFFFFFFF FFFFFFFF"

#define TM_OAKLEY_PRIME_MODP1024 \
    "FFFFFFFF FFFFFFFF C90FDAA2 2168C234 C4C6628B 80DC1CD1" \
    "29024E08 8A67CC74 020BBEA6 3B139B22 514A0879 8E3404DD" \
    "EF9519B3 CD3A431B 302B0A6D F25F1437 4FE1356D 6D51C245" \
    "E485B576 625E7EC6 F44C42E9 A637ED6B 0BFF5CB6 F406B7ED" \
    "EE386BFB 5A899FA5 AE9F2411 7C4B1FE6 49286651 ECE65381" \
    "FFFFFFFF FFFFFFFF"

#define TM_OAKLEY_PRIME_MODP1536 \
    "FFFFFFFF FFFFFFFF C90FDAA2 2168C234 C4C6628B 80DC1CD1" \
    "29024E08 8A67CC74 020BBEA6 3B139B22 514A0879 8E3404DD" \
    "EF9519B3 CD3A431B 302B0A6D F25F1437 4FE1356D 6D51C245" \
    "E485B576 625E7EC6 F44C42E9 A637ED6B 0BFF5CB6 F406B7ED" \
    "EE386BFB 5A899FA5 AE9F2411 7C4B1FE6 49286651 ECE45B3D" \
    "C2007CB8 A163BF05 98DA4836 1C55D39A 69163FA8 FD24CF5F" \
    "83655D23 DCA3AD96 1C62F356 208552BB 9ED52907 7096966D" \
    "670C354E 4ABC9804 F1746C08 CA237327 FFFFFFFF FFFFFFFF"

#define TM_OAKLEY_PRIME_MODP2048 \
    "FFFFFFFF FFFFFFFF C90FDAA2 2168C234 C4C6628B 80DC1CD1" \
    "29024E08 8A67CC74 020BBEA6 3B139B22 514A0879 8E3404DD" \
    "EF9519B3 CD3A431B 302B0A6D F25F1437 4FE1356D 6D51C245" \
    "E485B576 625E7EC6 F44C42E9 A637ED6B 0BFF5CB6 F406B7ED" \
    "EE386BFB 5A899FA5 AE9F2411 7C4B1FE6 49286651 ECE45B3D" \
    "C2007CB8 A163BF05 98DA4836 1C55D39A 69163FA8 FD24CF5F" \
    "83655D23 DCA3AD96 1C62F356 208552BB 9ED52907 7096966D" \
    "670C354E 4ABC9804 F1746C08 CA18217C 32905E46 2E36CE3B" \
    "E39E772C 180E8603 9B2783A2 EC07A28F B5C55DF0 6F4C52C9" \
    "DE2BCBF6 95581718 3995497C EA956AE5 15D22618 98FA0510" \
    "15728E5A 8AACAA68 FFFFFFFF FFFFFFFF"

/*character vector */
typedef struct tsCharVector 
{
/* length of the value */
    unsigned int    cvLength;    
/* place holder to the value in buffer */
    char TM_FAR *   cvValuePtr;  
}ttCharVector;
typedef struct tsCharVector TM_FAR * ttCharVectorPtr;
typedef ttCharVectorPtr TM_FAR * ttCharVectorPtrPtr;


/* ttMd5Ctx, because the ppp-chap option will use md5context sturcture, 
 * so this part is defined in
 * trtype.h 
*/








#ifdef TM_USE_RC5
/* esp-rc5 */
#define TM_RC5_ROUNDS_8     8
#define TM_RC5_ROUNDS_12    12
#define TM_RC5_ROUNDS_16    16

#define TM_RC5_DEFAULT_ROUND 16

/*    The structure for key information */
typedef struct ttRc5KeyInstance {
    int      rounds; 
    tt32Bit  TM_FAR rc5KeySched[2*(TM_RC5_ROUNDS_16+1)];
} ttRc5KeyInstance;
typedef ttRc5KeyInstance TM_FAR * ttRc5KeyInstancePtr;
#endif /* TM_USE_RC5*/

typedef  int  (*ttEspPktEncryptDecryptPtr)(tt8BitPtr           ivPtr,
                                           ttPacketPtr         packetPtr, 
                                           ttPktLen            offData, 
                                           ttPacketPtr         newPacketPtr,
                                           ttPktLen            payloadLen,
                                           ttGenericKeyPtr     keyPtr,
                                           ttEspAlgorithmPtr   algoPtr,
                                           ttVoidPtr           additionalData);

TM_PROTO_EXTERN int tfCryptoEngineInit(void);
TM_PROTO_EXTERN void tfCryptoEngineUninitialize(void);



TM_PROTO_EXTERN int tfAhContext(ttAhAlgorithmPtr algoPtr, 
                                ttGenericKeyPtr gkeyPtr);
TM_PROTO_EXTERN void tfAhClearContext(ttVoidPtr  contextPtr);
TM_PROTO_EXTERN int tfAhMac(ttPacketPtr      packetPtr,
                            int              startOff, 
/*TLS when using HMAC on packets, need hash in sequence number
 * which is not in network data. 
 */
                             tt8BitPtr        extraDataPtr,
                             ttPktLen         extraLen,
                             ttPktLen         totalAuth,
                             tt8BitPtr        outPositionPtr,
                             ttGenericKeyPtr  gkeyPtr, 
                             ttAhAlgorithmPtr algoPtr,
                             tt8Bit           needTruncate,
                             tt8Bit           doHmac);

TM_PROTO_EXTERN int tfEspDataCbcEncrypt(tt8BitPtr           ivPtr,
                                        tt8BitPtr           dataSrcPtr, 
                                        tt8BitPtr           dataDstPtr,
                                        ttPktLen            payloadLen,
                                        ttGenericKeyPtr     keyPtr,
                                        ttEspAlgorithmPtr   algoPtr);

TM_PROTO_EXTERN int tfEspDataCbcDecrypt (tt8BitPtr       ivPtr, 
                                         tt8BitPtr       dataSrcPtr, 
                                         tt8BitPtr       dataDstPtr,
                                         ttPktLen        payloadLen,
                                         ttGenericKeyPtr keyPtr, 
                                         ttEspAlgorithmPtr algoPtr);

TM_PROTO_EXTERN int tfEspDataCtrEncrypt(tt32BitPtr        nonce,
                                        tt8BitPtr         ivPtr,
                                        tt32Bit           ivLen,
                                        tt8BitPtr         dataSrcPtr,
                                        tt8BitPtr         dataDstPtr,
                                        ttPktLen          payloadLen,
                                        ttGenericKeyPtr   keyPtr,
                                        ttEspAlgorithmPtr algoPtr);

TM_PROTO_EXTERN int tfEspDataCtrDecrypt(tt32BitPtr        nonce,
                                        tt8BitPtr         ivPtr,
                                        tt32Bit           ivLen,
                                        tt8BitPtr         dataSrcPtr,
                                        tt8BitPtr         dataDstPtr,
                                        ttPktLen          payloadLen,
                                        ttGenericKeyPtr   keyPtr,
                                        ttEspAlgorithmPtr algoPtr);

TM_PROTO_EXTERN int tfEspPacketCbcEncrypt(tt8BitPtr           ivPtr,
                                          ttPacketPtr         packetPtr, 
                                          ttPktLen            offData,
/* if newPacketPtr is NULL, we write back to the original packet */
                                          ttPacketPtr         newPacketPtr,
                                          ttPktLen            payloadLen,
                                          ttGenericKeyPtr     keyPtr,
                                          ttEspAlgorithmPtr   algoPtr,
/* update IV */
                                          ttVoidPtr           additionalData);
TM_PROTO_EXTERN int tfEspPacketCtrModeCrypto(tt8BitPtr           ivPtr,
                                             ttPacketPtr         packetPtr, 
                                             ttPktLen            offData, 
                                             ttPacketPtr         newPacketPtr,
                                             ttPktLen            payloadLen,
                                             ttGenericKeyPtr     keyPtr,
                                             ttEspAlgorithmPtr   algoPtr,
                                             ttVoidPtr           additionalData);



TM_PROTO_EXTERN int tfEspPacketCbcDecrypt(tt8BitPtr           ivPtr,
                                          ttPacketPtr         packetPtr, 
                                          ttPktLen            offData, 
                                          ttPacketPtr         newPacketPtr,
                                          ttPktLen            payloadLen,
                                          ttGenericKeyPtr     keyPtr,
                                          ttEspAlgorithmPtr   algoPtr,
                                          ttVoidPtr           additionalData);


TM_PROTO_EXTERN int tfEspSchedule(ttEspAlgorithmPtr, 
                                  ttGenericKeyPtr);
TM_PROTO_EXTERN void tfEspClearSchedule(ttUserVoidPtr     schedulePtr);













/*
 * Put data in hex format, flush to log buffer when full,
 * and at the end if offSetPtr is null.
 */
TM_PROTO_EXTERN void tfCryptoDebugPrintf(ttUserCharPtrPtr  printBufPtrPtr,
                                         ttUser8BitPtr  dataPtr,
                                         ttUser32Bit    dataLength,
                                         ttUserIntPtr   offsetPtr);

/*
 * Flush to log buffer. Only needed to be called if last call to
 * tfCryptoDebugPrintf had a non null offsetPtr, and we are done.
 */
TM_PROTO_EXTERN void tfCryptoDebugFlush(ttUserCharPtrPtr printBufPtrPtr);

#ifdef __cplusplus
}
#endif


#endif /* _TRCRYLIB_H_ */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:34      For: Xilinx-Samsung
 * File Name: trcrylib.h    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 31168    Output checksum: 28149
 */
