/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: include/trlance.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:33:26CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * This LANCE driver was taken from the GNU sources for LINUX
 * and modified for the Treck TCP/IP protocol stack
 *
 *   Written 1993 by Donald Becker.
 *
 *   Copyright 1993 United States Government as represented by the
 *   Director, National Security Agency.  This software may be used and
 *   distributed according to the terms of the GNU Public License,
 *   incorporated herein by reference.
 *
 * Driver Modified by Paul and Jared at Treck Inc. 1999
 * Modification History
 * Date: $Date: 2010/01/18 06:33:26CST $
 * Author: Jared
 * Version:  $Revision: 6.0.2.2 $
 * Release:  $ProjectRevision: 6.0.1.23 $
 * Description: Driver for LANCE ethernet chips
 *
 * Developed on an AMD Net186 board (AMD PCNet-ISA II Ethernet controller
 * and AM186ES-40VC processor) using Borland BCC 5.0 compiler
 */
#ifndef _TRLANCE_H_

#define _TRLANCE_H_

#ifdef __cplusplus
extern "C" {
#endif

/*
 * The size of the receive and send descriptor rings (respectively). The
 * valid sizes for these rings are 1, 2, 4, 8, 16, 32, 64, and 128. They
 * can be set to different sizes.
 */
#define TM_LANCE_RING_SIZE_RX    8
#define TM_LANCE_RING_SIZE_TX    8

/* Constants for the Ioctl function */
#define TM_LANCE_IOCTL_REFILL         0x01
#define TM_LANCE_IOCTL_SEND_COMPLETE  0x02

/* Forward Definitions */
int  tfLanceOpen(ttUserInterface lanceHandle);
int  tfLanceSend(ttUserInterface lanceHandle,
                 char TM_FAR *   dataPtr,
                 int             len,
                 int             flag);
int  tfLanceClose(ttUserInterface lanceHandle);
void tfLanceIsrHandler0(void);
void tfLanceIsrHandler1(void);
/* Add more LANCE ISR handler here */

int  tfLanceReceive(ttUserInterface        interfaceHandle,
                    char TM_FAR * TM_FAR * dataPtr,
                    int  TM_FAR *          dataLength,
                    ttUserBufferPtr        bufHandlePtr);
int  tfLanceIoctl(ttUserInterface interfaceHandle,
                  int             flag,
                  void TM_FAR *   optionPtr,
                  int             optionLen);
int  tfLanceGetPhysicalAddress(ttUserInterface lanceHandle,
                               char TM_FAR *   physicalAddress);

#ifdef __cplusplus
}
#endif

#endif /* _TRLANCE_H_ */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:34      For: Xilinx-Samsung
 * File Name: trlance.h     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 20976    Output checksum: 39721
 */
