/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:
 *  The primary header file that the user needs to include in their
 *  application to be able to use IPsec/IKE.
 *
 * Filename: trsecapi.h
 * Author: Jin Zhang
 * Date Created: 03/03/02
 * $Source: include/trsecapi.h $
 *
 * Modification History
 * $Revision: 6.0.2.4 $
 * $Date: 2011/11/29 09:57:26CST $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifndef _TRSECAPI_H_
#define _TRSECAPI_H_


#ifdef __cplusplus
extern "C" {
#endif


#ifdef TM_USE_PRI_PROC
#error You must define TM_USE_PRIORITIES with TM_USE_PRI_PROC
#endif /* TM_USE_PRI_PROC */


#ifdef TM_IKE_VALIDATE_OUTGOING
#error You must define TM_USE_IKE_POLICIES with TM_IKE_VALIDATE_OUTGOING
#endif /* TM_IKE_VALIDATE_OUTGOING */





#if (defined TM_USE_SHA384 || defined TM_USE_SHA512)
#ifndef TM_AALG_VAR_BLOCK_SIZE
#error TM_AALG_VAR_BLOCK_SIZE must be defined when using SHA-384 or SHA 512. \
       See the crypto manual for details.
#endif /* !TM_AALG_VAR_BLOCK_SIZE */
#endif /* (defined TM_USE_SHA384 || defined TM_USE_SHA512) */

    /*IKE port*/
#define TM_IKE_PORT                        500


#define TM_CRYPTO_ENGINE_SOFTWARE            0
#define TM_CRYPTO_ENGINE_HIFN7951            1
#define TM_CRYPTO_ENGINE_MCF5235             2
#define TM_CRYPTO_ENGINE_MAX                 3

/* 
 * IPsec and IKE macros
 */

/*
 * This is the maximum number of IPsec SA pairs that can be present at one
 * time.  This value can be overridden in trsystem.h and must be a
 * tt16Bit value.
 */
#ifndef TM_IPSEC_SAD_MAX_SIZE
#define TM_IPSEC_SAD_MAX_SIZE 100
#endif /* TM_IPSEC_SAD_MAX_SIZE */

/*crypto engine request type */
#define TM_CEREQUEST_CRYPTO_HASH              1
#define TM_CEREQUEST_CRYPTO_HMACHASH          2
#define TM_CEREQUEST_CRYPTO_IPSEC             3
#define TM_CEREQUEST_CRYPTO_IKE               4
#define TM_CEREQUEST_PUBKEY_DHKEYPAIR         5
#define TM_CEREQUEST_PUBKEY_DHSHAREDKEY       6
/* For the following request, RSA/DSA keys needs to be stored 
 * as asn.1 structure. Structure to be used is ttGeneralRsaPtr
 * and ttGeneralDsaPtr;
 */
#define TM_CEREQUEST_PUBKEY_RSASIGN           7
#define TM_CEREQUEST_PUBKEY_RSAVERIFY         8
#define TM_CEREQUEST_PUBKEY_DSASIGN           9
#define TM_CEREQUEST_PUBKEY_DSAVERIFY        10
#define TM_CEREQUEST_PUBKEY_RSAENCRYPT       11
#define TM_CEREQUEST_PUBKEY_RSADECRYPT       12
/* For SSL crypto request */
#define TM_CEREQUEST_CRYPTO_SSL              13
#define TM_CEREQUEST_CRYPTO_CMACHASH         14

/* OAKLEY groups */
#define TM_DHGROUP_1                   1
#define TM_DHGROUP_2                   2
#define TM_DHGROUP_3                   3
#define TM_DHGROUP_4                   4
#define TM_DHGROUP_5                   5
#define TM_DHGROUP_14                  14
/* assign an arbitrary number to our exportable DH group. Got to be
 * 512 bits or less
 */
#define TM_DHGROUP_EXPORT              111
/* class value - encryption algorithm, during phase 1 */
/* The IKE_null encrypt algorithm is not interoperable with
 * other vendors. It only works as a testing tool, so that 
 * we don't use any encryption for IKE exchanges
 */
#define TM_IKE_DES_CBC          (ttUser16Bit)1
#define TM_IKE_IDEA_CBC         (ttUser16Bit)2
#define TM_IKE_BLOWFISH_CBC     (ttUser16Bit)3
#define TM_IKE_RC5_R16_B64_CBC  (ttUser16Bit)4
#define TM_IKE_3DES_CBC         (ttUser16Bit)5
#define TM_IKE_CAST_CBC         (ttUser16Bit)6
/* determined by IANA in RFC 3602*/
#define TM_IKE_AES_CBC          (ttUser16Bit)7
#ifdef TM_USE_IKE2
#define TM_IKE2_DES_IV64_CBC    (ttUser16Bit)1
#define TM_IKE2_DES_CBC         (ttUser16Bit)2
#define TM_IKE2_3DES_CBC        (ttUser16Bit)3
#define TM_IKE2_RC5_R16_B64_CBC (ttUser16Bit)4
#define TM_IKE2_IDEA_CBC        (ttUser16Bit)5
#define TM_IKE2_CAST_CBC        (ttUser16Bit)6
#define TM_IKE2_BLOWFISH_CBC    (ttUser16Bit)7
#define TM_IKE2_3IDEA_CBC       (ttUser16Bit)8
#define TM_IKE2_DES_IV32_CBC    (ttUser16Bit)9
#define TM_IKE2_NULL_ENCRYPT    (ttUser16Bit)11
#define TM_IKE2_AES_CBC         (ttUser16Bit)12
#define TM_IKE2_AES_CTR         (ttUser16Bit)13
#endif /* TM_USE_IKE2 */
/* to be determined by IANA - class values 65001-65535 are for private use
   among mutually consenting parties */
#define TM_IKE_NULL_ENCRYPT     (ttUser16Bit)65001
#define TM_IKE_TWOFISH_CBC      (ttUser16Bit)65005
/* class value - Hash Algorithm attribute , during phase 1*/
#define TM_IKE_MD5              (ttUser16Bit)1
#define TM_IKE_SHA1             (ttUser16Bit)2
#define TM_IKE_TIGER            (ttUser16Bit)3
#define TM_IKE_SHA256           (ttUser16Bit)4
#define TM_IKE_SHA384           (ttUser16Bit)5
#define TM_IKE_SHA512           (ttUser16Bit)6

#ifdef TM_USE_IKE2
#define TM_IKE2_NULL_AUTH       (ttUser16Bit)0
#define TM_IKE2_MD5_96          (ttUser16Bit)1
#define TM_IKE2_SHA1_96         (ttUser16Bit)2
#define TM_IKE2_AESXCBC         (ttUser16Bit)5
#define TM_IKE2_SHA256          (ttUser16Bit)12
#define TM_IKE2_SHA384          (ttUser16Bit)13
#define TM_IKE2_SHA512          (ttUser16Bit)14
#endif  /* TM_USE_IKE2 */
/* to be determined by IANA - class values 65001-65535 are for private use
   among mutually consenting parties */
#define TM_IKE_RIPEMD           (ttUser16Bit)65001

#ifdef TM_USE_IKE2
#define TM_IKE2_PRF_MD5         (ttUser16Bit)1
#define TM_IKE2_PRF_SHA1        (ttUser16Bit)2
#define TM_IKE2_PRF_AESXCBC     (ttUser16Bit)4
#define TM_IKE2_PRF_SHA256      (ttUser16Bit)5
#define TM_IKE2_PRF_SHA384      (ttUser16Bit)6
#define TM_IKE2_PRF_SHA512      (ttUser16Bit)7
#endif /* TM_USE_IKE2 */


/* class value - Authentication Method attribute, phase 1 */
#define TM_IKE_PRESHARED_KEY    (ttUser8Bit)1
#define TM_IKE_DSS_SIG          (ttUser8Bit)2
#define TM_IKE_RSA_SIG          (ttUser8Bit)3
#define TM_IKE_RSA_ENC          (ttUser8Bit)4
#define TM_IKE_RSA_REVISED      (ttUser8Bit)5


/* SSL uses TCP port 443 by default */
#define TM_SSL_PORT                    443

/*
 * Cryptography (hashing, encryption...)
 */
#define TM_ESP_ENCRYPT                       1
#define TM_ESP_DECRYPT                       0

/* authentication header algorithms
 * If adding a new algorithm, make sure the AH payload
 * length is represented below */
#define    SADB_AALG_NULL              0
#define    SADB_AALG_MD5HMAC           1 
#define    SADB_AALG_SHA1HMAC          2 
#define    SADB_AALG_RIPEMDHMAC        3 
#define    SADB_AALG_MD2HMAC           4 
#define    SADB_AALG_SHA256HMAC        5
#define    SADB_AALG_SHA384HMAC        6
#define    SADB_AALG_SHA512HMAC        7
#define    SADB_AALG_AESXCBCMAC        8

/* Valid lengths of AH payload field in authentication header.
 * When adding a new valid length, modify tfAhCheckPkt as well */
/* Length only valid for NULL encryption */
#define SADB_AALG_AH_PAYLOAD_LEN_1 ((tt8Bit)1)
/* Length for MD5 and SHA1 */
#define SADB_AALG_AH_PAYLOAD_LEN_4 ((tt8Bit)4)
/* Length of SHA-256 payload */
#define SADB_AALG_AH_PAYLOAD_LEN_5 ((tt8Bit)5)
/* Length of SHA-384 payload */
#define SADB_AALG_AH_PAYLOAD_LEN_7 ((tt8Bit)7)
/* Length of SHA-512 payload */
#define SADB_AALG_AH_PAYLOAD_LEN_9 ((tt8Bit)9)

/* ESP algorithms */
#define    SADB_EALG_NULL              0
#define    SADB_EALG_RC2CBC            6 
#define    SADB_EALG_DESIV64CBC        7 
#define    SADB_EALG_DESCBC            8 
#define    SADB_EALG_3DESCBC           9 
#define    SADB_EALG_ARCFOUR           10 
#define    SADB_EALG_RC5CBC            11 
#define    SADB_EALG_IDEACBC           12 
#define    SADB_EALG_CAST128CBC        13 
#define    SADB_EALG_BLOWFISHCBC       14 
#define    SADB_EALG_3IDEACBC          15
#define    SADB_EALG_AESCBC            16
#define    SADB_EALG_TWOFISHCBC        17
#define    SADB_EALG_AESCTR            33

#define    TM_AESCTR_NONCE_LENGTH_BYTES 4
#define    TM_AESCTR_IV_LENGTH_BYTES    8

/* IPCOMP algorithms */
#define    SADB_CALG_NULL              0
#define    SADB_CALG_LZS               1
#define    SADB_CALG_OUI               2
#define    SADB_CALG_DEFLATE           3

/* Public Key */
#define    SADB_PUBKEY_NULL            19
#define    SADB_PUBKEY_RSA             20
#define    SADB_PUBKEY_DSA             21
/* support Diffie-Hellman group 1 */
#define    SADB_PUBKEY_DIFFIEHELLMAN1  22
/* support Diffie-Hellman group 2 */
#define    SADB_PUBKEY_DIFFIEHELLMAN2  23
/* support Diffie-Hellman group 5 */
#define    SADB_PUBKEY_DIFFIEHELLMAN5  24
/* support Diffie-Hellman group 14 */
#define    SADB_PUBKEY_DIFFIEHELLMAN14 26
#define    SADB_PUBKEY_X509            25

#define    SADB_ALGORITHM_MAX          31

typedef struct tsGenericKey {
    ttUser8BitPtr        keyDataPtr;
    int                  keyRounds;
    ttUser32Bit          keyBits;
    union{
        void TM_FAR *    keySchedulePointer;
        void TM_FAR *    keyContextPointer;
    }schd_context_ptr;
    union{
        ttUser16Bit      keyScheduleLength;
        ttUser16Bit      keyContextLength;
    }schd_context_len;
    
#define keySchedulePtr schd_context_ptr.keySchedulePointer    
#define keyContextPtr schd_context_ptr.keyContextPointer  
#define keyScheduleLen schd_context_len.keyScheduleLength  
#define keyContextLen schd_context_len.keyContextLength  
}ttGenericKey;
typedef ttGenericKey TM_FAR * ttGenericKeyPtr;

typedef struct tsCounterBlock {
    ttUser32Bit  nonce;
    ttUser8Bit   iv[8];
/* always store in network byte order */
    ttUser32Bit  ctr;
}ttCounterBlock;
typedef ttCounterBlock TM_FAR * ttCounterBlockPtr;

typedef struct tsAhAlgorithm {
    ttUser16Bit     aalgName;
    ttUser16Bit     aalgKeyMin;    /* in bits */
    ttUser16Bit     aalgKeyMax;    /* in bits */
    ttUser16Bit     aalgKeyDefault;
    ttUser16Bit     aalgDigestOutBits; /*in bits*/
    ttUser16Bit     aalgDigestTruncateBits; /*in bits*/
#ifdef TM_AALG_VAR_BLOCK_SIZE
    ttUser16Bit     aalgBlockSize; /* in bytes */
#endif /* TM_AALG_VAR_BLOCK_SIZE */
    void          (TM_CODE_FAR *aalgInitFuncPtr) (ttUserVoidPtr contextPtr);
    void          (TM_CODE_FAR *aalgUpdateFuncPtr) (
                                        ttUserVoidPtr  contextPtr, 
                                        ttUser8BitPtr  moreDataPtr, 
                                        ttPktLen       size, 
                                        ttPktLen       offset);
    void          (TM_CODE_FAR *aalgFinalFuncPtr)(
                                      ttUser8BitPtr outDigest, 
                                      ttUserVoidPtr contextPtr);
    int           (TM_CODE_FAR *aalgContextLenFuncPtr) (void);
}ttAhAlgorithm;

typedef ttAhAlgorithm TM_FAR * ttAhAlgorithmPtr;

typedef struct  tsEspAlgorithm {
    int             ealgName;
    ttUser8Bit      ealgBlockSize;  /* block size, in byte */
    ttUser8Bit      ealgPaddingFlag; 
    ttUser16Bit     ealgKeyMin;    /* in bits */
    ttUser16Bit     ealgKeyMax;    /* in bits */
    ttUser16Bit     ealgKeyDefault; /* use for IKE */
    int           (TM_CODE_FAR *ealgScheduleLenFuncPtr) (void);
    int           (TM_CODE_FAR *ealgScheduleFuncPtr) (
                                  struct tsEspAlgorithm TM_FAR * algPtr, 
                                  ttGenericKeyPtr                keyPtr);
    int           (TM_CODE_FAR *ealgBlockDecryptFuncPtr) (
                                  struct tsEspAlgorithm TM_FAR * algPtr,
                                  ttGenericKeyPtr                keyPtr, 
                                  ttUser8BitPtr                  srcPtr, 
                                  ttUser8BitPtr                  dstPtr);
    int           (TM_CODE_FAR *ealgBlockEncryptFuncPtr) (
                                 struct tsEspAlgorithm TM_FAR * algPtr, 
                                  ttGenericKeyPtr               keyPtr, 
                                  ttUser8BitPtr                 srcPtr, 
                                  ttUser8BitPtr                 dstPtr);
}ttEspAlgorithm;
typedef ttEspAlgorithm TM_FAR * ttEspAlgorithmPtr;


/* direction */
#define TM_IPSEC_INBOUND          (ttUser8Bit)0x01 
#define TM_IPSEC_OUTBOUND         (ttUser8Bit)0x02
#define TM_IPSEC_BOTH_DIRECTION   (ttUser8Bit)0x03 /* = 0x01 | 0x02 */

#define TM_CRYPTOREQUEST_ENCODE              0x01
#define TM_CRYPTOREQUEST_DECODE              0x02
#define TM_CRYPTOREQUEST_AH                  0x04
/* We don't have packetPtr to hold the data yet. For example, we are sending
 * the FINISHED handshake message, or NO_WAIT user data. In this case, we 
 * need do SSL encapsulation immediately on the raw data. Otherwise, we 
 * have a packetPtr (or zero copy buffer handle) to hold the data
 */
#define TM_CRYPTOREQUEST_DONE                0x80


/*IPsec options user can change in running time */
#define TM_IPSECOPT_ANTIREPLAY               1
#define TM_IPSECOPT_PFSKEY                   2
#define TM_IPSECOPT_AGGRESS                  3
#define TM_IPSECOPT_AGGRESSDHGROUP           4
#define TM_IPSECOPT_INITCONTACT              5
#define TM_IPSECOPT_ICMPBYPASS               6
#define TM_IPSECOPT_ICMPSRCCHKBYPASS         7
#define TM_IPSECOPT_ICMP6NDMLDBYPASS         8
#define TM_IPSECOPT_NESTIKE_BYPASS           9
/* set the dfbit options */
#define TM_IPSECOPT_DFBIT                    10
#define TM_IPSECOPT_DEAD_PEER_DETECTION      11

/* PKI options */
#define TM_PKIOPT_PKICERTCHECKALIVE          1
#define TM_PKIOPT_PKICERTNONVERIFY           2


/*how to treat DF bit with regarding to outer header */
#define TM_IPSEC_DFBIT_CLEAR                 0
#define TM_IPSEC_DFBIT_SET                   1
#define TM_IPSEC_DFBIT_COPY                  2


/* 
 * Policy Content Flags
 */

/* policy content flags masks, from the lowest bit: Bit 0 
 * bit 0: set means tunnel mode
 * bit 1: set means transport mode, you can't set both tunnel and transport
 * bit 2  set means AH
 * bit 3  set means ESP. For one policy content, you can't set both AH and ESP
 * Bit 4: set means each remote ip needs its own SA. Prohibit sharing.
           see RFC 2401, 4.4.1 
 * bit 5: set means each remote port needs its own SA. Prohibit sharing.
 * Bit 6: set means each local ip needs its own SA. Prohibit sharing.
 * bit 7: set means each local port needs its own SA. seldom used.
 * Bit 8: set means each protocol needs its own SA, prohibit sharing
 * Bit 9: set means bypass ipsec policy check
 * Bit 10: set means discard the matched packet
 * Bit 11, 12 set to indicate if loc ip and remote ip is DNS format or not
 * Bit 13: set means to log this packet. (selector value and 40 bytes from
 * the ulp.
 * Bit 14: This policy is used for the tunnelling interface between MN's Co@ 
 * its HA. 
 * Bit 15: set means IP Compression (only negotiation is supported)
 * Bit 16: set means 64-bit sequence number support is enabled
 * Bit 17: set means anti-replay is enabled
 * Bit 18: used with IPsec2, traffic is protected with AH or ESP
 */
#define     TM_PFLAG_TUNNEL           0x01 /* bit 0 */
#define     TM_PFLAG_TRANSPORT        0x02 /* bit 1 */
#define     TM_PFLAG_AH               0x04 /* bit 2 */
#define     TM_PFLAG_ESP              0x08 /* bit 3 */
#define     TM_PFLAG_RIP_PACKET       0x10 /* bit 4 */
#define     TM_PFLAG_RPT_PACKET       0x20 /* bit 5 */
#define     TM_PFLAG_LIP_PACKET       0x40 /* bit 6 */
#define     TM_PFLAG_LPT_PACKET       0x80 /* bit 7 */
#define     TM_PFLAG_PROTO_PACKET     0x0100 /* bit 8 */
#define     TM_PFLAG_BYPASS           0x0200 /* bit 9 */ 
#define     TM_PFLAG_DISCARD          0x0400 /* bit 10 */
#define     TM_PFLAG_LOCIP_DNS_NAME   0x0800 /* bit 11 */
#define     TM_PFLAG_REMTIP_DNS_NAME  0x1000 /* bit 12 */
#define     TM_PFLAG_LOG              0x2000 /* bit 13 */
#define     TM_PFLAG_MIPV6_HA_TUNNEL  0x4000 /* bit 14 */
#define     TM_PFLAG_IPCOMP           0x8000 /* bit 15 */
#define     TM_PFLAG_64BIT_SEQ_NO     0x10000 /* bit 16 */
#define     TM_PFLAG_ANTI_REPLAY      0x20000 /* bit 17 */
#define     TM_PFLAG_PROTECT          0x40000 /* bit 18 */

#define     TM_PFLAG_SELMASK          (TM_PFLAG_RIP_PACKET | \
                                       TM_PFLAG_RPT_PACKET | \
                                       TM_PFLAG_LIP_PACKET | \
                                       TM_PFLAG_LPT_PACKET | \
                                       TM_PFLAG_PROTO_PACKET)

#define     TM_PFLAG_PROTOPORTMASK    (TM_PFLAG_RPT_PACKET | \
                                       TM_PFLAG_LPT_PACKET | \
                                       TM_PFLAG_PROTO_PACKET)


/* 
 * SA options, used in tfSadbRecordSetOptions. When SA expires, we don't
 * rekey this SA, just remove it
 */
#define     TM_IPSEC_SADB_NOREKEYING        1
/* SA options, used in tfSadbRecordSetOptions to indicate that the SA will 
 * not expire at all. 
 */
#define     TM_IPSEC_SADB_NOEXPIRING        2

/* Policy query options */
/* query all policies including those inactive. This is very 
 * important for MIPv6 HA. Originally HA sets all policy with MN
 * to inactive. When MN send a quick mode negotiation, HA should
 * query all its policies (including inactive ones.). Later,
 * when IKE negotiation finishes, HA is responsible to activate
 * the inactive policies.
 */
#define     TM_PQUERY_INCLUDE_INACTIVE    0x01
/* query the policy on MIP virtual interface (tunnelling to HA)*/
#define     TM_PQUERY_MIP_VIR_INTERFACE   0x02
/*
 * IPsec Selectors
 */

/* wild values */
#define     TM_SELECTOR_WILD_PORT       (ttUser16Bit)0
#define     TM_SELECTOR_WILD_PROTOCOL   (ttUser8Bit)0

/* Actually, to denote a wild ip address for either IPv4
 * and IPv6, we just store zero in the ss_family.
 */

/* local ip selector is a host IP address*/
#define     TM_SELECTOR_LOCIP_HOST      (ttUser16Bit)0x01
/* remote ip selector is a host IP address*/
#define     TM_SELECTOR_REMTIP_HOST     (ttUser16Bit)0x02
/* both ip selectors are host IP addresses */
#define     TM_SELECTOR_BOTHIP_HOST     (ttUser16Bit)0x03 /* = 0x01 | 0x02 */

/* local ip selector is a subnet*/
#define     TM_SELECTOR_LOCIP_SUBNET    (ttUser16Bit)0x04
/* remote ip selector is a subnet */
#define     TM_SELECTOR_REMTIP_SUBNET   (ttUser16Bit)0x08
/* both ip selectors are subnets  */
#define     TM_SELECTOR_BOTHIP_SUBNET   (ttUser16Bit)0x0c /* = 0x04 | 0x08 */

/* local ip selector is an ip range, min + max */
#define     TM_SELECTOR_LOCIP_RANGE     (ttUser16Bit)0x10
/* remote ip selector is an ip range, min + max */
#define     TM_SELECTOR_REMTIP_RANGE    (ttUser16Bit)0x20
/* both ip selectors are ip ranges */
#define     TM_SELECTOR_BOTHIP_RANGE    (ttUser16Bit)0x30 /* = 0x10 | 0x20 */

/* local ip selector is a domain name */
#define     TM_SELECTOR_LOCIP_DNS_NAME  (ttUser16Bit)0x40
/* remote ip selector is a domain name */
#define     TM_SELECTOR_REMTIP_DNS_NAME (ttUser16Bit)0x80
/* both ip selector are domain name */
#define     TM_SELECTOR_BOTHIP_DNS_NAME (ttUser16Bit)0xc0 /* = 0x40 | 0x80 */

/* use prefix length to indicate a subnet, rather than input masks */
#define     TM_SELECTOR_USE_PREFIX_LENGTH  (ttUser16Bit)0x100

/* local ip selector type mask */
#define     TM_SELECTOR_LOC_OPTION      (ttUser16Bit)0x15
/*0x15 = localIP_HOST 0x01 + localIP_SUBNET 0x04 + localSRCIP_RANGE 0x10*/

/* remote ip selector type mask */
#define     TM_SELECTOR_REMT_OPTION     (ttUser16Bit)0x2a
/*0x2a = remoteIP_HOST 0x02 + remoteIP_SUBNET 0x08 + localSRCIP_RANGE 0x20*/

#define     TM_SELECTOR_LOCPORT_RANGE   (ttUser16Bit)0x0400
#define     TM_SELECTOR_REMTPORT_RANGE  (ttUser16Bit)0x0800
#define     TM_SELECTOR_BOTHPORT_RANGE  (ttUser16Bit)0x0C00

/*4.6.2.1 Identification Type Values*/
#define TM_DOI_ID_RESERVED                 (ttUser8Bit)0
#define TM_DOI_ID_IPV4_ADDR                (ttUser8Bit)1
#define TM_DOI_ID_FQDN                     (ttUser8Bit)2
#define TM_DOI_ID_USER_FQDN                (ttUser8Bit)3
#define TM_DOI_ID_IPV4_ADDR_SUBNET         (ttUser8Bit)4
#define TM_DOI_ID_IPV6_ADDR                (ttUser8Bit)5
#define TM_DOI_ID_IPV6_ADDR_SUBNET         (ttUser8Bit)6
#define TM_DOI_ID_IPV4_ADDR_RANGE          (ttUser8Bit)7
#define TM_DOI_ID_IPV6_ADDR_RANGE          (ttUser8Bit)8
#define TM_DOI_ID_KEY_ID                   (ttUser8Bit)11
/* Used in parser2 to identify MIPv6 ID names 
 * Values 249-255 are reserved for private use
 */
#define TM_DOI_ID_MIPV6                    (ttUser8Bit)249

/* RFC 2408 */
#define TM_IKE_COOKIESECRET_REFRESH_RATE   900
#define TM_ISAKMP_COOKIE_SIZE              8
#define TM_ISAKMP_GENERICHEADER_SIZE       4
/* make sure NONCE is divisible by four */
#define TM_ISAKMP_NONCE_DEFAULT_SIZE       16
#define TM_MINIMUM_PUBLIC_VALUE_SIZE       32
#define TM_IKE_ATTRIBUTE_BASIC_LENGTH      4

/* definition for PKI IKE */
#define TM_PKI_RSA_KEY              0
#define TM_PKI_DSA_KEY              1
/* top-level CA ID */
#define TM_PKI_ROOT_CAID            "1.1.1.1"
#define TM_PKI_INTERMEDIARY_CAID    "2.2.2.2"

/* certificate with PEM format */
#define TM_PKI_CERTIFICATE_PEM      (ttUser8Bit)0x01
/* certificate with DER format */
#define TM_PKI_CERTIFICATE_DER      (ttUser8Bit)0x02
/* the input certificate is already string with PEM, instead of file */
#define TM_PKI_CERTIFICATE_STRING   (ttUser8Bit)0x04

/* the certificate is owner self */
#define TM_PKI_CERT_LOCAL           0x01
/* general certificate including CA or other peer's certificate */
#define TM_PKI_CERT_NONLOCAL        0x02
/* the certificate is CRL */
#define TM_PKI_CERT_CRL             0x04
/* this certificate is a root ca's */
#define TM_PKI_CERT_ROOTCA          0x08



typedef struct sadb_sa
{
    ttUser32Bit sadb_sa_spi ;
    ttUser8Bit  sadb_sa_replay ;
    ttUser8Bit  sadb_sa_state ;
    ttUser8Bit  sadb_sa_auth ;
    ttUser8Bit  sadb_sa_encrypt ;
}
sadb_sa ;
typedef sadb_sa  TM_FAR * sadb_saPtr ;

typedef struct tsSaIdentity{
    struct sockaddr_storage  siSrcSockaddr;
    struct sockaddr_storage  siDstSockaddr;
#ifdef TM_USE_STRONG_ESL
/* Physical port mask */
    ttUser32Bit              siPortMask;
#endif /* TM_USE_STRONG_ESL */
    ttUser32Bit              siSpi;
    ttUser8Bit               siProto;
}ttSaIdentity;
typedef ttSaIdentity TM_FAR * ttSaIdentityPtr;


typedef int (TM_CODE_FAR *ttCryptoSessionOpenFuncPtr)(
    ttUserVoidPtr TM_FAR * sessionPtrPtr,
    ttUserVoidPtr          param1Ptr, 
    ttUserVoidPtr          param2Ptr);
typedef int (TM_CODE_FAR *ttCryptoSessionFuncPtr)(
    ttUserVoidPtr              sessionPtr);
typedef int (TM_CODE_FAR *ttCryptoEngineInitFuncPtr)(
    ttUserVoidPtr           initParamPtr);
typedef int (TM_CODE_FAR *ttCryptoGetRandomWordFuncPtr)(
    ttUser32BitPtr       dataPtr,
    ttUser16Bit          wordSize);

typedef struct tsCryptoEngine
{
    ttUser32Bit                   ceEngineId;
    ttUser32Bit                   ceSupportFlags;
    ttCryptoEngineInitFuncPtr     ceInitFuncPtr;
    ttCryptoGetRandomWordFuncPtr  ceRandomWordFuncPtr;
    ttCryptoSessionOpenFuncPtr    ceSessionOpenFuncPtr;
    ttCryptoSessionFuncPtr        ceSessionProcessFuncPtr;
    ttCryptoSessionFuncPtr        ceSessionCloseFuncPtr;
}ttCryptoEngine;
typedef ttCryptoEngine TM_FAR * ttCryptoEnginePtr;
typedef ttCryptoEnginePtr TM_FAR * ttCryptoEnginePtrPtr;

typedef struct tsIpsecSelector
{
#ifdef TM_USE_STRONG_ESL
    ttUser32Bit                    selPortMask;
#endif /* TM_USE_STRONG_ESL */ 
/* a pair of IP */
    struct sockaddr_storage        selLocIp1; 
    struct sockaddr_storage        selRemtIp1;
/* if selector is a range, we need another pair */
    struct sockaddr_storage        selLocIp2;
    struct sockaddr_storage        selRemtIp2;
/* port number is already there */
    ttUser16Bit                    selIpFlags; 
    ttUser8Bit                     selProtocol;
    ttUser8Bit                     selPadding;
} 
ttIpsecSelector;
typedef ttIpsecSelector TM_FAR * ttIpsecSelectorPtr;
#define selLocPort   selLocIp1.addr.ipv4.sin_port
#define selRemtPort  selRemtIp1.addr.ipv4.sin_port
#ifdef TM_USE_IPSEC_PORT_RANGE
#define selLocPort2  selLocIp2.addr.ipv4.sin_port
#define selRemtPort2 selRemtIp2.addr.ipv4.sin_port
#endif /* TM_USE_IPSEC_PORT_RANGE */

typedef struct tsPolicyContent{
/* used for bundles, the outer policy content */
    struct tsPolicyContent TM_FAR * pctOuterContentPtr; 
#ifdef TM_USE_STRONG_ESL
    ttUser32Bit                     pctPortMask;
#endif /* TM_USE_STRONG_ESL */ 
/* for router, it may have multiple ip, need to specify,
 * anyway, we don't care the port number here
 */
    struct sockaddr_storage         pctLocIpAddr; 
    struct sockaddr_storage         pctRemtIpAddr;
    ttUserPacketPtr                 pctQueuePtr;
    ttUserPacketPtr                 pctLastQueuePtr;
/* AH|ESP, tunnel|transport, policy|packet determination */
    ttUser32Bit                     pctRuleFlags; 
/* along with pctQueuePtr, indicates how many bytes data are waiting
 * for an IKE negotiation results
 */
    ttUser32Bit                     pctLifeSeconds;
#ifdef TM_USE_IPSEC_64BIT_KB_LIFETIMES
    ttUser64Bit                     pctLifeKbytes;
#else /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser32Bit                     pctLifeKbytes;
#endif /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser32Bit                     pctContentIndex;
    ttUser16Bit                     pctEncryptKeyBits;
    ttUser16Bit                     pctQueueBytes;
/* How many entities own this policy content entry */
    ttUser16Bit                     pctOwnerCount;
/* esp may also have authentication algorithm*/
    ttUser8Bit                      pctAuthAlg;
    ttUser8Bit                      pctEncryptAlg;
#ifdef TM_USE_IPSEC_IPCOMP
    ttUser8Bit                      pctIpcompAlg;
#endif /* TM_USE_IPSEC_IPCOMP */
    ttUser8Bit                      pctP2PfsDhGroup;
}  ttPolicyContent;
typedef ttPolicyContent    TM_FAR * ttPolicyContentPtr;
typedef ttPolicyContentPtr TM_FAR * ttPolicyContentPtrPtr;

typedef struct tsSadbRecord
{
 
/* use single linked list for same-hash and same-policy
 * those with same hash key will be linked
 */
    struct tsSadbRecord  TM_FAR   * sadbNextSameHashPtr ;
/* those of the same policy must be linked in order for quick 
 * delete and search 
 */
    struct tsSadbRecord  TM_FAR   * sadbNextSamePolicyPtr; 
/* inbound sa  <----->   outbound sa. They are twins.*/
    struct tsSadbRecord  TM_FAR   * sadbTwinSaPtr; 
/* This is the back pointer to policy for policy check*/
    struct tsPolicyEntry TM_FAR   * sadbPolicyPtr; 
/* used by IKE rekeying */
    ttPolicyContentPtr              sadbPlcyContentPtr; 
    ttUser8BitPtr                   sadbReplayWindowMapPtr ;
    ttIpsecSelectorPtr              sadbSelectorPtr;
    ttUserVoidPtr                   sadbSoftTimerPtr; 
    ttUserVoidPtr                   sadbHardTimerPtr;
/* Session pointer, may be used for hardware accelerator */
    ttUserVoidPtr                   sadbSessionPtr;
/* lifetime remain bytes */
#ifdef TM_USE_IPSEC_64BIT_KB_LIFETIMES
    ttUser64Bit                     sadbRemainBytes;
    ttUser64Bit                     sadbLifetimeBytesSoftLimit;
    ttUser64Bit                     sadbLifetimeBytesHard;
#else /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser32Bit                     sadbRemainBytes;
    ttUser32Bit                     sadbLifetimeBytesSoftLimit;
    ttUser32Bit                     sadbLifetimeBytesHard;
#endif /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser32Bit                     sadbLifeSeconds;
/* the Src and Dst IP is required by ipsec*/
    struct sockaddr_storage         sadbSrcIpAddress; 
/* they may be different with the selectors */
    struct sockaddr_storage         sadbDstIpAddress; 
#ifdef TM_USE_STRONG_ESL
    ttUser32Bit                     sadbPortMask;
#endif /* TM_USE_STRONG_ESL */
    struct tsGenericKey             sadbAuthKey;
    struct tsGenericKey             sadbEncryptKey;
/* for software crypto engine, we need to assign values for
 * the followign two fields. For hardware crypto engine,
 * the following two fields are not used.
 */
    ttAhAlgorithmPtr                sadbAuthAlgPtr;
    ttEspAlgorithmPtr               sadbEncryptAlgPtr;
    ttCryptoEnginePtr               sadbCryptoEnginePtr;
    sadb_saPtr                      sadbSAPtr ;
    ttUser32Bit                     sadbSequenceNumber ;
    ttUser32Bit                     sadbReplayWindowBegin;

#ifdef TM_USE_SEC_STAT
    ttUser32Bit                     sadbTotalRekeys;
    ttUser32Bit                     sadbByteRekeys;
    ttUser32Bit                     sadbTimeRekeys;
    ttUser32Bit                     sadbPeerRekeys;
    ttUser32Bit                     sadbOtherRekeys;
#endif /* TM_USE_SEC_STAT */
    ttUser32Bit                     sadbSerialNumber;
    ttUser32Bit                     sadbNonceValue;
    ttUser16Bit                     sadbFlags;
/* this record may cached in a socketentryptr */
    ttUser16Bit                     sadbCheckOut;
/* How many entities own this sadb record */
    ttUser16Bit                     sadbOwnerCount;
/* incoming or outgoing */
    ttUser8Bit                      sadbDirection; 
/* Ah, Esp save TM_IP_AH. or TM_IP_ESP */
    ttUser8Bit                      sadbSaProtocol; 
/* transport tunnel */
    ttUser8Bit                      sadbIpsecMode;
    ttUser8Bit                      sadbIsInitiator;
#ifdef TM_USE_SEC_STAT
    ttUser8Bit                      sadbExpiredFlags;
#endif /* TM_USE_SEC_STAT */
}
ttSadbRecord ;
typedef ttSadbRecord TM_FAR * ttSadbRecordPtr ;
typedef ttSadbRecordPtr  TM_FAR * ttSadbRecordPtrPtr ;


typedef struct tsIpsecPolicyPair
{
    ttUser16Bit               ippSelectorIndex;
    ttUser8Bit                ippPlcycontentIndex;
    ttUser8Bit                ippDirection;
}ttIpsecPolicyPair;
typedef struct tsIpsecPolicyPair TM_FAR * ttIpsecPolicyPairPtr;



typedef struct tsIpsecSelectorInString{
/* first pair of local and remote */
    char    TM_FAR     * selstrLocIp1;
    union 
    {
        int              selstr_loc_prefix_length;
        char TM_FAR    * selstr_loc_ip2;
    } selstrLocIp2Union;
/* if in range, we need another pair */
    char TM_FAR       *  selstrRemtIp1;
    union 
    {
        int              selstr_remt_prefix_length;
        char TM_FAR   *  selstr_remt_ip2;
    } selstrRemtIp2Union;
#ifdef TM_USE_STRONG_ESL
    ttUser32Bit          selstrPortMask;
#endif /* TM_USE_STRONG_ESL */
    ttUser16Bit          selstrIpFlags; 
/* local port and remote port */
    ttUser16Bit          selstrLocPort;
    ttUser16Bit          selstrRemtPort;
    ttUser16Bit          selstrProtocol;
#ifdef TM_USE_IPSEC_PORT_RANGE
    ttUser16Bit          selstrLocPort2;
    ttUser16Bit          selstrRemtPort2;
#endif /* TM_USE_IPSEC_PORT_RANGE */
#define selstrLocPrefixLength  selstrLocIp2Union.selstr_loc_prefix_length
#define selstrLocIp2           selstrLocIp2Union.selstr_loc_ip2
#define selstrRemtPrefixLength selstrRemtIp2Union.selstr_remt_prefix_length
#define selstrRemtIp2          selstrRemtIp2Union.selstr_remt_ip2
} ttIpsecSelectorInString;
typedef ttIpsecSelectorInString TM_FAR * ttIpsecSelectorInStringPtr;


typedef struct tsPolicyContentInString{
/* for router, it may have multiple ip, need to specify */
    char TM_FAR *             pctstrLocIpAddr; 
    char TM_FAR *             pctstrRemtIpAddr;
/* AH|ESP, tunnel|transport, policy|packet determination */
    ttUser32Bit               pctstrRuleFlags; 
/* esp may also have authentication algorithm*/
    ttUser16Bit               pctstrAuthAlg;
    ttUser16Bit               pctstrEncryptAlg;
#ifdef TM_USE_IPSEC_IPCOMP
    ttUser8Bit                pctstrIpcompAlg;
#endif /* TM_USE_IPSEC_IPCOMP */
/* BUG 565, we decide to add the following two fields of lifetime*/
/* lifetime value in seconds. zero value(0) means to use the 
 * default value TM_IPSECSA_TLIFETIME_DEFAULT 
 */
    ttUser32Bit               pctstrLifeSeconds;
/* lifetime value in kilo bytes. zero value(0) means to use the 
 * default value TM_IPSECSA_VLIFETIME_DEFAULT 
 */
#ifdef TM_USE_IPSEC_64BIT_KB_LIFETIMES
    ttUser64Bit               pctstrLifeKbytes;
#else /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser32Bit               pctstrLifeKbytes;
#endif /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
#ifdef TM_USE_STRONG_ESL
    ttUser32Bit               pctstrPortMask;
#endif /* TM_USE_STRONG_ESL */
/* encryption algorithm key length in bits. 0 means to use the
 * the default key length 
 */
    ttUser16Bit               pctstrEncryptKeyBits;
/* Phase 2 PFS DH Group - 0 means Phase 2 PFS is disabled */
    ttUser8Bit                pctstrP2PfsDhGroup;
}  ttPolicyContentInString;
typedef ttPolicyContentInString TM_FAR * ttPolicyContentInStringPtr;

typedef struct tsPolicyEntry{
    struct tsPolicyEntry TM_FAR *   plcyNextPtr;
/* link all SA of the same policy*/
    ttSadbRecordPtr                 plcyInboundSadbPtr; 
    ttSadbRecordPtr                 plcyOutboundSadbPtr;
    ttIpsecSelector                 plcySelector;
    ttPolicyContentPtr              plcyContentPtr;
    ttUser32Bit                     plcySerialNumber;
/* How many entities own this policy entry */
    ttUser16Bit                     plcyOwnerCount;
/* How many entities own this policy content entry */
    ttUser16Bit                     plcyIndex; 
/* ipsec overhead header size*/
    ttUser16Bit                     plcyHdrLen; 
/* ipsec overhead trailer */
    ttUser16Bit                     plcyTrailerLen; 
/* 1 for in,2 for out, 3 for both */
    ttUser8Bit                      plcyDirection; 
    ttUser8Bit                      plcyAction;
#ifdef TM_IPSEC_POLICY_USER_DATA
    ttUserVoidPtr                   plcyUserDataPtr;
    ttUser32Bit                     plcyUserDataLen;
#endif /* TM_IPSEC_POLICY_USER_DATA */
#ifdef TM_USE_SEC_STAT
    ttUser16Bit                     plcySaInboundCount;
    ttUser16Bit                     plcySaOutboundCount;
#endif /* TM_USE_SEC_STAT */
}ttPolicyEntry;
typedef ttPolicyEntry TM_FAR *    ttPolicyEntryPtr;
typedef ttPolicyEntryPtr TM_FAR * ttPolicyEntryPtrPtr;

typedef struct tsDiffieHellman
{
/* Prime used, for IKE, we don't use this, because we know the prime
 * given the dhgroup number. However for TLS, the prime is unknown
 */
    ttUser8BitPtr           dhPrimePtr;
/* my secret value, xi or xr depends */
    ttUser8BitPtr           dhSecretPtr;       
/* Initiator public value  g^xi % p */
    ttUser8BitPtr           dhGxiPtr;           
/* Responder public value  g^xr % p */
    ttUser8BitPtr           dhGxrPtr;
/* Our derived shared Secret */
    ttUser8BitPtr           dhGxyPtr;
/* generator can be at most 4 octets */
    ttUser32Bit             dhGenerator;
/* length for prime, group 1 is 96, and group 2 is 128 */
    ttUser16Bit             dhGroupLen;
    ttUser16Bit             dhGxiLen;
    ttUser16Bit             dhGxrLen;
    ttUser16Bit             dhGxyLen;
/* group number, 1-4 currently */ 
    ttUser8Bit              dhGroup;        
/* length for dhSecretPtr, maximum 256 bytes */
    ttUser8Bit              dhSecretLen;  

}ttDiffieHellman;
typedef struct tsDiffieHellman TM_FAR * ttDiffieHellmanPtr;

typedef struct tsCryptoRequestDhrequest
{
    ttDiffieHellmanPtr    crdDhinfoPtr;
    ttUser32Bit           crdIsInitialtor;
}ttCryptoRequestDhrequest;
typedef ttCryptoRequestDhrequest TM_FAR * ttCryptoRequestDhrequestPtr;



/* the information used to generate brandnew SA or rekeying sa.*/
typedef struct tsChildSaInfo
{
    ttPolicyEntryPtr                  chdPlcyPtr;
    ttPolicyContentPtr                chdPlcyContentPtr;
    ttIpsecSelector                   chdPacketSelector;
    ttUser32Bit                       chdMySpi; /* in host order*/
    ttUser32Bit                       chdPeerSpi;
    ttUser32Bit                       chdLifetimeSeconds;
#ifdef TM_USE_IPSEC_64BIT_KB_LIFETIMES
    ttUser64Bit                       chdLifetimeKbytes;
#else /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser32Bit                       chdLifetimeKbytes;
#endif /* TM_USE_IPSEC_64BIT_KB_LIFETIMES */
    ttUser16Bit                       chdAuthKeyBytes;
    ttUser16Bit                       chdEncryptKeyBytes;
#ifdef TM_USE_SEC_STAT
    ttUser32Bit                       chdTotalRekeys;
    ttUser32Bit                       chdByteRekeys;
    ttUser32Bit                       chdTimeRekeys;
    ttUser32Bit                       chdPeerRekeys;
    ttUser32Bit                       chdOtherRekeys;
#endif /* TM_USE_SEC_STAT */
    ttUser8Bit                        chdSpiPending;
#ifdef TM_USE_IKE_VENDOR_ATTR
#ifdef TM_USE_TRECK_ONE_SIDE_REKEY
    ttUser8Bit                        chdVendorNegotFlags;
#endif /* TM_USE_TRECK_ONE_SIDE_REKEY */
#endif /* TM_USE_IKE_VENDOR_ATTR */
}ttChildSaInfo;
typedef ttChildSaInfo TM_FAR * ttChildSaInfoPtr;


/* this was moved from trike.h as it's needed by trsadb.c for bundles */

typedef struct tsCryptoEngineList
{
    struct tsCryptoEngineList TM_FAR * celNextPtr;
    ttCryptoEnginePtr                  celCryptoEnginePtr;
}ttCryptoEngineList;
typedef ttCryptoEngineList TM_FAR * ttCryptoEngineListPtr;

typedef struct tsCryptoChainData
{
    struct tsCryptoChainData TM_FAR * ccdNextPtr;
    ttUser8BitPtr                     ccdDataPtr;
    ttPktLen                          ccdDataLen;
}ttCryptoChainData;
typedef ttCryptoChainData TM_FAR * ttCryptoChainDataPtr;

typedef struct tsCryptoRequestHash
{
    ttAhAlgorithmPtr      crhAlgPtr;
    ttCryptoChainDataPtr  crhInDataPtr;
    ttUser8BitPtr         crhOutPtr;
    ttUser16Bit           crhHashAlg;
    ttUser16Bit           crhDataLength;
}ttCryptoRequestHash;
typedef ttCryptoRequestHash TM_FAR * ttCryptoRequestHashPtr;

typedef struct tsCryptoRequestHmacHash
{
    ttAhAlgorithmPtr      crhhAlgPtr;
    ttCryptoChainDataPtr  crhhInDataPtr;
    ttUser8BitPtr         crhhOutPtr;
    ttUser8BitPtr         crhhKeyPtr;
    ttUser32Bit           crhhDataLength;
    ttUser16Bit           crhhKeyLen;
    ttUser16Bit           crhhHashAlg;
}ttCryptoRequestHmacHash;
typedef ttCryptoRequestHmacHash TM_FAR * ttCryptoRequestHmacHashPtr;

typedef struct tsCryptoRequestIpsec
{
    ttUserPacketPtr       criPacketPtr;
    ttUserPacketPtr       criNewPacketPtr;
    ttSadbRecordPtr       criSadbPtr;
    ttUser8BitPtr         criIvPtr;
    ttUser8BitPtr         criMacOutPtr;
    ttUser8BitPtr         criOrigMacPtr;
    ttUser16Bit           criCryptHeadLength;
    ttUser16Bit           criCryptDataLength;
    ttUser16Bit           criMacHeadLength;
    ttUser16Bit           criMacDataLength;
}ttCryptoRequestIpsec;
typedef ttCryptoRequestIpsec TM_FAR * ttCryptoRequestIpsecPtr;





typedef struct tsCertificateEntry {
    const char TM_FAR * certDataPtr;/* points to start of raw certificate*/
    int                 certLength; /* length of raw certificate */
    int                 certFlags;  /* flag to indicate status of this cert*/
}ttCertificateEntry;
typedef ttCertificateEntry TM_FAR * ttCertificateEntryPtr;

typedef struct tsCryptoRequest
{
    union
    {
        ttCryptoRequestHashPtr        crHashParamPtr;
        ttCryptoRequestHmacHashPtr    crHmacHashParamPtr;
        ttCryptoRequestDhrequestPtr   crDhgroupParamPtr;
    }crParamUnion;

    ttUser16Bit                       crResult;
    ttUser8Bit                        crType;
    ttUser8Bit                        crFlags;
}ttCryptoRequest;
typedef ttCryptoRequest TM_FAR * ttCryptoRequestPtr;

/*
 * AH, ESP algorithm lookup API
 */
TM_PROTO_EXTERN ttAhAlgorithmPtr tfAhAlgorithmLookup(int idx);
TM_PROTO_EXTERN ttEspAlgorithmPtr tfEspAlgorithmLookup(int idx);

/*
 * Use IPsec API 
 */
TM_PROTO_EXTERN int tfUseIpsec(void);

TM_PROTO_EXTERN int tfIpsecUninitialize(void);

TM_PROTO_EXTERN int tfIpsecSetOptions(ttUser32Bit       option,
                                      ttUser8Bit        value);
TM_PROTO_EXTERN int tfPkiSetOptions(ttUser32Bit       option,
                                    ttUser8Bit        value);

/* Get crypto engine that supports algorithm */
TM_PROTO_EXTERN ttCryptoEnginePtr tfCryptoEngineGet(int       algorithm);
/* Get crypto engine that supports both algorithms */
TM_PROTO_EXTERN ttCryptoEnginePtr tfCryptoEngineGet2(
                                        int algorithm1, int algorithm2);
/* Get first crypto engine */
TM_PROTO_EXTERN ttCryptoEnginePtr tfCryptoEngineGetFirst(void);

TM_PROTO_EXTERN int tfCryptoEngineRegister(
                           unsigned int                 engineId,
                           ttUserVoidPtr                initParamPtr,
                           ttCryptoEnginePtrPtr         newEnginePtrPtr,
                           ttCryptoEngineInitFuncPtr    engineInitFuncPtr,
                           ttCryptoGetRandomWordFuncPtr randomFuncPtr,
                           ttCryptoSessionOpenFuncPtr   sessionOpenFuncPtr,
                           ttCryptoSessionFuncPtr       sessionProcessFuncPtr,
                           ttCryptoSessionFuncPtr       sessionCloseFuncPtr);
TM_PROTO_EXTERN int tfCryptoEngineDeRegister(unsigned int    engineId);

TM_PROTO_EXTERN int tfCryptoEngineAddAlgorithm(
                           ttCryptoEnginePtr          cenginePtr,
                           unsigned int               algorithmId);

TM_PROTO_EXTERN int tfCryptoEngineSwapUserBufsStart(
                           ttCryptoRequestIpsecPtr    reqPtr);

TM_PROTO_EXTERN void tfCryptoEngineSwapUserBufsFinish(
                           ttCryptoRequestIpsecPtr    reqPtr);

TM_PROTO_EXTERN void tfCryptoEncAlgToStr(ttUser8Bit module, 
                                         ttUser16Bit encAlg, 
                                         ttUserConstCharPtrPtr bufferPtrPtr);
TM_PROTO_EXTERN void tfCryptoHashAlgToStr(ttUser8Bit module, 
                                          ttUser16Bit hashAlg, 
                                          ttUserConstCharPtrPtr bufferPtrPtr);


#ifdef TM_USE_MCF5235_CRYPTOENGINE
int tfMcf5235EngineInit(ttUserVoidPtr initParamPtr);
int tfMcf5235GetRandomWord(ttUser32Bit TM_FAR * dataPtr,
                           ttUser16Bit          wordSize);
int tfMcf5235SessionProcess(ttUserVoidPtr cryptoRequestPtr);
#endif /* TM_USE_MCF5235_CRYPTOENGINE */



#ifdef TM_USE_SEC_STAT
#ifndef TM_SEC_STAT_MAX_IKE_ADDRESSES
#define TM_SEC_STAT_MAX_IKE_ADDRESSES 1
#elif TM_SEC_STAT_MAX_IKE_ADDRESSES > 32767
#error TM_SEC_STAT_MAX_IKE_ADDRESSES too big! Must not be > 32767.
#endif /* TM_SEC_STAT_MAX_IKE_ADDRESSES */

/*
 * The 2 MSBs are flags for initiator, responder, IKE, IPsec, etc.
 * The 2 LSBs represent the error code
 */
#define TM_STAT_INIT                        0x1000
#define TM_STAT_RESP                        0x2000

#define TM_STAT_IKE                         0x4000
#define TM_STAT_IPSEC                       0x8000

#define TM_STAT_IKE_FAIL_INVAL_COOKIE       0x0001
#define TM_STAT_IKE_FAIL_NORESPONSE         0x0002
#define TM_STAT_IKE_FAIL_BADMESSAGE         0x0003
#define TM_STAT_IKE_FAIL_LOGIC              0x0004
#define TM_STAT_IKE_SEND                    0x0005
#define TM_STAT_IKE_RECV                    0x0006
#define TM_STAT_IKE_DPD_RUT_SENT            0x0007
#define TM_STAT_IKE_DPD_RUT_SENT_RETRANS    0x0008
#define TM_STAT_IKE_DPD_RUT_RECV            0x0009
#define TM_STAT_IKE_DPD_ACK_SENT            0x000A
#define TM_STAT_IKE_DPD_ACK_RECV            0x000B
#define TM_STAT_IKE_DPD_RUT_SEQNUM_ERROR    0x000C
#define TM_STAT_IKE_DPD_ACK_SEQNUM_ERROR    0x000D
#define TM_STAT_IKE_DPD_COOKIE_ERROR        0x000E

#define TM_STAT_IPSEC_FAIL_INVAL_COOKIE     0x0081
#define TM_STAT_IPSEC_FAIL_NORESPONSE       0x0082
#define TM_STAT_IPSEC_FAIL_BADMESSAGE       0x0083
#define TM_STAT_IPSEC_FAIL_LOGIC            0x0084

#define TM_STAT_ERROR_MASK                  0x00FF

#define TM_STAT_DECREMENT                   (int)-1
#define TM_STAT_INCREMENT                   (int) 1

#define TM_STAT_FLAG_VALID                  0x0001
#define TM_STAT_FLAG_INVAL                  (ttS16Bit)-1

#define TM_IPSEC_EXPIRE_SOFT_BYTE           (tt8Bit)1
#define TM_IPSEC_EXPIRE_SOFT_TIME           (tt8Bit)2
#define TM_IPSEC_EXPIRE_PEER                (tt8Bit)3


typedef struct tsIsakmpStats
{
    struct sockaddr_storage     iksIpAddress;
/* IKE */
/* Byte and packet counts */
    ttUser64Bit                 iksIkeTotalPacketsIn;
    ttUser64Bit                 iksIkeTotalPacketsOut;
    ttUser64Bit                 iksIkeTotalBytesIn;
    ttUser64Bit                 iksIkeTotalBytesOut;
/* Negotiation statistics */
    ttUser32Bit                 iksIkeCurrentNumOfSas;
    ttUser32Bit                 iksIkeCurrentInitiatedSas;
    ttUser32Bit                 iksIkeCurrentRespondedSas;
    ttUser32Bit                 iksIkeTotalInitiatedSas;
    ttUser32Bit                 iksIkeTotalRespondedSas;
    ttUser32Bit                 iksIkeTotalAttempts;
    ttUser32Bit                 iksIkeTotalAMAttemptsAsInit;
    ttUser32Bit                 iksIkeTotalMMAttemptsAsInit;
    ttUser32Bit                 iksIkeTotalAMAttemptsAsResp;
    ttUser32Bit                 iksIkeTotalMMAttemptsAsResp;
/* Error statistics */
    ttUser32Bit                 iksIkeInitiatorFails;
    ttUser32Bit                 iksIkeResponderFails;
    ttUser32Bit                 iksIkeInvalCookieFails;
    ttUser32Bit                 iksIkeNoResponseFails;
    ttUser32Bit                 iksIkeBadMessageFails;
    ttUser32Bit                 iksIkeLogicFails;
/* IPsec */
/* Negotiation statistics */
    ttUser32Bit                 iksIpsecCurrentNumOfSas;
    ttUser32Bit                 iksIpsecCurrentInitiatedSas;
    ttUser32Bit                 iksIpsecCurrentRespondedSas;
    ttUser32Bit                 iksIpsecTotalInitiatedSas;
    ttUser32Bit                 iksIpsecTotalRespondedSas;
    ttUser32Bit                 iksIpsecTotalAttempts;
    ttUser32Bit                 iksIpsecTotalQMAttemptsAsInit;
    ttUser32Bit                 iksIpsecTotalQMAttemptsAsResp;
/* Error statistics */
    ttUser32Bit                 iksIpsecInitiatorFails;
    ttUser32Bit                 iksIpsecResponderFails;
    ttUser32Bit                 iksIpsecInvalCookieFails;
    ttUser32Bit                 iksIpsecNoResponseFails;
    ttUser32Bit                 iksIpsecBadMessageFails;
    ttUser32Bit                 iksIpsecLogicFails;
} ttIsakmpStats;
typedef ttIsakmpStats TM_FAR * ttIsakmpStatsPtr;

typedef struct tsSadbDetails
{
    struct sockaddr_storage     sadbDetLocalIp;
    struct sockaddr_storage     sadbDetRemoteIp;
    ttUser64Bit                 sadbDetInLifetimeHardByteLimit;
    ttUser64Bit                 sadbDetOutLifetimeHardByteLimit;
    ttUser64Bit                 sadbDetInLifetimeSoftByteLimit;
    ttUser64Bit                 sadbDetOutLifetimeSoftByteLimit;
    ttUser64Bit                 sadbDetInLifetimeBytesProcessed;
    ttUser64Bit                 sadbDetOutLifetimeBytesProcessed;
    ttUser32Bit                 sadbDetInLifetimeHardTimeLimit;
    ttUser32Bit                 sadbDetOutLifetimeHardTimeLimit;
    ttUser32Bit                 sadbDetInLifetimeSoftTimeLimit;
    ttUser32Bit                 sadbDetOutLifetimeSoftTimeLimit;
    ttUser32Bit                 sadbDetInLifetimeTimeRemain;
    ttUser32Bit                 sadbDetOutLifetimeTimeRemain;
    ttUser32Bit                 sadbDetInSpi;
    ttUser32Bit                 sadbDetOutSpi;
    ttUser32Bit                 sadbDetTotalRekeys;
    ttUser32Bit                 sadbDetByteRekeys;
    ttUser32Bit                 sadbDetTimeRekeys;
    ttUser32Bit                 sadbDetPeerRekeys;
    ttUser32Bit                 sadbDetOtherRekeys;
    ttUser32Bit                 sadbDetEncKeyLen;
    int                         sadbDetEncAlg;
    int                         sadbDetNumOfSas;
    ttUser16Bit                 sadbDetAuthAlg;
    ttUser8Bit                  sadbDetMode;
    ttUser8Bit                  sadbDetLocalProtocol;
    ttUser8Bit                  sadbDetRemoteProtocol;
    ttUser8Bit                  sadbDetKind;
    ttUser8Bit                  sadbDetDirection;
#ifdef TM_USE_IPSEC_IPCOMP
    ttUser8Bit                  sadbDetCompressAlg;
#endif /* TM_USE_IPSEC_IPCOMP */
} ttSadbDetails;
typedef ttSadbDetails TM_FAR * ttSadbDetailsPtr;

typedef struct tsSadbTotals
{
    ttUser96Bit                 sadbInUserByteCount;
    ttUser96Bit                 sadbInPacketCount;
    ttUser96Bit                 sadbInCompressedByteCount;
    ttUser96Bit                 sadbInCompressedPacketCount;
    ttUser96Bit                 sadbOutUserByteCount;
    ttUser96Bit                 sadbOutPacketCount;
    ttUser96Bit                 sadbOutCompressedByteCount;
    ttUser96Bit                 sadbOutCompressedPacketCount;
    ttUser64Bit                 sadbInAuthErrCount;
    ttUser64Bit                 sadbInPadErrCount;
    ttUser64Bit                 sadbInReplayErrCount;
} ttSadbTotals;
typedef ttSadbTotals TM_FAR * ttSadbTotalsPtr;

typedef struct tsSadbCount
{
    ttUser64Bit                 sadbTotalInEspSas;
    ttUser64Bit                 sadbTotalInAhSas;
    ttUser64Bit                 sadbTotalOutEspSas;
    ttUser64Bit                 sadbTotalOutAhSas;
#ifdef TM_USE_IPSEC_IPCOMP
    ttUser64Bit                 sadbTotalInIpcompSas;
    ttUser64Bit                 sadbTotalOutIpcompSas;
    ttUser32Bit                 sadbCurrentInIpcompSas;
    ttUser32Bit                 sadbCurrentOutIpcompSas;
#endif /* TM_USE_IPSEC_IPCOMP */
    ttUser32Bit                 sadbCurrentInEspSas;
    ttUser32Bit                 sadbCurrentInAhSas;
    ttUser32Bit                 sadbCurrentOutEspSas;
    ttUser32Bit                 sadbCurrentOutAhSas;
    
} ttSadbCount;
typedef ttSadbCount TM_FAR * ttSadbCountPtr;




TM_PROTO_EXTERN int tfSadbRecordGetByPolicy(ttUser32Bit         options,
                                            ttPolicyEntryPtr    plcyPtr,
                                            ttUser8Bit          direction,
                                            ttUserVoidPtrPtr    iteratorPtr,
                                            ttSadbDetailsPtr    sadbDetailsPtr);

TM_PROTO_EXTERN int tfGetIsakmpStats(ttUser8Bit             unit,
                                     ttIsakmpStatsPtr       isakmpStatsPtr);

TM_PROTO_EXTERN int tfGetSadbDetails(ttUser8Bit               unit,
                                     ttUser16Bit              priority,
                                     ttUser32Bit              saNumber,
                                     ttSadbRecordPtr          sadbRecordPtr,
                                     ttSadbDetailsPtr         sadbDetailsPtr);

TM_PROTO_EXTERN int tfGetSadbTotals(ttUser8Bit              unit,
                                    ttSadbTotalsPtr         sadbTotalsPtr);

TM_PROTO_EXTERN int tfGetSadbCount(ttUser8Bit               unit,
                                   ttSadbCountPtr           sadbCountPtr);

TM_PROTO_EXTERN int tfGetIkeAddrIndex(
                      const struct sockaddr_storage TM_FAR * sockAddrPtr);

TM_PROTO_EXTERN int tfRegisterIkeAddr(
                      const struct sockaddr_storage TM_FAR * sockAddrPtr);

TM_PROTO_EXTERN int tfUnregisterIkeAddr(
                      const struct sockaddr_storage TM_FAR * sockAddrPtr);

TM_PROTO_EXTERN void tfStatisticsUpdateCount(
                             int             index,
                             ttUser16Bit     counter,
                             int             addend);

#endif /* TM_USE_SEC_STAT */

/*
 *  Policy Database Management API 
 */
TM_PROTO_EXTERN int tfPolicyRestore(ttIpsecPolicyPairPtr        pairListPtr, 
                                    ttIpsecSelectorInStringPtr  sListPtr,
                                    ttPolicyContentInStringPtr  cListPtr,
                                    int                         npair);

TM_PROTO_EXTERN ttPolicyEntryPtr tfPolicyInsertAtIndex(
    ttUser16Bit         policyIndex,
    ttIpsecSelectorPtr  selectorPtr,
    ttPolicyContentPtr  contentPtr,
    ttUser8Bit          direction,
    int TM_FAR *        errorPtr);

TM_PROTO_EXTERN ttPolicyEntryPtr tfPolicyAdd(ttIpsecSelectorPtr  selectorPtr,
                                             ttPolicyContentPtr  contentPtr,
                                             ttUser8Bit          direction,
                                             int   TM_FAR      * errorPtr);

#ifdef TM_IPSEC_POLICY_USER_DATA
TM_PROTO_EXTERN ttPolicyEntryPtr tfPolicyAddWithUserData(
                                             ttIpsecSelectorPtr  selectorPtr,
                                             ttPolicyContentPtr  contentPtr,
                                             ttUser8Bit          direction,
                                             int   TM_FAR      * errorPtr,
                                             ttUserVoidPtr       userDataPtr,
                                             ttUser32Bit         userDataLen);
#endif /* TM_IPSEC_POLICY_USER_DATA */
TM_PROTO_EXTERN int tfPolicyAddBundle(ttPolicyEntryPtr    plcyPtr,
                                      ttPolicyContentPtr  outerContentPtr);


TM_PROTO_EXTERN 
int tfIpsecPolicyQueryBySelector(ttUser32Bit         options,
                                 ttIpsecSelectorPtr  keySelectorPtr, 
                                 ttPolicyEntryPtrPtr plcyPtrPtr,
                                 ttUser8Bit          direction);

/* get the applied policy, call with IPSEC not already locked */
TM_PROTO_EXTERN int tfIpsecPolicyQueryByIndex(int                 index,
                                              ttPolicyEntryPtrPtr plcyPtrPtr);


TM_PROTO_EXTERN int tfPolicyDelete(ttPolicyEntryPtr   plcyPtr,
                                   ttUser8Bit         direction);

TM_PROTO_EXTERN void tfPolicyClear(void);

/* 
 * SA Database Management API 
 */

TM_PROTO_EXTERN int tfSadbRecordManualAdd(
                          ttSadbRecordPtrPtr          sadbPtrPtr,
                          ttUser8Bit                  direction,
                          ttUser32Bit                 spi,
                          ttIpsecSelectorInStringPtr  selectorStringPtr,
                          int                         contentIndex,
                          const char TM_FAR *         manualKeyPtr,
                          ttUser32Bit                 keyLength);

TM_PROTO_EXTERN int tfSadbRecordGenerate(ttSadbRecordPtrPtr   saPtrPtr,
#ifdef TM_USE_STRONG_ESL
                                         ttUserInterface      interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                                         ttChildSaInfoPtr     saInfoPtr,
                                         ttUser8BitPtr        keyMatPtr,
                                         ttUser16Bit          keyMatLen,
                                         ttUser8Bit           direction);

TM_PROTO_EXTERN int tfSadbRecordGet(ttUser32Bit          options,
                                    ttSaIdentityPtr      saIdPtr, 
                                    ttSadbRecordPtrPtr   sadbPtrPtr);

TM_PROTO_EXTERN int tfSadbRecordFind(ttUser32Bit             options,
                                     ttIpsecSelectorPtr      pktSelectorPtr, 
                                     ttSadbRecordPtrPtr      sadbRecordPtrPtr,
                                     ttPolicyEntryPtr        plcyPtr,
                                     ttPolicyContentPtr      plcyContentPtr,
                                     ttUser8Bit              direction);


TM_PROTO_EXTERN int tfSadbRecordDelete(ttUser32Bit      options,
                                       ttSadbRecordPtr  sadbPtr
                                       );

TM_PROTO_EXTERN int tfSadbRecordDeleteByDestination(
    ttUser32Bit                      options,
    struct sockaddr_storage TM_FAR * destination
                                     );

TM_PROTO_EXTERN int tfSadbRecordDeleteByPolicy(
    ttUser32Bit            options,
    ttPolicyEntryPtr       plcyPtr,
    ttUser8Bit             direction);

TM_PROTO_EXTERN void tfSadbRecordClear(ttUser32Bit options);

TM_PROTO_EXTERN int tfSadbRecordClearAll(ttUser16Bit options);

TM_PROTO_EXTERN int tfSadbRecordClearByPriority(ttUser16Bit priority,
                                                ttUser16Bit options);

TM_PROTO_EXTERN void tfSadbClearSocketCache(ttSadbRecordPtr   sadbPtr);

TM_PROTO_EXTERN void tfSadbRecordSetOptions(ttSadbRecordPtr  sadbPtr, 
                                            ttUser16Bit      optionType,
                                            ttUser8Bit       optionValue);

/* get the applied policy, call with IPSEC not already locked */
int tfIpsecPolicyCKQueryBySelector(ttUser32Bit         options,
                                 ttIpsecSelectorPtr  keySelectorPtr, 
                                 ttPolicyEntryPtrPtr plcyPtrPtr,
                                 ttUser8Bit          direction);

/* get the applied policy, call with IPSEC not already locked */
TM_PROTO_EXTERN int tfIpsecPolicyCKQueryByIndex(int                 index,
                                              ttPolicyEntryPtrPtr plcyPtrPtr);

/* 
 * Unget a policy pointer, obtained via
 * tfIpsecPolicyCKQueryBySelector(), tfIpsecPolicyCKQueryByIndex(),
 * or tfIpsecPolicyCKLockedQueryByIndex(),
 * after the caller no longer needs to access the policy Pointer.
 * Failure to do so, will result in memory leak.
 */
TM_PROTO_EXTERN void tfPolicyUnget(ttUser32Bit      options,
                                   ttPolicyEntryPtr plcyPtr);

TM_PROTO_EXTERN int tfSadbRecordCKManualAdd(
                          ttSadbRecordPtrPtr          sadbPtrPtr,
                          ttUser8Bit                  direction,
                          ttUser32Bit                 spi,
                          ttIpsecSelectorInStringPtr  selectorStringPtr,
                          int                         contentIndex,
                          const char TM_FAR *         manualKeyPtr,
                          ttUser32Bit                 keyLength);

TM_PROTO_EXTERN int tfSadbRecordCKGenerate(ttSadbRecordPtrPtr   saPtrPtr,
#ifdef TM_USE_STRONG_ESL
                                           ttUserInterface      interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                                           ttChildSaInfoPtr     saInfoPtr,
                                           ttUser8BitPtr        keyMatPtr,
                                           ttUser16Bit          keyMatLen,
                                           ttUser8Bit           direction);


TM_PROTO_EXTERN int tfSadbRecordCKGet(ttUser32Bit          options,
                                      ttSaIdentityPtr      saIdPtr, 
                                      ttSadbRecordPtrPtr   sadbPtrPtr);

TM_PROTO_EXTERN int tfSadbRecordCKFind(ttUser32Bit             options,
                                       ttIpsecSelectorPtr      pktSelectorPtr, 
                                       ttSadbRecordPtrPtr      sadbRecordPtrPtr,
                                       ttPolicyEntryPtr        plcyPtr,
                                       ttPolicyContentPtr      plcyContentPtr,
                                       ttUser8Bit              direction);

/* 
 * Unget an sadb record pointer, obtained via tfSadbRecordCKManualAdd(),
 * tfSadbRecordCKGenerate(), tfSadbRecordCKFind(), or tfSadbRecordCKGet(),
 * after the caller no longer needs to access the sadb Record Pointer.
 * Failure to do so, will result in memory leak.
 */
TM_PROTO_EXTERN void tfSadbRecordUnget(ttUser32Bit     options,
                                       ttSadbRecordPtr sadbRecordPtr);

#ifdef TM_USE_IPSEC_DEBUG_TASK
TM_PROTO_EXTERN void tfSadbDumpSaTable(ttUser32Bit options);
#endif /* TM_USE_IPSEC_DEBUG_TASK */


/* Use Log */
TM_PROTO_EXTERN int tfUseIpsecLogging (ttUser16Bit      numLogMsgs);
TM_PROTO_EXTERN int tfIpsecLogWalk(ttLogWalkFuncPtr     funcPtr,
                                   int                  msgSeqNo,
                                   ttUserGenericUnion   genParam); 
#if defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)
TM_PROTO_EXTERN int tfIpsecInitializeFromFile(
    ttUserInterface   interfaceHandle,
    char TM_FAR     * fileName, 
    int TM_FAR      * lineNumberPtr);
#endif /* TM_KERNEL_VISUAL_X86 || TM_KERNEL_WIN32_X86 */

#ifdef TM_USE_PARSER2
TM_PROTO_EXTERN int tfIke2Initialize(char TM_FAR * configStrPtr,
                                     ttUser8Bit configStrType,
                                     ttUserInterface interfaceHandle);
#endif /* TM_USE_PARSER2 */

/*
 * SSL public apis
 */

#ifdef __cplusplus
}
#endif

#endif /* #ifndef _TRSECAPI_H_ */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:35      For: Xilinx-Samsung
 * File Name: trsecapi.h    File Revision: 6.0.2.4 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:     0    Output checksum:  9550
 */
