/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/drivers/lance/tram79c.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:01CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * This AM79C driver was taken from the GNU sources for LINUX
 * and modified for the Treck TCP/IP protocol stack
 *
 *   Written 1993 by Donald Becker.
 *
 *   Copyright 1993 United States Government as represented by the
 *   Director, National Security Agency.  This software may be used and
 *   distributed according to the terms of the GNU Public License,
 *   incorporated herein by reference.
 *
 * Driver Modified by Paul and Jared at Treck Inc. 1999
 * Driver Modefied by jshang for am79c973 in MIPS malta board
 * Modification History
 * Date: $Date: 2010/01/18 06:36:01CST $
 * Author: jshang
 * Version:  $Revision: 6.0.2.2 $
 * Release:  $ProjectRevision: 6.0.1.23 $
 * Description: Driver for AM79C ethernet chips
 *
 */
/* Yamon dump
 jin> pcicfg 0 b 0 0 50
 Read PCI (bus 0x00, dev 0x0b, func 0)
 addr 0x00 : 0x20001022 0x02900147
 addr 0x08 : 0x02000043 0x00002000
 addr 0x10 : 0x00001201 0x08011000
 addr 0x18 : 0x00000000 0x00000000
 addr 0x20 : 0x00000000 0x00000000
 addr 0x28 : 0x00000000 0x20001022
 addr 0x30 : 0x00000000 0x00000040
 addr 0x38 : 0x00000000 0x1818010A
 addr 0x40 : 0xC8010001 0x00000000
 addr 0x48 : 0x00000000 0x00000000
 */
/*
 * Include the Treck Interface types
 */
#include <stdio.h>
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#include <yamon_api.h>

#include <tram79c.h>

#ifdef TM_LITTLE_ENDIAN
#define tm_cpu_to_le(x) (x)
#define tm_le_to_cpu(x) (x)
#define tm_cpu_to_le16(x) (x)
#define tm_le16_to_cpu(x) (x)
#else /* TM_BIG_ENDIAN */
#define tm_cpu_to_le(x)((tt32Bit)((tt32Bit)(((x)>>24)&TM_UL(0xff))\
                                  |(tt32Bit)(((x)>>8)&TM_UL(0xff00))\
                                  |(tt32Bit)(((x)<<8)&TM_UL(0xff0000))\
                                  |(tt32Bit)(((x)<<24)&TM_UL(0xff000000))))  
#define tm_le_to_cpu(x) tm_cpu_to_le(x)
#define tm_cpu_to_le16(x)((tt32Bit)((tt32Bit)(((x)>>8)&TM_UL(0x00ff00ff))\
                                  |(tt32Bit)(((x)<<8)&TM_UL(0xff00ff00))))
#define tm_le16_to_cpu(x) tm_cpu_to_le16(x) 
#endif /* TM_LITTLE_ENDIAN */

/*
 * AM79C Memory mapping I/O, KSEG1
 */
#define TM_AM79C_BASE                   0xB0011000UL
#define TM_AM79C_INTERRUPT_TYPE         0x000a   
 
/* 
 * following paramters are to obtain Am79c973 paramter from yamon API (MIPS+malta) 
 * including timer, and interrup paramters
 * yamon timer: */
/*
#define SYSCON_BOARD_GET_MILLISEC_ID    63
 */

#define SYSCON_read YAMON_FUNC_SYSCON_READ

/* MIPS + Malta board need KSEG1 address */
#define KUSEG_MSK		  0x80000000
#define KSEG_MSK		  0xE0000000
#define KUSEGBASE		  0x00000000
#define KSEG0BASE		  0x80000000
#define KSEG1BASE		  0xA0000000
#define KSSEGBASE		  0xC0000000
#define KSEG3BASE		  0xE0000000
#define KSEG1(addr)     (((tt32Bit)(addr) & ~KSEG_MSK)  | KSEG1BASE)
#define PHYS(origAddress)   (((tt32Bit)(origAddress)) & ~KSEG_MSK )


/* Default packet buffer size */
#define TM_PKT_BUF_SZ               1518
#define TM_AM79C_FCS_SIZE           0x0004

/* Set info for the number of Tx and Rx buffers. */
#define TM_AM79C_RING_SIZE_RX_NUM   4
#define TM_AM79C_RING_SIZE_RX       (1<<TM_AM79C_RING_SIZE_RX_NUM)
#define TM_AM79C_RING_MOD_MASK_RX   (TM_AM79C_RING_SIZE_RX - 1)
#define TM_AM79C_RING_LEN_BITS_RX   (TM_AM79C_RING_SIZE_RX_NUM<<4)

#define TM_AM79C_RING_SIZE_TX_NUM   4
#define TM_AM79C_RING_SIZE_TX       (1<<TM_AM79C_RING_SIZE_TX_NUM)
#define TM_AM79C_RING_MOD_MASK_TX   (TM_AM79C_RING_SIZE_TX - 1)
#define TM_AM79C_RING_LEN_BITS_TX   (TM_AM79C_RING_SIZE_TX_NUM<<12)


/* Offsets from base I/O address. These offsets can be found on page
 */
#define TM_AM79C_ADDRESS            0x00    
#define TM_AM79C_W_RDP              0x10
#define TM_AM79C_W_RAP              0x14
#define TM_AM79C_W_RESET            0x18   
#define TM_AM79C_W_BDP              0x1c   
                                     

/* Initialization for AM79C chip */
#define TM_AM79C_INIT_MODE      0x0000    /* Standard initialization mode */

/* CSR Offsets */
#define TM_AM79C_CSR0           0x0000    /* CSR0 Controller Status Register */
#define TM_AM79C_CSR1           0x0001    /* CSR1 Lower Address of Init Block */
#define TM_AM79C_CSR2           0x0002    /* CSR2 Upper Address of Init Block */
#define TM_AM79C_CSR3           0x0003    /* CSR3    Interrupt Masks and
                                           *         Deferral Control */
#define TM_AM79C_CSR4           0x0004    /* CSR4    Test and Features Control
                                           *         Control */
#define TM_AM79C_CSR5           0x0005    /* CSR4    Test and Features Control
                                           *         Control */

#define TM_AM79C_CSR58          0x003A    /* Software Style mapped to BCR20 */

#define TM_AM79C_CSR80          0x0050    

#define TM_AM79C_CSR88          0x0058    /* CSR88 Chip ID */

/* Bit offsets for CSR0 */
#define TM_AM79C_CSR0_ERR       0x8000    /* Remains true as long as any error
                                           * flags are true */
#define TM_AM79C_CSR0_BABL      0x4000    /* Babble error */
#define TM_AM79C_CSR0_CERR      0x2000    /* Collision error */
#define TM_AM79C_CSR0_MISS      0x1000    /* Missed Frame error */
#define TM_AM79C_CSR0_MERR      0x0800    /* Memory error */
#define TM_AM79C_CSR0_RINT      0x0400    /* Receive Interrupt */
#define TM_AM79C_CSR0_TINT      0x0200    /* Transmit Interrupt */
#define TM_AM79C_CSR0_IDON      0x0100    /* Initialization Done Interrupt */
#define TM_AM79C_CSR0_INTR      0x0080    /* Interrupt flag - indicates one of
                                           * the interrupts is set */
#define TM_AM79C_CSR0_IENA      0x0040    /* Interrupt Enable */
#define TM_AM79C_CSR0_RXON      0x0020    /* Receive On */
#define TM_AM79C_CSR0_TXON      0x0010    /* Transmit On */
#define TM_AM79C_CSR0_TDMD      0x0008    /* Transmit Demand */
#define TM_AM79C_CSR0_STOP      0x0004    /* Stop the chip */
#define TM_AM79C_CSR0_STRT      0x0002    /* Start the chip */
#define TM_AM79C_CSR0_INIT      0x0001    /* Initialize the chip */

/* Bit offsets for CSR3 */
#define TM_AM79C_CSR3_BABLM     0x4000    /* Babble Interrupt Mask */
#define TM_AM79C_CSR3_MISSM     0x1000    /* Missed Frame Interrupt Mask */
#define TM_AM79C_CSR3_MERRM     0x0800    /* Memory Error Interrupt Mask */
#define TM_AM79C_CSR3_RINTM     0x0400    /* Receive Interrupt Mask */
#define TM_AM79C_CSR3_TINTM     0x0200    /* Transmit Interrupt Mask */
#define TM_AM79C_CSR3_IDONM     0x0100    /* Initialization Done Mask */
#define TM_AM79C_CSR3_DXSUFLO   0x0040    /* Disable transmit stop on underflow
                                           * error */
#define TM_AM79C_CSR3_LAPPEN    0x0020    /* Look Ahead Packet Processing */
#define TM_AM79C_CSR3_DXMT2PD   0x0010    /* Disable transmit two part
                                           * deferral */
#define TM_AM79C_CSR3_EMBA      0x0008    /* Enable modified backoff
                                           * algorithm */

/* Bit offsets for CSR4 */
#define TM_AM79C_CSR4_ENTEST    0x8000    /* Enable test mode */
#define TM_AM79C_CSR4_DMAPLUS   0x4000    /* Disable burst transaction counter
                                           * in CSR80 */
#define TM_AM79C_CSR4_TIMER     0x2000    /* enable Bus Activity Timer
                                           * register */
#define TM_AM79C_CSR4_DPOLL     0x1000    /* Disable transmit polling */
#define TM_AM79C_CSR4_APAD_XMT  0x0800    /* Autopad transmit */
#define TM_AM79C_CSR4_ASTRP_RCV 0x0400    /* Autostrip receives */
#define TM_AM79C_CSR4_MFCO      0x0200    /* Missed frame count overflow
                                           * interrupt */
#define TM_AM79C_CSR4_MFCOM     0x0100    /* Missed frame count overflow mask */
#define TM_AM79C_CSR4_RCVCCO    0x0020    /* Receive Collision Counter overflow
                                           * interrupt */
#define TM_AM79C_CSR4_RCVCCOM   0x0010    /* Receive Collision counter overflow
                                           * mask */
#define TM_AM79C_CSR4_TXSTRT    0x0008    /* Transmit start interrupt */
#define TM_AM79C_CSR4_TXSTRTM   0x0004    /* Transmit start mask */
#define TM_AM79C_CSR4_JAB       0x0002    /* Jabber error interrupt */
#define TM_AM79C_CSR4_JABM      0x0001    /* Jabber error mask */

/* BCR 20 for software style */
#define TM_AM79C_BCR20          0x0014   

/* BCR registers, value expected to be present in EEPROM */
#define TM_AM79C_BCR9           0x0009   
#define TM_AM79C_BCR18          0x0012   
#define TM_AM79C_BCR22          0x0016   
#define TM_AM79C_BCR25          0x0019   
#define TM_AM79C_BCR26          0x001A   
#define TM_AM79C_BCR27          0x001B  

/* BCR MII clock */
#define TM_AM79C_BCR32          0x0020  
#define TM_AM79C_BCR33          0x0021 
#define TM_AM79C_BCR34          0x0022 


/* Write out a word for specified port address */
#define tm_out_word(tp_value,tp_port) \
((*( volatile tt32Bit *) ((tt32Bit)am79cInfoPtr->infoDeviceBase + (tp_port))) = (tp_value) & 0xffffUL)

/* Read in a word from specified port address */

#undef tm_in_word
#define tm_in_word(tp_port) \
((*( volatile tt32Bit *)((tt32Bit)am79cInfoPtr->infoDeviceBase + (tp_port))) & 0xffffUL)

#define tm_csr_read( csr, readvar ) \
{ \
    tm_out_word ( csr, TM_AM79C_W_RAP ); \
    readvar = tm_in_word ( TM_AM79C_W_RDP) ; \
}

#define tm_csr_write( csr, writevar ) \
{ \
    tm_out_word ( csr, TM_AM79C_W_RAP );  \
    tm_out_word ( writevar, TM_AM79C_W_RDP ); \
}

#define tm_bcr_read( bcr, readvar ) \
{ \
    tm_out_word ( bcr, TM_AM79C_W_RAP ); \
    readvar = tm_in_word ( TM_AM79C_W_BDP) ; \
}

#define tm_bcr_write( bcr, writevar ) \
{ \
    tm_out_word ( bcr, TM_AM79C_W_RAP );  \
    tm_out_word ( writevar, TM_AM79C_W_BDP ); \
}


#define tm_get_abs_address(origAddress) PHYS(origAddress)

/* Get the upper 8 bits of an absolute address */
#define tm_get_upper8(absAddress) (tt16Bit)((absAddress&0xffff0000UL)>>16)
/* Get the lower 16 bits of an absolute address */
#define tm_get_lower16(absAddress) (tt16Bit)(absAddress&0x0000ffffUL)


#define TM_AM79C_TRUE                 1     /* TRUE */
#define TM_AM79C_FALSE                0     /* FALSE */

#define TM_AM79C_ALIGNMENT_MODIFIER   32     /* Bytes to add to malloc to insure
                                             * 8-byte alignment */
#define TM_AM79C_ALIGNMENT_MOD_MASK   0xffffffe0UL
                                            /* Mask for rounding to 8-byte
                                             * multiple */


#define TM_AM79C_SOFTWARE_OWNS        0x0000UL
                                            /* Bitmap for driver ownership */
#define TM_AM79C_HARDWARE_OWNS        0x8000UL
                                            /* Bitmap for hardware ownership */

#define TM_AM79C_END_OF_PACKET        0x0100UL
                                            /* Bitmap for end-of-packet flag */
#define TM_AM79C_START_OF_PACKET      0x0200UL
                                            /* Bitmap for start-of-packet flag*/

#define TM_AM79C_SET_UPPER_4_BITS     0xf000
                                            /* Bitmap for turning on top 4 bits
                                               in a 16-bit field */

#define TM_AM79C_INFO_FIELD_SHIFT     24    /* Bits to shift address in receive
                                             * buffers to get status byte */

#define TM_AM79C_RECEIVE_ERROR        0x40  /* Bitmap indicating an error in a 
                                             * receive descriptor ring status
                                             * byte */

#define TM_AM79C_PACKET_TYPE_OFFSET_1 12    /* Offset of start of packet-type
                                             * field from beginning of buffer */
#define TM_AM79C_PACKET_TYPE_OFFSET_2 13    /* Offset of end of packet-type
                                             * field from beginning of buffer */
#define TM_AM79C_INVALID_PACKET_TYPE  ((unsigned char)0xFF)
                                            /* An invalid packet type to get the
                                             * stack to discard packets with
                                             * errors in them */

/* The AM79C973 datasheet Rx and Tx ring descriptors. */
typedef struct tsAm79cRxHead 
{
    tt32Bit rxAddress;
    tt16Bit rxBufferBytes;
    tt16Bit rxFlag;

    tt16Bit rxMessageBytes;

    tt16Bit rxRtag;
    tt32Bit rxUser;

} ttAm79cRxHead; 
typedef ttAm79cRxHead TM_FAR * ttAm79cRxHeadPtr;

#define TM_AM79C_RX_HEAD_PTR_NULL (ttAm79cRxHeadPtr)0

typedef struct tsAm79cTxHead
{
    tt32Bit txAddress;

    tt16Bit txBufferBytes;
    tt16Bit txFlag;

    tt16Bit txTrc;

    tt16Bit txErrors;
    tt32Bit txUser;

} ttAm79cTxHead;
typedef ttAm79cTxHead TM_FAR * ttAm79cTxHeadPtr;
#define TM_AM79C_TX_HEAD_PTR_NULL (ttAm79cTxHeadPtr)0

/* The AM79C initialization block, described in databook. */
typedef struct tsAm79cInit 
{
  
/* Initialization Mode to be read into register 15 */
    tt16Bit            initMode;    
    tt16Bit            res1;
/* Physical ethernet address */
    ttEnetAddress      initPhysAddr;    
    tt16Bit            res2;
/* Multicast filter (unused). */
    tt32Bit            initFilter[2];        
/* Receive and transmit ring base, along with extra bits. */
    tt32Bit            initRxRing;        
    tt32Bit            initTxRing;
} ttAm79cInit;
typedef ttAm79cInit TM_FAR * ttAm79cInitPtr;
#define TM_AM79C_INIT_PTR_NULL (ttAm79cInitPtr)0

/* Some accounting info we need to keep about the receive ring */
typedef struct tsAm79cRxRingInfo
{
/* Buffer handle associated with the ring entry */
    ttUserBuffer     rxBufferHandle;
/* segment/offset address of receive buffer */
    char TM_FAR *    rxBufferAddress;
} ttAm79cRxRingInfo;
typedef ttAm79cRxRingInfo TM_FAR * ttAm79cRxRingInfoPtr;
#define TM_AM79C_RX_RING_INFO_PTR_NULL (ttAm79cRxRingInfoPtr)0

typedef struct tsAm79cInfo
{
/* The first 2 entries must be aligned on 8-byte boundaries. */
/* The Receive Descriptor Ring */
    ttAm79cRxHead                infoRxRing[TM_AM79C_RING_SIZE_RX];
/* The Transmit Descriptor Ring */
    ttAm79cTxHead                infoTxRing[TM_AM79C_RING_SIZE_TX];
/* The Initialization Block */
    ttAm79cInit                  infoInitBlock;
/* A corollary array to the receive descriptor ring containing more data */
    ttAm79cRxRingInfo            infoRxRingInfo[TM_AM79C_RING_SIZE_RX];

/* The interface handle */
    int                          infoDeviceIndex;
/* The next free receive ring entry */
    int                          infoCurRx;
/* The current rx ring entry to be sent up */
    int                          infoIsrRx;
/* The current tx ring entry */
    int                          infoCurTx;
/* The ring entries to be free()ed. */
    int                          infoIsrTx;    
    tt32Bit                       infoDeviceBase;
/* Set to 1 if packet is first in a scatter send */
    int                          infoFirstPacket;
    int                          infoXmitQueueOn;
} ttAm79cInfo;
typedef ttAm79cInfo TM_FAR * ttAm79cInfoPtr;
#define TM_AM79C_INFO_PTR_NULL (ttAm79cInfoPtr)0

/* Yamon timer function */
#if 0
static void LAN_AM79C973_wait( tt32Bit millisec )
{
    tt32Bit  first, latest, previous, accumulate ;
    int len; 
    accumulate = 0 ;
    len = sizeof(first);

    /* get millisecond count */
     SYSCON_read( SYSCON_BOARD_GET_MILLISEC_ID,
                      &first,
                      len ) ;
    previous = first ;
    while ( 1 )
    {
/* get millisecond count */
         SYSCON_read( SYSCON_BOARD_GET_MILLISEC_ID,
                          &latest,
                          sizeof(latest) ) ;
        if ( latest >= previous )
        {
/* counter still not wrapped */
            if ( (accumulate + (latest - first)) > millisec )
            {
                break ;
            }
        }
        else
        {
/* counter did wrap */
            accumulate = accumulate + (previous - first) ;
            if ( (accumulate + latest) > millisec )
            {
                break ;
            }
/* reset first */
            first = 0 ;
        }
        /* prepare next delta time */
        previous = latest ;
    }
}

#endif

/*
 * Get the physical address
 * return it to the Treck stack
 * The address of the card needs to be changed - this is just test data
 */
int tfAm79cGetPhysicalAddress(ttUserInterface am79cHandle,
                              char TM_FAR *   physicalAddress)
{
/* This line is to get rid of compiler warnings */
    TM_UNREF_IN_ARG(am79cHandle);

    physicalAddress[0] = (char)0;
    physicalAddress[1] = (char)0xd0;
    physicalAddress[2] = (char)0xa0;
    physicalAddress[3] = (char)0;
    physicalAddress[4] = (char)0x02;
    physicalAddress[5] = (char)0x99;

    return TM_ENOERROR;
}


/*
 * Open the am79c chip and do any necessary initializations 
 */
int tfAm79cOpen(ttUserInterface am79cHandle)
{
    tt32Bit             absAddr;
    char                ethAddress[6];
    ttAm79cInfoPtr      am79cInfoPtr;
    tt32Bit              pci_clk_freq ;
    tt32Bit             temp;

    int                 am79cIndex;
/* Device addresses */
    int                 am79cIsrBase;
    int                 retCode;
    int                 i;
    int                 errorCode;
    
/* defaults to no error */
    errorCode = TM_DEV_OKAY;
    am79cInfoPtr = (ttAm79cInfoPtr)0;

/* Find an unused AM79C parameter structure */
    for(am79cIndex = 0; am79cIndex < TM_MAX_NUMBER_DEVICES; am79cIndex++ )
    {
        if(tvDeviceIsrPara[am79cIndex].deviInterfaceHandle == (ttUserInterface)0)
            break;
    }

    if(am79cIndex >= TM_MAX_NUMBER_DEVICES)
    {
#ifdef TM_ERROR_CHECKING
        tfKernelError("tfAm79cOpen",
           "Device driver parameter full: Increase TM_MAX_NUMBER_DEVICES.");
#endif /* TM_ERROR_CHECKING */
        errorCode = TM_DEV_ERROR;
        goto TL_AM79C_OPEN_END;
    }
    if ( am79cIndex >1 )
    {
#ifdef TM_ERROR_CHECKING
        tfKernelError("tfAm79cOpen",  "should be first two driver");
#endif /* TM_ERROR_CHECKING */
        errorCode = TM_DEV_ERROR;
        goto TL_AM79C_OPEN_END;
    }

/* Get memory for the tsAm79cInfo structure and make sure it is
 * 8-byte aligned */
    am79cInfoPtr = (ttAm79cInfoPtr)tm_get_raw_buffer( sizeof(ttAm79cInfo)
                                           + TM_AM79C_ALIGNMENT_MODIFIER );
    
    if (am79cInfoPtr == TM_AM79C_INFO_PTR_NULL) 
    {
        errorCode = TM_DEV_ERROR;
        goto TL_AM79C_OPEN_END;
    }
    am79cInfoPtr = (ttAm79cInfoPtr)
                     ((((tt32Bit)(am79cInfoPtr)) +
                     (TM_AM79C_ALIGNMENT_MODIFIER-1)) &
                     TM_AM79C_ALIGNMENT_MOD_MASK);
    am79cInfoPtr = (ttAm79cInfoPtr)KSEG1((tt32Bit)am79cInfoPtr);
    tm_bzero ( am79cInfoPtr, sizeof(ttAm79cInfo));

/* Store driver variable to treck stack */
    retCode = tfDeviceStorePointer(am79cHandle, (ttVoidPtr)am79cInfoPtr); 
    if (retCode != TM_ENOERROR)
    {
#ifdef TM_ERROR_CHECKING
        tfKernelError("tfAm79cOpen",\
                        "Device driver already opened: corrupted memory.");
#endif /* TM_ERROR_CHECKING */
        errorCode = TM_DEV_ERROR;
        goto TL_AM79C_OPEN_END;
    }

/* Store the device indext the Info structure */
    am79cInfoPtr->infoDeviceIndex = am79cIndex;
    am79cInfoPtr->infoDeviceBase = TM_AM79C_BASE;
    am79cInfoPtr->infoFirstPacket = TM_AM79C_TRUE;
    am79cInfoPtr->infoXmitQueueOn = tfInterfaceXmitQueueOn(am79cHandle);

/* Store ISR handler parameters */
    tvDeviceIsrPara[am79cIndex].deviInterfaceHandle = am79cHandle;
#ifdef TM_MULTIPLE_CONTEXT
    tvDeviceIsrPara[am79cIndex].deviContextHandle = tfGetCurrentContext();
#endif /* TM_MULTIPLE_CONTEXT */


/* Reset the AM79C */
    tm_out_word(0, TM_AM79C_W_RDP);
    tm_in_word(TM_AM79C_W_RESET);
/* wait 10ms */
/*
    LAN_AM79C973_wait( 10 ) ; 
*/
    for (i=0;i<1000;i++);

    tm_csr_write ( 0, TM_AM79C_CSR0);

/* set 32bit mode */    
    tm_bcr_write ( TM_AM79C_BCR20, 2 ); 


/* Install tfAm79cIsrHandler on INT0 (interrupt type 10) in the IVT
 * Change this line to whatever your interrupt type is for your processor
 */
    switch(am79cIndex)
    {
        case 0:
            tfKernelInstallIsrHandler((ttUserIsrHandlerPtr)tfAm79cIsrHandler0,
                                       TM_AM79C_INTERRUPT_TYPE);
            break;

        case 1:
            tfKernelInstallIsrHandler((ttUserIsrHandlerPtr)tfAm79cIsrHandler1,
                                      TM_AM79C_INTERRUPT_TYPE);
            break;
        default:
            break;
    }


/* Initialize the AM79C Rx and Tx rings. */
    am79cInfoPtr->infoCurRx = 0;
    am79cInfoPtr->infoIsrRx = 0;
    am79cInfoPtr->infoCurTx = 0;
    am79cInfoPtr->infoIsrTx = 0;

/* Initialize the receive ring */
    for (i = 0; i < TM_AM79C_RING_SIZE_RX; i++)
    {
         am79cInfoPtr->infoRxRingInfo[i].rxBufferAddress =
            tm_malloc(TM_PKT_BUF_SZ + TM_AM79C_ALIGNMENT_MODIFIER);
/* Make sure we got back a valid data pointer. If not, return an error */
        if (am79cInfoPtr->infoRxRingInfo[i].rxBufferAddress ==
            (char TM_FAR *)0)
        {
            errorCode = TM_DEV_ERROR;
            goto TL_AM79C_OPEN_END;
        }
        am79cInfoPtr->infoRxRingInfo[i].rxBufferAddress =(char TM_FAR *)KSEG1( (char TM_FAR*)
            (( (tt32Bit)am79cInfoPtr->infoRxRingInfo[i].rxBufferAddress +
                TM_AM79C_ALIGNMENT_MODIFIER-1) & TM_AM79C_ALIGNMENT_MOD_MASK));
                
/* The ORing in this line is to set ownership of the buffer to the chip */
        am79cInfoPtr->infoRxRing[i].rxAddress =
            tm_get_abs_address(
            am79cInfoPtr->infoRxRingInfo[i].rxBufferAddress);

        am79cInfoPtr->infoRxRing[i].rxAddress = 
                        tm_cpu_to_le(am79cInfoPtr->infoRxRing[i].rxAddress); 

/* Upper 4 bits must be 1 */
        temp = ((tt32Bit)(TM_AM79C_HARDWARE_OWNS<<16))|((tt16Bit)(TM_AM79C_SET_UPPER_4_BITS | (-TM_PKT_BUF_SZ)));
        *((tt32Bit *)&am79cInfoPtr->infoRxRing[i].rxBufferBytes) = 
                                                    tm_cpu_to_le(temp);
    }

/* Initialize the transmit ring */
    for (i = 0; i < TM_AM79C_RING_SIZE_TX; i++)
    {
        am79cInfoPtr->infoTxRing[i].txAddress = 0;
        am79cInfoPtr->infoTxRing[i].txFlag = TM_AM79C_SOFTWARE_OWNS;
        am79cInfoPtr->infoTxRing[i].txBufferBytes = 0;
#if 0
        am79cInfoPtr->infoTxRing[i].txAddress =
            (tt32Bit)tm_malloc(TM_PKT_BUF_SZ + TM_AM79C_ALIGNMENT_MODIFIER);
/* Make sure we got back a valid data pointer. If not, return an error */
        if (am79cInfoPtr->infoTxRing[i].txAddress ==
            (tt32Bit)0)
        {
            errorCode = TM_DEV_ERROR;
            printf(" Error in alloc Txaddress\n");
            goto TL_AM79C_OPEN_END;
        }
        am79cInfoPtr->infoTxRing[i].txAddress =(tt32Bit)KSEG1( (tt32Bit)
            (( (tt32Bit)am79cInfoPtr->infoTxRing[i].txAddress +
                TM_AM79C_ALIGNMENT_MODIFIER-1) & TM_AM79C_ALIGNMENT_MOD_MASK));
                
/* The ORing in this line is to set ownership of the buffer to the chip */
        am79cInfoPtr->infoTxRing[i].txAddress =
            tm_get_abs_address(
            am79cInfoPtr->infoTxRing[i].txAddress);

        am79cInfoPtr->infoTxRing[i].txAddress = 
                        tm_cpu_to_le(am79cInfoPtr->infoTxRing[i].txAddress); 

#endif 
    }

/* Set up the ethernet addr */
    (void)tfAm79cGetPhysicalAddress(am79cHandle,ethAddress);
    for (i = 0; i < 6; i++)
    {
        am79cInfoPtr->infoInitBlock.initPhysAddr[i] = (tt8Bit)ethAddress[i];   
    }
    temp =*((tt32Bit *)&am79cInfoPtr->infoInitBlock.initPhysAddr[0]) ;
   //                                         tm_cpu_to_le(temp);
    temp =*((tt32Bit *)&am79cInfoPtr->infoInitBlock.initPhysAddr[4]) ;
    
/* Set the initialization mode */
    am79cInfoPtr->infoInitBlock.initMode = TM_AM79C_INIT_MODE;
/* Set Portsel */
    am79cInfoPtr->infoInitBlock.initMode |= 0x8180;

    am79cInfoPtr->infoInitBlock.res1 = TM_AM79C_RING_LEN_BITS_RX;
    am79cInfoPtr->infoInitBlock.res1 |=TM_AM79C_RING_LEN_BITS_TX;

    temp =*((tt32Bit *)&am79cInfoPtr->infoInitBlock.initMode) ;

    *((tt32Bit *)&am79cInfoPtr->infoInitBlock.initMode) = 
                                            tm_cpu_to_le16(temp);

//    *((tt32Bit *)&am79cInfoPtr->infoInitBlock.initMode) = 0x80810000; 

/* We don't do multicast, so set Logical Address Filters to 0 */
    am79cInfoPtr->infoInitBlock.initFilter[0] = 0x0UL;
    am79cInfoPtr->infoInitBlock.initFilter[1] = 0x0UL;

/* Set the address and length of receive descriptor ring */
    am79cInfoPtr->infoInitBlock.initRxRing = 
            tm_get_abs_address(am79cInfoPtr->infoRxRing);
    am79cInfoPtr->infoInitBlock.initTxRing = 
            tm_get_abs_address(am79cInfoPtr->infoTxRing);

    am79cInfoPtr->infoInitBlock.initRxRing = 
            tm_cpu_to_le(am79cInfoPtr->infoInitBlock.initRxRing);
    am79cInfoPtr->infoInitBlock.initTxRing = 
            tm_cpu_to_le( am79cInfoPtr->infoInitBlock.initTxRing); 
    am79cInfoPtr->infoInitBlock.res2 = 0x0;

/* Write the absolute address of the initialization block into appropriate
 * registers
 */
    absAddr = tm_get_abs_address(&(am79cInfoPtr->infoInitBlock));

    tm_out_word(TM_AM79C_CSR1, TM_AM79C_W_RAP);
    tm_out_word(tm_get_lower16(absAddr), TM_AM79C_W_RDP);

    tm_out_word(TM_AM79C_CSR2, TM_AM79C_W_RAP);
    tm_out_word(tm_get_upper8(absAddr), TM_AM79C_W_RDP);

    tm_bcr_write (TM_AM79C_BCR9, 0);
    tm_bcr_write (TM_AM79C_BCR18, 0x9ae1);
    tm_bcr_write (TM_AM79C_BCR22, 0x1818);
    tm_bcr_write (TM_AM79C_BCR25, 0x17);
    tm_bcr_write (TM_AM79C_BCR26, 0x8);
    tm_bcr_write (TM_AM79C_BCR27, 0x0);

    temp = 0;
    tm_bcr_read ( TM_AM79C_BCR18, temp);
    temp = temp | (1<<11);
    tm_bcr_write ( TM_AM79C_BCR18, temp);
    tm_csr_read ( TM_AM79C_CSR80, temp);
    temp = temp | (3<<10);
    tm_csr_write ( TM_AM79C_CSR80, temp);


/* Tell chip to read in the initialization block */
    tm_csr_write(TM_AM79C_CSR0, TM_AM79C_CSR0_INIT); 
    
/* Wait until chip has initialized */

    while (!(tm_in_word(TM_AM79C_W_RDP) & TM_AM79C_CSR0_IDON))
    {
        ;
    }

    tm_csr_write ( TM_AM79C_CSR0, TM_AM79C_CSR0_STRT); 
 
    tm_csr_write ( TM_AM79C_CSR4, 0x914);
    tm_csr_write ( TM_AM79C_CSR3, ( 1<<9 ));
    tm_csr_write ( TM_AM79C_CSR5, ( 1<<15 ));
    
    tm_csr_write ( TM_AM79C_CSR0, (1<<6));
    
    
TL_AM79C_OPEN_END:
    if (errorCode != TM_DEV_OKAY && am79cInfoPtr != (ttAm79cInfoPtr)0)
    {
        tm_free_raw_buffer((ttVoidPtr)am79cInfoPtr);
    }

    return(errorCode);            
}


/* Close the AM79C chip */
int tfAm79cClose(ttUserInterface interfaceHandle)
{
    int             i;
    int             am79cIndex;
    int             errorCode;
    ttAm79cInfoPtr  am79cInfoPtr;

    errorCode = TM_DEV_OKAY;
/* Deal with 'unused variable' compiler warnings */
    TM_UNREF_IN_ARG(interfaceHandle);

/* Get the chip info pointer */
    am79cInfoPtr = tm_device_get_pointer(interfaceHandle);
#ifdef TM_ERROR_CHECKING
    if( am79cInfoPtr == TM_AM79C_INFO_PTR_NULL )
    {
        tfKernelError("tfAm79cClose",\
                        "Device driver already closed: corrupted memory.");
        errorCode = TM_DEV_ERROR;
    }
    else
#endif /* TM_ERROR_CHECKING */
    {
        am79cIndex = am79cInfoPtr->infoDeviceIndex;
        /* Stop the chip */
        tm_out_word(TM_AM79C_CSR0, TM_AM79C_W_RAP);
        tm_out_word(TM_AM79C_CSR0_STOP, TM_AM79C_W_RDP);

/* Free ISR handler parameters structure */
        tvDeviceIsrPara[am79cIndex].deviInterfaceHandle = 
                                                        (ttUserInterface)0;

/* Free any buffers the driver is still holding on to */
        for (i = 0; i < TM_AM79C_RING_SIZE_RX; i++)
        {
            if (am79cInfoPtr->infoRxRing[i].rxAddress != 0)
            {
                  (void)tfFreeDriverBuffer(
                        am79cInfoPtr->infoRxRingInfo[i].rxBufferHandle);
            
                tm_free(am79cInfoPtr->infoRxRingInfo[i].rxBufferAddress);                    
            }
        }
        
/* Free device driver variable */
        am79cInfoPtr = tfDeviceClearPointer(interfaceHandle);
        if ( am79cInfoPtr != TM_AM79C_INFO_PTR_NULL )
        {
/* Free malloc'd memory */
            tm_free_raw_buffer((void TM_FAR *)am79cInfoPtr);
        }
#ifdef TM_ERROR_CHECKING
        else
        {
            tfKernelError("tfAm79cClose","Fail to clear device driver ptr");
            errorCode = TM_DEV_ERROR;
        }
#endif /* TM_ERROR_CHECKING */
    }

    return errorCode;
}


/* This function is responsible for handing packets-to-be-sent over to the
 * hardware. It first waits (if necessary) until there are descriptor entries
 * that are unused by the hardware, then calls sendcompletes on sent packets,
 * and then sends out any pending packets. This function is made a bit more
 * complicated because of scatter sends. It is not a good idea to start sending
 * scattered data before you've been given all of the data. Thus, the function
 * does not turn control over to the chip until it has received ALL the blocks
 * in a scattered packet.
 */
int tfAm79cSend ( ttUserInterface am79cHandle,
                  char TM_FAR *   dataPtr,
                  int             dataLength,
                  int             flag )
{

    int             entry;
    int             i;
    tt16Bit         txInfo;
    tt16Bit         tFlag;
    tt32Bit         temp;
    int             errorCode;
    char TM_FAR *   tempPtr;
    ttAm79cInfoPtr  am79cInfoPtr;
    char            temp2[80];
/* Define device base address macro */

/* Get the chip info pointer */
    am79cInfoPtr = tm_device_get_pointer(am79cHandle);
#ifdef TM_ERROR_CHECKING
    if( am79cInfoPtr == TM_AM79C_INFO_PTR_NULL )
    {
        tfKernelError("tfAm79cSend",\
                        "Device driver already closed: corrupted memory.");
        errorCode = TM_DEV_ERROR;
        goto am79cSendExit;
    }
#endif /* TM_ERROR_CHECKING */
/* Wait until the hardware has freed up some descriptor ring entries */
    errorCode = TM_DEV_OKAY;

    temp = *((tt32Bit *)&am79cInfoPtr->infoTxRing[am79cInfoPtr->infoCurTx].txBufferBytes); 
    temp = tm_le_to_cpu(temp);
    tFlag = (tt16Bit)(temp>>16);
 
    if (    (tFlag
           & TM_AM79C_HARDWARE_OWNS ) != TM_AM79C_FALSE )
    {
        if (am79cInfoPtr->infoXmitQueueOn)
        {
/* If user uses a transmit queue, queue the buffer in the transmit queue */
            errorCode = TM_DEV_ERROR;
            tfKernelWarning("tfAm79cSend",\
                        "Xmit queue.");
            goto am79cSendExit;
        }
        else
        {
            do
            {
/*
 * Do nothing function if user does not use transmit task. Otherwise will
 * yield the CPU.
 */
               tfKernelWarning("tfAm79cSend",\
                        "spin lock.!!!!!!!!!!!!!!");
            goto am79cSendExit;
                (void)tfInterfaceSpinLock(am79cHandle);
                temp = *((tt32Bit *)&am79cInfoPtr->infoTxRing[am79cInfoPtr->infoCurTx].txBufferBytes); 
                temp = tm_le_to_cpu(temp);
                tFlag = (tt16Bit)(temp>>16);
            }
            while 
           (  (tFlag& TM_AM79C_HARDWARE_OWNS ) != TM_AM79C_FALSE );
        }
    }
/* Call send complete on dirty packets. Only call send complete once per
 * scattered packet. This loop runs while the address isn't set to 0 (to keep
 * us from looping) and the hardware doesn't own the buffer in question
 */
    i = am79cInfoPtr->infoIsrTx;
    temp = *((tt32Bit *)&am79cInfoPtr->infoTxRing[i].txBufferBytes); 
    temp = tm_le_to_cpu(temp);
    tFlag = (tt16Bit)(temp>>16);

    while ((am79cInfoPtr->infoTxRing[i].txAddress !=0)
            &&(tFlag  &  TM_AM79C_HARDWARE_OWNS) == TM_AM79C_FALSE)
    {
/* Make sure the entry has its end-of-packet bit set. This ensures send complete
 * is only called once per packet.
 */
        if (tFlag & TM_AM79C_END_OF_PACKET)
        {       
            tfSendCompleteInterface(am79cHandle,TM_DEV_SEND_COMPLETE_DRIVER);
            am79cInfoPtr->infoTxRing[i].txBufferBytes = 0;
            am79cInfoPtr->infoTxRing[i].txFlag = 0;
        }
        am79cInfoPtr->infoTxRing[i].txAddress = 0;
        i = (i+1) & TM_AM79C_RING_MOD_MASK_TX;
        if ( i == am79cInfoPtr->infoIsrTx)
            break;       
        temp = *((tt32Bit *)&am79cInfoPtr->infoTxRing[i].txBufferBytes); 
        temp = tm_le_to_cpu(temp);
        tFlag = (tt16Bit)(temp>>16);
    }
    
    
/* Update dirty 'pointer' */
    am79cInfoPtr->infoIsrTx = i;
 
/* Get the entry to be sent */
    entry = am79cInfoPtr->infoCurTx;
    am79cInfoPtr->infoCurTx++;
    am79cInfoPtr->infoCurTx &= TM_AM79C_RING_MOD_MASK_TX;

/* Set the necessary fields in the transmit descriptor
 * Caution: the write order is important here.
 */
    am79cInfoPtr->infoTxRing[entry].txErrors = 0;
    am79cInfoPtr->infoTxRing[entry].txTrc = 0;
    am79cInfoPtr->infoTxRing[entry].txFlag = 0;
   
/* Make sure we got back a valid data pointer. If not, return an error */
/*    tempPtr = (char TM_FAR *)KSEG1((tt32Bit)am79cInfoPtr->infoTxRing[entry].txAddress);
    tm_bcopy( dataPtr,tempPtr,dataLength);
*/

    am79cInfoPtr->infoTxRing[entry].txAddress =
            tm_cpu_to_le( tm_get_abs_address(dataPtr));

/* If this is the last buffer in a packet... */
    if ((flag & TM_USER_BUFFER_LAST) != TM_AM79C_FALSE)
    {
/* and also the first packet... */
        if (am79cInfoPtr->infoFirstPacket == TM_AM79C_TRUE)
        {
/* Then set start-of-packet and end-of-packet bits, and set ownership to
 * hardware
 */
            txInfo = TM_AM79C_HARDWARE_OWNS |
                     TM_AM79C_START_OF_PACKET |
                     TM_AM79C_END_OF_PACKET;
        }
/* else it is not the first packet */
        else
        {
/* So just set the end-of-packet bit and ownership bit */
            txInfo = TM_AM79C_HARDWARE_OWNS |
                     TM_AM79C_END_OF_PACKET;
        }
/* Reset the isFirstPacket flag */
        am79cInfoPtr->infoFirstPacket = TM_AM79C_TRUE;
    }
/* Else it is not the last buffer in a packet... */
    else
    {
/* If it's the first packet... */
        if (am79cInfoPtr->infoFirstPacket == TM_AM79C_TRUE)
        {
/* Set first packet and ownership bits. Also turn the isFirstPacket flag off */
            am79cInfoPtr->infoFirstPacket = TM_AM79C_FALSE;
            txInfo = TM_AM79C_HARDWARE_OWNS |
                     TM_AM79C_START_OF_PACKET;
        }
        else
        {
/* Else it's neither start or end of packet. Just set ownership bit */
            txInfo = TM_AM79C_HARDWARE_OWNS;
        }
    }
/* Store the appropriate ownership & packet info bits in the correct field */
    temp = ((tt32Bit)(txInfo<<16))|( (tt16Bit)(TM_AM79C_SET_UPPER_4_BITS | (-dataLength)));      
    *((tt32Bit*)&am79cInfoPtr->infoTxRing[entry].txBufferBytes) = 
                                                tm_cpu_to_le(temp);



/* At this point, am79cInfoPtr->infoFirstPacket will have been reset in the
 * above 'if' structure. Thus, if it has been reset to true, that means we 
 * must have encountered a last packet above, and should send out all of the
 * waiting blocks
 */
    if (am79cInfoPtr->infoFirstPacket == TM_AM79C_TRUE)
    {
        tm_csr_write(TM_AM79C_CSR0,0x0048);
    }

am79cSendExit:
    return errorCode;
}


/* Pass packet received by hardware up the stack, and mark it as
 * having been dealt with, so that Ioctl knows to refill the buffer.
 * The chip generates an interrupt when a packet is received, which in turn
 * triggers a call to this function.
 */
int tfAm79cReceive(ttUserInterface        interfaceHandle,
                   char TM_FAR * TM_FAR * dataPtr,
                   int  TM_FAR *          dataLength,
                   ttUserBufferPtr        bufHandlePtr)
{
    ttDeviceEntryPtr    devEntryPtr;
    ttPacketPtr         packetPtr;
  
    tt32Bit         temp; 
    tt16Bit         rFlag, rMessageBytes, rBufferBytes;
    int             entry;
    int             status;
    int             errorCode;
    char            temp2[80];
    ttAm79cInfoPtr  am79cInfoPtr;

/* Get the chip info pointer */
    am79cInfoPtr = tm_device_get_pointer(interfaceHandle);
#ifdef TM_ERROR_CHECKING
    if( am79cInfoPtr == TM_AM79C_INFO_PTR_NULL )
    {
        tfKernelError("tfAm79cReceive",\
                        "Null Device Driver Pointer: corrupted memory.");
        errorCode = TM_DEV_ERROR;
    }
    else
#endif /* TM_ERROR_CHECKING */
    {
        devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    
        packetPtr = tfGetSharedBuffer(
                                 TM_LL_MAX_HDR_LEN + TM_ETHER_HW_ALIGN,
                                 (ttPktLen)(TM_PKT_BUF_SZ + TM_ETHER_IP_ALIGN),
                                 TM_16BIT_ZERO );
        if ( packetPtr == TM_PACKET_NULL_PTR)
        {
            errorCode = TM_ENOBUFS;
            tfKernelError("tfAm79cReceive",\
                        "no bufs.");
            return errorCode;
        }
        if (tm_ll_need_32bit_align(devEntryPtr))
        {
            if (   ((tt32Bit)packetPtr->pktLinkDataPtr)
                & (TM_ETHER_HW_ALIGN - 1) )
            {
/* Align the data area (according to TM_ETHER_HW_ALIGN) */
                packetPtr->pktLinkDataPtr = (tt8Bit TM_FAR *)
                             (    (tt32Bit)packetPtr->pktLinkDataPtr
                               & ~(TM_ETHER_HW_ALIGN - 1) );
            }
/*
 * Offset of ethernet frame to allow IP area to be long word aligned.
 */
            packetPtr->pktLinkDataPtr += TM_ETHER_IP_ALIGN;
        }
 

/* Get the current entry */
        entry = am79cInfoPtr->infoCurRx & TM_AM79C_RING_MOD_MASK_RX;
/* Make sure that the packet belongs to us, otherwise return an error */
        temp = *((tt32Bit *)&am79cInfoPtr->infoRxRing[entry].rxBufferBytes); 
        temp = tm_le_to_cpu(temp);
        rFlag = (tt16Bit)(temp>>16);
        temp = *((tt32Bit *)&am79cInfoPtr->infoRxRing[entry].rxMessageBytes); 
        rMessageBytes = (tt16Bit)((tm_le_to_cpu(temp)&0x0000ffffl));

        if ((rFlag & \
                            TM_AM79C_HARDWARE_OWNS) == TM_AM79C_FALSE)

        {
/* Set the passed-in variables to appropriate values */
            *dataLength = (rMessageBytes & 0x0fff) -
                            TM_AM79C_FCS_SIZE;
            packetPtr->pktUserFlags = TM_PKTF_USER_DEV_OWNS;

            tm_bcopy(am79cInfoPtr->infoRxRingInfo[entry].rxBufferAddress,
                        packetPtr->pktLinkDataPtr,
                        *dataLength);
            *dataPtr = packetPtr->pktLinkDataPtr;
            *bufHandlePtr = packetPtr;

/* Check if there was an error in receive */
            if (status & TM_AM79C_RECEIVE_ERROR)
            {    
/* The 12th and 13th bytes of an ethernet packet indicate type. If we set
 * them to something invalid (0xffff in this case), the stack will simply
 * discard the packet
 */
                *((*dataPtr)+TM_AM79C_PACKET_TYPE_OFFSET_1) =
                                          (char)TM_AM79C_INVALID_PACKET_TYPE;
                *((*dataPtr)+TM_AM79C_PACKET_TYPE_OFFSET_2) = 
                                           (char)TM_AM79C_INVALID_PACKET_TYPE;
            }
/* Set the address in the descriptor ring to 0, so Ioctl knows to clean it up */
            rFlag = TM_AM79C_HARDWARE_OWNS;
            rBufferBytes = (tt16Bit)(TM_AM79C_SET_UPPER_4_BITS | (-TM_PKT_BUF_SZ));
            rMessageBytes = 0;

            errorCode = TM_DEV_OKAY;
        }
        else
        {
            tfKernelError("tfAm79cReceive",\
                        "wrong flag.");
            tfFreeSharedBuffer(packetPtr, TM_SOCKET_UNLOCKED); 
            errorCode = TM_DEV_ERROR;
            return errorCode;
        }
        temp =(tt32Bit) (((tt32Bit)(rFlag <<16))|rBufferBytes);
        *((tt32Bit*)&am79cInfoPtr->infoRxRing[entry].rxBufferBytes) =
                                                tm_cpu_to_le(temp);
        temp = (tt32Bit)(rMessageBytes&0x0000ffffl);
        *((tt32Bit*)&am79cInfoPtr->infoRxRing[entry].rxMessageBytes) =
                                                tm_cpu_to_le(temp);
        if ( errorCode == TM_DEV_OKAY)
        {
            entry = (entry+1) & TM_AM79C_RING_MOD_MASK_RX;
        } 
/* Update the current entry */
        am79cInfoPtr->infoCurRx = entry;
    }
    return(errorCode);
}


/* 
 * The AM79C interrupt handler. This is only called on a receive-complete
 * interrupt.
 */
void tfAm79cIsrHandler (ttDeviceIsrParaPtr isrParaPtr)
{
    int              tlCsr0;
    int              tlRap;
    ttAm79cRxHeadPtr ringPtr;
    int              i;
    int              recvNotifyCount;
    ttAm79cInfoPtr   am79cInfoPtr;
    ttUserInterface  am79cHandle;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext    saveContextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tt32Bit          temp;
    tt16Bit          rFlag;
    tt32Bit          intstat;

    am79cHandle = (ttUserInterface)(isrParaPtr->deviInterfaceHandle);
#ifdef TM_MULTIPLE_CONTEXT
    saveContextHandle = tfGetCurrentContext();
    tfSetCurrentContext(isrParaPtr->deviContextHandle);
#endif /* TM_MULTIPLE_CONTEXT */
    am79cInfoPtr = tm_device_get_pointer(am79cHandle);
    recvNotifyCount = 0;

    if ( am79cInfoPtr == (ttAm79cInfoPtr)0)
    {
#ifdef TM_ERROR_CHECKING
        tfKernelError("tfAm79cIsrhandler", "Null Device Driver.");
#endif /* TM_ERROR_CHECKING */
        return;
    }

    i = am79cInfoPtr->infoIsrRx;
    ringPtr = am79cInfoPtr->infoRxRing;
/* call NotifyRecieve for each received packet we haven't dealt with yet */
    temp = *((tt32Bit *)&ringPtr[i].rxBufferBytes);    
    temp = tm_le_to_cpu(temp);
    rFlag = (tt16Bit)(temp>>16);
/* new add for ISR */
    tm_csr_read(0,intstat);
    tm_csr_write( 0, ((intstat & ~0x004f) & 0xffff) );

    for (;;)
    {
        if ((rFlag & (TM_AM79C_HARDWARE_OWNS | TM_AM79C_END_OF_PACKET)) == TM_AM79C_END_OF_PACKET) 
        {
/* Mark ring entry as dealt-with */
            rFlag &= ~TM_AM79C_END_OF_PACKET;  
            temp = ((tt32Bit)(rFlag<<16))|(temp&0x0000ffffl);
            *((tt32Bit *)&ringPtr[i].rxBufferBytes) = tm_cpu_to_le(temp);
/*            YAMON_FUNC_PRINT("recv \n"); */
            recvNotifyCount++;
/*          tfRecvInterface(isrParaPtr->deviInterfaceHandle); */
        }
        else
        {
            if ( recvNotifyCount!= 0)
            {
                break;
            }
        }    
        i = (i+1) & TM_AM79C_RING_MOD_MASK_RX;

        temp = *((tt32Bit *)&ringPtr[i].rxBufferBytes);    
        temp = tm_le_to_cpu(temp);
        rFlag = (tt16Bit)(temp>>16);
        if ( i == am79cInfoPtr->infoIsrRx)
            break;
    }
    
    am79cInfoPtr->infoIsrRx = i;
    
    tm_csr_write(0, 0x7940 );
    
    if ( recvNotifyCount > 0)
        tfNotifyInterfaceIsr( isrParaPtr->deviInterfaceHandle,
                          recvNotifyCount,
                          0, 0UL, 0UL );

#ifdef TM_MULTIPLE_CONTEXT
    tfSetCurrentContext(saveContextHandle);
#endif /* TM_MULTIPLE_CONTEXT */
    return;
}


void tfAm79cIsrHandler0(void)
{
    tfAm79cIsrHandler(&tvDeviceIsrPara[0]);
    return;
}


void tfAm79cIsrHandler1(void)
{
    tfAm79cIsrHandler(&tvDeviceIsrPara[1]);
    return;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:39      For: Xilinx-Samsung
 * File Name: tram79c.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:   269    Output checksum: 36452
 */
