/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/drivers/lance/trlance.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:03CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * This LANCE driver was taken from the GNU sources for LINUX
 * and modified for the Treck TCP/IP protocol stack
 *
 *   Written 1993 by Donald Becker.
 *
 *   Copyright 1993 United States Government as represented by the
 *   Director, National Security Agency.  This software may be used and
 *   distributed according to the terms of the GNU Public License,
 *   incorporated herein by reference.
 *
 * Driver Modified by Paul and Jared at Treck Inc. 1999
 * Modification History
 * Date: $Date: 2010/01/18 06:36:03CST $
 * Author: Jared
 * Version:  $Revision: 6.0.2.2 $
 * Release:  $ProjectRevision: 6.0.1.23 $
 * Description: Driver for LANCE ethernet chips
 */

/*
 * Developed on an AMD Net186 board (AMD PCNet-ISA II Ethernet controller
 * and AM186ES-40VC processor) using Borland BCC 5.0 compiler
 */


/*
 * Include the Treck Interface types
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* This is for writing to and reading from ports in the Borland compiler
 * Change as necessary.
 */
#include <dos.h>
#include "trlance.h"

/* Define PC-LINT compile options */
#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

#ifdef TM_LINT
LINT_UNREF_MACRO(TM_LANCE_PCB_PIO0_MODE)
LINT_UNREF_MACRO(TM_LANCE_PCB_PIO0_DIR)
LINT_UNREF_MACRO(TM_LANCE_PCB_PIO0_DATA)
LINT_UNREF_MACRO(TM_LANCE_ADDRESS)
LINT_UNREF_MACRO(TM_LANCE_TOTAL_SIZE)
LINT_UNREF_MACRO(TM_LANCE_CSR0_ERR)
LINT_UNREF_MACRO(TM_LANCE_CSR0_BABL)
LINT_UNREF_MACRO(TM_LANCE_CSR0_CERR)
LINT_UNREF_MACRO(TM_LANCE_CSR0_MISS)
LINT_UNREF_MACRO(TM_LANCE_CSR0_MERR)
LINT_UNREF_MACRO(TM_LANCE_CSR0_TINT)
LINT_UNREF_MACRO(TM_LANCE_CSR0_INTR)
LINT_UNREF_MACRO(TM_LANCE_CSR0_RXON)
LINT_UNREF_MACRO(TM_LANCE_CSR0_TXON)
LINT_UNREF_MACRO(TM_LANCE_CSR3_RINTM)
LINT_UNREF_MACRO(TM_LANCE_CSR3_LAPPEN)
LINT_UNREF_MACRO(TM_LANCE_CSR3_DXMT2PD)
LINT_UNREF_MACRO(TM_LANCE_CSR3_EMBA)
LINT_UNREF_MACRO(TM_LANCE_CSR4_ENTEST)
LINT_UNREF_MACRO(TM_LANCE_CSR4_TIMER)
LINT_UNREF_MACRO(TM_LANCE_CSR4_APAD_XMT)
LINT_UNREF_MACRO(TM_LANCE_CSR4_MFCO)
LINT_UNREF_MACRO(TM_LANCE_CSR4_RCVCCO)
LINT_UNREF_MACRO(TM_LANCE_CSR4_TXSTRT)
LINT_UNREF_MACRO(TM_LANCE_CSR4_JAB)
LINT_UNREF_MACRO(tm_out_byte)
LINT_UNREF_MACRO(tm_in_byte)
LINT_UNREF_MACRO(TM_LANCE_RX_HEAD_PTR_NULL)
LINT_UNREF_MACRO(TM_LANCE_TX_HEAD_PTR_NULL)
LINT_UNREF_MACRO(TM_LANCE_INIT_PTR_NULL)
LINT_UNREF_MACRO(TM_LANCE_RX_RING_INFO_PTR_NULL)
LINT_UNREF_MACRO(TM_LANCE_ISR_PARA_PTR_NULL)
#endif /* TM_LINT */

/*
 * Processor related header information
 */
#define TM_LANCE_EOI_INT0      0x000c  /* Clears INT0 in the EOI */

#define TM_LANCE_PCB           0xff00  /* Peripheral Control Block Address */
#define TM_LANCE_PCB_INT0      0x0038  /* INT0 Control Register */
#define TM_LANCE_PCB_EOI       0x0022  /* EOI Register to clear interrupts */
#define TM_LANCE_PCB_PIO0_MODE 0x0070  /* PIO0 Mode Register */
#define TM_LANCE_PCB_PIO0_DIR  0x0072  /* PIO0 Direction Register */
#define TM_LANCE_PCB_PIO0_DATA 0x0074  /* PIO0 Data Register */
#define TM_LANCE_INT0_CONFIG   0x0010  /* Standard config for INT0:
                                        * Fully Nested Mode
                                        * Level Sensitive
                                        * Unmasked
                                        * Highest Priority
                                        */


/*
 * LANCE Chip related header information
 */

/* Per the NET186 Evaulation Board */
/* Need to be changed for mulit-context, multi-chip */
#define TM_LANCE_BASE           0x0200

/* 
 * When using more than one Lance device for this device driver, this
 * need to be ported. TM_LANCE_BASE is the base port for the device. DeviceIndex
 * gives the device index in tfOpenInterface() order.
 */
#define tm_lance_base(lanceBaseVar, deviceIndex)             \
{                                                            \
        lanceBaseVar = TM_LANCE_BASE;                        \
}

/* 
 * When using more than one Lance device for this device driver, this
 * need to be ported. TM_LANCE_PCB+TM_LANCE_PCB_INT0 is the base port for 
 * the CPU Isr. DeviceIndex gives the device index in tfOpenInterface() 
 * order.
 */
#define tm_lance_isr_base(lanceIsrBaseVar, deviceIndex)      \
{                                                            \
        lanceIsrBaseVar = TM_LANCE_PCB+TM_LANCE_PCB_INT0;    \
}

/* Default packet buffer size */
#define TM_PKT_BUF_SZ 1518

/* Set info for the number of Tx and Rx buffers. */
#define TM_LANCE_RING_MOD_MASK_RX (TM_LANCE_RING_SIZE_RX - 1)
#if TM_LANCE_RING_SIZE_RX == 1
#define TM_LANCE_RING_LEN_BITS_RX 0x00000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 2
#define TM_LANCE_RING_LEN_BITS_RX 0x20000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 4
#define TM_LANCE_RING_LEN_BITS_RX 0x40000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 8
#define TM_LANCE_RING_LEN_BITS_RX 0x60000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 16
#define TM_LANCE_RING_LEN_BITS_RX 0x80000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 32
#define TM_LANCE_RING_LEN_BITS_RX 0xa0000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 64
#define TM_LANCE_RING_LEN_BITS_RX 0xc0000000UL
#endif
#if TM_LANCE_RING_SIZE_RX == 128
#define TM_LANCE_RING_LEN_BITS_RX 0xe0000000UL
#endif

#define TM_LANCE_RING_MOD_MASK_TX (TM_LANCE_RING_SIZE_TX - 1)
#if TM_LANCE_RING_SIZE_TX == 1
#define TM_LANCE_RING_LEN_BITS_TX 0x00000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 2
#define TM_LANCE_RING_LEN_BITS_TX 0x20000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 4
#define TM_LANCE_RING_LEN_BITS_TX 0x40000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 8
#define TM_LANCE_RING_LEN_BITS_TX 0x60000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 16
#define TM_LANCE_RING_LEN_BITS_TX 0x80000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 32
#define TM_LANCE_RING_LEN_BITS_TX 0xa0000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 64
#define TM_LANCE_RING_LEN_BITS_TX 0xc0000000UL
#endif
#if TM_LANCE_RING_SIZE_TX == 128
#define TM_LANCE_RING_LEN_BITS_TX 0xe0000000UL
#endif

/* Offsets from base I/O address. These offsets can be found on page
 * 125 of the AM79C961A Data Sheet.
 */
#define TM_LANCE_ADDRESS    0x00    /* Hardwired Ethernet hardware address
                                     */
#define TM_LANCE_RDP        0x10    /* Register Data Port. Enter the data to
                                     * be written to the CSR specified in the
                                     * RAP here.
                                     */
#define TM_LANCE_RAP        0x12    /* Register Address Port. Enter CSR (Control
                                     * and Status Register) offsets (found on
                                     * page 123 of the AM79C961A Data Sheet) or
                                     * ISACSR (ISA Bus Configuration Registers)
                                     * offsets (found on page 125) here.
                                     */
#define TM_LANCE_RESET      0x14    /* Reset the chip by reading from this
                                     * memory location
                                     */
#define TM_LANCE_IDP        0x16    /* ISACSR Register Data Port. Enter the
                                     * data to be written to the ISACSR
                                     * specified in the RAP here.
                                     */
#define TM_LANCE_TOTAL_SIZE 0x18    /* The total size of the offset area. This
                                     * doesn't appear to be used (or even
                                     * useful).
                                     */

/* Initialization for LANCE chip */
#define TM_LANCE_INIT_MODE      0x0000    /* Standard initialization mode */

/* CSR Offsets */
#define TM_LANCE_CSR0           0x0000    /* CSR0 Controller Status Register */
#define TM_LANCE_CSR1           0x0001    /* CSR1 Lower Address of Init Block */
#define TM_LANCE_CSR2           0x0002    /* CSR2 Upper Address of Init Block */
#define TM_LANCE_CSR3           0x0003    /* CSR3    Interrupt Masks and
                                           *         Deferral Control */
#define TM_LANCE_CSR4           0x0004    /* CSR4    Test and Features Control
                                           *         Control */
#define TM_LANCE_CSR15          0x000f    /* CSR15   Mode register */
#define TM_LANCE_CSR88          0x0058    /* CSR88 Chip ID */

/* ISACSR Offsets */
#define TM_LANCE_ISACSR2        0x0002    /* Miscellaneous Configuration 1 */

/* Bit offsets for CSR0 */
#define TM_LANCE_CSR0_ERR       0x8000    /* Remains true as long as any error
                                           * flags are true */
#define TM_LANCE_CSR0_BABL      0x4000    /* Babble error */
#define TM_LANCE_CSR0_CERR      0x2000    /* Collision error */
#define TM_LANCE_CSR0_MISS      0x1000    /* Missed Frame error */
#define TM_LANCE_CSR0_MERR      0x0800    /* Memory error */
#define TM_LANCE_CSR0_RINT      0x0400    /* Receive Interrupt */
#define TM_LANCE_CSR0_TINT      0x0200    /* Transmit Interrupt */
#define TM_LANCE_CSR0_IDON      0x0100    /* Initialization Done Interrupt */
#define TM_LANCE_CSR0_INTR      0x0080    /* Interrupt flag - indicates one of
                                           * the interrupts is set */
#define TM_LANCE_CSR0_IENA      0x0040    /* Interrupt Enable */
#define TM_LANCE_CSR0_RXON      0x0020    /* Receive On */
#define TM_LANCE_CSR0_TXON      0x0010    /* Transmit On */
#define TM_LANCE_CSR0_TDMD      0x0008    /* Transmit Demand */
#define TM_LANCE_CSR0_STOP      0x0004    /* Stop the chip */
#define TM_LANCE_CSR0_STRT      0x0002    /* Start the chip */
#define TM_LANCE_CSR0_INIT      0x0001    /* Initialize the chip */

/* Bit offsets for CSR3 */
#define TM_LANCE_CSR3_BABLM     0x4000    /* Babble Interrupt Mask */
#define TM_LANCE_CSR3_MISSM     0x1000    /* Missed Frame Interrupt Mask */
#define TM_LANCE_CSR3_MERRM     0x0800    /* Memory Error Interrupt Mask */
#define TM_LANCE_CSR3_RINTM     0x0400    /* Receive Interrupt Mask */
#define TM_LANCE_CSR3_TINTM     0x0200    /* Transmit Interrupt Mask */
#define TM_LANCE_CSR3_IDONM     0x0100    /* Initialization Done Mask */
#define TM_LANCE_CSR3_DXSUFLO   0x0040    /* Disable transmit stop on underflow
                                           * error */
#define TM_LANCE_CSR3_LAPPEN    0x0020    /* Look Ahead Packet Processing */
#define TM_LANCE_CSR3_DXMT2PD   0x0010    /* Disable transmit two part
                                           * deferral */
#define TM_LANCE_CSR3_EMBA      0x0008    /* Enable modified backoff
                                           * algorithm */

/* Bit offsets for CSR4 */
#define TM_LANCE_CSR4_ENTEST    0x8000    /* Enable test mode */
#define TM_LANCE_CSR4_DMAPLUS   0x4000    /* Disable burst transaction counter
                                           * in CSR80 */
#define TM_LANCE_CSR4_TIMER     0x2000    /* enable Bus Activity Timer
                                           * register */
#define TM_LANCE_CSR4_DPOLL     0x1000    /* Disable transmit polling */
#define TM_LANCE_CSR4_APAD_XMT  0x0800    /* Autopad transmit */
#define TM_LANCE_CSR4_ASTRP_RCV 0x0400    /* Autostrip receives */
#define TM_LANCE_CSR4_MFCO      0x0200    /* Missed frame count overflow
                                           * interrupt */
#define TM_LANCE_CSR4_MFCOM     0x0100    /* Missed frame count overflow mask */
#define TM_LANCE_CSR4_RCVCCO    0x0020    /* Receive Collision Counter overflow
                                           * interrupt */
#define TM_LANCE_CSR4_RCVCCOM   0x0010    /* Receive Collision counter overflow
                                           * mask */
#define TM_LANCE_CSR4_TXSTRT    0x0008    /* Transmit start interrupt */
#define TM_LANCE_CSR4_TXSTRTM   0x0004    /* Transmit start mask */
#define TM_LANCE_CSR4_JAB       0x0002    /* Jabber error interrupt */
#define TM_LANCE_CSR4_JABM      0x0001    /* Jabber error mask */

/* Bit offsets for CSR15 */
#define TM_LANCE_CSR15_PROM     0x8000    /* Promiscuous receive mode */
#define TM_LANCE_CSR15_DRCVBC   0x4000    /* DisableReceive Broadcast */
#define TM_LANCE_CSR15_DRCVPA   0x2000    /* Disable Receive Physical Address*/
#define TM_LANCE_CSR15_DLNKTST  0x1000    /* Disable Link Status */
#define TM_LANCE_CSR15_DAPC     0x0800 /* Disable Automatic Polarity Correctn*/
#define TM_LANCE_CSR15_MENDECL  0x0400    /* MENDEC Loopback Mode */
#define TM_LANCE_CSR15_LRT      0x0200    /* Low Receive Threshold (T-MAU) */
#define TM_LANCE_CSR15_TSEL     0x0200    /* Transmit Mode Select (AUI) */
#define TM_LANCE_CSR15_PORTSEL1 0x0100    /* Port Select */
#define TM_LANCE_CSR15_PORTSEL0 0x0080    /* Port Select */
#define TM_LANCE_CSR15_INTL     0x0040    /* Internal Loopback */
#define TM_LANCE_CSR15_DRTY     0x0020    /* Disable Retry */
#define TM_LANCE_CSR15_FCOLL    0x0010    /* Force Collision */
#define TM_LANCE_CSR15_DXMTFCS  0x0008    /* Disable Transmit CRC (FCS) */
#define TM_LANCE_CSR15_LOOP     0x0004    /* Loopback Enable */
#define TM_LANCE_CSR15_DTX      0x0002    /* Disable Transmit */
#define TM_LANCE_CSR15_DRX      0x0001    /* Disable Receiver */

/* Bit offsets for ISACSR2 */
#define TM_LANCE_ISACSR2_MEDSEL 0x0002    /* Auto-Select Media */

/* Write out a word to specified port address*/
#define tm_out_word(tp_value,tp_port)    outpw(tp_port,tp_value)
/* Write out a byte to specified port address */
#define tm_out_byte(tp_value,tp_port)    outp(tp_port,tp_value)
/* Read in a word from specified port address */
#define tm_in_word  inpw
/* Read in a byte from specified port address */
#define tm_in_byte  inp


/* Translate a segment/offset into an absolute address */
#define tm_get_abs_address(origAddress) \
            ((((tt32Bit)(origAddress)&0xffff0000UL)>>12) + \
            (((tt32Bit)(origAddress))&0x0000ffffUL))

/* Get the upper 8 bits of an absolute address */
#define tm_get_upper8(absAddress) (tt16Bit)((absAddress&0xffffff00UL)>>16)
/* Get the lower 16 bits of an absolute address */
#define tm_get_lower16(absAddress) (tt16Bit)(absAddress&0x0000ffffUL)


#define TM_LANCE_TRUE                 1     /* TRUE */
#define TM_LANCE_FALSE                0     /* FALSE */

#define TM_LANCE_ALIGNMENT_MODIFIER   8     /* Bytes to add to malloc to insure
                                             * 8-byte alignment */
#define TM_LANCE_ALIGNMENT_MOD_MASK   0xfffffff8UL
                                            /* Mask for rounding to 8-byte
                                             * multiple */
#define TM_LANCE_NEW_CHIP             0x3003
                                            /* Chip id for newer LANCE chips */
#define TM_LANCE_UNRESET              0     /* Value to write to chip to
                                             * unreset it */
#define TM_LANCE_INTERRUPT_TYPE       0xc   /* Interrupt type 12 */

#define TM_LANCE_SOFTWARE_OWNS        0x00000000UL
                                            /* Bitmap for driver ownership */
#define TM_LANCE_HARDWARE_OWNS        0x80000000UL
                                            /* Bitmap for hardware ownership */

#define TM_LANCE_END_OF_PACKET        0x01000000UL
                                            /* Bitmap for end-of-packet flag */
#define TM_LANCE_START_OF_PACKET      0x02000000UL
                                            /* Bitmap for start-of-packet flag*/

#define TM_LANCE_SET_UPPER_4_BITS     0xf000
                                            /* Bitmap for turning on top 4 bits
                                               in a 16-bit field */

#define TM_LANCE_INFO_FIELD_SHIFT     24    /* Bits to shift address in receive
                                             * buffers to get status byte */

#define TM_LANCE_RECEIVE_ERROR        0x40  /* Bitmap indicating an error in a 
                                             * receive descriptor ring status
                                             * byte */

#define TM_LANCE_PACKET_TYPE_OFFSET_1 12    /* Offset of start of packet-type
                                             * field from beginning of buffer */
#define TM_LANCE_PACKET_TYPE_OFFSET_2 13    /* Offset of end of packet-type
                                             * field from beginning of buffer */
#define TM_LANCE_INVALID_PACKET_TYPE  ((unsigned char)0xFF)
                                            /* An invalid packet type to get the
                                             * stack to discard packets with
                                             * errors in them */

/* The LANCE Rx and Tx ring descriptors. */
typedef struct tsLanceRxHead 
{
/* The upper 24 bits are the address field, the lower 8 are information about
 * the buffer, including owner (stack or hardware), errors, and start-of-packet
 * and end-of-packet flags
 */
    tt32Bit rxAddress;
/* Length of the buffer pointed to by this descriptor (12 bits). This is
 * expressed as the two's complement of the length of the buffer. The upper 4
 * bits must be 1's.
 */
    tt16Bit rxBufferBytes;
/* Length of the received message (12 bits). The upper 4 bits are reserved
 * and are read as 0
 */
    tt16Bit rxMessageBytes;
} ttLanceRxHead; 
typedef ttLanceRxHead TM_FAR * ttLanceRxHeadPtr;
#define TM_LANCE_RX_HEAD_PTR_NULL (ttLanceRxHeadPtr)0

#ifdef TM_LINT
LINT_UNREF_MEMBER_BEGIN /* PC-LINT compile options */
#endif /* TM_LINT */
typedef struct tsLanceTxHead
{
/* The upper 24 bits are the address field, the lower 8 are information about
 * the buffer, including owner (stack or hardware) and flags controlling the
 * manner of transmission (see Am79c961A Data Sheet pg. 120)
 */
    tt32Bit txAddress;
/* The length of the buffer pointed to by this descriptor (12 bits). This is
 * expressed as the two's complement of the length of the buffer. The upper 4
 * bits must be 1's.
 */
    tt16Bit txBufferBytes;
/* The 6 high bits are each error flags. The 10 lower bits are Time Domain
 * Reflectometry info (used in determining how long after the start of
 * transmission an error occurs).
 */
    tt16Bit txErrors;
} ttLanceTxHead;
typedef ttLanceTxHead TM_FAR * ttLanceTxHeadPtr;
#define TM_LANCE_TX_HEAD_PTR_NULL (ttLanceTxHeadPtr)0

/* The LANCE initialization block, described in databook. */
typedef struct tsLanceInit 
{
/* Initialization Mode to be read into register 15 */
    tt16Bit            initMode;    
/* Physical ethernet address */
    ttEnetAddress      initPhysAddr;    
/* Multicast filter (unused). */
    tt32Bit            initFilter[2];        
/* Receive and transmit ring base, along with extra bits. */
    tt32Bit            initRxRing;        
    tt32Bit            initTxRing;
} ttLanceInit;
typedef ttLanceInit TM_FAR * ttLanceInitPtr;
#define TM_LANCE_INIT_PTR_NULL (ttLanceInitPtr)0

/* Some accounting info we need to keep about the receive ring */
typedef struct tsLanceRxRingInfo
{
/* Buffer handle associated with the ring entry */
    ttUserBuffer     rxBufferHandle;
/* segment/offset address of receive buffer */
    char TM_FAR *    rxBufferAddress;
} ttLanceRxRingInfo;
typedef ttLanceRxRingInfo TM_FAR * ttLanceRxRingInfoPtr;
#define TM_LANCE_RX_RING_INFO_PTR_NULL (ttLanceRxRingInfoPtr)0


typedef struct tsLanceInfo
{
/* The first 2 entries must be aligned on 8-byte boundaries. */
/* The Receive Descriptor Ring */
    ttLanceRxHead                infoRxRing[TM_LANCE_RING_SIZE_RX];
/* The Transmit Descriptor Ring */
    ttLanceTxHead                infoTxRing[TM_LANCE_RING_SIZE_TX];
/* The Initialization Block */
    ttLanceInit                  infoInitBlock;
/* A corollary array to the receive descriptor ring containing more data */
    ttLanceRxRingInfo            infoRxRingInfo[TM_LANCE_RING_SIZE_RX];
/* The interface handle */
    int                          infoDeviceIndex;
/* The next free receive ring entry */
    int                          infoCurRx;
/* The current rx ring entry to be sent up */
    int                          infoIsrRx;
/* The current tx ring entry */
    int                          infoCurTx;
/* The ring entries to be free()ed. */
    int                          infoDirtyRx;
    int                          infoDirtyTx;    
/* Set to 1 if packet is first in a scatter send */
    int                          infoFirstPacket;
    int                          infoXmitQueueOn;
/* Device Base address */
    int                          infoDeviceBase;
/* Flag for old lance chips */
    char                         infoOldLanceFlag;
    char                         infoPadding[3];
} ttLanceInfo;
typedef ttLanceInfo TM_FAR * ttLanceInfoPtr;
#define TM_LANCE_INFO_PTR_NULL (ttLanceInfoPtr)0

#ifdef TM_LINT
LINT_UNREF_MEMBER_END /* PC-LINT compile options */
#endif /* TM_LINT */

/*
 * Get the physical address
 * return it to the Treck stack
 * The address of the card needs to be changed - this is just test data
 */
int tfLanceGetPhysicalAddress(ttUserInterface lanceHandle,
                              char TM_FAR *   physicalAddress)
{
/* This line is to get rid of compiler warnings */
    TM_UNREF_IN_ARG(lanceHandle);

    physicalAddress[0] = 0;
    physicalAddress[1] = 4;
    physicalAddress[2] = 5;
    physicalAddress[3] = 3;
    physicalAddress[4] = 7;
    physicalAddress[5] = 7;

    return TM_ENOERROR;
}


/*
 * Open the lance chip and do any necessary initializations 
 */
int tfLanceOpen(ttUserInterface lanceHandle)
{
    int                 i;
    tt32Bit             absAddr;
    char                ethAddress[6];
    int                 errorCode;
    ttLanceInfoPtr      lanceInfoPtr;
    int                 lanceIndex;
/* Device addresses */
    int                 lanceDeviceBase;
    int                 lanceIsrBase;
    int                 lanceDeviceRdp;
    int                 lanceDeviceRap;
    int                 lanceDeviceReset;
    int                 lanceDeviceIdp;
    int                 retCode;

/* defaults to no error */
    errorCode = TM_DEV_OKAY;
    lanceInfoPtr = (ttLanceInfoPtr)0;

/* Find an unused LANCE parameter structure */
    for(lanceIndex = 0; lanceIndex < TM_MAX_NUMBER_DEVICES; lanceIndex++ )
    {
        if(tvDeviceIsrPara[lanceIndex].deviInterfaceHandle == (ttUserInterface)0)
            break;
    }

    if(lanceIndex >= TM_MAX_NUMBER_DEVICES)
    {
#ifdef TM_ERROR_CHECKING
        tfKernelError("tfLanceOpen",
           "Device driver parameter full: Increase TM_MAX_NUMBER_DEVICES.");
#endif /* TM_ERROR_CHECKING */
        errorCode = TM_DEV_ERROR;
        goto TL_LANCE_OPEN_END;
    }

/* Get memory for the tsLanceInfo structure and make sure it is
 * 8-byte aligned */
    lanceInfoPtr = (ttLanceInfoPtr)tm_get_raw_buffer( sizeof(ttLanceInfo)
                                           + TM_LANCE_ALIGNMENT_MODIFIER );
    if (lanceInfoPtr == TM_LANCE_INFO_PTR_NULL) 
    {
        errorCode = TM_DEV_ERROR;
        goto TL_LANCE_OPEN_END;
    }
    lanceInfoPtr = (ttLanceInfoPtr)
                     ((((tt32Bit)(lanceInfoPtr)) +
                     (TM_LANCE_ALIGNMENT_MODIFIER-1)) &
                     TM_LANCE_ALIGNMENT_MOD_MASK);

/* Store driver variable to treck stack */
    retCode = tfDeviceStorePointer(lanceHandle, (ttVoidPtr)lanceInfoPtr); 
    if (retCode != TM_ENOERROR)
    {
#ifdef TM_ERROR_CHECKING
        tfKernelError("tfLanceOpen",\
                        "Device driver already opened: corrupted memory.");
#endif /* TM_ERROR_CHECKING */
        errorCode = TM_DEV_ERROR;
        goto TL_LANCE_OPEN_END;
    }

/* Store the device indext the Info structure */
    lanceInfoPtr->infoDeviceIndex = lanceIndex;
/* 
 * Store device base address. This should be a function of lanceIndex, i.e.
 * as lanceIndex grow, the lanceDeviceBase should get bigger
 */
    tm_lance_base(lanceDeviceBase, lanceIndex);

    lanceInfoPtr->infoDeviceBase = lanceDeviceBase;
/* Initialize to TRUE */
    lanceInfoPtr->infoFirstPacket = TM_LANCE_TRUE;

    lanceInfoPtr->infoXmitQueueOn = tfInterfaceXmitQueueOn(lanceHandle);

    lanceDeviceReset = lanceDeviceBase + TM_LANCE_RESET;
/* Reset the LANCE */
    tm_in_word(lanceDeviceReset);

/* Determine if this is an older lance chip. CSR88 contains the chip ID */
    lanceDeviceRap = lanceInfoPtr->infoDeviceBase + TM_LANCE_RAP;
    tm_out_word(TM_LANCE_CSR88, lanceDeviceRap);
    lanceDeviceRdp = lanceInfoPtr->infoDeviceBase + TM_LANCE_RDP;
    lanceInfoPtr->infoOldLanceFlag = 
            (char)(tm_in_word(lanceDeviceRdp) != TM_LANCE_NEW_CHIP);

/* Un-Reset the LANCE, needed only for the NE2100. */
    if (lanceInfoPtr->infoOldLanceFlag==TM_LANCE_TRUE)
    {
        tm_out_word(TM_LANCE_UNRESET, lanceDeviceReset);
    }

    if (lanceInfoPtr->infoOldLanceFlag == TM_LANCE_FALSE)
    {
/* This is 79C960-specific: Turn on auto-select of media (AUI, BNC). */
        tm_out_word(TM_LANCE_ISACSR2, lanceDeviceRap);
        lanceDeviceIdp = lanceDeviceBase + TM_LANCE_IDP;
        tm_out_word(TM_LANCE_ISACSR2_MEDSEL, lanceDeviceIdp);
    }

/* Warning!! Modify this code for your particular processor!!
 * Turn on INT0 on the processor (that is, turn off masking).
 * Warning several devices: 
 * TM_LANCE_PCB_INT0 + a function of lanceIndex ???
 */
    tm_lance_isr_base(lanceIsrBase, lanceIndex);
    tm_out_word(TM_LANCE_INT0_CONFIG, lanceIsrBase);


/* Store ISR handler parameters */
    tvDeviceIsrPara[lanceIndex].deviInterfaceHandle = lanceHandle;
#ifdef TM_MULTIPLE_CONTEXT
    tvDeviceIsrPara[lanceIndex].deviContextHandle = tfGetCurrentContext();
#endif /* TM_MULTIPLE_CONTEXT */

/* Install tfLanceIsrHandler on INT0 (interrupt type 12) in the IVT
 * Change this line to whatever your interrupt type is for your processor
 */
    switch(lanceIndex)
    {
        case 0:
            tfKernelInstallIsrHandler((ttUserIsrHandlerPtr)tfLanceIsrHandler0,
                                       TM_LANCE_INTERRUPT_TYPE);
            break;

        case 1:
            tfKernelInstallIsrHandler((ttUserIsrHandlerPtr)tfLanceIsrHandler1,
                                      TM_LANCE_INTERRUPT_TYPE);
            break;

/*
 * Install more LANCE ISR handler here if needed, the number of ISR handler 
 * is limited by TM_MAX_NUMBER_DEVICES. 
 */

        default:
            break;
    }

/* Initialize the LANCE Rx and Tx rings. */
    lanceInfoPtr->infoCurRx = 0;
    lanceInfoPtr->infoIsrRx = 0;
    lanceInfoPtr->infoCurTx = 0;
    lanceInfoPtr->infoDirtyRx = 0;
    lanceInfoPtr->infoDirtyTx = 0;

/* Initialize the receive ring */
    for (i = 0; i < TM_LANCE_RING_SIZE_RX; i++)
    {
        lanceInfoPtr->infoRxRingInfo[i].rxBufferAddress =
            tfGetEthernetBuffer((ttUserBufferPtr)
            &(lanceInfoPtr->infoRxRingInfo[i].rxBufferHandle));
/* JPA/02-07-00 BUGFIX 472 */
/* Make sure we got back a valid data pointer. If not, return an error */
        if (lanceInfoPtr->infoRxRingInfo[i].rxBufferAddress ==
            (char TM_FAR *)0)
        {
            errorCode = TM_DEV_ERROR;
            goto TL_LANCE_OPEN_END;
        }
/* The ORing in this line is to set ownership of the buffer to the chip */
        lanceInfoPtr->infoRxRing[i].rxAddress =
            tm_get_abs_address(
            lanceInfoPtr->infoRxRingInfo[i].rxBufferAddress) |
            TM_LANCE_HARDWARE_OWNS;

/* Upper 4 bits must be 1 */
        lanceInfoPtr->infoRxRing[i].rxBufferBytes =
            (tt16Bit)(TM_LANCE_SET_UPPER_4_BITS | (-TM_PKT_BUF_SZ));
    }

/* Initialize the transmit ring */
    for (i = 0; i < TM_LANCE_RING_SIZE_TX; i++)
    {
        lanceInfoPtr->infoTxRing[i].txAddress = TM_LANCE_SOFTWARE_OWNS;
/* Upper 4 bits must be 1 */
        lanceInfoPtr->infoTxRing[i].txBufferBytes =
            (tt16Bit)(TM_LANCE_SET_UPPER_4_BITS | (-TM_PKT_BUF_SZ));
    }

/* Set up the init block */
/* Load the phsical address into the init table */
    (void)tfLanceGetPhysicalAddress(lanceHandle,ethAddress);
    for (i = 0; i < 6; i++)
    {
        lanceInfoPtr->infoInitBlock.initPhysAddr[i] = (tt8Bit)ethAddress[i];   
    }

/* Set the initialization mode */
    lanceInfoPtr->infoInitBlock.initMode = TM_LANCE_INIT_MODE;

/* We don't do multicast, so set Logical Address Filters to 0 */
    lanceInfoPtr->infoInitBlock.initFilter[0] = 0x0UL;
    lanceInfoPtr->infoInitBlock.initFilter[1] = 0x0UL;

/* Set the address and length of receive descriptor ring */
    lanceInfoPtr->infoInitBlock.initRxRing = 
            tm_get_abs_address(lanceInfoPtr->infoRxRing) |
            TM_LANCE_RING_LEN_BITS_RX;

/* Set the address and length of transmit descriptor ring */
    lanceInfoPtr->infoInitBlock.initTxRing = 
            tm_get_abs_address(lanceInfoPtr->infoTxRing) |
            TM_LANCE_RING_LEN_BITS_TX;

/* Write the absolute address of the initialization block into appropriate
 * registers
 */
    absAddr = tm_get_abs_address(&(lanceInfoPtr->infoInitBlock));

    tm_out_word(TM_LANCE_CSR1, lanceDeviceRap);
    tm_out_word(tm_get_lower16(absAddr), lanceDeviceRdp);

    tm_out_word(TM_LANCE_CSR2, lanceDeviceRap);
    tm_out_word(tm_get_upper8(absAddr), lanceDeviceRdp);


/* Set receive interrupt enabled, all others masked */
    tm_out_word(TM_LANCE_CSR3, lanceDeviceRap);
    tm_out_word(TM_LANCE_CSR3_BABLM |
                TM_LANCE_CSR3_MISSM |
                TM_LANCE_CSR3_MERRM |
                TM_LANCE_CSR3_TINTM |
                TM_LANCE_CSR3_IDONM |
                TM_LANCE_CSR3_DXSUFLO,
                lanceDeviceRdp);

/* Set up various options. See definitions above for what each constant does */
    tm_out_word(TM_LANCE_CSR4, lanceDeviceRap);
    tm_out_word(TM_LANCE_CSR4_ASTRP_RCV |
                TM_LANCE_CSR4_MFCOM    |
                TM_LANCE_CSR4_RCVCCOM |
                TM_LANCE_CSR4_TXSTRTM |
                TM_LANCE_CSR4_JABM |
                TM_LANCE_CSR4_DPOLL |
                TM_LANCE_CSR4_DMAPLUS,
                lanceDeviceRdp);

#if (defined(TM_IGMP) || defined(TM_6_USE_MLD))
/* For now, to get MLD to work for IPv6, we just set the receiver to be in
   promiscuous receive mode. */
    tm_out_word(TM_LANCE_CSR15, lanceDeviceRap);
    tm_out_word(TM_LANCE_CSR15_PROM, lanceDeviceRdp);
#endif /* TM_IGMP or TM_6_USE_MLD */

/* Tell chip to read in the initialization block */
    tm_out_word(TM_LANCE_CSR0, lanceDeviceRap);
    tm_out_word(TM_LANCE_CSR0_INIT,
                lanceDeviceRdp);

/* Wait until chip has initialized */
    while (!(tm_in_word(lanceDeviceRdp) & TM_LANCE_CSR0_IDON))
    {
        ;
    }

/* Start the chip */
    tm_out_word(TM_LANCE_CSR0_IDON |
                TM_LANCE_CSR0_IENA |
                TM_LANCE_CSR0_STRT,
                lanceDeviceRdp);

TL_LANCE_OPEN_END:
    if (errorCode != TM_DEV_OKAY && lanceInfoPtr != (ttLanceInfoPtr)0)
    {
        tm_free_raw_buffer((ttVoidPtr)lanceInfoPtr);
    }
    return(errorCode);            
}


/* Close the LANCE chip */
int tfLanceClose(ttUserInterface interfaceHandle)
{
    int             i;
    int             lanceIndex;
    int             lanceDeviceRdp;
    int             lanceDeviceRap;
    int             errorCode;
    ttLanceInfoPtr  lanceInfoPtr;

    errorCode = TM_DEV_OKAY;
/* Deal with 'unused variable' compiler warnings */
    TM_UNREF_IN_ARG(interfaceHandle);

/* Get the chip info pointer */
    lanceInfoPtr = tm_device_get_pointer(interfaceHandle);
#ifdef TM_ERROR_CHECKING
    if( lanceInfoPtr == TM_LANCE_INFO_PTR_NULL )
    {
        tfKernelError("tfLanceClose",\
                        "Device driver already closed: corrupted memory.");
        errorCode = TM_DEV_ERROR;
    }
    else
#endif /* TM_ERROR_CHECKING */
    {
        lanceIndex = lanceInfoPtr->infoDeviceIndex;
        lanceDeviceRap = lanceInfoPtr->infoDeviceBase + TM_LANCE_RAP;
        lanceDeviceRdp = lanceInfoPtr->infoDeviceBase + TM_LANCE_RDP;
/* Stop the chip */
        tm_out_word(TM_LANCE_CSR0, lanceDeviceRap);
        tm_out_word(TM_LANCE_CSR0_STOP, lanceDeviceRdp);
/* JPA/02-07-00 BUGFIX 463 */

/* Free ISR handler parameters structure */
        tvDeviceIsrPara[lanceIndex].deviInterfaceHandle = 
                                                        (ttUserInterface)0;

/* Free any buffers the driver is still holding on to */
        for (i = 0; i < TM_LANCE_RING_SIZE_RX; i++)
        {
            if (lanceInfoPtr->infoRxRing[i].rxAddress != 0)
            {
                (void)tfFreeDriverBuffer(
                        lanceInfoPtr->infoRxRingInfo[i].rxBufferHandle);
            }
        }

/* Free device driver variable */
        lanceInfoPtr = tfDeviceClearPointer(interfaceHandle);
        if ( lanceInfoPtr != TM_LANCE_INFO_PTR_NULL )
        {
/* Free malloc'd memory */
            tm_free_raw_buffer((void TM_FAR *)lanceInfoPtr);
        }
#ifdef TM_ERROR_CHECKING
        else
        {
            tfKernelError("tfLanceClose","Fail to clear device driver ptr");
            errorCode = TM_DEV_ERROR;
        }
#endif /* TM_ERROR_CHECKING */
    }

    return errorCode;
}


/* This function should be called from the timer function in the application.
 * Right now, it only supports refilling receive buffers, but, if need be,
 * you can also include send completes in here (though it should not be
 * necessary). Just add another if and define another constant. All
 * sendcompletes should be taken care of by the send routine itself.
 */
int tfLanceIoctl(ttUserInterface interfaceHandle,
                 int             flag,
                 void TM_FAR *   optionPtr,
                 int             optionLen)
{
    int             errorCode;
    int             i;
    ttLanceInfoPtr  lanceInfoPtr;

/* Deal with 'unused variable' compiler warnings */
    TM_UNREF_IN_ARG(interfaceHandle);
    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);


/* Get the chip info pointer */
    lanceInfoPtr = tm_device_get_pointer(interfaceHandle);
#ifdef TM_ERROR_CHECKING
    if( lanceInfoPtr == TM_LANCE_INFO_PTR_NULL )
    {
        tfKernelError("tfLanceIoctl",\
                        "Device driver already closed: corrupted memory.");
        errorCode = TM_DEV_ERROR;
    }
    else
#endif /* TM_ERROR_CHECKING */
    {
        errorCode = TM_DEV_OKAY;
/* If the refill flag is passed in, refill the receive buffers */
        if (flag & TM_LANCE_IOCTL_REFILL)
        {
            i = lanceInfoPtr->infoDirtyRx;
/* Loop until we hit a receive buffer that is still in use */
            while ((lanceInfoPtr->infoRxRing[i].rxAddress) == 0x0UL)
            {
/* Get a new buffer */
                lanceInfoPtr->infoRxRingInfo[i].rxBufferAddress =
                        tfGetEthernetBuffer((ttUserBufferPtr)
                        &(lanceInfoPtr->infoRxRingInfo[i].rxBufferHandle));
/* JPA/02-07-00 BUGFIX 472 */
/*
 * Make sure we actually got memory. If not, break the loop and return an
 * error.
 */
                if (lanceInfoPtr->infoRxRingInfo[i].rxBufferAddress ==
                        (char TM_FAR *)0)
                {
                    errorCode = TM_DEV_ERROR;
                    break;
                }
/* The ORing in this line is to set ownership of the buffer to the chip */
                lanceInfoPtr->infoRxRing[i].rxAddress =
                    tm_get_abs_address(
                        lanceInfoPtr->infoRxRingInfo[i].rxBufferAddress);
                lanceInfoPtr->infoRxRing[i].rxAddress |= TM_LANCE_HARDWARE_OWNS;
/* Modulus operation to keep i within the ring values */
                i = (i+1) & TM_LANCE_RING_MOD_MASK_RX;
            }
/* Update the dirty transmit 'pointer' */
            lanceInfoPtr->infoDirtyRx = i;
        }
        if (flag & TM_LANCE_IOCTL_SEND_COMPLETE)
        {
/* Call send complete on dirty packets. Only call send complete once per
 * scattered packet. This loop runs while the address isn't set to 0 (to keep
 * us from looping) and the hardware doesn't own the buffer in question
 */
            i = lanceInfoPtr->infoDirtyTx;
            while (((lanceInfoPtr->infoTxRing[i].txAddress) != 0x0UL) &&
                  (((lanceInfoPtr->infoTxRing[i].txAddress) & 
                  TM_LANCE_HARDWARE_OWNS) == TM_LANCE_FALSE))
            {
/* Make sure the entry has its end-of-packet bit set. This ensures send complete
 * is only called once per packet.
 */
                if (   lanceInfoPtr->infoTxRing[i].txAddress 
                     & TM_LANCE_END_OF_PACKET )
                {       
                    tfSendCompleteInterface( interfaceHandle,
                                             TM_DEV_SEND_COMPLETE_DRIVER );
                }
                lanceInfoPtr->infoTxRing[i].txAddress = 0UL;
                i = (i+1) & TM_LANCE_RING_MOD_MASK_TX;
            }
/* Update dirty 'pointer' */
            lanceInfoPtr->infoDirtyTx = i;
        }
    }

    return(errorCode);
}


/* This function is responsible for handing packets-to-be-sent over to the
 * hardware. It first waits (if necessary) until there are descriptor entries
 * that are unused by the hardware, then calls sendcompletes on sent packets,
 * and then sends out any pending packets. This function is made a bit more
 * complicated because of scatter sends. It is not a good idea to start sending
 * scattered data before you've been given all of the data. Thus, the function
 * does not turn control over to the chip until it has received ALL the blocks
 * in a scattered packet.
 */
int tfLanceSend ( ttUserInterface lanceHandle,
                  char TM_FAR *   dataPtr,
                  int             dataLength,
                  int             flag )
{

    int             entry;
    int             i;
    unsigned long   txInfo;
    int             errorCode;
    int             lanceDeviceRdp;
    int             lanceDeviceRap;
    ttLanceInfoPtr  lanceInfoPtr;
/* Define device base address macro */

/* Get the chip info pointer */
    lanceInfoPtr = tm_device_get_pointer(lanceHandle);
#ifdef TM_ERROR_CHECKING
    if( lanceInfoPtr == TM_LANCE_INFO_PTR_NULL )
    {
        tfKernelError("tfLanceSend",\
                        "Device driver already closed: corrupted memory.");
        errorCode = TM_DEV_ERROR;
        goto lanceSendExit;
    }
#endif /* TM_ERROR_CHECKING */
    lanceDeviceRap = lanceInfoPtr->infoDeviceBase + TM_LANCE_RAP;
    lanceDeviceRdp = lanceInfoPtr->infoDeviceBase + TM_LANCE_RDP;
/* Wait until the hardware has freed up some descriptor ring entries */
    errorCode = TM_DEV_OKAY;
    if ( (   (lanceInfoPtr->infoTxRing[lanceInfoPtr->infoCurTx].txAddress)
           & TM_LANCE_HARDWARE_OWNS ) != TM_LANCE_FALSE )
    {
        if (lanceInfoPtr->infoXmitQueueOn)
        {
/* If user uses a transmit queue, queue the buffer in the transmit queue */
            errorCode = TM_DEV_ERROR;
            goto lanceSendExit;
        }
        else
        {
            do
            {
/*
 * Do nothing function if user does not use transmit task. Otherwise will
 * yield the CPU.
 */
                (void)tfInterfaceSpinLock(lanceHandle);
            }
            while (
          (  (lanceInfoPtr->infoTxRing[lanceInfoPtr->infoCurTx].txAddress)
           & TM_LANCE_HARDWARE_OWNS ) != TM_LANCE_FALSE );
        }
    }
/* Call send complete on dirty packets. Only call send complete once per
 * scattered packet. This loop runs while the address isn't set to 0 (to keep
 * us from looping) and the hardware doesn't own the buffer in question
 */
    i = lanceInfoPtr->infoDirtyTx;
    while (((lanceInfoPtr->infoTxRing[i].txAddress) != 0x0UL) &&
             (((lanceInfoPtr->infoTxRing[i].txAddress) & 
             TM_LANCE_HARDWARE_OWNS) == TM_LANCE_FALSE))
    {
/* Make sure the entry has its end-of-packet bit set. This ensures send complete
 * is only called once per packet.
 */
        if (lanceInfoPtr->infoTxRing[i].txAddress & TM_LANCE_END_OF_PACKET)
        {       
            tfSendCompleteInterface(lanceHandle,TM_DEV_SEND_COMPLETE_DRIVER);
        }
        lanceInfoPtr->infoTxRing[i].txAddress = 0UL;
        i = (i+1) & TM_LANCE_RING_MOD_MASK_TX;
    }
/* Update dirty 'pointer' */
    lanceInfoPtr->infoDirtyTx = i;
 
/* Get the entry to be sent */
    entry = lanceInfoPtr->infoCurTx++;
    lanceInfoPtr->infoCurTx &= TM_LANCE_RING_MOD_MASK_TX;

/* Set the necessary fields in the transmit descriptor
 * Caution: the write order is important here.
 */
    lanceInfoPtr->infoTxRing[entry].txBufferBytes = 
                (tt16Bit)(TM_LANCE_SET_UPPER_4_BITS | (-dataLength));
    lanceInfoPtr->infoTxRing[entry].txErrors = 0;
    lanceInfoPtr->infoTxRing[entry].txAddress = tm_get_abs_address(dataPtr);

/* If this is the last buffer in a packet... */
    if ((flag & TM_USER_BUFFER_LAST) != TM_LANCE_FALSE)
    {
/* and also the first packet... */
        if (lanceInfoPtr->infoFirstPacket == TM_LANCE_TRUE)
        {
/* Then set start-of-packet and end-of-packet bits, and set ownership to
 * hardware
 */
            txInfo = TM_LANCE_HARDWARE_OWNS |
                     TM_LANCE_START_OF_PACKET |
                     TM_LANCE_END_OF_PACKET;
        }
/* else it is not the first packet */
        else
        {
/* So just set the end-of-packet bit and ownership bit */
            txInfo = TM_LANCE_HARDWARE_OWNS |
                     TM_LANCE_END_OF_PACKET;
        }
/* Reset the isFirstPacket flag */
        lanceInfoPtr->infoFirstPacket = TM_LANCE_TRUE;
    }
/* Else it is not the last buffer in a packet... */
    else
    {
/* If it's the first packet... */
        if (lanceInfoPtr->infoFirstPacket == TM_LANCE_TRUE)
        {
/* Set first packet and ownership bits. Also turn the isFirstPacket flag off */
            lanceInfoPtr->infoFirstPacket = TM_LANCE_FALSE;
            txInfo = TM_LANCE_HARDWARE_OWNS |
                     TM_LANCE_START_OF_PACKET;
        }
        else
        {
/* Else it's neither start or end of packet. Just set ownership bit */
            txInfo = TM_LANCE_HARDWARE_OWNS;
        }
    }
/* Store the appropriate ownership & packet info bits in the correct field */
    lanceInfoPtr->infoTxRing[entry].txAddress |= txInfo;
        
/* At this point, lanceInfoPtr->infoFirstPacket will have been reset in the
 * above 'if' structure. Thus, if it has been reset to true, that means we 
 * must have encountered a last packet above, and should send out all of the
 * waiting blocks
 */
    if (lanceInfoPtr->infoFirstPacket == TM_LANCE_TRUE)
    {
        tm_out_word(TM_LANCE_CSR0, lanceDeviceRap);
        tm_out_word(TM_LANCE_CSR0_TDMD |
                    TM_LANCE_CSR0_IENA, 
                    lanceDeviceRdp);
    }

lanceSendExit:
    return errorCode;
}


/* Pass packet received by hardware up the stack, and mark it as
 * having been dealt with, so that Ioctl knows to refill the buffer.
 * The chip generates an interrupt when a packet is received, which in turn
 * triggers a call to this function.
 */
int tfLanceReceive(ttUserInterface        interfaceHandle,
                   char TM_FAR * TM_FAR * dataPtr,
                   int  TM_FAR *          dataLength,
                   ttUserBufferPtr        bufHandlePtr)
{
    int             entry;
    int             status;
    int             errorCode;
    ttLanceInfoPtr  lanceInfoPtr;

/* Get the chip info pointer */
    lanceInfoPtr = tm_device_get_pointer(interfaceHandle);
#ifdef TM_ERROR_CHECKING
    if( lanceInfoPtr == TM_LANCE_INFO_PTR_NULL )
    {
        tfKernelError("tfLanceReceive",\
                        "Null Device Driver Pointer: corrupted memory.");
        errorCode = TM_DEV_ERROR;
    }
    else
#endif /* TM_ERROR_CHECKING */
    {

/* Deal with 'unused variable' compiler warning */
        TM_UNREF_IN_ARG(interfaceHandle);

/* Get the current entry */
        entry = lanceInfoPtr->infoCurRx & TM_LANCE_RING_MOD_MASK_RX;
    
/* Make sure that the packet belongs to us, otherwise return an error */
        if (((lanceInfoPtr->infoRxRing[entry].rxAddress) & \
                            TM_LANCE_HARDWARE_OWNS) == TM_LANCE_FALSE)
        {
            status = (int) ((lanceInfoPtr->infoRxRing[entry].rxAddress) >>
                            TM_LANCE_INFO_FIELD_SHIFT);
/* Set the passed-in variables to appropriate values */
            *dataLength = lanceInfoPtr->infoRxRing[entry].rxMessageBytes;
            *dataPtr = lanceInfoPtr->infoRxRingInfo[entry].rxBufferAddress;
            *bufHandlePtr = 
                        lanceInfoPtr->infoRxRingInfo[entry].rxBufferHandle;
/* Check if there was an error in receive */
            if (status & TM_LANCE_RECEIVE_ERROR)
            {    
/* The 12th and 13th bytes of an ethernet packet indicate type. If we set
 * them to something invalid (0xffff in this case), the stack will simply
 * discard the packet
 */
                *((*dataPtr)+TM_LANCE_PACKET_TYPE_OFFSET_1) =
                                          (char)TM_LANCE_INVALID_PACKET_TYPE;
                *((*dataPtr)+TM_LANCE_PACKET_TYPE_OFFSET_2) = 
                                           (char)TM_LANCE_INVALID_PACKET_TYPE;
            }
/* Set the address in the descriptor ring to 0, so Ioctl knows to clean it up */
            lanceInfoPtr->infoRxRing[entry].rxAddress = 0UL;
            entry = (entry+1) & TM_LANCE_RING_MOD_MASK_RX;
            errorCode = TM_DEV_OKAY;
        }
        else
        {
            errorCode = TM_DEV_ERROR;
        }
/* Update the current entry */
        lanceInfoPtr->infoCurRx = entry;
    }

    return(errorCode);
}


/* 
 * The LANCE interrupt handler. This is only called on a receive-complete
 * interrupt.
 */
void tfLanceIsrHandler (ttDeviceIsrParaPtr isrParaPtr)
{
    int              tlCsr0;
    int              tlRap;
    ttLanceRxHeadPtr ringPtr;
    int              i;
    int              recvNotifyCount;
    int              lanceDeviceRap;
    int              lanceDeviceRdp;
    ttLanceInfoPtr   lanceInfoPtr;
    ttUserInterface  lanceHandle;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext    saveContextHandle;
#endif /* TM_MULTIPLE_CONTEXT */

    lanceHandle = (ttUserInterface)(isrParaPtr->deviInterfaceHandle);
#ifdef TM_MULTIPLE_CONTEXT
    saveContextHandle = tfGetCurrentContext();
    tfSetCurrentContext(isrParaPtr->deviContextHandle);
#endif /* TM_MULTIPLE_CONTEXT */
    lanceInfoPtr = tm_device_get_pointer(lanceHandle);
    lanceDeviceRap = lanceInfoPtr->infoDeviceBase + TM_LANCE_RAP;
    lanceDeviceRdp = lanceInfoPtr->infoDeviceBase + TM_LANCE_RDP;
    recvNotifyCount = 0;
/* Save current RAP setting */
    tlRap = tm_in_word(lanceDeviceRap);

/* Get current state of CSR0 */
    tm_out_word(TM_LANCE_CSR0, lanceDeviceRap);
    tlCsr0 = tm_in_word(lanceDeviceRdp);

/* Clear receive interrupt from LANCE */
    tm_out_word(tlCsr0, lanceDeviceRdp);

/* Handle the Rx Interrupt */
    if (tlCsr0 & TM_LANCE_CSR0_RINT)
    {
        i = lanceInfoPtr->infoIsrRx;
        ringPtr = lanceInfoPtr->infoRxRing;
/* call NotifyRecieve for each received packet we haven't dealt with yet */
        while ((ringPtr[i].rxAddress & (TM_LANCE_HARDWARE_OWNS |
               TM_LANCE_END_OF_PACKET)) == TM_LANCE_END_OF_PACKET)
        {
/* Mark ring entry as dealt-with */
            ringPtr[i].rxAddress &= ~TM_LANCE_END_OF_PACKET;
            recvNotifyCount++;
            i = (i+1) & TM_LANCE_RING_MOD_MASK_RX;
        }
        lanceInfoPtr->infoIsrRx = i;
    }

/*
 * Set a critical section so that when we clear the interrupt, we will
 * not be interrupted again until we exit the ISR handler. This driver
 * was written on an x86 running AMX, which handles the IRET for us
 */
    tm_kernel_set_critical;

/* Warning!! Change next line of code for your particular processor!! */
/* Clear the interrupt in the processor */
    tm_out_word(TM_LANCE_EOI_INT0,TM_LANCE_PCB+TM_LANCE_PCB_EOI);

/* Restore the RAP */
    tm_out_word(tlRap, lanceDeviceRap);
    tfNotifyInterfaceIsr( isrParaPtr->deviInterfaceHandle,
                          recvNotifyCount,
                          0, 0UL, 0UL );

#ifdef TM_MULTIPLE_CONTEXT
    tfSetCurrentContext(saveContextHandle);
#endif /* TM_MULTIPLE_CONTEXT */
    return;
}


void tfLanceIsrHandler0(void)
{
    tfLanceIsrHandler(&tvDeviceIsrPara[0]);
    return;
}


void tfLanceIsrHandler1(void)
{
    tfLanceIsrHandler(&tvDeviceIsrPara[1]);
    return;
}

/*
 * Add more LANCE ISR handler here, the number of ISR handler is limited 
 * by TM_MAX_NUMBER_DEVICES.
 */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:39      For: Xilinx-Samsung
 * File Name: trlance.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 22739    Output checksum: 41484
 */
