/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/drivers/m68360/trquicc.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:04CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/* 
 * SOURCE FILE FOR Motorola 68360 QUICC Sample Driver
 *
 * This device driver is derived from the Ethernet Bridge example from
 * Motorola.
 * It has been modified to interface to the Treck TCP/IP stack and to
 * provide readability.
 *
 * Since it is a derived work, it remains the property of Motorola Corp.
 *
 * We have left the definintions of the quicc registers as they
 * are in the QUICC user manual
 *
 * This is why they do not match TRECK coding standards.
 *
 * Modification History
 * Date: $Date: 2010/01/18 06:36:04CST $
 * Author:
 * Version:  $Revision: 6.0.2.2 $
 * Release:  $ProjectRevision: 6.0.1.23 $
 * Description:
 */


#include <trsocket.h>
#include <trquicc.h>


/* 
 * IMPORTANT
 * If the quicc device only receives a few frames and stops forever,
 * make sure that you are calling the tfQuiccRefillScc1RecvPool()
 * function from a low priority task
 */


/*    
 * Since we do not accept SHORT frames, we use the SHORT bit to 
 * mean that this frame has been notified
 * IF THIS DRIVER IS CHANGED TO ALLOW SHORT FRAMES, ANOTHER BIT
 * WILL NEED TO BE USED
 */
#define TM_QUICC_NOTIFIED TM_QUICC_RECV_E_SHORT
#define TM_QUICC_ERROR_RECV (TM_QUICC_RECV_ERROR & (~TM_QUICC_NOTIFIED))

/*
 * Local Function Prototypes
 */

/* Ethernet Interface */
void tfQuiccBusy(ttUserInterface interfaceHandle, int channel);
void tfQuiccXmitError(ttUserInterface interfaceHandle,int channel);
void tfQuiccScc1SendComplete(ttUserInterface interfaceHandle);
void tfQuiccEtherUp(int channel);
void tfQuiccEtherScc1BoardInit(void);
void tfQuiccEtherScc1Init(void);
/* defined in trquicc.h */
/* int  tfQuiccEtherOpen(ttUserInterface interfaceHandle); */
/* int  tfQuiccEtherClose(ttUserInterface interfaceHandle); */
/* void tfQuiccScc1HandlerIsr(void); */
void tfQuiccRefillScc1RecvPool(void);


/* Serial Interface */
int  tfQuiccSerialClose(ttUserInterface interfaceHandle);
void tfQuiccSerialUp(int channel);
int  tfQuiccSerialOpen(ttUserInterface interfaceHandle);
void tfQuiccSerialScc4BoardInit(void);
void tfQuiccSerialScc4Init(void);
void tfQuiccScc4HandlerIsr(void);
void tfQuiccRefillScc4RecvPool(void);
void tfQuiccScc4SendComplete(ttUserInterface interfaceHandle);

/* General functions */
void tfQuiccCreateXmitPool(int channel, int maxBd);
void tfQuiccCreateRecvPool(int channel, int maxBd);
void tfQuiccIssueCommand(unsigned short command);
void tfQuiccDown(int channel);
int  tfQuiccOnline(int channel);

/* 
 * Local variables 
 */

static ttQuiccPtr tlQuiccPtr;

static ttQuiccBufDescPtr tlQuiccScc1XmitBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc4XmitBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc1XmitIsrBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc4XmitIsrBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc1XmitBufDescStartPtr;
static ttQuiccBufDescPtr tlQuiccScc4XmitBufDescStartPtr;

static ttQuiccBufDescPtr tlQuiccScc1RecvBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc4RecvBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc1RecvIsrBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc4RecvIsrBufDescPtr;
static ttQuiccBufDescPtr tlQuiccScc1RecvBufDescStartPtr; 
static ttQuiccBufDescPtr tlQuiccScc4RecvBufDescStartPtr; 
static ttQuiccBufDescPtr tlQuiccScc1RecvRefillBufDescPtr; 
static ttQuiccBufDescPtr tlQuiccScc4RecvRefillBufDescPtr;

static ttQuiccBufDescPtr tlScatterBufferStart;

unsigned long     tvQuiccScc1RecvErrorCount;
unsigned long     tvQuiccScc4RecvErrorCount;
unsigned long     tvQuiccScc1RecvPacketCount;
unsigned long     tvQuiccScc4RecvPacketCount;
unsigned long     tvQuiccScc1SendPacketCount;
unsigned long     tvQuiccScc4SendPacketCount;
unsigned long     tvQuiccBusyPacketCount;
unsigned long     tvQuiccScc1XmitWait;
unsigned long     tvQuiccScc4XmitWait;
unsigned long     tvQuiccIsrRecvErrorCount;

/*
 * Four Interface handles for four SCC's 
 */
static ttUserInterface tlInterfaceArray[4];
/*
 * Four transmit queue flags for four SCC's
 */
static int             tlQuiccXmitQueueArray[4];

/*
 * Table of ttUserBuffer Handles
 */
static ttUserBuffer    tlUserScc1BufferList[TM_QUICC_ETHER_MAX_RECV_BUFD];
static ttUserBuffer    tlUserScc4BufferList[TM_QUICC_SERIAL_MAX_RECV_BUFD];

/*
 * Our index to the ttUserBuffer's
 */
static ttUserBufferPtr tlUserScc1BufferPtr; 
static ttUserBufferPtr tlUserScc4BufferPtr; 
static ttUserBufferPtr tlUserScc1RefillBufferPtr; 
static ttUserBufferPtr tlUserScc4RefillBufferPtr; 


char tlQuiccXmitBuffer[TM_QUICC_SERIAL_MAX_XMIT_BUFD];
int  tlQuiccXmitBufNum;



/*
 *
 * ETHERNET INTERFACE  (SCC1)
 *
 */




/*
 * Return the Physical Address for the device
 * For the QUICC, it should be read from EEPROM
 * because every ethernet address is Unique
 *
 * THIS ROUTINE MUST BE MODIFIED!
 *
 * To get an ethernet address address block for 
 * your company, you need to contact IEEE Standards
 * or visit http://standards.ieee.org/faqs/OUI.html
 * for information on getting an Company ID/OUI
 */
int tfQuiccGetPhyAddr ( ttUserInterface userInterface,
                        char   TM_FAR * address )
{
/* 
 * We don't use the interface handle here, but we keep the compiler
 * from complaining about unused formal parameters
 */
    TM_UNREF_IN_ARG(userInterface);
    address[0] = (char)0x00;
    address[1] = (char)(unsigned char)0xea;
    address[2] = (char)(unsigned char)0xc0;
    address[3] = (char)(unsigned char)0xf0;
    address[4] = (char)0x00;
    address[5] = (char)0x00;
    return 0;
}


/* 
 * Initialize SCC1 to be an ethernet channel
 */
void tfQuiccEtherScc1Init (void)
{
    char                          etherAddr[6];
    struct ethernet_pram TM_FAR * enet_pram;
    struct scc_regs      TM_FAR * regs;

/* Retrieve the Ethernet Address */
    tfQuiccGetPhyAddr((ttUserInterface)0, etherAddr);

    enet_pram = &tlQuiccPtr->pram[0].enet_scc;          
    regs = &tlQuiccPtr->scc_regs[0];

/* Disable transmit/receive */
    regs->scc_gsmra = 0;

/* 
 * Initialize paramater ram 
 */

/* points to start of receiving ring */
    enet_pram->rbase =   TM_QUICC_SCC1_RECV_BASE;
/* points to start of transmiting ring */
    enet_pram->tbase =   TM_QUICC_SCC1_XMIT_BASE;   
/* Function Codes for Transmit and Receive */
    enet_pram->rfcr =    TM_QUICC_SCC1_RECV_FUNC_CODE;
    enet_pram->tfcr =    TM_QUICC_SCC1_XMIT_FUNC_CODE;
/* We always receive a full ethernet frame so we set it to 1518 */
    enet_pram->mrblr =   TM_QUICC_SCC1_MAX_RECV_BUF_LEN;
/* A good ethernet CRC */
    enet_pram->c_mask =  TM_QUICC_ETHER_CRC_MASK;
/* The CRC we start with */
    enet_pram->c_pres =  TM_QUICC_ETHER_CRC_PRES;
/* These three registers are cleared for clarity sake */
    enet_pram->crcec =   TM_QUICC_ETHER_CRCEC;
    enet_pram->alec =    TM_QUICC_ETHER_ALEC;
    enet_pram->disfc =   TM_QUICC_ETHER_DISFC;
/* Pad Character */
    enet_pram->pads =    TM_QUICC_ETHER_PADS;
/* Max Retries when we have a collision */
    enet_pram->ret_lim = TM_QUICC_ETHER_RETRY_LIMIT;
/* Maximum Ethernet Frame Length (Always 1518) */
    enet_pram->mflr =    TM_QUICC_ETHER_MAX_FRAME_LEN;
/* Minimum Ethernet Frame Length (Always 64) */
    enet_pram->minflr =  TM_QUICC_ETHER_MIN_FRAME_LEN;
/* Maximum DMA Counts */
    enet_pram->maxd1 =   TM_QUICC_ETHER_MAX_DMA1_COUNT;
    enet_pram->maxd2 =   TM_QUICC_ETHER_MAX_DMA2_COUNT; 
/* Group Addresses */
    enet_pram->gaddr1 =  TM_QUICC_ETHER_GROUP_ADDR1;    
    enet_pram->gaddr2 =  TM_QUICC_ETHER_GROUP_ADDR2;    
    enet_pram->gaddr3 =  TM_QUICC_ETHER_GROUP_ADDR3;    
    enet_pram->gaddr4 =  TM_QUICC_ETHER_GROUP_ADDR4;    
/* 
 * Our physical (Ethernet) address (IN LITTLE ENDIAN)
 * If the physical address is:
 * 0x112233445566
 *
 * It is stored like this:
 * paddr_l=0x2211
 * paddr_m=0x4433
 * paddr_h=0x6655
 * 
 * If not done correctly, the QUICC will NOT receive any packets
 * with its destination address 
 */
    enet_pram->paddr_l = (unsigned short)
                         (   ((unsigned short)etherAddr[0]        & 0x00ff)
                           | ((((unsigned short)etherAddr[1])<<8) & 0xff00) );
    enet_pram->paddr_m = (unsigned short)
                         (   ((unsigned short)etherAddr[2]        & 0x00ff)
                           | ((((unsigned short)etherAddr[3])<<8) & 0xff00) );
    enet_pram->paddr_h = (unsigned short)
                         (   ((unsigned short)etherAddr[4]        & 0x00ff)
                           | ((((unsigned short)etherAddr[5])<<8) & 0xff00) );

    enet_pram->p_per =   TM_QUICC_ETHER_P_PER;
/* Individual Hash Addresses */
    enet_pram->iaddr1 =  TM_QUICC_ETHER_INDV_ADDR1; 
    enet_pram->iaddr2 =  TM_QUICC_ETHER_INDV_ADDR2; 
    enet_pram->iaddr3 =  TM_QUICC_ETHER_INDV_ADDR3; 
    enet_pram->iaddr4 =  TM_QUICC_ETHER_INDV_ADDR4; 
/* Temp Registers */
    enet_pram->taddr_h = TM_QUICC_ETHER_T_ADDR_H;   
    enet_pram->taddr_m = TM_QUICC_ETHER_T_ADDR_M;
    enet_pram->taddr_l = TM_QUICC_ETHER_T_ADDR_L;
    tvQuiccScc1RecvPacketCount = 0L;
    tvQuiccScc4RecvPacketCount = 0L;
    tvQuiccScc1SendPacketCount = 0L;
    tvQuiccScc4SendPacketCount = 0L;
    tvQuiccScc1XmitWait        = 0L;
    tvQuiccScc4XmitWait        = 0L;
    tvQuiccScc1RecvErrorCount  = 0L;
    tvQuiccScc4RecvErrorCount  = 0L;
    tvQuiccIsrRecvErrorCount   = 0L;
    tlScatterBufferStart = (ttQuiccBufDescPtr)0;

/* Tell the CP to initialize our Recv/Xmit parameters */
    tfQuiccIssueCommand(TM_QUICC_INIT_RXTX_SCC1);
}

/* 
 * Initialize SCC1 to be an ethernet channel    
 */
void tfQuiccEtherScc1BoardInit (void)
{
    unsigned long  scc1Vector;
    unsigned long  scc1Location;

/*  tlQuiccPtr->sim_mcr =   0x608f; */

/*
 * IMPLEMENTATION NOTE:
 *
 * This routine is very board specific
 * In this example we use the Atlas Communications Engines
 * HSB devlopment board for an example
 * Please refer to your boards design to modify this routine
 */

/* Atlas HSB Setup */
    tlQuiccPtr->si_sicr |=  TM_QUICC_SI_SCC1_RCS_CLK2 |
                            TM_QUICC_SI_SCC1_TCS_CLK1;
                            
/* Make PortA Pins 0,1,8,9 for onchip use */
    tlQuiccPtr->pio_padir &= ~(TM_QUICC_PORTA_PA0 |
                               TM_QUICC_PORTA_PA1 |
                               TM_QUICC_PORTA_PA8 |
                               TM_QUICC_PORTA_PA9);
    
/* 
 * Use PortA pins 0,1 for SCC1 Ethernet RX/TX and 
 * pind 6,7 for SCC4 Xmit and Recv and
 * pins 8,9 for Ethernet TCLK and RCLK 
 */
    tlQuiccPtr->pio_papar |= TM_QUICC_PORTA_PA0 |
                             TM_QUICC_PORTA_PA1 |
                             TM_QUICC_PORTA_PA8 |
                             TM_QUICC_PORTA_PA9;

    tlQuiccPtr->intr_cicr=TM_QUICC_CICR_DEFAULT;
    
/* Clear the pending and in-service registers */
    tlQuiccPtr->intr_cipr =  TM_QUICC_SCC1_INT_MASK;
    tlQuiccPtr->intr_cisr =  TM_QUICC_SCC1_INT_MASK;
/* Set the CP mask register to accept interrupts from SCC1 */
    tlQuiccPtr->intr_cimr |=  TM_QUICC_SCC1_INT_MASK;

    tlQuiccPtr->sdma_sdcr =   TM_QUICC_SDMA_INT_MASK |
                              TM_QUICC_SDMA_ARB_ID;

    
 /* PortC Pins 4 & 5 are onchip */
    tlQuiccPtr->pio_pcdir &=   ~(TM_QUICC_PORTC_PC4 |
                                 TM_QUICC_PORTC_PC5);
                                 
/*
 * Use Pins 4 and 5 (SCC1 CTS and CD) for Ethernet CLSN and RENA 
 */
    tlQuiccPtr->pio_pcso |=   TM_QUICC_PORTC_PC4 |
                              TM_QUICC_PORTC_PC5;
                              
/* Clear the data register for pins 4 and 5*/
    tlQuiccPtr->pio_pcdat &=  ~( TM_QUICC_PORTC_PC4 |
                                 TM_QUICC_PORTC_PC5 );

/* Port B pin 12 is Ethernet TENA for HSB */
    tlQuiccPtr->pip_pbpar |= TM_QUICC_PORTB_PB12;
    tlQuiccPtr->pip_pbdir |= TM_QUICC_PORTB_PB12;
    
    scc1Vector = (unsigned long)(TM_QUICC_SCCA_INT | TM_QUICC_CICR_VECT);
    scc1Location = scc1Vector * 4;

/*
 * IMPLEMENTATION NOTE:
 *
 * The Actual SCC1 ISR Wrapper Function in installed here
 * It must call the tfQuiccScc1HandlerIsr function
 * This routine may be in "C" if it is supported
 * or assembly if not.  It is VERY RTOS dependent
 * so it cannot be provided with the driver
 *
 * With some RTOS's a wrapper function may not be
 * needed.  All the driver cares about is that the
 * function tfQuiccScc1HandlerIsr function is called
 * when the interrupt occurs.
 *
 * An example of this routine in assembly for uC/OS:

*
* The Interrupt Routine for a CPU32 to call the Quicc Driver
*
_tfQuiccScc1Isr
* Save the Registers
    MOVEM.L D0-D7/A0-A6,-(A7)

* uC/OS Call to tell RTOS we are in an ISR
    JSR _OSIntEnter

* Call the handler
    JSR _tfQuiccScc1HandlerIsr

* uC/OS Call to tell the RTOS that we are leaving the ISR
    JSR _OSIntExit

* Restore the registers
    MOVEM.L (A7)+,D0-D7/A0-A6

* Return from Exception (Interrupt)
    RTE 

 */

/* Install the wrapper function */
    tfKernelInstallIsrHandler(tfQuiccScc1Isr,scc1Location);
}

/* 
 * Enable the SCC's  to recieve and transmit 
 */
void tfQuiccEtherUp ( int channel )
{
    struct scc_regs TM_FAR * regs;


    regs = &tlQuiccPtr->scc_regs[channel];

/* clear all events on the SCC event register and CP event reg*/
    regs->scc_scce = 0xffff;

/* Clear the appropriate bit in the CISR */
    switch (channel)
    {
        case TM_QUICC_SCC1_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC1_INT_MASK;
            break;
        case TM_QUICC_SCC2_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC2_INT_MASK;
            break;
        case TM_QUICC_SCC3_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC3_INT_MASK;
            break;
        case TM_QUICC_SCC4_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC4_INT_MASK;
            break;
        default:
            break;
    }


/* 
 * mask the events we sich to interrupt on
 */
    regs->scc_sccm = 
/* An error in transmitting has occured */
                     TM_QUICC_INTR_ETHER_E_XMIT |
/* A buffer has completed transmitting */
                     TM_QUICC_INTR_ETHER_XMIT_B |
/* A ethernet frame has been received */
                     TM_QUICC_INTR_ETHER_RECV_F |
/* The busy condition has occured */
                     TM_QUICC_INTR_ETHER_BUSY;
    
/* SCC1 data sync reg  (The pattern for Ethernet)*/
    regs->scc_dsr =  TM_QUICC_SCC_FRAME_SYNC_ETHER;

/* protocol specific mode register */
    regs->scc_psmr = 
/* To disable the reception of Broadcast packets uncomment this line */
/*                     TM_QUICC_ETHER_BROADCAST | */
/* Turn on the CRC for ethernet */
                     TM_QUICC_ETHER_ENABLE_CRC | 
/* Ignore 22 bits for ethernet */
                     TM_QUICC_ETHER_NBITS_IGNORED;


/* SCC general mode reg */
    regs->scc_gsmrb = 
/* Carrier Detect Sampling */
                      TM_QUICC_CD_SAMPLE |
/* Clear to Send Sampling */
                      TM_QUICC_CTS_SAMPLE;

/* SCC general mode reg */
    regs->scc_gsmra = 
/* Transmit Clock Inversion */
                      TM_QUICC_XMIT_CLOCK_INV | 
/* Preamble Bit Length (48 always for ethernet) */
                      TM_QUICC_PREAMBLE_48 | 
/* Preamble Bit Pattern (Alternating '1' and '0' for ethernet */
                      TM_QUICC_PREAMBLE_PAT_10 | 
/* Ethernet Mode */
                      TM_QUICC_ETHERNET_PORT;

/* enable SCC1 receive/transmit operation */
    regs->scc_gsmra |= TM_QUICC_ENABLE_RECV | TM_QUICC_ENABLE_XMIT;
        
}

void tfQuiccDown (int channel)
{
    struct scc_regs TM_FAR * regs;


    regs = &tlQuiccPtr->scc_regs[channel];

/* clear all events on the SCC event register and CP event reg*/
    regs->scc_scce = 0xffff;


/* Disable SCCx receive/transmit operation */
    regs->scc_gsmra &= ~(TM_QUICC_ENABLE_RECV | TM_QUICC_ENABLE_XMIT);
        
}


/* 
 * Check to see if the quicc is online
 */
int tfQuiccOnline ( int channel )
{
    struct scc_regs TM_FAR * regs;
    int                      online;


    regs = &tlQuiccPtr->scc_regs[channel];

/* Disable SCCx receive/transmit operation */
    if (regs->scc_gsmra & (TM_QUICC_ENABLE_RECV | TM_QUICC_ENABLE_XMIT))
    {
        online = 1;
    }
    else
    {
        online = 0;
    }
    return online;    
}


/*
 * Disable the Ethernet on SCC1
 */
int tfQuiccEtherClose (ttUserInterface interfaceHandle)
{
    ttQuiccBufDescPtr       bufDescPtr;
    ttUserBufferPtr         userBufPtr;
    int                     i;


/* Shut down the SCC1 Ethernet hardware */
    tfQuiccDown(TM_QUICC_SCC1_CHANNEL);

/* 
 * Free leftover memory:
 *   First, send complete on any remaining packets in the transmit ring.
 */
    bufDescPtr = tlQuiccScc1XmitBufDescStartPtr;
    for (i = 0; i < TM_QUICC_ETHER_MAX_XMIT_BUFD; i++)
    {
/* Only call send complete when the last piece has been transmitted */
        if ( ((bufDescPtr->bdStatus) & TM_QUICC_XMIT_LAST ) &&
             (bufDescPtr->bdDataPtr != (char TM_FAR *) 0) )
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER );
        }
        bufDescPtr++;
    }


/* Next, free any buffers in the receive ring */
    bufDescPtr = tlQuiccScc1RecvBufDescStartPtr;
    userBufPtr = &(tlUserScc1BufferList[0]);
    for (i = 0; i < TM_QUICC_ETHER_MAX_RECV_BUFD; i++)
    {
        if ((bufDescPtr->bdDataPtr) != (char TM_FAR *) 0)
        {
            tfFreeDriverBuffer(*userBufPtr);
        }
        bufDescPtr++;
        userBufPtr++;
    }
        

    return TM_DEV_OKAY;
}

/* 
 * The main initialization routine for the quicc and the scc's 
 */
int tfQuiccEtherOpen (ttUserInterface interfaceHandle)
{

    unsigned long  quiccRegisters;
    int            errorCode;

    quiccRegisters = TM_QUICC_MBAR_LOC|0x00000001UL;
    tlQuiccPtr = (ttQuicc *)(quiccRegisters&0xfffffffcUL);

    errorCode = TM_DEV_OKAY;
/* 
 * Check that the number of transmit descriptors exceed to the number
 * of buffers per frame that the user specified in tfOpenInterface.
 */
    if (tfInterfaceScatterCount(interfaceHandle) >
            TM_QUICC_ETHER_MAX_XMIT_BUFD)
    {
        tfKernelWarning("tfQuiccEtherOpen",
                        "Scatter buffers exceed descriptors");
        errorCode = TM_DEV_ERROR;
    }

    if (errorCode == TM_DEV_OKAY)
    {

/* Initialize default Ethernet parameters */
        tfQuiccEtherScc1Init();
/* Create our transmit pool */
        tfQuiccCreateXmitPool(TM_QUICC_SCC1_CHANNEL,
                              TM_QUICC_ETHER_MAX_XMIT_BUFD);
/* Create our receive pool (with receive buffers) */
        tfQuiccCreateRecvPool(TM_QUICC_SCC1_CHANNEL,
                              TM_QUICC_ETHER_MAX_RECV_BUFD);
/* Initial board specific parameters */
        tfQuiccEtherScc1BoardInit();
/* Bring the ethernet scc port online */
        tfQuiccEtherUp(TM_QUICC_SCC1_CHANNEL);

/* Save our interfaceHandle for ISR use */
        tlInterfaceArray[TM_QUICC_SCC1_CHANNEL] = interfaceHandle;
/* Save whether the user enabled the Treck transmit queue */
        tlQuiccXmitQueueArray[TM_QUICC_SCC1_CHANNEL] =
                                      tfInterfaceXmitQueueOn(interfaceHandle);
    }
  
    return errorCode;
}


/* 
 * Interrupt service routine called from actual
 * ISR handlers
 */ 
void tfQuiccScc1HandlerIsr(void)
{
    ttUserInterface           interfaceHandle;
    struct scc_regs         * regs;
    int                       channel;
    unsigned short  TM_FAR  * bdStatusPtr;
#ifdef TM_TASK_SEND
    void            TM_FAR * TM_FAR * bdDataPtrPtr;
#endif /* TM_TASK_SEND */
    register unsigned short   event;
    int                       sentNotifyCount;
    unsigned long             sentNotifyBytes;
    int                       recvNotifyCount;
    
    channel = TM_QUICC_SCC1_CHANNEL;
    interfaceHandle = tlInterfaceArray[channel];
/* Points to SCC regs */
    regs = &tlQuiccPtr->scc_regs[channel]; 

/* Check to see what events have occured */
    event = regs->scc_scce;
/* Cleared event bits we hill handle */
    regs->scc_scce = event;
    recvNotifyCount = 0;
    sentNotifyCount = 0;
    sentNotifyBytes = 0UL;

#ifdef TM_TASK_SEND
    if( event & TM_QUICC_INTR_ETHER_XMIT_B ) 
    {
        bdStatusPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdStatus;
        bdDataPtrPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdDataPtr;
/* An ethernet frame has completed transmitting so notify the user*/
        while (    ( (*bdStatusPtr & TM_QUICC_XMIT_READY) == 0 )
                && ( *bdDataPtrPtr != (void TM_FAR *)0 ) )
        {
/* Zero it out so we know that we have freed this one before */
            *bdDataPtrPtr = (void TM_FAR *)0;
            if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
            {
                tlQuiccScc1XmitIsrBufDescPtr = tlQuiccScc1XmitBufDescStartPtr;
            }
            else
            {
                tlQuiccScc1XmitIsrBufDescPtr++;
            }
            bdStatusPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdStatus;
            bdDataPtrPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdDataPtr;
            sentNotifyCount++;
            sentNotifyBytes = (unsigned long)
                  (   sentNotifyBytes
                    + (unsigned long)tlQuiccScc1XmitIsrBufDescPtr->bdLength );
        }
    }
#endif /* TM_TASK_SEND */
    if ( event & TM_QUICC_INTR_ETHER_RECV_F )
    {
/*
 * loop on frames with the Empty bit and SHORT (notify) bit is NOT set
 * "==0" .
 * Only notify while the frames are not empty
 */
        bdStatusPtr = &tlQuiccScc1RecvIsrBufDescPtr->bdStatus;
        while (!(*bdStatusPtr & (TM_QUICC_RECV_EMPTY|TM_QUICC_NOTIFIED)))
        {
            recvNotifyCount++;
            *bdStatusPtr |= TM_QUICC_NOTIFIED;
            if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
            {
                tlQuiccScc1RecvIsrBufDescPtr = tlQuiccScc1RecvBufDescStartPtr;
            }
            else
            {
                tlQuiccScc1RecvIsrBufDescPtr++;
            }
            bdStatusPtr = &tlQuiccScc1RecvIsrBufDescPtr->bdStatus;
        }
    }
    if ( event &  TM_QUICC_INTR_ETHER_E_XMIT )
    {
/* An error on Transmit has occured */
        tfQuiccXmitError(interfaceHandle, channel);
    }
    if ( event & TM_QUICC_INTR_ETHER_BUSY )
    {
/* Busy Condition has been set */
        tfQuiccBusy(interfaceHandle,channel);
    }
    if ( (recvNotifyCount != 0) || (sentNotifyCount != 0) )
    {
        tfNotifyInterfaceIsr( interfaceHandle, recvNotifyCount,
                              sentNotifyCount, sentNotifyBytes, 0UL );
    }
/* Clear the appropriate bit in the CISR */
    tlQuiccPtr->intr_cisr |= TM_QUICC_SCC1_INT_MASK;
}

/*
 * Send a packet to the network
 */
int tfQuiccEtherSend ( ttUserInterface  interfaceHandle,
                       char TM_FAR    * bufferPtr,
                       int              length,
                       int              flag )
{
    ttQuiccBufDescPtr       bufDescPtr;
    ttQuiccBufDescPtr       startBufDescPtr;
    unsigned short TM_FAR * bdStatusPtr;
    void  TM_FAR * TM_FAR * bdDataPtrPtr;
    int                     errorCode;

    errorCode = TM_DEV_OKAY;
/* Wait for a free buffer descriptor */    
    if (tlQuiccScc1XmitBufDescPtr->bdStatus & TM_QUICC_XMIT_READY)
    {
        if (tlQuiccXmitQueueArray[TM_QUICC_SCC1_CHANNEL])
        {
/* If user uses a transmit queue, queue the buffer in the transmit queue */
            errorCode = TM_DEV_ERROR;
            goto quiccSendExit;
        }
        else
        {
/* Otherwise */
            do
            {
/*
 * Do nothing function if user does not use transmit task. Otherwise will
 * yield the CPU.
 */
                tfInterfaceSpinLock(interfaceHandle);
                tvQuiccScc1XmitWait++;
            }
            while (tlQuiccScc1XmitBufDescPtr->bdStatus & TM_QUICC_XMIT_READY);
        }
    }
#ifndef TM_TASK_SEND
/* Inlined send complete */
    bdStatusPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdStatus;
    bdDataPtrPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdDataPtr;
/* An ethernet frame has completed transmitting so notify the user*/
    while (   (    ((*bdStatusPtr & TM_QUICC_XMIT_READY) == 0)
                && (tlQuiccScc1XmitIsrBufDescPtr != tlScatterBufferStart) )
           && ( *bdDataPtrPtr != (void TM_FAR *)0 ) )
/* 
 * Watch out for a scattered frame we have not yet finished
 * loading up
 */
    {
/* Zero it out so we know that we have freed this one before */
        *bdDataPtrPtr = (void TM_FAR *)0;
        if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
        {
            tlQuiccScc1XmitIsrBufDescPtr = tlQuiccScc1XmitBufDescStartPtr;
        }
        else
        {
            tlQuiccScc1XmitIsrBufDescPtr++;
        }
/* Only call send complete when the last piece has been transmitted */
        if (*bdStatusPtr & TM_QUICC_XMIT_LAST)
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER );
        }
        bdStatusPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdStatus;
        bdDataPtrPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdDataPtr;
    }
#endif /* NOT TM_TASK_SEND */
    bufDescPtr=tlQuiccScc1XmitBufDescPtr++;
    bdStatusPtr=&bufDescPtr->bdStatus;
    if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
    {
        tlQuiccScc1XmitBufDescPtr = tlQuiccScc1XmitBufDescStartPtr;
    }
    bufDescPtr->bdDataPtr=(void TM_FAR *)bufferPtr;
    bufDescPtr->bdLength = (unsigned short)length;
/* Clear the previous status (but preserve the WRAP bit) */
    bufDescPtr->bdStatus &= TM_QUICC_XMIT_WRAP;
/* If this is the last buffer in the frame then xmit it */
    if (flag==TM_USER_BUFFER_LAST)
    {
/* Now mark the last bd ready */
        bdStatusPtr=&bufDescPtr->bdStatus;
        *bdStatusPtr |= (TM_QUICC_XMIT_READY |
                        TM_QUICC_XMIT_PAD |

#ifdef TM_TASK_SEND
                        TM_QUICC_XMIT_INTR | 
#endif /* TM_TASK_SEND */

                        TM_QUICC_XMIT_LAST |
                        TM_QUICC_XMIT_CRC);
/* Check to see if we are xmitting scattered data */
        if (tlScatterBufferStart != (ttQuiccBufDescPtr)0)
        {
/* We have at least one more piece to xmit if we are here */
            startBufDescPtr=tlScatterBufferStart;
            tlScatterBufferStart=(ttQuiccBufDescPtr)0;
/* 
 * Go ahead and xmit the first packets (with more set) 
 * by walking the list backwards
 * We do this so that we don't need a critical section
 * in order to keep from getting gaps inbetween the 
 * pieces
 * All we need to do here is update the
 * ready flag
 */
            do
            {
                if (bufDescPtr==tlQuiccScc1XmitBufDescStartPtr)
                {
/* We are at the front of the list so go to the end */
                    bufDescPtr += TM_QUICC_ETHER_MAX_XMIT_BUFD;
                }
                else
                {
/* Go back one buffer descriptor */
                    bufDescPtr--;
                }
                bdStatusPtr=&bufDescPtr->bdStatus;
                *bdStatusPtr |= TM_QUICC_XMIT_READY;
            } while (bufDescPtr != startBufDescPtr);
        }        
    }
    else
    {
/* More buffers to come */
        if (tlScatterBufferStart == (ttQuiccBufDescPtr)0)
        {
            tlScatterBufferStart=bufDescPtr;
        }
    }
    tvQuiccScc1SendPacketCount++;

quiccSendExit:
    return errorCode;
}


/*
 * A Transmit Error has occured
 */ 
void tfQuiccXmitError ( ttUserInterface interfaceHandle, int channel )
{
    TM_UNREF_IN_ARG(interfaceHandle);
/* issue the RESTART TRANSMIT command to the CP */
    channel = TM_QUICC_RESTART_TX | ( channel << 6);
    tfQuiccIssueCommand((unsigned short)channel);
}


/*
 * tfQuiccEtherReceive
 * Receive a buffer from the chip and pass back to the stack
 */
int tfQuiccEtherReceive ( ttUserInterface        interfaceHandle,
                          char TM_FAR * TM_FAR * dataPtrPtr,
                          int           TM_FAR * lengthPtr,
                          ttUserBufferPtr        userBufferHandlePtr )
{
    ttQuiccBufDescPtr       bufDescPtr;
    unsigned short TM_FAR * bdStatusPtr;
    int                     errorCode;
    
    TM_UNREF_IN_ARG(interfaceHandle);
    errorCode=TM_DEV_OKAY;
    
    bdStatusPtr = &tlQuiccScc1RecvBufDescPtr->bdStatus;

/* Skip pass error packets who are not empty */
    if (    (!(*bdStatusPtr &  TM_QUICC_RECV_EMPTY))
         && (*bdStatusPtr & TM_QUICC_ERROR_RECV) )
    {
        tvQuiccScc1RecvErrorCount++;
/* Return the userBuffer Handle for this data pointer */
        if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
        {
            tlQuiccScc1RecvBufDescPtr = tlQuiccScc1RecvBufDescStartPtr;
            tlUserScc1BufferPtr = tlUserScc1BufferList;
        }
        else
        {
            tlQuiccScc1RecvBufDescPtr++;
            tlUserScc1BufferPtr++;
        }
        errorCode = TM_EWOULDBLOCK;
    }
    else
    {
/* Check to see if we own the Buffer Descriptor */
        if (*bdStatusPtr & TM_QUICC_RECV_EMPTY)
        {
        
/* Will only happen when refill cleans up error before we are called here */
            tvQuiccScc1RecvErrorCount++;
/* Return the userBuffer Handle for this data pointer */
            if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
            {
                tlQuiccScc1RecvBufDescPtr = tlQuiccScc1RecvBufDescStartPtr;
                tlUserScc1BufferPtr = tlUserScc1BufferList;
            }
            else
            {
                tlQuiccScc1RecvBufDescPtr++;
                tlUserScc1BufferPtr++;
            }
            errorCode = TM_EWOULDBLOCK;
        }
        else
        {
            bufDescPtr = tlQuiccScc1RecvBufDescPtr++;
/* Return the userBuffer Handle for this data pointer */
            *userBufferHandlePtr = *tlUserScc1BufferPtr++;
            if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
            {
                tlQuiccScc1RecvBufDescPtr = tlQuiccScc1RecvBufDescStartPtr;
                tlUserScc1BufferPtr=tlUserScc1BufferList;
            }
/* We are returning the pointer to the data */
            *dataPtrPtr = (char TM_FAR *)(bufDescPtr->bdDataPtr);

/* Zero out the bufferPtr so we can refill it later */
            bufDescPtr->bdDataPtr = (void TM_FAR *)0;

/* We are returning the length */
            *lengthPtr = bufDescPtr->bdLength;
/* Up our packet count */
            tvQuiccScc1RecvPacketCount++;
        }
    }

    tfQuiccRefillScc1RecvPool();

    return(errorCode);
}

/*
 * SET GROUP ADDRESS COMMAND for SCC1:
 * (pages 7-7, 7-8, 7-253, 7-254, and 7-258 of the MC68360 manual)
 * Reset bit (bit 15) is 0. Bits 12-14 are reserved.
 * Opcode (bits 8 - 11) is 8.
 * Channel number bits 4-7 is 0.
 * Flag (bit 0) is set in tfQuiccIssueCommand
 */
#define TM_QUICC_SET_GROUP_ADDRESS          0x0800

/*
 * Can be used to configure the device or refill the receive pool, or
 * enable reception of a list of multicast addresses.
 */
int tfQuiccIoctl (ttUserInterface interfaceHandle, int flag,
                  void TM_FAR * optionPtr, int optionLen)
{
#if (defined(TM_IGMP) || defined(TM_6_USE_MLD))

    struct ethernet_pram    * enetPramAddrPtr;
    struct scc_regs         * regsPtr;
    unsigned char    TM_FAR * etherPtr;
    unsigned char    TM_FAR * msbHashPtr;
    int                       index;

    if ( (flag == TM_DEV_SET_MCAST_LIST) || (flag == TM_DEV_SET_ALL_MCAST) )
    {
/* SCC1 Registers */
        regsPtr = &tlQuiccPtr->scc_regs[TM_QUICC_SCC1_CHANNEL];

/*
 * Disable SCC1 receive/transmit operation, before clearing the Group
 * address hash table
 */
        regsPtr->scc_gsmra &= ~(TM_QUICC_ENABLE_RECV | TM_QUICC_ENABLE_XMIT);

        enetPramAddrPtr = &tlQuiccPtr->pram[TM_QUICC_SCC1_CHANNEL].enet_scc;
/*
 * Clear hash table. If optionLen is zero, this will clear reception
 * of all multicast addresses (as intended).
 */
        enetPramAddrPtr->gaddr1 = 0;
        enetPramAddrPtr->gaddr2 = 0;
        enetPramAddrPtr->gaddr3 = 0;
        enetPramAddrPtr->gaddr4 = 0;

        if (flag == TM_DEV_SET_MCAST_LIST)
        {
/* Most significant byte of entry in the hash table is in low memory. */
            msbHashPtr = (unsigned char TM_FAR *)
                                           (&(enetPramAddrPtr->taddr_h));
            etherPtr = (unsigned char TM_FAR *)optionPtr;
/* Add multicast address list */
            while (optionLen > 0)
            {
                index = 0;
                while (index < 6)
                {
/* Copy Ethernet address MSB (high memory) to Hash table MSB (low memory) */
                    msbHashPtr[index] = etherPtr[6 - 1 - index];
                    index++;
                }
                tfQuiccIssueCommand(TM_QUICC_SET_GROUP_ADDRESS);
/* Next Ethernet address in the list */
                optionLen--;
                etherPtr += 6;
            }
        }
        else
        {
            if (flag == TM_DEV_SET_ALL_MCAST)
            {
/* Enable reception of all multicast addresses */
                enetPramAddrPtr->gaddr1 = 0xFFFF;
                enetPramAddrPtr->gaddr2 = 0xFFFF;
                enetPramAddrPtr->gaddr3 = 0xFFFF;
                enetPramAddrPtr->gaddr4 = 0xFFFF;
            }
        }
/* Enable SCC1 receive/transmit operation */
        regsPtr->scc_gsmra |= (TM_QUICC_ENABLE_RECV | TM_QUICC_ENABLE_XMIT);
    }

#else /* !(TM_IGMP or TM_6_USE_MLD) */

    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);

#endif /* !(TM_IGMP or TM_6_USE_MLD) */

    if (flag & TM_QUICC_REFILL_SCC1)
    {
        tfQuiccRefillScc1RecvPool();
    }
#ifndef TM_TASK_SEND
    if (flag & TM_QUICC_SEND_COMPLETE)
    {
        tfQuiccScc1SendComplete(interfaceHandle);
    }
#endif /* NOT TM_TASK_SEND */
    return TM_DEV_OKAY;
}

/*
 * Walk through receive list and allocate new buffers
 * after they have been processed
 */
void tfQuiccRefillScc1RecvPool (void)
{
    ttUserBuffer            userBuffer;
    unsigned short TM_FAR * bdStatusPtr;
    void           TM_FAR * dataPtr;
    void           TM_FAR * TM_FAR * bdDataPtrPtr;

    bdDataPtrPtr = &tlQuiccScc1RecvRefillBufDescPtr->bdDataPtr;
    bdStatusPtr  = &tlQuiccScc1RecvRefillBufDescPtr->bdStatus;
/* Loop while it is empty or an error */
    while (    (*bdDataPtrPtr == (void TM_FAR *)0)
            || (*bdStatusPtr & TM_QUICC_ERROR_RECV) )
    {
/* Check to see if it is an error */
        if (!(*bdStatusPtr & TM_QUICC_ERROR_RECV))
        {
/* If it is not get a new buffer, otherwise just reuse it */
/* bdDataPtr need to be aligned on a 4-byte boundary */
            dataPtr = (void TM_FAR *)
                    (tfGetEthernetBuffer(&userBuffer) - TM_ETHER_IP_ALIGN);
            if (userBuffer == (ttUserBuffer)0)
            {
/* No memory so stop the refill */
                break;
            }
            tlQuiccScc1RecvRefillBufDescPtr->bdDataPtr = dataPtr;
            *tlUserScc1RefillBufferPtr = userBuffer;
        }
        tlQuiccScc1RecvRefillBufDescPtr->bdLength = 0;
/* Clear the status Except for the wrap bit */
        *bdStatusPtr &= TM_QUICC_RECV_WRAP;
/* Mark this BD as ready to use */
        *bdStatusPtr |= (TM_QUICC_RECV_EMPTY|TM_QUICC_RECV_INTR);
        if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
        {
            tlQuiccScc1RecvRefillBufDescPtr = tlQuiccScc1RecvBufDescStartPtr;
            tlUserScc1RefillBufferPtr = tlUserScc1BufferList;
        }
        else
        {
            tlQuiccScc1RecvRefillBufDescPtr++;
            tlUserScc1RefillBufferPtr++;
        }
        bdDataPtrPtr = &tlQuiccScc1RecvRefillBufDescPtr->bdDataPtr;
        bdStatusPtr  = &tlQuiccScc1RecvRefillBufDescPtr->bdStatus;
    }
}

#ifndef TM_TASK_SEND

/*
 * Walk through send list and post send completes
 */
void tfQuiccScc1SendComplete(ttUserInterface interfaceHandle)
{
    unsigned short TM_FAR  * bdStatusPtr;
    void            TM_FAR * TM_FAR * bdDataPtrPtr;

    bdStatusPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdStatus;
    bdDataPtrPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdDataPtr;
/* An ethernet frame has completed transmitting so notify the user*/
    while (    ( (*bdStatusPtr & TM_QUICC_XMIT_READY) == 0 )
            && ( tlQuiccScc1XmitIsrBufDescPtr != tlScatterBufferStart )
            && ( *bdDataPtrPtr != (void TM_FAR *)0 ) )
    {
/* Zero it out so we know that we have freed this one before */
        *bdDataPtrPtr = (void TM_FAR *)0;
        if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
        {
            tlQuiccScc1XmitIsrBufDescPtr = tlQuiccScc1XmitBufDescStartPtr;
        }
        else
        {
            tlQuiccScc1XmitIsrBufDescPtr++;
        }
/* Only call send complete when the last piece has been transmitted */
        if (*bdStatusPtr & TM_QUICC_XMIT_LAST)
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER );
        }
        bdStatusPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdStatus;
        bdDataPtrPtr = &tlQuiccScc1XmitIsrBufDescPtr->bdDataPtr;
    }
}
#endif /* NOT TM_TASK_SEND */




 
/*
 *
 * SERIAL INTERFACE   (SCC4)
 *
 */




/*
 * tfQuiccSerialReceive
 * Receive a buffer from the chip and pass back to the stack
 */
int tfQuiccSerialReceive( ttUserInterface        interfaceHandle,
                          char TM_FAR * TM_FAR * dataPtrPtr,
                          int           TM_FAR * lengthPtr,
                          ttUserBufferPtr        userBufferHandlePtr )
{
    ttQuiccBufDescPtr        bufDescPtr;
    unsigned short TM_FAR  * bdStatusPtr;
    int                      errorCode;
    
    TM_UNREF_IN_ARG(interfaceHandle);
    errorCode = TM_DEV_OKAY;
    
    bdStatusPtr = &tlQuiccScc4RecvBufDescPtr->bdStatus;

/* Skip pass error packets who are not empty */
    if (    (!(*bdStatusPtr &  TM_QUICC_RECV_EMPTY))
         && (*bdStatusPtr & TM_QUICC_ERROR_RECV) )
    {
        tvQuiccScc4RecvErrorCount++;
/* Return the userBuffer Handle for this data pointer */
        if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
        {
            tlQuiccScc4RecvBufDescPtr = tlQuiccScc4RecvBufDescStartPtr;
            tlUserScc4BufferPtr = tlUserScc4BufferList;
        }
        else
        {
            tlQuiccScc4RecvBufDescPtr++;
            tlUserScc4BufferPtr++;
        }
        errorCode = TM_EWOULDBLOCK;
    }
    else
    {
/* Check to see if we own the Buffer Descriptor */
        if (*bdStatusPtr & TM_QUICC_RECV_EMPTY)
        {
        
/* Will only happen when refill cleans up error before we are called here */
            tvQuiccScc4RecvErrorCount++;
/* Return the userBuffer Handle for this data pointer */
            if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
            {
                tlQuiccScc4RecvBufDescPtr = tlQuiccScc4RecvBufDescStartPtr;
                tlUserScc4BufferPtr = tlUserScc4BufferList;
            }
            else
            {
                tlQuiccScc4RecvBufDescPtr++;
                tlUserScc4BufferPtr++;
            }
            errorCode = TM_EWOULDBLOCK;
        }
        else
        {
            bufDescPtr = tlQuiccScc4RecvBufDescPtr++;
/* Return the userBuffer Handle for this data pointer */
            *userBufferHandlePtr = *tlUserScc4BufferPtr++;
            if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
            {
                tlQuiccScc4RecvBufDescPtr = tlQuiccScc4RecvBufDescStartPtr;
                tlUserScc4BufferPtr = tlUserScc4BufferList;
            }
/* We are returning the pointer to the data */
            *dataPtrPtr = (char TM_FAR *)(bufDescPtr->bdDataPtr);

/* Zero out the bufferPtr so we can refill it later */
            bufDescPtr->bdDataPtr = (void TM_FAR *)0;

/* We are returning the length */
            *lengthPtr = bufDescPtr->bdLength;
/* Up our packet count */
            tvQuiccScc4RecvPacketCount++;
        }
    }
    return(errorCode);
}

/* 
 * Initialize SCC4 to be an UART channel
 */
void tfQuiccSerialScc4Init (void)
{
    struct uart_pram TM_FAR * rs232_pram;
    struct scc_regs  TM_FAR * regs;
    int                       i;


    rs232_pram = &tlQuiccPtr->pram[3].scc.pscc.u;
    regs = &tlQuiccPtr->scc_regs[3];

/* General SCC initializations */

/* Disable transmit/receive */
    regs->scc_gsmra = 0;    /* reset General SCC Mode Register */

/* points to start of receiving ring */
    rs232_pram->rbase =   TM_QUICC_SCC4_RECV_BASE;
/* points to start of transmiting ring */
    rs232_pram->tbase =   TM_QUICC_SCC4_XMIT_BASE;   

/* Function Codes for Transmit and Receive */
    rs232_pram->rfcr =    TM_QUICC_SCC4_RECV_FUNC_CODE;
    rs232_pram->tfcr =    TM_QUICC_SCC4_XMIT_FUNC_CODE;

/* Maximum recieve buffer length */
    rs232_pram->mrblr =   TM_QUICC_SCC4_MAX_RECV_BUF_LEN;

/* UART specific initializations */
    
/* maximum idle characters */
    rs232_pram->max_idl = TM_QUICC_SCC4_MAX_IDL16;        
/* Number of break characters to send on a STOP TRANSMIT */
    rs232_pram->brkcr = TM_QUICC_SCC4_BREAK_COUNT;

/* Initialize error count registers (parity error, frame error,etc) */
    rs232_pram->parec = 0;
    rs232_pram->frmer = 0;
    rs232_pram->nosec = 0;
    rs232_pram->brkec = 0;

/* Set our control characters */
    rs232_pram->cc[0] = TM_QUICC_UART_LAST_CHAR;
    for ( i = 0; i < TM_QUICC_NUM_CONTROL_CHARS; i++ )
    {
/*
        rs232_pram->cc[i] = tlQuiccControlChars[i].ch & 0xFF;
        if (tlQuiccControlChars[i].reject)
        {
            rs232_pram->cc[i] |= TM_QUICC_UART_REJECT_CHAR;
        }
*/
    }
    rs232_pram->cc[i] |= TM_QUICC_UART_LAST_CHAR;

    rs232_pram->rccm =   0x00FF
                       | TM_QUICC_UART_REJECT_CHAR
                       | TM_QUICC_UART_LAST_CHAR;

    


/* Tell the CP to initialize our Recv/Xmit parameters */
    tfQuiccIssueCommand(TM_QUICC_INIT_RXTX_SCC4);
}

#ifndef TM_TASK_SEND
/*
 * Walk through send list and post send completes
 */
void tfQuiccScc4SendComplete ( ttUserInterface interfaceHandle )
{
    unsigned short           TM_FAR  * bdStatusPtr;
    void            TM_FAR * TM_FAR * bdDataPtrPtr;

    bdStatusPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdStatus;
    bdDataPtrPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdDataPtr;
/* An ethernet frame has completed transmitting so notify the user*/
    while (    ((*bdStatusPtr & TM_QUICC_XMIT_READY) == 0)
            && (tlQuiccScc4XmitIsrBufDescPtr != tlScatterBufferStart)
            && (*bdDataPtrPtr != (void TM_FAR *)0) )
    {
/* Zero it out so we know that we have freed this one before */
        *bdDataPtrPtr = (void TM_FAR *)0;
        if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
        {
            tlQuiccScc4XmitIsrBufDescPtr = tlQuiccScc4XmitBufDescStartPtr;
        }
        else
        {
            tlQuiccScc4XmitIsrBufDescPtr++;
        }
/* Only call send complete when the last piece has been transmitted */
        if (*bdStatusPtr & TM_QUICC_XMIT_LAST)
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER );
        }
        bdStatusPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdStatus;
        bdDataPtrPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdDataPtr;
    }
}
#endif /* NOT TM_TASK_SEND */


/* 
 * Initialize SCC4 to be a UART channel
 */
void tfQuiccSerialScc4BoardInit (void)
{
    unsigned long  scc4Location;
    char  TM_FAR * cs7out;
    long           divider;
    unsigned long  scc4Vector;

    tlQuiccPtr->sim_mcr =   0x608f;

/*
 * IMPLEMENTATION NOTE:
 *
 * This routine is very board specific
 * In this example we use the Atlas Communications Engines
 * HSB devlopment board for an example
 * Please refer to your boards design to modify this routine
 */

/* Atlas HSB Setup */
    tlQuiccPtr->si_sicr |=   TM_QUICC_SI_SCC4_RCS_BRG3 |
                             TM_QUICC_SI_SCC4_TCS_BRG3;

/* Set the divider to 0x130 and RESET and no DIV16 
 * NOTE: The divider starts in bit location 1 not 0
 * The formula is
 * Divider=ClockRate/DesiredBaudRate/16-1*2
 * Example for 24.576 clock
 * 318=((24576000/9600)/16)-1*2
 * 0x130=318
 */
    divider = (((TM_PROC_CLOCK_RATE/TM_QUICC_BAUD_RATE)/16)-1)*2;
    tlQuiccPtr->brgc3 = divider | TM_QUICC_BRGC_RST;
/*
    tlQuiccPtr->brgc3 = (((TM_PROC_CLOCK_RATE/TM_QUICC_BAUD_RATE)>>4-1)<<1)
                            |TM_QUICC_BRGC_RST;
*/

/* Set USI = RS-232 */
    tlQuiccPtr->memc_br7 = 0x88000001UL;
    tlQuiccPtr->memc_or7 = 0x2FFF0005UL;

    cs7out = (char *) 0x88000001UL;
    *cs7out = 0x02;


/* Set PA12 = BRGO3 output */
    tlQuiccPtr->pio_papar |= TM_QUICC_PORTA_PA12;
    tlQuiccPtr->pio_padir |= TM_QUICC_PORTA_PA12;

/* Set PB12 = 1 */
    tlQuiccPtr->pip_pbpar &= (unsigned long)~TM_QUICC_PORTB_PB17;
    tlQuiccPtr->pip_pbdir |=  TM_QUICC_PORTB_PB17;
    tlQuiccPtr->pip_pbdat &= (unsigned long)~TM_QUICC_PORTB_PB17;

/* Make PortA Pins 6,7 for onchip use */
    tlQuiccPtr->pio_padir &= ~(TM_QUICC_PORTA_PA6 | TM_QUICC_PORTA_PA7);
    
/* 
 * pins 6,7 for SCC4 Xmit and Recv
 */
    tlQuiccPtr->pio_papar |= TM_QUICC_PORTA_PA6 |
                             TM_QUICC_PORTA_PA7;

    tlQuiccPtr->intr_cicr=TM_QUICC_CICR_DEFAULT;
    
/* Clear the pending and in-service registers */
    tlQuiccPtr->intr_cipr = TM_QUICC_SCC4_INT_MASK;
    tlQuiccPtr->intr_cisr = TM_QUICC_SCC4_INT_MASK;

/* Set the CP mask register to accept interrupts from SCC4 */
    tlQuiccPtr->intr_cimr |= TM_QUICC_SCC4_INT_MASK;

    tlQuiccPtr->sdma_sdcr =  TM_QUICC_SDMA_INT_MASK |
                             TM_QUICC_SDMA_ARB_ID;

    
/* PortC Pins 10 & 11 ar CD and CTS */
    tlQuiccPtr->pio_pcpar &=   ~(TM_QUICC_PORTC_PC10 |
                                 TM_QUICC_PORTC_PC11);

    tlQuiccPtr->pio_pcdir &=   ~(TM_QUICC_PORTC_PC10 |
                                 TM_QUICC_PORTC_PC11);
                                 
/*
 * Use Pins 10 and 11 (SCC4 CTS and CD)
 */
    tlQuiccPtr->pio_pcso |=   TM_QUICC_PORTC_PC10|
                              TM_QUICC_PORTC_PC11;


/* Port B pin 12 is Ethernet TENA for HSB */
/*
    tlQuiccPtr->pip_pbpar |= TM_QUICC_PORTB_PB15;
    tlQuiccPtr->pip_pbdir |= TM_QUICC_PORTB_PB15;
*/
    
    scc4Vector = (unsigned long)(TM_QUICC_SCCD_INT | TM_QUICC_CICR_VECT);
    scc4Location = scc4Vector * 4;

/*
 * IMPLEMENTATION NOTE:
 *
 * The Actual SCC4 ISR Wrapper Function in installed here
 * It must call the tfQuiccScc1HandlerIsr function
 * This routine may be in "C" if it is supported
 * or assembly if not.  It is VERY RTOS dependent
 * so it cannot be provided with the driver
 *
 * With some RTOS's a wrapper function may not be
 * needed.  All the driver cares about is that the
 * function tfQuiccScc1HandlerIsr function is called
 * when the interrupt occurs.
 *
 * An example of this routine in assembly for uC/OS:

*
* The Interrupt Routine for a CPU32 to call the Quicc Driver
*
_tfQuiccScc4Isr
* Save the Registers
    MOVEM.L D0-D7/A0-A6,-(A7)

* uC/OS Call to tell RTOS we are in an ISR
    JSR _OSIntEnter

* Call the handler
    JSR _tfQuiccScc4HandlerIsr

* uC/OS Call to tell the RTOS that we are leaving the ISR
    JSR _OSIntExit

* Restore the registers
    MOVEM.L (A7)+,D0-D7/A0-A6

* Return from Exception (Interrupt)
    RTE 

 */

/* Install the wrapper function */
/* NOTE: TO ADD ISR HANDLER */

    tfKernelInstallIsrHandler(tfQuiccScc4Isr,scc4Location); 
}

/* 
 * The main initialization routine for the quicc and the scc's 
 */
int tfQuiccSerialOpen ( ttUserInterface interfaceHandle )
{

    unsigned long            quiccRegisters;
    struct scc_regs TM_FAR * regs;

    quiccRegisters = TM_QUICC_MBAR_LOC|0x00000001UL;
    tlQuiccPtr = (ttQuicc *)(quiccRegisters&0xfffffffcUL);

/* Check to see if we have already initialized this channel */

    regs = &tlQuiccPtr->scc_regs[TM_QUICC_SCC4_CHANNEL];

    if (regs->scc_gsmra == 0UL)
    {
        tlScatterBufferStart = (ttQuiccBufDescPtr)0;
/* Initialize default UART parameters */
        tfQuiccSerialScc4Init();

/* Create and init our send and recieve pools */    
        tfQuiccCreateXmitPool( TM_QUICC_SCC4_CHANNEL,
                               TM_QUICC_SERIAL_MAX_XMIT_BUFD);
        tfQuiccCreateRecvPool( TM_QUICC_SCC4_CHANNEL,
                               TM_QUICC_SERIAL_MAX_RECV_BUFD);
    
/* Initial board specific parameters */
        tfQuiccSerialScc4BoardInit();
        tlQuiccXmitBufNum = 0;
/* Save our interfaceHandle for ISR use */
        tlInterfaceArray[TM_QUICC_SCC4_CHANNEL] = interfaceHandle;
    }
/* Bring the UART scc port online */
    tfQuiccSerialUp(TM_QUICC_SCC4_CHANNEL);



    return TM_DEV_OKAY;
}


/*
 * Disable the Serial (RS-232) on SCC4
 */
int tfQuiccSerialClose (ttUserInterface interfaceHandle)
{
    TM_UNREF_IN_ARG(interfaceHandle);
    tfQuiccDown(TM_QUICC_SCC4_CHANNEL);
    return 0;
}


/* 
 * Enable the SCC's  to recieve and transmit 
 */
void tfQuiccSerialUp (int channel)
{
    struct scc_regs TM_FAR * regs;


    regs = &tlQuiccPtr->scc_regs[channel];

/* clear all events on the SCC event register and CP event reg*/
    regs->scc_scce = 0xffff;

/* Clear the appropriate bit in the CISR */
    switch (channel)
    {
        case TM_QUICC_SCC1_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC1_INT_MASK;
            break;
        case TM_QUICC_SCC2_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC2_INT_MASK;
            break;
        case TM_QUICC_SCC3_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC3_INT_MASK;
            break;
        case TM_QUICC_SCC4_CHANNEL:
            tlQuiccPtr->intr_cisr = TM_QUICC_SCC4_INT_MASK;
            break;
        default:
            break;
    }

/* 
 * mask the events we sich to interrupt on
 */
    regs->scc_sccm = 
/* A buffer has completed transmitting */
                     TM_QUICC_INTR_SERIAL_XMIT_B |
/* A ethernet frame has been received */
                     TM_QUICC_INTR_SERIAL_RECV_B |
/* The busy condition has occured */
                     TM_QUICC_INTR_SERIAL_BUSY;
    
/* protocol specific mode register */
    regs->scc_psmr = 
/* CTS Flow Control On */
                     TM_QUICC_UART_FLOW | 
/* CHaracters are 8 Bits */
                     TM_QUICC_CHAR_8;
                     


/* SCC general mode reg */
/* Set receive FIFO to 1 byte for UART mode */
    regs->scc_gsmrb = TM_QUICC_UART_RECV_FIFO_1;

/* SCC general mode reg */
    regs->scc_gsmra = 
/* 16x Sample Rate */
                      TM_QUICC_UART_RECV_SAMPLE | 
                      TM_QUICC_UART_XMIT_SAMPLE | 
/* Serial UART Mode */
                      TM_QUICC_UART;

/* enable SCC1 receive/transmit operation */
    regs->scc_gsmra |= TM_QUICC_ENABLE_RECV | TM_QUICC_ENABLE_XMIT;
     
}


/* 
 * Interrupt service routine called from actual
 * ISR handlers
 */ 
void tfQuiccScc4HandlerIsr (void)
{
    ttUserInterface          interfaceHandle;
    struct scc_regs        * regs;
    unsigned short TM_FAR  * bdStatusPtr;
#ifdef TM_TASK_SEND
    void   TM_FAR * TM_FAR * bdDataPtrPtr;
#endif /* TM_TASK_SEND */
    int                      channel;
    register unsigned short  event;
    int                      sentNotifyCount;
    unsigned long            sentNotifyBytes;
    int                      recvNotifyCount;
    
    channel = TM_QUICC_SCC4_CHANNEL;
    interfaceHandle = tlInterfaceArray[channel];
/* Points to SCC regs */
    regs = &tlQuiccPtr->scc_regs[channel]; 

/* Check to see what events have occured */
    event = regs->scc_scce;
/* Cleared event bits we hill handle */
    regs->scc_scce = event;

    recvNotifyCount = 0;
    sentNotifyCount = 0;
    sentNotifyBytes = 0UL;

#ifdef TM_TASK_SEND
    if( event & TM_QUICC_INTR_SERIAL_XMIT_B ) 
    {
        bdStatusPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdStatus;
        bdDataPtrPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdDataPtr;
/* An ethernet frame has completed transmitting so notify the user*/
        while (    ((*bdStatusPtr & TM_QUICC_XMIT_READY) == 0)
                && (*bdDataPtrPtr != (void TM_FAR *)0))
        {
/* Zero it out so we know that we have freed this one before */
            *bdDataPtrPtr = (void TM_FAR *)0;
            if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
            {
                tlQuiccScc4XmitIsrBufDescPtr = tlQuiccScc4XmitBufDescStartPtr;
            }
            else
            {
                tlQuiccScc4XmitIsrBufDescPtr++;
            }
            bdStatusPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdStatus;
            bdDataPtrPtr = &tlQuiccScc4XmitIsrBufDescPtr->bdDataPtr;
            sentNotifyCount++;
            sentNotifyBytes = (unsigned long)
                  (   sentNotifyBytes
                    + (unsigned long)tlQuiccScc4XmitIsrBufDescPtr->bdLength );
        }
    }
#endif /* TM_TASK_SEND */
    if ( event & TM_QUICC_INTR_SERIAL_RECV_B )
    {
/*
 * loop on frames with the Empty bit and SHORT (notify) bit is NOT set
 * "== 0" .
 * Only notify while the frames are not empty
 */
        bdStatusPtr = &tlQuiccScc4RecvIsrBufDescPtr->bdStatus;
        while (!(*bdStatusPtr & (TM_QUICC_RECV_EMPTY|TM_QUICC_NOTIFIED)))
        {
            recvNotifyCount++;
            *bdStatusPtr |= TM_QUICC_NOTIFIED;
            if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
            {
                tlQuiccScc4RecvIsrBufDescPtr = tlQuiccScc4RecvBufDescStartPtr;
            }
            else
            {
                tlQuiccScc4RecvIsrBufDescPtr++;
            }
            bdStatusPtr = &tlQuiccScc4RecvIsrBufDescPtr->bdStatus;
        }
    }
    if ( event & TM_QUICC_INTR_SERIAL_BUSY )
    {
/* Busy Condition has been set */
        tfQuiccBusy(interfaceHandle,channel);
    }
    if ( (recvNotifyCount != 0) || (sentNotifyCount != 0) )
    {
        tfNotifyInterfaceIsr( interfaceHandle, recvNotifyCount,
                              sentNotifyCount, sentNotifyBytes, 0UL );
    }
/* Clear the appropriate bit in the CISR */
    tlQuiccPtr->intr_cisr |= TM_QUICC_SCC4_INT_MASK;
}



/*
 * Refill the serial recieve pool
 */
void tfQuiccRefillScc4RecvPool (void)
{
    ttUserBuffer             userBuffer;
    unsigned short  TM_FAR * bdStatusPtr;
    void            TM_FAR * dataPtr;
    void            TM_FAR * TM_FAR * bdDataPtrPtr;

    bdDataPtrPtr = &tlQuiccScc4RecvRefillBufDescPtr->bdDataPtr;
    bdStatusPtr  = &tlQuiccScc4RecvRefillBufDescPtr->bdStatus;
/* Loop while it is empty or an error */
    while (    (*bdDataPtrPtr == (void TM_FAR *)0)
            || (*bdStatusPtr & TM_QUICC_ERROR_RECV) )
    {
/* Check to see if it is an error */
        if (!(*bdStatusPtr & TM_QUICC_ERROR_RECV))
        {
/* If it is not get a new buffer, otherwise just reuse it */
            dataPtr = (void TM_FAR *)
                    (tfGetEthernetBuffer(&userBuffer) - TM_ETHER_IP_ALIGN);
            if (userBuffer == (ttUserBuffer)0)
            {
/* No memory so stop the refill */
                break;
            }
            *tlUserScc4RefillBufferPtr = userBuffer;
            tlQuiccScc4RecvRefillBufDescPtr->bdDataPtr = dataPtr;
        }
        tlQuiccScc4RecvRefillBufDescPtr->bdLength = 0;
/* Clear the status Except for the wrap bit */
        *bdStatusPtr &= TM_QUICC_RECV_WRAP;
/* Mark this BD as ready to use */
        *bdStatusPtr |= (TM_QUICC_RECV_EMPTY|TM_QUICC_RECV_INTR);
        if (*bdStatusPtr & TM_QUICC_RECV_WRAP)
        {
            tlQuiccScc4RecvRefillBufDescPtr = tlQuiccScc4RecvBufDescStartPtr;
            tlUserScc4RefillBufferPtr = tlUserScc4BufferList;
        }
        else
        {
            tlQuiccScc4RecvRefillBufDescPtr++;
            tlUserScc4RefillBufferPtr++;
        }
        bdDataPtrPtr = &tlQuiccScc4RecvRefillBufDescPtr->bdDataPtr;
        bdStatusPtr  = &tlQuiccScc4RecvRefillBufDescPtr->bdStatus;
    }


}

/*
 * Can be used to configure the device
 * or refill the receive pool
 */
int tfQuiccSerialIoctl ( ttUserInterface interfaceHandle, int flag,
                         void TM_FAR * optionPtr, int optionLen )
{
    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);
/* Make sure the device is online before calling these */
    if (tfQuiccOnline(TM_QUICC_SCC4_CHANNEL))
    {
        if (flag&TM_QUICC_REFILL_SCC4)
        {
            tfQuiccRefillScc4RecvPool();
        }
#ifndef TM_TASK_SEND
        if (flag&TM_QUICC_SEND_COMPLETE)
        {
            tfQuiccScc4SendComplete(interfaceHandle);
        }
#endif /* NOT TM_TASK_SEND */
    }
    return TM_DEV_OKAY;
}

/*
 * Send a packet to the network
 */
int tfQuiccSerialSend ( ttUserInterface   interfaceHandle,
                        char TM_FAR     * bufferPtr,
                        int               length,
                        int               flag )
{
    ttQuiccBufDescPtr         bufDescPtr;
    unsigned short  TM_FAR  * bdStatusPtr;

/* Wait for a free buffer descriptor */    
    while (tlQuiccScc4XmitBufDescPtr->bdStatus & TM_QUICC_XMIT_READY)
    {
/*
 * Do nothing function if user does not use transmit task. Otherwise will
 * yield the CPU.
 */
        tfInterfaceSpinLock(interfaceHandle);
        tvQuiccScc4XmitWait++;
    }
    bufDescPtr = tlQuiccScc4XmitBufDescPtr++;
    bdStatusPtr = &bufDescPtr->bdStatus;

    tlQuiccXmitBuffer[tlQuiccXmitBufNum] = *bufferPtr;
    bufDescPtr->bdDataPtr = &tlQuiccXmitBuffer[tlQuiccXmitBufNum++];
    bufDescPtr->bdLength = (unsigned short)length;

    if (*bdStatusPtr & TM_QUICC_XMIT_WRAP)
    {
        tlQuiccScc4XmitBufDescPtr = tlQuiccScc4XmitBufDescStartPtr;
        tlQuiccXmitBufNum = 0;
    }


/* Clear the previous status (but preserve the WRAP bit) */
    bufDescPtr->bdStatus &= TM_QUICC_XMIT_WRAP;
/* If this is the last buffer in the frame then xmit it */
/* Now mark the last bd ready */
        bdStatusPtr = &bufDescPtr->bdStatus;
        *bdStatusPtr |= (TM_QUICC_XMIT_READY |

#ifdef TM_TASK_SEND
                        TM_QUICC_XMIT_INTR | 
#endif /* TM_TASK_SEND */

                        TM_QUICC_XMIT_LAST);
        
    tvQuiccScc4SendPacketCount++;

#ifndef TM_TASK_SEND
/* Inlined send complete */
    if (flag == TM_USER_BUFFER_LAST)
    {
       tfSendCompleteInterface(interfaceHandle,TM_DEV_SEND_COMPLETE_DRIVER);
    }
#endif /* NOT TM_TASK_SEND */
    return TM_DEV_OKAY;
}

/*
 *
 * GENERAL ROUTINES (used for both ethernet and serial interfaces)
 *
 */

/* 
 * Interrupt called on a busy condition on the line. 
 * Lost some frames here.
 */
void tfQuiccBusy(ttUserInterface interfaceHandle, int channel)
{
    TM_UNREF_IN_ARG(interfaceHandle);
    TM_UNREF_IN_ARG(channel);
    tvQuiccBusyPacketCount++;
}


/* 
 * tfQuiccIssueCmd() is used to send commands to the communication processor
 * and waits till the command is completed
 */
void tfQuiccIssueCommand(unsigned short command)
{
    volatile unsigned short TM_FAR * commandReg;

/* Save the command register for the CP */  
    commandReg = &tlQuiccPtr->cp_cr;
/* Issue the command */
    *commandReg = (unsigned short)(command | TM_QUICC_COMMAND_FLAG);
/* wait for the CP zero the CMD_FLG */
    while( (*(volatile short *)commandReg) & TM_QUICC_COMMAND_FLAG )
    {
/* 
 * If this loop never exits:
 *
 * 1) check to make sure that the TM_QUICC_MBAR_LOC is correct 
 *
 * 2) that the tlQuicc value is correct and has been initialized
 *
 * 3) Make sure the compiler has generated good code
 *    Some compilers have been known to just AND a CPU register with
 *    TM_QUICC_COMMAND_FLAG instead of ANDING the contents of the 
 *    memory location for commandReg with TM_QUICC_COMMAND_FLAG.
 *    WORK AROUND: move the check into a seperate function
 *    and call that function from inside this loop.
 */
       ;
    } 
}


/* 
 * This routine is preparing for each SCC a pool of READY buffer 
 * descriptors that are ready to recieve. The last buffer descriptor
 * has the wrap bit set.
 */
void tfQuiccCreateRecvPool(int channel, int maxBd)
{
    ttQuiccBufDescPtr             tempBufDescPtr;
    struct ethernet_pram TM_FAR * enetPramAddr;
    ttUserBuffer                  userBuffer;
    void                 TM_FAR * dataPtr;
    int                           i;

    enetPramAddr = &tlQuiccPtr->pram[channel].enet_scc;

    tempBufDescPtr = TM_QUICC_RECV_BD_ADDR(tlQuiccPtr,enetPramAddr);
/* Save it off for our use in the recv routine */
    if (channel == TM_QUICC_SCC1_CHANNEL)
    {
        tlQuiccScc1RecvRefillBufDescPtr = tempBufDescPtr;
        tlQuiccScc1RecvBufDescStartPtr = tempBufDescPtr;
        tlQuiccScc1RecvIsrBufDescPtr = tempBufDescPtr;
        tlQuiccScc1RecvBufDescPtr = tempBufDescPtr;
        tlUserScc1RefillBufferPtr = tlUserScc1BufferList;
        tlUserScc1BufferPtr = tlUserScc1BufferList;
    }
    else 
    {
        if (channel == TM_QUICC_SCC4_CHANNEL)
        {
            tlQuiccScc4RecvRefillBufDescPtr = tempBufDescPtr;
            tlQuiccScc4RecvBufDescStartPtr = tempBufDescPtr;
            tlQuiccScc4RecvIsrBufDescPtr = tempBufDescPtr;
            tlQuiccScc4RecvBufDescPtr = tempBufDescPtr;
            tlUserScc4RefillBufferPtr = tlUserScc4BufferList;
            tlUserScc4BufferPtr = tlUserScc4BufferList;
        }

    }

    for( i = 0; i < maxBd; i++, tempBufDescPtr++ )
    {
/* bdDataPtr need to be aligned on a 4-byte boundary */
        dataPtr = (void TM_FAR *)
                    (tfGetEthernetBuffer(&userBuffer) - TM_ETHER_IP_ALIGN);
        if (userBuffer == (ttUserBuffer)0)
        {
/* No memory so stop the create here */
            break;
        }
        tempBufDescPtr->bdDataPtr = dataPtr;
        if (channel == TM_QUICC_SCC1_CHANNEL)
        {
            tlUserScc1BufferList[i] = userBuffer;
        }
        else
        {
            if (channel == TM_QUICC_SCC4_CHANNEL)
            {
                tlUserScc4BufferList[i] = userBuffer;
            }
        }

        tempBufDescPtr->bdLength = 0;
        tempBufDescPtr->bdStatus = (TM_QUICC_RECV_EMPTY|TM_QUICC_RECV_INTR);
    }
    --tempBufDescPtr;
 /* signify last bit on */
    tempBufDescPtr->bdStatus |= TM_QUICC_RECV_WRAP;

}

/* 
 * Prepare a transmit pool, set the status bits in the buffer descriptors
 * to be not READY to transmit and set the last buffer descriptor to
 * be wrapped
 */
void tfQuiccCreateXmitPool (int channel, int maxBd)
{
    int                           i;
    ttQuiccBufDescPtr             tempBufDescPtr;
    struct ethernet_pram TM_FAR * enetPramAddr;

/* Save the ethernet parameter address */
    enetPramAddr = &tlQuiccPtr->pram[channel].enet_scc;

    tempBufDescPtr = TM_QUICC_XMIT_BD_ADDR(tlQuiccPtr,enetPramAddr);

/* Save it off for our use in the xmit routine */    
    if (channel==TM_QUICC_SCC1_CHANNEL)
    {
        tlQuiccScc1XmitIsrBufDescPtr = tempBufDescPtr;
        tlQuiccScc1XmitBufDescStartPtr = tempBufDescPtr;
        tlQuiccScc1XmitBufDescPtr = tempBufDescPtr;
    }
    else
    {
        if (channel==TM_QUICC_SCC4_CHANNEL)
        {
            tlQuiccScc4XmitIsrBufDescPtr = tempBufDescPtr;
            tlQuiccScc4XmitBufDescStartPtr = tempBufDescPtr;
            tlQuiccScc4XmitBufDescPtr = tempBufDescPtr;
        }
    }

    for(i=0;i< maxBd;i++,tempBufDescPtr++ )
    {
        tempBufDescPtr->bdStatus = 0;
        tempBufDescPtr->bdLength = 0;
        tempBufDescPtr->bdDataPtr = (void TM_FAR *)0;
    }
    --tempBufDescPtr;
/* signify last bit on */
    tempBufDescPtr->bdStatus |= TM_QUICC_XMIT_WRAP; 
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:39      For: Xilinx-Samsung
 * File Name: trquicc.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 52912    Output checksum: 31917
 */
