/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Source file for Motorola Coldfire FEC sample driver
 *
 * $Source: source/drivers/mcf5282/trmcffec.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:05CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/* NOTE: you must use the Quadros configuration tool "rtxcgen" to setup
   the isvctfMcf5282IntRxHandler to be associated with vector
   ETHERRX_VECTOR (91) */
   
#define ETHERRX_VECTOR 91
#define ETHERTX_VECTOR 87 /* not used */

#ifdef TM_TASK_SEND
#error This driver does not support a send complete task!
#endif /* TM_TASK_SEND */

#define DIAG_SECTION_KERNEL DIAG_SECTION_DRIVER

#include "trsocket.h"
#include "trmacro.h"
#include "trtype.h"
#include "trproto.h"
#include "trmcffec.h"
#include "trglobal.h"

#ifdef TM_ERROR_CHECKING
/* The following 4 includes are for rtxc_printf only */
#include "rtxcapi.h"
#include "ktask.h"
#include "kproject.h"
#include "rtxcstdio.h"
#else /* TM_ERROR_CHECKING */
#define rtxc_printf
#endif /* TM_ERROR_CHECKING */

static void tfMcf5282RefillRecvPool(void);
static void tfMcf5282CleanupXmitQueue(ttUserInterface interfaceHandle);

#define PHYAUTO 4 /* set phy to autonegotiate speed */
#define PHY100  2 /* set phy to 100BaseT speed */
#define PHY10   0 /* set phy to 10BaseT speed */

/* ********************************************************************   */
/* MII interface functions */
/* ********************************************************************   */

#define MII_EVENT_TIMEOUT 1000  /* retry count before MII event timeout   */

tt32Bit tfEthfecReadMII(tt32Bit phy,tt32Bit reg, tt16Bit *data); 
tt32Bit tfEthfecWriteMII(tt32Bit phy,tt32Bit reg, tt32Bit val);
tt32Bit tfEthfecSetSpeed(tt32Bit phy,tt32Bit speed);
tt32Bit tfEthfecPollLinkStatus(tt32Bit phy);

/* Register definitions for the PHY. */
#define MII_REG_CR          0  /* Control Register                         */
#define MII_REG_SR          1  /* Status Register                          */
#define MII_REG_PHYIR1      2  /* PHY Identification Register 1            */
#define MII_REG_PHYIR2      3  /* PHY Identification Register 2            */
#define MII_REG_ANAR        4  /* A-N Advertisement Register               */ 
#define MII_REG_ANLPAR      5  /* A-N Link Partner Ability Register        */
#define MII_REG_ANER        6  /* A-N Expansion Register                   */
#define MII_REG_ANNPTR      7  /* A-N Next Page Transmit Register          */
#define MII_REG_ANLPRNPR    8  /* A-N Link Partner Received Next Page Reg. */

typedef struct
{
   unsigned short flags;
   unsigned short length;
   void         * buffer;
} ttBufDesc;

typedef ttBufDesc * ttBufDescPtr;

/* NOTE: you must use the Quadros configuration tool "rtxcgen" to setup
   the isvctfMcf5282IntRxHandler to be associated with vector
   ETHERRX_VECTOR (91) */
extern void isvctfMcf5282IntRxHandler(void);

/* ********************************************************************   */
/* GLOBAL DATA                                                            */
/* ********************************************************************   */

typedef struct tsMcf5282
{
  ttUserInterface       mcfInterfaceHandle;
  tt8BitPtr             mcfDescPtr;

/* For the FAST Ethernet controller, Buffer descriptors are in main memory */
/* and must be quad word aligned */
/* Pointer to first receive frame buffer descriptor */
  volatile ttBufDescPtr mcfRxFirstBufDescPtr;
/* Pointer to first transmit frame buffer descriptor */
  volatile ttBufDescPtr mcfTxFirstBufDescPtr;

  volatile ttBufDescPtr mcfTxBufDescPtr;    /* transmit descriptors */
  volatile ttBufDescPtr mcfTxScatterStartBufDescPtr;
  volatile ttBufDescPtr mcfTxIsrBufDescPtr; /* sent descriptors */
  volatile ttBufDescPtr mcfRxBufDescPtr;    /* receive descriptors */
  volatile ttBufDescPtr mcfRxIsrBufDescPtr; /* received descriptors */
  volatile ttBufDescPtr mcfRxRefillBufDescPtr;/* refill received descriptors */
/* Keep track of allocated received buffers */
  ttUserBuffer          mcfRxUserBufferList[TM_MCF5282_ETHER_MAX_RECV_BUFD];
/* next packet buffer to process for receive */
  volatile ttUserBuffer *mcfRxUserBufferPtr;
/* next packet buffer to process for refill received */
  volatile ttUserBuffer *mcfRxUserRefillBufferPtr;
  volatile unsigned long        mcfRecvErrorCount;
  volatile unsigned long        mcfRecvPacketCount;
  volatile unsigned long        mcfSendPacketCount;
  volatile unsigned long        mcfBusyPacketCount;
  volatile unsigned long        mcfXmitWait;
  volatile unsigned long        mcfIsrRecvErrorCount;
  int                           mcfFECXmitQueueOn;
/* array of 16-byte aligned mini-buffers that we use to align transmit data */
  tt32Bit                       mcfScatSendAlignBuf[
                                    TM_MCF5282_ETHER_MAX_XMIT_BUFD+1];
} ttMcf5282;

static ttMcf5282 tlMcfStruct;

#define tm_scat_send_align_buf          tlMcfStruct.mcfScatSendAlignBuf
#define tm_tx_bd_ptr                    tlMcfStruct.mcfTxBufDescPtr
#define tm_tx_isr_bd_ptr                tlMcfStruct.mcfTxIsrBufDescPtr
#define tm_rx_bd_ptr                    tlMcfStruct.mcfRxBufDescPtr
#define tm_rx_isr_bd_ptr                tlMcfStruct.mcfRxIsrBufDescPtr
#define tm_rx_refill_bd_ptr             tlMcfStruct.mcfRxRefillBufDescPtr
#define tm_rx_first_bd_ptr              tlMcfStruct.mcfRxFirstBufDescPtr
#define tm_tx_first_bd_ptr              tlMcfStruct.mcfTxFirstBufDescPtr
#define tm_rx_user_packet_ptr(i)        tlMcfStruct.mcfRxUserBufferList[i]
#define tm_rx_user_buf_list             tlMcfStruct.mcfRxUserBufferList
#define tm_rx_user_buf_ptr              tlMcfStruct.mcfRxUserBufferPtr
#define tm_rx_refill_user_buf_ptr       tlMcfStruct.mcfRxUserRefillBufferPtr
#define tm_tx_scat_start_bd_ptr         tlMcfStruct.mcfTxScatterStartBufDescPtr

#define tm_rx_err_stat                  tlMcfStruct.mcfRecvErrorCount
#define tm_rx_pkt_stat                  tlMcfStruct.mcfRecvPacketCount
#define tm_tx_pkt_stat                  tlMcfStruct.mcfSendPacketCount
#define tm_tx_wait_stat                 tlMcfStruct.mcfXmitWait
#define tm_rx_isr_err_stat              tlMcfStruct.mcfIsrRecvErrorCount
#define tm_fec_xmit_q_on                tlMcfStruct.mcfFECXmitQueueOn

typedef union
{
  unsigned char bytes [6];

  struct
    {
    unsigned long dword;
    unsigned short word;
    } words;
} NODE_ADDRESS;

#define BD_SCC_ETH_RX_EMPTY        0x8000
#define BD_SCC_ETH_RX_SW_ISR       0x4000 /* for software use */
#define BD_SCC_ETH_RX_WRAP         0x2000
#define BD_SCC_ETH_RX_SW_REFILL    0x1000 /* for software use */
#define BD_SCC_ETH_RX_LAST         0x0800
#define BD_SCC_ETH_RX_MISS         0x0100
#define BD_SCC_ETH_RX_VIOLATION    0x0020
#define BD_SCC_ETH_RX_NONALIGNED   0x0010
#define BD_SCC_ETH_RX_CRC          0x0004
#define BD_SCC_ETH_RX_OVERRUN      0x0002
#define BD_SCC_ETH_RX_TRUNCATED    0x0001
#define BD_SCC_ETH_RX_STAT_BITS    0x0fff
#define TM_BD_SCC_ETH_RX_ERROR \
    (BD_SCC_ETH_RX_VIOLATION \
     | BD_SCC_ETH_RX_NONALIGNED \
     | BD_SCC_ETH_RX_CRC \
     | BD_SCC_ETH_RX_OVERRUN \
     | BD_SCC_ETH_RX_TRUNCATED)

#define BD_SCC_ETH_TX_READY        0x8000
#define BD_SCC_ETH_TX_SW_COMPLETE  0x4000 /* tfSendCompleteInterface called */
#define BD_SCC_ETH_TX_WRAP         0x2000
#define BD_SCC_ETH_TX_SW_OWN2      0x1000
#define BD_SCC_ETH_TX_LAST         0x0800
#define BD_SCC_ETH_TX_CRC          0x0400
#define BD_SCC_ETH_TX_ABC          0x0200

#define RTIP_FEC_LEVEL  4 /* must match RTXCgen Exception level */

#define RTIP_FEC_RX_PRIORITY  1
#define RTIP_FEC_TX_PRIORITY  2
#define RTIP_FEC_XX_PRIORITY  3

/* MAC address */
NODE_ADDRESS myEthAddr = { { 0x00, 0xcf, 0x52, 0x82, 0xc3, 0x01 } };

/* Multicast address filter, receive all multicast for IGMP and IPv6 */
tt32Bit ethfec_mcaf[2] = { 0xffffffff, 0xffffffff };

/*
 * Return the Physical Address for the device
 * For the trmcffec, it is hard coded. 
 * because every ethernet address is Unique
 *
 * THIS ROUTINE MUST BE MODIFIED!
 *
 * To get an ethernet address address block for 
 * your company, you need to contact IEEE Standards
 * or visit http://standards.ieee.org/faqs/OUI.html
 * for information on getting an Company ID/OUI
 */
int tfMcf5282GetPhyAddr ( ttUserInterface userInterface,
                         char   TM_FAR * address )
{
/* 
 * We don't use the interface handle here, but we keep the compiler
 * from complaining about unused formal parameters
 */
    TM_UNREF_IN_ARG(userInterface);
/* Copy hard coded value in myEthAddr */
    tm_bcopy(myEthAddr.bytes, address, sizeof(myEthAddr.bytes));
    return 0;
}

/*************************************************************************
 * open the packet driver interface.                                     
 *                                                                      
 * This routine opens a packet device driver and copies the board's    
 * ethernet address into the interface structure. If the open is successful
 * and the ethernet address was ascertained it returns TRUE otherwise FALSE.
*/                                                                         
int tfMcf5282Open(ttUserInterface interfaceHandle)
{
    int         errorCode;
    int         i;
    tt8BitPtr   descPtr;
    tt32Bit     phyNum;
    tt32Bit     phyReg;
    tt16Bit     phyTemp;
    int         phyStatus;

    errorCode = TM_DEV_OKAY;
    tm_bzero((ttUserVoidPtr) &tlMcfStruct, sizeof(tlMcfStruct));

/* Save whether the user enabled the Treck transmit queue */
    tm_fec_xmit_q_on = tfInterfaceXmitQueueOn(interfaceHandle);

/*******************************************************
* Begin FAST Ethernet Controller Initialization 
*******************************************************  */

  /* Reset the FEC - equivalent to a hard reset */
    MCF5282_FEC_ECR = MCF5282_FEC_ECR_RESET;
  /* Wait for the reset sequence to complete */
    i = 0;
    while(MCF5282_FEC_ECR & MCF5282_FEC_ECR_RESET)
    {
        if(i++ > 10000) 
        {
            errorCode = TM_DEV_ERROR;
        }
    }
    if (errorCode == TM_DEV_OKAY)
    {
        MCF5282_FEC_ECR = 0;

/* Initialize PEHLPAR to enable ethernet signals */
        MCF5282_GPIO_PEHLPAR = 0xC0;

/* Setup pin multplexing for MII lines */
        MCF5282_GPIO_PASPAR = 0x0f00;
    }
    if (errorCode == TM_DEV_OKAY)
    {

/* copy the ethernet address to my_hw_addr       */
/* remember our interface ID */
        tlMcfStruct.mcfInterfaceHandle = interfaceHandle;

/* This is already handled by the quadrOS functino rtxcinit() */
/* Hook recv interrupt handler: Quadros should already do this */
        tfKernelInstallIsrHandler(isvctfMcf5282IntRxHandler, ETHERRX_VECTOR);
    }
    if (errorCode == TM_DEV_OKAY)
    {
/*
 *  Enable FEC interrupts to ColdFire core - receive and transmit
 *  are separate interrupts; There are 9 additional individual
 *  interrupts for error conditions. These are currently not implemented
 */

        if (checkICR(27, RTIP_FEC_LEVEL, RTIP_FEC_RX_PRIORITY)) /* Rx Frame*/
        {
            errorCode = TM_DEV_ERROR;
        }
        else
        {

            MCF5282_INTC0_ICR27 = MCF5282_INTC_ICR_IL(RTIP_FEC_LEVEL) |
                MCF5282_INTC_ICR_IP(RTIP_FEC_RX_PRIORITY);

            MCF5282_INTC0_IMRL &= (unsigned long) ~(MCF5282_INTC_IMRL_INT27 |
                                                    MCF5282_INTC_IMRL_MASKALL);
        }
    }
    if (errorCode == TM_DEV_OKAY)
    {

/* Set the interrupt mask and clear any pending events */
/* enable Transmit frame(bit 4), Receive frame (bit 6) */

        MCF5282_FEC_EIR   = MCF5282_FEC_EIR_RXF;
        MCF5282_FEC_EIMR |= MCF5282_FEC_EIR_RXF;

/* Initialize pointers to transmit and receive buffer descriptor rings.
   Buffer descriptors must be long word aligned (ie. bit 0,1 = 0) but
   Motorola recommends 16-byte alignment to improve bus utilization.
   For this controller, receive data buffer must be 16 byte aligned.    */

/* Allocate memory for Transmit and receive descriptors */
        tlMcfStruct.mcfDescPtr = tm_get_raw_buffer( 
            (sizeof(ttBufDesc) * (TM_MCF5282_ETHER_MAX_RECV_BUFD
                                  + TM_MCF5282_ETHER_MAX_XMIT_BUFD))
            + TM_ETHER_HW_ALIGN);
        descPtr = tlMcfStruct.mcfDescPtr;
        if (descPtr == TM_8BIT_NULL_PTR)
        {
            errorCode = TM_DEV_ERROR;
        }
        else
        {
            tm_bzero((ttUserVoidPtr) tlMcfStruct.mcfDescPtr, 
                     (sizeof(ttBufDesc) * (TM_MCF5282_ETHER_MAX_RECV_BUFD
                                           + TM_MCF5282_ETHER_MAX_XMIT_BUFD))
                     + TM_ETHER_HW_ALIGN);
            if (((tt32Bit)descPtr) & (tt32Bit)(TM_ETHER_HW_ALIGN - 1) )
            {
/* Align the data area (according to TM_ETHER_HW_ALIGN) */
                descPtr = (tt8Bit TM_FAR *)
                    (    (tt32Bit)descPtr
                         & ~(tt32Bit)(TM_ETHER_HW_ALIGN - 1) );
            }
            tm_rx_first_bd_ptr = (ttBufDesc *)descPtr;

            tm_rx_bd_ptr = tm_rx_first_bd_ptr;
            tm_rx_isr_bd_ptr = tm_rx_first_bd_ptr;
            tm_rx_refill_bd_ptr = tm_rx_first_bd_ptr;
            tm_rx_refill_user_buf_ptr = tm_rx_user_buf_list;
            tm_rx_user_buf_ptr = tm_rx_user_buf_list;
            tm_tx_first_bd_ptr = (tm_rx_first_bd_ptr
                                  + TM_MCF5282_ETHER_MAX_RECV_BUFD);
            tm_tx_bd_ptr = tm_tx_first_bd_ptr;
            tm_tx_isr_bd_ptr = tm_tx_first_bd_ptr;
            tm_tx_scat_start_bd_ptr = (ttBufDescPtr)0;
        }
    }
    if (errorCode == TM_DEV_OKAY)
    {
/* set up tx/rx buffer descriptors - defined in mpc860ad.h       */
        for (i = 0; i < TM_MCF5282_ETHER_MAX_RECV_BUFD; i++)
        {
            tm_rx_bd_ptr[i].length = 0;
/* get a pre-allocated packet */
            tm_rx_bd_ptr[i].buffer = 
                (tfGetDriverBuffer(
                    &(tm_rx_user_packet_ptr(i)),
                    TM_MCF5282_RX_BUF_LEN, TM_ETHER_HW_ALIGN));
            if (tm_rx_user_packet_ptr(i) == (ttUserBuffer)0)
            {
                break;
            }
            tm_rx_bd_ptr[i].flags = BD_SCC_ETH_RX_EMPTY;
        }
        if (i != 0)
        {
            tm_rx_bd_ptr[i-1].flags |= BD_SCC_ETH_RX_WRAP;
        }

        for (i = 0; i < TM_MCF5282_ETHER_MAX_XMIT_BUFD; i++)
        {
            tm_tx_bd_ptr[i].flags = BD_SCC_ETH_TX_SW_COMPLETE;
        }
        tm_tx_bd_ptr[TM_MCF5282_ETHER_MAX_XMIT_BUFD-1].flags
            |= BD_SCC_ETH_TX_WRAP;

/* Fill in our physical address */
        MCF5282_FEC_PALR = myEthAddr.words.dword;
        MCF5282_FEC_PAUR = (myEthAddr.words.word << 16);

/* Set hash table to group address for multicast packets */
        MCF5282_FEC_GALR = ethfec_mcaf[0]; /* hash table low */
        MCF5282_FEC_GAUR = ethfec_mcaf[1]; /* hash table high */

/* We will only accept our own address so no IA hash table entries */
        MCF5282_FEC_IALR = 0x00000000;
        MCF5282_FEC_IAUR = 0x00000000;
    
/* Define maximum receive buffer length */
/* NOTE: this value must be evenly divible by 16 */
        MCF5282_FEC_EMRBR = (tt16Bit)TM_MCF5282_RX_BUF_LEN;

/* receive descriptor ring pointer */
        MCF5282_FEC_ERDSR = (tt32Bit)tm_rx_bd_ptr;
/* xmit descriptor ring pointer */
        MCF5282_FEC_ETDSR = (tt32Bit)tm_tx_bd_ptr;

/* set receive/transmit control regs */
        MCF5282_FEC_RCR = (MCF5282_FEC_RCR_MII_MODE | MCF5282_FEC_RCR_DRT)
            | (1518<<16); /* MII mode, disable rx on tx */
/*        MCF5282_FEC_TCR = MCF5282_FEC_TCR_FDEN;   /* full duplex */
        MCF5282_FEC_TCR = 0x00000000;   /* no full duplex */

/* set MII management frame speed to 2.5Mhz... this reg is a clock divider
   and we assume that the CPU clock is 66 Mhz */
        MCF5282_FEC_MSCR = 0x001a;

/* enable the controller, then tell the controller the receive descriptors
 are active */
        MCF5282_FEC_ECR |= MCF5282_FEC_ECR_ETHER_EN;

/* Indicate Empty buffers have been produced */
        MCF5282_FEC_RDAR = MCF5282_FEC_RDAR_R_DES_ACTIVE;

/* Phy probe: read reg 0 from each possible phy number until we hit a valid
   response */
        phyReg = 0;
        for (phyNum = 0; phyNum < 32; phyNum++)
        {
            phyStatus = tfEthfecReadMII(phyNum, phyReg, &phyTemp);
            if ((phyStatus == TM_ENOERROR) && (phyTemp != 0x0000FFFF))
            {
/* set phy speed */
                phyStatus = tfEthfecSetSpeed(phyNum, PHYAUTO);

#ifdef TM_DISPLAY_MII
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_CR, &phyTemp);
                rtxc_printf("MII control register: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_SR, &phyTemp);
                rtxc_printf("MII status register: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_PHYIR1, &phyTemp);
                rtxc_printf(
                    "MII PHY identification register 1: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_PHYIR2, &phyTemp);
                rtxc_printf(
                    "MII PHY identification register 2: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_ANAR, &phyTemp);
                rtxc_printf(
                    "MII A-N Advertisement register: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_ANLPAR, &phyTemp);
                rtxc_printf(
                    "MII A-N Link Partner Ability register: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_ANER, &phyTemp);
                rtxc_printf(
                    "MII A-N Expansion register: 0x%hx\n", phyTemp);
                phyStatus = tfEthfecReadMII(
                    phyNum, MII_REG_ANNPTR, &phyTemp);
                rtxc_printf(
                    "MII A-N Next Page Transmit register: 0x%hx\n", phyTemp);
#endif /* TM_DISPLAY_MII */

                break;
            }
        }
    }

    return(errorCode);
}

/* ********************************************************************
 * close the packet driver interface.                                  
 *                                                                     
 * This routine is called when the device interface is no longer needed
 * it should stop the driver from delivering packets to the upper levels
 * and shut off packet delivery to the network.                        
 *                                                                    
 * The address of this function must be placed into the "devices" table in
 * iface.c either at compile time or before a device open is called.   
 *                                                                    
 *                                                                   
 * Non packet drivers should behave the same way.                   
 */
int tfMcf5282Close(ttUserInterface interfaceHandle)
{
    ttBufDescPtr        bufDescPtr;
    ttUserBufferPtr     userBufPtr;
    int                 i;

/* avoid compiler warning */
    TM_UNREF_IN_ARG(interfaceHandle);

/* disable the controller */
    MCF5282_FEC_ECR = 0x00000000;

/* 
 * Free leftover memory:
 *   First, send complete on any remaining packets in the transmit ring.
 */
    bufDescPtr = tm_tx_first_bd_ptr;
    for (i = 0; i < TM_MCF5282_ETHER_MAX_XMIT_BUFD; i++)
    {
/* Call send complete to free this entry */
        if ( ((bufDescPtr->flags) & BD_SCC_ETH_TX_LAST ) &&
             (bufDescPtr->buffer != (char TM_FAR *) 0) )
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER );
        }
        bufDescPtr++;
    }

/* Next, free any buffers in the receive ring */
    bufDescPtr = tm_rx_first_bd_ptr;
    userBufPtr = (void**) &(tm_rx_user_buf_list[0]);
    for (i = 0; i < TM_MCF5282_ETHER_MAX_RECV_BUFD; i++)
    {
        if ((bufDescPtr->buffer) != (char TM_FAR *) 0)
        {
            tfFreeDriverBuffer(*userBufPtr);
        }
        bufDescPtr++;
        userBufPtr++;
    }
        
/* Free our Buffer descriptor ring last */
    if (tlMcfStruct.mcfDescPtr != TM_8BIT_NULL_PTR)
    {
        tm_free_raw_buffer(tlMcfStruct.mcfDescPtr);
    }

    return TM_DEV_OKAY;
}

/*
 * tfMcf5282Ioctl()
 *
 * Handles general I/O control functions for the EMAC driver, which includes
 * giving new receive buffers to the EMAC and calling tfSendComplete for
 * packets that the EMAC has completed transmitting.
 *
 */
int tfMcf5282Ioctl(
    ttUserInterface interfaceHandle, int flag,
    void TM_FAR * optionPtr, int optionLen)
{
    int              errorCode;

/* fix compiler warnings */
    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);

    errorCode = TM_DEV_OKAY;

/* Refill the receive and transmit queues. */
    if (flag & TM_MCF5282_IOCTL_REFILL_RX)
    {
        tfMcf5282RefillRecvPool();
    }

    if (flag & TM_MCF5282_SEND_COMPLETE)
    {
        tfMcf5282CleanupXmitQueue(interfaceHandle);
    }

    return errorCode;
}

void tfMcf5282RefillRecvPool(void)
{
    ttUserBuffer            userBuffer;
    unsigned short TM_FAR * flagsPtr;
    void           TM_FAR * dataPtr;
        
    flagsPtr  = &tm_rx_refill_bd_ptr->flags;
/* Loop while it is empty or an error */
    while (*flagsPtr & BD_SCC_ETH_RX_SW_REFILL)
    {
        if (tm_rx_refill_bd_ptr->buffer == (void TM_FAR *)0 )
        {
/* Get a new buffer aligned on a 4-byte boundary */
            dataPtr = (void TM_FAR *)
                tfGetDriverBuffer(
                    &userBuffer,
                    TM_MCF5282_RX_BUF_LEN, TM_ETHER_HW_ALIGN);
            if (userBuffer == (ttUserBuffer)0)
            {
/* No memory so stop the refill */
                break;
            }

            tm_rx_refill_bd_ptr->buffer = dataPtr;
            *tm_rx_refill_user_buf_ptr = userBuffer;
        }
        
        tm_rx_refill_bd_ptr->length = 0;
/* Clear the status Except for the wrap bit */
        *flagsPtr &= BD_SCC_ETH_RX_WRAP;
/* Mark this BD as ready to use */
        *flagsPtr |= BD_SCC_ETH_RX_EMPTY;
        if (*flagsPtr & BD_SCC_ETH_RX_WRAP)
        {
            tm_rx_refill_bd_ptr = tm_rx_first_bd_ptr;
            tm_rx_refill_user_buf_ptr = tm_rx_user_buf_list;
        }
        else
        {
            tm_rx_refill_bd_ptr++;
            tm_rx_refill_user_buf_ptr++;
        }
        flagsPtr  = &tm_rx_refill_bd_ptr->flags;

/* Indicate Empty buffers have been produced */
        MCF5282_FEC_RDAR = MCF5282_FEC_RDAR_R_DES_ACTIVE;
    }
}

int tfMcf5282Receive(
    ttUserInterface interfaceHandle,
    char TM_FAR    ** dataPtrPtr,
    int  TM_FAR     * lengthPtr,
    ttUserBufferPtr   userBufferHandlePtr)
{
    int                         errorCode;
    register ttBufDescPtr       bufDescPtr;
    unsigned short TM_FAR *     flagsPtr;
    
    TM_UNREF_IN_ARG(interfaceHandle);
    errorCode = TM_ENOERROR;
    
    flagsPtr = &tm_rx_bd_ptr->flags;

/* Skip pass error packets who are not empty */
    if (    ( !( *flagsPtr &  BD_SCC_ETH_RX_EMPTY ) )
            && ( *flagsPtr & TM_BD_SCC_ETH_RX_ERROR ) )
    {
        tm_rx_err_stat++;
        
/* give tfMcf5282RefillRecvPool ownership of this frame descriptor */
        *flagsPtr |= BD_SCC_ETH_RX_SW_REFILL;
                
/* Return the userBuffer Handle for this data pointer */
        if (*flagsPtr & BD_SCC_ETH_RX_WRAP)
        {
            tm_rx_bd_ptr = tm_rx_first_bd_ptr;
            tm_rx_user_buf_ptr = tm_rx_user_buf_list;
        }
        else
        {
            tm_rx_bd_ptr++;
            tm_rx_user_buf_ptr++;
        }

        errorCode = TM_EWOULDBLOCK;
    }
    else
    {
/* Check to see if we own the Buffer Descriptor */
        if (*flagsPtr & BD_SCC_ETH_RX_EMPTY)
        {
/* odd, we could not find a complete received frame! */
            tm_rx_err_stat++;

            errorCode = TM_EWOULDBLOCK;
        }
        else
        {
            bufDescPtr = tm_rx_bd_ptr++;
            *userBufferHandlePtr = *tm_rx_user_buf_ptr++;
            if (*flagsPtr & BD_SCC_ETH_RX_WRAP)
            {
                tm_rx_bd_ptr = tm_rx_first_bd_ptr;
                tm_rx_user_buf_ptr = tm_rx_user_buf_list;
            }

/* We are returning the pointer to the data */
            *dataPtrPtr = (char TM_FAR *)(bufDescPtr->buffer);

/* Zero out the bufferPtr so we can refill it later */
            bufDescPtr->buffer=(void TM_FAR *)0;
/* give tfMcf5282RefillRecvPool ownership of this frame descriptor */
            *flagsPtr |= BD_SCC_ETH_RX_SW_REFILL;

/* We are returning the length */
            *lengthPtr  = bufDescPtr->length;

/* Up our packet count */
            tm_rx_pkt_stat++;
/* refill this receive descriptor */
            tfMcf5282RefillRecvPool();
        }
    }

    return(errorCode);
}

#ifdef TM_USE_DRV_SCAT_RECV  
int tfMcf5282ScatRecv (
    ttUserInterface interfaceHandle,
    ttDruBlockPtrPtr uDevBlockPtrPtr,
    int * uDevBlockCountPtr,
    int * flagPtr)
{
    int                                 errorCode;
    register ttBufDescPtr               bufDescPtr;
    int                                 rxBufDescCount;
    int                                 i;
    register unsigned short TM_FAR *    flagsPtr;
    register ttDruBlockPtr              devBlockPtr;
    ttDeviceEntryPtr                    devEntryPtr;
    unsigned int                        frameTotalLen;
    
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    errorCode = TM_ENOERROR;
    
/* first, determine where our received frame starts and ends, w/ regards
   to rx frame descriptors. Also, we keep a count of frame descriptors. */
    rxBufDescCount = 1;
    bufDescPtr = tm_rx_bd_ptr;
    while(1)
    {
        flagsPtr = &(bufDescPtr->flags);

        if (*flagsPtr & BD_SCC_ETH_RX_EMPTY)
        {
/* odd, we could not find a complete received frame! */
            errorCode = TM_EWOULDBLOCK;
            break;
        }
        
/* if this frame descriptor is the last one in a frame, then we are
   at the end of the frame */
        if (*flagsPtr & BD_SCC_ETH_RX_LAST)
        {
/* we found a complete received frame */
            break;
        }
        
        if (*flagsPtr & BD_SCC_ETH_RX_WRAP)
        {
            bufDescPtr = tm_rx_first_bd_ptr;
        }
        else
        {
            bufDescPtr++;
        }
        
        rxBufDescCount++;
        if (rxBufDescCount > TM_MCF5282_ETHER_MAX_RECV_BUFD)
        {
/* odd, we could not find a complete received frame! */
            errorCode = TM_EWOULDBLOCK;
            break;
        }
    }

    if (errorCode == TM_ENOERROR)
    {
        devBlockPtr = (ttDruBlockPtr)(devEntryPtr->devReceiveDataPtr);
        if (devBlockPtr != (ttDruBlockPtr)0)
        {
/* free previous instance of output parameter */
            tm_free(((ttVoidPtr)devBlockPtr));
        }

/* allocate our return arguments */
        devBlockPtr =
            (ttDruBlockPtr)tm_malloc(sizeof(ttDruBlock) * (rxBufDescCount));
        
        if (devBlockPtr != (ttDruBlockPtr)0)
        {
            frameTotalLen = 0;
            for (i = 0; i < rxBufDescCount; i++)
            {
                bufDescPtr = tm_rx_bd_ptr;
                flagsPtr = &(bufDescPtr->flags);

/* store this scattered rx buffer in the list we will return to the Treck
   stack */
                devBlockPtr[i].druStackBufferPtr =
                    (ttUserBufferPtr) *tm_rx_user_buf_ptr;
                if (*flagsPtr & BD_SCC_ETH_RX_LAST)
                {
                    if (bufDescPtr->length > frameTotalLen)
                    {
/* the last frame descriptor contains the total frame length */
                        devBlockPtr[i].druDataLength =
                            bufDescPtr->length - frameTotalLen;
                    }
                    else
                    {
/* not sure why we got here */
                        devBlockPtr[i].druDataLength = TM_MCF5282_RX_BUF_LEN;
                    }
                }
                else
                {
                    devBlockPtr[i].druDataLength = bufDescPtr->length;
                    frameTotalLen += bufDescPtr->length;
                }
                devBlockPtr[i].druDataPtr =
                    (char TM_FAR *) (bufDescPtr->buffer);

/* Skip past error packets, already checked that they are not empty */
                if ( *flagsPtr & TM_BD_SCC_ETH_RX_ERROR )
                {
                    tm_rx_err_stat++;
                    errorCode = TM_EWOULDBLOCK;
                }
                else
                {
/* Zero out the bufferPtr so we can refill it later */
                    bufDescPtr->buffer=(void TM_FAR *)0;
                }

/* give tfMcf5282RefillRecvPool ownership of this frame descriptor */
                *flagsPtr |= BD_SCC_ETH_RX_SW_REFILL;
                
/* advance to the next rx frame descriptor */
                if (*flagsPtr & BD_SCC_ETH_RX_WRAP)
                {
                    tm_rx_bd_ptr = tm_rx_first_bd_ptr;
                    tm_rx_user_buf_ptr = tm_rx_user_buf_list;
                }
                else
                {
                    tm_rx_bd_ptr++;
                    tm_rx_user_buf_ptr++;
                }
            }

            if (errorCode == TM_ENOERROR)
            {
/* Up our packet count */
                tm_rx_pkt_stat++;
                
/* set the output parameters for the scattered receive */
                *flagPtr = TM_DEV_SCAT_RECV_STACK_BUFFER;
                *uDevBlockPtrPtr = devBlockPtr;
                *uDevBlockCountPtr = rxBufDescCount;

/* to be freed later */
                devEntryPtr->devReceiveDataPtr =
                    (tt8BitPtr)((ttVoidPtr)devBlockPtr);
            }
            else
            {
                tm_free(((ttVoidPtr)devBlockPtr));

/* already freed this earlier, we don't want to try to free it twice */
                devEntryPtr->devReceiveDataPtr = (tt8BitPtr) 0;
            }
        }
    }
    
/* refill this receive descriptor */
    tfMcf5282RefillRecvPool();

    return(errorCode);
}
#endif /* TM_USE_DRV_SCAT_RECV */

static void tfMcf5282CleanupXmitQueue(ttUserInterface interfaceHandle)
{
    register unsigned short TM_FAR       *flagsPtr;
    register void  TM_FAR * TM_FAR       *bdDataPtrPtr;

/* complete any transmits */        
    flagsPtr = &tm_tx_isr_bd_ptr->flags;
    bdDataPtrPtr = &tm_tx_isr_bd_ptr->buffer;

    while ((((*flagsPtr & (BD_SCC_ETH_TX_READY | BD_SCC_ETH_TX_SW_COMPLETE))
             == 0)
            && (tm_tx_isr_bd_ptr != tm_tx_scat_start_bd_ptr)
            && ( *bdDataPtrPtr != (void TM_FAR *)0 )))
/* 
 * Watch out for a scattered frame we have not yet finished
 * loading up
 */
    {
/* Zero it out so we know that we have freed this one before */
        *bdDataPtrPtr = (void TM_FAR *)0;
        if (*flagsPtr & BD_SCC_ETH_TX_WRAP)
        {
            tm_tx_isr_bd_ptr = tm_tx_first_bd_ptr;
        }
        else
        {
            tm_tx_isr_bd_ptr++;
        }

/* Call send complete when the last piece has been transmitted */
        if (*flagsPtr & BD_SCC_ETH_TX_LAST)
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER);
        }

/* notify device driver send that this frame descriptor is available */
        *flagsPtr |= BD_SCC_ETH_TX_SW_COMPLETE;

        flagsPtr = &tm_tx_isr_bd_ptr->flags;
        bdDataPtrPtr = &tm_tx_isr_bd_ptr->buffer;
    }
}

int tfMcf5282Send(
    ttUserInterface  interfaceHandle,
    char TM_FAR    * bufferPtr,
    int              length,
    int              flag)
{
    ttBufDescPtr                bufDescPtr;
    ttBufDescPtr                startBufDescPtr;
    unsigned short TM_FAR       *flagsPtr;
    int                         errorCode;
    tt8BitPtr                   alignedBufferPtr;
    register tt8BitPtr          scatSendAlignBufPtr;
    register int                i;
    int                         nonAlignedBytes;
    int                         scatSendAlignBufIndex;

    errorCode = TM_DEV_OKAY;

/* if the transmit queue is enabled, then check to make sure that the required
   number of frame descriptors are ready - if they aren't, then we return
   TM_DEV_ERROR so that this packet is queued in the transmit queue for
   later */
    if (tm_fec_xmit_q_on)
    {
        i = 1;
        if ( ((tt32Bit)bufferPtr) & 3)
        {
/* we will need another frame descriptor for data alignment */            
            i++;
        }

        for (bufDescPtr = tm_tx_bd_ptr; i != 0; i--)
        {
            if (!(bufDescPtr->flags & BD_SCC_ETH_TX_SW_COMPLETE))
            {
/* this frame descriptor isn't available, retry the transmit later */
                errorCode = TM_DEV_ERROR;
                goto mcfSendExit;
            }
            
/* advance to the next transmit frame descriptor */
            if (bufDescPtr->flags & BD_SCC_ETH_TX_WRAP)
            {
                bufDescPtr = tm_tx_first_bd_ptr;
            }
            else
            {
                bufDescPtr++;
            }
        }
    }

/* align the packet data to start on a 4-byte boundary */

/* check if not align before trying to align
 * copy the non-aligned bytes from the front of the packet to the extra space
 * past the end of the packet so that they are aligned, the first frame
 * descriptor points here. The remaining aligned bytes are pointed by
 * by a 2nd frame descriptor.
 */
    if ( ((tt32Bit)bufferPtr) & 3)
    {
        alignedBufferPtr = 
            (tt8BitPtr) ((((tt32Bit) bufferPtr) + 3) & ~((tt32Bit) 3));
        nonAlignedBytes = alignedBufferPtr - ((tt8BitPtr) bufferPtr);
        length -= nonAlignedBytes;

/* Wait for a free buffer descriptor */
        if (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE))
        {
            do
            {
/* If user transmit task, yield the CPU, otherwise, do nothing */
                tfInterfaceSpinLock(interfaceHandle);
                tm_tx_wait_stat++;

/* cleanup any transmits that have already completed */
                tfMcf5282CleanupXmitQueue(interfaceHandle);
            }
            while (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE ));
        }

/* get an aligned buffer for this specific frame descriptor to copy the
   non-aligned data to */
        scatSendAlignBufIndex = tm_tx_bd_ptr - tm_tx_first_bd_ptr;
        
/* get a 4-byte aligned buffer to copy the non-aligned data to */
        scatSendAlignBufPtr = 
            (tt8BitPtr) ((((tt32Bit)
                           &(tm_scat_send_align_buf[
                                 scatSendAlignBufIndex]) ) + 3)
                         & ~((tt32Bit) 3));

/* copy the non-aligned bytes from the front of the packet data to the
   4-byte aligned buffer */
        for (i = (nonAlignedBytes - 1); i >= 0; i--)
        {
            scatSendAlignBufPtr[i] = bufferPtr[i];
        }
        bufferPtr = (char TM_FAR *) alignedBufferPtr;
    
/* use this transmit buffer descriptor for the non-aligned bytes, which we
   have copied to a 4-byte aligned buffer */
        bufDescPtr = tm_tx_bd_ptr++;
        if (bufDescPtr->flags & BD_SCC_ETH_TX_WRAP)
        {
            tm_tx_bd_ptr = tm_tx_first_bd_ptr;
        }
        bufDescPtr->buffer = (void TM_FAR *)scatSendAlignBufPtr;
        bufDescPtr->length = nonAlignedBytes;
    
/* Clear the previous status (but preserve the WRAP bit) */
        bufDescPtr->flags &= BD_SCC_ETH_TX_WRAP;

/* More buffers to come */
        if (tm_tx_scat_start_bd_ptr == (ttBufDescPtr)0)
        {
            tm_tx_scat_start_bd_ptr = bufDescPtr;
        }
    }

/* Wait for a free buffer descriptor */    
    if (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE))
    {
        do
        {
/* If user transmit task, yield the CPU, otherwise, do nothing */
            tfInterfaceSpinLock(interfaceHandle);
            tm_tx_wait_stat++;

/* cleanup any transmits that have already completed */
            tfMcf5282CleanupXmitQueue(interfaceHandle);
        }
        while (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE ));
    }

    bufDescPtr = tm_tx_bd_ptr++;
    if (bufDescPtr->flags & BD_SCC_ETH_TX_WRAP)
    {
        tm_tx_bd_ptr = tm_tx_first_bd_ptr;
    }
    bufDescPtr->buffer = (void TM_FAR *)bufferPtr;
    bufDescPtr->length = length;
    
/* Clear the previous status (but preserve the WRAP bit) */
    bufDescPtr->flags &= BD_SCC_ETH_TX_WRAP;
    
/* If this is the last buffer in the frame then xmit it */
    if (flag == TM_USER_BUFFER_LAST)
    {
/* Now mark the last bd ready */
        flagsPtr = &bufDescPtr->flags;
        *flagsPtr |= (BD_SCC_ETH_TX_READY |
                      BD_SCC_ETH_TX_LAST |
                      BD_SCC_ETH_TX_CRC);

/* Check to see if we are xmitting scattered data */
        if (tm_tx_scat_start_bd_ptr != (ttBufDescPtr)0)
        {
/* We have at least one more piece to xmit if we are here */
            startBufDescPtr = tm_tx_scat_start_bd_ptr;
            tm_tx_scat_start_bd_ptr = (ttBufDescPtr)0;
/* 
 * Go ahead and xmit the first packets (with more set) 
 * by walking the list backwards
 * We do this so that we don't need a critical section
 * in order to keep from getting gaps inbetween the 
 * pieces
 * All we need to do here is update the
 * ready flag
 */
            do
            {
                if (bufDescPtr == tm_tx_first_bd_ptr)
                {
/* We are at the front of the list so go to the end */
                    bufDescPtr += TM_MCF5282_ETHER_MAX_XMIT_BUFD;
                }
                else
                {
/* Go back one buffer descriptor */
                    bufDescPtr--;
                }
                flagsPtr = &bufDescPtr->flags;
                *flagsPtr |= BD_SCC_ETH_TX_READY;
            } while (bufDescPtr != startBufDescPtr);
        }

/* Inform the FEC that there are new transmit frames to process */
        MCF5282_FEC_TDAR = MCF5282_FEC_TDAR_X_DES_ACTIVE;

/* cleanup any transmits that have already completed */
        tfMcf5282CleanupXmitQueue(interfaceHandle);
    }
    else
    {
/* More buffers to come */
        if (tm_tx_scat_start_bd_ptr == (ttBufDescPtr)0)
        {
            tm_tx_scat_start_bd_ptr = bufDescPtr;
        }
    }

mcfSendExit:
    return errorCode;
}

#ifdef TM_USE_DRV_ONE_SCAT_SEND
/*
 * tfDm642EmacScatSend
 * Scatter send function.
 */
int tfMcf5282ScatSend(
    ttUserInterface interfaceHandle,
    ttUserPacketPtr  uPacketPtr )
{
    char TM_FAR *               dataPtr;
    int                         dataLength;
    int                         scatDataLen;
    int                         scatterIndex;
    ttBufDescPtr                bufDescPtr;
    ttBufDescPtr                startBufDescPtr;
    unsigned short TM_FAR       *flagsPtr;
    int                         errorCode;
    tt8BitPtr                   alignedBufferPtr;
    register tt8BitPtr          scatSendAlignBufPtr;
    register int                i;
    ttUserPacketPtr             txQPacketPtr;
    int                         nonAlignedBytes;
    int                         scatSendAlignBufIndex;

    scatterIndex = uPacketPtr->pktuLinkExtraCount;
    dataLength = uPacketPtr->pktuChainDataLength;

    if (scatterIndex == 0)
    {
/* if this is not a scattered send, then call the regular driver send */
        errorCode = tfMcf5282Send(
            interfaceHandle,
            (char TM_FAR *) uPacketPtr->pktuLinkDataPtr,
            dataLength, TM_USER_BUFFER_LAST); 
    }
    else
    {
        errorCode = TM_DEV_OKAY;

/* if the transmit queue is enabled, then check to make sure that the required
   number of frame descriptors are ready - if they aren't, then we return
   TM_DEV_ERROR so that this packet is queued in the transmit queue for
   later */
        if (tm_fec_xmit_q_on)
        {
            i = scatterIndex + 1;
            for (txQPacketPtr = uPacketPtr; scatterIndex >= 0; scatterIndex--)
            {
                dataPtr = (char TM_FAR *) uPacketPtr->pktuLinkDataPtr;
                txQPacketPtr  = txQPacketPtr->pktuLinkNextPtr;
                if ( ((tt32Bit)dataPtr) & 3)
                {
/* we will need another frame descriptor for data alignment */            
                    i++;
                }
            }

            for (bufDescPtr = tm_tx_bd_ptr; i != 0; i--)
            {
                if (!(bufDescPtr->flags & BD_SCC_ETH_TX_SW_COMPLETE))
                {
/* this frame descriptor isn't available, retry the transmit later */
                    errorCode = TM_DEV_ERROR;
                    goto mcfScatSendExit;
                }

/* advance to the next transmit frame descriptor */
                if (bufDescPtr->flags & BD_SCC_ETH_TX_WRAP)
                {
                    bufDescPtr = tm_tx_first_bd_ptr;
                }
                else
                {
                    bufDescPtr++;
                }
            }

/* restore the local variables that we used */
            scatterIndex = uPacketPtr->pktuLinkExtraCount;
        }

        for (; scatterIndex >= 0; scatterIndex--)
        {
            dataPtr = (char TM_FAR *) uPacketPtr->pktuLinkDataPtr;
            scatDataLen = uPacketPtr->pktuLinkDataLength;
            uPacketPtr  = uPacketPtr->pktuLinkNextPtr; 

            if ( ((tt32Bit) dataPtr) & 3)
            {
                alignedBufferPtr = 
                    (tt8BitPtr) ((((tt32Bit) dataPtr) + 3) & ~((tt32Bit) 3));
                nonAlignedBytes = alignedBufferPtr - ((tt8BitPtr) dataPtr);
                scatDataLen -= nonAlignedBytes;

/* Wait for a free buffer descriptor */
                if (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE))
                {
                    do
                    {
/* If user transmit task, yield the CPU, otherwise, do nothing */
                        tfInterfaceSpinLock(interfaceHandle);
                        tm_tx_wait_stat++;

/* cleanup any transmits that have already completed */
                        tfMcf5282CleanupXmitQueue(interfaceHandle);
                    }
                    while(!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE ));
                }

/* get an aligned buffer for this specific frame descriptor to copy the
   non-aligned data to */
                scatSendAlignBufIndex = tm_tx_bd_ptr - tm_tx_first_bd_ptr;
        
/* get a 4-byte aligned buffer to copy the non-aligned data to */
                scatSendAlignBufPtr = 
                    (tt8BitPtr) ((((tt32Bit)
                                   &(tm_scat_send_align_buf[
                                         scatSendAlignBufIndex]) ) + 3)
                                 & ~((tt32Bit) 3));

/* copy the non-aligned bytes from the front of the packet data to the
   4-byte aligned buffer */
                for (i = (nonAlignedBytes - 1); i >= 0; i--)
                {
                    scatSendAlignBufPtr[i] = dataPtr[i];
                }
                dataPtr = (char TM_FAR *) alignedBufferPtr;
    
/* use this transmit buffer descriptor for the non-aligned bytes, which we
   have copied to a 4-byte aligned buffer */
                bufDescPtr = tm_tx_bd_ptr++;
                if (bufDescPtr->flags & BD_SCC_ETH_TX_WRAP)
                {
                    tm_tx_bd_ptr = tm_tx_first_bd_ptr;
                }
                bufDescPtr->buffer = (void TM_FAR *)scatSendAlignBufPtr;
                bufDescPtr->length = nonAlignedBytes;
    
/* Clear the previous status (but preserve the WRAP bit) */
                bufDescPtr->flags &= BD_SCC_ETH_TX_WRAP;

/* More buffers to come */
                if (tm_tx_scat_start_bd_ptr == (ttBufDescPtr)0)
                {
                    tm_tx_scat_start_bd_ptr = bufDescPtr;
                }
            }

/* Wait for a free buffer descriptor */    
            if (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE))
            {
                do
                {
/* If user transmit task, yield the CPU, otherwise, do nothing */
                    tfInterfaceSpinLock(interfaceHandle);
                    tm_tx_wait_stat++;

/* cleanup any transmits that have already completed */
                    tfMcf5282CleanupXmitQueue(interfaceHandle);
                }
                while (!(tm_tx_bd_ptr->flags & BD_SCC_ETH_TX_SW_COMPLETE ));
            }

            bufDescPtr = tm_tx_bd_ptr++;
            if (bufDescPtr->flags & BD_SCC_ETH_TX_WRAP)
            {
                tm_tx_bd_ptr = tm_tx_first_bd_ptr;
            }
            bufDescPtr->buffer = (void TM_FAR *)dataPtr;
            bufDescPtr->length = scatDataLen;
    
/* Clear the previous status (but preserve the WRAP bit) */
            bufDescPtr->flags &= BD_SCC_ETH_TX_WRAP;
    
/* If this is the last buffer in the frame then xmit it */
            if (scatterIndex == 0)
            {
/* Now mark the last bd ready */
                flagsPtr = &bufDescPtr->flags;
                *flagsPtr |= (BD_SCC_ETH_TX_READY |
                              BD_SCC_ETH_TX_LAST |
                              BD_SCC_ETH_TX_CRC);

/* Check to see if we are xmitting scattered data */
                if (tm_tx_scat_start_bd_ptr != (ttBufDescPtr)0)
                {
/* We have at least one more piece to xmit if we are here */
                    startBufDescPtr = tm_tx_scat_start_bd_ptr;
                    tm_tx_scat_start_bd_ptr = (ttBufDescPtr)0;
/* 
 * Go ahead and xmit the first packets (with more set) 
 * by walking the list backwards
 * We do this so that we don't need a critical section
 * in order to keep from getting gaps inbetween the 
 * pieces
 * All we need to do here is update the
 * ready flag
 */
                    do
                    {
                        if (bufDescPtr == tm_tx_first_bd_ptr)
                        {
/* We are at the front of the list so go to the end */
                            bufDescPtr += TM_MCF5282_ETHER_MAX_XMIT_BUFD;
                        }
                        else
                        {
/* Go back one buffer descriptor */
                            bufDescPtr--;
                        }
                        flagsPtr = &bufDescPtr->flags;
                        *flagsPtr |= BD_SCC_ETH_TX_READY;
                    } while (bufDescPtr != startBufDescPtr);
                }

/* Inform the FEC that there are new transmit frames to process */
                MCF5282_FEC_TDAR = MCF5282_FEC_TDAR_X_DES_ACTIVE;
                
/* cleanup any transmits that have already completed */
                tfMcf5282CleanupXmitQueue(interfaceHandle);
            }
            else
            {
/* More buffers to come */
                if (tm_tx_scat_start_bd_ptr == (ttBufDescPtr)0)
                {
                    tm_tx_scat_start_bd_ptr = bufDescPtr;
                }
            }
        }
    }

mcfScatSendExit:
    return errorCode;
}
#endif /* TM_USE_DRV_ONE_SCAT_SEND */

/* handle a receive interrupt */
void tfMcf5282IntRxHandler(void)
{
  register unsigned short  TM_FAR       *flagsPtr;
  int                                   recvNotifyCount;
  tt32Bit                               my_scce;

/* Clear all interrupt events that will be serviced */
/* only receiver interrupts */ 
  my_scce = MCF5282_FEC_EIR & MCF5282_FEC_EIR_RXF;
  MCF5282_FEC_EIMR &= ~my_scce;  /* mask off source */
  MCF5282_FEC_EIR = my_scce;   /* acknowledge source */

  recvNotifyCount = 0;

/* loop on frames having the Empty and SW_ISR bits NOT set */
/* Only notify while the frames are not empty */
  flagsPtr = &tm_rx_isr_bd_ptr->flags;
  while (!(*flagsPtr & (BD_SCC_ETH_RX_EMPTY|BD_SCC_ETH_RX_SW_ISR)))
  {
#ifdef TM_USE_DRV_SCAT_RECV
/* if this frame descriptor is the last one in a frame, then we have
   completed receiving a frame */
      if (*flagsPtr & BD_SCC_ETH_RX_LAST)
      {
#endif /* TM_USE_DRV_SCAT_RECV */
          recvNotifyCount++;
#ifdef TM_USE_DRV_SCAT_RECV
      }
#endif /* TM_USE_DRV_SCAT_RECV */
      *flagsPtr |= BD_SCC_ETH_RX_SW_ISR;
      if (*flagsPtr & BD_SCC_ETH_RX_WRAP)
      {
          tm_rx_isr_bd_ptr = tm_rx_first_bd_ptr;
      }
      else
      {
          tm_rx_isr_bd_ptr++;
      }

      flagsPtr = &tm_rx_isr_bd_ptr->flags;
  }
   
  if (recvNotifyCount != 0)
  {
      tfNotifyInterfaceIsr(
          tlMcfStruct.mcfInterfaceHandle, recvNotifyCount,
          0, 0UL, 0UL );
  }

  MCF5282_FEC_EIMR |= my_scce; /* re-enable interrupts serviced */
}

/**************************************************************************
*
*  FUNCTION: tfEthfecReadMII()
*  
*  DESCRIPTION: read an ethernet MII phy register
*
*  INPUT:  tt32Bit phy    1 of 32 possible MII physical interfaces
*          tt32Bit reg    1 of 32 possible registers of the MII physical
*                         interface
*          tt16bit *data  pointer to a tt16bit where the MII read data will
*                         be stored
*
*  RETURN: tt32Bit        0 if successful, non-zero otherwise
*
************************************************************************/
tt32Bit tfEthfecReadMII(tt32Bit phy,tt32Bit reg, tt16Bit *data) 
{  
   tt32Bit temp, status, i;

/* clear any pending MII events */
   MCF5282_FEC_EIR |= MCF5282_FEC_EIR_MII;
   
/* Set up MII register data */
   temp = MCF5282_FEC_MMFR_ST        |
          MCF5282_FEC_MMFR_OP_RD     |
          MCF5282_FEC_MMFR_TA        |
          MCF5282_FEC_MMFR_PA(phy)   |
          MCF5282_FEC_MMFR_RA(reg)   |
          MCF5282_FEC_MMFR_DATA(0);

/* send the command  */ 
   MCF5282_FEC_MMFR = temp;

/* Wait for MII event to post */
   for (i = 0; i < MII_EVENT_TIMEOUT; i++)
   {
/* check if the event posted */
       if(MCF5282_FEC_EIR & MCF5282_FEC_EIR_MII)
       {
/* the event posted - exit the loop */           
           break;
      }
   } 
   
   if (i == MII_EVENT_TIMEOUT)
   {
/* MII event post timeout - bad status */       
      status = 1;
   }
   else
   {
/* read the MII register data */        
      *data = MCF5282_FEC_MMFR_DATA(MCF5282_FEC_MMFR);
/* signal good status to caller */      
      status = TM_ENOERROR;
   }
   
   return status;
}

/**************************************************************************
*
*  FUNCTION: tfEthfecWriteMII()
*  
*  DESCRIPTION: write an ethernet MII phy register
*
*  INPUT:  tt32Bit phy    1 of 32 possible MII physical interfaces
*          tt32Bit reg    1 of 32 possible registers of the MII physical
*                         interface
*          tt32Bit val    data to be written to the MII register
*
*  RETURN: 0 if successful, non-zero otherwise
*
************************************************************************/
tt32Bit tfEthfecWriteMII(tt32Bit phy,tt32Bit reg, tt32Bit val)
{
   tt32Bit temp, status, i;

/* clear any pending MII events */ 
   MCF5282_FEC_EIR |= MCF5282_FEC_EIR_MII;
   
/* Set up MII register data */
   temp = MCF5282_FEC_MMFR_ST        |
          MCF5282_FEC_MMFR_OP_WR     |
          MCF5282_FEC_MMFR_TA        |
          MCF5282_FEC_MMFR_PA(phy)   |
          MCF5282_FEC_MMFR_RA(reg)   |
          MCF5282_FEC_MMFR_DATA(val);
    
/* send the command */    
   MCF5282_FEC_MMFR = temp;

   for (i=0; i < MII_EVENT_TIMEOUT; i++)
   {
/* check if the event posted */
       if(MCF5282_FEC_EIR & MCF5282_FEC_EIR_MII)
       {
/* the event posted - exit the loop */ 
           break;
       }
   } 
   
   if (i == MII_EVENT_TIMEOUT)
   {
/* MII event post timeout - bad status */ 
      status = 1;
   }
   else
   {
/* signal good status */ 
      status = 0;
   }
   
   return status;
}

/**************************************************************************
*
*  FUNCTION: tfEthfecSetSpeed()
*  
*  DESCRIPTION: set the speed of the MII phy interface
*
*  INPUT:  tt32Bit phy    1 of 32 possible MII physical interfaces
*          tt32Bit speed  desired speed of MII physical interface
*
*  RETURN: 0 if successful, non-zero otherwise
*
************************************************************************/
tt32Bit tfEthfecSetSpeed(tt32Bit phy, tt32Bit speed)    
{
   tt32Bit status;
   
/* write a reset to phy command register */
   status = tfEthfecWriteMII(phy, MII_REG_CR, 0x00008000);
   
   if(!status)
   {
/* set the speed and duplex we support for auto-negotiation */
       status = tfEthfecWriteMII(phy, MII_REG_ANAR, 0x000001e0);
       
       switch (speed)
       {
       case PHYAUTO:
/* set to autonegotiate */
           status = tfEthfecWriteMII(phy, MII_REG_CR, 0x00001000);
/* force autonegotiate restart */             
           status = tfEthfecWriteMII(phy, MII_REG_CR, 0x00001200);
           break;
           
       case PHY100:
/* set to 100bt, half dup */ 
           status = tfEthfecWriteMII(phy, MII_REG_CR, 0x00002000);
           break;
         
       case PHY10:
/* set to 10bt, half dup */           
           status = tfEthfecWriteMII(phy, MII_REG_CR, 0x00000000);
           break;
         
         default:
/* bad return status */             
            status = 1;
       }
   } /* if(!status) */
   
   return status;
}

/**************************************************************************
*
*  FUNCTION: tfEthfecPollLinkStatus()
*  
*  DESCRIPTION: determine if the MII phy link is active
*
*  INPUT:  tt32Bit phy    1 of 32 possible MII physical interfaces
*        
*
*  RETURN: 0 if phy link is active, non-zero otherwise
*
************************************************************************/
tt32Bit tfEthfecPollLinkStatus(tt32Bit phy)     
{
    tt32Bit status;
    tt16Bit data;
    
/* read the phy link status */
    status = tfEthfecReadMII(phy, 1, &data);
    
    if(!status)
    {
        if (data & 0x000000004) 
       {
/* phy link is active */           
           status = 0;
       }
       else
       {
/* phy link is inactive */           
           status = 1;
       }
    }
    
    return status;
}

/* end of file - trmcffec.c */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: trmcffec.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 41261    Output checksum: 12009
 */
