/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Source file for Freescale Coldfire MCF5485 FEC driver
 *              This driver is based off of the example driver provided by
 *              Freescale.  It functionality for the Fase Ethernet Controller
 *              (FEC), the multi-channel DMA, and the programmable serial port
 *              (PSC).  Depending on what phy you are using, you will need to
 *              change the function tfPhyInit to operate with the phy that you
 *              are currently using.
 *
 *              At this time, the second ethernet port has not been shown to
 *              work.  The same problem occurs with the example drivers.  All
 *              of the code is present for the second ethernet port, and when
 *              it is shown to work this paragraph will be removed.
 *
 * $Source: source/drivers/mcf5485/trm5485.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:07CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include "common.h"
#include "stdlib.h"

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>

#include "trm5485.h"
#include <trglobal.h>

/*
 * FEC function prototypes
 */
int tfEthfecMIIWrite(tt8Bit ch, tt8Bit phy_addr, tt8Bit reg_addr,
                     tt16Bit data);
int tfEthfecMIIRead(tt8Bit ch, tt8Bit phy_addr, tt8Bit reg_addr,
                    tt16Bit *data);
void tfEthfecMIIInit(tt8Bit ch, tt32Bit sys_clk);
void tfEthfecSetDuplex(tt8Bit ch, tt8Bit duplex);
tt8Bit tfEthfecSetHashAddress(const tt8Bit *addr);
void tfEthfecSetAddress(tt8Bit ch, const tt8Bit *pa);
void tfEthfecReset(tt8Bit ch);
void tfEthfecInit(tt8Bit ch, tt8Bit mode, tt8Bit duplex, const tt8Bit *pa);
void tfEthfecRxStart(tt8Bit ch, char *rxbd, tt8Bit pri);
void tfEthfecRxContinue(tt8Bit ch);
void tfEthfecRxStop(tt8Bit ch);
void tfEthfecTxStart(tt8Bit ch, char *txbd, tt8Bit pri);
void tfEthfecTxContinue(tt8Bit ch);
void tfEthfecTxStop(tt8Bit ch);
void tfEthfecIRQEnable(tt8Bit ch, tt8Bit lvl, tt8Bit pri);
void tfEthfecIRQDisable(tt8Bit ch);
int tfEthfecStart(tt8Bit ch, tt8Bit trcvr, tt8Bit speed, tt8Bit duplex,
                  tt8Bit *mac, int sys_clk, tt8Bit phya, tt8Bit flvl,
                  tt8Bit fpri, tt8Bit dtxpri, tt8Bit drxpri);
void tfEthfecStop(tt8Bit ch);

void tfMcf5485RefillRecvPool(int ch);
void tfMcf5485CleanupXmitQueue(ttUserInterface interfaceHandle, int ch);
int tfEthfecBuffDescInit(tt8Bit ch);

/*
 * PSC Function prototypes
 */
void tfPscUartInit(int port, int sys_clk, int baud);
void tfPscSirInit (int, int, int);
char tfPscInChar (int);
void tfPscOutChar (int, char);
int  tfPscCharPresent (int);

/*
 * DMA function prototypes
 */
void   tfEthdmaIRQEnable(tt8Bit lvl, tt8Bit pri);
void   tfEthdmaIRQDisable(void);
int    tfEthdmaSetInitiator(int initiator);
tt32Bit tfEthdmaGetInitiator(int initiator);
void   tfEthdmaFreeInitiator(int initiator);
int    tfEthdmaSetChannel(int requestor, void (*handler)(void));
int    tfEthdmaGetChannel(int requestor);
void   tfEthdmaFreeChannel(int requestor);
void   tfEthdmaIRQHandler(void);

/*
 * Multichannel DMA API (MCD) functions
 * These functions are from an example API that Freescale provides on their
 * website.
 */
int tfMcdStartDma(int channel, char *srcAddr, ttS16Bit srcIncr,
                  char *destAddr, ttS16Bit destIncr, tt32Bit dmaSize,
                  tt32Bit xferSize, tt32Bit initiator, int priority,
                  tt32Bit flags, tt32Bit funcDesc);
int tfMcdInitDma (dmaRegs *sDmaBarAddr, void *taskTableDest, tt32Bit flags);
int tfMcdDmaStatus (int channel);
int tfMcdXfreProgrQuery (int channel, MCD_XferProg *progRep);
int tfMcdKillDma (int channel);
int tfMcdContinueDma (int channel);
int tfMcdPauseDma (int channel);
int tfMcdResumeDma (int channel);
int tfMcdChecksumQuery (int channel, tt32Bit *csum);
int tfMcdGetCodeSize(void);
int tfMcdGetVersion(char **longVersion);
void tfMcdmemcpy (int *dest, int *src, tt32Bit size);
void tfMcdResumeActions (int channel);

/* macro for setting a location in the variable table */
#define MCD_SET_VAR(taskTab,idx,value) \
    ((tt32Bit *)(taskTab)->varTab)[idx] = value
/* Note that MCD_SET_VAR() is invoked many times in firing up a DMA function,
  so I'm avoiding surrounding it with "do {} while(0)" */

/*
 * MCD Tasks
 */

void  tfMcdStartDmaChainNoEu(int *currBD, short srcIncr, short destIncr,
                             int xferSize, short xferSizeIncr, int *cSave,
                             volatile TaskTableEntry *taskTable, int channel);
void  tfMcdStartDmaSingleNoEu(char *srcAddr, short srcIncr, char *destAddr,
                              short destIncr, int dmaSize, short xferSizeIncr,
                              int flags, int *currBD, int *cSave,
                              volatile TaskTableEntry *taskTable,
                              int channel);
void  tfMcdStartDmaChainEu(int *currBD, short srcIncr, short destIncr,
                           int xferSize, short xferSizeIncr, int *cSave,
                           volatile TaskTableEntry *taskTable, int channel);
void  tfMcdStartDmaSingleEu(char *srcAddr, short srcIncr, char *destAddr,
                            short destIncr, int dmaSize, short xferSizeIncr,
                            int flags, int *currBD, int *cSave,
                            volatile TaskTableEntry *taskTable, int channel);
void  tfMcdStartDmaENetRcv(char *bDBase, char *currBD, char *rcvFifoPtr,
                           volatile TaskTableEntry *taskTable, int channel);
void  tfMcdStartDmaENetXmit(char *bDBase, char *currBD, char *xmitFifoPtr,
                            volatile TaskTableEntry *taskTable, int channel);

/*
 * Phy function prototypes
 */
int tfPhyInit(tt8Bit fec_ch, tt8Bit phy_addr, tt8Bit speed, tt8Bit duplex);
void tfPhyGetReg(tt16Bit* status0, tt16Bit* status1);

ETH_ADDR src[2] = {{0x00,0xCF,0x54,0x85,0xCF,0x00},
                   {0x00,0xCF,0x54,0x85,0xCF,0x01}};

/*
 * This is an API-internal pointer to the DMA's registers
 */
dmaRegs *MCD_dmaBar;

/*
 * These are the real and model task tables as generated by the
 * build process
 */
extern TaskTableEntry MCD_realTaskTableSrc[NCHANNELS];
extern TaskTableEntry MCD_modelTaskTableSrc[NUMOFVARIANTS];

/*
 * However, this (usually) gets relocated to on-chip SRAM, at which
 * point we access them as these tables
 */
volatile TaskTableEntry *MCD_taskTable;
TaskTableEntry *MCD_modelTaskTable;


/*
 * MCD_chStatus[] is an array of status indicators for remembering
 * whether a DMA has ever been attempted on each channel, pausing
 * status, etc.
 */
static int MCD_chStatus[NCHANNELS] =
{
    MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA,
    MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA,
    MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA,
    MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA, MCD_NO_DMA
};

/*
 * Buffer descriptors used for storage of progress info for single Dmas
 * Also used as storage for the DMA for CRCs for single DMAs
 * Otherwise, the DMA does not parse these buffer descriptors
 */
#ifdef MCD_INCLUDE_EU
extern MCD_bufDesc MCD_singleBufDescs[NCHANNELS];
#else
MCD_bufDesc MCD_singleBufDescs[NCHANNELS];
#endif
MCD_bufDesc *MCD_relocBuffDesc;

/*
 * Structure to remember which variant is on which channel
 * TBD- need this?
 */
typedef struct MCD_remVariants_struct MCD_remVariant;
struct MCD_remVariants_struct
{
   int remDestRsdIncr[NCHANNELS];  /* -1,0,1 */
   int remSrcRsdIncr[NCHANNELS];   /* -1,0,1 */
   ttS16Bit remDestIncr[NCHANNELS];     /* DestIncr */
   ttS16Bit remSrcIncr[NCHANNELS];      /* srcIncr */
   tt32Bit remXferSize[NCHANNELS];     /* xferSize */
};

/*
 * Structure to remember the startDma parameters for each channel
 */
MCD_remVariant MCD_remVariants;

typedef struct
{
   unsigned short flags;
   unsigned short length;
   void         * buffer;
} ttBufDesc;

typedef ttBufDesc * ttBufDescPtr;

typedef struct tsMcf5485
{
  ttUserInterface       mcfInterfaceHandle;
  tt8BitPtr             mcfDescPtr;

/*
 * For the FAST Ethernet controller, buffer descriptors are in main memory
 * and must be quad word aligned.
 */

/* Transmit buffer descriptor pointers */
  volatile ttBufDescPtr mcfTxBufDescPtr;
  volatile ttBufDescPtr mcfTxIsrBufDescPtr;
  volatile ttBufDescPtr mcfTxFirstBufDescPtr;
  volatile ttBufDescPtr mcfTxScatterStartBufDescPtr;

/* Receive buffer descriptor pointers */
  volatile ttBufDescPtr mcfRxFirstBufDescPtr;
  volatile ttBufDescPtr mcfRxBufDescPtr;    
  volatile ttBufDescPtr mcfRxIsrBufDescPtr; 
  volatile ttBufDescPtr mcfRxRefillBufDescPtr;
  
/* Keep track of allocated received buffers */
  ttUserBuffer          mcfRxUserBufferList[TM_MCF5485_ETHER_MAX_RECV_BUFD];

/* next packet buffer to process for receive */
  volatile ttUserBufferPtr mcfRxUserBufferPtr;

/* next packet buffer to process for refill received */
  volatile ttUserBufferPtr mcfRxUserRefillBufferPtr;

/* statistics */
  volatile unsigned long        mcfRecvErrorCount;
  volatile unsigned long        mcfRecvPacketCount;
  volatile unsigned long        mcfSendPacketCount;
  volatile unsigned long        mcfBusyPacketCount;
  volatile unsigned long        mcfXmitWait;
  volatile unsigned long        mcfIsrRecvErrorCount;
  int                           mcfFECXmitQueueOn;

/* array of 16-byte aligned mini-buffers that we use to align transmit data */
  tt32Bit                       mcfScatSendAlignBuf[
                                    TM_MCF5485_ETHER_MAX_XMIT_BUFD+1];
} ttMcf5485;

static ttMcf5485 tlMcfStruct[NUM_ETH_PORTS];

/* 
 * This global keeps track of which initiators have been
 * used of the available assignments.  Initiators 0-15 are
 * hardwired.  Initiators 16-31 are multiplexed and controlled
 * via the Initiatior Mux Control Registe (IMCR).  The 
 * assigned requestor is stored with the associated initiator
 * number.  
 */
static char used_reqs[32] = 
{
    DMA_ALWAYS,  DMA_DSPI_RX, DMA_DSPI_TX, DMA_DREQ0,
    DMA_PSC0_RX, DMA_PSC0_TX, DMA_USBEP0,  DMA_USBEP1,
    DMA_USBEP2,  DMA_USBEP3,  DMA_PCI_TX,  DMA_PCI_RX,
    DMA_PSC1_RX, DMA_PSC1_TX, DMA_I2C_RX,  DMA_I2C_TX,
    0,           0,           0,           0,  
    0,           0,           0,           0,  
    0,           0,           0,           0,  
    0,           0,           0,           0
};

/*
 * This global keeps track of which channels have been assigned
 * to tasks.  This methology assumes that no single initiator
 * will be tied to more than one task/channel
 */
typedef struct
{
    int     req;
    void    (*handler)(void);
} DMA_CHANNEL_STRUCT;

static DMA_CHANNEL_STRUCT dma_channel[NCHANNELS] = 
{
    {-1,NULL}, {-1,NULL}, {-1,NULL}, {-1,NULL}, 
    {-1,NULL}, {-1,NULL}, {-1,NULL}, {-1,NULL}, 
    {-1,NULL}, {-1,NULL}, {-1,NULL}, {-1,NULL}, 
    {-1,NULL}, {-1,NULL}, {-1,NULL}, {-1,NULL}
};

/********************************************************************/
/*
 * Write a value to a PHY's MII register.
 *
 * Parameters:
 *  ch          FEC channel
 *  phy_addr    Address of the PHY.
 *  reg_addr    Address of the register in the PHY.
 *  data        Data to be written to the PHY register.
 *
 * Return Values:
 *  0 on failure
 *  1 on success.
 *
 * Please refer to your PHY manual for registers and their meanings.
 * mii_write() polls for the FEC's MII interrupt event (which should
 * be masked from the interrupt handler) and clears it. If after a
 * suitable amount of time the event isn't triggered, a value of 0
 * is returned.
 */
int tfEthfecMIIWrite(tt8Bit ch, tt8Bit phy_addr, tt8Bit reg_addr,
                     tt16Bit data)
{
    int     timeout;
    tt32Bit  eimr;

    tm_assert(tfEthfecMIIWrite, (ch == 0 || ch == 1));

/* Clear the MII interrupt bit */
    MCF_FEC_EIR(ch) = MCF_FEC_EIR_MII;

/* Write to the MII Management Frame Register to kick-off the MII write */
    MCF_FEC_MMFR(ch) = 0
        | MCF_FEC_MMFR_ST_01
        | MCF_FEC_MMFR_OP_WRITE
        | MCF_FEC_MMFR_PA(phy_addr)
        | MCF_FEC_MMFR_RA(reg_addr)
        | MCF_FEC_MMFR_TA_10
        | MCF_FEC_MMFR_DATA(data);

/* Mask the MII interrupt */
    eimr = MCF_FEC_EIMR(ch);
    MCF_FEC_EIMR(ch) &= ~MCF_FEC_EIMR_MII;

/* Poll for the MII interrupt (interrupt should be masked) */
    for (timeout = 0; timeout < FEC_MII_TIMEOUT; timeout++)
    {
        if (MCF_FEC_EIR(ch) & MCF_FEC_EIR_MII)
            break;
    }
    if(timeout == FEC_MII_TIMEOUT)
        return 0;

/* Clear the MII interrupt bit */
    MCF_FEC_EIR(ch) = MCF_FEC_EIR_MII;

/* Restore the EIMR */
    MCF_FEC_EIMR(ch) = eimr;

    return 1;
}
/********************************************************************/
/*
 * Read a value from a PHY's MII register.
 *
 * Parameters:
 *  ch          FEC channel
 *  phy_addr    Address of the PHY.
 *  reg_addr    Address of the register in the PHY.
 *  data        Pointer to storage for the Data to be read
 *              from the PHY register (passed by reference)
 *
 * Return Values:
 *  0 on failure
 *  1 on success.
 *
 * Please refer to your PHY manual for registers and their meanings.
 * mii_read() polls for the FEC's MII interrupt event (which should
 * be masked from the interrupt handler) and clears it. If after a
 * suitable amount of time the event isn't triggered, a value of 0
 * is returned.
 */
int tfEthfecMIIRead(tt8Bit ch, tt8Bit phy_addr, tt8Bit reg_addr,
                    tt16Bit *data)
{
    int timeout;

    tm_assert(tfEthfecMIIRead, (ch == 0 || ch == 1));

/* Clear the MII interrupt bit */
    MCF_FEC_EIR(ch) = MCF_FEC_EIR_MII;

/* Write to the MII Management Frame Register to kick-off the MII read */
    MCF_FEC_MMFR(ch) = 0
        | MCF_FEC_MMFR_ST_01
        | MCF_FEC_MMFR_OP_READ
        | MCF_FEC_MMFR_PA(phy_addr)
        | MCF_FEC_MMFR_RA(reg_addr)
        | MCF_FEC_MMFR_TA_10;

/* Poll for the MII interrupt (interrupt should be masked) */
    for (timeout = 0; timeout < FEC_MII_TIMEOUT; timeout++)
    {
        if (MCF_FEC_EIR(ch) & MCF_FEC_EIR_MII)
            break;
    }

    if(timeout == FEC_MII_TIMEOUT)
        return 0;

/* Clear the MII interrupt bit */
    MCF_FEC_EIR(ch) = MCF_FEC_EIR_MII;

    *data = (tt16Bit)(MCF_FEC_MMFR(ch) & 0x0000FFFF);

    return 1;
}
/********************************************************************/
/*
 * Initialize the MII interface controller
 *
 * Parameters:
 *  ch      FEC channel
 *  sys_clk System Clock Frequency (in MHz)
 */
void tfEthfecMIIInit(tt8Bit ch, tt32Bit sys_clk)
{
    tm_assert(tfEthfecMIIInit, (ch == 0 || ch == 1));

/* NOTE:  IF YOU ARE NOT USING THE MCF5485EVB - READ THIS
 * The MCF5485EVB has a dual port phy which only has the MII control pins
 * connected to FEC0.  If you are using different hardware that is using
 * both ports fully, change MCF_FEC_MSCR(0) to change MCF_FEC_MSCR(ch).
 */


/*
 * Initialize the MII clock (EMDC) frequency
 *
 * Desired MII clock is 2.5MHz
 * MII Speed Setting = System_Clock / (2.5MHz * 2)
 * (plus 1 to make sure we round up)
 */
    MCF_FEC_MSCR(0) = MCF_FEC_MSCR_MII_SPEED((sys_clk/5)+1);
}

/********************************************************************/
/*
 * Set the duplex on the selected FEC controller
 *
 * Parameters:
 *  ch      FEC channel
 *  duplex  FEC_MII_FULL_DUPLEX or FEC_MII_HALF_DUPLEX
 */
void tfEthfecSetDuplex(tt8Bit ch, tt8Bit duplex)
{
    tm_assert(tfEthfecSetDuplex, (ch == 0 || ch == 1));

    switch (duplex)
    {
        case FEC_MII_HALF_DUPLEX:
            MCF_FEC_RCR(ch) |= MCF_FEC_RCR_DRT;
            MCF_FEC_TCR(ch) &= (tt32Bit)~MCF_FEC_TCR_FDEN;
            break;
        case FEC_MII_FULL_DUPLEX:
        default:
            MCF_FEC_RCR(ch) &= (tt32Bit)~MCF_FEC_RCR_DRT;
            MCF_FEC_TCR(ch) |= MCF_FEC_TCR_FDEN;
            break;
    }
}
/********************************************************************/
/*
 * Generate the hash table settings for the given address
 *
 * Parameters:
 *  addr    48-bit (6 byte) Address to generate the hash for
 *
 * Return Value:
 *  The 6 most significant bits of the 32-bit CRC result
 */
tt8Bit tfEthfecSetHashAddress(const tt8Bit *addr)
{
    tt32Bit  crc;
    tt8Bit   byte;
    int     i;
    int     j;

    crc = 0xFFFFFFFF;
    for(i=0; i<6; ++i)
    {
        byte = addr[i];
        for(j=0; j<8; ++j)
        {
            if((byte & 0x01)^(crc & 0x01))
            {
                crc >>= 1;
                crc = crc ^ 0xEDB88320;
            }
            else
                crc >>= 1;
            byte >>= 1;
        }
    }
    return (tt8Bit)(crc >> 26);
}
/********************************************************************/
/*
 * Set the Physical (Hardware) Address and the Individual Address
 * Hash in the selected FEC
 *
 * Parameters:
 *  ch  FEC channel
 *  pa  Physical (Hardware) Address for the selected FEC
 */
void tfEthfecSetAddress(tt8Bit ch, const tt8Bit *pa)
{
    tt8Bit crc;

    tm_assert(tfEthfecSetAddress, (ch == 0 || ch == 1));

/* Set the Physical Address */
    MCF_FEC_PALR(ch) = (tt32Bit)((pa[0]<<24) | (pa[1]<<16) |
                                (pa[2]<<8) | pa[3]);
    MCF_FEC_PAUR(ch) = (tt32Bit)((pa[4]<<24) | (pa[5]<<16));

/* 
 * Calculate and set the hash for given Physical Address
 * in the  Individual Address Hash registers
 */
    crc = tfEthfecSetHashAddress(pa);
    if(crc >= 32)
        MCF_FEC_IAUR(ch) |= (tt32Bit)(1 << (crc - 32));
    else
        MCF_FEC_IALR(ch) |= (tt32Bit)(1 << crc);
}
/********************************************************************/
/*
 * Reset the selected FEC controller
 *
 * Parameters:
 *  ch      FEC channel
 */
void tfEthfecReset(tt8Bit ch)
{
    int i;

    tm_assert(tfEthfecReset, (ch == 0 || ch == 1));

/* Clear any events in the FIFO status registers */
    MCF_FEC_FECRFSR(ch) = (0
        | MCF_FEC_FECRFSR_OF
        | MCF_FEC_FECRFSR_UF
        | MCF_FEC_FECRFSR_RXW
        | MCF_FEC_FECRFSR_FAE
        | MCF_FEC_FECRFSR_IP);
    MCF_FEC_FECTFSR(ch) = (0
        | MCF_FEC_FECRFSR_OF
        | MCF_FEC_FECRFSR_UF
        | MCF_FEC_FECRFSR_RXW
        | MCF_FEC_FECRFSR_FAE
        | MCF_FEC_FECRFSR_IP);

/* Reset the FIFOs */
    MCF_FEC_FRST(ch) |= MCF_FEC_FRST_SW_RST;
    MCF_FEC_FRST(ch) &= ~MCF_FEC_FRST_SW_RST;

/* Set the Reset bit and clear the Enable bit */
    MCF_FEC_ECR(ch) = MCF_FEC_ECR_RESET;

/* Wait at least 8 clock cycles */
    for (i=0; i<10; ++i)
        nop();
}
/********************************************************************/
/*
 * Initialize the selected FEC
 *
 * Parameters:
 *  ch      FEC channel
 *  mode    External interface mode (MII, 7-wire, or internal loopback)
 *  pa      Physical (Hardware) Address for the selected FEC
 */
void tfEthfecInit(tt8Bit ch, tt8Bit mode, tt8Bit duplex, const tt8Bit *pa)
{
    tm_assert(tfEthfecInit, (ch == 0 || ch == 1));

/* Enable all the external interface signals */
    if (mode == FEC_MODE_7WIRE)
    {
        if (ch == 1)
            MCF_GPIO_PAR_FECI2CIRQ |= MCF_GPIO_PAR_FECI2CIRQ_PAR_E17;
        else
            MCF_GPIO_PAR_FECI2CIRQ |= MCF_GPIO_PAR_FECI2CIRQ_PAR_E07;
    }
    else if (mode == FEC_MODE_MII)
    {
        if (ch == 1)
            MCF_GPIO_PAR_FECI2CIRQ |= 0
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDC_EMDC
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDIO_EMDIO
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MII
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E17;
        else
            MCF_GPIO_PAR_FECI2CIRQ |= 0
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E0MDC
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E0MDIO
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E0MII
                | MCF_GPIO_PAR_FECI2CIRQ_PAR_E07;
    }

/* Clear the Individual and Group Address Hash registers */
    MCF_FEC_IALR(ch) = 0;
    MCF_FEC_IAUR(ch) = 0;
    MCF_FEC_GALR(ch) = 0;
    MCF_FEC_GAUR(ch) = 0;

/* Set the Physical Address for the selected FEC */
    tfEthfecSetAddress(ch, pa);

/* Mask all FEC interrupts */
    MCF_FEC_EIMR(ch) = MCF_FEC_EIMR_MASK_ALL;

/* Clear all FEC interrupt events */
    MCF_FEC_EIR(ch) = MCF_FEC_EIR_CLEAR_ALL;

/* Initialize the Receive Control Register */
    MCF_FEC_RCR(ch) = 0
        | MCF_FEC_RCR_MAX_FL(ETH_MAX_FRM)
    #ifdef FEC_PROMISCUOUS
        | MCF_FEC_RCR_PROM
    #endif
        | MCF_FEC_RCR_FCE;

    if (mode == FEC_MODE_MII)
        MCF_FEC_RCR(ch) |= MCF_FEC_RCR_MII_MODE;

    else if (mode == FEC_MODE_LOOPBACK)
        MCF_FEC_RCR(ch) |= MCF_FEC_RCR_LOOP;

/* Set the duplex */
    tfEthfecSetDuplex(ch, duplex);

/* Set Rx FIFO alarm and granularity */
    MCF_FEC_FECRFCR(ch) = 0
        | MCF_FEC_FECRFCR_FRM
        | MCF_FEC_FECRFCR_RXW_MSK
        | MCF_FEC_FECRFCR_GR(7);
    MCF_FEC_FECRFAR(ch) = MCF_FEC_FECRFAR_ALARM(768);

/* Set Tx FIFO watermark, alarm and granularity */
    MCF_FEC_FECTFCR(ch) = 0
        | MCF_FEC_FECTFCR_FRM
        | MCF_FEC_FECTFCR_TXW_MSK
        | MCF_FEC_FECTFCR_GR(7);
    MCF_FEC_FECTFAR(ch) = MCF_FEC_FECTFAR_ALARM(256);
    MCF_FEC_FECTFWR(ch) = MCF_FEC_FECTFWR_X_WMRK_256;

/* Enable the transmitter to append the CRC */
    MCF_FEC_CTCWR(ch) = 0
        | MCF_FEC_CTCWR_TFCW
        | MCF_FEC_CTCWR_CRC;
}
/********************************************************************/
/*
 * Start the FEC Rx DMA task
 *
 * Parameters:
 *  ch      FEC channel
 *  rxbd    First Rx buffer descriptor in the chain
 */
void tfEthfecRxStart(tt8Bit ch, char *rxbd, tt8Bit pri)
{
    tt32Bit  initiator;
    int     channel;
    int     result;

   tm_assert(tfEthfecRxStart, (ch == 0 || ch == 1));

/* Make the initiator assignment */
    result = tfEthdmaSetInitiator(DMA_FEC_RX(ch));
    tm_assert(tfEthfecRxStart, (result == 0));

/* Grab the initiator number */
    initiator = tfEthdmaGetInitiator(DMA_FEC_RX(ch));

/* Determine the DMA channel running the task for the selected FEC */
    channel = tfEthdmaSetChannel(DMA_FEC_RX(ch),
                              (ch == 0) ? tfMcf5485ReceiveISRFec0 :
                                          tfMcf5485ReceiveISRFec1);
    tm_assert(tfEthfecRxStart, (channel != -1));

/* Start the Rx DMA task */
    tfMcdStartDma(channel,
                  (char*)rxbd,
                  0,
                  (char*)MCF_FEC_FECRFDR_ADDR(ch),
                  0,
                  TM_MCF5485_RX_BUF_LEN,
                  0,
                  initiator,
                  (int)pri,
                  0 
                    | MCD_FECRX_DMA   
                    | MCD_INTERRUPT
                    | MCD_TT_FLAGS_CW 
                    | MCD_TT_FLAGS_RL
                    | MCD_TT_FLAGS_SP
                    ,
                  0 
                    | MCD_NO_CSUM
                    | MCD_NO_BYTE_SWAP
                  );
}
/********************************************************************/
/*
 * Continue the Rx DMA task
 *
 * This routine is called after the DMA task has halted after
 * encountering an Rx buffer descriptor that wasn't marked as
 * ready. There is no harm in calling the DMA continue routine
 * if the DMA is not halted.
 *
 * Parameters:
 *  ch      FEC channel
 */
void tfEthfecRxContinue(tt8Bit ch)
{
    int channel;

    tm_assert(tfEthfecRxContinue, (ch == 0 || ch == 1));

/* Determine the DMA channel running the task for the selected FEC */
    channel = tfEthdmaGetChannel(DMA_FEC_RX(ch));
    tm_assert(tfEthfecRxContinue, (channel != -1));

/* Continue/restart the DMA task */
    tfMcdContinueDma(channel);
}
/********************************************************************/
/*
 * Stop all frame receptions on the selected FEC
 *
 * Parameters:
 *  ch  FEC channel
 */
void tfEthfecRxStop(tt8Bit ch)
{
    tt32Bit  mask;
    int     channel;

    tm_assert(tfEthfecRxStop, (ch == 0 || ch == 1));

/* Save off the EIMR value */
    mask = MCF_FEC_EIMR(ch);

/* Mask all interrupts */
    MCF_FEC_EIMR(ch) = 0;

/* Determine the DMA channel running the task for the selected FEC */
    channel = tfEthdmaGetChannel(DMA_FEC_RX(ch));

/* Kill the FEC Tx DMA task */
    if (channel != -1)
        tfMcdKillDma(channel);

/* Free up the FEC requestor from the software maintained initiator list */
    tfEthdmaFreeInitiator(DMA_FEC_RX(ch));

/* Free up the DMA channel */
    tfEthdmaFreeChannel(DMA_FEC_RX(ch));

/* Restore the interrupt mask register value */
    MCF_FEC_EIMR(ch) = mask;
}

/********************************************************************/
/*
 * Start the FEC Tx DMA task
 *
 * Parameters:
 *  ch      FEC channel
 *  txbd    First Tx buffer descriptor in the chain
 */
void tfEthfecTxStart(tt8Bit ch, char *txbd, tt8Bit pri)
{
    tt32Bit  initiator;
    int     channel;
    int     result;

/* Make the initiator assignment */
    result = tfEthdmaSetInitiator(DMA_FEC_TX(ch));
    tm_assert(tfEthfecTxStart, (result == 0));

/* Grab the initiator number */
    initiator = tfEthdmaGetInitiator(DMA_FEC_TX(ch));
    tm_assert(tfEthfecTxStart, (initiator != 0));

/*
 * Determine the DMA channel running the task for the selected FEC.  We are
 * not using an interrupt on transmit, so do not provide a handler function
 * to tfEthdamSetChannel.
 */
    channel = tfEthdmaSetChannel(DMA_FEC_TX(ch), NULL);
    tm_assert(tfEthfecTxStart, (channel != -1));

/* Start the Tx DMA task */
    tfMcdStartDma(channel,
                  (char*)txbd,
                  0,
                  (char*)MCF_FEC_FECTFDR_ADDR(ch),
                  0,
                  ETH_MTU,
                  0,
                  initiator,
                  (int)pri,
                  0 
                    | MCD_FECTX_DMA   
                    | MCD_TT_FLAGS_CW 
                    | MCD_TT_FLAGS_RL
                    | MCD_TT_FLAGS_SP
                    ,
                  0 
                    | MCD_NO_CSUM
                    | MCD_NO_BYTE_SWAP
                  );
}
/********************************************************************/
/*
 * Continue the Tx DMA task
 *
 * This routine is called after the DMA task has halted after
 * encountering an Tx buffer descriptor that wasn't marked as
 * ready.  There is no harm in calling the continue DMA routine
 * if the DMA was not paused.
 *
 * Parameters:
 *  ch      FEC channel
 */
void tfEthfecTxContinue(tt8Bit ch)
{
    int channel;

/* Determine the DMA channel running the task for the selected FEC */
    channel = tfEthdmaGetChannel(DMA_FEC_TX(ch));
    tm_assert(tfEthfecTxContinue, (channel != -1));

/* Continue/restart the DMA task */
    tfMcdContinueDma((int)channel);
}
/********************************************************************/
/*
 * Stop all transmissions on the selected FEC and kill the DMA task
 *
 * Parameters:
 *  ch  FEC channel
 */
void tfEthfecTxStop(tt8Bit ch)
{
    tt32Bit  mask;
    int     channel;

    tm_assert(tfEthfecTxStop, (ch == 0 || ch == 1));

/* Save off the EIMR value */
    mask = MCF_FEC_EIMR(ch);

/* Mask all interrupts */
    MCF_FEC_EIMR(ch) = 0;

/* If the Ethernet is still enabled... */
    if (MCF_FEC_ECR(ch) & MCF_FEC_ECR_ETHER_EN)
    {
/* Issue the Graceful Transmit Stop */
        MCF_FEC_TCR(ch) |= MCF_FEC_TCR_GTS;

/* Wait for the Graceful Stop Complete interrupt */
        while(!(MCF_FEC_EIR(ch) & MCF_FEC_EIR_GRA))
        {
            if (!(MCF_FEC_ECR(ch) & MCF_FEC_ECR_ETHER_EN))
                break;
        }

/* Clear the Graceful Stop Complete interrupt */
        MCF_FEC_EIR(ch) = MCF_FEC_EIR_GRA;
    }

/* Determine the DMA channel running the task for the selected FEC */
    channel = tfEthdmaGetChannel(DMA_FEC_TX(ch));

/* Kill the FEC Tx DMA task */
    if (channel != -1)
        tfMcdKillDma(channel);

/* Free up the FEC requestor from the software maintained initiator list */
    tfEthdmaFreeInitiator(DMA_FEC_TX(ch));

/* Free up the DMA channel */
    tfEthdmaFreeChannel(DMA_FEC_TX(ch));

/* Restore the interrupt mask register value */
    MCF_FEC_EIMR(ch) = mask;
}


/********************************************************************/
/*
 * Enable interrupts on the selected FEC
 *
 * Parameters:
 *  ch      FEC channel
 *  pri     Interrupt Priority
 *  lvl     Interrupt Level
 */
void tfEthfecIRQEnable(tt8Bit ch, tt8Bit lvl, tt8Bit pri)
{
    tm_assert(tfEthfecIRQEnable, (ch == 0 || ch == 1));
    tm_assert(tfEthfecIRQEnable, (lvl > 0 && lvl < 8));
    tm_assert(tfEthfecIRQEnable, (pri < 8));

/* Setup the appropriate ICR */
    MCF_INTC_ICRn((ch == 0) ? 39 : 38) = (tt8Bit)(0
        | MCF_INTC_ICRn_IP(pri)
        | MCF_INTC_ICRn_IL(lvl));

/* Clear any pending FEC interrupt events */
    MCF_FEC_EIR(ch) = MCF_FEC_EIR_CLEAR_ALL;

/* Unmask all FEC interrupts */
    MCF_FEC_EIMR(ch) = MCF_FEC_EIMR_UNMASK_ALL;

/* Unmask the FEC interrupt in the interrupt controller */
    if (ch == 0)
        MCF_INTC_IMRH &= ~MCF_INTC_IMRH_INT_MASK39;
    else
        MCF_INTC_IMRH &= ~MCF_INTC_IMRH_INT_MASK38;
}
/********************************************************************/
/*
 * Disable interrupts on the selected FEC
 *
 * Parameters:
 *  ch      FEC channel
 */
void tfEthfecIRQDisable(tt8Bit ch)
{
    tm_assert(tfEthfecIRQDisable, (ch == 0 || ch == 1));

/* Mask all FEC interrupts */
    MCF_FEC_EIMR(ch) = MCF_FEC_EIMR_MASK_ALL;

/* Mask the FEC interrupt in the interrupt controller */
    if (ch == 0)
        MCF_INTC_IMRH |= MCF_INTC_IMRH_INT_MASK39;
    else
        MCF_INTC_IMRH |= MCF_INTC_IMRH_INT_MASK38;
}
/********************************************************************/
/*
 * FEC interrupt handler
 * All interrupts are multiplexed into a single vector for each
 * FEC module. The lower level interrupt handler passes in the
 * channel to this handler. Note that the receive interrupt is
 * generated by the Multi-channel DMA FEC Rx task.
 *
 * Parameters:
 * ch       FEC channel
 */
static void tfEthfecIRQHandler(tt8Bit ch)
{
    tt32Bit event;

/*
 * Determine which interrupt(s) asserted by AND'ing the
 * pending interrupts with those that aren't masked.
 */
    event = MCF_FEC_EIR(ch) & MCF_FEC_EIMR(ch);

    #ifdef DEBUG
    if (event != MCF_FEC_EIR(ch))
        printf("Pending but not enabled: 0x%08X\n",(event ^ MCF_FEC_EIR(ch)));
    #endif

/* Clear the event(s) in the EIR immediately */
    MCF_FEC_EIR(ch) = event;

    if (event & MCF_FEC_EIR_RFERR)
    {
        #ifdef DEBUG
        printf("RFERR\n");
        printf("FECRFSR%d = 0x%08x\n",ch,MCF_FEC_FECRFSR(ch));
        tfEthfecStop(ch);
        #endif
    }
    if (event & MCF_FEC_EIR_XFERR)
    {
        #ifdef DEBUG
        printf("XFERR\n");
        #endif
    }
    if (event & MCF_FEC_EIR_XFUN)
    {
        #ifdef DEBUG
        printf("XFUN\n");
        tfEthfecStop(ch);
        #endif
    }
    if (event & MCF_FEC_EIR_RL)
    {
        #ifdef DEBUG
        printf("RL\n");
        #endif
    }
    if (event & MCF_FEC_EIR_LC)
    {
        #ifdef DEBUG
        printf("LC\n");
        #endif
    }
    if (event & MCF_FEC_EIR_BABT)
    {
        #ifdef DEBUG
        printf("BABT\n");
        #endif
    }
    if (event & MCF_FEC_EIR_BABR)
    {
        #ifdef DEBUG
        printf("BABR\n");
        #endif
    }
    if (event & MCF_FEC_EIR_HBERR)
    {
        #ifdef DEBUG
        printf("HBERR\n");
        #endif
    }
}

__interrupt__
void tfEthfec0IRQHandler(void)
{
    tfEthfecIRQHandler(0);
}

__interrupt__
void tfEthfec1IRQHandler(void)
{
    tfEthfecIRQHandler(1);
}
/********************************************************************/
/*
 * Configure the selected Ethernet port and enable all operations
 *
 * Parameters:
 *  ch      FEC channel
 *  trcvr   Transceiver mode (MII, 7-Wire or internal loopback)
 *  speed   Maximum operating speed (valid in MII mode only)
 *  duplex  Full or Half-duplex (MII only)
 *  mac     Physical (MAC) Address
 *  sys_clk System clock frequency - used for MII speed calculation
 *  phya    Address of the MII Phy (valid in MII mode only)
 *  flvl    FEC interrupt level
 *  fpri    FEC interrupt priority
 *  dtxpri  Tx DMA task priority
 *  drxpri  Rx DMA task priority
 *
 * Return Value:
 *  1 if the Ethernet was successfully started
 *  0 otherwise 
 */
int tfEthfecStart(tt8Bit ch, tt8Bit trcvr, tt8Bit speed, tt8Bit duplex,
                  tt8Bit *mac, int sys_clk, tt8Bit phya, tt8Bit flvl,
                  tt8Bit fpri, tt8Bit dtxpri, tt8Bit drxpri)
{
    int errorCode = TM_DEV_OKAY;

    tm_assert(tfEthfecStart, (ch == 0 || ch == 1));

/* Disable FEC interrupts */
    tfEthfecIRQDisable(ch);
    if(ch == 1)
    {
        tfEthfecIRQDisable(0);
//        MCF_FEC_ECR(0) &= (~MCF_FEC_ECR_ETHER_EN);
    }

/* Initialize the FEC */
    tfEthfecReset(ch);
    tfEthfecInit(ch, trcvr, duplex, mac);

    if (trcvr == FEC_MODE_MII)
    {

/* Initialize the MII interface */
        tfEthfecMIIInit(ch, (tt32Bit)sys_clk);

/* Initialize the PHY interface */
/* 
 * NOTE:  tfPhyInit was written to interface to the BCM5222 on the MCF5485EVB.
 *        On this board, the MII control pins are only connected for FEC0.
 *        For this reason, all of the MII read and write commands are called
 *        using FEC0 only.  Depending on your hardware, you may want to change
 *        this function.
 */
        errorCode = tfPhyInit(ch, phya, speed, duplex);
    }

    if(errorCode == TM_DEV_OKAY)
    {
/* Enable the multi-channel DMA tasks */
        tfEthfecRxStart(ch, (char*)tlMcfStruct[ch].mcfRxFirstBufDescPtr,
                        drxpri);

        tfEthfecTxStart(ch, (char*)tlMcfStruct[ch].mcfTxFirstBufDescPtr,
                        dtxpri);

/* Initialize and enable FEC interrupts */
        tfEthfecIRQEnable(ch, flvl, fpri);

/* Enable the FEC channel */
        MCF_FEC_ECR(ch) |= MCF_FEC_ECR_ETHER_EN;

        if(ch == 1)
        {
            tfEthfecIRQEnable(0, FEC_INTC_LVL(0), FEC_INTC_PRI(0));
//            MCF_FEC_ECR(0) |= MCF_FEC_ECR_ETHER_EN;
        }
    }

    return errorCode;
}
/********************************************************************/
/*
 * Stop the selected Ethernet port
 *
 * Parameters:
 *  ch      FEC channel
 */
void tfEthfecStop(tt8Bit ch)
{
    tt32Bit level;
    
/* Disable interrupts */
    level = (tt32Bit)asm_set_ipl(7);

/* Gracefully disable the receiver and transmitter */
    tfEthfecTxStop(ch);
    tfEthfecRxStop(ch);

/* Disable FEC interrupts */
    tfEthfecIRQDisable(ch);

/* Disable the FEC channel */
    MCF_FEC_ECR(ch) &= ~MCF_FEC_ECR_ETHER_EN;

/* Restore interrupt level */
    asm_set_ipl(level);
}
/********************************************************************/

int tfMcf5485Open(ttUserInterface interfaceHandle)
{
    tt8Bit ch = 0;
    int errorCode = TM_DEV_OKAY;

/*
 * For this setup, channels are opened in order (channel 0 first, then
 * channel 1).  This maintains an easy method of checking which channel
 * we are working with by just checking the index of the interface handle
 * in our interface handle array.
 */
/* Use the first available channel */
    for(ch=0; ch<NUM_ETH_PORTS; ch++)
    {
        if(tlMcfStruct[ch].mcfInterfaceHandle == NULL)
        {
            break;
        }
    }

/* Exit if we are unable to find an empty structure */
    if(ch >= NUM_ETH_PORTS)
    {
        errorCode = TM_DEV_ERROR;
        goto mcfOpenEnd;
    }

/* Clear out the main data structure */
    tm_bzero((ttUserVoidPtr) &tlMcfStruct[ch], sizeof(tlMcfStruct[ch]));

/* Remember our interface ID */
    tlMcfStruct[ch].mcfInterfaceHandle = interfaceHandle;

/* Save whether the user enabled the Treck transmit queue */
    tlMcfStruct[ch].mcfFECXmitQueueOn = 
        tfInterfaceXmitQueueOn(interfaceHandle);

/* Allocate and initialize the buffer descriptors */
    errorCode = tfEthfecBuffDescInit(ch);

/* Start up our ethernet port. */
    if(errorCode == TM_DEV_OKAY)
    {
        errorCode = tfEthfecStart(ch,
                                  FEC_MODE_MII,
                                  FEC_MII_100BASE_TX,
                                  FEC_MII_FULL_DUPLEX,
                                  src[ch], 
                                  SYSTEM_CLOCK, 
                                  (tt8Bit)FEC_PHY(ch), 
                                  (tt8Bit)FEC_INTC_LVL(ch),
                                  (tt8Bit)FEC_INTC_PRI(ch),
                                  (tt8Bit)FECTX_DMA_PRI(ch),
                                  (tt8Bit)FECRX_DMA_PRI(ch));
    }
mcfOpenEnd:
    return errorCode;
}

int tfMcf5485Close(ttUserInterface interfaceHandle)
{
    ttBufDescPtr    bufDescPtr;
    ttUserBufferPtr userBufPtr;
    tt32Bit         level;
    int             i;
    tt8Bit          ch;
    int             errorCode = TM_DEV_OKAY;

/* Disable interrupts */
    level = (tt32Bit)asm_set_ipl(7);

/* Figure out what channel we are working with */
    ch = 0;
    while((ch < NUM_ETH_PORTS)
            && (interfaceHandle != tlMcfStruct[ch].mcfInterfaceHandle))
    {
        ch++;
    }

/* Make sure we found the channel we are working with */
    if(ch < NUM_ETH_PORTS)
    {
/* 
 * Free leftover memory:
 *   First, send complete on any remaining packets in the transmit ring.
 */
        bufDescPtr = tlMcfStruct[ch].mcfTxFirstBufDescPtr;
        for (i = 0; i < TM_MCF5485_ETHER_MAX_XMIT_BUFD; i++)
        {
/* Call send complete to free this entry */
            if ( ((bufDescPtr->flags) & TX_BD_L ) &&
                 (bufDescPtr->buffer != (char TM_FAR *) 0) )
            {
                tfSendCompleteInterface( interfaceHandle,
                                         TM_DEV_SEND_COMPLETE_DRIVER );
            }
            bufDescPtr++;
        }

/* Next, free any buffers in the receive ring */
        bufDescPtr = tlMcfStruct[ch].mcfRxFirstBufDescPtr;
        userBufPtr = (void**) &(tlMcfStruct[ch].mcfRxUserBufferList[0]);
        for (i = 0; i < TM_MCF5485_ETHER_MAX_RECV_BUFD; i++)
        {
            if ((bufDescPtr->buffer) != (char TM_FAR *) 0)
            {
                tfFreeDriverBuffer(*userBufPtr);
            }
            bufDescPtr++;
            userBufPtr++;
        }
        
/* Free our Buffer descriptor ring last */
        if (tlMcfStruct[ch].mcfDescPtr != TM_8BIT_NULL_PTR)
        {
            tm_free_raw_buffer(tlMcfStruct[ch].mcfDescPtr);
        }

/* Enable interrupts */
        asm_set_ipl(level);

/* Stop the ethernet port */
        tfEthfecStop(ch);
    }
    else
    {
        errorCode = TM_DEV_OKAY;
    }
    return errorCode;
}

int tfMcf5485Send(
    ttUserInterface  interfaceHandle,
    char TM_FAR    * bufferPtr,
    int              length,
    int              flag)
{
/* Just send the data out port 0 for now */
    ttBufDescPtr            bufDescPtr;
    ttBufDescPtr            startBufDescPtr;
    int                     i;
    int                     errorCode = TM_DEV_OKAY;
    tt8BitPtr               alignedBufferPtr;
    unsigned short          nonAlignedBytes;
    int                     scatSendAlignBufIndex;
    register tt8BitPtr      scatSendAlignBufPtr;
    unsigned short TM_FAR   *flagsPtr;
    int                     ch = 0;

/* Figure out what channel we are working with */
    while((ch < NUM_ETH_PORTS)
            && (interfaceHandle != tlMcfStruct[ch].mcfInterfaceHandle))
    {
        ch++;
    }

/* Make sure we found the channel we are working with */
    if(ch >= NUM_ETH_PORTS)
    {
        errorCode = TM_DEV_ERROR;
        goto mcfSendExit;
    }
/*
 * If the transmit queue is enabled, then check to make sure that the required
 * number of frame descriptors are ready - if they aren't, then we return
 * TM_DEV_ERROR so that this packet is queued in the transmit queue for
 * later
 */
    if (tlMcfStruct[ch].mcfFECXmitQueueOn != 0)
    {
        i = 1;
        if (((tt32Bit)bufferPtr) & 3)
        {
/* we will need another frame descriptor for data alignment */            
            i++;
        }

        for (bufDescPtr = tlMcfStruct[ch].mcfTxBufDescPtr; i != 0; i--)
        {
            if (!(bufDescPtr->flags & TX_BD_SW_COMPLETE))
            {
/* this frame descriptor isn't available, retry the transmit later */
                errorCode = TM_DEV_ERROR;
                goto mcfSendExit;
            }
            
/* advance to the next transmit frame descriptor */
            if (bufDescPtr->flags & TX_BD_W)
            {
                bufDescPtr = tlMcfStruct[ch].mcfTxFirstBufDescPtr;
            }
            else
            {
                bufDescPtr++;
            }
        }
    }

/* align the packet data to start on a 4-byte boundary */

/* check if not align before trying to align
 * copy the non-aligned bytes from the front of the packet to the extra space
 * past the end of the packet so that they are aligned, the first frame
 * descriptor points here. The remaining aligned bytes are pointed by
 * by a 2nd frame descriptor.
 */
    if ( ((tt32Bit)bufferPtr) & 3)
    {
        alignedBufferPtr = 
            (tt8BitPtr) ((((tt32Bit) bufferPtr) + 3) & ~((tt32Bit) 3));
        nonAlignedBytes = (unsigned short)(alignedBufferPtr - ((tt8BitPtr) bufferPtr));
        length -= nonAlignedBytes;

/* Wait for a free buffer descriptor */
        if (!(tlMcfStruct[ch].mcfTxBufDescPtr->flags
              & TX_BD_SW_COMPLETE))
        {
            do
            {
/* If user transmit task, yield the CPU, otherwise, do nothing */
                tfInterfaceSpinLock(interfaceHandle);
                tlMcfStruct[ch].mcfXmitWait++;

/* cleanup any transmits that have already completed */
                tfMcf5485CleanupXmitQueue(interfaceHandle, ch);
            }
            while (!(tlMcfStruct[ch].mcfTxBufDescPtr->flags
                     & TX_BD_SW_COMPLETE ));
        }

/* get an aligned buffer for this specific frame descriptor to copy the
   non-aligned data to */
        scatSendAlignBufIndex = tlMcfStruct[ch].mcfTxBufDescPtr
                                - tlMcfStruct[ch].mcfTxFirstBufDescPtr;
        
/* get a 4-byte aligned buffer to copy the non-aligned data to */
        scatSendAlignBufPtr = 
            (tt8BitPtr) ((((tt32Bit)
                           &(tlMcfStruct[ch].mcfScatSendAlignBuf[
                                 scatSendAlignBufIndex]) ) + 3)
                         & ~((tt32Bit) 3));

/* copy the non-aligned bytes from the front of the packet data to the
   4-byte aligned buffer */
        for (i = (nonAlignedBytes - 1); i >= 0; i--)
        {
            scatSendAlignBufPtr[i] = (unsigned char)bufferPtr[i];
        }
        bufferPtr = (char TM_FAR *) alignedBufferPtr;
    
/* use this transmit buffer descriptor for the non-aligned bytes, which we
   have copied to a 4-byte aligned buffer */
        bufDescPtr = tlMcfStruct[ch].mcfTxBufDescPtr++;
        if (bufDescPtr->flags & TX_BD_W)
        {
            tlMcfStruct[ch].mcfTxBufDescPtr =
                tlMcfStruct[ch].mcfTxFirstBufDescPtr;
        }
        bufDescPtr->buffer = (void TM_FAR *)scatSendAlignBufPtr;
        bufDescPtr->length = nonAlignedBytes;
    
/* Clear the previous status (but preserve the WRAP bit */
        bufDescPtr->flags &= TX_BD_W;

/* More buffers to come */
        if (tlMcfStruct[ch].mcfTxScatterStartBufDescPtr ==
                (ttBufDescPtr)0)
        {
            tlMcfStruct[ch].mcfTxScatterStartBufDescPtr = bufDescPtr;
        }
    }

/* Wait for a free buffer descriptor */    
    if (!(tlMcfStruct[ch].mcfTxBufDescPtr->flags
          & TX_BD_SW_COMPLETE))
    {
        do
        {
/* If user transmit task, yield the CPU, otherwise, do nothing */
            tfInterfaceSpinLock(interfaceHandle);
            tlMcfStruct[ch].mcfXmitWait++;

/* cleanup any transmits that have already completed */
            tfMcf5485CleanupXmitQueue(interfaceHandle, ch);
        }
        while (!(tlMcfStruct[ch].mcfTxBufDescPtr->flags
                 & TX_BD_SW_COMPLETE ));
    }

    bufDescPtr = tlMcfStruct[ch].mcfTxBufDescPtr++;
    if (bufDescPtr->flags & TX_BD_W)
    {
        tlMcfStruct[ch].mcfTxBufDescPtr =
            tlMcfStruct[ch].mcfTxFirstBufDescPtr;
    }
    bufDescPtr->buffer = (void TM_FAR *)bufferPtr;
    bufDescPtr->length = (unsigned short)length;
    
/* Clear the previous status (but preserve the WRAP bit and interrupt bit) */
    bufDescPtr->flags &= TX_BD_W;
    
/* If this is the last buffer in the frame then xmit it */
    if (flag == TM_USER_BUFFER_LAST)
    {
/* Now mark the last bd ready */
        flagsPtr = &bufDescPtr->flags;
        *flagsPtr |= (TX_BD_R |
                      TX_BD_L |
                      TX_BD_TC);

/* Check to see if we are xmitting scattered data */
        if (tlMcfStruct[ch].mcfTxScatterStartBufDescPtr !=
                (ttBufDescPtr)0)
        {
/* We have at least one more piece to xmit if we are here */
            startBufDescPtr = tlMcfStruct[ch].mcfTxScatterStartBufDescPtr;
            tlMcfStruct[ch].mcfTxScatterStartBufDescPtr = (ttBufDescPtr)0;
/* 
 * Go ahead and xmit the first packets (with more set) 
 * by walking the list backwards
 * We do this so that we don't need a critical section
 * in order to keep from getting gaps inbetween the 
 * pieces
 * All we need to do here is update the
 * ready flag
 */
            do
            {
                if (bufDescPtr == tlMcfStruct[ch].mcfTxFirstBufDescPtr)
                {
/* We are at the front of the list so go to the end */
                    bufDescPtr += TM_MCF5485_ETHER_MAX_XMIT_BUFD;
                }
                else
                {
/* Go back one buffer descriptor */
                    bufDescPtr--;
                }
                flagsPtr = &bufDescPtr->flags;
                *flagsPtr |= TX_BD_R;
            } while (bufDescPtr != startBufDescPtr);
        }

/* Tell the DMA to check for more buffers to transmit */
        tfEthfecTxContinue((tt8Bit)ch);

/* cleanup any transmits that have already completed */
        tfMcf5485CleanupXmitQueue(interfaceHandle, ch);
    }
    else
    {
/* More buffers to come */
        if (tlMcfStruct[ch].mcfTxScatterStartBufDescPtr ==
                (ttBufDescPtr)0)
        {
            tlMcfStruct[ch].mcfTxScatterStartBufDescPtr = bufDescPtr;
        }
    }

mcfSendExit:
    return errorCode;
}

int tfMcf5485Receive(
    ttUserInterface interfaceHandle,
    char TM_FAR    ** dataPtrPtr,
    int  TM_FAR     * lengthPtr,
    ttUserBufferPtr   userBufferHandlePtr)
{
    int                         errorCode;
    int                         ch;
    register ttBufDescPtr       bufDescPtr;
    unsigned short TM_FAR *     flagsPtr;

    TM_UNREF_IN_ARG(interfaceHandle);
    errorCode = TM_ENOERROR;
    ch = 0;
    
/* Figure out what channel we are working with */
    while((ch < NUM_ETH_PORTS)
            && (interfaceHandle != tlMcfStruct[ch].mcfInterfaceHandle))
    {
        ch++;
    }

    if(ch >= NUM_ETH_PORTS)
    {
        errorCode = TM_DEV_ERROR;
        goto mcfReceiveEnd;
    }

    flagsPtr = &tlMcfStruct[ch].mcfRxBufDescPtr->flags;

/* Skip pass error packets who are not empty or have an error */
    if (( !( *flagsPtr &  RX_BD_E ) ) && ( *flagsPtr & RX_BD_ERROR ))
    {
        tlMcfStruct[ch].mcfRecvErrorCount++;
        
/* give tfMcf5282RefillRecvPool ownership of this frame descriptor */
        *flagsPtr |= RX_BD_SW_REFILL;
                
/* Return the userBuffer Handle for this data pointer */
        if (*flagsPtr & RX_BD_W)
        {
            tlMcfStruct[ch].mcfRxBufDescPtr =
                tlMcfStruct[ch].mcfRxFirstBufDescPtr;

            tlMcfStruct[ch].mcfRxUserBufferPtr =
                tlMcfStruct[ch].mcfRxUserBufferList;
        }
        else
        {
            tlMcfStruct[ch].mcfRxBufDescPtr++;
            tlMcfStruct[ch].mcfRxUserBufferPtr++;
        }

        errorCode = TM_EWOULDBLOCK;
    }
    else
    {
/* Check to see if we own the Buffer Descriptor */
        if (*flagsPtr & RX_BD_E)
        {
/* odd, we could not find a complete received frame! */
            tlMcfStruct[ch].mcfRecvErrorCount++;

            errorCode = TM_EWOULDBLOCK;
        }
        else
        {
            bufDescPtr = tlMcfStruct[ch].mcfRxBufDescPtr++;
            *userBufferHandlePtr = *tlMcfStruct[ch].mcfRxUserBufferPtr++;
            if (*flagsPtr & RX_BD_W)
            {
                tlMcfStruct[ch].mcfRxBufDescPtr =
                    tlMcfStruct[ch].mcfRxFirstBufDescPtr;

                tlMcfStruct[ch].mcfRxUserBufferPtr =
                    tlMcfStruct[ch].mcfRxUserBufferList;
            }

/* We are returning the pointer to the data */
            *dataPtrPtr = (char TM_FAR *)(bufDescPtr->buffer);

/* Zero out the bufferPtr so we can refill it later */
            bufDescPtr->buffer=(void TM_FAR *)0;
/* give tfMcf5282RefillRecvPool ownership of this frame descriptor */
            *flagsPtr |= RX_BD_SW_REFILL;

/* We are returning the length */
            *lengthPtr  = bufDescPtr->length;

/* Up our packet count */
            tlMcfStruct[ch].mcfRecvPacketCount++;
/* refill this receive descriptor */
            tfMcf5485RefillRecvPool(ch);
        }
    }
mcfReceiveEnd:
    return(errorCode);
}

void tfMcf5485ReceiveISRFec0(void)
{
  register unsigned short  TM_FAR       *flagsPtr;
  int                                   recvNotifyCount = 0;

/* loop on frames having the Empty and SW_ISR bits NOT set */
/* Only notify while the frames are not empty */
    flagsPtr = &tlMcfStruct[0].mcfRxIsrBufDescPtr->flags;
    while (!(*flagsPtr & (RX_BD_E|RX_BD_SW_ISR)))
    {
#ifdef TM_USE_DRV_SCAT_RECV
/* if this frame descriptor is the last one in a frame, then we have
   completed receiving a frame */
        if (*flagsPtr & RX_BD_L)
        {
#endif /* TM_USE_DRV_SCAT_RECV */
            recvNotifyCount++;
#ifdef TM_USE_DRV_SCAT_RECV
        }
#endif /* TM_USE_DRV_SCAT_RECV */
        *flagsPtr |= RX_BD_SW_ISR;
        if (*flagsPtr & RX_BD_W)
        {
            tlMcfStruct[0].mcfRxIsrBufDescPtr =
                tlMcfStruct[0].mcfRxFirstBufDescPtr;
        }
        else
        {
            tlMcfStruct[0].mcfRxIsrBufDescPtr++;
        }

        flagsPtr = &tlMcfStruct[0].mcfRxIsrBufDescPtr->flags;
    }

    if (recvNotifyCount != 0)
    {
        tfNotifyInterfaceIsr(tlMcfStruct[0].mcfInterfaceHandle,
                             recvNotifyCount, 0, 0UL, 0UL );
    }
}

void tfMcf5485ReceiveISRFec1(void)
{
  register unsigned short  TM_FAR       *flagsPtr;
  int                                   recvNotifyCount = 0;

/* loop on frames having the Empty and SW_ISR bits NOT set */
/* Only notify while the frames are not empty */
    flagsPtr = &tlMcfStruct[1].mcfRxIsrBufDescPtr->flags;
    while (!(*flagsPtr & (RX_BD_E|RX_BD_SW_ISR)))
    {
#ifdef TM_USE_DRV_SCAT_RECV
/* if this frame descriptor is the last one in a frame, then we have
   completed receiving a frame */
        if (*flagsPtr & RX_BD_L)
        {
#endif /* TM_USE_DRV_SCAT_RECV */
            recvNotifyCount++;
#ifdef TM_USE_DRV_SCAT_RECV
        }
#endif /* TM_USE_DRV_SCAT_RECV */
        *flagsPtr |= RX_BD_SW_ISR;
        if (*flagsPtr & RX_BD_W)
        {
            tlMcfStruct[1].mcfRxIsrBufDescPtr =
                tlMcfStruct[1].mcfRxFirstBufDescPtr;
        }
        else
        {
            tlMcfStruct[1].mcfRxIsrBufDescPtr++;
        }

        flagsPtr = &tlMcfStruct[1].mcfRxIsrBufDescPtr->flags;
    }

    if (recvNotifyCount != 0)
    {
        tfNotifyInterfaceIsr(tlMcfStruct[1].mcfInterfaceHandle,
                             recvNotifyCount, 0, 0UL, 0UL );
    }
}

void tfMcf5485RefillRecvPool(int ch)
{
    ttUserBuffer            userBuffer;
    unsigned short TM_FAR * flagsPtr;
    void           TM_FAR * dataPtr;
        
    flagsPtr  = &tlMcfStruct[ch].mcfRxRefillBufDescPtr->flags;
/* Loop while it is empty or an error */
    while (*flagsPtr & RX_BD_SW_REFILL)
    {
        if (tlMcfStruct[ch].mcfRxRefillBufDescPtr->buffer
                == (void TM_FAR *)0 )
        {
/* Get a new buffer aligned on a 4-byte boundary */
            dataPtr = (void TM_FAR *)
                tfGetDriverBuffer(
                    &userBuffer,
                    TM_MCF5485_RX_BUF_LEN, TM_ETHER_HW_ALIGN);
            if (userBuffer == (ttUserBuffer)0)
            {
/* No memory so stop the refill */
                break;
            }

            tlMcfStruct[ch].mcfRxRefillBufDescPtr->buffer = dataPtr;
            *tlMcfStruct[ch].mcfRxUserRefillBufferPtr = userBuffer;
        }
        
        tlMcfStruct[ch].mcfRxRefillBufDescPtr->length = TM_MCF5485_RX_BUF_LEN;
/* Clear the status Except for the wrap bit and the interrupt bit */
        *flagsPtr &= RX_BD_W | RX_BD_INTERRUPT;
/* Mark this BD as ready to use */
        *flagsPtr |= RX_BD_E;
        if (*flagsPtr & RX_BD_W)
        {
            tlMcfStruct[ch].mcfRxRefillBufDescPtr =
                tlMcfStruct[ch].mcfRxFirstBufDescPtr;

            tlMcfStruct[ch].mcfRxUserRefillBufferPtr =
                tlMcfStruct[ch].mcfRxUserBufferList;
        }
        else
        {
            tlMcfStruct[ch].mcfRxRefillBufDescPtr++;
            tlMcfStruct[ch].mcfRxUserRefillBufferPtr++;
        }
        flagsPtr  = &tlMcfStruct[ch].mcfRxRefillBufDescPtr->flags;

/*
 * Let the DMA know that there are new Rx BDs (in case it is waiting for an
 * empty one).
 */
    tfEthfecRxContinue((tt8Bit)ch);
    }
}

int tfMcf5485Ioctl(
    ttUserInterface interfaceHandle, int flag,
    void TM_FAR * optionPtr, int optionLen)
{
    tt8Bit           ch = 0;
    int              errorCode;

/* fix compiler warnings */
    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);

    errorCode = TM_DEV_OKAY;

/* Refill the receive and transmit queues. */
    if (flag & TM_MCF5485_IOCTL_REFILL_RX)
    {
        for(ch=0; ch<NUM_ETH_PORTS; ch++)
        {
            if(tlMcfStruct[ch].mcfInterfaceHandle == interfaceHandle)
            {
                tfMcf5485RefillRecvPool(ch);
                break;
            }
        }
    }

    if (flag & TM_MCF5485_SEND_COMPLETE)
    {
        for(ch=0; ch<NUM_ETH_PORTS; ch++)
        {
            if(tlMcfStruct[ch].mcfInterfaceHandle == interfaceHandle)
            {
                tfMcf5485CleanupXmitQueue(interfaceHandle, ch);
                break;
            }
        }
    }

    return errorCode;
}

static void tfMcf5485CleanupXmitQueue(ttUserInterface interfaceHandle, int ch)
{
    register unsigned short TM_FAR       *flagsPtr;
    register void  TM_FAR * TM_FAR       *bdDataPtrPtr;

/* complete any transmits */        
    flagsPtr = &tlMcfStruct[ch].mcfTxIsrBufDescPtr->flags;
    bdDataPtrPtr = &tlMcfStruct[ch].mcfTxIsrBufDescPtr->buffer;

    while ((((*flagsPtr & (TX_BD_R | TX_BD_SW_COMPLETE))
             == 0)
            && (tlMcfStruct[ch].mcfTxIsrBufDescPtr
                    != tlMcfStruct[ch].mcfTxScatterStartBufDescPtr)
            && ( *bdDataPtrPtr != (void TM_FAR *)0 )))
/* 
 * Watch out for a scattered frame we have not yet finished
 * loading up
 */
    {
/* Zero it out so we know that we have freed this one before */
        *bdDataPtrPtr = (void TM_FAR *)0;
        if (*flagsPtr & TX_BD_W)
        {
            tlMcfStruct[ch].mcfTxIsrBufDescPtr =
                tlMcfStruct[ch].mcfTxFirstBufDescPtr;
        }
        else
        {
            tlMcfStruct[ch].mcfTxIsrBufDescPtr++;
        }

/* Call send complete when the last piece has been transmitted */
        if (*flagsPtr & TX_BD_L)
        {
            tfSendCompleteInterface( interfaceHandle,
                                     TM_DEV_SEND_COMPLETE_DRIVER);
        }

/* notify device driver send that this frame descriptor is available */
        *flagsPtr |= TX_BD_SW_COMPLETE;

        flagsPtr = &tlMcfStruct[ch].mcfTxIsrBufDescPtr->flags;
        bdDataPtrPtr = &tlMcfStruct[ch].mcfTxIsrBufDescPtr->buffer;
    }
}


int tfMcf5485GetPhyAddr ( ttUserInterface userInterface,
                         char   TM_FAR * address )
{
    int ch = 0;
    int errorCode = TM_DEV_OKAY;
/* Determine which channel we are on */
    while((ch < NUM_ETH_PORTS)
            && (userInterface != tlMcfStruct[ch].mcfInterfaceHandle))
    {
        ch++;
    }
    
    if(ch < NUM_ETH_PORTS)
    {
        address[0] = (char)src[ch][0];
        address[1] = (char)src[ch][1];
        address[2] = (char)src[ch][2];
        address[3] = (char)src[ch][3];
        address[4] = (char)src[ch][4];
        address[5] = (char)src[ch][5];
    }
    else
    {
        errorCode = TM_DEV_ERROR;
    }
    return errorCode;
}

int tfEthfecBuffDescInit(tt8Bit ch)
{
    int         errorCode;
    int         i;
    tt8BitPtr   descPtr;

    errorCode = TM_DEV_OKAY;
/* 
 * Initialize pointers to transmit and receive buffer descriptor rings.
 * Buffer descriptors must be long word aligned (ie. bit 0,1 = 0) but Motorola
 * recommends 16-byte alignment to improve bus utilization.  For this
 * controller, receive data buffer must be 16 byte aligned.
 */

/* Allocate memory for Transmit and receive descriptors */
    tlMcfStruct[ch].mcfDescPtr = tm_get_raw_buffer( 
        (sizeof(ttBufDesc) * (TM_MCF5485_ETHER_MAX_RECV_BUFD +
                              TM_MCF5485_ETHER_MAX_XMIT_BUFD))
        + TM_ETHER_HW_ALIGN);
    descPtr = tlMcfStruct[ch].mcfDescPtr;
    if (descPtr == TM_8BIT_NULL_PTR)
    {
        errorCode = TM_DEV_ERROR;
    }
    else
    {
        tm_bzero((ttUserVoidPtr) tlMcfStruct[ch].mcfDescPtr, 
                 (sizeof(ttBufDesc) * (TM_MCF5485_ETHER_MAX_RECV_BUFD +
                                       TM_MCF5485_ETHER_MAX_XMIT_BUFD))
                 + TM_ETHER_HW_ALIGN);
        if (((tt32Bit)descPtr) & (tt32Bit)(TM_ETHER_HW_ALIGN - 1) )
        {
/* Align the data area (according to TM_ETHER_HW_ALIGN) */
            descPtr = ((tt8Bit TM_FAR *)
                (    (tt32Bit)descPtr
                     & ~(tt32Bit)(TM_ETHER_HW_ALIGN - 1) ))
                        + TM_ETHER_HW_ALIGN;
        }
        tlMcfStruct[ch].mcfRxFirstBufDescPtr = (ttBufDesc *)descPtr;

/* Set the intial values for all of our buffer descriptor pointers */
        tlMcfStruct[ch].mcfRxBufDescPtr =
            tlMcfStruct[ch].mcfRxFirstBufDescPtr;

        tlMcfStruct[ch].mcfRxIsrBufDescPtr =
            tlMcfStruct[ch].mcfRxFirstBufDescPtr;

        tlMcfStruct[ch].mcfRxRefillBufDescPtr =
            tlMcfStruct[ch].mcfRxFirstBufDescPtr;

        tlMcfStruct[ch].mcfRxUserRefillBufferPtr =
            tlMcfStruct[ch].mcfRxUserBufferList;

        tlMcfStruct[ch].mcfRxUserBufferPtr =
            tlMcfStruct[ch].mcfRxUserBufferList;

        tlMcfStruct[ch].mcfTxFirstBufDescPtr =
            (tlMcfStruct[ch].mcfRxFirstBufDescPtr
             + TM_MCF5485_ETHER_MAX_RECV_BUFD);

        tlMcfStruct[ch].mcfTxBufDescPtr =
            tlMcfStruct[ch].mcfTxFirstBufDescPtr;

        tlMcfStruct[ch].mcfTxIsrBufDescPtr =
            tlMcfStruct[ch].mcfTxFirstBufDescPtr;

        tlMcfStruct[ch].mcfTxScatterStartBufDescPtr = (ttBufDescPtr)0;
    }

    if (errorCode == TM_DEV_OKAY)
    {
/* set up tx/rx buffer descriptors */
        for (i = 0; i < TM_MCF5485_ETHER_MAX_RECV_BUFD; i++)
        {
            tlMcfStruct[ch].mcfRxBufDescPtr[i].length = TM_MCF5485_RX_BUF_LEN;
/* get a pre-allocated packet */
            tlMcfStruct[ch].mcfRxBufDescPtr[i].buffer = 
                (tfGetDriverBuffer(&(tlMcfStruct[ch].mcfRxUserBufferList[i]),
                                   TM_MCF5485_RX_BUF_LEN,
                                   TM_ETHER_HW_ALIGN));
            if (tlMcfStruct[ch].mcfRxUserBufferList[i] == (ttUserBuffer)0)
            {
                break;
            }
/* 
 * Mark all buffers as empty, and allow all buffers to fire a
 * DMA interrupt.
 */
            tlMcfStruct[ch].mcfRxBufDescPtr[i].flags =
                RX_BD_E | RX_BD_INTERRUPT;
        }
        if (i != 0)
        {
/* Mark the wrap bit in the last buffer */
            tlMcfStruct[ch].mcfRxBufDescPtr[i-1].flags |= RX_BD_W;
        }

        for (i = 0; i < TM_MCF5485_ETHER_MAX_XMIT_BUFD; i++)
        {
/*
 * Mark all buffers as empty, and allow all buffers to fire a 
 * DMA interrupt.
 */
            tlMcfStruct[ch].mcfTxBufDescPtr[i].flags =
                TX_BD_SW_COMPLETE;
        }
/* Mark the wrap bit in the last buffer */
        tlMcfStruct[ch].mcfTxBufDescPtr[
            TM_MCF5485_ETHER_MAX_XMIT_BUFD-1].flags |= RX_BD_W;
    }

    return errorCode;
}


/*
 * Programmable Serial Controller functions
 * These functions are used to setup the PSC as a UART and send data debug
 * data out the serial port.
 */

/*
 * Initialize the PSC port
 */
void tfPscUartInit(int port, int sys_clk, int baud)
{
    register tt16Bit divider;

    tm_assert(tfPscUartInit, (port >= 0 && port <= 3));

/* Set GPIO port register to enable PSC(port) signals */
    switch (port)
    {
        case 0:
            MCF_GPIO_PAR_PSC0 = (0
                #ifdef HARDWARE_FLOW_CONTROL
                | MCF_GPIO_PAR_PSC0_PAR_CTS0_CTS
                | MCF_GPIO_PAR_PSC0_PAR_RTS0_RTS
                #endif
                | MCF_GPIO_PAR_PSC0_PAR_TXD0
                | MCF_GPIO_PAR_PSC0_PAR_RXD0);
            break;
        case 1:
            MCF_GPIO_PAR_PSC1 = (0
                #ifdef HARDWARE_FLOW_CONTROL
                | MCF_GPIO_PAR_PSC1_PAR_CTS1_CTS
                | MCF_GPIO_PAR_PSC1_PAR_RTS1_RTS
                #endif
                | MCF_GPIO_PAR_PSC1_PAR_TXD1
                | MCF_GPIO_PAR_PSC1_PAR_RXD1);
            break;
        case 2:
            MCF_GPIO_PAR_PSC2 = (0
                #ifdef HARDWARE_FLOW_CONTROL
                | MCF_GPIO_PAR_PSC2_PAR_CTS2_CTS
                | MCF_GPIO_PAR_PSC2_PAR_RTS2_RTS
                #endif
                | MCF_GPIO_PAR_PSC2_PAR_TXD2
                | MCF_GPIO_PAR_PSC2_PAR_RXD2);
            break;
        case 3:
            MCF_GPIO_PAR_PSC3 = (0
                #ifdef HARDWARE_FLOW_CONTROL
                | MCF_GPIO_PAR_PSC3_PAR_CTS3_CTS
                | MCF_GPIO_PAR_PSC3_PAR_RTS3_RTS
                #endif
                | MCF_GPIO_PAR_PSC3_PAR_TXD3
                | MCF_GPIO_PAR_PSC3_PAR_RXD3);
            break;
        default:
            return;
    }

/* Put PSC in UART mode */
    MCF_PSC_SICR(port) = MCF_PSC_SICR_SIM_UART;

/* Generate the Rx and Tx baud rate from the System Clock */
    MCF_PSC_CSR(port) = (0
        | MCF_PSC_CSR_RCSEL_SYS_CLK 
        | MCF_PSC_CSR_TCSEL_SYS_CLK);

/* Calculate baud settings */
    divider = (tt16Bit)((sys_clk*1000000)/(baud * 32));
    MCF_PSC_CTUR(port) =  (tt8Bit) ((divider >> 8) & 0xFF);
    MCF_PSC_CTLR(port) =  (tt8Bit) (divider & 0xFF);

/* Reset transmitter, receiver, mode register, and error conditions */
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_RX;
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_TX;
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_ERROR;
    MCF_PSC_CR(port) = MCF_PSC_CR_BKCHGINT;
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_MR;

/* 8-bit data, no parity */
    MCF_PSC_MR(port) = (0
        #ifdef UART_HARDWARE_FLOW_CONTROL
        | MCF_PSC_MR_RXRTS
        #endif
        | MCF_PSC_MR_PM_NONE
        | MCF_PSC_MR_BC_8);

/* No echo or loopback, 1 stop bit */
    MCF_PSC_MR(port) = (0
        #ifdef UART_HARDWARE_FLOW_CONTROL
        | MCF_PSC_MR_TXCTS
        #endif 
        | MCF_PSC_MR_CM_NORMAL
        | MCF_PSC_MR_SB_STOP_BITS_1);

/* Mask all UART interrupts */
    MCF_PSC_IMR(port) = 0x0000;

/* Enable RTS to send */
    MCF_PSC_OPSET(port) = MCF_PSC_OPSET_RTS;

/* Setup FIFO Alarms */
    MCF_PSC_RFAR(port) = MCF_PSC_RFAR_ALARM(248);
    MCF_PSC_TFAR(port) = MCF_PSC_TFAR_ALARM(248);

/* Enable receiver and transmitter */
    MCF_PSC_CR(port) =(0
        | MCF_PSC_CR_RX_ENABLED
        | MCF_PSC_CR_TX_ENABLED);
}

/*
 * Place the PSC port into SIR mode
 */
void tfPscSirInit(int port, int sys_clk, int baud)
{
    register tt16Bit divider;

    tm_assert(tfPscSirInit, (port >= 0 && port <= 3));

    /* 
     * Set GPIO port register to enable PSC(port) signals 
     */
    switch (port)
    {
        case 0:
            MCF_GPIO_PAR_PSC0 = (0
                | MCF_GPIO_PAR_PSC0_PAR_TXD0
                | MCF_GPIO_PAR_PSC0_PAR_RXD0);
            break;
        case 1:
            MCF_GPIO_PAR_PSC1 = (0
                | MCF_GPIO_PAR_PSC1_PAR_TXD1
                | MCF_GPIO_PAR_PSC1_PAR_RXD1);
            break;
        case 2:
            MCF_GPIO_PAR_PSC2 = (0
                | MCF_GPIO_PAR_PSC2_PAR_TXD2
                | MCF_GPIO_PAR_PSC2_PAR_RXD2);
            break;
        case 3:
            MCF_GPIO_PAR_PSC3 = (0
                | MCF_GPIO_PAR_PSC3_PAR_TXD3
                | MCF_GPIO_PAR_PSC3_PAR_RXD3);
            break;
        default:
            return;
    }

/* Put PSC in SIR mode */
    MCF_PSC_SICR(port) = MCF_PSC_SICR_SIM_SIR;

/* Generate the Rx and Tx baud rate from the System Clock */
    MCF_PSC_CSR(port) = (0
        | MCF_PSC_CSR_RCSEL_SYS_CLK 
        | MCF_PSC_CSR_TCSEL_SYS_CLK);

/* Calculate baud settings */
    divider = (tt16Bit)((sys_clk*1000000)/(baud * 32));
    MCF_PSC_CTUR(port) =  (tt8Bit) ((divider >> 8) & 0xFF);
    MCF_PSC_CTLR(port) =  (tt8Bit) (divider & 0xFF);

/* Reset transmitter, receiver, mode register, and error conditions */
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_RX;
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_TX;
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_ERROR;
    MCF_PSC_CR(port) = MCF_PSC_CR_BKCHGINT;
    MCF_PSC_CR(port) = MCF_PSC_CR_RESET_MR;

/* Write Mode Register 1 */
    MCF_PSC_MR(port) = (0
        | MCF_PSC_MR_PM_NONE
        | MCF_PSC_MR_BC_8);

/* Write Mode Register 2 - Normal Mode */
    MCF_PSC_MR(port) = (0
        | MCF_PSC_MR_CM_NORMAL
        | MCF_PSC_MR_SB_STOP_BITS_1);

/* Put the SIR port in half-duplex mode */
    MCF_PSC_IRCR1(port) = 0;

/* Mask all UART interrupts */
    MCF_PSC_IMR(port) = 0x0000;

/* Enable RTS to send */
    MCF_PSC_OPSET(port) = MCF_PSC_OPSET_RTS;

/* Setup FIFO Alarms */
    MCF_PSC_RFAR(port) = MCF_PSC_RFAR_ALARM(248);
    MCF_PSC_TFAR(port) = MCF_PSC_TFAR_ALARM(248);

/* Enable receiver and transmitter */
    MCF_PSC_CR(port) =(0
        | MCF_PSC_CR_RX_ENABLED
        | MCF_PSC_CR_TX_ENABLED);
}

/* 
 * Get a character from the selected port
 */
char tfPscInChar (int port)
{
    tm_assert(tfPscInChar, (port >= 0 && port <= 3));

/* Wait until character has been received */
    while (!(MCF_PSC_SR(port) & MCF_PSC_SR_RXRDY))
    {
        ;
    }
    return (char)*((tt8Bit *) &MCF_PSC_RB(port));
}

/* 
 * Send a character out the selected port
 */
void tfPscOutChar (int port, char ch)
{
    tm_assert(tfPscOutChar, (port >= 0 && port <= 3));

/* Wait until space is available in the FIFO */
    while (!(MCF_PSC_SR(port) & MCF_PSC_SR_TXRDY))
    {
        ;
    }
    *((tt8Bit *) &MCF_PSC_TB(port)) = (tt8Bit)ch;
}

/*
 * Notify that there is a character ready in the Rx FIFO
 */
int tfPscCharPresent (int port)
{
    tm_assert(tfPscCharPresent, (port >= 0 && port <= 3));

    return (MCF_PSC_SR(port) & MCF_PSC_SR_RXRDY);
}

/*
 * DMA Controller funcitons
 */

/********************************************************************/
/*
 * Enable all DMA interrupts
 *
 * Parameters:
 *  pri     Interrupt Priority
 *  lvl     Interrupt Level
 */
void tfEthdmaIRQEnable(tt8Bit lvl, tt8Bit pri)
{
    tm_assert(tfEthdmaIRQEnable ,(lvl > 0 && lvl < 8));
    tm_assert(tfEthdmaIRQEnable, (pri < 8));

/* Setup the DMA ICR (#48) */
    MCF_INTC_ICR48 = (tt8Bit) (0
        | MCF_INTC_ICRn_IP(pri)
        | MCF_INTC_ICRn_IL(lvl));

/* Unmask all task interrupts */
    MCF_DMA_DIMR = 0;

/* Clear the interrupt pending register */
    MCF_DMA_DIPR = 0;

/* Unmask the DMA interrupt in the interrupt controller */
    MCF_INTC_IMRH &= ~MCF_INTC_IMRH_INT_MASK48;
}
/********************************************************************/
/*
 * Disable all DMA interrupts
 */
void tfEthdmaIRQDisable(void)
{
/* Mask all task interrupts */
    MCF_DMA_DIMR = (tt32Bit)~0;

/* Clear any pending task interrupts */
    MCF_DMA_DIPR = (tt32Bit)~0;

/* Mask the DMA interrupt in the interrupt controller */
    MCF_INTC_IMRH |= MCF_INTC_IMRH_INT_MASK48;
}
/********************************************************************/
/*
 * Attempt to enable the provided Initiator in the Initiator
 * Mux Control Register
 *
 * Parameters:
 *  initiator   Initiator identifier
 *
 * Return Value:
 *  TM_DEV_ERROR    if unable to make the assignment
 *  TM_DEV_OKAY     successful
 */
int tfEthdmaSetInitiator(int initiator)
{
    int errorCode = TM_DEV_OKAY;

    switch (initiator)
    {
/* These initiators are always active */
        case DMA_ALWAYS:
        case DMA_DSPI_RX:
        case DMA_DSPI_TX: 
        case DMA_DREQ0: 
        case DMA_PSC0_RX: 
        case DMA_PSC0_TX: 
        case DMA_USBEP0: 
        case DMA_USBEP1: 
        case DMA_USBEP2: 
        case DMA_USBEP3: 
        case DMA_PCI_TX: 
        case DMA_PCI_RX: 
        case DMA_PSC1_RX: 
        case DMA_PSC1_TX: 
        case DMA_I2C_RX: 
        case DMA_I2C_TX:
            break;
        case DMA_FEC0_RX:
            MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC16(3)) 
                            | MCF_DMA_IMCR_SRC16_FEC0RX;
            used_reqs[16] = DMA_FEC0_RX;
            break;
        case DMA_FEC0_TX:
            MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC17(3)) 
                            | MCF_DMA_IMCR_SRC17_FEC0TX;
            used_reqs[17] = DMA_FEC0_TX;
            break;
        case DMA_FEC1_RX:
            MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC20(3)) 
                            | MCF_DMA_IMCR_SRC20_FEC1RX;
            used_reqs[20] = DMA_FEC1_RX;
            break;
        case DMA_FEC1_TX:
            if (used_reqs[21] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC21(3)) 
                                | MCF_DMA_IMCR_SRC21_FEC1TX;
                used_reqs[21] = DMA_FEC1_TX;
            }
            else if (used_reqs[25] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC25(3)) 
                                | MCF_DMA_IMCR_SRC25_FEC1TX;
                used_reqs[25] = DMA_FEC1_TX;
            }
            else if (used_reqs[31] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC31(3)) 
                                | MCF_DMA_IMCR_SRC31_FEC1TX;
                used_reqs[31] = DMA_FEC1_TX;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_DREQ1:
            if (used_reqs[29] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC29(3)) 
                                | MCF_DMA_IMCR_SRC29_DREQ1;
                used_reqs[29] = DMA_DREQ1;
            }
            else if (used_reqs[21] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC21(3)) 
                                | MCF_DMA_IMCR_SRC21_DREQ1;
                used_reqs[21] = DMA_DREQ1;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM0:
            if (used_reqs[24] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC24(3)) 
                                | MCF_DMA_IMCR_SRC24_CTM0;
                used_reqs[24] = DMA_CTM0;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM1:
            if (used_reqs[25] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC25(3)) 
                                | MCF_DMA_IMCR_SRC25_CTM1;
                used_reqs[25] = DMA_CTM1;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM2:
            if (used_reqs[26] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC26(3)) 
                                | MCF_DMA_IMCR_SRC26_CTM2;
                used_reqs[26] = DMA_CTM2;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM3:
            if (used_reqs[27] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC27(3)) 
                                | MCF_DMA_IMCR_SRC27_CTM3;
                used_reqs[27] = DMA_CTM3;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM4:
            if (used_reqs[28] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC28(3)) 
                                | MCF_DMA_IMCR_SRC28_CTM4;
                used_reqs[28] = DMA_CTM4;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM5:
            if (used_reqs[29] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC29(3)) 
                                | MCF_DMA_IMCR_SRC29_CTM5;
                used_reqs[29] = DMA_CTM5;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM6:
            if (used_reqs[30] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC30(3)) 
                                | MCF_DMA_IMCR_SRC30_CTM6;
                used_reqs[30] = DMA_CTM6;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_CTM7:
            if (used_reqs[31] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC31(3)) 
                                | MCF_DMA_IMCR_SRC31_CTM7;
                used_reqs[31] = DMA_CTM7;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_USBEP4:
            if (used_reqs[26] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC26(3)) 
                                | MCF_DMA_IMCR_SRC26_USBEP4;
                used_reqs[26] = DMA_USBEP4;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_USBEP5:
            if (used_reqs[27] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC27(3)) 
                                | MCF_DMA_IMCR_SRC27_USBEP5;
                used_reqs[27] = DMA_USBEP5;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_USBEP6:
            if (used_reqs[28] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC28(3)) 
                                | MCF_DMA_IMCR_SRC28_USBEP6;
                used_reqs[28] = DMA_USBEP6;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_PSC2_RX:
            if (used_reqs[28] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC28(3)) 
                                | MCF_DMA_IMCR_SRC28_PSC2RX;
                used_reqs[28] = DMA_PSC2_RX;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_PSC2_TX:
            if (used_reqs[29] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC29(3)) 
                                | MCF_DMA_IMCR_SRC29_PSC2TX;
                used_reqs[29] = DMA_PSC2_TX;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_PSC3_RX:
            if (used_reqs[30] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC30(3)) 
                                | MCF_DMA_IMCR_SRC30_PSC3RX;
                used_reqs[30] = DMA_PSC3_RX;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        case DMA_PSC3_TX:
            if (used_reqs[31] == 0)
            {
                MCF_DMA_IMCR = (MCF_DMA_IMCR & ~MCF_DMA_IMCR_SRC31(3)) 
                                | MCF_DMA_IMCR_SRC31_PSC3TX;
                used_reqs[31] = DMA_PSC3_TX;
            }
            else
            {
/* No empty slots */
                errorCode = TM_DEV_ERROR;
            }
            break;
        default:
            errorCode = TM_DEV_ERROR;
    }
    return errorCode;
}
/********************************************************************/
/*
 * Return the initiator number for the given requestor
 *
 * Parameters:
 *  requestor   Initiator/Requestor identifier
 *
 * Return Value:
 *  The initiator number (0-31) if initiator has been assigned
 *  0 (always initiator) otherwise
 */
tt32Bit tfEthdmaGetInitiator(int requestor)
{
    tt32Bit i;

    for (i=0; i<sizeof(used_reqs); i++)
    {
        if (used_reqs[i] == requestor)
        {
            break;
        }
    }
/* Reset i if we didn't find an initiator */
    if(i == sizeof(used_reqs))
    {
        i = 0;
    }
    return i;
}
/********************************************************************/
/*
 * Remove the given initiator from the active list
 *
 * Parameters:
 *  requestor   Initiator/Requestor identifier
 */
void tfEthdmaFreeInitiator(int requestor)
{
    tt32Bit i;

    for (i=16; i<sizeof(used_reqs); i++)
    {
        if (used_reqs[i] == requestor)
        {
            used_reqs[i] = 0;
            break;
        }
    }
}
/********************************************************************/
/*
 * Attempt to find an available channel and mark it as used
 *
 * Parameters:
 *  requestor   Initiator/Requestor identifier
 *
 * Return Value:
 *  First available channel or -1 if they are all occupied
 */
int tfEthdmaSetChannel(int requestor, void (*handler)(void))
{
    int i;

/* Check to see if this requestor is already assigned to a channel */
    if ((i = tfEthdmaGetChannel(requestor)) == -1)
    {
/* Not assigned.  Find an open requestor */
        for (i=0; i<NCHANNELS; i++)
        {
            if (dma_channel[i].req == -1)
            {
                dma_channel[i].req = requestor;
                dma_channel[i].handler = handler;
                break;
            }
        }
    }

    if(i == NCHANNELS)
    {
/* All channels taken */
        i = -1;
    }

    return i;
}
/********************************************************************/
/*
 * Return the channel being initiated by the given requestor
 *
 * Parameters:
 *  requestor   Initiator/Requestor identifier
 *
 * Return Value:
 *  Channel that the requestor is controlling or -1 if hasn't been
 *  activated
 */
int tfEthdmaGetChannel(int requestor)
{
    int i;

    for (i=0; i<NCHANNELS; ++i)
    {
        if (dma_channel[i].req == requestor)
        {
            break;
        }
    }
    if(i == NCHANNELS)
    {
        i = -1;
    }
    return i;
}
/********************************************************************/
/*
 * Remove the channel being initiated by the given requestor from 
 * the active list
 *
 * Parameters:
 *  requestor   Initiator/Requestor identifier
 */
void tfEthdmaFreeChannel(int requestor)
{
    tt32Bit i;

    for (i=0; i<NCHANNELS; ++i)
    {
        if (dma_channel[i].req == requestor)
        {
            dma_channel[i].req = -1;
            dma_channel[i].handler = NULL;
            break;
        }
    }
}
/********************************************************************/
/* 
 * This is the catch-all interrupt handler for the mult-channel DMA 
 */
__interrupt__  void tfEthdmaIRQHandler (void) 
{
    tt32Bit i, interrupts;

/* 
 * Determine which interrupt(s) triggered by AND'ing the 
 * pending interrupts with those that aren't masked.
 */
    interrupts = MCF_DMA_DIPR & ~MCF_DMA_DIMR;

/* Make sure we are here for a reason */
    tm_assert(tfEthdmaIRQHandler, (interrupts != 0));

/* Clear the interrupt in the pending register */
    MCF_DMA_DIPR = interrupts;

    for (i=0; i<16; ++i, interrupts>>=1)
    {
        if (interrupts & 0x1)
        {
/* If there is a handler, call it */
            if (dma_channel[i].handler != NULL)
                dma_channel[i].handler();
        }
    }
}


/********************************************************************/
/*
 * Function: tfMcdInitDma
 * Purpose:  Initializes the DMA API by setting up a pointer to the DMA
 *           registers, relocating and creating the appropriate task
 *           structures, and setting up some global settings
 * Arguments:
 *  dmaBarAddr    - pointer to the multichannel DMA registers
 *  taskTableDest - location to move DMA task code and structs to
 *  flags         - operational parameters
 * Return Value:
 *  MCD_TABLE_UNALIGNED if taskTableDest is not 512-byte aligned
 *  MCD_OK otherwise
 */
extern tt32Bit MCD_funcDescTab0[];

int tfMcdInitDma (dmaRegs *dmaBarAddr, void *taskTableDest, tt32Bit flags)
{
    int i;
    TaskTableEntry *entryPtr;

/* setup the local pointer to register set */
    MCD_dmaBar = dmaBarAddr;

/* do we need to move/create a task table */
    if ((flags & MCD_RELOC_TASKS) != 0)
    {
        int fixedSize;
        tt32Bit *fixedPtr;
/*int *tablePtr = taskTableDest;TBD*/
        int varTabsOffset, funcDescTabsOffset, contextSavesOffset;
        tt32Bit taskDescTabsOffset;
        int taskTableSize, varTabsSize, funcDescTabsSize, contextSavesSize;
        tt32Bit taskDescTabSize;

        int i;

/* check if physical address is aligned on 512 byte boundary */
        if (((tt32Bit)taskTableDest & 0x000001ff) != 0)
            return(MCD_TABLE_UNALIGNED);

/* set up local pointer to task Table */
        MCD_taskTable = taskTableDest;

/*
 * Create a task table:
 * - compute aligned base offsets for variable tables and
 *   function descriptor tables, then
 * - loop through the task table and setup the pointers
 * - copy over model task table with the the actual task descriptor
 *   tables
 */

        taskTableSize = NCHANNELS * sizeof(TaskTableEntry);
/* align variable tables to size */
        varTabsOffset = taskTableSize + (int)taskTableDest;
        if ((varTabsOffset & (VAR_TAB_SIZE - 1)) != 0)
            varTabsOffset = (varTabsOffset + VAR_TAB_SIZE) & (~VAR_TAB_SIZE);
/* align function descriptor tables */
        varTabsSize = NCHANNELS * VAR_TAB_SIZE;
        funcDescTabsOffset = varTabsOffset + varTabsSize;

        if ((funcDescTabsOffset & (FUNCDESC_TAB_SIZE - 1)) != 0)
            funcDescTabsOffset = (funcDescTabsOffset + FUNCDESC_TAB_SIZE) &
            (~FUNCDESC_TAB_SIZE);

        funcDescTabsSize = FUNCDESC_TAB_NUM * FUNCDESC_TAB_SIZE;
        contextSavesOffset = funcDescTabsOffset + funcDescTabsSize;
        contextSavesSize = (NCHANNELS * CONTEXT_SAVE_SIZE);
        fixedSize = taskTableSize + varTabsSize + funcDescTabsSize +
                    contextSavesSize;

/* zero the thing out */
        fixedPtr = (tt32Bit *)taskTableDest;
        for (i = 0;i<(fixedSize/4);i++)
            fixedPtr[i] = 0;

        entryPtr = (TaskTableEntry*)MCD_taskTable;
/* set up fixed pointers */
        for (i = 0; i < NCHANNELS; i++)
        {
/* update ptr to local value */
            entryPtr[i].varTab = (tt32Bit)varTabsOffset; 
            entryPtr[i].FDTandFlags = (tt32Bit)funcDescTabsOffset |
                                        MCD_TT_FLAGS_DEF;
            entryPtr[i].contextSaveSpace = (tt32Bit)contextSavesOffset;
            varTabsOffset += VAR_TAB_SIZE;
/* if not there is only one, just point to the same one */
#ifdef MCD_INCLUDE_EU 
            funcDescTabsOffset += FUNCDESC_TAB_SIZE;
#endif
            contextSavesOffset += CONTEXT_SAVE_SIZE;
        }
/* copy over the function descriptor table */
        for ( i = 0; i < FUNCDESC_TAB_NUM; i++)
        {
            tfMcdmemcpy((void*)(entryPtr[i].FDTandFlags & ~MCD_TT_FLAGS_MASK),
                       (void*)MCD_funcDescTab0, FUNCDESC_TAB_SIZE);
        }

/* copy model task table to where the context saves stuff leaves off*/
        MCD_modelTaskTable = (TaskTableEntry*)contextSavesOffset;

        tfMcdmemcpy ((void*)MCD_modelTaskTable, (void*)MCD_modelTaskTableSrc,
                    NUMOFVARIANTS * sizeof(TaskTableEntry));

/* point to local version of model task table */
        entryPtr = MCD_modelTaskTable; 
        taskDescTabsOffset = (tt32Bit)MCD_modelTaskTable +
                            (NUMOFVARIANTS * sizeof(TaskTableEntry));

/* copy actual task code and update TDT ptrs in local model task table */
        for (i = 0; i < NUMOFVARIANTS; i++)
        {
            taskDescTabSize = entryPtr[i].TDTend - entryPtr[i].TDTstart + 4;
            tfMcdmemcpy ((void*)taskDescTabsOffset,
                (void*)entryPtr[i].TDTstart, taskDescTabSize);

            entryPtr[i].TDTstart = (tt32Bit)taskDescTabsOffset;
            taskDescTabsOffset += taskDescTabSize;
            entryPtr[i].TDTend = (tt32Bit)taskDescTabsOffset - 4;
        }
/* 
 * Tack single DMA BDs onto end of code so API controls where they are since
 * DMA might write to them
 */
#ifdef MCD_INCLUDE_EU 
        MCD_relocBuffDesc = (MCD_bufDesc*)(entryPtr[NUMOFVARIANTS - 1].TDTend
                                           + 4);
#else
/*
 * DMA does not touch them so they can be wherever and we don't need to waste
 * SRAM on them
 */
        MCD_relocBuffDesc = MCD_singleBufDescs;
#endif
    }
    else
    {
/*
 * point the would-be relocated task tables and the buffer descriptors to the
 * ones the linker generated
 */
        if (((tt32Bit)MCD_realTaskTableSrc & 0x000001ff) != 0)
            return(MCD_TABLE_UNALIGNED);

/*
 * need to add code to make sure that every thing else is aligned properly TBD * this is problematic if we init more than once or after running tasks,
 * need to add variable to see if we have aleady init'd
 */
        entryPtr = MCD_realTaskTableSrc;
        for (i = 0; i < NCHANNELS; i++)
        {
            if (((entryPtr[i].varTab & (VAR_TAB_SIZE - 1)) != 0) ||
                ((entryPtr[i].FDTandFlags & (FUNCDESC_TAB_SIZE - 1)) != 0))
                return(MCD_TABLE_UNALIGNED);
        }

        MCD_taskTable = MCD_realTaskTableSrc;
        MCD_modelTaskTable = MCD_modelTaskTableSrc;
        MCD_relocBuffDesc = MCD_singleBufDescs;
    }


/* Make all channels as totally inactive, and remember them as such: */

    MCD_dmaBar->taskbar = (tt32Bit) MCD_taskTable;
    for (i = 0;  i < NCHANNELS;  i++)
    {
        MCD_dmaBar->taskControl[i] = 0x0;
        MCD_chStatus[i] = MCD_NO_DMA;
    }

/* 
 * Set up pausing mechanism to inactive state: 
 * no particular values yet for either comparator registers
 */
    MCD_dmaBar->debugComp1 = 0;  
    MCD_dmaBar->debugComp2 = 0;
    MCD_dmaBar->debugControl = DBG_CTL_DISABLE;
    MCD_dmaBar->debugStatus = DBG_KILL_ALL_STAT;

/* 
 * enable or disable commbus prefetch, really need an ifdef or something to
 * keep from trying to set this in the 8220
 */
    if ((flags & MCD_COMM_PREFETCH_EN) != 0)
        MCD_dmaBar->ptdControl &= (tt16Bit)(~PTD_CTL_COMM_PREFETCH);
    else
        MCD_dmaBar->ptdControl |= PTD_CTL_COMM_PREFETCH;

    return(MCD_OK);
}

/********************************************************************/
/* Function:   tfMcdDmaStatus
 * Purpose:    Returns the status of the DMA on the requested channel
 * Arguments:  channel - channel number
 * Returns:    Predefined status indicators
 */
int tfMcdDmaStatus (int channel)
{
    tt16Bit tcrValue;

    if((channel < 0) || (channel >= NCHANNELS))
        return(MCD_CHANNEL_INVALID);

    tcrValue = MCD_dmaBar->taskControl[channel];
    if ((tcrValue & TASK_CTL_EN) == 0)
    {
/* nothing running if last reported with task enabled */
        if (   MCD_chStatus[channel] == MCD_RUNNING
            || MCD_chStatus[channel] == MCD_IDLE)
            MCD_chStatus[channel] = MCD_DONE;
    }
/* something is running */
    else
    {
/* There are three possibilities: paused, running or idle. */
        if (   MCD_chStatus[channel] == MCD_RUNNING
            || MCD_chStatus[channel] == MCD_IDLE)
        {
            MCD_dmaBar->ptdDebug = PTD_DBG_TSK_VLD_INIT;
/* This register is selected to know which initiator is actually asserted. */
            if ((MCD_dmaBar->ptdDebug >> channel ) & 0x1 )
                MCD_chStatus[channel] = MCD_RUNNING;
            else
                MCD_chStatus[channel] = MCD_IDLE;
/* do not change the status if it is already paused. */
        }
    }
    return MCD_chStatus[channel];
}

/********************************************************************/
/* Function:    tfMcdStartDma
 * Ppurpose:    Starts a particular kind of DMA
 * Arguments:   see below
 * Returns:     MCD_CHANNEL_INVALID if channel is invalid, else MCD_OK
 */

int tfMcdStartDma (
    int         channel,   /* the channel on which to run the DMA */
    char        *srcAddr,  /* the address to move data from, or physical
                              buffer-descriptor address */
    ttS16Bit    srcIncr,   /* the amount to increment the source address per
                              transfer */
    char        *destAddr, /* the address to move data to */
    ttS16Bit    destIncr,  /* the amount to increment the destination address
                              per transfer */
    tt32Bit     dmaSize,   /* the number of bytes to transfer independent of
                              the transfer size */
    tt32Bit     xferSize,  /* the number bytes in of each data movement
                              (1, 2, or 4) */
    tt32Bit     initiator, /* what device initiates the DMA */
    int         priority,  /* priority of the DMA */
    tt32Bit     flags,     /* flags describing the DMA */
    tt32Bit     funcDesc   /* a description of byte swapping, bit swapping,
                              and CRC actions */
#ifdef MCD_NEED_ADDR_TRANS
    char        *srcAddrVirt /* virtual buffer descriptor address TBD*/
#endif
)
{
    int srcRsdIncr, destRsdIncr;
    int *cSave;
    short xferSizeIncr;
    int tcrCount = 0;
#ifdef MCD_INCLUDE_EU
    tt32Bit *realFuncArray;
#endif

    if((channel < 0) || (channel >= NCHANNELS))
        return(MCD_CHANNEL_INVALID);
        
/*
 * tbd - need to determine the proper response to a bad funcDesc when not 
 * including EU functions, for now, assign a benign funcDesc, but maybe
 * should return an error
 */
#ifndef MCD_INCLUDE_EU
    funcDesc = MCD_FUNC_NOEU1;
#endif
        
#ifdef MCD_DEBUG
printf("startDma:Setting up params\n");
#endif
/*
 * Set us up for task-wise priority.  We don't technically need to do this on
 * every start, but since the register involved is in the same longword as
 * other registers that users are in control of, setting it more than once is
 * probably preferable.  That since the documentation doesn't seem to be
 * completely consistent about the nature of the PTD control register.
 */
    MCD_dmaBar->ptdControl |= (tt16Bit) 0x8000;
#if 1
/* Not sure what we need to keep here rtm TBD */
/* Calculate additional parameters to the regular DMA calls. */
    srcRsdIncr = srcIncr < 0 ? -1 : (srcIncr > 0 ? 1 : 0);
    destRsdIncr = destIncr < 0 ? -1 : (destIncr > 0 ? 1 : 0);

    xferSizeIncr = (xferSize & 0xffff) | 0x20000000;

/* Remember for each channel which variant is running. */
    MCD_remVariants.remSrcRsdIncr[channel] = srcRsdIncr;
    MCD_remVariants.remDestRsdIncr[channel] = destRsdIncr;
    MCD_remVariants.remDestIncr[channel] = destIncr;
    MCD_remVariants.remSrcIncr[channel] = srcIncr;
    MCD_remVariants.remXferSize[channel] = xferSize;
#endif

    cSave = (int*)(MCD_taskTable[channel].contextSaveSpace) + CSAVE_OFFSET
                + CURRBD;

#ifdef MCD_INCLUDE_EU
/* may move this to EU specific calls */
    realFuncArray = (tt32Bit *) (MCD_taskTable[channel].FDTandFlags
                        & 0xffffff00);
/*
 * Modify the LURC's normal and byte-residue-loop functions according to
 * parameter.
 */
    realFuncArray[(LURC*16)] = xferSize == 4 ?
                                 funcDesc : xferSize == 2 ?
                                     funcDesc & 0xfffff00f
                                        : funcDesc & 0xffff000f;
    realFuncArray[(LURC*16+1)] = (funcDesc & MCD_BYTE_SWAP_KILLER)
                                    | MCD_NO_BYTE_SWAP_ATALL;
#endif
/* 
 * Write the initiator field in the TCR, and also set the initiator-hold bit.
 * Note that,due to a hardware quirk, this could collide with an MDE access to
 * the initiator-register file, so we have to verify that the write reads
 * back correctly.
 */

    MCD_dmaBar->taskControl[channel] =
        (initiator << 8) | TASK_CTL_HIPRITSKEN | TASK_CTL_HLDINITNUM;

    while(((MCD_dmaBar->taskControl[channel] & 0x1fff) !=
          ((initiator << 8) | TASK_CTL_HIPRITSKEN | TASK_CTL_HLDINITNUM)) &&
            (tcrCount < 1000))
    {
        tcrCount++;
/*MCD_dmaBar->ptd_tcr[channel] = (initiator << 8) | 0x0020;*/
        MCD_dmaBar->taskControl[channel] =
            (initiator << 8) | TASK_CTL_HIPRITSKEN | TASK_CTL_HLDINITNUM;
    }

    MCD_dmaBar->priority[channel] = (tt8Bit)(priority & PRIORITY_PRI_MASK);
/* should be albe to handle this stuff with only one write to ts reg - tbd */
    if (channel < 8 && channel >= 0)
    {
        MCD_dmaBar->taskSize0 &= (tt32Bit)(~(0xf << (7-channel)*4));
        MCD_dmaBar->taskSize0 |= ((xferSize & 3) << (((7 - channel)*4) + 2));
        MCD_dmaBar->taskSize0 |= (xferSize & 3) << ((7 - channel)*4);
    }
    else
    {
        MCD_dmaBar->taskSize1 &= (tt32Bit)(~(0xf << (15-channel)*4));
        MCD_dmaBar->taskSize1 |= (xferSize & 3) << (((15 - channel)*4) + 2);
        MCD_dmaBar->taskSize1 |= (xferSize & 3) << ((15 - channel)*4);
    }

/* setup task table flags/options which mostly control the line buffers */
    MCD_taskTable[channel].FDTandFlags &= (tt32Bit)(~MCD_TT_FLAGS_MASK);
    MCD_taskTable[channel].FDTandFlags |= (MCD_TT_FLAGS_MASK & flags);

    if (flags & MCD_FECTX_DMA)
    {
/* TDTStart and TDTEnd */
        MCD_taskTable[channel].TDTstart =
            MCD_modelTaskTable[TASK_FECTX].TDTstart;

        MCD_taskTable[channel].TDTend = MCD_modelTaskTable[TASK_FECTX].TDTend;
        tfMcdStartDmaENetXmit(srcAddr, srcAddr, destAddr,
                              MCD_taskTable, channel);
    }
    else if (flags & MCD_FECRX_DMA)
    {
/* TDTStart and TDTEnd */
        MCD_taskTable[channel].TDTstart =
            MCD_modelTaskTable[TASK_FECRX].TDTstart;

        MCD_taskTable[channel].TDTend = MCD_modelTaskTable[TASK_FECRX].TDTend;
        tfMcdStartDmaENetRcv(srcAddr, srcAddr, destAddr,
                             MCD_taskTable, channel);
    }
    else if(flags & MCD_SINGLE_DMA)
    {
/*
 * this buffer descriptor is used for storing off initial parameters for later
 * progress query calculation and for the DMA to write the resulting checksum
 * The DMA does not use this to determine how to operate, that info is passed
 * with the init routine
 */
        MCD_relocBuffDesc[channel].srcAddr = srcAddr;
        MCD_relocBuffDesc[channel].destAddr = destAddr;
/* definitely not its final value */
        MCD_relocBuffDesc[channel].lastDestAddr = destAddr;
        MCD_relocBuffDesc[channel].dmaSize = dmaSize;
        MCD_relocBuffDesc[channel].flags = 0;       /* not used */
        MCD_relocBuffDesc[channel].csumResult = 0;  /* not used */
        MCD_relocBuffDesc[channel].next = 0;        /* not used */

/* Initialize the progress-querying stuff to show no progress:*/
        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[
                            SRCPTR + CSAVE_OFFSET] = (int)srcAddr;

        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[
                            DESTPTR + CSAVE_OFFSET] = (int)destAddr;

        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[
                            DCOUNT + CSAVE_OFFSET] = 0;
        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[
                            CURRBD + CSAVE_OFFSET] =
                                (tt32Bit) &(MCD_relocBuffDesc[channel]);
/*
 * tbd - need to keep the user from trying to call the EU routine
 * when MCD_INCLUDE_EU is not defined
 */
        if( funcDesc == MCD_FUNC_NOEU1 || funcDesc == MCD_FUNC_NOEU2)
        {
/* TDTStart and TDTEnd */
            MCD_taskTable[channel].TDTstart =
                MCD_modelTaskTable[TASK_SINGLENOEU].TDTstart;

            MCD_taskTable[channel].TDTend =
                MCD_modelTaskTable[TASK_SINGLENOEU].TDTend;

            tfMcdStartDmaSingleNoEu(srcAddr, srcIncr, destAddr, destIncr,
                                    dmaSize, xferSizeIncr, flags,
                                    (int *)&(MCD_relocBuffDesc[channel]),
                                    cSave, MCD_taskTable, channel);
        }
        else
        {
/* TDTStart and TDTEnd */
            MCD_taskTable[channel].TDTstart =
                MCD_modelTaskTable[TASK_SINGLEEU].TDTstart;

            MCD_taskTable[channel].TDTend =
                MCD_modelTaskTable[TASK_SINGLEEU].TDTend;

            tfMcdStartDmaSingleEu(srcAddr, srcIncr, destAddr, destIncr,
                                  dmaSize, xferSizeIncr, flags,
                                  (int *)&(MCD_relocBuffDesc[channel]), cSave,
                                  MCD_taskTable, channel);
        }
    }
    else
    {
/* chained DMAS */
/* Initialize the progress-querying stuff to show no progress:*/
#if 1 /* (!defined(MCD_NEED_ADDR_TRANS)) */
        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[SRCPTR
            + CSAVE_OFFSET] =
                (int)((MCD_bufDesc*) srcAddr)->srcAddr;

        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[DESTPTR
            + CSAVE_OFFSET]  =
                (int)((MCD_bufDesc*) srcAddr)->destAddr;
#else
/*
 * if using address translation, need the virtual addr of the first buffer
 * descriptor
 */
        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[SRCPTR
            + CSAVE_OFFSET] =
                (int)((MCD_bufDesc*) srcAddrVirt)->srcAddr;

        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[DESTPTR
            + CSAVE_OFFSET] =
                (int)((MCD_bufDesc*) srcAddrVirt)->destAddr;
#endif
        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[DCOUNT
            + CSAVE_OFFSET] = 0;

        ((volatile int *)MCD_taskTable[channel].contextSaveSpace)[CURRBD
            + CSAVE_OFFSET] = (tt32Bit) srcAddr;

        if( funcDesc == MCD_FUNC_NOEU1 || funcDesc == MCD_FUNC_NOEU2)
        {
            /*TDTStart and TDTEnd*/
            MCD_taskTable[channel].TDTstart =
                MCD_modelTaskTable[TASK_CHAINNOEU].TDTstart;

            MCD_taskTable[channel].TDTend =
                MCD_modelTaskTable[TASK_CHAINNOEU].TDTend;

            tfMcdStartDmaChainNoEu((int *)srcAddr, srcIncr, destIncr,
                                   xferSize, xferSizeIncr, cSave,
                                   MCD_taskTable,channel);
        }
        else
        {
/*TDTStart and TDTEnd*/
            MCD_taskTable[channel].TDTstart =
                MCD_modelTaskTable[TASK_CHAINEU].TDTstart;

            MCD_taskTable[channel].TDTend =
                MCD_modelTaskTable[TASK_CHAINEU].TDTend;

            tfMcdStartDmaChainEu((int *)srcAddr, srcIncr, destIncr, xferSize,
                                 xferSizeIncr, cSave, MCD_taskTable, channel);
        }
    }
    MCD_chStatus[channel] = MCD_IDLE;
    return(MCD_OK);
}

/********************************************************************/
/* Function:    tfMcdXfreProgrQuery
 * Purpose:     Returns progress of DMA on requested channel
 * Arguments:   channel - channel to retrieve progress for
 *              progRep - pointer to user supplied MCD_XferProg struct
 * Returns:     MCD_CHANNEL_INVALID if channel is invalid, else MCD_OK
 *
 * Notes:
 *  tfMcdXfreProgrQuery() upon completing or after aborting a DMA, or
 *  while the DMA is in progress, this function returns the first
 *  DMA-destination address not (or not yet) used in the DMA. When 
 *  encountering a non-ready buffer descriptor, the information for
 *  the last completed descriptor is returned.
 *
 *  MCD_XferProgQuery() has to avoid the possibility of getting
 *  partially-updated information in the event that we should happen
 *  to query DMA progress just as the DMA is updating it. It does that
 *  by taking advantage of the fact context is not saved frequently for
 *  the most part. We therefore read it at least twice until we get the
 *  same information twice in a row.
 *
 *  Because a small, but not insignificant, amount of time is required
 *  to write out the progress-query information, especially upon
 *  completion of the DMA, it would be wise to guarantee some time lag
 *  between successive readings of the progress-query information.
 */

/*
 * How many iterations of the loop below to execute to stabilize values
 */
#define STABTIME 0

int tfMcdXfreProgrQuery (int channel, MCD_XferProg *progRep)
{
    MCD_XferProg prevRep;
    int again;  /* true if we are to try again to get consistent results */
    int i;  /* used as a time-waste counter */
    int destDiffBytes; /* Total number of bytes that we think actually got
                          xfered. */
    int numIterations; /* number of iterations */
    int bytesNotXfered; /* bytes that did not get xfered. */
    char *LWAlignedInitDestAddr, *LWAlignedCurrDestAddr;
    int subModVal, addModVal; /* Mode values to added and subtracted from the
                                final destAddr */

    if((channel < 0) || (channel >= NCHANNELS))
    {
        return(MCD_CHANNEL_INVALID);
    }

/* Read a trial value for the progress-reporting values*/
    prevRep.lastSrcAddr = (char *) ((volatile int*) MCD_taskTable[
        channel].contextSaveSpace)[SRCPTR + CSAVE_OFFSET];

    prevRep.lastDestAddr = (char *) ((volatile int*) MCD_taskTable[
        channel].contextSaveSpace)[DESTPTR + CSAVE_OFFSET];

    prevRep.dmaSize = ((volatile int*) MCD_taskTable[
        channel].contextSaveSpace)[DCOUNT + CSAVE_OFFSET];

    prevRep.currBufDesc = (MCD_bufDesc*) ((volatile int*) MCD_taskTable[
        channel].contextSaveSpace)[CURRBD + CSAVE_OFFSET];

/* Repeatedly reread those values until they match previous values: */
    do
    {
/* Waste a little bit of time to ensure stability: */
        for (i = 0;  i < STABTIME;  i++)
        {
/* make sure this loop does something so that it doesn't get optimized out */
            i += i >> 2;
        }
/* Check them again: */
        progRep->lastSrcAddr = (char *) ((volatile int*) MCD_taskTable[
            channel].contextSaveSpace)[SRCPTR + CSAVE_OFFSET];

        progRep->lastDestAddr = (char *) ((volatile int*) MCD_taskTable[
            channel].contextSaveSpace)[DESTPTR + CSAVE_OFFSET];

        progRep->dmaSize = ((volatile int*) MCD_taskTable[
            channel].contextSaveSpace)[DCOUNT + CSAVE_OFFSET];

        progRep->currBufDesc = (MCD_bufDesc*) ((volatile int*) MCD_taskTable[
            channel].contextSaveSpace)[CURRBD + CSAVE_OFFSET];

/* See if they match: */
        if (   prevRep.lastSrcAddr != progRep->lastSrcAddr
           || prevRep.lastDestAddr != progRep->lastDestAddr
           || prevRep.dmaSize != progRep->dmaSize
           || prevRep.currBufDesc != progRep->currBufDesc)
        {
/* If they don't match, remember previous values and try again:*/
            prevRep.lastSrcAddr = progRep->lastSrcAddr;
            prevRep.lastDestAddr = progRep->lastDestAddr;
            prevRep.dmaSize = progRep->dmaSize;
            prevRep.currBufDesc = progRep->currBufDesc;
            again = MCD_TRUE;
        }
        else
        {
            again = MCD_FALSE;
        }
    } while (again == MCD_TRUE);


/*
 * Update the dCount, srcAddr and destAddr.  To calculate dmaCount, we
 * consider destination address. C overs M1,P1,Z for destination
 */
    switch(MCD_remVariants.remDestRsdIncr[channel])
    {
        case MINUS1:
            subModVal = ((int)progRep->lastDestAddr)
                & ((MCD_remVariants.remXferSize[channel]) - 1);

            addModVal = ((int)progRep->currBufDesc->destAddr)
                & ((MCD_remVariants.remXferSize[channel]) - 1);

            LWAlignedInitDestAddr = (progRep->currBufDesc->destAddr)
                - addModVal;

            LWAlignedCurrDestAddr = (progRep->lastDestAddr) - subModVal;
            destDiffBytes = LWAlignedInitDestAddr - LWAlignedCurrDestAddr;
            bytesNotXfered = (destDiffBytes/MCD_remVariants.remDestIncr[
                channel]) * ( MCD_remVariants.remDestIncr[channel]
                    + MCD_remVariants.remXferSize[channel]);

            progRep->dmaSize = (tt32Bit)(destDiffBytes - bytesNotXfered
                + addModVal - subModVal);
            break;

        case ZERO:
            progRep->lastDestAddr = progRep->currBufDesc->destAddr;
            break;

        case PLUS1:
/* This value has to be subtracted from the final calculated dCount. */
            subModVal = ((int)progRep->currBufDesc->destAddr)
                & ((MCD_remVariants.remXferSize[channel]) - 1);

/* These bytes are already in lastDestAddr. */
            addModVal = ((int)progRep->lastDestAddr)
                & ((MCD_remVariants.remXferSize[channel]) - 1);

            LWAlignedInitDestAddr = (progRep->currBufDesc->destAddr)
                - subModVal;

            LWAlignedCurrDestAddr = (progRep->lastDestAddr) - addModVal;
            destDiffBytes = (progRep->lastDestAddr - LWAlignedInitDestAddr);
            numIterations = ( LWAlignedCurrDestAddr - LWAlignedInitDestAddr)
                / MCD_remVariants.remDestIncr[channel];

            bytesNotXfered =  numIterations *
                ( MCD_remVariants.remDestIncr[channel]
                    - MCD_remVariants.remXferSize[channel]);
            progRep->dmaSize = destDiffBytes - bytesNotXfered - subModVal;
            break;

            default:
            break;
    }

    /* This covers M1,P1,Z for source */
    switch(MCD_remVariants.remSrcRsdIncr[channel]) {
        case MINUS1:
            progRep->lastSrcAddr =
                progRep->currBufDesc->srcAddr +
                 ( MCD_remVariants.remSrcIncr[channel] *
                   (progRep->dmaSize/MCD_remVariants.remXferSize[channel]));
            break;

        case ZERO:
            progRep->lastSrcAddr = progRep->currBufDesc->srcAddr;
            break;

       case PLUS1:
            progRep->lastSrcAddr =
                progRep->currBufDesc->srcAddr +
                 ( MCD_remVariants.remSrcIncr[channel] *
                   (progRep->dmaSize/MCD_remVariants.remXferSize[channel]));
            break;

       default:
            break;
    }
    return(MCD_OK);
}

/********************************************************************/
/* tfMcdResumeActions() does the majority of the actions of a DMA resume.
 * It is called from tfMcdKillDma() and tfMcdResumeDma().  It has to be
 * a separate function because the kill function has to negate the task
 * enable before resuming it, but the resume function has to do nothing
 * if there is no DMA on that channel (i.e., if the enable bit is 0).
 */
static void tfMcdResumeActions (int channel)
{
   MCD_dmaBar->debugControl = DBG_CTL_DISABLE;
   MCD_dmaBar->debugStatus = MCD_dmaBar->debugStatus;

/* This register is selected to know which initiator is actually asserted. */
   MCD_dmaBar->ptdDebug = PTD_DBG_TSK_VLD_INIT; 
   if((MCD_dmaBar->ptdDebug >> channel ) & 0x1)
       MCD_chStatus[channel] = MCD_RUNNING;
   else
       MCD_chStatus[channel] = MCD_IDLE;
}

/********************************************************************/
/* Function:    tfMcdKillDma
 * Purpose:     Halt the DMA on the requested channel, without any
 *              intention of resuming the DMA.
 * Arguments:   channel - requested channel
 * Returns:     MCD_CHANNEL_INVALID if channel is invalid, else MCD_OK
 *
 * Notes:
 *  A DMA may be killed from any state, including paused state, and it
 *  always goes to the MCD_HALTED state even if it is killed while in
 *  the MCD_NO_DMA or MCD_IDLE states.
 */
int tfMcdKillDma (int channel)
{
/* MCD_XferProg progRep; */
    if((channel < 0) || (channel >= NCHANNELS))
        return(MCD_CHANNEL_INVALID);

    MCD_dmaBar->taskControl[channel] = 0x0;
    tfMcdResumeDma (channel);
/*
 * This must be after the write to the TCR so that the task doesn't
 * start up again momentarily, and before the status assignment so
 * as to override whatever tfMcdResumeDma() may do to the channel
 * status.
 */
    MCD_chStatus[channel] = MCD_HALTED;

/*
 * Update the current buffer descriptor's lastDestAddr field
 *
 * tfMcdXfreProgrQuery (channel, &progRep);
 * progRep.currBufDesc->lastDestAddr = progRep.lastDestAddr;
 */
    return(MCD_OK);
}

/********************************************************************/
/* Function:    tfMcdContinueDma
 * Purpose:     Continue a DMA which as stopped due to encountering an
 *              unready buffer descriptor.
 * Arguments:   channel - channel to continue the DMA on
 * Returns:     MCD_CHANNEL_INVALID if channel is invalid, else MCD_OK
 *
 * Notes:
 *  This routine does not check to see if there is a task which can
 *  be continued. Also this routine should not be used with single DMAs.
 */
int tfMcdContinueDma (int channel)
{
    if((channel < 0) || (channel >= NCHANNELS))
        return(MCD_CHANNEL_INVALID);

    MCD_dmaBar->taskControl[channel] |= TASK_CTL_EN;
    MCD_chStatus[channel] = MCD_RUNNING;

    return(MCD_OK);
}

/*********************************************************************
 * tfMcdPauseDma() and tfMcdResumeDma() below use the DMA's debug unit
 * to freeze a task and resume it.  We freeze a task by breakpointing
 * on the stated task.  That is, not any specific place in the task,
 * but any time that task executes.  In particular, when that task
 * executes, we want to freeze that task and only that task.
 *
 * The bits of the debug control register influence interrupts vs.
 * breakpoints as follows:
 * - Bits 14 and 0 enable or disable debug functions.  If enabled, you
 *   will get the interrupt but you may or may not get a breakpoint.
 * - Bits 2 and 1 decide whether you also get a breakpoint in addition
 *   to an interrupt.
 *
 * The debug unit can do these actions in response to either internally
 * detected breakpoint conditions from the comparators, or in response
 * to the external breakpoint pin, or both.
 * - Bits 14 and 1 perform the above-described functions for
 *   internally-generated conditions, i.e., the debug comparators.
 * - Bits 0 and 2 perform the above-described functions for external
 *   conditions, i.e., the breakpoint external pin.
 *
 * Note that, although you "always" get the interrupt when you turn
 * the debug functions, the interrupt can nevertheless, if desired, be
 * masked by the corresponding bit in the PTD's IMR. Note also that
 * this means that bits 14 and 0 must enable debug functions before
 * bits 1 and 2, respectively, have any effect.
 *
 * NOTE: It's extremely important to not pause more than one DMA channel 
 *  at a time.
 ********************************************************************/

/********************************************************************/
/* Function:    tfMcdPauseDma
 * Purpose:     Pauses the DMA on a given channel (if any DMA is running
 *              on that channel).
 * Arguments:   channel
 * Returns:     MCD_CHANNEL_INVALID if channel is invalid, else MCD_OK
 */
int tfMcdPauseDma (int channel)
{
/* MCD_XferProg progRep; */

    if((channel < 0) || (channel >= NCHANNELS))
        return(MCD_CHANNEL_INVALID);

    if (MCD_dmaBar->taskControl[channel] & TASK_CTL_EN)
    {
        MCD_dmaBar->debugComp1 = (tt32Bit)channel;
        MCD_dmaBar->debugControl = (tt32Bit)(DBG_CTL_ENABLE | (1 << (channel + 16)));
        MCD_chStatus[channel] = MCD_PAUSED;

/*
 * Update the current buffer descriptor's lastDestAddr field
 *
 * tfMcdXfreProgrQuery (channel, &progRep);
 * progRep.currBufDesc->lastDestAddr = progRep.lastDestAddr;
 */
    }
    return(MCD_OK);
}

/********************************************************************/
/* Function:    tfMcdResumeDma
 * Purpose:     Resumes the DMA on a given channel (if any DMA is
 *              running on that channel).
 * Arguments:   channel - channel on which to resume DMA
 * Returns:     MCD_CHANNEL_INVALID if channel is invalid, else MCD_OK
 */
int tfMcdResumeDma (int channel)
{
    if((channel < 0) || (channel >= NCHANNELS))
        return(MCD_CHANNEL_INVALID);

    if (MCD_dmaBar->taskControl[channel] & TASK_CTL_EN)
        tfMcdResumeActions (channel);

    return(MCD_OK);
}

/********************************************************************/
/* Function:    tfMcdChecksumQuery
 * Purpose:     Provide the checksum after performing a non-chained DMA
 * Arguments:   channel - channel to report on
 *              csum - pointer to where to write the checksum/CRC
 * Returns:     MCD_ERROR if the channel is invalid, else MCD_OK
 *
 * Notes:
 *
 */
int tfMcdChecksumQuery (int channel, tt32Bit *csum)
{
#ifdef MCD_INCLUDE_EU
    if((channel < 0) || (channel >= NCHANNELS))
    {
        return(MCD_CHANNEL_INVALID);
    }

    *csum = MCD_relocBuffDesc[channel].csumResult;
    return(MCD_OK);
#else
    TM_UNREF_IN_ARG(channel);
    TM_UNREF_IN_ARG(csum);
    return(MCD_ERROR);
#endif
}

/********************************************************************/
/* Function:    tfMcdGetCodeSize
 * Purpose:     Provide the size requirements of the microcoded tasks
 * Returns:     Size in bytes
 */
int tfMcdGetCodeSize(void)
{
#ifdef MCD_INCLUDE_EU
    return(0x2b5c);
#else
    return(0x173c);
#endif
}

/********************************************************************/
/* Function:    tfMcdGetVersion
 * Purpose:     Provide the version string and number
 * Arguments:   longVersion - user supplied pointer to a pointer to a char
 *                    which points to the version string
 * Returns:     Version number and version string (by reference)
 */
char MCD_versionString[] = "Multi-channel DMA API Alpha v0.3 (2004-04-26)";
#define MCD_REV_MAJOR   0x00
#define MCD_REV_MINOR   0x03

int tfMcdGetVersion(char **longVersion)
{
    *longVersion = MCD_versionString;
    return((MCD_REV_MAJOR << 8) | MCD_REV_MINOR);
}

/********************************************************************/
/* Private version of memcpy()
 * Note that everything this is used for is longword-aligned.
 */
static void tfMcdmemcpy (int *dest, int *src, tt32Bit size)
{
    tt32Bit i;

    for (i = 0;  i < size;  i += sizeof(int), dest++, src++)
    {
        *dest = *src;
    }
}

/*
 * Task 0
 */
 
void  tfMcdStartDmaChainNoEu(int *currBD, short srcIncr, short destIncr,
                             int xferSize, short xferSizeIncr, int *cSave,
                             volatile TaskTableEntry *taskTable, int channel)
{

    MCD_SET_VAR(taskTable+channel, 2, (tt32Bit)currBD);         /* var[2] */
    MCD_SET_VAR(taskTable+channel, 25, (tt32Bit)(0xe000 << 16)
                    | (0xffff & srcIncr));                  /* inc[1] */
    MCD_SET_VAR(taskTable+channel, 24, (tt32Bit)(0xe000 << 16)
                    | (0xffff & destIncr));                 /* inc[0] */
    MCD_SET_VAR(taskTable+channel, 11, (tt32Bit)xferSize);      /* var[11] */
    MCD_SET_VAR(taskTable+channel, 26, (tt32Bit)(0x2000 << 16)
                    | (0xffff & xferSizeIncr));             /* inc[2] */
    MCD_SET_VAR(taskTable+channel, 0, (tt32Bit)cSave);          /* var[0] */
    MCD_SET_VAR(taskTable+channel, 1, (tt32Bit)0x00000000);     /* var[1] */
    MCD_SET_VAR(taskTable+channel, 3, (tt32Bit)0x00000000);     /* var[3] */
    MCD_SET_VAR(taskTable+channel, 4, (tt32Bit)0x00000000);     /* var[4] */
    MCD_SET_VAR(taskTable+channel, 5, (tt32Bit)0x00000000);     /* var[5] */
    MCD_SET_VAR(taskTable+channel, 6, (tt32Bit)0x00000000);     /* var[6] */
    MCD_SET_VAR(taskTable+channel, 7, (tt32Bit)0x00000000);     /* var[7] */
    MCD_SET_VAR(taskTable+channel, 8, (tt32Bit)0x00000000);     /* var[8] */
    MCD_SET_VAR(taskTable+channel, 9, (tt32Bit)0x00000000);     /* var[9] */
    MCD_SET_VAR(taskTable+channel, 10, (tt32Bit)0x00000000);    /* var[10] */
    MCD_SET_VAR(taskTable+channel, 12, (tt32Bit)0x00000000);    /* var[12] */
    MCD_SET_VAR(taskTable+channel, 13, (tt32Bit)0x80000000);    /* var[13] */
    MCD_SET_VAR(taskTable+channel, 14, (tt32Bit)0x00000010);    /* var[14] */
    MCD_SET_VAR(taskTable+channel, 15, (tt32Bit)0x00000004);    /* var[15] */
    MCD_SET_VAR(taskTable+channel, 16, (tt32Bit)0x08000000);    /* var[16] */
    MCD_SET_VAR(taskTable+channel, 27, (tt32Bit)0x00000000);    /* inc[3] */
    MCD_SET_VAR(taskTable+channel, 28, (tt32Bit)0x80000000);    /* inc[4] */
    MCD_SET_VAR(taskTable+channel, 29, (tt32Bit)0x80000001);    /* inc[5] */
    MCD_SET_VAR(taskTable+channel, 30, (tt32Bit)0x40000000);    /* inc[6] */

    /* Set the task's Enable bit in its Task Control Register */
    MCD_dmaBar->taskControl[channel] |= (tt16Bit)0x8000;
}


/*
 * Task 1
 */
 
void  tfMcdStartDmaSingleNoEu(char *srcAddr, short srcIncr, char *destAddr,
                              short destIncr, int dmaSize, short xferSizeIncr,
                              int flags, int *currBD, int *cSave,
                              volatile TaskTableEntry *taskTable, int channel)
{

    MCD_SET_VAR(taskTable+channel, 7, (tt32Bit)srcAddr);        /* var[7] */
    MCD_SET_VAR(taskTable+channel, 25, (tt32Bit)(0xe000 << 16)
                    | (0xffff & srcIncr));                  /* inc[1] */
    MCD_SET_VAR(taskTable+channel, 2, (tt32Bit)destAddr);       /* var[2] */
    MCD_SET_VAR(taskTable+channel, 24, (tt32Bit)(0xe000 << 16)
                    | (0xffff & destIncr));                 /* inc[0] */
    MCD_SET_VAR(taskTable+channel, 3, (tt32Bit)dmaSize);        /* var[3] */
    MCD_SET_VAR(taskTable+channel, 26, (tt32Bit)(0x2000 << 16)
                    | (0xffff & xferSizeIncr));             /* inc[2] */
    MCD_SET_VAR(taskTable+channel, 5, (tt32Bit)flags);          /* var[5] */
    MCD_SET_VAR(taskTable+channel, 1, (tt32Bit)currBD);         /* var[1] */
    MCD_SET_VAR(taskTable+channel, 0, (tt32Bit)cSave);          /* var[0] */
    MCD_SET_VAR(taskTable+channel, 4, (tt32Bit)0x00000000);     /* var[4] */
    MCD_SET_VAR(taskTable+channel, 6, (tt32Bit)0x00000000);     /* var[6] */
    MCD_SET_VAR(taskTable+channel, 8, (tt32Bit)0x00000000);     /* var[8] */
    MCD_SET_VAR(taskTable+channel, 9, (tt32Bit)0x00000004);     /* var[9] */
    MCD_SET_VAR(taskTable+channel, 10, (tt32Bit)0x08000000);    /* var[10] */
    MCD_SET_VAR(taskTable+channel, 27, (tt32Bit)0x00000000);    /* inc[3] */
    MCD_SET_VAR(taskTable+channel, 28, (tt32Bit)0x80000001);    /* inc[4] */
    MCD_SET_VAR(taskTable+channel, 29, (tt32Bit)0x40000000);    /* inc[5] */

    /* Set the task's Enable bit in its Task Control Register */
    MCD_dmaBar->taskControl[channel] |= (tt16Bit)0x8000;
}


/*
 * Task 2
 */
 
void  tfMcdStartDmaChainEu(int *currBD, short srcIncr, short destIncr,
                           int xferSize, short xferSizeIncr, int *cSave,
                           volatile TaskTableEntry *taskTable, int channel)
{

    MCD_SET_VAR(taskTable+channel, 3, (tt32Bit)currBD);         /* var[3] */
    MCD_SET_VAR(taskTable+channel, 25, (tt32Bit)(0xe000 << 16)
                    | (0xffff & srcIncr));                  /* inc[1] */
    MCD_SET_VAR(taskTable+channel, 24, (tt32Bit)(0xe000 << 16)
                    | (0xffff & destIncr));                 /* inc[0] */
    MCD_SET_VAR(taskTable+channel, 12, (tt32Bit)xferSize);      /* var[12] */
    MCD_SET_VAR(taskTable+channel, 26, (tt32Bit)(0x2000 << 16)
                    | (0xffff & xferSizeIncr));             /* inc[2] */
    MCD_SET_VAR(taskTable+channel, 0, (tt32Bit)cSave);          /* var[0] */
    MCD_SET_VAR(taskTable+channel, 1, (tt32Bit)0x00000000);     /* var[1] */
    MCD_SET_VAR(taskTable+channel, 2, (tt32Bit)0x00000000);     /* var[2] */
    MCD_SET_VAR(taskTable+channel, 4, (tt32Bit)0x00000000);     /* var[4] */
    MCD_SET_VAR(taskTable+channel, 5, (tt32Bit)0x00000000);     /* var[5] */
    MCD_SET_VAR(taskTable+channel, 6, (tt32Bit)0x00000000);     /* var[6] */
    MCD_SET_VAR(taskTable+channel, 7, (tt32Bit)0x00000000);     /* var[7] */
    MCD_SET_VAR(taskTable+channel, 8, (tt32Bit)0x00000000);     /* var[8] */
    MCD_SET_VAR(taskTable+channel, 9, (tt32Bit)0x00000000);     /* var[9] */
    MCD_SET_VAR(taskTable+channel, 10, (tt32Bit)0x00000000);    /* var[10] */
    MCD_SET_VAR(taskTable+channel, 11, (tt32Bit)0x00000000);    /* var[11] */
    MCD_SET_VAR(taskTable+channel, 13, (tt32Bit)0x00000000);    /* var[13] */
    MCD_SET_VAR(taskTable+channel, 14, (tt32Bit)0x80000000);    /* var[14] */
    MCD_SET_VAR(taskTable+channel, 15, (tt32Bit)0x00000010);    /* var[15] */
    MCD_SET_VAR(taskTable+channel, 16, (tt32Bit)0x00000001);    /* var[16] */
    MCD_SET_VAR(taskTable+channel, 17, (tt32Bit)0x00000004);    /* var[17] */
    MCD_SET_VAR(taskTable+channel, 18, (tt32Bit)0x08000000);    /* var[18] */
    MCD_SET_VAR(taskTable+channel, 27, (tt32Bit)0x00000000);    /* inc[3] */
    MCD_SET_VAR(taskTable+channel, 28, (tt32Bit)0x80000000);    /* inc[4] */
    MCD_SET_VAR(taskTable+channel, 29, (tt32Bit)0xc0000000);    /* inc[5] */
    MCD_SET_VAR(taskTable+channel, 30, (tt32Bit)0x80000001);    /* inc[6] */
    MCD_SET_VAR(taskTable+channel, 31, (tt32Bit)0x40000000);    /* inc[7] */

    /* Set the task's Enable bit in its Task Control Register */
    MCD_dmaBar->taskControl[channel] |= (tt16Bit)0x8000;
}


/*
 * Task 3
 */
 
void  tfMcdStartDmaSingleEu(char *srcAddr, short srcIncr, char *destAddr,
                            short destIncr, int dmaSize, short xferSizeIncr,
                            int flags, int *currBD, int *cSave,
                            volatile TaskTableEntry *taskTable, int channel)
{

    MCD_SET_VAR(taskTable+channel, 8, (tt32Bit)srcAddr);        /* var[8] */
    MCD_SET_VAR(taskTable+channel, 25, (tt32Bit)(0xe000 << 16)
                    | (0xffff & srcIncr));                  /* inc[1] */
    MCD_SET_VAR(taskTable+channel, 3, (tt32Bit)destAddr);       /* var[3] */
    MCD_SET_VAR(taskTable+channel, 24, (tt32Bit)(0xe000 << 16)
                    | (0xffff & destIncr));                 /* inc[0] */
    MCD_SET_VAR(taskTable+channel, 4, (tt32Bit)dmaSize);        /* var[4] */
    MCD_SET_VAR(taskTable+channel, 26, (tt32Bit)(0x2000 << 16)
                    | (0xffff & xferSizeIncr));             /* inc[2] */
    MCD_SET_VAR(taskTable+channel, 6, (tt32Bit)flags);          /* var[6] */
    MCD_SET_VAR(taskTable+channel, 2, (tt32Bit)currBD);         /* var[2] */
    MCD_SET_VAR(taskTable+channel, 0, (tt32Bit)cSave);          /* var[0] */
    MCD_SET_VAR(taskTable+channel, 1, (tt32Bit)0x00000000);     /* var[1] */
    MCD_SET_VAR(taskTable+channel, 5, (tt32Bit)0x00000000);     /* var[5] */
    MCD_SET_VAR(taskTable+channel, 7, (tt32Bit)0x00000000);     /* var[7] */
    MCD_SET_VAR(taskTable+channel, 9, (tt32Bit)0x00000000);     /* var[9] */
    MCD_SET_VAR(taskTable+channel, 10, (tt32Bit)0x00000001);    /* var[10] */
    MCD_SET_VAR(taskTable+channel, 11, (tt32Bit)0x00000004);    /* var[11] */
    MCD_SET_VAR(taskTable+channel, 12, (tt32Bit)0x08000000);    /* var[12] */
    MCD_SET_VAR(taskTable+channel, 27, (tt32Bit)0x00000000);    /* inc[3] */
    MCD_SET_VAR(taskTable+channel, 28, (tt32Bit)0xc0000000);    /* inc[4] */
    MCD_SET_VAR(taskTable+channel, 29, (tt32Bit)0x80000000);    /* inc[5] */
    MCD_SET_VAR(taskTable+channel, 30, (tt32Bit)0x80000001);    /* inc[6] */
    MCD_SET_VAR(taskTable+channel, 31, (tt32Bit)0x40000000);    /* inc[7] */

    /* Set the task's Enable bit in its Task Control Register */
    MCD_dmaBar->taskControl[channel] |= (tt16Bit)0x8000;
}


/*
 * Task 4
 */
 
void  tfMcdStartDmaENetRcv(char *bDBase, char *currBD, char *rcvFifoPtr,
                           volatile TaskTableEntry *taskTable, int channel)
{

    MCD_SET_VAR(taskTable+channel, 0, (tt32Bit)bDBase);         /* var[0] */
    MCD_SET_VAR(taskTable+channel, 3, (tt32Bit)currBD);         /* var[3] */
    MCD_SET_VAR(taskTable+channel, 6, (tt32Bit)rcvFifoPtr);     /* var[6] */
    MCD_SET_VAR(taskTable+channel, 1, (tt32Bit)0x00000000);     /* var[1] */
    MCD_SET_VAR(taskTable+channel, 2, (tt32Bit)0x00000000);     /* var[2] */
    MCD_SET_VAR(taskTable+channel, 4, (tt32Bit)0x00000000);     /* var[4] */
    MCD_SET_VAR(taskTable+channel, 5, (tt32Bit)0x00000000);     /* var[5] */
    MCD_SET_VAR(taskTable+channel, 7, (tt32Bit)0x00000000);     /* var[7] */
    MCD_SET_VAR(taskTable+channel, 8, (tt32Bit)0x00000000);     /* var[8] */
    MCD_SET_VAR(taskTable+channel, 9, (tt32Bit)0x0000ffff);     /* var[9] */
    MCD_SET_VAR(taskTable+channel, 10, (tt32Bit)0x30000000);    /* var[10] */
    MCD_SET_VAR(taskTable+channel, 11, (tt32Bit)0x0fffffff);    /* var[11] */
    MCD_SET_VAR(taskTable+channel, 12, (tt32Bit)0x00000008);    /* var[12] */
    MCD_SET_VAR(taskTable+channel, 24, (tt32Bit)0x00000000);    /* inc[0] */
    MCD_SET_VAR(taskTable+channel, 25, (tt32Bit)0x60000000);    /* inc[1] */
    MCD_SET_VAR(taskTable+channel, 26, (tt32Bit)0x20000004);    /* inc[2] */
    MCD_SET_VAR(taskTable+channel, 27, (tt32Bit)0x40000000);    /* inc[3] */

    /* Set the task's Enable bit in its Task Control Register */
    MCD_dmaBar->taskControl[channel] |= (tt16Bit)0x8000;
}


/*
 * Task 5
 */
 
void  tfMcdStartDmaENetXmit(char *bDBase, char *currBD, char *xmitFifoPtr,
                            volatile TaskTableEntry *taskTable, int channel)
{
    MCD_SET_VAR(taskTable+channel, 0, (tt32Bit)bDBase);         /* var[0] */
    MCD_SET_VAR(taskTable+channel, 3, (tt32Bit)currBD);         /* var[3] */
    MCD_SET_VAR(taskTable+channel, 11, (tt32Bit)xmitFifoPtr);   /* var[11] */
    MCD_SET_VAR(taskTable+channel, 1, (tt32Bit)0x00000000);     /* var[1] */
    MCD_SET_VAR(taskTable+channel, 2, (tt32Bit)0x00000000);     /* var[2] */
    MCD_SET_VAR(taskTable+channel, 4, (tt32Bit)0x00000000);     /* var[4] */
    MCD_SET_VAR(taskTable+channel, 5, (tt32Bit)0x00000000);     /* var[5] */
    MCD_SET_VAR(taskTable+channel, 6, (tt32Bit)0x00000000);     /* var[6] */
    MCD_SET_VAR(taskTable+channel, 7, (tt32Bit)0x00000000);     /* var[7] */
    MCD_SET_VAR(taskTable+channel, 8, (tt32Bit)0x00000000);     /* var[8] */
    MCD_SET_VAR(taskTable+channel, 9, (tt32Bit)0x00000000);     /* var[9] */
    MCD_SET_VAR(taskTable+channel, 10, (tt32Bit)0x00000000);    /* var[10] */
    MCD_SET_VAR(taskTable+channel, 12, (tt32Bit)0x00000000);    /* var[12] */
    MCD_SET_VAR(taskTable+channel, 13, (tt32Bit)0x0000ffff);    /* var[13] */
    MCD_SET_VAR(taskTable+channel, 14, (tt32Bit)0xffffffff);    /* var[14] */
    MCD_SET_VAR(taskTable+channel, 15, (tt32Bit)0x00000004);    /* var[15] */
    MCD_SET_VAR(taskTable+channel, 16, (tt32Bit)0x00000008);    /* var[16] */
    MCD_SET_VAR(taskTable+channel, 24, (tt32Bit)0x00000000);    /* inc[0] */
    MCD_SET_VAR(taskTable+channel, 25, (tt32Bit)0x60000000);    /* inc[1] */
    MCD_SET_VAR(taskTable+channel, 26, (tt32Bit)0x40000000);    /* inc[2] */
    MCD_SET_VAR(taskTable+channel, 27, (tt32Bit)0xc000fffc);    /* inc[3] */
    MCD_SET_VAR(taskTable+channel, 28, (tt32Bit)0xe0000004);    /* inc[4] */
    MCD_SET_VAR(taskTable+channel, 29, (tt32Bit)0x80000000);    /* inc[5] */
    MCD_SET_VAR(taskTable+channel, 30, (tt32Bit)0x4000ffff);    /* inc[6] */
    MCD_SET_VAR(taskTable+channel, 31, (tt32Bit)0xe0000001);    /* inc[7] */

    /* Set the task's Enable bit in its Task Control Register */
    MCD_dmaBar->taskControl[channel] |= (tt16Bit)0x8000;
}

/********************************************************************/
/* Initialize the BCM5222 PHY
 *
 * This function sets up the Auto-Negotiate Advertisement register
 * within the PHY and then forces the PHY to auto-negotiate for
 * it's settings.
 * 
 * Params:
 *  fec_ch      FEC channel
 *  phy_addr    Address of the PHY.
 *  speed       Desired speed (10BaseT or 100BaseTX)
 *  duplex      Desired duplex (Full or Half)
 *
 * Return Value:
 *  0 if MII commands fail
 *  1 otherwise
 */
int tfPhyInit(tt8Bit fec_ch, tt8Bit phy_addr, tt8Bit speed, tt8Bit duplex)
{
    int timeout;
    tt16Bit settings;
    int errorCode = TM_DEV_OKAY;

    tm_assert(tfPhyInit, (speed == FEC_MII_10BASE_T
                            || speed == FEC_MII_100BASE_TX));
    tm_assert(tfPhyInit, (duplex == FEC_MII_HALF_DUPLEX
                            || duplex == FEC_MII_FULL_DUPLEX));

/* Reset the PHY */
    if (!tfEthfecMIIWrite(0, phy_addr, BCM5222_CTRL,
                          BCM5222_CTRL_RESET | BCM5222_CTRL_ANE))
    {
        errorCode = TM_DEV_ERROR;
        goto tfPhyInitEnd;
    }

/* Wait for the PHY to reset */
    for (timeout = 0; timeout < FEC_MII_TIMEOUT; ++timeout)
    {
        tfEthfecMIIRead(0, phy_addr, BCM5222_CTRL, &settings);
        if (!(settings & BCM5222_CTRL_RESET))
        {
            break;
        }
    }
    if (FEC_MII_TIMEOUT <= timeout)
    {
        errorCode = TM_DEV_ERROR;
        goto tfPhyInitEnd;
    }

    settings = (BCM5222_AN_ADV_NEXT_PAGE | BCM5222_AN_ADV_PAUSE);

    if (speed == FEC_MII_10BASE_T)
    {
        settings |= (tt16Bit)((duplex == FEC_MII_FULL_DUPLEX) 
            ? (BCM5222_AN_ADV_10BT_FDX | BCM5222_AN_ADV_10BT) 
            : BCM5222_AN_ADV_10BT);
    }
    else
/* (speed == FEC_MII_100BASE_TX) */
    {
        settings = (tt16Bit)((duplex == FEC_MII_FULL_DUPLEX)  
            ? (BCM5222_AN_ADV_100BTX_FDX | BCM5222_AN_ADV_100BTX | 
               BCM5222_AN_ADV_10BT_FDX   | BCM5222_AN_ADV_10BT    ) 
            : (BCM5222_AN_ADV_100BTX | BCM5222_AN_ADV_10BT));
    }
    
/* Set the Auto-Negotiation Advertisement Register */
    if (!tfEthfecMIIWrite(0, phy_addr, BCM5222_AN_ADV, settings))
    {
        errorCode = TM_DEV_ERROR;
        goto tfPhyInitEnd;
    }

/* Enable Auto-Negotiation */
    if (!tfEthfecMIIWrite(0, phy_addr, BCM5222_CTRL,
                          (BCM5222_CTRL_ANE | BCM5222_CTRL_RESTART_AN)))
    {
        errorCode = TM_DEV_ERROR;
        goto tfPhyInitEnd;
    }


/* Wait for auto-negotiation to complete */
    for (timeout = 0; timeout < FEC_MII_TIMEOUT; ++timeout)
    {
        if (!tfEthfecMIIRead(0, phy_addr, BCM5222_STAT, &settings))
        {
            errorCode = TM_DEV_ERROR;
            goto tfPhyInitEnd;
        }
        if (settings & BCM5222_STAT_AN_COMPLETE)
        {
            break;
        }
    }
    if (FEC_MII_TIMEOUT <= timeout)
    {
        errorCode = TM_DEV_ERROR;
        goto tfPhyInitEnd;
    }

/* Read Auxiliary Control/Status Register  */
    if (!tfEthfecMIIRead(0, phy_addr, BCM5222_ACSR, &settings))
    {
        errorCode = TM_DEV_ERROR;
        goto tfPhyInitEnd;
    }

/* Set the proper duplex in the FEC now that we have auto-negotiated */
    if (settings & BCM5222_ACSR_FDX)
    {
        tfEthfecSetDuplex(fec_ch, FEC_MII_FULL_DUPLEX);
    }
    else
    {
        tfEthfecSetDuplex(fec_ch, FEC_MII_HALF_DUPLEX);
    }

#ifdef DEBUG
        printf("\nPHY Mode: ");
        if (settings & BCM5222_ACSR_100BTX)
        {
            printf("100Mbps ");
        }
        else
        {
            printf("10Mbps ");
        }
        if (settings & BCM5222_ACSR_FDX)
        {
            printf("Full-duplex\n");
        }
        else
        {
            printf("Half-duplex\n");
        }
#endif

tfPhyInitEnd:
    return errorCode;
}

/*
 * Read out all of the phy register values
 */
void tfPhyGetReg(tt16Bit* status0, tt16Bit* status1)
{
   
   tfEthfecMIIRead(0, 0x00, 0x00000000, &status0[0]); 
   tfEthfecMIIRead(0, 0x00, 0x00000001, &status0[1]); 
   tfEthfecMIIRead(0, 0x00, 0x00000004, &status0[4]); 
   tfEthfecMIIRead(0, 0x00, 0x00000005, &status0[5]); 
   tfEthfecMIIRead(0, 0x00, 0x00000006, &status0[6]); 
   tfEthfecMIIRead(0, 0x00, 0x00000007, &status0[7]); 
   tfEthfecMIIRead(0, 0x00, 0x00000008, &status0[8]); 
   tfEthfecMIIRead(0, 0x00, 0x00000010, &status0[16]);
   tfEthfecMIIRead(0, 0x00, 0x00000011, &status0[17]);
   tfEthfecMIIRead(0, 0x00, 0x00000012, &status0[18]);
   tfEthfecMIIRead(0, 0x00, 0x00000013, &status0[19]);
   tfEthfecMIIRead(0, 0x00, 0x00000018, &status0[24]);
   tfEthfecMIIRead(0, 0x00, 0x00000019, &status0[25]);
   tfEthfecMIIRead(0, 0x00, 0x0000001B, &status0[27]);
   tfEthfecMIIRead(0, 0x00, 0x0000001C, &status0[28]);
   tfEthfecMIIRead(0, 0x00, 0x0000001E, &status0[30]);

   tfEthfecMIIRead(0, 0x01, 0x00000000, &status1[0]);
   tfEthfecMIIRead(0, 0x01, 0x00000001, &status1[1]);
   tfEthfecMIIRead(0, 0x01, 0x00000004, &status1[4]);
   tfEthfecMIIRead(0, 0x01, 0x00000005, &status1[5]);
   tfEthfecMIIRead(0, 0x01, 0x00000006, &status1[6]);
   tfEthfecMIIRead(0, 0x01, 0x00000007, &status1[7]);
   tfEthfecMIIRead(0, 0x01, 0x00000008, &status1[8]);
   tfEthfecMIIRead(0, 0x01, 0x00000010, &status1[16]);
   tfEthfecMIIRead(0, 0x01, 0x00000011, &status1[17]);
   tfEthfecMIIRead(0, 0x01, 0x00000012, &status1[18]);
   tfEthfecMIIRead(0, 0x01, 0x00000013, &status1[19]);
   tfEthfecMIIRead(0, 0x01, 0x00000018, &status1[24]);
   tfEthfecMIIRead(0, 0x01, 0x00000019, &status1[25]);
   tfEthfecMIIRead(0, 0x01, 0x0000001B, &status1[27]);
   tfEthfecMIIRead(0, 0x01, 0x0000001C, &status1[28]);
   tfEthfecMIIRead(0, 0x01, 0x0000001E, &status1[30]);
}


/********************************************************************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: trm5485.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 53870    Output checksum: 36350
 */
