/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:
 *
 * Filename: tr7616e.c
 * Author: Sebastien Windal
 * Date Created:
 * $Source: source/drivers/sh7616/tr7616e.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:21CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <machine.h>
#include "includes.h"
#include "tr7616e.h"

#include <trsystem.h>
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>

#ifdef TM_USE_CLIB
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#endif /* TM_USE_CLIB */


#define TM_SYS_MACADR_High 0x12345678
#define TM_SYS_MACADR_Low  0x00009abc


ttSH7616Ether tvEther;

#pragma section     _EDMAC_DESC
 
volatile ttEdmacDesc tvRcvDesc[DSC_COUNT],
                     tvSndDesc[DSC_COUNT];

/* E-DMAC transfers data by block of 16 bytes.
 * Instead of pointing to ubRcvBuff and  ubSndBuff,
 * The Send and Recv desciptor may point to the closest
 * 16 byte address following the address of ubRcvBuff
 * and ubSndBuff
 */
static unsigned char ubRcvBuff[ DSC_COUNT ][ BUFFSIZE + 16 ],
                     ubSndBuff[ DSC_COUNT ][ BUFFSIZE + 16 ];

#pragma section

/* static functions */
static void tfDescInit(ttEdmacDesc *, unsigned long);
static void tfSH7616EtherEdmacStatusCheck(void);

void DP83843_reset(void);
void DP83843_init(void);
static void write_PHY(unsigned short);
static void read_PHY(unsigned short);
static void write_0(void);
static void write_1(void);

void tfSH7616EthernetInit()
{
    unsigned long   uwI;

    /* initialize the MAC layer */
    DP83843_init();

    tm_memset(&tvEther, 0x00, sizeof(ttSH7616Ether));

    tvEther.uwMacAddrHigh = TM_SYS_MACADR_High;
    tvEther.uwMacAddrLow  = TM_SYS_MACADR_Low;

    tfDescInit(&tvRcvDesc[0], (unsigned long)ACT);
    tfDescInit(&tvSndDesc[0], (unsigned long)0);

/* software reset the Ethernet controller and MAC layer */
    EDMAC.EDMR.LONG = 0x00000001;
    for (uwI = 0; uwI < 10000; uwI ++)    {    ;    } /* Delay */
/* disable transmit and receive at the MAC layer */
    
    EDMAC.RDLAR = &tvRcvDesc[0]; /* Sets start address of rcv decriptor list */
    EDMAC.TDLAR = &tvSndDesc[0]; /* Sets start address of snd decriptor list */

    EDMAC.TRSCER.LONG = 0x00000000;
    EDMAC.TFTR        = 0x00000080;
    EDMAC.FDR.LONG    = 0x00000101;
    EDMAC.RCR.LONG    = 0x00000001;
    EDMAC.EDOCR.LONG  = 0x00000000;

    EC.MAHR = tvEther.uwMacAddrHigh;
    EC.MALR = tvEther.uwMacAddrLow;

    EC.ECSR.LONG   = 0x00000007;
    EC.ECSIPR.LONG = 0x00000000;

    EC.RFLR        = 1518;
    EC.ECMR.LONG   = 0x00000062;

    tvEther.uhSendDescPtr  = &tvSndDesc[0];
    tvEther.uhRecvDescPtr  = &tvRcvDesc[0];
    tvEther.uhSendBufCount = 0;
    tvEther.uhRecvBufCount = 0;

    tvEther.uwSaveEdmacRDFAR = (unsigned long)&tvRcvDesc[0];
    tvEther.uwSaveEdmacTDFAR = (unsigned long)&tvSndDesc[0];

/* Enable Interrupts */
    EDMAC.EESIPR.BIT.FRIP    = 1; /* enable frame receives Buffer ITR        */
    EDMAC.EESIPR.BIT.TCIP    = 1; /* enable Frame Transmit Complete ITR      */
    EDMAC.EESIPR.BIT.ECIIP   = 1; /* enable EtherC status ITR                */
    EDMAC.EESIPR.BIT.RFCOFIP = 1; /* enable rcv frame counter overflow ITR   */
    EDMAC.EESIPR.BIT.TDEIP   = 1; /* enable Transmit descriptor exhausted ITR*/
    EDMAC.EESIPR.BIT.TFUFIP  = 1; /* enable transmit FIFO underflow ITR      */
    EDMAC.EESIPR.BIT.RDEIP   = 1; /* enable receive descriptor exhausted ITR */
    EDMAC.EESIPR.BIT.RFOFIP  = 1; /* enable receive FIFO overflow ITR        */
    EDMAC.EESIPR.BIT.ITFIP   = 1; /* enable illegal transmit frame ITR       */
    EDMAC.EESIPR.BIT.CNDIP   = 1; /* enable carrier not detected ITR         */
    EDMAC.EESIPR.BIT.DLCIP   = 1; /* enable detect loss of carrier ITR       */
    EDMAC.EESIPR.BIT.CDIP    = 1; /* enable collision detection ITR          */
    EDMAC.EESIPR.BIT.TROIP   = 1; /* enable transmit retry over ITR          */
    EDMAC.EESIPR.BIT.RMAFIP  = 1; /* enable rcv multicast addres frame ITR   */
    EDMAC.EESIPR.BIT.RRFIP   = 1; /* enable recevie residual bit frame       */
    EDMAC.EESIPR.BIT.RTLFIP  = 1; /* enable receive too long frame ITR       */
    EDMAC.EESIPR.BIT.RTSFIP  = 1; /* enable receive too short frame ITR      */
    EDMAC.EESIPR.BIT.PREIP   = 1; /* enable PHY-LSI receive error ITR        */
    EDMAC.EESIPR.BIT.CERFIP  = 1; /* enable CRC error on received frame ITR  */
    tvEther.uwOpenFlag = 0;

/* Install the EtherC interrupt function in the vector table */
    tfKernelInstallIsrHandler(tfSH7616EtherIntEntry, EXC_INT_LAN);

/* add start ELJP */
    tfKernelSetCritical();
    /* enable E-DMAC interrupt */
    *IPRB |= 0xF000;        /* interrupt signal for Ether-C         */
    *VCRA  = 0x4500;        /* IRL1011 (69)                         */
    tfKernelReleaseCritical();
/* add end ELJP */
}


void tfSH7616EthernetOpen()
{
    int            erCode;
    unsigned long  uwI;

/* initialize the MAC layer */
    DP83843_init();

    tm_memset(&tvEther.uwEdmacEESR, 0x00,
              ((unsigned long)&tvEther.uhReserved -
               (unsigned long)&tvEther.uwEdmacEESR));

    tfDescInit(&tvRcvDesc[0], (unsigned long)ACT);
    tfDescInit(&tvSndDesc[0], (unsigned long)0);


/* software reset the Ethernet controller and MAC layer */
    EDMAC.EDMR.LONG = 0x00000001;
    for (uwI = 0; uwI < 10000; uwI ++)    {    ;    } /* Delay */
/* disable transmit and receive at the MAC layer */

    EDMAC.RDLAR = &tvRcvDesc[0];
    EDMAC.TDLAR = &tvSndDesc[0];

    EDMAC.TRSCER.LONG = 0x00000000;
    EDMAC.TFTR        = 0x00000080;
    EDMAC.FDR.LONG    = 0x00000101;
    EDMAC.RCR.LONG    = 0x00000001;
    EDMAC.EDOCR.LONG  = 0x00000000;

    EC.MAHR = tvEther.uwMacAddrHigh;
    EC.MALR = tvEther.uwMacAddrLow;

    EC.ECSR.LONG   = 0x00000007;
    EC.ECSIPR.LONG = 0x00000000;
    EC.RFLR        = 1518;
    EC.ECMR.LONG   = 0x00000062;

    tvEther.uhSendDescPtr  = &tvSndDesc[0];
    tvEther.uhRecvDescPtr  = &tvRcvDesc[0];
    tvEther.uhSendBufCount = 0;
    tvEther.uhRecvBufCount = 0;

    tvEther.uwSaveEdmacRDFAR = (unsigned long)&tvRcvDesc[0];
    tvEther.uwSaveEdmacTDFAR = (unsigned long)&tvSndDesc[0];

    EDMAC.EESIPR.BIT.TCIP    = 1;
    EDMAC.EESIPR.BIT.FRIP    = 1;
    EDMAC.EESIPR.BIT.ECIIP   = 1;

    EDMAC.EESIPR.BIT.RFCOFIP = 1;
    EDMAC.EESIPR.BIT.TFUFIP  = 1;
    EDMAC.EESIPR.BIT.RDEIP   = 1;
    EDMAC.EESIPR.BIT.RFOFIP  = 1;
    EDMAC.EESIPR.BIT.TDEIP   = 1;
    EDMAC.EESIPR.BIT.ITFIP   = 1;
    EDMAC.EESIPR.BIT.CNDIP   = 1;
    EDMAC.EESIPR.BIT.DLCIP   = 1;
    EDMAC.EESIPR.BIT.CDIP    = 1;
    EDMAC.EESIPR.BIT.TROIP   = 1;
    EDMAC.EESIPR.BIT.RMAFIP  = 1;
    EDMAC.EESIPR.BIT.RRFIP   = 1;
    EDMAC.EESIPR.BIT.RTLFIP  = 1;
    EDMAC.EESIPR.BIT.RTSFIP  = 1;
    EDMAC.EESIPR.BIT.PREIP   = 1;
    EDMAC.EESIPR.BIT.CERFIP  = 1;

    EDMAC.EDRRR.LONG = 0x00000001;
}


void tfDescInit(ttEdmacDesc   *stpDesc,
                unsigned long  uwStatus)
{
    int             erCode;
    int             iI, iJ;
    unsigned char  *tmpBuf;
    int             shift;
    ttEdmacDesc    *stpCurrent;

    for(iI = 0; iI < DSC_COUNT; iI++)
    {
        stpCurrent = &stpDesc[iI];

        if(uwStatus == 0)
        {
/* Make sure the send buffer is 16 bytes boundary aligned */
            tmpBuf = (unsigned char *) &ubSndBuff[iI][0];
            shift = ((unsigned long) tmpBuf) & 0x0000000F;
            if (shift)
            {
                tmpBuf += 16 - shift;
            }
            stpCurrent->ubpBuf = tmpBuf;
        }
        else
        {
/* Make sure the receive buffer is 16 byte boundary aligned */
            tmpBuf = (unsigned char *) &ubRcvBuff[iI][0];
            shift = ((unsigned long) tmpBuf) & 0x0000000F;
            if (shift)
            {
                tmpBuf += 16 - shift;
            }

            stpCurrent->ubpBuf = tmpBuf;
        }

        for(iJ = 0; iJ < BUFFSIZE; iJ ++)
        {
            stpCurrent->ubpBuf[iJ] = 0;
        }

        stpCurrent->uhBufSize = ETHERSIZE;
        stpCurrent->uhSize    = 0;
        stpCurrent->uwStatus  = uwStatus;
        stpCurrent->stpNext   = &stpDesc[iI + 1];
    }

    stpCurrent->uwStatus |= DL_BIT;
    stpCurrent->stpNext   = &stpDesc[0];
}


void tfSH7616EthernetClose()
{
    unsigned long uwI;

    EDMAC.EDMR.LONG = 0x00000001;

    for (uwI = 0; uwI < 10000; uwI ++)    {    ;    }  /* delay */

    EC.ECMR.LONG    = 0x00000000;
}



    
 

int tfSetMaskLevel(int  iIntMask)
{
    int     erCode;
    int     iNowMask;


    iNowMask = get_imask();
    set_imask(iIntMask);

    return(iNowMask);
}

#pragma interrupt(tfSH7616EtherIntEntry)
void tfSH7616EtherIntEntry()
{
    int             erCode;
    unsigned long   uwRecvPktCnt;
    unsigned long   uwSendPktCnt;
    
    tvEther.uwEdmacEESR  = EDMAC.EESR.LONG;

    if ((tvEther.uwEdmacEESR & EESR_TC) != 0)
    {
/* Transmit completed */
        EDMAC.EESR.LONG |= EESR_TC;
        tvEther.uwEdmacTDFAR = (unsigned long)EDMAC.TDFAR;

        if(tvEther.uwEdmacTDFAR > tvEther.uwSaveEdmacTDFAR)
        {
            uwSendPktCnt = (tvEther.uwEdmacTDFAR - tvEther.uwSaveEdmacTDFAR) /
                sizeof(ttEdmacDesc);
        }
        else
        {
            uwSendPktCnt = DSC_COUNT -
                (tvEther.uwSaveEdmacTDFAR - tvEther.uwEdmacTDFAR) /
                sizeof(ttEdmacDesc);
        }

        tvEther.uwSaveEdmacTDFAR = tvEther.uwEdmacTDFAR;
        tvEther.uhIntStaTC++;
    }

    if((tvEther.uwEdmacEESR & EESR_FR) != 0)
    {
/* Frame received */
        EDMAC.EESR.LONG |= EESR_FR;
        tvEther.uwEdmacRDFAR = (unsigned long)EDMAC.RDFAR;

        if(tvEther.uwEdmacRDFAR == tvEther.uwSaveEdmacRDFAR)
        {
            tvEther.uhIntStaFR ++;
            EDMAC.EESR.LONG |= EESR_FR;
            return;
        }
        else
        {
            if(tvEther.uwEdmacRDFAR > tvEther.uwSaveEdmacRDFAR)
            {
                uwRecvPktCnt = (tvEther.uwEdmacRDFAR -
                                tvEther.uwSaveEdmacRDFAR)
                    / sizeof(ttEdmacDesc);
            }
            else
            {
                uwRecvPktCnt = DSC_COUNT -
                    (tvEther.uwSaveEdmacRDFAR - tvEther.uwEdmacRDFAR) /
                    sizeof(ttEdmacDesc);
            }
        }

        tvEther.uhRecvBufCount += (unsigned short)uwRecvPktCnt;
        tvEther.uwSaveEdmacRDFAR = tvEther.uwEdmacRDFAR;

        if(tvEther.uwOpenFlag == 1)
        {
            tfNotifyInterfaceIsr(tvEther.hnInterfaceHandle,
                                 uwRecvPktCnt,
                                 0,
                                 0,
                                 0);
        }

        tvEther.uhIntStaFR ++;
    }


    if (tvEther.uwEdmacEESR &
        (EESR_TRO|EESR_TDE|EESR_TFUF|EESR_ITF))
    {
/* Transmit error */
    }

    tfSH7616EtherEdmacStatusCheck();

}

void tfSH7616EtherEdmacStatusCheck()
{
    int  erCode;

    if ((tvEther.uwEdmacEESR & EESR_RFCOF) != 0)
    {
/* Receive frame counter overflow */
        tvEther.uhIntErrRFCOF ++;
        EDMAC.EESR.LONG |= EESR_RFCOF;
    }

    if((tvEther.uwEdmacEESR & EESR_ECI) != 0)
    {
/* EtherC status register interrupt */
        tvEther.uhIntErrECI ++;
        EDMAC.EESR.LONG |= EESR_ECI;
    }

    if((tvEther.uwEdmacEESR & EESR_TDE) != 0)
    {
/* Transmit descriptor exhausted */
        tvEther.uhIntErrTDE ++;
        EDMAC.EESR.LONG |= EESR_TDE;
    }

    if((tvEther.uwEdmacEESR & EESR_TFUF) != 0)
    {
/* Transmit FIFO underflow */
        tvEther.uhIntErrTFUF ++;
        EDMAC.EESR.LONG |= EESR_TFUF;
    }

    if((tvEther.uwEdmacEESR & EESR_RDE) != 0)
    {
/* Receive descriptor exhausted */
        tvEther.uhIntErrRDE ++;
        EDMAC.EESR.LONG |= EESR_RDE;
    }

    if((tvEther.uwEdmacEESR & EESR_RFOF) != 0)
    {
/* Receive FIFO overflow */
        tvEther.uhIntErrRFOF ++;
        EDMAC.EESR.LONG |= EESR_RFOF;
    }

    if((tvEther.uwEdmacEESR & EESR_ITF) != 0)
    {
/* illegal transmit frame */
        tvEther.uhIntErrITF ++;
        EDMAC.EESR.LONG |= EESR_ITF;
    }

    if((tvEther.uwEdmacEESR & EESR_CND) != 0)
    {
/* Carrier not detected */
        tvEther.uhIntErrCND ++;
        EDMAC.EESR.LONG |= EESR_CND;
    }

    if((tvEther.uwEdmacEESR & EESR_DLC) != 0)
    {
/* Lost of carrier detected */
        tvEther.uhIntErrDLC ++;
        EDMAC.EESR.LONG |= EESR_DLC;
    }


    if((tvEther.uwEdmacEESR & EESR_CD) != 0)
    {
/* Collision detected */
        tvEther.uhIntErrCD ++;
        EDMAC.EESR.LONG |= EESR_CD;
    }

    if((tvEther.uwEdmacEESR & EESR_TRO) != 0)
    {
/* Transmit retry over condition detected */
        tvEther.uhIntErrTRO ++;
        EDMAC.EESR.LONG |= EESR_TRO;
    }

    if((tvEther.uwEdmacEESR & EESR_RMAF) != 0)
    {
/* Receive Multicast address frame */
        tvEther.uhIntStaRMAF ++;
        EDMAC.EESR.LONG |= EESR_RMAF;
    }

    if((tvEther.uwEdmacEESR & EESR_RRF) != 0)
    {
/* Receive residual bit frame*/
        tvEther.uhIntErrRRF ++;
        EDMAC.EESR.LONG |= EESR_RRF;
    }

    if((tvEther.uwEdmacEESR & EESR_RTLF) != 0)
    {
 /* Receive too long frame */
        tvEther.uhIntErrRTLF ++;
        EDMAC.EESR.LONG |= EESR_RTLF;
    }

    if((tvEther.uwEdmacEESR & EESR_RTSF) != 0)
    {
/* Receive too short frame */
        tvEther.uhIntErrRTSF ++;
        EDMAC.EESR.LONG |= EESR_RTSF;
    }

    if((tvEther.uwEdmacEESR & EESR_PRE) != 0)
    {
/* PHY-LSI receive error */
        tvEther.uhIntErrPRE ++;
        EDMAC.EESR.LONG |= EESR_PRE;
    }

    if((tvEther.uwEdmacEESR & EESR_CERF) != 0)
    {
/* CRC error in received Frame */
        tvEther.uhIntErrCERF ++;
        EDMAC.EESR.LONG |= EESR_CERF;
    }
}




/* Functions to interface the device driver and the Treck stack */


int tfSH7616DriverOpen(ttUserInterface interfaceHandle)
{
    int erCode;
    int iMaskLevel;

    iMaskLevel = tfSetMaskLevel( KNL_MASK_LEVEL );

    tfSH7616EthernetOpen();

    tvEther.uwOpenFlag = 1;

    tfSetMaskLevel( iMaskLevel );

    return TM_DEV_OKAY;
}


int tfSH7616EthDriverClose(ttUserInterface interfaceHandle)
{
    int erCode;
    int iMaskLevel;

    iMaskLevel = tfSetMaskLevel( KNL_MASK_LEVEL );

    tvEther.uwOpenFlag = 0;

    tfSH7616EthernetClose( );

    tfSetMaskLevel( iMaskLevel );

    return TM_DEV_OKAY;
}



int tfSH7616ScatSend(ttUserInterface interfaceHandle,
                     ttUserPacketPtr packetUPtr)
{
    int errCode;
    char                  *srcRawBuf;  /* Src Raw Buffer (in the stack)    */
    char                  *dstRawBuf;  /* Dest Raw buffer (send descr      */
    volatile ttEdmacDesc  *curSndDesc; /* current send desc (written to)   */
    ttUserPacketPtr        subPktPtr;  /* current sub packet (read from)   */
    int                    subPktSize;
    int                    pktSize;
    unsigned long          i;
        
    curSndDesc = tvEther.uhSendDescPtr;
    dstRawBuf  = (char *)curSndDesc->ubpBuf;

    i = 0;

/* wait for our packet buffer to become available */
    while (((curSndDesc->uwStatus) & ACT ) != 0)
    {
/* Yield the CPU */
#ifdef TM_TASK_XMIT
        (void)tfInterfaceSpinLock(interfaceHandle);
#endif /* TM_TASK_XMIT */
        i++;

        if (i > 0x100000)
        {
            tm_sh7616_edmac_tr_set; /* kick the EDMAC just in case it is stuck */

/* If we have to wait for so long, there must be something wrong */
            errCode = TM_DEV_ERROR;
            goto tfSH7616ScatSendExit;
        }

     }

   pktSize = 0;
    for (subPktPtr = packetUPtr;
         subPktPtr;
         subPktPtr = (ttUserPacketPtr)subPktPtr->pktuLinkNextPtr)
    {
        subPktSize = subPktPtr->pktuLinkDataLength;
        pktSize += subPktSize; /* accumulate the total packet size */
        srcRawBuf  = (char *)subPktPtr->pktuLinkDataPtr;
/* Copy scattered piece in send descriptor buffer */
        tm_bcopy(srcRawBuf, dstRawBuf, subPktSize);
        dstRawBuf += subPktSize; /* offset for next scattered piece */
    }

    curSndDesc->uhBufSize = pktSize;

    tfKernelSetCritical();
    curSndDesc->uwStatus |= FP1|FP0|ACT; /* Release it */
/* Move to the next send descriptor */
    tvEther.uhSendDescPtr = curSndDesc->stpNext;

    tm_sh7616_edmac_tr_set; /* send pkts  */

    tvEther.uhSendBufCount++;
    tfKernelReleaseCritical();

/* tell the stack that we have transmitted this packet */
    tfSendCompletePacketInterface(interfaceHandle,
                                  packetUPtr,
                                  TM_DEV_SEND_COMPLETE_DRIVER);

   errCode = TM_DEV_OKAY;

tfSH7616ScatSendExit:
    return errCode;
}




int tfSH7616EthDriverRecv(ttUserInterface   interfaceHandle,
                          char TM_FAR     **ptrData,
                          int TM_FAR       *iRcvLen,
                          ttUserBufferPtr   bufHandlePtr)
{
    int errorCode;
    unsigned short status;
    volatile ttEdmacDesc  *curRcvDesc;

    curRcvDesc = tvEther.uhRecvDescPtr;

    status = curRcvDesc->uwStatus;
   
    if ((status & ACT) == 0)
    {
        errorCode = TM_DEV_OKAY;
        if (status & FE)
        {
            if (!(status & EESR_RMAF ) || 
                (tvEther.uwMultiCastFlag == MULTIC_OFF))
            {
                 errorCode = TM_DEV_ERROR;
                 
            }
        }
        if (errorCode == TM_DEV_OKAY)
        {
            *ptrData = tfGetEthernetBuffer(bufHandlePtr);

           tm_bcopy(curRcvDesc->ubpBuf,
                  *ptrData,
                   curRcvDesc->uhSize + 1);
           *iRcvLen = curRcvDesc->uhSize;
/* Release the descriptor */        
           tvEther.uhRecvDescPtr = curRcvDesc->stpNext;
           
        }
        curRcvDesc->uwStatus |= ACT;
        tm_sh7616_edmac_rr_set;
     }
     else
     {
        errorCode = TM_DEV_ERROR;
     }
     return errorCode;
}


int tfSH7616EthDriverIoctl(ttUserInterface  interfaceHandle,
                           int              flag,
                           void TM_FAR     *optionPtr,
                           int              optionLen)
{
    int erCode;

    if ((flag & TM_DEV_SET_MCAST_LIST) || (flag & TM_DEV_SET_ALL_MCAST))
    {
        if (   ((flag & TM_DEV_SET_MCAST_LIST) && optionLen != 0)
            || (flag & TM_DEV_SET_ALL_MCAST) )
        {
            tvEther.uwMultiCastFlag = MULTIC_ON;
        }
        else
        {
            tvEther.uwMultiCastFlag = MULTIC_OFF;
        }    
        erCode = TM_DEV_OKAY;
    }
    else
    {
        erCode = TM_DEV_OKAY;
    }

    return erCode;
}

int tfSH7616EthDriverGetPhysAddr(ttUserInterface    interfaceHandle,
                                 char TM_FAR       *physicalAddress)
{
    int             erCode;
    unsigned char  *ubpTemp;

    union {
        unsigned long            uwAddr;
        unsigned char            ubAddr[4];
    } stMac;


    ubpTemp = (unsigned char *)physicalAddress;

#ifdef TM_BIG_ENDIAN
    stMac.uwAddr = tvEther.uwMacAddrHigh;

    ubpTemp[0] = (unsigned char)( stMac.ubAddr[0] );
    ubpTemp[1] = (unsigned char)( stMac.ubAddr[1] );
    ubpTemp[2] = (unsigned char)( stMac.ubAddr[2] );
    ubpTemp[3] = (unsigned char)( stMac.ubAddr[3] );
    stMac.uwAddr = tvEther.uwMacAddrLow;
    ubpTemp[4] = (unsigned char)( stMac.ubAddr[2] );
    ubpTemp[5] = (unsigned char)( stMac.ubAddr[3] );
#else /* TM_BIG_ENDIAN */
    stMac.uwAddr = tvEther.uwMacAddrHigh;
    ubpTemp[0] = (unsigned char)( stMac.ubAddr[3] );
    ubpTemp[1] = (unsigned char)( stMac.ubAddr[2] );
    ubpTemp[2] = (unsigned char)( stMac.ubAddr[1] );
    ubpTemp[3] = (unsigned char)( stMac.ubAddr[0] );
    stMac.uwAddr = tvEther.uwMacAddrLow;
    ubpTemp[4] = (unsigned char)( stMac.ubAddr[1] );
    ubpTemp[5] = (unsigned char)( stMac.ubAddr[0] );
#endif /* !TM_BIG_ENDIAN    */

    return TM_DEV_OKAY;
}


int tfSH7617DriverGetCtrlInfo(ttUserInterface  interfaceHandle,
                              ttSH7616Ether   *stpEther)
{
    int erCode;

    tfKernelSetCritical();
    tm_memcpy(stpEther, &tvEther, sizeof(ttSH7616Ether));
    tm_memset(&tvEther.uhIntErrRFCOF, 0x00,
              ((unsigned long)&tvEther.uhReserved - 
               (unsigned long)&tvEther.uhIntErrRFCOF));
    tfKernelReleaseCritical();

    return TM_DEV_OKAY;
}


int tfSH7616EthDriverPromiscuousMode(ttUserInterface interfaceHandle,
                                     int             mode)
{
    int erCode;

    if( mode == PROMISC_ON )
    {
        EC.ECMR.BIT.PRM = 1;
    }
    else
    {
        EC.ECMR.BIT.PRM = 0;
    }

    return TM_DEV_OKAY;
}



/* Physical layer (MII interface) initialization routine and definition */


/* Format of PHY register address:
 *
 *  example of ANAR (offset 04)
 *
 *    ST OP PHYADDR REFADDR  TA
 *    01 01  00001   00100   10  = 0x5092
 */
/* address to set registers */
#define BMCR        0x5082  /* Basic Mode Control Register                  */
#define ANAR        0x5092  /* Auto-Negotiation Advertsement Register       */
#define ANLPAR      0x5096  /* Auto-Negotiation Link Partner Ability Reg    */
#define ANER        0x509a  /* Auto-Negotiation Expansion Register          */
#define ANNPTR      0x509e  /* Auto-Negotiation Next Page TX                */
#define MIPSCR      0x50c6  /* MII Interrupt PHY Specific Control Register  */
#define DCR         0x50ce  /* Disconnect Counter Register                  */
#define FCSCR       0x50d2  /* False Carrier Sense Counter Register         */
#define RECR        0x50d6  /* Receive Error Counter Register               */
#define PCSR        0x50da  /* PCS Sub-Layer Config and Status Register     */
#define LBR         0x50de  /* Loopback and Bypass Register                 */
#define BTSCR       0x50e2  /* 10BASE-T Status & Control Register           */
#define PHYCTRL     0x50e6  /* PHY Control Register                         */

/* address to read registers */
#define R_BMCR      0x6082  /*Basic Mode Control Register                   */
#define R_BMSR      0x6086 /*Basic Mode Status Register                     */
#define R_ANAR      0x6092 /*Auto-Negotiation Advertsement Register         */
#define R_ANLPAR    0x6096 /*Auto-Negotiation Link Partner Ability Register */
#define R_ANER      0x609a /*Auto-Negotiation Expansion Register            */
#define R_ANNPTR    0x609e /*Auto-Negotiation Next Page TX                  */
#define R_PHYSTS    0x60c2 /*PHY Status Register                            */
#define R_MIPSCR    0x60c6 /*MII Interrupt PHY Specific Control Register    */
#define R_DCR       0x60ce /*Disconnect Counter Register                    */
#define R_FCSCR     0x60d2 /*False Carrier Sense Counter Register           */
#define R_RECR      0x60d6 /*Receive Error Counter Register                 */
#define R_PCSR      0x60da /*PCS Sub-Layer Config and Status Register       */
#define R_LBR       0x60de /*Loopback and Bypass Register                   */
#define R_BTSCR     0x60e2 /*10BASE-T Status & Control Register             */
#define R_PHYCTRL   0x60e6 /*PHY Control Register                           */


/* register values */
#define D_BMCR     0x0000
#define D_ANAR     0x0021
#define D_ANLPAR   0x0000
#define D_ANER     0x0000
#define D_ANNPTR   0x3801
#define D_MIPSCR   0x0002
#define D_DCR      0x0000
#define D_FCSCR    0x0000
#define D_RECR     0x0000
#define D_PCSR     0x0000
#define D_LBR      0x0000
#define D_10BTSCR  0x1880

#define D_PHYCTRL  0x0801


/* DP83843 functions */
void DP83843_init(void)
{

        DP83843_reset();         /* DP83843 reset */

        write_PHY(0xffff);       /* Write BMCR(PHY_Register) */
        write_PHY(0xffff);
        write_PHY(BMCR);
        write_PHY(D_BMCR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);       /* Write ANAR(PHY_Register)*/
        write_PHY(0xffff);
        write_PHY(ANAR);
        write_PHY(D_ANAR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(ANLPAR);
        write_PHY(D_ANLPAR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(ANER);
        write_PHY(D_ANER);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(ANNPTR);
        write_PHY(D_ANNPTR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(MIPSCR);
        write_PHY(D_MIPSCR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(DCR);
        write_PHY(D_DCR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(FCSCR);
        write_PHY(D_FCSCR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(RECR);
        write_PHY(D_RECR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(PCSR);
        write_PHY(D_PCSR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(LBR);
        write_PHY(D_LBR);
        EC.PIR.BIT.MDC=0;

        write_PHY(0xffff);
        write_PHY(0xffff);
        write_PHY(BTSCR);
        write_PHY(D_10BTSCR);
        EC.PIR.BIT.MDC=0;

}

void DP83843_reset()
{
    write_PHY(0xffff);
    write_PHY(0xffff);
    write_PHY(BMCR);
    write_PHY(0x8000);                /* Initial Software Reset */    
}

static void write_PHY(unsigned short wdata)
{
	int i;
	for (i=0; i<16; i++)
    {
		if (wdata & 0x8000)
        {
			write_1();
        }
		else
        {
			write_0();
        }
		wdata<<1;
	}
}

static void write_1()
{
    EC.PIR.LONG=0x00000006;
    EC.PIR.BIT.MDC=1;
    EC.PIR.BIT.MDC=0;
}

static void write_0()
{
    EC.PIR.LONG=0x00000002;
    EC.PIR.BIT.MDC=1;
    EC.PIR.BIT.MDC=0;
}

static void out_hiz()
{
    EC.PIR.LONG=0;
    EC.PIR.BIT.MDC=1;
    EC.PIR.LONG=0;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: tr7616e.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 16570    Output checksum:  1355
 */
