/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: SMC9000 Device Driver modified for chip SMSC's LAN91C113I
 *              Infineon C161P processor, Keil uVision2 IDE. CMX RTOS
 *
 * Filename: trsmcc16.c
 * Author: Jason (original), Bryan Wang (modified)
 * Date Created: 10/9/2000
 * $Source: source/drivers/smc/trsmc113.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:25CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifndef	trSmc113_h
#define	trSmc113_h

/*
 * Procedure : tfSmc113InWord, tfSmc113InWord
 *             Read Byte/Word from the SMSC113 at a given register.
 * Parameter : reg  - Offset added to the SMSC memory base
 * Returns   : Byte/Word
 */

unsigned char tfSmc113InByte (unsigned int reg);
unsigned int  tfSmc113InWord (unsigned int reg);

/*
 * Procedure : tfSmc113OutByte, tfSmc113OutWord
 *           : Write Byte/Word to the SMSC113 at a given register.
 * Parameter : reg  - Offset added to the SMSC memory base
 *             data - Data to be written
 * Returns   : None
 */
void tfSmc113OutByte(unsigned int reg, unsigned char data);
void tfSmc113OutWord(unsigned int reg, unsigned int  data);

/*
 * Procedure : tfSmc113RcvData and tfSmc113SndData
 *           : Provides a high speed transfer from/to the SMSC113 data
 *             register (Data transfer register is located at Base + 8)
 * Parameter : dataPtr - Huge pointer to data to transfer from/into.
 *             len    - Length of transfer
 * Returns   : Byte/Word
 */
void tfSmc113RcvData (unsigned char huge * dataPtr, unsigned int len);
void tfSmc113SndData (unsigned char huge * dataPtr, unsigned int len);

/*
 * This routine does the same with tfSmc113SndData except     
 * it does not send the length and the final odd byte, it is     
 * meant to be called multiple times to send the even chunks  of 
 * scattered packets
 */
 void tfSmc113SndScatData (unsigned char huge * dataPtr, unsigned int len);

/*
 * Procedure : tfSmc113CheckSum
 *           : Provides a high speed checksum routine (RFC 1071).
 * Parameter : startCs  - Starting CRC value, allows multiple calls
 *             dataPtr - Huge pointer to data to calc checksum
 *             len    - Length of data.
 * Returns   : WORD     - CheckSum
 */
unsigned int tfSmc113CheckSum (unsigned int startCs,unsigned char huge * dataPtr, unsigned int len);

#endif /* trSmc113_h */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: trsmc113.h    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 16182    Output checksum: 20725
 */
