;/*
; * Copyright Notice:
; * Copyright Treck Incorporated  1997 - 2004
; * Copyright Elmic Systems Japan 1997 - 2004
; * No portions or the complete contents of this file may be copied or
; * distributed in any form (including but not limited to printed or electronic
; * forms) without the expressed written consent of Treck Incorporated OR
; * Elmic Systems.  Copyright laws and International Treaties protect the
; * contents of this file.  Unauthorized use is prohibited.
; * All rights reserved.
; *
; * Description: SMC9000 Device Driver modified for chip SMSC's LAN91C113I
; *              Infineon C161P processor, Keil uVision2 IDE. CMX RTOS
; *
; * Filename: trsmcc16.c
; * Author: Jason (original), Bryan Wang (modified)
; * Date Created: 10/9/2000
; * $Source: source/drivers/smc/trsmc113.inc $
; *
; * Modification History
; * $Revision: 6.0.2.2 $
; * $Date: 2010/01/18 06:36:28CST $
; * $Author: odile $
; * $ProjectName: /home/mks/proj/tcpip.pj $
; * $ProjectRevision: 6.0.1.23 $
; */

;******************************************************************************
;* System Equates                                                             *
;******************************************************************************

BASE_FREQ		equ	5000000 			;  drive clock frequency * 4 through PLL
CLOCK_FREQ		equ	20000000			;  clock frequency
								; ***** Verbage for maintenance *****
MS_PER_SECOND     	equ	1000				; Number of the miliseconds in one second 
uS_PER_SECOND     	equ	1000000				; Number of the microseconds in one second
NS_PER_SECOND     	equ	1000000000			; Number of the nanoseconds in one second

;******************************************************************************
;* Display Equates                                                            *
;******************************************************************************

DISPLAY_FRAMES_PER_SEC	equ	24				; Frames Per Second 
DISPLAY_DUTY		equ	16				; 1/16 Duty Cycle *MUST BE A POWER OF 2* (ScreenRender Code)
DISPLAY_WIDTH		equ	240				; Current Width
DISPLAY_HEIGHT		equ	32				; Current Depth 
%set	 		(DISPLAY_SEGS, 2)			; Total number of Row Segments (Must be equal ...
								;  ... to DISPLAY_HEIGHT / DISPLAY_DUTY )
;******************************************************************************
;* Syscon Equates                                                             *
;*******************************************************************************

XBUS_SHARE		equ     (1b SHL 0)
XBUS_VIS		equ     (1b SHL 1)
XBUS_EN			equ     (1b SHL 2)
BIDIR_RESET		equ     (1b SHL 3)
OSC_WATCHDOG		equ     (1b SHL 4)
CS_LATCH		equ	(1b SHL 6)
WR_HWRL			equ	(1b SHL 7)
CLK_EN			equ	(1b SHL 8)
BHE_DIS			equ	(1b SHL 9)
ROM_EN			equ	(1b SHL 10)
SEG_DIS			equ	(1b SHL 11)
ROM_SEG1		equ	(1b SHL 12)

;*****************************************************************************
;* Stack Size Equates                                                        *
;*****************************************************************************

STK_SIZE_32		equ	(011b SHL 13)
STK_SIZE_64		equ     (010b SHL 13)
STK_SIZE_128		equ	(001b SHL 13)
STK_SIZE_256		equ	(000b SHL 13)
STK_SIZE_512		equ	(100b SHL 13)
STK_SIZE_1024		equ	(111b SHL 13)

;******************************************************************************
;* BusCon Equates                                                             *
;******************************************************************************

NO_WAIT_STATE		equ	15
MCT_0			equ	NO_WAIT_STATE
MCT_1			equ	14
MCT_2			equ	13
MCT_3			equ	12
MCT_4			equ	11
MCT_5			equ	10
MCT_6			equ	9
MCT_7			equ	8
MCT_8			equ	7
MCT_9			equ	6
MCT_10			equ	5
MCT_11			equ	4
MCT_12			equ	3
MCT_13			equ	2
MCT_14			equ	1
MCT_15			equ	0
	
NO_RW_DELAY		equ	(1b SHL 4)
NO_TRISTATE		equ	(1b SHL 5)
BUS_MLTPLEXED		equ	(1b SHL 6)
BUS_SIZE_16		equ	(1b SHL 7)
LONG_ALE		equ	(1b SHL 9)
BUS_ACTIVE		equ	(1b SHL 10)
RDY_ACTIVE		equ	(1b SHL 12)
READ_CSEN		equ	(1b SHL 14)
WRITE_CSEN		equ	(1b SHL 15)
	
RGSZ_4K			equ	0000b
RGSZ_8K			equ	0001b
RGSZ_16K		equ	0010b
RGSZ_32K		equ	0011b
RGSZ_64K		equ	0100b
RGSZ_128K		equ	0101b
RGSZ_256K		equ	0110b
RGSZ_512K		equ	0111b
RGSZ_1M			equ	1000b
RGSZ_2M			equ	1001b
RGSZ_4M			equ	1011b
RGSZ_8M			equ	1100b

;******************************************************************************
;* Serial Control Equates                                                     *
;******************************************************************************

CHECK_PARITY		equ	(1b SHL 5)			; Check for Parity errors
			
A8_BIT_SYNC		equ	000b				; 8 Bit Synchronous
A8_BIT_NO_PARITY	equ	001b				; 8 Bit Asynchronous
A7_BIT_W_PARITY		equ	(011b |	CHECK_PARITY)
								; 7 Bit Asynchronous + Parity
A9_BIT_NO_PAR		equ	100b				; 9 Bit Asynchronous
A8_BIT_WAKEUP		equ	101b				; 8 Bit Asynchronous + Wakeup Bit
A8_BIT_PARITY		equ	(111b | CHECK_PARITY)
								; 8 Bit Asynchronous + Parity Bit
STOP_BIT_1		equ	(0b SHL 3)			; Select 1 Stop Bit
STOP_BIT_2		equ	(1b SHL 3)			; Select 2 Stop Bits
ODD_PARITY		equ	(0b SHL 12)			; Select Odd parity
EVEN_PARITY		equ	(1b SHL 12)			; Select Even parity
			
RECEIVER_ENABLE		equ	(1b SHL 4)			; Enable Serial Receiver
CHECK_FRAMING		equ	(1b SHL 6)			; Check for Framing errors
CHECK_OVERRUN		equ	(1b SHL 7)			; Check for Overrun errors
			
RTS_CTS			equ	(1b SHL 11)			; Enable RTS/CTS flow control
LOOPBACK		equ	(1b SHL 14)			; Enable loop back test
BAUD_GEN_RUN		equ	(1b SHL 15)			; Enable buad rate generator to run
BAUD_GEN_DIV		equ	(1b SHL 13)			; Divide baud rate by 2/3
			
BAUD_DIVISOR		equ	(CLOCK_FREQ / 16)		; Baud divisor for reload calculation
			
FRAME_ERR		equ	S0FE				; Error flags
OVERRUN_ERR		equ	S0OE
PARITY_ERR		equ	S0PE
	
;Designator			Reload value			   Deviation (in percents)
	
BAUD_110		equ	5681				;	0.0  	Baudrate reload values calculated @ 20Mhz
BAUD_300		equ	2083				;	0.0    using a 40Mhz clock source (165 divides by 2)
BAUD_600		equ	1041				;	0.0
BAUD_1P2K		equ	520				;       0.0
BAUD_2P4K		equ	260				;       0.2
BAUD_4P8K		equ	131     			;	0.2
BAUD_9P6K		equ	64      			;	0.2
BAUD_14P4K		equ	42      			;	0.9
BAUD_19P2K		equ	32      			;      -1.4
BAUD_38P4K		equ	15      			; 	1.7
BAUD_56K		equ	10      			; 	1.4
BAUD_57P6K		equ	10     				;      -1.4
BAUD_115P2K		equ	4      				; 	7.8
BAUD_128K		equ	4      				;      -2.4
BAUD_256K		equ	1       			;      18.1

;******************************************************************************
;* Interupt Vector Locations                                                  *
;******************************************************************************

ISRVECT_IP_PIN1		equ	060h				; ISR Vector to ISR InputISR_PIN1() located in InputPortISR.a66
ISRVECT_IP_PIN2		equ	064h				; ISR Vector to ISR InputISR_PIN2() located in InputPortISR.a66
ISRVECT_IP_PIN3		equ	068h				; ISR Vector to ISR InputISR_PIN3() located in InputPortISR.a66
ISRVECT_IP_PIN4		equ	06Ch				; ISR Vector to ISR InputISR_PIN4() located in InputPortISR.a66
ISRVECT_IP_PIN5		equ	070h				; ISR Vector to ISR InputISR_PIN5() located in InputPortISR.a66
ISRVECT_IP_PIN6		equ	074h				; ISR Vector to ISR InputISR_PIN6() located in InputPortISR.a66

ISRVECT_IP_PIN7		equ	070h				; External Interupt 6 (not implemented)
ISRVECT_IP_PIN8		equ	074h				; External Interupt 7 (not implemented)
			
T2_INT			equ	088h				; GPT 1 Timer 2
T3_INT			equ	08Ch            		; GPT 1 Timer 3
T4_INT			equ	090h            		; GPT 1 Timer 4
T5_INT			equ	094h            		; GPT 2 Timer 5
T6_INT			equ	098h            		; GPT 2 Timer 6
CR_INT			equ	09Ch				; GPT 2 CapRel
ISRVECT_FPGA		equ	CR_INT				; ISR Vector to ISR FPGAIsr() located in AlphaDriver.a66
			
A2D_COMP_INT		equ	0A0h				; A2D Complete
ISRVECT_IP1_EXEC	equ 	A2D_COMP_INT			; ISR Vector to ISR IP1Execute() located in InputPortISR.a66

A2D_ERROR_INT		equ	0A4h				; A2D Overrun error
ISRVECT_CMX_RECHEDULE	equ	A2D_ERROR_INT			; ISR Vector to ISR CMXDeferSchedule() located in CXSKV5M.SRC 
								; Define our interrupt that is going to be used to force CMX
								; to do a reschedule

S0T_INT			equ	0A8h				; Async Transmit
S0TB_INT		equ     11Ch				; Async Buffer Empty
S0R_INT          	equ     0ACh				; Async Receive
S0E_INT          	equ     0B0h				; Async Error
			
SCT_INT			equ	0B4h				; Sync Transmit
SCR_INT          	equ     0B8h				; Sync Receive
SCE_INT          	equ     0BCh				; Sync Error
			
I2C_TRANS_INT		equ	100h				; I2C data transfer
I2C_PROT_INT		equ	100h				; I2C Protocol error
			
PLL_RTC_INT		equ	10Ch				; RTC/PLLOWD via ISNC
ISRVECT_DIG_DISPLAY	equ	PLL_RTC_INT			; ISR Vector to ISR  DisplayISR () located in DisplayDriver.a66
			
RESET_INT		equ	000h				; Reset Vector
NMI_INT			equ	008h				; NMI Vector
ST_OT_INT		equ	010h				; Stack overflow Vector
ST_UT_INT		equ	018h				; Stack underflow vector
B_TRAP_INT		equ	028h				; B Level Trap

;******************************************************************************
;* Interupt Priority Levels - As required by Interupt Control Registers       *
;******************************************************************************

ILVL0			equ	(0 SHL 2)			; Priority Level - 0	Lowest
ILVL1			equ	(1 SHL 2)			; Priority Level - 1
ILVL2			equ	(2 SHL 2)			; Priority Level - 2
ILVL3			equ	(3 SHL 2)			; Priority Level - 3
ILVL4			equ	(4 SHL 2)			; Priority Level - 4
ILVL5			equ	(5 SHL 2)			; Priority Level - 5
ILVL6			equ	(6 SHL 2)			; Priority Level - 6
ILVL7			equ	(7 SHL 2)			; Priority Level - 7
ILVL8			equ	(8 SHL 2)			; Priority Level - 8
ILVL9			equ	(9 SHL 2)			; Priority Level - 9
ILVL10			equ	(10 SHL 2)			; Priority Level - 10
ILVL11			equ	(11 SHL 2)			; Priority Level - 11
ILVL12			equ	(12 SHL 2)			; Priority Level - 12
ILVL13			equ	(13 SHL 2)			; Priority Level - 13
ILVL14			equ	(14 SHL 2)			; Priority Level - 14
ILVL15			equ	(15 SHL 2)			; Priority Level - 15	Highest

;******************************************************************************
;* Interupt Group Levels                                                      *
;******************************************************************************

GLVL0			equ	0				; Group 0 Priority	Lowest
GLVL1			equ	1				; Group 1 Priority
GLVL2			equ	2				; Group 2 Priority
GLVL3			equ	3				; Group 3 Priority	Highest

;******************************************************************************
;* Interupt Control Codes                                                     *
;******************************************************************************

INT_EN			equ	(1 SHL 6) 			; Enable Interupt
INT_REQ			equ	(1 SHL 7) 			; Interupt Request Flag

;******************************************************************************
;* PEC Control Codes                                                          *
;******************************************************************************

PEC_BYTE_TRAN		equ	00100000000b			; Transfer a BYTE
PEC_WORD_TRAN		equ	00000000000b			; Transfer a BYTE
PEC_INC_NONE		equ	00000000000b			; No pointer increment
PEC_INC_DEST		equ	01000000000b			; Increment Destination pointer
PEC_INC_SRC		equ	10000000000b			; Increment Source pointer

;******************************************************************************
;* PEC Channel Select Codes                                                   *
;******************************************************************************

PEC_CHANNEL_0		equ	(ILVL14 | GLVL0)		; Select Channel 0 * Lowest  Priority *
PEC_CHANNEL_1		equ	(ILVL14 | GLVL1)		; Select Channel 1	   7
PEC_CHANNEL_2		equ	(ILVL14 | GLVL2)		; Select Channel 2	   6
PEC_CHANNEL_3		equ	(ILVL14 | GLVL3)		; Select Channel 3	   5
PEC_CHANNEL_4		equ	(ILVL15 | GLVL0)		; Select Channel 4	   4
PEC_CHANNEL_5		equ	(ILVL15 | GLVL1)		; Select Channel 5	   3
PEC_CHANNEL_6		equ	(ILVL15 | GLVL2)		; Select Channel 6	   2
PEC_CHANNEL_7		equ	(ILVL15 | GLVL3)		; Select Channel 7 * Highest Priority *

;******************************************************************************
;* Fast Interupt Control Codes                                                *
;******************************************************************************

INT0_DIS		equ	(00b SHL 00)
INT0_EDGE_POS		equ	(01b SHL 00)
INT0_EDGE_NEG		equ	(10b SHL 00)
INT0_EDGE_ANY		equ	(11b SHL 00)
INT0_MASK		equ	INT0_EDGE_ANY
	
INT1_DIS		equ	(00b SHL 02)
INT1_EDGE_POS		equ	(01b SHL 02)
INT1_EDGE_NEG		equ	(10b SHL 02)
INT1_EDGE_ANY		equ	(11b SHL 02)
INT1_MASK		equ	INT1_EDGE_ANY
	
INT2_DIS		equ	(00b SHL 04)
INT2_EDGE_POS		equ	(01b SHL 04)
INT2_EDGE_NEG		equ	(10b SHL 04)
INT2_EDGE_ANY		equ	(11b SHL 04)
INT2_MASK		equ	INT2_EDGE_ANY
	
INT3_DIS		equ	(00b SHL 06)
INT3_EDGE_POS		equ	(01b SHL 06)
INT3_EDGE_NEG		equ	(10b SHL 06)
INT3_EDGE_ANY		equ	(11b SHL 06)
INT3_MASK		equ	INT3_EDGE_ANY
	
INT4_DIS		equ	(00b SHL 08)
INT4_EDGE_POS		equ	(01b SHL 08)
INT4_EDGE_NEG		equ	(10b SHL 08)
INT4_EDGE_ANY		equ	(11b SHL 08)
INT4_MASK		equ	INT4_EDGE_ANY
	
INT5_DIS		equ	(00b SHL 10)
INT5_EDGE_POS		equ	(01b SHL 10)
INT5_EDGE_NEG		equ	(10b SHL 10)
INT5_EDGE_ANY		equ	(11b SHL 10)
INT5_MASK		equ	INT5_EDGE_ANY
	
INT6_DIS		equ	(00b SHL 12)
INT6_EDGE_POS		equ	(01b SHL 12)
INT6_EDGE_NEG		equ	(10b SHL 12)
INT6_EDGE_ANY		equ	(11b SHL 12)
INT6_MASK		equ	INT6_EDGE_ANY
	
INT7_DIS		equ	(00b SHL 14)
INT7_EDGE_POS		equ	(01b SHL 14)
INT7_EDGE_NEG		equ	(10b SHL 14)
INT7_EDGE_ANY		equ	(11b SHL 14)
INT7_MASK		equ	INT7_EDGE_ANY
	
CAP_INT_DIS		equ	(00b SHL 12)
CAP_INT_EDGE_POS	equ	(01b SHL 12)
CAP_INT_EDGE_NEG	equ	(10b SHL 12)
CAP_INT_EDGE_ANY	equ	(11b SHL 12)
CAP_INT_MASK		equ	CAP_INT_EDGE_ANY

;******************************************************************************
;* Timer Control Codes                                                        *
;******************************************************************************

GPT1_PRESCALE_8		equ	(000b SHL 0)
GPT1_PRESCALE_16	equ	(001b SHL 0)
GPT1_PRESCALE_32	equ	(010b SHL 0)
GPT1_PRESCALE_64	equ	(011b SHL 0)
GPT1_PRESCALE_128	equ	(100b SHL 0)
GPT1_PRESCALE_256	equ	(101b SHL 0)
GPT1_PRESCALE_512	equ	(110b SHL 0)
GPT1_PRESCALE_1024	equ	(111b SHL 0)

GPT2_PRESCALE_4		equ	(000b SHL 0)
GPT2_PRESCALE_8		equ	(001b SHL 0)
GPT2_PRESCALE_16	equ	(010b SHL 0)
GPT2_PRESCALE_32	equ	(011b SHL 0)
GPT2_PRESCALE_64	equ	(100b SHL 0)
GPT2_PRESCALE_128	equ	(101b SHL 0)
GPT2_PRESCALE_256	equ	(110b SHL 0)
GPT2_PRESCALE_512	equ	(111b SHL 0)

T2_EDGE_SEL_NONE	equ	(000b SHL 0)
T2_EDGE_SEL_POS		equ	(001b SHL 0)
T2_EDGE_SEL_NEG		equ	(010b SHL 0)
T2_EDGE_SEL_ANY		equ	(011b SHL 0)
T2_POS_T3OTL		equ	(101b SHL 0)
T2_NEG_T3OTL		equ	(110b SHL 0)
T2_ANY_T3OTL		equ	(111b SHL 0)
	
T2_TIMER_MODE		equ	(000b SHL 3)
T2_CNT_MODE		equ	(001b SHL 3)
T2_GATED_ACT_LOW	equ	(010b SHL 3)
T2_GATED_ACT_HIGH	equ	(011b SHL 3)
T2_RLD_MODE		equ	(100b SHL 3)
T2_CAPTURE_MODE		equ	(101b SHL 3)
	
T2_RUN			equ	(1b SHL 6)
T2_CNT_UP		equ	0
T2_CNT_DOWN		equ	(1b SHL 7)
T2_EXT_UP_DOWN_EN	equ	(1b SHL 8)
	
T3_EDGE_SEL_NONE	equ	T2_EDGE_SEL_NONE
T3_EDGE_SEL_POS		equ	T2_EDGE_SEL_POS
T3_EDGE_SEL_NEG		equ	T2_EDGE_SEL_NEG
T3_EDGE_SEL_ANY		equ	T2_EDGE_SEL_ANY
	
T3_TIMER_MODE		equ	T2_TIMER_MODE
T3_CNT_MODE		equ	T2_CNT_MODE
T3_GATED_ACT_LOW	equ	T2_GATED_ACT_LOW
T3_GATED_ACT_HIGH	equ	T2_GATED_ACT_HIGH
	
T3_RUN			equ	T2_RUN
T3_CNT_UP		equ     T2_CNT_UP
T3_CNT_DOWN		equ	T2_CNT_DOWN
T3_EXT_UP_DOWN_EN	equ	T2_EXT_UP_DOWN_EN
T3_ALT_OUT_EN		equ	(1b SHL 9)
T3_OUT_TOGGLE		equ     (1b SHL 10)
	
T4_EDGE_SEL_NONE	equ	T2_EDGE_SEL_NONE
T4_EDGE_SEL_POS		equ	T2_EDGE_SEL_POS
T4_EDGE_SEL_NEG		equ	T2_EDGE_SEL_NEG
T4_EDGE_SEL_ANY		equ	T2_EDGE_SEL_ANY
T4_POS_T3OTL		equ	T2_POS_T3OTL
T4_NEG_T3OTL		equ	T2_NEG_T3OTL
T4_ANY_T3OTL		equ	T2_ANY_T3OTL

T4_TIMER_MODE		equ	T2_TIMER_MODE
T4_CNT_MODE		equ	T2_CNT_MODE
T4_GATED_ACT_LOW	equ	T2_GATED_ACT_LOW
T4_GATED_ACT_HIGH	equ	T2_GATED_ACT_HIGH
T4_RLD_MODE		equ	T2_RLD_MODE
T4_CAPTURE_MODE		equ	T2_CAPTURE_MODE

T4_RUN			equ	T2_RUN
T4_CNT_UP		equ     T2_CNT_UP
T4_CNT_DOWN		equ	T2_CNT_DOWN
T4_EXT_UP_DOWN_EN	equ	T2_EXT_UP_DOWN_EN
	
T5_EDGE_SEL_NONE	equ	T2_EDGE_SEL_NONE
T5_EDGE_SEL_POS		equ	T2_EDGE_SEL_POS
T5_EDGE_SEL_NEG		equ	T2_EDGE_SEL_NEG
T5_EDGE_SEL_ANY		equ	T2_EDGE_SEL_ANY
T5_POS_T6OTL		equ	T2_POS_T3OTL
T5_NEG_T6OTL		equ	T2_NEG_T3OTL
T5_ANY_T6OTL		equ	T2_ANY_T3OTL

T5_TIMER_MODE		equ	T2_TIMER_MODE
T5_CNT_MODE		equ	T2_CNT_MODE
T5_GATED_ACT_LOW	equ	T2_GATED_ACT_LOW
T5_GATED_ACT_HIGH	equ	T2_GATED_ACT_HIGH

T5_RUN			equ	T2_RUN
T5_CNT_UP		equ     T2_CNT_UP
T5_CNT_DOWN		equ	T2_CNT_DOWN
T5_EXT_UP_DOWN_EN	equ	T2_EXT_UP_DOWN_EN

T5_POS_CAPTURE		equ	(01b SHL 12)
T5_NEG_CAPTURE		equ	(10b SHL 12)
T5_ANY_CAPTURE		equ	(11b SHL 12)
	
T5_CLR_CAPTURE		equ	(1b SHL 14)
	
T6_EDGE_SEL_NONE	equ	T2_EDGE_SEL_NONE
T6_EDGE_SEL_POS		equ	T2_EDGE_SEL_POS
T6_EDGE_SEL_NEG		equ	T2_EDGE_SEL_NEG
T6_EDGE_SEL_ANY		equ	T2_EDGE_SEL_ANY
	
T6_TIMER_MODE		equ	T2_TIMER_MODE
T6_CNT_MODE		equ	T2_CNT_MODE
T6_GATED_ACT_LOW	equ	T2_GATED_ACT_LOW
T6_GATED_ACT_HIGH	equ	T2_GATED_ACT_HIGH
	
T6_RUN			equ	T2_RUN
T6_CNT_UP		equ     T2_CNT_UP
T6_CNT_DOWN		equ	T2_CNT_DOWN
T6_EXT_UP_DOWN_EN	equ	T2_EXT_UP_DOWN_EN


T6_ALT_OUT_EN		equ	T3_ALT_OUT_EN
T6_OUT_TOGGLE		equ	T3_OUT_TOGGLE
T6_CAP_RLD		equ	(1b SHL 15)

%*define (CalcTimerValMs(PreScale,TimeMs))	((((CLOCK_FREQ / (2 * (2 SHL %PreScale))) * %TimeMs) / 1000))
%*define (CalcTimerValUSec(PreScale,TimeUSec))	((((CLOCK_FREQ / 10 / (2 * (2 SHL %PreScale))) * %TimeUSec) / 100000))
%*define (CalcTimerValNSec(PreScale,TimeNSec))	((((CLOCK_FREQ / 100 / (2 * (2 SHL %PreScale))) * %TimeNSec) / 10000000))

%*define (CalcTimerResNSec(PreScale))		((((8 SHL %PreScale) * 1000) / (CLOCK_FREQ / 1000000)))

T14ResNSec      equ                             (256000000000 / BASE_FREQ)
%*define (CalcT14Val(TimeUSec)) 		(0 - ((%TimeUSec * 1000) / T14ResNSec))

;******************************************************************************
;* Memory Mapped device address's and BUSCON setup.                           *
;******************************************************************************

DATA_FLASH_ADDR_SZ	equ	RGSZ_4M				; 4Meg Address Window
DATA_FLASH_ADDR_LOC	equ	8192 * 1024			; Starting at address 8192K (800000h)
DATA_FLASH_ADDR_SEL	lit	'ADDRSEL1'			; Data Flash attached to CS1
DATA_FLASH_ADDR_VAL	equ	(((DATA_FLASH_ADDR_LOC / 4096) << 4) + DATA_FLASH_ADDR_SZ)
DATA_FLASH_ADDR_DPP	equ	(DATA_FLASH_ADDR_LOC / 16384)
DATA_FLASH_ADDR_OFS	equ	(DATA_FLASH_ADDR_LOC & 16383)
DATA_FLASH_MCT		equ	NO_WAIT_STATE
DATAF_LASH_BUSCON	lit	'BUSCON1'
DATAF_LASH_BUSCON_VAL	equ	(BUS_ACTIVE | DATA_FLASH_MCT | NO_TRISTATE | NO_RW_DELAY)

MSG_FLASH_ADDR_SZ	equ	RGSZ_4M				; 4Meg Address Window
MSG_FLASH_ADDR_LOC	equ	4096 * 1024			; Starting at address 4096K (400000h)
MSG_FLASH_ADDR_SEL	lit	'ADDRSEL2'			; Message Flash attached to CS2
MSG_FLASH_ADDR_VAL	equ	(((MSG_FLASH_ADDR_LOC / 4096) << 4) + MSG_FLASH_ADDR_SZ)
MSG_FLASH_ADDR_DPP	equ	(MSG_FLASH_ADDR_LOC / 16384)
MSG_FLASH_ADDR_OFS	equ	(MSG_FLASH_ADDR_LOC & 16383)
MSG_FLASH_MCT		equ	NO_WAIT_STATE
MSG_FLASH_BUSCON	lit	'BUSCON2'
MSG_FLASH_BUSCON_VAL	equ	(BUS_ACTIVE | MSG_FLASH_MCT | NO_TRISTATE | NO_RW_DELAY)

SYS_RAM_ADDR_SZ		equ	RGSZ_2M				; Maximum 1Meg of System Ram (Allow for recovery of SFR areas)
								; Doubled to recover overlayed areas
SYS_RAM_ADDR_LOC	equ	0				; Starting at address 0
SYS_RAM_ADDR_SEL	lit	'ADDRSEL3'			; SysRam attached to CS3
SYS_RAM_ADDR_VAL	equ	(((SYS_RAM_ADDR_LOC / 4096) << 4) + SYS_RAM_ADDR_SZ)
SYS_RAM_ADDR_DPP	equ	(SYS_RAM_ADDR_LOC / 16384)
SYS_RAM_ADDR_OFS	equ	(SYS_RAM_ADDR_LOC & 16383)
SYS_RAM_MCT		equ	NO_WAIT_STATE
SYS_RAM_BUSCON		lit	'BUSCON3'
SYS_RAM_BUSCON_VAL	equ	(BUS_ACTIVE | SYS_RAM_MCT | NO_TRISTATE | NO_RW_DELAY)

FPGA_ADDR_SZ		equ	RGSZ_2M				; 2Meg Address Window
FPGA_ADDR_LOC		equ	2048 * 1024			; Starting at address 2048 (200000h)

FPGA_ADDR_SEL		lit	'ADDRSEL4'			; FPGA attached to CS4
FPGA_ADDR_VAL		equ	(((FPGA_ADDR_LOC / 4096) << 4) + FPGA_ADDR_SZ)
FPGA_ADDR_DPP		equ	(FPGA_ADDR_LOC / 16384)
FPGA_ADDR_SEG		equ	(FPGA_ADDR_LOC / 65536)
FPGA_ADDR_OFS		equ	(FPGA_ADDR_LOC & 16383)
FPGA_ADDR_SOF		equ	(FPGA_ADDR_LOC & 65535)
FPGA_MCT		equ	NO_WAIT_STATE
FPGA_BUSCON		lit	'BUSCON4'
FPGA_BUSCON_VAL		equ	(BUS_ACTIVE | FPGA_MCT | NO_TRISTATE | NO_RW_DELAY)

;******************************************************************************
;* Port Equates                                                               *
;******************************************************************************

P_FASTIN1		bit	P2.8				; Fast Input 1
DP_FASTIN1		bit	DP2.8           		
ODP_FASTIN1		bit	ODP2.8          		
	
P_FASTIN2		bit	P2.9				; Fast Input 2 
DP_FASTIN2		bit	DP2.9           		
ODP_FASTIN2		bit	ODP2.9          		
	
P_FASTIN3		bit	P2.10				; Fast Input 3 
DP_FASTIN3		bit	DP2.10          		
ODP_FASTIN3		bit	ODP2.10         		
	
P_FASTIN4		bit	P2.11				; Fast Input 4 
DP_FASTIN4		bit	DP2.11          		
ODP_FASTIN4		bit	ODP2.11         		
	
P_FASTIN5		bit	P2.12				; Fast Input 5 
DP_FASTIN5		bit	DP2.12          		
ODP_FASTIN5		bit	ODP2.12         		
	
P_FASTIN6		bit	P2.13				; Fast Input 6 
DP_FASTIN6		bit	DP2.13          		
ODP_FASTIN6		bit	ODP2.13         		
	
P_OPT_CARD_IRQ		bit	P2.14				; Option Card Interrupt 1 & 3 (Active Low)
DP_OPT_CARD_IRQ		bit	DP2.14          		
ODP_OPT_CARD_IRQ	bit	ODP2.14                         
	
P_ETHER_IRQ		bit	P2.15				; Ethernet IRQ 
DP_ETHER_IRQ 		bit	DP2.15          		
ODP_ETHER_IRQ 		bit	ODP2.15         		
	
P_I2C0_CLOCK		bit	P3.0				; I2C Buss A Clock
DP_I2C0_CLOCK 		bit	DP3.0           		
	
P_I2C0_DATA		bit	P3.1				; I2C Buss A Clock
DP_I2C0_DATA 		bit	DP3.1           		
	
P_FPGA_IRQ		bit	P3.2				; FPGA Port IRQ (AN0 & AN1 Used to Decode Cause)
DP_FPGA_IRQ		bit	DP3.2           		
ODP_FPGA_IRQ		bit	ODP3.2          		
	
P_DISPLAY_CLK		bit	P3.3				; Display Clock
DP_DISPLAY_CLK 		bit	DP3.3           		
ODP_DISPLAY_CLK		bit	ODP3.3          		
	
P_DISPLAY_RED		bit	P3.4				; Display Out Red bit
DP_DISPLAY_RED		bit	DP3.4           		
ODP_DISPLAY_RED		bit	ODP3.4          		
	
P_DISPLAY_GREEN		bit	P3.5				; Display Out Green bit
DP_DISPLAY_GREEN	bit	DP3.5           		
ODP_DISPLAY_GREEN	bit	ODP3.5          		

P_INPUT_MODE		bit	P3.6				; Input mode select
DP_INPUT_MODE		bit	DP3.6           		
ODP_INPUT_MODE		bit	ODP3.6          		
	
P_RTC_SYNC		bit	P3.7				; RTC FOUT Signal used to drive our T2
DP_RTC_SYNC		bit	DP3.7           		
ODP_RTC_SYNC		bit	ODP3.7          		
	
P_SPI_DATA_OUT		bit	P3.8				; SPI Data Out
DP_SPI_DATA_OUT		bit	DP3.8           		
ODP_SPI_DATA_OUT	bit	ODP3.8          		
	
P_SPI_DATA_IN		bit	P3.9				; SPI Data In
DP_SPI_DATA_IN		bit	DP3.9           		
ODP_SPI_DATA_IN		bit	ODP3.9          		
	
P_ASC0_TX		bit	P3.10				; ASC0 TX Pin
DP_ASC0_TX		bit	DP3.10                          
ODP_ASC0_TX		bit	ODP3.10                         
	
P_ASC0_RX		bit	P3.11				; ASC0 RX Pin
DP_ASC0_RX		bit	DP3.11          		
ODP_ASC0_RX		bit	ODP3.11         		
	
P_BHE			bit	P3.12				; BHE Line - Not normally configured by application
DP_BHE			bit	DP3.12          		
ODP_BHE			bit	ODP3.12         		
	
P_SPI_CLOCK		bit	P3.13				; SPI Clock out
DP_SPI_CLOCK		bit	DP3.13          		
ODP_SPI_CLOCK		bit	ODP3.13         		
	
P_CLK_OUT		bit	P3.15				; Master Clock out - Configured by CXStart
DP_CLK_OUT		bit	DP3.15          		
ODP_CLK_OUT		bit	ODP3.15         		
	
P_EXPANSION_ID2		bit	P5.3				; Expansion ID2 Data In
	
P_FPGA_DONE		bit	P5.14				; FPGA Done Line (Asserted after programming)
	
P_NMI_ACTIVE		bit	P5.15				; NMI Line State - Used for brownouts
	
P_RELAY			bit	P6.5				; Onboard Relay
DP_RELAY		bit	DP6.5           		
	
P_FPGA_PROGRAM		bit	P6.6				; FPGA Program Line
DP_FPGA_PROGRAM		bit	DP6.6           		
	
P_DISPLAY_LATCH		bit	P6.7				; Display Latch enable
DP_DISPLAY_LATCH	bit	DP6.7                           

;******************************************************************************
;* PEC Channels
;******************************************************************************

ALPHA_DRV_PEC_CTRL	equ     PECC7				; PEC Channel for the AL Alpha Driver
ALPHA_DRV_PEC_SRC 	lit     'SRCP7'         		
ALPHA_DRV_PEC_DST 	lit     'DSTP7'         		
ALPHA_DRV_PEC_CHAN	lit     'PEC_CHANNEL_7'   		
			
SSC_RX_PEC_CTRL		equ     PECC6				; SSC Receive PEC
SSC_RX_PEC_SRC		lit     'SRCP6'         		
SSC_RX_PEC_DST 		lit     'DSTP6'         		
SSC_RX_PEC_CHAN		lit     'PEC_CHANNEL_6'   		
			
SSC_TX_PEC_CTRL		equ     PECC5				; SSC Transmit PEC
SSC_TX_PEC_SRC		lit     'SRCP5'         		
SSC_TX_PEC_DST 		lit     'DSTP5'         		
SSC_TX_PEC_CHAN		lit     'PEC_CHANNEL_5'   		

;*** ASC0PEC Is No longer used but maintained for Library Compatiability ***
			
ASC0_RX_PEC_CTRL	equ     PECC4				; ASC0 Serial Receive PEC
ASC0_RX_PEC_SRC		lit     'SRCP4'         		
ASC0_RX_PEC_DST		lit     'DSTP4'         		
ASC0_RX_PEC_CHAN	lit     'PEC_CHANNEL_4'   		
			
ASC0_TX_PEC_CTRL	equ     PECC3				; ASC0 Serial Transmit PEC
ASC0_TX_PEC_SRC		lit     'SRCP3'         		
ASC0_TX_PEC_DST 	lit     'DSTP3'         		
ASC0_TX_PEC_CHAN	lit     'PEC_CHANNEL_3'   		
			
I2C_COMM_PEC_CTRL	equ     PECC2				; I2C Buss PEC, RX & TX
I2C_COMM_PEC_SRC	lit     'SRCP2'
I2C_COMM_PEC_DST	lit     'DSTP2'
I2C_COMM_PEC_CHAN	lit     'PEC_CHANNEL_2'

;******************************************************************************
;* IRQ Equates                                                                *
;******************************************************************************

FAST_IN1_INT_IE		bit	CC8IE				; Fast Input 1
FAST_IN1_INT_IR		bit	CC8IR           			
FAST_IN1_INT_IC		lit	'CC8IC'         			
FAST_IN1_INT_VEC	lit	'CC8INT'        			
FAST_IN1_INT_MASK	equ	INT0_MASK       			
FAST_IN1_EDGE		equ	INT0_EDGE_ANY    			
				
FAST_IN2_INT_IE		bit	CC9IE				; Fast Input 2
FAST_IN2_INT_IR		bit	CC9IR           			
FAST_IN2_INT_IC		lit	'CC9IC'         			
FAST_IN2_INT_VEC	lit	'CC9INT'        			
FAST_IN2_INT_MASK	equ	INT1_MASK       			
FAST_IN2_EDGE		equ	INT1_EDGE_ANY    			
				
FAST_IN3_INT_IE		bit	CC10IE				; Fast Input 3
FAST_IN3_INT_IR		bit	CC10IR          			
FAST_IN3_INT_IC		lit	'CC10IC'        			
FAST_IN3_INT_VEC	lit	'CC10INT'       			
FAST_IN3_INT_MASK	equ	INT2_MASK       			
FAST_IN3_EDGE		equ	INT2_EDGE_ANY    			
				
FAST_IN4_INT_IE		bit	CC11IE				; Fast Input 4
FAST_IN4_INT_IR		bit	CC11IR 
FAST_IN4_INT_IC		lit	'CC11IC'
FAST_IN4_INT_VEC	lit	'CC11INT'
FAST_IN4_INT_MASK	equ	INT3_MASK
FAST_IN4_EDGE		equ	INT3_EDGE_ANY
	
FAST_IN5_INT_IE		bit	CC12IE				; Fast Input 5
FAST_IN5_INT_IR		bit	CC12IR          			
FAST_IN5_INT_IC		lit	'CC12IC'        			
FAST_IN5_INT_VEC	lit	'CC12INT'       			
FAST_IN5_INT_MASK	equ	INT4_MASK       			
FAST_IN5_EDGE		equ	INT4_EDGE_ANY    			
				
FAST_IN6_INT_IE		bit	CC13IE				; Fast Input 6
FAST_IN6_INT_IR		bit	CC13IR          			
FAST_IN6_INT_IC		lit	'CC13IC'        			
FAST_IN6_INT_VEC	lit	'CC13INT'       			
FAST_IN6_INT_MASK	equ	INT5_MASK       			
FAST_IN6_EDGE		equ	INT5_EDGE_ANY    			

FAST_IN_EXEC_INT_IE	bit	ADCIE				; Input Defered Executioner
FAST_IN_EXEC_INT_IR	bit	ADCIR
FAST_IN_EXEC_INT_IC	lit	'ASCIC'
FAST_IN_EXEC_INT_VEC	lit	'ASCINT'

OPTION_INT_IE		bit	CC14IE				; Option Card
OPTION_INT_IR		bit	CC14IR          			
OPTION_INT_IC		lit	'CC14IC'        			
OPTION_INT_VEC		lit	'CC14INT'       			
OPTION_INT_MASK		equ	INT6_MASK       			
OPTION_EDGE		equ	INT6_EDGE_POS    			
				
ETHERNET_INT_IE		bit	CC15IE				; Ethernet Service Request
ETHERNET_INT_IR		bit	CC15IR          			
ETHERNET_INT_IC		lit	'CC15IC'        			
ETHERNET_INT_VEC	lit	'CC15INT'       			
ETHERNET_INT_MASK	equ	INT7_MASK       			
ETHERNET_EDGE		equ	INT7_EDGE_POS    			
			
FPGA_INT_IE		bit	CRIE				; FPGA Service Request
FPGA_INT_IR		bit	CRIR            			
FPGA_INT_IC		lit	'CRIC'
FPGA_INT_MASK		equ	CAP_INT_MASK
FPGA_EDGE		equ	CAP_INT_EDGE_POS
FPGA_EDGE_SEL		lit	'T5CON'				; T5 is used to set the CapIn IRQ Edge Selection

;******************************************************************************
;* General TIMER Equates                                                      *
;******************************************************************************

RTC_TICK_INT_IE		bit	T2IE
RTC_TICK_INT_IR		bit	T2IR
RTC_TICK_INT_IC		lit	'T2IC'
RTC_TICK_INT_VEC	lit	'T2_INT'
RTC_TICK_T_VAL		lit	'T2'
RTC_TICK_T_CTRL		lit	'T2CON'
RTC_TICK_T_OTL		lit	'T2OTL'
RTC_TICK_T_RUN		lit	'T2R'
	
DEBOUNCE_INT_IE		bit	T3IE
DEBOUNCE_INT_IR		bit	T3IR
DEBOUNCE_INT_IC		lit	'T3IC'
DEBOUNCE_INT_VEC	lit	'T3_INT'
DEBOUNCE_T_Val		lit	'T3'
DEBOUNCE_T_Ctrl		lit	'T3CON'
DEBOUNCE_T_OTL		lit	'T3OTL'
DEBOUNCE_T_RUN		lit	'T3R'
	
RS232_TICK_INT_IE	bit	T5IE
RS232_TICK_INT_IR	bit	T5IR
RS232_TICK_INT_IC	lit	'T5IC'
RS232_TICK_INT_VEC	lit	'T5_INT'
RS232_TICK_T_VAL	lit	'T5'
RS232_TICK_T_CTRL	lit	'T5CON'
RS232_TICK_T_OTL	lit	'T5OTL'
RS232_TICK_T_RUN	lit	'T5R'
	
RTOS_TICK_INT_IE	bit	T6IE
RTOS_TICK_INT_IR	bit	T6IR
RTOS_TICK_INT_IC	lit	'T6IC'
RTOS_TICK_INT_VEC	lit	'T6_INT'
RTOS_TICK_T_VAL		lit	'T6'
RTOS_TICK_T_CTRL	lit	'T6CON'
RTOS_TICK_T_OTL		lit	'T6OTL'
RTOS_TICK_T_RUN		lit	'T6R'
	
DISPLAY_INT_IE		bit	RTCIE
DISPLAY_INT_IR		bit	RTCIR
DISPLAY_INT_IC		lit	'XP3IC'
DISPLAY_T_VAL		lit	'T14'
DISPLAY_T_RLD		lit	'T14REL'

;******************************************************************************
;* IRQ Levels and Groups
;*                                  
;* NOTE: Levels 14 & 15 are reserved for PEC transfers)                 
;******************************************************************************

FPGA_LVL		equ     (ILVL8  | GLVL0)		; FPGA Interrupt Level - This is temporary
T6_TIMER_LVL      	equ     (ILVL13 | GLVL0)        	; Master 10ms timer

ASC0_HAL_RX_LVL		equ	(ILVL13 | GLVL3)		; ASC0/HAL RX Interrupt
ASC0_HAL_TX_LVL		equ	(ILVL13 | GLVL2)		; ASC0/HAL TX Interrupt
		
INPUT1_LVL		equ     (ILVL11 | GLVL3)        	
INPUT2_LVL		equ     (ILVL11 | GLVL2)        	
INPUT3_LVL		equ     (ILVL11 | GLVL1)        	
INPUT4_LVL		equ     (ILVL11 | GLVL0)        			
INPUT5_LVL		equ     (ILVL10 | GLVL3)        	
INPUT6_LVL		equ     (ILVL10 | GLVL2)        	
		
DEBOUNCE_LVL     	equ     (ILVL7  | GLVL0)        	; Debounce 1ms timer
INPUT_DEFER_LVL		equ     (ILVL6	| GLVL0)		; MUST be changed also in
I2C_DEFER_LVL     	equ     (ILVL5  | GLVL2)        	; I2C ISR ILVL/GLVL for execution of the CallBack.
SER_ERR_LVL       	equ     (ILVL2  | GLVL2)        	; ASC0 Serial Line Error                                              
DISPLAY_LVL      	equ     (ILVL1  | GLVL2)        	; Display ISR ILVL/GLVL
		
SER_BSL_TIMER_LVL  	equ     DEBOUNCE_LVL             	; Allow the Serial BSL to use the DEBOUCE LVL when running

;******************************************************************************
;* System CRC Equates                                                         *
;******************************************************************************

IP_LIMG_CRC		equ	28974
MASTER_CRC		equ	4079546297
STRUCT_CRC		equ	27592

;******************************************************************************
;* Serial Packet equates                                                      *
;******************************************************************************

SOH	        	equ	01h
ACK	        	equ	06h
NAK	        	equ	15h
ETX	        	equ	04h

;******************************************************************************
;* Input Port 1 ISR equates                                                   *
;******************************************************************************

IP_NUMBER_OF_INPUTS     equ     6               		; Number of Fast Inputs
IP_PIN_PORT             equ     P2              		; All inputs attached to P2.8..P2.13

;******************************************************************************
;* I2C SFR and Bit Definitions / not Available in REG161.INC                  *
;*                                                                            *
;* NOTE: None of the I2C control registers are bit addressable as they        *
;*       are held in XRAM - The I2C controller is an XBUS peripheral          *
;*                                                                            *
;******************************************************************************

ICCFG           	equ     0ED00h          		; I2C Configuration Register
SCL0            	equ     4               		; I2C Clock Channel 0 - Bit 4 of ICCFG
SCL1            	equ     5               		; I2C Clock Channel 1
SDA0            	equ     0               		; I2C Data  Channel 0 - Bit 0 of ICCFG 
SDA1            	equ     1               		; I2C Data  Channel 1 - Bit 1 of ICCFG 
SDA2            	equ     2               		; I2C Data  Channel 2 - Bit 2 of ICCFG 
								
ICCON           	equ     0ED02h          		; I2C Control Register
M10             	equ     0               		; 0 = 7 Bit Addressing / 1 = 10 Bit
RSC             	equ     1               		; 0 = No Operation / 1 = Generate a Restart (Self Clearing)
MOD0            	equ     2               		; The Mode bits are defined as
MOD1            	equ     3               		; 00 = Initialized 
								; 01 = Slave mode 
								; 10 = Master mode
								; 11 = Multi-Master mode
BUM             	equ     4               		; 0 = Generates a Stop Condition if set / 1 = Generate a Start Condition
ACKDIS          	equ     5               		; 0 = Generate ACKS , 1 = No ACKS generated 
AIRDIS          	equ     6               		; 0 = IRQD Cleared on reload of ICRTB , 1 = No Action on Reloads
TRX             	equ     7               		; 0 = I2C Receiving , 1 = I2C Transmitting
			
ICST            	equ     0ED04h          		; I2C Status Register
ADR             	equ     0               		; Set when in Slave mode if address is received
AL              	equ     1               		; Set when arbitration is lost in Multi Master mode
SLA             	equ     2               		; 0 = Bus Inactive , 1 = Bus Active
LRB             	equ     3               		; Value of Last Bit Received
BB              	equ     4               		; 0 = Stop Condition Occurred , 1 = Start Condition Occurred
IRQD            	equ     5               		; 0 = No Pending Data Interrupts , 1 = Pending Data IRQ
IRQP            	equ     6               		; 0 = No Pending Event Interrupts , 1 = Pending Event IRQ
								
ICRTB           	equ     0ED08h          		; Transmit and Receive register
								
XP0INT			equ	100h	        		; XBUS 0 Interupt Vector
								
XP0IC           	DEFR    0F186H          		; XBUS 0 Interupt Controller in ESFR space
XP0IE           	BIT	XP0IC.6         		; XBUS 0 Interupt Enable Bit 
XP0IR           	BIT	XP0IC.7         		; XBUS 0 Interupt Request Bit
								
I2C_INT_IE		bit	XP0IE           		
I2C_INT_IR		bit	XP0IR           		
I2C_INT_IC		lit	'XP0IC'         		
I2C_INT_VEC		lit	'XP0INT'        		
	
I2C_MODE_SLAVE   	equ     00000100b
I2C_MODE_MASTER  	equ     00001000b
I2C_MODE_MULTI   	equ     (I2C_MODE_SLAVE | I2C_MODE_MASTER)
								; Define the number of nanoseconds in one FRAME time
I2C_FRAME_PERIOD  	equ     ((NS_PER_SECOND / (CLOCK_FREQ / 4)) * 9)

;******************************************************************************
;* Serial Driver Equates.                                                     *
;******************************************************************************

SERIAL_DATA_LEN7        equ     0       			; Data Length is 7 Bits
SERIAL_DATA_LEN8        equ     1       			; Data Length is 8 Bits
			
SERIAL_PARITY_NONE      equ     0       			; Select No Parity
SERIAL_PARITY_ODD       equ     1       			; Select Odd Parity
SERIAL_PARITY_EVEN      equ     2       			; Select Even Parity
			
SERIAL_STOPBITS_1       equ     0       			; Generate 1 Stop Bit
SERIAL_STOPBITS_2       equ     1       			; Generate 2 Stop Bits
			
SER_ERR_NONE            equ     0       			; Configuration was successful - no error occurred
SER_ERR_CONFIG_ERR      equ     1       			; Configuration is not supported (TX/RX buffers not PEC addressable)
SER_ERR_NO_DEVICE       equ     2       			; Device is not present (16550 plug in card)
SER_ERR_BAUD_RATE       equ     3       			; Specified Baudrate is not supported
SER_ERR_FRAME_TYPE      equ     4       			; Requested Parity, Stop-bits and data length not supported
			
SERIAL_NO_DATA          equ     100h    			; Specifier for no data used with SerXXXX_GetByte

;******************************************************************************
;* 16C550 Equates for internal registers.                                     *
;******************************************************************************

; Control Register Address
SER550_RHR              equ     0       			; Receive Holding Register
SER550_THR              equ     0       			; Transmit Holding Register
SER550_IER              equ     1       			; Interupt Enable Register
SER550_ISR              equ     2       			; Interupt Status Register
SER550_FCR              equ     2       			; FIFO Control Register
SER550_LCR              equ     3       			; Line Control Register
SER550_MCR              equ     4       			; Modem Control Register
SER550_LSR              equ     5       			; Line Status Register
SER550_MSR              equ     6       			; Modem Status Register
			
; Interupt Enable Bits                  			
SER550_IE_REC           equ     1       			; Receive Holding Register Interrupt Enable
SER550_IE_TRAN          equ     2       			; Transmitter Holding Register Interrupt Enable
SER550_IE_LINE          equ     4       			; Line Status Interrupt Enable
SER550_IE_MODEM         equ     8       			; Modem Status Interrupt Enable
SER550_IE_SPECIAL       equ    32       			; Special Modem Interrupt Enable
			
; FIFO Control Bits                     			
SER550_F_TRIG_LVL1      equ     0       			; Trigger on 1 byte
SER550_F_TRIG_LVL4      equ     1       			; Trigger on 4 bytes
SER550_F_TRIG_LVL8      equ     2       			; Trigger on 8 bytes
SER550_F_TRIG_LVL14     equ     3       			; Trigger on 14 bytes
			
SER550_FRX_SEL       	equ     6       			; Bit position of the Receive FIFO trigger
SER550_FTX_SEL	        equ     4       			; Bit position of the Transmit FIFO trigger
			
SER550_FIFO_EN      	equ     1       			; Bit 0 must be set for the FIFOs to function
			
; Interrupt Source                      			
SER550_IRQ_REQ          equ     1       			; Bit 0 is cleared when an IRQ is pending
SER550_IRQ_LINE         equ     6       			; Line Status interrupt pending
SER550_IRQ_RECV         equ     4       			; Receive data interrupt pending
SER550_IRQ_RECV_TIMEOUT equ     12      			; Receive timeout interrupt pending
SER550_IRQ_TRANS        equ     2       			; Transmitter holding empty interrupt pending
SER550_IRQ_MODEM        equ     0       			; Modem status register interrupt pending
			
; Line Control Register Setup           			
SER550_LCR_5_BITS       equ     0       			; Frame WORD length is 5 bits
SER550_LCR_6_BITS       equ     1       			; Frame WORD length is 6 bits 
SER550_LCR_7_BITS       equ     2       			; Frame WORD length is 7 bits 
SER550_LCR_8_BITS       equ     3       			; Frame WORD length is 8 bits 
SER550_LCR_STOP_1       equ     0       			; Select 1 stop bit
SER550_LCR_STOP_2       equ     4       			; Select 2 stop bits
			
SER550_LCR_PARITY_ODD   equ     8       			; Select Odd Parity
SER550_LCR_PARITY_EVEN  equ     24      			; Select Even Parity
			
SER550_LCR_DIV_LATCH    equ     128     			; Baud rate divisor latch enable
			
; Line Status Register                  			
SER550_LSR_RX_FIFO_EMPTY equ     1       			; Bit set when no data is currently in the FIFO
SER550_LSR_OVERRUN     	 equ     2       			; Bit set when an overrun error occurred
SER550_LSR_PARITY      	 equ     4       			; Bit set when an parity error occurred
SER550_LSR_FRAME         equ     8       			; Bit set when a framing error occurred
SER550_LSR_TX_FIFO_EMPTY equ     32      			; Bit set when the Transmitter FIFO is empty

;******************************************************************************
;* SerialBSL Receive Size                                                     *
;******************************************************************************

SER_BSL_Rx_BUF_SIZE    equ     200     				; Number of bytes to reserve in IRAM for the BSL Serial Routines

;******************************************************************************
;* FPGA Control Structures                                                    *
;******************************************************************************

FPGA_PRI_CTRL_RESET	equ	(1 << 0)			; Force the FPGA to do an internal reset
FPGA_PRI_CTRL_VIDEO_EN  equ	(1 << 1)			; En Video Processing from the FPGA (out. data to the alpha display) 
FPGA_PRI_CTRL_DIP_OUT	equ	(1 << 2)			; Dipswitch output to write information to a latch
FPGA_PRI_CTRL_DIP_DIS	equ	(1 << 3)			; Dipswitch disable allow read back from a latch
FPGA_PRI_CTRL_DSP_FL0	equ	(1 << 4)			; Forces a reload of the first Row into the FPGA
								
FPGA_PRI_CTRL_485_DE	equ	(1 << 14)			; Force the DE line to stay high
FPGA_PRI_CTRL_UART	equ	(1 << 15)			; Enables the UART

								; Mask is used to only modify required bits
FPGA_PRI_CTRL_MASK	equ	(FPGA_PRI_CTRL_RESET | FPGA_PRI_CTRL_VIDEO_EN | FPGA_PRI_CTRL_DIP_OUT | FPGA_PRI_CTRL_DIP_DIS | FPGA_PRI_CTRL_DSP_FL0)

FPGA_PXL_CNT_MASK	equ	0111111111b			; Pixel Counter Mask - 9 Bits in length

FPGA_PEC_DELAY_MASK	equ	01111b				; Total Delay Time Mask - 4 bits in length

FPGA_INTENSITY_MASK	equ	011111111b			; Total Intensity Mask - 8 Bits

FPGA_IRQ_MSK_VID_ROW		equ	0			; Row Interrupt for updating the Display
FPGA_IRQ_MSK_VID_FRAME		equ	1			; Frame Interrupt / Display needs to be resynced
FPGA_IRQ_MSK_UART_TX_NOT_FULL 	equ	2			; Uart TX FIFO is full
FPGA_IRQ_MSK_UART_TX_HALF_EMPTY equ	3			; Uart TX FIFO is half full 
FPGA_IRQ_MSK_UART_TX_EMPTY	equ	4			; Uart TX FIFO is empty
FPGA_IRQ_MSK_UART_RX_FULL	equ	5			; Uart RX FIFO is full , possible RX overrunn
FPGA_IRQ_MSK_UART_RX_HALF	equ	6			; Uart RX FIFO is half full and should be serviced as soon as possible
FPGA_IRQ_MSK_UART_RX_ONE	equ	7			; Uart RX FIFO is empty

FPGA_IRQ_ABS_VID_ROW		equ	(1 << FPGA_IRQ_MSK_VID_ROW)		; Row Interrupt for updating the Display
FPGA_IRQ_ABS_VID_FRAME		equ	(1 << FPGA_IRQ_MSK_VID_FRAME)		; Frame Interrupt / Display needs to be resynced
FPGA_IRQ_ABS_UART_TX_NOT_FULL 	equ	(1 << FPGA_IRQ_MSK_UART_TX_NOT_FULL)	; Uart TX FIFO is NOT full
FPGA_IRQ_ABS_UART_TX_HALF_EMPTY equ	(1 << FPGA_IRQ_MSK_UART_TX_HALF_EMPTY)	; Uart TX FIFO is half empty
FPGA_IRQ_ABS_UART_TX_EMPTY	equ	(1 << FPGA_IRQ_MSK_UART_TX_EMPTY)	; Uart TX FIFO is empty
FPGA_IRQ_ABS_UART_RX_FULL	equ	(1 << FPGA_IRQ_MSK_UART_RX_FULL) 	; Uart RX FIFO is full , possible RX overrunn
FPGA_IRQ_ABS_UART_RX_HALF	equ	(1 << FPGA_IRQ_MSK_UART_RX_HALF) 	; Uart RX FIFO is half full and should be serviced...
										; ... as soon as possible
FPGA_IRQ_ABS_UART_RX_ONE	equ	(1 << FPGA_IRQ_MSK_UART_RX_ONE)  	; Uart RX FIFO is empty

;******************************************************************************
;* FPGA Memory Map                                                            *
;******************************************************************************

FPGA_PIXEL_DATA		equ	(FPGA_ADDR_OFS + 0x0000)	; All PIXEL Data is stored from this addr up until the width of ...
								; ... the Display
FPGA_CONTROL		equ	(FPGA_ADDR_OFS + 0x2000)	; Global control register (Reset , Display output etc)
FPGA_DIP_SWITCH		equ	(FPGA_ADDR_OFS + 0x2002)	; Data Register that reflects the Dipswitch settings.
FPGA_SPI_CHI_SEL	equ	(FPGA_ADDR_OFS + 0x2004)	; SPI Chip Select logic for enabling SPI Devices            
FPGA_INTENSITY		equ	(FPGA_ADDR_OFS + 0x2006)	; Sets the display intensity, must be adjusted for size of screen
FPGA_PIXEL_ADDR		equ	(FPGA_ADDR_OFS + 0x2008)	; Address Register for where pixel data will be written
FPGA_PEC_DELAY		equ	(FPGA_ADDR_OFS + 0x200a)	; Specified in 50ns units the delay time between the FPGA PEC IRQ's
FPGA_IRQ_STATUS		equ	(FPGA_ADDR_OFS + 0x200c)	; IRQ Status Register
FPGA_IRQ_REQUEST	equ	(FPGA_ADDR_OFS + 0x200e)	; Pending IRQ Register
FPGA_IRQ_ENABLE		equ	(FPGA_ADDR_OFS + 0x2010)	; IRQ Masking Register
FPGA_UART_DATA		equ	(FPGA_ADDR_OFS + 0x2020)	; UART I/O buffer
FPGA_UART_CTRL		equ	(FPGA_ADDR_OFS + 0x2022)	; UART Control Register

;******************************************************************************
;* Helper Macros & Literals                                                   *
;******************************************************************************

FAKE_IRET LIT "DB 0FBh,088h"

;******************************************************************************
;* Base DPP values assigned by linker                                         *
;******************************************************************************

	extrn	?C_PAGEDPP0 : DATA16				; Linker predefined DPP allocation numbers
	extrn	?C_PAGEDPP1 : DATA16
	extrn	?C_PAGEDPP2 : DATA16

?C_PAGEDPP3			equ	3			; Linker does not provide an entry for DPP3
								; it is assume to always be set to the SYSTEM
;******************************************************************************
;* Flash Macro Storage Requirements                                           *
;******************************************************************************

FLASH_MACROS_SZ			equ	102400			; Total space allocated for all Flash Macros
FLASH_MACRO_INDEX_ENTRIES	equ	1000			; Number of entries allocated for the index
FLASH_MACRO_LISTS_SZ		equ	22			; Sizeof List Structure
FLASH_MACRO_LISTS_ENTRIES	equ	32			; Total number of lists
FLASH_MACRO_RESERVE		equ	(128 * 1024)		; Total amount of memory to reserve for Flash Macros


Off	equ	0
On	equ	1
False	equ	Off
FALSE	equ	Off
True	equ	On
TRUE	equ	On
No	equ	Off
Yes	equ	On

;******************************************************************************
;* Relative Jump Definitions                                                  *
;******************************************************************************

%*define (reti)	     (db 0xfb,0x88)	; Fake Reti to stop assembler from issueing a warning

%*define (jz (dest)) (jmp cc_z,%dest)	; Zero
%*define (jnz(dest)) (jmp cc_nz,%dest)	; Not Zero
%*define (je (dest)) (jmp cc_eq,%dest)	; Equal (Zero)
%*define (jne(dest)) (jmp cc_ne,%dest)	; Not Equal (Not Zero)
%*define (jo (dest)) (jmp cc_v,%dest)	; Overflow
%*define (jno(dest)) (jmp cc_nv,%dest)	; No Overflow
%*define (jn (dest)) (jmp cc_n,%dest)	; Negative
%*define (jnn(dest)) (jmp cc_nn,%dest)	; Not Negative
%*define (jc (dest)) (jmp cc_c,%dest)	; Carry
%*define (jnc(dest)) (jmp cc_nc,%dest)	; No Carry
%*define (jl (dest)) (jmp cc_ult,%dest)	; Less than
%*define (jb (dest)) (jmp cc_ult,%dest)	; Below
%*define (jle(dest)) (jmp cc_ule,%dest)	; Less than or equal
%*define (jbe(dest)) (jmp cc_ule,%dest)	; Below or equal
%*define (ja (dest)) (jmp cc_ugt,%dest)	; Greater than (no carry + Zero)
%*define (jae(dest)) (jmp cc_uge,%dest)	; Greater than or equal (no carry)
%*define (jsb(dest)) (jmp cc_slt,%dest)	; Signed below
%*define (jsl(dest)) (jmp cc_slt,%dest)	; Signed less than
%*define (jsbe(dest))(jmp cc_sle,%dest)	; Signed below or equal
%*define (jsle(dest))(jmp cc_sle,%dest)	; Signed less than or equal
%*define (jsa(dest)) (jmp cc_sgt,%dest)	; Signed greater
%*define (jsae(dest))(jmp cc_sge,%dest)	; Signed greater than or equal

%*define (clr (data))(and %data,#0)	; Clear reg or memory
%*define (dec (data))(sub %data,#1)	; Decrement reg or memory
%*define (inc (data))(add %data,#1)	; Increment reg or memory
%*define (cplb(data))(bmovn %data,%data); Complement bit



