/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/dos/intelx86/real/msc60/test1.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:49CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/* 
 *
 * TRECK TCP Client example application for DOS, using packet driver
 *
 */
#include <dos.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include "trsocket.h"
#include "trpacket.h"

/* IP address, netmask and default gateway for this host */
#define TM_DEVICE_ADDR    "208.229.201.133"
#define TM_DEVICE_MASK    "255.255.255.0"
#define TM_DEVICE_GATEWAY "208.229.201.250"

/* IP Address and TCP port to send data to */
#define TM_DEST_PORT 9
#define TM_DEST_ADDR "208.229.201.115"

/* Number of times to send the data buffer to the remote host */
#define TEST_PKT_COUNT 1000

/* Test buffer length */
#define TEST_BUF_LENGTH 1448

/*
 * Application states (initial, connecting to remote host, 
 * connected and sending data and closed).
 */
#define INITIAL    0
#define CONNECTING 1
#define CONNECTED  2
#define CLOSED     3


/*
 * Internal functions 
 */
void _interrupt tfTimerIsrHandler();
void tfAbort (int sig);
static void tfSignalInit (void);
int tfStartup(void);


ttUserInterface interfaceHandle;
void (_interrupt TM_FAR *tlOldIsr)(void);
unsigned char appState;

/*
 * Socket callback function: called when a connect has completed or an error has
 * occured on the socket.
 */
void socketCallback( int sock, int eventFlags )
{
    int errorCode;

/* 
 * We are connected to the remote host: move to connected state to start sending
 * data.
 */
    if (eventFlags & TM_CB_CONNECT_COMPLT)
    {
        printf("connected!\n");
        appState = CONNECTED;
    }

/* An error has occured: close the socket and move to closed state */    
    if (eventFlags & TM_CB_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sock);
        printf("Socket error: %d\n", errorCode);
        (void)tfClose(sock);

        appState = CLOSED;
    }

    return;
}

void main(void)
{
    int                dataSock;
    int                loopCounter1;
    int                loopCounter2;
    struct sockaddr_in destAddr;
    int                errorCode;
    int                packetsToSend;
    int                bytes;
    int                unsentBytes;
    char  TM_FAR *     dataPtr;
    ttUserMessage      bufferHandle;

    printf("Init started...\n");
    tlOldIsr = 0;

/* Init our signal handler */
    tfSignalInit();

/* Start the Treck stack: add and open the ethernet interface */
    (void)tfStartup();

/* Trap the timer interrupt */
    tlOldIsr = _dos_getvect(0x08);

    _dos_setvect(0x08, tfTimerIsrHandler);

    printf("Init complete...\n");


    appState = INITIAL;
    packetsToSend = TEST_PKT_COUNT;

/* Main processing loop */
    loopCounter1 = loopCounter2 = 0;
    unsentBytes = TEST_BUF_LENGTH;
    bufferHandle = (ttUserMessage)0;
    while (1)
    {
mainLoopStart:

/* 
 * Allow the Treck stack to execute any timer related tasks that need to be 
 * handled (retransmissions, etc).
 */
        tfTimerExecute();
        (void)tfIoctlInterface(interfaceHandle, TM_PD_IOCTL_REFILL,0,0);

/* Print out a counter periodically just so we know we're still alive */
        if (loopCounter1++ == 0)
        {
            if ((loopCounter2++ % 200) == 0)
            {
                printf("-%d-\n",loopCounter2);
            }
        }


/*
 * Poll the device driver to see if there is any data waiting to be recieved:
 * if there is, pass it up to the protocol stack to be processed, and
 * refill the device's receive buffers.
 */

        if (!tfCheckReceiveInterface(interfaceHandle))
        {
            (void)tfRecvInterface(interfaceHandle);
        }


/* Application code */

/* 
 * Initial state: create a data socket, register a callback for this socket and
 * then initiate a connection to the remote host.
 */
        if (appState == INITIAL)
        {

/* Create socket */
            dataSock = socket (PF_INET, SOCK_STREAM, 0);

            if (dataSock == TM_SOCKET_ERROR)
            {
                printf("Couldn't open TCP socket\n");
                appState = CLOSED;
                goto mainLoopStart;
            }

/* 
 * Register a callback for this socket so we are notified when the connection
 * is made and if there are any errors.
 */
            errorCode = tfRegisterSocketCB( dataSock, 
                                            socketCallback, 
                                            TM_CB_CONNECT_COMPLT | 
                                                            TM_CB_SOCKET_ERROR);

            if (errorCode == TM_SOCKET_ERROR)
            {
                errorCode = tfGetSocketError(dataSock);
                printf("Error registering socket callback: %d\n", errorCode);
                appState = CLOSED;
                goto mainLoopStart;
            }

/* Specify the address family */
            destAddr.sin_family = AF_INET;

/* Specify the destination port */
            destAddr.sin_port = htons(TM_DEST_PORT);

/* Specify the destination IP address */
            destAddr.sin_addr.s_addr = inet_addr(TM_DEST_ADDR);

            printf("connecting...\n");        

/* Initiate connection to remote host */
            errorCode = connect( dataSock, (struct sockaddr *) &destAddr, 
                                 sizeof(destAddr) );

            if (errorCode == TM_SOCKET_ERROR)
            {
                errorCode = tfGetSocketError(dataSock);
                if (errorCode == TM_EINPROGRESS)
                {

/* Since we're in non-blocking mode, 'connect' just returns with an EINPROGRESS 
 * to let us know that the connection establishment has been started.  We move 
 * to the connecting state and wait for the callback to notify us that the
 * connection is established.
 */
                    appState = CONNECTING;
                }
                else
                {
                    printf("Error connecting to remote machine: %d\n",
                           errorCode);
                    appState = CLOSED;
                    goto mainLoopStart;
                }
            }
            else
            {
/* The connection to the remote host is complete */
                appState = CONNECTED;
            }

        }

/* 
 * Connected state: send the specified amount of data to the remote host and
 * then close the connection.
 */
        if (appState == CONNECTED)
        {
/* Get zero copy buffer */
            if ( bufferHandle == (ttUserMessage)0 )
            {
                bufferHandle = tfGetZeroCopyBuffer(TEST_BUF_LENGTH,&dataPtr);
                unsentBytes = TEST_BUF_LENGTH;
            }

/* Send data */
            do
            {
                bytes = tfZeroCopySend(dataSock, bufferHandle,
                             unsentBytes, 0);
                if (bytes > 0)
                {
                    unsentBytes -= bytes;
                }
            } while ( (bytes > 0) && ( unsentBytes > 0 ) );
            
            if ( bytes > 0 )
            {
                if (packetsToSend-- == 0)
                {
/* We've sent all of our data: close the connection. */
                    printf("Send test complete.");
                    (void)tfClose(dataSock);
                    appState = CLOSED;
                }
/* Reset buffer. We no longer own it. */
                bufferHandle = (ttUserMessage)0;
            }

            if (bytes == 0)
            {
/* The server closed the connection on us */
                (void)tfClose(dataSock);
                appState = CLOSED;
            }            

            if (bytes < 0)
            {
                errorCode = tfGetSocketError(dataSock);
                if (errorCode != TM_EWOULDBLOCK)
                {
                    printf("Error sending data: %d\n", errorCode);
                    (void)tfClose(dataSock);
                    appState = CLOSED;
/* Reset buffer. We no longer own it. */
                    bufferHandle = (ttUserMessage)0;
                }
            }

        }


    }
}

/*
 * Perform system startup (init link layer, etc)
 */

int tfStartup(void)
{

    ttUserIpAddress ipAddress;
    ttUserIpAddress netMask;
    ttUserLinkLayer linkLayerHandle;
    int             errorCode;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */

#ifndef TM_MULTIPLE_CONTEXT
    tfInitTreck();
#else /* TM_MULTIPLE_CONTEXT */
    tfInitTreckMultipleContext();
    contextHandle = tfCreateTreckContext();
    if (contextHandle == (void TM_FAR *)0)
    {
        tfKernelError("main","Failed to allocate Treck context\n");
    }
    else
    {
        tfSetCurrentContext(contextHandle);
    }
#endif /* TM_MULTIPLE_CONTEXT */

    (void)tfInitTreckOptions(TM_OPTION_TICK_LENGTH,55);

/* Start the Treck TCP/IP Stack */
    (void)tfStartTreck();

/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();

/* Add our interface */
    interfaceHandle=tfAddInterface(
                       "ETHER.DOS",
                        linkLayerHandle,
                        tfPacketOpen,
                        tfPacketClose,
                        tfPacketSend,
                        tfPacketRecv,
                        (ttDevFreeRecvFuncPtr)0,
                        tfPacketIoctl,
                        tfPacketGetPhyAddr,
                        &errorCode);

    if (errorCode != TM_ENOERROR)
    {
        printf("Interface Error %u", errorCode);
        tfAbort(0);
    }

    ipAddress=inet_addr(TM_DEVICE_ADDR);
    netMask=inet_addr(TM_DEVICE_MASK);

 /* Config the Interface */    
    errorCode=tfConfigInterface(
            interfaceHandle,
            ipAddress,
            netMask,
            0,
            1,
            0);

    if (errorCode != TM_ENOERROR)
    {
        tfAbort(errorCode);
    }

    (void)tfAddDefaultGateway(interfaceHandle, inet_addr(TM_DEVICE_GATEWAY));

    return(0);
}

/*
 * Timer ISR Handler
 */

void _interrupt tfTimerIsrHandler()
{
    tfTimerUpdateIsr();
    (*tlOldIsr)();
}

/*
 * Initialize abort functions so that we can clean up before exiting
 * the program
 */
static void tfSignalInit (void)
{
        (void)signal(SIGINT, tfAbort); /* Ctrl-C */
        (void)signal(SIGILL, tfAbort); /* illegal instruction */
        (void)signal(SIGSEGV, tfAbort); /* segment violation */
        (void)signal(SIGTERM, tfAbort); /* Software termination signal from kill() */
        (void)signal(SIGABRT, tfAbort); /* abort call */
}

/*
 * Clean up before exiting
 */

void tfAbort (int sig)
{
    printf("Program Terminated\n");

    if (tlOldIsr != 0)
    {
        _dos_setvect(0x08, tlOldIsr);
    }

    (void)tfPacketClose(interfaceHandle);

    printf("error code = %u\n",sig);

    exit(1);
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: test1.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 28714    Output checksum: 58871
 */
