/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Kernel Hooks
 *              NOTE: This File is intended to be modified by the user
 *                    In order to add a new operating system
 * Filename: trkernel.c
 * Author: Paul
 * Date Created: 11/10/97
 * $Source: source/kernel/dos/intelx86/real/msc60/trdos.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:50CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/* Modify the include file for your Kernel/OS */
/*#include <kernel.h> */
#include <stdio.h>
#include <stdlib.h>
#include <trsocket.h>
#include <dos.h>

/*
 * Generic Parameter Passing Union Typedef
 */


#define TM_KERN_OKAY   0
#define TM_KERN_ERROR -1

/*
 * Allocate a memory block of size from the kernel
 */
void TM_FAR *tfKernelMalloc(unsigned size)
{
#ifdef TM_USE_SHEAP
    return tfSheapMalloc(size);
#else /* !TM_USE_SHEAP */
    return (void TM_FAR *)(malloc((unsigned)size));
#endif /* !TM_USE_SHEAP */
}

/*
 * Free a memory block of size back to the kernel
 */
void tfKernelFree(void TM_FAR *memoryBlockPtr)
{
#ifdef TM_USE_SHEAP
    tfSheapFree(memoryBlockPtr);
#else /* !TM_USE_SHEAP */
    free(memoryBlockPtr);
#endif /* !TM_USE_SHEAP */
}

/*
 * Counting Semaphore Operations
 *
 * The examples assume the following:
 * A semaphore is of type int
 * Success from a Kernel call is 0
 * Failure from a Kernel Call is -1
 *
 */

/*
 * Create a counting semaphore
 */
int tfKernelCreateCountSem(ttUserGenericUnionPtr countingSemaphore)
{
    countingSemaphore=countingSemaphore;
    return TM_KERN_ERROR;
}

/*
 * Wait on a counting semaphore
 */
int tfKernelPendCountSem(ttUserGenericUnionPtr countingSemaphore)
{
    countingSemaphore=countingSemaphore;
    return TM_KERN_ERROR;
}

#ifdef TM_USE_PEND_TIMEOUT
/*
 * Wait on a counting semaphore, up to tickTimeout in ticks.
 */
int tfKernelPendCountSemTimeout ( ttUserGenericUnionPtr countingSemaphorePtr, 
                                  ttUser32Bit tickTimeout )
{
    countingSemaphorePtr=countingSemaphorePtr;
    TM_UNREF_IN_ARG(tickTimeout);
    return TM_KERN_ERROR;
}
#endif /* TM_USE_PEND_TIMEOUT */

/*
 * Resume waiting tasks on a counting semaphore
 */
int tfKernelPostCountSem(ttUserGenericUnionPtr countingSemaphore)
{
    countingSemaphore=countingSemaphore;
    return TM_KERN_ERROR;
}

/*
 * Delete a counting semaphore
 */
int tfKernelDeleteCountSem(ttUserGenericUnionPtr countingSemaphore)
{
    countingSemaphore=countingSemaphore;
    return TM_KERN_ERROR;
}


void  tfKernelError(const char TM_FAR *functionName,
                    const char TM_FAR *errorMessage)
{
    printf("tfKernelError! %s: %s\n", functionName, errorMessage);
    tm_thread_stop;
}

void  tfKernelWarning(const char TM_FAR *functionName,
                      const char TM_FAR *errorMessage)
{
    printf("tfKernelWarning! %s: %s\n", functionName, errorMessage);
}

extern unsigned char tvPendPostEnabled;

void  tfKernelInitialize(void)
{
/* for TM_TRECK_NO_KERNEL, or TM_TRECK_TASK, disable Pend, and Post */
    tvPendPostEnabled = 0;
}

/*
 * Create an event structure for pend/post from an ISR
 */
void tfKernelCreateEvent(ttUserGenericUnionPtr eventPtr)
{
    TM_UNREF_IN_ARG(eventPtr);
}

/*
 * Wait on an Event from an ISR
 */
void tfKernelPendEvent(ttUserGenericUnionPtr eventPtr)
{
    TM_UNREF_IN_ARG(eventPtr);
}

/*
 * Resume waiting tasks that ware waiting on this event
 */
void tfKernelIsrPostEvent(ttUserGenericUnionPtr eventPtr)
{
    TM_UNREF_IN_ARG(eventPtr);
}

/*
 * Yield the CPU. Used by tfInterfaceSpinLock().
 */
void tfKernelTaskYield(void)
{
    return;
}

/*
 * Install an ISR Handler
 */
void tfKernelInstallIsrHandler(
    ttUserIsrHandlerPtr funcPtr, unsigned long offSet)
{
    _dos_setvect((unsigned int)offSet, (void (__interrupt __far *)())funcPtr);
    return;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: trdos.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 52096    Output checksum: 26094
 */
