/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/dos/intelx86/real/msc60/tstnat.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:51CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 *
 * TRECK NAT Router example application for DOS, using packet driver over
 * Ethernet for the private network and PPP dialup for the public network.
 *
 */
#include <dos.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <trpacket.h>
#include <tr16550.h>

/*
 * IP addresses and netmask
 */
#define TM_DEVICE_ADDR             "192.168.111.112"   /* my ip address      */
#define TM_DEVICE_MASK             "255.255.255.0"     /* my netmask         */
#define TM_DEVICE_INNERSERVER      "192.168.111.111"   /* some other machine */

/*
 * Dialer customization
 */
#define TM_DIALER_PHONENUM_WITHCR "XXX-XXX-XXXX\r"
#define TM_DIALER_PROMPT_USERNAME "login:"
#define TM_DIALER_USERNAME_WITHCR "XXXXXX\r"
#define TM_DIALER_PROMPT_PASSWORD "Password:"
#define TM_DIALER_PASSWORD_WITHCR "xxxxxx\r"

/*
 * Special ports and synthetic port ranges
 */
#define TM_INNERSERVERPORT_PUBLIC  23000
#define TM_INNERSERVERPORT_PRIVATE 23
#define TM_NAPT_FIRST              0xC000   /* for NAPT */
#define TM_NAPT_LAST               0xCFFF
#define TM_PASV_FIRST              0xD000   /* for PASV connections with */
#define TM_PASV_LAST               0xDFFF   /* inner FTP servers         */

/*
 * Local variables
 */
int pppConnectedStatus = 0;
#ifdef TM_MULTIPLE_CONTEXT
ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
ttUserInterface interfaceHandleEthernet;
ttUserInterface interfaceHandlePpp;
void (_interrupt TM_FAR *tlOldIsr)();

/*
 * Internal functions
 */
void _interrupt tfTimerIsrHandler();
void tfAbort (int sig);
static void tfSignalInit (void);
int tfStartup(void);
void tfPppDialerNotify(ttUserInterface interfaceHandle, unsigned int flag);
void tfPppLinkNotify(ttUserInterface interfaceHandle, unsigned int flag);

/*
 * Socket callback function: called when a connect has completed or an error has
 * occured on the socket.
 */
void socketCallback( int sock, int eventFlags )
{
    int errorCode;

/*
 * We are connected to the remote host: move to connected state to start sending
 * data.
 */
    if (eventFlags & TM_CB_CONNECT_COMPLT)
    {
        tm_debug_log0("connected!");
    }

/* An error has occured: close the socket and move to closed state */
    if (eventFlags & TM_CB_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sock);
        tm_error_log1("Socket error: %d", errorCode);
        (void)tfClose(sock);

    }

    return;
}

void main(void)
{
    int                loopCounter1;
    int                loopCounter2;
    int                errorCode;
    ttUserIpAddress    ipaddr;
    ttUserIpAddress    ipppppeer;
    char               myiptext[20];
    char               peeriptext[20];

    tm_debug_log0("Init started...");
    tlOldIsr = 0;

/* Init our signal handler */
    tfSignalInit();

/* Start the Treck stack: add and open the ethernet interface */
    (void)tfStartup();

/* Trap the timer interrupt */
    tlOldIsr = _dos_getvect(0x08);

    _dos_setvect(0x08, tfTimerIsrHandler);

    tm_debug_log0("Init complete.");



    tm_debug_log0("Waiting for PPP to connect...");
    loopCounter1 = 0;
    loopCounter2 = 0;
    while (pppConnectedStatus == 0)
    {
/*
 * Allow the Treck stack to execute any timer related tasks that need to be
 * handled (retransmissions, etc).
 */
        tfTimerExecute();

/* Print out a counter periodically just so we know we're still alive */
        if ((loopCounter1++ % 32768) == 0)
        {
            if ((loopCounter2++ % 256) == 0)
            {
                tm_debug_log1("-%x-",loopCounter2);
            }
        }

/*
 * Poll the device driver to see if there is any data waiting to be recieved:
 * if there is, pass it up to the protocol stack to be processed, and
 * refill the device's receive buffers.
 */

        if (tfCheckReceiveInterface(interfaceHandleEthernet) == TM_ENOERROR)
        {
            (void)tfRecvInterface(interfaceHandleEthernet);
        }


        if (tfCheckReceiveInterface(interfaceHandlePpp) == TM_ENOERROR)
        {
            int rc;
            rc=tfRecvInterface(interfaceHandlePpp);
            if (rc != TM_ENOERROR)
            {
                tm_error_log1("tfRecvInterface error %d",rc);
            }
        }

    }

    if (tfGetIpAddress(interfaceHandlePpp,&ipaddr,0) == 0)
    {
        tfInetToAscii(ipaddr,myiptext);
    }
    else
    {
        strcpy(myiptext,"<unknown>");
    }
    if (tfGetPppPeerIpAddress(interfaceHandlePpp,&ipppppeer) == 0)
    {
        errorCode=tfAddDefaultGateway(interfaceHandlePpp, ipppppeer);
        if (errorCode != TM_ENOERROR)
        {
            tm_error_log1("tfAddDefaultGateway '%s'", tfStrError(errorCode));
            tfAbort(0);
        }
        tfInetToAscii(ipppppeer,peeriptext);
    }
    else
    {
        strcpy(peeriptext,"<unknown>");
    }
    tm_debug_log2("PPP connected.  My IP is %s, peer is %s",myiptext,peeriptext);

    tm_debug_log0("Type 'N' to see a dump of NAT triggers");

    errorCode = tfNatConfig(interfaceHandlePpp);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfig failed '%s'", tfStrError(errorCode));
        tfAbort(0);
    }

/* Now configure either NAPT/Inner Server  -or-  Static/Dynamic */


/*  ONE-IP CONFIGURATION: NAPT and Inner Servers  */

    errorCode = tfNatConfigNapt(interfaceHandlePpp,
                                ipaddr,
                                TM_NAPT_FIRST,
                                TM_NAPT_LAST);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfigNapt failed '%s'", tfStrError(errorCode));
        tfAbort(0);
    }

    errorCode = tfNatConfigInnerTcpServer(interfaceHandlePpp,
                                          ipaddr,
                                          inet_addr(TM_DEVICE_INNERSERVER),
                                          TM_INNERSERVERPORT_PUBLIC,
                                          TM_INNERSERVERPORT_PRIVATE);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfigInnerTcpServer failed '%s'",
               tfStrError(errorCode));
        tfAbort(0);
    }

    errorCode = tfNatConfigInnerUdpServer(interfaceHandlePpp,
                                          ipaddr,
                                          inet_addr(TM_DEVICE_INNERSERVER),
                                          TM_INNERSERVERPORT_PUBLIC,
                                          TM_INNERSERVERPORT_PRIVATE);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfigInnerUdpServer failed '%s'",
               tfStrError(errorCode));
        tfAbort(0);
    }

    errorCode = tfNatConfigInnerFtpServer(interfaceHandlePpp,
                                          ipaddr,
                                          inet_addr(TM_DEVICE_INNERSERVER),
                                          TM_PASV_FIRST,
                                          TM_PASV_LAST);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfigInnerServer failed '%s'",
               tfStrError(errorCode));
        tfAbort(0);
    }


/*
  MULTIPLE-IP CONFIGURATION: Dynamic or Static triggers

    errorCode = tfNatConfigStatic(interfaceHandlePpp,
                                  ipaddr,
                                  inet_addr(TM_DEVICE_INNERSERVER));
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfigStatic failed '%s'", tfStrError(errorCode));
        tfAbort(0);
    }

    errorCode = tfNatConfigDynamic(interfaceHandlePpp,
                                   ipaddr);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("tfNatConfigDynamic failed '%s'", tfStrError(errorCode));
        tfAbort(0);
    }
*/

/* Main processing loop */
    loopCounter1 = loopCounter2 = 0;
    while (1)
    {

/*
 * Allow the Treck stack to execute any timer related tasks that need to be
 * handled (retransmissions, etc).
 */
        tfTimerExecute();
        (void)tfIoctlInterface(interfaceHandleEthernet, TM_PD_IOCTL_REFILL,0,0);

/* Print out a counter periodically just so we know we're still alive */
        if ((loopCounter1++ % 32768) == 0)
        {
            if ((loopCounter2++ % 256) == 0)
            {
                tm_debug_log1("-%x-",loopCounter2);
            }
            if (_kbhit()) switch (_getch())
            {
                case '\x1B':
                    tm_debug_log0("User Abort");
                    tfAbort(0);
                    break;
#ifdef TM_DEBUG_LOGGING
                case 'N':
                case 'n':
                    tfNatDump(interfaceHandlePpp);
                    break;
#endif /* TM_DEBUG_LOGGING */
                    break;
            }
        }


/*
 * Poll the device driver to see if there is any data waiting to be recieved:
 * if there is, pass it up to the protocol stack to be processed, and
 * refill the device's receive buffers.
 */

        if (!tfCheckReceiveInterface(interfaceHandleEthernet))
        {
            int rc;
            rc=tfRecvInterface(interfaceHandleEthernet);
            if (rc != 0) {
                tm_error_log1("tfRecvInterface error %d",rc);
            }
        }


        if (!tfCheckReceiveInterface(interfaceHandlePpp))
        {
            (void)tfRecvInterface(interfaceHandlePpp);
        }


        if (pppConnectedStatus == 0)
        {
             tm_debug_log0("PPP Connection lost!");
             tfAbort(1);
        }
    }
}

/*
 * Perform system startup (init link layer, etc)
 */

int tfStartup(void)
{

    ttUserIpAddress ipAddress;
    ttUserIpAddress netMask;
    ttUserLinkLayer linkLayerHandlePpp;
    ttUserLinkLayer linkLayerHandleEthernet;
    int             errorCode;
    int             failed;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */


    failed = 0;
#ifndef TM_MULTIPLE_CONTEXT
    tfInitTreck();
#else /* TM_MULTIPLE_CONTEXT */
    tfInitTreckMultipleContext();
    contextHandle = tfCreateTreckContext();
    if (contextHandle == (void TM_FAR *)0)
    {
        tm_error_log0("Failed to allocate Treck context");
        failed = 1;
    }
    else
    {
        tfSetCurrentContext(contextHandle);
    }
#endif /* TM_MULTIPLE_CONTEXT */
    (void)tfInitTreckOptions(TM_OPTION_TICK_LENGTH,55);

/* Start the Treck TCP/IP Stack */
    errorCode=tfStartTreck();
    tm_error_log1("tfStartTreck returned %d",errorCode);

    errorCode=tfSetTreckOptions (TM_OPTION_IP_FORWARDING, 1UL);
    tm_error_log1("tfSetTreckOptions(IP_FORWARDING) returned %d",errorCode);

/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandleEthernet=tfUseEthernet();

/* Add the Ethernet interface -- the Private network */
    interfaceHandleEthernet=tfAddInterface(
                       "ETHER.DOS",
                        linkLayerHandleEthernet,
                        tfPacketOpen,
                        tfPacketClose,
                        tfPacketSend,
                        tfPacketRecv,
                        (ttDevFreeRecvFuncPtr)0,
                        tfPacketIoctl,
                        tfPacketGetPhyAddr,
                        &errorCode);
    if (errorCode != TM_ENOERROR)
    {
        tm_error_log1("Interface Error %u", errorCode);
        tfAbort(0);
    }

/* Use the PPP Link Layer */
    linkLayerHandlePpp = tfUseAsyncPpp(
        (ttUserLnkNotifyFuncPtr) tfPppLinkNotify);

/* Add PPP interface -- the Public network */
    if (failed == 0)
    {
        interfaceHandlePpp = tfAddInterface(
            "PPPDEVICE.001",
            linkLayerHandlePpp,
            tf16550Open,
            tf16550Close,
            tf16550Send,
            tf16550Receive,
            tf16550FreeReceiveBuffer,
            tf16550Ioctl,
            (ttDevGetPhysAddrFuncPtr) 0,
            &errorCode);
        if (errorCode != TM_ENOERROR)
        {
            tm_error_log1( "Adding interface 16550 driver failed '%s'",
                    tfStrError(errorCode) );
            failed = 1;
        }
    }

    ipAddress=inet_addr(TM_DEVICE_ADDR);
    netMask=inet_addr(TM_DEVICE_MASK);

 /* Config the Ethernet Interface */
    errorCode=tfOpenInterface(
            interfaceHandleEthernet,
            ipAddress,
            netMask,
            TM_DEV_IP_FORW_ENB,
            1);

    if (errorCode != TM_ENOERROR)
    {
        tfAbort(errorCode);
    }

    if (failed == 0)
    {
        errorCode=tfUseDialer(interfaceHandlePpp,tfPppDialerNotify);
        if (errorCode != TM_ENOERROR)
        {
            tm_error_log1("tfUseDialer failed '%s'", tfStrError(errorCode));
            failed = 1;
        }
    }

    if (failed == 0)
    {
        (void)tfDialerAddSendExpect ( interfaceHandlePpp,
            "\rAT\r",
            "OK",
            "ERROR",
            1,
            45,
            TM_DIALER_FAIL_ON_ERROR );

        (void)tfDialerAddSendExpect ( interfaceHandlePpp,
            "ATDT " TM_DIALER_PHONENUM_WITHCR,
            "CONNECT",
            "ERROR",
            1,
            45,
            TM_DIALER_FAIL_ON_ERROR );

        (void)tfDialerAddExpectSend ( interfaceHandlePpp,
            TM_DIALER_USERNAME_WITHCR,
            TM_DIALER_PROMPT_USERNAME,
            "",
            1,
            30,
            0 );

        (void)tfDialerAddExpectSend ( interfaceHandlePpp,
            TM_DIALER_PASSWORD_WITHCR,
            TM_DIALER_PROMPT_PASSWORD,
            "",
            1,
            30,
            0 );

/* Configure the PPP interface */
        ipAddress=0;
        netMask=inet_addr("255.255.255.0");

        errorCode=tfOpenInterface(
            interfaceHandlePpp,
            ipAddress,
            netMask,
            TM_DEV_IP_FORW_ENB,
            1);
        if (errorCode == TM_ENOERROR || errorCode == TM_EINPROGRESS)
        {
            errorCode = TM_ENOERROR;
        }
        else
        {
            tm_error_log1("tfOpenInterface PPP failed '%s'", tfStrError(errorCode));
            failed = 1;
        }
    }

    if (errorCode != TM_ENOERROR)
    {
        tfAbort(errorCode);
    }
    return(0);
}

/*
 * Timer ISR Handler
 */

void _interrupt tfTimerIsrHandler()
{
    tfTimerUpdateIsr();
    (*tlOldIsr)();
}

/*
 * Initialize abort functions so that we can clean up before exiting
 * the program
 */
static void tfSignalInit (void)
{
        (void)signal(SIGINT, tfAbort); /* Ctrl-C */
        (void)signal(SIGILL, tfAbort); /* illegal instruction */
        (void)signal(SIGSEGV, tfAbort); /* segment violation */
        (void)signal(SIGTERM, tfAbort); /* Software termination signal from kill() */
        (void)signal(SIGABRT, tfAbort); /* abort call */
}

/*
 * Clean up before exiting
 */

void tfAbort (int sig)
{
    int errorCode;

    tm_debug_log0("Program Terminated");

    if (tlOldIsr != 0)
    {
        _dos_setvect(0x08, tlOldIsr);
    }

    if (interfaceHandleEthernet != (ttUserInterface)0) \
    {
        errorCode = tfCloseInterface(interfaceHandleEthernet);
        if (errorCode != TM_ENOERROR)
        {
            tm_error_log1("Ethernet tfCloseInterface failed '%s'",
                   tfStrError(errorCode));
        }
    }

    if (interfaceHandlePpp != (ttUserInterface)0) \
    {
        tfNatUnConfig(interfaceHandlePpp);
        errorCode = tfCloseInterface(interfaceHandlePpp);
        if (errorCode != TM_ENOERROR)
        {
            tm_error_log1("PPP tfCloseInterface failed '%s'",
                   tfStrError(errorCode));
        }
    }

    if (sig != 0)
    {
        tm_debug_log1("exit code = %u",sig);
    }

    exit(sig);
}


void tfPppDialerNotify(ttUserInterface interfaceHandle, unsigned int flag)
{
    TM_UNREF_IN_ARG(interfaceHandle); /* (not used) */

    switch (flag)
    {
    case TM_DIALER_OPEN_FAILED:
        tm_error_log0("tfPppDialerNotify: Dial failed.");
        pppConnectedStatus = 2;
        break;

    case TM_DIALER_OPEN_COMPLETE:
        tm_debug_log0("tfPppDialerNotify: Dial complete.");
        break;

    default:
        tm_debug_log1("tfPppDialerNotify flag %d", flag);
        break;
    }
    return;
}

void tfPppLinkNotify(ttUserInterface interfaceHandle, unsigned int flag)
{
    TM_UNREF_IN_ARG(interfaceHandle); /* (not used) */

    switch (flag)
    {
    case TM_LL_LQM_LINK_BAD:
        tm_debug_log0("tfPppLinkNotify: Link quality is bad.");
        break;

    case TM_LL_LQM_UP:
        tm_debug_log0("tfPppLinkNotify: LQM enabled on the link.");
        break;

    case TM_LL_LQM_DISABLED:
        tm_debug_log0("tfPppLinkNotify: LQM disabled on the link.");
        break;

    case TM_LL_OPEN_COMPLETE:
        tm_debug_log0("tfPppLinkNotify: PPP link opened.");
        pppConnectedStatus = 1;
        break;

    case TM_LL_CLOSE_COMPLETE:
        tm_debug_log0("tfPppLinkNotify: PPP link closed.");
        pppConnectedStatus = 0;
        break;

    default:
        tm_debug_log1("tfPppLinkNotify flag %d", flag);
        break;
    }
    return;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: tstnat.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 34693    Output checksum: 22634
 */
