/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:
 *  "C" main program for port to TI C5x DSP
 *
 * Filename: tcpdemo.c
 * Author: Ed Remmell
 * Date Created: 11/10/2003
 * $Source: source/kernel/dsp_bios/c5510dsk/tcpdemo.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:37:12CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/* Elmic stack includes */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

/* DSP/BIOS includes */
#include <std.h>

#include "tcpcfg.h"
#include "trtic5mx.h"

/*
 *  The 5510 DSK Board Support Library is divided into several modules, each
 *  of which has its own include file.  The file dsk5510.h must be included
 *  in every program that uses the BSL.  This example also includes
 *  dsk5510_led.h and dsk5510_dip.h because it uses the LED and DIP modules.
 */
#include "dsk5510.h"
#include "dsk5510_led.h"
#include "dsk5510_dip.h"

ttUserInterface myInterfaceHandle;
ttUserLinkLayer etherLinkLayerHandle;
extern volatile int tlTxFree;
int peerReachable;

/* #define DO_LOOPBACK_TEST */
#define DO_PING_TEST
/* #define DO_LOCK_TEST */
/* #define DO_DHCP_TEST */
#define DO_PERF_TEST
#define TM_PERF_TEST	3
#define TM_DISCARD_PORT 9

/* macros used to read from and write to GMAC registers */
#define READ32(x)    (*(volatile tt32Bit *)x)
#define WRITE32(x,y) (*(volatile tt32Bit *)(x) = (y))

#ifdef DO_DHCP_TEST
#define TM_ETHER_OUR_IP_ADDR    "0.0.0.0"
#define TM_ETHER_PEER_IP_ADDR   "192.168.0.11"
#else /* ! DO_DHCP_TEST */
#define TM_ETHER_OUR_IP_ADDR    "169.254.114.21"
#define TM_ETHER_PEER_IP_ADDR   "169.254.114.20"
#endif /* ! DO_DHCP_TEST */

/*
 *  ======== main ========
 */
Void main()
{
    int errorCode;

    peerReachable = 0;

    tfInitTreck();
/* main routine is called with interrupts disabled, can call hardware
   initialization routines. Can enable individual interrupt mask bits,
   but should not call HWI_enable to globally enable interrupts. */
    errorCode = tfStartTreck();
    tm_assert(main, (errorCode == TM_ENOERROR));

    etherLinkLayerHandle = tfUseEthernet();
    myInterfaceHandle = tfAddInterface(
        "MX98728EC",
        etherLinkLayerHandle,
        tfTiMxOpen,
        tfTiMxClose,
        tfTiMxSend,
        tfTiMxReceive,
        (ttDevFreeRecvFuncPtr)0,
        tfTiMxIoctl,
        tfTiMxGetPhysAddr,
        &errorCode);
    tm_assert(main, (errorCode == TM_ENOERROR));
    
#ifdef TM_USE_DRV_ONE_SCAT_SEND
    errorCode = tfUseInterfaceOneScatSend
        (myInterfaceHandle, tfTiMxScatSend);
    tm_assert(main, (errorCode == TM_ENOERROR));
#endif /* TM_USE_DRV_ONE_SCAT_SEND */

    errorCode = tfUseInterfaceXmitQueue( myInterfaceHandle, 7);
    tm_assert(main, (errorCode == TM_ENOERROR));

/* fall into DSP/BIOS idle loop */
    return;
}

/* Tasks we run in this demo. Note that each needs to have a distinct
   priority, so that we get preemptive multi-tasking with DSP BIOS */
Void tfTreckTimerTask()
{
    tt32Bit loResTicksPerElmicTick;
    int msecCount;
    int errorCode;

    loResTicksPerElmicTick =
        (CLK_countspms() * TM_TICK_LENGTH) / (CLK_getprd() + 1);

    /* Toggle LED #0 */
    DSK5510_LED_toggle(0);
    msecCount = 0;

    while(1)
    {
        TSK_sleep(loResTicksPerElmicTick);

        tfTimerUpdate();
        tfTimerExecute();

        msecCount += TM_TICK_LENGTH;
        if (msecCount >= 1000)
        {
            msecCount = 0;
            /* Toggle LED #0 */
            DSK5510_LED_toggle(0);
        }

/* refill the receive buffer pool */
        errorCode = tfIoctlInterface(
            myInterfaceHandle, TM_DEV_IOCTL_REFILL_POOL_FLAG, (void *)0, 0);

        if (tlTxFree)
        {
/* transmit any packets that failed to transmit before */
            errorCode = tfIoctlInterface(
                myInterfaceHandle,
                TM_DEV_IOCTL_EMPTY_XMIT_FLAG, (void *)0, 0);
        }
    }
}

Void tfTreckReceiveTask()
{
    /* Toggle LED #1 */
    DSK5510_LED_toggle(1);

    while (1)
    {
        (void)tfWaitReceiveInterface(myInterfaceHandle);

        /* Toggle LED #1 */
        DSK5510_LED_toggle(1);

        (void)tfRecvInterface(myInterfaceHandle);
    }
}

#define TM_PORT_LOOPBACK_TEST htons(10)
#define TM_BUF_ARRAY_SIZE     2048

char buffer[TM_BUF_ARRAY_SIZE];

#ifdef DO_PING_TEST
static unsigned long lastPingTransmittedCount;
static unsigned long lastPingReceivedCount;
static void tfxPingCB(int pingSd)
{
    ttPingInfo pingInfo;

    tfPingGetStatistics(pingSd, &pingInfo);

    if (pingInfo.pgiSendErrorCode == TM_ENOERROR)
    {
        if (lastPingTransmittedCount != pingInfo.pgiTransmitted)
        {
            lastPingTransmittedCount = pingInfo.pgiTransmitted;

/* Toggle LED #2 when we send an Echo Request packet */
            DSK5510_LED_toggle(2);
        }
    }

    if (pingInfo.pgiRecvErrorCode == TM_ENOERROR)
    {
        if (lastPingReceivedCount != pingInfo.pgiReceived)
        {
            lastPingReceivedCount = pingInfo.pgiReceived;

/* Toggle LED #3 when we receive an Echo Reply packet */
            DSK5510_LED_toggle(3);
            peerReachable = 1;
        }
    }
}
#endif /* DO_PING_TEST */

#ifdef DO_DHCP_TEST
int addrConfigStatus;
void myDhcpNotifyFunc(
    ttUserInterface interfaceHandle, int errorCode )
{
    if (errorCode == 0)
    {
        addrConfigStatus = 1;
    }
    return;
}
#endif /* DO_DHCP_TEST */

Void tfTreckTcpServerTask()
{
    int                 errorCode;
    int                 retCode;
    int                 failed;
    struct sockaddr_in  addr;
    int                 sd;
    int                 serverSd;
    int                 acceptSd;
    int                 len;
    struct sockaddr_storage ipNgAddr;
#ifdef DO_PING_TEST
    int pingSd;
#endif /* DO_PING_TEST */
    int     sentLen;
    int     recvLen;
    char TM_FAR *      dataPtr;
    ttUserMessage      bufHandle;
    int                receivedSize;
    int                tcpNoDelay;
    int                devFlags;

    /* Toggle LED #2 */
    DSK5510_LED_toggle(2);

#ifdef TM_USE_DRV_ONE_SCAT_SEND
    devFlags = TM_DEV_SCATTER_SEND_ENB;
#else /* ! TM_USE_DRV_ONE_SCAT_SEND */
    devFlags = 0;
#endif /* ! TM_USE_DRV_ONE_SCAT_SEND */
#ifdef DO_DHCP_TEST
    devFlags |= TM_DEV_IP_DHCP;
    addrConfigStatus = 0;
    errorCode = tfUseDhcp (myInterfaceHandle, myDhcpNotifyFunc);
#endif /* DO_DHCP_TEST */

    errorCode = tfOpenInterface(
        myInterfaceHandle,
        inet_addr (TM_ETHER_OUR_IP_ADDR),
        inet_addr ("255.255.0.0"),
        devFlags,
        2);
    tm_assert(
        tfTreckTcpServerTask,
        ((errorCode == TM_ENOERROR) || (errorCode == TM_EINPROGRESS)));

#ifdef DO_DHCP_TEST
    while (!addrConfigStatus)
    {
        TSK_sleep(1);
    }
#endif /* DO_DHCP_TEST */

#ifdef DO_LOOPBACK_TEST
    failed = 0;
    sd = TM_SOCKET_ERROR;
    recvLen = TM_SOCKET_ERROR;
    sentLen = TM_SOCKET_ERROR;
/* Amount of data to send, and receive */
    len = sizeof(buffer);
    if (failed == 0)
    {
/* Open a TCP socket */
        sd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
    }
    if ( (sd == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed socket call */
        errorCode = tfGetSocketError(sd);
        tm_assert(tfTreckTcpServerTask, (sd != TM_SOCKET_ERROR));
        failed = 1;
    }
    if (failed == 0)
    {
/* Bind the TCP socket to a well known TCP port */
        addr.sin_family = PF_INET;
        addr.sin_port = TM_PORT_LOOPBACK_TEST;
        addr.sin_addr.s_addr = 0;
        errorCode = bind( sd,
                          (struct sockaddr TM_FAR *)&addr,
                          sizeof(struct sockaddr_in));
    }
    if ( (errorCode == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed bind call */
        errorCode = tfGetSocketError(sd);
        tm_assert(tfTreckTcpServerTask, (errorCode == TM_ENOERROR));
        failed = 1;
    }
    if (failed == 0)
    {
/* connect to our own socket: simultaneous TCP open. */
        addr.sin_addr.s_addr = inet_addr("127.0.0.1");
        errorCode = connect ( sd,
                              (struct sockaddr TM_FAR *)&addr,
                              sizeof(struct sockaddr_in));
    }
    if ( (errorCode == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed bind call */
        errorCode = tfGetSocketError(sd);
        tm_assert(tfTreckTcpServerTask, (errorCode == TM_ENOERROR));
        failed = 1;
    }
    if (failed == 0)
    {
/* Send some loop back data to our own socket */
        sentLen = send( sd,
                           buffer,
                           len,
                           0);
    }
    if ( (sentLen == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed send call */
        errorCode = tfGetSocketError(sd);
        tm_assert(tfTreckTcpServerTask, (sentLen != TM_SOCKET_ERROR));
        failed = 1;
    }
    if (failed == 0)
    {
/* Receive some loop back data from our own socket */
        recvLen = recv( sd,
                            buffer,
                            len,
                            0);
    }
    if ( (recvLen == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed recv call */
        errorCode = tfGetSocketError(sd);
        tm_assert(tfTreckTcpServerTask, (recvLen != TM_SOCKET_ERROR));
        failed = 1;
    }
    if (sd != TM_SOCKET_ERROR)
    {
/* All done. Close the socket: simultaneous TCP close. */
        (void)tfClose(sd);
    }
#endif /* DO_LOOPBACK_TEST */

#ifdef DO_PING_TEST
    tm_memset(&ipNgAddr, 0, sizeof(struct sockaddr_storage));
    ipNgAddr.ss_family = AF_INET; /* IPv6 address */
    ipNgAddr.ss_len = sizeof(struct sockaddr_storage);
    lastPingTransmittedCount = 0;
    lastPingReceivedCount = 0;

/* set the address we want to ping */
    ipNgAddr.addr.ipv4.sin_addr.s_addr = inet_addr (TM_ETHER_PEER_IP_ADDR);

/* start the ping */
    pingSd = tfNgPingOpenStart(
        &ipNgAddr,
        20, /* 20 milliseconds between retransmissions */
        100, /* 100 bytes of data */
        (ttPingCBFuncPtr) tfxPingCB);

    while(!peerReachable)
    {
        TSK_sleep(1);
    }

#ifdef DO_PERF_TEST
/* stop the ping for the performance test */
    (void) tfPingClose(pingSd);
#endif /* DO_PERF_TEST */
#endif /* DO_PING_TEST */

#ifdef DO_LOCK_TEST
    tfTestTreck(TM_TEST_LOCK,
                "10.0.1.5",
                htons(9),
                500,
                25,
                TM_TEST_FLAG_ZEROCOPY,
                (ttUserTestHandlePtr)0,
                0);
#endif /* DO_LOCK_TEST */

#ifndef DO_PERF_TEST
    failed = 0;
    serverSd = TM_SOCKET_ERROR;
    recvLen = TM_SOCKET_ERROR;
    sentLen = TM_SOCKET_ERROR;
/* Amount of data to send, and receive */
    len = sizeof(buffer);
    if (failed == 0)
    {
/* Open a TCP socket */
        serverSd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
    }
    if ( (serverSd == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed socket call */
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfTreckTcpServerTask, (serverSd != TM_SOCKET_ERROR));
        failed = 1;
    }
    if (failed == 0)
    {
/* Bind the TCP socket to a well known TCP port */
        addr.sin_family = PF_INET;
        addr.sin_port = htons(7); /* echo server */
        addr.sin_addr.s_addr = 0;
        errorCode = bind( serverSd,
                          (struct sockaddr TM_FAR *)&addr,
                          sizeof(struct sockaddr_in));
    }
    if ( (errorCode == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed bind call */
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfTreckTcpServerTask, (errorCode == TM_ENOERROR));
        failed = 1;
    }
    if (failed == 0)
    {
        errorCode = listen(serverSd, 1);
        if(errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(serverSd);
            tm_assert(tfTreckTcpServerTask, (errorCode == TM_ENOERROR));
            failed = 1;
        }
    
        for(;;)
        {
            acceptSd = accept(
                serverSd,
                (struct sockaddr TM_FAR *)0,
                (int TM_FAR             *)0 );

            if(acceptSd == TM_SOCKET_ERROR)
            {
                errorCode = tfGetSocketError(serverSd);
                tm_assert(tfTreckTcpServerTask, (errorCode == TM_ENOERROR));
                break;
            }

            tcpNoDelay = 1;
            errorCode = setsockopt(
                acceptSd, IPPROTO_TCP, TCP_NODELAY,
                (const char TM_FAR *)&tcpNoDelay, sizeof(tcpNoDelay));

            for(;;)
            {
                retCode = tfZeroCopyRecv( acceptSd, &bufHandle, &dataPtr, 
                                          TM_BUF_ARRAY_SIZE, MSG_DONTWAIT );
                
                switch(retCode)
                {
                case TM_SOCKET_ERROR:
                    errorCode = tfGetSocketError(acceptSd);
                    if (errorCode == TM_EWOULDBLOCK)
                    {
                        break;
                    }
/* fall through */
                case 0:
                    goto tcpServerClose;

                default:
                    receivedSize = retCode;
                    retCode =
                        tfZeroCopySend(acceptSd, bufHandle, receivedSize, 0);
                    break;
                }
            }

tcpServerClose:
            tfClose(acceptSd);
        }
    }
#endif /* ! DO_PERF_TEST */
}

Void tfTreckUdpServerTask()
{
    int errorCode;
    ttUserTestHandle testHandle;

    /* Toggle LED #3 */
    DSK5510_LED_toggle(3);

#ifdef DO_DHCP_TEST
    while (!addrConfigStatus)
    {
        TSK_sleep(1);
    }
#endif /* DO_DHCP_TEST */
#ifdef DO_PING_TEST
    while(!peerReachable)
    {
        TSK_sleep(1);
    }
#endif /* DO_PING_TEST */

    TSK_sleep(4000);
    DSK5510_LED_toggle(3);

#ifdef DO_PERF_TEST

/* Performance test start function */

    errorCode = TM_ENOERROR;
#if (TM_PERF_TEST == 1)
/* 1.1 UDP short packet send test: disable UDP checksum */
    errorCode = tfTestTreck(
        TM_TEST_UDP_SEND,         /* test type */
        TM_ETHER_PEER_IP_ADDR,    /* ip address */
        htons(TM_DISCARD_PORT),   /* port number */
        36,                       /* data size */
        60000,                    /* test count */
        (unsigned int)(TM_TEST_FLAG_ZEROCOPY
                       | TM_TEST_FLAG_UDP_CS_OFF), /* flags */
        &testHandle,        /* call back */
        (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 1) */

#if (TM_PERF_TEST == 2)
/* 1.2 UDP short packet send test: enable UDP checksum */
    errorCode = tfTestTreck(
        TM_TEST_UDP_SEND,         /* test type */
        TM_ETHER_PEER_IP_ADDR,    /* ip address */
        htons(TM_DISCARD_PORT),   /* port number */
        36,                       /* data size */
        60000,                    /* test count */
        (unsigned int)TM_TEST_FLAG_ZEROCOPY, /* flags */
        &testHandle,        /* call back */
        (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 2) */

#if (TM_PERF_TEST == 3)
/* 2.1 UDP long packet send test: disable UDP checksum */
    errorCode = tfTestTreck(
        TM_TEST_UDP_SEND,         /* test type */
        TM_ETHER_PEER_IP_ADDR,    /* ip address */
        htons(TM_DISCARD_PORT),   /* port number */
        1472,                     /* data size */
        60000,                    /* test count */
        (unsigned int)(TM_TEST_FLAG_ZEROCOPY
                       | TM_TEST_FLAG_UDP_CS_OFF), /* flags */
        &testHandle,        /* call back */
        (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 3) */

#if (TM_PERF_TEST == 4)
/* 2.2 UDP long packet send test: enable UDP checksum */
    errorCode = tfTestTreck(
        TM_TEST_UDP_SEND,         /* test type */
        TM_ETHER_PEER_IP_ADDR,    /* ip address */
        htons(TM_DISCARD_PORT),   /* port number */
        1472,                     /* data size */
        60000,                    /* test count */
        (unsigned int) TM_TEST_FLAG_ZEROCOPY, /* flags */
        &testHandle,        /* call back */
        (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 4) */

#if (TM_PERF_TEST == 5)
/* 3.1 TCP short packet send test: TCP_NODELAY */
    errorCode = tfTestTreck(
        TM_TEST_TCP_SEND,         /* test type */
        TM_ETHER_PEER_IP_ADDR,    /* ip address */
        htons(TM_DISCARD_PORT),   /* port number */
        12,                       /* data size */
        60000,                    /* test count */
        (unsigned int)(TM_TEST_FLAG_ZEROCOPY
                       | TM_TEST_FLAG_TCP_NODELAY), /* flags */
        &testHandle,    /* call back */
        (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 5) */

#if (TM_PERF_TEST == 6)
/* 4 TCP long packet send test */
    errorCode = tfTestTreck(
        TM_TEST_TCP_SEND,         /* test type */
        TM_ETHER_PEER_IP_ADDR,    /* ip address */
        htons(TM_DISCARD_PORT),   /* port number */
        1448,                     /* data size */
        60000,                    /* test count */
        (unsigned int) TM_TEST_FLAG_ZEROCOPY, /* flags */
        &testHandle,        /* call back */
        (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 7) */

#else /* ! DO_PERF_TEST */
    errorCode = tfTestTreck(
        TM_TEST_UDP_ECHO_SERVER,
        TM_ETHER_OUR_IP_ADDR,
        htons(7),
        4000,
        0,
        (unsigned long int)(TM_TEST_FLAG_ZEROCOPY
                            | TM_TEST_FLAG_UDP_CS_OFF),
        &testHandle,
        (void TM_FAR *)0UL);
#endif /* ! DO_PERF_TEST */
}

Void myUTL_doError(String s, Int errno)
{
    LOG_error("SYS_error called: error id = 0x%x", errno);
    LOG_error("SYS_error called: string = '%s'", s);
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:41      For: Xilinx-Samsung
 * File Name: tcpdemo.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 28110    Output checksum: 43034
 */
