/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:     System Description Macro's
 * This file is intended for the user to modify for their hardware/os platform
 * Filename:        trsystem.h
 * Author:          Odile & Paul
 * Date Created:    9/22/97
 * $Source: source/kernel/dsp_bios/c5510dsk/trsystem.h $
 *
 * Modification History 
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:37:14CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifndef _TRSYSTEM_H_

#define _TRSYSTEM_H_

#ifdef __cplusplus
extern "C" {
#endif

/* comment out the following line if you do not want to use IPv4 protocols */
#define TM_USE_IPV4

/* uncomment the following line if you want to use IPv6 protocols */
/* #define TM_USE_IPV6 */
     
/* 
 * TM_6_USE_NUD macro. Enable IPv6 Neighbor Unreachability Detection code.
 * Comment out to save code space when IPv6 Neighbor Unreachability Detection
 * is not needed.
 */

/* 
 * TM_6_USE_MLD macro. Enable IPv6 Multicast Listener Discovery code. Comment
 * out to save code space when IPv6 Multicast Listener Discovery is not needed.
 */

/* 
 * TM_6_USE_DAD macro. Enable IPv6 Duplicate Address Detection code. Comment
 * out to save code space when IPv6 Duplicate Address Detection is not needed.
 */

/* 
 * TM_6_USE_PREFIX_DISCOVERY macro. Enable IPv6 Prefix Discovery code, part
 * of stateless address auto-configuration. Comment out to save code space
 * when IPv6 Prefix Discovery is not needed.
 */

/* 
 * TM_6_USE_MIP_CN macro. Enable Mobile IPv6 correspondent node code.
 * Comment out to save code space when Mobile IPv6 correspondent node
 * functionality is not needed.
 */
/* #define TM_6_USE_MIP_CN */

/* 
 * TM_6_USE_MIP_MN macro. Enable Mobile IPv6 mobile node code. Comment out
 * to save code space when Mobile IPv6 mobile node functionality is not
 * needed.
 */
/* #define TM_6_USE_MIP_MN */

/* 
 * TM_6_USE_MIP_HA macro. Enable Mobile IPv6 home agent code. Comment out
 * to save code space when Mobile IPv6 mobile node functionality is not
 * needed.
 * NOTE: As of IPv6 phase II release, this is only a HA lite that dones't 
 *       implement the whole functionality of a MIP6 home agent
 */
/* #define TM_6_USE_MIP_HA */


/* HA needs CN functionality */

/* 
 * TM_6_USE_MIP_RO macro. Enable Mobile IPv6 route optimization. Comment
 * out to save code space when Mobile IPv6 route optimization is not needed.
 * ([MIPV6_18++].R14.4.5:10, [MIPV6_18++]R14.4.5:20)
 */

/* 
 * TM_6_MN_DISABLE_HOME_DETECT macro. Disable Mobile IPv6 Mobile Node home
 * detection/return to home functionality. If you know that the mobile node
 * never returns home, then you can define this macro to save some code
 * space.       
 */
/* #define TM_6_MN_DISABLE_HOME_DETECT */

/*
 * TM_6_USE_MIP_RA_RTR_ADDR macro. When enabled, the mobile node uses the
 * Router Address option (when available) in received Router Advertisement
 * messages to determine if a Router Advertisement is from the current
 * default router.
 */

/*
 * Uncomment the following line, if you want to use the netstat tool for
 * for outputting the routing entries, ARP entries and socket entries
 */
/* #define TM_USE_NETSTAT */

/*
 * Uncomment the following line to enable support for splitting out IPv4
 * and IPv6 traffic in the send path to separate virtual channels to
 * simulate multiple hosts as multiple IP aliases. Note that this is
 * incompatible with the default mode of IPv6 operation, therefore IPv6
 * prefix discovery must be disabled.
 */
/* #define TM_USE_VCHAN */

/*
 * Uncomment the following line to speed up lookup in the recieve-path of IP
 * aliases configured on the interface. This is intended for situations where
 * the user has configured many (i.e. >50) IP aliases on a single interface. 
 */
/* #define TM_OPTIMIZE_MANY_MHOMES */

/*
 * Uncomment the following line for SNMPv1 only.
 * (SNMP option only.)
 */
/* #define TM_SNMP_VERSION 1 */

/*
 * Uncomment the following line for SNMPv1 and SNMPv2c (bilingual).
 * (SNMP option only.)
 */
/* #define TM_SNMP_VERSION 2 */

/*
 * Uncomment the following line for SNMPv1, SNMPv2c and SNMPv3 (trilingual).
 * (SNMP option only.)
 */
/* #define TM_SNMP_VERSION 3 */

/*
 * Uncomment the TM_SNMP_CACHE macro if you want the stack to keep a cache of
 * the UDP sockets, TCP vectors, ARP table, and Routing table for SNMP.
 * (SNMP option only.)
 */
/* #define TM_SNMP_CACHE */

/*
 * Uncomment the following line for 8-bit OIDs.
 * (SNMP option only.)
 */
/* #define TM_SNMP_8BIT_SUBIDS */

/*
 * Uncomment the following line 16-bit OIDs.
 * (SNMP option only.)
 */	
/* #define TM_SNMP_16BIT_SUBIDS */

/*
 * TM_IP_FRAGMENT macro. Enable support for IP fragmentation and reassembly.
 * Comment out to save code space when IP fragmentation and reassembly are
 * not needed.
 */
#define TM_IP_FRAGMENT

/* 
 * This macro is commented out by default.
 * If you define TM_IP_FRAGMENT, and your device driver supports scattered 
 * send, then define TM_IP_FRAGMENT_NO_COPY to avoid an extra internal data
 * copy when an IP datagram need is fragmented.
 */
/* #define TM_IP_FRAGMENT_NO_COPY */


/*
 * TM_6_IP_FRAGMENT macro. Enable support for IPv6 fragmentation and
 * reassembly. Comment out to save code space when IPv6 fragmentation and
 * reassembly are not needed.
 */

/*
 * TM_6_USE_IP_FORWARD macro. Enable IPv6 forwarding code. Comment out to save
 * code space when IPv6 forwarding is not needed.
 */

/*
 * Uncomment the TM_DISABLE_TCP_SACK macro, if you want to remove the TCP
 * Selective Acknowlegment Options (RFC2018) code.
 */
/* #define TM_DISABLE_TCP_SACK */

/*
 * Uncomment the TM_DISABLE_TCP_RFC2414 macro, if you want to disable
 * the TCP Initial Send Window Increase as described in RFC 2414.
 */
/* #define TM_DISABLE_TCP_RFC2414 */

/*
 * Uncomment the TM_DISABLE_TCP_RFC2581 macro, if you want to remove the TCP
 * congestion control update to RFC2001 as described in RFC2581.
 */
/* #define TM_DISABLE_TCP_RFC2581 */

/*
 * Uncomment the TM_DISABLE_TCP_RFC3042 macro, if you want to remove the
 * Limited Transmit Algorithm that enhances TCP's Loss Recovery as described 
 * in RFC3042. 
 */
/* #define TM_DISABLE_TCP_RFC3042 */

/*
 * Comment out the TM_DISABLE_TCP_ACK_PUSH macro, if you want the
 * stack to send an immediate TCP ACK, when receiving a TCP segment with
 * the PUSH bit set.
 * (Important: this macro should be commented out when the peer runs
 *  windows 2000.)
 */
#define TM_DISABLE_TCP_ACK_PUSH

/*
 * Uncomment the TM_DISABLE_PMTU_DISC macro, if you want to disable automatic
 * Path MTU discovery on indirect routes.
 */
/* #define TM_DISABLE_PMTU_DISC */

/*
 * Uncomment the TM_6_DISABLE_PMTU_DISC macro, if you want to disable automatic
 * IPv6 Path MTU discovery.
 */
/* #define TM_6_DISABLE_PMTU_DISC */
    
/*
 * Uncomment the TM_ARP_UPDATE_ON_RECV, if you want to avoid sending an ARP
 * request every 10 minutes on an active ARP entry, i.e. an ARP entry for
 * a destination IP address that the stack is actively sending to.
 * Note that uncommenting out this macro will affect speed and size, since
 * every incoming packet will be checked for an ARP cache match on its
 * destination IP address.
 */
/* #define TM_ARP_UPDATE_ON_RECV */
 
/*
 * Uncomment the TM_TCP_ANVL macro, if you want the stack to strictly follow
 * the RFC recommendations for the TCP protocol, as checked by the ANVL tools.
 */
/* #define TM_TCP_ANVL */

/*
 * Uncomment the TM_USE_TCP_PACKET macro, if you want to modify the
 * TCP behaviour and force TCP to send data on user send packet boundaries.
 * Warning: uncommenting this macro, will make TCP less efficient, and
 * will also disable Path MTU discovery, and TCP Selective Acknowledgements.
 */
/* #define TM_USE_TCP_PACKET */

/*
 * Uncomment the TM_USE_REUSEADDR_LIST macro, if you want to use the
 * SO_REUSEADDR socket level option with setsockopt(). This enables use of
 * the socket reuse address list, i.e. enable binding on the same port using
 * different local IP addresses.
 */
/* #define TM_USE_REUSEADDR_LIST */

/*
 * Uncomment the TM_USE_AUTO_IP macro, if you want to use Auto configuration,
 * or want to add collision detection.
 */
/* #define TM_USE_AUTO_IP */

/*
 * Uncomment the TM_USE_RAW_SOCKET macro, if you want to use raw sockets.
 * (tfRawSocket() need to be called, instead of socket())
 */
/* #define TM_USE_RAW_SOCKET */
/*
 * Uncomment the TM_RAW_SOCKET_INPUT_ICMP_REQUESTS, if you want ICMP echo
 * requests, and ICMP address mask requests to be delivered to a raw ICMP
 * socket.
 */
/* #define TM_RAW_SOCKET_INPUT_ICMP_REQUESTS */

/*
 * Uncomment the TM_IGMP macro, if you have purchased the IGMP protocol
 */
/* #define TM_IGMP */

/*
 * Uncomment the TM_DEV_SEND_OFFLOAD macro, if you want to use the
 * device driver TCP segmentation/checksum offload capabilities.
 */
/* #define TM_DEV_SEND_OFFLOAD */

/*
 * Uncomment the TM_DEV_RECV_OFFLOAD macro, if you want to use the
 * device driver recv packet checksum offload capabilities.
 */
/* #define TM_DEV_RECV_OFFLOAD */

/* 
 * Uncomment the TM_USE_DRV_ONE_SCAT_SEND macro, if you want to use
 * a single call to the device driver, passing the packet handle, even
 * when sending a frame with scattered buffers. Note that to enable 
 * this feature, this macro needs to be added, and tfUseInterfaceOneScatSend
 * needs to be called on the interface that supports it.
 */
#define TM_USE_DRV_ONE_SCAT_SEND

/* 
 * Uncomment the TM_USE_DRV_SCAT_RECV macro, if you want to allow
 * the device driver recv function to pass back a frame to the stack
 * in scattered buffers ("Gather Read"). Note that to enable this feature, 
 * this macro needs to be added, and  tfUseInterfaceScatRecv needs to be 
 * called on the interface that supports it.
 */
/* #define TM_USE_DRV_SCAT_RECV */

/*
 * Uncommment TM_NEED_ETHER_32BIT_ALIGNMENT and uncomment TM_USE_DRV_SCAT_RECV,
 * without using tfUseInterfaceScatRecv(),
 * if you want tfRecvInterface to make the TCP/IP header aligned on a
 * 4-byte boundary, if it is not aligned coming from a single buffer in
 * an Ethernet device driver.
 */
/* #define TM_NEED_ETHER_32BIT_ALIGNMENT */

/*
 * Uncomment the TM_LOOP_TO_DRIVER, if you want to loop back all the way
 * to the driver when sending to an Ethernet interface IP address.
 */
/* #define TM_LOOP_TO_DRIVER */

/*
 * Undefine/delete the TM_INDRV_INLINE_SEND_RECV macro to test intra 
 * machine driver with a separate recv task. In that case the intra machine 
 * received data will no longer be processed in the send path, but will be 
 * processed when tfRecvInterface/tfRecvScatInterface is called. In the
 * examples directory, the txscatlp.c, and txscatdr.c modules contain sample
 * code that uses this feature.
 */
#define TM_INDRV_INLINE_SEND_RECV 

/*
 * Uncomment the TM_USE_INDRT, if you want to use the GHS INDRT debugging
 * tools . You also need to call tfUseIndrt() on the Ethernet interface
 * you want the INDRT debug messages to be sent and received on.
 */
/* #define TM_USE_INDRT */

/*
 * Uncomment the TM_MULTIPLE_CONTEXT macro, if you want to use multiple
 * instances of the Treck stack.
 */
/* #define TM_MULTIPLE_CONTEXT */

/*
 * Uncomment the TM_PPP_LQM macro, if you want PPP Link Quality Monitoring
 */
/* #define TM_PPP_LQM */

/*
 * Uncomment the TM_USE_IPHC to enable IP header compression (RFC-2507) with
 * PPP.
 */
/* #define TM_USE_IPHC */

/*
 * Uncomment the TM_USE_EAP to enable Extensible Authentication Protocol with
 * PPP.
 */
/* #define TM_USE_EAP */
    
/*
 * Uncomment the TM_USE_POP3 to enable POP3
 */
/* #define TM_USE_POP3 */

/*
 * Uncomment the TM_USE_SMTP to enable SMTP
 */
/* #define TM_USE_SMTP */

/*
 * Uncomment the TM_MOBILE_IP4 macro, if you have purchased the Mobile IPv4
 * option.
 */
/* #define TM_MOBILE_IP4 */

/*
 * IPsec Option
 * Uncomment the TM_USE_IPSEC macro, if you have purchased the IPSEC option.
 */
/*#define TM_USE_IPSEC */

/*
 * IKE Option
 * Uncomment the TM_USE_IKE macro, if you are going to use KEY management rather
 * than manual keying
 */
/*#define TM_USE_IKE */


/*
 * By default the prototypes in the header files are not declared extern.
 * If your linker dictates that they should be declared extern, then
 * uncomment the following macro.
 */

/* #define TM_PROTO_EXTERN extern */

/*
 * Uncomment the TM_DISABLE_ANSI_LINE_FILE macro if your compiler does not
 * support the ANSI __LINE__ and __FILE__ macros.
 * The tm_assert() macro will use the ANSI __LINE__ and __FILE__ macros (if
 * available) to print the source line number and source file name identifying
 * where in the code an assertion failure occurred for debugging purposes.
 */
#define TM_DISABLE_ANSI_LINE_FILE

/* Uncomment the TM_PC_LINT macro, if you are using PC-LINT */
/*#define TM_PC_LINT */


/*
 * Thread stop is used to stop a thread which has had an unrecoverable error
 * by default defined to be a forever loop
 *
 * Unrecoverable errors dictate that the system MUST be reset!
 * You can define this to the reset for your board
 */

/* #define tm_thread_stop rtosReset() */

/*
 * Define the TM_PING_MILLISECONDS_INTERVAL macro, if you want the
 * pingInterval parameter passed to the obsolete tfPingOpenStart API
 * to be in milliseconds rather than in seconds.
 */
/* #define TM_PING_MILLISECONDS_INTERVAL */


/*
 * Performance Macros
 */

/*
 * If we are using ethernet, we can use it inline to increase speed.
 */
/* #define TM_BYPASS_ETHER_LL */

/*
 * Optimize for speed
 */
#define TM_OPTIMIZE_SPEED

/*
 * Optimize for size
 */
/* #define TM_OPTIMIZE_SIZE */

/*
 * If you have only a single interface, and you only configure a single IP
 * address on that interface (i.e. multi-homing is not required), then
 * uncomment the TM_SINGLE_INTERFACE_HOME macro to reduce code size.
 */
#define TM_SINGLE_INTERFACE_HOME

/*
 * If you have only a single interface, and you are using IPv6, then
 * uncomment the TM_SINGLE_INTERFACE macro to reduce code size. Note that
 * all IPv6 interfaces are multi-homed, so you cannot use the
 * TM_SINGLE_INTERFACE_HOME macro with IPv6.
 */
/* #define TM_SINGLE_INTERFACE */

/*
 * User defined checksum routine
 */
/* #define TM_USER_OCS */

/*
 * RTOS/Kernel Macros
 */
/* #define TM_KERNEL_WIN32_X86 */
/* #define TM_KERNEL_VISUAL_X86 */
/* #define TM_KERNEL_LINUX_APP */
/* #define TM_KERNEL_ELX_86 */
/* #define TM_KERNEL_UCOS_CPU32 */
/* #define TM_KERNEL_UCOS_X86 */
/* #define TM_KERNEL_UCOS_PPC */
/* #define TM_KERNEL_UCOS_MIPS */
/* #define TM_KERNEL_AMX_CPU32 */
/* #define TM_KERNEL_AMX_X86 */
/* #define TM_KERNEL_DOS_X86 */
/* #define TM_KERNEL_THREADX_ARM7 */
/* #define TM_KERNEL_THREADX_CPU32 */    
/* #define TM_KERNEL_NONE_EZ80 */
/* #define TM_KERNEL_RZK_EZ80 */
/* #define TM_KERNEL_UCOS_XSCALE */
/* #define TM_KERNEL_NONE_H8S */
/* #define TM_KERNEL_UCOS_SH2 */
#define TM_KERNEL_TI_DSP_BIOS

/* The following is added on 2002/05/02 by Kuzumaki */
/* #define TM_KERNEL_REALOS_FR */



/*
 * Simple heap size (used by simple heap allocation routines in trsheap.c)
 * This is used by tfSheapMalloc() and tfSheapFree() provided in
 * trsheap.c, and called by tfKernelMalloc() and tfKernelFree() when
 * the RTOS does not provide heap support, and you do not want to use
 * C library malloc and free.
 * This is not documented because it is an OS supplement.
 * On a DSP platform, the heap size should be specified in words; on all
 * other targets the heap size should be specified in bytes.
 */
#define TM_SHEAP_SIZE TM_UL(75)*TM_UL(1024)

/*
 * Uncomment the TM_DYNAMIC_CREATE_SHEAP macro if you want the Treck simple
 * heap to be dynamically allocated by calling tfKernelSheapCreate rather
 * than implemented as a static array.
 */
#define TM_DYNAMIC_CREATE_SHEAP

/*
 * Number of memory pages. Instead of a unique array, the simple heap can be
 * divided into several ones. Uncomment TM_SHEAP_NUM_PAGE and indicate the
 * number of pages if you are using the simple heap with a system that has
 * paged memory and that one page is too small to hold the whole simple heap
 * memory (static or preallocated by tfKernelSheapCreate).
 * In this case the simple heap will be divided in several non-contiguous
 * pages.
 * The default for TM_DYNAMIC_CREATE_SHEAP is 1. Each page of memory must
 * have the same size. The page size is TM_SHEAP_SIZE/TM_SHEAP_NUM_PAGE.
 */
/* The first page of 25K words is allocated in DARAM */
#define TM_SHEAP_NUM_PAGE 3


/*
 * Do we have seperate tasks for processing receive or send complete
 */

/*
 * Received packets are processed in a seperate Receive task
 * This all depends on how we setup our device driver and if we
 * are using mode 2 or mode 4 (see below)
 */
#define TM_TASK_RECV

/*
 * By default, packets are sent in the context of the application
 * sending the data, or of the timer task for retransmission of data,
 * or of the receive task for TCP acks, or PING echo replies for
 * example. If you wish to use a separate transmit task to
 * send the data to the driver, then define TM_TASK_XMIT.
 */
/* #define TM_TASK_XMIT */

/*
 * When a send complete occurs, we process it in a seperate task
 * This all depends on how we setup our device driver and if we
 * are using mode 2 or mode 4 (see below)
 */
/* #define TM_TASK_SEND */

/*
 * Device driver macros
 */

/* #define TM_QUICC */


/*
 * Compiler macros to take advantage on inline assembly (when needed)
 */

/* #define TM_COMPILER_SDS */     /* The SDS Crosscode Compiler for 68K */
/* #define TM_COMPILER_DDI_PPC */ /* The Diab Data Compiler for the PPC */
/* #define TM_COMPILER_GHS_PPC */ /* The Green Hills Compiler for the PPC */
/* #define TM_COMPILER_MRI_68K */ /* The MRI Compiler for 68K           */
/* #define TM_COMPILER_GHS_ARM */ /* The Green Hills Compiler for the ARM */
/* #define TM_COMPILER_GHS_68K */ /* The Green Hills Compiler for the 68K */
    
/* 
 * Define this macro, if compiler cannot handle nested braces
 * in the constant initialization of in6_addr structure 
 * (trsocket.h/trglobal.h/trinit.c)
 */
/* #define TM_IN6ADDR_NO_NESTED_BRACES */

/*--------------------------------------------------------------------------*/

/*
 * The following macros are primitives that can be turned on/off to suite
 * your design (use these if you do NOT use the RTOS macros)
 */

/*
 * Processor Macros
 */
/* #define TM_EZ80 */
/* #define TM_INTEL_X86 */
/* #define TM_MOTOROLA_CPU32 */
/* #define TM_MOTOROLA_68K */
/* #define TM_MOTOROLA_PPC */
/* #define TM_TMS320_C3 */
/* #define TM_TMS320_C3_BIG_ENDIAN */ /* For TI C3 like DSP with byte order reversed */
/* #define TM_TMS320_C6 */

/*
 * Word Order
 */

/* #define TM_LITTLE_ENDIAN */

/* To perform internal checking for the Treck Stack */
/* #define TM_ERROR_CHECKING */

/* Uncomment the TM_6_IP_SRC_ERROR_CHECKING macro if you want validation of
   the source address of received IPv6 packets. */
/* #define TM_6_IP_SRC_ERROR_CHECKING */

/*
 * Number of milliseconds per tick
 */
/* #define TM_TICK_LENGTH  10 */

/*
 * Mode 1:
 * Treck TCP/IP is used without an RTOS or Kernel
 */
/* #define TM_TRECK_NO_KERNEL */

/*
 * Mode 2:
 * Treck TC/IP is used as a shared library on a non-preemptive RTOS
 */
/* #define TM_TRECK_NONPREEMPTIVE_KERNEL */

/*
 * Mode 3:
 * Treck TCP/IP is used as an independent task on an RTOS
 * (preemptive or nonpreemtive)
 */
/* #define TM_TRECK_TASK */

/*
 * Mode 4:
 * Treck TCP/IP is used as a shared library on a premptive kernel
 */
/* #define TM_TRECK_PREEMPTIVE_KERNEL */


/*
 * Predefined Kernel Macros
 */
/*
 * REALOS/FR
 * FUJITSU iTRON(ver3.0) for the MBxxxxx
 * It runs on a Fujitsu MBxxxxx platform
 */
#ifdef  TM_KERNEL_REALOS_FR
#define TM_BIG_ENDIAN                      /* Use CPU Endian */
#define TM_TRECK_PREEMPTIVE_KERNEL         /* REAL/OS is Pre-Emptive Kernel */
#define TM_TICK_LENGTH  100                /* number of milliseconds per tick */
/*#define TM_BYPASS_ETHER_LL*/             /* Use Ethernet Module */
#define TM_TASK_RECV                       /* Use Recieve Task */
#define TM_USE_SHEAP                       /* Use Simple Heap Memory */
#ifdef  TM_USE_SHEAP
/* Simple Heap Memory Size is 256Kbyte */    
#define TM_SHEAP_SIZE TM_UL(256)*TM_UL(1024) 
#endif /* TM_USE_SHEAP */
#endif /* TM_KERNEL_REALOS_FR */



/*
 * ELX-86
 * This Kernel runs Treck TCP/IP as it's own task
 * It runs on a Intel x86 platform in Real mode
 * It also uses far memory(heap)
 */
#ifdef TM_KERNEL_ELX_86
#define TM_TRECK_TASK
#define TM_TICK_LENGTH  100 /* number of milliseconds per tick */
#define TM_INTEL_X86
#define TM_FAR far
/*
 * Define the ELX movsb for high speed copy
 */
void movsb(void TM_FAR *dest, const void TM_FAR *src, unsigned int len);
/* Use movsb for our tm_bcopy */
#define tm_bcopy(a,b,c) \
        movsb((void TM_FAR *)(b),(const void TM_FAR *)(a),(unsigned int)(c))

#endif /* TM_KERNEL_ELX_86 */


/*
 * uC/OS for the XSCALE Core of INTEL processors
 * uC/OS is preemptive and we use Treck TCP/IP as a shared library
 */
#ifdef TM_KERNEL_UCOS_XSCALE_RTASK

#define TM_KERNEL_UCOS_XSCALE
#define TM_LITTLE_ENDIAN
#define TM_USE_DRV_ONE_SCAT_SEND
#define TM_DEV_SEND_OFFLOAD
#define TM_DEV_RECV_OFFLOAD

#define TM_TICK_LENGTH  5 /* number of milliseconds per tick */
#define TM_OPTIMIZE_SPEED
/*
 * TM_ETHER_HW_ALIGN specifies what type of boundary (4 byte, 16 byte, etc)
 * the ethernet buffer is aligned on.  This is useful for devices that require
 * a DMA'ed recieve buffer to be aligned on a certain boundary.
 */
#define TM_ETHER_HW_ALIGN TM_UL(16) /* The XSCALE requires 16-byte alignment */

#define TM_BYPASS_ETHER_LL
/*
 * large heap size because of the large numbers of driver receive and
 * transmit descriptors.
 */
#define TM_SHEAP_SIZE TM_UL(512)*TM_UL(1024)
#define TM_USE_SHEAP

#endif /* TM_KERNEL_UCOS_XSCALE_RTASK */

#ifdef TM_KERNEL_UCOS_XSCALE
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#endif /*TM_KERNEL_UCOS_XSCALE */

/*
 * uC/OS for the PPC Core of the Motorola processors
 * PPC includes the MPC860
 * uC/OS is preemptive and we use Treck TCP/IP as a shared library
 */
#ifdef TM_KERNEL_UCOS_PPC_RTASK
#define TM_KERNEL_UCOS_PPC
#define TM_TASK_RECV
#endif

#ifdef TM_KERNEL_UCOS_PPC
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_MOTOROLA_PPC

/*
 * TM_ETHER_HW_ALIGN specifies what type of boundary (4 byte, 16 byte, etc)
 * the ethernet buffer is aligned on.  This is useful for devices that require
 * a DMA'ed recieve buffer to be aligned on a certain boundary.
 */
#define TM_ETHER_HW_ALIGN TM_UL(16) /* The MPC860 requires 16-byte alignment */

#define TM_BYPASS_ETHER_LL
/*
 * large heap size because of the large numbers of driver receive and
 * transmit descriptors.
 */
#define TM_SHEAP_SIZE TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#endif /*TM_KERNEL_UCOS_PPC */



/*
 * uC/OS for the PPC Core of the Motorola processors
 * PPC includes the MPC860
 * uC/OS is NON-Preemptive and we use Treck TCP/IP as a shared library
 */
#ifdef TM_KERNEL_NP_UCOS_PPC_RTASK
#define TM_KERNEL_NP_UCOS_PPC
#define TM_TASK_RECV
#endif

#ifdef TM_KERNEL_NP_UCOS_PPC
#define TM_TRECK_NONPREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_MOTOROLA_PPC

/*
 * TM_ETHER_HW_ALIGN specifies what type of boundary (4 byte, 16 byte, etc)
 * the ethernet buffer is aligned on.  This is useful for devices that require
 * a DMA'ed recieve buffer to be aligned on a certain boundary.
 */
#define TM_ETHER_HW_ALIGN TM_UL(16) /* The MPC860 requires 16-byte alignment */
#define TM_BYPASS_ETHER_LL
/*
 * large heap size because of the large numbers of driver receive and
 * transmit descriptors.
 */
#define TM_SHEAP_SIZE TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#endif /*TM_KERNEL_NP_UCOS_PPC */

/*
 * H8S, no kernel
 * CPU: H8S/2674R
 */
#ifdef TM_KERNEL_NONE_H8S
#define TM_HITACHI_H8S
#define TM_FAR
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_BYPASS_ETHER_LL
#define TM_USE_DRV_ONE_SCAT_SEND
#define TM_LOOP_TO_DRIVER
#define TM_USE_SHEAP
#define TM_SHEAP_SIZE TM_UL(128)*TM_UL(1024)
#define TM_TRECK_NO_KERNEL
#define TM_ETHER_POLLING_MODE
#define TM_INTERRUPT
#define TM_IGMP
#endif /*TM_KERNEL_NONE_H8S */


/*
 * uC/OS for the CPU32 Core of the Motorola 68K processors
 * CPU32 includes 68332 and 68360
 * uC/OS is preemptive and we use Treck TCP/IP as a shared library
 */
#ifdef TM_KERNEL_UCOS_RTASK
#define TM_KERNEL_UCOS_CPU32
#define TM_TASK_RECV
#endif

#ifdef TM_KERNEL_UCOS_CPU32
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_MOTOROLA_CPU32
#ifdef TM_QUICC
#ifndef TM_SOC_SEND_Q_BYTES
/* Default send queue size in bytes */
#define TM_SOC_SEND_Q_BYTES 16384
#endif /* TM_SOC_SEND_Q_BYTES */

#define TM_BYPASS_ETHER_LL
#endif /* TM_QUICC */

/*
 * large heap size because of the large numbers of driver receive and
 * transmit descriptors.
 */
#define TM_SHEAP_SIZE TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#endif /*TM_KERNEL_UCOS_CPU32 */

/*
 * Non Preemptive uC/OS for the CPU32 Core of the
 * Motorola 68K processors
 * CPU32 includes 68332 and 68360
 * uC/OS is non-preemptive and we use Treck TCP/IP as a shared library
 */
#ifdef TM_KERNEL_NP_UCOS_RTASK
#define TM_KERNEL_NP_UCOS_CPU32
#define TM_TASK_RECV
#endif

#ifdef TM_KERNEL_NP_UCOS_CPU32
#define TM_TRECK_NONPREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_MOTOROLA_CPU32
#ifdef TM_QUICC
#ifndef TM_SOC_SEND_Q_BYTES
/* Default send queue size in bytes */
#define TM_SOC_SEND_Q_BYTES 16384
#endif /* TM_SOC_SEND_Q_BYTES */

#define TM_BYPASS_ETHER_LL
#endif /* TM_QUICC */
/*
 * large heap size because of the large numbers of driver receive and
 * transmit descriptors.
 */
#define TM_SHEAP_SIZE TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#endif /*TM_KERNEL_NP_UCOS_CPU32 */

#ifdef TM_KERNEL_AMX_CPU32
#define TM_TASK_RECV
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_MOTOROLA_CPU32
#ifdef TM_QUICC            /* defined in sds360cc.bat */
#ifndef TM_SOC_SEND_Q_BYTES
/* Default send queue size in bytes */
#define TM_SOC_SEND_Q_BYTES 16384
#endif /* TM_SOC_SEND_Q_BYTES */

#define TM_BYPASS_ETHER_LL
#endif /* TM_QUICC */

/*
 * large heap size because of the large numbers of driver receive and
 * transmit descriptors.
 */
#define TM_SHEAP_SIZE TM_UL(256)*TM_UL(1024)
/* Use AMX memory pool code. Do not use our SHEAP (Simple HEAP) */
#endif /*TM_KERNEL_AMX_CPU32 */


#ifdef TM_KERNEL_AMX_X86
#define TM_TASK_RECV
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  55 /* number of milliseconds per tick */
#define TM_INTEL_X86
#endif /*TM_KERNEL_AMX_X86 */


#ifdef TM_KERNEL_DOS_X86
#define TM_TICK_LENGTH 55
#define TM_TRECK_NO_KERNEL
#define TM_INTEL_X86
/* #define TM_USE_SHEAP */
/* #define TM_SHEAP_SIZE TM_UL(54)*TM_UL(1024) */
/* #define TM_BYPASS_ETHER_LL */
#define TM_FAR far
#endif /* TM_KERNEL_DOS_X86 */


/*
 * uC/OS for the Intel Family
 * uC/OS is preemptive and we use Treck TCP/IP as a shared library
 */
#ifdef TM_KERNEL_UCOS_X86
#define TM_TICK_LENGTH  55 /* number of milliseconds per tick */
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_INTEL_X86
#define TM_OPTIMIZE_SIZE
#define TM_BYPASS_ETHER_LL
#define TM_SHEAP_SIZE TM_UL(29)*TM_UL(1024)
#define TM_USE_SHEAP
#define TM_FAR far
#endif /*TM_KERNEL_UCOS_X86 */

/*
 * Windows 32 for the x86 family, 
 * Treck runs as multi-thread windows application.
 */
#ifdef TM_KERNEL_WIN32_X86
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_LITTLE_ENDIAN
#define TM_OPTIMIZE_SPEED
#define TM_BYPASS_ETHER_LL
#define TM_SHEAP_SIZE   TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#define TM_BSD_CLASH
#define TM_FAR
#define TM_USE_CLIB
#undef TM_INDRV_INLINE_SEND_RECV 
#endif /* TM_KERNEL_WIN32_X86 */

 
/*
 * Loop back testing using Microsoft Visual C++
 */
#ifdef TM_KERNEL_VISUAL_X86
#define TM_FAR
#define TM_TICK_LENGTH  10 /* number of milliseconds per tick */
#define TM_TRECK_TASK /* Turbo Treck run in a single task */
#define TM_LOCK_NEEDED
#define TM_OPTIMIZE_SPEED
#define TM_LITTLE_ENDIAN
#define TM_X86_ASM_SWAP
#define TM_BYPASS_ETHER_LL
#define TM_SHEAP_SIZE   TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#define TM_BSD_CLASH
#define TM_USE_CLIB
/*
 * To use interface driver instead of loop back driver, when
 * sending to our own IP address using the Ethernet link layer.
 */ 
#define TM_LOOP_TO_DRIVER 
#define TM_USE_DRV_ONE_SCAT_SEND /* Allow more efficient driver send */
/* Allow incoming scattered device driver recv data */
#define TM_USE_DRV_SCAT_RECV 
/* 
 * More efficient IP fragmentation when device driver supports scattered
 * send
 */
#define TM_IP_FRAGMENT_NO_COPY 
#endif /* TM_KERNEL_VISUAL_X86 */

/* Linux Raw_socket Driver */
/* #define TM_KERNEL_LINUX_APP */

#ifdef TM_KERNEL_LINUX_APP
/* For swap macro definitions, Linux socket calls */
#define TM_KERNEL_LINUX
#define TM_INTEL_GNU_ASM_SWAP
#define TM_FAR
#define TM_TICK_LENGTH        10 /* number of milliseconds per tick */
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_OPTIMIZE_SPEED
#define TM_TASK_RECV
#define TM_LITTLE_ENDIAN
#define TM_BYPASS_ETHER_LL
#define TM_SHEAP_SIZE   TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#define TM_DYNAMIC_CREATE_SHEAP
#define TM_BSD_CLASH
/* following define is for SNMPD Trace*/
#define TM_DEBUG_LOGGING
#define TM_TRACE_LOGGING
#define TM_ERROR_LOGGING 
#define TM_DEF_MAX_LOG_MSG_LEN 4096
#define TM_DEF_NUM_LOG_MSGS    10
#endif /* TM_KERNEL_LINUX_APP */

/*
 * Nucleus RTX for the TI TMS320C3x Family
 * Running Nucleus RTX in non-preemptive mode
 */
#ifdef TM_KERNEL_NP_NUCLEUSRTX_C3X
#define TM_TRECK_NONPREEMPTIVE_KERNEL
#define TM_TICK_LENGTH  10
#define TM_BYPASS_ETHER_LL
#endif /* TM_KERNEL_NUCLEUSRTX_C3X */

/*
 * ARM7 processor, no RTOS
 */
#ifdef TM_KERNEL_NONE_ARM7
#define TM_ARM7
#define TM_TICK_LENGTH 10
#define TM_USE_SHEAP
#define TM_SHEAP_SIZE TM_UL(128)*TM_UL(1024)
#define TM_TRECK_NO_KERNEL
#define TM_BYPASS_ETHER_LL
#endif /* TM_KERNEL_NONE_ARM7 */

/*
 * ThreadX for the ARM7
 */
#ifdef TM_KERNEL_THREADX_ARM7
#define TM_ARM7
#define TM_TICK_LENGTH 10
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_BYPASS_ETHER_LL
#endif /* TM_KERNEL_THREADX_ARM7 */

/*
 * ThreadX for the Motorola CPU32 core (68332 & 68360)
 */
#ifdef TM_KERNEL_THREADX_CPU32
#define TM_MOTOROLA_CPU32
#define TM_TICK_LENGTH 10
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_BYPASS_ETHER_LL
#endif /* TM_KERNEL_THREADX_CPU32 */
    
/*
 * TI DSP BIOS 
 */
#ifdef TM_KERNEL_TI_DSP_BIOS
#define TM_TICK_LENGTH 10
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_USE_CLIB
#define TM_USE_SHEAP
#endif /* TM_KERNEL_TI_DSP_BIOS */

/*
 * EZ80 evaluation board, no kernel
 */
#ifdef TM_KERNEL_NONE_EZ80
#define TM_EZ80
#define TM_USE_SHEAP
#define TM_SHEAP_SIZE TM_UL(128)*TM_UL(1024)
#define TM_TRECK_NO_KERNEL
#define TM_BYPASS_ETHER_LL
#define TM_Z80L92_ASM_OCS
#endif /* TM_KERNEL_NONE_EZ80 */

/*
 * EZ80 evaluation board, RZK(RTOS)
 */
#ifdef TM_KERNEL_RZK_EZ80
#define TM_EZ80
#define TM_USE_SHEAP
#define TM_SHEAP_SIZE TM_UL(128)*TM_UL(1024)
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_Z80L92_ASM_OCS
#define TM_USE_KERNEL_CRITICAL_STATUS

#ifndef TM_USER_KERNEL_CRITICAL_STATUS_TYPE
#define TM_USER_KERNEL_CRITICAL_STATUS_TYPE
typedef unsigned int ttUserKernelCriticalStatus;
#endif /* TM_USER_KERNEL_CRITICAL_STATUS_TYPE */

extern unsigned int RZKDisableInterrupts(void);
extern void RZKEnableInterrupts(unsigned int interruptsMask);

#define tm_kernel_set_critical \
            tvKernelCriticalStatus = RZKDisableInterrupts()
#define tm_kernel_release_critical \
            RZKEnableInterrupts(tvKernelCriticalStatus)

#endif /*TM_KERNEL_NONE_EZ80 */


/*
 * MIPS4kc, Malta evaluation board, ucos kernel
 * gnu tools: sde-mips
 */
#ifdef TM_KERNEL_UCOS_MIPS
#define TM_KERNEL_MIPS4k
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_TASK_RECV
#define TM_TICK_LENGTH        10 
#define TM_GNUMIPS4K_ASM_CRITICAL
/* define ENDIAN in makefile for little and big */
#define TM_BYPASS_ETHER_LL
#define TM_SHEAP_SIZE   TM_UL(256)*TM_UL(1024)
#define TM_USE_SHEAP
#define printf tfMipsPrintf
#endif /* TM_KERNEL_UCOS_MIPS */


/*
 * HITACHI SH2 Processor - uCOS-II ( big endian )
 *  SH7616 (SH2-DSP) evaluation board.
 */
#ifdef TM_KERNEL_UCOS_SH2
#define TM_HITACHI_SH2
#define TM_TASK_RECV
#define TM_TASK_SEND
#define TM_TASK_XMIT
#define TM_TICK_LENGTH  10   /* number of milliseconds per tick */
#define TM_TRECK_PREEMPTIVE_KERNEL
#define TM_OPTIMIZE_SPEED
#define TM_BYPASS_ETHER_LL
#define TM_USE_SHEAP
#define TM_SHEAP_SIZE ( 512UL * 1024UL )
#define TM_USE_DRV_ONE_SCAT_SEND
#endif /* TM_KERNEL_UCOS_SH2 */



/*
 * Intel x86
 */
#ifdef TM_INTEL_X86
#define TM_LITTLE_ENDIAN
#ifdef __BORLANDC__
/* In line assembly critical section */
#define TM_X86_ASM_CRITICAL
/* In line network byte ordering */
#define TM_X86_ASM_SWAP
/* Use assembly checksum routine */
#define TM_BORLAND_ASM_OCS
#endif /* __BORLANDC__ */
#ifdef _MSC_VER
/* In line assembly critical section */
#define TM_X86_ASM_CRITICAL
/* In line network byte ordering */
#define TM_X86_ASM_SWAP
#endif /* _MSC_VER */
#endif /* TM_INTEL_X86 */


/*
 * Motorola Power PC Core
 */
#ifdef TM_MOTOROLA_PPC
#undef TM_LITTLE_ENDIAN
/* In line assembly critical section */
#ifdef TM_COMPILER_DDI_PPC
#define TM_DDIPPC_ASM_CRITICAL
#define TM_DDIPPC_ASM_OCS
#endif /* TM_COMPILER_DDI_PPC */

#ifdef TM_COMPILER_GHS_PPC
#define TM_GHSPPC_ASM_CRITICAL
#define TM_GHSPPC_ASM_OCS
#endif /* TM_COMPILER_GHS_PPC */
#endif /* TM_MOTOROLA_PPC */

/*
 * Motorola CPU32 Core (68332 & 68360)
 */
#ifdef TM_MOTOROLA_CPU32
#undef TM_LITTLE_ENDIAN
/* In line assembly critical section */
#ifdef TM_COMPILER_SDS
#define TM_SDS68K_ASM_CRITICAL
#define TM_SDS68K_ASM_OCS
#endif /* TM_COMPILER_SDS */

#ifdef TM_COMPILER_MRI
#define TM_MRI_ASM_CRITICAL
#endif /* TM_COMPILER_MRI */

#endif /* TM_MOTOROLA_CPU32 */



/*
 * Motorola 68000 (EC) Core
 */
#ifdef TM_MOTOROLA_68K
#undef TM_LITTLE_ENDIAN

#ifdef TM_COMPILER_SDS
#define TM_SDS68K_ASM_CRITICAL
#endif /* TM_COMPILER_SDS */

#ifdef TM_COMPILER_MRI
#define TM_MRI_ASM_CRITICAL
#endif /* TM_COMPILER_MRI */

#endif /* TM_MOTOROLA_68K */

/*
 * Hitachi H8S
 */
#ifdef TM_HITACHI_H8S
#undef TM_LITTLE_ENDIAN
#endif /* TM_HITACHI_H8S */

/*
 * Hitachi SH2
 */
#ifdef TM_HITACHI_SH2
#define TM_BIG_ENDIAN
#endif /* TM_HITACHI_SH2 */


/*
 * ARM7 Core
 */
#ifdef TM_ARM7
#define TM_LITTLE_ENDIAN

#ifdef TM_COMPILER_GHS_ARM
#define TM_GHSARM7_ASM_CRITICAL
#define TM_GHSARM7_ASM_OCS
#endif /* TM_COMPILER_GHS_ASM */

#endif /* TM_ARM7 */

/* TI DSP C3 like processor but with byte order reversed */

/*
 * Texas Instruments TMS320C3x Core
 */


/*
 * Texas Instruments TMS320C5x Core
 */

/*
 * Texas Instruments TMS320C6000 core
 */
#ifdef TM_TMS320_C6

#define TM_USER_32BIT_TYPE
typedef unsigned int ttUser32Bit;
typedef signed int   ttUserS32Bit;

#define TM_UL(x) x##U
#define TM_L(x) x

#endif /* TM_TMS320_C6 */


/*
 * Zilog eZ80 processor
 */
#ifdef TM_EZ80
#define TM_FAR
#define TM_TICK_LENGTH  55 /* number of milliseconds per tick */
#define TM_LITTLE_ENDIAN
#endif /* TM_EZ80 */

#ifndef TM_TICK_LENGTH
#define TM_TICK_LENGTH 0
#endif /* TM_TICK_LENGTH */


/* PPP options */
/* define the following macro to use MSCHAP */
/*#define TM_USE_PPP_MSCHAP*/

/*
 * NAT Options
 */

/* Enable NAT progress display to stdout */
#define TM_NAT_TRACE

/* Enable tfNatDump(), which can display a list of NAT triggers to stderr */
#define TM_NAT_DUMP

/* Discard packets that NAT does not transform */
#define TM_NAT_STRICT



/*
 * IPsec and IKE Options
 */


/*
 * Uncomment the following line, if you want to use C library routines,
 * instead of the equivalent library routines provided by the Turbo Treck 
 * stack.
 */
/* #define TM_USE_CLIB */

/* 
 * NOTE: you may need to modify the following, if you #define TM_USE_CLIB
 */
#ifdef TM_USE_CLIB
/* For isupper, tolower ... */
#include <ctype.h>
/* For memcpy, strchr ...   */
#include <string.h>
/* For printf, sprintf...   */
#include <stdio.h>
/* For strtoul... */
#include <stdlib.h>
#endif /* TM_USE_CLIB */

/* 
 * if TM_USE_SNIFF is defined, interface drivers will dump the packets to the
 * file system in libpcap format, to be opened by tcpdump, windump, ethereal, 
 * or other libpcap file readers.
 */
/* #define TM_USE_SNIFF */

#ifdef __cplusplus
}
#endif

#endif /* _TRSYSTEM_H_ */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:41      For: Xilinx-Samsung
 * File Name: trsystem.h    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6610    Output checksum: 54319
 */
