/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/dsp_bios/dm642evm/txtelntd.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:37:45CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
#include <assert.h>
#include <string.h>

#define TM_TELD_BUFFER_SIZE     1024
#define TM_TELD_WELCOME_STR     "Welcome to Elmic TurboTreck Demo! \r\n"
#define TM_TELD_PROMPT_STR      "Treck # "

#define TM_TELD_COMMAND_HELP    "help"
#define TM_TELD_COMMAND_HELLO   "hello"
#define TM_TELD_COMMAND_EXIT    "exit"
#define TM_TELD_COMMAND_TCP     "tcp"
#define TM_TELD_COMMAND_UDP     "udp"
#define TM_TELD_COMMAND_ARP     "arp"
#define TM_TELD_COMMAND_ROUTE   "route"
#define TM_TELD_COMMAND_DEVICE  "device"
#define TM_TELD_COMMAND_BINDING "binding"
#define TM_TELD_COMMAND_SADB    "sadb"
#define TM_TELD_COMMAND_SPD     "spd"

#define TM_TELD_RESPOND_INVALID "Invalid Command, type help for supported commands\r\n"
#define TM_TELD_RESPOND_HELLO   "Hi There!\r\n"
#define TM_TELD_RESPOND_EXIT    "Byebye!\r\n"

#define TM_TELD_RESPOND_HELP_START \
"Supported command:\r\n\
  help    - this message\r\n\
  hello   - greeting message\r\n\
  exit    - terminate the telnet connection\r\n\
  tcp     - print TCP sockets\r\n\
  udp     - print UDP sockets\r\n\
  arp     - print ARP table\r\n\
  route   - print routing table\r\n\
  device  - print device table and IP address\r\n"
#define TM_TELD_RESPOND_HELP_IPSEC \
"  sadb    - print IPsec security associations\r\n\
  spd     - print IPsec Security Policy Database\r\n"
#define TM_TELD_RESPOND_HELP_MIP_CN \
"  binding - print Mobile IPv6 CN bindings\r\n"

#ifdef TM_USE_NETSTAT

#define TM_TELD_RESPOND_HELP TM_TELD_RESPOND_HELP_START

#else /* TM_USE_NETSTAT */
#define TM_TELD_RESPOND_HELP  \
"Supported command:\r\n\
  help    - this message\r\n\
  hello   - greeting message\r\n\
  exit    - terminate the telnet connection\r\n"
#endif /* TM_USE_NETSTAT */


/* local variable definitions for telnet deamon*/
ttUserTeldHandle    tlTelnetHandle;
char                tlServerDataBuf[TM_TELD_BUFFER_SIZE];
int                 tlServerDataLength;
char *              tlTableName[] = {
                        "ARP Table:\r\n",
                        "Routing Table:\r\n",
                        "TCP Sockets:\r\n",
                        "UDP Sockets:\r\n",
                        "Device Table:\r\n"
};

#ifdef TM_USE_NETSTAT
void tfxTelnetNetstat(
    ttUserTeldHandle    teldHandle,
    ttNtTableId         tableId);

int tfxNtEntryCBTelnet(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2);
#endif /* TM_USE_NETSTAT */

/* telnet functions */
void tfTeldOpened( 
    ttUserTeldHandle            teldHandle,
    struct sockaddr_in TM_FAR * sockAddrPtr )
{
    TM_UNREF_IN_ARG(sockAddrPtr);

    tlTelnetHandle      = teldHandle;

    tfTeldUserSend(teldHandle,
                   TM_TELD_PROMPT_STR,
                   strlen(TM_TELD_PROMPT_STR),
                   0);

    tfTeldUserSend(teldHandle,
                   TM_TELD_WELCOME_STR,
                   strlen(TM_TELD_WELCOME_STR),
                   0);

    tfTeldUserSend(teldHandle,
                   TM_TELD_PROMPT_STR,
                   strlen(TM_TELD_PROMPT_STR),
                   0);
    return;
}

int tfTeldIncoming( 
    ttUserTeldHandle teldHandle,
    char    TM_FAR * teldRecvBuf,
    int              teldBytes,
    int              eolFlag )
{
    int     i;
    int     closeSession; 

    closeSession = 0;
   
/* avoid buffer overflow */
    if(tlServerDataLength + teldBytes >= TM_TELD_BUFFER_SIZE)
    {
        memset(tlServerDataBuf, 0, TM_TELD_BUFFER_SIZE);
        tlServerDataLength = 0;
    }
    else
    {
/* process the received buffer */
        for (i = 0; i< teldBytes; i++)
        {
/* backspace handling */
            if (teldRecvBuf[i] == '\b')
            {
/* avoid under flow */
                if (tlServerDataLength == 0)
                {
                    return TM_ENOERROR;
                }
                else
                {
                    tlServerDataLength --;
/* erase the char on the client's screen */
                    tfTeldUserSend(teldHandle, " \b", 2, 0);
                }
            }
            else if(teldRecvBuf[i] < 0x20)
            {
/* ignore other special chars */
            }
            else
            {
/* store the normal chars */
                tlServerDataBuf[tlServerDataLength ++] = teldRecvBuf[i];
            }
        }
    }

/* repond to the command */
    if (eolFlag)
    {

        tfTeldUserSend(teldHandle, "\r\n", 2, 0);

        tfTeldUserSend(teldHandle,
                       TM_TELD_PROMPT_STR,
                       strlen(TM_TELD_PROMPT_STR),
                       0);

        if (strstr(tlServerDataBuf, TM_TELD_COMMAND_HELP))
        {
            tfTeldUserSend(teldHandle, TM_TELD_RESPOND_HELP, strlen(TM_TELD_RESPOND_HELP), 0);
        }
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_HELLO))
        {
            tfTeldUserSend(teldHandle, TM_TELD_RESPOND_HELLO, strlen(TM_TELD_RESPOND_HELLO), 0);
        }
#ifdef TM_USE_NETSTAT
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_TCP))
        {
            tfxTelnetNetstat(teldHandle, TM_NT_TABLE_TCP);
        }
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_UDP))
        {
            tfxTelnetNetstat(teldHandle, TM_NT_TABLE_UDP);
        }
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_ROUTE))
        {
            tfxTelnetNetstat(teldHandle, TM_NT_TABLE_RTE);
        }
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_ARP))
        {
            tfxTelnetNetstat(teldHandle, TM_NT_TABLE_ARP);
        }
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_DEVICE))
        {
            tfxTelnetNetstat(teldHandle, TM_NT_TABLE_DEVICE);
        }
#endif /* TM_USE_NETSTAT */
        else if (strstr(tlServerDataBuf, TM_TELD_COMMAND_EXIT))
        {
            tfTeldUserSend(teldHandle, TM_TELD_RESPOND_EXIT, strlen(TM_TELD_RESPOND_EXIT), 0);
            closeSession = 1;
        }
        else if (tlServerDataLength == 0)
        {
            tfTeldUserSend(teldHandle, " ", 1, 0);
        }
        else
        {
            tfTeldUserSend(teldHandle, TM_TELD_RESPOND_INVALID, strlen(TM_TELD_RESPOND_INVALID), 0);
        }


        if(closeSession)
        {
            tfTeldUserClose(teldHandle);
        }
        else if ((tlServerDataLength != 0))
        {
            tfTeldUserSend(teldHandle,
                           TM_TELD_PROMPT_STR,
                           strlen(TM_TELD_PROMPT_STR),
                           0);
        }

        memset(tlServerDataBuf, 0, tlServerDataLength);
        tlServerDataLength = 0;
    }
    return TM_ENOERROR;
}

void tfTeldClosed(ttUserTeldHandle teldHandle)
{
    tfTeldUserClose(teldHandle);
}

#ifdef TM_USE_NETSTAT
void tfxTelnetNetstat(
    ttUserTeldHandle    teldHandle,
    ttNtTableId         tableId)
{
    int                 headerStrLen;
    char                buffer[TM_NT_ENTRY_STR_LEN];
    ttUserGenericUnion  genUnion1;
    ttUserGenericUnion  genUnion2;


/* Table name */
    tfTeldUserSend(
        tlTelnetHandle, 
        tlTableName[tableId], 
        strlen(tlTableName[tableId]), 
        0);

    headerStrLen = TM_NT_ENTRY_STR_LEN;
    switch(tableId)
    {
    case TM_NT_TABLE_TCP:
        tfNtGetTcpHeaderStr(buffer, &headerStrLen);
        break;
    case TM_NT_TABLE_UDP:
        tfNtGetUdpHeaderStr(buffer, &headerStrLen);
        break;
    case TM_NT_TABLE_RTE:
        tfNtGetRteHeaderStr(buffer, &headerStrLen);
        break;
    case TM_NT_TABLE_ARP:
        tfNtGetArpHeaderStr(buffer, &headerStrLen);
        break;
    case TM_NT_TABLE_DEVICE:
        tfNtGetDeviceHeaderStr(buffer, &headerStrLen);
        break;
    }

    strcat(buffer, "\r\n");
/* Table header */
    tfTeldUserSend(tlTelnetHandle, buffer, strlen(buffer), 0);

/* the table contents */    
    genUnion1.gen8BitParm = (ttUser8Bit)tableId;
    genUnion2.genVoidParmPtr = teldHandle;

    tfNetStat(tableId, 
              tfxNtEntryCBTelnet,
              genUnion1,
              genUnion2);
}
#endif /* TM_USE_NETSTAT */

#ifdef TM_USE_NETSTAT
int tfxNtEntryCBTelnet(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{
    char                buffer[TM_NT_ENTRY_STR_LEN];
    int                 entryStrLen;
    ttNtDevEntryPtr     ntDevPtr;
    int                 i;
    int                 size;

    entryStrLen = TM_NT_ENTRY_STR_LEN;

    if (genParam1.gen8BitParm == TM_NT_TABLE_DEVICE)
    {
        ntDevPtr = &(ntEntryUPtr->ntDevEntry);

/* print the basic device information */
        size = TM_NT_ENTRY_STR_LEN;
        tfNtDeviceEntryToStr(ntDevPtr, buffer, &size);
        strcat(buffer, "\r\n");
        tfTeldUserSend(tlTelnetHandle, buffer, strlen(buffer), 0);

        if (ntDevPtr->ntDevStatus != TM_NT_DEVICE_STATUS_CLOSED)
        {
/* print the address infor header */
            size = TM_NT_ENTRY_STR_LEN;
            tfNtGetAddrConfHeaderStr(buffer, &size);
            strcat(buffer, "\r\n");
            tfTeldUserSend(tlTelnetHandle, buffer, strlen(buffer), 0);


/* print out each of the IPv4 address info in a row */
            for (i=0; i<TM_NT_MAX_IP_PER_IF; i++)
            {
                if (   ntDevPtr->ntDevAddrConfArray[i].ntAddrType
                    != TM_NT_ADDR_CONF_TYPE_NONE)
                {
                    size = TM_NT_ENTRY_STR_LEN;
                    tfNtAddrConfEntryToStr(
                        &ntDevPtr->ntDevAddrConfArray[i], 
                        buffer, 
                        &size);
                    strcat(buffer, "\r\n");
                    tfTeldUserSend(tlTelnetHandle, buffer, strlen(buffer), 0);
                }
            }

			tfTeldUserSend(tlTelnetHandle, "\r\n", 1, 0);
        }

    }
    else
    {
        switch(genParam1.gen8BitParm)
        {
        case TM_NT_TABLE_ARP:
            tfNtArpEntryToStr(&ntEntryUPtr->ntArpEntry,
                              buffer,
                              &entryStrLen);
            break;
        case TM_NT_TABLE_RTE:
            tfNtRteEntryToStr(&ntEntryUPtr->ntRteEntry,
                              buffer,
                              &entryStrLen);
            break;
        case TM_NT_TABLE_TCP:
            tfNtTcpEntryToStr(&ntEntryUPtr->ntTcpEntry,
                              buffer,
                              &entryStrLen);
            break;
        case TM_NT_TABLE_UDP:
            tfNtUdpEntryToStr(&ntEntryUPtr->ntUdpEntry,
                              buffer,
                              &entryStrLen);
            break;

        default:
            break;
        }
        strcat(buffer, "\r\n");
        tfTeldUserSend(tlTelnetHandle, buffer, strlen(buffer), 0);
    }
/* 
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries 
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}
#endif /* TM_USE_NETSTAT */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: txtelntd.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 26152    Output checksum: 64067
 */
