;****************************************************************************
;  Purpose: This file contain boot code for ZiLOG eZ80 L92 board. At the
;         end of boot code, will call function _main
;         1. Board level initialization. (ex. configures Chip select,
;            GPIO, UART, PRT)
;         2. Initialize Interrupt Vector Table and Jump Table; Install ISRs;
;         3. Setup Stack;
;         4. Call main()
;
;   Program Developed by : Qin Zhang
;   Unit                 : 
;   Date                 : 02/10/2003
;
; $Source: source/kernel/eZ80_none/F91boot.s $
;
; Modification History
;   $Revision: 6.0.2.2 $
;   $Date: 2010/01/18 06:37:46CST $
;   $Author: odile $
;   $ProjectName: /home/mks/proj/tcpip.pj $
;   $ProjectRevision: 6.0.1.23 $
;****************************************************************************

	public __exit
	extern _main

; Chip Select / Wait State Generator
CS0_BMC: .equ %F0 
CS1_BMC: .equ %F1 
CS2_BMC: .equ %F2 
CS3_BMC: .equ %F3 

CS_BMC0_DATA:	.equ	%82
CS_BMC1_DATA:	.equ	%01
CS_BMC2_DATA:	.equ	%81
CS_BMC3_DATA:	.equ	%84

; RAM register
RAM_CTL: .equ %B4
RAM_ADDR_U: .equ %B5

RAM_CTL_DATA: .equ %C0
RAM_ADDR_U_DATA: .equ %7A


;*******************************************************************************
;* Section:		.bootstrap                                                     *
;*******************************************************************************

;    public	__exit
	extern _RZKStartup
	extern _tfCrystalIsrHandler	; cs8900a interrupt handler


	define	.bootstrap, space=RAM, org=000000h
	section .bootstrap
	.assume ADL=1

	di
	rsmix	
	jp.lil	_ADL_START

;*******************************************************************************
;* Section:		IRQVect                                                        *
;*                                                                             *
;* This section will hold the Interrupt Vector Table .                         *
;*******************************************************************************
	define  .IRQVect, space=RAM, org=000040h
	section .IRQVect
	.def	_IVT_BASE

	IVT_SIZE	.equ	%28
_IVT_BASE:
	ds    	IVT_SIZE * 4

;*******************************************************************************
;* Section:		.startup                                                        *
;*                                                                             *
;* After a HW reset, all peripheral devices are initialized to a known state.  *
;* However, the state of the peripherals is unknown after a SW reset.  This    *
;* routine ensures none of the peripheral devices will generate interrupts.    *
;* In addition, the I register is set to point to the .IRQVect table.          *
;*******************************************************************************
	define	.startup, space=RAM
	section .startup
	.assume ADL=1
	.def _ADL_START
	.ref	.TOSPS
	.ref	.TOSPL
	.ref    .BSS_BASE, .BSS_LENGTH
	.ref    __low_data	; Address of initialized data section
	.ref    __low_romdata	; Address of initialized data section copy
	.ref    __len_data	; Length of initialized data sectrion


_ADL_START:
;
; At this point we're in ADL mode because we got here
; via a jp.lil at the reset location 000000h
;
;***	Set I register to start of interrupt vector table, use interrupt mode 2
	di
	RSMIX
	im		2
	ld		bc, _IVT_BASE
	ld		a, b
	ld		i, a

;***	Set the Bus Mode for external perihperals
	ld	a,CS_BMC0_DATA
	out0	(CS0_BMC),a
	ld	a,CS_BMC1_DATA
	out0	(CS1_BMC),a
	ld	a,CS_BMC2_DATA
	out0	(CS2_BMC),a
	ld	a,CS_BMC3_DATA
	out0	(CS3_BMC),a

; Config RAM
    ld  a, RAM_ADDR_U_DATA
	out0    (RAM_ADDR_U),a
    ld  a, RAM_CTL_DATA
	out0    (RAM_CTL),a

; Initialize the stack pointer
	ld.sis	sp,.TOSPS	; Setup SPS
	ld.lil	sp,.TOSPL	; Setup SPL

;*** 	Initialize the .BSS section to zero
	ld.lil	HL, .BSS_BASE		; BSS start Address
	ld.lil	BC, .BSS_LENGTH		; Assume BSS Length multiple of 4
	ld.lil	DE, .BSS_BASE		; BSS start Address

	inc	DE
	dec	BC    
	
	ld	(HL), 0			; Load 0 in memory

	ldir

;------ Copy the initialized data section
	ld	hl,__len_data	; Check for non-zero length
	ld	bc,0		; *
	or	a,a		; *
	sbc	hl,bc		; *
	jr	z,_c_data_done	; .DATA is zero-length ...
$$:
	ld	hl,__low_romdata	; [hl]=__low_romdata
	ld	de,__low_data	; [de]=.data
	ld	bc,__len_data	; [bc]= data length
loaddata:
	add hl, bc		;
	push hl
	or a, a
	sbc hl,bc		
	ldir			; Copy the data section
	push hl			; load next address to bc
	pop  bc			;
	pop hl			; load end address to hl
	or a,a			; reset cary flag
	sbc hl,bc 		; check if done transfer
	jr z, _c_data_done	;
	push bc
	push hl
	pop bc
	pop hl
	jr loaddata
_c_data_done:

;***	Enable interrupts	
    ei

    nop      

; jump to main function
	jp	_main		; Transfer to C-function.

	.def	__exit
__exit:
	jr	$		; ?

;************************************************************************
;* Default interrupt handler
;************************************************************************
__default_hndlr:
    reti.l

