;********************************************************************************************************
;*	source  : L92bootRzkStartup.s (  Runtime Startup )
;*	
;   Scope : The file will initialize the L92 board and do preparation for RZK to run.
;			1. Configures Chip select:
;						LBR		UBR		CTL
;				CS0		F0		FF		00(Disabled)		(ROM)
;				CS1		00		07		28(one wait state)	(RAM)
;				CS2		B0		BF		08(Enabled)			(ext RAM)
;				CS3		FF		FF		08(Enabled)			(CS8900A)
;			2. Configures GPIO, UART, PRT;
;			3. Initialize Interrupt Vector Table and Jump Table; Install ISRs;
;			4. Setup Stack;
;           5. Call main()
;
;  Copyright (c) , ZiLOG & ELMIC 2002
;  Project : 
;
; * Program Developed by : Rajiv I.Pujar & Qin Zhang
; * Unit                 : 
; * Date                 : 07/27/2002
;
; $Source: source/kernel/eZ80_none/L92boot.s $
;
; Modification History
;   $Revision: 6.0.2.2 $
;   $Date: 2010/01/18 06:37:47CST $
;   $Author: odile $
;   $ProjectName: /home/mks/proj/tcpip.pj $
;   $ProjectRevision: 6.0.1.23 $
;*******************************************************************************************************

	public __exit
	extern _main
	extern _tfCrystalIsrHandler

;-------------------------------------------
; Programmable Reload Counter/Timers

TMR0_CTL: .equ %80 
TMR0_DR_L: .equ %81 
TMR0_RR_L: .equ %81 
TMR0_DR_H: .equ %82 
TMR0_RR_H: .equ %82 
TMR1_CTL: .equ %83 
TMR1_DR_L: .equ %84 
TMR1_RR_L: .equ %84 
TMR1_DR_H: .equ %85 
TMR1_RR_H: .equ %85 
TMR2_CTL: .equ %86 
TMR2_DR_L: .equ %87 
TMR2_RR_L: .equ %87 
TMR2_DR_H: .equ %88 
TMR2_RR_H: .equ %88 
TMR3_CTL: .equ %89 
TMR3_DR_L: .equ %8A 
TMR3_RR_L: .equ %8A 
TMR3_DR_H: .equ %8B 
TMR3_RR_H: .equ %8B 
TMR4_CTL: .equ %8C 
TMR4_DR_L: .equ %8D 
TMR4_RR_L: .equ %8D 
TMR4_DR_H: .equ %8E 
TMR4_RR_H: .equ %8E 
TMR5_CTL: .equ %8F 
TMR5_DR_L: .equ %90 
TMR5_RR_L: .equ %90 
TMR5_DR_H: .equ %91 
TMR5_RR_H: .equ %91 
TMR_ISS: .equ %92

;-------------------------------------------
; Watch Dog Timer

WDT_CTL: .equ %93 
WDT_RR: .equ %94 

;-------------------------------------------
; General Purpose I/O

PB_DR: .equ %9A 
PB_DDR: .equ %9B 
PB_ALT1: .equ %9C 
PB_ALT2: .equ %9D 
PC_DR: .equ %9E 
PC_DDR: .equ %9F 
PC_ALT1: .equ %A0 
PC_ALT2: .equ %A1 
PD_DR: .equ %A2 
PD_DDR: .equ %A3 
PD_ALT1: .equ %A4 
PD_ALT2: .equ %A5 

;-------------------------------------------
; Chip Select / Wait State Generator

CS0_LBR: .equ %A8 
CS0_UBR: .equ %A9 
CS0_CTL: .equ %AA 
CS1_LBR: .equ %AB 
CS1_UBR: .equ %AC 
CS1_CTL: .equ %AD 
CS2_LBR: .equ %AE 
CS2_UBR: .equ %AF 
CS2_CTL: .equ %B0 
CS3_LBR: .equ %B1 
CS3_UBR: .equ %B2 
CS3_CTL: .equ %B3 
CS0_BMC: .equ %F0 
CS1_BMC: .equ %F1 
CS2_BMC: .equ %F2 
CS3_BMC: .equ %F3 

SPI_BRG_L: .equ %B8 
SPI_BRG_H: .equ %B9 
SPI_CTL: .equ %BA 
SPI_SR: .equ %BB 
SPI_TSR: .equ %BC 
SPI_RBR: .equ %BC 

IR_CTL: .equ %BF 

UART0_RBR: .equ %C0 
UART0_THR: .equ %C0 
UART0_BRG_L: .equ %C0 
UART0_IER: .equ %C1 
UART0_BRG_H: .equ %C1 
UART0_IIR: .equ %C2 
UART0_FCTL: .equ %C2 
UART0_LCTL: .equ %C3 
UART0_MCTL: .equ %C4 
UART0_LSR: .equ %C5 
UART0_MSR: .equ %C6 
UART0_SPR: .equ %C7 

I2C_SAR: .equ %C8 
I2C_XSAR: .equ %C9 
I2C_DR: .equ %CA 
I2C_CTL: .equ %CB 
I2C_SR: .equ %CC 
I2C_CCR: .equ %CC 
I2C_SRR: .equ %CD 

UART1_RBR: .equ %D0 
UART1_THR: .equ %D0 
UART1_BRG_L: .equ %D0 
UART1_IER: .equ %D1 
UART1_BRG_H: .equ %D1 
UART1_IIR: .equ %D2 
UART1_FCTL: .equ %D2 
UART1_LCTL: .equ %D3 
UART1_MCTL: .equ %D4 
UART1_LSR: .equ %D5 
UART1_MSR: .equ %D6 
UART1_SPR: .equ %D7 

;-------------------------------------------
; Clock and Power-Down

CLK_PPD1: .equ %DB 
CLK_PPD2: .equ %DC 

;-------------------------------------------
; Real-Time Clock

RTC_SEC: .equ %E0 
RTC_MIN: .equ %E1 
RTC_HRS: .equ %E2 
RTC_DOW: .equ %E3 
RTC_DOM: .equ %E4 
RTC_MON: .equ %E5 
RTC_YR: .equ %E6 
RTC_CEN: .equ %E7 
RTC_ASEC: .equ %E8 
RTC_AMIN: .equ %E9 
RTC_AHRS: .equ %EA 
RTC_ADOW: .equ %EB 
RTC_ACTRL: .equ %EC 
RTC_CTRL: .equ %ED 


; CS0: FLASH (disabled)
;  1 Megabyte, lower bound and upper bound chosen so they don't conflict with RAM/etc.
;  bus mode: Intel mode, 2 clocks
CS_LBR0_DATA:	.equ	%f0
CS_UBR0_DATA:	.equ	%ff
CS_CTL0_DATA:	.equ	%00 ; %08
CS_BMC0_DATA:	.equ	%82

; CS1: RAM (enabled, 1 wait state)
;  512 Kilobytes
;  bus mode: eZ80 mode, 1 clock
CS_LBR1_DATA:	.equ	%00    ; Chip select CS1= 28( one wait state ),LB=00, UB=00, (64 KB BOUNDRY )
CS_UBR1_DATA:	.equ	%07
CS_CTL1_DATA:	.equ	%28
CS_BMC1_DATA:	.equ	%01

; CS2: extension RAM (enabled)
;  512 Kilobytes, assumes expansion board is not fully populated
;  bus mode: Intel mode, 1 clock
CS_LBR2_DATA:	.equ	%b8
CS_UBR2_DATA:	.equ	%bf
CS_CTL2_DATA:	.equ	%08
CS_BMC2_DATA:	.equ	%81

; CS3: Crystal LAN chip (enabled, I/O mode)
;  physically wired to start at I/O port base address x300
;  bus mode: Intel mode, 4 clocks
;CS_LBR3_DATA:	.equ	%03
;CS_UBR3_DATA:	.equ	%03
;CS_CTL3_DATA:	.equ	%18
;CS_BMC3_DATA:	.equ	%84

; CS3: Crystal LAN chip (enabled, Memory mode)
;  Memory map address 0xFF0000 - OxFFFFFF
;  bus mode: Intel mode, 4 clocks
CS_LBR3_DATA:	.equ	%FF
CS_UBR3_DATA:	.equ	%FF
CS_CTL3_DATA:	.equ	%08
CS_BMC3_DATA:	.equ	%84


; GPIO Port B
PB_DR_DATA: .equ %FF
PB_DDR_DATA: .equ %FF
PB_ALT1_DATA: .equ %00
PB_ALT2_DATA: .equ %00

; GPIO Port C
PC_DR_DATA: .equ %00
PC_DDR_DATA: .equ %FF
PC_ALT1_DATA: .equ %00
PC_ALT2_DATA: .equ %FF

; GPIO Port D without CS8900 IRQ
;PD_DR_DATA: .equ %FC
;PD_DDR_DATA: .equ %FF
;PD_ALT1_DATA: .equ %00
;PD_ALT2_DATA: .equ %03

; GPIO Port D with CS8900 IRQ
PD_DR_DATA: .equ %FC
PD_DDR_DATA: .equ %EF
PD_ALT1_DATA: .equ %10
PD_ALT2_DATA: .equ %13

; PD4 Jump Table offset
JP_PD4:	.equ	%DC ; 58h * 5 / 2

; Realtime Clock Jump Table offset
JP_RTC:	.equ	%37	; 16h *5 / 2

;
; Save Interrupt Mask
;
SAVEIMASK	macro		; sets a=0 if interrupts disabled else a=1
	ld	a,i		; sets parity bit to value of IEF2
	ld	a,0
	jp	po,$+5		; parity bit is IEF2
	inc	a
	macend

;
; Restore Interrupt Mask
;
RESTOREIMASK	macro		; restore interrupt state
	bit	0,a
	jr	z,$+3
	ei
				; enable intr if enabled on entry
	macend

;*******************************************************************************
;* Section:		.bootstrap                                                     *
;*                                                                             *
;* The bootstrap code contains the SW reset handlers.  The Reset instruction   *
;* is of the form RST n.  This instruction causes the CPU to save the current  *
;* PC to the stack and then starts executing code at {0000nn}.  Valid values   *
;* of n are:	00h, 08h, 10h, 18h, 20h, 28h, 30h, 38h.                        *
;*                                                                             *
;* Note: When the HW is powered on (or Reset), Flash is mapped to the entire   *
;* 24-bit address space (000000-FFFFFF) and the PC is set to 0.  This will     *
;* cause the boot loader to begin executing RST0.                              *
;*                                                                             *
;* Unlike a HW reset, the RST n instruction does not re-initialize the         *
;* peripheral devices.  This is done by the startup routine.                   *
;*                                                                             *
;* Illegal instruction Traps will also reult in the execution of RST0          *
;*******************************************************************************

;    public	__exit
	extern _RZKStartup
	extern _tfCrystalIsrHandler	; cs8900a interrupt handler


	define	.bootstrap, space=RAM, org=000000h
	section .bootstrap
	.assume ADL=1
	.def	_RST0
	.def	_RST8
	.def	_RST10
	.def	_RST18
	.def	_RST20
	.def	_RST28
	.def	_RST30
	.def	_RST38
	.def	_NMI66

_RST0:
	di
	rsmix	
	jp.lil	_ADL_START

_RST8:
	di
	rsmix
	jp.lil	_ADL_START

_RST10:
	di
	rsmix
	jp.lil	_ADL_START

_RST18:
	di
	rsmix
	jp.lil	_ADL_START

_RST20:
	di
	rsmix
	jp.lil	_ADL_START

_RST28:
	di
	rsmix
	jp.lil	_ADL_START

_RST30:
	di
	rsmix
	jp.lil	_ADL_START

_RST38:
	di
	rsmix
	jp.lil	_ADL_START
	
	ds 26

	.org	%66
_NMI66:
	di
	rsmix
	jp.lil	_ADL_START

	ds 90
;*******************************************************************************
;* Section:		IRQVect                                                         *
;*                                                                             *
;* This section will hold the Interrupt Vector Table and the ISR Jump Table.   *
;* When an interrupt occurs on the eZ80 in ADL mode, the CPU retrieves the     *
;* 2-byte vector number from IVT.  It will then start executing code (the ISR) *
;* at address {0 | I | [vect]}.  Therefore, the vector table must be located   *
;* in the first 64kB of addressable memory.  It is unlikely that all ISRs can  *
;* be located in the first 64kB of the address space, so a Jump Table is       *
;* created in memory immediately after the Vector table.  This table just      *
;* contains code of the form jmp.lil 24-bit ISR address.  This allows the ISR  *
;* to exist anywhere in the 24-bit ADL address space.                          *
;*******************************************************************************
	define  .IRQVect, space=RAM, org=000100h
	section .IRQVect
	.def	_IVT_Base
	.def	_IVT_Jump_Table

	IVT_SIZE	.equ	%40
	IV_ENTRY:	MACRO offset
				dw (_IVT_Jump_Table + (5*offset))
	ENDMAC 	IV_ENTRY

	.org	000180h
_IVT_Jump_Table:
	ds    	IVT_SIZE * 5

	.org	000100h
_IVT_Base:
	IV_ENTRY 00h
	IV_ENTRY 01h
	IV_ENTRY 02h
	IV_ENTRY 03h
	IV_ENTRY 04h
	IV_ENTRY 05h
	IV_ENTRY 06h
	IV_ENTRY 07h
	IV_ENTRY 08h
	IV_ENTRY 09h
	IV_ENTRY 0Ah
	IV_ENTRY 0Bh
	IV_ENTRY 0Ch
	IV_ENTRY 0Dh
	IV_ENTRY 0Eh
	IV_ENTRY 0Fh
	IV_ENTRY 10h
	IV_ENTRY 11h
	IV_ENTRY 12h
	IV_ENTRY 13h
	IV_ENTRY 14h
	IV_ENTRY 15h
	IV_ENTRY 16h
	IV_ENTRY 17h
	IV_ENTRY 18h
	IV_ENTRY 19h
	IV_ENTRY 1Ah
	IV_ENTRY 1Bh
	IV_ENTRY 1Ch
	IV_ENTRY 1Dh
	IV_ENTRY 1Eh
	IV_ENTRY 1Fh
	IV_ENTRY 20h
	IV_ENTRY 21h
	IV_ENTRY 22h
	IV_ENTRY 23h
	IV_ENTRY 24h
	IV_ENTRY 25h
	IV_ENTRY 26h
	IV_ENTRY 27h
	IV_ENTRY 28h
	IV_ENTRY 29h
	IV_ENTRY 2Ah
	IV_ENTRY 2Bh
	IV_ENTRY 2Ch
	IV_ENTRY 2Dh
	IV_ENTRY 2Eh
	IV_ENTRY 2Fh
	IV_ENTRY 30h
	IV_ENTRY 31h
	IV_ENTRY 32h
	IV_ENTRY 33h
	IV_ENTRY 34h
	IV_ENTRY 35h
	IV_ENTRY 36h
	IV_ENTRY 37h
	IV_ENTRY 38h
	IV_ENTRY 39h
	IV_ENTRY 3Ah
	IV_ENTRY 3Bh
	IV_ENTRY 3Ch
	IV_ENTRY 3Dh
	IV_ENTRY 3Eh
	IV_ENTRY 3Fh

;*******************************************************************************
;* Section:		.startup                                                        *
;*                                                                             *
;* After a HW reset, all peripheral devices are initialized to a known state.  *
;* However, the state of the peripherals is unknown after a SW reset.  This    *
;* routine ensures none of the peripheral devices will generate interrupts.    *
;* In addition, the I register is set to point to the .IRQVect table.          *
;*******************************************************************************
; program counter must be set to x2c0, so we have space for the interrupt vector table
	define	.startup, space=RAM, org=0002C0h
	section .startup
	.assume ADL=1
	.def _ADL_START

_ADL_START:
;
; At this point we're in ADL mode because we got here
; via a jp.lil at the reset location 000000h
;
;***	Set I register to start of interrupt vector table, use interrupt mode 2
	di
	im		2
	ld		bc, _IVT_Base
	ld		a, b
	ld		i, a


_c_int1:

    .assume ADL=1

;***	Set the Bus Mode for external perihperals
	ld	a,CS_BMC0_DATA
	out0	(CS0_BMC),a
	ld	a,CS_BMC1_DATA
	out0	(CS1_BMC),a
	ld	a,CS_BMC2_DATA
	out0	(CS2_BMC),a
	ld	a,CS_BMC3_DATA
	out0	(CS3_BMC),a

; setup chip selects
	ld	a,CS_LBR0_DATA
	out0	(CS0_LBR),a
	ld	a,CS_UBR0_DATA
	out0	(CS0_UBR),a
	ld	a,CS_CTL0_DATA
	out0	(CS0_CTL),a

	ld	a,CS_LBR1_DATA
	out0	(CS1_LBR),a
	ld	a,CS_UBR1_DATA
	out0	(CS1_UBR),a
	ld	a,CS_CTL1_DATA
	out0	(CS1_CTL),a

	ld	a,CS_LBR2_DATA
	out0	(CS2_LBR),a
	ld	a,CS_UBR2_DATA
	out0	(CS2_UBR),a
	ld	a,CS_CTL2_DATA
	out0	(CS2_CTL),a

	ld	a,CS_LBR3_DATA
	out0	(CS3_LBR),a
	ld	a,CS_UBR3_DATA
	out0	(CS3_UBR),a
	ld	a,CS_CTL3_DATA
	out0	(CS3_CTL),a

;****	Disable all PRTCs
	ld		a, 00h
	out0	(TMR0_CTL), a
	out0	(TMR1_CTL), a
	out0	(TMR2_CTL), a
	out0	(TMR3_CTL), a
	out0	(TMR4_CTL), a
	out0	(TMR5_CTL), a

;***	Disable Watchdog timer
	out0	(WDT_CTL), a

;***	Port B GPIO
	ld		a, PB_DR_DATA
	out0	(PB_DR), a
	ld		a, PB_DDR_DATA
	out0	(PB_DDR), a
	ld		a, PB_ALT1_DATA
	out0	(PB_ALT1), a
	ld		a, PB_ALT2_DATA
	out0	(PB_ALT2), a
;***	Port C GPIO
	ld		a, PC_DR_DATA
	out0	(PC_DR), a
	ld		a, PC_DDR_DATA
	out0	(PC_DDR), a
	ld		a, PC_ALT1_DATA
	out0	(PC_ALT1), a
	ld		a, PC_ALT2_DATA
	out0	(PC_ALT2), a
;***	Port D GPIO
	ld		a, PD_DR_DATA
	out0	(PD_DR), a
	ld		a, PD_DDR_DATA
	out0	(PD_DDR), a
	ld		a, PD_ALT1_DATA
	out0	(PD_ALT1), a
	ld		a, PD_ALT2_DATA
	out0	(PD_ALT2), a

;***	Disable UART interrupts
;	out0( UART0_IIR ), a
;	out0( UART1_IIR ), a
	ld a, 00h
	out0 (UART0_IER), a
	out0 (UART1_IER), a

;***	Set PRTC0..3 input sources as System Clock / Prescaler.
	ld		a, 00h
	out0	(TMR_ISS),  a

;***	Disable IR
	out0( IR_CTL ), a

;***	Disable I2C interrupts
	out0( I2C_CTL ), a

;***	Apply system clock to all peripherals
	out0( CLK_PPD1 ), a
	out0( CLK_PPD2 ), a

;***	Disable RTC interrupts
	out0( RTC_CTRL ), a

;***	Disable SPI interrupts
	ld		a, 04
	out0( SPI_CTL ), a

;*** 	Make sure flash in Read Array Mode.
;		If your platform does not include a Flash, remove this block of code.
;		Different Flash memories will use different commands to ensure the Flash
; 		is available for read access.  This command is for the MT28F008B3.
;--------------------------------------------------------------------------------
;	.extern	.FLASH_BASE
;	ld	a, 0FFh
;	ld hl, .FLASH_BASE
;	ld (hl), a

	
;***	Setup the stack
; 		It is not necessrary to setup the Z80 stack if the system is left in     
;		ADL mode.
;--------------------------------------------------------------------------------
	extern	.TOS
	ld.sis	sp, 0FFFFh	; Setup SPS
	ld.lil	sp, .TOS 	; Setup SPL


;*** 	Initialize the .BSS section to zero
	extern	.BSS_BASE
	extern	.BSS_LENGTH
	ld		bc, .BSS_LENGTH
	ld		hl, 0		
	or 	    a, a
	sbc	    hl, bc
	jr		z, _c_bss_done

	ld		hl, .BSS_BASE	

	ld		(hl), 0				; zero first byte of BSS
	dec		bc					; reduce count
	ld		hl, 0	
	or 	    a, a
	sbc	    hl, bc	
	jr		z, _c_bss_done	
	ld		hl, .BSS_BASE	
	ld		de, .BSS_BASE+1	
	ldir
_c_bss_done:


; --------------------------------------------------------------------------
; Re-Initialize Variables
; --------------------------------------------------------------------------
;***	Re-Initialize Ram Variables
	extern	__low_data
	extern	__len_data
	extern 	__low_romdata
	;* If the length is 0 we're done
	ld		hl, __len_data
	ld		bc, 0		
	or		a, a		
	sbc		hl, bc		
	jr		z, _copy_data_done	

	ld		de, __low_data
	ld		hl, __low_romdata
	ld		bc, __len_data
	ldir
_copy_data_done:

;***	Initialize interrupt jump table 
    ld      hl,_IVT_Jump_Table
    ld      b,IVT_SIZE
    ld      iy,__default_hndlr
$1:
    ld      (hl),05bh     ; jp.lil Mmn
    inc     hl
    ld      (hl),0c3h
    inc     hl
    ld      (hl),iy       ; __default_hndlr
    inc     hl
    inc     hl
    inc     hl
    dec     b
    jr      nz,$1

;***	Instarll cs8900a interrupt handler
	ld		hl, JP_PD4		; Interrupt jump table offset
    ld      bc,_IVT_Jump_Table
    add     hl,bc           ; hl is address of jp
	inc		hl
	inc		hl
    ld      bc,_tfCrystalIsrHandler	; Interrupt handler
    ld      (hl),bc         ; store new vector address

;***	Enable interrupts	
    ei

;*** 	Call 'C' entry point.
;		Be sure to set up any parameters required by the called routine.
; 		For eZ80 assembly, Param1 is in DE, 
;	   	    				 Param2 is in HL
;								 Param3 is in BC
; 		If more than 3 parameters or a variable parameter list is used,
; 		parameters must be placed on the stack.
;---------------------------------------------
	ld	hl,0		; hl=NULL
	push	hl		; argv[0] = NULL
	ld	ix,0
	add	ix,sp		; ix=&argv[0]
	push	ix		; &argv[0]
	
	pop	hl
	ld	de,0		; argc==0
	call	_main		; main()
	pop	af		; clean the stack
__exit:
	jr	$		; ?

;************************************************************************
;* Default interrupt handler
;************************************************************************
__default_hndlr:
    reti.l


;************************************************************************
;* Crystal CS8900A interrupt handler
;************************************************************************
;__cs8900a_isr_hndlr:
;	di
;	call.l _tfCrystalIsrHandler
;	ei
;   reti.l

;************************************************************************
;* Define __set_vector to install a user interrupt handler
;*
;*      void _set_vector(unsigned short vector,void (*hndlr)(void));
;*
;*      NOTE: THIS CODE DOES NOT INSERT THE JP BECAUSE IT ASSUMES
;*            THAT __init_default_vectors HAS ALREADY BEEN CALLED.
;*
;*      (IX+9)  hndlr - address of user interrupt handler
;*      (IX+6)  vector - defined in ez80.h. Actually an index into
;*                       __jump_table
;*
;************************************************************************
    .def    __set_vector
__set_vector:
    push    ix
    ld      ix,0
    add     ix,sp           ; Standard prologue
	push	af
	SAVEIMASK
    ld      hl,0            ; Clear UHL
;    ld.sis  hl,(ix+6)       ; Vector offset
    ld  	hl,(ix+6)       ; Vector offset
    ld      bc,_IVT_Jump_Table
    add     hl,bc           ; hl is address of jp
    inc     hl              ; hl is address of handler
    ld      bc,(ix+9)       ; handler
    ld      (hl),bc         ; store new vector address
	RESTOREIMASK
	pop	af
    ld      sp,ix           ; Standard epilogue
    pop     ix
    ret.l

;***	Storage for Linker defined variables
;	.extern		.FLASH_BASE
;	.extern		.TOTAL_FLASH
	.extern 	.RAM_BASE
	.extern 	.RAM_LENGTH
	.extern 	.TOS
	.extern 	.CODE_SIZE
	.extern 	.DATA_SIZE
	.extern		.INT_VT_BASE
	.extern		.INT_JT_BASE
;	.def		_flash_ptr
;	.def		_flash_length
	.def		_ram_base
	.def		_ram_length
	.def		_TOS
	.def		_code_size
	.def		_data_size
	.def		_p_INT_VT_base
	.def		_p_INT_JT_base

;_flash_ptr:
;	.trio		.FLASH_BASE
;_flash_length:
;	.trio		.TOTAL_FLASH
_ram_base:
	.trio		.RAM_BASE
_ram_length:
	.trio		.RAM_LENGTH
_TOS:
	.trio		.TOS
_code_size:
	.trio		.CODE_SIZE
_data_size:
	.trio		.DATA_SIZE
_p_INT_VT_base:
	.trio		.INT_VT_BASE
_p_INT_JT_base:
	.trio		.INT_JT_BASE





 

	 

