/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: eZ80 L92/F91 + demo application (without RTOS)
 *
 * Filename: txz80app.c
 * Author: Qin Zhang
 * Date Created: 10/1/02
 * $Source: source/kernel/eZ80_none/txz80app.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:37:48CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trzuart.h>
#ifdef _EZ80F91
#include <tremac.h>
#endif /* _EZ80F91 */
#ifdef _EZ80L92
#include <trcrystl.h>
#endif /* _EZ80L92 */


/*
 * Local Macro Definitions
 */
/*
 * Use async interface: if you want to use serial interface, uncomment
 * the following macro
 */
/* #define TM_USE_SERIAL_INTERFACE */

/*
 * Use Ethernet interface: if you do not want to use Ethernet interface,
 * comment out the following macro
 */
#define TM_USE_ETHERNET_INTERFACE

/*
 * System options
 */
#define TM_DEMO_FTPD      1
#define TM_DEMO_UDPECHO   1
#define TM_DEMO_TCPECHO   1
#define TM_DEMO_PPP       0
#define TM_DEMO_PERFTEST  0

/* PPP loopback test */
/* #define TM_PPP_LOOPBACK_TEST */

/* Ram file system size */
#define TM_RAMFS_BLOCKS     64

#if TM_DEMO_PPP
#ifndef TM_USE_SERIAL_INTERFACE
#define TM_USE_SERIAL_INTERFACE
#endif /* TM_USE_SERIAL_INTERFACE */
#endif /* TM_DEMO_PPP */

/* IP address & mask */
#ifdef TM_USE_SERIAL_INTERFACE
#define TM_SERIAL_OUR_IP_ADDR      "192.168.100.10"
#define TM_SERIAL_PEER_IP_ADDR     "192.168.100.20"
#endif /* TM_USE_SERIAL_INTERFACE */

#ifdef TM_USE_ETHERNET_INTERFACE
#define TM_ETHER_OUR_IP_ADDR      "12.99.240.215"
#define TM_ETHER_PEER_IP_ADDR     "12.99.240.205"
#endif /* TM_USE_ETHERNET_INTERFACE */
#define TM_NET_MASK               "255.255.255.0"

/* Echo port define */
#define TM_ECHO_PORT        7
#define TM_DISCARD_PORT     9

/* Number of times to send the data buffer to the remote host */
#define TEST_PKT_COUNT 1000

/*
 * PPP application states (for TCP discard server).
 */
#define TM_INITIAL    0
#define TM_ACCEPT     1
#define TM_RECV       2
#define TM_APPERROR   3

/* define kerword interrupt */
#ifdef TM_KERNEL_NONE_EZ80
#define TM_INTERRUPT interrupt
#else /* TM_KERNEL_NONE_EZ80 */
#define TM_INTERRUPT
#endif /* TM_KERNEL_NONE_EZ80 */

/* Register definitions */
#ifdef _EZ80L92
/* PRT0(TIMER0) reg on eZ80 board */
#define TM_Z80_TMR0_CTL     ((TM_INTIO)0x80)  /* control reg          */
#define TM_Z80_TMR0_RR_L    ((TM_INTIO)0x81)  /* reload reg low       */
#define TM_Z80_TMR0_RR_H    ((TM_INTIO)0x82)  /* reload reg high      */
#define TM_Z80_TMR_ISS      ((TM_INTIO)0x92)  /* input source select  */

/* PD reg on eZ80 board */
#define TM_Z80_PD_DR        ((TM_INTIO)0xA2)
#define TM_Z80_PD_DDR       ((TM_INTIO)0xA3)
#define TM_Z80_PD_ALT1      ((TM_INTIO)0xA4)
#define TM_Z80_PD_ALT2      ((TM_INTIO)0xA5)
#endif /* _EZ80L92 */

#ifdef _EZ80F91
/* PRT0(TIMER0) reg on eZ80 board */
#define TM_Z80_TMR0_CTL     ((TM_INTIO)0x60)  /* control reg            */
#define TM_Z80_TMR0_IER     ((TM_INTIO)0x61)  /* interrupt enable reg   */
#define TM_Z80_TMR0_IIR     ((TM_INTIO)0x62)  /* int identification reg */
#define TM_Z80_TMR0_RR_L    ((TM_INTIO)0x63)  /* reload reg low         */
#define TM_Z80_TMR0_RR_H    ((TM_INTIO)0x64)  /* reload reg high        */

/* PD reg on eZ80 board */
#define TM_Z80_PD_DR        ((TM_INTIO)0xA2)
#define TM_Z80_PD_DDR       ((TM_INTIO)0xA3)
#define TM_Z80_PD_ALT1      ((TM_INTIO)0xA4)
#define TM_Z80_PD_ALT2      ((TM_INTIO)0xA5)
#endif /* _EZ80F91 */

/*
 * Timer0 regs value
 */
/*
 * Reload Value = SYSTEM CLOCK / DIV / TIMER RATE
 *              = 48M/256(DIV)/500
 *              = 375 = 177H
 */
#define TM_TMR0_RR_L_DATA   0x77
#define TM_TMR0_RR_H_DATA   0x01

/*
 * Reload Value = SYSTEM CLOCK / DIV / TIMER RATE
 *              = 48M/256(DIV)/100
 *              = 1875 = 753H
 */
#define TM_TMR1_RR_L_DATA   0x53
#define TM_TMR1_RR_H_DATA   0x07

#ifdef _EZ80L92
/* Timer0 Contrl reg value */
#define TM_TMR0_CTL_DATA    0x5F

/* Timer1 Contrl reg value */
#define TM_TMR1_CTL_DATA    0x5F

/* Timer0 ISS reg value */
#define TM_TMR0_ISS_MASK    0x03
#define TM_TMR0_ISS_DATA    0x00 /* 00 : system clock */

/* Timer1 ISS reg value */
#define TM_TMR1_ISS_MASK    0x0C
#define TM_TMR1_ISS_DATA    0x00 /* 00 : system clock */
#endif _EZ80L92

#ifdef _EZ80F91
/*
 * Timer0 Contrl reg value:
 * BRK_STOP + system clock + 256 DIV + TIM_CONT + RLD + TIM_EN
 */
#define TM_TMR0_CTL_DATA    0x9F

/*
 * Timer1 Contrl reg value:
 * BRK_STOP + system clock + 256 DIV + TIM_CONT + RLD + TIM_EN
 */
#define TM_TMR1_CTL_DATA    0x9F
#endif /* _EZ80F91 */

/* PD registers value */
#define TM_Z80_PD_DR_DATA        0xFC
#define TM_Z80_PD_DDR_DATA       0xEF
#define TM_Z80_PD_ALT1_DATA      0x10
#define TM_Z80_PD_ALT2_DATA      0x13

#ifdef _EZ80L92
/* Interrupt Jump table base for L92 */
#define TM_IV_BASE          0x180

/* Timer0 Jump table entry offset */
#define TM_IV_TMR0        0x19

/* Timer1 Jump table entry offset */
#define TM_IV_TIMER1        0x1E
#endif /* _EZ80L92 */

#ifdef _EZ80F91
/* Interrupt Vector table base for F91 */
#define TM_IV_BASE          0x0

/* Timer0 Jump table entry offset */
#define TM_IV_TMR0        0x54

/* Timer1 Jump table entry offset */
#define TM_IV_TIMER1        0x58
#endif /* _EZ80F91 */

/* eZ80 reg macro */
#define tm_z80_reg(x)  ((x)[0])

/* Tyep defines */
typedef volatile unsigned char __INTIO * TM_INTIO;
/* User socket address pointer union */
typedef union tuUserSockPtr
{
    struct sockaddr_in TM_FAR   *sockInPtr;
    struct sockaddr TM_FAR      *sockPtr;
} ttUserSockPtrUnion;


/*
 * Global Variable Definitions
 */
#if (TM_DEMO_PPP || defined(TM_USE_SERIAL_INTERFACE))
/* PPP connection status */
int tlPppConnectedStatus = 0;
#endif /* (TM_DEMO_PPP || (defined(TM_USE_SERIAL_INTERFACE)) */


/*
 * External function definitions
 */
extern int tfRamfsInit(ttUser16Bit blCount);

/*
 * Function prototype definitions
 */
void main(void);
ttUserInterface tfStartupEthernet( ttUserIpAddress ipAddress,
                                   ttUserIpAddress netMask);
ttUserInterface tfStartupSerial( ttUserIpAddress ipAddress,
                                 ttUserIpAddress netMask);
void tfPppNotifyFunc(ttUserInterface pInterface, unsigned int flags);
void tfInstallTimerIsr0(void);
#ifdef _EZ80L92
void tfInitEthernet(void);
#endif /* _EZ80L92 */
void tfInitUart0 ( void );

void TM_INTERRUPT tfTimerIsrHandler(void);


#if TM_DEMO_PPP
void tfPppTestExecute(void);
#endif /* TM_DEMO_PPP */

#if TM_DEMO_PERFTEST
int tfPerfTestStart(ttUserTestHandlePtr perfTestHandlePtr);
#endif /* TM_DEMO_PERFTEST */

#ifdef TM_USE_SERIAL_INTERFACE
void tfPppLinkNotify(ttUserInterface interfaceHandle, int flag);
int myHandleEchoReply(
    ttUserInterface interfaceHandle,
    ttUser8Bit echoRequestId,
    const char* dataPtr,
    int dataLen);
#endif /* TM_USE_SERIAL_INTERFACE */

#ifdef TM_PPP_LQM
ttUser8Bit myLinkQualityMonitor(
    ttUserInterface interfaceHandle,
    int reasonCode,
    unsigned long timeElapsedMsec,
    ttLqrCountDeltasPtr countDeltasPtr,
    ttConstLqrCountsPtr countsPtr,
    ttUser32Bit outLqrs,
    ttUser32Bit outPackets,
    ttUser32Bit outOctets );
#endif /* TM_PPP_LQM */

void tfZ80InstallIsr( ttUserIsrHandlerPtr   funcPtr,
                      int                   ivBase,
                      ttUser8Bit            offset);

/* Functions */
void main(void)
{
	int                 errorCode;
#if TM_DEMO_UDPECHO
    ttUserTestHandle    udpEchoHandle;
#endif /* TM_DEMO_UDPECHO */
#if TM_DEMO_TCPECHO
    ttUserTestHandle    tcpEchoHandle;
#endif /* TM_DEMO_TCPECHO */
#if TM_DEMO_PERFTEST
    ttUserTestHandle    perfTestHandle;
#endif /* TM_DEMO_PERFTEST */
#ifdef TM_USE_ETHERNET_INTERFACE
    ttUserInterface     ethernetHandle;
#endif /* TM_USE_ETHERNET_INTERFACE */
#ifdef TM_USE_SERIAL_INTERFACE
    ttUserInterface     serialHandle;
#endif /* TM_USE_SERIAL_INTERFACE */

    tfInitTreck();
/* Set tick length */
    errorCode = tfInitTreckOptions(TM_OPTION_TICK_LENGTH, 10);

/* Start the Treck TCP/IP Stack */
	tfStartTreck();

#ifdef TM_USE_ETHERNET_INTERFACE
#ifdef _EZ80L92
/* Initialize Ethernet */
    tfInitEthernet();
#endif /* _EZ80L92 */

/* Start the Treck stack: add and open the ethernet interface */
    ethernetHandle = tfStartupEthernet( inet_addr(TM_ETHER_OUR_IP_ADDR),
                                        inet_addr(TM_NET_MASK));

    if( ethernetHandle == 0 )
    {
        tfKernelError("main","Failed to startup ethernet");
    }
#endif /* TM_USE_ETHERNET_INTERFACE */

#ifdef TM_USE_SERIAL_INTERFACE
/* Initialize UART */
    tfInitUart0();

/* Start the Treck stack: add and open the PPP interface */
    serialHandle = tfStartupSerial( inet_addr(TM_SERIAL_OUR_IP_ADDR),
                                    inet_addr(TM_NET_MASK));

    if( serialHandle == 0 )
    {
        tfKernelError("main","Failed to startup serial");
    }
#endif /* TM_USE_SERIAL_INTERFACE */

/* Config timer1 ISR & install TRECK timer ISR handler */
    tfInstallTimerIsr0();

#if TM_DEMO_FTPD
/* Initialize RAM file system */
    (void)tfRamfsInit(TM_RAMFS_BLOCKS);

    errorCode = tfFtpdUserStart(TM_FS_ALLCMND_MASK,/* File command allowed */
                                2,                 /* Max connections      */
                                2,                 /* Max concurrent conns */
                                1200,              /* Idle timeout(seconds)*/
                                TM_BLOCKING_OFF ); /* Blocking state       */
    if( errorCode != TM_ENOERROR )
    {
        tfKernelError("main","Failed to start ftp server");
    }
#endif /* TM_DEMO_FTPD */

#if TM_DEMO_UDPECHO
    errorCode = tfTestTreck(  TM_TEST_UDP_ECHO_SERVER,
                              TM_ETHER_OUR_IP_ADDR,
                              htons(TM_ECHO_PORT),
                              64,
                              0,
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_UDP_CS_OFF
                              | TM_TEST_FLAG_NONBLOCKING),
                              &udpEchoHandle,
                              (void TM_FAR *)0UL );
    if( errorCode != TM_EWOULDBLOCK )
    {
        tfKernelError("main","Failed to start UDP echo server");
    }
#endif /* TM_DEMO_UDPECHO */

#if TM_DEMO_TCPECHO
    errorCode = tfTestTreck(  TM_TEST_TCP_ECHO_SERVER,
                              TM_ETHER_OUR_IP_ADDR,
                              htons(TM_ECHO_PORT),
                              1,
                              0,
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
                              | TM_TEST_FLAG_NONBLOCKING),
                              &tcpEchoHandle,
                              (void TM_FAR *)0UL);
    if( errorCode != TM_EWOULDBLOCK )
    {
        tfKernelError("main","Failed to start TCP echo server");
    }
#endif /* TM_DEMO_TCPECHO */

#if TM_DEMO_PERFTEST
    errorCode = tfPerfTestStart( &perfTestHandle );
    if( errorCode != TM_EWOULDBLOCK )
    {
        tfKernelError("main","Failed to start performance test");
    }
#endif /* TM_DEMO_PERFTEST */

/* Main loop */
    while(1)
    {
/*
 * Allow the Treck stack to execute any timer related tasks that need to be
 * handled (retransmissions, etc).
 */
		tfTimerExecute();

#if TM_DEMO_PPP
        tfPppTestExecute();
#endif /* TM_DEMO_PPP */

#if TM_DEMO_FTP
        errorCode = tfFtpdUserExecute();
        if( errorCode != TM_ENOERROR )
        {
            tfKernelError("main","FTPD execute error");
        }
#endif /* TM_DEMO_FTP */

#if TM_DEMO_UDPECHO
        errorCode = tfTestUserExecute(udpEchoHandle);
        if( errorCode != TM_EWOULDBLOCK )
        {
            tfKernelError("main","UDP echo execute error");
        }
#endif /* TM_DEMO_UDPECHO */

#if TM_DEMO_TCPECHO
        errorCode = tfTestUserExecute(tcpEchoHandle);
        if( errorCode != TM_EWOULDBLOCK )
        {
            tfKernelError("main","TCP echo execute error");
        }
#endif /* TM_DEMO_TCPECHO */

#if TM_DEMO_PERFTEST
        errorCode = tfTestUserExecute(perfTestHandle);
        if( errorCode != TM_EWOULDBLOCK )
        {
            tfKernelError("main","Performance test execute error");
        }
#endif /* TM_DEMO_PERFTEST */

#ifdef TM_USE_ETHERNET_INTERFACE
		if( !tfCheckReceiveInterface(ethernetHandle) )
		{
			errorCode = tfRecvInterface(ethernetHandle);
			if(errorCode != TM_ENOERROR)
            {
                tfKernelError("main","Ethernet receive error");
            }
		}
#endif /* TM_USE_ETHERNET_INTERFACE */

#ifdef TM_USE_SERIAL_INTERFACE
		if( !tfCheckReceiveInterface(serialHandle) )
		{
			errorCode = tfRecvInterface(serialHandle);
			if(errorCode != TM_ENOERROR)
            {
                tfKernelError("main","Serial interface receive error");
            }
		}
#endif /* TM_USE_SERIAL_INTERFACE */
    }

    return;
}


#if TM_DEMO_PERFTEST

#define TM_PERF_TEST	4
/* Performance test start function */
int tfPerfTestStart(ttUserTestHandlePtr perfTestHandlePtr)
{
    int                 errorCode;

    errorCode = TM_ENOERROR;
#if (TM_PERF_TEST == 1)
/* 1.1 UDP short packet send test: disable UDP checksum */
    errorCode = tfTestTreck(  TM_TEST_UDP_SEND,         /* test type */
                              TM_ETHER_PEER_IP_ADDR,    /* ip address */
                              htons(TM_DISCARD_PORT),   /* port number */
                              36,                       /* data size */
                              10000,                    /* test count */
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_NONBLOCKING
                              | TM_TEST_FLAG_UDP_CS_OFF), /* flags */
                              perfTestHandlePtr,        /* call back */
                              (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 1) */

#if (TM_PERF_TEST == 2)
/* 1.2 UDP short packet send test: enable UDP checksum */
    errorCode = tfTestTreck(  TM_TEST_UDP_SEND,         /* test type */
                              TM_ETHER_PEER_IP_ADDR,    /* ip address */
                              htons(TM_DISCARD_PORT),   /* port number */
                              36,                       /* data size */
                              10000,                    /* test count */
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_NONBLOCKING), /* flags */
                              perfTestHandlePtr,        /* call back */
                              (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 2) */

#if (TM_PERF_TEST == 3)
/* 2.1 UDP long packet send test: disable UCP checksum */
    errorCode = tfTestTreck(  TM_TEST_UDP_SEND,         /* test type */
                              TM_ETHER_PEER_IP_ADDR,    /* ip address */
                              htons(TM_DISCARD_PORT),   /* port number */
                              1472,                     /* data size */
                              10000,                    /* test count */
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_NONBLOCKING
                              | TM_TEST_FLAG_UDP_CS_OFF), /* flags */
                              perfTestHandlePtr,        /* call back */
                              (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 3) */

#if (TM_PERF_TEST == 4)
/* 2.2 UDP long packet send test: enable UCP checksum */
    errorCode = tfTestTreck(  TM_TEST_UDP_SEND,         /* test type */
                              TM_ETHER_PEER_IP_ADDR,    /* ip address */
                              htons(TM_DISCARD_PORT),   /* port number */
                              1472,                     /* data size */
                              10000,                    /* test count */
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_NONBLOCKING), /* flags */
                              perfTestHandlePtr,        /* call back */
                              (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 4) */

#if (TM_PERF_TEST == 5)
/* 3.1 TCP short packet send test: TCP_NODELAY */
    errorCode = tfTestTreck(  TM_TEST_TCP_SEND,         /* test type */
                              TM_ETHER_PEER_IP_ADDR,    /* ip address */
                              htons(TM_DISCARD_PORT),   /* port number */
                              12,                       /* data size */
                              10000,                    /* test count */
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_NONBLOCKING
                              | TM_TEST_FLAG_TCP_NODELAY), /* flags */
                              perfTestHandlePtr,    /* call back */
                              (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 5) */

#if (TM_PERF_TEST == 6)
/* 4 TCP long packet send test */
    errorCode = tfTestTreck(  TM_TEST_TCP_SEND,         /* test type */
                              TM_ETHER_PEER_IP_ADDR,    /* ip address */
                              htons(TM_DISCARD_PORT),   /* port number */
                              1448,                     /* data size */
                              10000,                    /* test count */
                              (unsigned int)(TM_TEST_FLAG_ZEROCOPY
							  | TM_TEST_FLAG_NONBLOCKING), /* flags */
                              perfTestHandlePtr,        /* call back */
                              (void TM_FAR *)0UL);      /* parameter */
#endif /* (TM_PERF_TEST == 7) */

    return errorCode;
}
#endif /* TM_DEMO_PERFTEST */


/*
 * System internal timer call back function(ISR)
 */
void TM_INTERRUPT tfTimerIsrHandler(void)
{
/* Update timer value */
	tfTimerUpdateIsr();
    return;
}

void tfInstallTimerIsr0(void)
{
    unsigned char tmp;

/* Disable timer 0 */
    tm_z80_reg(TM_Z80_TMR0_CTL) = 0x00;

/* Set timer 0 input source select */
#ifdef _EZ80L92
    tmp = tm_z80_reg(TM_Z80_TMR_ISS);
    tmp &= ~TM_TMR0_ISS_MASK;
    tmp |= (TM_TMR0_ISS_MASK & TM_TMR0_ISS_DATA);
    tm_z80_reg(TM_Z80_TMR_ISS) = tmp;
#endif /* _EZ80L92 */

/* Set timer 0 reload register */
    tm_z80_reg(TM_Z80_TMR0_RR_L) = TM_TMR0_RR_L_DATA;
    tm_z80_reg(TM_Z80_TMR0_RR_H) = TM_TMR0_RR_H_DATA;

/* Install isr handler */
    tfZ80InstallIsr(tfTimerIsrHandler, TM_IV_BASE, TM_IV_TMR0);

/* Enable timer 0 */
    tm_z80_reg(TM_Z80_TMR0_CTL) = TM_TMR0_CTL_DATA;

    return;
}


/*
 * tfInitEthernet description:
 * Initialize board settings for Crystal 8900a chipset: Config PD4 for
 * cs8900a interrupt input.
 */
#ifdef _EZ80L92
void tfInitEthernet(void)
{
    unsigned char value;

/* Config PD4 for cs8900a interrupt */
    value = tm_z80_reg(TM_Z80_PD_DR);
    tm_z80_reg(TM_Z80_PD_DR)    = value | 0x10;
    value = tm_z80_reg(TM_Z80_PD_DDR);
    tm_z80_reg(TM_Z80_PD_DDR)   = value & 0xEF;
    value = tm_z80_reg(TM_Z80_PD_ALT1);
    tm_z80_reg(TM_Z80_PD_ALT1)  = value | 0x10;
    value = tm_z80_reg(TM_Z80_PD_ALT2);
    tm_z80_reg(TM_Z80_PD_ALT2)  = value | 0x10;

    return;
}
#endif /* _EZ80L92 */


/*
 * Perform system startup (init link layer, etc)
 */
ttUserInterface tfStartupEthernet( ttUserIpAddress ipAddress,
                                   ttUserIpAddress netMask)
{
	ttUserLinkLayer linkLayerHandle;
    ttUserInterface interfaceHandle;
	int             errorCode;

/* Use the Ethernet Link Layer (Including ARP) */
	linkLayerHandle=tfUseEthernet();
	
/* Add our interface */
#ifdef _EZ80L92
    interfaceHandle = tfAddInterface( "CRYSTAL", 
                                      linkLayerHandle,
                                      tfCrystalOpen,
                                      tfCrystalClose,
									  tfCrystalSend,
                                      tfCrystalReceive,
                                      (ttDevFreeRecvFuncPtr)0,
                                      tfCrystalIoctl,
                                      tfCrystalGetPhysicalAddress,
                                      &errorCode);
#endif /* _EZ80L92 */
#ifdef _EZ80F91
    interfaceHandle = tfAddInterface( "CRYSTAL", 
                                      linkLayerHandle,
                                      tfEmacOpen,
                                      tfEmacClose,
									  tfEmacSend,
                                      tfEmacReceive,
                                      tfEmacFreeRecvBuf,
                                      tfEmacIoctl,
                                      tfEmacGetPhysicalAddress,
                                      &errorCode);
#endif /* _EZ80F91 */
    if( errorCode != TM_ENOERROR )
    {
        tfKernelError("tfStartupEthernet","Failed to add interface");
    }
	
/* Config the Interface */
	errorCode = tfConfigInterface(  interfaceHandle,
			                        ipAddress,
			                        netMask,
			                        0,
			                        1,
			                        0);
	
	if( errorCode != TM_ENOERROR )
	{
        errorCode = tfCloseInterface(interfaceHandle);
        interfaceHandle = (ttUserInterface)0;
	}
	
	return(interfaceHandle);
}

#ifdef TM_USE_SERIAL_INTERFACE
/* Fot test UART */
/* #define TM_UART_TEST */
void tfInitUart0 ( void ) 
{
    unsigned char       value;
#ifdef TM_UART_TEST
	char				testSend[] = "Qin";
#endif /* TM_UART_TEST */

/*  uzictl0 = UARTMODE; */
/*    tm_z80_reg((TM_INTIO)0xCF) = 0x01; */

/* PD0 = uart0_tx, PD1 = uart0_rx */
    value = tm_z80_reg(TM_Z80_PD_ALT1);
	tm_z80_reg(TM_Z80_PD_ALT1) = value & 0xFC;
    value = tm_z80_reg(TM_Z80_PD_ALT2);
	tm_z80_reg(TM_Z80_PD_ALT2) = value | 0x03;

/* PD_DDR is not read/write, but WRITE-ONLY */
    value = tm_z80_reg(TM_Z80_PD_DDR);
	tm_z80_reg(TM_Z80_PD_DDR) = value | 0x03;

/*
 * Config Infrared Encoder/Decoder Control Register:
 * Disable Encoder/Decoder, Pass IR_RXD data to UART0 RxD
 */
/*    tm_z80_reg(TM_Z80_IR_CTL) = 0x02; */

#ifdef TM_UART_TEST
    tfUartOpen(tlSerialIfHandle);
    (void)tfUartSend(tlSerialIfHandle, testSend, 3, 0);
    tm_thread_stop;
#endif /* TM_UART_TEST */
    return;
}  


/*
 * Perform system startup (init link layer, etc)
 */
ttUserInterface tfStartupSerial( ttUserIpAddress ipAddress,
                                 ttUserIpAddress netMask)
{
	ttUserLinkLayer linkLayerHandle;
    ttUserInterface interfaceHandle;
	int             errorCode;
	ttUser8Bit      timeoutValue;
#ifdef TM_PPP_LQM
    ttUser32Bit     reportingPeriod;
#endif /* TM_PPP_LQM */
    ttUser8Bit      magicNumberStatus;
	ttUserIpAddress peerIpAddress;

/* Use the Serial Link Layer */
#ifdef TM_ASYNC_SERVER
    linkLayerHandle = tfUseAsyncServerPpp( tfPppNotifyFunc );
#else /* TM_ASYNC_SERVER */
    linkLayerHandle = tfUseAsyncPpp( tfPppNotifyFunc );
#endif /* TM_ASYNC_SERVER */
	
/* Add our interface */
    interfaceHandle = tfAddInterface( "SERIAL", 
                                      linkLayerHandle,
                                      tfUartOpen,
                                      tfUartClose,
									  tfUartSend,
                                      tfUartReceive,
                                      tfUartFreeReceiveBuffer,
                                      tfUartIoctl,
                                      0,
                                      &errorCode);
    if( errorCode != TM_ENOERROR )
    {
        tfKernelError("tfStartupSerial","Failed to add serial interface");
    }

/* Set options */
#ifdef TM_PPP_LQM
    if (errorCode == TM_ENOERROR)
    {
        errorCode = tfUsePppLqm(
            interfaceHandle,
            (ttUser32Bit) 3000);

        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","Use PPP LQM failed");
        }
    }
#endif /* TM_PPP_LQM */

    if (errorCode == TM_ENOERROR)
    {
        timeoutValue = 20;
        errorCode = tfPppSetOption( interfaceHandle, TM_PPP_LCP_PROTOCOL,
					 TM_PPP_OPT_WANT, TM_LCP_TIMEOUT, 
					 (const char TM_FAR *)&timeoutValue, 
					 1 );
        
        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","PPP set option failed");
        }
    }

    if (errorCode == TM_ENOERROR)
    {
        magicNumberStatus = 1;
        errorCode = tfPppSetOption(
            interfaceHandle,
            TM_PPP_LCP_PROTOCOL,
            TM_PPP_OPT_WANT,
            TM_LCP_MAGIC_NUMBER,
            (const char*) &magicNumberStatus,
            1);
        
        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","PPP set option failed");
        }
    }

#ifdef TM_PPP_LQM
    if (errorCode == TM_ENOERROR)
    {
        reportingPeriod = 5000;
        errorCode = tfPppSetOption(
            interfaceHandle,
            TM_PPP_LCP_PROTOCOL,
            TM_PPP_OPT_WANT,
            TM_LCP_QUALITY_PROTOCOL,
            (const char*) &reportingPeriod,
            4);
        
        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","PPP set option failed");
        }
    }

    if (errorCode == TM_ENOERROR)
    {
        reportingPeriod = 5000;
        errorCode = tfPppSetOption(
            interfaceHandle,
            TM_PPP_LCP_PROTOCOL,
            TM_PPP_OPT_ALLOW,
            TM_LCP_QUALITY_PROTOCOL,
            (const char*) &reportingPeriod,
            4);
        
        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","PPP set option failed");
        }
    }
#endif /* TM_PPP_LQM */

    if (errorCode == TM_ENOERROR)
    {
        peerIpAddress=inet_addr(TM_SERIAL_PEER_IP_ADDR);
        errorCode = tfPppSetOption(
            interfaceHandle,
            TM_PPP_IPCP_PROTOCOL,
            TM_PPP_OPT_ALLOW,
            TM_IPCP_IP_ADDRESS,
            (char *) &peerIpAddress,
            4);

        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","PPP set option failed");
        }
    }

    if (errorCode == TM_ENOERROR)
    {
        errorCode = tfPppSetOption(
            interfaceHandle,
            TM_PPP_IPCP_PROTOCOL,
            TM_PPP_OPT_WANT,
            TM_IPCP_IP_ADDRESS,
            (char *) &ipAddress,
            4);

        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial","PPP set option failed");
        }
    }

    if (errorCode == TM_ENOERROR)
    {
/* Configure the interface */
        errorCode=tfConfigInterface(
            interfaceHandle,
            ipAddress,
            netMask,
            0,
            1,
			0);
        
        if (errorCode != TM_ENOERROR)
        {
            tfKernelWarning("tfStartupSerial",
                            "Config serial interface failed");
        }
    }

	if( errorCode != TM_ENOERROR )
	{
	    (void)tfCloseInterface(interfaceHandle);
#ifdef TM_PPP_LQM
        (void)tfFreePppLqm(interfaceHandle);
#endif /* TM_PPP_LQM */
        interfaceHandle = (ttUserInterface)0;
	}
	
	return(interfaceHandle);
}


void tfPppNotifyFunc(ttUserInterface interfaceHandle, unsigned int flag)
{
    TM_UNREF_IN_ARG(interfaceHandle);

    switch (flag)
    {
#ifdef TM_PPP_LQM
    case TM_LL_LQM_LINK_BAD:
        tfKernelWarning("tfPppNotifyFunc","Link quality is bad.");
/* TBD: attempt recovery of the PPP link */
        break;

    case TM_LL_LQM_UP:
        tfKernelWarning("tfPppNotifyFunc","LQM enabled on the link.");
        break;

    case TM_LL_LQM_DISABLED:
        tfKernelWarning("tfPppNotifyFunc","LQM disabled on the link.");
        break;
#endif /* TM_PPP_LQM */

    case TM_LL_OPEN_COMPLETE:
        tfKernelWarning("tfPppNotifyFunc","PPP link tlOpened.");
        tlPppConnectedStatus = 1;
        break;

    case TM_LL_CLOSE_COMPLETE:
        tfKernelWarning("tfPppNotifyFunc","PPP link closed.");
        tlPppConnectedStatus = 0;
        break;

    default:
        tfKernelWarning("tfPppNotifyFunc","error flag.");
        break;
    }
    return;
}
#endif /* TM_USE_SERIAL_INTERFACE */


#if TM_DEMO_PPP
#define TM_TEST_MAX_PPP_PACKET_LEN  500
void tfPppTestExecute(void)
{
    int                 loopCounter;
    int                 errorCode;
	static int          sock;
	static int          dataSock;
    struct sockaddr_in  addr;
	int                 addrLen;
	char				dataPtr[TM_TEST_MAX_PPP_PACKET_LEN];
    static ttUserSockPtrUnion  tempSockAddr;
    static int          appState = TM_INITIAL;

/* wait for the PPP link to come up */
    if(tlPppConnectedStatus)
    {
        switch(appState)
        {
        case TM_INITIAL:
#ifdef TM_PPP_LQM
            errorCode = tfLqmRegisterMonitor(
                tlSerialIfHandle,
                myLinkQualityMonitor,
                TM_LQM_HYSTERESIS_MAX_FAILURES,
                TM_LQM_HYSTERESIS_SAMPLES);
#endif /* TM_PPP_LQM */

/* Discard server */
            sock = socket( PF_INET, SOCK_STREAM, IP_PROTOTCP);
	        if(sock == TM_SOCKET_ERROR)
	        {
	            errorCode = TM_SOCKET_ERROR;
                appState = TM_APPERROR;
                break;
            }
	        else
	        {
	            addr.sin_family      = PF_INET;
	            addr.sin_addr.s_addr = inet_addr("0.0.0.0");
	            addr.sin_port        = htons(TM_DISCARD_PORT);
		        tempSockAddr.sockInPtr = &addr;
                errorCode = bind( sock, tempSockAddr.sockPtr, sizeof(addr) );
		        if(errorCode == TM_ENOERROR)
		        {
		            errorCode = listen(sock, 10);
                    if (errorCode != TM_ENOERROR)
                    {
                        errorCode = tfGetSocketError(sock);
                        appState = TM_APPERROR;
                        break;
                    }
			        else
			        {
                        appState = TM_ACCEPT;
                    }
                }
                else
                {
                    appState = TM_APPERROR;
                    break;
                }
            }

        case TM_ACCEPT:
			addrLen = sizeof(addr);
		    dataSock = accept( sock, tempSockAddr.sockPtr, &addrLen );
		    if (dataSock < 0)
		    {
		        errorCode = tfGetSocketError(sock);
                if(errorCode != TM_EWOULDBLOCK)
                {
                    appState = TM_APPERROR;
                    break;
                }
		    }
		    else 
		    {
                appState = TM_RECV;
            }

        case TM_RECV:
            errorCode = recv(
                dataSock, dataPtr, TM_TEST_MAX_PPP_PACKET_LEN, 0);
		    if (errorCode < 0)
		    {
		        errorCode = tfGetSocketError(dataSock);
                appState = TM_APPERROR;
		    }
		    break;

        default:
            break;
        }
    }

    return;
}


#ifdef TM_PPP_LQM
ttUser8Bit myLinkQualityMonitor(
    ttUserInterface interfaceHandle,
    int reasonCode,
    unsigned long timeElapsedMsec,
    ttLqrCountDeltasPtr countDeltasPtr,
    ttConstLqrCountsPtr countsPtr,
    ttUser32Bit outLqrs,
    ttUser32Bit outPackets,
    ttUser32Bit outOctets )
{
    int linkQuality;
    int outboundLqrsInPipeline;
    
    linkQuality = 0;
    switch(reasonCode)
    {
    case TM_LQM_MONITOR_LQR:
/* calculate link quality of the outgoing link */
        if (countDeltasPtr->deltaLastOutPackets >
            (countDeltasPtr->deltaPeerInPackets 
             + countDeltasPtr->deltaPeerInDiscards))
        {
            linkQuality += countDeltasPtr->deltaLastOutPackets
                - (countDeltasPtr->deltaPeerInPackets 
                   + countDeltasPtr->deltaPeerInDiscards);
        }

/* calculate link quality of the incoming link */
        if (countDeltasPtr->deltaPeerOutPackets 
            > countDeltasPtr->deltaSaveInPackets)
        {
            linkQuality += countDeltasPtr->deltaPeerOutPackets
                - countDeltasPtr->deltaSaveInPackets;
        }

/* calculate the number of outbound LQRs still in the pipeline */
        outboundLqrsInPipeline = (int) ((ttUser32Bit) 0xffffffff 
                                        & (outLqrs - countsPtr->lastOutLqrs));
        if (outboundLqrsInPipeline > 1)
        {
            linkQuality += outboundLqrsInPipeline - 1;
        }
        break;

    case TM_LQM_MONITOR_TIMEOUT:
        tfKernelWarning("myLinkQualityMonitor","LQM timeout occurred.");
        linkQuality = 1;
        break;

    default:
        tfKernelWarning("myLinkQualityMonitor","LQM invalid reasonCode.");
        break;
    }
    
    /* make it take at least 2 sample periods to declare the link bad */
    if (linkQuality > TM_LQM_HYSTERESIS_MAX_FAILURES)
    {
        linkQuality = TM_LQM_HYSTERESIS_MAX_FAILURES;
    }
    return (ttUser8Bit) linkQuality;
}
#endif /* TM_PPP_LQM */
#endif /* TM_DEMO_PPP */


void tfZ80InstallIsr( ttUserIsrHandlerPtr   funcPtr,
                      int                   ivBase,
                      ttUser8Bit            offset)
{
/* To avoid compiler warning */
    TM_UNREF_IN_ARG(funcPtr);
    TM_UNREF_IN_ARG(ivBase);
    TM_UNREF_IN_ARG(offset);

    asm("\tLD  HL, (IX+12)");  /* Interrupt jump table offset */
    asm("\tLD  BC, (IX+9)");   /* Interrupt jump table base */
    asm("\tADD HL, BC");
#ifdef _EZ80L92
    asm("\tINC HL");
    asm("\tINC HL");
#endif /* _EZ80L92 */
    asm("\tLD  BC, (IX+6)");   /* Interrupt handler */
    asm("\tLD  (HL), BC");     /* Store new vector address */

    return;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: txz80app.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6008    Output checksum: 61473
 */
