/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Linux Application Level threads Hooks
 *
 * Filename: trlinuxa.c
 * Author: jshang
 * Date Created: 04/18/02
 * $Source: source/kernel/linux/app/trlinuxa.c $
 *
 * Modification History
 * $Revision: 6.0.2.18 $
 * $Date: 2011/09/14 14:57:37CDT $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Linux Raw_Socket Driver, and application level threads hooks.
 */ 
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <netdb.h>
#include <sys/poll.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <net/if.h>
#include <time.h>
#include <malloc.h> 

#include <features.h> /* for the glibc version number */
#if __GLIBC__ >= 2 && __GLIBC_MINOR__ >= 1
#include <netpacket/packet.h>
#include <net/ethernet.h>    /* the L2 protocols */
#else
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>  /* The L2 protocols */
#endif
#include <net/route.h>               
#include <semaphore.h>

#include <pthread.h>


#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#ifdef TM_KERNEL_LINUX_APP

#undef socket
#undef bind
#undef send
#undef write
#undef recv
#undef poll
#undef close
#undef ioctl 
#undef connect
#undef listen
#undef accept
#undef sockaddr
#undef sockaddr_in


#define LinuxSocket             socket
#define LinuxBind               bind
#define LinuxSend               send
#define LinuxWrite              write
#define LinuxRecv               recv
#define LinuxPoll               poll
#define LinuxClose              close
#define LinuxIoctl              ioctl
#define LinuxConnect            connect
#define LinuxListen             listen
#define LinuxAccept             accept

#define LinuxPF_PACKET          PF_PACKET
#define LinuxSOCK_RAW           SOCK_RAW
#define LinuxPF_INET            PF_INET
#define LinuxSOCK_DGRAM         SOCK_DGRAM
#define LinuxSOCK_STREAM        SOCK_STREAM

#define TM_INTF_MTU             5000
#define TM_LINUX_MAX_INTF       TM_MAX_NUM_IFS
#define TM_LINUX_MAX_SEM        15 + TM_LINUX_MAX_INTF


typedef struct eth_type
{
  char eth_da[6];
  char eth_sa[6];
  unsigned short int eth_type;
} *eth_pt;

typedef struct tsLinuxInterfaceInfo 
{
    ttUserInterface             interfaceHandle;
    char                        hwAddr[6];
    int                         socketFd;
    char                      * scattBufferPtr;
    int                         scattLength;
} ttLinuxInterfaceInfo;    

typedef struct tsLinuxSemSt
{
    sem_t               sem;
/* semaphore pool flag, 0: free, 1: use */    
    int                 flag;
} ttLinuxSemSt;

typedef void *thrfunc(void *);

static int                     tlLinuxThreadsStarted = 0;
static int                     tlLinuxMaxIntf = 0;
static pthread_mutex_t         tlLinuxMutex;
static pthread_mutex_t         tlLinuxInterfaceMutex;
static ttLinuxInterfaceInfo    tlLinuxIntfInfo[TM_LINUX_MAX_INTF];
static ttLinuxSemSt            tlLinuxSem[TM_LINUX_MAX_SEM]; 
#ifdef TM_MULTIPLE_CONTEXT
static pthread_key_t		   tlLinuxThreadKey;
#endif /* TM_MULTIPLE_CONTEXT */

static void tfLinuxIntfClose(int index);
static void tfLinuxSetInterfaceCritical(void);
static void tfLinuxReleaseInterfaceCritical(void);
#ifdef TM_USE_STOP_TRECK
int tfDeleteLinuxDriver(ttCharPtr        namePtr);
#endif /* TM_USE_STOP_TRECK */

/* 
 * Dynamically allocate the Treck simple heap. Only called when
 * TM_DYNAMIC_CREATE_SHEAP has been #define'd.
 * User can change the size of the created heap by updating *sheapSizePtr
 * prior to calling malloc.
 * The start of the simple heap (i.e. the pointer returned) being
 * Return:
 * Pointer to beginning of dynamically created heap.
 */
ttUser32Bit TM_FAR * tfKernelSheapCreate(ttUser32Bit TM_FAR * sheapSizePtr)
{
    return (ttUser32Bit TM_FAR *)(malloc((*sheapSizePtr)));
}

#ifdef TM_USE_STOP_TRECK
void tfKernelSheapFree (ttUser32Bit TM_FAR * sheapPtr)
{
    free(sheapPtr);
    return;
}
#endif /* TM_USE_STOP_TRECK */

/*
 * Protect linux interface array
 */
static void tfLinuxSetInterfaceCritical(void)
{
    int errorCode;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    /* kernelDisableInterrupt() */
    errorCode = pthread_mutex_lock ( &tlLinuxInterfaceMutex );    
    if ( errorCode != 0)
    {
        printf(" tfLinuxSetInterfaceCritical error \n");
    }
    tm_set_current_context(contextHandle);
}
/*
 * Resume normal operation (from tfLinuxSetInterfaceCritical)
 */
static void tfLinuxReleaseInterfaceCritical(void)
{
    int errorCode;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    /*  kernelEnableInterrupt () */
    errorCode = pthread_mutex_unlock ( &tlLinuxInterfaceMutex );
    if ( errorCode != 0)
    {
        printf(" tfLinuxReleaseInterfaceCritical error \n");
    }
    tm_set_current_context(contextHandle);
}

/*
 * Stop all other calls into Treck by 
 * a kernel context switch
 */
void tfKernelSetCritical(void)
{
    int errorCode;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    /* kernelDisableInterrupt() */
    errorCode = pthread_mutex_lock ( &tlLinuxMutex );    
    if ( errorCode != 0)
    {
        printf(" tfKernelSetCritical error \n");
    }
    tm_set_current_context(contextHandle);
}
/*
 * Allocate a memory block of size from the kernel
 */
void TM_FAR *tfKernelMalloc(unsigned size)
{
    void TM_FAR * memoryPtr;
    memoryPtr = (void TM_FAR *)(malloc((unsigned)size));
    return memoryPtr;
}

/*
 * Free a memory block back to the kernel
 */
void tfKernelFree(void TM_FAR *memoryBlockPtr)
{
    free(memoryBlockPtr);
}

/*
 * Resume normal operation (from tm_kernel_set_critical)
 */
void tfKernelReleaseCritical(void)
{
    int errorCode;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    /*  kernelEnableInterrupt () */
    errorCode = pthread_mutex_unlock ( &tlLinuxMutex );
    if ( errorCode != 0)
    {
        printf(" tfKernelReleaseCritical error \n");
    }
    tm_set_current_context(contextHandle);
}


/*
 * Counting Semaphore Operations
 *
 * The examples assume the following:
 * A semaphore is of type int
 * Success from a Kernel call is 0
 * Failure from a Kernel Call is -1
 *
 */

/*
 * Create a counting semaphore
 */
int tfKernelCreateCountSem(ttUserGenericUnionPtr countingSemaphorePtr)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */

    int     kernelSemaphore;
    int     errorCode;
    int     i;

    errorCode = TM_KERN_ERROR;
/* search for an empty slot, and reserve it */
    tm_kernel_set_critical;
    for ( i = 0; i < TM_LINUX_MAX_SEM; i++)
    {
        if ( tlLinuxSem[i].flag == 0)
        {
/* Reserve the slot */
            tlLinuxSem[i].flag = 1;
            break;
        }
    }
    tm_kernel_release_critical;
    if (i < TM_LINUX_MAX_SEM)
/* Found, and reserved an empty slot */
    {
/* Save current context */
        tm_get_current_context(contextHandle);
/* Create semaphore with OS */
        kernelSemaphore=sem_init(&tlLinuxSem[i].sem, 0, 0);
/* Context could have changed during OS call */
        tm_set_current_context(contextHandle);
        if(kernelSemaphore != -1)
/* OS semaphore granted. Give user corresponding internal slot. */
        {
            countingSemaphorePtr->genIntParm = i;
            errorCode = TM_KERN_OKAY;
        }
        else
/* OS could not create a semaphore. Return the slot */
        {
            countingSemaphorePtr->genIntParm = -1;
/* Next operation is probably atomic, so critical section probably not needed */
            tm_kernel_set_critical; 
            tlLinuxSem[i].flag = 0;
            tm_kernel_release_critical;
        }
    }
    return errorCode;
}

/*
 * Wait on a counting semaphore
 */
int tfKernelPendCountSem(ttUserGenericUnionPtr countingSemaphorePtr)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    int             errorCode;
    int             i;

    errorCode = TM_KERN_ERROR;
    i = countingSemaphorePtr->genIntParm; 
    if (i != -1)
    {
        tm_get_current_context(contextHandle);
        if (sem_wait(&tlLinuxSem[i].sem)==0)
        {
            errorCode = TM_KERN_OKAY;
        }
        tm_set_current_context(contextHandle);
    }
    return errorCode;
}

#ifdef TM_USE_PEND_TIMEOUT
/*
 * Wait on a counting semaphore
 */
int tfKernelPendCountSemTimeout( ttUserGenericUnionPtr countingSemaphorePtr, 
                                 ttUser32Bit           timeOut )
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    struct timespec absTimeout;
    int             errorCode;
    int             i;

    errorCode = TM_KERN_ERROR;
    i = countingSemaphorePtr->genIntParm; 
    if (i != -1)
    {
        tm_get_current_context(contextHandle);
        timeOut = timeOut * tvTimerTickLength; /* in Milliseconds */
/* Seconds */
        absTimeout.tv_sec = (long) (timeOut / TM_UL(1000));
/* Milliseconds left */
        timeOut =   timeOut 
                  - ((ttUser32Bit)(absTimeout.tv_sec) * TM_UL(1000));
/* Nano seconds*/
        absTimeout.tv_nsec = (long)(timeOut * TM_UL(1000000));
        if ( sem_timedwait(&tlLinuxSem[i].sem, &absTimeout) == 0 )
        {
            errorCode = TM_KERN_OKAY;
        }
        else
        {
            if (errno == ETIMEDOUT)
            {
                errorCode = TM_ETIMEDOUT;
            }
        }
        tm_set_current_context(contextHandle);
    }
    return errorCode;
}
#endif /* TM_USE_PEND_TIMEOUT */

/*
 * Resume waiting tasks on a counting semaphore
 */
int tfKernelPostCountSem(ttUserGenericUnionPtr countingSemaphorePtr)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    int             errorCode;
    int             i;

    errorCode = TM_KERN_ERROR;
    i = countingSemaphorePtr->genIntParm; 
    if (i != -1)
    {
        tm_get_current_context(contextHandle);
        if (sem_post(&tlLinuxSem[i].sem)==0)
        {
            errorCode = TM_KERN_OKAY;
        }
        tm_set_current_context(contextHandle);
    }
    return errorCode;
}

/*
 * Delete a counting semaphore
 */
int tfKernelDeleteCountSem(ttUserGenericUnionPtr countingSemaphorePtr)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext   contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    int             errorCode;
    int             i;

    errorCode = TM_KERN_ERROR;
    i = countingSemaphorePtr->genIntParm; 
    if (i != -1)
    {
        tm_get_current_context(contextHandle);
        if (sem_destroy(&tlLinuxSem[i].sem)==0)
        {
            errorCode = TM_KERN_OKAY;
        }
        tm_set_current_context(contextHandle);
        tm_kernel_set_critical;
/* return semaphore slot to the pool */
        tlLinuxSem[i].flag = 0;
        tm_kernel_release_critical;
        countingSemaphorePtr->genIntParm = -1;
    }
    return errorCode;
}


void  tfKernelError(const char TM_FAR *functionName,
                    const char TM_FAR *errorMessage)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    printf("tfKernelError! %s: %s\n", functionName, errorMessage);
    tm_set_current_context(contextHandle);
    tm_thread_stop;
}

void  tfKernelWarning(const char TM_FAR *functionName,
                      const char TM_FAR *errorMessage)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    printf("tfKernelWarning! %s: %s\n", functionName, errorMessage);
    tm_set_current_context(contextHandle);
}

void  tfKernelInitialize(void)
{
    int i;
#ifdef TM_MULTIPLE_CONTEXT
    pthread_key_create(&tlLinuxThreadKey, NULL);
#endif /* TM_MULTIPLE_CONTEXT */    
    tlLinuxMaxIntf = 0;
    tm_bzero(tlLinuxIntfInfo, sizeof(ttLinuxInterfaceInfo)*TM_LINUX_MAX_INTF);
    for (i = 0; i < TM_LINUX_MAX_INTF; i++)
    {
        tlLinuxIntfInfo[i].socketFd = -1;
    }
    tm_bzero(&tlLinuxMutex, sizeof(tlLinuxMutex));
    tm_bzero(&tlLinuxInterfaceMutex, sizeof(tlLinuxMutex));
    tm_bzero(tlLinuxSem, sizeof(ttLinuxSemSt)*TM_LINUX_MAX_SEM);
}

void  tfKernelUnInitialize(void)
{
    return;
}

/*
 * Create an event structure for pend/post from an ISR, and
 * also for XMIT task pend/post.
 */
void tfKernelCreateEvent(ttUserGenericUnionPtr eventPtr)
{ 
    int     errorCode;

    errorCode = tfKernelCreateCountSem(eventPtr);
    if (errorCode == TM_KERN_ERROR)
    {
        printf(" Create Event error \n");
    }
    return;
}

/*
 * Task wait on an Event
 */
void tfKernelPendEvent(ttUserGenericUnionPtr eventPtr)
{
    (void)tfKernelPendCountSem(eventPtr);
}

/*
 * Resume task waiting on this Event. Called from a task.
 */
void tfKernelTaskPostEvent(ttUserGenericUnionPtr eventPtr)
{ 
    (void)tfKernelPostCountSem(eventPtr);
}

/*
 * Resume task waiting on this event. Called from an ISR.
 */
void tfKernelIsrPostEvent(ttUserGenericUnionPtr eventPtr)
{
    int i;

    i = eventPtr->genIntParm; 
    if (i != -1)
    {
        if (sem_post(&tlLinuxSem[i].sem)!=0)
        {
            printf(" Post Event error \n");
        }
    }
    return;
}

/*
 * Delete a counting semaphore
 */
int tfKernelDeleteEvent (ttUserGenericUnionPtr eventPtr)
{
    int             errorCode;

    errorCode = tfKernelDeleteCountSem(eventPtr);
    return errorCode;
}

/*
 * Yield the CPU. Used by tfInterfaceSpinLock().
 */
void tfKernelTaskYield(void)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    usleep(1000);
    tm_set_current_context(contextHandle);
    return;
}

void tfLinuxRecvThread  (void);
void tfLinuxTimerThread (void);

int tfLinuxDrvClose( ttUserInterface     interfaceHandle);

int tfLinuxDrvOpen( ttUserInterface     interfaceHandle);

int tfLinuxDrvSend( ttUserInterface     interfaceHandle,
                    char   TM_FAR       *dataPtr,
                    int                 dataLength,
                    int                 flag );

int tfLinuxDrvRecv( ttUserInterface     interfaceHandle,
                    ttCharPtr TM_FAR    *dataPtrPtr,
                    ttIntPtr            dataSizePtr,
                    ttUserBufferPtr     packetPtrPtr );

int tfLinuxDrvGetPhyAddr( ttUserInterface interfaceHandle,
                          char TM_FAR *physicalAddress );

int tfLinuxDrvIoctl( ttUserInterface interfaceHandle, int flag,
                     void TM_FAR * optionPtr, int optionLen );

int tfLinuxDrvGetPhyAddr ( ttUserInterface interfaceHandle,
                           char TM_FAR *physicalAddress )
{
    int i;
    int errorCode;

    i = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
    tfLinuxSetInterfaceCritical();
    if (tlLinuxIntfInfo[i].interfaceHandle == interfaceHandle)
    {
        tm_bcopy(tlLinuxIntfInfo[i].hwAddr, physicalAddress, 6);
        errorCode = TM_ENOERROR;
    }
    else
    {
        printf (" Can't find the hwaddr you request !\n");
        errorCode = TM_EINVAL;
    }
    tfLinuxReleaseInterfaceCritical();
    return errorCode;
}


/*
 * tfUseLinuxDriver() Function Description
 * This function is called first.
 */
ttUserInterface tfUseLinuxDriver(ttCharPtr        namePtr,
                                 ttUserLinkLayer  linkLayerHandle,
                                 int TM_FAR    *  errorCodePtr) 
{
    ttUserInterface     interfaceHandle;
    pthread_t           linuxRecvThread;
    pthread_t           linuxTimerThread;
    int                 index;
    int                 fd;

    *errorCodePtr = TM_ENOERROR;
    fd = -1;
    interfaceHandle = tfAddInterface (namePtr,
                                      linkLayerHandle,
                                      tfLinuxDrvOpen,
                                      tfLinuxDrvClose,
                                      tfLinuxDrvSend,
                                      tfLinuxDrvRecv,
                                      (ttDevFreeRecvFuncPtr)0,
                                      tfLinuxDrvIoctl,
                                      tfLinuxDrvGetPhyAddr,
                                      errorCodePtr); 
    if (interfaceHandle != (ttUserInterface)0)
    {
        index = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
        tfLinuxSetInterfaceCritical();
        tlLinuxIntfInfo[index].interfaceHandle = interfaceHandle;
        tlLinuxIntfInfo[index].socketFd = -1;
/* How many we have added */
        if (index >= tlLinuxMaxIntf)
        {
            tlLinuxMaxIntf = index + 1;
        }
        tfLinuxReleaseInterfaceCritical();
/*  Create recv and timer threads
 */
        if (!tlLinuxThreadsStarted)
        {
            tlLinuxThreadsStarted = 1;
            pthread_mutex_init (&tlLinuxMutex, NULL);
            pthread_create(&linuxRecvThread, NULL, 
                            (thrfunc *)tfLinuxRecvThread, NULL);
            pthread_create(&linuxTimerThread, NULL, 
                            (thrfunc *)tfLinuxTimerThread, NULL);
            printf (" Receive and Timer threads  are running !\n\n");
        }   
    }
    else
    {
/* Do not overwrite the error code from tfAddInterface() */
        if (*errorCodePtr == TM_ENOERROR)
        {
            *errorCodePtr = TM_ENOENT;
        }
    }

    if (*errorCodePtr != TM_ENOERROR)
    {
        if (interfaceHandle != (ttVoidPtr)0)
        {
#ifdef TM_USE_STOP_TRECK
            tfDeleteInterface(interfaceHandle);
#endif /* TM_USE_STOP_TRECK */
            interfaceHandle = (ttUserInterface)0;
            if (fd != -1)
            {
                LinuxClose(fd);
            }
        }
    }
    return interfaceHandle;
}

int tfLinuxDrvClose(ttUserInterface interfaceHandle)
{
    int index;

    tfLinuxSetInterfaceCritical();
    index = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
    if (tlLinuxIntfInfo[index].interfaceHandle == interfaceHandle)
    {
        tfLinuxIntfClose(index);
    }
    tfLinuxReleaseInterfaceCritical();
    return TM_ENOERROR;
}

static void tfLinuxIntfClose(int index)
{
    int fd;

    if (tlLinuxIntfInfo[index].interfaceHandle != 0)
    {
        tlLinuxIntfInfo[index].scattLength = 0;
        if (tlLinuxIntfInfo[index].scattBufferPtr != (char *)0)
        {
            tm_kernel_free(tlLinuxIntfInfo[index].scattBufferPtr);
            tlLinuxIntfInfo[index].scattBufferPtr = (char *)0;
        }
        fd = tlLinuxIntfInfo[index].socketFd; 
        if (fd != -1)
        {
            tlLinuxIntfInfo[index].socketFd = -1;
            (void)LinuxClose(fd);
        }
    }
}

int tfLinuxDrvOpen(ttUserInterface     interfaceHandle)
{
    int     fd, ifindex;
    int     index;
    int     result;
    struct  ifreq ifreq;
    struct  sockaddr_ll addr_ll; 
    int     errorCode;

    errorCode = TM_ENOERROR;
    tfLinuxSetInterfaceCritical();
    index = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
    if (tlLinuxIntfInfo[index].interfaceHandle == interfaceHandle)
    {
/* Open PF_PACKET socket, set interface */
        fd = LinuxSocket(LinuxPF_PACKET, LinuxSOCK_RAW, htons(ETH_P_ALL));
        if (fd < 0) 
        {
            printf (" Socket open error \n");
            errorCode = TM_EBADF;
            goto linuxDrvOpenExit; 
        }
        printf("New Linux Socket: %d\n", fd);
        tm_bcopy(((ttDeviceEntryPtr)interfaceHandle)->devNameArray,
                 ifreq.ifr_name,
                 IFNAMSIZ);
        if (LinuxIoctl(fd, SIOCGIFHWADDR, &ifreq) < 0) 
        {
    	    printf (" Error in hwaddr\n");
            errorCode = TM_EIO;
            goto linuxDrvOpenExit; 
        }
        tm_bcopy(ifreq.ifr_hwaddr.sa_data, tlLinuxIntfInfo[index].hwAddr, 6);

        if (LinuxIoctl(fd, SIOCGIFINDEX, &ifreq) < 0) 
        {
            printf (" Error in SIOCGIFINDEX \n");
            errorCode = TM_EIO;
            goto linuxDrvOpenExit; 
        }
        ifindex = ifreq.ifr_ifindex;
 
        if (LinuxIoctl(fd, SIOCGIFFLAGS, &ifreq) < 0) 
        {
            printf ("Error in SIOCGIFFLAGS \n");
            errorCode = TM_EIO;
            goto linuxDrvOpenExit; 
        }
 
/* enable interface in promiscuous mode */
        ifreq.ifr_flags |= IFF_PROMISC;
        ifreq.ifr_flags |= IFF_ALLMULTI;
        ifreq.ifr_flags |= IFF_UP;
  
        if (ioctl(fd, SIOCSIFFLAGS, &ifreq) < 0) 
        {           
            printf ("Error in SIOCSIFFLAGS \n");
            errorCode = TM_EIO;
            goto linuxDrvOpenExit; 
        }
 
        addr_ll.sll_family = AF_PACKET;
        addr_ll.sll_protocol = htons(ETH_P_ALL);
        addr_ll.sll_ifindex = ifindex;
        if ((result=LinuxBind(fd, (struct sockaddr *)&addr_ll, 
                                                sizeof(addr_ll))) < 0) 
        {
            printf ("Error in bind %d \n",result);
            errorCode = TM_EIO;
            goto linuxDrvOpenExit; 
        }
    
        tlLinuxIntfInfo[index].socketFd = fd;
    }
linuxDrvOpenExit: 
    tfLinuxReleaseInterfaceCritical();
    return errorCode;
}

#ifdef TM_USE_STOP_TRECK
/*
 * tfDeleteLinuxDriver() Function Description
 * Delete Linux driver interface to the Treck stack
 * 
 */
int tfDeleteLinuxDriver(ttCharPtr        namePtr)
{
    ttUserInterface     interfaceHandle;
    int                 i;
    int                 errorCode;

    errorCode = TM_ENOERROR;
    tfLinuxSetInterfaceCritical();
    interfaceHandle = tfInterfaceNameToDevice(namePtr);
    if (interfaceHandle != (ttUserInterface)0)
    {
        i = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
        if (tlLinuxIntfInfo[i].interfaceHandle == interfaceHandle)
        {
            if (i == (tlLinuxMaxIntf - 1))
            {
                tlLinuxMaxIntf--;
            }
            tfLinuxIntfClose(i);
            tlLinuxIntfInfo[i].interfaceHandle = (ttUserInterface)0;
            tfLinuxReleaseInterfaceCritical();
/* Release the critical section, because tfDeleteInterface could call
 * tfLinuxDrvClose() (which itself set the critical section), if the user had
 * not closed the interface
 */
            errorCode = tfDeleteInterface(interfaceHandle);
            tfLinuxSetInterfaceCritical();
        }
        else
        {
            errorCode = TM_EFAULT;
        }
    }
    else
    {
        errorCode = TM_ENOENT;
    }
    tfLinuxReleaseInterfaceCritical();
    return errorCode;
}
#endif /* TM_USE_STOP_TRECK */


int tfLinuxDrvSend  (   ttUserInterface     interfaceHandle,
                        char     TM_FAR     *dataPtr,
                        int                 dataLength,
                        int                 flag )
{
    int errorCode;
    int i;
    int totalLength;
    int bytesSent;

    errorCode = TM_ENOERROR;
    i = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
    tfLinuxSetInterfaceCritical();
    if (tlLinuxIntfInfo[i].interfaceHandle == interfaceHandle)
    {
        totalLength = tlLinuxIntfInfo[i].scattLength + dataLength;
        if ( (flag != TM_USER_BUFFER_MORE)) 
/* whole packet, or last piece of scattered buffer */
        {
            bytesSent = 0;
            if ( tlLinuxIntfInfo[i].scattLength == 0)
/* Non scattered send */
            {
                bytesSent = LinuxWrite(tlLinuxIntfInfo[i].socketFd,
                                               dataPtr, dataLength);
            }
            else
            {
                if (tlLinuxIntfInfo[i].scattBufferPtr != (char *)0)
                {
                    if (totalLength > TM_INTF_MTU)
                    {
                        printf (" Error in LinuDrvSend, MTU \n");
                        errorCode = TM_EINVAL;
                    }
                    else
                    {
                        tm_bcopy ( dataPtr, tlLinuxIntfInfo[i].scattBufferPtr + 
                                tlLinuxIntfInfo[i].scattLength, dataLength);
                        tlLinuxIntfInfo[i].scattLength = totalLength;
                        bytesSent = LinuxWrite(tlLinuxIntfInfo[i].socketFd,
                                               tlLinuxIntfInfo[i].scattBufferPtr,
                                               totalLength);
                    }
                    tm_kernel_free(tlLinuxIntfInfo[i].scattBufferPtr);
                    tlLinuxIntfInfo[i].scattBufferPtr = (char *)0;
                }
/* Commented out to ignore the error:
                else
                {
                    errorCode = TM_ENOBUFS;
                }
*/
                tlLinuxIntfInfo[i].scattLength = 0; /* reset for next packet */
            }
            if (bytesSent < 0) 
            {
                printf ("Error in Write !  \n"); /* ignore the error */
            }
            if (errorCode == TM_ENOERROR)
            {
/* No need to call tfSendCompleteInterface if we return an error */
                tfSendCompleteInterface (interfaceHandle, 
                                          TM_DEV_SEND_COMPLETE_DRIVER);
            }
        }
        else /* TM_USER_BUFFER_MORE */
        {
            if (tlLinuxIntfInfo[i].scattBufferPtr == (char *)0)
            {
                if (tlLinuxIntfInfo[i].scattLength == 0)
/* Beginning of new packet */
                {
                    tlLinuxIntfInfo[i].scattBufferPtr =
                            tm_kernel_malloc(TM_INTF_MTU);
                    if (tlLinuxIntfInfo[i].scattBufferPtr == (char *)0)
                    {
                        errorCode = TM_ENOBUFS;
                    }
                }
            }
            if (tlLinuxIntfInfo[i].scattBufferPtr != (char *)0)
            {
                if (totalLength <= TM_INTF_MTU)
                {
                    tm_bcopy ( dataPtr, tlLinuxIntfInfo[i].scattBufferPtr + 
                               tlLinuxIntfInfo[i].scattLength, dataLength);
                }
                else
                {
                    printf (" Error in LinuDrvSend, MTU \n");
                }
            }
/* Update length even if we could not allocate */
            tlLinuxIntfInfo[i].scattLength = totalLength;
        }
    }
    else
    {
        errorCode = TM_EINVAL;
    }
    tfLinuxReleaseInterfaceCritical();
    return errorCode;
}

int tfLinuxDrvRecv (    ttUserInterface     interfaceHandle,
                        ttCharPtr TM_FAR    *dataPtrPtr,
                        ttIntPtr            dataSizePtr,
                        ttUserBufferPtr     packetPtrPtr )
{
    ttDeviceEntryPtr    devEntryPtr;
    ttPacketPtr         packetPtr;
    int                 dataLength;
    int                 errorCode;
    int                 i;

    dataLength = 0;

    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    
    packetPtr = tfGetSharedBuffer(
        TM_LL_MAX_HDR_LEN + TM_ETHER_HW_ALIGN,
        (ttPktLen)(TM_INTF_MTU + TM_ETHER_IP_ALIGN),
        TM_16BIT_ZERO );
    if ( packetPtr == TM_PACKET_NULL_PTR)
    {
        errorCode = TM_ENOBUFS;
        return errorCode;
    }
    if (tm_ll_need_32bit_align(devEntryPtr))
    {
        if (   ((tt32Bit)packetPtr->pktLinkDataPtr)
             & (TM_ETHER_HW_ALIGN - 1) )
        {
/* Align the data area (according to TM_ETHER_HW_ALIGN) */
            packetPtr->pktLinkDataPtr = (tt8Bit TM_FAR *)
                             (    (tt32Bit)packetPtr->pktLinkDataPtr
                               & ~(TM_ETHER_HW_ALIGN - 1) );
        }
/*
 * Offset of ethernet frame to allow IP area to be long word aligned.
 */
        packetPtr->pktLinkDataPtr += TM_ETHER_IP_ALIGN;
    }
    i = ((ttDeviceEntryPtr)interfaceHandle)->devIndex - 1;
    if ( tlLinuxIntfInfo[i].interfaceHandle == interfaceHandle)
    {
        dataLength = LinuxRecv( tlLinuxIntfInfo[i].socketFd,
                                packetPtr->pktLinkDataPtr, TM_INTF_MTU , 0);
        if (dataLength < 0)
        {
            errorCode = TM_ENOENT;
            tfFreeSharedBuffer(packetPtr, TM_SOCKET_UNLOCKED);
        }
        else
        {
            packetPtr->pktUserFlags = TM_PKTF_USER_DEV_OWNS;
            *packetPtrPtr = packetPtr;
            *dataPtrPtr = (ttCharPtr)(packetPtr->pktLinkDataPtr);
            *dataSizePtr = dataLength;
            errorCode = TM_ENOERROR;
        }
    }
    else
    {
        errorCode = TM_EINVAL;
    }
    return errorCode;    
}

int tfLinuxDrvIoctl ( ttUserInterface interfaceHandle, int flag,
                     void TM_FAR * optionPtr, int optionLen )
{
    TM_UNREF_IN_ARG(interfaceHandle);
    TM_UNREF_IN_ARG(flag);
    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);

    return TM_ENOERROR;
}



/*
 * tfLinuxRecvThread() Function Description
 */

void tfLinuxRecvThread (void)
{
    struct pollfd pollfd[TM_LINUX_MAX_INTF];
    int nfds, rfds, i;
    ttUserInterface interfaceHandle;
   
    while (1) 
    {
        tfLinuxSetInterfaceCritical();
        nfds = tlLinuxMaxIntf;
        for ( i = 0; i < nfds; i++)
        {
            pollfd[i].fd = tlLinuxIntfInfo[i].socketFd;
            if (pollfd[i].fd != -1)
            {
                pollfd[i].events = POLLIN;
            }
            else
            {
                pollfd[i].events = 0;
            }
            pollfd[i].revents = 0;
        }
        tfLinuxReleaseInterfaceCritical();
/*  check pollfd every 1 sec */
        rfds = LinuxPoll(pollfd, nfds, 1000); 
    
        if ( rfds <= 0)
        {
            continue;
        }
        for ( i = 0; i < nfds ; i++)
        {
            if ( pollfd[i].revents & POLLIN ) 
            {
                tfLinuxSetInterfaceCritical();
                interfaceHandle = tlLinuxIntfInfo[i].interfaceHandle; 
                tfLinuxReleaseInterfaceCritical();
#ifdef TM_MULTIPLE_CONTEXT
                if (interfaceHandle != 0)
                {
                    tfNotifyInterfaceTask(
                                  interfaceHandle,
                                  1,
                                  0, 0UL, 0UL ); 
                }
#else
                if (interfaceHandle != 0)
                {
                    tfRecvInterface(interfaceHandle);
                }
#endif /* TM_MULTIPLE_CONTEXT */
            }
        }
    }
}
/*  Timer Task *
 */
void tfLinuxTimerThread(void)
{
    while (1)
    {
        usleep(TM_TICK_LENGTH * 1000);
        tfTimerUpdate();
#ifndef TM_MULTIPLE_CONTEXT
/* in Multiple context, user should define TimerExecute thread by self */
        tfTimerExecute();
#endif /* TM_MULTIPLE_CONTEXT */
    }
}

#ifdef TM_MULTIPLE_CONTEXT

ttUserContext tfKernelGetContext(void)
{
    void * voidPtr;
    
    voidPtr = pthread_getspecific(tlLinuxThreadKey);
    if ( voidPtr == (void * )0)
    {
	printf(" wrong thread key\n");
        voidPtr = tvCurrentContext;
    } 
    tvCurrentContext = (ttContextPtr)voidPtr;
    return (ttUserContext)voidPtr;
}

void tfKernelSetContext(ttUserContext context)
{
    int errorCode;

    errorCode = pthread_setspecific(tlLinuxThreadKey, context);
    tm_assert( tfKernelSetContext, errorCode==0);
}

#endif /* TM_MULTIPLE_CONTEXT */

/* 
 * Treck Stack calls this function to ask for the system time. It is 
 * used in the appliction protocols such as the web server and SMTP client.
 * User could get the time from the system clock, from network, 
 * or from application user interface.
 * 
 * Parameters:
 *  secondsPtr: 
 *      Pointer to store number of seconds elapsed since midnight (00:00:00), 
 *      January 1, 1970, Coordinated Universal Time (UTC). 
 *      UTC is the international time standard. It is the current term for
 *      what was commonly referred to as Greenwich Meridian Time (GMT). 
 *      This matches the return value of the ANSI C library 
 *      function time_t time(time_t *)
 *  daysPtr:
 *      The maximum number of seconds a 32 bit unsigned integer can hold is
 *      2**32 = 4G. this is approximately 126 year. 
 *      In case this value ever wrap arounds, we need another space to hold the 
 *      a bigger units of time. This is the purpose of daysPtr. 
 *  NOTE: The total laps of time is the days stored in daysPtr plus the seconds
 *        stored in the secondsPtr  
 * Returns:
 *  TM_KERN_OKAY:   Sucessful
 *  TM_KERN_ERROR:  Failed to obtain the system time.
 *  
 */


int tfKernelGetSystemTime(ttUser32Bit * daysPtr, ttUser32Bit * secondsPtr)
{
    time_t  times; /* long integer */
#ifdef _DEBUG
    ttTime  utcTime;
#endif /* _DEBUG */
    int     errorCode; 
  
    errorCode = TM_KERN_OKAY;
    times= time(&times);
    *daysPtr = 0;
    *secondsPtr = (ttUser32Bit)times; /* cast down on 64-bit CPUs */
#ifdef _DEBUG
    tfGetUtcTime(daysPtr, secondsPtr, &utcTime);
#endif /* _DEBUG */

    return errorCode;
}

#else /* ! TM_KERNEL_LINUX_APP */
int tlLinuxDump = 0;
#endif /* TM_KERNEL_LINUX_APP */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: trlinuxa.c    File Revision: 6.0.2.18 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 13318    Output checksum: 38575
 */
