/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Linux Application Level  Hooks
 *              NOTE: This File is intended to be modified by the user
 *                    In order to add a new operating system
 * Filename: txlinuxa.c
 * Author: jshang
 * Date Created: 04/18/02
 * $Source: source/kernel/linux/app/txlinuxa.c $
 *
 * Modification History
 * $Revision: 6.0.2.10 $
 * $Date: 2011/06/17 16:43:54CDT $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trglobal.h>
#include <trproto.h>
#ifdef TM_SNMP_VERSION
#include <trapi.h>
#endif /* TM_SNMP_VERSION */
#include <stdio.h> /* for printf */
#include <string.h> /* for strstr */
#undef timeval
#undef fd_set
#undef select
#include <stdlib.h> /* for atoi */

#include <pthread.h>

#define __socklen_t_defined /* for socklen_t in trsocket.h 720 */
#include <unistd.h> /* for getcwd */

#include <sys/stat.h>
#include <sys/types.h>
int mkdir(const char *pathname, mode_t mode);

/*
 * Local macro definitions
 */
#ifndef tm_getcwd
#define tm_getcwd    getcwd
#endif /* tm_getcwd */

#ifndef tm_chdir
#define tm_chdir     chdir
#endif /* tm_chdir */

#ifndef tm_rmdir
#define tm_rmdir     rmdir
#endif /* tm_rmdir */

#ifndef tm_mkdir
#define tm_mkdir     mkdir
#endif /* tm_mkdir */

#define tm_rename    rename
#define tm_remove    remove

#define tm_fopen(fh,pchParam)       fopen(fh,pchParam)
#define tm_fread(bufPtr,size,count,filePtr) \
                fread(bufPtr,(size_t)size,(size_t)count,filePtr)
#define tm_fwrite(bufPtr,size,count,filePtr) \
            fwrite(bufPtr,(size_t)size,(size_t)count,filePtr)

#define tm_fflush       fflush
#define tm_fclose       fclose


typedef struct tsUserLogin
{
    char             userName[20];
    char             userPassword[20];
} ttUserLogin;
typedef const ttUserLogin TM_FAR * ttUserLoginPtr;

#define TM_FS_WORKINGDIR_SIZE 256

#ifdef TM_LINT
LINT_UNREF_MEMBER_BEGIN
#endif /* TM_LINT */
typedef struct tsUserFsData
{
    ttUserLoginPtr   userLoginPtr;
    ttVoidPtr        userDirPtr;
    char             userWorkingDirectory[TM_FS_WORKINGDIR_SIZE];
/* Long or short directory open */
    int              userDirFlag;
} ttUserFsData;
#ifdef TM_LINT
LINT_UNREF_MEMBER_END
#endif /* TM_LINT */

typedef ttUserFsData TM_FAR * ttUserFsDataPtr;

typedef const ttUserFsData TM_FAR * ttConstUserFsDataPtr;


#define TM_LINUX_TESTCASE       1
#define TM_LINUX_SNMPD_AGENT    2

#define TM_LOGSENDTO_SCREEN

#if (defined(TM_DEBUG_LOGGING) || defined(TM_TRACE_LOGGING) || \
     defined(TM_ERROR_LOGGING) || defined(TM_MEMORY_LOGGING) || \
     defined(TM_USE_LOGGING_LEVELS))
#define TM_LINUX_LOGGING_ENABLED
#endif /* TM_DEBUG_LOGGING ... */

typedef struct tsLinuxTest
{
    char            localIpAddrStr[20];
    char            maskIpAddrStr[20];
    char            etherIntf[6];

    int             flag;
    char            remoteIpAddrStr[20];
    int             localPortNum;
    int             remotePortNum;
    int             dataLength;
    int             count ;
    int             intTime;
    int             startCase;
    int             endCase;

    char            trapIpAddrStr[20];
    int             trapPortNum;
    char            logIpAddrStr[20];
    int             logPortNum;

/* if want to dump log msg to file, uncomment all related to snmpDump */
/*     FILE  *         snmpDump; */
} ttLinuxTest;

/*
 * Local variables
 */

static char    tlFSInitWorkingDirectory[TM_FS_WORKINGDIR_SIZE];

const ttUserLogin tlFsNamePasswd[] =
{
/*   username,      password    */
    {"elmic",       "elmic"     },
    {"guest",       "1Aguests"  },
    {"ftp",         ""          },  /* no password */
    {"anonymous",   ""          },  /* no password */
};

/*
 * Note: with a DOS OS we need to open the files in binary mode, that
 * way there is no translation from CR,LF to LF on input, and from LF
 * to CR,LF on output. We can keep the network CR,LF and store that
 * in the file. Since we are opening the files in binary mode, then
 * we will have to explicitely write CR,LF for end of line ourselves.
 */
const char tlFsOpenMap[TM_FS_APPEND+1][4] = { "rb", "wb", "ab"};

ttUserInterface     tvInterfaceHandle;
ttLinuxTest         tvLT;
int tvLoggingSd;
int tvLastSeqNo;
struct sockaddr_in tvToAddress;
char tvLogMsg[(TM_DEF_MAX_LOG_MSG_LEN * 2) + 22];

/*
 * local functions prototype
 */
static void tfFSUserDir(ttConstUserFsDataPtr userFsDataPtr);

void tfLinuxTest (  char*   localIpAddrStr,
                    int     localPortNum,
                    char*   remoteIpAddrStr,
                    int     remotePortNum,
                    char*   maskIpAddrStr,
                    int     dataLength,
                    int     count,
                    int     startCase, 
                    int     endCase,
                    int     intTime  );

void tfLinuxTcpEchoServer(void);

#ifdef TM_SNMP_VERSION
void tfLinuxSnmpThread(void);
void tfLinuxSnmpAgent ( char *trapIpAddrStr, char *trapNetMask, int trapPortNum,
                            char *logIpAddrStr, int logPortNum );
#endif /* TM_SNMP_VERSION */
#ifdef TM_LINUX_LOGGING_ENABLED
void tfLinuxLogThread(void);
void loggingTask(char *logIpAddrStr, int logPortNum);
int walkLog(int msgSeqNo, ttConstCharPtr pszLogMsg);
#endif /* TM_LINUX_LOGGING_ENABLED */

int tfLinuxReadConf(char *fname);
int tfLinuxReadConf(char *fname)
{
    int errorCode;
    FILE *fh;
    char buf[1024];
    ttCharPtr token;
    ttCharPtr readbuf;

    errorCode = TM_ENOERROR;
    fh = fopen (fname, "rt");
    if (!fh)
    {
        errorCode = TM_EIO;
        goto LINUX_READ_CONF_RETURN;
    }
/* Read Description */
    tvLT.flag = 0;
    while((readbuf = fgets(buf, 1024, fh)) != (char *)0)
    {
        if(*readbuf == '*' || *readbuf == 0x0d || *readbuf == 0x0a)
        {
            continue;
        }
        
        token=strtok(readbuf,"\r\n:");
        if(!token)
        {
            errorCode = TM_EINVAL;
            goto LINUX_READ_CONF_RETURN;
        }
        
        if(strstr(token, "Local Host Interface") != 0)
        {
            token=strtok(NULL," \r\n:");
            (void)strncpy(tvLT.etherIntf,token,6);
        }
        if(strstr(token, "Local Host IP Address") != 0)
        {
            token=strtok(NULL," \r\n:");
            (void)strncpy(tvLT.localIpAddrStr,token,20);
        }
        if(strstr(token, "Network Mask Address") != 0)
        {
            token=strtok(NULL," \r\n:");
            (void)strncpy(tvLT.maskIpAddrStr,token,20);
        }
        if(strstr(token, "Run Test Case (y/n)") != 0)
        {
            token=strtok(NULL," \r\n:");
            if ((strstr(token, "y") != 0) || (strstr(token, "Y") != 0))
            {
                tvLT.flag = TM_LINUX_TESTCASE;
            }
        }
        if(strstr(token, "Local Host Port Num.") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.localPortNum = atoi(token);
        }
        if(strstr(token, "Remote Host IP Address") != 0)
        {
            token=strtok(NULL," \r\n:");
            (void)strncpy(tvLT.remoteIpAddrStr,token,20);
        }
        if(strstr(token, "Remote Port Num") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.remotePortNum = atoi(token);
        }
        if(strstr(token, "Start Test Case No. (1-9)") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.startCase = atoi(token);
        }
        if(strstr(token, "End Test Case No. (1-9)") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.endCase = atoi(token);
        }
        if(strstr(token, "Data Length of each test (byte)") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.dataLength = atoi(token);
        }
        if(strstr(token, "Times of each test") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.count = atoi(token);
        }
        if(strstr(token, "Interval time between two tests (sec)") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.intTime = atoi(token);
        }
#ifdef TM_SNMP_VERSION
        if(strstr(token, "Start SNMPD Agent (y/n)") != 0)
        {
            token=strtok(NULL," \r\n:");
            if ((strstr(token, "y") != 0) || (strstr(token, "Y") != 0))
            {
                if (tvLT.flag == TM_LINUX_TESTCASE)
                {
                    printf ("Error: you cannot run test case and snmpd agent at same time!\n");
                    errorCode = TM_EINVAL;
                    goto LINUX_READ_CONF_RETURN;
                }
                else
                {
                    tvLT.flag = TM_LINUX_SNMPD_AGENT;
                }
            }
        }
        if(strstr(token, "Agent Trap IP Address") != 0)
        {
            token=strtok(NULL," \r\n:");
            (void)strncpy(tvLT.trapIpAddrStr,token,20);
        }
        if(strstr(token, "Agent Trap Port Num.") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.trapPortNum = atoi(token);
        }
#endif /* TM_SNMP_VERSION */
        if(strstr(token, "Log IP Address") != 0)
        {
            token=strtok(NULL," \r\n:");
            (void)strncpy(tvLT.logIpAddrStr,token,20);
        }
        if(strstr(token, "Log Port Num.") != 0)
        {
            token=strtok(NULL," \r\n:");
            tvLT.logPortNum = atoi(token);
        }
    }
    
LINUX_READ_CONF_RETURN:
    return errorCode;
}
          
    
int main(int argc, char *argv[])
{
    ttUserLinkLayer         linkLayerHandle;
    int                     i;
    int                     errorCode;
    char                    filename[64] = "txlinuxa.conf";
    char                    testCaseName[][25] = {
                                            "1: TEST_UDP_SEND",
                                            "2: TEST_UDP_RECV",
                                            "3: TEST_TCP_CONNECT",
                                            "4: TEST_TCP_SEND",
                                            "5: TEST_TCP_RECV",
                                            "6: TEST_UDP_ECHO_CLIENT",
                                            "7: TEST_UDP_ECHO_SERVER",
                                            "8: TEST_TCP_ECHO_CLIENT",
                                            "9: TEST_TCP_ECHO_SERVER" };


    errorCode = TM_ENOERROR;

    if (argc > 1)
    {
        if (sizeof(argv[1]) > 64)
        {
            printf("Error: Configuration filename too long!\n");
            return 0;
        }
        tm_bcopy(argv[1],filename,sizeof(argv[1])); 
    }

    errorCode = tfLinuxReadConf(filename);
    if (errorCode == TM_EIO)
    {
        printf ("Error: Invalid filename!\n");
        return 0;
    }
    if (errorCode == TM_EINVAL)
    {
        printf ("Error when parsing file.\n");
        return 0;
    }
    if (errorCode != TM_ENOERROR)
    {
        printf ("Unknown error code %d.\n", errorCode);
        return 0;
    }

    if ( tvLT.flag == TM_LINUX_TESTCASE )
    {
        printf (" \nWelcome to Turbo Treck Linux Raw_Socket Driver Test !\n");
        printf (" Ethernet interface            : %s\n",tvLT.etherIntf);
        printf (" Local IP Address              : %s\n",tvLT.localIpAddrStr);
        printf (" Mask Address                  : %s\n",tvLT.maskIpAddrStr);
        printf (" Local Port Number (TCP & UDP) : %d\n",tvLT.localPortNum);
        printf (" Remote IP Address             : %s\n", tvLT.remoteIpAddrStr); 
        printf (" Remote Port Number (TCP & UDP): %d\n",tvLT.remotePortNum);
        printf (" Data length of each test      : %d\n",tvLT.dataLength);
        printf (" Times for each test           : %d\n",tvLT.count);
        printf (" Test Cases                    :\n");
        for (i = tvLT.startCase; i <= tvLT.endCase ; i++)
            printf (" %s\n",testCaseName[i-1]);
        printf ( "\nThe interval time between two test cases is %d sec\n", tvLT.intTime);
        printf (" Start Test: \n");
    }
#ifdef TM_SNMP_VERSION
    if ( tvLT.flag == TM_LINUX_SNMPD_AGENT )
    {
        printf (" \nWelcome to Turbo Treck Linux Raw_Socket Driver SNMPD Agent !\n");
        printf (" Ethernet interface            : %s\n",tvLT.etherIntf);
        printf (" Local IP Address              : %s\n",tvLT.localIpAddrStr);
        printf (" Mask Address                  : %s\n\n",tvLT.maskIpAddrStr);
        printf (" Agent Trap IP Address         : %s\n", tvLT.trapIpAddrStr); 
        printf (" Agent Trap Port Number        : %d\n",tvLT.trapPortNum);
    }
#endif /* TM_SNMP_VERSION */
    
    tfInitTreck();
    errorCode = tfStartTreck();
    if (errorCode != TM_ENOERROR)
    {
        printf( "tfStartTreck failed, '%s'\n", tfStrError(errorCode));
        return errorCode;
    }

/* open Interface */ 
    linkLayerHandle = tfUseEthernet();
/* open Linux Raw_socket, recv and timer threadsi for specified eth0,eth1,... */
    tvInterfaceHandle = tfUseLinuxDriver(tvLT.etherIntf, linkLayerHandle, &errorCode);
    if ((errorCode != TM_ENOERROR) || (tvInterfaceHandle == (ttUserInterface)0))
    {
        printf("tfUseLinuxDriver failed with errorCode %d (%s)!\n",
               errorCode,
               tfStrError(errorCode));
        return 0;
    }
 
/* Open the added Linux interface, here is eth1 in the above */
    errorCode = tfOpenInterface( tvInterfaceHandle,
                                 inet_addr(tvLT.localIpAddrStr),
                                 inet_addr(tvLT.maskIpAddrStr),
                                 0,
                                 1 );
 
    if (errorCode != TM_ENOERROR)
    {
        printf("tfConfigInterface failed '%s'\n", tfStrError(errorCode));
        return errorCode;
    }

    switch ( tvLT.flag )
    {
        case TM_LINUX_TESTCASE:
            tfLinuxTest (   tvLT.localIpAddrStr, 
                            tvLT.localPortNum, 
                            tvLT.remoteIpAddrStr, 
                            tvLT.remotePortNum, 
                            tvLT.maskIpAddrStr,
                            tvLT.dataLength, 
                            tvLT.count,
                            tvLT.startCase, 
                            tvLT.endCase, 
                            tvLT.intTime );
            break;
#ifdef TM_SNMP_VERSION
        case TM_LINUX_SNMPD_AGENT:
            tfLinuxSnmpAgent(tvLT.trapIpAddrStr, tvLT.maskIpAddrStr,tvLT.trapPortNum,
                                tvLT.logIpAddrStr, tvLT.logPortNum);        
            while (1);
            break;
#endif /* TM_SNMP_VERSION */
        default:
            break;
    }

    return 0;
}

/* Test Case
 * 1: TEST_UDP_SEND  
 * 2: TEST_UDP_RECV
 * 3: TEST_TCP_CONNECT
 * 4: TEST_TCP_SEND
 * 5: TEST_TCP_RECV
 * 6: TEST_UDP_ECHO_CLIENT
 * 7: TEST_UDP_ECHO_SERVER
 * 8: TEST_TCP_ECHO_CLIENT
 * 9: TEST_TCP_ECHO_SERVER
 * function parameter: treck 1, treck 1,5 (from case 1-5), treck (test all cases )
 */

void tfLinuxTest (  char*   localIpAddrStr,
                    int     localPortNum,
                    char*   remoteIpAddrStr,
                    int     remotePortNum,
                    char*   maskIpAddrStr,
                    int     dataLength,
                    int     count,
                    int     startCase, 
                    int     endCase,  
                    int     intTime  )
{
    int errorCode;

    TM_UNREF_IN_ARG(maskIpAddrStr);

    remotePortNum = htons (remotePortNum);
    localPortNum  = htons (localPortNum);
    
    while ( endCase >= startCase)
    {
        sleep(intTime);
        printf (" \nTest Case %d \n", startCase);   
        switch ( startCase ) 
        {
            case 1:
                errorCode = tfTestTreck(    TM_TEST_UDP_SEND,
                                            remoteIpAddrStr,
                                            remotePortNum,
                                            dataLength,
                                            count,
                                            TM_TEST_FLAG_FILL_DATA,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_UDP_SEND failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_UDP_SEND successfully \n");
                }
                break;
            case 2:
                errorCode = tfTestTreck(    TM_TEST_UDP_RECV,
                                            localIpAddrStr,
                                            localPortNum,
                                            dataLength,
                                            count,
                                            0,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_UDP_RECV failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf ("TEST_UDP_RECV successfully \n");
                }
                break;
            case 3:
                errorCode = tfTestTreck(    TM_TEST_TCP_CONNECT,
                                            remoteIpAddrStr,
                                            remotePortNum,
                                            dataLength,
                                            count,
                                            0,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_TCP_CONNECT failed '%s'\n", \
                                            tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_TCP_CONNECT successfully \n");
                }
                break;
            case 4:
                errorCode = tfTestTreck(    TM_TEST_TCP_SEND,
                                            remoteIpAddrStr,
                                            remotePortNum,
                                            dataLength,
                                            count,
                                            TM_TEST_FLAG_FILL_DATA,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_TCP_SEND failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_TCP_SEND successfully \n");
                }
                break;
            case 5:
                errorCode = tfTestTreck(    TM_TEST_TCP_RECV,
                                            localIpAddrStr,
                                            localPortNum,
                                            dataLength,
                                            count,
                                            0,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  

                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_TCP_RECV failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_TCP_RECV successfully \n");
                }
                break;
            case 6:
                errorCode = tfTestTreck(    TM_TEST_UDP_ECHO_CLIENT,
                                            remoteIpAddrStr,
                                            remotePortNum,
                                            dataLength,
                                            count,
                                            TM_TEST_FLAG_FILL_DATA,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_UDP_ECHO_CLIENT failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_UDP_ECHO_CLIENT successfully \n");
                }
                break;
            case 7:
                errorCode = tfTestTreck(    TM_TEST_UDP_ECHO_SERVER,
                                            localIpAddrStr,
                                            localPortNum,
                                            dataLength,
                                            count,
                                            0,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_UDP_ECHO_SERVER failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_UDP_ECHO_SERVER successfully \n");
                }
                break;
            case 8:
                errorCode = tfTestTreck(    TM_TEST_TCP_ECHO_CLIENT,
                                            remoteIpAddrStr,
                                            remotePortNum,
                                            dataLength,
                                            count,
                                            TM_TEST_FLAG_FILL_DATA,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_TCP_ECHO_CLIENT failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_TCP_ECHO_CLIENT successfully \n");
                }
                break;
            case 9:
                errorCode = tfTestTreck(    TM_TEST_TCP_ECHO_SERVER,
                                            localIpAddrStr,
                                            localPortNum,
                                            dataLength,
                                            count,
                                            0,
                                            (ttUserTestHandlePtr) 0,
                                            ( void TM_FAR * )0 );  
                if (errorCode != TM_ENOERROR)
                {
                    printf("TEST_TCP_ECHO_SERVER failed '%s'\n", tfStrError(errorCode));
                }
                else
                {
                    printf("TEST_TCP_ECHO_SERVER successfully \n");
                }
                break;
            default:
                break;
        }
        startCase++;
    }
    return ;
}

void tfLinuxTcpEchoServer(void)
{
    int errorCode;
    errorCode = tfTestTreck(    TM_TEST_TCP_ECHO_SERVER,
                                 tvLT.localIpAddrStr,
                                 htons(tvLT.localPortNum),
/* Keep TCP ECHO SERVER working forever for SNMP */
                                 tvLT.dataLength,
                                 0,
                                 0,
                                 (ttUserTestHandlePtr) 0,
                                 ( void TM_FAR * )0 );  
    if (errorCode != TM_ENOERROR)
    {
        printf("TCP_ECHO_SERVER failed '%s'\n", tfStrError(errorCode));
    }
    else
    {
        printf("TCP_ECHO_SERVER already closed \n");
    }
    printf (" You need to restart Treck in order TCP_ECHO_SERVER works \n"); 
    return;
}

#ifdef TM_SNMP_VERSION
void tfLinuxSnmpAgent(char *    trapIpAddrStr,
                      char *    trapNetMask,
                      int       trapPortNum,
                      char *    logIpAddrStr,
                      int       logPortNum)
{
/*  create thread */
    pthread_t               tvLinuxSnmpThread;
#ifdef TM_LINUX_LOGGING_ENABLED
    pthread_t               tvLinuxLogThread;
#endif /* TM_LINUX_LOGGING_ENABLED */
    pthread_t               tvLinuxTcpEchoServer;
    typedef void * thrfunc(void *);    

/* Create Snmp Agent Thread */
    ttSnmpTrapParmsPtr      trapParmsPtr;
    ttUserIpAddress         trapDestIpAddr;
    ttUserIpAddress         netMask;
    int                     ret;

    TM_UNREF_IN_ARG(logIpAddrStr);
    TM_UNREF_IN_ARG(logPortNum);

    pthread_create(&tvLinuxTcpEchoServer, NULL,
                        (thrfunc *)tfLinuxTcpEchoServer, NULL);
    printf (" TCP ECHO SERVER is running ! \n");

#ifdef TM_LINUX_LOGGING_ENABLED
    pthread_create(&tvLinuxLogThread, NULL,
                        (thrfunc *)tfLinuxLogThread, NULL);
    printf (" LOG Thread is running ! \n");
/* the following causes address resolution for the trap destination IP
   address to occur, since that is the same address as the destination
   of the logging messages. So, trace logging must be enabled via the
   TM_TRACE_LOGGING macro. */
    tm_trace_func_entry(startSnmpAgent);
#endif /* TM_LINUX_LOGGING_ENABLED */

/*    ret = tfSnmpdMain(TM_BLOCKING_OFF, 0); */

/* set the destination IP address for SNMP traps */
    trapDestIpAddr=inet_addr(trapIpAddrStr);
/*    netMask=htonl(0xffffff00); */
    netMask = inet_addr(trapNetMask);
    trapParmsPtr = (ttSnmpTrapParmsPtr)
        tm_malloc((unsigned)sizeof(ttSnmpTrapParms));
    tfBzero(trapParmsPtr, sizeof(ttSnmpTrapParms));
    tfStrCpy((char *) trapParmsPtr->stpCommunity, "trap comm");
    trapParmsPtr->stpCommunityLen = tfStrLen(
        (const char*) trapParmsPtr->stpCommunity);
    trapParmsPtr->stpRemotePort = trapPortNum;
    trapParmsPtr->stpDestIp = trapDestIpAddr;
    trapParmsPtr->stpSourceIp = tfGetRouterId();
    trapParmsPtr->stpEnterpriseNum = 32925; /* TM_DEFAULT_ENTERPRISE_NUMBER */
    trapParmsPtr->stpEnterpriseLen = TM_DEFAULT_ENTERPRISE_LENGTH;
    ret = tfAddTrapEntry (trapParmsPtr);
    tm_assert(snmpdTask, (ret == TM_SNMPAPI_NOERROR));
    tm_free(trapParmsPtr);

    ret = tfAddCommunity(TM_READ_COMMUNITY_FLAG, "read_comm");
    tm_assert(snmpdTask, (ret == TM_SNMPAPI_NOERROR));
    ret = tfAddCommunity(TM_WRITE_COMMUNITY_FLAG, "write_comm");
    tm_assert(snmpdTask, (ret == TM_SNMPAPI_NOERROR));

/* wait for SNMP trap destination address to become reachable, so that
   we can verify cold-start trap functionality */
    while(tfRtDestExists(
#ifdef TM_USE_STRONG_ESL
                         tvInterfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                         trapDestIpAddr, netMask) != TM_ENOERROR)
    {
        printf("tfLinuxSnmpAgent: Waiting for trap destination"
               " to become reachable...\n");
        usleep (50000);
    }

    pthread_create(&tvLinuxSnmpThread, NULL,
                        (thrfunc *)tfLinuxSnmpThread, NULL);
    printf (" SNMPv3 Thread is running ! \n");


    return ;
}

void tfLinuxSnmpThread ()
{
    int ret;

    ret = tfSnmpdMain(TM_BLOCKING_ON, 0);
}
#endif /* TM_SNMP_VERSION */

#ifdef TM_LINUX_LOGGING_ENABLED
void tfLinuxLogThread()
{
    loggingTask(tvLT.logIpAddrStr, tvLT.logPortNum);
}

int walkLog(int msgSeqNo, ttConstCharPtr pszLogMsg)
{
#ifndef TM_LOGSENDTO_SCREEN
    int len;
    int treckError;
#endif /* !TM_LOGSENDTO_SCREEN */
    int errorCode = 0;

    if (tvLastSeqNo < msgSeqNo)
    {
#ifdef TM_LOGSENDTO_SCREEN       
        printf ("Log seqNo=%d, msg=%s\n", msgSeqNo, pszLogMsg);
/*      fprintf (tvLT.snmpDump,"Log seqNo=%d, msg=%s\n", msgSeqNo, pszLogMsg); */
        usleep(1000);
        errorCode = TM_ENOERROR;
        tvLastSeqNo = msgSeqNo + 1;
#else 
        len = tfSPrintF(tvLogMsg, "LOG seqNo=%d, msg=%s", msgSeqNo, pszLogMsg);
        errorCode = sendto(
            tvLoggingSd,
            tvLogMsg,
            len,
            0,
            (const struct sockaddr TM_FAR *) &tvToAddress,
            (int)sizeof(struct sockaddr));
        usleep(1000);
        if (errorCode == -1)
        {
            treckError=tfGetSocketError(tvLoggingSd);
        }
        else
        {
            errorCode = TM_ENOERROR;
            tvLastSeqNo = msgSeqNo + 1;

#if 0
            if (tvLastSeqNo == 150)
            {
/* test SNMP Agent shutdown */
                (void) tfSnmpdStop();
            }
#endif
        }
#endif /* TM_LOGSENDTO_SCREEN */
    }

    return errorCode; /* continue the walk */
}

void loggingTask(char *logIpAddrStr, int logPortNum)
{
    ttUserGenericUnion genParam;    
    int                ret;

    tvLastSeqNo = -1;
#ifdef TM_LOGSENDTO_SCREEN
    TM_UNREF_IN_ARG(logIpAddrStr);
    TM_UNREF_IN_ARG(logPortNum);
#else 
    tvLoggingSd = socket(AF_INET, SOCK_DGRAM, IP_PROTOUDP);
    tvToAddress.sin_family=PF_INET;
    tvToAddress.sin_addr.s_addr=inet_addr(logIpAddrStr);
    tvToAddress.sin_port=htons(logPortNum);
    if (tvLoggingSd == TM_SOCKET_ERROR)
    {
        tfKernelError("loggingTask", "Could not open a socket\n");
    }
    else
#endif /* TM_LOGSENDTO_SCREEN  */

/*  tvLT.snmpDump = fopen ("snmpdump","wt"); */
    {
        while (1)
        {
            usleep (10000);
            ret = tfLogWalk(
                (ttLogWalkFuncPtr) walkLog,
                tm_context(tvLogCtrlBlkPtr),
                tvLastSeqNo,
                genParam); /* genParam is unused. */
                tm_assert(loggingTask, (ret == TM_ENOERROR));
        }
    }
}
#endif /* TM_LINUX_LOGGING_ENABLED */

/*
 * Example File System Interfaces from TRECK stack to Linux file system
 */

/*
 * Function Description
 * Verifies whether a user is allowed on the system.
 *
 * Parameters
 *
 * Parameter       Description
 * userNamePtr     Pointer to a null terminated string containing the
 *                 user name.
 *
 * Returns
 * Value   Meaning
 * 0       Success
 * -1      User is not allowed on the system
 */
int tfFSUserAllowed (const char TM_FAR *  userNamePtr)
{
    int            maxEntries;
    int            retCode;
    int            i;

    retCode = -1;
    maxEntries = sizeof(tlFsNamePasswd)/sizeof(tlFsNamePasswd[0]);
    for (i = 0; i < maxEntries; i++)
    {
        if (    tm_strcmp(userNamePtr, tlFsNamePasswd[i].userName)
             == TM_STRCMP_EQUAL )
        {
            retCode = 0;
            break;
        }
    }

    return retCode;
}

/*
 * tfFSUserLogin Function Description
 * Given a User name string , and Password string, login the user
 * if the user is allowed, and password correct. Allocate a data structure
 * to store information unique to the just logged in user, such as its
 * current working directory.
 *
 * Parameters
 *
 * Parameter       Description
 * userNamePtr     Pointer to a null terminated string containing the
 *                 user name.
 * passwordPtr     Pointer to a null terminated string containing the
 *                 password.
 *
 * Returns
 * Value         Meaning
 * (void *)0     Failure.
 * userDataPtr   Pointer to a unique user data structure containing
 *               information about the given user (such as its working
 *               directory.)
 */

void TM_FAR * tfFSUserLogin ( const char TM_FAR *  userNamePtr,
                              const char TM_FAR *  passwordPtr )
{
    ttUserFsDataPtr   userFsDataPtr;
    int               maxEntries;
    int               i;
    int               retCode;

    userFsDataPtr = (ttUserFsDataPtr)0;
    retCode = -1;
    maxEntries = sizeof(tlFsNamePasswd)/sizeof(tlFsNamePasswd[0]);
    for (i = 0; i < maxEntries; i++)
    {
        if (    tm_strcmp(userNamePtr, tlFsNamePasswd[i].userName)
             == TM_STRCMP_EQUAL )
        {
            if (    (tlFsNamePasswd[i].userPassword[0] == '\0')
                 || (    tm_strcmp( passwordPtr,
                                    tlFsNamePasswd[i].userPassword )
                      == TM_STRCMP_EQUAL ) )
            {
                retCode = 0;
            }
            break;
        }
    }
    if (retCode == 0)
    {
        userFsDataPtr = (ttUserFsDataPtr)tm_get_raw_buffer(
                                                        sizeof(ttUserFsData));
        if (userFsDataPtr != (ttUserFsDataPtr)0)
        {
            tm_bzero(userFsDataPtr, sizeof(ttUserFsData));
/* set the initial working directory */
            if (tvFSInitWorkingDirSet == TM_8BIT_ZERO)
            {
                (void)tm_getcwd(tlFSInitWorkingDirectory, 
                                TM_FS_WORKINGDIR_SIZE );
                tvFSInitWorkingDirSet = TM_8BIT_YES;

            }
            tm_strcpy(userFsDataPtr->userWorkingDirectory, 
                      tlFSInitWorkingDirectory);
        }
    }

    return (void TM_FAR *)userFsDataPtr;
}

/*
 * tfFSUserLogout Function Description
 * Given the unique user pointer as returned by tfFSUserLogin, log the
 * user out, free the structure pointed to by userDataPtr.
 *
 * Parameters
 * Parameter   Description
 * userDataPtr Pointer to user data structure as returned by tfFSUserLogin.
 * Returns
 * No return value.
 */
void tfFSUserLogout (void TM_FAR *  userDataPtr)
{
    tm_free_raw_buffer((ttRawBufferPtr)userDataPtr);

    return;
}

/*
 * tfFSSystem function description:
 * Copy into bufferPtr (up to bufferSize bytes), the official system name,
 * as assigned in the list of "OPERATING SYSTEM NAMES" in the
 * "Assigned Numbers" RFC (RFC 1700). For example the DOS operating system
 * has been assigned DOS as system name. If the file system is a DOS file
 * system, then this function should copy "DOS" into bufferPtr. If the
 * system has not been assigned a system name in the RFC, then this function 
 * should return -1.
 * Parameters
 * Parameter    Description
 * bufferPtr    Pointer to a buffer where to copy the system name
 * bufferSize   size in bytes of the buffer.
 * Returns
 * Value        Meaning
 * -1           failure
 *  > 0         number of bytes copied.
 */
int tfFSSystem (char TM_FAR * bufferPtr, int bufferSize)
{
    TM_UNREF_IN_ARG(bufferSize);
    tm_strcpy(bufferPtr, "LINUX");

    return 5;
}

/*
 * tfFSStructureMount function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and 
 * a file system, mount the user to the new file system.
 * Parameters
 * Parameter    Description
 * userDataPtr  Pointer to user data structure as returned by tfFsUserLogin.
 * pathNamePtr  Pointer to a null terminated string containing a file system
 *              name.
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSStructureMount ( void TM_FAR *      userDataPtr,
                         const char TM_FAR * pathNamePtr )
{
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(pathNamePtr);

    return -1;
}

/*
 * tfFSRenameFile function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and 
 * a current file name, and a new file name, rename the file to the new
 * file name. 
 * Parameters
 * Parameter       Description
 * userDataPtr     Pointer to user data structure as returned by tfFsUserLogin
 * fromPathNamePtr Pointer to null terminated string containing current
 *                 file name
 * toPathNamePtr   Pointer to null terminated string containing new
 *                 file name
 *
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSRenameFile (const void TM_FAR * userDataPtr,
                    const char TM_FAR * fromPathNamePtr,
                    const char TM_FAR * toPathNamePtr )
{
    int retCode;

#define userFsDataPtr ((ttUserFsDataPtr)userDataPtr)
    tfFSUserDir(userFsDataPtr);
    retCode = tm_rename(fromPathNamePtr, toPathNamePtr);
#undef userFsDataPtr

    return retCode;
}

/*
 * tfFSGetUniqueFileName function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * find and copy a unique file name that does not
 * conflict with any existing file name in the user working
 * directory in the buffer pointed to by bufferPtr, up to bufferSize bytes.
 * Returns -1 on failure, otherwise the number of copyied bytes.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFsUserLogin
 * bufferPtr     Pointer to buffer where to store the unique file name.
 * bufferSize    size in bytes of the buffer.
 * Returns
 * Value         Meaning
 *  -1           failure
 *  > 0          number of copyied bytes
 */
int tfFSGetUniqueFileName ( void TM_FAR * userDataPtr,
                            char TM_FAR * bufferPtr,
                            int           bufferSize )
{
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(bufferPtr);
    TM_UNREF_IN_ARG(bufferSize);

    return -1;
}

/*
 * tfFSDeleteFile function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * and a file name, delete the file.
 *
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFsUserLogin
 * pathNamePtr   Pointer to a null terminated string containing file name.
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSDeleteFile (const void TM_FAR * userDataPtr, 
                    const char TM_FAR * pathNamePtr )
{
    int retCode;

#define userFsDataPtr ((ttUserFsDataPtr)userDataPtr)
    tfFSUserDir(userFsDataPtr);
    retCode = tm_remove(pathNamePtr);
#undef userFsDataPtr

    return retCode;
}

/*
 * tfFSOpenFile function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * and a file name, open the file for either read (if flag is 
 * TM_FS_READ), write (if flag is TM_FS_WRITE), or apppend (if flag is
 * TM_FS_APPEND). Parameter type specifies if file type is ascii 
 * (TM_TYPE_ASCII), or (TM_TYPE_BINARY). Parameter structure specifies if
 * the file structure is stream (TM_STRU_STREAM) or record
 * (TM_STRU_RECORD).). Allocate a file data structure to store
 * the file pointer, file type, file structure, etc.. 
 * Note: This call should fail if the file name is a directory.
 * Returns pointer to the newly allocated file structure on success,
 * null pointer on failure.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFsUserLogin
 * pathNamePtr   Pointer to a null terminated string containing file name.
 * flag          open flag: TM_FS_READ, TM_FS_WRITE, TM_FS_APPEND
 * type          file type: TM_TYPE_ASCII, or TM_TYPE_BINARY
 * structure     file structure: TM_STRU_RECORD, or TM_STRU_STREAM
 * Returns
 * Value             Meaning
 * (void TM_FAR *)0  failure
 * fileDataPtr       pointer to newly allocated file data structure.
 */

void TM_FAR * tfFSOpenFile (const void TM_FAR * userDataPtr,
                            const char TM_FAR * pathNamePtr,
                            int                 flag,
                            int                 type,
                            int                 structure )
{
    void TM_FAR * fileHandle;

    TM_UNREF_IN_ARG(flag);
    TM_UNREF_IN_ARG(type);
    TM_UNREF_IN_ARG(structure);
    tfFSUserDir((ttConstUserFsDataPtr)userDataPtr);

    fileHandle = (void TM_FAR *)tm_fopen(pathNamePtr, tlFsOpenMap[flag]);

    return fileHandle;
}

/*
 * tfFSReadFile function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * and a file data pointer as returned by tfFSOpenFile, read up to 
 * bufferSize in the buffer. Return the number of bytes actually read, 0 if
 * end of file has been reached, -1 on error.
 *
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * fileDataPtr   Pointer to file data structure as returned by tfFSOpenFile.
 * bufferPtr     Pointer to buffer where to copy the data from the file.
 * bufferSize    size in bytes of the buffer.
 *
 * Returns
 * Value         Meaning
 * > 0           number of copied bytes
 *  0            end of file
 *  -1           failure
 */
int tfFSReadFile( void TM_FAR * userDataPtr,
                  void TM_FAR * fileDataPtr,
                  char TM_FAR * bufferPtr,
                  int           bufferSize )
{
    int retCode;

    TM_UNREF_IN_ARG(userDataPtr);
    retCode = (int)tm_fread(bufferPtr, 1, bufferSize, (FILE *)fileDataPtr);

    return retCode;
}

/*
 * tfFSReadFileRecord function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * and a file data pointer as returned by tfFSOpenFile, read up to 
 * bufferSize in the buffer, or end of record whichever comes first.
 * If end of record has been reached, store 1 in the integer pointed to
 * by eorPtr, otherwise store 0. If the file system does not support
 * records, then the system end of line (i.e <CR><LF> for DOS, <LF> for
 * Unix) is an end of record, and this routine should convert every end of
 * line (i.e <CR><LF> for DOS, <LF> for Unix) to an end of record, i.e
 * store 1 in eorPtr when the end of line character(s) have been read;
 * the end of line character(s) themselves should not be copied into
 * the buffer pointed to by bufferPtr.
 *
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * fileDataPtr   Pointer to file data structure as returned by tfFSOpenFile.
 * bufferPtr     Pointer to buffer where to copy the data from the file.
 * bufferSize    size in bytes of the buffer.
 * eorPtr        Pointer to end of record.
 * Returns
 * Value         Meaning
 * > 0           number of copied bytes (not including end of record)
 *  0            end of file has been reached
 *  -1           failure
 */
int tfFSReadFileRecord ( void TM_FAR * userDataPtr,
                         void TM_FAR * fileDataPtr,
                         char TM_FAR * bufferPtr,
                         int           bufferSize,
                         int  TM_FAR * eorPtr )
{
/* Not needed on DOS. We use the TM_FS_CRLF_FLAG */
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(fileDataPtr);
    TM_UNREF_IN_ARG(bufferPtr);
    TM_UNREF_IN_ARG(bufferSize);
    TM_UNREF_IN_ARG(eorPtr);

    return -1;
}

/*
 * tfFSWriteFile function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * and a file data pointer as returned by tfFSOpenFile, write 
 * bytes from the buffer pointed to by bufferPtr to the file.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * fileDataPtr   Pointer to file data structure as returned by tfFSOpenFile.
 * bufferPtr     Pointer to buffer data to copy into the file.
 * bytes         size in bytes of the data in the buffer.
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSWriteFile ( void TM_FAR *       userDataPtr,
                    void TM_FAR *       fileDataPtr,
                    const char TM_FAR * bufferPtr,
                    int                 bytes)
{
    int     retCode;
    int     length;

    TM_UNREF_IN_ARG(userDataPtr);
    length = (int)tm_fwrite(bufferPtr, 1, bytes, fileDataPtr);
    if (length == bytes)
    {
        retCode = 0;
    }
    else
    {
        retCode = -1;
    }

    return retCode;
}

/*
 * tfFSWriteFileRecord function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, 
 * and a file data pointer as returned by tfFSOpenFile, write up to 
 * bytes from the buffer, plus an end of record if eor is set to 1.
 * If the file system does not support records, then the system end of
 * line (i.e <CR><LF> for DOS, <LF> for Unix) should be written to the
 * file after all the bytes from the buffer have been written if eor is
 * set to 1.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * fileDataPtr   Pointer to file data structure as returned by tfFSOpenFile.
 * bufferPtr     Pointer to buffer data to copy into the file.
 * bytes         size in bytes of the data in the buffer.
 * eor           end of record (1 if end of record need to be written)
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSWriteFileRecord ( void TM_FAR * userDataPtr,
                          void TM_FAR * fileDataPtr,
                          char TM_FAR * bufferPtr,
                          int           bytes,
                          int           eor )
{
/* Not needed on DOS. We use the TM_FS_CRLF_FLAG */
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(fileDataPtr);
    TM_UNREF_IN_ARG(bufferPtr);
    TM_UNREF_IN_ARG(bytes);
    TM_UNREF_IN_ARG(eor);

    return -1;
}

/*
 * tfFSFlushFile Function description:
 * Given a unique user data pointer as returned by tfFSUserLogin, and a
 * file data pointer as returned by tfFSOpenFile, flush the file.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * fileDataPtr   Pointer to file data structure as returned by tfFSOpenFile.
 * Returns
 * No return value.
 */
int tfFSFlushFile ( void TM_FAR * userDataPtr, void TM_FAR * fileDataPtr)
{
    TM_UNREF_IN_ARG(userDataPtr);
    (void)tm_fflush(fileDataPtr);

    return 0;
}


/*
 * tfFSCloseFile Function description:
 * Given a unique user data pointer as returned by tfFSUserLogin, and a
 * file data pointer as returned by tfFSOpenFile, close the file,
 * and free the file data structure pointed to by fileDataPtr.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * fileDataPtr   Pointer to file data structure as returned by tfFSOpenFile.
 * Returns
 * No return value.
 */
int tfFSCloseFile ( void TM_FAR * userDataPtr, void TM_FAR * fileDataPtr)
{
    TM_UNREF_IN_ARG(userDataPtr);
    (void)tm_fclose(fileDataPtr);

    return 0;
}

/* 
 * tfFSChangeDir function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and
 * a directory name, change the user working directory to the new
 * directory.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * pathNamePtr   Pointer to a null terminated string containing directory
 *               path name.
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSChangeDir (void TM_FAR *        userDataPtr, 
                   const char TM_FAR *  pathNamePtr )
{
    int retCode;

#define userFsDataPtr ((ttUserFsDataPtr)userDataPtr)
    
    if (userDataPtr != TM_VOID_NULL_PTR)
    {
        tfFSUserDir(userFsDataPtr);
        retCode = tm_chdir(pathNamePtr);
        if (retCode != -1)
        {
            (void)tm_getcwd(userFsDataPtr->userWorkingDirectory,
                            TM_FS_WORKINGDIR_SIZE);
        }
    }
    else
    {
/* This command is only valid if a user is logged in. */
        retCode = -1;
    }

#undef userFsDataPtr    
    return retCode;
}

/* 
 * tfFSChangeParentDir function description:
 * Given the unique user data pointer as returned by tfFSUserLogin 
 * change the user working directory to its parent directory.
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSChangeParentDir ( void TM_FAR * userDataPtr )
{
    int retCode;

    if (userDataPtr != TM_VOID_NULL_PTR)
    {
        retCode = tfFSChangeDir(userDataPtr, "..");
    }
    else
    {
/* This command is only valid if a user is logged in. */        
        retCode = -1;
    }

    return retCode;
}

/* 
 * tfFSRemoveDir function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and
 * a directory name, remove the directory.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * pathNamePtr   Pointer to a null terminated string containing directory
 *               path name.
 * Returns
 * Value         Meaning
 *  0            success
 *  -1           failure
 */
int tfFSRemoveDir(const void TM_FAR * userDataPtr, 
                  const char TM_FAR * pathNamePtr )
{
#define userFsDataPtr ((ttUserFsDataPtr)userDataPtr)
/* Point to current user directory */
    tfFSUserDir(userFsDataPtr);

    return tm_rmdir(pathNamePtr);
#undef userFsDataPtr
}

/* 
 * tfFSMakeDir function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and
 * a directory name, create the directory. If successfull, copy the
 * directory path name in bufferPtr (up to bufferSize). The directory
 * path name could be either absolute or relative to the user working
 * directory path, but should be such that a subsequent tfFSChangeDir
 * with that pathname as an argument should not fail.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * pathNamePtr   Pointer to a null terminated string containing directory
 *               path name.
 * bufferPtr     pointer to a buffer where to copy the pathname of the newly
 *               created directory.
 * bufferSize    size in bytes of the buffer.
 * Returns
 * Value         Meaning
 *  -1           failure
 *  > 0          number of bytes copyied into the buffer pointed to by
 *               bufferPtr.
 */
int tfFSMakeDir (const void TM_FAR *    userDataPtr, 
                 const char TM_FAR *    pathNamePtr,
                 char TM_FAR *          bufferPtr, 
                 int                    bufferSize )
{
    int     retCode;

#define userFsDataPtr ((ttUserFsDataPtr)userDataPtr)
/* Point to current user directory */
    tfFSUserDir(userFsDataPtr);
    retCode = tm_mkdir(pathNamePtr, 0);
    if (retCode == 0)
    {
/* created directory, copy path */
        retCode = (int)tm_strlen(pathNamePtr);
        if (retCode >= bufferSize)
        {
            retCode = bufferSize - 1;
        }
        tm_bcopy(pathNamePtr, bufferPtr, retCode + 1);
        bufferPtr[retCode] = '\0';
    }
#undef userFsDataPtr

    return retCode;
}

/* 
 * tfFSGetWorkingDir function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, copy
 * the user working directory in the buffer pointed to by bufferPtr (up
 * to bufferSize).
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * bufferPtr     pointer to a buffer where to copy the pathname of the user
 *               working directory.
 * bufferSize    size in bytes of the buffer.
 * Returns
 * Value         Meaning
 *  -1           failure
 *  > 0          number of bytes copyied into the buffer pointed to by
 *               bufferPtr.
 */
int tfFSGetWorkingDir ( const void TM_FAR * userDataPtr,
                        char TM_FAR *       bufferPtr,
                        int                 bufferSize )
{
    int retCode;

    tfFSUserDir((ttUserFsDataPtr)userDataPtr);
    (void)tm_getcwd(bufferPtr, bufferSize);
    if (bufferPtr != (char TM_FAR *) 0)
    {
        bufferPtr[bufferSize - 1] = '\0';
        retCode = (int)tm_strlen(bufferPtr);
    }
    else
	{
        retCode = -1;
    }

    return retCode;
}


/* 
 * tfFSOpenDir function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and
 * a pointer to a path name, and a flag, open the directory corresponding
 * to path for reading either a long directory (flag == TM_DIR_LONG) or
 * short directory (TM_DIR_SHORT). Subsequent calls to tfFSGetNextDirEntry
 * will fetch each entry in the directory matching the pattern as pointed to
 * by pathNamePtr.
 * The tfFSOpenDir implementor should allocate a directory data structure
 * to keep track of the path name matching pattern, the reading position
 * in the directory, and the directory read flag (TM_DIR_LONG or
 * TM_DIR_SHORT). Note that if pathNamePtr points to a directory name,
 * then the matching pattern is "*.*". If pathNamePtr points to "*.*",
 * then the user working directory should be open, and the matching pattern
 * is "*.*".
 * Returns a pointer to a just allocated directory structure pointer on
 * success, null pointer on failure.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * pathNamePtr   Pointer to a null terminated string containing directory
 *               path name, or matching pattern path name.
 * flag          Either TM_DIR_LONG or TM_DIR_SHORT
 * Returns
 * Value            Meaning
 * (void TM_FAR*)0  failure
 * dirDataPtr       Pointer to newly allocated directory data structure
 */
void TM_FAR * tfFSOpenDir ( void TM_FAR *       userDataPtr,
                            const char TM_FAR * pathNamePtr,
                            int                 flag )
{
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(pathNamePtr);
    TM_UNREF_IN_ARG(flag);

    return (void TM_FAR *)0;
}

/* 
 * tfFSGetNextDirEntry function description:
 * Given the unique user data pointer as returned by tfFSUserLogin, and
 * a directory data pointer as returned by tfFSOpenDir, get the next
 * entry in the directory that matches the path given as an argument to
 * tfFSOpenDir. The next entry should be either a long listing
 * of the next directory entry (either volume name, sub-directory,
 * or file name with its attribute), or a short listing of the directory
 * entry (file name only without any attribute), and should be stored in
 * the buffer pointed to by bufferPtr up to bufferSize.
 * The FTP server will keep on calling tfFSGetNextDirEntry until it returns
 * 0 bytes, to read all the matching directory entries. It is up to the
 * implementor of tfFSGetNextDirEntry to keep track of how many entries of
 * the listing have been read so far, and to keep track of whether the
 * directory was open for long or short listing, and to keep track of the
 * matching pattern (using dirDataPtr).
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * dirDataPtr    Pointer to newly allocated directory data structure
 * bufferPtr     pointer to a buffer where to copy the next directory entry
 * bufferSize    size in bytes of the buffer.
 *
 * Returns
 * Value         Meaning
 *  -1           failure
 *  > 0          number of bytes copyied into the buffer pointed to by
 *               bufferPtr.
 *   0           end of directory.
 */
int tfFSGetNextDirEntry ( void TM_FAR * userDataPtr,
                          void TM_FAR * dirDataPtr,
                          char TM_FAR * bufferPtr,
                          int           bufferSize )
{
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(dirDataPtr);
    TM_UNREF_IN_ARG(bufferPtr);
    TM_UNREF_IN_ARG(bufferSize);

    return -1;
}


/* 
 * tfFSCloseDir function description:
 * Given a unique user data pointer as returned by tfFSUserLogin, and a
 * directory data pointer as returned by tfFSOpenDir, close the directory,
 * and free the directory data structure pointed to by dirDataPtr.
 * Parameters
 * Parameter     Description
 * userDataPtr   Pointer to user data structure as returned by tfFSUserLogin
 * dirDataPtr    Pointer to newly allocated directory data structure
 *
 * Returns
 * no return value
 */
void tfFSCloseDir ( void TM_FAR * userDataPtr, void TM_FAR * dirDataPtr )
{
#define userFsDataPtr ((ttUserFsDataPtr)userDataPtr)
    TM_UNREF_IN_ARG(dirDataPtr);

/* This command is only valid if a user has logged in. */
    if (userDataPtr != TM_VOID_NULL_PTR)
    {
        userFsDataPtr->userDirPtr = (ttVoidPtr)0;
    }
#undef userFsDataPtr

    return;
}

static void tfFSUserDir(ttConstUserFsDataPtr userFsDataPtr)
{

/* This command is only valid if a user has logged in. */    
    if (userFsDataPtr != (ttConstUserFsDataPtr)0)
    {
        (void)tm_chdir(userFsDataPtr->userWorkingDirectory);
    }

    return;
}

/* This interface function expects the user to provide the most efficient 
 * file block size. Normally it would be the cluster size of the 
 * underlying file system. This is used to optimize the file read and 
 * write performance. If returns zero, the default size 
 * TM_FS_FILE_BLOCK_SIZE (512) will be used by the stack.
 */
int tfFSGetFileBlockSize(void)
{
    return 0;
}

int tfFSFileExists(ttConstVoidPtr userDataPtr, ttConstCharPtr pathNamePtr)
{
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(pathNamePtr);

    return 0;
}

/* Seek to a file position */
int tfFSSeekFile(
    ttVoidPtr       userDataPtr,
    ttVoidPtr       fileDataPtr,
    long            offset,
    int             whence)
{
    int                 errorCode;

    TM_UNREF_IN_ARG(userDataPtr);
    if (fileDataPtr == (ttVoidPtr)0)
    {
        errorCode = TM_EINVAL;
    }
    else if (fseek(fileDataPtr, offset, whence) == 0)
    {
        errorCode = TM_ENOERROR;
    }
    else
    {
        errorCode = TM_EIO;
    }

    return errorCode;
}

ttUser32Bit tfFSGetFileLen(
    const void TM_FAR * userDataPtr,
    const char TM_FAR * pathNamePtr )
{
    TM_UNREF_IN_ARG(userDataPtr);
    TM_UNREF_IN_ARG(pathNamePtr);

    return 0;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: txlinuxa.c    File Revision: 6.0.2.10 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  7386    Output checksum: 31324
 */
