/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2008
 * Copyright Elmic Systems Japan 1997 - 2008
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Kernel Hooks
 *              NOTE: This File is intended to be modified by the user
 *                    In order to add a new operating system
 * Filename: trkernel.c
 * Author: Lorne
 * Date Created: 2008/10/27
 * $Source: source/kernel/none/rabbit/rcm3209/softools/trkernel.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:07CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/* Modify the include file for your Kernel/OS */
/*#include <kernel.h> */

/*#define TM_USE_SEM_FOR_EVENT*/

/* For Treck Kernel interface */
#include <trsocket.h>

/* For malloc, free */
#include <stdlib.h>

/* For printf to the serial console */
#include <stdio.h>


#ifdef TM_DYNAMIC_CREATE_SHEAP
/* For Treck simple heap */
#define PAGE_COUNT (TM_SHEAP_SIZE/TM_SHEAP_PAGE_SIZE)
ttUser8Bit TM_FAR buf0[TM_SHEAP_PAGE_SIZE];
#if PAGE_COUNT > 1
#error
ttUser8Bit TM_FAR buf1[TM_SHEAP_PAGE_SIZE];
#endif
#if PAGE_COUNT > 2
ttUser8Bit TM_FAR buf2[TM_SHEAP_PAGE_SIZE];
#endif
struct tsBufferInfo {
	ttUser8BitPtr ptr;
	ttUser32Bit size;
	};
struct tsBufferInfo buffers[] = { { buf0, sizeof(buf0) }
#if PAGE_COUNT > 1
		, { buf1, sizeof(buf1) }
#endif
#if PAGE_COUNT > 2
		, { buf2, sizeof(buf2) }
#endif
		};

/* numbers of dynamic sheap buffers we're allocated */
static int TM_GLOBAL_QLF tlSheapBufCount = 0;

/*
 * Dynamically allocate the Treck simple heap. Only called when
 * TM_DYNAMIC_CREATE_SHEAP has been #define'd.
 * User can change the size of the created heap by updating *sheapSizePtr
 * prior to calling malloc.
 * The start of the simple heap (i.e. the pointer returned) being
 * aligned on a 32-bit boundary.
 * Return:
 * Pointer to beginning of dynamically created heap.
 */
ttUser32Bit TM_FAR * tfKernelSheapCreate(ttUser32Bit TM_FAR * sheapSizePtr)
{
	ttUser8BitPtr ptr;
	int bump;

	if (tlSheapBufCount >= sizeof(buffers) / sizeof(struct tsBufferInfo))
	{
		tfKernelError("tfKernelSheapCreate",
					  "Two many sheap blocks requested");
		return NULL;
	}

	/* Align pointer to ttUser32Bit boundary and adjust size accordingly */
	ptr = buffers[tlSheapBufCount].ptr;
	bump = -(int)ptr & (sizeof(ttUser32Bit) - 1);
	*sheapSizePtr = buffers[tlSheapBufCount].size - bump;
	return ptr + bump;
}
#endif /* TM_DYNAMIC_CREATE_SHEAP */


#ifdef TM_CRITICAL_NEEDED
/*
 * Stop all other calls into Treck by a kernel context switch
 */
void tfKernelSetCritical()
{
	CriticalSection_Start();
}

/*
 * Resume normal operation (from tm_kernel_set_critical)
 */
void tfKernelReleaseCritical()
{
	CriticalSection_End();
}
#endif /* TM_CRITICAL_NEEDED */


#if defined(TM_LOCK_NEEDED) || defined(TM_PEND_POST_NEEDED)
/*
 * Create a counting semaphore
 */
int tfKernelCreateCountSem(ttUserGenericUnionPtr countingSemaphore)
{
/* CMS requires the applcation to keep track of the reference to the semaphor. */
	int errorCode;

	if ( K_Semaphore_Create(tlSemCount, 0)  == K_OK)
	{
		countingSemaphore->gen16BitParm = tlSemCount;
		tlSemCount ++;
		errorCode = TM_KERN_OKAY;
	}
	else
	{
		errorCode = TM_KERN_ERROR;
	}

	return errorCode;
}

/*
 * Wait on a counting semaphore
 */
int tfKernelPendCountSem(ttUserGenericUnionPtr countingSemaphore)
{
	int errorCode;

/* Wait on the semaphore indefinately */
	if (K_Semaphore_Wait(countingSemaphore->gen16BitParm, 0) == K_OK)
	{
		errorCode = TM_KERN_OKAY;
	}
	else
	{
		errorCode = TM_KERN_ERROR;
	}

	return errorCode;
}

#ifdef TM_USE_PEND_TIMEOUT
/*
 * Wait on a counting semaphore up to a timeout value (in ticks)
 */
int tfKernelPendCountSemTimeout( ttUserGenericUnionPtr countingSemaphore,
								 ttUser32Bit tickTimeout )
{
	int errorCode;

/* Wait on the semaphore for  tickTimeout in system ticks*/
	if (K_Semaphore_Wait(countingSemaphore->gen16BitParm, tickTimeout) == K_OK)
	{
		errorCode = TM_KERN_OKAY;
	}
	else
	{
		errorCode = TM_KERN_ERROR;
	}

	return errorCode;

}
#endif /* TM_USE_PEND_TIMEOUT */

/*
 * Resume waiting tasks on a counting semaphore
 */
int tfKernelPostCountSem(ttUserGenericUnionPtr countingSemaphore)
{
	int errorCode;

	if (K_Semaphore_Post(countingSemaphore->gen16BitParm) == K_OK)
	{
		errorCode = TM_KERN_OKAY;
	}
	else
	{
		errorCode = TM_KERN_ERROR;
	}

	return errorCode;
}

/*
 * Delete a counting semaphore
 */
int tfKernelDeleteCountSem(ttUserGenericUnionPtr countingSemaphore)
{
	int errorCode;
/* CMX doesn't have API for deleting a semaphor, however, reset might be the
 * next match, it flushes the semaphore queue, even if it is owned by other
 * tasks
 */
	if (K_Semaphore_Reset(countingSemaphore->gen16BitParm, 1) == K_OK)
	{
		errorCode = TM_KERN_OKAY;
	}
	else
	{
		errorCode = TM_KERN_ERROR;
	}

	return errorCode;
}
#endif /* defined(TM_LOCK_NEEDED) || defined(TM_PEND_POST_NEEDED) */


void  tfKernelError(const char TM_FAR * functionName,
					const char TM_FAR *errorMessage)
{
/* print to the serial console and stop */
	printf("tfKernelError! %s %s\n", functionName, errorMessage);
}

void  tfKernelWarning(const char TM_FAR *functionName,
					  const char TM_FAR *errorMessage)
{
/* print to the serial console and stop */
	printf("tfKernelWarning! %s %s\n", functionName, errorMessage);
}

void  tfKernelInitialize(void)
{
}

/*
 * Yield the CPU. Used by tfInterfaceSpinLock().
 */
void tfKernelTaskYield(void)
{
	printf("\ntfKernelTaskYield\n");
}

#ifdef TM_PEND_EVENT_NEEDED
/*
 * Create an event structure for pend/post from an ISR, and
 * also for XMIT task pend/post.
 */
void tfKernelCreateEvent(ttUserGenericUnionPtr eventPtr)
{
	eventPtr->gen16BitParm = tlEventCount++;
}

/*
 * Task wait on an Event
 */
void tfKernelPendEvent(ttUserGenericUnionPtr eventPtr)
{
	TM_UNREF_IN_ARG(eventPtr);
	K_Event_Wait(eventPtr->gen8BitParm,
				 0,       /* wait indefinately*/
				 3 );     /* clear the event before waiting and after awaken*/
}

#ifdef TM_USE_PEND_TIMEOUT
/*
 * Task wait on an Event up to a timeout value (in ticks).
 */
int tfKernelPendEventTimeout( ttUserGenericUnionPtr eventPtr,
							   ttUser32Bit tickTimeout )
{
	K_Event_Wait(eventPtr->gen8BitParm ,
				 tickTimeout,/* wait */,
				 3 );        /* clear the event before waiting and after awaken*/
}
#endif /* TM_USE_PEND_TIMEOUT */

/*
 * Resume task waiting on this event. Called from an ISR.
 */
void tfKernelIsrPostEvent(ttUserGenericUnionPtr eventPtr)
{
	K_Event_Signal(3,/* All tasks wating for this event */
				   0,
				   eventPtr->gen8BitParm);
}

/*
 * Resume task waiting on this Event. Called from a task.
 */
void tfKernelTaskPostEvent(ttUserGenericUnionPtr eventPtr)
{
	K_Event_Signal(3,/* All tasks wating for this event */
				   0,
				   eventPtr->gen8BitParm);
}

#endif /* TM_PEND_EVENT_NEEDED */

/***/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: trkernel.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  5725    Output checksum: 48862
 */
