/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2005
 * Copyright Elmic Systems Japan 1997 - 2005
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:
 * Kernel hooks for Stretch S56XX
 *
 * Filename: kernel.c
 * Author: Jared August
 * Date Created: 8/26/2005
 * $Source: source/kernel/none/s56xx/kernel.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:10CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <stdio.h>
#include <stdlib.h>

#include <trsystem.h>
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

#include "main.h"

#if (defined TM_56XX_HEAP_IN_SRAM) && (defined TM_56XX_HEAP_IN_SDRAM)
#error TM_56XX_HEAP_IN_SRAM and TM_56XX_HEAP_IN_SDRAM both defined!
#endif /* (defined TM_56XX_HEAP_IN_SRAM) && (define TM_56XX_HEAP_IN_SDRAM) */

#ifdef TM_56XX_HEAP_IN_SRAM
tt8Bit simpleHeap[TM_SHEAP_SIZE] __attribute__ ((aligned(4)))
                            __attribute__ ((section(".sram.data")));
#endif /* TM_56XX_HEAP_IN_SRAM */
#ifdef TM_56XX_HEAP_IN_SDRAM
tt8Bit simpleHeap[TM_SHEAP_SIZE] __attribute__ ((aligned(4)));
#endif /* TM_56XX_HEAP_IN_SRAM */

/* Output an unrecoverable error and halt execution */
void  tfKernelError(const char TM_FAR *functionName,
                    const char TM_FAR *errorMessage)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    printf("tfKernelError! %s: %s\n", functionName, errorMessage);
    tm_set_current_context(contextHandle);
    while(1);
}

/* Output a warning */
void  tfKernelWarning(const char TM_FAR *functionName,
                      const char TM_FAR *errorMessage)
{
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    tm_get_current_context(contextHandle);
    printf("tfKernelWarning! %s: %s\n", functionName, errorMessage);
    tm_set_current_context(contextHandle);
}

/* Global variable used by in-line critical section assembly code */
unsigned rval __attribute__ ((section(".dram.data")));

/* Don't have any initialization to do */
void  tfKernelInitialize(void)
{
    ;
}

/* Return a pointer to the simple heap */
ttUser32Bit * tfKernelSheapCreate(ttUser32Bit * sheapSize)
{
    return (ttUser32Bit*)simpleHeap;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: kernel.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 21898    Output checksum: 11141
 */
