/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/none/tic3x/tiger31/ti/bus30.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:17CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*****************************************************************************/
/* bus30.h v4.5                                                              */
/* Copyright (c) 1992 Texas Instruments Incorporated			     */
/*****************************************************************************/
#ifndef _BUS30
#define _BUS30


#define exp_gcontrol      _egctrl._intval
#define exp_gcontrol_bit  _egctrl._bitval
#define prim_gcontrol     _pgctrl._intval
#define prim_gcontrol_bit _pgctrl._bitval

/**************************************************************************/
/* MACRO DEFINITION FOR BUS BASE ADDRESS                                  */
/**************************************************************************/
#define BUS_BASE        0x808060
#define BUS_ADDR        ((volatile BUS_REG *) ((char *) BUS_BASE))

/**************************************************************************/
/* STRUCTURE DEFINITION FOR PRIM AND EXP BUS GLOBAL CONTROL REGISTERS     */
/**************************************************************************/
typedef union
{
    unsigned int _intval;
    struct
    {
         unsigned int holdst   :1;                     /* Hold status bit */
         unsigned int nohold   :1;                    /* Port hold signal */
         unsigned int hiz      :1;                       /* Internal hold */
         unsigned int sww      :2;                       /* S/W wait mode */
         unsigned int wtcnt    :3;                      /* S/W wait count */
         unsigned int bnkcmp   :5;                        /* Bank compare */
         unsigned int r_rest   :19;                           /* reserved */
    } _bitval;
} PRIMARY_BUS_CONTROL;

typedef union
{
    unsigned int _intval;
    struct
    {
         unsigned int r_012    :3;                            /* reserved */
         unsigned int sww      :2;                       /* S/W wait mode */
         unsigned int wtcnt    :3;                      /* S/W wait count */
         unsigned int r_rest   :24;                           /* reserved */
    } _bitval;
} EXPANSION_BUS_CONTROL;


/**************************************************************************/
/* STRUCTURE DEFINITION FOR BUS REGISTERS                                 */
/**************************************************************************/
typedef struct
{
    EXPANSION_BUS_CONTROL _egctrl;     /* Exapansion bus control register */
    unsigned int          reserved1[3];                       /* reserved */
    PRIMARY_BUS_CONTROL   _pgctrl;        /* Primary bus control register */
} BUS_REG;



/**************************************************************************/
/* BUS MACROS FOR BIT FIELD DEFINITIONS                                   */
/**************************************************************************/
/* Bus macros for global control registers integer assignments            */
/* e.g. BUS_ADDR->prim_gcontrol = EXTERNAL_RDY | WS_1;   sets the         */
/* corresponding bits of the primary bus control register.                */
/**************************************************************************/
#define HOLDST          0x1
#define NOHOLD          0x2
#define HIZ             0x4
#define EXTERNAL_RDY    0x00
#define INTERNAL_RDY    0x08
#define OR_EXT_INT      0x10
#define AND_EXT_INT     0x18
#define WS_0            0x00
#define WS_1            0x20
#define WS_2            0x40
#define WS_3            0x60
#define WS_4            0x80
#define WS_5            0xA0
#define WS_6            0xC0
#define WS_7            0xE0
#define BANK_16M        0x00
#define BANK_8M         0x100
#define BANK_4M         0x200
#define BANK_2M         0x300
#define BANK_1M         0x400
#define BANK_512K       0x500
#define BANK_256K       0x600
#define BANK_128K       0x700
#define BANK_64K        0x800
#define BANK_32K        0x900
#define BANK_16K        0xA00
#define BANK_8K         0xB00
#define BANK_4K         0xC00
#define BANK_2K         0xD00
#define BANK_1K         0xE00
#define BANK_512        0xF00
#define BANK_256        0x1000


/**************************************************************************/
/* Bus macros for global control registers bit field assignments          */
/* e.g. BUS_ADDR->prim_gcontrol_bit.sww = AND_RDY;                        */
/*      BUS_ADDR->exp_gcontrol_bit.wtcnt = WAIT_2;                        */
/* sets the appropriate bits of bus control registers.                    */
/**************************************************************************/
#define HOLD            1
#define INT_RDY         0
#define EXT_RDY         1
#define OR_RDY          2
#define AND_RDY         3
#define BANK_SIZE_16M   0
#define BANK_SIZE_8M    1
#define BANK_SIZE_4M    2
#define BANK_SIZE_2M    3
#define BANK_SIZE_1M    4
#define BANK_SIZE_512K  5
#define BANK_SIZE_256K  6
#define BANK_SIZE_128K  7
#define BANK_SIZE_64K   8
#define BANK_SIZE_32K   9
#define BANK_SIZE_16K   10
#define BANK_SIZE_8K    11
#define BANK_SIZE_4K    12
#define BANK_SIZE_2K    13
#define BANK_SIZE_1K    14
#define BANK_SIZE_512   15
#define BANK_SIZE_256   16
#define WAIT_0          0
#define WAIT_1          1
#define WAIT_2          2
#define WAIT_3          3
#define WAIT_4          4
#define WAIT_5          5
#define WAIT_6          6
#define WAIT_7          7

#endif /* #ifndef _BUS30 */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: bus30.h       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6184    Output checksum: 61902
 */
