/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/none/tic3x/tiger31/ti/main.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:19CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
                 
#include <trsocket.h>                    
#include <stdio.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <tr16550.h>

#include "bus30.h"
#include "tiger31.h"

#define OUR_IP_ADDR "192.168.100.10"
#define PEER_IP_ADDR "192.168.100.1"

#define CONFIG_V4
/* #define CONFIG_V6 */

int pppOpened;
int pppClosed;

#define TM_BUF_LEN 1024
int outDataBuf[TM_BUF_LEN];

void pppNotify(ttUserInterface interfaceHandle, unsigned int flags)
{
    interfaceHandle=interfaceHandle;
    if (flags & TM_LL_CLOSE_COMPLETE)
    {
        pppClosed = 1;
    }

    if (flags & TM_LL_OPEN_COMPLETE)
    {
        pppOpened = 1;
    }
        
}

void executeTreckLoop(ttUserInterface interfaceHandle)
{
    tfTimerExecute();

    if (tfCheckReceiveInterface(interfaceHandle) == 0)
    {
        tfRecvInterface(interfaceHandle);
    }
}


#define T31_CMD0   (*(volatile ttUser32Bit *) 0xcffff0)  /* command 0 reg   */

#ifndef BIT
#define	BIT(n)	((1)<<(n))
#endif

/* Command Register 0 Bits definitions */
#define	INTROMDIS	BIT(0)	/* 0x01 */
#define	URESET_		BIT(1)	/* 0x02 */
#define	CTFCLR_		BIT(2)	/* 0x04 */
#define	C31IRQ		BIT(3)	/* 0x08 */
#define	RED_LED		BIT(4)	/* 0x10 */
#define	GREEN_LED	BIT(5)	/* 0x20 */
#define	GPOUT0		BIT(6)	/* 0x40 */
#define	GPOUT1		BIT(7)	/* 0x80 */

unsigned T31_control(unsigned mask, unsigned pattn)
{
    static unsigned shadow = 0;

    shadow = ( ~mask & shadow ) | ( mask & pattn );
    T31_CMD0 = shadow;
    return shadow;
}

void main(void)
{
    int errorCode;
    int sd;
    struct sockaddr_in addr;
    int i, j;
    int bufLength;
    ttUserInterface interfaceHandle;
    ttUserLinkLayer linkLayerHandle;
    ttUserIpAddress ipAddress;
    ttUserIpAddress netmask;
    struct sockaddr_storage ipAddr;

/* Setup the external bus.  This is important to access the 16550 controller
   correctly. */
    BUS_ADDR->prim_gcontrol = BANK_1M | WS_7 | OR_EXT_INT;
/* release UART reset, set LEDs */    
    T31_control(URESET_ | GREEN_LED | RED_LED, URESET_ | GREEN_LED | RED_LED);

    tfInitTreck();

    errorCode = tfInitTreckOptions(TM_OPTION_TICK_LENGTH, 50);
        
    errorCode = tfStartTreck();

    if (errorCode != TM_ENOERROR)
    {
        printf("tfStartTreck: %d\n", errorCode);
    }
    
    T31_timer_init();

    linkLayerHandle = tfUseAsyncPpp(pppNotify);
    
    interfaceHandle=tfAddInterface("T31.RS232",
                                   linkLayerHandle,
                                   tf16550Open,
                                   tf16550Close,
                                   tf16550Send,
                                   tf16550Receive,
                                   tf16550FreeReceiveBuffer,
                                   tf16550Ioctl,
                                   (ttDevGetPhysAddrFuncPtr) 0,
                                   &errorCode);

#ifdef CONFIG_V4
/* Allow server to set it's IP address. */
    if (errorCode == TM_ENOERROR)
    {
        ipAddress = 0UL;    
        errorCode = tfPppSetOption(interfaceHandle,
                                   TM_PPP_IPCP_PROTOCOL,
                                   TM_PPP_OPT_ALLOW,
                                   TM_IPCP_IP_ADDRESS,
                                   (const char *) &ipAddress,
                                   sizeof(ipAddress));
    }

/* Request an IP address from the server. */
    if (errorCode == TM_ENOERROR)
    {    
        ipAddress = 0UL;    
        errorCode = tfPppSetOption(interfaceHandle,
                                   TM_PPP_IPCP_PROTOCOL,
                                   TM_PPP_OPT_WANT,
                                   TM_IPCP_IP_ADDRESS,
                                   (const char *) &ipAddress,
                                   sizeof(ipAddress));
    }
    
    pppOpened = 0;
    pppClosed = 0;

    ipAddress=0UL;
    netmask=0UL;
    errorCode=tfConfigInterface(interfaceHandle,
                                ipAddress,
                                netmask,
                                TM_DEV_SCATTER_SEND_ENB,
                                5,  
                                0);
#endif /* CONFIG_V4 */
    
#ifdef CONFIG_V6
    tm_bzero(&ipAddr, sizeof(ipAddr));
    ipAddr.ss_family = AF_INET6;
    ipAddr.ss_len = sizeof(ipAddr);
    errorCode = tfNgOpenInterface(interfaceHandle,
                                  &ipAddr,
                                  0,
                                  TM_DEV_SCATTER_SEND_ENB,
                                  0,
                                  5,
                                  ipv6Notify);
#endif /* CONFIG_V6 */
    
    while ( (pppOpened == 0) && (pppClosed == 0))
    {
        executeTreckLoop(interfaceHandle);
    }

    sd = socket(AF_INET, SOCK_DGRAM, 0);
        
    if (sd == TM_SOCKET_ERROR)
    {
        printf("Socket error\n");
    }

    tm_bzero(&ipAddr, sizeof(ipAddr));

    errorCode = inet_pton(AF_INET,
                          PEER_IP_ADDR,
                          &ipAddr.addr.ipv4.sin_addr);

    ipAddr.ss_family = AF_INET;
    ipAddr.ss_port = htons(9);
    ipAddr.ss_len = sizeof(ipAddr);
    
    bufLength = 16;
    for (i=0;i<100;i++)
    {
        for (j=0;j<(bufLength>>2);j++)
        {
            outDataBuf[j] = ((j+i) % 255);
        }

        errorCode = sendto(sd, 
                         (char *) outDataBuf, bufLength, 0,
                         (const struct sockaddr *) &ipAddr, sizeof(ipAddr));
        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(sd);
            tm_assert(main, (errorCode == TM_EWOULDBLOCK));
        }
        
        executeTreckLoop(interfaceHandle);
        bufLength += 16;
        if (bufLength >= TM_BUF_LEN)
        {
            bufLength = 16;
        }
    }         
    
    return;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: main.c        File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 59949    Output checksum:  8017
 */
