/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/none/tic3x/tiger31/ti/main_lb.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:20CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
                 
#include <trsocket.h>                    
#include <stdio.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <tr16550.h>

#include "bus30.h"
#include "tiger31.h"

#define DEST_IP_ADDR "127.0.0.1"
#define DISCARD_PORT 9

int outDataBuf[32];
unsigned int bytesSent;
unsigned int bytesRcvd;

				/* pointer to command 0 and status reg */
volatile unsigned long *comd_stat = (unsigned long *)(0xcffff0);

					/* pointer to transfer registers */
volatile unsigned long *xfer_reg = (unsigned long *)(0xcffff3);

#define TM_PORT_LOOPBACK_TEST htons(10)
#define TM_BUF_ARRAY_SIZE     1460

char bufferArray[TM_BUF_ARRAY_SIZE + 1];
char rxBufferArray[TM_BUF_ARRAY_SIZE + 1];
/*
 * Test using TCP send/recv
 * Gives the client IP address, server IP address, the data length in one
 * send(), and number of iterations in the send/recv loop, and whether
 * to send the time stamp option in every packet.
 */
void testTcp ( ttUserIpAddress clientIpAddr, ttUserIpAddress peerIpAddr,
               int sendDataLength, int iterations, int tsOn );

/*
 * Test using TCP zero copy send / zero copy recv
 * Gives the client IP address, server IP address, the data length in one
 * send(), and number of iterations in the send/recv loop, and whether
 * to send the time stamp option in every packet.
 */
void testzTcp ( ttUserIpAddress clientIpAddr, ttUserIpAddress peerIpAddr,
                int sendDataLength, int iterations, int tsOn );

#ifndef BIT
#define	BIT(n)	((1)<<(n))
#endif

/* Command Register 0 Bits definitions */
#define	INTROMDIS	BIT(0)	/* 0x01 */
#define	URESET_		BIT(1)	/* 0x02 */
#define	CTFCLR_		BIT(2)	/* 0x04 */
#define	C31IRQ		BIT(3)	/* 0x08 */
#define	RED_LED		BIT(4)	/* 0x10 */
#define	GREEN_LED	BIT(5)	/* 0x20 */
#define	GPOUT0		BIT(6)	/* 0x40 */
#define	GPOUT1		BIT(7)	/* 0x80 */

unsigned T31_control(unsigned mask, unsigned pattn)
{
    static unsigned shadow = 0;

    shadow = ( ~mask & shadow ) | ( mask & pattn );
    T31_CMD0 = shadow;
    return shadow;
}

void main(void)
{
    int errorCode;
    int csd, lsd, asd;
    struct sockaddr_in addr;
    int i, j;
    unsigned ledTest;
    int bufLength;
    ttUserInterface interfaceHandle;
    ttUserLinkLayer linkLayerHandle;
    ttUserIpAddress ipAddress;
    ttUserIpAddress netmask;
    struct sockaddr_storage ipAddr;

/* Setup the external bus.  This is important to access the 16550 controller
   correctly. */
    BUS_ADDR->prim_gcontrol = BANK_1M | WS_7 | OR_EXT_INT;
/* release UART reset, set LEDs */    
    T31_control(URESET_ | GREEN_LED | RED_LED, URESET_ | GREEN_LED | RED_LED);

    bytesSent = 0;
    bytesRcvd = 0;
    tfInitTreck();

    errorCode = tfInitTreckOptions(TM_OPTION_TICK_LENGTH, 50);
        
    errorCode = tfStartTreck();

    if (errorCode != TM_ENOERROR)
    {
        printf("tfStartTreck: %d\n", errorCode);
    }
    printf("Treck started\n");
    
    T31_timer_init();

/* test the LEDs */
	*comd_stat = 0x34;		/* set both LEDs */

	while (*comd_stat & 0x40)	/* sync */
		;
	*xfer_reg = 0xa5a5a5a5;

	*comd_stat = 0x14;		/* set RED LED */

	while (*comd_stat & 0x40)	/* sync */
		;
	*xfer_reg = ~0xa5a5a5a5;

	*comd_stat = 0x24;		/* set GREEN LED */

#ifdef LED_TEST
/*	LED 0 will toggle with a steady rate				*/
/*	LED 1 will toggle with half the LED 0 rate			*/
	for (ledTest = 0; ; ledTest++)
		*comd_stat = (ledTest >> (20 - 4)) & 0xF0 | 0x04;
#endif /* LED_TEST */
    printf("test");

    ipAddress= inet_addr(DEST_IP_ADDR);
    netmask= inet_addr("255.255.0.0");

    testTcp(
        ipAddress, ipAddress, 3, 1000000, 0 /* tsOn */);
    
    return;
}

void testTcp ( ttUserIpAddress clientIpAddr, ttUserIpAddress peerIpAddr,
               int sendDataLength, int iterations, int tsOn )
{
    int                 errorCode;
    int                 failed;
    struct sockaddr_in  addr;
    int                 lsd;
    int                 csd;
    int                 asd;
    int                 recvdLength;
    int                 retCode;
    int                 sentLength;
    ttSockAddrPtrUnion  tempSockAddr;
    unsigned int        tempUInt;
    int					i;
    int					origSendDataLength;

	tempUInt = 0;
    for (i = 0; i < TM_BUF_ARRAY_SIZE; i++)
    {
        bufferArray[i] = tempUInt;
        if (tempUInt == 0xffffffff)
        {
            tempUInt = 0;
        }
        else
        {
            tempUInt += 0x01010101;
        }
    }
    i = 0;
    origSendDataLength = sendDataLength;
    
    tempSockAddr.sockInPtr = &addr;
    TM_UNREF_IN_ARG(clientIpAddr);
    failed = 0;
    csd = TM_SOCKET_ERROR;
    asd = TM_SOCKET_ERROR;
/* Open the listening TCP socket */
    lsd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
    if (lsd == TM_SOCKET_ERROR)
    {
/* Retrieve the socket error for failed socket call */
        errorCode = tfGetSocketError(lsd);
        printf( "socket failed '%s'\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
/* Bind the TCP listening socket to a well known TCP port */
        addr.sin_family = PF_INET;
        addr.sin_port = TM_PORT_LOOPBACK_TEST;
        addr.sin_addr.s_addr = peerIpAddr;
        errorCode = bind( lsd,
                          tempSockAddr.sockPtr,
                          sizeof(struct sockaddr_in));
        if (errorCode == TM_SOCKET_ERROR)
        {
/* Retrieve the socket error for failed bind call */
            errorCode = tfGetSocketError(lsd);
            printf( "bind failed '%s'\n",
                     tfStrError(errorCode));
            failed = 1;
        }
    }
    if (failed == 0)
    {
        (void)tfBlockingState(lsd, TM_BLOCKING_OFF);
/* Listen for incoming connections */
        errorCode = listen(lsd, 1);
        if (errorCode == TM_SOCKET_ERROR)
        {
/* Retrieve the socket error for failed listen call */
            errorCode = tfGetSocketError(lsd);
            printf( "listen failed '%s'\n",
                     tfStrError(errorCode));
            failed = 1;
        }
    }
/* Open the connecting socket */
    if (failed == 0)
    {
        csd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
        if (csd == TM_SOCKET_ERROR)
        {
/* Retrieve the socket error for failed socket call */
            errorCode = tfGetSocketError(csd);
            printf( "socket failed '%s'\n",
                     tfStrError(errorCode));
            failed = 1;
        }
    }
    if (failed == 0)
    {
        (void)tfBlockingState(csd, TM_BLOCKING_OFF);
/* Turn off (tsOn == 0), or on (tsOn == 1) the time stamp option */
        errorCode = setsockopt(csd, IPPROTO_TCP, TM_TCP_TS,
                               (const char TM_FAR *)&tsOn, sizeof(tsOn));

        if (errorCode == TM_SOCKET_ERROR)
        {
/* Retrieve the socket error for failed bind call */
            errorCode = tfGetSocketError(csd);
            printf( "setsockopt TM_TCP_TS failed '%s'\n",
                     tfStrError(errorCode));
            failed = 1;
        }
    }
    if (failed == 0)
    {
/* Connect to the listening socket * (sin_family already initialized) */
        addr.sin_port = TM_PORT_LOOPBACK_TEST;
        addr.sin_addr.s_addr = peerIpAddr;
        errorCode = connect ( csd,
                              tempSockAddr.sockPtr,
                              sizeof(struct sockaddr_in));
        if (errorCode == TM_SOCKET_ERROR)
        {
/* Retrieve the socket error for failed bind call */
            errorCode = tfGetSocketError(csd);
            printf( "connect failed '%s'\n",
                     tfStrError(errorCode));
            failed = 1;
        }
    }
    if (failed == 0)
    {
        printf("Connected to the listening socket\n");
/* Accept the connection */
        asd = accept(lsd,
                     (struct sockaddr TM_FAR *)0,
                     (int TM_FAR             *)0);
        if (asd == TM_SOCKET_ERROR)
        {
/* Retrieve the socket error for failed bind call */
            errorCode = tfGetSocketError(lsd);
            printf( "accept failed '%s'\n",
                     tfStrError(errorCode));
            failed = 1;
        }
    }
    if (failed == 0)
    {
        do
        {
/* Send some loop back data to the accepted socket */
            sentLength = send( csd,
                               bufferArray + i,
                               sendDataLength,
                               0);
            if (sentLength == TM_SOCKET_ERROR)
            {
/* Retrieve the socket error for failed send call */
                errorCode = tfGetSocketError(csd);
                if (errorCode != TM_EWOULDBLOCK)
                {
                    printf( "send failed '%s'\n", tfStrError(errorCode));
                    failed = 1 ;
                    break;
                }
                else
                {
                    printf("Send blocked\n");
                }
            }
            else
            {
                if (sentLength < sendDataLength)
                {
                    sendDataLength -= sentLength;
                }
                else
                {
                    sendDataLength = origSendDataLength;
                    i ++;
                    if ((i + sendDataLength) >= TM_BUF_ARRAY_SIZE)
                    {
                        i = 0;
                    }
                }
                bytesSent += sentLength;
                printf("%d sent successfully\n", sentLength);
            }
/*
 * Receive some loop back data from the connecting socket. Loop to make sure
 * we receive all the data, in case the send size exceeds the eff. MSS.
 */
            recvdLength = 0;
            errorCode = TM_ENOERROR;
            do
            {
                retCode = recv( asd,
                                rxBufferArray,
                                sizeof(rxBufferArray),
                                0);
                if (retCode > 0)
                {
                    recvdLength += retCode;
                }
                else
                {
                    if (retCode == TM_SOCKET_ERROR)
                    {
/* Retrieve the socket error for failed recv call */
                        errorCode = tfGetSocketError(asd);
                        if (errorCode != TM_EWOULDBLOCK)
                        {
/*
 * ErrorCode makes us exit the inner loop, and failed makes us exit the
 * outer loop
 */
                            failed = 1;
                            printf( "recv failed '%s'\n",
                                     tfStrError(errorCode));
                        }
                    }
                    else
                    {
/* End of file */
                        printf("Remote closed the connection\n");
                        iterations = 1; /* to make us exit the outer loop */
                        break; /* to make us exit the inner loop */
                    }
                }
            }
/* Loop until there is no more data to recv, or we failed */
            while (errorCode == TM_ENOERROR);
            if (recvdLength != 0)
            {
                bytesRcvd += recvdLength;
                printf("%d received succesfully\n", recvdLength);
            }
            iterations--;
        }
/* Loop until we have reached our limit, or we failed */
        while (iterations && failed == 0);
    }
/* All done. */
    if (csd != TM_SOCKET_ERROR)
    {
/* Close the connecting socket */
        (void)tfClose(csd);
    }
    if (asd != TM_SOCKET_ERROR)
    {
/* Close the accepted socket */
        (void)tfClose(asd);
    }
    if (lsd != TM_SOCKET_ERROR)
    {
/* Close the listening socket */
        (void)tfClose(lsd);
    }
    if (failed == 0)
    {
        printf("TCP LOOP BACK Test with send/recv success\n");
    }
}



/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: main_lb.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 29634    Output checksum: 42798
 */
