/*
 * $Source: source/kernel/none/tic3x/tiger31/ti/t31.cmd $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:22CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $ 
 *
 *  ======== t31.cmd ========
 *
 *  Linker command file for DSP Research TIGER 31 Board
 *  - The Run Time System library must be the last library listed below.
 *  - Code is external ram, while data is in internal ram.
 *  - Location 0x809800 is used by the boot loader.
 *  - Make Sure startc3x.obj is linked first (_c_int00 must be at adr 0x1000
 *    in .map file).
 *  - Make Sure cinit = ffffffff in .map file.
 *
 */
-cr				/* RAM MODEL only with startc3x.asm */
-stack 0x800
-heap 0x100

/* make sure libraries are in directory defined by environment var: C_DIR */

-l rts30r.lib			/* "standard" C functions, register model */

MEMORY
{
    SRAM0 : org = 0x001000, len = 0x40000	    /* external ram        */

/* When the C31 is used in "microprocessor mode", the interrupt vector table
   starts at address 0x0.  Otherwise, the vector table resides at 0x809fc0 */
/*    VECTORS (RWI) : org = 0x0, len = 0x00040 */   /* reset/intr. vectors */
    VECTORS (RWI) : org = 0x809fc0, len = 0x00040   /* reset/intr. vectors */
	UART (RW): org = 0xfffff0, len = 0x00010
}

SECTIONS
{
    GROUP:
    {
	.text: { }				/* CODE SECTION	*/

	.textend: {_TEXTEND = . ;}		/* USED BY MEMORY TEST */

	.cinit:	{ }	/* INITIALIZED DATA VALUES FOR ROM MODEL (NOT USED) */

	.const: { }				/* CONSTANTS */

        .bss: { }				/* GLOBAL AND STATIC VARS */

        .data: { }				/* INITIALIZED VARS */

        .stack: { }				/* SYSTEM STACK */
        					/* ALSO INITIALIZED DATA! */    
    } > SRAM0

    .vectors	: {
        _VECTORS = .; 
			} > VECTORS
}


/* Memory map description for CodeComposer debugger. */
/*
MA 0x000000,0x001000,RAM ; RAM if internal ROM is disabled
MA 0x001000,0x040000,RAM ; 256k Words of External SRAM
MA 0x808020,0x000010,RAM ; On-Chip Timer 0
MA 0x809fc0,0x000040,RAM ; Interrupt Vectors (On-Chip)
*/
