/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Support routines for the Tiger 31 DSP development board
 *              Most functions derived from sample/tutorial code included
 *              with Tiger 31.
 *
 * Filename: tiger31.c
 * Author: Jason Schmidlapp
 * Date Created: 4/28/2003
 * $Source: source/kernel/none/tic3x/tiger31/ti/tiger31.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:23CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <tiger31.h>
#include <trsocket.h>

/* constant declarations and enumerations */

/* type definitions */

/* local function prototypes */
interrupt void T31_timer_interrupt(void);

/* local variable definitions */

/* static const tables */

/* macro definitions */

/* STatus register bits. */
#define ST_GIE_BIT 0x00002000 /* Enable/disable global interrupts. */

#define T31_TMR0_CTL_PTR 0x808020
#define T31_TMR0_PRD_PTR 0x808028

/* Timer global-control register bits. */
#define T31_TMR0_CTL_FUNC       0x00000001 /* Configures TCLK as a timer pin. */
#define T31_TMR0_CTL_IO         0x00000002 /* Configures TCLK for output.     */
#define T31_TMR0_CTL_STOP       0x00000000 /* Stops the timer.                */
#define T31_TMR0_CTL_START      0x000000C0 /* Resets and starts the timer.    */
#define T31_TMR0_CTL_CLOCK_MODE 0x00000100 /* Configure timer in 'clock mode' */
#define T31_TMR0_CTL_INT_CLKSRC 0x00000200 /* Configure timer to use internal
                                              clock.                          */


/* Returns the value of the Interrupt Enable register. */
int ie_value(void)
{
    asm("        LDI   IE,R0     ;Get IE value");
}


/* Returns the value of the CPU status register. */
int st_value(void)
{
    asm("        LDI   ST,R0     ;Get ST value");
}


/* Disables global interrupts */
void T31_disable_interrupts(void)
{
/*
    asm("        RPTS  0");
    asm("        AND   0DFFFh,ST     ;Set GIE = 0");
*/
    
    unsigned st;
    st = st_value();
    st = st & ~ST_GIE_BIT;
    load_st(st);

}

/* Enables global interrupts.
   Use the RPTS command to ensure that we're not interrupted while changing
   the GIE bit.  This is a potential problem due to the pipelining of the
   C3x */
void T31_enable_interrupts(void)
{
/*
    asm("        RPTS  0");
    asm("        OR    2000h,ST     ;Set GIE = 1");
*/    

    unsigned st;
    st = st_value();
    st = st | ST_GIE_BIT;
    load_st(st);

}


/* Updates the Interrupt Enable register.
   Use the RPTS command to ensure that we're not interrupted while changing
   the GIE bit.  This is a potential problem due to the pipelining of the
   C3x */
unsigned T31_ie(unsigned mask, unsigned data)
{
    unsigned ie;
    ie = ie_value();
    ie = (ie & ~mask) | (data & mask);
    load_ie(ie);
    return ie;
}

/*
 * Install ISR handler and enable specific interrupt.
 */ 
int T31_setISR(int vector, void (*c_int)(void))
{
    long        mask, d;
    unsigned long        address;

    address = vector;
    vector &= 0x3f;
    d = (unsigned)c_int;
    d |= 0x60000000;    /* BR instruction */
    *(unsigned *) address = d;
    mask = BIT(vector);
    mask >>= 1;
    T31_ie(mask, mask);
    return 0;
}

extern void timerIsr(void);

void T31_timer_init(void)
{
    unsigned int * timerCtlPtr;
    unsigned int * timerPeriodPtr;

    timerCtlPtr = (unsigned int *) T31_TMR0_CTL_PTR;
     timerPeriodPtr = (unsigned int *) T31_TMR0_PRD_PTR;
    
/* Stop the timer. */
    *timerCtlPtr =   T31_TMR0_CTL_FUNC
                   | T31_TMR0_CTL_IO
                   | T31_TMR0_CTL_STOP
                   | T31_TMR0_CTL_CLOCK_MODE
                   | T31_TMR0_CTL_INT_CLKSRC;

/* Install timer interrupt. */
    T31_setISR((long) &TINT0, T31_timer_interrupt);    

/* Set period of timer to 50 ms */
    *timerPeriodPtr = 390000;


    *timerCtlPtr =   T31_TMR0_CTL_FUNC
                   | T31_TMR0_CTL_IO
                   | T31_TMR0_CTL_START
                   | T31_TMR0_CTL_CLOCK_MODE
                   | T31_TMR0_CTL_INT_CLKSRC;

    
    T31_enable_interrupts();
}


int timerCount = 0;

interrupt void T31_timer_interrupt(void)
{
    timerCount++;
    tfTimerUpdateIsr();
}

 

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: tiger31.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 25230    Output checksum: 59387
 */
