/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Test for IQ80321 board + Intel 8254x Device Driver
 *
 * Filename: ipsec.c
 * Author: Jin Shang
 * Date Created:
 * $Source: source/kernel/none/xscale/iq80314/gnu/ipsec.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:29CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/* printf in console window*/
#include <stdio.h>
#include <string.h>

#include <trsocket.h>
#include <tr80314.h>

#define TM_LOCAL_IP "192.168.7.111"
#define TM_PEER_IP  "192.168.7.222"




int main()
{
    struct sockaddr_storage localIpAddr;
    struct sockaddr_storage peerIpAddr;
    ttUserIpAddress         netMask;
    ttUserLinkLayer         linkLayerHandle;
    ttUserInterface         interfaceHandle;
    int                     errorCode;
    int                     configFlags;
    int                     scatteredBufferCount;
    int                     ssend;
    ttUserMessage           bufferHandle;
    char      *             dataPtr;
    int                     bufferLength;
    int                     caseNumber;
/* print out test info in console window */


/* Start Treck */
    tfInitTreck();
    tfStartTreck();
    
/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();

/* Add ethernet interface */

    interfaceHandle=tfAddInterface(
            "forestlake",
            linkLayerHandle,
            tf80314Open,
            tf80314Close,
            tf80314Send,
            tf80314Receive,
			(ttDevFreeRecvFuncPtr)0,
            tf80314Ioctl,
            tf80314GetPhysAddr,
            &errorCode);

    tf80314Initialize(interfaceHandle,
                     0);

/* Allow scatter send */
    configFlags     =  0; /*TM_DEV_SCATTER_SEND_ENB;*/
    scatteredBufferCount = 1;

    memset(&localIpAddr, 0, sizeof(localIpAddr));
    localIpAddr.ss_family       = PF_INET;
    localIpAddr.ss_port = 1000;
    localIpAddr.addr.ipv4.sin_addr.s_addr = inet_addr(TM_LOCAL_IP);
    netMask                     = htonl(0xffffff00); 

    memset(&peerIpAddr, 0, sizeof(peerIpAddr));
    peerIpAddr.ss_family       = PF_INET;
    peerIpAddr.ss_port = htons(9);
    peerIpAddr.addr.ipv4.sin_addr.s_addr = inet_addr(TM_PEER_IP);



/* Config the Interface */    
    errorCode = tfConfigInterface(
                interfaceHandle,
                localIpAddr.addr.ipv4.sin_addr.s_addr,
                netMask,
                configFlags,  
                scatteredBufferCount,  
                0);

    caseNumber = 1; /* UDP 1472 */
//    caseNumber = 2; /* UDP 64 */
//    caseNumber = 3; /* TCP 1448 */
//    caseNumber = 4; /* TCP 64 */


    if((caseNumber == 1) || (caseNumber == 2))
    {
        if(caseNumber == 1)
        {
            bufferLength = 1472;
        }
        else
        {
            bufferLength = 64;
        }
        ssend = socket(PF_INET, SOCK_DGRAM, IP_PROTOUDP);
/* Turn UDP checksum on or off, according to flags from user */
        (void)tfSetTreckOptions( TM_OPTION_UDP_CHECKSUM, 
                           1 );
        
        errorCode = bind(ssend,
                     (struct sockaddr TM_FAR *)&localIpAddr.addr.ipSockAddr,
                     sizeof(localIpAddr)
                    );

        while(1)
        {
            
            bufferHandle=tfGetZeroCopyBuffer(bufferLength,&dataPtr);
            errorCode = tfZeroCopySendTo(ssend,
                                         bufferHandle,
                                         bufferLength,
                                         0,
                                         (const struct sockaddr*)&peerIpAddr,
                                         sizeof(peerIpAddr));
            tfTimerExecute();
            if(tfCheckReceiveInterface(interfaceHandle)
                == TM_ENOERROR)
            {
                tfRecvInterface(interfaceHandle); 
                break;
            }
        }

        while(1)
        {
            
            bufferHandle=tfGetZeroCopyBuffer(bufferLength,&dataPtr);
            errorCode = tfZeroCopySendTo(ssend,
                                         bufferHandle,
                                         bufferLength,
                                         0,
                                         (const struct sockaddr*)&peerIpAddr,
                                         sizeof(peerIpAddr));
        }

    }
    else
    {
        if(caseNumber == 3)
        {
            bufferLength = 1448;
        }
        else
        {
            bufferLength = 64;
        }
        ssend = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
        (void)tfBlockingState(ssend, TM_BLOCKING_OFF);
        errorCode = bind(ssend,
                     (struct sockaddr TM_FAR *)&localIpAddr.addr.ipSockAddr,
                     sizeof(localIpAddr)
                    );
        if(errorCode) return -1;

        errorCode=connect(ssend,
            (const struct sockaddr TM_FAR *)&peerIpAddr,
            sizeof(peerIpAddr));
/*        if(errorCode) return -1; */


        while(1)
        {
            tfTimerExecute();
            if(tfCheckReceiveInterface(interfaceHandle)
                == TM_ENOERROR)
            {
                tfRecvInterface(interfaceHandle); 
            }
            
            bufferHandle=tfGetZeroCopyBuffer(bufferLength,&dataPtr);
            tfZeroCopySend ( ssend,
                bufferHandle,
                bufferLength,
                0);
//            tf80314Nanosleep(2000000);
        }
    }
    
    
    return 0;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:42      For: Xilinx-Samsung
 * File Name: ipsec.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 47757    Output checksum:  3122
 */
