/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/quadros/motorola/coldfire/metrowerks/quaddemo.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:38:36CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * Demo of the Turbo Treck TCP/IP stack and the Quadros RTOS with the
 * Metrowerks toolset.
 */

#include <trsocket.h> 
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <trmcffec.h>

#include "rtxcapi.h"
#include "ktask.h"
#include "kproject.h"
#include "rtxcstdio.h"

/* #define only one of the following: */
#define TM_DEMO_USE_DHCP
/* #define TM_DEMO_USE_STATIC_IP */

#ifdef TM_DEMO_USE_STATIC_IP
#define TM_OUR_IP_ADDRESS     "192.168.200.200"
#define TM_OUR_IP_NETMASK     "255.255.255.0"
#define TM_GATEWAY_IP_ADDRESS "192.168.200.1"
#endif /* TM_DEMO_USE_STATIC_IP */

/* If this file is being compiled with EventAnalyzer support... */
#ifdef TX_ENABLE_EVENT_LOGGING

/* then declare the linker-defined begin and end symbols for the .eventlog
   section.  */
extern char __ghsbegin_eventlog[];
extern char __ghsend_eventlog[];
#endif

/* declare the linker_defined end symbol for the .free_mem section.  */
extern char __ghsend_free_mem[];

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Support for multiple interfaces */
ttUserInterface interfaceHandleArray[5];
#define ETHERNET_INTERFACE_1  0


/* Define thread and other function prototypes.  */
void startTreck(void);
void treckAppTask(void);
void treckTimerTask(void);
void treckRecvTask(void);
void treckTcpServerTask(void);
void treckUdpServerTask(void);

#define TM_TEST_BUFFER_SIZE      1468
#define TM_TEST_TCP_SERVER_PORT  7
#define TM_TEST_UDP_SERVER_PORT  7



void pingCallback(int sockDesc);

int sendtoNoCopy(struct sockaddr_in * toAddress, int size);
int sendNoCopy(struct sockaddr_in * toAddress, 
               int                  size, 
               int                  noDelayFlag, 
               int                  echoFlag);
int recvfromNoCopy(int size);

/* our local IP address and netmask */
ttUserIpAddress localIpAddr;
ttUserIpAddress localNetMask;
ttUserIpAddress gatewayIpAddr;
int addrConfigStatus;

/* Define what the initial system looks like.  */

void startTreck(void) 
{
    ttUserLinkLayer    linkLayerHandle;
    ttUserInterface    interfaceHandle;
    int                errorCode;
    KSRC status;

    addrConfigStatus = 0;
    rtxc_printf("Treck Starting");

/* Initialize the stack */
    tfInitTreck();
    tfStartTreck();
    tm_bzero((ttUserCharPtr)(ttUserVoidPtr)&interfaceHandleArray[0], 
              sizeof(interfaceHandleArray));

/* Use the Ethernet link layer */    
    linkLayerHandle = tfUseEthernet();
/* Add the network interface */
    interfaceHandle=tfAddInterface(
            "MCF5282.1",
            linkLayerHandle,
            tfMcf5282Open,
            tfMcf5282Close,
            tfMcf5282Send,
            tfMcf5282Receive,
            (ttDevFreeRecvFuncPtr)0,
            tfMcf5282Ioctl,
            tfMcf5282GetPhyAddr,
            &errorCode);

    interfaceHandleArray[ETHERNET_INTERFACE_1] = interfaceHandle;

#ifdef TM_USE_DRV_ONE_SCAT_SEND
    errorCode = tfUseInterfaceOneScatSend(interfaceHandle, tfMcf5282ScatSend);
#endif /* TM_USE_DRV_ONE_SCAT_SEND */
    errorCode = tfUseInterfaceXmitQueue(interfaceHandle, 100);
#ifdef TM_USE_DRV_SCAT_RECV  
    errorCode = tfUseInterfaceScatRecv(interfaceHandle, tfMcf5282ScatRecv);
#endif /* TM_USE_DRV_SCAT_RECV */

/* Start all threads in the system.  */

/* Receive Task */
    status = KS_ExecuteTask(TRKRCV);

    if (status != RC_GOOD)
    {
        tfKernelError("startTreck",
                      "Failed to start the TRKRCV task");

    }
    
/* Timer Task */    
    status = KS_ExecuteTask(TRKTIME);
    if (status != RC_GOOD)
    {
        tfKernelError("startTreck",
                      "Failed to start the TRKTIME task");
    }
    
/* Application Task */    
    status = KS_ExecuteTask(APPTASK);

    if (status != RC_GOOD)
    {
        tfKernelError("startTreck",
                      "Failed to start the APPTASK task");
    }    

/* Tcp Echo Server Task */    
    status = KS_ExecuteTask(TRKTCP);

    if (status != RC_GOOD)
    {
        tfKernelError("startTreck TCP echo server",
                      "Failed to start the TRKTCP task");
    }    

/* Udp Echo Server Task */    
    status = KS_ExecuteTask(TRKUDP);

    if (status != RC_GOOD)
    {
        tfKernelError("startTreck UDP echo server",
                      "Failed to start the TRKUDP task");
    }    

    return;
}

#ifdef TM_DEMO_USE_DHCP
/* notification function for DHCP address configuration */
void myDhcpNotifyFunc(
    ttUserInterface interfaceHandle, int errorCode)
{
    switch(errorCode)
    {
    case TM_ENOERROR:
/* we have successfully auto-configured an address using DHCP */
        addrConfigStatus = 1;
        errorCode = tfGetIpAddress(
            interfaceHandle,
            &localIpAddr,
            0);
        errorCode = tfGetNetMask(
            interfaceHandle,
            &localNetMask,
            0);
        break;

    case TM_ETIMEDOUT:
        rtxc_printf("DHCP client timed-out, no DHCP server was found.\n");
        break;

    default:
        rtxc_printf("DHCP client error %u.\n", errorCode);
        break;
    }

    return;
}
#endif /* TM_DEMO_USE_DHCP */

void treckAppTask(void)
{
    ttUserInterface             interfaceHandle;
    int                         errorCode;
    int                         testType;
    int                         testSize;    
    struct sockaddr_in          toAddress;
    struct sockaddr_storage     localSockAddr;
    struct in_addr              tempInAddr;



/* Avoid compiler warning */
    rtxc_printf("Turbo Treck task starting\n");

    interfaceHandle = interfaceHandleArray[ETHERNET_INTERFACE_1]; 
/* configure our local IP address */
    

#ifdef TM_DEMO_USE_STATIC_IP
    localIpAddr = inet_addr(TM_OUR_IP_ADDRESS);
    localNetMask = inet_addr(TM_OUR_IP_NETMASK);
    gatewayIpAddr = inet_addr(TM_GATEWAY_IP_ADDRESS);

/* Open the ethernet interface */
    errorCode=tfOpenInterface(
        interfaceHandle,
        localIpAddr,
        localNetMask,
        TM_DEV_MCAST_ENB | TM_DEV_SCATTER_SEND_ENB,  
        2); /* scattered buffers per frame count */

    if (errorCode == TM_ENOERROR)
    {
        addrConfigStatus = 1;
    }

    tfAddDefaultGateway(interfaceHandle, gatewayIpAddr);
#endif /* TM_DEMO_USE_STATIC_IP */
#ifdef TM_DEMO_USE_DHCP
    rtxc_printf("Starting DHCP auto-configuration...\n");

    tfUseDhcp(interfaceHandle, myDhcpNotifyFunc);

/* Open the ethernet interface */
    errorCode=tfOpenInterface(
        interfaceHandle,
        0,
        0,
        TM_DEV_MCAST_ENB | TM_DEV_SCATTER_SEND_ENB | TM_DEV_IP_DHCP,  
        2); /* scattered buffers per frame count */

    while (!addrConfigStatus)
    {
        KS_SleepTask(TIMEBASE, 1000/CLKTICK);
    }
#endif /* TM_DEMO_USE_DHCP */

    memset(&localSockAddr, 0, sizeof(localSockAddr));
    localSockAddr.ss_family = PF_INET;
    localSockAddr.ss_len = sizeof(localSockAddr);
    localSockAddr.addr.ipv4.sin_addr.s_addr = localIpAddr;
    localSockAddr.addr.ipv4.sin_port = htons(80);
    
    rtxc_printf(
        "  local IP address:   %s\n",
        inet_ntoa(localSockAddr.addr.ipv4.sin_addr));
    tempInAddr.s_addr = localNetMask;
    rtxc_printf("  local network mask: %s\n", inet_ntoa(tempInAddr));

    errorCode = tfGetDefaultGateway(&gatewayIpAddr);
    if (errorCode == TM_ENOERROR)
    {
        tempInAddr.s_addr = gatewayIpAddr;
        rtxc_printf("  gateway IP address: %s\n", inet_ntoa(tempInAddr));
    }

    toAddress.sin_family      = PF_INET;
    toAddress.sin_addr.s_addr = inet_addr("192.168.0.1");
    toAddress.sin_port        = htons(9);

    testType = 6;
    switch (testType)
    {
/* Test #1 - small UDP zero copy send */        
        case 1:
        testSize = 16;
        sendtoNoCopy(&toAddress, testSize);
        break;

/* Test #2 - large UDP zero copy send */
    case 2:
        testSize = 1472;
        sendtoNoCopy(&toAddress, testSize);
        break;
        
/* Test #3 - TCP zero copy send */      
    case 3:
        testSize = 1448;
        sendNoCopy(&toAddress, testSize, 0, 0);
        break;

/* Test #4 - TCP zero copy echo */      
    case 4:
        testSize = 512;
        toAddress.sin_port = htons(7);
        sendNoCopy(&toAddress, testSize, 0, 1);
        break;

/* Test #5 - Ping */
    case 5:    
        errorCode = tfPingOpenStart (
            "192.168.200.1",
            0,
            100,
            pingCallback );
	break;

/* Test #6 - Web Server */

    }

    while (1)
    {
        KS_SleepTask(TIMEBASE, 1000/CLKTICK);
    }
} 

int udpRecvCount;

/*
 * recvfromNoCopy
 */
int recvfromNoCopy(int size)
{
    char *         testPtr;
    int                errorCode;
    ttUserMessage      bufferHandle;
    int            udpSd;
    struct sockaddr_in bindAddr;

    udpSd = socket(PF_INET, SOCK_DGRAM, 0);

    bindAddr.sin_family      = PF_INET;
    bindAddr.sin_addr.s_addr = 0UL;
    bindAddr.sin_port        = htons(9);
    
    errorCode = bind(udpSd,
                     (struct sockaddr *) &bindAddr,
                     sizeof(bindAddr));
    
    udpRecvCount = 0;
    while (1) 
    {
        errorCode = tfZeroCopyRecvFrom(
            udpSd,
            &bufferHandle,
            &testPtr,
            size,
            0,
            0,
            0);
    
        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(udpSd);
            if (errorCode != TM_EWOULDBLOCK)
            {
                rtxc_printf("tfZeroCopyRecvFrom error = %d\n", errorCode);
                break;
            }
        }
        else
        {
            tfFreeZeroCopyBuffer(bufferHandle);
        }

        udpRecvCount++;

    }
    
    return errorCode;
}    
    
int udpSendCount;

/*
 * sendtoNoCopy
 */
int sendtoNoCopy(struct sockaddr_in *toAddress, int size)
{
    char *         testPtr;
    int            errorCode;
    ttUserMessage  bufferHandle;
    int            udpSd;

    udpSd = socket(PF_INET, SOCK_DGRAM, 0);
    
    udpSendCount = 0;
    while (1) 
    {
        bufferHandle=tfGetZeroCopyBuffer(size, &testPtr);

        errorCode = tfZeroCopySendTo(
            udpSd,
            bufferHandle,
            size,
            0,
            (const struct sockaddr TM_FAR *) toAddress,
            (int) sizeof(struct sockaddr) ); 

        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(udpSd);
            if (errorCode != TM_EWOULDBLOCK)
            {
                rtxc_printf("tfZeroCopySendTo error = %d\n", errorCode);
                break;
            }
        }

        udpSendCount++;

    }
    
    return errorCode;
}

int tcpSendCount;

char sendBuffer[1000];

/*
 * sendNoCopy
 */
int sendNoCopy(struct sockaddr_in * toAddress, 
               int                  size, 
               int                  noDelayFlag, 
               int                  echoFlag)
{
    char *         testPtr;
    int            errorCode;
    ttUserMessage  bufferHandle;
    int            tcpSd;
    int            optParam;
    
    tcpSd = socket(PF_INET, SOCK_STREAM, 0);

    if (noDelayFlag)
    {
    optParam = 1;
    errorCode = setsockopt(tcpSd,
                           IP_PROTOTCP,
                           TCP_NODELAY,
                           (const char *) &optParam,
                           sizeof(optParam));
    
    if (errorCode != TM_ENOERROR)
    {
        goto sendNoCopyFinish;
    }
    }


    errorCode = connect(tcpSd,
                        (struct sockaddr *) toAddress,
                        sizeof(struct sockaddr_in));

    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(tcpSd);
        rtxc_printf("connect error = %d\n", errorCode);  
        goto sendNoCopyFinish;
    }
    
    tcpSendCount = 0;
    testPtr = sendBuffer;
    while (1) 
    {

        bufferHandle=tfGetZeroCopyBuffer(size, &testPtr);

        errorCode = tfZeroCopySend(tcpSd,
                                   bufferHandle,
                                   size,
                                   0);

        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(tcpSd);
            rtxc_printf("tfZeroCopySend error = %d\n", errorCode);
            goto sendNoCopyFinish;
        }

        if (echoFlag)
        {

            errorCode = tfZeroCopyRecv(tcpSd,
                                       &bufferHandle,
                                       &testPtr,
                                       size,
                                       0);
        
            if (errorCode == TM_SOCKET_ERROR)
            {       
                errorCode = tfGetSocketError(tcpSd);
                rtxc_printf("tfZeroCopyRecv error = %d\n", errorCode);
                goto sendNoCopyFinish;
            }
            else
            {
                if (bufferHandle != (ttUserMessage) 0)
                {
                    tfFreeZeroCopyBuffer(bufferHandle);
                }

            }
        
        }

        tcpSendCount++;
    }

sendNoCopyFinish:    
    return errorCode;
}    



/*
 * Treck timer task
 */
void treckTimerTask(void)
{
    ttUserInterface interfaceHandle;

    interfaceHandle = interfaceHandleArray[ETHERNET_INTERFACE_1];
    
    while (1)
    {
        tfTimerUpdate();
        tfTimerExecute();
        tfIoctlInterface(
            interfaceHandle, 
            TM_DEV_IOCTL_EMPTY_XMIT_FLAG | TM_MCF5282_IOCTL_REFILL_RX
            | TM_MCF5282_SEND_COMPLETE,
            (void *) 0, 
            0);

        KS_SleepTask(TIMEBASE, 1/CLKTICK);
    }
}    

/*
 * treckRecvTask
 */
void treckRecvTask(void)
{
    ttUserInterface interfaceHandle;
    unsigned int interfaceIndex;

    interfaceIndex = ETHERNET_INTERFACE_1;
    while (interfaceHandleArray[interfaceIndex] == (ttUserInterface)0)
    {
        KS_SleepTask(TIMEBASE, 10/CLKTICK);
    }
    interfaceHandle = interfaceHandleArray[interfaceIndex];

    while (1)
    {
        tfWaitReceiveInterface(interfaceHandle);
#ifdef TM_USE_DRV_SCAT_RECV  
        tfRecvScatInterface(interfaceHandle);
#else /* !TM_USE_DRV_SCAT_RECV */
        tfRecvInterface(interfaceHandle);
#endif /* !TM_USE_DRV_SCAT_RECV */
    }
}    

void pingCallback(int sockDesc)
{
    rtxc_printf("Ping reply\n");
}

/* TCP echo server task */
void treckTcpServerTask(void)
{
    int                         errorCode;
    int                         serverSd;
    int                         acceptSd;
    int                         optValue;
    struct sockaddr_storage     remoteAddr;
    int                         remoteAddrLen;
    struct sockaddr_storage     localSockAddr;
    ttUserMessage               bufferHandle;
    char *                      dataPtr;
    int                         bytesRecv;
    int                         bytesSent;

    rtxc_printf("Entering Treck TCP echo server task.\n");

/* wait for interface to be opened */
    while (!addrConfigStatus)
    {
        KS_SleepTask(TIMEBASE, 1000/CLKTICK);
    }

    memset(&localSockAddr, 0, sizeof(localSockAddr));
    localSockAddr.ss_family = PF_INET;
    localSockAddr.ss_len = sizeof(localSockAddr);
    localSockAddr.addr.ipv4.sin_addr.s_addr = 0;
    localSockAddr.addr.ipv4.sin_port = htons(7);
    
    serverSd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
    if(serverSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        rtxc_printf(
            "taskTreckTcpEcho failed to create socket, %d.\n", errorCode);
        return;
    }

    errorCode = bind(
        serverSd, (struct sockaddr*)&localSockAddr, sizeof(localSockAddr));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        rtxc_printf("taskTreckTcpEcho failed to bind, %d.\n", errorCode);
        tfClose(serverSd);
        return;
    }

    errorCode = listen(serverSd, 2);
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        rtxc_printf(
            "taskTreckTcpEcho failed to listen, %d.\n", errorCode);
        tfClose(serverSd);
        return;
    }

    for(;;)
    {
        acceptSd = accept(
            serverSd,
            (struct sockaddr TM_FAR *)0,
            (int TM_FAR             *)0 );

        if(acceptSd == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(serverSd);
            rtxc_printf(
                "taskTreckTcpEcho failed to accept, %d.\n", errorCode);
            break;
        }
        else
        {
            remoteAddrLen = sizeof(remoteAddr);
            errorCode = getpeername(
                acceptSd, (struct sockaddr *)&remoteAddr, &remoteAddrLen);
            if (errorCode == TM_SOCKET_ERROR)
            {
                errorCode = tfGetSocketError(acceptSd);
                rtxc_printf(
                    "TCP echo client Get Peer name, %s connected\n",
                    tfStrError(errorCode));
            }
            else
            {
                rtxc_printf("TCP echo client %s connected\n",
                       inet_ntoa(remoteAddr.addr.ipv4.sin_addr));
            }

            for(;;)
            {
#ifdef TM_USE_DRV_SCAT_RECV
                bufferHandle = tfGetZeroCopyBuffer(
                    TM_TEST_BUFFER_SIZE, &dataPtr);
                if (bufferHandle == (ttUserMessage)0)
                {
                    rtxc_printf("tfGetZeroCopyBuffer failed\n");
                    break;
                }
                
/* recv is more efficient when using scattered receive */
                bytesRecv = recv(acceptSd,
                                 dataPtr,
                                 TM_TEST_BUFFER_SIZE,
                                 0);
#else /* ! TM_USE_DRV_SCAT_RECV */
                bytesRecv = tfZeroCopyRecv(
                    acceptSd,
                    &bufferHandle,
                    &dataPtr,
                    1500,
                    0);
#endif /* ! TM_USE_DRV_SCAT_RECV */
        
                if (bytesRecv > 0)
                {
                    bytesSent = tfZeroCopySend(
                        acceptSd,
                        bufferHandle,
                        bytesRecv,
                        0);

                    if (bytesSent == TM_SOCKET_ERROR)
                    {
                        errorCode = tfGetSocketError(acceptSd);
                        switch(errorCode)
                        {
                        case TM_ECONNRESET:
                            rtxc_printf("TCP echo client connection reset.\n");
                            break;
                            
                        default:
                            rtxc_printf(
                                "taskTreckTcpEcho failed to send, %d.\n",
                                errorCode);
                            break;
                        }
                        tfClose(acceptSd);
                        break;
                    }
                }
                else
                {
                    if (bytesRecv == TM_SOCKET_ERROR)
                    {
                        errorCode = tfGetSocketError(acceptSd);
                        switch(errorCode)
                        {
                        case TM_ECONNRESET:
                            rtxc_printf("TCP echo client connection reset.\n");
                            break;
                            
                        default:
                            rtxc_printf(
                                "taskTreckTcpEcho failed to recv, %d.\n",
                                errorCode);
                            break;
                        }
                        tfClose(acceptSd);
                        break;
                    }
                    else
                    {
                        rtxc_printf("TCP echo client disconnected.\n");
                        tfClose(acceptSd);
                        break;
                    }
                }
            }
        }
    }

    tfClose(serverSd);

/*    errorCode = tfTestTreck(
        TM_TEST_TCP_ECHO_SERVER,
        TM_OUR_IP_ADDRESS,
        htons(TM_ECHO_PORT),
        8192,
        0,
        (unsigned long int)TM_TEST_FLAG_ZEROCOPY,
        (ttUserTestHandlePtr)0,
        (void TM_FAR *)0UL); */
    
    return;
}

/*
 * treckUdpServerTask
 */
void treckUdpServerTask(void)

{
    int                         errorCode;
    int                         serverSd;
    struct sockaddr_storage     remoteAddr;
    int                         remoteAddrLen;
    struct sockaddr_storage     localSockAddr;
    ttUserMessage               bufferHandle;
    char *                      dataPtr;
    int                         bytesRecv;
    int                         bytesSent;
    
    rtxc_printf("Entering Treck UDP echo server task.\n");

/* wait for interface to be opened */
    while (!addrConfigStatus)
    {
        KS_SleepTask(TIMEBASE, 1000/CLKTICK);
    }
    
    memset(&localSockAddr, 0, sizeof(localSockAddr));
    localSockAddr.ss_family = PF_INET;
    localSockAddr.ss_len = sizeof(localSockAddr);
    localSockAddr.addr.ipv4.sin_addr.s_addr = 0;
    localSockAddr.addr.ipv4.sin_port = htons(7);
    
startUdpTest:
    serverSd = socket(PF_INET, SOCK_DGRAM, IP_PROTOUDP);
    if(serverSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        rtxc_printf(
            "taskTreckUdpEcho failed to create socket, %d.\n", errorCode);
        return;
    }

    errorCode = bind(
        serverSd, (struct sockaddr*)&localSockAddr, sizeof(localSockAddr));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        rtxc_printf("taskTreckUdpEcho failed to bind, %d.\n", errorCode);
        tfClose(serverSd);
        return;
    }

    remoteAddrLen = sizeof(remoteAddr);
    for(;;)
    {
        bufferHandle = tfGetZeroCopyBuffer(
            TM_TEST_BUFFER_SIZE, &dataPtr);
        if (bufferHandle == (ttUserMessage)0)
        {
            rtxc_printf("tfGetZeroCopyBuffer failed\n");
            break;
        }
                
/* recvfrom is more efficient when using scattered receive */
        bytesRecv = recvfrom(
            serverSd,
            dataPtr,
            TM_TEST_BUFFER_SIZE,
            0,
            (struct sockaddr *) &remoteAddr,
            &remoteAddrLen);
        
        if (bytesRecv > 0)
        {
            bytesSent = tfZeroCopySendTo(
                serverSd,
                bufferHandle,
                bytesRecv,
                0,
                (struct sockaddr *) &remoteAddr,
                remoteAddrLen);
            
            if (bytesSent == TM_SOCKET_ERROR)
            {
                errorCode = tfGetSocketError(serverSd);
                rtxc_printf(
                    "taskTreckUdpEcho failed to send, %d.\n",
                    errorCode);
            }
        }
        else
        {
            if (bytesRecv == TM_SOCKET_ERROR)
            {
                errorCode = tfGetSocketError(serverSd);
                tfClose(serverSd);
                goto startUdpTest;
            }
            else
            {
/* we were just sent a 0-byte UDP message, we can't send a 0-byte
   UDP message so instead we just free the received message here */
                if (bufferHandle != (ttUserMessage) 0)
                {
                    tfFreeZeroCopyBuffer(bufferHandle);
                }
            }
        }
    }
    
/*    errorCode = tfTestTreck(
        TM_TEST_UDP_ECHO_SERVER,
        TM_OUR_IP_ADDRESS,
        htons(TM_ECHO_PORT),
        3000,
        0,
        (unsigned long int)(TM_TEST_FLAG_ZEROCOPY
                            | TM_TEST_FLAG_UDP_CS_OFF),
        (ttUserTestHandlePtr)0,
        (void TM_FAR *)0UL); */
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:43      For: Xilinx-Samsung
 * File Name: quaddemo.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 54984    Output checksum:   180
 */
