;***********************************************************
; RZK	RTOS  - eZ80
; Copyright (c)  2000,  ZiLOG.
;
;
;
; Function	: 	RZKStartup
; Purpose	:	This function will clear memory and will 
;				set up the stack.
; $Source: source/kernel/rzk/ez80/l92boot.s $
;
; Modification History
;   $Revision: 6.0.2.2 $
;   $Date: 2010/01/18 06:38:40CST $
;   $Author: odile $
;   $ProjectName: /home/mks/proj/tcpip.pj $
;   $ProjectRevision: 6.0.1.23 $
;					
;**************************************************************
;-------------------------------------------
; Chip Select / Wait State Generator
CS0_BMC: .equ %F0 
CS1_BMC: .equ %F1 
CS2_BMC: .equ %F2 
CS3_BMC: .equ %F3 

CS_BMC0_DATA:	.equ	%82
CS_BMC1_DATA:	.equ	%01
CS_BMC2_DATA:	.equ	%81
CS_BMC3_DATA:	.equ	%84

;*******************************************************************************
;* Section:		IRQVect                                                         *
;*                                                                             *
;* This section will hold the Interrupt Vector Table and the ISR Jump Table.   *
;* When an interrupt occurs on the eZ80 in ADL mode, the CPU retrieves the     *
;* 2-byte vector number from IVT.  It will then start executing code (the ISR) *
;* at address {0 | I | [vect]}.  Therefore, the vector table must be located   *
;* in the first 64kB of addressable memory.  It is unlikely that all ISRs can  *
;* be located in the first 64kB of the address space, so a Jump Table is       *
;* created in memory immediately after the Vector table.  This table just      *
;* contains code of the form jmp.lil 24-bit ISR address.  This allows the ISR  *
;* to exist anywhere in the 24-bit ADL address space.                          *
;*******************************************************************************
;******************************************************************
	define .save, space=ROM, org=%0000
	segment .save
	.def _INT_VECTOR_BASE
_INT_VECTOR_BASE:
	ds %0100 - %0000

;/***********************************************************
;	Function	:	tfTimerISRProlog
;	Dependencies	:	all externs
;**************************************************************/
    .sect ".isr"
	.assume adl=1
	.def _InvalidInterrupt
_InvalidInterrupt:
	JP.il _InvalidInterruptHandler

	.extern _RZKISRProlog
	.extern _RZKISREpilog
	.extern _tfTimerIsrHandler
	.def _tfTimerIsrProlog
_tfTimerIsrProlog:
.if 1
	PUSH	AF
	IN0     A,(83H)			; Read control reg.
	PUSH	BC
	PUSH	DE
	PUSH	HL		; scratch reg for function call from isr
	PUSH	IX
	PUSH	IY
	CALL	_RZKISRProlog
;	EI
	CALL	_tfTimerIsrHandler	;Interrupt specific C code
	DI
	CALL	_RZKISREpilog
	POP		IY
	POP     IX
	POP		HL
	POP		DE
	POP		BC
	POP		AF
.else
	DI
	IN0     A,(83H)			; Read control reg.
	CALL	_tfTimerIsrHandler	;Interrupt specific C code
.endif
	EI
	RETI

	.extern _tfCrystalIsrHandler
	.def _tfCrystalIsrProlog
_tfCrystalIsrProlog:
.if 1
	PUSH	AF
	PUSH	BC
	PUSH	DE
	PUSH	HL		; scratch reg for function call from isr
	PUSH	IX
	PUSH	IY
	CALL	_RZKISRProlog
;	EI
	CALL	_tfCrystalIsrHandler	;Interrupt specific C code
	DI
	CALL	_RZKISREpilog
	POP		IY
	POP     IX
	POP		HL
	POP		DE
	POP		BC
	POP		AF
.else
	DI
	CALL	_tfCrystalIsrHandler	;Interrupt specific C code
.endif
	EI
	RETI

	.extern _tfUartIsrHandler
	.def _tfUartIsrProlog
_tfUartIsrProlog:
.if 1
	PUSH	AF
	PUSH	BC
	PUSH	DE
	PUSH	HL		; scratch reg for function call from isr
	PUSH	IX
	PUSH	IY
	CALL	_RZKISRProlog
;	EI
	CALL	_tfUartIsrHandler	;Interrupt specific C code
	DI
	CALL	_RZKISREpilog
	POP		IY
	POP     IX
	POP		HL
	POP		DE
	POP		BC
	POP		AF
.else
	DI
	CALL	_tfUartIsrHandler	;Interrupt specific C code
.endif
	EI
	RETI

;******************************************************************
    .sect	".startup"
;	.org 00E400h
	RZK_ROM: EQU	0	; 1 ->ROMbased, 0 ->Download through DebugMon
	.INITCOPY:	EQU	1	; Copy the initialized tabels?
	.ref	.TOSPL
   	IF RZK_ROM = 1
	.ref    .RAM_START_ADDR
	.ref    __low_data, __len_data
	ELSE
	.ref    .BSS_BASE, .BSS_LENGTH
	ENDIF

	.ref    .RAM_END_ADDR

	START_STACK: EQU (.RAM_END_ADDR - 4)
	.extern _RZKHwInitC
	.extern _RZKApplicationEntry
    .extern _InvalidInterruptHandler
	.ref	.TOSPS
	.ref	.TOSPL

	.assume ADL=1

	.def _RZKStartup
_RZKStartup:
; Disable CPU interrupts            
	DI
	RSMIX
 	JP.lil	NEXT			; will set the ADL= 1
NEXT:
	IM 2
	LD bc, _INT_VECTOR_BASE
	LD A, b     		; Setting I for vector table base;	10-05-2002, M.S.Sridhar ZIEL 
	LD I, A				; Initialization of I register done after copying the data section to RAM

;***	Set the Bus Mode for external perihperals
	ld	a,CS_BMC0_DATA
	out0	(CS0_BMC),a
	ld	a,CS_BMC1_DATA
	out0	(CS1_BMC),a
	ld	a,CS_BMC2_DATA
	out0	(CS2_BMC),a
	ld	a,CS_BMC3_DATA
	out0	(CS3_BMC),a
.if 0
;***   Config CS3
	ld	a,CS_LBR3_DATA
	out0	(CS3_LBR),a
	ld	a,CS_UBR3_DATA
	out0	(CS3_UBR),a
	ld	a,CS_CTL3_DATA
	out0	(CS3_CTL),a
;***	Port D GPIO
	ld		a, PD_DR_DATA
	out0	(PD_DR), a
	ld		a, PD_DDR_DATA
	out0	(PD_DDR), a
	ld		a, PD_ALT1_DATA
	out0	(PD_ALT1), a
	ld		a, PD_ALT2_DATA
	out0	(PD_ALT2), a
.endif

; Initialize the stack pointer
	ld.sis	sp,.TOSPS	; Setup SPS
	ld.lil	sp,.TOSPL	; Setup SPL

	IF RZK_ROM = 1
		LD	HL, .RAM_START_ADDR	; It includes BSS, as well DRAM requirements.
		LD	BC, .RAM_END_ADDR	; The End of RAM Area.
		SUB	BC, HL
		LD	DE, .RAM_START_ADDR	; It includes BSS, as well DRAM requirements.
	ELSE            
		LD.lil	HL, .BSS_BASE		; BSS start Address
		LD.lil	BC, .BSS_LENGTH		; Assume BSS Length multiple of 4
		LD.lil	DE, .BSS_BASE		; BSS start Address
	ENDIF

	INC	DE
	DEC	BC    
	
	LD	(HL), 0			; Load 0 in memory

;	PUSH.l	BC
;	LD.lil 		IX,		0
;	ADD.l		IX,		SP
;	LD		A,		(IX+2)
;	POP.l		BC
;	ADD		A,		01

;MemClear:
	LDIR
;	LD		BC, 0FFFFH
;	DEC		A
;	JP		NZ,	MemClear

	.if	.INITCOPY	; Copy Initialized tabels
	.ref    __low_data	; Address of initialized data section
	.ref    __low_romdata	; Address of initialized data section copy
	.ref    __len_data	; Length of initialized data sectrion

;------ Copy the initialized data section
	ld	hl,__len_data	; Check for non-zero length
	ld	bc,0		; *
	or	a,a		; *
	sbc	hl,bc		; *
	jr	z,_c_data_done	; .DATA is zero-length ...
$$:
	ld	hl,__low_romdata	; [hl]=__low_romdata
	ld	de,__low_data	; [de]=.data
	ld	bc,__len_data	; [bc]= data length
loaddata:
	add hl, bc		;
	push hl
	or a, a
	sbc hl,bc		
	ldir			; Copy the data section
	push hl			; load next address to bc
	pop  bc			;
	pop hl			; load end address to hl
	or a,a			; reset cary flag
	sbc hl,bc 		; check if done transfer
	jr z, _c_data_done	;
	push bc
	push hl
	pop bc
	pop hl
	jr loaddata
_c_data_done:
	.endif

; Initialize interrupt vector table & jump table
;	call _z80InitInterruptTable

;	ld.lil	sp,.TOSPL	; Setup SPL
   nop      
;	JP	_RZKApplicationEntry
	JP	_RZKHwInitC		; Transfer to C-function.
	.def _GetIntrTableBase
_GetIntrTableBase:
	PUSH AF		; mahadev added
;	LD	HL,0h
;	LD H,010h
	LD	A, I		; Read MSW of Vector Base
	LD	HL,0h
	LD H,A
	POP AF		; mahadev added
	RET

	.def	__exit
__exit:
	jr	$		; ?


.if 0
	.def _z80InitIntJumpTable
_z80InitIntJumpTable:
;***	Initialize interrupt jump table 
    ld      hl,_IVT_Jump_Table
    ld      b,IVT_SIZE
    ld      iy,__default_hndlr
$1:
    ld      (hl),05bh     ; jp.lil Mmn
    inc     hl
    ld      (hl),0c3h
    inc     hl
    ld      (hl),iy       ; __default_hndlr
    inc     hl
    inc     hl
    inc     hl
    dec     b
    jr      nz,$1
.endif	
