; $Source: source/kernel/threadx/motorola/ppc/ghs/860init.s $
;
; Modification History
;   $Revision: 6.0.2.2 $
;   $Date: 2010/01/18 06:39:05CST $
;   $Author: odile $
;   $ProjectName: /home/mks/proj/tcpip.pj $
;   $ProjectRevision: 6.0.1.23 $
;
; Darwin Init Code
;------------------------------------------------------------------------
; On-Chip Core Register Definitions
;------------------------------------------------------------------------
;
ROMBASE    =    0x02000000
RAMBASE    =    0x00000000

ICR        =    148          # Interrupt Cause Register
ICTRL      =    158          # Instruction Control Register
DER        =    149          # Debug Enable Register
DPDR       =    630          # Development Port Data Register
IMMR       =    638          # Internal I/O base register
IC_CST     =    560          # ICache Control Status Register
IC_ADR     =    561          # ICache Address Register
IC_DAT     =    562          # ICache Data Register
DC_CST     =    568          # DCache Control Status Register
DC_ADR     =    569          # DCache Address Register
DC_DAT     =    570          # DCache Data Register

SIUMCR     =    0x000        # SIU Module configuration
SYPCR      =    0x004        # SIU System Protection Control
SWSR       =    0x00E        # Software service register
SIMASK     =    0x014        # SI Mask Register
MCR        =    0x168        # Memory Command
MAMR       =    0x170        # Machine A Mode Register
MPTPR      =    0x17A        # Memory Periodic Timer Prescaler
MDR        =    0x17C        # Memory Data
BR0        =    0x100        # Base Register 0
OR0        =    0x104        # Option Register 0
BR1        =    0x108        # Base Register 1
OR1        =    0x10C        # Option Register 1
BR2        =    0x110        # Base Register 2
OR2        =    0x114        # Option Register 2
BR3        =    0x118        # Base Register 3
OR3        =    0x11C        # Option Register 3
BR4        =    0x120        # Base Register 4
OR4        =    0x124        # Option Register 4
BR5        =    0x128        # Base Register 5
OR5        =    0x12C        # Option Register 5
PLPRCR     =    0x284        # PLL, Low Power and Reset Control Reg


;------------------------------------------------------------------------
; EQUATE SECTIONS
;------------------------------------------------------------------------

BAUD_CONSOLE_VAL = 0x10102

;
;
;------------------------------------------------------------------------
; Instruction and Data Cache definition
; Note: must use with lis instruction to load into bit 0-15
;------------------------------------------------------------------------
;
CacheUnlockAllCmd  =   0x0A00   # Cache Unlock_All command
CacheDisableCmd    =   0x0400   # Cache Disable command
CacheInvAllCmd     =   0x0C00   # Cache Invalidate_All command
CacheEnableCmd     =   0x0200   # DCache_Enable Command
CacheEnableBit     =   0x8000   # Cache Enable bit in I/DC_CST


;------------------------------------------------------------------------
; MACROS SECTION
;------------------------------------------------------------------------

;------------------------------------------------------------------------
; LA macro - load a register with a word value
;------------------------------------------------------------------------
    .macro    LA reg,symbol
    lis       reg,%hiadj(symbol)
    addi      reg,reg,%lo(symbol)
    .endm

;------------------------------------------------------------------------
; write_half_word macro - write a half word to offset + r4
;------------------------------------------------------------------------
    .macro  write_half_word data,offset
    li  r3,data
    sth r3,offset(r4)
    .endm

;------------------------------------------------------------------------
; SetCSReg macro - write a word to offset + r4
;------------------------------------------------------------------------
    .macro SetCSReg data,offset
    lis r3,%hi(data)
    ori r3,r3,%lo(data)
    stw r3,offset(r4)
    .endm

;
;------------------------------------------------------------------------
; ACE860 Initialization Routine 
;
; Core registers:
;  MSR and SRR1 = 0x00009002
;  ICTRL = 7
;  DER   = 0
;  ICR   = 0
;  IMMR  = 0xFF000000
;  
; SIU registers:
;  SIUMCR |= 0x00600040  
;  SYPCR = 0xffffff08    
;  TBSCR = 0x0001        
;  RTCSC = 0x0102
;  PISCR = 0x0002
;  
;      MPTPR = 0x0200
;      MAMR  = depends on speed
;  
; The initial memory map is as following:
;  Chip Select 0 - Flash Memory - 0x01000000 - 0x0107FFFF
;  Chip Select 1 - DRAM         - 0x00000000 - 0x003FFFFF
;  Internal I/O space (programmed in IMMR register)
;                               - 0xFF000000
;------------------------------------------------------------------------
;
;
;------------------------------------------------------------------------
; Input clock multiplication factor used to program the PLL conrtol
;------------------------------------------------------------------------
;************************************************************************
;      HdwInit: Do startup initialization that can't be done in C.      *
;                                                                       *
;       SYNTAX: void HdwInit(void);                                     *
;         NOTE: Called from "begin.s", not C code.  Doesn't return.     *
;                                                                       *
;************************************************************************
    .text
    .align  4

;************************************************************************
;
;  !!!! NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE NOTE !!!!
; 
;  r4 is loaded with offset to immr , DO NOT USE R4 FOR ANYTHING ESLE!!
;                                                        
;************************************************************************
    .globl    _tf860Init
_tf860Init:

;------------------------------------------------------------------------
; Set up IMMR
;------------------------------------------------------------------------
     li     r0,0x0000       
     lis    r4,0xFF00       #  IMMR = 0xFF000000 
     ori    r4,r4,0x0000
     mtspr  IMMR,r4

;------------------------------------------------------------------------
; Disable Data Cache before accesing any more registers
; Save the status of DC_CST in R8 for the flush routine
;------------------------------------------------------------------------
    mfspr   r3,DC_CST
    ori     r8,r3,0      # save DC_CST in r8
    lis     r3,0x0400    # DCache Disable command
    mtspr   DC_CST,r3
;------------------------------------------------------------------------
; Set the decrementer to a very high value 
;------------------------------------------------------------------------
    LA      r3,0x7FFFFFFF
    bl      .ppcDECwr    # decrementer

;------------------------------------------------------------------------
; Zero out all of DPRAM 
;------------------------------------------------------------------------
;CleardpRam:
;   LA  r3,DUAL_PORT_RAM_BASE
;   li  r10,0           # zero value
;   LA  r12,DUAL_PORT_RAM_BASE+DUAL_PORT_RAM_SIZE
    
;WrtZeroLoopdp:
;   stw     r10,0(r3)       # write to memory
;   addi    r3,r3,4     #point to next word
;   cmpw    r3,r12
;   bne     WrtZeroLoopdp

;------------------------------------------------------------------------
; Initialize Core registers
;------------------------------------------------------------------------

    lis     r3,0x0000    #  MSR = 0x00009002
    ori     r3,r3,0x9002
    mtmsr   r3
    lis     r3,0x0000    #  SRR1 = 0x00009002
    ori     r3,r3,0x9002
    mtspr   SRR1,r3

    lis     r3,0x0       #  DER = 0
    ori     r3,r3,0x0   
    mtspr   DER,r3
 

    lis     r3,0x0000    #  ICTRL = 7
    ori     r3,r3,0x0007
    mtspr   ICTRL,r3

    lis     r3,0x0000    #  ICR = 0
    ori     r3,r3,0x0000
    mtspr   ICR,r3

;   Set CPU Clock to 40MHZ from 5MHZ osc
    lis     r3,0x0070     
    stw     r3,PLPRCR(r4)  
;   Set CPU Clock to 25MHZ from 5MHZ osc
;    lis     r3,0x0040     
;    stw     r3,PLPRCR(r4)  

;------------------------------------------------------------------------
; Initialize SIU registers
;------------------------------------------------------------------------
    lwz     r3,SIUMCR(r4) #  SIUMCR = The old value bitwize
    oris    r3,r3,0x0060  #           ored with 0x00600040  
    ori     r3,r3,0x0040
    stw     r3,SIUMCR(r4)

    lis     r3,0xffff     #  SYPCR = 0xffffff08
    ori     r3,r3,0xff08
    stw     r3,SYPCR(r4)

    write_half_word 0x556C,SWSR  # Prevent SWT timeout
    write_half_word 0xAA93,SWSR

;------------------------------------------------------------------------
; ROM Version start...
;------------------------------------------------------------------------
;   lis     r12,ROMBASE@ha
;   ori     r12,r12,InitCS@l
;   mtlr    r12
;   blr
    
InitCS:
;   CS0 = Boot ROM
    SetCSReg    ROMBASE+0x0401,BR0
    SetCSReg    0xFFF80D30,OR0

UPMInit: 
    LA      r5,UpmTable
    LA      r6,UpmTableEnd
    sub     r7,r6,r5        # UpmTableEnd - UpmTable
    srawi   r7,r7,2         # /4
    
    li      r6,0x0000     # Command: OP=Write, UPMA, MAD=0
UpmWriteLoop:
    lwz     r3,0(r5)      # get data from table
    stw     r3,MDR(r4)    # store the data to MD register
    
    stw     r6,MCR(r4)    # issue command to MCR register
    addi    r5,r5,4       # next entry in the table
    addi    r6,r6,1       # next MAD address
    cmpw    r6,r7         # done yet ?
    blt     UpmWriteLoop

; set up refresh for 15.6 usec at 25 MHz
; uses MPTPR & MSB of MAMR
    
    li      r3,0x0200    # Memory Periodic Timer Clk = CPU CLK/32
    sth     r3,MPTPR(r4)

; PTA = 0x0C (for 25 MHz)
; PTAE = 1 (enabled)
; AMA[0:2] = 010 4 MB DRAM
; DSA[0:1] = 01 (RAS PRECHARGE = 2 CLKS)
; G0CLA = 000 (A12)
; GPLA4DIS = 1
; RLFA[0:3] = 1 (loop is executed 1 time
; WLFA[0:3] = 1
; TLFA[0:3] = 1
            
; 40Mhz Part
    lis     r3,0x14A0     # MAMR = 0x14A01334
    ori     r3,r3,0x1334
    stw     r3,MAMR(r4)
; 25Mhz part
;    lis     r3,0x0CA0     # MAMR = 0x0CA01334
;    ori     r3,r3,0x1334
;    stw     r3,MAMR(r4)

;
;                      MEMORY CONTROLLER INITIALIZATION
;
;       Setup Chip Selects
;

;   CS1 = DRAM Bank 1

    SetCSReg    0xFFC00800,OR1
    SetCSReg    RAMBASE+0x0081,BR1

;------------------------------------------------------------------------
; Wait at least 200 usec for DRAM to stabilize
;------------------------------------------------------------------------
    lis    r5,0x0000     # count 0x0379 times ~225nsec
    ori    r5,r5,0x379
    lis    r6,0x0000     # clear r6
delayloop4:
    addi   r6,r6,0x01    # decrement the counter
    cmpw   cr0,r6,r5     # done yet ?       
    bne    delayloop4
;------------------------------------------------------------------------
; Need at least 10 DRAM accesses to stabilize
;------------------------------------------------------------------------
    lis    r5,0x0000     # count 10 times
    ori    r5,r5,0x0a
    lis    r6,0x0000     # clear r6

    lis    r15,RAMBASE     # load address
    ori    r15,r15,0x1000
    lis    r16,0x1234     # load data
    ori    r16,r16,0x5678


;   CS2 = DRAM Bank 2

;   if it is enabled, the refresh frequency must be doubled

;   SetCSReg    0xFFE00D32,OR2
;   SetCSReg    ROM_BASE+0x0400000+0x0401,BR2

;   CS3 = 1M Flash ROM

    SetCSReg    0xFFE00D32,OR3
    SetCSReg    0x02080801,BR3

;   CS4 = Network Interface Modules

    SetCSReg    0xFFFF0182,OR4
    SetCSReg    0x40000401,BR4

;   CS5 = DRAM ID Register

    SetCSReg    0xFFFF0122,OR5
    SetCSReg    0x50000401,BR5

;------------------------------------------------------------------------
; Enable Decrementer and RTC
;------------------------------------------------------------------------
;    li      r3,0x0001     #  TBSCR = 0x00c2 (16bit register)
;    sth     r3,TBSCR(r4)

;   lis     r3,0x55cc     #  Unlock the RTC registers
;   ori     r3,r3,0xaa33
;   stw     r3,RTCSCK(r4)
;   stw     r3,RTCK(r4)

;    li      r3,0x0102     #  RTCSC = 0x01c2 (16bit register)
;    sth     r3,RTCSC(r4)
    
;    li      r3,0x0002     #  PISCR = 0x0082 (16bit register)
;    sth     r3,PISCR(r4)
    
;   li      r3,0x0000     # Clear SIMASK
;   stw     r3,SIMASK(r4)

;------------------------------------------------------------------------
; Quick enable of SMC to output banner and DRAM test message
;
;   !! assumes r4 is offset to IMMR !!
;------------------------------------------------------------------------
;       write_half_word 0x8001,CPCR     # RESET THE CP

; wait for reset complete

GetCPResetComplete:
;   lhz             r6,CPCR(r4)
;   andi.           r6,r6,1
;   bne             GetCPResetComplete

;   SetCSReg        0,SIMODE
;   write_half_word 0x2800,SMC1RBAS
;   write_half_word 0x2808,SMC1TBAS
;   SetCSReg        0xC0,PBPAR

;   write_half_word 0x91,CPCR

; wait for reset complete
GetSMCResetComplete:
;   lhz             r6,CPCR(r4)
;   andi.           r6,r6,1
;   bne             GetSMCResetComplete

;   write_half_word 0x1818,SMC1TFCR
;   write_half_word 1,SMC1MRBL
;   write_half_word 0,SMC1MAXI
;   write_half_word 0,SMC1BRKL
;   write_half_word 0,SMC1BRKE
;   write_half_word 1,SMC1BRKC
;   SetCSReg        BAUD_CONSOLE_VAL,BRGC1      # For 19200bps
;   write_half_word 0x4822,SMCMR1

;------------------------------------------------------------------------
; Print Hello Banner
;------------------------------------------------------------------------
;   SetCSReg        PwrUpMsg,0x280C # pointer to buffer
;   write_half_word EndPwrUp-PwrUpMsg,0x280A        # length
;   write_half_word 0xA000,0x2808

; wait for message to be sent
GetHelloComplete:
;   lhz     r6,0x2808(r4)
;   andi.   r6,r6,0x8000
;   bne     GetHelloComplete

; Test Code Start

; Zero 0x400 BYTES starting at RAM Location 0x80000 
;    li         r7,0x0400
;    srawi      r7,r7,2      
;    lis        r5,0x0008    
;    li         r6,0x0000     
;    li         r3,0     
;ZeroLoop:
;    stw        r3,0(r5)
;    addi       r5,r5,4
;    addi       r6,r6,1
;    cmpw       r6,r7        
;    blt        ZeroLoop

;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop
;    nop

CacheInit:
#------------------------------------------------------------------------
# If Instruction cache was enabled, disable and invalidate all
#------------------------------------------------------------------------
ICacheInit:
ICacheUnlock:
        lis      r3,CacheUnlockAllCmd
        mtspr    IC_CST,r3       # Unlock Icache
        isync
   
ICacheInv:
        lis      r3,CacheInvAllCmd
        mtspr    IC_CST,r3       # Invalidate Icache
        isync
ICacheEnable:
         lis     r3,CacheEnableCmd
         mtspr   IC_CST,r3       # Enable Icache
         sync


    b       _start

    .align  2
    .globl  .ppcDECwr
;
.ppcDECwr:
;        mtdec   r3
        mtspr   22,r3
        isync
        blr



;/* UPMA contents: */
UpmTable:
;/* single read. (offset 0 in upm RAM) */
     .long    0x0fffcc24, 0x0fffcc04, 0x00ffcc00, 0x71ffcc07
     .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
;/* burst read. (offset 8 in upm RAM) */
     .long    0x0fffcc24, 0x0fffcc04, 0x00ffcc80, 0x03ffcccc
     .long    0x00ffcc00, 0x73ffcc07, 0xffffcc04, 0xfffffc04
     .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
     .long    0xfffffe04, 0xfffffc04, 0xfffffc04, 0xfffffc04
;/* single write. (offset 18 in upm RAM) */
     .long    0x0fafec24, 0x0fafec04, 0x00afec00, 0x73bfec07
     .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
; /* burst write. (offset 20 in upm RAM) */
     .long    0x0fafec24, 0x0fafec04, 0x00afec80, 0x0fafeccc
     .long    0x00afec00, 0x7fbfec07, 0xfffffc04, 0xfffffc04
     .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
     .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
; /* refresh. (offset 30 in upm RAM) */
     .long    0xf0ffec04, 0x00ffec04, 0x01ffec04, 0x3fffec04
     .long    0xffffec04, 0xffffec05, 0xfffffc04, 0xfffffc04
     .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
; /* exception. (offset 3c in upm RAM) */
     .long    0xffffec07, 0xfffffc04, 0xfffffc04, 0xfffffc04
UpmTableEnd:

PwrUpMsg:
    .byte "Treck Real-Time TCP/IP (tm) "
    .byte "Initialization Complete"
EndPwrUp:
