/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/threadx/motorola/ppc/ghs/darwin.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:06CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
#include <trmpc860.h>
#define     TX_DISABLE_ERROR_CHECKING  1
#include   "tx_api.h"


void darwinInstallSuiIsr(void *funcPtr, unsigned long offSet);
void darwinCpicInterrupt(void);
void darwinNoInterrupt(void);
void darwinSIUInterrupt(void);
#define TM_MAX_VECTORS       16
#define TM_MAX_CPIC_VECTORS  30 

/*
 * Typedef for the SIU
 */
typedef struct {
    void (*vectFuncPtr)(void);
} ttVector;

typedef void (*ttVectPtr)();

/*
 * The SIU Vector Table
 */
ttVector tvVectorTable[TM_MAX_VECTORS];

/*
 * The CPIC Vector Table
 */
ttVector tvCpicVectorTable[TM_MAX_CPIC_VECTORS];

volatile static ttMpc860 *tl860Ptr;
unsigned long tlIsrCount;

/*
 * Initialize the SUI (System Integration Unit)
 */
void tfSetUpSUI()
{
	int vectorIndex;
    unsigned long  mpc860Registers;
    unsigned char  timerVector;
    unsigned long  timerLocation;

    
    mpc860Registers=TM_MPC860_IMMR_LOC;
    tl860Ptr = (ttMpc860 *)(mpc860Registers&0xfffffffc);
/* Reset the CP on the 860 */
    tl860Ptr->cp_cr = 0x8001;
    while (tl860Ptr->cp_cr != 0)
    {
        ;
    }
/* Clear the SUI interrupt table */
	for ( vectorIndex = 0; vectorIndex < TM_MAX_VECTORS; vectorIndex++ )
		tvVectorTable[vectorIndex].vectFuncPtr = darwinNoInterrupt;
/* Clear the CPIC interrupt table */
	for ( vectorIndex = 0; vectorIndex < TM_MAX_CPIC_VECTORS; vectorIndex++ )
		tvCpicVectorTable[vectorIndex].vectFuncPtr = darwinNoInterrupt;
    
/* Install the master cpic vector */
    tfInstallSuiIsr((void *)darwinCpicInterrupt,TM_MPC860_CPIC_SIVEC);
/* Unmask the SUI Level */
    tl860Ptr->siu_simask |= TM_MPC860_CPIC_MASK; 
    
}




/*
 * MPC 860 Main Interrupt Handler from the SIU
 * All MPC860 Interrupts are called through the SIU
 */
void darwinSIUInterrupt()
{
    (void)tx_interrupt_control(TX_INT_DISABLE);
	tlIsrCount++; 
/* Call the specific Interrupt Handler	*/
	(tvVectorTable[tl860Ptr->siu_sivec>>26].vectFuncPtr)();		
    (void)tx_interrupt_control(TX_INT_ENABLE);
}



/*
 * NULL Interrupt for ISR that are not processed
 */
void darwinNoInterrupt(void)
{
}

/*
 * Install a SUI ISR
 */
void darwinInstallSuiIsr(void *funcPtr, unsigned long offSet)
{
    tlIsrCount=0;
    tvVectorTable[offSet>>26].vectFuncPtr=(ttVectPtr)funcPtr;
}

/*
 * Install a CPIC ISR
 */
void tfInstallISR(void *funcPtr, unsigned long offSet)
{
    tvCpicVectorTable[offSet].vectFuncPtr=(ttVectPtr)funcPtr;
}
/*
 * Master CPIC Interrupt Handler
 */
void darwinCpicInterrupt(void)
{
    tl860Ptr->intr_civr |= TM_MPC860_CIVR_IACK; 
	(tvCpicVectorTable[tl860Ptr->intr_civr>>11].vectFuncPtr)();		
}



/*
 * MPC860 ISR
 */
void tfMpc860Scc1Isr(void)
{
    tfMpc860Scc1HandlerIsr();
}

void tfMpc860Scc4Isr(void)
{
    tfMpc860Scc4HandlerIsr();
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:43      For: Xilinx-Samsung
 * File Name: darwin.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 43144    Output checksum: 40032
 */
