/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/threadx/motorola/ppc/ghs/test.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:12CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/* This is a small demo of the high-performance ThreadX kernel.  It includes examples of six
   threads of different priorities, message queues, semaphores, and event flags.  */

#include <trsocket.h>
#include <trmpc860.h>

#define     TX_DISABLE_ERROR_CHECKING  1
#include   "tx_api.h"


#define     DEMO_STACK_SIZE 2048

#define     TEST_BUF_SIZE   20000

/* Define the ThreadX object control blocks...  */

TX_THREAD               treckTaskTcb;
TX_THREAD               timerTaskTcb;
TX_THREAD               recvTaskTcb;
TX_THREAD               sendTaskTcb;
TX_THREAD               serverTaskTcb;



/* Define thread prototypes.  */

void    treckTask(ULONG thread_input);
void    timerTask(ULONG thread_input);
void    recvTask(ULONG thread_input);
void    sendTask(ULONG thread_input);
void    serverTask(ULONG thread_input);

/* Support for multiple interfaces */
ttUserInterface tlInterfaceHandle[5];
extern           unsigned char tvUdpChecksumOn;
typedef int (*ttFreeRecvBufferFuncPtr)(ttUserInterface interfaceHandle,
             char TM_FAR    *dataPtr);


/*
 * Test Driver
 */

int sendSmallCopy(int sd, struct sockaddr_in *toAddress, char *testPtr);
int sendLargeCopy(int sd, struct sockaddr_in *toAddress, char *testPtr);
int sendSmallNoCopy(int sd, struct sockaddr_in *toAddress);
int sendLargeNoCopy(int sd, struct sockaddr_in *toAddress);
int sendTcpSmallCopy(int sd, char *testPtr);
int sendTcpLargeCopy(int sd, char *testPtr);

volatile int treckError;

#define TM_SCC1 0

/* Define main entry point.  */

int main()
{

    /* Enter the ThreadX kernel.  */
    tx_kernel_enter();
}


/* Define what the initial system looks like.  */

void    tx_application_define(void *first_unused_memory)
{

CHAR    *pointer;

    /* Put first available memory address into a character pointer.  */
    pointer =  (CHAR *) first_unused_memory;

    /* Create the Treck thread.  */
    tx_thread_create(&treckTaskTcb, "Treck Task", treckTask, 0,  
			pointer, DEMO_STACK_SIZE, 
			9, 9, TX_NO_TIME_SLICE, TX_AUTO_START);
    pointer = pointer + DEMO_STACK_SIZE;

    tx_thread_create(&timerTaskTcb, "Timer Task", timerTask, 0,  
			pointer, DEMO_STACK_SIZE, 
			1, 1, 1, TX_DONT_START);
    pointer = pointer + DEMO_STACK_SIZE;

    tx_thread_create(&recvTaskTcb, "Receive Task", recvTask, TM_SCC1,  
			pointer, DEMO_STACK_SIZE, 
			4, 4, TX_NO_TIME_SLICE, TX_DONT_START);
}

volatile int masterError;
volatile int testType;
volatile unsigned long dataCount;
void   treckTask(ULONG thread_input)
{
    int                 sd;
    int                 tsd;
    int                 errorCode;
    ttUserIpAddress     ipAddress;
    ttUserIpAddress     netMask;
    ttUserLinkLayer     linkLayerHandle;
    ttUserInterface     interfaceHandle;
    struct sockaddr_in  toAddress;
    char TM_FAR         *bufPtr;
    int                 portNumber;
    int                 toPortNumber;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext           contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    
#ifndef TM_MULTIPLE_CONTEXT
    tfInitTreck();
#else /* TM_MULTIPLE_CONTEXT */
    tfInitTreckMultipleContext();
    contextHandle = tfCreateTreckContext();
    if (contextHandle == (void TM_FAR *)0)
    {
        tfKernelError("main","Failed to allocate Treck context\n");
    }
    else
    {
        tfSetCurrentContext(contextHandle);
    }
#endif /* TM_MULTIPLE_CONTEXT */
/* Start Treck */
    tfStartTreck();

/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();
/* Add the Interface */
    interfaceHandle=tfAddInterface(
            "MPC860.SCC1",
            linkLayerHandle,
            tfMpc860EtherOpen,
            tfMpc860EtherClose,
            tfMpc860EtherSend,
            tfMpc860EtherReceive,
            (ttFreeRecvBufferFuncPtr)0,
/*            tfMpc860EtherFreeRecvBuffer, */
            tfMpc860Ioctl,
            tfMpc860GetPhyAddr,
            &errorCode);
	tlInterfaceHandle[TM_SCC1]=interfaceHandle;
    portNumber=5050;    
/*    toPortNumber=2081;  */
    toPortNumber=7; 

/* Start the tasks */
	tx_thread_resume(&timerTaskTcb);
	tx_thread_resume(&recvTaskTcb);

    ipAddress=inet_addr("208.229.201.110");
/* 255.255.255.192 */    
    netMask=htonl(0xffffffc0);
/* Config the Interface */    
    errorCode=tfConfigInterface(
            interfaceHandle,
            ipAddress,
            netMask,
            TM_DEV_SCATTER_SEND_ENB,  
            10,   
            0);
/* Open a socket */
    toAddress.sin_family=PF_INET;
    toAddress.sin_addr.s_addr=inet_addr("208.229.201.80");
    toAddress.sin_port=htons(toPortNumber);
/*    tvUdpChecksumOn=0; */

/* Malloc test buffer */
    bufPtr=(char TM_FAR *)tm_kernel_malloc(TEST_BUF_SIZE);
/* Test Type */
    testType=5; 
    dataCount=4;
    sd=socket(PF_INET,SOCK_DGRAM,0);
    tsd=socket(PF_INET,SOCK_STREAM,0);
    switch (testType)
    {
        case 0:
            while (1)
            {
                errorCode=sendSmallCopy(sd, &toAddress,bufPtr);
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(sd);
                }
            }
            break;
        case 1:
            while (1)
            {
                errorCode=sendLargeCopy(sd, &toAddress,bufPtr);
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(sd);
                }
            }
            break;
        case 2:
            while (1)
            {
                errorCode=sendSmallNoCopy(sd, &toAddress);
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(sd);
                }
            }
            break;
        case 3:
            while (1)
            {
                errorCode=sendLargeNoCopy(sd, &toAddress);
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(sd);
                }
            }
            break;
        case 4:
            while (1)
            {
                errorCode=connect(tsd,
                         (const struct sockaddr TM_FAR *)&toAddress,
                         sizeof(toAddress));
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(tsd);
                }
                errorCode=sendTcpSmallCopy(tsd,bufPtr);
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(tsd);
                }
            }
            break;
        case 5:
            while (1)
            {
                errorCode=connect(tsd,
                         (const struct sockaddr TM_FAR *)&toAddress,
                         sizeof(toAddress));
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(tsd);
                }
                errorCode=sendTcpLargeCopy(tsd,bufPtr);
                if (errorCode!=TM_ENOERROR)
                {
                    treckError=tfGetSocketError(tsd);
                }
            }
            break;
        case 9:
            while (1)
            {
                ;
            }
        default:
            break;
    }
/* free test buffer */
    tm_kernel_free(bufPtr);

    masterError=errorCode;
}

extern unsigned long     tvMpc860SendPacketCount;
unsigned long tvSocketSendCount;
int sendSmallCopy(int sd, struct sockaddr_in *toAddress, char *testPtr)
{
   int errorCode;
   
   tvSocketSendCount=0UL;
   
   while (1) {
        errorCode = sendto(sd,testPtr,18,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
        tvSocketSendCount++;
        if (errorCode == -1)
        {
            treckError=tfGetSocketError(sd);
            break;
        }
    }
    return errorCode;
}

int sendTcpSmallCopy(int sd, char *testPtr)
{
   int errorCode;
   
   tvSocketSendCount=0UL;
   while (1) {
        *((unsigned long *)testPtr)=htonl(dataCount++);
        errorCode = send(sd,testPtr,18,0);
        tvSocketSendCount++;
        if (errorCode == -1)
        {
            treckError=tfGetSocketError(sd);
            break;
        }
    }
    return errorCode;
}

int sendTcpLargeCopy(int sd, char *testPtr)
{
   int errorCode;
   int bytes;
   
   tvSocketSendCount=0UL;
   while (1) {
        *((unsigned long *)testPtr)=htonl(dataCount++);
        errorCode = send(sd,testPtr,2048,0);
        tvSocketSendCount++;
        if (errorCode == -1)
        {
            treckError=tfGetSocketError(sd);
            break;
        }
        bytes=recv(sd,testPtr,10000,0);
        if (bytes < 0)
        {
            treckError=tfGetSocketError(sd);
        }
    }
    return errorCode;
}

static unsigned long tlSendCount;
static unsigned long tlRecvCount;

int sendLargeCopy(int sd, struct sockaddr_in *toAddress, char *testPtr)
{
   int                  errorCode;
   struct sockaddr_in   fromAddress;
   int                  recvFromLen;
   int                  bytes;
   int                  count;
   char TM_FAR          *inBufPtr;
   
   tlSendCount=0UL;
   tlRecvCount=0UL;
/* Malloc test in buffer */
   inBufPtr = tm_kernel_malloc(TEST_BUF_SIZE);
   while (1) 
   {
       errorCode = sendto(sd,testPtr,1472,0,
           (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
       if (errorCode == -1)
       {
           treckError=tfGetSocketError(sd);
           break;
       }
       else
       {
           tlSendCount++;
           for (count=0; count<10000;count++)
           {
               recvFromLen=sizeof(fromAddress);
               bytes=recvfrom(sd,inBufPtr,1472,MSG_DONTWAIT,
                    (struct sockaddr TM_FAR *)&fromAddress,
                    (int TM_FAR *)&recvFromLen);
               if (bytes < 0)
               {
                   treckError=tfGetSocketError(sd);
                   if (treckError!=235)
                   {
                       while(1);
                   }
               }
               else
               {
                   tlRecvCount++;
                   break;
               }
           }
       }
   }

/* free test in buffer */
   tm_kernel_free(inBufPtr);

   return errorCode;
}

int sendSmallNoCopy(int sd, struct sockaddr_in *toAddress)
{
   char         *testPtr;
   int           errorCode;
   ttUserMessage bufferHandle;

   while (1) 
   {
        bufferHandle=tfGetZeroCopyBuffer(18,&testPtr);
        errorCode = tfZeroCopySendTo(sd,bufferHandle,18,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 

        if (errorCode == -1)
        {
            break;
        }
    }
    return errorCode;
}

int sendLargeNoCopy(int sd, struct sockaddr_in *toAddress)
{
   char         *testPtr;
   int           errorCode;
   ttUserMessage bufferHandle;

   while (1) {
        bufferHandle=tfGetZeroCopyBuffer(1472,&testPtr);
        errorCode = tfZeroCopySendTo(sd,bufferHandle,1472,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
        if (errorCode == -1)
        {
            break;
        }
    }
    return errorCode;
}

void timerTask(ULONG interfaceIndex)
{
    ttUserInterface interfaceHandle;

	interfaceHandle=tlInterfaceHandle[interfaceIndex];
    while (1) 
    {
        tfTimerUpdate();
        tfTimerExecute(); 
        (void)tx_thread_sleep(1);
        (void)tfIoctlInterface(interfaceHandle,
                               TM_MPC860_REFILL_SCC1|TM_MPC860_SEND_COMPLETE,
                               (void TM_FAR *)0, 0);

    }
}

void recvTask(ULONG interfaceIndex)
{
    ttUserInterface interfaceHandle;

	interfaceHandle=tlInterfaceHandle[interfaceIndex];
    while (1) 
    {
        (void)tfWaitReceiveInterface(interfaceHandle);
        (void)tfRecvInterface(interfaceHandle);
    }
}


void serverTask(ULONG dataPort)
{
    int errorCode;
    int bytes;
    int sd;
    struct sockaddr_in fromAddress;
    struct sockaddr_in ourAddress;
    int    recvFromLen;
    char TM_FAR *inBufPtr;

    sd=socket(PF_INET,SOCK_DGRAM,0);
    if (sd<0)
    {
        treckError=tfGetSocketError(sd);
    }
/* Open a socket */
    ourAddress.sin_family=PF_INET;
    ourAddress.sin_addr.s_addr=inet_addr("127.0.0.1");
    ourAddress.sin_port=htons((int)dataPort);
    errorCode=bind(sd,(const struct sockaddr TM_FAR *)&ourAddress,sizeof(ourAddress));
    if (errorCode!= TM_ENOERROR)
    {
        treckError=tfGetSocketError(sd);
    }
/* Malloc test in buffer */
    inBufPtr = tm_kernel_malloc(TEST_BUF_SIZE);
    while (1) 
    {
        recvFromLen=sizeof(fromAddress);
        bytes=recvfrom(sd,inBufPtr,1472,0,
             (struct sockaddr TM_FAR *)&fromAddress,
             (int TM_FAR *)&recvFromLen);
        if (bytes > 0)
        {
            errorCode = sendto(sd,inBufPtr,bytes,0,
                (const struct sockaddr TM_FAR *)&fromAddress,(int)sizeof(struct sockaddr)); 
            if (errorCode <= 0)
            {
                treckError=tfGetSocketError(sd);
            }
        }
        else
        {
            treckError=tfGetSocketError(sd);
        }
             
    }
/* Free test in buffer */
    tm_kernel_free(inBufPtr);
}
#if 0
void sendTask(ULONG interfaceIndex)
{
    int errorCode;
    while (1) 
    {
        errorCode=tfWaitSentInterface(*(ttUserInterface *)
        		tlInterfaceHandle[interfaceIndex]);
        if (errorCode==TM_ENOERROR)
        {
            tfSendCompleteInterface(*(ttUserInterface *)
            tlInterfaceHandle[interfaceIndex],
                TM_DEV_SEND_COMPLETE_APP);
        }
    }
}
#endif


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:43      For: Xilinx-Samsung
 * File Name: test.c        File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  2508    Output checksum: 19164
 */
