/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/threadx/motorola/ppc/ghs/trmpcint.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:13CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
#include <trmpc860.h>
#define     TX_DISABLE_ERROR_CHECKING  1
#include   "tx_api.h"


void tfMpcInstallSuiIsr(void *funcPtr, unsigned long offSet);
void tfTxInstallISR(void *funcPtr, unsigned long offSet);
void tfMpcCpicInterrupt(void);
void tfMpcNoInterrupt(void);
void tfMpcSIUInterrupt(void);
void tfTxIntrInit(void);
void tfMpcSetUpSUI(void);
void tfMpcTickISR(void);

#define TM_MAX_VECTORS       16
#define TM_MAX_CPIC_VECTORS  30 

/*
 * Typedef for the SIU
 */
typedef struct {
    void (*vectFuncPtr)(void);
} ttVector;

typedef void (*ttVectPtr)();

/*
 * The SIU Vector Table
 */
ttVector tvVectorTable[TM_MAX_VECTORS];

/*
 * The CPIC Vector Table
 */
ttVector tvCpicVectorTable[TM_MAX_CPIC_VECTORS];

volatile static ttMpc860 *tl860Ptr;
unsigned long tlIsrCount;


void tfTxIntrInit(void)
{
    tfMpcSetUpSUI();
}

/*
 * Initialize the SUI (System Integration Unit)
 */
void tfMpcSetUpSUI()
{
	int vectorIndex;
    unsigned long  mpc860Registers;
    unsigned char  timerVector;
    unsigned long  timerLocation;

    
    mpc860Registers=TM_MPC860_IMMR_LOC;
    tl860Ptr = (ttMpc860 *)(mpc860Registers&0xfffffffc);
/* Reset the CP on the 860 */
    tl860Ptr->cp_cr = 0x8001;
    while (tl860Ptr->cp_cr != 0)
    {
        ;
    }
/* Clear the SUI interrupt table */
	for ( vectorIndex = 0; vectorIndex < TM_MAX_VECTORS; vectorIndex++ )
		tvVectorTable[vectorIndex].vectFuncPtr = tfMpcNoInterrupt;
/* Clear the CPIC interrupt table */
	for ( vectorIndex = 0; vectorIndex < TM_MAX_CPIC_VECTORS; vectorIndex++ )
		tvCpicVectorTable[vectorIndex].vectFuncPtr = tfMpcNoInterrupt;
    
/* Install the master cpic vector */
    tfMpcInstallSuiIsr((void *)tfMpcCpicInterrupt,TM_MPC860_CPIC_SIVEC);
/* Unmask the SUI Level */
    tl860Ptr->siu_simask |= TM_MPC860_CPIC_MASK; 
    
    tl860Ptr->timer_tgcr=0x0000;
/* Install the Timer Cpic Vector */
    tfTxInstallISR((void *)tfMpcTickISR,TM_MPC860_TMR1_INT);

    tl860Ptr->intr_cicr=TM_MPC860_CICR_DEFAULT;
    tl860Ptr->intr_cipr = 0xffffffffUL;
    tl860Ptr->intr_cisr |= TM_MPC860_TMR1_INT_MASK;
    tl860Ptr->intr_cimr = TM_MPC860_TMR1_INT_MASK;
/* 
 * The MPC860 Timer Clock is either 25mhz or 40mhz
 * for 25mhz:
 * We use a divide by value of 4 so that the clock appears to be
 * running at 6.25mhz.  We then wait for the clock to count down
 * from 62500 to 0 to get a 10ms resolution
 *
 * for 40Mhz:
 * We use a divide by value of 8 so that the clock appears to be
 * running at 5.00mhz.  We then wait for the clock to count down
 * from 50000 to 0 to get a 10ms resolution
 */

/* 
 * Set the Mode register to divide by x, Interrupt, Auto Restart (FRR)
 * and use the internal system clock
 * Note: The prescale value ranges 0 - 255 and represents the divide by
 * number of 1 - 256.  That is why there is a x-1 for the prescale 
 */
/*    tl860Ptr->timer_tmr1=0x031a; */
    tl860Ptr->timer_tmr1=0x071a; 
/* Clear the count */
    tl860Ptr->timer_tcn1=0x0000;
/* Set the reference value to 62500 */
/*    tl860Ptr->timer_trr1=0xF424; */
/* Set the reference value to 50000 */
    tl860Ptr->timer_trr1=0xC350; 
/* Clear the events */
    tl860Ptr->timer_ter1=0xFFFF;
/* Start the timer */
    tl860Ptr->timer_tgcr=0x0005;
}


/*
 * MPC 860 Main Interrupt Handler from the SIU
 * All MPC860 Interrupts are called through the SIU
 */
void tfMpcSIUInterrupt()
{
/*    (void)tx_interrupt_control(TX_INT_DISABLE); */
	tlIsrCount++; 
/* Call the specific Interrupt Handler	*/
	(tvVectorTable[tl860Ptr->siu_sivec>>26].vectFuncPtr)();		
/*    (void)tx_interrupt_control(TX_INT_ENABLE); */
}



/*
 * NULL Interrupt for ISR that are not processed
 */
void tfMpcNoInterrupt(void)
{
}

/*
 * Install a SUI ISR
 */
void tfMpcInstallSuiIsr(void *funcPtr, unsigned long offSet)
{
    tlIsrCount=0;
    tvVectorTable[offSet>>26].vectFuncPtr=(ttVectPtr)funcPtr;
}

/*
 * Install a CPIC ISR
 */
void tfTxInstallISR(void *funcPtr, unsigned long offSet)
{
    tvCpicVectorTable[offSet].vectFuncPtr=(ttVectPtr)funcPtr;
}
/*
 * Master CPIC Interrupt Handler
 */
void tfMpcCpicInterrupt(void)
{
    tl860Ptr->intr_civr |= TM_MPC860_CIVR_IACK; 
	(tvCpicVectorTable[tl860Ptr->intr_civr>>11].vectFuncPtr)();		
}



/*
 * MPC860 SCC1 ISR
 */
void tfMpc860Scc1Isr(void)
{
    tfMpc860Scc1HandlerIsr();
}

/*
 * Timer ISR
 */
void tfMpcTickISR(void)
{
    tl860Ptr->timer_ter1=0xFFFF;
    tl860Ptr->intr_cisr |= TM_MPC860_TMR1_INT_MASK;
}

void tfMpc860Scc4Isr(void)
{
/*    tfMpc860Scc4HandlerIsr();  */
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:43      For: Xilinx-Samsung
 * File Name: trmpcint.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 48468    Output checksum: 22489
 */
