/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/intelx86/real/borland/ix86l_c.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:21CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
*********************************************************************************************************
*                                                uC/OS
*                                         The Real-Time Kernel
*
*                      (c) Copyright 1992, 1993, Jean J. Labrosse, Plantation, FL
*                                          All Rights Reserved
*
*
*                                       80x86/80x88 Specific code
*                                           LARGE MEMORY MODEL
*
* File : Ix86L_C.C
* By   : Jean J. Labrosse
*********************************************************************************************************
*/

#include "INCLUDES.H"
/*
*********************************************************************************************************
*                                           CREATE A TASK
*********************************************************************************************************
*/
UBYTE OSTaskCreate(void (far *task)(void *pd), void *pdata, void *pstk, UBYTE p)
{
    UWORD *stk;
    UBYTE  err;


    OS_ENTER_CRITICAL();
    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {        /* Make sure task doesn't already exist at this prio.  */
        OS_EXIT_CRITICAL();
        stk    = (UWORD *)pstk;                  /* Load stack pointer                                  */
        *--stk = (UWORD)FP_SEG(pdata);           /* Simulate call to function with argument             */
        *--stk = (UWORD)FP_OFF(pdata);
        *--stk = (UWORD)FP_SEG(task);
        *--stk = (UWORD)FP_OFF(task);
        *--stk = (UWORD)0x0202;                  /* PSW = Interrupts enabled                            */
        *--stk = (UWORD)FP_SEG(task);            /* Put pointer to task   on top of stack               */
        *--stk = (UWORD)FP_OFF(task);
        *--stk = (UWORD)0x0000;                  /* AX = 0                                              */
        *--stk = (UWORD)0x0000;                  /* CX = 0                                              */
        *--stk = (UWORD)0x0000;                  /* DX = 0                                              */
        *--stk = (UWORD)0x0000;                  /* BX = 0                                              */
        *--stk = (UWORD)0x0000;                  /* SP = 0                                              */
        *--stk = (UWORD)0x0000;                  /* BP = 0                                              */
        *--stk = (UWORD)0x0000;                  /* SI = 0                                              */
        *--stk = (UWORD)0x0000;                  /* DI = 0                                              */
        *--stk = (UWORD)0x0000;                  /* ES = 0                                              */
        *--stk = _DS;                            /* Save current value of DS                            */
        err = OSTCBInit(p, (void far *)stk);     /* Get and initialize a TCB                            */
        if (err == OS_NO_ERR) {
            if (OSRunning) {                     /* Find highest prio. task if multitasking has started */
                OSSched();
            }
        }
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

/* 
 * Install an ISR handler
 */
void OSInstallISR(void far *funcPtr, unsigned long offSet)
{
    setvect((int) offSet,(void interrupt (*)(void))funcPtr);

    return;
}

/* 
 * Install the uC/OS handler (context switches)
 */
void OSSetupUcosHandler()
{
    OSInstallISR((void far *)OSCtxSw, uCOS);

	return;
}

/*
 * Install the Timer Handler
 */
void OSSetUpTimer()
{
    void interrupt (*OldTickISR)(void);

/*
 * Save the old timer tick vector and store it at vector 0x81 (which is 
 * unused).  This is only necessary under DOS, and in fact should *not* be
 * used if not running uC/OS 'on-top' of another OS (eg, DOS).	 [JNS]
 */
    OldTickISR = getvect(0x08);
    setvect(0x81, OldTickISR);  

/* Hooks our ISR into the timer tick interrupt */
	OSInstallISR(OSTickISR,0x08);

	return;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:43      For: Xilinx-Samsung
 * File Name: ix86l_c.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 31486    Output checksum:  3424
 */
