/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/intelx86/real/borland/test1.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:30CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
************************************************************
*                         TEST1.C
*                      uCOS EXAMPLE
************************************************************
*/
#include "INCLUDES.H"
#include <trsocket.h>
#include <trpacket.h>

#define          STK_SIZE       2048
OS_TCB           OSTCBTbl[OS_MAX_TASKS];
UWORD            OSIdleTaskStk[STK_SIZE];
UWORD            Stk1[STK_SIZE];
UWORD            Stk2[STK_SIZE];
UWORD            Stk3[STK_SIZE];
char             Data1;
char             Data2;
char             Data3;

typedef void (OS_FAR *ttTaskPtr)(void *pd);

void treckTask(void *data);
void timerTask(void *data);
void recvTask(void *interfaceHandle);

extern           unsigned char tvUdpChecksumOn;


typedef int (*ttFreeRecvBufferFuncPtr)(ttUserInterface interfaceHandle,
             char TM_FAR    *dataPtr);



int sendSmallCopy(int sd, struct sockaddr_in *toAddress, char *testPtr);
int sendLargeCopy(int sd, struct sockaddr_in *toAddress, char *testPtr);
int sendSmallNoCopy(int sd, struct sockaddr_in *toAddress);
int sendLargeNoCopy(int sd, struct sockaddr_in *toAddress);

static unsigned long testBuf[380];
volatile int treckError;
void main(void)
{
    Data1 = '1';
    Data2 = '2';
    Data3 = '3';

/* Setup Tick ISR */
    OSSetupUcosHandler();
    OSInit();
    OSTaskCreate((ttTaskPtr)treckTask, (void *)&Data3, (void *)&Stk3[STK_SIZE], 4);
    OSStart();
}
volatile int masterError;
volatile int testType;
void treckTask(void *data)
{
    int sd;
    int errorCode;
    ttUserIpAddress ipAddress;
    ttUserIpAddress netMask;
    ttUserLinkLayer linkLayerHandle;
    ttUserInterface interfaceHandle;
    struct sockaddr_in toAddress;
    char TM_FAR *testBufPtr;
#ifdef TM_MULTIPLE_CONTEXT
    ttUserContext           contextHandle;
#endif /* TM_MULTIPLE_CONTEXT */
    
    data=data;

#ifndef TM_MULTIPLE_CONTEXT
    tfInitTreck();
#else /* TM_MULTIPLE_CONTEXT */
    tfInitTreckMultipleContext();
    contextHandle = tfCreateTreckContext();
    if (contextHandle == (void TM_FAR *)0)
    {
        tfKernelError("main","Failed to allocate Treck context\n");
    }
    else
    {
        tfSetCurrentContext(contextHandle);
    }
#endif /* TM_MULTIPLE_CONTEXT */
    
/* Start Treck */
    tfStartTreck();
/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();
    OSSetUpTimer();
/* Add the Interface */
    interfaceHandle=tfAddInterface(
            "PACKET.001",
            linkLayerHandle,
            tfPacketOpen,
            tfPacketClose,
            tfPacketSend,
            tfPacketRecv,
            (ttFreeRecvBufferFuncPtr)0,
            tfPacketIoctl,
            tfPacketGetPhyAddr,
            &errorCode);
    
/* Start the tasks */
    OSTaskCreate((ttTaskPtr)timerTask, (void *)&interfaceHandle, (void *)&Stk1[STK_SIZE],1);
    OSTaskCreate((ttTaskPtr)recvTask, (void *)&interfaceHandle, (void *)&Stk2[STK_SIZE], 2);

/* 208.229.201.110 */
    ipAddress=htonl(0xd0e5c96eUL);
/* 255.255.255.192 */    
    netMask=htonl(0xffffffc0UL);
/* Config the Interface */    
    errorCode=tfConfigInterface(
            interfaceHandle,
            ipAddress,
            netMask,
            0,
            1,
            0);

/* Open a socket */
    sd=socket(PF_INET,SOCK_DGRAM,0);
    toAddress.sin_family=PF_INET;
    toAddress.sin_addr.s_addr=htonl(0xd0e5c950UL);
    toAddress.sin_port=htons(0x0009UL);
    tvUdpChecksumOn=0;
   
    testBufPtr=(char *)testBuf;
    while (1)
    {
        for (testType=0;testType<4;testType++)
        {
            switch (testType)
            {
                case 0:
                    errorCode=sendSmallCopy(sd, &toAddress,testBufPtr);
                    if (errorCode!=TM_ENOERROR)
                    {
                        treckError=errorCode;
                    }
                    break;
                case 1:
                    errorCode=sendLargeCopy(sd, &toAddress,testBufPtr);
                    if (errorCode!=TM_ENOERROR)
                    {
                        treckError=errorCode;
                    }
                    break;
                case 2:
                    errorCode=sendSmallNoCopy(sd, &toAddress);
                    if (errorCode!=TM_ENOERROR)
                    {
                        treckError=errorCode;
                    }
                    break;
                case 3:
                    errorCode=sendLargeNoCopy(sd, &toAddress);
                    if (errorCode!=TM_ENOERROR)
                    {
                        treckError=errorCode;
                    }
                    break;
                case 4:
                    while (1)
                    {
                        ;
                    }
                default:
                    break;
            }
        }
    }
}

int sendSmallCopy(int sd, struct sockaddr_in *toAddress, char *testPtr)
{
   int errorCode;
   
   while (1) {
        errorCode = sendto(sd,testPtr,18,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
        if (errorCode == -1)
        {
            break;
        }
    }
    return errorCode;
}

int sendLargeCopy(int sd, struct sockaddr_in *toAddress, char *testPtr)
{
   int errorCode;
   
   while (1) {
        errorCode = sendto(sd,testPtr,1472,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
        if (errorCode == -1)
        {
            break;
        }
    }
    return errorCode;
}

int sendSmallNoCopy(int sd, struct sockaddr_in *toAddress)
{
   char         *testPtr;
   int           errorCode;
   ttUserMessage bufferHandle;

   while (1) {
        bufferHandle=tfGetZeroCopyBuffer(18,&testPtr);
        errorCode = tfZeroCopySendTo(sd,bufferHandle,18,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
        if (errorCode == -1)
        {
            break;
        }
    }
    return errorCode;
}

int sendLargeNoCopy(int sd, struct sockaddr_in *toAddress)
{
   char         *testPtr;
   int           errorCode;
   ttUserMessage bufferHandle;

   while (1) {
        bufferHandle=tfGetZeroCopyBuffer(1472,&testPtr);
        errorCode = tfZeroCopySendTo(sd,bufferHandle,1472,0,
            (const struct sockaddr TM_FAR *)toAddress,(int)sizeof(struct sockaddr)); 
        if (errorCode == -1)
        {
            break;
        }
    }
    return errorCode;
}

void timerTask(void *interfaceHandle)
{
    while (1) 
    {
        tfTimerUpdate();
        tfTimerExecute();
        OSTimeDly(1);
        (void)tfIoctlInterface( *(ttUserInterface *)interfaceHandle,
                                TM_DEV_IOCTL_REFILL_POOL_FLAG,
                                (void *)0,
                                0 );
    }
}

void recvTask(void *interfaceHandle)
{
    int errorCode;
    while (1) 
    {
        errorCode=tfWaitReceiveInterface(*(ttUserInterface *)interfaceHandle);
        if (errorCode==TM_ENOERROR)
        {
            (void)tfRecvInterface(*(ttUserInterface *)interfaceHandle); 
        }
    }
}




/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:43      For: Xilinx-Samsung
 * File Name: test1.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 49483    Output checksum: 14102
 */
