/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/intelx86/real/msc60/ucos186c.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:46CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
***********************************************************
*                      UCOS186C.C
*               80186/80188 Specific Code
*                  SMALL MEMORY MODEL
***********************************************************
*                     INCLUDE FILES
***********************************************************
*/
#include "INCLUDES.H"
extern UBYTE    OSMapTbl[]; 
extern UBYTE    OSUnMapTbl[]; 

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/

UBYTE OSTaskCreate(void (far *task)(void *dptr), void  *data, void  *pstk, UBYTE  p)
{
    OS_TCB *ptr;
    UWORD  *stk;


    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {                       /* Avoid creating task if already exist */
        ptr              = OSTCBGetFree();
        ptr->OSTCBPrio   = (UBYTE)p;
        ptr->OSTCBStat   = OS_STAT_RDY;
        ptr->OSTCBDly    = 0;

        stk              = (UWORD *)pstk;                       /* 80186/80188 Small Model              */
        *--stk           = (UWORD)FP_OFF(data);
        *--stk           = (UWORD)FP_SEG(task);
        *--stk           = (UWORD)FP_OFF(task);
        *--stk           = (UWORD)0x0200;                       /* PSW = Int. En.                       */
        *--stk           = (UWORD)FP_SEG(task);
        *--stk           = (UWORD)FP_OFF(task);
        *--stk           = (UWORD)0x0000;                       /* AX = 0                               */
        *--stk           = (UWORD)0x0000;                       /* CX = 0                               */
        *--stk           = (UWORD)0x0000;                       /* DX = 0                               */
        *--stk           = (UWORD)0x0000;                       /* BX = 0                               */
        *--stk           = (UWORD)0x0000;                       /* SP = 0                               */
        *--stk           = (UWORD)0x0000;                       /* BP = 0                               */
        *--stk           = (UWORD)0x0000;                       /* SI = 0                               */
        *--stk           = (UWORD)0x0000;                       /* DI = 0                               */
        *--stk           = (UWORD)0x0000;                       /* ES = 0                               */
        ptr->OSTCBStkPtr = (void *)stk;                         /* Load SP in TCB                       */
        OSTCBPrioTbl[p]  = ptr;
        OS_ENTER_CRITICAL();
        ptr->OSTCBNext        = OSTCBList;
        ptr->OSTCBPrev        = (OS_TCB *)0;
        if (OSTCBList != (OS_TCB *)0) {                         /* Rev. A, This line was missing        */
            OSTCBList->OSTCBPrev = ptr;
        }
        OSTCBList             = ptr;
        OSRdyGrp             |= OSMapTbl[p >> 3];
        OSRdyTbl[p >> 3]     |= OSMapTbl[p & 0x07];
        OS_EXIT_CRITICAL();
        if (OSRunning) {
            OSSched();
        }
        return (OS_NO_ERR);
    } else {
        return (OS_PRIO_EXIST);
    }
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:44      For: Xilinx-Samsung
 * File Name: ucos186c.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 60459    Output checksum: 54550
 */
