/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/mips/gnu/cpu.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:48CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * File: cpu.c
 *
 * MIPS-specific stuff for the uC/OS Real-time multitasking kernel
 *
 * Created by Jean J. Labrosse. 
 * MIPS port by Phil Bunce (pjb@carmel.com) based on the ARM port 
 * by Marco Graziano (marcog@crl.com).
 *
 */

/* This port requires YAMON 2.04 +*/

#include "includes.h"
#include "yamon_api.h"
#include <malta.h>

#ifdef EL
  #define RTC_ADDR 0x70
  #define RTC_DATA 0x71
  #define UART_DAT 0x3f8
#endif
#ifdef EB
  /* Swap byte lanes of PCI byte addresses */
  #define RTC_ADDR 0x73
  #define RTC_DATA 0x72
  #define UART_DAT 0x3fb
#endif


extern UBYTE OSIntNesting; /* in cpu.c*/
extern void OSMySched(void);

/***********************************************************************
*  uint *BuildNewStack(uint *stk,PTV task,void *pdata)
*/
uint *BuildNewStack(uint *stk,UCOStask task,void *pdata)
{
    stk -= 32/*E_SIZE*/;
    stk[/*E_RA*/0] = (uint)task;
    stk[/*E_A0*/4] = (uint)pdata;
    stk[/*E_EPC*/29] = (uint)task;
    return(stk);
}
/* create a task */
UBYTE	
OSTaskCreate(UCOStask task, void *pdata, void *ptsk, UBYTE prio)
{
	uint	*stk;
	UBYTE	Ret;

	OS_ENTER_CRITICAL(); 
	if (OSTCBPrioTbl[prio] != (OS_TCB *)0) {
		/* only one task allowed at its priority */
		OS_EXIT_CRITICAL(); 
		Ret = OS_PRIO_EXIST;
	}
	else {
		OS_EXIT_CRITICAL(); 
		stk = (uint *)ptsk;
		/* build a context for the new task */
		stk = BuildNewStack(stk,task,pdata);
		Ret = OSTCBInit(prio, (void *)stk);
		if (Ret == OS_NO_ERR && OSRunning) 
			OSSched();
	}

	return(Ret);
} /* OSCreateTask */



void TimerIsr(void *data)
{
    char c;
    volatile char * malta_io_base = (volatile char *)0xb8000000;

/* clear interrupt */

    malta_io_base[RTC_ADDR] = 0x0c;
    c = malta_io_base[RTC_DATA];
    
    OSIntNesting ++;
/*    YAMON_FUNC_PRINT("T"); */
	OSTimeTick();
    OSIntExit();
}


void ReceiveIsr(void *data)
{
    char c;
    
    OSIntNesting ++;
    tfAm79cIsrHandler0();
    OSIntExit();
}

void bspInit()
{
    t_yamon_ref myref;
    char mydat;
    volatile char * malta_io_base;
   
/* set up slow periodic interrupt */
    malta_io_base = (volatile char *)0xb8000000;
    malta_io_base[RTC_ADDR] = 0x0a;
    malta_io_base[RTC_DATA] = 0x29; /*0x2f*/
/* The last four bits controls the timer speed
 *  1000    ~4ms          8
 *  1001    ~8ms          9
 *  1010    ~16ms         a
 *  1011    ~32ms         b
 *  1100    ~64ms         c
 *  1101    ~125ms        d
 *  1110    ~250ms        e
 *  1111    ~500ms        f
 */
    malta_io_base[RTC_ADDR] = 0x0b;
    malta_io_base[RTC_DATA] = 0x47;
    malta_io_base[RTC_ADDR] = 0x0c;
    mydat = malta_io_base[RTC_DATA];

    YAMON_FUNC_REGISTER_IC_ISR(8, TimerIsr, &mydat, &myref );
    sys_enable_int();
}

void OSIntCtxSw(void)
{
    UINT32  malta_pci_io_base;
    volatile UINT8 *piix4_pci_io_base ;

/*save the stack value from Yamon, to the block where $sp points to*/
    OSMySched();
/* the following load the stack of OSTCBHighRdy, and jump to its EPC*/
    __asm("nop");
    __asm("lw	$t1, OSTCBCur");
	__asm("sw	$t0, ($t1)");
    __asm("lw	$t0, OSTCBHighRdy");
	__asm("lw	$sp, ($t0)");
    __asm("sw   $t0, OSTCBCur");
    /* restore the new context */
    __asm("lw   $t0, 27*4($sp)"); 
    __asm("mthi $t0");
    __asm("lw   $t0, 28*4($sp)");
    __asm("mtlo $t0");
    __asm("lw   $ra, 0*4($sp)");
    __asm("lw   $v0, 2*4($sp)"); 
    __asm("lw   $v1, 3*4($sp)"); 
    __asm("lw   $a0, 4*4($sp)");  
    __asm("lw   $a1, 5*4($sp)"); 
    __asm("lw   $a2, 6*4($sp)"); 
    __asm("lw   $a3, 7*4($sp)"); 
    __asm("lw   $t0, 8*4($sp)"); 
    __asm("lw   $t1, 9*4($sp)"); 
    __asm("lw   $t2, 10*4($sp)"); 
    __asm("lw   $t3, 11*4($sp)");
    __asm("lw   $t4, 12*4($sp)");
    __asm("lw   $t5, 13*4($sp)");  
    __asm("lw   $t6, 14*4($sp)"); 
    __asm("lw   $t7, 15*4($sp)");
    __asm("lw   $t8, 16*4($sp)"); 
    __asm("lw   $t9, 17*4($sp)"); 
    __asm("lw   $s0, 18*4($sp)"); 
    __asm("lw   $s1, 19*4($sp)"); 
    __asm("lw   $s2, 20*4($sp)");   
    __asm("lw   $s3, 21*4($sp)");  
    __asm("lw   $s4, 22*4($sp)");   
    __asm("lw   $s5, 23*4($sp)");  
    __asm("lw   $s6, 24*4($sp)"); 
    __asm("lw   $s7, 25*4($sp)");  
    __asm("lw   $s8, 26*4($sp)"); 
    __asm("lw   $gp, 31*4($sp)"); 
    
    __asm(".set noat");
    __asm("lw   $at, 1*4 ($sp)");
    __asm(".set at");

    __asm("lw   $k0, 29*4($sp)");
    __asm("addu $sp, 32*4");
/*move k0 to EPC*/
    __asm__ volatile ("mtc0  $k0, $14");

/* set IE, 0x01 
 * it seems IE will be set after eret, so that we don't need to do it
 * by ourselves  __asm__ volatile ("mfc0  $t0, $12"); C0_SR
 */
    
    __asm__ volatile ("mfc0  $t0, $12");
    __asm__ volatile ("ori  $t0, $t0, 0x01");
    __asm__ volatile ("mtc0 $t0, $12");
/* send of EOI, otherwise, YAMON will think we are still in our ISR */
    malta_pci_io_base = 0x18000000;
    piix4_pci_io_base =
              (volatile UINT8*)KSEG1(MALTA_PCI0_IO_BASE + 
                                     PIIX4_IO_BRIDGE_START);

#define NSEOI    ( (PIIX4_OCW2_CODE_NSEOI << PIIX4_OCW2_CODE_SHF) | \
		   (PIIX4_OCW2_OCWSEL_2   << PIIX4_OCW2_OCWSEL_SHF) )

    /* Perform Non-specific EOI for master (ignore index) */
	MALTA_PIIX4_IO8( PIIX4_OCW2M_OFS ) = NSEOI;

	/* Perform Non-specific EOI for slave (ignore index) */
	MALTA_PIIX4_IO8( PIIX4_OCW2S_OFS ) = NSEOI;

/*back to the interrupted task. eret will set the EXL*/    
    __asm("eret");
    __asm("nop");

}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:44      For: Xilinx-Samsung
 * File Name: cpu.c         File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 44485    Output checksum: 38788
 */
