/*************************************************************
 *
 * File: cpu_a.s
 *
 * uC/OS Real-time multitasking kernel for the MIPS processor.
 * Low-level routines.
 *
 * Written by Phil Bunce (pjb@carmel.com).
 * Based on the ARM implementaion by Marco Graziano (marcog@crl.com).
 *
 * $Source: source/kernel/ucos/mips/gnu/cpu_a.S $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:50CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 *
 */

#include "maltareg.h"

	.extern OSTCBCur,4
	.extern OSTCBHighRdy,4


/*************************************************************
*	void OSDisableInt(void)
*	void OSEnableInt(void)
*
*	Disable and enable IRQ and FIQ preserving current CPU mode.
*
*/
	.globl	OSDisableInt
	.ent	OSDisableInt
OSDisableInt:
	.set noreorder
	mfc0	t0,C0_SR
	nop
	and	t0,~SR_IEC
	mtc0	t0,C0_SR
	.set reorder
	j	ra
	.end	OSDisableInt

	.globl	OSEnableInt
	.ent	OSEnableInt
OSEnableInt:
	.set noreorder
	mfc0	t0,C0_SR
	nop
	or	t0,SR_IEC
	mtc0	t0,C0_SR
	.set reorder
	j	ra
	.end	OSEnableInt


/*************************************************************
*	void OSCtxSw(void)
*		Perform a context switch.
*	void OSStartHighRdy(void)
*		Start the task with the highest priority;
*
*/
	.globl	OSCtxSw
	.globl	OSStartHighRdy
	.ent	OSCtxSw
	#.set noat
OSCtxSw:
	# save the current context
	# we do not need to save all regs because we know that we
	# got here via a normal call.
	subu	sp,E_SIZE*4
	sw	s0,E_S0*4(sp) ; sw	s1,E_S1*4(sp)
	sw	s2,E_S2*4(sp) ; sw	s3,E_S3*4(sp)
	sw	s4,E_S4*4(sp) ; sw	s5,E_S5*4(sp)
	sw	s6,E_S6*4(sp) ; sw	s7,E_S7*4(sp)
	sw	s8,E_S8*4(sp) ; sw	ra,E_RA*4(sp)
	sw	ra,E_EPC*4(sp)		# also save ra as EPC
    sw  gp,E_GP*4(sp)
	mfc0	t2,C0_SR  #also save SR by Jin
    nop
    nop
    sw  t2,E_SR*4(sp)

	# Get current task TCB address
	lw	t0, OSTCBCur		# get TCB addr
	sw	sp, (t0)		# store sp in preempted tasks's TCB

OSStartHighRdy:
	# Get highest priority task TCB address
	lw	t0, OSTCBHighRdy	# get TCB addr
	lw	sp, (t0)		# get new task's stack pointer
	sw	t0, OSTCBCur		# set new current task TCB address

	# restore the new context
	lw	t0,E_HI*4(sp)
	mthi	t0
	lw	t0,E_LO*4(sp)
	mtlo	t0
	lw	v1,E_V1*4(sp) ; lw	a0,E_A0*4(sp)
	lw	a1,E_A1*4(sp) ; lw	a2,E_A2*4(sp)
	lw	a3,E_A3*4(sp) ; lw	t0,E_T0*4(sp)
	# t1 and t2 are restored later
	lw	t3,E_T3*4(sp) ; lw	t4,E_T4*4(sp)
	lw	t5,E_T5*4(sp) ; lw	t6,E_T6*4(sp)
	lw	t7,E_T7*4(sp) ; lw	t8,E_T8*4(sp)
	lw	t9,E_T9*4(sp) ; lw	s0,E_S0*4(sp)
	lw	s1,E_S1*4(sp) ; lw	s2,E_S2*4(sp)
	lw	s3,E_S3*4(sp) ; lw	s4,E_S4*4(sp)
	lw	s5,E_S5*4(sp) ; lw	s6,E_S6*4(sp)
	lw	s7,E_S7*4(sp) ; lw	s8,E_S8*4(sp)
	lw	ra,E_RA*4(sp) ; lw	v0,E_V0*4(sp)


	# now restore our two working regs & AT
	lw	t1,E_T1*4(sp)
	lw	t2,E_T2*4(sp)
	.set noat
	lw	AT,E_AT*4(sp)

    #set the exception bit, (and the IE bit), eret will set it back
    mfc0 k0, C0_SR
    nop
    nop
    ori k0, k0, 0x03 
    mtc0 k0, C0_SR
    nop
    nop


	.set noreorder
	lw	k0,E_EPC*4(sp)
	addu	sp,E_SIZE*4
    mtc0   k0, C0_EPC
    nop
    nop
	eret
    nop  
# added by Jin, rfe is handled by Yamon, using eret
#	rfe
	.set reorder
	.set at
	.end	OSCtxSw




/*************************************************************
*	void OSMySched(void)
*		Perform a context switch after TimeIsr.
*	For simplicity, we just 1) save Yamon's context to the 
*   switch victim's stack. 2) load the higher priorty task's 
*   stack to Yamon's stack. We leave all other stuff to Yamon.
*   This results in one additional memory->register->memory 
*   saving. Instead of this, we could be able to load the higher
*   priority task's stack directly to the running registers, and
*   take care function such as eret ourselves.
*
*/
	.globl	OSMySched
	.ent	OSMySched
OSMySched:
#   we know that Yamon saved everything use the base address k1
#   Get the sp->t0 first
    
/* the following is to get k1 and exception*/
#    move  t0, k1
#    move  t1, k1
#    move  zero, t2
#    sw    t1, (t2) 
   
    lw  t0, OFS_END + 29*8(k1); 
	subu	t0,E_SIZE*4

    lw  t1, OFS_END + 1*8(k1);      sw   t1, E_AT*4(t0);
    lw  t1, OFS_END + 2*8(k1);      sw   t1, E_V0*4(t0);
    lw  t1, OFS_END + 3*8(k1);      sw   t1, E_V1*4(t0);
    lw  t1, OFS_END + 4*8(k1);      sw   t1, E_A0*4(t0);
    lw  t1, OFS_END + 5*8(k1);      sw   t1, E_A1*4(t0);
    lw  t1, OFS_END + 6*8(k1);      sw   t1, E_A2*4(t0);
    lw  t1, OFS_END + 7*8(k1);      sw   t1, E_A3*4(t0);
    lw  t1, OFS_END + 8*8(k1);      sw   t1, E_T0*4(t0);
    lw  t1, OFS_END + 9*8(k1);      sw   t1, E_T1*4(t0);
    lw  t1, OFS_END + 10*8(k1);     sw   t1, E_T2*4(t0);
    lw  t1, OFS_END + 11*8(k1);     sw   t1, E_T3*4(t0);
    lw  t1, OFS_END + 12*8(k1);     sw   t1, E_T4*4(t0);
    lw  t1, OFS_END + 13*8(k1);     sw   t1, E_T5*4(t0);
    lw  t1, OFS_END + 14*8(k1);     sw   t1, E_T6*4(t0);
    lw  t1, OFS_END + 15*8(k1);     sw   t1, E_T7*4(t0);
    lw  t1, OFS_END + 16*8(k1);     sw   t1, E_S0*4(t0);
    lw  t1, OFS_END + 17*8(k1);     sw   t1, E_S1*4(t0);
    lw  t1, OFS_END + 18*8(k1);     sw   t1, E_S2*4(t0);
    lw  t1, OFS_END + 19*8(k1);     sw   t1, E_S3*4(t0);
    lw  t1, OFS_END + 20*8(k1);     sw   t1, E_S4*4(t0);
    lw  t1, OFS_END + 21*8(k1);     sw   t1, E_S5*4(t0);
    lw  t1, OFS_END + 22*8(k1);     sw   t1, E_S6*4(t0);
    lw  t1, OFS_END + 23*8(k1);     sw   t1, E_S7*4(t0);
    lw  t1, OFS_END + 24*8(k1);     sw   t1, E_T8*4(t0);
    lw  t1, OFS_END + 25*8(k1);     sw   t1, E_T9*4(t0);
# we don't save k0, k1, sp
    lw  t1, OFS_END + 28*8(k1);     sw   t1, E_GP*4(t0);
    lw  t1, OFS_END + 30*8(k1);     sw   t1, E_S8*4(t0);
    lw  t1, OFS_END + 31*8(k1);     sw   t1, E_RA*4(t0);
    lw  t1, OFS_END + 32*8(k1);     sw   t1, E_SR*4(t0);
    lw  t1, OFS_END + 33*8(k1);     sw   t1, E_LO*4(t0);
    lw  t1, OFS_END + 34*8(k1);     sw   t1, E_HI*4(t0);
    lw  t1, OFS_END + 37*8(k1);     sw   t1, E_EPC*4(t0);

    j   ra
    nop
	.end	OSMySched


