/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/mips/gnu/maltareg.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:54CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * MIPS-specific stuff for the uC/OS multitasking kernel
 */

#ifndef _MALTAREG_H_
#define _MALTAREG_H_


#ifdef EB
#define  OFS_END 4
#else
#if defined(EL)
#define  OFS_END 0
#endif
#endif



/* aliases for general registers */
#define zero		$0
#define	AT		$1		/* assembler temporaries */
#define	v0		$2		/* value holders */
#define	v1		$3
#define	a0		$4		/* arguments */
#define	a1		$5
#define	a2		$6
#define	a3		$7
#define	t0		$8		/* temporaries */
#define	t1		$9
#define	t2		$10
#define	t3		$11
#define	t4		$12
#define	t5		$13
#define	t6		$14
#define	t7		$15
#define	s0		$16		/* saved registers */
#define	s1		$17
#define	s2		$18
#define	s3		$19
#define	s4		$20
#define	s5		$21
#define	s6		$22
#define	s7		$23
#define	t8		$24		/* temporaries */
#define	t9		$25
#define	k0		$26		/* kernel registers */
#define	k1		$27
#define	gp		$28		/* global pointer */
#define	sp		$29		/* stack pointer */
#define	s8		$30		/* saved register */
#define	fp		$30		/* frame pointer (old usage) */
#define	ra		$31		/* return address */

#ifdef LANGUAGE_ASSEMBLY
/* System Control Coprocessor (CP0) registers */
#define C0_SR		$12		/* Processor Status */
#define C0_CAUSE	$13		/* Exception Cause */
#define C0_EPC		$14		/* Exception PC */
#define C0_BADADDR	$8		/* Bad Address */
#define C0_BADVADDR	$8		/* Bad Virtual Address */
#define C0_PRID		$15		/* Processor Rev Indentifier */

/* Floating-Point Control registers */
#define FPA_CSR		$31		/* Control/Status register */
#define FPA_IRR		$0		/* Implementation/Rev register */
#define C1_CSR		$31
#define C1_FRID		$0

#else /* ================ LANGUAGE_C ======================== */

#define C0_SR		12		/* Processor Status */
#define C0_CAUSE	13		/* Exception Cause */
#define C0_EPC		14		/* Exception PC */
#define C0_BADADDR	8		/* Bad Address */
#define C0_BADVADDR	8		/* Bad Virtual Address */
#define C0_PRID		15		/* Processor Rev Indentifier */

/* Floating-Point Control registers */
#define FPA_CSR		31		/* Control/Status register */
#define FPA_IRR		0		/* Implementation/Rev register */
#define C1_CSR		31
#define C1_FRID		0
#endif /* ============== LANGUAGE_C */


#define SR_IEC		0x00000001	/* Interrupt Enable, current */

/* uC/OS-specific stuff from here on.. */
#define E_RA  0
#define E_AT  1
#define E_V0  2
#define E_V1  3
#define E_A0  4
#define E_A1  5
#define E_A2  6
#define E_A3  7
#define E_T0  8
#define E_T1  9
#define E_T2  10
#define E_T3  11
#define E_T4  12
#define E_T5  13
#define E_T6  14
#define E_T7  15
#define E_T8  16
#define E_T9  17
#define E_S0  18
#define E_S1  19
#define E_S2  20
#define E_S3  21
#define E_S4  22
#define E_S5  23
#define E_S6  24
#define E_S7  25
#define E_S8  26
#define E_HI  27
#define E_LO  28
#define E_EPC 29
#define E_SR  30
#define E_GP  31
#define E_SIZE 32 /* must be even */

#endif /* _MALTAREG_H_ */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:44      For: Xilinx-Samsung
 * File Name: maltareg.h    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 62540    Output checksum: 39651
 */
