/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/mips/gnu/test1.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:39:57CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * File:        example1.c
 *
 * uC/OS Real-time multitasking kernel for the MIPS processor.
 *
 * Simple example of using multiple tasks and mailboxes.
 *
 * Created by Marco Graziano (marcog@crl.com).
 *
 */
#include	"includes.h" 	/* uC/OS interface */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include "yamon_api.h"
#include "tram79c.h"

/* allocate memory for tasks' stacks */
#define	STACKSIZE	4096

static unsigned int	Stack1[STACKSIZE];
static unsigned int	Stack2[STACKSIZE];
static unsigned int	Stack3[STACKSIZE];
static char Id1, Id2;
static void Task2(void *Id);
static void Task1(void *Id);
void recvTask(void *interfaceHandle);



static void Task1(void *Id)
{
    int errorCode;
    ttUserIpAddress         ipAddress;
    ttUserIpAddress         netMask;
    ttUserLinkLayer         linkLayerHandle;
    ttUserInterface         interfaceHandle;
    struct sockaddr_in      toAddress;
   
    tfInitTreck();
    tfStartTreck();
/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();
/* Add the Interface */
    interfaceHandle=tfAddInterface(
            "malta1",
            linkLayerHandle,
            tfAm79cOpen,
            0,
            tfAm79cSend,
            tfAm79cReceive,
			(ttDevFreeRecvFuncPtr)0,
            0,
            tfAm79cGetPhysicalAddress,
            &errorCode);

    ipAddress=inet_addr("192.168.1.9");
    netMask=htonl(0xffffff00);
/* Config the Interface */    
    errorCode=tfConfigInterface(
            interfaceHandle,
            ipAddress,
            netMask,
            TM_DEV_SCATTER_SEND_ENB,  
            5,  
            0);

    OSTaskCreate(Task2, (void *)&Id2, (void *)&Stack2[STACKSIZE - 6], 1);
    OSTaskCreate(recvTask, (void *)&interfaceHandle, (void *)&Stack3[STACKSIZE - 6], 2);

	for (;;) {

	}
}

static void
Task2(void *Id)
{

   for(;;)
   {
        
        tfTimerUpdate();
        tfTimerExecute();
        OSTimeDly(2); 
	}
}


void tfAm79cIsrHandler0(void);  
void recvTask(void *interfaceHandle)
{
    while (1) 
    {
        (void)tfWaitReceiveInterface(*(ttUserInterface *)interfaceHandle);
        (void)tfRecvInterface(*(ttUserInterface *)interfaceHandle);
    }
}



/*
 * Main function.
 */
int
main(int argc, char **argv)
{
    Id1 = '1';
	Id2 = '2';


	/* needed by uC/OS */
	OSInit();
	/* 
	 * create the tasks in uC/OS and assign increasing
	 * priorities to them so that Task3 at the end of
	 * the pipeline has the highest priority.
	 */
	OSTaskCreate(Task1, (void *)&Id1, (void *)&Stack1[STACKSIZE - 6], 3);

	bspInit();
	/* start the os */
	OSStart();

	/* never reached */
} /* main */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:44      For: Xilinx-Samsung
 * File Name: test1.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 61004    Output checksum: 47042
 */
