/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/motorola/cpu32/mri/ucosc32c.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:40:14CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/* file: ucosc32c.c */

/*
***********************************************************
*                      UCOSC32C.C
*                  CPU32 Specific Code
***********************************************************
*                     INCLUDE FILES
***********************************************************
*/
#include "INCLUDES.H"
#ifdef TM_68360
#include <trsocket.h>
#include <trquicc.h>
#endif /* TM_68360 */

/*
***********************************************************
*                    CREATE A TASK
***********************************************************
*/

long *globaldata; /* each task's A5 should be set to 'globaldata' */

 /* The INITIAL_SR makes each task start up with the following attributes:
    1. supervisor mode
    2. all interrupts enabled
 */

#define INITIAL_SR 0x2000
#ifdef TM_68360
void OSSetUpTimer(void);
void OSTimerClear(void);
void OSIdmaBcopy(const void *src, void *dest, unsigned len);
#endif


void OSTickISR(void);
unsigned long OSGetVBR(void);

UBYTE OSTaskCreate(void (*task)(void *dptr), void  *data, void  *pstk, UBYTE  p)
{
    long   *stk;
    UBYTE  err;


    if (OSTCBPrioTbl[p] == (OS_TCB *)0) {            /* Avoid creating task if already exist */
        stk              = (long *)((long) pstk &             /* pstk points to highest-address       */
                           (long) 0xfffffffcL);       /* long-word of the task's stack        */
        

        *--stk           = (long)data;
        *--stk           = (long)task;

        *--((UWORD *)stk)  = 0x0080;                   /* exception format=0, offset=TRAP #0   */
        *--stk           = (long)task;
        *--((UWORD *)stk)  = INITIAL_SR;               /* initial SR value                     */

        *--stk           = 0L;                       /* Sim A6-A0 (32-bit) reg's on stack    */
        *--stk           = (long)globaldata;         /* A5 = global data ptr */
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;

        *--stk           = 0L;                       /* Sim D7-D0 (32-bit) reg's on stack    */
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;
        *--stk           = 0L;

        err = OSTCBInit(p, (void *)stk);     /* Get and initialize a TCB                            */
        if (err == OS_NO_ERR) {
            if (OSRunning) {                     /* Find highest prio. task if multitasking has started */
                OSSched();
            }
        }
        return (err);
    } else {
        OS_EXIT_CRITICAL();
        return (OS_PRIO_EXIST);
    }
}

/*
 * Setup the timer
 */
#ifdef TM_68360
volatile static ttQuicc *tlQuiccPtr;

void OSSetUpTimer()
{
    unsigned long *mbar;
    unsigned long  quiccRegisters;
    unsigned char  timerVector;
    unsigned long  timerLocation;

/* The mbar is ALWAYS at 0x03ff00 on a 68360 */
    mbar=(unsigned long *)0x03ff00UL;
    quiccRegisters=TM_QUICC_MBAR_LOC|0x00000001UL;
    *mbar=quiccRegisters;
    tlQuiccPtr = (ttQuicc *)(quiccRegisters&0xfffffffc);
    tlQuiccPtr->timer_tgcr=0x0000;
    timerVector=TM_QUICC_TMR1_INT | TM_QUICC_CICR_VECT;
    timerLocation = timerVector*4;
/* Install the timer vector */
    OSInstallISR((void *)OSTickISR,timerLocation);
    tlQuiccPtr->intr_cicr=TM_QUICC_CICR_DEFAULT;
    tlQuiccPtr->intr_cipr = 0xffffffffUL;
    tlQuiccPtr->intr_cisr |= TM_QUICC_TMR1_INT_MASK;
    tlQuiccPtr->intr_cimr = TM_QUICC_TMR1_INT_MASK;
    tlQuiccPtr->timer_tmr1=0x031a;
    tlQuiccPtr->timer_tcn1=0x0000;
    tlQuiccPtr->timer_trr1=0xF424;
    tlQuiccPtr->timer_ter1=0xFFFF;
/* Start the timer */
    tlQuiccPtr->timer_tgcr=0x0005;
}

/*
 * Clear the timer interrupt
 */
void OSTimerClear()
{
    tlQuiccPtr->timer_ter1=0xFFFF;
    tlQuiccPtr->intr_cisr |= TM_QUICC_TMR1_INT_MASK;
}

/* 
 * Use the IDMA of the 68360 to perform the bcopy
 * len can never be 3
 */
void OSIdmaBcopy(const void *src, void *dest, unsigned len)
{
    if (len==3)
    {
/* ERROR */
        while(1);
    }
/* Normal Configuration */
    tlQuiccPtr->idma_iccr=(unsigned short)0x0020;
/* Source function code = 0b1000 destingation function code=0b1001 */
    tlQuiccPtr->idma1_fcr=0x89;
/* Source address */
    tlQuiccPtr->idma1_sapr=(unsigned long)src;
/* Destination Address */
    tlQuiccPtr->idma1_dapr=(unsigned long)dest;
/* Length of the transfer */
    tlQuiccPtr->idma1_bcr=(unsigned long)len;
/* Clear any status */
    tlQuiccPtr->idma1_csr=(unsigned char)0xff;
/* Disable Interrupts (We will Poll) */
    tlQuiccPtr->idma1_cmar=(unsigned char)0x00;
/* Start the copy MAXIMUM TRANSFER RATE */
    tlQuiccPtr->idma1_cmr=(unsigned short)0x4305;
/* Wait for completion */
    while(tlQuiccPtr->idma1_cmr & (unsigned short)0x0001);
    
}
#endif /* TM_68360 */

/* 
 * Install an ISR
 */
void OSInstallISR(void *funcPtr, unsigned long offSet)
{
    unsigned long vbr;
    unsigned long address;
/* Get the VBR register with an ASM call */
    vbr=OSGetVBR();
    address=vbr+offSet;
/* Store this ISR at the vbr+offset of the vector */
    *((void **)address) = funcPtr;
    return;
}    
    
/*
 * Install the uC/OS Handler
 */
void OSSetupUcosHandler()
{
    OSInstallISR((void *)OSCtxSw, (unsigned long)((UCOS*4)+0x80));
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:44      For: Xilinx-Samsung
 * File Name: ucosc32c.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 12266    Output checksum: 60631
 */
