* file: ucosc32a.asm
* $Source: source/kernel/ucos/motorola/cpu32/sds/ucosc32a.asm $
*
* Modification History
* $Revision: 6.0.2.2 $
* $Date: 2010/01/18 06:40:30CST $
* $Author: odile $
* $ProjectName: /home/mks/proj/tcpip.pj $
* $ProjectRevision: 6.0.1.23 $
*
            XDEF START
            .REGION reset
            DC.L    START
            DC.L    STKTOP                  ; initial stack pointer
            .REGION code
*****************************************************************
*                        UCOSC32A.ASM
*                CPU32 Real Time Operating System
*****************************************************************
            XDEF _OSStartHighRdy
            XDEF _OSCtxSw
            XDEF _OSIntCtxSw
            XDEF _OSTickISR
            XDEF _OSGetVBR
**** The Assembly Handler for SCC1
            XDEF _tfQuiccScc1Isr
**** The Assembly Handler for SCC4
            XDEF _tfQuiccScc4Isr
            
**** The "C" Handler for SCC1
            XREF  _tfQuiccScc1HandlerIsr
**** The "C" Handler for SCC4
            XREF  _tfQuiccScc4HandlerIsr
            XREF  _OSIntEnter
            XREF  _OSIntExit
            XREF  _OSTimeTick
            XREF  _OSTCBCur
            XREF  _OSTCBHighRdy
            XREF  _OSTimerClear
            XREF  _main
            XREF   DATA
            XREF   STKTOP 

START
            ori     #$0700,sr
            move.l  #0,a0
            movec   a0,vbr              ;0x0000 <= vectors <= 0x03ff

            move.w  #$2700,SR           ;initial status register/ints OFF
            MOVE.L  #STKTOP,A7          ; set the stack pointer
            MOVE.L  #0,A6               ; terminate call chain for -Og

***********************************************************
;               Zero out uninitialized RAM.
***********************************************************
        MOVE.L  #`BASE(ram),A1          ; A1 = base of region ram
        MOVE.L  #`SIZE(ram),D0          ; D0 = size of region ram
        LSR.L   #2,D0                   ; compute size in longs
        BRA     ZDBF                    ; enter a fast loop
  ZLP:  CLR.L   (A1)+                   ; clear four bytes at a time
  ZDBF: DBF     D0,ZLP                  ; up to 256K in inner loop
        SUB.L   #$10000,D0              ; rest in outer loop
        BHS     ZLP

;***********************************************************
;               Initialize other RAM from ROM.
;***********************************************************
        MOVE.L  #DATA,A0                ; A0 = ROM base of region data
        MOVE.L  #`BASE(data),A1         ; A1 = RAM base of region data
        MOVE.L  #`SIZE(data),D0         ; D0 = size of region data
        LSR.L   #2,D0                   ; compute size in longs
        BRA     IDBF                    ; enter a fast loop
  ILP:  MOVE.L  (A0)+,(A1)+             ; move four bytes at a time
  IDBF: DBF     D0,ILP                  ; up to 256K in inner loop
        SUB.L   #$10000,D0              ; rest in outer loop
        BHS     ILP
            
;
; Here is where you might want to add the initial chip select settings
;
; If you change the mbar from TM_QUICC_MBAR_LOC, then you must change
; the TM_QUICC_MBAR_LOC macro to match
            JSR     _main


*****************************************************************
*                      START MULTITASKING
*                   void OSStartHighRdy(void)
*****************************************************************
            
_OSStartHighRdy

            MOVE.L   (_OSTCBHighRdy),A1 ;Get highest prio. task
            MOVE.L   A1,(_OSTCBCur)
            MOVE.L   (A1),A7            ;Get ptr to top of stack
            MOVEM.L  (A7)+,A0-A6/D0-D7
            RTE                         ;Return to task


*****************************************************************
*            PERFORM A CONTEXT SWITCH (From task level)
*                       void OSCtxSw(void)
*****************************************************************
_OSCtxSw    MOVEM.L  A0-A6/D0-D7,-(A7)  ;Save current task's context

            MOVE.L   (_OSTCBCur),A1     ;Save stack ptr in TCB
            MOVE.L   A7,(A1)

            MOVE.L   (_OSTCBHighRdy),A1 ;Point to HI Prio. Task Rdy
            MOVE.L   A1,(_OSTCBCur)     ;This is now current TCB

            MOVE.L   (A1),A7            ;Get new task's stack ptr
            MOVEM.L  (A7)+,A0-A6/D0-D7
            RTE                         ;Return to new task

*****************************************************************
*            PERFORM A CONTEXT SWITCH (From an ISR)
*                     void OSIntCtxSw(void)
*****************************************************************
_OSIntCtxSw
            ADDA   #8,A7               ;Ignore calls to OSIntExit,
*                                       ;OSIntCtxSw and locals.
*
*   The interrupt context save and the full register set must
*   be on the stack in the correct order prior to the OSIntExit call!
*

            MOVE.L   (_OSTCBCur),A1     ;Save stack ptr in TCB
            MOVE.L   A7,(A1)

            MOVE.L   (_OSTCBHighRdy),A1 ;Point to HI Prio. Task Rdy
            MOVE.L   A1,(_OSTCBCur)     ;This is now current TCB

            MOVE.L   (A1),A7            ;Get new task's stack ptr
            MOVEM.L  (A7)+,A0-A6/D0-D7
            RTE                         ;Return to new task

*****************************************************************
*                        HANDLE TICK ISR
*                      void OSTickISR(void)
*****************************************************************


_OSTickISR


        MOVEM.L D0-D7/A0-A6,-(A7)

        JSR     _OSIntEnter
        JSR     _OSTimeTick
    JSR _OSTimerClear
        JSR     _OSIntExit

        MOVEM.L (A7)+,D0-D7/A0-A6
        RTE     

*****************************************************************
*                        HANDLE SCC1 ISR
*                      void tfQuiccScc4Isr(void)
*****************************************************************


_tfQuiccScc1Isr


        MOVEM.L D0-D7/A0-A6,-(A7)

        JSR     _OSIntEnter
        JSR     _tfQuiccScc1HandlerIsr
        JSR     _OSIntExit

        MOVEM.L (A7)+,D0-D7/A0-A6
        RTE     

_tfQuiccScc4Isr


        MOVEM.L D0-D7/A0-A6,-(A7)

        JSR     _OSIntEnter
        JSR     _tfQuiccScc4HandlerIsr
        JSR     _OSIntExit

        MOVEM.L (A7)+,D0-D7/A0-A6
        RTE     

_OSGetVBR
    movec.l vbr,d0
    RTS
   
    END
