; $Source: source/kernel/ucos/motorola/ppc/ddi/crt0.s $
;
; Modification History
; $Revision: 6.0.2.2 $
; $Date: 2010/01/18 06:40:34CST $
; $Author: odile $
; $ProjectName: /home/mks/proj/tcpip.pj $
; $ProjectRevision: 6.0.1.23 $
;
# Start-up function for an embedded environment

    .file       "crt0.c"
    .text
    .extern     SIUIntr
    .globl      _start
    .globl      UpmTable
    .globl      UpmTableEnd
    .align      2
#------------------------------------------------------------------------
# On-Chip Core Register Definitions
#------------------------------------------------------------------------
#
RomBase    .equ    0x02000000
RamBase    .equ    0x00000000
RamBaseUncached	   .equ	   0x80000000

ICR        .equ    148          # Interrupt Cause Register
ICTRL      .equ    158          # Instruction Control Register
DER        .equ    149          # Debug Enable Register
DPDR       .equ    630          # Development Port Data Register
IMMR       .equ    638          # Internal I/O base register
IC_CST     .equ    560          # ICache Control Status Register
IC_ADR     .equ    561          # ICache Address Register
IC_DAT     .equ    562          # ICache Data Register
DC_CST     .equ    568          # DCache Control Status Register
DC_ADR     .equ    569          # DCache Address Register
DC_DAT     .equ    570          # DCache Data Register

SIUMCR     .equ    0x000        # SIU Module configuration
SYPCR      .equ    0x004        # SIU System Protection Control
SWSR       .equ    0x00E        # Software service register
SIMASK     .equ    0x014        # SI Mask Register
MCR        .equ    0x168        # Memory Command
MAMR       .equ    0x170        # Machine A Mode Register
MPTPR      .equ    0x17A        # Memory Periodic Timer Prescaler
MDR        .equ    0x17C        # Memory Data
BR0        .equ    0x100        # Base Register 0
OR0        .equ    0x104        # Option Register 0
BR1        .equ    0x108        # Base Register 1
OR1        .equ    0x10C        # Option Register 1
BR2        .equ    0x110        # Base Register 2
OR2        .equ    0x114        # Option Register 2
BR3        .equ    0x118        # Base Register 3
OR3        .equ    0x11C        # Option Register 3
BR4        .equ    0x120        # Base Register 4
OR4        .equ    0x124        # Option Register 4
BR5        .equ    0x128        # Base Register 5
OR5        .equ    0x12C        # Option Register 5
PLPRCR     .equ    0x284        # PLL, Low Power and Reset Control Reg


#------------------------------------------------------------------------
# LA macro - load a register with a word value
#------------------------------------------------------------------------
    .macro    LA reg,symbol
    lis       reg,%hiadj(symbol)
    addi      reg,reg,%lo(symbol)
    .endm

#
#
#------------------------------------------------------------------------
# Instruction and Data Cache definition
# Note: must use with lis instruction to load into bit 0-15
#------------------------------------------------------------------------
#
CacheUnlockAllCmd  .equ   0x0A00   # Cache Unlock_All command
CacheDisableCmd    .equ   0x0400   # Cache Disable command
CacheInvAllCmd     .equ   0x0C00   # Cache Invalidate_All command
CacheEnableCmd     .equ   0x0200   # DCache_Enable Command
CacheEnableBit     .equ   0x8000   # Cache Enable bit in I/DC_CST

#
#
# Speed of the CPU is 40mhz
# CHANGE THIS VALUE TO 0 for 25MHZ PARTS
#

TM_SPEED40 .equ   1

TM_DRAM70NS .equ 0
TM_DRAM60NS .equ 0


#------------------------------------------------------------------------
# SetCSReg macro - write a word to offset + r4
#------------------------------------------------------------------------
SetCSReg:   macro data,offset
    lis r3,%hi(data)
    ori r3,r3,%lo(data)
    stw r3,offset(r4)
    endm


#
#           INTERRUPT VECTORS               
#

#
#       RESERVED, UNUSED INTERRUPT              
#

_Ires0:
    .skip   256-(.-_Ires0)

#
#           SYSTEM RESET                    
#
_reset:
    xor        r0,r0,r0
_start:
    xor        r0,r0,r0
# init stack pointer
    addis       r11,r0,__SP_INIT@ha
    addi        r1,r11,__SP_INIT@l
    addis       r13,r0,_SDA_BASE_@ha
    addi        r13,r13,_SDA_BASE_@l
    addis       r2,r0,_SDA2_BASE_@ha
    addi        r2,r2,_SDA2_BASE_@l
    addi        r0,r0,0
    stwu        r0,-64(r1)

#
# insert other init code here
#
# do rest in C
    bl      __init_main
#   b       exit
    bl      main        # pull in main()
    .skip   256-(.-_reset)


#
#           MACHINE CHECK                   
#
_IMachineCheck:
	b		GenericIntHandler
    .skip   256-(.-_IMachineCheck)

#
#           DATA STORAGE                    
#
_IDataStore:
	b		GenericIntHandler
    .skip   256-(.-_IDataStore)

#
#           INSTRUCTION STORAGE             
#
_IInstStore:
	b		GenericIntHandler
    .skip   256-(.-_IInstStore)

#
#           EXTERNAL INTERRUPT              
#
_IExtInterrupt:
    b   SIUIntr
    rfi
    .skip   256-(.-_IExtInterrupt)

#
#           ALIGNMENT                   
#
_IAlignment:
	b		GenericIntHandler
    .skip   256-(.-_IAlignment)

#
#           PROGRAM                     
#
_IProgram:
	b		GenericIntHandler
    .skip   256-(.-_IProgram)

#
#       FLOATING POINT (NOT USED ON 860)            
#
_IFloat:
    rfi
    .skip   256-(.-_IFloat)

#
#           DECREMENTER                 
#
_IDecrementer:
	b		GenericIntHandler
    .skip   256-(.-_IDecrementer)

#
#           RESERVED                    
#
    .skip   512

#
#           SYSTEM CALL                 
#
_ISystemCall:
	b		GenericIntHandler
    .skip   256-(.-_ISystemCall)


#
#               TRACE                   
#
_ITrace:
    rfi
    .skip   256-(.-_ITrace)


#
#           FLOATING POINT ASSIST               
#
_IFloatAssist:
    rfi
    .skip   512-(.-_IFloatAssist)


#
#           SOFTWARE EMULATION              
#
_ISoftEmulate:
	b		GenericIntHandler
    .skip   256-(.-_ISoftEmulate)


#
#           INSTRUCTION TLB MISS                
#
_IInstTLB:
	b		GenericIntHandler
    .skip   256-(.-_IInstTLB)


#
#           DATA TLB MISS                   
#
_IDataTLB:
	b		GenericIntHandler
    .skip   256-(.-_IDataTLB)


#
#           INSTRUCTION TLB ERROR               
#
_IInstTLBError:
	b		GenericIntHandler
    .skip   256-(.-_IInstTLBError)


#
#           DATA TLB ERROR                  
#
_IDataTLBError:
	b		GenericIntHandler
    .skip   256-(.-_IDataTLBError)


#
#           RESERVED                    
#
    .skip   0x700

#
#           DATA BREAKPOINT                 
#
_IDataBreak:
    rfi
    .skip   256-(.-_IDataBreak)


#
#           INSTRUCTION BREAKPOINT              
#
_IInstBreak:
    rfi
    .skip   256-(.-_IInstBreak)


#
#           PERIPHERAL BREAKPOINT               
#
_IPeripBreak:
    rfi
    .skip   256-(.-_IPeripBreak)


#
#       NON-MASKABLE DEVELOPMENT PORT               
#
_IDevPort:
    rfi
    .skip   256-(.-_IDevPort)

# Init Code
# Do the rest in "C"
    .section    .init,2,C
    .globl      __init
    .align      2
__init:
    mfspr       r0,8
    stwu        r1,-64(r1)
    stw     r0,68(r1)

    .section    .fini,2,C
# Return from __init
    lwz     r0,68(r1)
    addi        r1,r1,64
    mtspr       8,r0
    blr
    .globl      __fini
__fini:
    mfspr       r0,8
    stwu        r1,-64(r1)
    stw     r0,68(r1)

    .section    .eini,2,C
# Return from __fini
    lwz     r0,68(r1)
    addi        r1,r1,64
    mtspr       8,r0
    blr


# MPC860 Init Code
# Do the rest in "C"
    .section    .tfInit860,2,C
    .globl      __tfInit860
    .globl      UpmTable
    .globl      UpmTableEnd
    .align      2

__tfInit860:

#
# Atlas Communications Engines
# DARWIN INITIALIZATION (MPC860 Board)
# You will need to modify four your hardware
# "C" Syntax:
# void init860(unsigned long immrValue);
#

#save IMMR in r4
    mr         r4,r3
    mtspr      IMMR,r3

#------------------------------------------------------------------------
# Disable Data Cache before accesing any more registers
# Save the status of DC_CST in R8 for the flush routine
#------------------------------------------------------------------------
    mfspr      r3,DC_CST
    ori        r8,r3,0      # save DC_CST in r8
    lis        r3,0x0400    # DCache Disable command
    mtspr      DC_CST,r3

#------------------------------------------------------------------------
# Set the decrementer to a very high value 
#------------------------------------------------------------------------
    lis        r3,0x7FFF
    addi       r3,r3,0xFFFF
    mtspr      22,r3
    isync

#------------------------------------------------------------------------
# Initialize Core registers
#------------------------------------------------------------------------

    lis        r3,0x0000    #  MSR and SRR1 = 0x00001002
    ori        r3,r3,0x9002
    mtmsr      r3
    mtspr      SRR1,r3

    lis        r3,0x0       #  DER = 0
    ori        r3,r3,0x0    
    mtspr      DER,r3
 

    lis        r3,0x0000    #  ICTRL = 6
    ori        r3,r3,0x0006
    mtspr      ICTRL,r3

    lis        r3,0x0000    #  ICR = 0
    ori        r3,r3,0x0000
    mtspr      ICR,r3
 
    .if TM_SPEED40=1
#   Set CPU Clock to 40MHZ from 5MHZ osc
        lis        r3,0x0070     
        stw        r3,PLPRCR(r4)  
    .elsec #/* speed not defined  assume 25 */ 
#   Set CPU Clock to 25MHZ from 5MHZ osc
        lis        r3,0x0040     
        stw        r3,PLPRCR(r4)  
    .endc



#------------------------------------------------------------------------
# Initialize SIU registers
#------------------------------------------------------------------------
    lwz        r3,SIUMCR(r4) #  SIUMCR = The old value bitwize
    oris       r3,r3,0x0060  #           ored with 0x00600040  
    ori        r3,r3,0x0040
    stw        r3,SIUMCR(r4)
#
# Watch Dog Timer
#
    lis        r3,0xffff     #  SYPCR = 0xffffff08
    ori        r3,r3,0xff08
    stw        r3,SYPCR(r4)

    li         r3,0x556c
    sth        r3,SWSR(r4)
    
    li         r3,0xAA93
    sth        r3,SWSR(r4)

# Setup Chip Selects
#   CS0 = Boot ROM
    SetCSReg    RomBase+0x0401,BR0
    SetCSReg    0xFFF80D30,OR0

UPMInit: 
    LA  r5,UpmTable
    LA  r6,UpmTableEnd
    sub        r7,r6,r5         # UpmTableEnd - UpmTable
    srawi      r7,r7,2          # /4
        
    li         r6,0x0000     # Command: OP=Write, UPMA, MAD=0
UpmWriteLoop:
    lwz        r3,0(r5)      # get data from table
    stw        r3,MDR(r4)    # store the data to MD register
        
    stw        r6,MCR(r4)    # issue command to MCR register
    addi       r5,r5,4       # next entry in the table
    addi       r6,r6,1       # next MAD address
    cmp        r6,r7         # done yet ?
    blt        UpmWriteLoop

#   CS1 = DRAM Bank 1

    SetCSReg    0xFFC00800,OR1
    SetCSReg    RamBase+0x0081,BR1
# set up refresh for 15.6 
# uses MPTPR & MSB of MAMR
        
    li         r8,0x0200    # Memory Periodic Timer Clk = CPU CLK/32
    sth        r8,MPTPR(r4)

    .if TM_SPEED40=1    
        lis        r8,0x1480        ; set PTA,PTAE
        ori        r8,r8,0x00001334 ; set DSA,G0CLA,RFLA,WLFA,TLFA
    .elsec
        lis        r8,0x0C80        ; set PTA,PTAE
        ori        r8,r8,0x00001334 ; set DSA,G0CLA,RFLA,WLFA,TLFA
    .endc

;*** Initially set for 4 MB
    LA      r5,0xFF8FFFFF
    and     r8,r8,r5                ; clear AMA
    LA      r5,0x00200000
    or        r8,r8,r5              ; Set AMA
    stw        r8,MAMR(r4)          ; set MAMR

    SetCSReg    0xFFC00800,OR1
    SetCSReg    RamBase+0x0081,BR1

;*** Wait at least 200 usec for DRAM to stabilize
    lis        r5,0x0000     # count 0x0379 times ~225nsec
    ori        r5,r5,0x379
    lis    r6,0x0000     # clear r6
delayloop4:
    addi       r6,r6,0x01    # decrement the counter
    cmpw       cr0,r6,r5     # done yet ?       
    bne    delayloop4

;*** Need at least 10 DRAM accesses to stabilize
    lis        r5,0x0000     # count 10 times
    ori        r5,r5,0x0a
    lis        r6,0x0000     # clear r6

    lis        r15,RamBase     # load address
    ori        r15,r15,0x1000
    lis        r16,0x1234     # load data
    ori        r16,r16,0x5678

ramloop:
    stw        r16,0(r15)
    lwz        r16,0(r15)
    addi       r6,r6,0x01    # decrement the counter
    cmpw       cr0,r6,r5     # done yet ?       
    bne    ramloop


#   CS4 = Network Interface Modules

    SetCSReg    0xFFFF0182,OR4
    SetCSReg    0x40000401,BR4

#   CS5 = DRAM ID Register

    SetCSReg    0xFFFF0122,OR5
    SetCSReg    0x50000401,BR5




    mfspr   r8,DC_CST
#
#************************************************************************
# Cache Initialization
#
# Function: CacheInit
#       If the Data cache is enabled, flush then
#       disable it so we can finish the rest of initialization
#
# Input Registers:
#       R8 contains the content of DC_CST before enterring
#          usr_init()
#
# Output Registers:
#       R3, R7, R5 and R6 will be destroyed
#************************************************************************
#
CacheInit:
#------------------------------------------------------------------------
# If Instruction cache was enabled, disable and invalidate all
#------------------------------------------------------------------------
ICacheInit:
ICacheUnlock:
        lis      r3,CacheUnlockAllCmd
        mtspr    IC_CST,r3       # Unlock Icache
        isync
   
ICacheInv:
        lis      r3,CacheInvAllCmd
        mtspr    IC_CST,r3       # Invalidate Icache
        isync

ICacheDisable:
		lis		r3,CacheDisableCmd
		mtspr	IC_CST,r3
		isync

DCacheInit:
DCacheUnlock:
        lis     r3,CacheUnlockAllCmd
        sync
        mtspr   DC_CST,r3       # Unlock Dcache

DCacheInv:
        lis     r3,CacheInvAllCmd
        sync
        mtspr   DC_CST,r3       # Invalidate Dcache


DCacheDisable:
		lis		r3,CacheDisableCmd
        sync
		mtspr	DC_CST,r3


DCacheSetupMMU:

# 
# Setup Memory Management unit
#       

# 
# A few notes about the MMU:  It is important that your entire memory map is
# defined below.  Otherwise, you'll generate a lot of Data TLB Miss interrupts,
# and will cause erratic results.  The reason for using the MMU even though
# your OS doesn't support paging is to 1) be able to specify certain areas of
# RAM (such as those used for DMA) as non-cacheable 2) according to Motorola you
# should use it for performance reasons, as enabling the MMU actually speeds up
# regular memory accesses.  Some of this code is derived from the mmu_lite.c 
# and init860.c example files provided by Motorola.
#

# Disable MMU
		mfmsr	r3
		LA		r5,0x0000ffcf
		and		r3, r3, r5
		mtmsr	r3

#
# Invalidate all of the TLB entries
# 

        tlbia
        isync

# set the MMU instruction control register        
		addi	r3, 0, 0x1F00
        mtspr   MI_CTR, r3        


# set the MMU data control register                     
		oris	r3,r3,0x0400
		ori		r3,r3,0x1F00
        mtspr   MD_CTR, r3

# set access permissions
        oris    r3, 0, 0x5555
        ori     r3, r3, 0x5555
        mtspr   MD_AP, r3   

# 
# Setup DRAM bank 1 (8MB) (data cache enabled)
#

        LA      r3, RamBase
        LA      r5, 0xfffff000
        and     r3, r3, r5
        ori     r3, r3, 0x0200
        mtspr   MD_EPN, r3

## Set effective page number. 8M page size, writethrough policy, unguarded storage
		LA		r3, 0x0000000F
        mtspr   MD_TWC, r3      

# Set real page number. 8M page size, shared page, cache enabled
        LA      r3, RamBase
        LA      r5, 0xfffff000
        and     r3, r3, r5
        ori     r3, r3, 0x09FD
        mtspr   MD_RPN, r3
        isync       


# 
# Setup DRAM bank 1 (8MB) (data cache disabled)
#

        LA      r3, RamBaseUncached
        LA      r5, 0xfffff000
        and     r3, r3, r5
        ori     r3, r3, 0x0200
        mtspr   MD_EPN, r3

## Set effective page number. 8M page size, copy-back policy, guarded storage
		LA		r3, 0x0000001F
        mtspr   MD_TWC, r3      

# Set real page number. 8M page size, shared page, cache enabled
        LA      r3, RamBase
        LA      r5, 0xfffff000
        and     r3, r3, r5
        ori     r3, r3, 0x09FF
        mtspr   MD_RPN, r3
        isync       



#
# Setup IMMR region (data cache disabled)
#
        mfspr   r3, IMMR
        LA      r5, 0xfffff000
        and     r3, r3, r5
        ori     r3, r3, 0x0200
        mtspr   MD_EPN, r3

# Set effective page number. 4K page size, copy-back policy, guarded storage
		LA		r3, 0x00000001
        mtspr   MD_TWC, r3      

# Set real page number. 16K page size, shared page, cache inhibited
        mfspr   r3, IMMR
        LA      r5, 0xfffff000
        and     r3, r3, r5
        ori     r3, r3, 0x09FF
        mtspr   MD_RPN,r3
        isync

#
# Setup instruction area (DRAM)
#

        LA       r3, RamBase
        LA       r5, 0xfffff000
        and      r3, r3, r5
        addi     r3,r3,0x0200
        mtspr    MI_EPN,r3       

        addi     r3,0,0x000D     
        mtspr    MI_TWC,r3

        LA       r3, RamBase
        LA       r5, 0xfffff000
        and      r3, r3, r5
        ori      r3,r3,0x0CFD    
        mtspr    MI_RPN,r3
        isync



# Enable MMU
		mfmsr   r3
		ori		r3,r3,0x0030
		mtmsr	r3



ICacheEnable:
		 addis	r3,0,CacheEnableCmd
         mtspr  IC_CST,r3       # Enable Icache
         sync

DCacheEnable:
  		addis	r3,0,CacheEnableCmd
        mtspr   DC_CST,r3       # Enable Dcache
        sync


# Return from __tfInit860
        blr



    .if TM_SPEED40=1  #/* assumes 60 NS drams */


# /* UPMA contents: */
UpmTable:
# /* single read. (offset 0 in upm RAM) */
         .long    0x0fffcc24, 0x0fffcc04, 0x00ffcc00, 0x71ffcc07
         .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
# /* burst read. (offset 8 in upm RAM) */
         .long    0x0fffcc24, 0x0fffcc04, 0x00ffcc80, 0x03ffcccc
         .long    0x00ffcc00, 0x73ffcc07, 0xffffcc04, 0xfffffc04
         .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
         .long    0xfffffe04, 0xfffffc04, 0xfffffc04, 0xfffffc04
# /* single write. (offset 18 in upm RAM) */
         .long    0x0fafec24, 0x0fafec04, 0x00afec00, 0x73bfec07
         .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
# /* burst write. (offset 20 in upm RAM) */
         .long    0x0fafec24, 0x0fafec04, 0x00afec80, 0x0fafeccc
         .long    0x00afec00, 0x7fbfec07, 0xfffffc04, 0xfffffc04
         .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
         .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
# /* refresh. (offset 30 in upm RAM) */
         .long    0xf0ffec04, 0x00ffec04, 0x01ffec04, 0x3fffec04
         .long    0xffffec04, 0xffffec05, 0xfffffc04, 0xfffffc04
         .long    0xfffffc04, 0xfffffc04, 0xfffffc04, 0xfffffc04
# /* exception. (offset 3c in upm RAM) */
         .long    0xffffec07, 0xfffffc04, 0xfffffc04, 0xfffffc04
#
UpmTableEnd:


    .elsec  # speed not defined 
UpmTable:
# UPM contents for the default 860 memory configuration:

    .if TM_DRAM60NS
  
# /* DRAM 60ns - single read. (offset 0 in upm RAM) */
# /* offsets 6-7 not used */
        .long   0x0FFFCC04, 0x00FFCC00, 0xF1FFCC05, 0xFFFFFC04
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04       

# /* DRAM 60ns - burst read. (offset 8 in upm RAM) */
# /* offsets 16-17 not used */
        .long   0x0FFFEC04, 0x00FFEC08, 0x04FFEC40, 0x02FFCC4C
        .long   0x00FFEC08, 0x04FFEC40, 0x33FFCC07, 0xFFFFFC04
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04

# /* DRAM 60ns - single write. (offset 18 in upm RAM) */
# /* offsets 1d-1f not used */
        .long   0x0FAFEC00,0x11BFEC07, 0xFFFFFC04, 0xFFFFFC04
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04

# /* DRAM 60ns - burst write. (offset 20 in upm RAM) */
# /* offsets 2a-2f not used */
        .long   0x0FAFEC00, 0x01AFEC4C, 0x08AFEC00, 0x01AFEC4C
        .long   0x08AFEC00, 0x01AFEC4C, 0x08AFEC00, 0x11BFEC07
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04
    
# /* refresh 60ns. (offset 30 in upm RAM) */
# /* offsets 37-3b not used */

        .long   0xC0FFEC04, 0x03FFEC04, 0x3FFFEC04, 0xFFFFEC05
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04
        .long   0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04

# /* exception. (offset 3c in upm RAM) */
# /* offset 3d-3f not used */
#
        .long   0xFFFFEC07, 0xFFFFFC04, 0xFFFFFC04, 0xFFFFFC04
    
    .endc

    .if TM_DRAM70NS
    
# /* DRAM 70ns - single read. (offset 0 in upm RAM) */
# /* offsets 6-7 not used */
        .long      0x0fffcc24, 0x0fffcc04, 0x0cffcc04
        .long      0x00ffcc04, 0x00ffcc00, 0x37ffcc47
        .long      0xffffffff, 0xffffffff

# /* DRAM 70ns - burst read. (offset 8 in upm RAM) */
# /* offsets 16-17 not used */
        .long      0x0fffcc24, 0x0fffcc04, 0x08ffcc04, 0x00ffcc04
        .long      0x00ffcc08, 0x0cffcc44, 0x00ffec0c, 0x03ffec00
        .long      0x00ffec44, 0x00ffcc08, 0x0cffcc44, 0x00ffec04
        .long      0x00ffec00, 0x3fffec47
        .long      0xffffffff, 0xffffffff

# /* DRAM 70ns - single write. (offset 18 in upm RAM) */
# /* offsets 1d-1f not used */
        .long      0x0fafcc24, 0x0fafcc04, 0x08afcc04, 0x00afcc00, 0x37ffcc47
        .long      0xffffffff, 0xffffffff, 0xffffffff

# /* DRAM 70ns - burst write. (offset 20 in upm RAM) */
# /* offsets 2a-2f not used */
        .long      0x0fafcc24, 0x0fafcc04, 0x08afcc00, 0x07afcc4c
        .long      0x08afcc00, 0x07afcc4c, 0x08afcc00, 0x07afcc4c
        .long      0x08afcc00, 0x37afcc47
        .long      0xffffffff, 0xffffffff, 0xffffffff
        .long      0xffffffff, 0xffffffff, 0xffffffff

# /* refresh 70ns. (offset 30 in upm RAM) */
# /* offsets 37-3b not used */
# /* exception. (offset 3c in upm RAM) */
# /* offset 3d-3f not used */
#
        .long      0xe0ffcc84, 0x00ffcc04, 0x00ffcc04, 0x0fffcc04
        .long      0x7fffcc04, 0xffffcc86, 0xffffcc05
        .long      0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff
        .long      0x33ffcc07
        .long      0xffffffff, 0xffffffff, 0x40004650

    .endc

UpmTableEnd:
    
    .endc  #/* no speed defined */

# This is a generic interrupt handler.  Any unused entries in the interrupt
# vector table are set to jump here, which is useful for debugging.  It
# especially useful for ensuring that the MMU is setup correctly, because
# if it is not, you will recieve a large number of TLB misses, which will
# jump here to be handled...
GenericIntHandler:
            mfspr   r3, MD_EPN
            mfspr   r3, MD_TWC
            mfspr   r3, MD_RPN
            rfi
