/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/motorola/ppc/ddi/ucosppcc.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:40:52CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/* file: ucosppcc.h */


/*
************************************************************
*                      UCOSPPCC.H
*                 Power PC Specific Code
************************************************************
*                      CONSTANTS
************************************************************
*/


/*
************************************************************
*                        MACROS
************************************************************
*/

#define  OS_TASK_SW()   OSCtxSw()



asm void OS_ENTER_CRITICAL (void)
{
    addi    r1,r1,-8        # save r11, r12
    stw     r12,0(r1)
    stw     r11,-4(r1)

    mfmsr   r12             # get the MSR (Machine Status Register)
    xor     r11,r11,r11     # Clear R11
    ori     r11,r11,0x8000  # set the EE Bit (External Interrupt Enable) in r11
    andc    r12,r12,r11     # and r11 with r12 (Clearing the EE Bit)
    mtmsr   r12             # Save the MSR

    lwz     r11,-4(r1)      # restore r11, r12
    lwz     r12,0(r1)
    addi    r1,r1,8
}

asm void OS_EXIT_CRITICAL (void)
{
    addi    r1,r1,-4          # save r12  
    stw     r12,0(r1)

    mfmsr   r12               # get the MSR (Machine Status Register)
    ori     r12,r12,0x8000    # set the EE Bit in r12
    mtmsr   r12               # save the MSR

    lwz     r12,0(r1)         # restore r12  
    addi    r1,r1,4
}

typedef unsigned char  BOOLEAN;
typedef unsigned char  UBYTE;                    /* Unsigned  8 bit quantity                           */
typedef signed   char  BYTE;                     /* Signed    8 bit quantity                           */
typedef unsigned short UWORD;                    /* Unsigned 16 bit quantity                           */
typedef signed   short WORD;                     /* Signed   16 bit quantity                           */
typedef unsigned long  ULONG;                    /* Unsigned 32 bit quantity                           */
typedef signed   long  LONG;                     /* Signed   32 bit quantity                           */



/*
*********************************************************************************************************
*                                              CONSTANTS
*********************************************************************************************************
*/

#ifndef  FALSE
#define  FALSE    0
#endif

#ifndef  TRUE
#define  TRUE     1
#endif

#define OS_FAR
#define OS_STK_TYPE          UBYTE
#define OS_MAX_TASKS             63    /* Maximum number of tasks in your application                  */
#define OS_MAX_EVENTS            30    /* Maximum number of event control blocks in your application   */
#define OS_MAX_QS                 5    /* Maximum number of queue control blocks in your application   */

#define OS_IDLE_TASK_STK_SIZE  1024    /* Idle task stack size (WORDs)                                 */
#define OS_IDLE_TASK_STK_TOP   1023    /* Index into idle task top of stack                            */

#define OS_TASK_CHANGE_PRIO_EN    1    /* Include code for OSTaskChangePrio()                          */
#define OS_TASK_DEL_EN            1    /* Include code for OSTaskDel()                                 */
#define OS_SEM_EN                 1    /* Include code for SEMAPHORES                                  */
#define OS_MBOX_EN                1    /* Include code for MAILBOXES                                   */
#define OS_Q_EN                   1    /* Include code for QUEUES                                      */
#define OS_TASK_SUSPEND_EN        1    /* Include code for OSTaskSuspend() and OSTaskResume()          */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:45      For: Xilinx-Samsung
 * File Name: ucosppcc.h    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 30124    Output checksum: 62295
 */
