/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * #####DESCRIPTIONBEGIN####
 * 
 *  Author(s):   Scott Coulter, Jeff Frazier, Eric Breeden
 *  Contributors:
 *  Date:        2001-01-25
 *  Purpose:     
 *  Description: 
 * 
 * ####DESCRIPTIONEND####
 * 
 * =========================================================================== */
 * $Source: source/kernel/ucos/xscale/iq80310/gnu/interrupts.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:41:36CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
 * =============================================================================
 * 
 *       interrupts.c - Cyclone Diagnostics
 * 
 * =============================================================================
 * ####COPYRIGHTBEGIN####
 *                                                                           
 *  -------------------------------------------                              
 *  The contents of this file are subject to the Red Hat eCos Public License 
 *  Version 1.1 (the "License"); you may not use this file except in         
 *  compliance with the License.  You may obtain a copy of the License at    
 *  http: * www.redhat.com/                                                   
 *                                                                           
 *  Software distributed under the License is distributed on an "AS IS"      
 *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
 *  License for the specific language governing rights and limitations under 
 *  the License.                                                             
 *                                                                           
 *  The Original Code is eCos - Embedded Configurable Operating System,      
 *  released September 30, 1998.                                             
 *                                                                           
 *  The Initial Developer of the Original Code is Red Hat.                   
 *  Portions created by Red Hat are                                          
 *  Copyright (C) 2001 Red Hat, Inc.                             
 *  All Rights Reserved.                                                     
 *  -------------------------------------------                              
 *                                                                           
 * ####COPYRIGHTEND####
 * =============================================================================
 * #####DESCRIPTIONBEGIN####
 * 
 *  Author(s):   Scott Coulter, Jeff Frazier, Eric Breeden
 *  Contributors:
 *  Date:        2001-01-25
 *  Purpose:     
 *  Description: 
 * 
 * ####DESCRIPTIONEND####
 * 
 * =========================================================================== */

/******************************************************************************/
/* interrupts.c - Interrupt dispatcher routines for IQ80310 Board             */
/*                                                                            */
/* modification history                                                       */
/* --------------------                                                       */
/* 07sep00, ejb, Written for IQ80310 Cygmon diagnostics                       */
/* 11oct00, ejb, Switched FIQ and IRQ interrupt handlers                      */
/* 18dec00  snc and jwf                                                       */
/* 02feb01  jwf for snc                                                       */
/******************************************************************************/

#include "iq80310.h"
#include "pci_bios.h"

#include "includes.h" /* For OSIntEnter(), OSIntExit(). */

extern long _cspr_enable_fiq_int();
extern long _cspr_enable_irq_int();
extern long _scrub_ecc();

extern int nmi_verbose; /* for NMI, only print NMI info if this is TRUE */
extern int pci_config_cycle; /* don't handle NMI if in a config cycle */
extern int pci_config_error;

#define AND_WORD(addr,val)   *addr = *addr & val

/*** External board interrupts (XINT3) ***/
#define TM_VECTOR_TABLE_BASE 0xa000a004

#define CYGNUM_HAL_INTERRUPT_TIMER        27 /* external timer     */
#define CYGNUM_HAL_INTERRUPT_ETHERNET     28 /* onboard enet       */
           

#define CYGNUM_HAL_INTERRUPT_TIMER_VECT \
(TM_VECTOR_TABLE_BASE + (CYGNUM_HAL_INTERRUPT_TIMER << 2))

#define CYGNUM_HAL_INTERRUPT_ETHERNET_VECT \
(TM_VECTOR_TABLE_BASE + (CYGNUM_HAL_INTERRUPT_ETHERNET << 2))


/* Other User Interrupt Service Routines */

void (*usr_timer_isr)(int) = NULL;
int usr_timer_arg = 0;

void (*usr_enet_isr)(int) = NULL;
int usr_enet_arg = 0;

void (*usr_sintb_isr)(int) = NULL;
int usr_sintb_arg = 0;


/********************************************************************************
* iq80310_irq_handler - Interrupt dispatcher for IQ80310 IRQ Interrupts
*
* This function determines the source of the IRQ Interrupt, and calls the 
* corresponding interrupt service routine.  If multiple sources are interrupting
* the dispatcher will call all interrupt handles.  Users must clear the interrupt
* within the interrupt service routine before exiting.
*
* IRQ Interrupts are multiplexed from SPCI INTA - INTD, External Device
* Interrupts, and XINT6 and XINT7 Internal device interrupts.
*/
int iq80310_irq_handler()
{
    UINT8*        int_status_reg;
    UINT8         int_status;
    unsigned int  data;
    unsigned char ri_state;    
    unsigned char board_rev;
    unsigned char sint_status;

/* 12/18/00 jwf */

    OSIntEnter();

/*
 * Right now, we're only interested in INTB from the secondary PCI controller.
 */ 

/* access uart u2 msr reg at addr fe810006 */    
    ri_state = *( unsigned char * ) 0xfe810006; 
    ri_state &= RI_MASK;
    if(ri_state == RI_MASK) /* RI# pin on UART2 is grounded */
    {
        board_rev = *BOARD_REV_REG_ADDR; /* read Board Revision register */
        board_rev &= BOARD_REV_MASK;     /* isolate LSN */
        if (board_rev >= BOARD_REV_E)    /* Board Rev is at E or higher */
        {
/* read current secondary pci interrupt status */            
            sint_status = *SINT_REG_ADDR;
/* isolate SINTA, SINTB, and SINTC */            
            sint_status &= SINT_MASK;                       
            switch(sint_status)
            {
                case SINTB_INT:
                    if (usr_sintb_isr != NULL)
                    {
                        (*usr_sintb_isr)(usr_sintb_arg);
                    }
                    break;                  

                default:
                    break;                                          
            }
        }
    }
    else
    {
/* RI# pin on UART2 is pulled up to 3.3V. Cannot read board revision register,
   not implemented on hardware */
        if (usr_sintb_isr != NULL)
        {
            (*usr_sintb_isr)(usr_sintb_arg);
        }
    }
        
    
/* Read IRQ3 Status Register, and if any of the multiple sources are 
   interrupting, call corresponding handler */
    int_status_reg = (UINT8 *)X3ISR_ADDR;
    int_status = *int_status_reg;
    if (int_status & TIMER_INT) /* timer interrupt? */
    {
/* call user ISR, if connected */
        if (usr_timer_isr != NULL)
        {
            (*usr_timer_isr)(usr_timer_arg);
        }
    }
    else if (int_status & ENET_INT)      /* ethernet interrupt */
    {
/* call user ISR, if connected */
        if (usr_enet_isr != NULL)
        {
            (*usr_enet_isr)(usr_enet_arg);
        }
    }

    OSIntExit();

    return (0);
}




/****************************************************************
*  nmi_ecc_isr - ECC NMI Interrupt Handler
*
*  This module handles the NMI caused by an ECC error.
*  For a Single-bit error it does a read-nodify-write
*  to correct the error in memory. For a multi-bit or
*  nibble error it does absolutely nothing.
*/
void nmi_ecc_isr(void)
{
    UINT32 eccr_register;
    UINT32* reg32;
    
/* Read current state of ECC register */
    eccr_register = *(UINT32 *)ECCR_ADDR;
    
/* Turn off all ecc error reporting */
    *(UINT32 *)ECCR_ADDR = 0x4;
    
/* Check for ECC Error 0 */
    if(*(UINT32 *)MCISR_ADDR & 0x1)
    {
        reg32 = (UINT32*)ELOG0_ADDR;
        
        reg32 = (UINT32*)ECAR0_ADDR;
        
/* Check for single-bit error */
        if(!(*(UINT32 *)ELOG0_ADDR & 0x00000100))
        {
/* call ECC restoration function */
            _scrub_ecc(*reg32);
            
/* Clear the MCISR */
            *(UINT32 *)MCISR_ADDR = 0x1;
        }
    }
    
/* Check for ECC Error 1 */
    if(*(UINT32 *)MCISR_ADDR & 0x2)
    {
        reg32 = (UINT32*)ELOG1_ADDR;
        
        reg32 = (UINT32*)ECAR1_ADDR;
        
/* Check for single-bit error */
        if(!(*(UINT32 *)ELOG1_ADDR & 0x00000100))
        {
/* call ECC restoration function */
            _scrub_ecc(*reg32);
            
/* Clear the MCISR */
            *(UINT32 *)MCISR_ADDR = 0x2;
        }
    }
    
/* Check for ECC Error N */
    if(*(UINT32 *)MCISR_ADDR & 0x4)
    {
/* Clear the MCISR */
        *(UINT32 *)MCISR_ADDR = 0x4;
    }
    
/* Turn on  ecc error reporting */
    *(UINT32 *)ECCR_ADDR = eccr_register;
}



/******************************************************************************
* iq80310_fiq_handler - Interrupt dispatcher for IQ80310 FIQ Interrupts
*
*
*/
int iq80310_fiq_handler()
{
    unsigned long nmi_status;

    nmi_status = *(volatile unsigned long *)NISR_ADDR;
    
    if (nmi_status & MCU_ERROR)
    {
/* call ecc interrupt handler */
        nmi_ecc_isr(); 
        
/* clear the interrupt condition*/
        AND_WORD((volatile unsigned long *)MCISR_ADDR, 0x07);
    }

    
    if (nmi_status & PATU_ERROR)
    {
/* clear the interrupt conditions */
        AND_WORD((volatile unsigned long *)PATUISR_ADDR, 0x79f);
        CLEAR_PATU_STATUS();
        
/* tell the config cleanup code about error */
        if (pci_config_cycle == 1) 
            pci_config_error = TRUE;
    }


    if (nmi_status & SATU_ERROR)
    {
/* clear the interrupt conditions */
        AND_WORD((volatile unsigned long *)SATUISR_ADDR, 0x69f);
        CLEAR_SATU_STATUS();

/* tell the config cleanup code about error */
        if (pci_config_cycle == 1) 
            pci_config_error = TRUE;
    }

    if (nmi_status & PBRIDGE_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)PBISR_ADDR, 0x3f);
        CLEAR_PBRIDGE_STATUS();
        
/* tell the config cleanup code about error */
        if (pci_config_cycle == 1) 
            pci_config_error = TRUE;
    }

    if (nmi_status & SBRIDGE_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)SBISR_ADDR, 0x7f);
        CLEAR_SBRIDGE_STATUS();
        
/* tell the config cleanup code about error */
        if (pci_config_cycle == 1) 
            pci_config_error = TRUE;
    }

    if (nmi_status & DMA_0_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)CSR0_ADDR, 0x2D);
    }
    
    if (nmi_status & DMA_1_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)CSR1_ADDR, 0x2D);
    }

    if (nmi_status & DMA_2_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)CSR2_ADDR, 0x2D);
    }
    
    if (nmi_status & MU_ERROR)
    {       
/* clear the interrupt condition; note that the clearing of the NMI doorbell
   is handled by the PCI comms code */
        AND_WORD((volatile unsigned long *)IISR_ADDR, 0x20);    
    }

    if (nmi_status & AAU_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)ASR_ADDR, 0x20);
    }
    
    if (nmi_status & BIU_ERROR)
    {
/* clear the interrupt condition */
        AND_WORD((volatile unsigned long *)BIUISR_ADDR, 0x04);
    }
    
    return 0;
}

#ifdef TM_XSCALE_REDBOOT
int redboot_isr_handler(unsigned vector, unsigned data)
{
    switch (vector)
    {
        case CYGNUM_HAL_INTERRUPT_TIMER:
            if (usr_timer_isr != NULL)
            {
                (*usr_timer_isr)(data);
            }
            break;

        case CYGNUM_HAL_INTERRUPT_ETHERNET:
            if (usr_enet_isr != NULL)
            {
                (*usr_enet_isr)(data);
            }
            break;
        default:
            break;
    }
}
#endif /* TM_XSCALE_REDBOOT */

/**********************************************************************
* isr_connect - Connect a user Interrupt Service Routine
*/
int isr_connect(int int_num, void (*handler)(int), int arg)
{
    unsigned int * vectPtr;
    
    switch (int_num)
    {
        case TIMER_INT_ID:
#ifdef TM_XSCALE_REDBOOT
            vectPtr = (unsigned int *) CYGNUM_HAL_INTERRUPT_TIMER_VECT;
            *vectPtr = (unsigned int) redboot_isr_handler;
#endif /* TM_XSCALE_REDBOOT */
            usr_timer_isr = handler;
            usr_timer_arg = arg;
            break;
        case ENET_INT_ID:
#ifdef TM_XSCALE_REDBOOT
            vectPtr = (unsigned int *) CYGNUM_HAL_INTERRUPT_ETHERNET_VECT;
            *vectPtr = (unsigned int) redboot_isr_handler;
#endif /* TM_XSCALE_REDBOOT */
            usr_enet_isr = handler;
            usr_enet_arg = arg;
            break;
        case SINTB_INT_ID:
            usr_sintb_isr = handler;
            usr_sintb_arg = arg;
        default:
            return (ERROR);
            break;
    }

    return (OK);
}

/**********************************************************************
* isr_disconnect - Disconnect a user Interrupt Service Routine
*
* NOT TO BE USED FOR SPCI INTERRUPTS! - use pci_isr_disconnect instead
*
*/
int isr_disconnect(int int_num)
{
    switch (int_num)
    {
        case TIMER_INT_ID:
                usr_timer_isr = NULL;
                usr_timer_arg = 0;
                break;
        case ENET_INT_ID:
                usr_enet_isr = NULL;
                usr_enet_arg = 0;
                break;
        case SINTB_INT_ID:
            usr_sintb_isr = NULL;
            usr_sintb_arg = 0;
        default:
                return (ERROR);
                break;
    }

    return (OK);
}

/********************************************************************
* disable_external_interrupt - Mask an external interrupt 
*
*/
int disable_external_interrupt(int int_id)
{

    unsigned char* ext_mask_reg = (unsigned char*) X3MASK_ADDR;
    unsigned char  new_mask_value;

/* make sure interrupt to enable is an external interrupt */
    if ((int_id < TIMER_INT_ID) || (int_id > SINTD_INT_ID))
        return (ERROR);
    
    new_mask_value = *ext_mask_reg; /* read current mask status */
    
    switch (int_id)
    {
        case TIMER_INT_ID:
            new_mask_value |= TIMER_INT;
            break;
        case ENET_INT_ID:
            new_mask_value |= ENET_INT;
            break;
        default:
            break; /* leave mask register as it was */
    }
    
    *ext_mask_reg = new_mask_value; /* set new mask value */
    
    return (OK);
    
}



/********************************************************************
* enable_external_interrupt - Unmask an external interrupt 
*
*/
int enable_external_interrupt(int int_id)
{
    
    unsigned char* ext_mask_reg = (unsigned char*) X3MASK_ADDR;
    unsigned char  new_mask_value;

/* make sure interrupt to enable is an external interrupt */
    if ((int_id < TIMER_INT_ID) || (int_id > SINTD_INT_ID))
        return (ERROR);
    
    
    new_mask_value = *ext_mask_reg; /* read current mask status */
    
    switch (int_id)
    {
        case TIMER_INT_ID:
            new_mask_value &= ~(TIMER_INT);
            break;
        case ENET_INT_ID:
            new_mask_value &= ~(ENET_INT);
            break;
        default:
            break; /* leave mask register as it was */
    }
    
    *ext_mask_reg = new_mask_value; /* set new mask value */
    
    return (OK);
}


void config_ints()
{
    unsigned int* pirsr_ptr;

    pirsr_ptr = (unsigned int*) PIRSR_ADDR;
    
    *pirsr_ptr = 0xf; /* this is an errata in the original Yavapai manual.
                         The interrupt steering bits are reversed, so a '1'
                         routes XINT interrupts to FIQ */

/* make sure interrupts are enabled in CSPR */

    _cspr_enable_irq_int(); 
    
    _cspr_enable_fiq_int(); 
}





/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: interrupts.c  File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6020    Output checksum: 13013
 */
