/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Author: odile $
 * Description:
 *      uC/OS-II V2.03 LH79531 Processor Specific 'C' Code
 *
 * Revision History:
 * $Log: source/kernel/ucos/xscale/iq80310/gnu/os_cpu_c.c  $
 * Revision 6.0.2.2 2010/01/18 06:41:45CST odile 
 * 6.0.2.x
 * Revision 6.0.2.1 2010/01/18 02:39:50PST odile 
 * Duplicate revision
 * Revision 5.0.1.1 2008/05/26 15:11:56PDT odile 
 * Duplicate revision
 * Revision 4.7.1.1 2007/03/11 21:31:01PST odile 
 * Duplicate revision
 * Revision 4.5.1.1 2006/12/02 20:22:39PST odile 
 * Duplicate revision
 * Revision 4.2.1.1 2006/02/05 18:39:08PST odile 
 * Duplicate revision
 * Revision 4.1.1.1 2004/11/07 21:08:10PST xwang 
 * Duplicate revision
 * Revision 4.0.1.1 2004/02/27 17:31:46PST odile 
 * Duplicate revision
 * Revision 3.0.1.6 2004/02/02 17:55:24GMT-08:00 xwang 
 * Update copyright information for year '04
 * Revision 3.0.1.5 2003/10/22 07:01:48GMT-08:00 xwang 
 * Undo global change of const to TM_CONST
 * Revision 3.0.1.3 2003/07/18 12:17:34PDT xwang 
 * Standardised/Added file header for TCP/IP
 * Revision 3.0.1.2 2003/07/11 15:04:30PDT xwang 
 * Adding the project tag to each file
 * Revision 3.0.1.1 2003/03/14 11:56:49PST jason 
 * Duplicate revision
 * 
 *    Rev 1.0   Sep 08 2000 08:31:56   BarnettH
 * Initial revision.
 * 
 * Notes:
 *              (1) This file defines the global 'swap_context_flag'
 *
 * COPYRIGHT (C) 2000 SHARP MICROELECTRONICS OF THE AMERICAS, INC.
 *                     CAMAS, WA 
 *********************************************************************/
 * $Source: source/kernel/ucos/xscale/iq80310/gnu/os_cpu_c.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:41:45CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/**********************************************************************
 * $Workfile:   Os_cpu_c.c  $
 * $Revision: 6.0.2.2 $
 * $Author: odile $
 * $Date: 2010/01/18 06:41:45CST $
 *
 * Project: LH79531 EVB
 *
 * Description:
 *      uC/OS-II V2.03 LH79531 Processor Specific 'C' Code
 *
 * Revision History:
 * $Log: source/kernel/ucos/xscale/iq80310/gnu/os_cpu_c.c  $
 * Revision 6.0.2.2 2010/01/18 06:41:45CST odile 
 * 6.0.2.x
 * Revision 6.0.2.1 2010/01/18 02:39:50PST odile 
 * Duplicate revision
 * Revision 5.0.1.1 2008/05/26 15:11:56PDT odile 
 * Duplicate revision
 * Revision 4.7.1.1 2007/03/11 21:31:01PST odile 
 * Duplicate revision
 * Revision 4.5.1.1 2006/12/02 20:22:39PST odile 
 * Duplicate revision
 * Revision 4.2.1.1 2006/02/05 18:39:08PST odile 
 * Duplicate revision
 * Revision 4.1.1.1 2004/11/07 21:08:10PST xwang 
 * Duplicate revision
 * Revision 4.0.1.1 2004/02/27 17:31:46PST odile 
 * Duplicate revision
 * Revision 3.0.1.6 2004/02/02 17:55:24GMT-08:00 xwang 
 * Update copyright information for year '04
 * Revision 3.0.1.5 2003/10/22 07:01:48GMT-08:00 xwang 
 * Undo global change of const to TM_CONST
 * Revision 3.0.1.3 2003/07/18 12:17:34PDT xwang 
 * Standardised/Added file header for TCP/IP
 * Revision 3.0.1.2 2003/07/11 15:04:30PDT xwang 
 * Adding the project tag to each file
 * Revision 3.0.1.1 2003/03/14 11:56:49PST jason 
 * Duplicate revision
 * 
 *    Rev 1.0   Sep 08 2000 08:31:56   BarnettH
 * Initial revision.
 * 
 * Notes:
 *              (1) This file defines the global 'swap_context_flag'
 *
 * COPYRIGHT (C) 2000 SHARP MICROELECTRONICS OF THE AMERICAS, INC.
 *                     CAMAS, WA 
 *********************************************************************/

#define  OS_CPU_GLOBALS
#include "includes.h"
#include "iq80310.h"

#include "led.h"


/***********************************************************************
 *
 * Function: OSTaskStkInit
 *
 * Purpose:
 *      This function is called by either OSTaskCreate() or
 *      OSTaskCreateExt() to initialize the stack frame of the task
 *      being created.
 *
 * Processing:
 *      Initialize stack pointer
 *      Initialize first stack frame
 *      Return stack pointer
 *
 * Parameters:
 *      void (* task)(void *pd) - is a function pointer to the task code.
 *
 *      void * pdata - is a pointer to a user supplied data area that will
 *                      be passed to the task when the task first executes.
 *                      
 *      OS_STK * ptos - is a pointer to the top of stack for the task.
 *                      In this LH79531 Full Descending stack implementation,
 *                      OS_STK_GROWTH is set to 1, so 'ptos' contains the HIGHEST
 *                      valid address of the stack.
 *
 *      INT16U opt - specifies options that can be used to alter the
 *                      behavior of OSTaskStkInit().
 *                      (see uCOS_II.H for OS_TASK_OPT_???).
 *
 * Outputs:  None
 *
 * Returns:  current stack pointer of task after initialization
 *
 * Notes:
 *      This function is highly processor specific.
 *      Interrupts are enabled when this task starts executing.
 *
 **********************************************************************/

OS_STK *OSTaskStkInit (void (*task)(void *pd),
                                                void *pdata,
                                                OS_STK *ptos,
                                                INT16U opt)
{
    OS_STK * stk ;

    stk = ptos; /* Init stack pointer */

    /* build a context for the new task */
    *--stk = (unsigned int) task;       /* lr */
    *--stk = 0;                         /* r12 */
    *--stk = 0;                         /* r11 */
    *--stk = 0;                         /* r10 */
    *--stk = 0;                         /* r9 */
    *--stk = 0;                         /* r8 */
    *--stk = 0;                         /* r7 */
    *--stk = 0;                         /* r6 */
    *--stk = 0;                         /* r5 */
    *--stk = 0;                         /* r4 */
    *--stk = 0;                         /* r3 */
    *--stk = 0;                         /* r2 */
    *--stk = 0;                         /* r1 */
    *--stk = (unsigned int) pdata;      /* r0 */
    *--stk = SVC32MODE;                 /* spsr */
    *--stk = SVC32MODE;                 /* cpsr */

    return ((OS_STK *)stk);
}

/***********************************************************************
 *
 * Function: OSIntCtxSw
 *
 * Purpose:
 *      To set a flag indicating to IRQ Handler that a context change is 
 *      needed at the Interrupt Level.
 *
 * Processing:
 *      Initialize stack pointer
 *      Initialize first stack frame
 *      Return stack pointer
 *
 * Parameters: void
 *
 * Outputs: swap_context_flag global set to 1
 *
 * Returns:  void
 *
 * Notes:
 *      swap_context_flag global is only used by OSIntCtxSw ()
 *      and the IRQ Handler. The IRQ Handler resets the flag to 0 
 *
 **********************************************************************
 */
 
unsigned int swap_context_flag = 0 ;

void OSIntCtxSw(void)
{
    swap_context_flag = 1;
}

#if OS_CPU_HOOKS_EN
/*
 **************************************************************************
 *                                          TASK CREATION HOOK
 *
 * Description: This function is called when a task is created.
 *
 * Arguments  : ptcb is a pointer to the task control block of the task
 *                   being created.
 *
 * Note(s)    : 1) Interrupts are disabled during this call.
 **************************************************************************
 */
void OSTaskCreateHook (OS_TCB *ptcb)
{
}

/*
 **************************************************************************
 *                                           TASK DELETION HOOK
 *
 * Description: This function is called when a task is deleted.
 *
 * Arguments  : ptcb is a pointer to the task control block of the task
 *                   being deleted.
 *
 * Note(s)    : 1) Interrupts are disabled during this call.
 **************************************************************************
 */
void OSTaskDelHook (OS_TCB *ptcb)
{
}

/*
 **************************************************************************
 *                                           TASK SWITCH HOOK
 *
 * Description: This function is called when a task switch is performed.
 *              This allows other operations to be performed operations
 *              during a context switch.
 *
 * Arguments  : none
 *
 * Note(s)    : 1) Interrupts are disabled during this call.
 *              2) It is assumed that the global pointer 'OSTCBHighRdy'
 *                 points to the TCB of the task that will be 'switched in'
 *                 (i.e. the highest priority task) and, 'OSTCBCur' points
 *                 to the task being switched out (i.e. the preempted task).
 **************************************************************************
 */
void OSTaskSwHook (void)
{
}

/*
 **************************************************************************
 *                                           STATISTIC TASK HOOK
 *
 * Description: This function is called every second by uC/OS-II's statistics
 *              task.  This allows your application to add functionality to
 *              the statistics task.
 *
 * Arguments  : none
 **************************************************************************
 */
void OSTaskStatHook (void)
{
}

/*
 **************************************************************************
 *                                               TICK HOOK
 *
 * Description: This function is called every tick.
 *
 * Arguments  : none
 *
 * Note(s)    : 1) Interrupts may or may not be ENABLED during this call.
 ***************************************************************************
 */
void OSTimeTickHook (void)
{
}

#endif /* OS_CPU_HOOKS_EN */ 

void _enableFiqIrq()
{
    asm ("mrc p15, 0, r0, c13, c0, 1;"
         "orr r0, r0, #0x2000;"
         "mrc p15, 0, r0, c13, c0, 1;"
         "mrc p13, 0, r0, c0, c0, 0;"
         "orr           r0, r0, #3;"
         "mcr   p13, 0, r0, c0, c0, 0;"
         : : );
}



void _scrub_ecc(unsigned p)
{
    asm volatile ("ldrb r4, [%0]\n"
                  "strb r4, [%0]\n" : : "r"(p) );
}




/**********************************************************************
*
* Function: Install_IRQ_Handle()
*
* Purpose:
*       To install the handler for ARM interrupt vector 0x18
*
* Processing:
*       Installs IRQ Handler entry address in Exception jump table for
*        "Load pc" method of branching to IRQ Handler.
*       Calls Install_Handler () with pointer to IRQ vector 0x18
*               and pointer to jump table address
*
* Parameters: 
*    unsigned int, location of IRQ Handler entry point
*
* Outputs:  None.
*
* Returns:      unsigned int, old contents of IRQ vector
*
* Notes:
*    ARM Architecture specific
*
**********************************************************************/

unsigned int Install_IRQ_Handle(unsigned int IRQHandler)
{
    unsigned int * irqvec = (unsigned int *) IRQ_VEC;
    unsigned int * irqaddr = (unsigned int *) IRQ_ADDR;

    *irqaddr = IRQHandler;
    return (Install_Handler_iq80310 ((unsigned int) irqaddr, irqvec));
}



unsigned int Install_FIQ_Handle(unsigned int FIQHandler)
{
    unsigned int * irqvec = (unsigned int *) FIQ_VEC;
    unsigned int * irqaddr = (unsigned int *) FIQ_ADDR;

    *irqaddr = FIQHandler;
    return (Install_Handler_iq80310 ((unsigned int) irqaddr, irqvec));
}


/**********************************************************************
*
* Function: Install_Handler ()
*
* Author: barnetth
*
* Purpose:
*       To install the handler for generic ARM interrupt vector using
*       the "Load pc" method
*
* Processing:
*       Installs IRQ Handler entry address in Exception jump table for
*        "Load pc" method of branching to IRQ Handler.
*       Calls Install_Handler () with pointer to IRQ vector 0x18
*               and pointer to jump table address
*
* Parameters: 
*    unsigned int containing location of Handler entry point
*    unsigned int pointer containing location of vector
*
* Outputs:  None.
*
* Returns:      unsigned int, old contents of vector
*
* Notes: "0xe59ff000" is machine language for "LDR PC,#0x000" 
*    When or'd with address within 4k, it results in the PC
*    being loaded with the entry point of the handler when exception
*    associated with "vector" is invoked, provided entry point of
*    handler has been stored at address "location".
*
*    ARM Architecture specific.
*
*
**********************************************************************/

int Install_Handler_iq80310 (unsigned int location, unsigned int * vector)
{
    unsigned int vec, oldvec;
    vec = ((unsigned int) location -
           (unsigned int)vector - 0x08) | 0xe59ff000;
    oldvec = *vector;
    *vector = vec;
    _cspr_enable_irq_int(); 
    return ( oldvec );

}

/* interrupt handler for the PAL-based external timer */
void ext_timer_handler (int arg)
{
/* increment tick counter */
    OSTimeTick(); 

/* to clear the timer interrupt, clear the timer interrupt
   enable, then re-set the int. enable bit */
    EXT_TIMER_INT_DISAB();
    EXT_TIMER_INT_ENAB();

    return;
}


void init_external_timer( void )
{
/* disable timer in case it was running */
    EXT_TIMER_INT_DISAB();
    EXT_TIMER_CNT_DISAB();

    write_timer_count ( EXT_TIMER_CLK_FREQ / OS_TICKS_PER_SEC);
/* connect the timer ISR */
    isr_connect (TIMER_INT_ID, ext_timer_handler, 0);

/* enable the external interrupt */
    enable_external_interrupt(TIMER_INT_ID);
/* enable timer count and interrupt */
    EXT_TIMER_INT_ENAB();
    EXT_TIMER_CNT_ENAB();
}

/* timer count must be written 8 bits at a time */
int write_timer_count (UINT32 count)
{
    UINT8 cnt_word;

/* first ensure that there are only 22 bits of count data */
    count &= 0x003fffff;

/* grab least significant 8 bits of timer value */
    cnt_word = (UINT8)(count & 0xff);
    *TIMER_LA0_REG_ADDR = cnt_word;

/* grab next 8 bits of timer value */
    count = (count >> 8);
    cnt_word = (UINT8)(count & 0xff);
    *TIMER_LA1_REG_ADDR = cnt_word;

/* grab last 6 bits of timer value */
    count = (count >> 8);
    cnt_word = (UINT8)(count & 0x3f);
    *TIMER_LA2_REG_ADDR = cnt_word;
    
    return;
}





        

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: os_cpu_c.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 47520    Output checksum: 25911
 */
