/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/xscale/iq80310/gnu/test1.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:41:56CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
************************************************************
*                         TEST1.C
*                      uC/OS-II EXAMPLE
************************************************************
*/
#include "INCLUDES.H"
#include "trsocket.h"
#include "trmacro.h"
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <tr8255x.h>
#include "led.h"

/*
 * Test Type:
 *
 * 1 - Small UDP zero copy send
 * 2 - Large UDP zero copy send
 * 3 - TCP zero copy send
 * 4 - TCP zero copy echo
 * 5 - Ping (send)
 * 9 - Idle (possibly to receive and respond to pings)
 */
#define TEST_TYPE 3

#define LOCAL_IP_ADDR "63.144.81.55"
#define LOCAL_NETMASK "255.255.255.0"
#define DEFAULT_GW_ADDR "63.144.81.32"

#define DEST_IP_ADDR "63.144.81.35"
#define DEST_PORT 9


#define STK_SIZE 5120

OS_TCB  OSTCBTbl[OS_MAX_TASKS];
UWORD   OSIdleTaskStk[STK_SIZE+20];
UWORD   Stk1[STK_SIZE+20];
UWORD   Stk2[STK_SIZE+20];
UWORD   Stk9[STK_SIZE+20];
char    Data1;
char    Data2;
char    Data9;

/* uC/OS tasks */
void treckTask(void *data);
void timerTask(void *data);
void recvTask(void *interfaceHandle);

/* Interrupt handlers */
extern void IQ80310_IRQHandler();
extern void IQ80310_FIQHandler();
extern unsigned int Install_FIQ_Handle(unsigned int FIQHandler);
extern unsigned int Install_IRQ_Handle(unsigned int IRQHandler);    
extern void init_external_timer( void );

/*
 * Test Driver
 */
void pingCallback(int sockDesc);
int sendtoNoCopy(struct sockaddr_in * toAddress, int size);
int sendNoCopy(struct sockaddr_in * toAddress, 
               int                  size, 
               int                  noDelayFlag, 
               int                  echoFlag);
int recvfromNoCopy(int size);


/* skip exception handling when performing pci config cycle */ 
int pci_config_cycle = 0;

int main(void)
{
    Data1 = '1';
    Data2 = '2';
    Data9 = '9';

   HAL_DCACHE_SYNC();
   HAL_DCACHE_DISABLE();

   Install_FIQ_Handle( (unsigned int) IQ80310_FIQHandler); 
   Install_IRQ_Handle( (unsigned int) IQ80310_IRQHandler);
   
   OSInit();
   OSTaskCreate(treckTask, (void *)&Data9, (void *)&Stk9[STK_SIZE], 9);

   OSStart(); /* Never returns... */

   return 0;
}

volatile int testType;

void treckTask(void *data)
{
    int                errorCode;
    int                testSize;
    ttUserIpAddress    ipAddress;
    ttUserIpAddress    netMask;
    ttUserLinkLayer    linkLayerHandle;
    ttUserInterface    interfaceHandle;
    struct sockaddr_in toAddress;

    tfInitTreck();
/* Start Treck */
    tfStartTreck();

/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();

    init_external_timer();   

    interfaceHandle=tfAddInterface(
            "82559",
            linkLayerHandle,
            tf8255xOpen,
            tf8255xClose,
            tf8255xSend,
            tf8255xReceive,
            0,
            tf8255xIoctl,
            tf8255xGetPhysAddr,
            &errorCode);

/* Start the tasks */
    OSTaskCreate(timerTask,
                 (void *)&interfaceHandle,
                 (void *)&Stk1[STK_SIZE],1);

    OSTaskCreate(recvTask,
                 (void *)&interfaceHandle,
                 (void *)&Stk2[STK_SIZE], 2);
    
    ipAddress = inet_addr(LOCAL_IP_ADDR);
    netMask   = inet_addr(LOCAL_NETMASK);
/* Config the Interface */    
    errorCode = tfConfigInterface(interfaceHandle,
                                  ipAddress,
                                  netMask,
                                  TM_DEV_SCATTER_SEND_ENB,  
                                  5,  
                                  0);

    if (errorCode != TM_ENOERROR)
    {
        printf("Failed to open Ethernet interface: %d\n", errorCode);
    }

/* Destination address for the tests below */
    toAddress.sin_family      = PF_INET;
    toAddress.sin_addr.s_addr = inet_addr(DEST_IP_ADDR);
    toAddress.sin_port        = htons(DEST_PORT);

    testType = TEST_TYPE;
    switch (testType)
    {
/* Test #1 - small UDP zero copy send */        
        case 1:
            testSize = 16;
            sendtoNoCopy(&toAddress, testSize);
            break;

/* Test #2 - large UDP zero copy send */
        case 2:
            testSize = 1472;
            sendtoNoCopy(&toAddress, testSize);
            break;
        
/* Test #3 - TCP zero copy send */      
        case 3:
            testSize = 1448;
            sendNoCopy(&toAddress, testSize, 0, 0);
            break;

/* Test #4 - TCP zero copy echo */      
        case 4:
            testSize = 512;
            toAddress.sin_port = htons(7);
            sendNoCopy(&toAddress, testSize, 0, 1);
            break;

/* Test #5 - Ping */
        case 5:    
            tfPingOpenStart ("10.2.2.160",
                             0,
                             100,
                             pingCallback );
/* Fall through and just wait for replies */
   
        case 9:
            while (1)
            {
                OSTimeDly(100);
            }
    }       

    while (1)
    {
        OSTimeDly(100);
    }
}

int udpRecvCount;

/*
 * Zero copy UDP receive
 */
int recvfromNoCopy(int size)
{
    char *         testPtr;
    int                errorCode;
    ttUserMessage      bufferHandle;
    int            udpSd;
    struct sockaddr_in bindAddr;

    udpSd = socket(PF_INET, SOCK_DGRAM, 0);

    bindAddr.sin_family      = PF_INET;
    bindAddr.sin_addr.s_addr = 0UL;
    bindAddr.sin_port        = htons(9);
    
    errorCode = bind(udpSd,
                     (struct sockaddr *) &bindAddr,
                     sizeof(bindAddr));
    
    udpRecvCount = 0;
    while (1) 
    {
        errorCode = tfZeroCopyRecvFrom(udpSd,
                                       &bufferHandle,
                                       &testPtr,
                                       size,
                                       0,
                                       0,
                                       0);
    
        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(udpSd);
            if (errorCode != TM_EWOULDBLOCK)
            {
                printf("tfZeroCopyRecvFrom error = %d\n", errorCode);
                break;
            }
        }
        else
        {
            tfFreeZeroCopyBuffer(bufferHandle);
        }

        udpRecvCount++;

    }
    
    return errorCode;
}    
    
int udpSendCount;

/*
 * Zero copy UDP send
 */
int sendtoNoCopy(struct sockaddr_in *toAddress, int size)
{
    char *         testPtr;
    int            errorCode;
    ttUserMessage  bufferHandle;
    int            udpSd;

    udpSd = socket(PF_INET, SOCK_DGRAM, 0);
    
    udpSendCount = 0;
    while (1) 
    {
        bufferHandle=tfGetZeroCopyBuffer(size, &testPtr);

        errorCode = tfZeroCopySendTo(udpSd,
                                     bufferHandle,
                                     size,
                                     0,
                                     (const struct sockaddr TM_FAR *) toAddress,
                                     (int) sizeof(struct sockaddr) ); 

        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(udpSd);
            if (errorCode != TM_EWOULDBLOCK)
            {
                printf("tfZeroCopySendTo error = %d\n", errorCode);
                break;
            }
        }

        udpSendCount++;

    }
    
    return errorCode;
}

int tcpSendCount;

char sendBuffer[1000];

/*
 * Zero copy TCP send
 */
int sendNoCopy(struct sockaddr_in * toAddress, 
               int                  size, 
               int                  noDelayFlag, 
               int                  echoFlag)
{
    char *         testPtr;
    int            errorCode;
    ttUserMessage  bufferHandle;
    int            tcpSd;
    int            optParam;
    
    tcpSd = socket(PF_INET, SOCK_STREAM, 0);

    if (noDelayFlag)
    {
    optParam = 1;
    errorCode = setsockopt(tcpSd,
                           IP_PROTOTCP,
                           TCP_NODELAY,
                           (const char *) &optParam,
                           sizeof(optParam));
    
    if (errorCode != TM_ENOERROR)
    {
        goto sendNoCopyFinish;
    }
    }


    errorCode = connect(tcpSd,
                        (struct sockaddr *) toAddress,
                        sizeof(struct sockaddr_in));

    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(tcpSd);
        printf("connect error = %d\n", errorCode);  
        goto sendNoCopyFinish;
    }
    
    tcpSendCount = 0;
    testPtr = sendBuffer;
    while (1) 
    {

        bufferHandle=tfGetZeroCopyBuffer(size, &testPtr);

        errorCode = tfZeroCopySend(tcpSd,
                                   bufferHandle,
                                   size,
                                   0);

        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(tcpSd);
            printf("tfZeroCopySend error = %d\n", errorCode);
            goto sendNoCopyFinish;
        }

        if (echoFlag)
        {

            errorCode = tfZeroCopyRecv(tcpSd,
                                       &bufferHandle,
                                       &testPtr,
                                       size,
                                       0);
        
            if (errorCode == TM_SOCKET_ERROR)
            {       
                errorCode = tfGetSocketError(tcpSd);
                printf("tfZeroCopyRecv error = %d\n", errorCode);
                goto sendNoCopyFinish;
            }
            else
            {
                if (bufferHandle != (ttUserMessage) 0)
                {
                    tfFreeZeroCopyBuffer(bufferHandle);
                }

            }
        
        }

        tcpSendCount++;
    }

sendNoCopyFinish:    
    return errorCode;
}    

/*
 * Called by the stack when a PING reply is received.
 */ 
void pingCallback(int sockDesc)
{
    printf("Ping reply\n");
}

/*
 * Treck timer task
 */ 
void timerTask(void *interfaceHandlePtr)
{
    ttUserInterface interfaceHandle;
    int ticks;
    int count;
    int i;

    interfaceHandle = *((ttUserInterface*) interfaceHandlePtr);

    ticks = 0;
    count = 0;
    while (1) 
    {
        OSTimeDly(1);

        tfTimerUpdate();
        tfTimerExecute();

        (void)tfIoctlInterface(
            interfaceHandle,
            TM_8255X_IOCTL_REFILL|TM_8255X_IOCTL_SEND_COMPLETE,
            (void TM_FAR *)0, 0);

/* Periodically, increment the number on the LED display, to verify that the
   timer task is still running. */
        if (ticks == 200)
        {
            ticks = 0;
            displayLED( leds[1], leds[count] );
            if (count++ > 4)
            {
                count = 0;
            }
        }

    }

}

/*
 * Ethernet driver receive task
 */
void recvTask(void *interfaceHandle)
{
    while (1) 
    {
        (void)tfWaitReceiveInterface(*(ttUserInterface *)interfaceHandle);
        (void)tfRecvInterface(*(ttUserInterface *)interfaceHandle);
    }
}

/*
 * Subroutine accepts bit patterns as defined in led.h and posts to
 * board registers resulting in display of value to board leds
 */ 
int displayLED(unsigned int display_left, unsigned int display_right)
{
    unsigned int *left;
    unsigned int *right;
    
    left        = DISPLAY_LEFT;  /* Load register address for left LED     */
    right       = DISPLAY_RIGHT; /* Load register address for right LED    */
    *left       = display_left;  /* Load bit pattern to register left LED  */
    *right      = display_right; /* Load bit pattern to register right LED */
                                                                        
    return 1;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: test1.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 12831    Output checksum: 63892
 */
