/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: uC/OS-II V2.03 LH79531 Processor Specific 'C' Code
 *
 * Filename:        trsocket.h
 * Author:          Jin Shang
 * Date Created:    03/05/2003
 * $Source: source/kernel/ucos/xscale/iq80321/gnu/Os_cpu_c.c $
 *
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:01CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 *
 * COPYRIGHT (C) 2000 SHARP MICROELECTRONICS OF THE AMERICAS, INC.
 * 		       CAMAS, WA 
 *    Rev 1.0   Sep 08 2000 08:31:56   BarnettH
 *
 * Description:
 * 	uC/OS-II V2.03 LH79531 Processor Specific 'C' Code
 * 
 * Notes:
 * 		(1) This file defines the global 'swap_context_flag'
 *
 * Initial revision.
 *********************************************************************/

#define  OS_CPU_GLOBALS
#include "includes.h"
#include "trsocket.h"
#include "trmacro.h"
#include "trtype.h"
#include "trglobal.h"
#include "led.h"

/* ISR init */
extern void      IQ80321_IRQHandler();
extern void      IQ80321_FIQHandler();
void (*usr_timer_isr)(int) = NULL;
int usr_timer_arg = 0;
void (*usr_enet_isr)(int) = NULL;
ttUser32Bit usr_enet_arg = 0;

extern INT32U tvInterruptMask;

/* in order to test without UCOS */
/*#define TM_TEST_NOOS*/
/**********************************************************************************
* iq80321_irq_handler - Interrupt dispatcher for IQ80321 IRQ Interrupts
*
* This function determines the source of the IRQ Interrupt, and calls the 
* corresponding interrupt service routine.  If multiple sources are interrupting
* the dispatcher will call all interrupt handles.  Users must clear the interrupt
* within the interrupt service routine before exiting.
*
* IRQ Interrupts are multiplexed from SPCI INTA - INTD, External Device Interrupts,
* and XINT6 and XINT7 Internal device interrupts.
*/
int iq80321_irq_handler()
{
    INT32U      mask;
    INT32U      sources;
    
    OSIntEnter();
    if ( OSIntNesting >1 )
    {
/* TBD , in our current porting, disable net-ISR */
        mask = mask ;
    }


    INTCTL_READ(mask); 
    IINTSRC_READ(sources);

    sources &= mask; 
    
    if (sources) 
    {    
        if ( sources & (1<<CYGNUM_HAL_INTERRUPT_TIMER0) )
        {
            if (usr_timer_isr != NULL)
				(*usr_timer_isr)(usr_timer_arg);
        }
        else if ( sources & (1<<CYGNUM_HAL_INTERRUPT_XINT0) )
        {
            if (usr_enet_isr != NULL)
				(*usr_enet_isr)(usr_enet_arg);

            /* more interrupt */
            ;
        }
    }
#ifdef TM_TEST_NOOS   
    OSIntNesting--;
#else
    OSIntExit();
#endif /* TM_TEST_NOOS */  
    
    return 0;
}


/******************************************************************************
* iq80321_fiq_handler - Interrupt dispatcher for IQ80321 FIQ Interrupts
*
*
*/
int iq80321_fiq_handler()
{
    int srcs_found = 0;
    return (srcs_found);
}


/**********************************************************************
* isr_connect - Disconnect a user Interrupt Service Routine
*
* NOT TO BE USED FOR SPCI INTERRUPTS! - use pci_isr_connect instead
*
*/
int isr_connect(int int_num, void (*handler)(int), int arg)
{
    switch (int_num)
    {

	case TIMER_INT_ID:
		usr_timer_isr = handler;
		usr_timer_arg = arg;
		break;
	case ENET_INT_ID:
		usr_enet_isr = handler;
		usr_enet_arg = arg;
		break;
	default:
		return (ERROR);
		break;
	}

	return (OK);
}

/**********************************************************************
* isr_disconnect - Disconnect a user Interrupt Service Routine
*
* NOT TO BE USED FOR SPCI INTERRUPTS! - use pci_isr_disconnect instead
*
*/
int isr_disconnect(int int_num)
{
    switch (int_num)
    {

	case TIMER_INT_ID:
		usr_timer_isr = NULL;
		usr_timer_arg = 0;
		break;
	case ENET_INT_ID:
		usr_enet_isr = NULL;
		usr_enet_arg = 0;
		break;
	default:
		return (ERROR);
		break;
	}

	/* i960 disabled interrupt here - should we? */

	return (OK);
}

/***********************************************************************
 *
 * Function: OSTaskStkInit
 *
 * Purpose:
 *	This function is called by either OSTaskCreate() or
 *	OSTaskCreateExt() to initialize the stack frame of the task
 *	being created.
 *
 * Processing:
 * 	Initialize stack pointer
 * 	Initialize first stack frame
 * 	Return stack pointer
 *
 * Parameters:
 *	void (* task)(void *pd) - is a function pointer to the task code.
 *
 *	void * pdata - is a pointer to a user supplied data area that will
 *			be passed to the task when the task first executes.
 *			
 *	OS_STK * ptos - is a pointer to the top of stack for the task.
 *			In this LH79531 Full Descending stack implementation,
 *			OS_STK_GROWTH is set to 1, so 'ptos' contains the HIGHEST
 *			valid address of the stack.
 *
 *	INT16U opt - specifies options that can be used to alter the
 *			behavior of OSTaskStkInit().
 *			(see uCOS_II.H for OS_TASK_OPT_???).
 *
 * Outputs:  None
 *
 * Returns:  current stack pointer of task after initialization
 *
 * Notes:
 *	This function is highly processor specific.
 *	Interrupts are enabled when this task starts executing.
 *
 **********************************************************************/

OS_STK *OSTaskStkInit (void (*task)(void *pd),
						void *pdata,
						OS_STK *ptos,
						INT16U opt)
{
    OS_STK * stk ;

    stk = ptos;	/* Init stack pointer */

    /* build a context for the new task */
    *--stk = (unsigned int) task;       /* lr */
    *--stk = 0;                         /* r12 */
    *--stk = 0;                         /* r11 */
    *--stk = 0;                         /* r10 */
    *--stk = 0;                         /* r9 */
    *--stk = 0;                         /* r8 */
    *--stk = 0;                         /* r7 */
    *--stk = 0;                         /* r6 */
    *--stk = 0;                         /* r5 */
    *--stk = 0;                         /* r4 */
    *--stk = 0;                         /* r3 */
    *--stk = 0;                         /* r2 */
    *--stk = 0;                         /* r1 */
    *--stk = (unsigned int) pdata;      /* r0 */
    *--stk = SVC32MODE;                 /* spsr */
	*--stk = SVC32MODE;                 /* cpsr */

    return ((OS_STK *)stk);
}

/***********************************************************************
 *
 * Function: OSIntCtxSw
 *
 * Purpose:
 *	To set a flag indicating to IRQ Handler that a context change is 
 *	needed at the Interrupt Level.
 *
 * Processing:
 * 	Initialize stack pointer
 * 	Initialize first stack frame
 * 	Return stack pointer
 *
 * Parameters: void
 *
 * Outputs: swap_context_flag global set to 1
 *
 * Returns:  void
 *
 * Notes:
 *	swap_context_flag global is only used by OSIntCtxSw ()
 *	and the IRQ Handler. The IRQ Handler resets the flag to 0 
 *
 **********************************************************************
 */
 
unsigned int swap_context_flag = 0 ;

void OSIntCtxSw(void)
{
    swap_context_flag = 1;
}

#if OS_CPU_HOOKS_EN
/*
 **************************************************************************
 *                                          TASK CREATION HOOK
 *
 * Description: This function is called when a task is created.
 *
 * Arguments  : ptcb is a pointer to the task control block of the task
 *                   being created.
 *
 * Note(s)    : 1) Interrupts are disabled during this call.
 **************************************************************************
 */
void OSTaskCreateHook (OS_TCB *ptcb)
{
}

/*
 **************************************************************************
 *                                           TASK DELETION HOOK
 *
 * Description: This function is called when a task is deleted.
 *
 * Arguments  : ptcb is a pointer to the task control block of the task
 *                   being deleted.
 *
 * Note(s)    : 1) Interrupts are disabled during this call.
 **************************************************************************
 */
void OSTaskDelHook (OS_TCB *ptcb)
{
}

/*
 **************************************************************************
 *                                           TASK SWITCH HOOK
 *
 * Description: This function is called when a task switch is performed.
 *              This allows other operations to be performed operations
 *              during a context switch.
 *
 * Arguments  : none
 *
 * Note(s)    : 1) Interrupts are disabled during this call.
 *              2) It is assumed that the global pointer 'OSTCBHighRdy'
 *                 points to the TCB of the task that will be 'switched in'
 *                 (i.e. the highest priority task) and, 'OSTCBCur' points
 *                 to the task being switched out (i.e. the preempted task).
 **************************************************************************
 */
void OSTaskSwHook (void)
{
}

/*
 **************************************************************************
 *                                           STATISTIC TASK HOOK
 *
 * Description: This function is called every second by uC/OS-II's statistics
 *              task.  This allows your application to add functionality to
 *              the statistics task.
 *
 * Arguments  : none
 **************************************************************************
 */
/* output CPU percentage per 30 sec */
extern double  tvLengthScale;
extern ttUser32Bit tvSocketSendCount;
void OSTaskStatHook (void)
{
    static int statTime = 0;
    static ttUser32Bit lastSendCount = 0;
    static int cpuUsage = 0;
    static int limitation = 0;
    
    int   countTime = 30;
    statTime++;
    cpuUsage += OSCPUUsage;

    
    if ( statTime == countTime)
    {
#if 0
        if ( limitation++ >= 60)
        {   
/* 30 mins stop */
            OS_ENTER_CRITICAL();
            INTCTL_WRITE(0);
            CPWAIT();
            tfKernelError("Demo Limitation", " Time Out!!!\n");
        }
#endif
        if ( tvLengthScale != 0)
        {
            printf("cpu:%d%c, %dMbs \n", cpuUsage/countTime,'%', 
                (int)((tvSocketSendCount - lastSendCount)*tvLengthScale)); 
        }
        else
        {
            printf("cpu:%d%c, %ld pkt/sec\n", cpuUsage/countTime,'%', 
                (ttUser32Bit)((tvSocketSendCount - lastSendCount)/30)); 

        }
        lastSendCount = tvSocketSendCount;
        statTime = 0;
        cpuUsage = 0;
    }
}

/*
 **************************************************************************
 *                                               TICK HOOK
 *
 * Description: This function is called every tick.
 *
 * Arguments  : none
 *
 * Note(s)    : 1) Interrupts may or may not be ENABLED during this call.
 ***************************************************************************
 */
void OSTimeTickHook (void)
{
}

#endif /* OS_CPU_HOOKS_EN */ 





/**********************************************************************
*
* Function: Install_IRQ_Handle()
*
* Purpose:
*	To install the handler for ARM interrupt vector 0x18
*
* Processing:
*	Installs IRQ Handler entry address in Exception jump table for
*	 "Load pc" method of branching to IRQ Handler.
*	Calls Install_Handler () with pointer to IRQ vector 0x18
*		and pointer to jump table address
*
* Parameters: 
*    unsigned int, location of IRQ Handler entry point
*
* Outputs:  None.
*
* Returns:	unsigned int, old contents of IRQ vector
*
* Notes:
*    ARM Architecture specific
*
**********************************************************************/

unsigned int Install_IRQ_Handle(unsigned int IRQHandler)
{
	unsigned int * irqvec = (unsigned int *) IRQ_VEC;
	unsigned int * irqaddr = (unsigned int *) IRQ_ADDR;

	*irqaddr = IRQHandler;
	return (Install_Handler_iq80321 ((unsigned int) irqaddr, irqvec));
}



unsigned int Install_FIQ_Handle(unsigned int FIQHandler)
{
	unsigned int * irqvec = (unsigned int *) FIQ_VEC;
	unsigned int * irqaddr = (unsigned int *) FIQ_ADDR;

	*irqaddr = FIQHandler;
	return (Install_Handler_iq80321 ((unsigned int) irqaddr, irqvec));
}


/**********************************************************************
*
* Function: Install_Handler ()
*
* Author: barnetth
*
* Purpose:
*	To install the handler for generic ARM interrupt vector using
*	the "Load pc" method
*
* Processing:
*	Installs IRQ Handler entry address in Exception jump table for
*	 "Load pc" method of branching to IRQ Handler.
*	Calls Install_Handler () with pointer to IRQ vector 0x18
*		and pointer to jump table address
*
* Parameters: 
*    unsigned int containing location of Handler entry point
*    unsigned int pointer containing location of vector
*
* Outputs:  None.
*
* Returns:	unsigned int, old contents of vector
*
* Notes: "0xe59ff000" is machine language for "LDR PC,#0x000" 
*    When or'd with address within 4k, it results in the PC
*    being loaded with the entry point of the handler when exception
*    associated with "vector" is invoked, provided entry point of
*    handler has been stored at address "location".
*
*    ARM Architecture specific.
*
*
**********************************************************************/

int Install_Handler_iq80321 (unsigned int location, unsigned int * vector)
{
	unsigned int vec, oldvec;
	vec = ((unsigned int) location - (unsigned int)vector - 0x08) | 0xe59ff000;
	oldvec = *vector;
	*vector = vec;
        
	return ( oldvec );

}


int tf8254xSetupInterrupts(unsigned char intLine, void (*fun)(int), void *arg)
{

/* connect the timer ISR */
	isr_connect (ENET_INT_ID, fun, (ttUser32Bit)arg);
    /* enable interrupt */
    tvInterruptMask |= (1<<CYGNUM_HAL_INTERRUPT_XINT0);
    OS_ENTER_CRITICAL();

    INTCTL_WRITE(tvInterruptMask);
    CPWAIT();
    
    OS_EXIT_CRITICAL();
    return 0;
}

#define CLOCK_MULTIPLIER 200

/* interrupt handler for the PAL-based external timer */
void TimerIsr (int arg)
{
	/* increment tick counter */
#ifdef TM_TEST_NOOS   
    tfTimerUpdateIsr();
#else
	OSTimeTick(); 
#endif
	/* to clear the timer interrupt, clear the timer interrupt
	   enable, then re-set the int. enable bit */

    TISR_WRITE(1);

	return;
}

void hal_clock_initialize(INT32U period)
{
    INT32U val;
    INT32U tmr_period;
    
    tmr_period = period * CLOCK_MULTIPLIER;
    
    // disable timer
    TMR0_WRITE(0);

    // clear interrupts
    TISR_WRITE(1);

    // set reload/count value
    TRR0_WRITE(tmr_period);
    TCR0_WRITE(tmr_period);

    // let it run
    TMR0_WRITE(TMR_ENABLE | TMR_RELOAD | TMR_CLK_1);

    TMR0_READ(val);
    isr_connect (TIMER_INT_ID, TimerIsr, 0);
}


void    IQ80321_ISR_INIT()
{
    INTSTR_WRITE(0);  
    INTCTL_READ(tvInterruptMask);       

    Install_FIQ_Handle((unsigned int)IQ80321_FIQHandler); 
    Install_IRQ_Handle((unsigned int)IQ80321_IRQHandler); 

}

void    IQ80321_ISR_TIMER()
{
    tvInterruptMask |= ( (1<<CYGNUM_HAL_INTERRUPT_TIMER0)  
                        |(1<<CYGNUM_HAL_INTERRUPT_TIMER1));
   
    OS_ENTER_CRITICAL();

    INTCTL_WRITE(tvInterruptMask);
    CPWAIT();
   
    OS_EXIT_CRITICAL();

}

	

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: Os_cpu_c.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6569    Output checksum: 53332
 */
