/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/xscale/iq80321/gnu/led.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:03CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/************************************************************************/
/* File: led.h 															*/
/* This header file defines constants for display values and data 		*/ 
/* structure for conversion												*/
/************************************************************************/
#ifndef LED_H
#define LED_H

#include <stdio.h>
#include <stdlib.h>

/************************************************************************/
/* See IQ80310 Board Manual for 7-Segment Display Bit Definition		*/ 
/* Display values: bit=0 turns on, bit=1 turns off 						*/
/************************************************************************/

#define 	DISPLAY_0				0x3 /* 11000000b Display value 0	*/
#define		DISPLAY_1				0x9f /* 11111001b 				1	*/
#define		DISPLAY_2				0x25 /* 10100100b 				2	*/
#define		DISPLAY_3				0x0d /* 10110000b 				3	*/
#define		DISPLAY_4				0x99 /* 10011001b 				4	*/
#define		DISPLAY_5				0x49 /* 10010010b 				5	*/
#define		DISPLAY_6				0x41 /* 10000010b 				6	*/
#define		DISPLAY_7				0x1f /* 11111000b 				7	*/
#define		DISPLAY_8				0x1 /* 10000000b 				8	*/
#define		DISPLAY_9				0x19 /* 10011000b 				9	*/
#define		DISPLAY_A				0x11 /* 10001000b 				A	*/
#define		DISPLAY_B				0xc1 /* 10000011b 				B	*/
#define		DISPLAY_C				0xe5 /* 10100111b 				C	*/
#define		DISPLAY_D				0x85 /* 10100001b 				D	*/
#define		DISPLAY_E				0x61 /* 10000110b 				E	*/
#define		DISPLAY_F				0x71 /* 10001110b 				F	*/
#define		DISPLAY__				0xff /* 11111111b All leds off		*/			
#define  	DELAY					100000000					


/*Memory map																		*/
#define 	TIMER_LA0			   (unsigned int *)	0xfe880000	   /* Timer LA0		*/
#define		TIMER_LA1			   (unsigned int *)	0xfe890000	   /* Timer LA1		*/
#define		TIMER_LA2			   (unsigned int *)	0xfe8a0000	   /* Timer LA2		*/
#define		TIMER_LA3			   (unsigned int *)	0xfe8b0000	   /* Timer LA3		*/
#define		TIMER_ENABLE		   (unsigned int *)	0xfe8c0000	   /* Timer Enable	*/
#define		DISPLAY_LEFT		   (unsigned int *)	0xfe840000	   /* Left LED		*/
#define		DISPLAY_RIGHT		   (unsigned int *)	0xfe850000	   /* Right LED		*/	
		 
/*Bit definitions																	*/
#define		COUNTER_ENABLED			0x0001				/* Used with 0xfe860000 	*/

/* Addresses of the 7-segment displays registers */

/* 08/25/00 jwf */
/* iq80303 address decode */
/*
#define MSB_DISPLAY_REG		(volatile unsigned char *)0xe0040000
#define LSB_DISPLAY_REG		(volatile unsigned char *)0xe0050000
*/

/* 08/25/00 jwf */
/* iq80310 address decode */
#define MSB_DISPLAY_REG		(volatile unsigned char *)0xfe840000	/* 7 segment 0 */
#define LSB_DISPLAY_REG		(volatile unsigned char *)0xfe850000	/* 7 segment 1 */


/* Values for the 7-segment displays */
#define DISPLAY_OFF				0xFF
#define ZERO					0xC0
#define ONE						0xF9
#define TWO						0xA4
#define THREE					0xB0
#define FOUR					0x99
#define FIVE					0x92
#define SIX						0x82
#define SEVEN					0xF8
#define EIGHT					0x80
#define NINE					0x90
#define LETTER_A				0x88
#define LETTER_B				0x83
#define LETTER_C				0xC6
#define LETTER_D				0xA1
#define LETTER_E				0x86
#define LETTER_F				0x8E
#define	LETTER_I				0xCF
#define LETTER_L				0xC7
#define LETTER_P				0x8C
#define LETTER_S				0x92
#define DECIMAL_POINT			0x7F
#define DISPLAY_ERROR			0x06  /* Displays "E." */


/* Parameters for functions */
#define	MSB							0
#define	LSB							1
#define	BOTH						2

static int leds[] =
{			DISPLAY_0 ,
			DISPLAY_1 ,
			DISPLAY_2 ,
			DISPLAY_3 ,
			DISPLAY_4 ,
			DISPLAY_5 ,
			DISPLAY_6 ,
			DISPLAY_7 ,
			DISPLAY_8 ,
			DISPLAY_9 ,
			DISPLAY_A ,
			DISPLAY_B ,
			DISPLAY_C ,
			DISPLAY_D ,
			DISPLAY_E ,
			DISPLAY_F ,
			DISPLAY__
};

int displayLED(unsigned int,unsigned int);

#endif

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: led.h         File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 44323    Output checksum: 30478
 */
