/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * $Source: source/kernel/ucos/xscale/iq80321/gnu/pci80321.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:06CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
/*
//==========================================================================
//####COPYRIGHTBEGIN####
//                                                                          
// -------------------------------------------                              
// The contents of this file are subject to the Red Hat eCos Public License 
// Version 1.1 (the "License"); you may not use this file except in         
// compliance with the License.  You may obtain a copy of the License at    
// http://www.redhat.com/                                                   
//                                                                          
// Software distributed under the License is distributed on an "AS IS"      
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the 
// License for the specific language governing rights and limitations under 
// the License.                                                             
//                                                                          
// The Original Code is eCos - Embedded Configurable Operating System,      
// released September 30, 1998.                                             
//                                                                          
// The Initial Developer of the Original Code is Red Hat.                   
// Portions created by Red Hat are                                          
// Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.                             
// All Rights Reserved.                                                     

 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <tr8254x.h>

#include "led.h"

//----------------------------------------------------------------------
// Bus probing limits.
// Note: these can be overridden by the platform
#ifndef CYG_PCI_MAX_BUS
#define CYG_PCI_MAX_BUS                      8  // Eight is enough?
#endif
#ifndef CYG_PCI_MAX_DEV
#define CYG_PCI_MAX_DEV                      32
#endif
#ifndef CYG_PCI_MIN_DEV
#define CYG_PCI_MIN_DEV                      0
#endif
#ifndef CYG_PCI_MAX_FN
#define CYG_PCI_MAX_FN                       8
#endif
#ifndef CYG_PCI_MAX_BAR
#define CYG_PCI_MAX_BAR                      6
#endif

#define CYG_PCI_VENDOR_UNDEFINED             0xffff
// Some buggy PCI chips force us to ignore certain devices so that
// they may be handled specially.
#define CYG_PCI_IGNORE_DEVICE(__bus, __dev, __fn) 0


#define true    1
#define false -1
/* for flash memory demo, avoid conflict to redboot */
#define cyg_pcihw_read_config_uint8  my_read_uint8
#define cyg_pcihw_read_config_uint16  my_read_uint16
#define cyg_pcihw_read_config_uint32  my_read_uint32
#define cyg_pcihw_write_config_uint8  my_write_uint8
#define cyg_pcihw_write_config_uint16  my_write_uint16
#define cyg_pcihw_write_config_uint32  my_write_uint32

#define cyg_pci_get_device_info         my_get_device_info
#define cyg_hal_plf_pci_init            my_pci_init


int tf8254xSetupPci(unsigned char TM_FAR * intLine,
                    unsigned char TM_FAR * TM_FAR * memAddrPtr,
                    ttUser32Bit TM_FAR * ioAddr);

#define CYG_HAL_MEMORY_MAP_NORMAL

#define TM_PCI_ID_INTEL       0x8086
#define TM_PCI_ID_INTEL_82544 0x1008

#define REG8_PTR(a)  ((volatile unsigned char *)(a))
#define REG32_PTR(a) ((volatile unsigned int *)(a))
#define REG16_PTR(a) ((volatile unsigned short *)(a))

// Address Translation Unit  (Chapter 3)
#define ATU_ATUVID	REG16_PTR(0xffffe100)
#define ATU_ATUDID	REG16_PTR(0xffffe102)
#define ATU_ATUCMD	REG16_PTR(0xffffe104)
#define ATU_ATUSR	REG16_PTR(0xffffe106)
#define ATU_ATURID	REG8_PTR(0xffffe108)
#define ATU_ATUCCR	REG8_PTR(0xffffe109)
#define ATU_ATUCLSR	REG8_PTR(0xffffe10c)
#define ATU_ATULT	REG8_PTR(0xffffe10d)
#define ATU_ATUHTR	REG8_PTR(0xffffe10e)
#define ATU_ATUBIST	REG8_PTR(0xffffe10f)
#define ATU_IABAR0	REG32_PTR(0xffffe110)
#define ATU_IAUBAR0	REG32_PTR(0xffffe114)
#define ATU_IABAR1	REG32_PTR(0xffffe118)
#define ATU_IAUBAR1	REG32_PTR(0xffffe11c)
#define ATU_IABAR2	REG32_PTR(0xffffe120)
#define ATU_IAUBAR2	REG32_PTR(0xffffe124)
#define ATU_ASVIR	REG16_PTR(0xffffe12c)
#define ATU_ASIR	REG16_PTR(0xffffe12e)
#define ATU_ERBAR	REG32_PTR(0xffffe130)
#define ATU_ATUILR	REG8_PTR(0xffffe13c)
#define ATU_ATUIPR	REG8_PTR(0xffffe13d)
#define ATU_ATUMGNT	REG8_PTR(0xffffe13e)
#define ATU_ATUMLAT	REG8_PTR(0xffffe13f)
#define ATU_IALR0	REG32_PTR(0xffffe140)
#define ATU_IATVR0	REG32_PTR(0xffffe144)
#define ATU_ERLR	REG32_PTR(0xffffe148)
#define ATU_ERTVR	REG32_PTR(0xffffe14c)
#define ATU_IALR1	REG32_PTR(0xffffe150)
#define ATU_IALR2	REG32_PTR(0xffffe154)
#define ATU_IATVR2	REG32_PTR(0xffffe158)
#define ATU_OIOWTVR	REG32_PTR(0xffffe15c)
#define ATU_OMWTVR0	REG32_PTR(0xffffe160)
#define ATU_OUMWTVR0	REG32_PTR(0xffffe164)
#define ATU_OMWTVR1	REG32_PTR(0xffffe168)
#define ATU_OUMWTVR1	REG32_PTR(0xffffe16c)
#define ATU_OUDWTVR	REG32_PTR(0xffffe178)
#define ATU_ATUCR	REG32_PTR(0xffffe180)
#define ATU_PCSR	REG32_PTR(0xffffe184)
#define ATU_ATUISR	REG32_PTR(0xffffe188)
#define ATU_ATUIMR	REG32_PTR(0xffffe18c)
#define ATU_IABAR3	REG32_PTR(0xffffe190)
#define ATU_IAUBAR3	REG32_PTR(0xffffe194)
#define ATU_IALR3	REG32_PTR(0xffffe198)
#define ATU_IATVR3	REG32_PTR(0xffffe19c)
#define ATU_OCCAR	REG32_PTR(0xffffe1a4)
#define ATU_OCCDR	REG32_PTR(0xffffe1ac)
#define ATU_PDSCR	REG32_PTR(0xffffe1bc)
#define ATU_PMCAPID	REG8_PTR(0xffffe1c0)
#define ATU_PMNEXT	REG8_PTR(0xffffe1c1)
#define ATU_APMCR	REG16_PTR(0xffffe1c2)
#define ATU_APMCSR	REG16_PTR(0xffffe1c4)
#define ATU_PCIXCAPID	REG8_PTR(0xffffe1e0)
#define ATU_PCIXNEXT	REG8_PTR(0xffffe1e1)
#define ATU_PCIXCMD	REG16_PTR(0xffffe1e2)
#define ATU_PCIXSR	REG32_PTR(0xffffe1e4)

#define GPIO_GPOE	REG8_PTR(0xffffe7c4)
#define GPIO_GPID	REG8_PTR(0xffffe7c8)
#define GPIO_GPOD	REG8_PTR(0xffffe7cc)

#define IQ80321_GBE_GPIO_PIN 4

#define _PCI_MEM_BASE       0x80000000
#define _PCI_MEM_DAC_BASE   0x00000000
#define _PCI_IO_BASE	    0x90000000
#define _PCI_MEM_LIMIT      0x83ffffff
#define _PCI_IO_LIMIT       0x9000ffff

#define HAL_PCI_PHYSICAL_MEMORY_BASE hal_pci_physical_memory_base
#define HAL_PCI_PHYSICAL_IO_BASE     hal_pci_physical_io_base

// Map PCI device resources starting from these addresses in PCI space.
#define HAL_PCI_ALLOC_BASE_MEMORY hal_pci_alloc_base_memory
#define HAL_PCI_ALLOC_BASE_IO     hal_pci_alloc_base_io



#define CYG_PCI_DEV_MAKE_ID(__bus,__devfn) (((__bus)<<16)|((__devfn)<<8))
#define CYG_PCI_DEV_MAKE_DEVFN(__dev, __fn) (((__dev)<<3)|(__fn))
#define CYG_PCI_DEV_GET_BUS(__devid) ((__devid>>16)&0xFF)
#define CYG_PCI_DEV_GET_DEVFN(__devid) ((__devid>>8)&0xFF)
#define CYG_PCI_DEV_GET_DEV(__devfn) ((__devfn>>3)&0x1f)
#define CYG_PCI_DEV_GET_FN(__devfn) (__devfn&0x7)


#define CYG_PCI_NULL_DEVID 0xffffffff
#define CYG_PCI_NULL_DEVFN 0xffff

#define __NIC_PUB  3
#define __NIC_PRIV 5

#define __NIC_PUB_1  8
#define __NIC_PRIV_1 4

#define CYG_PCI_CFG_VENDOR                      0x00
#define CYG_PCI_CFG_DEVICE                      0x02
#define CYG_PCI_CFG_COMMAND                     0x04
#define CYG_PCI_CFG_STATUS                      0x06
#define CYG_PCI_CFG_CLASS_REV                   0x08
#define CYG_PCI_CFG_CACHE_LINE_SIZE             0x0c
#define CYG_PCI_CFG_LATENCY_TIMER               0x0d
#define CYG_PCI_CFG_HEADER_TYPE                 0x0e
#define CYG_PCI_CFG_BIST                        0x0f
#define CYG_PCI_CFG_BAR_BASE                    0x10
#define CYG_PCI_CFG_BAR_0                       0x10
#define CYG_PCI_CFG_BAR_1                       0x14
#define CYG_PCI_CFG_BAR_2                       0x18
#define CYG_PCI_CFG_BAR_3                       0x1c
#define CYG_PCI_CFG_BAR_4                       0x20
#define CYG_PCI_CFG_BAR_5                       0x24
#define CYG_PCI_CFG_CARDBUS_CIS                 0x28
#define CYG_PCI_CFG_SUB_VENDOR                  0x2c
#define CYG_PCI_CFG_SUB_ID                      0x2e
#define CYG_PCI_CFG_ROM_ADDRESS                 0x30
#define CYG_PCI_CFG_CAP_LIST                    0x34
#define CYG_PCI_CFG_INT_LINE                    0x3c
#define CYG_PCI_CFG_INT_PIN                     0x3d
#define CYG_PCI_CFG_MIN_GNT                     0x3e
#define CYG_PCI_CFG_MAX_LAT                     0x3f

#define HAL_PCI_CFG_READ_UINT8( __bus, __devfn, __offset, __val )  \
    __val = cyg_hal_plf_pci_cfg_read_byte((__bus),  (__devfn), (__offset))

#define HAL_PCI_CFG_READ_UINT16( __bus, __devfn, __offset, __val ) \
    __val = cyg_hal_plf_pci_cfg_read_word((__bus),  (__devfn), (__offset))

#define HAL_PCI_CFG_READ_UINT32( __bus, __devfn, __offset, __val ) \
    __val = cyg_hal_plf_pci_cfg_read_dword((__bus),  (__devfn), (__offset))

#define HAL_PCI_CFG_WRITE_UINT8( __bus, __devfn, __offset, __val )  \
    cyg_hal_plf_pci_cfg_write_byte((__bus),  (__devfn), (__offset), (__val))

#define HAL_PCI_CFG_WRITE_UINT16( __bus, __devfn, __offset, __val ) \
    cyg_hal_plf_pci_cfg_write_word((__bus),  (__devfn), (__offset), (__val))

#define HAL_PCI_CFG_WRITE_UINT32( __bus, __devfn, __offset, __val ) \
    cyg_hal_plf_pci_cfg_write_dword((__bus),  (__devfn), (__offset), (__val))


typedef int                 cyg_bool;
typedef unsigned long long CYG_WORD64;
typedef ttUser32Bit      CYG_WORD32;
typedef CYG_WORD64 CYG_PCI_ADDRESS64;
typedef CYG_WORD32 CYG_PCI_ADDRESS32;

//------------------------------------------------------------------
// Macros for manufacturing and decomposing device ids
typedef CYG_WORD32 cyg_pci_device_id;           // PCI device ID
typedef tt8Bit  cyg_uint8;
typedef tt16Bit cyg_uint16;
typedef tt32Bit cyg_uint32;

#if 1
#define CYG_FAIL    printf

typedef enum { 
    CYG_PCI_HEADER_NORMAL = 0, 
    CYG_PCI_HEADER_BRIDGE = 1,
    CYG_PCI_HEADER_CARDBUS_BRIDGE = 2
} cyg_pci_header_type;

typedef struct                          // PCI device data
{
    cyg_pci_device_id devid;            // ID of this device

    // The following fields are read out of the
    // config space for this device.

    cyg_uint16  vendor;                 // vendor ID
    cyg_uint16  device;                 // device ID
    cyg_uint16  command;                // command register
    cyg_uint16  status;                 // status register
    cyg_uint32  class_rev;              // class+revision
    cyg_uint8   cache_line_size;        // cache line size
    cyg_uint8   latency_timer;          // latency timer
    cyg_pci_header_type header_type;    // header type
    cyg_uint8   bist;                   // Built-in Self-Test
    cyg_uint32  base_address[6];        // Memory base address registers

    // The following fields are used by the resource allocation
    // routines to keep track of allocated resources.
    cyg_uint32 num_bars;
        
    cyg_uint32 base_size[6];            // Memory size for each base address
    cyg_uint32 base_map[6];             // Physical address mapped
// add by js
 //   CYG_ADDRWORD hal_vector;            // HAL interrupt vector used by
                                        // device if int_line!=0

    // One of the following unions will be filled in according to
    // the value of the header_type field.

    union
    {
        struct
        {
            cyg_uint32  cardbus_cis;    // CardBus CIS Pointer
            cyg_uint16  sub_vendor;     // subsystem vendor id
            cyg_uint16  sub_id;         // subsystem id
            cyg_uint32  rom_address;    // ROM address register
            cyg_uint8   cap_list;       // capability list
            cyg_uint8   reserved1[7];
            cyg_uint8   int_line;       // interrupt line
            cyg_uint8   int_pin;        // interrupt pin
            cyg_uint8   min_gnt;        // timeslice request
            cyg_uint8   max_lat;        // priority-level request
        } normal;
        struct
        {
	    cyg_uint8	pri_bus;		// primary bus number
	    cyg_uint8	sec_bus;		// secondary bus number
	    cyg_uint8	sub_bus;		// subordinate bus number
	    cyg_uint8	sec_latency_timer;	// secondary bus latency
	    cyg_uint8	io_base;
	    cyg_uint8	io_limit;
	    cyg_uint16	sec_status;		// secondary bus status
	    cyg_uint16	mem_base;
	    cyg_uint16	mem_limit;
	    cyg_uint16	prefetch_base;
	    cyg_uint16	prefetch_limit;
	    cyg_uint32	prefetch_base_upper32;
	    cyg_uint32	prefetch_limit_upper32;	
	    cyg_uint16	io_base_upper16;
	    cyg_uint16	io_limit_upper16;
            cyg_uint8   reserved1[4];
            cyg_uint32  rom_address;    	// ROM address register
            cyg_uint8   int_line;       	// interrupt line
            cyg_uint8   int_pin;        	// interrupt pin
	    cyg_uint16	control;		// bridge control
        } bridge;
        struct
        {
            // Not yet supported
        } cardbus_bridge;
    } header;
} cyg_pci_device;

// PCI Configuration structure offsets
#define CYG_PCI_CFG_VENDOR                      0x00
#define CYG_PCI_CFG_DEVICE                      0x02
#define CYG_PCI_CFG_COMMAND                     0x04
#define CYG_PCI_CFG_STATUS                      0x06
#define CYG_PCI_CFG_CLASS_REV                   0x08
#define CYG_PCI_CFG_CACHE_LINE_SIZE             0x0c
#define CYG_PCI_CFG_LATENCY_TIMER               0x0d
#define CYG_PCI_CFG_HEADER_TYPE                 0x0e
#define CYG_PCI_CFG_BIST                        0x0f
#define CYG_PCI_CFG_BAR_BASE                    0x10
#define CYG_PCI_CFG_BAR_0                       0x10
#define CYG_PCI_CFG_BAR_1                       0x14
#define CYG_PCI_CFG_BAR_2                       0x18
#define CYG_PCI_CFG_BAR_3                       0x1c
#define CYG_PCI_CFG_BAR_4                       0x20
#define CYG_PCI_CFG_BAR_5                       0x24
#define CYG_PCI_CFG_CARDBUS_CIS                 0x28
#define CYG_PCI_CFG_SUB_VENDOR                  0x2c
#define CYG_PCI_CFG_SUB_ID                      0x2e
#define CYG_PCI_CFG_ROM_ADDRESS                 0x30
#define CYG_PCI_CFG_CAP_LIST                    0x34
#define CYG_PCI_CFG_INT_LINE                    0x3c
#define CYG_PCI_CFG_INT_PIN                     0x3d
#define CYG_PCI_CFG_MIN_GNT                     0x3e
#define CYG_PCI_CFG_MAX_LAT                     0x3f

// Bridge configuration offsets
#define CYG_PCI_CFG_PRI_BUS                     0x18
#define CYG_PCI_CFG_SEC_BUS                     0x19
#define CYG_PCI_CFG_SUB_BUS                     0x1a
#define CYG_PCI_CFG_SEC_LATENCY_TIMER           0x1b
#define CYG_PCI_CFG_IO_BASE                     0x1c
#define CYG_PCI_CFG_IO_LIMIT                    0x1d
#define CYG_PCI_CFG_SEC_STATUS                  0x1e
#define CYG_PCI_CFG_MEM_BASE                    0x20
#define CYG_PCI_CFG_MEM_LIMIT                   0x22
#define CYG_PCI_CFG_PREFETCH_BASE               0x24
#define CYG_PCI_CFG_PREFETCH_LIMIT              0x26
#define CYG_PCI_CFG_PREFETCH_BASE_UPPER32       0x28
#define CYG_PCI_CFG_PREFETCH_LIMIT_UPPER32      0x2c
#define CYG_PCI_CFG_IO_BASE_UPPER16             0x30
#define CYG_PCI_CFG_IO_LIMIT_UPPER16            0x32
#define CYG_PCI_CFG_BRIDGE_ROM_ADDRESS          0x38
#define CYG_PCI_CFG_BRIDGE_CONTROL              0x3e
// Command
#define CYG_PCI_CFG_COMMAND_IO                    0x0001
#define CYG_PCI_CFG_COMMAND_MEMORY                0x0002
#define CYG_PCI_CFG_COMMAND_MASTER                0x0004
#define CYG_PCI_CFG_COMMAND_SPECIAL               0x0008
#define CYG_PCI_CFG_COMMAND_INVALIDATE            0x0010
#define CYG_PCI_CFG_COMMAND_VGA_SNOOP             0x0020
#define CYG_PCI_CFG_COMMAND_PARITY                0x0040
#define CYG_PCI_CFG_COMMAND_WAIT                  0x0080
#define CYG_PCI_CFG_COMMAND_SERR                  0x0100
#define CYG_PCI_CFG_COMMAND_FAST_BACK             0x0200

// Consider the device active if any of these bits are set.
#define CYG_PCI_CFG_COMMAND_ACTIVE \
    (CYG_PCI_CFG_COMMAND_IO        \
     |CYG_PCI_CFG_COMMAND_MEMORY   \
     |CYG_PCI_CFG_COMMAND_MASTER)

// Header type
#define CYG_PCI_CFG_HEADER_TYPE_MASK              0x7f
#define CYG_PCI_CFG_HEADER_TYPE_MF                0x80

// BAR
#define CYG_PCI_CFG_BAR_SPACE_MASK                0x00000001
#define CYG_PCI_CFG_BAR_SPACE_MEM                 0x00000000
#define CYG_PCI_CFG_BAR_SPACE_IO                  0x00000001

#define CYG_PRI_CFG_BAR_MEM_TYPE_MASK             0x00000006
#define CYG_PRI_CFG_BAR_MEM_TYPE_32               0x00000000
#define CYG_PRI_CFG_BAR_MEM_TYPE_1M               0x00000002
#define CYG_PRI_CFG_BAR_MEM_TYPE_64               0x00000004

#define CYG_PRI_CFG_BAR_MEM_PREFETCH              0x00000008

//#define CYG_PRI_CFG_BAR_MEM_MASK                  0xfffffff0
#define CYG_PRI_CFG_BAR_IO_MASK                   0xfffffffc

// Bridge control
#define CYG_PCI_CFG_BRIDGE_CTL_PARITY		0x0001
#define CYG_PCI_CFG_BRIDGE_CTL_SERR		0x0002
#define CYG_PCI_CFG_BRIDGE_CTL_ISA		0x0004
#define CYG_PCI_CFG_BRIDGE_CTL_VGA		0x0008
#define CYG_PCI_CFG_BRIDGE_CTL_MASTER		0x0020
#define CYG_PCI_CFG_BRIDGE_CTL_RESET		0x0040
#define CYG_PCI_CFG_BRIDGE_CTL_FAST_BACK	0x0080

// This is where the PCI spaces are mapped in the CPU's address space.
cyg_uint32 hal_pci_alloc_base_memory;
cyg_uint32 hal_pci_alloc_base_io;

cyg_uint32 hal_pci_physical_memory_base;
cyg_uint32 hal_pci_physical_io_base;

cyg_uint32 hal_pci_inbound_window_base;
cyg_uint32 hal_pci_inbound_window_mask;



/* ------------------------------------------------------------------------*/
/*                   82544 EEPROM INTERFACE                                */
/* ------------------------------------------------------------------------*/


#define EE_SHIFT_CLK	0x01            // EEPROM shift clock.
#define EE_CS		    0x02            // EEPROM chip select.
#define EE_DATA_WRITE	0x04            // EEPROM chip data in.
#define EE_DATA_READ	0x08            // EEPROM chip data out.
#define EE_ENB		(0x10|EE_CS)

#define EE_DELAY() do { int z; for ( z = 0; z < 10000; z++ ) ; } while (0)

#define EE_PRINTF( foo )
#define EE_STUFF

// Pass ioAddr around "invisibly"
#define EE_SELECT()              ee_select(ioAddr)
#define EE_DESELECT()            ee_deselect(ioAddr)
#define EE_CLOCK_UP()            ee_clock_up(ioAddr)
#define EE_CLOCK_DOWN()          ee_clock_down(ioAddr)
#define EE_READ_DATA_BIT()       ee_read_data_bit(ioAddr)
#define EE_WRITE_DATA_BIT( _d_ ) ee_write_data_bit(ioAddr,(_d_))


#endif 

void cyg_hal_plf_pci_init(void);

cyg_uint8  cyg_hal_plf_pci_cfg_read_byte (cyg_uint32 bus,
			       cyg_uint32 devfn,
			       cyg_uint32 offset);

cyg_uint16 cyg_hal_plf_pci_cfg_read_word (cyg_uint32 bus,
			       cyg_uint32 devfn,
			       cyg_uint32 offset);
cyg_uint32 cyg_hal_plf_pci_cfg_read_dword (cyg_uint32 bus, 
                                cyg_uint32 devfn, 
                                cyg_uint32 offset);

void
cyg_hal_plf_pci_cfg_write_byte (cyg_uint32 bus,
				cyg_uint32 devfn,
				cyg_uint32 offset,
				cyg_uint8 data);
void
cyg_hal_plf_pci_cfg_write_word (cyg_uint32 bus,
				cyg_uint32 devfn,
				cyg_uint32 offset,
				cyg_uint16 data);
void
cyg_hal_plf_pci_cfg_write_dword (cyg_uint32 bus,
				 cyg_uint32 devfn,
				 cyg_uint32 offset,
				 cyg_uint32 data);


void 
cyg_pcihw_read_config_uint8( cyg_uint8 bus, cyg_uint8 devfn, 
                             cyg_uint8 offset, cyg_uint8 *val)
{
    HAL_PCI_CFG_READ_UINT8(bus, devfn, offset, *val);
}
    
void cyg_pcihw_read_config_uint16( cyg_uint8 bus, cyg_uint8 devfn, 
                              cyg_uint8 offset, cyg_uint16 *val)
{
    HAL_PCI_CFG_READ_UINT16(bus, devfn, offset, *val);
}

void
cyg_pcihw_read_config_uint32( cyg_uint8 bus, cyg_uint8 devfn, 
                              cyg_uint8 offset, cyg_uint32 *val)
{
    HAL_PCI_CFG_READ_UINT32(bus, devfn, offset, *val);
}

// Write functions
void 
cyg_pcihw_write_config_uint8( cyg_uint8 bus, cyg_uint8 devfn, 
                             cyg_uint8 offset, cyg_uint8 val)
{
    HAL_PCI_CFG_WRITE_UINT8(bus, devfn, offset, val);
}

void 
cyg_pcihw_write_config_uint16( cyg_uint8 bus, cyg_uint8 devfn, 
                              cyg_uint8 offset, cyg_uint16 val)
{
    HAL_PCI_CFG_WRITE_UINT16(bus, devfn, offset, val);
}

void
cyg_pcihw_write_config_uint32( cyg_uint8 bus, cyg_uint8 devfn, 
                              cyg_uint8 offset, cyg_uint32 val)
{
    HAL_PCI_CFG_WRITE_UINT32(bus, devfn, offset, val);
}



static void __attribute__ ((naked))
__pci_abort_handler(void) 
{
    asm ( "subs	pc, lr, #4\n" );
}
static cyg_uint32 orig_abort_vec;

static inline void
pci_config_setup(cyg_uint32 bus, cyg_uint32 devfn, cyg_uint32 offset)
{
    cyg_uint32 dev = CYG_PCI_DEV_GET_DEV(devfn);
    cyg_uint32 fn  = CYG_PCI_DEV_GET_FN(devfn);
    cyg_uint8 localbus;

    localbus = (*ATU_PCIXSR >> 8) & 0xff;
    if (localbus == 0xff)
	localbus = 0;

    /* Offsets must be dword-aligned */
    offset &= ~3;
	
    /* Immediate bus use type 0 config, all others use type 1 config */
#if DEBUG_CONFIG_VERBOSE
    diag_printf("config: localbus[%d] bus[%d] dev[%d] fn[%d] offset[0x%x]\n",
		localbus, bus, dev, fn, offset);
#endif

    if (bus == localbus)
	*ATU_OCCAR = ( (1 << (dev + 16)) | (fn << 8) | offset | 0 );
    else
        *ATU_OCCAR = ( (bus << 16) | (dev << 11) | (fn << 8) | offset | 1 );

    orig_abort_vec = ((volatile cyg_uint32 *)0x20)[4];
    ((volatile unsigned *)0x20)[4] = (unsigned)__pci_abort_handler;
}

static inline int
pci_config_cleanup(cyg_uint32 bus)
{
    cyg_uint32 status = 0, err = 0;

    status = *ATU_ATUSR;
    if ((status & 0xF900) != 0) {
	err = 1;
	*ATU_ATUSR = status & 0xF900;
    }

    ((volatile unsigned *)0x20)[4] = orig_abort_vec;

    return err;
}

cyg_uint8
cyg_hal_plf_pci_cfg_read_byte (cyg_uint32 bus,
			       cyg_uint32 devfn,
			       cyg_uint32 offset)
{
    int err;
    cyg_uint8 config_data;

    pci_config_setup(bus, devfn, offset & ~3);

    config_data = (cyg_uint8)(((*ATU_OCCDR) >> ((offset % 0x4) * 8)) & 0xff);

    err = pci_config_cleanup(bus);

#if DEBUG_CONFIG_VERBOSE
    diag_printf("config read byte: data[0x%x] err[%d]\n",
		config_data, err);
#endif
    if (err)
	return 0xff;
    else
	return config_data;
}


cyg_uint16 cyg_hal_plf_pci_cfg_read_word (cyg_uint32 bus,
			       cyg_uint32 devfn,
			       cyg_uint32 offset)
{
    cyg_uint16 config_data;
    int err;

    pci_config_setup(bus, devfn, offset & ~3);

    config_data = (cyg_uint16)(((*ATU_OCCDR) >> ((offset % 0x4) * 8)) & 0xffff);

    err = pci_config_cleanup(bus);

#if DEBUG_CONFIG_VERBOSE
    diag_printf("config read word: data[0x%x] err[%d]\n",
		config_data, err);
#endif
    if (err)
      return 0xffff;
    else
      return config_data;
}

cyg_uint32
cyg_hal_plf_pci_cfg_read_dword (cyg_uint32 bus, cyg_uint32 devfn, cyg_uint32 offset)
{
    cyg_uint32 config_data;
    int err;

    pci_config_setup(bus, devfn, offset);

    config_data = *ATU_OCCDR;

    err = pci_config_cleanup(bus);

#if DEBUG_CONFIG_VERBOSE
    diag_printf("config read dword: data[0x%x] err[%d]\n",
		config_data, err);
#endif
    if (err)
      return 0xffffffff;
    else
      return config_data;
}
void
cyg_hal_plf_pci_cfg_write_byte (cyg_uint32 bus,
				cyg_uint32 devfn,
				cyg_uint32 offset,
				cyg_uint8 data)
{
    int err;
    cyg_uint32 mask, temp;

    pci_config_setup(bus, devfn, offset & ~3);

    mask = ~(0x000000ff << ((offset % 0x4) * 8));
    temp = (cyg_uint32)(((cyg_uint32)data) << ((offset % 0x4) * 8));
    *ATU_OCCDR = (*ATU_OCCDR & mask) | temp; 

    err = pci_config_cleanup(bus);

#if DEBUG_CONFIG_VERBOSE
    diag_printf("config write byte: data[0x%x] err[%d]\n",
		data, err);
#endif
}
void
cyg_hal_plf_pci_cfg_write_word (cyg_uint32 bus,
				cyg_uint32 devfn,
				cyg_uint32 offset,
				cyg_uint16 data)
{
    int err;
    cyg_uint32 mask, temp;

    pci_config_setup(bus, devfn, offset & ~3);

    mask = ~(0x0000ffff << ((offset % 0x4) * 8));

    temp = (cyg_uint32)(((cyg_uint32)data) << ((offset % 0x4) * 8));
    *ATU_OCCDR = (*ATU_OCCDR & mask) | temp; 

    err = pci_config_cleanup(bus);

#if DEBUG_CONFIG_VERBOSE
    diag_printf("config write word: data[0x%x] err[%d]\n",
		data, err);
#endif
}

void
cyg_hal_plf_pci_cfg_write_dword (cyg_uint32 bus,
				 cyg_uint32 devfn,
				 cyg_uint32 offset,
				 cyg_uint32 data)
{
    int err;

    pci_config_setup(bus, devfn, offset);

    *ATU_OCCDR = data;

    err = pci_config_cleanup(bus);

#if DEBUG_CONFIG_VERBOSE
    diag_printf("config write dword: data[0x%x] err[%d]\n",
		data, err);
#endif
}


#if 1
void 
cyg_pci_get_device_info ( cyg_pci_device_id devid, cyg_pci_device *dev_info )
{
    int i;
    cyg_uint8 bus = CYG_PCI_DEV_GET_BUS(devid);
    cyg_uint8 devfn = CYG_PCI_DEV_GET_DEVFN(devid);
    cyg_uint8 header_type;

    dev_info->devid = devid;

    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_VENDOR,
                                 &dev_info->vendor);
    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_DEVICE,
                                 &dev_info->device);
    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_COMMAND,
                                 &dev_info->command);
    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_STATUS,
                                 &dev_info->status);
    cyg_pcihw_read_config_uint32(bus, devfn, CYG_PCI_CFG_CLASS_REV,
                                 &dev_info->class_rev);
    cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_CACHE_LINE_SIZE,
                                &dev_info->cache_line_size);
    cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_LATENCY_TIMER,
                                &dev_info->latency_timer);
    cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_HEADER_TYPE,
                                &header_type);
    dev_info->header_type = (cyg_pci_header_type) header_type;
    cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_BIST,
                                &dev_info->bist);

    if ((dev_info->header_type & CYG_PCI_CFG_HEADER_TYPE_MASK) == CYG_PCI_HEADER_BRIDGE)
	dev_info->num_bars = 2;
    else
	dev_info->num_bars = 6;

    for (i = 0; i < dev_info->num_bars; i++)
        cyg_pcihw_read_config_uint32(bus, devfn,
                                     CYG_PCI_CFG_BAR_BASE + 4*i,
                                     &dev_info->base_address[i]);

    // If device is disabled, probe BARs for sizes.
    if ((dev_info->command & CYG_PCI_CFG_COMMAND_ACTIVE) == 0) {

        for (i = 0; i < dev_info->num_bars; i++){
            cyg_uint32 size;

            cyg_pcihw_write_config_uint32(bus, devfn, 
                                          CYG_PCI_CFG_BAR_BASE + 4*i,
                                          0xffffffff);
            cyg_pcihw_read_config_uint32(bus, devfn, 
                                         CYG_PCI_CFG_BAR_BASE + 4*i,
                                         &size);
            dev_info->base_size[i] = size;
            dev_info->base_map[i] = 0xffffffff;

	    // No reason to scan beyond first inactive BAR.
	    if (size == 0) {
		dev_info->num_bars = i;
		break;
	    }

            // Check for a 64bit memory region.
            if (CYG_PCI_CFG_BAR_SPACE_MEM == 
                (size & CYG_PCI_CFG_BAR_SPACE_MASK)) {
                if (size & CYG_PRI_CFG_BAR_MEM_TYPE_64) {
                    // Clear fields for next BAR - it's the upper 32 bits.
                    i++;
                    dev_info->base_size[i] = 0;
                    dev_info->base_map[i] = 0xffffffff;
                }
            }
        }
    } else {
	// If the device is already configured. Fill in the base_map.
	CYG_PCI_ADDRESS64 tmp_addr;
	cyg_uint32 bar;

        for (i = 0; i < dev_info->num_bars; i++){

	    dev_info->base_size[i] = 0;

            bar = dev_info->base_address[i];

	    // No reason to scan beyond first inactive BAR.
	    if (bar == 0) {
		dev_info->num_bars = i;
		break;
	    }

	    if ((bar & CYG_PCI_CFG_BAR_SPACE_MASK) == CYG_PCI_CFG_BAR_SPACE_IO) {
		dev_info->base_map[i] = (bar & CYG_PRI_CFG_BAR_IO_MASK) + HAL_PCI_PHYSICAL_IO_BASE;
	    } else {
		tmp_addr = bar & CYG_PRI_CFG_BAR_MEM_MASK;

		if ((bar & CYG_PRI_CFG_BAR_MEM_TYPE_MASK) == CYG_PRI_CFG_BAR_MEM_TYPE_64)
		    tmp_addr |= ((CYG_PCI_ADDRESS64)(dev_info->base_address[i+1] & CYG_PRI_CFG_BAR_MEM_MASK)) << 32;
			
		tmp_addr += HAL_PCI_PHYSICAL_MEMORY_BASE;

		dev_info->base_map[i] = tmp_addr;
		if ((bar & CYG_PRI_CFG_BAR_MEM_TYPE_MASK) == CYG_PRI_CFG_BAR_MEM_TYPE_64)
		    dev_info->base_map[++i] = tmp_addr >> 32;
	    }
        }
    }
            

    switch (dev_info->header_type & CYG_PCI_CFG_HEADER_TYPE_MASK) {
    case CYG_PCI_HEADER_NORMAL:
        cyg_pcihw_read_config_uint32(bus, devfn, CYG_PCI_CFG_CARDBUS_CIS,
                                     &dev_info->header.normal.cardbus_cis);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_SUB_VENDOR,
                                     &dev_info->header.normal.sub_vendor);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_SUB_ID,
                                     &dev_info->header.normal.sub_id);
        cyg_pcihw_read_config_uint32(bus, devfn, CYG_PCI_CFG_ROM_ADDRESS,
                                     &dev_info->header.normal.rom_address);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_CAP_LIST,
                                    &dev_info->header.normal.cap_list);

        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_INT_LINE,
                                    &dev_info->header.normal.int_line);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_INT_PIN,
                                    &dev_info->header.normal.int_pin);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_MIN_GNT,
                                    &dev_info->header.normal.min_gnt);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_MAX_LAT,
                                    &dev_info->header.normal.max_lat);
        break;
    case CYG_PCI_HEADER_BRIDGE:
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_PRI_BUS,
                                    &dev_info->header.bridge.pri_bus);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_SEC_BUS,
                                    &dev_info->header.bridge.sec_bus);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_SUB_BUS,
                                    &dev_info->header.bridge.sub_bus);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_SEC_LATENCY_TIMER,
                                    &dev_info->header.bridge.sec_latency_timer);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_IO_BASE,
                                    &dev_info->header.bridge.io_base);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_IO_LIMIT,
                                    &dev_info->header.bridge.io_limit);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_SEC_STATUS,
                                    &dev_info->header.bridge.sec_status);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_MEM_BASE,
                                    &dev_info->header.bridge.mem_base);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_MEM_LIMIT,
                                    &dev_info->header.bridge.mem_limit);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_PREFETCH_BASE,
                                    &dev_info->header.bridge.prefetch_base);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_PREFETCH_LIMIT,
                                    &dev_info->header.bridge.prefetch_limit);
        cyg_pcihw_read_config_uint32(bus, devfn, CYG_PCI_CFG_PREFETCH_BASE_UPPER32,
                                    &dev_info->header.bridge.prefetch_base_upper32);
        cyg_pcihw_read_config_uint32(bus, devfn, CYG_PCI_CFG_PREFETCH_LIMIT_UPPER32,
                                    &dev_info->header.bridge.prefetch_limit_upper32);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_IO_BASE_UPPER16,
                                    &dev_info->header.bridge.io_base_upper16);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_IO_LIMIT_UPPER16,
                                    &dev_info->header.bridge.io_limit_upper16);
        cyg_pcihw_read_config_uint32(bus, devfn, CYG_PCI_CFG_BRIDGE_ROM_ADDRESS,
                                    &dev_info->header.bridge.rom_address);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_INT_LINE,
                                    &dev_info->header.bridge.int_line);
        cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_INT_PIN,
                                    &dev_info->header.bridge.int_pin);
        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_BRIDGE_CONTROL,
                                    &dev_info->header.bridge.control);
        break;
    case CYG_PCI_HEADER_CARDBUS_BRIDGE:
        CYG_FAIL("PCI device header 'cardbus bridge' support not implemented");
        break;
    default:
        CYG_FAIL("Unknown PCI device header type");
        break;
    }
}
#endif 

int tf8254xSetupPci(unsigned char TM_FAR * intLine,
                    unsigned char TM_FAR * TM_FAR * memAddrPtr,
                    ttUser32Bit TM_FAR * ioAddr)
{
   
    tt32Bit     devid;
    tt16Bit     vendor;                 // vendor ID
    tt16Bit     device;     
    tt8Bit      int_line;

    cyg_pci_device  devinfo;
    int i, bus1;
    cyg_uint8 bus; //= CYG_PCI_DEV_GET_BUS(devid);
    cyg_uint8 devfn;// = CYG_PCI_DEV_GET_DEVFN(devid);
      
    cyg_hal_plf_pci_init();
// First, look for NIC at private and public addresses

    bus1 = (*ATU_PCIXSR >> 8) & 0xff;
    if (bus1 == 0xff)
    {
        bus1 = 0;
    }

    devid = CYG_PCI_DEV_MAKE_ID(bus1, CYG_PCI_DEV_MAKE_DEVFN(__NIC_PUB, 0));
    bus = CYG_PCI_DEV_GET_BUS(devid);
    devfn = CYG_PCI_DEV_GET_DEVFN(devid);
    
    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_VENDOR,
                                 &vendor);

    if (vendor != TM_PCI_ID_INTEL) 
    {
	    devid = CYG_PCI_DEV_MAKE_ID(bus1, CYG_PCI_DEV_MAKE_DEVFN(__NIC_PRIV, 0));
        bus = CYG_PCI_DEV_GET_BUS(devid);
        devfn = CYG_PCI_DEV_GET_DEVFN(devid);
       	cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_VENDOR,
                                 &vendor);
        if (vendor != TM_PCI_ID_INTEL) 
        {
	        devid = CYG_PCI_DEV_MAKE_ID(bus1, CYG_PCI_DEV_MAKE_DEVFN(__NIC_PUB_1, 0));
            bus = CYG_PCI_DEV_GET_BUS(devid);
            devfn = CYG_PCI_DEV_GET_DEVFN(devid);
    
    	    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_VENDOR,
                                        &vendor);
            if (vendor != TM_PCI_ID_INTEL) 
            {
	            devid = CYG_PCI_DEV_MAKE_ID(bus1, CYG_PCI_DEV_MAKE_DEVFN(__NIC_PRIV_1, 0));
                bus = CYG_PCI_DEV_GET_BUS(devid);
                devfn = CYG_PCI_DEV_GET_DEVFN(devid);
    
    	        cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_VENDOR,
                                            &vendor);
                if ( vendor !=TM_PCI_ID_INTEL)
                {
                    printf(" no 82544 \n");
                    return -1;
                }
            }
        }
    }
    cyg_pcihw_read_config_uint16(bus, devfn, CYG_PCI_CFG_DEVICE,
                                 &device);
/* 82544 */
    if ( device != TM_PCI_ID_INTEL_82544) 
    {
        printf(" no 82544 \n");
        return -1;
    }
/*
    cyg_pcihw_read_config_uint8(bus, devfn, CYG_PCI_CFG_INT_LINE,
                                    &int_line);
 */
    cyg_pci_get_device_info( devid, &devinfo);
    
    *intLine    = devinfo.header.normal.int_line;
    *memAddrPtr = (char *)devinfo.base_map[0];
    *ioAddr     = devinfo.base_map[2];

    return 0;
}


void 
cyg_hal_plf_pci_init(void)
{
    cyg_uint32 hal_dram_size = 128<<20;
    cyg_uint32 dram_limit = (0xFFFFFFFF - (hal_dram_size - 1)) & 0xFFFFFFC0;

    static CYG_PCI_ADDRESS64 cyg_pci_memory_base;
    static CYG_PCI_ADDRESS32 cyg_pci_io_base;

    cyg_pci_memory_base = HAL_PCI_ALLOC_BASE_MEMORY;
    cyg_pci_io_base = HAL_PCI_ALLOC_BASE_IO;

    // Enable NIC through GPIO pin. This may not have an effect depending
    // on switch settings.
    *GPIO_GPOE &= ~(1 << IQ80321_GBE_GPIO_PIN);
    *GPIO_GPOD |= (1 << IQ80321_GBE_GPIO_PIN);

    hal_pci_inbound_window_mask = ~dram_limit;

#ifdef CYG_HAL_STARTUP_ROM
#ifdef CYGSEM_HAL_ARM_IQ80321_CLEAR_PCI_RETRY
    if (!hal_pcsr_cfg_retry || !cyg_hal_plf_wait_for_bios())
#endif  // CYGSEM_HAL_ARM_IQ80321_CLEAR_PCI_RETRY
    {
	// 64-bit prefetchable
	*ATU_IABAR2 = SDRAM_PHYS_BASE | \
                      CYG_PRI_CFG_BAR_MEM_TYPE_64 | \
                      CYG_PRI_CFG_BAR_MEM_PREFETCH;
	*ATU_IAUBAR2 = 0;

	// Outbound window will be set based on the memory reserved
        // by inbound window 1
	*ATU_IABAR1 = _PCI_MEM_BASE | \
	              CYG_PRI_CFG_BAR_MEM_TYPE_64 | \
                      CYG_PRI_CFG_BAR_MEM_PREFETCH;

    }
#endif  // CYG_HAL_STARTUP_ROM

    // allow ATU to act as a bus master, respond to PCI memory accesses,
    // and assert S_SERR#
    *ATU_ATUCMD = (CYG_PCI_CFG_COMMAND_SERR   | \
		   CYG_PCI_CFG_COMMAND_PARITY | \
		   CYG_PCI_CFG_COMMAND_MASTER | \
		   CYG_PCI_CFG_COMMAND_MEMORY);

    hal_pci_alloc_base_memory = *ATU_IABAR1 & CYG_PRI_CFG_BAR_MEM_MASK;
    hal_pci_alloc_base_io = _PCI_IO_BASE;

    hal_pci_inbound_window_base = *ATU_IABAR2 & CYG_PRI_CFG_BAR_MEM_MASK;


    // set the outbound window PCI address
    *ATU_OMWTVR0 = hal_pci_alloc_base_memory;
    *ATU_OUMWTVR0 = 0;

    // outbound I/O window
    *ATU_OIOWTVR = hal_pci_alloc_base_io;

    hal_pci_physical_memory_base = _PCI_MEM_BASE - hal_pci_alloc_base_memory;
    hal_pci_physical_io_base     = _PCI_IO_BASE - hal_pci_alloc_base_io;

#ifdef CYG_HAL_MEMORY_MAP_NORMAL
    // Adjust for Virt - Phys in CPU space
    hal_pci_physical_memory_base += 0x20000000;
    hal_pci_physical_io_base     += 0x20000000;
#endif

     cyg_pci_memory_base = HAL_PCI_ALLOC_BASE_MEMORY;
    cyg_pci_io_base = HAL_PCI_ALLOC_BASE_IO;;

    // enable outbound ATU
    *ATU_ATUCR = 2;

    *ATU_APMCSR = 3;
}



//------------------------------------------------------------------------
// Device find functions

cyg_bool
cyg_pci_find_next( cyg_pci_device_id cur_devid, 
                   cyg_pci_device_id *next_devid )
{
    cyg_uint8 bus = CYG_PCI_DEV_GET_BUS(cur_devid);
    cyg_uint8 devfn = CYG_PCI_DEV_GET_DEVFN(cur_devid);
    cyg_uint8 dev = CYG_PCI_DEV_GET_DEV(devfn);
    cyg_uint8 fn = CYG_PCI_DEV_GET_FN(devfn);

#ifdef CYGPKG_IO_PCI_DEBUG
    diag_printf("cyg_pci_find_next: start[%x] ...\n",(unsigned)cur_devid);
#endif

    // If this is the initializer, start with 0/0/0
    if (CYG_PCI_NULL_DEVID == cur_devid) {
        bus = dev = fn = 0;
        dev = CYG_PCI_MIN_DEV;
    } else if (CYG_PCI_NULL_DEVFN == (cur_devid & CYG_PCI_NULL_DEVFN)) {
        dev = fn = 0;
        dev = CYG_PCI_MIN_DEV;
    } else {
        // Otherwise, check multi-function bit of device's first function
        cyg_uint8 header;

	devfn = CYG_PCI_DEV_MAKE_DEVFN(dev, 0);
	cyg_pcihw_read_config_uint8(bus, devfn,
				    CYG_PCI_CFG_HEADER_TYPE, &header);
	if (header & CYG_PCI_CFG_HEADER_TYPE_MF) {
	    // Multi-function device. Increase fn.
	    fn++;
	    if (fn >= CYG_PCI_MAX_FN) {
		fn = 0;
		dev++;
	    }
	} else {
	    // Single-function device. Skip to next.
	    dev++;
	}
    }

    // Note: Reset iterators in enclosing statement's "next" part.
    //       Allows resuming scan with given input values. 
    for (;bus < CYG_PCI_MAX_BUS; bus++, dev=CYG_PCI_MIN_DEV) {
        for (;dev < CYG_PCI_MAX_DEV; dev++, fn=0) {
            for (;fn < CYG_PCI_MAX_FN; fn++) {
                cyg_uint16 vendor;

		if (CYG_PCI_IGNORE_DEVICE(bus, dev, fn))
		    continue;

                devfn = CYG_PCI_DEV_MAKE_DEVFN(dev, fn);
                cyg_pcihw_read_config_uint16(bus, devfn,
                                             CYG_PCI_CFG_VENDOR, &vendor);
                if (CYG_PCI_VENDOR_UNDEFINED != vendor) {
#ifdef CYGPKG_IO_PCI_DEBUG
                diag_printf("   Bus: %d, Dev: %d, Fn: %d, Vendor: %x\n", bus, dev, fn, vendor);
#endif
                    *next_devid = CYG_PCI_DEV_MAKE_ID(bus, devfn);
                    return true;
                }
            }
        }
    }

#ifdef CYGPKG_IO_PCI_DEBUG
    diag_printf("nothing.\n");
#endif

    return false;
}

//
// Scan for a particular device, starting with 'devid'
// 'devid' is updated with the next device if found.
//         is not changed if no suitable device is found.
cyg_bool
cyg_pci_find_device( cyg_uint16 vendor, cyg_uint16 device,
                     cyg_pci_device_id *devid )
{
    cyg_pci_device_id new_devid = *devid;

#ifdef CYGPKG_IO_PCI_DEBUG
    diag_printf("cyg_pci_find_device - vendor: %x, device: %x\n", vendor, device);
#endif
    // Scan entire bus, check for matches on valid devices.
    while (cyg_pci_find_next(new_devid, &new_devid)) {
        cyg_uint8 bus = CYG_PCI_DEV_GET_BUS(new_devid);
        cyg_uint8 devfn = CYG_PCI_DEV_GET_DEVFN(new_devid);
        cyg_uint16 v, d;

        // Check that vendor matches.
        cyg_pcihw_read_config_uint16(bus, devfn,
                                     CYG_PCI_CFG_VENDOR, &v);
        cyg_pcihw_read_config_uint16(bus, devfn,
                                     CYG_PCI_CFG_DEVICE, &d);
#ifdef CYGPKG_IO_PCI_DEBUG
        diag_printf("... PCI vendor = %x, device = %x\n", v, d);
#endif
        if (v != vendor) continue;

        // Check that device matches.
        if (d == device) {
            *devid = new_devid;
            return true;
        }
    }

    return false;
}



/*  Following functions are tf8254xSetupSpeed() and tf8254xGetMacAddr()
 *  in the initilization
 */


static inline void
ee_select( int ioAddr )
{
    CYG_WORD32 l;
    l = EE_ENB;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    EE_DELAY();
    l |= EE_CS;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    l = tm_inl( ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_select       : " EE_STUFF  );
    EE_DELAY();
}

static inline void
ee_deselect( int ioAddr )
{
    CYG_WORD32 l;
    l = EE_ENB;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_deselect 1   : " EE_STUFF  );
    EE_DELAY();
    EE_DELAY();
    EE_DELAY();
    l = EE_ENB & ~EE_CS;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    l = tm_inl( ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_deselect 2   : " EE_STUFF  );
    EE_DELAY();
    EE_DELAY();
    EE_DELAY();
}

static inline void
ee_clock_up( int ioAddr )
{
    CYG_WORD32 l;
    l = tm_inl( ioAddr + TM_I82544_EECD );
    l |= EE_SHIFT_CLK;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    EE_DELAY();
    l = tm_inl( ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_clock_up     : " EE_STUFF  );
    EE_DELAY();
}

static inline void
ee_clock_down( int ioAddr )
{
    CYG_WORD32 l;
    l = tm_inl( ioAddr + TM_I82544_EECD );
    l &=~ EE_SHIFT_CLK;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    EE_DELAY();
    l = tm_inl( ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_clock_down   : " EE_STUFF  );
    EE_DELAY();
}

static inline int
ee_read_data_bit( int ioAddr )
{
    CYG_WORD32 l;
    l = tm_inl( ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_read_data    : " EE_STUFF  );
    return EE_DATA_READ == (EE_DATA_READ & l);
}

static inline void
ee_write_data_bit( int ioAddr, int databit )
{
    CYG_WORD32 l;
    l = tm_inl( ioAddr + TM_I82544_EECD );
    if ( databit )
        l |= EE_DATA_WRITE;
    else
        l &= ~EE_DATA_WRITE;
    tm_outl( l, ioAddr + TM_I82544_EECD );
    l = tm_inl( ioAddr + TM_I82544_EECD );
    EE_PRINTF( "ee_write_data   : " EE_STUFF  );
    EE_DELAY();
}


// ------------------------------------------------------------------------

static int
read_eeprom_word( CYG_WORD32 ioAddr, int addrbits, int address )
{
    int i, tmp;

/*Should already be not-selected, but anyway:*/
    EE_SELECT();

/*     Shift the read command bits out.*/
    for (i = 3; i >= 0; i--) { // Doc says to shift out a zero then:
        tmp = (6 & (1 << i)) ? 1 : 0; // "6" is the "read" command.
        EE_WRITE_DATA_BIT(tmp);
        EE_CLOCK_UP();
        EE_CLOCK_DOWN();
    }

    // Now clock out address
    for ( i = addrbits - 1; i >= 0 ; i-- ) {
        tmp = (address & (1<<i)) ? 1 : 0;
        EE_WRITE_DATA_BIT(tmp);
        EE_CLOCK_UP();
        tmp = EE_READ_DATA_BIT();
        EE_CLOCK_DOWN();

//        CYG_ASSERT( (0 == tmp) == (0 == i), "Looking for zero handshake bit" );
    }

    // read in the data
    tmp = 0;
    for (i = 15; i >= 0; i--) {
        EE_CLOCK_UP();
        if ( EE_READ_DATA_BIT() )
            tmp |= (1<<i);
        EE_CLOCK_DOWN();
    }

    // Terminate the EEPROM access.
    EE_DESELECT();
 
#ifdef DEBUG_EE
//    diag_printf( "eeprom address %4x: data %4x\n", address, tmp );
#endif

    return tmp;
}


/* Setup Serial EEPROM for Ethernet Configuration */
void tf8254xGetMacAddr (ttUser32Bit nic_addr, char *macAddr)
{
    int i;
    unsigned short tmp[6];
    
    // Now read complete EEPROM contents
    for (i = 0; i < 3; i++) 
    {
        tmp[i] = (unsigned short )read_eeprom_word(nic_addr, 6, i);
        macAddr[2*i] = tmp[i] & 0xff;
        macAddr[2*i+1] = (tmp[i] & 0xff00)>>8;
    }
    
}




static int mii_read_register( ttUser32Bit ioAddr, int phy, int regnum )
{
    int value = 0;
// An 82544, read MII register via MDIC register.
// UNTESTED
        
    ttUser32Bit mdic = (2<<26) | (phy<<21) | (regnum<<16);

    tm_outl( mdic, ioAddr + TM_I82544_MDIC );

// Wait for ready
    do
    {
        mdic = tm_inl( ioAddr + TM_I82544_MDIC );
    } while( (mdic & (1<<28)) == 0 );

    value = mdic & 0xFFFF;
    return value;
}

static void mii_write_register( ttUser32Bit ioAddr, int phy, int regnum, int value )
{
    ttUser32Bit mdic = (1<<26) | (phy<<21) | (regnum<<16) | (value&0xFFFF);
    tm_outl( mdic, ioAddr + TM_I82544_MDIC );

/* Wait for ready*/
    do
    {
        mdic = tm_inl( ioAddr + TM_I82544_MDIC );
    } while( (mdic & (1<<28)) == 0 );
}

void tf8254xSetupSpeed(ttUser32Bit  ioAddr)
{
    ttUser32Bit           ctrl;
    unsigned short          phy_ctrl;
    unsigned short          phy_stat;
    unsigned short          phy_pssr;

    int delay_cnt = 500;

   
#if 1
/* setup : auto-negotiation */
    ctrl = tm_inl( ioAddr + TM_I82544_CTRL );
/* Set link up bit*/
    ctrl |= TM_I82544_CTRL_SLU | TM_I82544_CTRL_ASDE;
    ctrl &= ~(TM_I82544_CTRL_ILOS 
                | TM_I82544_CTRL_FRCSPD 
                | TM_I82544_CTRL_FRCDPLX);
    tm_outl( ctrl, ioAddr + TM_I82544_CTRL );

/* wait up to 5 seconds for link to come up*/
    while ((mii_read_register( ioAddr, TM_PHY_ADDRESS, 1 ) & 0x4) == 0) 
    {
	    if (--delay_cnt <= 0)
        {
		    break;
        }
    }
#else
// The following sequence of resets and bit twiddling seem to be
    // necessary to get the 82543 working. Not sure what is necessary
    // for the 82544.

    ctrl = tm_inl( ioAddr + TM_I82544_CTRL );
    
    // Set link up bit
    ctrl |= I82544_CTRL_SLU;    
    ctrl &= ~I82544_CTRL_ILOS;
    OUTL( ctrl, ioaddr + I82544_CTRL );
    udelay(20);

    // Force PHY physical reset
    // We can only access the PHY after we have done this.

    ctrl_ext = INL( ioaddr + I82544_CTRL_EXT );
    ctrl_ext |= I82544_CTRL_EXT_PHY_RESET_DIR4;
    OUTL( ctrl_ext, ioaddr + I82544_CTRL_EXT );
    udelay( 20000 );

    ctrl_ext = INL( ioaddr + I82544_CTRL_EXT );    
    ctrl_ext &= ~I82544_CTRL_EXT_PHY_RESET4;
    OUTL( ctrl_ext, ioaddr + I82544_CTRL_EXT );
    udelay( 20000 );

    ctrl_ext = INL( ioaddr + I82544_CTRL_EXT );    
    ctrl_ext |= I82544_CTRL_EXT_PHY_RESET4;
    OUTL( ctrl_ext, ioaddr + I82544_CTRL_EXT );
    udelay( 20000 );

   
    phy_ctrl = mii_read_register( ioAddr, TM_PHY_ADDRESS, 0 );
    
// add by js 
#if 0 
    phy_ctrl &= 0xcfbf;
#else
    phy_ctrl |= 0x1200;
#endif 
    mii_write_register( ioAddr, TM_PHY_ADDRESS, 0, phy_ctrl );
/*     Wait for it to complete*/
    do 
    {
        phy_stat = mii_read_register( ioAddr, TM_PHY_ADDRESS, 1 );
        phy_stat = mii_read_register( ioAddr, TM_PHY_ADDRESS, 1 );
    } while( (phy_stat & 0x0020) == 0 );

/* Transfer speed and duplicity settings from PHY to MAC

   In theory the MAC is supposed to auto-configure from what the
   PHY has autonegotiated. In practice this does not seem to work
   (on the 82543 at least, it always thinks it is 1000MHz full
   duplex) and we have to transfer the settings from the PHY by
   hand. Additionally, the settings in the PHY ctrl register seem
   bogus, so we read the values out of the PHY specific status
   register instead.
*/    
    phy_pssr = mii_read_register( ioAddr, TM_PHY_ADDRESS, 17 );

    ctrl = tm_inl( ioAddr + TM_I82544_CTRL );
    ctrl &= ~(TM_I82544_CTRL_SPEED | TM_I82544_CTRL_FD);
    if( phy_pssr & (1<<13) )
    {
        ctrl |= TM_I82544_CTRL_FD;
    }
/* Transfer speed*/
    ctrl |= ((phy_pssr>>14)&3)<<8;

    ctrl |= TM_I82544_CTRL_FRCDPLX | TM_I82544_CTRL_FRCSPD;

    tm_outl( ctrl, ioAddr + TM_I82544_CTRL );

// Hang around here for a bit to let the device settle down. We
// don't seem to get away without this.
    
#endif
/* Set up interrupts*/

/* Clear any pending interrupts*/
    ctrl = tm_inl( ioAddr + TM_I82544_ICR );

/* clear all mask bits*/
    tm_outl( 0xFFFFFFFFl, ioAddr + TM_I82544_IMC );
/* Set interrupt bits for:*/
/* 0: transmit complete */
/* 1 = Transmit queue empty*/
/* 7 = Receiver timeout interrupt*/
#ifdef TM_TASK_SEND
    tm_outl( (1<<0)|(1<<1)|(1<<7), ioAddr + TM_I82544_IMS );
#else /* !TM_TASK_SEND */
    tm_outl( (1<<7), ioAddr + TM_I82544_IMS );
#endif /* !TM_TASK_SEND */
}

/****************************************************************************************/
/* Subroutine accepts bit patterns as defined in led.h and posts to 					*/
/* board registers resulting in display of value to board leds							*/
/****************************************************************************************/											 
int displayLED(unsigned int display_left,unsigned int display_right){
	int 			i;
	unsigned int 	*left,*right;
	left 	= DISPLAY_LEFT;					 /* Load register address for left LED		*/
	right	= DISPLAY_RIGHT;				 /* Load register address for right LED		*/
	*left 	= display_left;					 /* Load bit pattern to register left LED	*/
	*right	= display_right;				 /* Load bit pattern to register right LED	*/
									
	return 1;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: pci80321.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 47048    Output checksum: 45347
 */
