/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Test for IQ80321 board + Intel 8254x Device Driver
 * Filename: test1.c
 * Author: jshang
 * Date Created: 1/14/2003
 * $Source: source/kernel/ucos/xscale/iq80321/gnu/test1.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:08CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/* printf in console window*/
#include <stdio.h>

#include <trsocket.h>

/* define test type 
 * all tests are from TM_LOCAL_IP (board) to TM_PEER_IP (discard port)
 * 1: UDP send , length: 1472, 
 * 2: UDP send , length: 64
 * 3: TCP send , length: 1448
 * 4: TCP send , length: 1448*5  (jumbo)
 * 5: TCP send , length: 1448*10 (jumbo)
 * 6: TCP send , length: 64
 * 7: Ping to Peer IP, length: 100
 * 8: TCP discard port server, for two boards test
 *        and also may receive ping from peer
 * 9: ftp server
 * 10: IPsec with UDP
 * 11: UDP with 80 data length
 * 12: TCP with 80 data length
 */
#define TM_TEST_TYPE        1

/* comment it if offload ( checksum + TCP segmentation ) is disable  */
#define TM_OFFLOAD_ENABLE   

/* packet (data) length for each test */
int tlPktLength[20] = { 0, 1472, 64, 1448, 1448*5, 1448*10,64,100,0,0,
                       1372, 80,80};

/* you can NOT change local IP address */
#define TM_LOCAL_IP         "192.168.1.200"

#define TM_PEER_IP          "192.168.1.1"

/* stack size and memory for each task */
#define                     STK_SIZE   5120

unsigned long               Stk2[STK_SIZE];
unsigned long               Stk3[STK_SIZE];
#ifdef TM_TASK_SEND
unsigned long               Stk4[STK_SIZE];
#endif /* TM_TASK_SEND */
unsigned long               Stk9[STK_SIZE];

char                        Data9;

void                        treckTask(void *data);
void                        timerTask(void *data);
void                        recvTask(void *interfaceHandle);
#ifdef TM_TASK_SEND
void                        sendTask(void *interfaceHandle); 
#endif /* TM_TASK_SEND */
void                        pingCallback(int sockDesc);


int sendUdpNoCopy (int sd, struct sockaddr_in *peerIpAddr);
int sendTcpNoCopy (int sd, int echoFlag);


#ifndef TM_TRECK_PREEMPTIVE_KERNEL
int     tlConnectComplete;
int     tlRecvDataLength;
int     tlSentLength;
ttUserInterface  tlInterfaceHandle;
void socketCB(int sock, int flag)
{
    int retCode;
    ttUserMessage       recvBufferHandle;
    char      TM_FAR *  dataPtr;
    if (flag == TM_CB_RECV )
    {
        retCode = tfZeroCopyRecv( sock,
                                  &recvBufferHandle,
                                  &dataPtr,
                                  tlSentLength,
                                  0 );
        if (retCode > 0)
        {
            (void)tfFreeZeroCopyBuffer(recvBufferHandle);
            tlRecvDataLength-= retCode;
        }
    }
    else if (flag == TM_CB_CONNECT_COMPLT )
    {
        tlConnectComplete = 1;
        tlRecvDataLength  = 0;
        tlSentLength      = 1448;
    }
}


#endif /* ! TM_TRECK_PREEMPTIVE_KERNEL */








/* following is external function/macro/variables definitions */
#ifdef TM_OFFLOAD_ENABLE
int     tv82544OffloadEnable = 1;
#else /* !TM_OFFLOAD_ENABLE */
int     tv82544OffloadEnable = 0;
#endif /* !TM_OFFLOAD_ENABLE */

unsigned long   tvSocketSendCount;
unsigned long   tvInterruptMask;

double          tvLengthScale;

/* 82544 driver */
#define TM_8254X_IOCTL_REFILL           0x0001
#define TM_8254X_IOCTL_SEND_COMPLETE    0x0002
#define TM_RAMFS_BLOCKS     256
/*
 * Driver API prototypes
 */
int tf8254xOpen (   ttUserInterface interfaceHandle);
int tf8254xClose(   ttUserInterface interfaceHandle);
int tf8254xIoctl(   ttUserInterface interfaceHandle,
                    int             flag,
                    void TM_FAR *   optionPtr,
                    int             optionLen);
int tf8254xSend(    ttUserInterface interfaceHandle,
                    char TM_FAR *   dataPtr,
                    int             dataLength,
                    int             flag);
int tf8254xReceive( ttUserInterface        interfaceHandle,
                    char TM_FAR * TM_FAR * dataPtr,
                    int  TM_FAR *          dataLength,
                    ttUserBufferPtr        bufHandlePtr
#ifdef TM_DEV_RECV_OFFLOAD
                 ,  ttDevRecvOffloadPtr    devoRPtr
#endif /* TM_DEV_RECV_OFFLOAD */
                 );

int tf8254xGetPhysAddr(ttUserInterface interfaceHandle,
                       char TM_FAR *   physicalAddress);

int tf8254xScatDrvSend ( ttUserInterface  interfaceHandle,
                         ttUserPacketPtr  uPacketPtr );

/* IQ80321 board function */
void        IQ80321_ISR_INIT();
void        IQ80321_ISR_TIMER();
void        hal_clock_initialize(unsigned long);
int         LED[2] = { 0x3, 0x9f}; 
/* uc/os functions */
void        OSInit(void);
void        OSStart(void);
unsigned char OSTaskCreate(void (*task)(void *pd), 
                           void *pdata, 
                           unsigned int *ptos, 
                           unsigned char prio);
void        OSStatInit(void);
void        OSTimeDly(unsigned short ticks);





/* main program */

int main(void)
{
    Data9 = '9';
    IQ80321_ISR_INIT();
    
    OSInit();
    OSTaskCreate(treckTask, (void *)&Data9, (void *)&Stk9[STK_SIZE-1], 9);

/* tick 5 ms, should not change tick time */
    hal_clock_initialize(5000);

    OSStart();

    return 0;
}


void treckTask(void *data)
{
    struct sockaddr_storage localIpAddr;
    struct sockaddr_storage peerIpAddr;
    ttUserIpAddress         netMask;
    ttUserLinkLayer         linkLayerHandle;
    ttUserInterface         interfaceHandle;
    int                     sd;
    int                     tsd;
    int                     errorCode;
    int                     portNumber;
    int                     toPortNumber;
    int                     configFlags;
    int                     scatteredBufferCount;
    int                     treckError;
    ttUserMessage           bufferHandle;
    int                     recvLength;
    char                    *testPtr;
      unsigned long temp;    
/* print out test info in console window */

    if ( TM_TEST_TYPE <= 2 )
    {
        printf(" UDP Test ");
    }
    else if ( TM_TEST_TYPE <= 6 )
    {
        printf(" TCP Test ");
    }
    else
    {
        printf(" other Tests ");
    }
    if ( tv82544OffloadEnable )
    {
        printf(" with OFFLOAD,");
    }
    else
    {
        printf(" without OFFLOAD, ");
    }
    printf(" packet length:%d\n", tlPktLength[TM_TEST_TYPE]);
    printf(" Result (CPU percentage and Speed) in each 30 sec\n");

/* scale factor for each print out, shown in console window 
 * in 30 sec, we have tvSocketSendPacket packets already sent out 
 */
    if ( tlPktLength[TM_TEST_TYPE] > 1500 )
    {
/* for segmenation offload */
        tvLengthScale = 
            1514*(tlPktLength[TM_TEST_TYPE]/1448)*8/30.0/1000000l;
    }
    else if ( tlPktLength[TM_TEST_TYPE] <100 )
    {
/* for small pakcet, only print out number packet/sec */
        tvLengthScale = 0;
    }
    else
    {
        tvLengthScale = 1514*8/30.0/1000000l;
    }


/* enable timer interrupt, must be called */
    IQ80321_ISR_TIMER();

    
    tvSocketSendCount = 0;

/* Start Treck */
    tfInitTreck();
    tfStartTreck();
    
/* Use the Ethernet Link Layer (Including ARP) */
    linkLayerHandle=tfUseEthernet();

/* Add ethernet interface */

    interfaceHandle=tfAddInterface(
            "82544",
            linkLayerHandle,
            tf8254xOpen,
            tf8254xClose,
            tf8254xSend,
            tf8254xReceive,
			0,
            tf8254xIoctl,
            tf8254xGetPhysAddr,
            &errorCode);
/* for IQ80321+82544, must use scatter send driver if using offload */
#ifdef TM_USE_DRV_ONE_SCAT_SEND    
    errorCode = tfUseInterfaceOneScatSend( interfaceHandle,
                                            tf8254xScatDrvSend );
#endif /* TM_USE_DRV_ONE_SCAT_SEND */
/* send/recv to/from discard port */    
    portNumber      = 50;    
    toPortNumber    = 9;

/* Allow scatter send */
    configFlags     =  TM_DEV_SCATTER_SEND_ENB;
    if (configFlags & TM_DEV_SCATTER_SEND_ENB)
    {
        scatteredBufferCount = 12;
    }
    else
    {
        scatteredBufferCount = 1;
    }

    localIpAddr.ss_family       = PF_INET;
    localIpAddr.ss_port         = htons(portNumber);
    localIpAddr.addr.ipv4.sin_addr.s_addr = inet_addr(TM_LOCAL_IP);

    peerIpAddr.ss_family        = PF_INET;
    peerIpAddr.ss_port          = htons(toPortNumber);
    peerIpAddr.addr.ipv4.sin_addr.s_addr = inet_addr(TM_PEER_IP);
   
    netMask                     = htonl(0xffffff00); 


/* Config the Interface */    
    errorCode = tfConfigInterface(
                interfaceHandle,
                localIpAddr.addr.ipv4.sin_addr.s_addr,
                netMask,
                configFlags,  
                scatteredBufferCount,  
                0);

/* udp socket */
    sd  = socket(PF_INET,SOCK_DGRAM,0);
/* tcp socket */
    tsd = socket(PF_INET,SOCK_STREAM,0);

#ifdef TM_TRECK_PREEMPTIVE_KERNEL
/* Start the tasks */
    OSTaskCreate(timerTask, (void *)&interfaceHandle, (void *)&Stk2[STK_SIZE-1],2);
    OSTaskCreate(recvTask, (void *)&interfaceHandle, (void *)&Stk3[STK_SIZE-1], 3);
#ifdef TM_TASK_SEND
    OSTaskCreate(sendTask, (void *)&interfaceHandle, (void *)&Stk4[STK_SIZE-1], 4); 
#endif /* TM_TASK_SEND */
/* creat statistic task as priority 1, it will print out result each 30 second  */
    OSStatInit();
    OSTimeDly(250);
#endif /* TM_TRECK_PREEMPTIVE_KERNEL */


    switch (TM_TEST_TYPE)
    {
        case 10:
            tvLengthScale = 0;
        case 11:
        case 1:
        case 2:
/* UDP send test */
            errorCode = sendUdpNoCopy(sd, &peerIpAddr.addr.ipv4);
            break;
        case 3:
        case 4:
        case 5:
        case 6:
        case 12:
/* TCP send test */
#ifndef TM_TRECK_PREEMPTIVE_KERNEL
            tlInterfaceHandle   = interfaceHandle;
            tfRegisterSocketCB(tsd, (ttUserSocketCBFuncPtr)socketCB,
                                TM_CB_CONNECT_COMPLT);

#endif /* !TM_TRECK_PREEMPTIVE_KERNEL */
#ifdef TM_USE_TCP_PACKET
            temp = 1;
            setsockopt(tsd, IP_PROTOTCP,TM_TCP_PACKET, (const char *)&temp,sizeof(temp));
#endif /* TM_USE_TCP_PACKET */
            errorCode = connect(tsd,
                         (const struct sockaddr TM_FAR *)&peerIpAddr,
                         sizeof(peerIpAddr)
                         );
            if (errorCode!=TM_ENOERROR)
            {
                treckError = tfGetSocketError(tsd);
                printf("treckError = %d\n", treckError);
            }
            errorCode = sendTcpNoCopy(tsd, 0);
            break;
        case 7:
/* ping test */
            tfPingOpenStart(
                TM_PEER_IP,
                1,
                100, /* 100 bytes of data */
                (ttPingCBFuncPtr) pingCallback);
            while(1)
            {
                ;
            }
            break;

        case 8:
/* TCP discard port server */
            errorCode = bind(tsd,
                             (struct sockaddr TM_FAR *)
                                    &localIpAddr.addr.ipSockAddr,
                             sizeof(localIpAddr));
            if ( errorCode == TM_SOCKET_ERROR)
            {
                break;
            }
          
            temp = 256*1024;
            setsockopt(tsd, SOL_SOCKET, SO_RCVBUF, (const char *)&temp, 4);

            errorCode = listen(tsd, 1);
            if ( errorCode == TM_SOCKET_ERROR)
            {
                break;
            }
            sd = accept(tsd, (struct sockaddr TM_FAR *)0,(int TM_FAR  *)0);
            recvLength = 1500;
            while (1)
            {
                errorCode = tfZeroCopyRecv(sd,
                                    &bufferHandle,
                                    &testPtr,
                                    recvLength,
                                    0);
        
                if (errorCode == TM_SOCKET_ERROR)
                {       
                    errorCode = tfGetSocketError(sd);
                    printf("tfZeroCopyRecv error = %d\n", errorCode);
                    break;
                }
                else
                {
                    if (bufferHandle != (ttUserMessage) 0)
                    {
                        tfFreeZeroCopyBuffer(bufferHandle);
                    }
                }
            }
            break;
        case 9:
            (void)tfRamfsInit(TM_RAMFS_BLOCKS);
            tfFtpdUserStart(TM_FS_ALLCMND_MASK,2,2,1200,TM_BLOCKING_ON);
            break;
        default:
            break;
    }
/* never exit from a task in ucos */
    while (1)
    {
        ;
    }
}

int sendTcpNoCopy(int sd, int echoFlag)
{
    unsigned long   temp;
    char            *testPtr;
    ttUserMessage   bufferHandle;
    int             errorCode;
    int             sentLength;
    int             recvLength;
    int             length;
    int             treckError;

    

    length = tlPktLength[TM_TEST_TYPE];
/* change send queue size */
    temp = 16384;
    if ( length > 1500)
    {
/* for jumbo packet, it is better to increase send queue size more */
        temp = 16384*4;
    }
    setsockopt(sd, SOL_SOCKET, SO_SNDBUF, (const char *)&temp, 4);

   
    while (1) 
    {
#ifndef TM_TRECK_PREEMPTIVE_KERNEL
        tfTimerUpdate();
        tfTimerExecute(); 
        (void)tfIoctlInterface(tlInterfaceHandle,
                               TM_8254X_IOCTL_REFILL|TM_8254X_IOCTL_SEND_COMPLETE,
                               (void TM_FAR *)0, 0);
        while (tfCheckReceiveInterface(tlInterfaceHandle)
                        == TM_ENOERROR)
        {
            tfRecvInterface(tlInterfaceHandle); 
        }

        if ( tlConnectComplete && (tlRecvDataLength <= 0 ))
        {
#endif /* TM_TRECK_PREEMPTIVE_KERNEL */

        bufferHandle = tfGetZeroCopyBuffer(length,&testPtr);
        if ( bufferHandle == (ttUserMessage)0)
        {
            printf("tfGetZeroCopyBuffer failed\n");
            break;
        }
        *((ttUser32Bit*)testPtr) = tvSocketSendCount ;
        sentLength = tfZeroCopySend(sd,bufferHandle,length,0); 

        if (sentLength == -1)
        {
            treckError = tfGetSocketError(sd);
            errorCode  = tfGetSocketError(sd);
/* because we use preemptive RTOS, so errorCode can not be TM_EWOULDBLOCK */
            printf("TCP send error %d, exit \n", errorCode);
            break;
        }
        else
        {
            tvSocketSendCount++;
        }
/* following part is for TCP echo test, 
 * right now we don't test this case 
 */
        if ( echoFlag)
        {
#ifdef TM_TRECK_PREEMPTIVE_KERNEL
            recvLength = 0;
            do
            {
                errorCode = tfZeroCopyRecv(sd,
                                    &bufferHandle,
                                    &testPtr,
                                    sentLength,
                                    0);
        
                if (errorCode == TM_SOCKET_ERROR)
                {       
                    errorCode = tfGetSocketError(sd);
                    printf("tfZeroCopyRecv error = %d\n", errorCode);
                    break;
                }
                else
                {
                    recvLength += errorCode;
                    if (bufferHandle != (ttUserMessage) 0)
                    {
                        tfFreeZeroCopyBuffer(bufferHandle);
                    }
                }
            }
            while ( recvLength < sentLength );
#else /* !TM_TRECK_PREEMPTIVE_KERNEL*/
            tlSentLength    = sentLength;
            tlRecvDataLength = sentLength;
        }
#endif /* !TM_TRECK_PREEMPTIVE_KERNEL */
        }
    } 
    return errorCode;
}


int sendUdpNoCopy(int sd, struct sockaddr_in *toAddress)
{
    unsigned long   temp;
    char            *testPtr;
    ttUserMessage   bufferHandle;
    int             sentLength;
    int             errorCode;
    int             length;
    
    length = tlPktLength[TM_TEST_TYPE];

    temp = 16*1024;
    setsockopt(sd, SOL_SOCKET, SO_SNDBUF, (const char *)&temp, 4);
   
    while (1) 
    {
        bufferHandle=tfGetZeroCopyBuffer(length,&testPtr);
        if ( bufferHandle ==(ttUserMessage)0)
        {
            printf("no memory \n");
            break;
        }
        *((unsigned long *)testPtr) = tvSocketSendCount;
        
        sentLength = tfZeroCopySendTo(sd,bufferHandle,length,0,
                        (const struct sockaddr TM_FAR *)toAddress,
                        (int)sizeof(struct sockaddr)); 

        if (sentLength == -1)
        {
            errorCode = tfGetSocketError(sd);
/* because we use preemptive RTOS, so errorCode can not be TM_EWOULDBLOCK */
            printf("UDP send error %d, exit \n", errorCode);
            break;
        }
        else
        {
            tvSocketSendCount++;
        }
    }
    return errorCode;
}
#ifdef TM_TRECK_PREEMPTIVE_KERNEL
void timerTask(void *interfaceHandle)
{
    int ticks=0;
    int count = 0;

    while (1) 
    {
        ticks++;
        tfTimerUpdate();
        tfTimerExecute(); 
        OSTimeDly(1); 

        (void)tfIoctlInterface(*(ttUserInterface *)interfaceHandle,
                               TM_8254X_IOCTL_REFILL|TM_8254X_IOCTL_SEND_COMPLETE,
                               (void TM_FAR *)0, 0);
/* led light in order to check stack is running */
        if (ticks == 200)
        {
            ticks = 0;
            displayLED( LED[count], LED[1-count] );
            count = 1- count;
        }
    }
}

void recvTask(void *interfaceHandle)
{
    while (1) 
    {
 	    (void)tfWaitReceiveInterface(*(ttUserInterface *)interfaceHandle);
        (void)tfRecvInterface(*(ttUserInterface *)interfaceHandle); 
    }
}
#endif /* TM_TRECK_PREEMPTIVE_KERNEL */

/*
 * Called by the stack when a PING reply is received.
 */ 
void pingCallback(int sockDesc)
{
    printf("Ping reply\n");
}


#ifdef TM_TASK_SEND
void sendTask(void *interfaceHandle)
{
    while (1) 
    {
        tfWaitSentInterface(*(ttUserInterface *)interfaceHandle);
        tfSendCompleteInterface(*(ttUserInterface *)interfaceHandle, 
                TM_DEV_SEND_COMPLETE_DRIVER);
    }
}
#endif /* TM_TASK_SEND */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: test1.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6255    Output checksum: 50953
 */
