/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:
 * memory leak detection for tm_get_raw_buffer and tm_free_raw_buffer only
 *
 * Filename: mddetect.c
 * Author: Jin Zhang
 * Date Created: 1/10/2003
 * $Source: source/kernel/win32/intelx86/mldetect.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:16CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#define   GET_RAWBUFFER    1
#define   FREE_RAWBUFFER   (!(GET_RAWBUFFER))

typedef struct tsMyBufferList
{
    struct tsMyBufferList *  blPrevPtr;
    struct tsMyBufferList *  blNextPtr;
    int                      blGetOrFree;
    int                      blRawNumber;
    char                     blFileName[80];
    int                      blLine;
    long                     blAddress;
    int                      blSize;
    long                     blTime;
}
ttMyBufferList;
typedef ttMyBufferList * ttMyBufferListPtr;

ttMyBufferListPtr     bufferHeadPtr        = (ttMyBufferListPtr)0;
ttMyBufferListPtr     bufferTailPtr        = (ttMyBufferListPtr)0;
ttMyBufferListPtr     badFreeBufferHeadPtr = (ttMyBufferListPtr)0;
int                   LineNumber;

ttMyBufferListPtr  tfNodeConstruct(char * aLine);
int                tfAddNode(ttMyBufferListPtr   newNodePtr);
int                tfFreeNode(ttMyBufferListPtr   freeNodePtr);
void               tfPrintNodeList(FILE * outfile, ttMyBufferListPtr headPtr);
int                tfAnalyzeMemoryUsage(FILE *infile, FILE* outfile);


ttMyBufferListPtr  tfNodeConstruct(char * aLine)
{
    ttMyBufferListPtr   aNodePtr = (ttMyBufferListPtr)0;
	char               *token;
    
    LineNumber ++;
    if(aLine)
    {
        aNodePtr = (ttMyBufferListPtr)malloc(sizeof(ttMyBufferList));
        assert(aNodePtr != (ttMyBufferListPtr)0);
        memset(aNodePtr, 0, sizeof(ttMyBufferList));
        
        token=strtok(aLine,","); 
        if(!token) goto ERRORRETURN;
        if(!memcmp(token, "GetBuffer", 9))
        {
            aNodePtr->blGetOrFree = GET_RAWBUFFER;
        }
        else if(!memcmp(token, "FreeBuffer", 10))
        {
            aNodePtr->blGetOrFree = FREE_RAWBUFFER;
        }
        else
        {
            goto ERRORRETURN;
        }

/* tvGetRawCount or tvFreeRawCount */
	    token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        aNodePtr->blRawNumber = atoi(token);
        if(!aNodePtr->blRawNumber) goto ERRORRETURN;

 /* Time stamp */
	    token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        aNodePtr->blTime = atol(token);

/* buffer pointer address */
	    token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        aNodePtr->blAddress = strtol(token, (char**)0, 16);
        if(!aNodePtr->blAddress) goto ERRORRETURN;

/* Buffer Size */
	    token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        aNodePtr->blSize = atoi(token);
        if(!aNodePtr->blSize) goto ERRORRETURN;

/* File name */
        token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        memcpy(aNodePtr->blFileName, token, strlen(token));
        aNodePtr->blFileName[strlen(token)] = '\0';

        token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        if(!memcmp(token, "line", 4)) goto ERRORRETURN;

/* line number */
	    token=strtok(NULL,","); 
        if(!token) goto ERRORRETURN;
        aNodePtr->blLine = atoi(token);
        if(!aNodePtr->blLine) goto ERRORRETURN;
    }
    goto RETURN;
ERRORRETURN:
    printf("-- INVALID LINE ENCOUNTERED AT LINE %d , SKIPPED --\n", LineNumber);
    free(aNodePtr);
    aNodePtr = (ttMyBufferListPtr)0;
RETURN:
    return aNodePtr;
}


int  tfAddNode(ttMyBufferListPtr   newNodePtr)
{
    if(!bufferHeadPtr)
    {
        bufferHeadPtr = newNodePtr;
    }

    if(bufferTailPtr)
    {
        bufferTailPtr->blNextPtr = newNodePtr;
        newNodePtr->blPrevPtr = bufferTailPtr;
    }
    
    bufferTailPtr = newNodePtr;
    return 0;

}


int tfFreeNode(ttMyBufferListPtr   freeNodePtr)
{
    ttMyBufferListPtr       nextPtr;

    nextPtr = bufferTailPtr;

    while(nextPtr)
    {
        if(nextPtr->blAddress == freeNodePtr->blAddress)
        {
            break;
        }
        nextPtr = nextPtr->blPrevPtr;
    }

    if(nextPtr)
    {
        if(bufferHeadPtr == nextPtr)
        {
            bufferHeadPtr = nextPtr->blNextPtr;
        }
        
        if(bufferTailPtr == nextPtr)
        {
            bufferTailPtr = nextPtr->blPrevPtr;
        }
        
        if(nextPtr->blPrevPtr)
        {
            nextPtr->blPrevPtr->blNextPtr = nextPtr->blNextPtr;
        }
        
        if(nextPtr->blNextPtr)
        {
            nextPtr->blNextPtr->blPrevPtr = nextPtr->blPrevPtr;
        }
        
        free(nextPtr);
        return 0;
    }
    else
    {
        if(badFreeBufferHeadPtr)
        {
            freeNodePtr->blNextPtr = badFreeBufferHeadPtr;
            badFreeBufferHeadPtr->blPrevPtr = freeNodePtr;
        }
        badFreeBufferHeadPtr = freeNodePtr;
        return -1;

    }
}

void tfPrintNodeList(FILE * outfile, ttMyBufferListPtr headPtr)
{
    ttMyBufferListPtr     prevPtr;
    while(headPtr)
    {
/*
        if(headPtr->blGetOrFree == GET_RAWBUFFER)
        {
            fprintf(outfile, "GetBuffer ");
        }
        else if(headPtr->blGetOrFree == FREE_RAWBUFFER)
        {
            fprintf(outfile, "FreeBuffer ");
        }
*/
        fprintf(outfile, 
                "t=%-8ld idx=%-5d 0x%-x sz=%-5d ln=%-6d %-s\n", 
                headPtr->blTime,
                headPtr->blRawNumber,
                headPtr->blAddress,
                headPtr->blSize,
                headPtr->blLine, 
                headPtr->blFileName);

        prevPtr = headPtr;
        headPtr = headPtr ->blNextPtr;
        free(prevPtr);
    }
}




/* If this file is used in treck demo, we aleady have a main() */
#ifndef TM_TRECK_DEMO
int main(int argc, char *argv[])
{
    FILE * infile;
    char * filename = "memoryleak.dat";

    LineNumber = 0;

    if(argc < 2)
    {
        infile = fopen(filename, "r");
        if(!infile)
        {
            printf("memoryleak.dat can't be opened. \nUSAGE: mldetect <input filename>\n");
            return -1;
        }

    }
    else
    {

       infile = fopen(argv[1], "r");
       assert(infile != (FILE*)0);
    }

	return tfAnalyzeMemoryUsage(infile, stdout);
}
#endif /* TM_TRECK_DEMO*/


/* Exportable public API, used by both main of this file and TreckDemo */
int tfAnalyzeMemoryUsage(FILE *infile, FILE* outfile)
{
    ttMyBufferListPtr   newNodePtr;
    char                buf[1024];
    char *              readbuf;
    int                 errorCode;

    bufferHeadPtr        = (ttMyBufferListPtr)0;
    bufferTailPtr        = (ttMyBufferListPtr)0;
    badFreeBufferHeadPtr = (ttMyBufferListPtr)0;

    while((readbuf = fgets(buf, 1024, infile)) != (char *)0)
    {
        newNodePtr = tfNodeConstruct(readbuf);
        if(!newNodePtr)
        {
            continue;
        }

        if(newNodePtr->blGetOrFree == GET_RAWBUFFER)
        {
            tfAddNode(newNodePtr);
        }
        else if(newNodePtr->blGetOrFree == FREE_RAWBUFFER)
        {
            errorCode = tfFreeNode(newNodePtr);
            if(errorCode == 0)
            {
                free(newNodePtr);
            }
        }

    }

    errorCode = fprintf(outfile, "------------------ UNFREED MEMORY ------------------\n");
    if (errorCode < 10)
    {
/* file write error */
        return 1;
    }
    tfPrintNodeList(outfile, bufferHeadPtr);
    fprintf(outfile, "\n\n------------------ WRONGLY FREED MEMORY ------------------\n");
    tfPrintNodeList(outfile, badFreeBufferHeadPtr);

    return 0;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: mldetect.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 45783    Output checksum: 25187
 */
