/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: print a string in the right location with good color
 *
 * Filename: trprint.c
 * Author: Bryan Wang
 * Date Created:
 * $Source: source/kernel/win32/intelx86/trprint.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:17CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifdef _WIN32
/* compile and run on Win32 platform */
/* to avoid an warning in MS VC++ header file */
struct _RPC_ASYNC_STATE;
#include <windows.h>
#endif /* _WIN32 */

#include <stdio.h>

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

#define TM_COLOR_PRINT_MAX_LINE 256

/* print a string in the right location with good color */
/* void tfColorPrint(int msgType, char *msgStr) */
int tfColorPrint(int msgType, const char TM_FAR * format, ...)
{

#ifdef _WIN32
    HANDLE                      outputHandle;
    CONSOLE_SCREEN_BUFFER_INFO  bufferInfo;
    COORD*                      pPos;
    WORD                        foregroundColor;
    char                        buffer[TM_COLOR_PRINT_MAX_LINE];
#endif /* _WIN32 */

    int                         nResult;
    va_list                     args;
    
    va_start(args, format);
    nResult = tfVSPrintF(buffer, format, args);
    va_end(args);

#ifdef _WIN32
    outputHandle = GetStdHandle(STD_OUTPUT_HANDLE);
    GetConsoleScreenBufferInfo(outputHandle, &bufferInfo);
    pPos = &bufferInfo.dwCursorPosition;


    switch(msgType)
    {
    case TM_PRINT_TYPE_TEST:
        foregroundColor = FOREGROUND_GREEN;
        pPos->X         = 0;
        break;
    case TM_PRINT_TYPE_INFO:
        foregroundColor = FOREGROUND_GREEN;
        break;
    case TM_PRINT_TYPE_PASS:
        foregroundColor = FOREGROUND_GREEN;
        pPos->X         = 74;
        pPos->Y         -= 1;
        break;
    case TM_PRINT_TYPE_FAIL:
        foregroundColor = FOREGROUND_RED;
        pPos->X         = 74;
        pPos->Y         -= 1;
        break;
    case TM_PRINT_TYPE_WARN:
        foregroundColor = FOREGROUND_BLUE;
        pPos->X         = 74;
        break;
    default:
        foregroundColor = FOREGROUND_GREEN | FOREGROUND_RED |FOREGROUND_BLUE;
        pPos->X         = 0;
        pPos->Y         = 0;
        break;
    }

    SetConsoleCursorPosition(outputHandle, *pPos);
    
    foregroundColor |= FOREGROUND_INTENSITY;
    SetConsoleTextAttribute(
            outputHandle,
            foregroundColor);

    printf("%s", buffer);

    SetConsoleTextAttribute(
            GetStdHandle(STD_OUTPUT_HANDLE),
            FOREGROUND_GREEN | FOREGROUND_RED |FOREGROUND_BLUE);

#else /* _WIN32 */
    printf("%s", buffer);
#endif /* _WIN32 */

    return nResult;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: trprint.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 12338    Output checksum: 28238
 */
