/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: pcap driver for windows,
 * 
 * Important node: 
 *   To run Treck with Winpcap driver, You need to add packet.lib found in 
 *   the same path with this file to your project, and also install the DLLs
 *   from the link below. 
 *   http://winpcap.polito.it/install/bin/WinPcap_3_0_a.exe
 *
 *   Since WinPcap is used, This module includes software developed by 
 *   the Politecnico di Torino, and its contributors.
 *
 * Filename: trwpcap.c
 * Author: jshang
 * Date Created: 08/15/2002
 * $Source: source/kernel/win32/intelx86/trwpcap.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2011/10/27 15:31:39CDT $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */



#ifdef _WIN32
/* compile and run on Win32 platform */
/* to avoid an warning in MS VC++ header file */
struct _RPC_ASYNC_STATE;
#include <winsock2.h>
#endif /* _WIN32 */


#include <stdio.h>
#include <conio.h>

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#if defined(TM_LOCK_NEEDED) || (defined(TM_ERROR_CHECKING) && defined(TM_USE_ANSI_LINE_FILE))
#include <trglobal.h>
#endif /* TM_LOCK_NEEDED || (TM_ERROR_CHECKING && TM_USE_ANSI_LINE_FILE) */

#undef  TM_WIN32_MAX_INTF
#define TM_INTF_MTU                             5000
#define TM_WIN32_MAX_INTF                       10

#define TM_WIN32_TASK_STACK_SIZE                128*1024

/* buffer size for local buffer to hold the ether multicast address list
 * realTeck supports 32, 3Com supports 64
 */
#define TM_WIN32_MAX_MCAST_ADRR                 32

#define TM_WIN32_SCATTBUFFER 

#define TM_OID_DATA_LEN        256
#define TM_ADAPTER_NAME_LEN    256

/* following definitions and structures obtained from pcap 
 * please refer original file for description */
#define Max_Num_Adapter                         TM_WIN32_MAX_INTF

/*  Ndis Packet Filter Bits (OID_GEN_CURRENT_PACKET_FILTER). */

#define NDIS_PACKET_TYPE_DIRECTED               0x0001
#define NDIS_PACKET_TYPE_MULTICAST              0x0002
#define NDIS_PACKET_TYPE_ALL_MULTICAST          0x0004
#define NDIS_PACKET_TYPE_BROADCAST              0x0008
#define NDIS_PACKET_TYPE_SOURCE_ROUTING         0x0010
#define NDIS_PACKET_TYPE_PROMISCUOUS            0x0020
#define NDIS_PACKET_TYPE_SMT                    0x0040
#define NDIS_PACKET_TYPE_ALL_LOCAL              0x0080
#define NDIS_PACKET_TYPE_MAC_FRAME              0x8000
#define NDIS_PACKET_TYPE_FUNCTIONAL             0x4000
#define NDIS_PACKET_TYPE_ALL_FUNCTIONAL         0x2000
#define NDIS_PACKET_TYPE_GROUP                  0x1000

#define MAX_LINK_NAME_LENGTH                    64

#define OID_802_3_PERMANENT_ADDRESS             0x01010101
#define OID_802_3_CURRENT_ADDRESS               0x01010102
#define OID_802_3_MULTICAST_LIST                0x01010103
#define OID_802_3_MAXIMUM_LIST_SIZE             0x01010104

#define OID_GEN_VENDOR_DESCRIPTION              0x0001010D
#define OID_GEN_HARDWARE_STATUS                 0x00010102
#define OID_GEN_MEDIA_CONNECT_STATUS            0x00010114

#define ADAPTER_NAME_LENGTH 256 + 12    
typedef struct _ADAPTER  { 
    HANDLE  hFile;      
    TCHAR   SymbolicLink[MAX_LINK_NAME_LENGTH]; 
    int     NumWrites;  
    HANDLE  ReadEvent;  
    UINT    ReadTimeOut;

/* fields added with v3.1 winpcap */    
    CHAR Name[ADAPTER_NAME_LENGTH];
    void *pWanAdapter;
    UINT Flags;
} ADAPTER, *LPADAPTER;

typedef struct _PACKET {  
    HANDLE       hEvent;
    OVERLAPPED   OverLapped;
    PVOID        Buffer;    
    UINT         Length;    
    UINT         ulBytesReceived;
    BOOLEAN      bIoComplete;   
}  PACKET, *LPPACKET;

struct _PACKET_OID_DATA {
    ULONG   Oid;
    ULONG   Length;
    UCHAR   Data[6 * TM_WIN32_MAX_MCAST_ADRR];
}; 
typedef struct _PACKET_OID_DATA PACKET_OID_DATA, *PPACKET_OID_DATA;

struct bpf_hdr {
    struct  timeval bh_tstamp;
    UINT    bh_caplen;      
    UINT    bh_datalen; 
    USHORT  bh_hdrlen;  
};

typedef struct NetType
{
    UINT LinkType;  /* The MAC of the current network adapter */
    UINT LinkSpeed; /* The speed of the network in bits per second */
    UINT v31LinkSpeed; /* LinkSpeed redefined to be ULONGLONG in v3.1 */
    UINT v31Pad; /* avoid data corruption */
}NetType;

/* NOTE: we should be using the winpcap public APIs instead of these
   packet.dll internal APIs, the public APIs start with "pcap_". Ethereal
   uses only the winpcap public APIs. */
PCHAR   PacketGetVersion();
BOOLEAN PacketSetMinToCopy(LPADAPTER AdapterObject,int nbytes);
BOOLEAN PacketSetNumWrites(LPADAPTER AdapterObject,int nwrites);
BOOLEAN PacketSetMode(LPADAPTER AdapterObject,int mode);
BOOLEAN PacketSetReadTimeout(LPADAPTER AdapterObject,int timeout);
BOOLEAN PacketSetBuff(LPADAPTER AdapterObject,int dim);
LPADAPTER PacketOpenAdapter(LPTSTR AdapterName);
BOOLEAN PacketSendPacket(LPADAPTER AdapterObject,LPPACKET pPacket,
                      BOOLEAN Sync);
LPPACKET PacketAllocatePacket(void);
VOID    PacketInitPacket(LPPACKET lpPacket,PVOID  Buffer,UINT  Length);
VOID    PacketFreePacket(LPPACKET lpPacket);
BOOLEAN PacketReceivePacket(LPADAPTER AdapterObject,LPPACKET lpPacket,
                      BOOLEAN Sync);
BOOLEAN PacketSetHwFilter(LPADAPTER AdapterObject,ULONG Filter);
/* the following API is significantly changed in v3.1 of winpcap:
   BOOLEAN PacketGetAdapterNames(PTSTR pStr,PULONG  BufferSize); */
/* the following API is removed in v3.1 of winpcap:
   BOOLEAN PacketGetNetInfo(LPTSTR AdapterName, PULONG netp, PULONG maskp); */

BOOLEAN PacketRequest(LPADAPTER  AdapterObject,BOOLEAN Set,
                      PPACKET_OID_DATA  OidData);
HANDLE  PacketGetReadEvent(LPADAPTER AdapterObject);
BOOLEAN PacketSetDumpName(LPADAPTER AdapterObject, void *name, int len);
BOOL    PacketStopDriver();
VOID    PacketCloseAdapter(LPADAPTER lpAdapter);
BOOLEAN PacketGetNetType (LPADAPTER AdapterObject,NetType *type);

/* use the winpcap public APIs "pcap_*" whenever possible, for portability
   with different versions of winpcap */
int  pcap_lookupnet(
    const char *device, tt32BitPtr netp, tt32BitPtr maskp, char *errbuf);
struct pcap_addr {
    struct pcap_addr *next;
    struct sockaddr *addr;      /* address */
    struct sockaddr *netmask;   /* netmask for that address */
    struct sockaddr *broadaddr; /* broadcast address for that address */
    struct sockaddr *dstaddr;   /* P2P destination address for that address */
};
typedef struct pcap_addr pcap_addr_t;
/*
 * Item in a list of interfaces.
 */
struct pcap_if {
    struct pcap_if *next;
    char *name;     /* name to hand to "pcap_open_live()" */
    char *description;  /* textual description of interface, or NULL */
    struct pcap_addr *addresses;
    tt32Bit flags;  /* PCAP_IF_ interface flags */
};
typedef struct pcap_if pcap_if_t;

#define PCAP_IF_LOOPBACK    0x00000001  /* interface is loopback */

int pcap_findalldevs(pcap_if_t **, char *);
void pcap_freealldevs(pcap_if_t *);

typedef struct tsWin32InterfaceInfo 
{
    int                         infoIntfId;
    ttUserInterface             infoIfHandle;
    ttUserInterface             infoRcvIfHandle;
    char                        *infoReadBuf;
#ifdef TM_WIN32_SCATTBUFFER
    char                        infoScatBuf[TM_INTF_MTU];
    int                         infoScatLen;
#endif /* TM_WIN32_SCATTBUFFER */
    char                        infoBuf[256000];  
    LPADAPTER                   infoLpAdapter ;
    LPPACKET                    infoRecvLpPacket;
    LPPACKET                    infoSendLpPacket;
    int                         mcastListSize;
    char                        physAddr[TM_ETHERNET_PHY_ADDR_LEN];
} ttWin32InterfaceInfo;    

typedef ttWin32InterfaceInfo * ttWin32InterfaceInfoPtr;

void tfWin32RecvTask  ();
void tfWin32TimerTask ();
void tfWin32RecvIsr   ();

int tfWin32DrvOpen(ttUserInterface interfaceHandle);
int tfWin32DrvClose(ttUserInterface interfaceHandle);

int tfWin32DrvSend(
    ttUserInterface     interfaceHandle,
    char   TM_FAR       *dataPtr,
    int                 dataLength,
    int                 flag );

int tfWin32DrvRecv(
    ttUserInterface     interfaceHandle,
    ttCharPtr TM_FAR    *dataPtrPtr,
    ttIntPtr            dataSizePtr,
    ttUserBufferPtr     packetPtrPtr );

int tfWin32DrvGetPhyAddr( 
    ttUserInterface     interfaceHandle,
    char TM_FAR *       physicalAddress );

int tfWin32Ioctl (
    ttUserInterface interfaceHandle,
    int             flag,
    void TM_FAR *   optionPtr,
    int             optionLen );

static int tfWin32AdapterIndexToName(
    int     index, 
    char*   name, int* 
    lenPtr);

int                     tlWin32NumIntf = 0;
HANDLE                  tlTimerThreadHandle = 0;
HANDLE                  tvReceiveThreadHandle = 0;
ttWin32InterfaceInfo    tlWin32IntfInfo[TM_WIN32_MAX_INTF];
/* in version 3.1 of winpcap, PCAP_ERRBUF_SIZE is 256 bytes */
#define PCAP_ERRBUF_SIZE 256
char                    tlPcapErrbuf[PCAP_ERRBUF_SIZE+1];


/*
 * This code was moved here from trsocket.h.
 * If you modify this, please see the TreckDemo.h file.
 */
/* WIN32 NIC media type */
typedef enum _NDIS_MEDIUM_TYPE {
    NdisMedium802_3,
    NdisMedium802_5,
    NdisMediumFddi,
    NdisMediumWan,
    NdisMediumLocalTalk,
    NdisMediumDix,            /* defined for convenience, not a real medium */
    NdisMediumArcnetRaw,
    NdisMediumArcnet878_2,
    NdisMediumAtm,
    NdisMediumWirelessWan,
    NdisMediumIrda,
    NdisMediumMax             /* Not a real medium, defined as an upper-bound */
} NDIS_MEDIUM_TYPE;

/* WIN32 NIC Hardware status codes (OID_GEN_HARDWARE_STATUS). */
typedef enum _NDIS_HW_STATUS {
    NdisHardwareStatusReady,
    NdisHardwareStatusInitializing,
    NdisHardwareStatusReset,
    NdisHardwareStatusClosing,
    NdisHardwareStatusNotReady
} NDIS_HW_STATUS;


/* descriptive information of an Win32 NIC, for testing and demo */
#define TM_WIN32_NIC_DRIVE_DESC_LEN     256
#define TM_WIN32_NIC_NAME_LEN           256
typedef struct tsWin32NicInfo
{
    char                name[TM_WIN32_NIC_NAME_LEN * 2]; /* WCHAR */
    char                driverDesc[TM_WIN32_NIC_DRIVE_DESC_LEN];
    int                 nameLen;
    NDIS_HW_STATUS      status;
    NDIS_MEDIUM_TYPE    type;
    int                 speed;  /* bps */
    char                phyAddr[tm_align(TM_ETHERNET_PHY_ADDR_LEN)];
    int                 ipAddr; /* Windows assigned IP address */
    int                 netMask;/* Windows assigned netmask */
} ttWin32NicInfo;

/* Get the information of a Win32 NIC by index */
int tfWin32GetNicInfo(ttWin32NicInfo TM_FAR * infoPtr, int index);

/* Get the number of NICs */
int tfWin32GetNicCount(void);

/* Check if the Ethernet link is connected */
int tfWin32GetConnectStatus(ttUserInterface interfaceHandle);



int tfWin32DrvGetPhyAddr ( 
    ttUserInterface interfaceHandle,
    char TM_FAR *   physicalAddress )
{
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;
    PPACKET_OID_DATA    oidDataPtr;
    char                oidDataData[TM_OID_DATA_LEN];
    int                 errorCode;

/* prepare the data structrue for driver command */
    oidDataPtr = (PPACKET_OID_DATA)oidDataData;
    oidDataPtr->Length = 6;
/* get the hardware address */
    oidDataPtr->Oid = OID_802_3_CURRENT_ADDRESS;
    win32IntfInfoPtr = (ttWin32InterfaceInfoPtr)
                            tfDeviceGetPointer(interfaceHandle);
    if (win32IntfInfoPtr != (ttWin32InterfaceInfoPtr)0) 
    {
        tm_memset(physicalAddress, 0, TM_ETHERNET_PHY_ADDR_LEN);
        errorCode = PacketRequest(
            win32IntfInfoPtr->infoLpAdapter,FALSE,oidDataPtr);
        if (errorCode == TRUE)
        {
            tm_memcpy(physicalAddress, 
                      oidDataPtr->Data, 
                      TM_ETHERNET_PHY_ADDR_LEN);

            tm_memcpy(win32IntfInfoPtr->physAddr, 
                      oidDataPtr->Data, 
                      TM_ETHERNET_PHY_ADDR_LEN);
        }
    }

    return 0;
}


/*
 * tfUseWin32Driver() Function Description
 * This function is called firstly before porting Treck on Win32
 * 
 */

ttUserInterface tfUseWin32Driver(   
    ttCharPtr           namePtr,
    ttUserLinkLayer     linkLayerHandle,
    int TM_FAR *        errorCodePtr) 
{
    ttUserInterface     interfaceHandle;
    int                 errorCode;
    char                adapterName[TM_ADAPTER_NAME_LEN];
    int                 adapterNameLen;
    int                 intfId;
#ifndef TM_APP_CREATE_TIMER_THREAD
    DWORD               idThread;
#endif /* TM_APP_CREATE_TIMER_THREAD */

    adapterNameLen = TM_ADAPTER_NAME_LEN;
    
    interfaceHandle = (ttVoidPtr)0;
    errorCode = TM_SOCKET_ERROR;

#ifndef TM_APP_CREATE_TIMER_THREAD
    if (tlTimerThreadHandle == 0) 
    {
        tlTimerThreadHandle =  CreateThread(    
                        NULL, 
                        TM_WIN32_TASK_STACK_SIZE,
                        (LPTHREAD_START_ROUTINE )tfWin32TimerTask, 
                        NULL, 
                        0,  
                        &idThread); 
        if ( tlTimerThreadHandle == NULL)
        {
            tfKernelError("tfUseWin32Driver", 
                             " Create tfWin32TimerTask error \n");
            goto COMMONRETURN;
        }
        errorCode = SetThreadPriority(
                            tlTimerThreadHandle, 
                            THREAD_PRIORITY_TIME_CRITICAL);
        tm_assert(tfWin32DrvOpen, errorCode != 0);
    }
#endif /* TM_APP_CREATE_TIMER_THREAD */

    tm_kernel_set_critical;
    intfId = tlWin32NumIntf;
    tlWin32NumIntf++;
    tm_kernel_release_critical;

    errorCode = tfWin32AdapterIndexToName(
                        namePtr[3] - '0', 
                        adapterName, 
                        &adapterNameLen);

    if (errorCode != TM_ENOERROR)
    {
        goto COMMONRETURN;
    }
    errorCode = TM_SOCKET_ERROR; /* assume failure */

/* initialize the winpcap interface information structure */
    tm_bzero(&tlWin32IntfInfo[intfId], sizeof(ttWin32InterfaceInfo));

    tlWin32IntfInfo[intfId].infoLpAdapter =   
            PacketOpenAdapter(adapterName);
    
    if (!tlWin32IntfInfo[intfId].infoLpAdapter 
            || (tlWin32IntfInfo[intfId].infoLpAdapter->hFile 
            == INVALID_HANDLE_VALUE))
    {
        tfKernelError("tfUseWin32Driver", "Unable to open the adapter"); 
        goto COMMONRETURN;
    }   

/* set a 512K buffer in the driver */
    if(PacketSetBuff(tlWin32IntfInfo[intfId].infoLpAdapter,512000)==FALSE)
    {
        tfKernelError(
            "tfUseWin32Driver", "Unable to set the kernel buffer!\n");
        goto COMMONRETURN;
    }

/* set a 5 ms read timeout */
    if(PacketSetReadTimeout(tlWin32IntfInfo[intfId].infoLpAdapter,5)
            ==FALSE)
    {
        tfKernelError("tfUseWin32Driver", "unable to set the read timeout!\n");
        goto COMMONRETURN;
    }

/* set the network adapter filter */
    if( !PacketSetHwFilter(
            tlWin32IntfInfo[intfId].infoLpAdapter,
            NDIS_PACKET_TYPE_BROADCAST
            | NDIS_PACKET_TYPE_DIRECTED | NDIS_PACKET_TYPE_ALL_LOCAL))
    {
        tfKernelError(
            "tfWin32DrvOpen", 
            "unable to set hardware filter !\n");
    }

/* allocate and initialize a packet structure that will be used to
 * receive the packets. */
    if((tlWin32IntfInfo[intfId].infoRecvLpPacket = 
            PacketAllocatePacket())==NULL)
    {
        tfKernelError(
            "tfUseWin32Driver", 
            "failed to allocate the receive LPPACKET structure.");

        goto COMMONRETURN;
    }

    PacketInitPacket(tlWin32IntfInfo[intfId].infoRecvLpPacket,
            (char*)tlWin32IntfInfo[intfId].infoBuf,256000);

    if((tlWin32IntfInfo[intfId].infoSendLpPacket 
            = PacketAllocatePacket())==NULL)
    {
        tfKernelError(
            "tfUseWin32Driver", 
            "failed to allocate the send LPPACKET structure.");

        goto COMMONRETURN;
    }

    tlWin32IntfInfo[intfId].infoReadBuf = (ttCharPtr)0; 
        
    interfaceHandle = tfAddInterface(namePtr,
                                    linkLayerHandle,
                                    (ttDevOpenCloseFuncPtr)tfWin32DrvOpen,
                                    (ttDevOpenCloseFuncPtr)tfWin32DrvClose,
                                    tfWin32DrvSend,
                                    tfWin32DrvRecv,
                                    (ttDevFreeRecvFuncPtr)0,
                                    tfWin32Ioctl,
                                    tfWin32DrvGetPhyAddr,
                                    &errorCode); 

    tlWin32IntfInfo[intfId].infoIfHandle = interfaceHandle;
/* interface is not ready to receive on, until it has been opened */
    tlWin32IntfInfo[intfId].infoRcvIfHandle = (ttUserInterface) 0;

    if (interfaceHandle != (ttDeviceEntryPtr)0)
    {
        tlWin32IntfInfo[intfId].infoIntfId = intfId;
        tfDeviceStorePointer(interfaceHandle, &(tlWin32IntfInfo[intfId]));
    }

COMMONRETURN:
    *errorCodePtr = errorCode;
    return interfaceHandle;
}

/* Free all previously added WinPcap drivers */
void tfFreeWin32Drivers()
{
/*    int  intfId;

    for ( intfId = 0; intfId < tlWin32NumIntf; intfId++)
    {
        if (tlWin32IntfInfo[intfId].infoSendLpPacket != NULL)
        {
            PacketFreePacket(tlWin32IntfInfo[intfId].infoSendLpPacket);
            tlWin32IntfInfo[intfId].infoSendLpPacket = NULL;
        }

        if (tlWin32IntfInfo[intfId].infoRecvLpPacket != NULL)
        {
            PacketFreePacket(tlWin32IntfInfo[intfId].infoRecvLpPacket);
            tlWin32IntfInfo[intfId].infoRecvLpPacket = NULL;
        }
    
        if (tlWin32IntfInfo[intfId].infoLpAdapter != NULL)
        {
            PacketCloseAdapter(tlWin32IntfInfo[intfId].infoLpAdapter);
        }
        } */
}

int tfWin32DrvOpen(ttUserInterface interfaceHandle)
{
    int                     errorCode;
    DWORD                   idThread;
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;
    struct _PACKET_OID_DATA oid;
 
    errorCode = TM_DEV_ERROR; /* assume failure */

    win32IntfInfoPtr = (ttWin32InterfaceInfoPtr)
                            tfDeviceGetPointer(interfaceHandle);

    if (win32IntfInfoPtr != (ttWin32InterfaceInfoPtr)0) 
    {
        if (tvReceiveThreadHandle == 0)
        {
            tvReceiveThreadHandle = CreateThread(   
                        NULL, 
                        TM_WIN32_TASK_STACK_SIZE,
                        (LPTHREAD_START_ROUTINE )tfWin32RecvIsr, 
                        NULL, 
                        0,  
                        &idThread);
            if ( tvReceiveThreadHandle == NULL)
            {
                tfKernelError("tfWin32DrvOpen", 
                                " Create tfWin32RecvIsr error");
            }
            else
            {
                errorCode = SetThreadPriority(
                    tvReceiveThreadHandle, 
                    THREAD_PRIORITY_ABOVE_NORMAL);
                tm_assert(tfWin32DrvOpen, errorCode != 0);
            }
        }   

/* get the multicast list size */
        oid.Oid = OID_802_3_MAXIMUM_LIST_SIZE;
        oid.Length = 4;

        if (!PacketRequest(win32IntfInfoPtr->infoLpAdapter, FALSE, &oid))
        {
            tfKernelWarning("tfWin32DrvOpen", 
                            "unable to get multicast addr list size!\n");
            win32IntfInfoPtr->mcastListSize = 0;
        }
        else
        {
            tm_memcpy(&win32IntfInfoPtr->mcastListSize, oid.Data, oid.Length);
        }

/* enable this interface for reception */
        tm_kernel_set_critical;
        win32IntfInfoPtr->infoRcvIfHandle = interfaceHandle;
        tm_kernel_release_critical;

        errorCode = TM_DEV_OKAY;
    }

    return errorCode;
}

int tfWin32DrvClose(ttUserInterface interfaceHandle)
{
    int errorCode;
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;

    errorCode = TM_DEV_ERROR; /* assume failure */

    win32IntfInfoPtr = (ttWin32InterfaceInfoPtr)
                            tfDeviceGetPointer(interfaceHandle);

    if (win32IntfInfoPtr != (ttWin32InterfaceInfoPtr)0) 
    {
/* disable this interface for reception */
        tm_kernel_set_critical;
        win32IntfInfoPtr->infoRcvIfHandle = (ttUserInterface) 0;
        tm_kernel_release_critical;

        errorCode = TM_DEV_OKAY;
    }

    return errorCode;
}


int tfWin32DrvSend  (   
    ttUserInterface     interfaceHandle,
    char     TM_FAR *   dataPtr,
    int                 dataLength,
    int                 flag )
{
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;
    int                     errorCode;
 
    errorCode = TM_DEV_OKAY;

    win32IntfInfoPtr = (ttWin32InterfaceInfoPtr)
                        tfDeviceGetPointer(interfaceHandle);
    if (win32IntfInfoPtr != (ttWin32InterfaceInfoPtr)0) 
    {
        if ( (flag != TM_USER_BUFFER_MORE)) 
        {
#ifdef TM_WIN32_SCATTBUFFER 
            if ( win32IntfInfoPtr->infoScatLen == 0)
            {
                    PacketInitPacket(win32IntfInfoPtr->infoSendLpPacket,
                        dataPtr, dataLength);
            }
            else
            {
                tm_bcopy ( dataPtr, win32IntfInfoPtr->infoScatBuf + 
                        win32IntfInfoPtr->infoScatLen, dataLength);
                win32IntfInfoPtr->infoScatLen += dataLength;
                if (win32IntfInfoPtr->infoScatLen > TM_INTF_MTU)
                {
                    tfKernelWarning("tfWin32DrvSend", "Trying to send more than MTU");
                    return TM_DEV_ERROR;
                }
                PacketInitPacket(   win32IntfInfoPtr->infoSendLpPacket,
                                    win32IntfInfoPtr->infoScatBuf, 
                                    win32IntfInfoPtr->infoScatLen );
                win32IntfInfoPtr->infoScatLen = 0;    
            }
#else /* ! TM_WIN32_SCATTBUFFER */
            PacketInitPacket(win32IntfInfoPtr->infoSendLpPacket,dataPtr, 
                    dataLength);
#endif /* TM_WIN32_SCATTBUFFER */
            if(PacketSetNumWrites(win32IntfInfoPtr->infoLpAdapter,1)==FALSE)
            {
                    tfKernelWarning("tfWin32DrvSend", "Unable to send !\n");
                }
    
                if(PacketSendPacket(win32IntfInfoPtr->infoLpAdapter,
                    win32IntfInfoPtr->infoSendLpPacket,TRUE)==FALSE)
            {
                    tfKernelWarning("tfWin32DrvSend", "Error sending the packets!\n");
                }

            tfSendCompleteInterface (interfaceHandle, 
                            TM_DEV_SEND_COMPLETE_DRIVER);
            errorCode = TM_DEV_OKAY;
        }
#ifdef TM_WIN32_SCATTBUFFER
        else /* TM_USER_BUFFER_MORE */
        {
            tm_bcopy ( dataPtr, win32IntfInfoPtr->infoScatBuf + 
                       win32IntfInfoPtr->infoScatLen, dataLength);
            win32IntfInfoPtr->infoScatLen += dataLength;
            if (win32IntfInfoPtr->infoScatLen > TM_INTF_MTU)
            {
                tfKernelWarning("tfWin32DrvSend", " MTU \n");
                return TM_DEV_ERROR;
            }
        }
#else /* !TM_WIN32_SCATTBUFFER */
        else
        {
           errorCode = TM_DEV_ERROR;
        }
#endif /* TM_WIN32_SCATTBUFFER */
    }
    return errorCode;
}

int tfWin32DrvRecv ( 
    ttUserInterface     interfaceHandle,
    ttCharPtr TM_FAR    *dataPtrPtr,
    ttIntPtr            dataSizePtr,
    ttUserBufferPtr     packetPtrPtr )
{
    ttDeviceEntryPtr        devEntryPtr;
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;
    ttPacketPtr             packetPtr;
    int                     dataLength;
    int                     errorCode;
    int                     wLen;
    int                     off;
    int                     align;
    ttEtherHeader           etherHdr;


    win32IntfInfoPtr =
        (ttWin32InterfaceInfoPtr) tfDeviceGetPointer(interfaceHandle);
    errorCode = TM_DEV_ERROR; /* assume we will drop this packet */

    if (    (win32IntfInfoPtr != (ttWin32InterfaceInfoPtr)0) 
            && (win32IntfInfoPtr->infoReadBuf != (char *)0))
    {
        align = sizeof(int);
        devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
        off  = ((struct bpf_hdr *)win32IntfInfoPtr->infoReadBuf)->bh_hdrlen;
        dataLength = ((struct bpf_hdr *)
                win32IntfInfoPtr->infoReadBuf)->bh_caplen;
        wLen = (int)(( off + dataLength + align -1) & (~(align-1)));

        if (
            tm_4_dev_forward(devEntryPtr)
            )
        {
/* enhancement ID# 928: Treck Win32 demo should not receive packets it sends */
            tm_bcopy(   (tt8BitPtr)win32IntfInfoPtr->infoReadBuf + off,
                        (tt8BitPtr) &etherHdr,
                        sizeof(ttEtherHeader));

            if (tm_ether_match(etherHdr.ethSrcAddr,
                               (unsigned char *)win32IntfInfoPtr->physAddr))
            {
                goto COMMONRETURN;
            }
        }
        packetPtr = tfGetSharedBuffer( 
                        TM_ETHER_HW_ALIGN,
                        (ttPktLen)(dataLength + TM_ETHER_IP_ALIGN),
                        TM_16BIT_ZERO );
        if ( packetPtr == TM_PACKET_NULL_PTR)
        {
            goto COMMONRETURN;
        }
        if (tm_ll_need_32bit_align(devEntryPtr))
        {
            if (   ((ttUserPtrCastToInt) packetPtr->pktLinkDataPtr)
                 & (ttUserPtrCastToInt)(TM_ETHER_HW_ALIGN - 1) )
            {
/* Align the data area (according to TM_ETHER_HW_ALIGN) */
                packetPtr->pktLinkDataPtr = (tt8BitPtr)
                    (((ttUserPtrCastToInt) packetPtr->pktLinkDataPtr)
                     & ~((ttUserPtrCastToInt) (TM_ETHER_HW_ALIGN - 1)) );
            }
/*
 * Offset of ethernet frame to allow IP area to be long word aligned.
 */
            packetPtr->pktLinkDataPtr += TM_ETHER_IP_ALIGN;
        }
        
        tm_bcopy(   (tt8BitPtr)win32IntfInfoPtr->infoReadBuf + off,
                    packetPtr->pktLinkDataPtr,
                    dataLength);

        win32IntfInfoPtr->infoReadBuf += wLen;
        if ( win32IntfInfoPtr->infoReadBuf == ( ttCharPtr)( 
                    (ttCharPtr)win32IntfInfoPtr->infoRecvLpPacket->Buffer
                    + win32IntfInfoPtr->infoRecvLpPacket->ulBytesReceived ))
        {
            win32IntfInfoPtr->infoReadBuf = (ttCharPtr)0;
        }
        
        packetPtr->pktUserFlags = TM_PKTF_USER_DEV_OWNS;
        *packetPtrPtr = packetPtr;
        *dataPtrPtr = (ttCharPtr)packetPtr->pktLinkDataPtr;
        *dataSizePtr = dataLength;
        errorCode = TM_DEV_OKAY;
    }

COMMONRETURN:
    return errorCode;    
}


int tfWin32Ioctl (
    ttUserInterface interfaceHandle,
    int             flag,
    void TM_FAR *   optionPtr,
    int             optionLen )
{
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;
    int                     errorCode;
    struct _PACKET_OID_DATA oid;
    int                     acceptableLen;

    errorCode = TM_DEV_OKAY;


/* To avoid compiler warnings */
    TM_UNREF_IN_ARG(interfaceHandle);
    TM_UNREF_IN_ARG(optionPtr);
    TM_UNREF_IN_ARG(optionLen);
    TM_UNREF_IN_ARG(flag);


    win32IntfInfoPtr = (ttWin32InterfaceInfoPtr)
                        tfDeviceGetPointer(interfaceHandle);

    if ((flag & TM_DEV_SET_MCAST_LIST) && win32IntfInfoPtr->mcastListSize)
    {
        if( !PacketSetHwFilter(
                win32IntfInfoPtr->infoLpAdapter,
                NDIS_PACKET_TYPE_BROADCAST
                | NDIS_PACKET_TYPE_DIRECTED | NDIS_PACKET_TYPE_MULTICAST
                | NDIS_PACKET_TYPE_ALL_LOCAL))
        {
            tfKernelWarning(
                "tfWin32DrvOpen", 
                "unable to set hardware filter !\n");
        }

/* ensure the list <= that supported by the device */
        if (optionLen > win32IntfInfoPtr->mcastListSize)
        {
            tfKernelWarning(
                "tfWin32DrvOpen", 
                "too many multicast address for the device, will do best!\n");

            acceptableLen =   win32IntfInfoPtr->mcastListSize
                            * TM_ETHERNET_PHY_ADDR_LEN;
        }
/* ensure the list <= that of the local buffer oid.data */
        else if(optionLen > TM_WIN32_MAX_MCAST_ADRR)
        {
            tfKernelWarning(
                "tfWin32DrvOpen", 
                "too many multicast address for the driver buffer, will do best!\n");

            acceptableLen =   TM_WIN32_MAX_MCAST_ADRR
                            * TM_ETHERNET_PHY_ADDR_LEN;
        }
        else
        {
            acceptableLen= optionLen* TM_ETHERNET_PHY_ADDR_LEN;
        }

        oid.Length = acceptableLen;

/* set list of multicast addresss to the driver */
        oid.Oid = OID_802_3_MULTICAST_LIST;
        tm_memcpy(oid.Data,
                  optionPtr, 
                  oid.Length);

        if (!PacketRequest(win32IntfInfoPtr->infoLpAdapter, TRUE, &oid))
        {
            tfKernelWarning(
                "tfWin32DrvOpen", 
                "unable to set multicast addr list !\n");
            win32IntfInfoPtr->mcastListSize = 0;
            goto checkSetAllMcast;
        }

#ifdef _DEBUG
/* get the list of multicast addresss to the diver */
        oid.Oid = OID_802_3_MULTICAST_LIST;

        if (!PacketRequest(win32IntfInfoPtr->infoLpAdapter, FALSE, &oid))
        {
            tfKernelWarning(
                "tfWin32DrvOpen", 
                "unable to get multicast addr list !\n");
        }

/* verify that the multicast list is sucessfully set, note that we can't use
 * tm_assert(tfWin32Ioctl, !memcmp(oid.Data, optionPtr, acceptableLen));
 * to automatically verify, since the orders of address in optionPtr and 
 * oid.Data is different sometimes
 */
#endif /* _DEBUG */


    }
    
 checkSetAllMcast:
    if ((flag & TM_DEV_SET_ALL_MCAST)
        || ((flag & TM_DEV_SET_MCAST_LIST)
            && !win32IntfInfoPtr->mcastListSize))
    {
        if(PacketSetHwFilter(win32IntfInfoPtr->infoLpAdapter,
                  NDIS_PACKET_TYPE_DIRECTED
                | NDIS_PACKET_TYPE_ALL_MULTICAST
                | NDIS_PACKET_TYPE_ALL_LOCAL
                | NDIS_PACKET_TYPE_BROADCAST ) ==FALSE )
        {
            tfKernelWarning(
                "tfWin32DrvOpen", 
                "unable to set hardware filter !\n");
        }
    }
    
    return(errorCode);
}


/*
 * tfWin32RecvThread() Function Description
 */

void tfWin32RecvIsr ()
{
    char    *buf;
    ttVoidPtr interfaceHandle;
    int     recvNotice;
    int     cond;
    int     tLen;
    int     pLen;
    int     off;
    int     align;
    int     wLen;
    int     i;

    cond  = 1;
    align = sizeof(int);

    while (cond)
    {
        for ( i = 0; i < tlWin32NumIntf; i++)
        {
            tm_kernel_set_critical;
            interfaceHandle = tlWin32IntfInfo[i].infoIfHandle;
            tm_kernel_release_critical;
            if (    (tlWin32IntfInfo[i].infoReadBuf == 0)
                 && (interfaceHandle != (ttVoidPtr)0) )
            {
                recvNotice = 0;
                if(PacketReceivePacket(tlWin32IntfInfo[i].infoLpAdapter,
                        tlWin32IntfInfo[i].infoRecvLpPacket,TRUE)==FALSE)
                {
                    tfKernelWarning(
                        "tfWin32RecvIsr",
                        "Error: PacketReceivePacket failed");
                }    
                else if (tlWin32IntfInfo[i].infoRcvIfHandle)
                {
                    tLen = tlWin32IntfInfo[i].infoRecvLpPacket->ulBytesReceived;
                    buf  = tlWin32IntfInfo[i].infoRecvLpPacket->Buffer;
                    while ( tLen > 0)
                    {
                        off  = ((struct bpf_hdr *)buf)->bh_hdrlen;
                        pLen = ((struct bpf_hdr *)buf)->bh_caplen; 
                        recvNotice++;
                        wLen = (int)
                                (( off + pLen + align -1) & (~(align-1)));
#ifndef TM_APP_CREATE_RECV_THREAD
                        tlWin32IntfInfo[i].infoReadBuf = buf;
                        tfRecvInterface(interfaceHandle);

#endif /* TM_APP_CREATE_RECV_THREAD */
                        buf += wLen;
                        tLen-= wLen;
                    }
#ifndef TM_APP_CREATE_RECV_THREAD
                    tlWin32IntfInfo[i].infoReadBuf = 0;
#else
                    if ( recvNotice > 0)
                    {
                        tlWin32IntfInfo[i].infoReadBuf = 
                                tlWin32IntfInfo[i].infoRecvLpPacket->Buffer;
                        tfNotifyInterfaceIsr(
                                interfaceHandle,
                                recvNotice,
                                0, 0UL, 0UL );
                    }
#endif /* TM_APP_CREATE_RECV_THREAD */
                }
            }
        }
    }
}

/*  Timer Task *
 */
void tfWin32TimerTask()
{
    DWORD   tSystem, tKasago;
    tKasago = GetTickCount( );
    for (;;)
    {
        Sleep(TM_TICK_LENGTH);

        tSystem = GetTickCount( );

        while( (tSystem - tKasago) >= TM_TICK_LENGTH )
        {
            tfTimerUpdate();
            tKasago += TM_TICK_LENGTH;
        }
        tfTimerExecute();
    }
}

int tfWin32GetNicInfo(ttWin32NicInfo TM_FAR * infoPtr, int index)
{
    tt32Bit             ipAddr;
    tt32Bit             netMask;
    char                adapterName[TM_ADAPTER_NAME_LEN];
    LPADAPTER           lpAdapter;
    int                 errorCode;
    PPACKET_OID_DATA    oidDataPtr;
    char                oidDataData[TM_OID_DATA_LEN];
    NetType             netType;
    int                 nameLen;

    tm_bzero(infoPtr, sizeof(ttWin32NicInfo));
    tm_bzero(&oidDataData, sizeof(oidDataData));

/* index to adapterName */
    nameLen = TM_ADAPTER_NAME_LEN;
    errorCode = tfWin32AdapterIndexToName(
                        index, 
                        adapterName, 
                        &nameLen);
    if (errorCode != TM_ENOERROR)
    {
        return errorCode;
    }

    tm_memcpy(infoPtr->name, adapterName, nameLen);
    infoPtr->nameLen = nameLen;
    lpAdapter =  PacketOpenAdapter(adapterName);
    if (lpAdapter == NULL)
    {
        return TM_SOCKET_ERROR;
    }

/* prepare the data structrue for driver command */
    oidDataPtr = (PPACKET_OID_DATA)oidDataData;
    oidDataPtr->Length =   TM_OID_DATA_LEN
                         - sizeof(oidDataPtr->Oid)
                         - sizeof(oidDataPtr->Length);

/* get adapter type and speed */
    netType.v31LinkSpeed = 0; /* hack for pre-v3.1 winpcap */
    errorCode = PacketGetNetType(lpAdapter, &netType);
    if(errorCode == TRUE)
    {
        infoPtr->type = netType.LinkType;
        if (netType.LinkSpeed == 0)
        {
/* we must be using v3.1 or later of winpcap packet.dll */
            infoPtr->speed = netType.v31LinkSpeed;
        }
        else
        {
/* assume pre-v3.1 winpcap */
            infoPtr->speed = netType.LinkSpeed;
        }
    }

/* get adapter address configure infor */
    errorCode = pcap_lookupnet(adapterName, &ipAddr, &netMask, tlPcapErrbuf);
/* pre-v3.1:
    errorCode = PacketGetNetInfo(adapterName, &ipAddr, &netMask);
    if(errorCode == TRUE) */
    if (errorCode >= 0)
    {
        infoPtr->ipAddr = ipAddr;
        infoPtr->netMask = netMask;
    }

/* get the driver vendor's description string */
    oidDataPtr->Oid = OID_GEN_VENDOR_DESCRIPTION;
    oidDataPtr->Length = 128;
    errorCode = PacketRequest(lpAdapter, FALSE, oidDataPtr);
    if (errorCode == TRUE)
    {
        strncpy(infoPtr->driverDesc, 
               (const char*)oidDataPtr->Data,
               oidDataPtr->Length);
    }

/* get the hardware status code, 0 is ready */
    oidDataPtr->Oid = OID_GEN_HARDWARE_STATUS;
    oidDataPtr->Length = 4;
    errorCode = PacketRequest(lpAdapter, FALSE, oidDataPtr);
    if (errorCode == TRUE)
    {
        infoPtr->status = *((int*)(oidDataPtr->Data));
    }

/* get the hardware address */
    oidDataPtr->Oid = OID_802_3_CURRENT_ADDRESS;

    oidDataPtr->Length = 6;
    tm_memset(infoPtr->phyAddr, 0, TM_ETHERNET_PHY_ADDR_LEN);
    errorCode = PacketRequest(lpAdapter, FALSE, oidDataPtr);
    if (errorCode == TRUE)
    {
        tm_memcpy(infoPtr->phyAddr, 
                  oidDataPtr->Data, 
                  TM_ETHERNET_PHY_ADDR_LEN);
    }

    PacketCloseAdapter(lpAdapter);
    return TM_ENOERROR;
}

/* get the number of NICs, returns -1 if there is an error */
int tfWin32GetNicCount(void)
{
    pcap_if_t   *alldevs, *dev;
    int         AdapterNum;

    AdapterNum = -1; /* assume we find no NICs */

    if (pcap_findalldevs(&alldevs, tlPcapErrbuf) != -1)
    {
        AdapterNum = 0;

        for (dev = alldevs; dev != NULL; dev = dev->next)
        {
/* Skip the WAN adapter, if we try to open it with winpcap v3.1beta4 this
   causes a crash */
            if ((dev->addresses != NULL)
                || (strstr(dev->name, "WanAdapter") == NULL))
            {
                AdapterNum++;
            }
        }
        
        pcap_freealldevs(alldevs);
    }

    return AdapterNum;
}

/* 
 * given a index, return the adapter name as retrieved by pcap_findalldevs, 
 * could be wchar 
 */
int tfWin32AdapterIndexToName(int index, char* name, int* lenPtr)
{
    pcap_if_t   *alldevs, *dev;
    int         AdapterNum;
    int         errorCode;
    int         nameLen;

    errorCode = TM_SOCKET_ERROR;

    if (pcap_findalldevs(&alldevs, tlPcapErrbuf) != -1)
    {
        AdapterNum = 0;

        for (dev = alldevs; dev != NULL; dev = dev->next)
        {
/* Skip the WAN adapter, if we try to open it with winpcap v3.1beta4 this
   causes a crash */
            if ((dev->addresses != NULL)
                || (strstr(dev->name, "WanAdapter") == NULL))
            {
                if (AdapterNum == index)
                {
                    nameLen = (int)tm_strlen(dev->name);
                    if ( *lenPtr < nameLen + 1 )
                    {
                        errorCode= TM_ENOSPC;
                    }
                    else
                    {
                        errorCode= TM_ENOERROR;
                        tm_memcpy(name, dev->name, nameLen);
/* ensure string terminated by '\0' */
                        name[nameLen] = '\0';
                        *lenPtr = nameLen;
                    }

                    pcap_freealldevs(alldevs);
                    goto toNameEnd;
                }

                AdapterNum++;
            }
        }

        pcap_freealldevs(alldevs);
    }

toNameEnd:
    return errorCode;
}

int tfWin32GetConnectStatus(ttUserInterface interfaceHandle)
{
    struct _PACKET_OID_DATA oid;
    ttWin32InterfaceInfoPtr win32IntfInfoPtr;

    win32IntfInfoPtr = (ttWin32InterfaceInfoPtr)
                        tfDeviceGetPointer(interfaceHandle);
    oid.Oid  = OID_GEN_MEDIA_CONNECT_STATUS;
    oid.Length = 4;

    PacketRequest( win32IntfInfoPtr->infoLpAdapter,
                   FALSE,
                   &oid
    );

    return *(int*)oid.Data;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: trwpcap.c     File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 65157    Output checksum: 60765
 */
