/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: 
 *          main function for automated testing as an WIN32 multithread app.
 *
 *          NOTE: all the files in the test\unittest directory must be 
 *          included into the project, since they implement the automated
 *          test code that this main function calls.
 *
 * Filename: txwin32.c
 * Author: Bryan Wang
 * Date Created:
 * $Source: source/kernel/win32/intelx86/txwin32.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:22CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifdef _WIN32
/* compile and run on Win32 platform */
/* to avoid an warning in MS VC++ header file */
struct _RPC_ASYNC_STATE;
#include <windows.h>
#endif /* _WIN32 */

#include <trsocket.h>
#ifdef TM_KERNEL_WIN32_X86
#ifdef TM_MULTIPLE_CONTEXT
#include <trmacro.h>
#include <trtype.h>
#include <trglobal.h>
#include <trproto.h>
#include "..\test\unittest\tutest.h"

#include <signal.h>
#include <conio.h>

DWORD WINAPI timerUpdateTask(void * pParam);
DWORD WINAPI timerExecTask(ttUserContext context);
DWORD WINAPI recvTask(ttUserContext context);
DWORD WINAPI v4TcpEchoServerTask(ttUserContext context);
DWORD WINAPI ftpServerTask(ttUserContext context);
DWORD WINAPI v6TcpEchoServerTask(ttUserContext context);
DWORD WINAPI v4UdpEchoServerTask(ttUserContext context);
DWORD WINAPI v6UdpEchoServerTask(ttUserContext context);
DWORD WINAPI testClientTask(ttUserContext context);
DWORD WINAPI TestBsdLibFunc();
DWORD WINAPI v4FtpClientTask(ttUserContext context);
#endif /* TM_MULTIPLE_CONTEXT */

main(void)
{
#ifdef TM_MULTIPLE_CONTEXT
    void *  handle;
    int     errorCode;
    ttUserLinkLayer linkLayerHandleServer;
    ttUserLinkLayer linkLayerHandleClient;
    char serverIpAddrStr1[] = "10.10.10.161";
    char serverIpAddrStr2[] = "10.10.10.162";
    char clientIpAddrStr1[] = "10.10.10.177";
    char clientIpAddrStr2[] = "10.10.10.178";
    ttUser16Bit serverPort1 = 4001;
    ttUser16Bit serverPort2 = 4002;
    
    
/* 
 * Thread Id carrying back from CreateThread, Not used, 
 * but CreateThread in win95, 98, and Me requries this parameter 
 */
    DWORD   tid;            

/* initialize the Treck stack */

/* tm_test(tfxCreateStartContexts); */
/* clear all the contexts */
    tfInitTreckMultipleContext();

/* server context  */
    tvContextHandleServer = tfCreateTreckContext();
    tm_assert(main, tvContextHandleServer != (void TM_FAR *)0)
    
    tfSetCurrentContext(tvContextHandleServer);
    errorCode = tfStartTreck();
    tm_assert(main, errorCode == TM_ENOERROR);


/* client context */
    tvContextHandleClient = tfCreateTreckContext();
    tm_assert(main, tvContextHandleClient != (void TM_FAR *)0)

    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfStartTreck();
    tm_assert(main, errorCode == TM_ENOERROR);
    
/* tm_test(tfxAddOnlinkInterfaces); */
/* server context  */
    tfSetCurrentContext(tvContextHandleServer);

    linkLayerHandleServer = tfUseEthernet();

    tvInterfaceHandleServer = tfUseIntfMCtxDriver(
                                    "Server", 
                                     linkLayerHandleServer, 
                                     &errorCode);

/* client context */
    tfSetCurrentContext(tvContextHandleClient);

    linkLayerHandleClient = tfUseEthernet();

    tvInterfaceHandleClient = tfUseIntfMCtxDriver(
                                    "Client",
                                     linkLayerHandleClient, 
                                     &errorCode);

/* tm_test(tfxConfigureIpAddr); */
/* tfx4ConfigureIpAddr */
    tm_strcpy(tv4ServerIpAddrStr[0], serverIpAddrStr1);
    tm_strcpy(tv4ServerIpAddrStr[1], serverIpAddrStr2);
    tm_strcpy(tv4ClientIpAddrStr[0], clientIpAddrStr1);
    tm_strcpy(tv4ClientIpAddrStr[1], clientIpAddrStr2);

    tm_memset(&tv4ServerSockAddr[0], 0, sizeof(struct sockaddr_storage));
    tm_memset(&tv4ServerSockAddr[1], 0, sizeof(struct sockaddr_storage));
    tm_memset(&tv4ClientSockAddr[0], 0, sizeof(struct sockaddr_storage));
    tm_memset(&tv4ClientSockAddr[1], 0, sizeof(struct sockaddr_storage));

    tm_memset(&tv4ServerSockAddr[0], 0, sizeof(struct sockaddr_storage));
    tm_memset(&tv4ServerSockAddr[1], 0, sizeof(struct sockaddr_storage));
    tm_memset(&tv4ClientSockAddr[0], 0, sizeof(struct sockaddr_storage));
    tm_memset(&tv4ClientSockAddr[1], 0, sizeof(struct sockaddr_storage));

    tv4ServerSockAddr[0].ss_family = AF_INET;
    tv4ServerSockAddr[1].ss_family = AF_INET;
    tv4ClientSockAddr[0].ss_family = AF_INET;
    tv4ClientSockAddr[1].ss_family = AF_INET;

    tv4ServerSockAddr[0].ss_len = sizeof(struct sockaddr_storage);
    tv4ServerSockAddr[1].ss_len = sizeof(struct sockaddr_storage);
    tv4ClientSockAddr[0].ss_len = sizeof(struct sockaddr_storage);
    tv4ClientSockAddr[1].ss_len = sizeof(struct sockaddr_storage);

    tv4ServerSockAddr[0].addr.ipv4.sin_port = htons(serverPort1);
    tv4ServerSockAddr[1].addr.ipv4.sin_port = htons(serverPort2);


    tfSetCurrentContext(tvContextHandleServer);
/* config first address on the server */
    if (serverIpAddrStr1 != NULL)
    {
        errorCode = inet_pton(AF_INET,
                              serverIpAddrStr1,
                              &tv4ServerSockAddr[0].addr.ipv4.sin_addr
                             );
        tm_assert(tfx6ConfigureIpAddr, errorCode == 1);

        errorCode = tfOpenInterface(
                        tvInterfaceHandleServer,
                        tv4ServerSockAddr[0].addr.ipv4.sin_addr.s_addr,
                        inet_addr("255.255.255.0"),
                        TM_DEV_MCAST_ENB,   /* flag */
                        1);
        tm_assert(tfx6ConfigureIpAddr, errorCode == TM_ENOERROR); 

    }    
#ifndef TM_SINGLE_INTERFACE_HOME
/* config second address on the server */
    if (serverIpAddrStr2 != NULL)
    {
        errorCode = inet_pton(AF_INET,
                              serverIpAddrStr2,
                              &tv4ServerSockAddr[1].addr.ipv4.sin_addr
                             );
        tm_assert(tfx6ConfigureIpAddr, errorCode == 1); 

        errorCode = tfConfigInterface(
                        tvInterfaceHandleServer,
                        tv4ServerSockAddr[1].addr.ipv4.sin_addr.s_addr,
                        inet_addr("255.255.255.0"),
                        TM_DEV_MCAST_ENB,   /* flag */
                        1,
                        1);
        tm_assert(tfx6ConfigureIpAddr, (errorCode == TM_ENOERROR));
    }    
#endif /* !TM_SINGLE_INTERFACE_HOME */

    tfSetCurrentContext(tvContextHandleClient);

/* config first addresse on the client */
    if (clientIpAddrStr1 != NULL)
    {
        errorCode = inet_pton(AF_INET,
                              clientIpAddrStr1,
                              &tv4ClientSockAddr[0].addr.ipv4.sin_addr
                             );
        tm_assert(tfx6ConfigureIpAddr, errorCode == 1); 

        errorCode = tfOpenInterface(
                        tvInterfaceHandleClient,
                        tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr,
                        inet_addr("255.255.255.0"),
                        TM_DEV_MCAST_ENB,   /* flag */
                        1);
        tm_assert(tfx6ConfigureIpAddr, errorCode == TM_ENOERROR); 
    }
    
#ifndef TM_SINGLE_INTERFACE_HOME
/* config second addresse on the client */
    if (clientIpAddrStr2 != NULL)
    {
        errorCode = inet_pton(AF_INET,
                              clientIpAddrStr2,
                              &tv4ClientSockAddr[1].addr.ipv4.sin_addr
                             );
        tm_assert(tfx6ConfigureIpAddr, errorCode == 1); 

        errorCode = tfConfigInterface(
                        tvInterfaceHandleClient,
                        tv4ClientSockAddr[1].addr.ipv4.sin_addr.s_addr,
                        inet_addr("255.255.255.0"),
                        TM_DEV_MCAST_ENB,   /* flag */
                        1,
                        1);
        tm_assert(tfx6ConfigureIpAddr, errorCode == TM_ENOERROR); 
    }
#endif /* !TM_SINGLE_INTERFACE_HOME */

    
/*
 * BSD library function testing, these function needs to be tested
 * only once here
 */
    TestBsdLibFunc();

/* 
 * the following tasks tests the socket functions in two contexts, 
 * simulating nodes, a echo server and echo client, uses Windows 
 * timer (Sleep), Thread, semaphore and events, as the kernel support
 */


/* 
 * global timer task, keeps updating tvTime every clock tick,
 * not specific to any context 
 */
    handle = CreateThread(
            NULL, 4096, timerUpdateTask, tvContextHandleServer, 0, &tid);
    tm_assert(main, handle != NULL);
    errorCode = SetThreadPriority(handle, THREAD_PRIORITY_TIME_CRITICAL);
    tm_assert(main, handle != 0);


/* server timer task */
    handle = CreateThread(
            NULL, 4096, timerExecTask, tvContextHandleServer, 0, &tid);
    tm_assert(main, handle != NULL);
    errorCode = SetThreadPriority(handle, THREAD_PRIORITY_TIME_CRITICAL);
    tm_assert(main, handle != 0);

/* server recv task */
    handle = CreateThread(
            NULL, 4096, recvTask, tvContextHandleServer, 0, &tid);
    tm_assert(main, handle != NULL);
    errorCode = SetThreadPriority(handle, THREAD_PRIORITY_ABOVE_NORMAL);
    tm_assert(main, handle != 0);

/* 4 server thread are running in the server context */

/* V4 UDP echo server task */
    handle = CreateThread(
            NULL, 4096, v4UdpEchoServerTask, tvContextHandleServer, 0, &tid);
    tm_assert(main, handle != NULL);

/* V4 TCP server task */
    handle = CreateThread(
            NULL, 4096, v4TcpEchoServerTask, tvContextHandleServer, 0, &tid);
    tm_assert(main, handle != NULL);
    handle = CreateThread(
            NULL, 4096, ftpServerTask, tvContextHandleServer, 0, &tid);
    tm_assert(main, handle != NULL);



/* client timer task */
    handle = CreateThread(
            NULL, 4096, timerExecTask, tvContextHandleClient, 0, &tid);
    tm_assert(main, handle != NULL);
    errorCode = SetThreadPriority(handle, THREAD_PRIORITY_TIME_CRITICAL);
    tm_assert(main, handle != 0);

/* client recv task */
    handle = CreateThread(
            NULL, 4096, recvTask, tvContextHandleClient, 0, &tid);
    tm_assert(main, handle != NULL);
    errorCode = SetThreadPriority(handle, THREAD_PRIORITY_ABOVE_NORMAL);
    tm_assert(main, handle != 0);

//#define TM_TEST_FTP

#ifdef TM_TEST_FTP
    handle = CreateThread(
            NULL, 4096, v4FtpClientTask, tvContextHandleClient, 0, &tid);
    tm_assert(main, handle != NULL);
#else /* !TM_TEST_FTP */
/* test client task */
    handle = CreateThread(
            NULL, 4096, testClientTask, tvContextHandleClient, 0, &tid);
    tm_assert(main, handle != NULL);
#endif /* !TM_TEST_FTP */


/* sleep a thousand years, while the child threads are busy testing */
    {
        int year = 1000;
        int day  = 365;
        while (year --)
        {
            while(day -- )
            {
                Sleep(1000 * 60 * 60 * 24);
            }
        }
    }
#else /* !TM_MULTIPLE_CONTEXT */
    printf("To run the automated testing code\n");
    printf("please compile the stack with TM_MULTIPLE_CONTEXT \n");
    getchar();
#endif /* !TM_MULTIPLE_CONTEXT */
}

#ifdef TM_MULTIPLE_CONTEXT
DWORD WINAPI  testClientTask(ttUserContext context)
{
    int     i;
#ifdef TM_USE_SHEAP
static int prevUsedBytes = 0;
#endif /* TM_USE_SHEAP */

    tfKernelSetContext(context);

    for (i = 1; i >0; i++)
    {
        tm_test_with_1_param(
                tfxUdpEchoClient, 
                (struct sockaddr *)&tv4ServerSockAddr[0]);
        tm_test_with_1_param(
                tfxTcpEchoClient, 
                (struct sockaddr *)&tv4ServerSockAddr[0]); 
        tm_test_with_1_param(tfxArp, tvContextHandleClient);

#ifdef TM_USE_SHEAP
#ifdef TM_USE_DYNAMIC_MEMORY
        tvCurrUsedBytes =   tm_global(tvSheapUsedLongs) * sizeof(tt32Bit)
                          - tm_global(tvMemRecycledBytes);
#else /* !TM_USE_DYNAMIC_MEMORY */
        tvCurrUsedBytes = tm_global(tvSheapUsedLongs) * sizeof(tt32Bit);
#endif /* !TM_USE_DYNAMIC_MEMORY */
        tvUsedBytesIncrement = tvCurrUsedBytes - prevUsedBytes;
        prevUsedBytes = tvCurrUsedBytes;
        printf("%dth round: sheap: %u(%+d)\n",
               i, 
               tvCurrUsedBytes, tvUsedBytesIncrement);
#endif /* TM_USE_SHEAP */
    }
    return 0;
}


DWORD WINAPI  timerUpdateTask(void * pParam)
{
    TM_UNREF_IN_ARG(pParam);
    for(;;)
    {
/* 
 * keeps tvTime to be accurate
 * of WIN32 thread 
 */
        tfTimerUpdate();
        Sleep(TM_TICK_LENGTH);
    }
}

DWORD WINAPI  timerExecTask(ttUserContext context)
{
/* 
 * store the pointer to the correct context in the thread local storage
 * of WIN32 thread 
 */
    tfKernelSetContext(context);

    for(;;)
    {
/* 
 * executes time specific code specific to the context 
 */
        tfTimerExecute();
        Sleep(TM_TICK_LENGTH);
    }
}


DWORD WINAPI  recvTask(ttUserContext context)
{
    tfKernelSetContext(context);
    for(;;)
    {
        (void)tfWaitReceiveInterface(tfxGetInterfaceByContext(context));
        (void)tfRecvInterface(tfxGetInterfaceByContext(context)); 
/* explicitly yield/sleep one millisecond, allow other tasks to run. */
        Sleep(1);
    }
}

DWORD WINAPI  v4TcpEchoServerTask(ttUserContext context)
{
    tfKernelSetContext(context);
    tfxTcpEchoServer((struct sockaddr *)&tv4ServerSockAddr[0]);
    return 0;
}

DWORD WINAPI  v4UdpEchoServerTask(ttUserContext context)
{
    tfKernelSetContext(context);
    tfxUdpEchoServer((struct sockaddr *)&tv4ServerSockAddr[0]);
    return 0;
}
DWORD WINAPI  ftpServerTask(ttUserContext context)
{
    int errorCode;
    tfKernelSetContext(context);
    errorCode =  tfNgFtpdUserStart(TM_FS_ALLCMND_MASK, 2, 2, 300, 
                                   TM_BLOCKING_ON, 
                                   0);
    if (errorCode != TM_ENOERROR)
    {
        printf("tfFtpdUserStart failed %s\n", tfStrError(errorCode));
    }
}


DWORD WINAPI  TestBsdLibFunc()
{
    tm_test(tfxInetNtoP);
    tm_test(tfxInetPtoN);
    tm_test_with_1_param(tfxIfIndexToName, tvContextHandleServer);
    tm_test_with_1_param(tfxIfIndexToName, tvContextHandleClient);
    tm_test_with_1_param(tfxIfNameToIndex, tvContextHandleServer);
    tm_test_with_1_param(tfxIfNameToIndex, tvContextHandleClient);
    tm_test(tfxSocket);
    return 0;
}

/*
 * FTP client test
 */
#define FS_USER_NAME     "ftp"
#define FS_PASSWORD      "ftp"
#define FTP_USER_NAME    "elmic"
#define FTP_PASSWORD     "elmic"
#define FTP_ACCOUNT_NAME "account"
#define FTP_TRANSMIT_LOCAL_FILE_NAME  "put.dat"
#define FTP_TRANSMIT_REMOTE_FILE_NAME "put.dat"
//#define FTP_RETRIEVE_LOCAL_FILE_NAME  "README"
//#define FTP_RETRIEVE_REMOTE_FILE_NAME "README"
#define FTP_RETRIEVE_LOCAL_FILE_NAME  "system.dat"
#define FTP_RETRIEVE_REMOTE_FILE_NAME "system.dat"
//#define TM_FAR far

char	dt[64];
int		dtcnt;
char	arv[5][20];
char	remote_file[32];
char	local_file[32];
char	user_name[32];
char	user_pass[32];
char	remote_pwd[128];
char	mkbuff[128];

/* FTP/TFTP Client Data */
#define	CMM_DIR			1
#define	CMM_PUT			2
#define	CMM_GET			3
#define	CMM_DEL			4
#define	CMM_REN			5
#define	CMM_PWD			6
#define	CMM_RD			7
#define	CMM_MD			8
#define	CMM_CD			9
#define	CMM_APP			10
#define	CMM_RESET		11
#define	CMM_ABOR		12
#define	CMM_BIN			13
#define	CMM_ASCII		14
#define	CMM_HELP		15
#define	CMM_QUIT		255

int command_check(void);


void key_input(char *buff)
{
	int cnt = 0;
	char c;

	printf("> ");
	while(1){
		if(kbhit()){
			c = getch();
			if(c == 0x0d){
				printf("\n");
				break;
			}
			if(c == 0x08){
				if(cnt != 0){
					putchar(0x08);
					putchar(0x20);
					putchar(0x08);
					cnt--;
					*(buff + cnt) = '\0';
				}
			}
			else{
				putchar(c);
				*(buff + cnt) = c;
				cnt++;
				*(buff + cnt) = '\0';
			}
		}
	}
}

int keycnt = 0;

int key_input2(char *buff)
{
	char c;

	if(kbhit()){
		c = getch();
		if(c == 0x0d){
			printf("\n");
			keycnt = 0;
			return(1);
		}
		if(c == 0x08){
			if(keycnt != 0){
				putchar(0x08);
				putchar(0x20);
				putchar(0x08);
				keycnt--;
				*(buff + keycnt) = '\0';
			}
		}
		else{
			putchar(c);
			*(buff + keycnt) = c;
			keycnt++;
			*(buff + keycnt) = '\0';
		}
	}
	return(0);
}

int command_check(void)

{
	char	*workp;
	int		i;
	int		cmm;

	i = 0;
	workp = strtok(dt,"\x20\t,");
	while(workp != NULL){
		strcpy(arv[i++], workp);
		workp = strtok(NULL,"\x20\t,");
	}
	if((strcmp(arv[0], "dir")) == 0)
		cmm = CMM_DIR;
	else if((strcmp(arv[0], "put")) == 0)
		cmm = CMM_PUT;
	else if((strcmp(arv[0], "get")) == 0)
		cmm = CMM_GET;
	else if((strcmp(arv[0], "del")) == 0)
		cmm = CMM_DEL;
	else if((strcmp(arv[0], "ren")) == 0)
		cmm = CMM_REN;
	else if((strcmp(arv[0], "pwd")) == 0)
		cmm = CMM_PWD;
	else if((strcmp(arv[0], "rd")) == 0)
		cmm = CMM_RD;
	else if((strcmp(arv[0], "md")) == 0)
		cmm = CMM_MD;
	else if((strcmp(arv[0], "cd")) == 0)
		cmm = CMM_CD;
	else if((strcmp(arv[0], "append")) == 0)
		cmm = CMM_APP;
	else if((strcmp(arv[0], "reset")) == 0)
		cmm = CMM_RESET;
	else if((strcmp(arv[0], "abo")) == 0)
		cmm = CMM_ABOR;
	else if((strcmp(arv[0], "bin")) == 0)
		cmm = CMM_BIN;
	else if((strcmp(arv[0], "ascii")) == 0)
		cmm = CMM_ASCII;
	else if((strcmp(arv[0], "help")) == 0)
		cmm = CMM_HELP;
	else if((strcmp(arv[0], "quit")) == 0)
		cmm = CMM_QUIT;
	else
		cmm = 0;
	return(cmm);
}
static void tfSignalInit (void);
static int ftpClient(ttUserContext context);

DWORD WINAPI v4FtpClientTask(ttUserContext context)
{
    int retCode;

    tfKernelSetContext(context);
    retCode = ftpClient(AF_INET);
    exit(retCode);
}

static int ftpClient(int af)
{
	void key_input(char *buff);
	int ftpCBFunc(ttUserFtpHandle ftpHandle, char *bufferPtr, int bufferSize);

	int		ret;
	ttUserFtpHandle ftpHandle;
	int		errorCode;
	char	c;
	int		cmm;


	printf("****************************************************************\n");
	printf("*      FTP Client Test                                         *\n");
	printf("****************************************************************\n");

    tfSignalInit();	
	ftpHandle = tfFtpNewSession( 0, TM_BLOCKING_ON, FS_USER_NAME, FS_PASSWORD );
	printf( "tfFtpNewSession = %d\n", ftpHandle );

    if (af == AF_INET)
    {
    	errorCode = tfNgFtpConnect( ftpHandle, &tv4ServerSockAddr[0] );
    }
	printf( "tfFtpConnect = %s\n", tfStrError(errorCode) );
    if (errorCode != TM_ENOERROR)
    {
        printf("FTP client Exiting with connect failure\n");
        errorCode = tfFtpFreeSession(ftpHandle);
  printf( "tfFtpFreeSession = %s\n", tfStrError(errorCode) );
  		return 1;
    }

/*------ Login Name/password ----------------------------*/
#if 0
/* Interactive login */
	do{
		printf("Login Name ");
		key_input(user_name);
		printf("password ");
		key_input(user_pass);
        */

		errorCode = tfFtpLogin( ftpHandle, user_name, user_pass, FTP_ACCOUNT_NAME );
		printf( "tfFtpLogin = %s\n", tfStrError(errorCode) );
	} while((errorCode != TM_ENOERROR) && (errorCode != TM_EWOULDBLOCK));
#endif
    errorCode = tfFtpLogin( ftpHandle, FTP_USER_NAME, FTP_PASSWORD, FTP_ACCOUNT_NAME );
    printf( "tfFtpLogin = %s\n", tfStrError(errorCode) );
    if (errorCode != TM_ENOERROR)
    {
        printf("FTP client Exiting with login failure\n");
        errorCode = tfFtpFreeSession(ftpHandle);
        printf( "tfFtpFreeSession = %s\n", tfStrError(errorCode) );
        return 1;
    }

	while(1){
		dt[0] = 0;
		key_input(dt);
		memset(arv, 0, sizeof(arv));
		cmm = command_check();
		switch(cmm){
			case CMM_PWD:
				errorCode = tfFtpPwd( ftpHandle, remote_pwd, 128 );
				printf( "tfFtpPwd = %s\n", tfStrError(errorCode) );
				printf("%s\n",remote_pwd);
				break;

			case CMM_DIR:
				errorCode = tfFtpDirList( ftpHandle, arv[1], TM_DIR_LONG, ftpCBFunc );
				printf( "tfFtpDirList = %s\n", tfStrError(errorCode) );
				break;

			case CMM_PUT:
				printf( "tfFtpType = %s\n", tfStrError(errorCode) );
				if(strlen(arv[1]) != 0){
					if(strlen(arv[2]) == 0){
						strcpy(arv[2],arv[1]);
					}
					errorCode = tfFtpStor( ftpHandle, 0, arv[1], arv[2]);
					printf( "tfFtpStor = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Name\n");
				break;

			case CMM_GET:
				printf( "tfFtpType = %s\n", tfStrError(errorCode) );

				if(strlen(arv[1]) != 0){
					if(strlen(arv[2]) == 0){
						strcpy(arv[2],arv[1]);
					}
					errorCode = tfFtpRetr( ftpHandle, 0,  arv[1], arv[2]);
					printf( "tfFtpRetr = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Name\n");
				break;

			case CMM_CD:
				if(strlen(arv[1]) != 0){
					if(strcmp(arv[1], "..") == 0){
						errorCode = tfFtpCdup( ftpHandle );
						printf( "tfFtpCdup = %s\n", tfStrError(errorCode) );
					}
					else{
						errorCode = tfFtpCwd( ftpHandle, arv[1] );
						printf( "tfFtpCwd = %s\n", tfStrError(errorCode) );
					}
                }
				else{
					printf("No Directry Name\n");
				}
				break;

			case CMM_DEL:
				if(strlen(arv[1]) != 0){
					errorCode = tfFtpDele( ftpHandle, arv[1] );
					printf( "tfFtpDele = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Name\n");
				break;

			case CMM_REN:
				if((strlen(arv[1]) != 0) && (strlen(arv[2]) != 0)){
					errorCode = tfFtpRename( ftpHandle, arv[1], arv[2] );
					printf( "tfFtpRename = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Name\n");
				break;

			case CMM_MD:
				if(strlen(arv[1]) != 0){
					errorCode = tfFtpMkd( ftpHandle, arv[1], mkbuff, 128 );
					printf( "tfFtpMkd = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Directry\n");
				break;

			case CMM_RD:
				if(strlen(arv[1]) != 0){
					errorCode = tfFtpRmd( ftpHandle, arv[1] );
					printf( "tfFtpRmd = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Directry\n");
				break;

			case CMM_RESET:
				errorCode = tfFtpRein( ftpHandle );
				printf( "tfFtpRein = %s\n", tfStrError(errorCode) );
				break;

			case CMM_ABOR:
				errorCode = tfFtpAbor( ftpHandle );
				printf( "tfFtpAbor = %s\n", tfStrError(errorCode) );
				break;

			case CMM_ASCII:
				errorCode = tfFtpType( ftpHandle, TM_TYPE_ASCII );
				break;

			case CMM_BIN:
				errorCode = tfFtpType( ftpHandle, TM_TYPE_BINARY );
				break;

			case CMM_APP:
				if(strlen(arv[1]) != 0){
					if(strlen(arv[2]) == 0){
						strcpy(arv[2],arv[1]);
					}
					errorCode = tfFtpAppe( ftpHandle, 0, arv[1], arv[2]);
					printf( "tfFtpStor = %s\n", tfStrError(errorCode) );
				}
				else
					printf("No File Name\n");
				break;

			case CMM_HELP:
				printf("dir  put  get  del  ren  pwd  rd  md  cd  help  reset abo ascii bin append quit\n");
				break;

			case CMM_QUIT:
				errorCode = tfFtpQuit(ftpHandle);
				printf( "tfFtpQuit = %s\n", tfStrError(errorCode) );

				errorCode = tfFtpFreeSession(ftpHandle);
				printf( "tfFtpFreeSession = %s\n", tfStrError(errorCode) );

				printf("FTP Quit\n");
				return 0;
            default:
                printf("Unrecognized command\n");
		}
	}
}


int ftpCBFunc(ttUserFtpHandle ftpHandle, char *bufferPtr, int bufferSize)

{
	printf("%s", bufferPtr);
}

void tfAbort (int sig);

/*
 * Initialize abort functions so that we can clean up before exiting
 * the program
 */
static void tfSignalInit (void)
{
	signal(SIGINT, tfAbort); /* Ctrl-C */
	signal(SIGILL, tfAbort); /* illegal instruction */
	signal(SIGSEGV, tfAbort); /* segment violation */
	signal(SIGTERM, tfAbort); /* Software termination signal from kill() */
	signal(SIGABRT, tfAbort); /* abort call */
}


/*
 * Clean up before exiting
 */
void tfAbort (int sig)
{
	TM_UNREF_IN_ARG(sig);
	
	printf("Program Terminated\n");
	
	printf("error code = %u\n",sig);
	
	exit(1);
}

#endif /* TM_MULTIPLE_CONTEXT */

#else /* !TM_KERNEL_WIN32_X86 */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltxWin32Dummy = 0;

#endif /* !TM_KERNEL_WIN32_X86 */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: txwin32.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 55765    Output checksum: 57247
 */
