/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfAddArpEntry)
 *
 * Filename: traddarp.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/traddarp.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:39CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

/* Add an ARP Entry to ARP table */
int tfAddArpEntry (
#ifdef TM_USE_STRONG_ESL
                   ttUserInterface      interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                   ttUserIpAddress      arpIpAddress,
                   const char TM_FAR *  physAddrPtr,
                   int                  physAddrLength)
{
    int           errorCode;

    if (    (tm_ip_zero(arpIpAddress))
            || (physAddrPtr == (const char TM_FAR *)0)
#if (TM_MAX_PHYS_ADDR <= 255)
            || (physAddrLength > TM_MAX_PHYS_ADDR)
#else /* !(TM_MAX_PHYS_ADDR <= 255) */
            || (physAddrLength > 255)
#endif /* !(TM_MAX_PHYS_ADDR <= 255) */
#ifdef TM_USE_STRONG_ESL
         || ( tfValidInterface(interfaceHandle) != TM_ENOERROR )
#endif /* TM_USE_STRONG_ESL */
       )
    {
        errorCode = TM_EINVAL;
    }
    else
    {
        errorCode = tf4RtArpAddEntry(
#ifdef TM_USE_STRONG_ESL
                                    (ttDeviceEntryPtr)interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                                    (tt4IpAddress)arpIpAddress,
                                    (tt8BitPtr)physAddrPtr,
                                    TM_RTE_INF,
                                    (tt8Bit) physAddrLength);
    }
    return errorCode;
}

/***************************************************************************
 * FUNCTION: tfNgAddArpEntry
 *
 * PURPOSE: Next Generation API replaces tfAddArpEntry.
 *
 * PARAMETERS:
 *   arpIpAddrPtr
 *       pointer to the IP addr information, 
 *   physAddrPtr
 *       pointer to a char array that contains the physical address
 *   physAddrLen
 *       length of the physical address
 * RETURNS:
 *   TM_ENOERROR
 *      sucess
 *   TM_EINVAL
 *       arpIpAddrPtr or physAddrPtr  is null pointer
 *       or physAddrLen is too short
 *
 ***************************************************************************/
int tfNgAddArpEntry(
#ifdef TM_USE_STRONG_ESL
    ttUserInterface                         interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
    const struct sockaddr_storage TM_FAR *  arpIpAddrPtr,
    const ttUser8Bit TM_FAR *               physAddrPtr, 
    int                                     physAddrLen )
{
    int          errorCode;

    if (    (physAddrPtr == (const ttUser8Bit TM_FAR *)0 )
         || (physAddrLen <= 0) 
         || (arpIpAddrPtr == (ttConstSockAddrStoragePtr)0)
#ifdef TM_USE_STRONG_ESL
         || ( tfValidInterface(interfaceHandle) != TM_ENOERROR )
#endif /* TM_USE_STRONG_ESL */
       )
    {
        errorCode = TM_EINVAL;
    }
    else
    {
        switch (arpIpAddrPtr->ss_family)
        {
        case AF_INET:
            errorCode = tf4RtArpAddEntry(
#ifdef TM_USE_STRONG_ESL
                        (ttDeviceEntryPtr)interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                        (tt4IpAddress)arpIpAddrPtr->addr.ipv4.sin_addr.s_addr,
                        (tt8BitPtr)physAddrPtr,
                        TM_RTE_INF,
                        (tt8Bit) physAddrLen);
            break;

        
        default:
            errorCode = TM_EAFNOSUPPORT;
            break;
        }
    }        

    return errorCode;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:46      For: Xilinx-Samsung
 * File Name: traddarp.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 12921    Output checksum: 47646
 */
