/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfDelDefaultGateway)
 *
 * Filename: trdelgw.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trdelgw.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:55CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>

#include <trtype.h>
#include <trproto.h>

/*
 * IPV4 only function.
 * tfDelDefaultGateway function description:
 * Called from the socket interface to delete the default gateway
 * Parameters:
 * gatewayIpAddress: default gateway Ip address
 * Return values:
 * See tfDelRoute()
 *
 */
int tfDelDefaultGateway (
#ifdef TM_USE_STRONG_ESL
                         ttUserInterface  interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                         ttUserIpAddress  gatewayIpAddress)
{
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr    devEntryPtr;
#endif /* TM_USE_STRONG_ESL */ 
    int                 errorCode;

#ifdef TM_USE_STRONG_ESL
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;     
    
    if ( tfValidInterface(devEntryPtr) != TM_ENOERROR) 
    {
        errorCode = TM_EINVAL;
    }
    else
#endif /* TM_USE_STRONG_ESL */ 
    {
        if (tm_ip_not_zero((tt4IpAddress)gatewayIpAddress))
        {
            errorCode = tf4RtDelRoute(
#ifdef TM_USE_STRONG_ESL
                                       devEntryPtr,
#endif /* TM_USE_STRONG_ESL */
                                       (tt4IpAddress)gatewayIpAddress,
                                       TM_IP_ZERO,
                                       TM_RTE_STATIC);
        }
        else
        {
            errorCode = TM_EINVAL;
        }
    }
    return (errorCode);
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trdelgw.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 28081    Output checksum: 26073
 */
