/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfDelStaticRoute)
 *
 * Filename: trdelsrt.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trdelsrt.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:42:56CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

/*
 * Delete a static route from the routing table
 */
int tfDelStaticRoute (
#ifdef TM_USE_STRONG_ESL
                      ttUserInterface  interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                      ttUserIpAddress  destIpAddress,
                      ttUserIpAddress  destNetMask)
{
    int                 errorCode;
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr    devEntryPtr;
#endif /* TM_USE_STRONG_ESL */ 

#ifdef TM_USE_STRONG_ESL
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;     
    if ( tfValidInterface(devEntryPtr) != TM_ENOERROR) 
    {
        errorCode = TM_EINVAL;
    }
    else
#endif /* TM_USE_STRONG_ESL */ 
    {
        if (    (tm_ip_not_zero((tt4IpAddress)destIpAddress))
             && (tm_ip_not_zero((tt4IpAddress)destNetMask))
             && (tm_ip_not_zero(tm_ip_net( (tt4IpAddress)destIpAddress,
                                           (tt4IpAddress)destNetMask ))) )
        {
            errorCode = tf4RtDelRoute(
#ifdef TM_USE_STRONG_ESL
                                      devEntryPtr,
#endif /* TM_USE_STRONG_ESL */
                                      (tt4IpAddress)destIpAddress,
                                      (tt4IpAddress)destNetMask,
                                      TM_RTE_STATIC);
        }
        else
        {
            errorCode = TM_EINVAL;
        }
    }
    return (errorCode);
} 

/****************************************************************************
* FUNCTION: tfNgDelStaticRoute
*
* PURPOSE: Next Generation API replaces tfDelStaticRoute
*
* PARAMETERS:
*   None
*
* RETURNS:
*   Nothing
*
* NOTES:
*
****************************************************************************/
int tfNgDelStaticRoute(
#ifdef TM_USE_STRONG_ESL
    ttUserInterface                         interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
    const struct sockaddr_storage TM_FAR *  destIpAddrPtr,
    int                                     prefixLen )
{
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr devEntryPtr;
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress    ipv4NetMask;    
    int             errorCode;
    
/* parameter validation */
#ifdef TM_USE_STRONG_ESL
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;     
    
    if ( tfValidInterface(devEntryPtr) != TM_ENOERROR) 
    {
        errorCode = TM_EINVAL;
    }
    else
#endif /* TM_USE_STRONG_ESL */ 
    {
        if (destIpAddrPtr == (ttConstSockAddrStoragePtr)0)
        {
            errorCode = TM_EINVAL;
        }
        else
        {
            switch (destIpAddrPtr->ss_family)
            {
            case AF_INET:
                tm_4_netmask(prefixLen, ipv4NetMask);
                errorCode = 
                    tfDelStaticRoute(
#ifdef TM_USE_STRONG_ESL
                                     devEntryPtr,
#endif /* TM_USE_STRONG_ESL */
                                     destIpAddrPtr->addr.ipv4.sin_addr.s_addr, 
                                     ipv4NetMask);
                break;

            default:
                errorCode = TM_EAFNOSUPPORT;
                break;
            }
        }
    }
    return errorCode;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trdelsrt.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 37961    Output checksum: 54324
 */
