/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2007
 * Copyright Elmic Systems Japan 1997 - 2007
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Device Driver options: tfInterfaceSetOptions()
 *
 * Filename: trdvstat.c
 * Author: Jared
 * Date Created: 07/11/07
 * $Source: source/sockapi/trdvstat.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:05CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#ifdef TM_LOCK_NEEDED
#include <trglobal.h>
#endif /* TM_LOCK_NEEDED */

#ifdef TM_FORWARDING_STATS

/*
 * tfInterfaceClearStats function description:
 * Called by the user to retrieve (and possibly clear) the specified interface
 * stat.
 */
int tfInterfaceGetStats(ttUserInterface interfaceHandle,
                        int             statName,
                        void TM_FAR *   statValuePtr,
                        int             statLen,
                        int             flag)
{
    ttDeviceEntryPtr   devEntryPtr;
    char TM_FAR *      srcPtr;
    int                errorCode;

    errorCode = TM_EINVAL;
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    srcPtr = (char TM_FAR *)0;
/* Check on parameters */
    if ( tfValidInterface(devEntryPtr) == TM_ENOERROR )
    {
/* Make sure the user passed in a valid length */
        switch (statName)
        {
            case TM_INTF_STAT_FORW_INC_PKT:
            case TM_INTF_STAT_FORW_OUT_PKT:
                if (statLen == sizeof(tt32Bit))
                {
                    errorCode = TM_ENOERROR;
                }
                break;
            case TM_INTF_STAT_FORW_INC_BYTE:
            case TM_INTF_STAT_FORW_OUT_BYTE:
                if (statLen == sizeof(ttIntfStatBignum))
                {
                    errorCode = TM_ENOERROR;
                }
                break;
            default:
                break;
        }

        if (errorCode == TM_ENOERROR)
        {

/* Set the correct stat to retrieve */
            switch (statName)
            {
                case TM_INTF_STAT_FORW_INC_PKT:
                    srcPtr = (char TM_FAR *)&(devEntryPtr->devForwRxPkts);
                    break;

                case TM_INTF_STAT_FORW_INC_BYTE:
                    srcPtr = (char TM_FAR *)&(devEntryPtr->devForwRxBytes);
                    break;

                case TM_INTF_STAT_FORW_OUT_PKT:
                    srcPtr = (char TM_FAR *)&(devEntryPtr->devForwTxPkts);
                    break;

                case TM_INTF_STAT_FORW_OUT_BYTE:
                    srcPtr = (char TM_FAR *)&(devEntryPtr->devForwTxBytes);
                    break;
            }

/* Lock device */
            tm_call_lock_wait(&(devEntryPtr->devLockEntry));

/* Save the data to the user's buffer */
            tm_bcopy(srcPtr, statValuePtr, statLen);
/* If requested, reset the stat to 0 */
            if (flag == TM_INTF_STAT_FLAG_CLEAR)
            {
                tm_bzero(srcPtr, statLen);
            }

/* Unlock the device */
            tm_call_unlock(&(devEntryPtr->devLockEntry));
        }

    }

    return errorCode;
}
#else /* TM_FORWARDING_STATS */

/* To allow link for builds when TM_FORWARDING_STATS is not defined */
int tvIntfStatsDummy = 0;

#endif /* TM_FORWARDING_STATS */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trdvstat.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 13205    Output checksum: 47646
 */
