/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Treck filtering functions
 * Filename: trfilter.c
 * Author: J Poffenbarger
 * Date Created: 05/09/07
 * $Source: source/sockapi/trfilter.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2010/05/11 12:44:43CDT $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#ifdef TM_USE_FILTERING
/*
 * tfUserRegisterFilter
 *
 * Purpose  This function registers an IPv4-based filter which allows a
 *          user to supply the filtering function to process packet content
 *          prior to continued stack processing.  The return value of the
 *          filter function determines the continued behavior of the stack.
 *          For more information, see the ttUserFilterCallback prototype.
 *
 * Note:  Only one function can be registered; it cannot be replaced
 *
 * Parms    filterCallback          The user-specified callback or NULL
 *
 * Returns  TM_EALREADY			    A filter has already been registered
 *          TM_ENOERROR             The new filter was registered
 *
 */
int tfUserRegisterFilter(ttUserFilterCallback filterCallback)
{
    int     errorCode;

    errorCode = TM_EALREADY;

    if (tm_context(tvUserFilterCallback) == (ttUserFilterCallback)0)
    {
        tm_context(tvUserFilterCallback) = filterCallback;
        errorCode = TM_ENOERROR;
    }

    return errorCode;
}
#endif /* TM_USE_FILTERING */


#ifdef TM_USE_IGMP_FILTERING
/*
 * tfUserRegisterIgmpFilter
 *
 * Purpose  This function registers an IPv4-based IGMP filter which allows a
 *          user to supply the filtering function to process IGMP packet content
 *          prior to continued stack processing.  The return value of the
 *          filter function determines the continued behavior of the stack.
 *          For more information, see the ttUserIgmpFilterCallback prototype.
 *
 * Note:  Only one function can be registered; it cannot be replaced
 *
 * Parms    igmpFilterCallback      The user-specified callback or NULL
 *
 * Returns  TM_EALREADY			    A filter has already been registered
 *          TM_ENOERROR             The new filter was registered
 *
 */
int tfUserRegisterIgmpFilter(ttUserIgmpFilterCallback igmpFilterCallback)
{
    int     errorCode;

    errorCode = TM_EALREADY;

    if (tm_context(tvUserIgmpFilterCallback) == (ttUserIgmpFilterCallback)0)
    {
        tm_context(tvUserIgmpFilterCallback) = igmpFilterCallback;
        errorCode = TM_ENOERROR;
    }

    return errorCode;
}
#endif /* TM_USE_IGMP_FILTERING */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trfilter.c    File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 14572    Output checksum: 16006
 */
