/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (getpeername)
 *
 * Filename: trgpeer.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trgpeer.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:20CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/*
 * Get the remote IP and Port if assigned
 */
int getpeername(int socketDescriptor, struct sockaddr TM_FAR *fromAddressPtr, 
                int TM_FAR *addressLengthPtr)
{
    int                        errorCode;
    int                        retCode;
    ttSocketEntryPtr           socketEntryPtr;
    int                        addressLength; 


/* PARAMETER CHECK */
    if (addressLengthPtr != (int TM_FAR *)0)
    {
        addressLength = *addressLengthPtr;
    }
    else
    {
        addressLength = 0;
    }

/*
 * Map from socket descriptor to a locked socket entry pointer and verify
 * that the address length is at least what we expect for this address type.
 */    
    socketEntryPtr = tfSocketCheckAddrLenLock( socketDescriptor,
                                               fromAddressPtr,
                                               addressLength,
                                               &errorCode
                                              );
    {
        if (socketEntryPtr == TM_SOCKET_ENTRY_NULL_PTR)
        {
/* errorCode set by tfSocketCheckAndLock */
            goto getpeernameFinish;
        }
        if ( fromAddressPtr == TM_SOCKADDR_NULL_PTR )
        {
            errorCode = TM_EINVAL;
            goto getpeernameFinish;
        }
/* Check to see that is connected */
        if (!(socketEntryPtr->socFlags & TM_SOCF_CONNECTED))
        {
            errorCode = TM_ENOTCONN;
            goto getpeernameFinish;
        }
    
        errorCode = TM_ENOERROR;
    
getpeernameFinish:
        retCode = tfSocketReturnAddr(socketEntryPtr, socketDescriptor,
                                     fromAddressPtr,
                                     errorCode, TM_ENOERROR, TM_ADDR_PEER_FLAG);
    }
    return retCode;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trgpeer.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 10782    Output checksum: 31085
 */
