/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfGetPppPeerIpAddress)
 *
 * Filename: trgpppip.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trgpppip.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2011/03/28 14:28:51CST $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#if defined(TM_LOCK_NEEDED) || defined(TM_4_USE_SCOPE_ID)
#include <trglobal.h>
#endif /* TM_LOCK_NEEDED || TM_4_USE_SCOPE_ID */

/* 
 * Get the IP address of a point to point peer (PPP, or SLIP)
 */
int tfGetPppPeerIpAddress (ttUserInterface interfaceHandle,
                           ttUserIpAddress TM_FAR *ifIpAddressPtr)
{
    ttDeviceEntryPtr devEntryPtr;
    int              errorCode;
    
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    errorCode = tfValidConfigInterface( devEntryPtr, TM_16BIT_ZERO );
    if (errorCode == TM_ENOERROR)
    {
        errorCode = TM_EINVAL; /* assume failure */
        if (ifIpAddressPtr != (ttUserIpAddress TM_FAR *)0)
        {
            if (tm_ll_is_pt2pt(devEntryPtr))
            {
                tm_call_lock_wait( &(devEntryPtr->devLockEntry) );
                tm_ip_copy( tfDeviceDestIpAddress(devEntryPtr),
                            *ifIpAddressPtr );
                tm_call_unlock( &(devEntryPtr->devLockEntry) );
                errorCode = TM_ENOERROR; /* success */
            }
        }
    }
    return errorCode;
}

    
/****************************************************************************
 * FUNCTION: tfNgGetPt2PtPeerIpAddress
 *
 * PURPOSE: Next Generation API replaces tfGetPppPeerIpAddress
 *
 * PARAMETERS:
 *   interfaceHandle
 *       Handle to the interface whose peer IP address is to be retrieved
 *   ifIpAddrPtr
 *       IP Address struct to save the retrieved Peer IP address, 
 *       ifIpAddrPtr->ss_family should be properly set.
 * RETURNS:
 *   TM_ENOERROR
 *      success    
 *   TM_EINVAL
 *      the interface handle is invalid, or the device is not point to point
 *   TM_ENETDOWN
 *      the point to point connection is not configured. 
 *
 * NOTES:
 *
 ****************************************************************************/
int tfNgGetPt2PtPeerIpAddress(
    ttUserInterface interfaceHandle,
    struct sockaddr_storage TM_FAR * ifIpAddrPtr )
{
    int              errorCode;
    tt8Bit           family;

/* parameter validation */
    if (ifIpAddrPtr == TM_SOCKADDR_STORAGE_NULL_PTR)
    {
        errorCode = TM_EINVAL;
    }
    else
    {
        family = ifIpAddrPtr->ss_family;
        tm_memset(ifIpAddrPtr, 0, sizeof(struct sockaddr_storage));
        ifIpAddrPtr->ss_family = family;
        ifIpAddrPtr->ss_len    = sizeof(struct sockaddr_storage);

        switch (family)
        {
        case AF_INET:
            errorCode = tfGetPppPeerIpAddress(
                                interfaceHandle,
                                &(ifIpAddrPtr->addr.ipv4.sin_addr.s_addr));
            break;

        
        default:
            errorCode = TM_EAFNOSUPPORT;
            break;
        }
    }

    return errorCode;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trgpppip.c    File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 28340    Output checksum: 55274
 */
