/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfGetSendBytesLeft)
 *
 * Filename: trgsendb.c
 * Author: Odile
 * Date Created: 10/27/03
 * $Source: source/sockapi/trgsendb.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:23CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>


/* 
 * Get the number of bytes left to be written on a socket.
 */ 
int tfGetSendBytesLeft (int socketDescriptor)
{
    int              retCode;
    int              errorCode;
    ttSocketEntryPtr socketEntryPtr;
    tt32Bit         totalInFlightBytes;
    tt32Bit         availableSpace;
    
/* PARAMETER CHECK */
    socketEntryPtr = tfSocketCheckValidLock(socketDescriptor
                                           );
    if (socketEntryPtr != TM_SOCKET_ENTRY_NULL_PTR)
    {
        totalInFlightBytes = socketEntryPtr->socSendQueueBytes;
        if (totalInFlightBytes >= socketEntryPtr->socMaxSendQueueBytes)
        {
            retCode = 0;
        }
        else
        {
            availableSpace =   socketEntryPtr->socMaxSendQueueBytes
                             - totalInFlightBytes;
            retCode = (int)availableSpace;
        }
        errorCode = TM_ENOERROR;
    }
    else
    {
        {
            errorCode = TM_EBADF;
        }
        retCode = TM_SOCKET_ERROR;
    }
    return tfSocketReturn(socketEntryPtr, socketDescriptor,
                          errorCode, retCode);
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trgsendb.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  7000    Output checksum: 14330
 */
