/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (getsockname)
 *
 * Filename: trgsockn.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trgsockn.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:25CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/*
 * Get our local IP and Port if assigned
 */
int getsockname(int socketDescriptor, struct sockaddr TM_FAR *myAddressPtr, 
                int TM_FAR *addressLengthPtr)
{
    int                        errorCode;
    int                        addressLength;
    int                        retCode;
    ttSocketEntryPtr           socketEntryPtr;

/* PARAMETER CHECK */
    if (addressLengthPtr != (int TM_FAR *)0)
    {
        addressLength = *addressLengthPtr;
    }
    else
    {
        addressLength = 0;
    }
    
    socketEntryPtr = tfSocketCheckAddrLenLock(  socketDescriptor,
                                                myAddressPtr,
                                                addressLength,
                                                &errorCode
                                             );
/* errorCode set by tfSocketCheckAndLock, if socketEntryPtr is null */
    {
        if (socketEntryPtr != TM_SOCKET_ENTRY_NULL_PTR)
        {
            if (myAddressPtr == TM_SOCKADDR_NULL_PTR)
            {
                errorCode = TM_EINVAL;
            }
            else
            {
                errorCode = TM_ENOERROR;
            }
        }
        retCode = tfSocketReturnAddr( socketEntryPtr, socketDescriptor,
                                      myAddressPtr, 
                                      errorCode, TM_ENOERROR, 
                                      TM_ADDR_LOCAL_FLAG );
    }
    return retCode;
}



/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trgsockn.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 23071    Output checksum: 20399
 */
