/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2009
 * Copyright Elmic Systems Japan 1997 - 2009
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: get/free array of name/index pairs for all interfaces
 *
 * Filename: trifarr.c
 * Author: Lorne
 * Date Created: 11/27/09
 * $Source: source/sockapi/trifarr.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:30CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>

#ifdef TM_USE_IFNAMEINDEX

#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>


/****************************************************************************
* FUNCTION: if_nameindex
*
* PURPOSE:  get an array containing the name and corresponding index of
*           all interfaces.
*
* PARAMETERS:
*   (none)
* RETURNS:
*   A pointer to an array of if_nameindex structures terminated with an entry
*   that has an if_index value of 0 and a NULL if_name value.
*   A NULL pointer will be returned if the memory could not be allocated.
* NOTES:
*   Call if_freenameindex() to release the storage returned by this function.
*
****************************************************************************/
struct if_nameindex TM_FAR * if_nameindex(void)
{
    struct if_nameindex TM_FAR *    niRetPtr;
    struct if_nameindex TM_FAR *    niPtr;
    tt8BitPtr                       namePtr;
    unsigned int                    ifIndex;
    unsigned int                    ifCount;

/* Allocate memory for the array of structures and names */
    ifCount = tm_context(tvAddedInterfaces);
    niRetPtr = (struct if_nameindex TM_FAR *) tm_get_raw_buffer(
                        ((ifCount + 1) * sizeof(struct if_nameindex))
                      + (ifCount * tm_align(TM_MAX_DEVICE_NAME)) );
    if (niRetPtr != (struct if_nameindex TM_FAR *)0)
    {
        niPtr = niRetPtr;
/* The names are stored after the array of structures */
        namePtr = (tt8BitPtr)(niPtr + ifCount + 1);
/* Iterate for all interface indices */
        for (ifIndex = 1; ifIndex <= ifCount; ++ifIndex)
        {
/* Copy the interface name */
            tm_memcpy( namePtr,
                       tm_context(tvDevIndexToDevPtr[ifIndex])->devNameArray,
                       tm_align(TM_MAX_DEVICE_NAME) );
/* Fill in the if_nameindex structure entry.
 * Move to the next free if_nameindex entry. */
            niPtr->if_index = ifIndex;
            niPtr++->if_name = (ttCharPtr)namePtr;
/* Move to the next free interface name space */
            namePtr += tm_align(TM_MAX_DEVICE_NAME);
        }
/* Terminate the list */
        niPtr->if_index = 0;
        niPtr->if_name = (ttCharPtr)0;
    }
    return niRetPtr;
}

/****************************************************************************
* FUNCTION: if_freenameindex
*
* PURPOSE:  free an if_nameindex structure array.
*
* PARAMETERS:
*   niPtr
*       pointer returned by function if_nameindex().
* RETURNS:
*   (nothing)
* NOTES:
*
****************************************************************************/
void if_freenameindex(struct if_nameindex TM_FAR * niPtr)
{
    if (niPtr != (struct if_nameindex TM_FAR *)0)
    {
        tm_free_raw_buffer(niPtr);
    }
}
#else /* !TM_USE_IFNAMEINDEX */

LINT_UNUSED_HEADER

/* To allow link for builds when TM_USE_IPV6 is not defined */
int tvIfArr = 0;
#endif /* !TM_USE_IFNAMEINDEX */

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trifarr.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 64606    Output checksum:  8365
 */
