/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (inet_addr, and inet_aton)
 *
 * Filename: trinetad.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trinetad.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2010/12/08 15:05:14CST $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>


/* Common code for inet_aton() and inet_addr() */
static ttUser32Bit tfInetAtoN(char TM_FAR *strPtr, int badRetCode);


static const int  TM_CONST_QLF tlPower10[4]={1,10,100,1};
/*
 * Convert from dotted string to unsigned long
 * The IP address is stored in network byte order
 * On failure, return (unsigned long)-1.
 */
ttUser32Bit inet_addr(char TM_FAR *strPtr)
{
    return tfInetAtoN(strPtr, TM_SOCKET_ERROR);
}

/*
 * Convert from dotted string to unsigned long
 * The IP address is stored in network byte order
 * On failure, return (unsigned long)0.
 */
ttUser32Bit inet_aton(char TM_FAR *strPtr)
{
    return tfInetAtoN(strPtr, 0);
}

/*
 * Called from inet_addr() or inet_aton().
 * Convert from dotted string to unsigned long
 * The IP address is stored in network byte order
 * On failure, return (unsigned long)badRetcode.
 */
static ttUser32Bit tfInetAtoN(char TM_FAR *strPtr, int badRetCode)
{
    int                   value[4];
    int                   strValIndex;
    int                   valueIndex;
    int                   errorCode;
    ttUser32Bit           retAddr;
    unsigned char TM_FAR *retAddrPtr;

    strValIndex=0;
    valueIndex=0;
    value[0] = -1;
    value[1] = -1;
    value[2] = -1;
    value[3] = -1;
/* Loop until we hit the end of the string */
    while (*strPtr != '\0')
    {
        if ( ( (*strPtr >='0') && (*strPtr <='9') ) || (*strPtr=='.') )
        {
            if (*strPtr=='.')
            {
/* Shift the value to right (base10) by the number of digits missing */
                value[valueIndex] /= tlPower10[3-strValIndex];
/* Dots cause us to go to the next value */
                strValIndex=0;
                valueIndex++;
                if (valueIndex > 3)
                {
/* Check for too many DOTS */
                    value[0] = -1;
                    break;
                }
            }
            else
            {
                if (strValIndex > 2)
                {
/* Check for too many digits (we only support 3 per number) */
                    value[0] = -1;
                    break;
                }
                if (value[valueIndex] == -1)
                {
/* First time we are adding into this value so zero it */
                    value[valueIndex]=0;
                }
                value[valueIndex] +=   (*strPtr-'0')
                                     * (tlPower10[(2-strValIndex)]);
                ++strValIndex;
            }
        }
        else
        {
/* Check for leading/trailing spaces */
            if (*strPtr == ' ')
            {
/* Leading Space check */
                if ((strValIndex != 0) || (valueIndex  != 0))
                {
/* Trailing space check */
                    if((valueIndex == 3) && (strValIndex !=0))
                    {
/* Trailing spaces cause us to drop out of the loop */
                        break;
                    }
                    else
                    {
/* Somewhere in the middle so it is an error */
                        value[0] = -1;
                        break;
                    }
                }
/* They both were zero so it must be a leading space, so ignore it */
            }
/* All other characters cause an error */
            else
            {
                value[0] = -1;
                break;
            }
        }
/* Look at the next character */
        ++strPtr;
    }
    if (strValIndex < 4)
    {
/* Shift the value to right (base10) by the number of digits missing */
        value[valueIndex] /= tlPower10[3-strValIndex];
    }
    errorCode=TM_ENOERROR;
/* We are going to write the address in network byte order in the long */
    retAddrPtr=(unsigned char TM_FAR *)&retAddr;
    for (valueIndex=0; valueIndex<4; valueIndex++)
    {
/* Make sure the number was set and that it is not too large */
        if (value[valueIndex] > 255 || value[valueIndex] == -1)
        {
/* Gice up on any value beign out of range */
            errorCode=TM_EINVAL;
            break;
        }
/* Save each value as a unsigned char */
        *retAddrPtr++ = (unsigned char)(value[valueIndex]);
    }
    if (errorCode==TM_EINVAL)
    {
/* If we had an error, return Error Code */
        retAddr = (ttUser32Bit)(badRetCode);
    }
    return (retAddr);
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trinetad.c    File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 19160    Output checksum: 45858
 */
