/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description:  BSD sockets inet_ntoa function
 *
 * Filename: trinetnt.c
 * Author: Odile
 * Date Created: 2/19/99
 * $Source: source/sockapi/trinetnt.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:35CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h>


#include <trtype.h>

#ifndef tm_get_byte_ptr
/* backward compatibility */
/* retrieve byte at offset in d data pointed to by dataPtr */
#define tm_get_byte_ptr(dataPtr, offset) \
        (((int)dataPtr[offset]) & 0xFF)
#endif /* !tm_get_bypte_ptr */

static char TM_FAR tlInetBuffer[16];


/*
 * Convert a socket address to the "h1.h2.h3.h4\0" format.
 * Parameters
 * Value          Meaning
 * in_addr        IN address structure containing the IP address
 *
 * Return value:
 * Pointer to converted string.
 */
char TM_FAR * inet_ntoa ( struct in_addr  inAddr )
{
        ttCharPtr   aHostPtr;
auto    tt4IpAddress hostIpAddress;
        int         commandIndex;
        int         dotIndex;
        int         tempInt;
        int         digit;
        char        delimiterChar;

static  const char   TM_CONST_QLF tlAsciiDigit[11] = "0123456789";

    tm_ip_copy(inAddr.s_addr, hostIpAddress);
    aHostPtr = (ttCharPtr)&hostIpAddress;
    commandIndex = 0;
    dotIndex = 0;
    delimiterChar = '.';
    while ( (unsigned)dotIndex < tm_byte_count(sizeof(tt4IpAddress) ) )
    {
        tempInt = (int)tm_get_byte_ptr(aHostPtr, dotIndex);
        if (tempInt >= 100)
        {
            digit = tempInt/100;
            tempInt = tempInt % 100;
            tlInetBuffer[commandIndex] = tlAsciiDigit[digit];
            commandIndex++;
            if (tempInt < 10)
/* Middle digit is zero */
            {
                tlInetBuffer[commandIndex] = tlAsciiDigit[0];
                commandIndex++;
            }
        }
        if (tempInt >= 10)
        {
            digit = tempInt/10;
            tempInt = tempInt % 10;
            tlInetBuffer[commandIndex] = tlAsciiDigit[digit];
            commandIndex++;
        }
        tlInetBuffer[commandIndex] = tlAsciiDigit[tempInt];
        commandIndex++;
        dotIndex++;
        if ( (unsigned)dotIndex == (tm_byte_count(sizeof(tt4IpAddress))) )
        {
            delimiterChar = '\0';
        }
        tlInetBuffer[commandIndex++] = delimiterChar;
    }
    return tlInetBuffer;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trinetnt.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 27530    Output checksum:   680
 */
