/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfIoctl)
 *
 * Filename: trioctl.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trioctl.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2011/11/08 23:51:27CST $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

typedef struct ip_msfilter TM_FAR * ttIpmsfilterPtr;
/*
 * BSD ioctl. Valid 'request' values are:
 *             FIONBIO (can also be done with tfBlockingState()),
 *             FIONREAD (can also be done with tfGetWaitingBytes()), and
 *             SIOCATMARK (can also be done with tfGetOobDataOffset()).
 * tfIoctl will return an error for any other request.
 */
int tfIoctl(int socketDescriptor, ttUser32Bit request, int TM_FAR * dataArg)
{
    ttSocketEntryPtr            socketEntryPtr;
    int                         errorCode;
    int                         retCode;
    
/* PARAMETER CHECK */
    socketEntryPtr = tfSocketCheckValidLock(socketDescriptor
                                           );
    {
        if (socketEntryPtr != TM_SOCKET_ENTRY_NULL_PTR)
        {
            errorCode = TM_ENOERROR;

            if (dataArg == (int TM_FAR *)0)
            {
                errorCode = TM_EINVAL;
                goto ioctlExit;
            }
            switch (request)
            {
                case FIONBIO:
                    if (*dataArg == 0)
                    {
/* clear non blocking IO */
                        socketEntryPtr->socFlags &= ~TM_SOCF_NONBLOCKING;
                    }
                    else
                    {
/* set non blocking IO */
                        socketEntryPtr->socFlags |= TM_SOCF_NONBLOCKING;
                    }
                    break;
                case FIONREAD:
                    *dataArg = (int)(socketEntryPtr->socRecvQueueBytes);
                    break;
                case SIOCATMARK:
                    if (tm_16bit_one_bit_set(socketEntryPtr->socFlags,
                                             TM_SOCF_RCVATMARK))
                    {
/* At out of band mark */
                        *dataArg = 1;
                    }
                    else
                    {
/* Not at out of band mark */
                        *dataArg = 0;
                    }
                    break;
                default:
                    errorCode = TM_EINVAL;
                    break;
            }
        }
        else
        {
            errorCode = TM_EBADF;
        }
ioctlExit:
        retCode = tfSocketReturn(socketEntryPtr, socketDescriptor,
                                 errorCode, TM_ENOERROR);
    }
    return retCode;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trioctl.c     File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 40949    Output checksum: 35479
 */
