/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (listen)
 *
 * Filename: trlisten.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trlisten.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:43CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/*
 * Setup the maximum amount of connections that we will allow
 */
int listen(int socketDescriptor, int backLog)
{
    ttSocketEntryPtr socketEntryPtr;
    int              errorCode;
    int              retCode;
    
/* PARAMETER CHECK */
    socketEntryPtr = tfSocketCheckValidLock(socketDescriptor
                                           );
    {
        if (socketEntryPtr == TM_SOCKET_ENTRY_NULL_PTR)
        {
            errorCode = TM_EBADF;
            goto listenFinish;
        }
        if (backLog < 0)
        {
            errorCode = TM_EINVAL;
            goto listenFinish;
        }
#ifdef TM_USE_TCP
/* Make sure we are setting up listen on a TCP socket only */
        if (socketEntryPtr->socProtocolNumber == (tt8Bit)IP_PROTOTCP)
        {
/* Set the backlog of maximum connections waiting for accept */
            socketEntryPtr->socFlags |= TM_SOCF_LISTENING;
            errorCode = tfTcpOpen( (ttTcpVectPtr)((ttVoidPtr)socketEntryPtr),
                                   TM_TCPE_PASSIVE_OPEN );
            if (errorCode == TM_ENOERROR)
            {
/* Allow matches in tree */
                if (tm_16bit_bits_not_set(socketEntryPtr->socFlags,
                                          TM_SOCF_IN_TREE))
                {
/* Pick random port in connection less tree */
                    retCode = (int)tfRandPortInsert(
                           socketEntryPtr,
                           &(socketEntryPtr->socTuple),
                           (tt16Bit)(  TM_SOC_TREE_CL_FLAG
                                     | (   socketEntryPtr->socOptions
                                         & (SO_REUSEADDR | SO_REUSEPORT) ) ) );
                    if (retCode == 0)
/* No port */
                    {
                        errorCode = TM_EADDRINUSE;
                    }
                }
                else
                {
                    socketEntryPtr->socCoFlags &= ~TM_SOC_TREE_NO_MATCH_FLAG;
                }
            }
            if (errorCode == TM_ENOERROR)
            {
                socketEntryPtr->socMaxBackLog = (tt16Bit)backLog;
            }
            else
            {
                socketEntryPtr->socFlags &= ~TM_SOCF_LISTENING;
            }
        }
        else
#endif /* TM_USE_TCP */
        {
            errorCode = TM_EOPNOTSUPP;
        }
listenFinish:
        retCode = tfSocketReturn(socketEntryPtr, socketDescriptor,
                                 errorCode, TM_ENOERROR);
    }
    return retCode;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trlisten.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 59504    Output checksum: 47975
 */
