/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Convert a signed long integer to string 
 *
 * Filename: trltoa.c
 * Author: Jin Zhang
 * Date Created: 09/09/03
 * $Source: source/sockapi/trltoa.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:43:44CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/* convert a signed long integer to string*/
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>

/*
 * Convert a signed long to a string buffer( right aligned). 
 * Return a pointer to the beginning of the converted number.
 */
char TM_FAR *tfLongToString(ttUserS32Bit  number, 
                            char TM_FAR * string, 
                            unsigned int  stringLength,
                            unsigned int  base)
{
    char          remainder;
    char TM_FAR * resultPtr;
    int           sign;

    resultPtr = (char TM_FAR *)0;
/* number of digits occupied by the sign. */
    sign = 0;
    if (base < 2 || base > 36)
    {
        goto ltoaExit;
    }

    if (number < TM_L(0))
    {
        sign = 1;
        number = -number;
    }
    string[--stringLength] = '\0';

    for (stringLength--; 
         ((int)stringLength >= sign) && (number != TM_L(0));
         stringLength--)
    {
        remainder = (char) (number % base);
        if ( remainder <= 9 )
        {
            string[stringLength] = (char)(remainder + '0');
        }
        else 
        {
            string[stringLength] = (char)(remainder - 10 + 'A');
        }
        number /= base;
    }
/* we don't want any unnecessory space */
    if (sign)
    {
        string[stringLength--] = '-';
    }
    if ( stringLength > 0 )
    {
        tm_memset(string, ' ', stringLength + 1);
    }
    resultPtr = string + stringLength + 1;

ltoaExit:
    return resultPtr;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trltoa.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 49612    Output checksum: 34239
 */
