/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2012
 * Copyright Elmic Systems Japan 1997 - 2012
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Treck User Realloc "C" library function
 *
 * Filename: trreallo.c
 * Author: Odile
 * Date Created: 02/18/99
 * $Source: source/sockapi/trreallo.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2011/12/19 15:22:19CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>

/*
 * Re-allocate the previously allocated block
 * in userPtr, making the new block size bytes long.
 * Unconditionally allocate and copy to a new region.
 */
ttVoidPtr tfRealloc(ttVoidPtr userPtr, unsigned size)
{
    ttVoidPtr       newUserPtr;
    unsigned int    oldSize;

    newUserPtr = tm_malloc(size);
    if ( userPtr != (ttVoidPtr)0 )
    {
/*
 * Make sure that userPtr, newUserPtr, and size are non zero, since
 * tfBcopy assumes that ptr, newPtr, and size are non zero
 */
        if (    (newUserPtr != (ttVoidPtr)0)
             && (size != 0) )
        {
/*
 * tm_malloc is defined as tfMalloc()
 * tfMalloc() calls tm_get_raw_buffer
 * tm_get_raw_buffer is defined as tfGetRawBuffer
 * tfGetRawBuffer stores the size in the 32-bit long preceding the allocated
 * buffer.
 */
            oldSize = *((tt32BitPtr)userPtr - 1);
            if (oldSize < size)
/* Do not copy more than was previously allocated */
            {
                size = oldSize; 
            }
            tm_bcopy(userPtr, newUserPtr, size);
        }
        tm_free(userPtr);
    }
    return newUserPtr;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trreallo.c    File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 63209    Output checksum: 30352
 */
