/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (recv)
 *
 * Filename: trrecv.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trrecv.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:44:04CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/*
 * Take some data from the receive queue and give it to the user 
 */
int recv(int socketDescriptor, char TM_FAR *bufferPtr, int bufferLength,
         int flags)
{
    ttSocketEntryPtr socketEntryPtr;
#ifdef TM_USE_TCP
    ttTcpVectPtr     tcpVectPtr;
    ttPacketPtr      nxtPacketPtr;
#endif /* TM_USE_TCP */
    ttPacketPtr      packetPtr;
    ttPacketPtr      recvdPacketPtr;
    ttPktLen         copyCount;
    ttPktLen         recvdPacketLength;
    ttPktLen         linkCopyCount;
#ifdef TM_USE_TCP
    ttPktLen         nowaitCopyCount;
    ttPktLen         bufferLeftSpace;
#endif /* TM_USE_TCP */
    int              errorCode;
    int              retCode;
#ifdef TM_USE_TCP
    tt8Bit           protocolNumber;
#endif /* TM_USE_TCP */

/* PARAMETER CHECK */
/* Map from socket descriptor to locked socket entry pointer */
    socketEntryPtr = tfSocketCheckValidLock(  socketDescriptor
                                           );
    {
        errorCode = TM_ENOERROR;
        recvdPacketLength = (ttPktLen)0;
        if ( socketEntryPtr == TM_SOCKET_ENTRY_NULL_PTR )
        {
            errorCode = TM_EBADF;
            goto recvFinish;
        }
#ifdef TM_USE_TCP
        tcpVectPtr = (ttTcpVectPtr)((ttVoidPtr)socketEntryPtr);
#endif /* TM_USE_TCP */
        if (    (bufferPtr == (char TM_FAR *)0)
/* we allow bufferlength between 0 and 65534 on 16-bit architecture */
             || (bufferLength == -1)
             || (bufferLength == 0)
             || ((flags & ~TM_RECV_FLAGS) != 0) )
        {
            errorCode = TM_EINVAL;
            goto recvFinish;
        }

#ifdef TM_USE_TCP
        protocolNumber = socketEntryPtr->socProtocolNumber;
#endif /* TM_USE_TCP */
/* Check to see if the socket is connected */
        if (   (!(socketEntryPtr->socFlags & TM_SOCF_CONNECTED))
#ifdef TM_USE_TCP
            && (protocolNumber != (tt8Bit)IP_PROTOTCP)
#endif /* TM_USE_TCP */
           )
        {
/* For a non TCP socket, can only use recv on connected sockets. */
            errorCode = TM_ENOTCONN;
            goto recvFinish;
        }
/* If user did not set any wait/dontwait flag */
        if ((flags & TM_SOCF_MSG_WAIT_FLAGS) ==  0)
        {
/* use the socket default */
            flags |= (socketEntryPtr->socFlags & TM_SOCF_NONBLOCKING);
        }
#ifdef TM_USE_TCP
        if (protocolNumber == (tt8Bit)IP_PROTOTCP)
        {
            if ((flags & MSG_OOB) != 0)
            {
                errorCode = tfTcpRecvOobData(tcpVectPtr, bufferPtr, flags);
                if (errorCode == TM_ENOERROR)
                {
                    recvdPacketLength = (ttPktLen)1; /* one byte */
                }
                goto recvFinish;
            }
        }
        bufferLeftSpace = (ttPktLen)bufferLength;
#ifdef TM_PEND_POST_NEEDED
        do
#endif /* TM_PEND_POST_NEEDED */
#endif /* TM_USE_TCP */
        {
            packetPtr = socketEntryPtr->socReceiveQueueNextPtr;
/* Check for Data */
#ifdef TM_PEND_POST_NEEDED
            while (packetPtr == TM_PACKET_NULL_PTR)
#else /* !TM_PEND_POST_NEEDED */
            if (packetPtr == TM_PACKET_NULL_PTR)
#endif /* !TM_PEND_POST_NEEDED */
/* 
 *  No data. 
 */
            {
                errorCode = socketEntryPtr->socRecvError;
                if (errorCode != TM_ENOERROR)
                {
#ifdef TM_USE_TCP
                    if (recvdPacketLength != (ttPktLen)0)
                    {
/* Return partial data */
                        errorCode = TM_ENOERROR;
                    }
                    else
#endif /* TM_USE_TCP */
                    {
/* No data, and error on socket. Return read error on socket */
                        socketEntryPtr->socRecvError = 0;
                    }
                    goto recvError;
                }
#ifdef TM_USE_TCP
                if (protocolNumber == (tt8Bit)IP_PROTOTCP)
                {
/*
 * Check whether there is any reason to wait for more data. OK to make
 * a call to TCP state machine (instead of in-lining check on TCP state)
 * since there is no data in the receive queue anyway.
 */
                    errorCode = tfTcpRecv(tcpVectPtr);
                    if (errorCode != TM_ENOERROR)
                    {
                        if (recvdPacketLength != (ttPktLen)0)
                        {
/* Return partial data */
                            errorCode = TM_ENOERROR;
                        }
                        goto recvError;
                    }
                }
#endif /* TM_USE_TCP */
#ifdef TM_PEND_POST_NEEDED
                if (!(flags & TM_SOCF_NONBLOCKING))
                {
                    (void)tm_pend(&socketEntryPtr->socRecvPendEntry,
                                  &socketEntryPtr->socLockEntry);
                    packetPtr = socketEntryPtr->socReceiveQueueNextPtr;
                }
                else
#endif /* TM_PEND_POST_NEEDED */
                {
#ifdef TM_USE_TCP
                    if (recvdPacketLength == (ttPktLen)0)
#endif /* TM_USE_TCP */
                    {
                        errorCode = TM_EWOULDBLOCK;
                    }
                    goto recvFinish;
                }
            }
/* TCP socket */
#ifdef TM_USE_TCP
            if (protocolNumber == (tt8Bit)IP_PROTOTCP)
            {
                if (tm_16bit_one_bit_set(tcpVectPtr->tcpsFlags, TM_TCPF_PACKET)
                    && (packetPtr->pktLinkDataLength > bufferLeftSpace))
                {
/* bug ID 105: if TM_TCP_PACKET is set, check if the specified buffer size is
 * smaller than the received TCP segment, in which case we return the error
 * TM_EMSGSIZE. 
 */
                    errorCode = TM_EMSGSIZE;
                    goto recvFinish;
                }

                if (    (socketEntryPtr->socOobMark != 0)
                     && ((tt32Bit)bufferLeftSpace > socketEntryPtr->socOobMark) )
                {
/* stop reading at out of band mark, in case user cares about finding it */
                    bufferLeftSpace = (ttPktLen)(socketEntryPtr->socOobMark);
                }
                nowaitCopyCount = (ttPktLen)0;
                while (    (bufferLeftSpace != (ttPktLen)0)
                        && (packetPtr != TM_PACKET_NULL_PTR) )
                {
/* Copy the data */
                
                    if (packetPtr->pktLinkDataLength > bufferLeftSpace)
                    {
/* 
 * There is more data in this packet than we have room for
 * So just copy and don't free up.
 */
                        tm_bcopy((ttConstVoidPtr)(packetPtr->pktLinkDataPtr),
                                 (ttVoidPtr)bufferPtr,
                                 (unsigned)bufferLeftSpace);

/* If the user is not peeking */
                        if ((flags & MSG_PEEK) == 0)
                        {
/* Update packet data pointer and length */
                            packetPtr->pktLinkDataPtr += bufferLeftSpace;
                            packetPtr->pktLinkDataLength -= bufferLeftSpace;
                        }
                        nowaitCopyCount += bufferLeftSpace;
                        bufferLeftSpace = (ttPktLen)0;
                    }
                    else
                    {
/* Copy this data area to the user buffer */
                        nowaitCopyCount += packetPtr->pktLinkDataLength;
                        bufferLeftSpace = (ttPktLen)(  
                                               bufferLeftSpace 
                                             - packetPtr->pktLinkDataLength);
                        {
                            tm_bcopy(
                                  (ttConstVoidPtr)(packetPtr->pktLinkDataPtr),
                                  (ttVoidPtr)bufferPtr,
                                  (unsigned)(packetPtr->pktLinkDataLength));
/* Update the buffer Pointer */
                            bufferPtr += packetPtr->pktLinkDataLength;
                        }

                        nxtPacketPtr = (ttPacketPtr)packetPtr->pktLinkNextPtr;

/* Since we copied the whole data area, free it if the user is not peeking */
                        if ((flags & MSG_PEEK) == 0)
                        {
                            tfFreeSharedBuffer(packetPtr, TM_SOCKET_LOCKED);
                        }
/* 
 * For TCP, since there is no boundary, data is queued using
 * pktLinkNextPtr.
 */
                        packetPtr = nxtPacketPtr;
                    }
                }
/* If the user is not peeking */
                if ((flags & MSG_PEEK) == 0)
                {
/* Update receive queue */
                    socketEntryPtr->socReceiveQueueNextPtr = packetPtr;
/*
 * Let TCP know that window is larger by nowaitCopyCount. 
 * Update receive queue size. Update out of band mark. Try and send
 * a window update if permitted by TCP algorithms.
 */
                    tfTcpRecvCmplt(tcpVectPtr, nowaitCopyCount, 0);
                }
                recvdPacketLength += nowaitCopyCount;
            }
            else
#endif /* TM_USE_TCP */
            {
/* Datagram protocols */
/* Most common case. User has enough room in its buffer. */
                copyCount = packetPtr->pktChainDataLength;
                recvdPacketPtr = packetPtr;
                recvdPacketLength = copyCount;
/* If the user is not peeking */
                if (copyCount > (ttPktLen)bufferLength)
                {
/* Return truncated packet */
                    errorCode = TM_EMSGSIZE;
/* Copy only up to user buffer length */
                    copyCount = (ttPktLen)bufferLength;
                }
                do
                {
                     if (copyCount < packetPtr->pktLinkDataLength)
                     {
/* Copy only up to user buffer length */
                        linkCopyCount = copyCount;
                     }
                     else
                     {
                        linkCopyCount = packetPtr->pktLinkDataLength;
                     }
/* Copy this data area to the user buffer */
                    {
                        tm_bcopy((ttConstVoidPtr)(packetPtr->pktLinkDataPtr),
                                 (ttVoidPtr)bufferPtr,
                                 (unsigned)linkCopyCount);
/* Update the buffer Pointer */
                        bufferPtr += linkCopyCount;
                    }
/* 
 * For Datagram protocols, Keep the datagram boundary
 */
                    packetPtr = (ttPacketPtr)packetPtr->pktLinkNextPtr;
                    copyCount -= linkCopyCount;
                } while (    (packetPtr != TM_PACKET_NULL_PTR) 
                          && (copyCount != (ttPktLen)0) );
/* 
 * Since we copied the data area, it if the user is not peeking, unqueue
 * and free the packet
 */
                if ((flags & MSG_PEEK) == 0)
                {
/* Update receive queue */
                    socketEntryPtr->socReceiveQueueNextPtr =
                                           recvdPacketPtr->pktChainNextPtr;
/* Update receive queue size */
                    socketEntryPtr->socRecvQueueBytes -= 
                                                (tt32Bit)recvdPacketLength;
                    socketEntryPtr->socRecvQueueDgrams--;
/* Free copied packet */
                    tfFreePacket(recvdPacketPtr, TM_SOCKET_LOCKED);
                }
            }
        }
#ifdef TM_USE_TCP
#ifdef TM_PEND_POST_NEEDED
        while (    
               (flags & MSG_WAITALL) 
            && (protocolNumber == (tt8Bit)IP_PROTOTCP)
            && (bufferLeftSpace != (ttPktLen)0)
            && (tm_16bit_bits_not_set(tcpVectPtr->tcpsFlags, TM_TCPF_PACKET)));

#endif /* TM_PEND_POST_NEEDED */
#endif /* TM_USE_TCP */
recvError:
        if (errorCode == TM_ESHUTDOWN)
        {
/* If the connection is shut down, return no error */
            if ( tm_16bit_one_bit_set(socketEntryPtr->socFlags, TM_SOCF_REOF) )
            {
                errorCode = TM_ENOERROR;
            }
        }
recvFinish:
/*
 * If errorCode not set, will return recvdPacketLength, otherwise it will set
 * the errorCode in the user error array, and return TM_SOCKET_ERROR instead
 */
        retCode = tfSocketReturn( socketEntryPtr, socketDescriptor,
                                  errorCode, (int)recvdPacketLength );
    }
    return retCode;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trrecv.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 50538    Output checksum: 37698
 */
