/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfSetTreckOptions)
 *
 * Filename: trsetopt.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trsetopt.c $
 *
 * Modification History
 * $Revision: 6.0.2.4 $
 * $Date: 2011/11/09 23:47:46CST $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>


/*
 * Set Treck options (only after a call to tfStartTreck)
 * Set a global variable with a new value.
 */

int tfSetTreckOptions(int optionName, ttUser32Bit optionValue)
{
    int         errorCode;

    errorCode = TM_ENOERROR;
    switch (optionName)
    {
#ifdef TM_USE_TCP
        case TM_OPTION_TCP_SOCKETS_MAX:
            if (    (optionValue == 0)
                 || (optionValue >= (tt32Bit)TM_SOC_NO_INDEX)
                 || (optionValue > tm_context(tvMaxNumberSockets)) )
            {
                errorCode = TM_EINVAL;
            }
            else
            {
                tm_context(tvMaxTcpVectAllocCount) = optionValue;
            }
            break;
        case TM_OPTION_TCP_TMWT_MAX:
            tm_context(tvMaxTcpTmWtAllocCount) = optionValue;
            break;
        case TM_OPTION_SOTB_TCP_CON_PORT_THRS:
/*
 * TCP sockets:
 * Threshold of number of sockets in the socket lookup table before we switch
 * from a random local port selection to a sequential local port selection
 */
            tm_context(tvSocketThresholdSequentialPort[TM_SOTB_TCP_CON]) =
                                                     (unsigned int)optionValue;
            break;
#endif /* TM_USE_TCP */
        case TM_OPTION_SOTB_NON_CON_PORT_THRS: 
/*
 * Other sockets:
 * Threshold of number of sockets in the socket lookup table before we switch
 * from a random local port selection to a sequential local port selection
 */
            tm_context(tvSocketThresholdSequentialPort[TM_SOTB_NON_CON]) =
                                                     (unsigned int)optionValue;
            break;
        case TM_OPTION_RT_TIMER_RES_MSEC:
            if (optionValue < tvTimerTickLength)
            {
/* the timer resolution should not be less than the tick length */
                optionValue = tvTimerTickLength;
            }
            tm_context(tvRtTimerResolution) = optionValue;
            if (tm_context(tvRtTimerPtr) != TM_TMR_NULL_PTR)
            {
/* if the timer is already running, then restart it with the new timer period*/
                tm_timer_new_time(tm_context(tvRtTimerPtr), optionValue);
            }
            break;

        case TM_OPTION_TIMER_MAX_EXECUTE:
/* Maximum number of timers that can be executed in a single call to
   tfTimerExecute */
            if ( (int)optionValue >= 1 )
            {
                tm_context(tvTimerMaxExecute) = (int)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;



        case TM_OPTION_ICMP_ROUTE_REPLY: 
            tm_context(tvIcmpRouteReply) = (tt8Bit)optionValue;
            break;

#ifdef TM_USE_ECHO_CONFIG
        case TM_OPTION_ICMP_ECHO_ENABLE:
            tm_context(tvIcmpEchoEnable) = (tt8Bit)optionValue;
#endif /* TM_USE_ECHO_CONFIG */

#ifndef TM_SINGLE_INTERFACE_HOME
        case TM_OPTION_ICMP_ERR_DST_AS_SRC:
            tm_context(tvIcmpErrDstAsSrc) = (tt8Bit)optionValue;
            break;
#endif /* TM_SINGLE_INTERFACE_HOME */

        case TM_OPTION_ARP_MAX_ENTRIES:
/* Maximum number of ARP entries */
            if ( (int)optionValue >= 1 )
            {
                tm_context(tvArpMaxEntries) = (int)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_ARP_MAX_RETRY:
            if ( (int)optionValue >= 1 )
            {
                tm_context(tvArpRequestTries) = optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_ARP_TIMEOUT_TIME:
            if ( optionValue >= TM_UL(1) )
            {
                tm_context(tvArpRetryTime) = optionValue*TM_UL(1000);
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_ARP_QUIET_TIME:
            if ( optionValue >= TM_UL(1) )
            {
                tm_context(tvArpQuietTime) = optionValue*TM_UL(1000);
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_ARP_TTL:
/* Arp cache entry time to live */
            if (optionValue != TM_RTE_INF)
            {
                tm_context(tvArpTimeout) = optionValue*TM_UL(1000);
            }
            else
            {
                tm_context(tvArpTimeout) = TM_RTE_INF;
            }
            break;

        case TM_OPTION_ROUTE_MAX_ENTRIES:
/* Maximum number of dynamic routing table entries */
            if ( (int)optionValue >= 1 )
            {
                tm_context(tvRtMaxEntries) = (int)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_ROUTER_ID:
/* Router ID */
            tm_ip_copy((tt4IpAddress)optionValue, tm_context(tvRouterId));
            break;

        case TM_OPTION_ROUTER_AGE_LIMIT:
/* Routing entry age limit */
            tm_context(tvRtTimeout) = optionValue*TM_UL(1000);
            break;

#ifdef TM_PMTU_DISC
        case TM_OPTION_DECREASED_PMTU_TTL:
/* Path MTU host entry age limit after the path MTU estimate is decreased */
            tm_context(tvRtDecreasedMtuTimeout) = optionValue*TM_UL(1000);
            break;

        case TM_OPTION_LARGER_PMTU_TTL:
/* Path MTU host entry age limit when a larger MTU is attempted */
            tm_context(tvRtLargerMtuTimeout) = optionValue*TM_UL(1000);
            break;
#endif /* TM_PMTU_DISC */

        case TM_OPTION_SEND_TRAILER_SIZE:
            if ( optionValue <= 0xFFFFUL)
            {
                tm_context(tvPakPktSendTrailerSize) =
                    (ttPktLen) tm_packed_byte_count(optionValue);
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;


        case TM_OPTION_RIP_ENABLE:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvRipOn) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_RIP_SEND_MODE:
            if (    ((tt8Bit)optionValue == TM_RIP_NONE)
                 || ((tt8Bit)optionValue == TM_RIP_1)
                 || ((tt8Bit)optionValue == TM_RIP_2)
                 || ((tt8Bit)optionValue == TM_RIP_2_BROADCAST) )
            {
                tm_context(tvRipSendMode) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_RIP_RECV_MODE:
            if (    ((tt8Bit)optionValue == TM_RIP_NONE)
                 || ((tt8Bit)optionValue == TM_RIP_1)
                 || ((tt8Bit)optionValue == TM_RIP_2)
                 || ((tt8Bit)optionValue == (TM_RIP_1|TM_RIP_2))
                 || ((tt8Bit)optionValue == TM_RIP_2_BROADCAST) )
            {
                tm_context(tvRipRecvMode) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_IP_FORWARDING:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvIpForward) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_IP_DBCAST_FORWARD:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvIpDbcastForward) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

#ifdef TM_USE_LBCAST_CONFIG
        case TM_OPTION_IP_LBCAST_ENABLE:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvIpLbcastEnable) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;
#endif /* TM_USE_LBCAST_CONFIG */

        case TM_OPTION_IP_TTL:
            if (optionValue <= TM_UL(0xFF))
            {
                tm_context(tvIpDefTtl) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_IP_TOS:
            if (optionValue <= TM_UL(0xFF))
            {
                tm_context(tvIpDefTos) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

#ifdef TM_IP_FRAGMENT
        case TM_OPTION_IP_FRAGMENT:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvIpFragment) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;
#endif /* TM_IP_FRAGMENT */

#ifdef TM_IP_REASSEMBLY
        case TM_OPTION_IP_FRAG_TTL:
            if ( (optionValue <= 0x7F) && (optionValue >= 1) )
            {
                tm_context(tvFragTtl) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_IP_FRAG_MAX_Q_SIZE:
/*
 * Option to change the maximum number of IP datagrams waiting to be
 * reassembled. If any fragment of a new IP datagram arrives when the
 * maximum number of IP datagrams waiting to be reassembled has been reached,
 * it is dropped. Default value is 5.
 */
            if (optionValue <= TM_UL(0x7F))
            {
                tm_context(tvFragMaxQSize) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_IP_FRAG_FAILED_MAX:
/*
 * Option to change the maximum number of IP datagrams that we will track that
 * were too big to be reassembled. If we receive a new "too large" datagram
 * when we've already reached the maximum, we will free the oldest.
 * Default value is 5.
 */
            if (optionValue <= TM_UL(0x7F))
            {
                tm_context(tvFragFailedMaxQSize) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_IP_FRAG_ENTRY_MAX_SIZE:
/*
 * Option to change the maximum size of an IP datagram waiting to be
 * reassembled. Beyond that size, we drop the entire IP datagram. Default value
 * is 8200 bytes.
 */
            tm_context(tvFragEntryMaxSize) = optionValue;
            break;
#endif /* TM_IP_REASSEMBLY */

        case TM_OPTION_ICMP_ADDR_MASK_AGENT:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvIcmpAddrMaskAgent) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_UDP_CHECKSUM:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvUdpChecksumOn) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;

        case TM_OPTION_ARP_SMART:
            if (    (tt8Bit)optionValue == TM_8BIT_YES
                 || (tt8Bit)optionValue == TM_8BIT_ZERO )
            {
                tm_context(tvArpSmart) = (tt8Bit)optionValue;
            }
            else
            {
                errorCode = TM_EINVAL;
            }
            break;


#if (defined(TM_IGMP) && defined(TM_USE_IPV4))
#endif /* TM_IGMP and TM_USE_IPV4 */
            





#ifdef TM_6_USE_TEMPADDR
/*
 * [RFC4941] Privacy Extensions for Stateless Address Autoconfig in IPv6.
 * The following options control the autoconfiguration of temporary
 * global addresses for all interfaces.
 */
/*
 * use_tempaddr option values (similar to Linux sysctl):
 *  0: disable privacy extensions.
 *  1: enable privacy extensions but prefer public addresses over
 *     temporary addresses.
 *  2: enable privacy extensions and prefer temporary addresses over
 *     public addresses.
 */
        case TM_6_OPTION_USE_TEMPADDR:
            switch (optionValue)
            {
            case TM_UL(0):
                tm_context(tv6TempDevFlags) = TM_8BIT_ZERO;
                break;
            case TM_UL(1):
                tm_context(tv6TempDevFlags) = TM_6_DEV_USE_TEMP;
                break;
            case TM_UL(2):
                tm_context(tv6TempDevFlags) = (  TM_6_DEV_USE_TEMP
                                               | TM_6_DEV_PREFER_TEMP);
                break;
            default:
                errorCode = TM_EINVAL;
            }
            break;
/*
 * Valid lifetime (seconds) of a temporary address (default: 1 week)
 */
        case TM_6_OPTION_TEMP_VALID_LFT:
            if (optionValue <= TM_6_TEMP_REGEN_ADVANCE)
            {
                errorCode = TM_EINVAL;
            }
            else
            {
                tm_context(tv6TempValidLifetime) = optionValue;
            }
            break;
/*
 * Preferred lifetime (seconds) of a temporary address (default: 1 day)
 */
        case TM_6_OPTION_TEMP_PREFER_LFT:
            if (optionValue <= TM_6_TEMP_REGEN_ADVANCE)
            {
                errorCode = TM_EINVAL;
            }
            else
            {
                tm_context(tv6TempPreferLifetime) = optionValue;
            }
            break;
/*
 * Maximum desynchronizing factor (seconds), which is a random value
 * that helps prevent interfaces from generating temporary addresses
 * at the same time (default: 10 minutes).
 */
        case TM_6_OPTION_TEMP_MAX_DESYNC:
            if (optionValue == TM_32BIT_ZERO)
            {
                errorCode = TM_EINVAL;
            }
            else
            {
                tm_context(tv6TempMaxDesyncFactor) = optionValue;
            }
            break;
/*
 * Maximum attempts to generate a valid temporary address
 * (default: 3 retries).
 */
        case TM_6_OPTION_TEMP_MAX_RETRY:
            if ((unsigned int)optionValue == 0)
            {
                errorCode = TM_EINVAL;
            }
            else
            {
                tm_context(tv6TempRegenMaxRetry) = (unsigned int)optionValue;
            }
            break;
#endif /* TM_6_USE_TEMPADDR */

        default:
            errorCode = TM_EINVAL;
            break;

    }
    return(errorCode);
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trsetopt.c    File Revision: 6.0.2.4 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 11258    Output checksum:  1423
 */
