/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfRegisterSocketCB)
 *
 * Filename: trsockcb.c
 * Author: Paul & Odile
 * Date Created: 11/10/97
 * $Source: source/sockapi/trsockcb.c $
 *
 * Modification History
 * $Revision: 6.0.2.4 $
 * $Date: 2011/07/25 22:45:07CDT $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/* 
 * Used to register a function to call upon completion of any socket
 * event as set in flags.
 */
int tfRegisterSocketCB(int                      socketDescriptor,
                       ttUserSocketCBFuncPtr    socketCBFuncPtr,
                       int                      flags)
{
    int              errorCode;
    ttSocketEntryPtr socketEntryPtr;
    
/* PARAMETER CHECK */
    socketEntryPtr = tfSocketCheckValidLock(  socketDescriptor
                                           );
    if (socketEntryPtr != TM_SOCKET_ENTRY_NULL_PTR)
    {
        if (    (socketCBFuncPtr == (ttUserSocketCBFuncPtr)0)
             && (flags != 0) )
        {
            errorCode = TM_EINVAL;
        }
        else
        {
            errorCode = TM_ENOERROR;
            socketEntryPtr->socUserCBFuncPtr=
                                       (ttUserSocketCBFuncPtr)socketCBFuncPtr;
            socketEntryPtr->socCBFlags = (tt16Bit)flags;
            socketEntryPtr->socFlags &= ~TM_SOCF_CB_PARAM_ON;
            if (socketCBFuncPtr != (ttUserSocketCBFuncPtr)0)
            {
                tfSockCBNotify(socketEntryPtr);
            }
        }
    }
    else
    {
        {
            errorCode = TM_EBADF;
        }
    }
    return tfSocketReturn(socketEntryPtr, socketDescriptor,
                          errorCode, TM_ENOERROR);
}

void tfSockCBNotify (ttSocketEntryPtr socketEntryPtr)
{
    int selectFlags;
    int socketCBFlags;

    selectFlags = 0;
    socketCBFlags = 0;
    if (socketEntryPtr->socRecvQueueBytes != 0)
    {
        selectFlags = TM_SELECT_READ;
        socketCBFlags = TM_CB_RECV;
    }
    if (tm_16bit_one_bit_set(socketEntryPtr->socFlags, TM_SOCF_REOF))
/* received EOF mark */
    {
        selectFlags = TM_SELECT_READ;
        socketCBFlags |= TM_CB_REMOTE_CLOSE;
    }
    if (tm_16bit_one_bit_set(socketEntryPtr->socFlags2, TM_SOCF2_RESET))
/* received/sent RST */
    {
        selectFlags = (TM_SELECT_READ | TM_SELECT_WRITE);
        socketCBFlags |= (TM_CB_RESET | TM_CB_SOCKET_ERROR);
    }
    if (selectFlags != 0)
    {
        tfSocketNotify( socketEntryPtr,
                        selectFlags,
                        socketCBFlags,
                        0 );
    }
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trsockcb.c    File Revision: 6.0.2.4 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 55409    Output checksum: 59205
 */
