/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2009
 * Copyright Elmic Systems Japan 1997 - 2009
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Register user callback per socket for incoming TCP packets
 *
 * Filename: trsockci.c
 * Author: Lorne
 * Date Created: 6/30/09
 * $Source: source/sockapi/trsockci.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:44:31CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>

#ifdef TM_USE_TCP_INCOMING_CB

#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/* 
 * Install or remove a user function on a particular TCP streaming
 * socket that will be called once for every valid incoming packet
 * that is destined for that socket.
 * Specify a NULL value for socketCBPacketFuncPtr to remove a
 * function pointer that was installed previously.
 */
int tfRegisterSocketPacketCB(
    int                            socketDescriptor,
    ttUserSocketCBPacketFuncPtr    socketCBPacketFuncPtr,
    int                            flags )
{
    int              errorCode;
    ttSocketEntryPtr socketEntryPtr;
    
    errorCode = TM_ENOERROR;

/* PARAMETER CHECK */
    socketEntryPtr = tfSocketCheckValidLock(  socketDescriptor
                                           );
    if (socketEntryPtr != TM_SOCKET_ENTRY_NULL_PTR)
    {
        socketEntryPtr->socUserCBPacketFuncPtr = socketCBPacketFuncPtr;
        socketEntryPtr->socCBPacketFlags = (tt16Bit)flags;
    }
    else
    {
        {
            errorCode = TM_EBADF;
        }
    }
    return tfSocketReturn(socketEntryPtr, socketDescriptor,
                          errorCode, TM_ENOERROR);
}

void tfSockCBPacketNotify(ttSocketEntryPtr socketEntryPtr, ttPacketPtr packetPtr)
{
    ttUserSocketCBPacketFuncPtr funcPtr;

    funcPtr = socketEntryPtr->socUserCBPacketFuncPtr;
    if (funcPtr != (ttUserSocketCBPacketFuncPtr)0)
    {
        (*funcPtr)( (int)socketEntryPtr->socIndex,
                    &packetPtr->pktUserStruct, 0 );
    }
}

#else /* !TM_USE_TCP_INCOMING_CB */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

/* To allow link for builds when TM_USE_TCP_INCOMING_CB is not defined */
int tvSockciDummy = 0;

#endif /* TM_USE_TCP_INCOMING_CB */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trsockci.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 21817    Output checksum: 33345
 */
