/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: BSD Sockets Interface (tfRegisterSocketCBParam)
 *
 * Filename: trsockcp.c
 * Author: Odile
 * Date Created: 11/30/98
 * $Source: source/sockapi/trsockcp.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:44:32CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

/* 
 * Used to register a function to call upon completion of any socket
 * event as set in flags. socketCBParamFuncPtr is a user defined pointer,
 * and is passed as an extra parameter by the call back function.
 */
int tfRegisterSocketCBParam(int                         socketDescriptor,
                            ttUserSocketCBParamFuncPtr  socketCBParamFuncPtr,
                            void TM_FAR                 *socketUserPtr,
                            int                         flags)
{
    int              errorCode;
    ttSocketEntryPtr socketEntryPtr;
    
/* PARAMETER CHECK */
    socketEntryPtr = tfSocketCheckValidLock(  socketDescriptor
                                           );
    if (socketEntryPtr != TM_SOCKET_ENTRY_NULL_PTR)
    {
        if (    (socketCBParamFuncPtr == (ttUserSocketCBParamFuncPtr)0)
             && (flags != 0) )
        {
            errorCode = TM_EINVAL;
        }
        else
        {
            errorCode = TM_ENOERROR;
            socketEntryPtr->socUserCBParamFuncPtr =
                             (ttUserSocketCBParamFuncPtr)socketCBParamFuncPtr;
            socketEntryPtr->socApiEntryPtr = socketUserPtr;
            socketEntryPtr->socCBFlags = (tt16Bit)flags;
            socketEntryPtr->socFlags |= TM_SOCF_CB_PARAM_ON;
            if (socketCBParamFuncPtr != (ttUserSocketCBParamFuncPtr)0)
            {
                tfSockCBNotify(socketEntryPtr);
            }
        }
    }
    else
    {
        {
            errorCode = TM_EBADF;
        }
    }
    return tfSocketReturn(socketEntryPtr, socketDescriptor,
                          errorCode, TM_ENOERROR);
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trsockcp.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 10843    Output checksum: 60424
 */
