/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: tfSocketKickAll/tfSocketKickOne socket API
 *
 * Filename: trsocki.c
 * Author: Odile Lapidus
 * Date Created: 04/07/2011
 * $Source: source/sockapi/trsocki.c $
 *
 * Modification History
 * $Revision: 6.0.2.1 $
 * $Date: 2011/04/21 16:25:19CDT $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

/*
 * tfSocketKickOne Function Description
 * Wake-up all tasks pending on a given socket.
 * . If the user passes a zero errorCode, then:
 *   .. any pending send/recv API for that socket will return -1, and a
 *      subsequent tfGetSocketError() call for the same socket will return
 *      TM_EWOULDBLOCK.
 *   .. any pending select won't return.
 *
 * . If the user passes a non zero errorCode, then:
 *   .. any pending send/recv API for that socket will return -1, and a
 *      subsequent tfGetSocketError() call for the same socket will return the
 *      passed errorCode
 *   .. any pending select will return a hit; a subsequent send/recv API for
 *      the socket will return -1, and a subsequent tfGetSocketError() call
 *      for the same socket will return the passed errorCode
 * Parameters
 *  sd          Socket descriptor
 *  errorCode   Error code to use when notifiying the socket.
 * Returns:
 * TM_ENOERROR  The passed socket descriptor was valid.
 * TM_EBADF     The passed socket descriptor was invalid.
 */
int tfSocketKickOne (int sd, int errorCode)
{
    ttSocketEntryPtr    socketPtr;

    socketPtr = tfSocketCheckValidLock(  sd
                                       );

    if (socketPtr != TM_SOCKET_ENTRY_NULL_PTR )
    {
        tfSocketNotify(socketPtr,
                       TM_SELECT_READ|TM_SELECT_WRITE |TM_SELECT_EXCEPTION, 
                       TM_CB_SOCKET_ERROR,
                       errorCode);
        errorCode = TM_ENOERROR;
        (void)tfSocketReturn(socketPtr, sd, TM_ENOERROR, 0);
    }
    else
    {
        {
            errorCode = TM_EBADF;
        }
    }
    return errorCode;
}

/*
 * tfSocketKickAll Function Description
 * Wake-up all tasks pending on any socket.
 * . If the user passes a zero errorCode, then:
 *   .. any pending send/recv API will return -1, and a subsequent
 *      tfGetSocketError() call for the corresponding socket will return
 *      TM_EWOULDBLOCK.
 *   .. any pending select won't return.
 *
 * . If the user passes a non zero errorCode, then:
 *   .. any pending send/recv API will return -1, and a subsequent
 *      tfGetSocketError() call for the corresponding socket will return the
 *      passed errorCode
 *   .. any pending select will return a hit; a subsequent send/recv API for
 *      the socket(s) registering a hit will return -1, and a subsequent
 *      tfGetSocketError() call for the corresponding socket(s) will return the
 *      passed errorCode
 * Parameters
 *  errorCode   Error code to use when notifiying any valid socket.
 * No Return
 */
void tfSocketKickAll (int errorCode)
{
    unsigned int        sockIndex;

    tm_call_lock_wait(&tm_context(tvSocketArrayLock));
    if (tm_context(tvSocketArray) != 0)
    {
        for ( sockIndex = 0;
              (tt32Bit)sockIndex < tm_context(tvMaxNumberSockets);
              sockIndex++ )
        {
            tm_call_unlock(&tm_context(tvSocketArrayLock));
            (void)tfSocketKickOne(sockIndex, errorCode);
            tm_call_lock_wait(&tm_context(tvSocketArrayLock));
        }
    }
    tm_call_unlock(&tm_context(tvSocketArrayLock));
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:47      For: Xilinx-Samsung
 * File Name: trsocki.c     File Revision: 6.0.2.1 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  8780    Output checksum: 26242
 */
