/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2010
 * Copyright Elmic Systems Japan 1997 - 2010
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Scattered Interface Loopback Driver. Can be used with 
 *              any link layer to test scattered driver send, and recv.
 *
 * Filename: trinscdr.c
 * Author: Odile
 * Date Created: 06/13/2002
 * $Source: source/trinscdr.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2010/07/20 16:33:10CDT $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#if defined(TM_LOCK_NEEDED) || defined(TM_ERROR_CHECKING)
#include <trglobal.h>
#endif /* TM_LOCK_NEEDED || TM_ERROR_CHECKING */

#define TESTING_RECV_USER_BUFFERS
/*#define TESTING_OUT_OF_ORDER_SEND_COMPLETE */

#ifdef TESTING_OUT_OF_ORDER_SEND_COMPLETE
#define TM_TESTING_SAVE_INDEX 2
#define TM_TESTING_SAVE_FLUSH_COUNT (TM_TESTING_SAVE_INDEX + 1) 
ttPacketPtr tlSavSendQPacketPtr[TM_TESTING_SAVE_INDEX];
int         tlSavSendQPacketIndex;
int         tlSavFlushCount;
#endif /* TESTING_OUT_OF_ORDER_SEND_COMPLETE */

#ifdef TM_USE_DRV_ONE_SCAT_SEND
int tfIntfScatDrvSend ( ttUserInterface  interfaceHandle,
                        ttUserPacketPtr  uPacketPtr );
#endif /* TM_USE_DRV_ONE_SCAT_SEND */

#ifdef TM_USE_DRV_SCAT_RECV
int tfIntfScatDrvRecv ( ttUserInterface   interfaceHandle,
                        ttDruBlockPtrPtr  uDevBlockPtr,
                        ttIntPtr          uDevBlockCountPtr,
                        ttIntPtr          flagPtr 
#ifdef TM_DEV_RECV_OFFLOAD
                      , ttDevRecvOffloadPtr recvOffloadPtr
#endif /* TM_DEV_RECV_OFFLOAD */
                      );
#endif /* TM_USE_DRV_SCAT_RECV */

int tfIntfScatDrvOpen(ttUserInterface  interfaceHandle);

int tfIntfScatDrvClose(ttUserInterface  interfaceHandle);

int tfIntfScatDrvFreeFunc ( ttUserInterface interfaceHandle, 
                            ttCharPtr       bufferPtr );

ttUserInterface tfUseScatIntfDriver ( ttCharPtr         namePtr,
                                      ttUserLinkLayer   linkLayerHandle,
                                      int TM_FAR      * errorCodePtr )
{
    ttUserInterface interfaceHandle;
#if defined(TM_USE_DRV_SCAT_RECV) && defined(TM_USE_DRV_ONE_SCAT_SEND)
    int             errorCode;
#endif /* TM_USE_DRV_SCAT_RECV && TM_USE_DRV_ONE_SCAT_SEND */

    interfaceHandle = tfAddInterface( namePtr,
                                      linkLayerHandle,
                                      tfIntfScatDrvOpen,
                                      tfIntfScatDrvClose,
                                      tfIntfDrvSend,
                                      tfIntfDrvRecv,
#ifdef TESTING_RECV_USER_BUFFERS
                                      tfIntfScatDrvFreeFunc,
#else /* !TESTING_RECV_USER_BUFFERS */
                    *                (ttDevFreeRecvFuncPtr)0,
#endif /* TESTING_RECV_USER_BUFFERS */
                                      tfIntfDrvIoctl,
                                      tfIntfDrvGetPhyAddr,
                                      errorCodePtr);
    if (interfaceHandle != (ttUserInterface)0)
    {
#if defined(TM_USE_DRV_SCAT_RECV) && defined(TM_USE_DRV_ONE_SCAT_SEND)
/* Replace tfIntfDrvSend with tfIntfScatDrvSend */
        errorCode = tfUseInterfaceOneScatSend( interfaceHandle,
                                               tfIntfScatDrvSend );
        if (errorCode == TM_ENOERROR)
        {
/* Replace tfIntfDrvRecv with tfIntfScatDrvRecv */
            errorCode = tfUseInterfaceScatRecv( interfaceHandle,
                                                tfIntfScatDrvRecv );
        }
        if (errorCode != TM_ENOERROR)
        {
            (*errorCodePtr) = errorCode;
        }
#endif /* TM_USE_DRV_SCAT_RECV && TM_USE_DRV_ONE_SCAT_SEND */
    }
    return interfaceHandle;
}


#ifdef TM_USE_DRV_ONE_SCAT_SEND
int tfIntfScatDrvSend ( ttUserInterface  interfaceHandle,
                        ttUserPacketPtr  uPacketPtr )
{
    ttDeviceEntryPtr devEntryPtr;
    ttPacketPtr      packetPtr;
    int              errorCode;

/* 
 * dump the packet to the file system in libpcap format,
 * to be opened by tcpdump, windump, ethereal, or other libpcap file readers.
 */
#ifdef TM_USE_INF_DRV_SNIFF
/* 
 * PPP and SLIP sniffing is done in tfPppStuffPacket and tfSlipStuffPacket,
 * before framing data and escape data is applied. 
 */
    if ((   ((ttDeviceEntryPtr)interfaceHandle)->devLinkLayerProtocol
          & TM_LINK_PT2PT_MASK) == 0)
    {
        tfPcapWriteFrame(
            ((ttDeviceEntryPtr)interfaceHandle)->devLinkLayerProtocol,
            (char*)uPacketPtr->pktuLinkDataPtr,
            uPacketPtr->pktuLinkDataLength);
    }
#endif /* TM_USE_INF_DRV_SNIFF */

    errorCode = TM_ENOERROR;
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    packetPtr = (ttPacketPtr)uPacketPtr;

#ifndef TM_INDRV_INLINE_SEND_RECV
    packetPtr->pktRecvNextPtr = (ttPacketPtr)0;
/* Queue packet */
    if (devEntryPtr->devReceiveQueuePtr == TM_PACKET_NULL_PTR)
    {
/* Empty queue. */
        devEntryPtr->devReceiveQueuePtr = packetPtr;
        devEntryPtr->devRcvLastQueuePtr = packetPtr;
    }
    else
    {
/* Non Empty queue. Queue at the end of the queue */
        devEntryPtr->devRcvLastQueuePtr->pktRecvNextPtr = packetPtr;
        devEntryPtr->devRcvLastQueuePtr = packetPtr;
    }
#else /* TM_INDRV_INLINE_SEND_RECV */
/* Single packet queue */
    tm_assert(tfIntfScatDrvSend, 
              devEntryPtr->devReceiveQueuePtr == TM_PACKET_NULL_PTR);
    devEntryPtr->devReceiveQueuePtr = packetPtr;
#endif /* TM_INDRV_INLINE_SEND_RECV */
/*
 * We can call tfRecvScatInterface in-line, or notify the stack, in
 * which case tfRecvScatInteraface needs to be called by the user.
 */
#ifdef TM_INDRV_INLINE_SEND_RECV
/* Unlock the device send/receive */
    tm_unlock(&(devEntryPtr->devDriverLockEntry));
#ifdef TM_USE_DRV_SCAT_RECV
    tfRecvScatInterface(interfaceHandle);
#endif /* TM_USE_DRV_SCAT_RECV */
    tm_lock_wait(&(devEntryPtr->devDriverLockEntry));
/* End of in line tfRecvInterface */
#else /* !TM_INDRV_INLINE_SEND_RECV */
/* Notify the Treck stack that we've received a packet */
    tfNotifyInterfaceTask(interfaceHandle, 1, 0, 0UL, 0UL);
#endif /* TM_INDRV_INLINE_SEND_RECV */

    return errorCode;    

}
#endif /* TM_USE_DRV_ONE_SCAT_SEND */

#define TM_MAX_SCATTER_COUNT 20

#ifdef TM_USE_DRV_SCAT_RECV
int tfIntfScatDrvRecv ( ttUserInterface     interfaceHandle,
                        ttDruBlockPtrPtr    uDevBlockPtrPtr,
                        ttIntPtr            uDevBlockCountPtr,
                        ttIntPtr            flagPtr 
#ifdef TM_DEV_RECV_OFFLOAD
                      , ttDevRecvOffloadPtr recvOffloadPtr
#endif /* TM_DEV_RECV_OFFLOAD */
                      )
{
    ttDeviceEntryPtr devEntryPtr;
    ttDruBlockPtr    devBlockPtr;
#ifdef TM_USE_DRV_ONE_SCAT_SEND
    ttPacketPtr      sendPacketPtr;
#endif /* TM_USE_DRV_ONE_SCAT_SEND*/
    ttPacketPtr      packetPtr;
    ttPacketPtr      newPacketPtr;
    int              errorCode;
    int              i;
#ifdef TESTING_OUT_OF_ORDER_SEND_COMPLETE
    int              j;
    int              saved;
#endif /* TESTING_OUT_OF_ORDER_SEND_COMPLETE */
#ifdef TM_ERROR_CHECKING
    ttPktLen         chainDataLength;
    ttPktLen         totalLinkDataLength;
#ifdef TM_DEBUG_LOGGING
    ttPktLen         lastLinkDataLength;
    ttPktLen         firstLinkDataLength;
#endif /* TM_DEBUG_LOGGING */
#endif /* TM_ERROR_CHECKING */

#ifdef TM_DEV_RECV_OFFLOAD
    TM_UNREF_IN_ARG(recvOffloadPtr); /* compiler warning for unused parameter */
#endif /* TM_DEV_RECV_OFFLOAD */
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
/* Check at head of the queue */
    packetPtr = devEntryPtr->devReceiveQueuePtr;
#ifdef TM_USE_DRV_ONE_SCAT_SEND
    sendPacketPtr = packetPtr;
#endif /* TM_USE_DRV_ONE_SCAT_SEND */
    errorCode = TM_ENOBUFS;
    if (packetPtr != TM_PACKET_NULL_PTR) 
/* Packet in the queue, and ready to be received */
    {
/* unlink this packet from the head of the queue of received packets */
#ifdef TM_INDRV_INLINE_SEND_RECV
        devEntryPtr->devReceiveQueuePtr = TM_PACKET_NULL_PTR;
#else /* !TM_INDRV_INLINE_SEND_RECV */
        devEntryPtr->devReceiveQueuePtr = 
                devEntryPtr->devReceiveQueuePtr->pktRecvNextPtr;
        if (devEntryPtr->devReceiveQueuePtr == (ttPacketPtr)0)
        {
/* Empty queue */
            devEntryPtr->devRcvLastQueuePtr = (ttPacketPtr)0;
        }
        packetPtr->pktRecvNextPtr = (ttPacketPtr)0;
#endif /* !TM_INDRV_INLINE_SEND_RECV */
        devBlockPtr = (ttDruBlockPtr)(devEntryPtr->devReceiveDataPtr);
        if (devBlockPtr != (ttDruBlockPtr)0)
        {
            tm_free(((ttVoidPtr)devBlockPtr));
        }
        *uDevBlockCountPtr = packetPtr->pktLinkExtraCount + 1;
        devBlockPtr = 
            (ttDruBlockPtr)tm_malloc(sizeof(ttDruBlock) * (*uDevBlockCountPtr));
        *uDevBlockPtrPtr = devBlockPtr;
        errorCode = TM_ENOERROR;
        if (devBlockPtr != (ttDruBlockPtr)0)
        {
            i = 0;
#ifdef TESTING_RECV_USER_BUFFERS
/* Testing using recv user buffers */
            *flagPtr = TM_DEV_SCAT_RECV_USER_BUFFER;
#else /* !TESTING_RECV_USER_BUFFERS*/
/* Testing using recv stack buffers (tfGetDriverBuffer) */
            *flagPtr = TM_DEV_SCAT_RECV_STACK_BUFFER;
#endif /* TESTING_RECV_USER_BUFFERS */
#ifdef TM_ERROR_CHECKING
            chainDataLength = packetPtr->pktChainDataLength;
            totalLinkDataLength = (ttPktLen)0;
#ifdef TM_DEBUG_LOGGING
            lastLinkDataLength = (ttPktLen)0;
            firstLinkDataLength = packetPtr->pktLinkDataLength;
#endif /* TM_DEBUG_LOGGING */
#endif /* TM_ERROR_CHECKING */
            while (    (packetPtr != (ttPacketPtr)0) 
                    && (i != *uDevBlockCountPtr ) )
            {
                if (   packetPtr->pktSharedDataPtr->dataFlags 
                    & TM_BUF_TCP_HDR_BLOCK)
                {
/* Cannot share a TCP recycle header */
                    newPacketPtr = tfGetSharedBuffer(0, 
                                                 packetPtr->pktLinkDataLength,
                                                 TM_16BIT_ZERO);
                    if (newPacketPtr != (ttPacketPtr)0)
                    {
                        tm_bcopy(
                          packetPtr->pktLinkDataPtr, 
                          newPacketPtr->pktLinkDataPtr, 
                          tm_packed_byte_count(packetPtr->pktLinkDataLength));
                    }
                    else
                    {
                        errorCode = TM_ENOBUFS;
                        break;
                    }
                }
                else
                {
                    newPacketPtr = tfDuplicateSharedBuffer(packetPtr);
                    if (newPacketPtr != (ttPacketPtr)0)
                    {
                        newPacketPtr->pktFlags = TM_16BIT_ZERO;
                        newPacketPtr->pktFlags2 = TM_16BIT_ZERO;
/* Stack buffer */
                        newPacketPtr->pktUserFlags = TM_PKTF_USER_DEV_OWNS;
                    }
                    else
                    {
                        errorCode = TM_ENOBUFS;
                        break;
                    }
                }
                devBlockPtr[i].druStackBufferPtr = 
                                         (ttUserBufferPtr)newPacketPtr;
                devBlockPtr[i].druDataPtr = (ttCharPtr)
                                         newPacketPtr->pktLinkDataPtr;
                devBlockPtr[i].druDataLength = packetPtr->pktLinkDataLength;
#ifdef TM_ERROR_CHECKING
                totalLinkDataLength += packetPtr->pktLinkDataLength;
#ifdef TM_DEBUG_LOGGING
                lastLinkDataLength = packetPtr->pktLinkDataLength;
#endif /* TM_DEBUG_LOGGING */
#endif /* TM_ERROR_CHECKING */
                packetPtr = (ttPacketPtr)packetPtr->pktLinkNextPtr;
                i++;
            }
            if (errorCode == TM_ENOERROR)
            {
                tm_assert(tfIntfScatDrvRecv, (i == *uDevBlockCountPtr));
                tm_assert(tfIntfScatDrvRecv, packetPtr == TM_PACKET_NULL_PTR);
#ifdef TM_ERROR_CHECKING
#ifdef TM_DEBUG_LOGGING
                if (totalLinkDataLength != chainDataLength)
                {
                    tm_debug_log5("FirstData %ld lastData %ld, totalLinkDataLength %ld, chainDataLength %ld, linkCount %d\n", 
                     firstLinkDataLength, lastLinkDataLength, 
                     totalLinkDataLength, chainDataLength, i);
                }
#endif /* TM_DEBUG_LOGGING */
#endif /* TM_ERROR_CHECKING */
                tm_assert(tfIntfScatDrvRecv, 
                                   (totalLinkDataLength == chainDataLength));
            }
            else
            {
/* clean up */
                for (i = 0; i < *uDevBlockCountPtr; i++)
                {
                    packetPtr = (ttPacketPtr)devBlockPtr[i].druStackBufferPtr;
                    if (packetPtr != (ttPacketPtr)0)
                    {
                        tfFreeSharedBuffer(packetPtr, TM_SOCKET_UNLOCKED);
                    }
                    else
                    {
                        break; /* done */
                    }
                }
            }
        }
#ifdef TM_USE_DRV_ONE_SCAT_SEND
/* 
 * Testing using tfSendCompletePacketInterface(), 
 * instead of tfSendCompleteInterface() 
 */
#ifdef TESTING_OUT_OF_ORDER_SEND_COMPLETE
        if (tlSavFlushCount >= TM_TESTING_SAVE_FLUSH_COUNT) 
        {
tm_debug_log1("Flushing, flushCount %d!!\n", tlSavFlushCount);
            tfIntfScatDrvClose(interfaceHandle);
        }
        if (tlSavSendQPacketPtr[tlSavSendQPacketIndex] == (ttPacketPtr)0)
        {
            tlSavSendQPacketPtr[tlSavSendQPacketIndex] = sendPacketPtr;
tm_debug_log2("                 saving 0x%x at %d\n", sendPacketPtr, tlSavSendQPacketIndex);
            if ( tlSavSendQPacketIndex == (TM_TESTING_SAVE_INDEX - 1) )
            {
                tlSavSendQPacketIndex = 0;
            }
            else
            {
                tlSavSendQPacketIndex++;
            }
        }
        else
        {
            saved = 0;
            if (tlSavFlushCount != (TM_TESTING_SAVE_FLUSH_COUNT - 1))
            {
                if (tlSavSendQPacketIndex == TM_TESTING_SAVE_INDEX - 1)
                {
                    j = 0;
                }
                else
                {
                    j = tlSavSendQPacketIndex + 1;
                }
                tlSavSendQPacketIndex = j;
                for (i = j; i < TM_TESTING_SAVE_INDEX; i++)
                {
                    if (tlSavSendQPacketPtr[i] != (ttPacketPtr)0)
                    {
                         tfSendCompletePacketInterface( interfaceHandle,
                                   &(tlSavSendQPacketPtr[i]->pktUserStruct),
                                   TM_DEV_SEND_COMPLETE_DRIVER );
tm_debug_log3("sending 0x%x and saving 0x%x at %d\n", tlSavSendQPacketPtr[i],
                                                sendPacketPtr, i); 
                         tlSavSendQPacketPtr[i] = sendPacketPtr;
                         saved = 1;
                         break;
                    }
                }
                if (!saved)
                {
                    for (i = 0; i < j; i++)
                    {
                        if (tlSavSendQPacketPtr[i] != (ttPacketPtr)0)
                        {
                            tfSendCompletePacketInterface( interfaceHandle,
                                       &(tlSavSendQPacketPtr[i]->pktUserStruct),
                                       TM_DEV_SEND_COMPLETE_DRIVER );
tm_debug_log3("sending 0x%x and saving 0x%x at %d\n", tlSavSendQPacketPtr[i],
                                                sendPacketPtr, i); 
                            tlSavSendQPacketPtr[i] = sendPacketPtr;
                            saved = 1;
                            break;
                        }
                    }
                }
            }
            tlSavFlushCount++;
            if (!saved)
            {
tm_debug_log2("Freeing last sent packet 0x%x, flushCount %d\n", sendPacketPtr, tlSavFlushCount);
                tfSendCompletePacketInterface( interfaceHandle,
                                           &(sendPacketPtr->pktUserStruct),
                                           TM_DEV_SEND_COMPLETE_DRIVER );
            }
        }
#else /* !TESTING_OUT_OF_ORDER_SEND_COMPLETE */
        tfSendCompletePacketInterface( interfaceHandle,
                                       &(sendPacketPtr->pktUserStruct),
                                       TM_DEV_SEND_COMPLETE_DRIVER );
#endif /* !TESTING_OUT_OF_ORDER_SEND_COMPLETE */
#else /* !TM_USE_DRV_ONE_SCAT_SEND */
        tfSendCompleteInterface( interfaceHandle,
                                 TM_DEV_SEND_COMPLETE_DRIVER );
#endif /* !TM_USE_DRV_ONE_SCAT_SEND */
/* To be freed later */
        devEntryPtr->devReceiveDataPtr = (tt8BitPtr)((ttVoidPtr)devBlockPtr);
    }
    return errorCode;
}
#endif /* TM_USE_DRV_SCAT_RECV */


int tfIntfScatDrvFreeFunc ( ttUserInterface interfaceHandle, 
                            ttCharPtr       bufferPtr )
{
    TM_UNREF_IN_ARG(interfaceHandle);
    tfFreeSharedBuffer((ttPacketPtr)bufferPtr, TM_SOCKET_UNLOCKED);
    return TM_ENOERROR;
}

int tfIntfScatDrvOpen(ttUserInterface interfaceHandle)
{
#if defined(TESTING_OUT_OF_ORDER_SEND_COMPLETE) && defined(TM_USE_DRV_ONE_SCAT_SEND)
    int i;

    tlSavSendQPacketIndex = 0;
    tlSavFlushCount = 0;
    for (i = 0; i < TM_TESTING_SAVE_INDEX; i++)
    {
        tlSavSendQPacketPtr[i] = (ttPacketPtr)0;
    }
#endif /* TESTING_OUT_OF_ORDER_SEND_COMPLETE && TM_USE_DRV_ONE_SCAT_SEND */
    TM_UNREF_IN_ARG(interfaceHandle);
    return TM_ENOERROR;
}

int tfIntfScatDrvClose( ttUserInterface  interfaceHandle)
{
    ttDeviceEntryPtr    devPtr;
#if defined(TESTING_OUT_OF_ORDER_SEND_COMPLETE) && defined(TM_USE_DRV_ONE_SCAT_SEND)
    int i;

    for (i = 0; i < TM_TESTING_SAVE_INDEX; i++)
    {
        if (tlSavSendQPacketPtr[i] != (ttPacketPtr)0)
        {
            tfSendCompletePacketInterface( interfaceHandle,
                                   &(tlSavSendQPacketPtr[i]->pktUserStruct),
                                   TM_DEV_SEND_COMPLETE_DRIVER );
            tlSavSendQPacketPtr[i] = (ttPacketPtr)0;
        }
    }
    tlSavSendQPacketIndex = 0;
    tlSavFlushCount = 0;
#endif /* TESTING_OUT_OF_ORDER_SEND_COMPLETE && TM_USE_DRV_ONE_SCAT_SEND */
/*
 * Free memory allocated in tfIntfScatDrvRecv()
 */
    devPtr = (ttDeviceEntryPtr)interfaceHandle;
    if (devPtr->devReceiveDataPtr != TM_8BIT_NULL_PTR)
    {
        tm_free(devPtr->devReceiveDataPtr);
        devPtr->devReceiveDataPtr = TM_8BIT_NULL_PTR;
    }
    return TM_ENOERROR;
}

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:48      For: Xilinx-Samsung
 * File Name: trinscdr.c    File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 26278    Output checksum:  2596
 */
