/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Network id support for multiple context.
 *
 * Filename: trnetid.c
 * Author: Bryan Wang
 * Date Created: 02/19/2004
 * $Source: source/trnetid.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:48:05CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#ifdef TM_MULTIPLE_CONTEXT

typedef struct tsIntfDrvInfo
{
/* Device parameter array index */
    int             infoDeviceIndex;
} ttIntfDrvInfo;
typedef ttIntfDrvInfo TM_FAR * ttIntfDrvInfoPtr;
#define TM_INTFDRV_INFO_PTR_NULL (ttIntfDrvInfoPtr)0

/* Get a network ID that hasn't been used yet */
ttUser32Bit tfNetworkGetUnusedId(void)
{

    return ++tm_global(tvCurrNetworkId);
}


/* Associate a interface to an network, this is to simulate plugging 
 * a interface into a certain network
 */
int tfNetworkSetId(
    ttUserInterface interfaceHandle,
    ttUser32Bit     networkId)
{
    int                     errorCode;

    errorCode = TM_DEV_OKAY;

/* Set the network id of the device to the specified */
    tm_device_set_network_id(interfaceHandle, networkId);

/* If user has given a network ID bigger than we known of, 
 * we update our knowledge.
 */
    if (tm_global(tvCurrNetworkId) < networkId)
    {
        tm_global(tvCurrNetworkId) = networkId;
    }
    return errorCode;
}


#else /* TM_MULTIPLE_CONTEXT */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

/* To allow link for builds when TM_MULTIPLE_CONTEXT 6 is not defined */
int tvNetworkIdDummy = 0;

#endif /* TM_MULTIPLE_CONTEXT */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:49      For: Xilinx-Samsung
 * File Name: trnetid.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 33525    Output checksum: 45367
 */
