/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: SLIP Set Options function (tfSlipSetOptions())
 *
 * Filename: trslipop.c
 * Author: Odile
 * Date Created: 01/24/00
 * $Source: source/trslipop.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:48:30CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>


#include <trtype.h>
#include <trproto.h>
#if    defined(TM_LOCK_NEEDED) \
    || (    defined(TM_ERROR_CHECKING) \
         && defined(TM_USE_ANSI_LINE_FILE)  \
         &&  (   defined(TM_KERNEL_VISUAL_X86) \
              || defined(TM_KERNEL_WIN32_X86) \
              || defined(TM_MEMORY_LOGGING) ) )
#include <trglobal.h>
#endif /*    TM_LOCK_NEEDED 
          || (    TM_ERROR_CHECKING
               && TM_USE_ANSI_LINE_FILE 
               && (    TM_KERNEL_VISUAL_X86
                    || TM_KERNEL_WIN32_X86
                    || TM_MEMORY_LOGGING ) ) */

/*
 * Local macros
 */

#define TM_SLIP_VECT_NULL_PTR (ttSlipVectPtr)0


int tfSlipSetOptions ( ttUserInterface interfaceHandle,
                       int             optionName,
                       void   TM_FAR * optionValuePtr,
                       int             optionLength )
{
    ttDeviceEntryPtr  devEntryPtr;
    ttSlipVectPtr     slipVectPtr;
    tt8BitPtr         sendDataPtr;
    short             optionValue;
    int               errorCode;

    errorCode = TM_EINVAL;
    devEntryPtr = (ttDeviceEntryPtr)interfaceHandle;
    if ( tfValidInterface(devEntryPtr) == TM_ENOERROR )
    {
        tm_call_lock_wait(&(devEntryPtr->devLockEntry));
        if ( tm_ll_is_slip(devEntryPtr) )
        {
/*
 * Lock device driver lock. OK to keep the device lock, since we never lock
 * the device lock after locking the device driver lock
 */
            tm_call_lock_wait(&(devEntryPtr->devDriverLockEntry));
            switch (optionName)
            {
                case TM_SLIP_OPT_SEND_BUF_SIZE:
                    if (optionLength == (int)sizeof(short))
                    {
                        optionValue = *((short TM_FAR *)optionValuePtr);
                        if (optionValue >= 1)
                        {

                            errorCode = TM_ENOERROR;
                            slipVectPtr = (ttSlipVectPtr)
                                            (devEntryPtr->devStateVectorPtr);
                            if (slipVectPtr == TM_SLIP_VECT_NULL_PTR)
                            {
                                errorCode = tfSlipVectInit(devEntryPtr);
                                slipVectPtr = (ttSlipVectPtr)
                                            (devEntryPtr->devStateVectorPtr);
                            }
                            else
                            {
                                if (    slipVectPtr->slipsSendDataPtr
                                     != &(slipVectPtr->slipsSendBuf[0]) )
                                {
                                    tm_free_raw_buffer(
                                             slipVectPtr->slipsSendDataPtr);
                                    slipVectPtr->slipsSendDataPtr =
                                             &(slipVectPtr->slipsSendBuf[0]);
                                    slipVectPtr->slipsSendBufSize =
                                                       TM_SLIP_DEF_SEND_SIZE;
                                }
                            }
                            if (slipVectPtr != TM_SLIP_VECT_NULL_PTR)
                            {
                                if (optionValue > TM_SLIP_DEF_BUF_SIZE)
                                {
                                    sendDataPtr =
                                            (tt8BitPtr)tm_get_raw_buffer( \
                                            (ttPktLen)(tt16Bit)optionValue);
                                    if ( sendDataPtr != (tt8BitPtr)0 )
                                    {
                                        slipVectPtr->slipsSendDataPtr =
                                                                sendDataPtr;
                                        slipVectPtr->slipsSendBufSize =
                                                                 optionValue;
                                    }
                                    else
                                    {
                                        errorCode = TM_ENOBUFS;
                                    }
                                }
                                else
                                {
                                    slipVectPtr->slipsSendBufSize =
                                                                 optionValue;
                                }
                            }
                        }
                    }
                    break;
                default:
                    errorCode = TM_EPROTONOSUPPORT;

            }
/* UnLock device driver lock. */
            tm_call_unlock(&(devEntryPtr->devDriverLockEntry));
        }
        else
        {
            errorCode = TM_EPERM;
        }
/* UnLock device lock. */
        tm_call_unlock(&(devEntryPtr->devLockEntry));
    }
    return errorCode;
}


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:52      For: Xilinx-Samsung
 * File Name: trslipop.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 64131    Output checksum: 39710
 */
