/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: testing the ARP manipulation APIs
 * Filename: tuarp.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002
 * $Source: test/unittest/tuarp.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:20CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"

#ifdef TM_MULTIPLE_CONTEXT


void tfxArp(ttUserContext context)
{
    ttUserContext               oldContext;
    ttDeviceEntryPtr            devPtr;
    tt8BitPtr                   physAddrIn;
    struct  sockaddr_storage    peerSockAddr;
    struct  sockaddr_storage    retSockAddr;
    int                         physAddrLen;
    int                         errorCode;
    tt8Bit                      physAddrArray[10];  
    tt8Bit                      physAddrInArray[10];

    tm_strcpy(physAddrInArray, "mypeer");
    physAddrIn = (tt8BitPtr)physAddrInArray;

    oldContext = tfGetCurrentContext();
#ifdef TM_MULTIPLE_CONTEXT
    devPtr = (ttDeviceEntryPtr)tfxGetInterfaceByContext(context);
    tfSetCurrentContext(context);
#else /* !TM_MULTIPLE_CONTEXT */
    devPtr = (ttDeviceEntryPtr)0;
#endif /* !TM_MULTIPLE_CONTEXT */
    tm_bzero(&peerSockAddr, sizeof(struct sockaddr_storage));
    tm_bzero(physAddrArray, sizeof(physAddrArray));

/* in either context, let's test adding and deleting arp entries. */




/* add a ARP entry */
    peerSockAddr.ss_len = sizeof(struct sockaddr_storage);
#ifdef TM_4_USE_SCOPE_ID
    errorCode = tf6AddressToIpv4Mapped
        (inet_addr("10.10.10.10"), &peerSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOERROR);
    errorCode = tf6SockaddrSetScopeId
        (tfxGetInterfaceByContext(tfGetCurrentContext()), &peerSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOERROR);
#else /* ! TM_4_USE_SCOPE_ID */
    peerSockAddr.ss_family = AF_INET;
    errorCode = inet_pton(AF_INET,
                          "10.10.10.10",
                          &(peerSockAddr.addr.ipv4.sin_addr));
    tm_assert(tfxArp, errorCode == 1);
#endif /* ! TM_4_USE_SCOPE_ID */
    errorCode = tfNgAddArpEntry(
#ifdef TM_USE_STRONG_ESL
                    devPtr,
#endif /* TM_USE_STRONG_ESL */
                    &peerSockAddr, physAddrIn, (int)tm_strlen(physAddrIn));
    tm_assert(tfxArp, errorCode == TM_ENOERROR);

/* get it back by IP */
    errorCode = tfNgGetArpEntryByIpAddr(
#ifdef TM_USE_STRONG_ESL
                            devPtr,
#endif /* TM_USE_STRONG_ESL */
                            &peerSockAddr,
                            physAddrArray,
                            sizeof(physAddrArray),
                            &physAddrLen);
    tm_assert(tfxArp, errorCode == TM_ENOERROR);
    tm_assert(tfxArp, !tm_memcmp(physAddrArray, physAddrIn, physAddrLen));

/* get it back by physical */
    errorCode = tfNgGetArpEntryByPhysAddr(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        AF_INET,
                        physAddrIn, 
                        (int)tm_strlen(physAddrIn),
                        &retSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOERROR);
#ifdef TM_4_USE_SCOPE_ID
    tm_assert(tfxArp, retSockAddr.ss_family  == AF_INET);
    tm_assert(tfxArp,    peerSockAddr.addr.ipv6.sin6_addr.s6LAddr[3]
                      == retSockAddr.addr.ipv4.sin_addr.s_addr);
#else /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxArp, retSockAddr.ss_family  == peerSockAddr.ss_family );
    tm_assert(tfxArp, retSockAddr.ss_len     == peerSockAddr.ss_len );
    tm_assert(tfxArp,    peerSockAddr.addr.ipv4.sin_addr.s_addr
                      == retSockAddr.addr.ipv4.sin_addr.s_addr);
#endif /* ! TM_4_USE_SCOPE_ID */
        
    tm_assert(tfxArp, errorCode == TM_ENOERROR);
    tm_assert(tfxArp, errorCode == TM_ENOERROR);

/* delete it by IP*/
    errorCode = tfNgDelArpEntryByIpAddr(
#ifdef TM_USE_STRONG_ESL
                    devPtr,
#endif /* TM_USE_STRONG_ESL */
                    &peerSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOERROR);

/* delete it by IP again, should fail */
    errorCode = tfNgDelArpEntryByIpAddr(
#ifdef TM_USE_STRONG_ESL
                    devPtr,
#endif /* TM_USE_STRONG_ESL */
                    &peerSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* delete it by physical again, should fail */
    errorCode = tfNgDelArpEntryByPhysAddr(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        AF_INET,
                        physAddrIn, 
                        (int)tm_strlen(physAddrIn));
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* get it back by IP, should fail */
    errorCode = tfNgGetArpEntryByIpAddr(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        &peerSockAddr,
                        physAddrArray,
                        sizeof(physAddrArray),
                        &physAddrLen);
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* get it back by physical, should fail */
    errorCode = tfNgGetArpEntryByPhysAddr(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        AF_INET,
                        physAddrIn, 
                        (int)tm_strlen(physAddrIn),
                        &retSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* add it back */
    errorCode = tfNgAddArpEntry(
#ifdef TM_USE_STRONG_ESL
                    devPtr,
#endif /* TM_USE_STRONG_ESL */
                    &peerSockAddr, physAddrIn, (int)tm_strlen(physAddrIn));
    tm_assert(tfxArp, errorCode == TM_ENOERROR);

/* delete it by physical */
    errorCode = tfNgDelArpEntryByPhysAddr(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        AF_INET,
                        physAddrIn, 
                        (int)tm_strlen(physAddrIn));
    tm_assert(tfxArp, errorCode == TM_ENOERROR);

/* delete it by IP again, should fail */
    errorCode = tfNgDelArpEntryByIpAddr(
#ifdef TM_USE_STRONG_ESL
                    devPtr,
#endif /* TM_USE_STRONG_ESL */
                    &peerSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* delete it by physical again, should fail */
    errorCode = tfNgDelArpEntryByPhysAddr(
#ifdef TM_USE_STRONG_ESL
                       devPtr,
#endif /* TM_USE_STRONG_ESL */
                        AF_INET,
                        physAddrIn, 
                        (int)tm_strlen(physAddrIn));
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* get it back by IP again, should fail */
    errorCode = tfNgGetArpEntryByIpAddr(
#ifdef TM_USE_STRONG_ESL
                           devPtr,
#endif /* TM_USE_STRONG_ESL */
                            &peerSockAddr,
                            physAddrArray,
                            sizeof(physAddrArray),
                            &physAddrLen);
    tm_assert(tfxArp, errorCode == TM_ENOENT);

/* get it back by physical again, should fail */
    errorCode = tfNgGetArpEntryByPhysAddr(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        AF_INET,
                        physAddrIn, 
                        (int)tm_strlen(physAddrIn),
                        &peerSockAddr);
    tm_assert(tfxArp, errorCode == TM_ENOENT);

}

#else /* TM_MULTIPLE_CONTEXT */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltfxArpDummy = 0;

#endif /* TM_MULTIPLE_CONTEXT */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuarp.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:    17    Output checksum: 58404
 */
