/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Common funcitons used by the testing code
 * Filename: tucommon.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002
 * $Source: test/unittest/tucommon.c $
 *
 * Modification History
 * $Revision: 6.0.2.8 $
 * $Date: 2011/11/08 23:57:03CST $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#ifdef TM_MULTIPLE_CONTEXT
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"
#ifdef TM_USE_RAM_FS
#include <trramfs.h>
#endif /* TM_USE_RAM_FS */

#ifndef TM_DISABLE_ALL_FILE_LOGGING
#define TM_SHEAP_LOG_FILE   "sheaplog.txt"
#endif /* !TM_DISABLE_ALL_FILE_LOGGING */

/* global varibales for testing */

char* tvRaPrefixStr = "3FFE:AAAA:BBBB:CCCC::";

ttUserInterface tvInterfaceHandle[TM_NUMBER_OF_INTERFACE];
ttUserInterface tvInterfaceHandleServer;
ttUserInterface tvInterfaceHandleClient;

#ifndef TM_SINGLE_INTERFACE_HOME
ttUserInterface tvOffLinkInterfaceServer;
ttUserInterface tvOffLinkInterfaceClient;
ttUserInterface tvInterfaceHandleExtraClient;
#endif /* ! TM_SINGLE_INTERFACE_HOME */

ttUserContext   tvContextHandleServer;
ttUserContext   tvContextHandleClient;

int tvTestRound = 0;
int tvLastRoundUsedBytes = 0;
int tvCurrUsedBytes;
int tvUsedBytesIncrement = 0;


/* both server and client have two set of V4 and V6 address */
char    tv4ServerIpAddrStr[TM_MAX_IPS_PER_IF][INET_ADDRSTRLEN];
char    tv4ClientIpAddrStr[TM_MAX_IPS_PER_IF][INET_ADDRSTRLEN];
struct  sockaddr_storage tv4ServerSockAddr[TM_MAX_IPS_PER_IF];
struct  sockaddr_storage tv4ClientSockAddr[TM_MAX_IPS_PER_IF];
#ifdef TM_4_USE_SCOPE_ID
struct  sockaddr_storage tv4MappedServerSockAddr[TM_MAX_IPS_PER_IF];
struct  sockaddr_storage tv4MappedClientSockAddr[TM_MAX_IPS_PER_IF];
#endif /* TM_4_USE_SCOPE_ID */
ttUser16Bit  tv4ServerPort[TM_MAX_IPS_PER_IF];



#ifdef TM_USE_DYNAMIC_MEMORY
static tt32Bit tfxGetMemRecycledBytes(void);
#endif /* TM_USE_DYNAMIC_MEMORY */

#ifndef TM_SINGLE_INTERFACE_HOME
static int txDriverGetPhysicalAddress(
    ttUserInterface interfaceHandle,
    char TM_FAR *   physicalAddress)
{

    TM_UNREF_IN_ARG(interfaceHandle);

    physicalAddress[0] = 0x00;
    physicalAddress[1] = 0x11;
    physicalAddress[2] = 0x22;
    physicalAddress[3] = 0x33;
    physicalAddress[4] = 0x44;
    physicalAddress[5] = 0x55;

    return TM_ENOERROR;
}

static int txDriverSend ( ttUserInterface interfaceHandle,
                    char TM_FAR *   dataPtr,
                    int             dataLength,
                    int             flag )
{
    TM_UNREF_IN_ARG(interfaceHandle);
    flag            = flag;
    dataPtr         = dataPtr;
    dataLength      = dataLength;

    return TM_DEV_OKAY;
}

static int txDriverReceive(ttUserInterface        interfaceHandle,
                           char TM_FAR * TM_FAR * dataPtrPtr,
                           int  TM_FAR *          dataLengthPtr,
                           ttUserBufferPtr        bufHandlePtr
#ifdef TM_DEV_RECV_OFFLOAD
                         , ttDevRecvOffloadPtr recvOffloadPtr
#endif /* TM_DEV_RECV_OFFLOAD */
                           )
{
    TM_UNREF_IN_ARG(interfaceHandle);
    dataPtrPtr      = dataPtrPtr;
    dataLengthPtr   = dataLengthPtr;
    bufHandlePtr    = bufHandlePtr;
#ifdef TM_DEV_RECV_OFFLOAD
    TM_UNREF_IN_ARG(recvOffloadPtr); /* compiler warning for unused parameter */
#endif /* TM_DEV_RECV_OFFLOAD */
    return TM_DEV_ERROR;
}
#endif /* ! TM_SINGLE_INTERFACE_HOME */

void tfxAddOnlinkInterfaces()
{
    int errorCode;
    ttUserLinkLayer linkLayerHandleServer;
    ttUserLinkLayer linkLayerHandleClient;
#ifdef TM_USE_VCHAN
    int mhomeIndex;
#endif /* TM_USE_VCHAN */

/* server context  */
    tfSetCurrentContext(tvContextHandleServer);

#ifdef TM_TEST_NULL_LINK_LAYER
    linkLayerHandleServer = tfUseNullLinkLayer();
#else /* ! TM_TEST_NULL_LINK_LAYER */
    linkLayerHandleServer = tfUseEthernet();
#endif /* ! TM_TEST_NULL_LINK_LAYER */

    tvInterfaceHandleServer = tfUseIntfMCtxDriver(
                                    "Server", 
                                     linkLayerHandleServer, 
                                     &errorCode);
    tm_assert(tfxAddOnlinkInterfaces, errorCode == TM_ENOERROR);



#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
    for (mhomeIndex = 0;
         mhomeIndex < TM_MAX_IPS_PER_IF;
         mhomeIndex++)
    {
        errorCode = tfNgInterfaceSetVirtualChannel(
            tvInterfaceHandleServer,
            (ttUser32Bit) mhomeIndex,
            AF_INET,
            mhomeIndex );
        tm_assert(tfxAddOnlinkInterfaces, errorCode == TM_ENOERROR);
    }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */

/* client context */
    tfSetCurrentContext(tvContextHandleClient);

#ifdef TM_TEST_NULL_LINK_LAYER
    linkLayerHandleClient = tfUseNullLinkLayer();
#else /* ! TM_TEST_NULL_LINK_LAYER */
    linkLayerHandleClient = tfUseEthernet();
#endif /* ! TM_TEST_NULL_LINK_LAYER */


    tvInterfaceHandleClient = tfUseIntfMCtxDriver(
                                    "Client",
                                     linkLayerHandleClient, 
                                     &errorCode);


    tm_assert(tfxAddOnlinkInterfaces, errorCode == TM_ENOERROR);

#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
    for (mhomeIndex = 0;
         mhomeIndex < TM_MAX_IPS_PER_IF;
         mhomeIndex++)
    {
        errorCode = tfNgInterfaceSetVirtualChannel(
            tvInterfaceHandleClient,
            (ttUser32Bit) mhomeIndex,
            AF_INET,
            mhomeIndex );
        tm_assert(tfxAddOnlinkInterfaces, errorCode == TM_ENOERROR);
    }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
}

void tfxAddOfflinkInterfaces(void)
{
#ifndef TM_SINGLE_INTERFACE_HOME
    int             errorCode;
#endif /* ! TM_SINGLE_INTERFACE_HOME */
    ttUserLinkLayer linkLayerHandleServer;
    ttUserLinkLayer linkLayerHandleClient;
#ifdef TM_USE_VCHAN
    int mhomeIndex;
#endif /* TM_USE_VCHAN */

/* server context  */
    tfSetCurrentContext(tvContextHandleServer);

#ifdef TM_TEST_NULL_LINK_LAYER
    linkLayerHandleServer = tfUseNullLinkLayer();
#else /* ! TM_TEST_NULL_LINK_LAYER */
    linkLayerHandleServer = tfUseEthernet();
#endif /* ! TM_TEST_NULL_LINK_LAYER */

#ifndef TM_SINGLE_INTERFACE_HOME
/* the offlink interface must be added first, otherwise the server will
   choose this interface when trying to send off-link, which is not what
   we want */
    tvOffLinkInterfaceServer = tfAddInterface(
        "offlinkS",
        linkLayerHandleServer,
        0,
        0,
        txDriverSend,
        txDriverReceive,
        (ttDevFreeRecvFuncPtr)0,
        0,
        txDriverGetPhysicalAddress,
        &errorCode);
    tm_assert(tfxAddInterfaces, errorCode == TM_ENOERROR);
#endif /* ! TM_SINGLE_INTERFACE_HOME */


#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
    for (mhomeIndex = 0;
         mhomeIndex < TM_MAX_IPS_PER_IF;
         mhomeIndex++)
    {
        errorCode = tfNgInterfaceSetVirtualChannel(
            tvOffLinkInterfaceServer,
            (ttUser32Bit) mhomeIndex,
            AF_INET,
            mhomeIndex );
        tm_assert(tfxAddInterfaces, errorCode == TM_ENOERROR);
    }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */



/* client context */
    tfSetCurrentContext(tvContextHandleClient);

#ifdef TM_TEST_NULL_LINK_LAYER
    linkLayerHandleClient = tfUseNullLinkLayer();
#else /* ! TM_TEST_NULL_LINK_LAYER */
    linkLayerHandleClient = tfUseEthernet();
#endif /* ! TM_TEST_NULL_LINK_LAYER */


#ifndef TM_SINGLE_INTERFACE_HOME
/* we only create the off-link interface for the client so that the
   interface indexes used in the socket address structures (i.e. sin6_scope_id)
   match between client and server, otherwise tests will fail. */
    tvOffLinkInterfaceClient = tfAddInterface(
        "offlinkC",
        linkLayerHandleClient,
        0,
        0,
        txDriverSend,
        txDriverReceive,
        (ttDevFreeRecvFuncPtr)0,
        0,
        txDriverGetPhysicalAddress,
        &errorCode);
    tm_assert(tfxAddInterfaces, errorCode == TM_ENOERROR);
#endif /* ! TM_SINGLE_INTERFACE_HOME */


#if defined(TM_USE_VCHAN)
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
    for (mhomeIndex = 0;
         mhomeIndex < TM_MAX_IPS_PER_IF;
         mhomeIndex++)
    {
        errorCode = tfNgInterfaceSetVirtualChannel(
            tvOffLinkInterfaceClient,
            (ttUser32Bit) mhomeIndex,
            AF_INET,
            mhomeIndex );
        tm_assert(tfxAddInterfaces, errorCode == TM_ENOERROR);
    }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
}


void tfxAddExtraClientInterfaces(void)
{
#ifndef TM_SINGLE_INTERFACE_HOME
    int errorCode;
#endif /* ! TM_SINGLE_INTERFACE_HOME */
    ttUserLinkLayer linkLayerHandleClient;
#ifdef TM_USE_VCHAN
    int mhomeIndex;
#endif /* TM_USE_VCHAN */

/* server context  */
    tfSetCurrentContext(tvContextHandleClient);
#ifdef TM_TEST_NULL_LINK_LAYER
    linkLayerHandleClient = tfUseNullLinkLayer();
#else /* ! TM_TEST_NULL_LINK_LAYER */
    linkLayerHandleClient = tfUseEthernet();
#endif /* ! TM_TEST_NULL_LINK_LAYER */

#ifndef TM_SINGLE_INTERFACE_HOME
/* this is tricky - we create an extra client interface last, so that if the
   client tries to send to a non-on-link destination without a router, it will
   pick this interface instead so the packets will be discarded. */
    tvInterfaceHandleExtraClient = tfAddInterface(
        "extraC",
        linkLayerHandleClient,
        0,
        0,
        txDriverSend,
        txDriverReceive,
        (ttDevFreeRecvFuncPtr)0,
        0,
        txDriverGetPhysicalAddress,
        &errorCode);
    tm_assert(tfxAddOnlinkInterfaces, errorCode == TM_ENOERROR);
#endif /* ! TM_SINGLE_INTERFACE_HOME */


#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
    for (mhomeIndex = 0;
         mhomeIndex < TM_MAX_IPS_PER_IF;
         mhomeIndex++)
    {
        errorCode = tfNgInterfaceSetVirtualChannel(
            tvInterfaceHandleExtraClient,
            (ttUser32Bit) mhomeIndex,
            AF_INET,
            mhomeIndex );
        tm_assert(tfxAddOnlinkInterfaces, errorCode == TM_ENOERROR);
    }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
}

void tfxCreateStartClientContext(void)
{
    int errorCode;
/* client context */
    tvContextHandleClient = tfCreateTreckContext();
    tm_assert(tfxCreateStartContexts, tvContextHandleClient != (void TM_FAR *)0)

    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfStartTreck();
#ifdef TM_USE_LOGGING_LEVELS
    tfLogSetLevel(TM_LOG_MODULE_ALL, TM_LOG_LEVEL_MAX);
    tfLogSetVerbosity(TM_LOG_VERBOSE);
#endif /* TM_USE_LOGGING_LEVELS */
    tm_assert(tfxCreateStartClientContext, errorCode == TM_ENOERROR);
}

void tfxCreateStartServerContext(void)
{
    int errorCode;
/* server context  */
    tvContextHandleServer = tfCreateTreckContext();
    tm_assert(tfxCreateStartContexts, tvContextHandleServer != (void TM_FAR *)0)
    
    tfSetCurrentContext(tvContextHandleServer);
#ifdef TM_USE_TIMER_CACHE
#ifdef TM_USE_TIMER_WHEEL
/* Test a bigger wheel on the server */
    errorCode = tfInitTreckOptions(TM_OPTION_TIMER_MAX_WHEEL_INTV, 4000); 
    tm_assert(tfxCreateStartContexts, errorCode == TM_ENOERROR);
#endif /* TM_USE_TIMER_WHEEL */
#endif /* TM_USE_TIMER_CACHE */
    errorCode = tfStartTreck();
#ifdef TM_USE_LOGGING_LEVELS
    tfLogSetLevel(TM_LOG_MODULE_ALL, TM_LOG_LEVEL_MAX);
    tfLogSetVerbosity(TM_LOG_VERBOSE);
#endif /* TM_USE_LOGGING_LEVELS */
    tm_assert(tfxCreateStartServerContext, errorCode == TM_ENOERROR);
}
/* 
 * set up testing environment for two contexts 
 * Clear all the contexts. Reinitialize the client and server contexts. Start
 * the Treck stack on each context.
 */
void tfxCreateStartContexts()
{
    if (tvContextHandleServer == (ttUserContext)0)
    {
        tfxCreateStartServerContext();
    }
    if (tvContextHandleClient == (ttUserContext)0)
    {
        tfxCreateStartClientContext();
    }
#ifdef TM_USE_RAM_FS
    tfRamfsInit(256);
#endif /* TM_USE_RAM_FS */
}


#if (defined(TM_USE_FILTERING) || defined(TM_6_USE_FILTERING))
/*
 * NOTES:
 *
 * If TM_USE_FILTERING_CONTIGUOUS_DATA is defined, we guarantee the
 * entire IPv4 packet to be contiguous in the filter callback.
 *
 * If TM_6_USE_FILTERING_CONT_DATA is defined, we guarantee the
 * entire IPv6 packet to be contiguous in the filter callback.
 * 
 * Without the CONTIGUOUS_DATA compile-time macros, we do not guarantee
 * the ULP to be contiguous for tunneled packets. Here are the details:
 *
 * IPv4:
 * The first eight (8) bytes of the ULP is guaranteed to be contiguous.
 *
 * IPv6:
 * We do not guarantee the ULP to be contiguous.
 *
 * Therefore, without the CONTIGUOUS_DATA compile-time macros, we will
 * filter the packets based on their ULP ports only if we have a guarantee
 * of contiguous data, otherwise we will filter at the IP level.
 *
 * Also, we now only have one callback function that will ever be registered
 * which will handle incoming and outgoing data for both IPv4 and IPv6.
 *
 * Here are the tests that will be run:
 * IPv4 and IPv6 + TCP and (connected) UDP
 * tfxExcerciseChannelSize sends packets from the client to the server
 * (1) we drop outgoing packets on the client side
 * (2) we drop incoming packets on the server side
 */
int tfxFilterCallback(ttUserInterface interfaceHandle,
                      void TM_FAR *   iphPtr,
                      void TM_FAR *   ulpPtr,
                      int             totalLength,
                      int             direction)
{
    ttIpHeaderPtr   ip4HeaderPtr;
    int             errorCode;
    unsigned int    dstPort;
    tt8Bit          ipHdrType;
    tt8Bit          ulpHdrType;
    tt8Bit          ulpSafe;

/* Filter for all length of packets */
    TM_UNREF_IN_ARG(totalLength);

/* Allow the packet by default */
    errorCode = TM_ENOERROR;

    ulpSafe = TM_8BIT_NO;
    ulpHdrType = 0;
    ipHdrType = *((ttCharPtr)iphPtr) >> 4;
    if (ipHdrType == TM_IP_4)
    {
        ip4HeaderPtr = (ttIpHeaderPtr)iphPtr;
        ipHdrType = TM_IP_4;
/* For IPv4, the ULP ports (first 8 bytes) are always contiguous */
        ulpSafe = TM_8BIT_YES;
        ulpHdrType = ip4HeaderPtr->iphUlp;
#ifdef TM_4_USE_SCOPE_ID
        dstPort = tv4MappedServerSockAddr[0].ss_port;
#else /* TM_4_USE_SCOPE_ID */
        dstPort = tv4ServerSockAddr[0].ss_port;
#endif /* TM_4_USE_SCOPE_ID */
    }
    else
    {
        goto TFX_FILTER_CALLBACK_EXIT;
    }

/* Process packet by default */
    errorCode = TM_ENOERROR;

    if (   (   (interfaceHandle == tvInterfaceHandleClient)
            && (direction == TM_FILTER_OUTGOING))
        || (   (interfaceHandle == tvInterfaceHandleServer)
            && (direction == TM_FILTER_INCOMING)) )
    {
        if (ulpSafe == TM_8BIT_YES)
        {
            switch (ulpHdrType)
            {
            case TM_IP_TCP:
/* Block all TCP traffic destined for port 0x8000 */
                if (((ttTcpHeaderPtr)ulpPtr)->tcpDstPort == dstPort)
                {
                    errorCode = TM_EOPNOTSUPP;
                }
                break;
            case TM_IP_UDP:
/* Block all UDP traffic destined for port 0x8000 */
                if (((ttUdpHeaderPtr)ulpPtr)->udpDstPort == dstPort)
                {
                    errorCode = TM_EOPNOTSUPP;
                }
                break;
            default:
/* Process packet by default */
                break;
            }
        }
        else
        {
/* Drop all IP traffic for above if-statement */
            errorCode = TM_EOPNOTSUPP;
        }
    }

TFX_FILTER_CALLBACK_EXIT:
    return errorCode;
}


#endif /* TM_USE_FILTERING || TM_6_USE_FILTERING */


/*
 * configure the IPv4 address in the two contexts, 
 * setup two addresses for each context.
 * store the ip address and port information in the globle address varibales.
 */


/*
 * Store the given IP address, netmask and port in the global variables
 * Configure the interfaces with the given information
 */
void tfx4ConfigureIpAddr(char * ipaddrStr,
                         char * netmaskStr,
                         ttUser16Bit port,
                         unsigned char mhomeIndex,
                         ttUser8Bit isServer)
{
    int errorCode;

    tm_assert(tfx4ConfigureIpAddr, mhomeIndex < TM_MAX_IPS_PER_IF);

    if (isServer == (ttUser8Bit)1)
    {
        tfSetCurrentContext(tvContextHandleServer);
        tm_strcpy(tv4ServerIpAddrStr[mhomeIndex], ipaddrStr);
        tm_bzero(&tv4ServerSockAddr[mhomeIndex],
                 sizeof(struct sockaddr_storage));
        tv4ServerSockAddr[mhomeIndex].ss_family = AF_INET;
        tv4ServerSockAddr[mhomeIndex].ss_len = sizeof(struct sockaddr_in);
        tv4ServerSockAddr[mhomeIndex].ss_port = htons(port);

        errorCode = inet_pton(
                            AF_INET,
                            ipaddrStr,
                            &tv4ServerSockAddr[mhomeIndex].addr.ipv4.sin_addr);
        tm_assert(tfx4ConfigureIpAddr, errorCode == 1);

        if (mhomeIndex == 0)
        {
#ifdef TM_USE_DHCP
            errorCode = tfUseDhcp( tvInterfaceHandleServer, 0);
            tm_assert(tfx4ConfigureIpAddr, errorCode == TM_ENOERROR);
            errorCode = tfOpenInterface(
                            tvInterfaceHandleServer,
                            0,
                            inet_addr(netmaskStr),
                            TM_DEV_MCAST_ENB | TM_DEV_IP_DHCP,   /* flag */
                            1);
            tm_assert(tfx4ConfigureIpAddr,
                         (errorCode == TM_EINPROGRESS)
                      || (errorCode == TM_ENOERROR));
            errorCode = tfUnConfigInterface(
                            tvInterfaceHandleServer,
                            0);
            tm_assert(tfx4ConfigureIpAddr, errorCode == TM_ENOERROR);
#endif /* TM_USE_DHCP */
            errorCode = tfOpenInterface(
                            tvInterfaceHandleServer,
                            tv4ServerSockAddr[0].addr.ipv4.sin_addr.s_addr,
                            inet_addr(netmaskStr),
                            TM_DEV_MCAST_ENB,   /* flag */
                            1);
        }
#ifndef TM_SINGLE_INTERFACE_HOME
        else
        {
            errorCode = tfConfigInterface(
                            tvInterfaceHandleServer,
                            tv4ServerSockAddr[mhomeIndex].
                                                addr.ipv4.sin_addr.s_addr,
                            inet_addr(netmaskStr),
                            TM_DEV_MCAST_ENB,   /* flag */
                            1,
                            mhomeIndex);
        }
        tm_assert(tfx4ConfigureIpAddr, errorCode == TM_ENOERROR);
#endif /* !TM_SINGLE_INTERFACE_HOME */

#ifdef TM_4_USE_SCOPE_ID
        tm_bzero(&tv4MappedServerSockAddr[mhomeIndex],
                 sizeof(struct sockaddr_storage));
        tv4MappedServerSockAddr[mhomeIndex].ss_family = AF_INET6;
        tv4MappedServerSockAddr[mhomeIndex].ss_len = 
                                     sizeof(struct sockaddr_in6);
        tv4MappedServerSockAddr[mhomeIndex].ss_port = htons(port);
        tm_6_addr_to_ipv4_mapped(
            tv4ServerSockAddr[mhomeIndex].addr.ipv4.sin_addr.s_addr,
            &tv4MappedServerSockAddr[mhomeIndex].addr.ipv6.sin6_addr);
        errorCode = tf6SockaddrSetScopeId(tvInterfaceHandleServer,
                                          &tv4MappedServerSockAddr[mhomeIndex]);
        tm_assert(tfx4ConfigureIpAddr, errorCode == TM_ENOERROR);
#endif /* TM_4_USE_SCOPE_ID */
    }
    else
    {
        tfSetCurrentContext(tvContextHandleClient);
        tm_strcpy(tv4ClientIpAddrStr[mhomeIndex], ipaddrStr);
        tm_bzero(&tv4ClientSockAddr[mhomeIndex],
                 sizeof(struct sockaddr_storage));
        tv4ClientSockAddr[mhomeIndex].ss_family = AF_INET;
        tv4ClientSockAddr[mhomeIndex].ss_len = sizeof(struct sockaddr_in);

        errorCode = inet_pton(
                            AF_INET,
                            ipaddrStr,
                            &tv4ClientSockAddr[mhomeIndex].addr.ipv4.sin_addr);
        tm_assert(tfx4ConfigureIpAddr, errorCode == 1);

        if (mhomeIndex == 0)
        {
            errorCode = tfOpenInterface(
                            tvInterfaceHandleClient,
                            tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr,
                            inet_addr(netmaskStr),
                            TM_DEV_MCAST_ENB,   /* flag */
                            1);
        }
#ifndef TM_SINGLE_INTERFACE_HOME
        else
        {
            errorCode = tfConfigInterface(
                            tvInterfaceHandleClient,
                            tv4ClientSockAddr[mhomeIndex].
                                                addr.ipv4.sin_addr.s_addr,
                            inet_addr(netmaskStr),
                            TM_DEV_MCAST_ENB,   /* flag */
                            1,
                            mhomeIndex);
        }
        tm_assert(tfx4ConfigureIpAddr, errorCode == TM_ENOERROR);
#endif /* !TM_SINGLE_INTERFACE_HOME */
#ifdef TM_4_USE_SCOPE_ID
        tm_bzero(&tv4MappedClientSockAddr[mhomeIndex],
                 sizeof(struct sockaddr_storage));
        tv4MappedClientSockAddr[mhomeIndex].ss_family = AF_INET6;
        tv4MappedClientSockAddr[mhomeIndex].ss_len = 
                                     sizeof(struct sockaddr_in6);
        tv4MappedClientSockAddr[mhomeIndex].ss_port = htons(port);
        tm_6_addr_to_ipv4_mapped(
            tv4ClientSockAddr[mhomeIndex].addr.ipv4.sin_addr.s_addr,
            &tv4MappedClientSockAddr[mhomeIndex].addr.ipv6.sin6_addr);
        errorCode = tf6SockaddrSetScopeId(tvInterfaceHandleClient,
                                          &tv4MappedClientSockAddr[mhomeIndex]);
        tm_assert(tfx4ConfigureIpAddr, errorCode == TM_ENOERROR);
#endif /* TM_4_USE_SCOPE_ID */
    }
}


int tfxWaitConfigueInterface(
    ttUserInterface interfaceHandle,
    int protocolFamily )
{
    int errorCode;

    errorCode = TM_EINPROGRESS;

    for(;;)
    {
        tfTimerUpdate();
        tfTimerExecute();

        errorCode = tfCheckOpenInterface(
                        interfaceHandle,
                        protocolFamily);

        if (errorCode  == TM_EINPROGRESS)
        {
            continue;
        }
        else if (errorCode == 0)
        {
/* configue finished or an error happened */
            break;
        }
        else
        {
/* there is an error, but let's ignore it now, maybe it'll solve itself. */
            if ( (errorCode == TM_EPERM) || (errorCode == TM_EINVAL))
            {
                break;
            }
        }
    }
    return errorCode;
}


int tfxWaitConfigureIndex(
    ttUserInterface interfaceHandle,
    int             protocolFamily,
    unsigned int    mHomeIndex,
    tt8Bit          dualContext)
{
    ttDeviceEntryPtr    devPtr;
    tt8BitPtr           flagPtr;
    int                 errorCode;
    tt8Bit              flagSuccess;
    tt8Bit              flagBusy;

#ifdef TM_SINGLE_INTERFACE_HOME
    TM_UNREF_IN_ARG(mHomeIndex);
#endif /* TM_SINGLE_INTERFACE_HOME */

    for (;;)
    {
/* Basic check on interface */
        errorCode = tfCheckOpenInterface(interfaceHandle, protocolFamily);
        if (errorCode != TM_EINPROGRESS)
        {
/* Pass or fail on basic check */
            break;
        }
/* Pass the time pending result of basic check */
        tfTimerUpdate();
        tfTimerExecute();
        if (dualContext)
        {
            tfSwitchContext();
            tfTimerExecute();
            tfSwitchContext();
        }
    }
/*
 * If the basic check passed then wait for the specified multihome index to
 * get configured.
 */
    if (errorCode == TM_ENOERROR)
    {
        devPtr = (ttDeviceEntryPtr)interfaceHandle;
        {
            flagPtr = &(tm_ip_dev_conf_flag(devPtr, mHomeIndex));
            flagSuccess = TM_DEV_IP_CONFIG;
            flagBusy = TM_DEV_IP_CONF_STARTED;
        }
        while (*flagPtr & flagBusy)
        {
/* Pass the time while the address is configuring */
            tfTimerUpdate();
            tfTimerExecute();
            if (dualContext)
            {
                tfSwitchContext();
                tfTimerExecute();
                tfSwitchContext();
            }
        }
/* Return success or failure result */
        errorCode = (*flagPtr & flagSuccess) ? TM_ENOERROR : TM_ENETDOWN;
    }
    return errorCode;
}


/* 
 * based on the family, AF_INET or AF_INET6,
 * choose an appropriate client , 
 *
 */
void tfxGetClientSockAddr(
    int                         index, /* choose a address slot */
    int                         family,
    struct sockaddr TM_FAR **   sockAddrPtrPtr,
    int TM_FAR *                sockAddrLengthPtr)
{
    if (family == AF_INET)
    {
#ifdef TM_4_USE_SCOPE_ID
        *sockAddrPtrPtr = (struct sockaddr*)&tv4MappedClientSockAddr[index];
        *sockAddrLengthPtr = tv4MappedClientSockAddr[index].ss_len;
#else /* ! TM_4_USE_SCOPE_ID */
        *sockAddrPtrPtr = (struct sockaddr*)&tv4ClientSockAddr[index];
        *sockAddrLengthPtr = tv4ClientSockAddr[index].ss_len;
#endif /* ! TM_4_USE_SCOPE_ID */
    }
}

/* 
 * based on the family, AF_INET or AF_INET6,
 * choose an appropriate server , 
 *
 */
void tfxGetServerSockAddr(
    int                         index, /* choose a address slot */
    int                         family,
    struct sockaddr TM_FAR **   sockAddrPtrPtr,
    int TM_FAR *                sockAddrLengthPtr)
{
    if (family == AF_INET)
    {
#ifdef TM_4_USE_SCOPE_ID
        *sockAddrPtrPtr = (struct sockaddr*)&tv4MappedServerSockAddr[index];
        *sockAddrLengthPtr = tv4MappedServerSockAddr[index].ss_len;
#else /* ! TM_4_USE_SCOPE_ID */
        *sockAddrPtrPtr = (struct sockaddr*)&tv4ServerSockAddr[index];
        *sockAddrLengthPtr = tv4ServerSockAddr[index].ss_len;
#endif /* ! TM_4_USE_SCOPE_ID */
    }
}

/**
 ** Convert an IP address from text to sockaddr_storage form.
 ** Returns the pointer to the sockaddr_storage structure.
 **/
ttSockAddrStoragePtr tfxInetAddr(
    ttConstCharPtr          addrStr,
    ttSockAddrStoragePtr    addrPtr )
{
    int             errorCode;

    tm_bzero(addrPtr, sizeof(struct sockaddr_storage));
    {
        errorCode = inet_pton(AF_INET, addrStr, &addrPtr->addr.ipv4.sin_addr);
        if (errorCode == 1)
        {
            addrPtr->ss_len = sizeof(struct sockaddr_in);
            addrPtr->ss_family = AF_INET;
        }
        else
        {
            errorCode = TM_EINVAL;
        }
    }
    tm_assert(tfxInetAddr, errorCode == 1);
    return addrPtr;
}

ttUserInterface tfxGetInterfaceByContext(ttUserContext context)
{
    ttUserInterface interfaceHanlde;

    if ( context == tvContextHandleServer)
    {
        interfaceHanlde = tvInterfaceHandleServer;
    }
    else if ( context == tvContextHandleClient)
    {
        interfaceHanlde = tvInterfaceHandleClient;
    }
    else
    {
        interfaceHanlde = 0;
    }

    return interfaceHanlde;
}

void tfSwitchContext(void)
{
    ttUserContext currentContext;

    currentContext = tfGetCurrentContext();

    if (currentContext == tvContextHandleClient)
    {
        tfSetCurrentContext(tvContextHandleServer);
    }
    else
    {
        tfSetCurrentContext(tvContextHandleClient);
    }
}

void FillSendBuffer(char * buffer, int size)
{
    while(size > 0)
    {
        buffer[--size] = 'E';
    }
}

void tfxSetupMCastInterface(void)
{
    int             errorCode;
    ttUserContext   oldContext;

    oldContext = tfGetCurrentContext();

    tfSetCurrentContext(tvContextHandleServer);
    errorCode = tfSetMcastInterface(tvInterfaceHandleServer, 1);
    tm_assert(tfxSetMcastInterface, 
             (errorCode == TM_ENOERROR) || (errorCode == TM_EALREADY))


    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfSetMcastInterface(tvInterfaceHandleClient, 1);
    tm_assert(tfxSetMcastInterface, 
             (errorCode == TM_ENOERROR) || (errorCode == TM_EALREADY))

    tfSetCurrentContext(oldContext);
}



void tfxCleanUp()
{

    int errorCode;

    tfSetCurrentContext(tvContextHandleServer);


/* unconfigure manual V4 address */
    errorCode = tfNgUnConfigInterface(
        tvInterfaceHandleServer,
        AF_INET,
        0); /* multi-home index 4 */
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));

/* unconfigure manual V4 address */
#ifndef TM_SINGLE_INTERFACE_HOME
    errorCode = tfNgUnConfigInterface(
        tvInterfaceHandleServer,
        AF_INET,
        1); /* multi-home index 4 */
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
#endif /* !TM_SINGLE_INTERFACE_HOME */

/* verify that the interface is disabled for IPv4 */
    errorCode = tfCheckOpenInterface(tvInterfaceHandleServer, AF_INET);
/* TBD: this following test failt */
/*     tm_assert(tfxCleanUp, (errorCode == TM_EPERM)); */

#ifndef TM_SINGLE_INTERFACE_HOME
    errorCode = tfCloseInterface(tvInterfaceHandleServer);
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
#endif /* ! TM_SINGLE_INTERFACE_HOME */

#ifndef TM_SINGLE_INTERFACE_HOME
/* Close the offlink interface that Ed added, addresses will be 
 *  unconfigured automatically */
    errorCode = tfCloseInterface(tvOffLinkInterfaceServer);
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
#endif /* ! TM_SINGLE_INTERFACE_HOME */

    tfSetCurrentContext(tvContextHandleClient);



/* unconfigure manual address */
    errorCode = tfNgUnConfigInterface(
        tvInterfaceHandleClient,
        AF_INET,
        0); /* multi-home index 4 */
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));

#ifndef TM_SINGLE_INTERFACE_HOME
/* unconfigure manual address */
    errorCode = tfNgUnConfigInterface(
        tvInterfaceHandleClient,
        AF_INET,
        1); /* multi-home index 4 */
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
#endif /* !TM_SINGLE_INTERFACE_HOME */


    errorCode = tfCloseInterface(tvInterfaceHandleClient);
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
#ifndef TM_SINGLE_INTERFACE_HOME
/* Close the offlink interface that Ed added, addresses will be 
 *  unconfigured automatically */
    errorCode = tfCloseInterface(tvOffLinkInterfaceClient);
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
/* Close the Extra client interface that Ed added, addresses will be 
 *  unconfigured automatically */
    errorCode = tfCloseInterface(tvInterfaceHandleExtraClient);
    tm_assert(tfxCleanUp, (errorCode == TM_ENOERROR));
#endif /* ! TM_SINGLE_INTERFACE_HOME */
}


void tfxInterfaceSetOptions()
{
    unsigned char optionValue;
    int errorCode;

    errorCode = TM_ENOERROR;
    optionValue = 1;


}


#ifdef TM_USE_DYNAMIC_MEMORY
static tt32Bit tfxGetMemRecycledBytes (void)
{
    tt32Bit memRecycledBytes;

    memRecycledBytes = 0;
    if (tvContextHandleServer)
    {
        memRecycledBytes =
            ((ttContextPtr)tvContextHandleServer)->tvMemRecycledBytes;
    }
    if (tvContextHandleClient)
    {
        memRecycledBytes +=
            ((ttContextPtr)tvContextHandleClient)->tvMemRecycledBytes;
    }
    return memRecycledBytes;
}
#endif /* TM_USE_DYNAMIC_MEMORY */

void tfxPrintMemUse()
{
#ifdef TM_USE_SHEAP
static int prevUsedBytes = 0;
#ifdef TM_USE_DYNAMIC_MEMORY
       tt32Bit memRecycledBytes;
#endif /* TM_USE_DYNAMIC_MEMORY */

#ifdef TM_USE_DYNAMIC_MEMORY

    memRecycledBytes = tfxGetMemRecycledBytes();
    tvCurrUsedBytes =   tm_global(tvSheapUsedLongs) * sizeof(tt32Bit)
                      - memRecycledBytes;
#else /* !TM_USE_DYNAMIC_MEMORY */
    tvCurrUsedBytes = tm_global(tvSheapUsedLongs) * sizeof(tt32Bit); 
#endif /* !TM_USE_DYNAMIC_MEMORY */
    tvUsedBytesIncrement = tvCurrUsedBytes - prevUsedBytes;
    prevUsedBytes = tvCurrUsedBytes;
    printf(" sheap: %u(%+d)\n", tvCurrUsedBytes, tvUsedBytesIncrement);
#endif /* TM_USE_SHEAP */
}

void tfxPrintRoundStats()
{
#ifdef TM_USE_SHEAP
#ifdef TM_USE_DYNAMIC_MEMORY
    tt32Bit memRecycledBytes;
#endif /* TM_USE_DYNAMIC_MEMORY */
    int increment;
    static int tlLastChangedRound = 0;
    static int tlLastChange = 0;
#ifndef TM_DISABLE_ALL_FILE_LOGGING
    FILE * logFilePtr;
#endif /* !TM_DISABLE_ALL_FILE_LOGGING */

#ifdef TM_USE_DYNAMIC_MEMORY
    memRecycledBytes = tfxGetMemRecycledBytes();
    tvCurrUsedBytes =   tm_global(tvSheapUsedLongs) * sizeof(tt32Bit)
                      - memRecycledBytes;
#else /* !TM_USE_DYNAMIC_MEMORY */
    tvCurrUsedBytes = tm_global(tvSheapUsedLongs) * sizeof(tt32Bit); 
#endif /* !TM_USE_DYNAMIC_MEMORY */
    increment = tvCurrUsedBytes - tvLastRoundUsedBytes;
    if (increment != 0)
    {
        tlLastChangedRound = tvTestRound;
        tlLastChange = increment;
    }

    tfKernelPrint(TM_PRINT_TYPE_STAT, 
                  "\nRnd %d Sheap usage: "
                  "this round: %d(%+d), "
                  "last change: %+d in Rnd %d\n\n",
                  tvTestRound, 
                  tvCurrUsedBytes, 
                  increment,
                  tlLastChange,
                  tlLastChangedRound);

#ifdef _WIN32
#ifndef TM_DISABLE_ALL_FILE_LOGGING
    logFilePtr = fopen(TM_SHEAP_LOG_FILE, "a+");
    if (logFilePtr )
    {
        fprintf(  logFilePtr ,
                  "Rnd %d Sheap usage: "
                  "this round: \t%d(%+d), "
                  "\tlast change: %+d \tin Rnd %d\n",
                  tvTestRound, 
                  tvCurrUsedBytes, 
                  increment,
                  tlLastChange,
                  tlLastChangedRound);

        fclose(logFilePtr );
    }
#endif /* !TM_DISABLE_ALL_FILE_LOGGING */
#endif /* _WIN32 */

#ifndef TM_DISABLE_DYNAMIC_MEMORY
    tm_assert(tfxPrintRoundStats, tlLastChangedRound < 200);
#endif /* TM_DISABLE_DYNAMIC_MEMORY */

    tvLastRoundUsedBytes = tvCurrUsedBytes;
#endif /* TM_USE_SHEAP */
}



#else /* !TM_MULTIPLE_CONTEXT*/

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltx6CommDummy = 0;

#endif /* !TM_MULTIPLE_CONTEXT */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tucommon.c    File Revision: 6.0.2.8 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 26613    Output checksum: 50068
 */
