/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Tests the software crypto modules that have their associated
 *              TM_USE_***_TEST macro defined.
 *
 * Filename: tucrypto.c
 * Author: J Poffenbarger
 * Date Created: 07/17/2007
 * $Source: test/unittest/tucrypto.c $
 *
 * Modification History
 * $Revision: 6.0.2.4 $
 * $Date: 2011/06/15 07:06:14CDT $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#ifdef TM_USE_CRYPTO_TEST
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <trcrylib.h>
/* for automated testing */
#include "tutest.h"

#include <tripsec.h>






//#define TM_CRYPTO_TEST_DEBUG
#ifdef TM_CRYPTO_TEST_DEBUG
void tfxCryptoPrint(tt8BitPtr inputPtr,
                    tt8BitPtr resultPtr,
                    tt8BitPtr hashResultPtr,
                    int       inputLen,
                    int       hashLen,
                    tt8BitPtr decryptBuffPtr);
#endif /* TM_CRYPTO_TEST_DEBUG */

/* Define how many tests are performed for each alg */
/* For the extra test 3, which requires 1,000,000 bytes */
#define CRYPTO_USE_BIG_TEST
#if defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)
#define TM_CRYPTO_SHA_EXTRA_TEST 1
#else /* defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86) */
#define TM_CRYPTO_SHA_EXTRA_TEST 0
#endif /* defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86) */

/* Include tests which need extra bits, not yet supported */
#ifdef TM_USE_FAILED_SHA_TESTS
#define FAILED_TEST_COUNT 3
#else /* !TM_USE_FAILED_SHA_TESTS */
#define FAILED_TEST_COUNT 0
#endif /* TM_USE_FAILED_SHA_TESTS */

/* The maximum number of tests for a hash */
#define MAX_TEST_COUNT 11

#ifdef TM_USE_SHA256
/* Number of vectors in SHA256 hash test */
#define CRYPTO_SHA256_COUNT (7 + TM_CRYPTO_SHA_EXTRA_TEST + FAILED_TEST_COUNT)
/* HMAC hash test count */
#ifdef TM_AALG_VAR_BLOCK_SIZE
#define TM_HMAC_SHA256_TEST_COUNT 7
#else /* !TM_AALG_VAR_BLOCK_SIZE */
#define TM_HMAC_SHA256_TEST_COUNT 5
#endif /* TM_AALG_VAR_BLOCK_SIZE */
/* Test SHA-256 Algorithm, two tests: hash and HMAC */
#define CRYPTO_HASH_COUNT_SHA256 2
#else /* !TM_USE_SHA256 */
#define CRYPTO_HASH_COUNT_SHA256 0
#endif /* TM_USE_SHA256 */

#ifdef TM_USE_SHA384
/* Number of vectors in SHA384 hash test */
#define CRYPTO_SHA384_COUNT (6 + TM_CRYPTO_SHA_EXTRA_TEST + FAILED_TEST_COUNT)
/* HMAC hash test count */
#define TM_HMAC_SHA384_TEST_COUNT 7
/* Test SHA-384 Algorithm, two tests: hash and HMAC */
#define CRYPTO_HASH_COUNT_SHA384 2
#else /* !TM_USE_SHA384 */
#define CRYPTO_HASH_COUNT_SHA384 0
#endif /* TM_USE_SHA384 */

#ifdef TM_USE_SHA512
/* Number of vectors in SHA512 hash test */
#define CRYPTO_SHA512_COUNT (6 + TM_CRYPTO_SHA_EXTRA_TEST + FAILED_TEST_COUNT)
/* HMAC hash test count */
#define TM_HMAC_SHA512_TEST_COUNT 7
/* Test SHA-512 Algorithm, two tests: hash and HMAC */
#define CRYPTO_HASH_COUNT_SHA512 2
#else /* !TM_USE_SHA512 */
#define CRYPTO_HASH_COUNT_SHA512 0
#endif /* TM_USE_SHA512 */

#define CRYPTO_AES_XCBC_MAC_COUNT 0

#define CRYPTO_ENC_COUNT_AES_CBC 0
#define CRYPTO_ENC_COUNT_AES_CTR 0

#define CRYPTO_HASH_ALG_COUNT (CRYPTO_HASH_COUNT_SHA256  + \
                               CRYPTO_HASH_COUNT_SHA384  + \
                               CRYPTO_HASH_COUNT_SHA512  + \
                               CRYPTO_AES_XCBC_MAC_COUNT   )

#define CRYPTO_CIPHER_ALG_COUNT  (CRYPTO_ENC_COUNT_AES_CBC + \
                                  CRYPTO_ENC_COUNT_AES_CTR   )

/* Begin test vectors */

/* For the SHA HMAC tests, the vectors are the same */
#if ( defined(TM_USE_SHA256) || \
      defined(TM_USE_SHA384) || \
      defined(TM_USE_SHA512)    )
/* HMAC Test 1 */
#define HMAC_TEST1_KEY \
        "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b" \
        "\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b"
#define HMAC_TEST1_KEY_LEN sizeof(HMAC_TEST1_KEY)-1
#define HMAC_TEST1_DATA \
        "\x48\x69\x20\x54\x68\x65\x72\x65"
#define HMAC_TEST1_DATA_LEN sizeof(HMAC_TEST1_DATA)-1

/* HMAC Test 2 */
#define HMAC_TEST2_KEY \
    "\x4a\x65\x66\x65"
#define HMAC_TEST2_KEY_LEN 4
/*HMAC_TEST2_DATA ("what do ya want for nothing")*/
#define HMAC_TEST2_DATA \
    "\x77\x68\x61\x74\x20\x64\x6f\x20\x79\x61\x20\x77\x61\x6e\x74\x20" \
    "\x66\x6f\x72\x20\x6e\x6f\x74\x68\x69\x6e\x67\x3f"
#define HMAC_TEST2_DATA_LEN 28

/* HMAC Test 3 */
#define HMAC_TEST3_KEY \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa"
#define HMAC_TEST3_KEY_LEN 20
#define HMAC_TEST3_DATA \
    "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd" \
    "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd" \
    "\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd\xdd" \
    "\xdd\xdd"
#define HMAC_TEST3_DATA_LEN 50

/* HMAC Test 4 */
#define HMAC_TEST4_KEY \
    "\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10" \
    "\x11\x12\x13\x14\x15\x16\x17\x18\x19"
#define HMAC_TEST4_KEY_LEN 25
#define HMAC_TEST4_DATA \
    "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd" \
    "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd" \
    "\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd\xcd" \
    "\xcd\xcd"
#define HMAC_TEST4_DATA_LEN 50

/* HMAC Test 5 */
#define HMAC_TEST5_KEY \
    "\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c\x0c" \
    "\x0c\x0c\x0c\x0c"
#define HMAC_TEST5_KEY_LEN 20
/* HMAC_TEST5_DATA: ("Test With Truncation") */
#define HMAC_TEST5_DATA \
    "\x54\x65\x73\x74\x20\x57\x69\x74\x68\x20\x54\x72\x75\x6e\x63\x61" \
    "\x74\x69\x6f\x6e"
#define HMAC_TEST5_DATA_LEN 20

/* HMAC tests 6 and 7 have longer-than-block-size keys. This requires special
 * processing, TM_AALG_VAR_BLOCK_SIZE is needed */
#ifdef TM_AALG_VAR_BLOCK_SIZE
/* HMAC Test 6 */
#define HMAC_TEST6_KEY \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa"
#define HMAC_TEST6_KEY_LEN sizeof(HMAC_TEST6_KEY)-1
/* "Test Using Larger Than Block-LEN Key - Hash Key" */
#define HMAC_TEST6_DATA \
    "\x54\x65\x73\x74\x20\x55\x73\x69\x6e\x67\x20\x4c\x61\x72\x67\x65" \
    "\x72\x20\x54\x68\x61\x6e\x20\x42\x6c\x6f\x63\x6b\x2d\x53\x69\x7a" \
    "\x65\x20\x4b\x65\x79\x20\x2d\x20\x48\x61\x73\x68\x20\x4b\x65\x79" \
    "\x20\x46\x69\x72\x73\x74"
#define HMAC_TEST6_DATA_LEN sizeof(HMAC_TEST6_DATA)-1

/* HMAC Test 7 */
#define HMAC_TEST7_KEY \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa\xaa" \
    "\xaa\xaa\xaa"
#define HMAC_TEST7_KEY_LEN 131
/* 
 * "This is a test using a larger than block-LEN key and a larger than
 * block-LEN data. The key needs to be hashed before being used by the 
 * HMAC algorithm.
 */
#define HMAC_TEST7_DATA \
    "\x54\x68\x69\x73\x20\x69\x73\x20\x61\x20\x74\x65\x73\x74\x20\x75" \
    "\x73\x69\x6e\x67\x20\x61\x20\x6c\x61\x72\x67\x65\x72\x20\x74\x68" \
    "\x61\x6e\x20\x62\x6c\x6f\x63\x6b\x2d\x73\x69\x7a\x65\x20\x6b\x65" \
    "\x79\x20\x61\x6e\x64\x20\x61\x20\x6c\x61\x72\x67\x65\x72\x20\x74" \
    "\x68\x61\x6e\x20\x62\x6c\x6f\x63\x6b\x2d\x73\x69\x7a\x65\x20\x64" \
    "\x61\x74\x61\x2e\x20\x54\x68\x65\x20\x6b\x65\x79\x20\x6e\x65\x65" \
    "\x64\x73\x20\x74\x6f\x20\x62\x65\x20\x68\x61\x73\x68\x65\x64\x20" \
    "\x62\x65\x66\x6f\x72\x65\x20\x62\x65\x69\x6e\x67\x20\x75\x73\x65" \
    "\x64\x20\x62\x79\x20\x74\x68\x65\x20\x48\x4d\x41\x43\x20\x61\x6c" \
    "\x67\x6f\x72\x69\x74\x68\x6d\x2e"
#define HMAC_TEST7_DATA_LEN 152
#endif /* TM_AALG_VAR_BLOCK_SIZE */
#endif /* TM_USE_SHA256 || TM_USE_SHA384 || TM_USE_SHA512 */

/* The results for the HMAC tests are different for each algorithm */
#ifdef TM_USE_SHA256
#define HMAC_TEST1_RESULT_256 \
        "\xb0\x34\x4c\x61\xd8\xdb\x38\x53\x5c\xa8\xaf\xce\xaf\x0b\xf1\x2b" \
        "\x88\x1d\xc2\x00\xc9\x83\x3d\xa7\x26\xe9\x37\x6c\x2e\x32\xcf\xf7"
#define HMAC_TEST2_RESULT_256 \
    "\x5b\xdc\xc1\x46\xbf\x60\x75\x4e\x6a\x04\x24\x26\x08\x95\x75\xc7" \
    "\x5a\x00\x3f\x08\x9d\x27\x39\x83\x9d\xec\x58\xb9\x64\xec\x38\x43"
#define HMAC_TEST3_RESULT_256 \
    "\x77\x3e\xa9\x1e\x36\x80\x0e\x46\x85\x4d\xb8\xeb\xd0\x91\x81\xa7" \
    "\x29\x59\x09\x8b\x3e\xf8\xc1\x22\xd9\x63\x55\x14\xce\xd5\x65\xfe"
#define HMAC_TEST4_RESULT_256 \
    "\x82\x55\x8a\x38\x9a\x44\x3c\x0e\xa4\xcc\x81\x98\x99\xf2\x08\x3a" \
    "\x85\xf0\xfa\xa3\xe5\x78\xf8\x07\x7a\x2e\x3f\xf4\x67\x29\x66\x5b"
#define HMAC_TEST5_RESULT_256 \
    "\xa3\xb6\x16\x74\x73\x10\x0e\xe0\x6e\x0c\x79\x6c\x29\x55\x55\x2b"
#define HMAC_TEST1_RESULT_256_LEN sizeof(HMAC_TEST1_RESULT_256) - 1
#define HMAC_TEST2_RESULT_256_LEN sizeof(HMAC_TEST2_RESULT_256) - 1
#define HMAC_TEST3_RESULT_256_LEN sizeof(HMAC_TEST3_RESULT_256) - 1
#define HMAC_TEST4_RESULT_256_LEN sizeof(HMAC_TEST4_RESULT_256) - 1
#define HMAC_TEST5_RESULT_256_LEN sizeof(HMAC_TEST5_RESULT_256) - 1
#ifdef TM_AALG_VAR_BLOCK_SIZE
#define HMAC_TEST6_RESULT_256 \
    "\x60\xe4\x31\x59\x1e\xe0\xb6\x7f\x0d\x8a\x26\xaa\xcb\xf5\xb7\x7f" \
    "\x8e\x0b\xc6\x21\x37\x28\xc5\x14\x05\x46\x04\x0f\x0e\xe3\x7f\x54"
#define HMAC_TEST7_RESULT_256 \
    "\x9b\x09\xff\xa7\x1b\x94\x2f\xcb\x27\x63\x5f\xbc\xd5\xb0\xe9\x44" \
    "\xbf\xdc\x63\x64\x4f\x07\x13\x93\x8a\x7f\x51\x53\x5c\x3a\x35\xe2"
#define HMAC_TEST6_RESULT_256_LEN sizeof(HMAC_TEST6_RESULT_256) - 1
#define HMAC_TEST7_RESULT_256_LEN sizeof(HMAC_TEST7_RESULT_256) - 1
#endif /* TM_AALG_VAR_BLOCK_SIZE */
#endif /* TM_USE_SHA256 */

#ifdef TM_USE_SHA384
#define HMAC_TEST1_RESULT_384 \
        "\xaf\xd0\x39\x44\xd8\x48\x95\x62\x6b\x08\x25\xf4\xab\x46\x90\x7f" \
        "\x15\xf9\xda\xdb\xe4\x10\x1e\xc6\x82\xaa\x03\x4c\x7c\xeb\xc5\x9c" \
        "\xfa\xea\x9e\xa9\x07\x6e\xde\x7f\x4a\xf1\x52\xe8\xb2\xfa\x9c\xb6"
#define HMAC_TEST2_RESULT_384 \
    "\xaf\x45\xd2\xe3\x76\x48\x40\x31\x61\x7f\x78\xd2\xb5\x8a\x6b\x1b" \
    "\x9c\x7e\xf4\x64\xf5\xa0\x1b\x47\xe4\x2e\xc3\x73\x63\x22\x44\x5e" \
    "\x8e\x22\x40\xca\x5e\x69\xe2\xc7\x8b\x32\x39\xec\xfa\xb2\x16\x49"
#define HMAC_TEST3_RESULT_384 \
    "\x88\x06\x26\x08\xd3\xe6\xad\x8a\x0a\xa2\xac\xe0\x14\xc8\xa8\x6f" \
    "\x0a\xa6\x35\xd9\x47\xac\x9f\xeb\xe8\x3e\xf4\xe5\x59\x66\x14\x4b" \
    "\x2a\x5a\xb3\x9d\xc1\x38\x14\xb9\x4e\x3a\xb6\xe1\x01\xa3\x4f\x27"
#define HMAC_TEST4_RESULT_384 \
    "\x3e\x8a\x69\xb7\x78\x3c\x25\x85\x19\x33\xab\x62\x90\xaf\x6c\xa7" \
    "\x7a\x99\x81\x48\x08\x50\x00\x9c\xc5\x57\x7c\x6e\x1f\x57\x3b\x4e" \
    "\x68\x01\xdd\x23\xc4\xa7\xd6\x79\xcc\xf8\xa3\x86\xc6\x74\xcf\xfb"
#define HMAC_TEST5_RESULT_384 \
    "\x3a\xbf\x34\xc3\x50\x3b\x2a\x23\xa4\x6e\xfc\x61\x9b\xae\xf8\x97"
#define HMAC_TEST1_RESULT_384_LEN sizeof(HMAC_TEST1_RESULT_384) - 1
#define HMAC_TEST2_RESULT_384_LEN sizeof(HMAC_TEST2_RESULT_384) - 1
#define HMAC_TEST3_RESULT_384_LEN sizeof(HMAC_TEST3_RESULT_384) - 1
#define HMAC_TEST4_RESULT_384_LEN sizeof(HMAC_TEST4_RESULT_384) - 1
#define HMAC_TEST5_RESULT_384_LEN sizeof(HMAC_TEST5_RESULT_384) - 1

#ifdef TM_AALG_VAR_BLOCK_SIZE
#define HMAC_TEST6_RESULT_384 \
    "\x4e\xce\x08\x44\x85\x81\x3e\x90\x88\xd2\xc6\x3a\x04\x1b\xc5\xb4" \
    "\x4f\x9e\xf1\x01\x2a\x2b\x58\x8f\x3c\xd1\x1f\x05\x03\x3a\xc4\xc6" \
    "\x0c\x2e\xf6\xab\x40\x30\xfe\x82\x96\x24\x8d\xf1\x63\xf4\x49\x52"
#define HMAC_TEST7_RESULT_384 \
    "\x66\x17\x17\x8e\x94\x1f\x02\x0d\x35\x1e\x2f\x25\x4e\x8f\xd3\x2c" \
    "\x60\x24\x20\xfe\xb0\xb8\xfb\x9a\xdc\xce\xbb\x82\x46\x1e\x99\xc5" \
    "\xa6\x78\xcc\x31\xe7\x99\x17\x6d\x38\x60\xe6\x11\x0c\x46\x52\x3e"
#define HMAC_TEST6_RESULT_384_LEN sizeof(HMAC_TEST6_RESULT_384) - 1
#define HMAC_TEST7_RESULT_384_LEN sizeof(HMAC_TEST7_RESULT_384) - 1
#endif /* TM_AALG_VAR_BLOCK_SIZE */
#endif /* TM_USE_SHA384 */

#ifdef TM_USE_SHA512
#define HMAC_TEST1_RESULT_512 \
        "\x87\xaa\x7c\xde\xa5\xef\x61\x9d\x4f\xf0\xb4\x24\x1a\x1d\x6c\xb0" \
        "\x23\x79\xf4\xe2\xce\x4e\xc2\x78\x7a\xd0\xb3\x05\x45\xe1\x7c\xde" \
        "\xda\xa8\x33\xb7\xd6\xb8\xa7\x02\x03\x8b\x27\x4e\xae\xa3\xf4\xe4" \
        "\xbe\x9d\x91\x4e\xeb\x61\xf1\x70\x2e\x69\x6c\x20\x3a\x12\x68\x54"
#define HMAC_TEST2_RESULT_512 \
    "\x16\x4b\x7a\x7b\xfc\xf8\x19\xe2\xe3\x95\xfb\xe7\x3b\x56\xe0\xa3" \
    "\x87\xbd\x64\x22\x2e\x83\x1f\xd6\x10\x27\x0c\xd7\xea\x25\x05\x54" \
    "\x97\x58\xbf\x75\xc0\x5a\x99\x4a\x6d\x03\x4f\x65\xf8\xf0\xe6\xfd" \
    "\xca\xea\xb1\xa3\x4d\x4a\x6b\x4b\x63\x6e\x07\x0a\x38\xbc\xe7\x37"
#define HMAC_TEST3_RESULT_512 \
    "\xfa\x73\xb0\x08\x9d\x56\xa2\x84\xef\xb0\xf0\x75\x6c\x89\x0b\xe9" \
    "\xb1\xb5\xdb\xdd\x8e\xe8\x1a\x36\x55\xf8\x3e\x33\xb2\x27\x9d\x39" \
    "\xbf\x3e\x84\x82\x79\xa7\x22\xc8\x06\xb4\x85\xa4\x7e\x67\xc8\x07" \
    "\xb9\x46\xa3\x37\xbe\xe8\x94\x26\x74\x27\x88\x59\xe1\x32\x92\xfb"
#define HMAC_TEST4_RESULT_512 \
    "\xb0\xba\x46\x56\x37\x45\x8c\x69\x90\xe5\xa8\xc5\xf6\x1d\x4a\xf7" \
    "\xe5\x76\xd9\x7f\xf9\x4b\x87\x2d\xe7\x6f\x80\x50\x36\x1e\xe3\xdb" \
    "\xa9\x1c\xa5\xc1\x1a\xa2\x5e\xb4\xd6\x79\x27\x5c\xc5\x78\x80\x63" \
    "\xa5\xf1\x97\x41\x12\x0c\x4f\x2d\xe2\xad\xeb\xeb\x10\xa2\x98\xdd"
#define HMAC_TEST5_RESULT_512 \
    "\x41\x5f\xad\x62\x71\x58\x0a\x53\x1d\x41\x79\xbc\x89\x1d\x87\xa6"
#define HMAC_TEST1_RESULT_512_LEN sizeof(HMAC_TEST1_RESULT_512) - 1
#define HMAC_TEST2_RESULT_512_LEN sizeof(HMAC_TEST2_RESULT_512) - 1
#define HMAC_TEST3_RESULT_512_LEN sizeof(HMAC_TEST3_RESULT_512) - 1
#define HMAC_TEST4_RESULT_512_LEN sizeof(HMAC_TEST4_RESULT_512) - 1
#define HMAC_TEST5_RESULT_512_LEN sizeof(HMAC_TEST5_RESULT_512) - 1

#ifdef TM_AALG_VAR_BLOCK_SIZE
#define HMAC_TEST6_RESULT_512 \
    "\x80\xb2\x42\x63\xc7\xc1\xa3\xeb\xb7\x14\x93\xc1\xdd\x7b\xe8\xb4" \
    "\x9b\x46\xd1\xf4\x1b\x4a\xee\xc1\x12\x1b\x01\x37\x83\xf8\xf3\x52" \
    "\x6b\x56\xd0\x37\xe0\x5f\x25\x98\xbd\x0f\xd2\x21\x5d\x6a\x1e\x52" \
    "\x95\xe6\x4f\x73\xf6\x3f\x0a\xec\x8b\x91\x5a\x98\x5d\x78\x65\x98"
#define HMAC_TEST7_RESULT_512 \
    "\xe3\x7b\x6a\x77\x5d\xc8\x7d\xba\xa4\xdf\xa9\xf9\x6e\x5e\x3f\xfd" \
    "\xde\xbd\x71\xf8\x86\x72\x89\x86\x5d\xf5\xa3\x2d\x20\xcd\xc9\x44" \
    "\xb6\x02\x2c\xac\x3c\x49\x82\xb1\x0d\x5e\xeb\x55\xc3\xe4\xde\x15" \
    "\x13\x46\x76\xfb\x6d\xe0\x44\x60\x65\xc9\x74\x40\xfa\x8c\x6a\x58"
#define HMAC_TEST6_RESULT_512_LEN sizeof(HMAC_TEST6_RESULT_512) - 1
#define HMAC_TEST7_RESULT_512_LEN sizeof(HMAC_TEST7_RESULT_512) - 1
#endif /* TM_AALG_VAR_BLOCK_SIZE */
#endif /* TM_USE_SHA512 */
/* End HMAC #defines */

/* This test is used for comparison purposes, it needs to be available for
 * any algorithms
 */
#define TEST3 \
    "Junk String"

/* These are test vectors common to all three SHA algorithms */
#if ( defined(TM_USE_SHA256) || \
      defined(TM_USE_SHA384) || \
      defined(TM_USE_SHA512)    )
#define TEST1 \
    "abc"

/* Test 2 is different for SHA-256 */
#ifdef TM_USE_SHA256
#define TEST2_1  \
    "abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
#endif /* TM_USE_SHA256 */

/* Test 2 is the same for SHA-384 and SHA-512 */
#if ( defined(TM_USE_SHA384) || defined(TM_USE_SHA512) )
#define TEST2_2 \
    "abcdefghbcdefghicdefghijdefghijkefghijklfghijklmghijklmn" \
    "hijklmnoijklmnopjklmnopqklmnopqrlmnopqrsmnopqrstnopqrstu"
#endif /* defined(TM_USE_SHA384) || defined(TM_USE_SHA512) */

#define TEST4 \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567" \
    "0123456701234567012345670123456701234567012345670123456701234567"

/* needs extra bits */
#define TEST5 \
    ""

/* Tests 6-11 are unique */
#ifdef TM_USE_SHA256
#define TEST6_256 \
    "\x19"
/* needs extra bits */
#define TEST7_256 \
    "\xbe\x27\x46\xc6\xdb\x52\x76\x5f\xdb\x2f\x88\x70\x0f\x9a\x73"
#define TEST8_256 \
    "\xe3\xd7\x25\x70\xdc\xdd\x78\x7c\xe3\x88\x7a\xb2\xcd\x68\x46\x52"
/* needs extra bits */
#define TEST9_256 \
    "\x3e\x74\x03\x71\xc8\x10\xc2\xb9\x9f\xc0\x4e\x80\x49\x07\xef\x7c" \
    "\xf2\x6b\xe2\x8b\x57\xcb\x58\xa3\xe2\xf3\xc0\x07\x16\x6e\x49\xc1" \
    "\x2e\x9b\xa3\x4c\x01\x04\x06\x91\x29\xea\x76\x15\x64\x25\x45\x70" \
    "\x3a\x2b\xd9\x01\xe1\x6e\xb0\xe0\x5d\xeb\xa0\x14\xeb\xff\x64\x06" \
    "\xa0\x7d\x54\x36\x4e\xff\x74\x2d\xa7\x79\xb0\xb3"
#define TEST10_256 \
    "\x83\x26\x75\x4e\x22\x77\x37\x2f\x4f\xc1\x2b\x20\x52\x7a\xfe\xf0" \
    "\x4d\x8a\x05\x69\x71\xb1\x1a\xd5\x71\x23\xa7\xc1\x37\x76\x00\x00" \
    "\xd7\xbe\xf6\xf3\xc1\xf7\xa9\x08\x3a\xa3\x9d\x81\x0d\xb3\x10\x77" \
    "\x7d\xab\x8b\x1e\x7f\x02\xb8\x4a\x26\xc7\x73\x32\x5f\x8b\x23\x74" \
    "\xde\x7a\x4b\x5a\x58\xcb\x5c\x5c\xf3\x5b\xce\xe6\xfb\x94\x6e\x5b" \
    "\xd6\x94\xfa\x59\x3a\x8b\xeb\x3f\x9d\x65\x92\xec\xed\xaa\x66\xca" \
    "\x82\xa2\x9d\x0c\x51\xbc\xf9\x33\x62\x30\xe5\xd7\x84\xe4\xc0\xa4" \
    "\x3f\x8d\x79\xa3\x0a\x16\x5c\xba\xbe\x45\x2b\x77\x4b\x9c\x71\x09" \
    "\xa9\x7d\x13\x8f\x12\x92\x28\x96\x6f\x6c\x0a\xdc\x10\x6a\xad\x5a" \
    "\x9f\xdd\x30\x82\x57\x69\xb2\xc6\x71\xaf\x67\x59\xdf\x28\xeb\x39" \
    "\x3d\x54\xd6"
#define TEST11_256 \
    "Test vector from febooti.com"
#endif /* TM_USE_SHA256 */

/* Tests 6-10 are unique */
#ifdef TM_USE_SHA384
#define TEST6_384 \
    "\xb9"
#define TEST7_384 \
    "\x8b\xc5\x00\xc7\x7c\xee\xd9\x87\x9d\xa9\x89\x10\x7c\xe0\xaa"
#define TEST8_384 \
    "\xa4\x1c\x49\x77\x79\xc0\x37\x5f\xf1\x0a\x7f\x4e\x08\x59\x17\x39"
#define TEST9_384 \
    "\x68\xf5\x01\x79\x2d\xea\x97\x96\x76\x70\x22\xd9\x3d\xa7\x16\x79" \
    "\x30\x99\x20\xfa\x10\x12\xae\xa3\x57\xb2\xb1\x33\x1d\x40\xa1\xd0" \
    "\x3c\x41\xc2\x40\xb3\xc9\xa7\x5b\x48\x92\xf4\xc0\x72\x4b\x68\xc8" \
    "\x75\x32\x1a\xb8\xcf\xe5\x02\x3b\xd3\x75\xbc\x0f\x94\xbd\x89\xfe" \
    "\x04\xf2\x97\x10\x5d\x7b\x82\xff\xc0\x02\x1a\xeb\x1c\xcb\x67\x4f" \
    "\x52\x44\xea\x34\x97\xde\x26\xa4\x19\x1c\x5f\x62\xe5\xe9\xa2\xd8" \
    "\x08\x2f\x05\x51\xf4\xa5\x30\x68\x26\xe9\x1c\xc0\x06\xce\x1b\xf6" \
    "\x0f\xf7\x19\xd4\x2f\xa5\x21\xc8\x71\xcd\x23\x94\xd9\x6e\xf4\x46" \
    "\x8f\x21\x96\x6b\x41\xf2\xba\x80\xc2\x6e\x83\xa9"
#define TEST10_384 \
    "\x39\x96\x69\xe2\x8f\x6b\x9c\x6d\xbc\xbb\x69\x12\xec\x10\xff\xcf" \
    "\x74\x79\x03\x49\xb7\xdc\x8f\xbe\x4a\x8e\x7b\x3b\x56\x21\xdb\x0f" \
    "\x3e\x7d\xc8\x7f\x82\x32\x64\xbb\xe4\x0d\x18\x11\xc9\xea\x20\x61" \
    "\xe1\xc8\x4a\xd1\x0a\x23\xfa\xc1\x72\x7e\x72\x02\xfc\x3f\x50\x42" \
    "\xe6\xbf\x58\xcb\xa8\xa2\x74\x6e\x1f\x64\xf9\xb9\xea\x35\x2c\x71" \
    "\x15\x07\x05\x3c\xf4\xe5\x33\x9d\x52\x86\x5f\x25\xcc\x22\xb5\xe8" \
    "\x77\x84\xa1\x2f\xc9\x61\xd6\x6c\xb6\xe8\x95\x73\x19\x9a\x2c\xe6" \
    "\x56\x5c\xbd\xf1\x3d\xca\x40\x38\x32\xcf\xcb\x0e\x8b\x72\x11\xe8" \
    "\x3a\xf3\x2a\x11\xac\x17\x92\x9f\xf1\xc0\x73\xa5\x1c\xc0\x27\xaa" \
    "\xed\xef\xf8\x5a\xad\x7c\x2b\x7c\x5a\x80\x3e\x24\x04\xd9\x6d\x2a" \
    "\x77\x35\x7b\xda\x1a\x6d\xae\xed\x17\x15\x1c\xb9\xbc\x51\x25\xa4" \
    "\x22\xe9\x41\xde\x0c\xa0\xfc\x50\x11\xc2\x3e\xcf\xfe\xfd\xd0\x96" \
    "\x76\x71\x1c\xf3\xdb\x0a\x34\x40\x72\x0e\x16\x15\xc1\xf2\x2f\xbc" \
    "\x3c\x72\x1d\xe5\x21\xe1\xb9\x9b\xa1\xbd\x55\x77\x40\x86\x42\x14" \
    "\x7e\xd0\x96"
#endif /* TM_USE_SHA384 */

/* Tests 6-10 are unique */
#ifdef TM_USE_SHA512
#define TEST6_512 \
    "\xd0"
#define TEST7_512 \
    "\x08\xec\xb5\x2e\xba\xe1\xf7\x42\x2d\xb6\x2b\xcd\x54\x26\x70"
#define TEST8_512 \
    "\x8d\x4e\x3c\x0e\x38\x89\x19\x14\x91\x81\x6e\x9d\x98\xbf\xf0\xa0"
#define TEST9_512 \
    "\x3a\xdd\xec\x85\x59\x32\x16\xd1\x61\x9a\xa0\x2d\x97\x56\x97\x0b" \
    "\xfc\x70\xac\xe2\x74\x4f\x7c\x6b\x27\x88\x15\x10\x28\xf7\xb6\xa2" \
    "\x55\x0f\xd7\x4a\x7e\x6e\x69\xc2\xc9\xb4\x5f\xc4\x54\x96\x6d\xc3" \
    "\x1d\x2e\x10\xda\x1f\x95\xce\x02\xbe\xb4\xbf\x87\x65\x57\x4c\xbd" \
    "\x6e\x83\x37\xef\x42\x0a\xdc\x98\xc1\x5c\xb6\xd5\xe4\xa0\x24\x1b" \
    "\xa0\x04\x6d\x25\x0e\x51\x02\x31\xca\xc2\x04\x6c\x99\x16\x06\xab" \
    "\x4e\xe4\x14\x5b\xee\x2f\xf4\xbb\x12\x3a\xab\x49\x8d\x9d\x44\x79" \
    "\x4f\x99\xcc\xad\x89\xa9\xa1\x62\x12\x59\xed\xa7\x0a\x5b\x6d\xd4" \
    "\xbd\xd8\x77\x78\xc9\x04\x3b\x93\x84\xf5\x49\x06"
#define TEST10_512 \
    "\xa5\x5f\x20\xc4\x11\xaa\xd1\x32\x80\x7a\x50\x2d\x65\x82\x4e\x31" \
    "\xa2\x30\x54\x32\xaa\x3d\x06\xd3\xe2\x82\xa8\xd8\x4e\x0d\xe1\xde" \
    "\x69\x74\xbf\x49\x54\x69\xfc\x7f\x33\x8f\x80\x54\xd5\x8c\x26\xc4" \
    "\x93\x60\xc3\xe8\x7a\xf5\x65\x23\xac\xf6\xd8\x9d\x03\xe5\x6f\xf2" \
    "\xf8\x68\x00\x2b\xc3\xe4\x31\xed\xc4\x4d\xf2\xf0\x22\x3d\x4b\xb3" \
    "\xb2\x43\x58\x6e\x1a\x7d\x92\x49\x36\x69\x4f\xcb\xba\xf8\x8d\x95" \
    "\x19\xe4\xeb\x50\xa6\x44\xf8\xe4\xf9\x5e\xb0\xea\x95\xbc\x44\x65" \
    "\xc8\x82\x1a\xac\xd2\xfe\x15\xab\x49\x81\x16\x4b\xbb\x6d\xc3\x2f" \
    "\x96\x90\x87\xa1\x45\xb0\xd9\xcc\x9c\x67\xc2\x2b\x76\x32\x99\x41" \
    "\x9c\xc4\x12\x8b\xe9\xa0\x77\xb3\xac\xe6\x34\x06\x4e\x6d\x99\x28" \
    "\x35\x13\xdc\x06\xe7\x51\x5d\x0d\x73\x13\x2e\x9a\x0d\xc6\xd3\xb1" \
    "\xf8\xb2\x46\xf1\xa9\x8a\x3f\xc7\x29\x41\xb1\xe3\xbb\x20\x98\xe8" \
    "\xbf\x16\xf2\x68\xd6\x4f\x0b\x0f\x47\x07\xfe\x1e\xa1\xa1\x79\x1b" \
    "\xa2\xf3\xc0\xc7\x58\xe5\xf5\x51\x86\x3a\x96\xc9\x49\xad\x47\xd7" \
    "\xfb\x40\xd2"
#endif /* TM_USE_SHA512 */

#define TEST1_LEN       sizeof(TEST1) - 1

/* Test 2 lengths */
#ifdef TM_USE_SHA256
#define TEST2_1_LEN     sizeof(TEST2_1) - 1
#endif /* TM_USE_SHA256 */
#if ( defined(TM_USE_SHA384) || defined(TM_USE_SHA512) )
#define TEST2_2_LEN     sizeof(TEST2_2) - 1
#endif /* defined(TM_USE_SHA384) || defined(TM_USE_SHA512) */

#define TEST3_LEN       1000000
#define TEST4_LEN       sizeof(TEST4) - 1
#define TEST5_LEN       sizeof(TEST5) - 1

#ifdef TM_USE_SHA256
#define TEST6_256_LEN   sizeof(TEST6_256) - 1
#define TEST7_256_LEN   sizeof(TEST7_256) - 1
#define TEST8_256_LEN   sizeof(TEST8_256) - 1
#define TEST9_256_LEN   sizeof(TEST9_256) - 1
#define TEST10_256_LEN  sizeof(TEST10_256) - 1
#define TEST11_256_LEN  sizeof(TEST11_256) - 1
#endif /* TM_USE_SHA256 */
#ifdef TM_USE_SHA384
#define TEST6_384_LEN   sizeof(TEST6_384) - 1
#define TEST7_384_LEN   sizeof(TEST7_384) - 1
#define TEST8_384_LEN   sizeof(TEST8_384) - 1
#define TEST9_384_LEN   sizeof(TEST9_384) - 1
#define TEST10_384_LEN  sizeof(TEST10_384) - 1
#endif /* TM_USE_SHA384 */
#ifdef TM_USE_SHA512
#define TEST6_512_LEN   sizeof(TEST6_512) - 1
#define TEST7_512_LEN   sizeof(TEST7_512) - 1
#define TEST8_512_LEN   sizeof(TEST8_512) - 1
#define TEST9_512_LEN   sizeof(TEST9_512) - 1
#define TEST10_512_LEN  sizeof(TEST10_512) - 1
#endif /* TM_USE_SHA512 */

/*
 * Results for Hash Tests from RFC 4634
 *
 */
#ifdef TM_USE_SHA256
#define TEST1_RESULT_256 \
    "\xBA\x78\x16\xBF\x8F\x01\xCF\xEA\x41\x41\x40\xDE\x5D\xAE\x22\x23" \
    "\xB0\x03\x61\xA3\x96\x17\x7A\x9C\xB4\x10\xFF\x61\xF2\x00\x15\xAD"
#define TEST2_1_RESULT_256 \
    "\x24\x8D\x6A\x61\xD2\x06\x38\xB8\xE5\xC0\x26\x93\x0C\x3E\x60\x39" \
    "\xA3\x3C\xE4\x59\x64\xFF\x21\x67\xF6\xEC\xED\xD4\x19\xDB\x06\xC1"
#define TEST3_RESULT_256 \
    "\xCD\xC7\x6E\x5C\x99\x14\xFB\x92\x81\xA1\xC7\xE2\x84\xD7\x3E\x67" \
    "\xF1\x80\x9A\x48\xA4\x97\x20\x0E\x04\x6D\x39\xCC\xC7\x11\x2C\xD0"
#define TEST4_RESULT_256 \
    "\x59\x48\x47\x32\x84\x51\xBD\xFA\x85\x05\x62\x25\x46\x2C\xC1\xD8" \
    "\x67\xD8\x77\xFB\x38\x8D\xF0\xCE\x35\xF2\x5A\xB5\x56\x2B\xFB\xB5"
#define TEST5_RESULT_256 \
    "\xD6\xD3\xE0\x2A\x31\xA8\x4A\x8C\xAA\x97\x18\xED\x6C\x20\x57\xBE" \
    "\x09\xDB\x45\xE7\x82\x3E\xB5\x07\x9C\xE7\xA5\x73\xA3\x76\x0F\x95"
#define TEST6_256_RESULT \
    "\x68\xAA\x2E\x2E\xE5\xDF\xF9\x6E\x33\x55\xE6\xC7\xEE\x37\x3E\x3D" \
    "\x6A\x4E\x17\xF7\x5F\x95\x18\xD8\x43\x70\x9C\x0C\x9B\xC3\xE3\xD4"
#define TEST7_256_RESULT \
    "\x77\xEC\x1D\xC8\x9C\x82\x1F\xF2\xA1\x27\x90\x89\xFA\x09\x1B\x35" \
    "\xB8\xCD\x96\x0B\xCA\xF7\xDE\x01\xC6\xA7\x68\x07\x56\xBE\xB9\x72"
#define TEST8_256_RESULT \
    "\x17\x5E\xE6\x9B\x02\xBA\x9B\x58\xE2\xB0\xA5\xFD\x13\x81\x9C\xEA" \
    "\x57\x3F\x39\x40\xA9\x4F\x82\x51\x28\xCF\x42\x09\xBE\xAB\xB4\xE8"
#define TEST9_256_RESULT \
    "\x3E\x9A\xD6\x46\x8B\xBB\xAD\x2A\xC3\xC2\xCD\xC2\x92\xE0\x18\xBA" \
    "\x5F\xD7\x0B\x96\x0C\xF1\x67\x97\x77\xFC\xE7\x08\xFD\xB0\x66\xE9"
#define TEST10_256_RESULT \
    "\x97\xDB\xCA\x7D\xF4\x6D\x62\xC8\xA4\x22\xC9\x41\xDD\x7E\x83\x5B" \
    "\x8A\xD3\x36\x17\x63\xF7\xE9\xB2\xD9\x5F\x4F\x0D\xA6\xE1\xCC\xBC"
#define TEST11_256_RESULT \
    "\x07\x7b\x18\xfe\x29\x03\x6a\xda\x48\x90\xbd\xec\x19\x21\x86\xe1" \
    "\x06\x78\x59\x7a\x67\x88\x02\x90\x52\x1d\xf7\x0d\xf4\xba\xc9\xab"

#define TEST1_RESULT_256_LEN        sizeof(TEST1_RESULT_256) - 1
#define TEST2_1_RESULT_256_LEN      sizeof(TEST2_1_RESULT_256) - 1
#define TEST3_RESULT_256_LEN        sizeof(TEST3_RESULT_256) -1
#define TEST4_RESULT_256_LEN        sizeof(TEST4_RESULT_256) - 1
#define TEST5_RESULT_256_LEN        sizeof(TEST5_RESULT_256) - 1
#define TEST6_256_RESULT_LEN        sizeof(TEST6_256_RESULT) - 1
#define TEST7_256_RESULT_LEN        sizeof(TEST7_256_RESULT) - 1
#define TEST8_256_RESULT_LEN        sizeof(TEST8_256_RESULT) - 1
#define TEST9_256_RESULT_LEN        sizeof(TEST9_256_RESULT) - 1
#define TEST10_256_RESULT_LEN       sizeof(TEST10_256_RESULT) - 1
#define TEST11_256_RESULT_LEN       sizeof(TEST11_256_RESULT) - 1
#endif /* TM_USE_SHA256 */

#ifdef TM_USE_SHA384
#define TEST1_RESULT_384 \
    "\xCB\x00\x75\x3F\x45\xA3\x5E\x8B\xB5\xA0\x3D\x69\x9A\xC6\x50\x07" \
    "\x27\x2C\x32\xAB\x0E\xDE\xD1\x63\x1A\x8B\x60\x5A\x43\xFF\x5B\xED" \
    "\x80\x86\x07\x2B\xA1\xE7\xCC\x23\x58\xBA\xEC\xA1\x34\xC8\x25\xA7"
#define TEST2_2_RESULT_384 \
    "\x09\x33\x0C\x33\xF7\x11\x47\xE8\x3D\x19\x2F\xC7\x82\xCD\x1B\x47" \
    "\x53\x11\x1B\x17\x3B\x3B\x05\xD2\x2F\xA0\x80\x86\xE3\xB0\xF7\x12" \
    "\xFC\xC7\xC7\x1A\x55\x7E\x2D\xB9\x66\xC3\xE9\xFA\x91\x74\x60\x39"
#define TEST3_RESULT_384 \
    "\x9D\x0E\x18\x09\x71\x64\x74\xCB\x08\x6E\x83\x4E\x31\x0A\x4A\x1C" \
    "\xED\x14\x9E\x9C\x00\xF2\x48\x52\x79\x72\xCE\xC5\x70\x4C\x2A\x5B" \
    "\x07\xB8\xB3\xDC\x38\xEC\xC4\xEB\xAE\x97\xDD\xD8\x7F\x3D\x89\x85"
#define TEST4_RESULT_384 \
    "\x2F\xC6\x4A\x4F\x50\x0D\xDB\x68\x28\xF6\xA3\x43\x0B\x8D\xD7\x2A" \
    "\x36\x8E\xB7\xF3\xA8\x32\x2A\x70\xBC\x84\x27\x5B\x9C\x0B\x3A\xB0" \
    "\x0D\x27\xA5\xCC\x3C\x2D\x22\x4A\xA6\xB6\x1A\x0D\x79\xFB\x45\x96"     
#define TEST5_RESULT_384 \
    "\x8D\x17\xBE\x79\xE3\x2B\x67\x18\xE0\x7D\x8A\x60\x3E\xB8\x4B\xA0" \
    "\x47\x8F\x7F\xCF\xD1\xBB\x93\x99\x5F\x7D\x11\x49\xE0\x91\x43\xAC" \
    "\x1F\xFC\xFC\x56\x82\x0E\x46\x9F\x38\x78\xD9\x57\xA1\x5A\x3F\xE4" 
#define TEST6_384_RESULT \
    "\xBC\x80\x89\xA1\x90\x07\xC0\xB1\x41\x95\xF4\xEC\xC7\x40\x94\xFE" \
    "\xC6\x4F\x01\xF9\x09\x29\x28\x2C\x2F\xB3\x92\x88\x15\x78\x20\x8A" \
    "\xD4\x66\x82\x8B\x1C\x6C\x28\x3D\x27\x22\xCF\x0A\xD1\xAB\x69\x38" 
#define TEST7_384_RESULT \
    "\xD8\xC4\x3B\x38\xE1\x2E\x7C\x42\xA7\xC9\xB8\x10\x29\x9F\xD6\xA7" \
    "\x70\xBE\xF3\x09\x20\xF1\x75\x32\xA8\x98\xDE\x62\xC7\xA0\x7E\x42" \
    "\x93\x44\x9C\x0B\x5F\xA7\x01\x09\xF0\x78\x32\x11\xCF\xC4\xBC\xE3" 
#define TEST8_384_RESULT \
    "\xC9\xA6\x84\x43\xA0\x05\x81\x22\x56\xB8\xEC\x76\xB0\x05\x16\xF0" \
    "\xDB\xB7\x4F\xAB\x26\xD6\x65\x91\x3F\x19\x4B\x6F\xFB\x0E\x91\xEA" \
    "\x99\x67\x56\x6B\x58\x10\x9C\xBC\x67\x5C\xC2\x08\xE4\xC8\x23\xF7"
#define TEST9_384_RESULT \
    "\x58\x60\xE8\xDE\x91\xC2\x15\x78\xBB\x41\x74\xD2\x27\x89\x8A\x98" \
    "\xE0\xB4\x5C\x4C\x76\x0F\x00\x95\x49\x49\x56\x14\xDA\xED\xC0\x77" \
    "\x5D\x92\xD1\x1D\x9F\x8C\xE9\xB0\x64\xEE\xAC\x8D\xAF\xC3\xA2\x97"
#define TEST10_384_RESULT \
    "\x4F\x44\x0D\xB1\xE6\xED\xD2\x89\x9F\xA3\x35\xF0\x95\x15\xAA\x02" \
    "\x5E\xE1\x77\xA7\x9F\x4B\x4A\xAF\x38\xE4\x2B\x5C\x4D\xE6\x60\xF5" \
    "\xDE\x8F\xB2\xA5\xB2\xFB\xD2\xA3\xCB\xFF\xD2\x0C\xFF\x12\x88\xC0"

#define TEST1_RESULT_384_LEN        sizeof(TEST1_RESULT_384) - 1
#define TEST2_2_RESULT_384_LEN      sizeof(TEST2_2_RESULT_384) - 1
#define TEST3_RESULT_384_LEN        sizeof(TEST3_RESULT_384) -1
#define TEST4_RESULT_384_LEN        sizeof(TEST4_RESULT_384) - 1
#define TEST5_RESULT_384_LEN        sizeof(TEST5_RESULT_384) - 1
#define TEST6_384_RESULT_LEN        sizeof(TEST6_384_RESULT) - 1
#define TEST7_384_RESULT_LEN        sizeof(TEST7_384_RESULT) - 1
#define TEST8_384_RESULT_LEN        sizeof(TEST8_384_RESULT) - 1
#define TEST9_384_RESULT_LEN        sizeof(TEST9_384_RESULT) - 1
#define TEST10_384_RESULT_LEN       sizeof(TEST10_384_RESULT) - 1
#define TEST11_384_RESULT_LEN       sizeof(TEST11_384_RESULT) - 1
#endif /* TM_USE_SHA384 */

#ifdef TM_USE_SHA512
#define TEST1_RESULT_512 \
    "\xDD\xAF\x35\xA1\x93\x61\x7A\xBA\xCC\x41\x73\x49\xAE\x20\x41\x31" \
    "\x12\xE6\xFA\x4E\x89\xA9\x7E\xA2\x0A\x9E\xEE\xE6\x4B\x55\xD3\x9A" \
    "\x21\x92\x99\x2A\x27\x4F\xC1\xA8\x36\xBA\x3C\x23\xA3\xFE\xEB\xBD" \
    "\x45\x4D\x44\x23\x64\x3C\xE8\x0E\x2A\x9A\xC9\x4F\xA5\x4C\xA4\x9F"
#define TEST2_2_RESULT_512 \
    "\x8E\x95\x9B\x75\xDA\xE3\x13\xDA\x8C\xF4\xF7\x28\x14\xFC\x14\x3F" \
    "\x8F\x77\x79\xC6\xEB\x9F\x7F\xA1\x72\x99\xAE\xAD\xB6\x88\x90\x18" \
    "\x50\x1D\x28\x9E\x49\x00\xF7\xE4\x33\x1B\x99\xDE\xC4\xB5\x43\x3A" \
    "\xC7\xD3\x29\xEE\xB6\xDD\x26\x54\x5E\x96\xE5\x5B\x87\x4B\xE9\x09"
#define TEST3_RESULT_512 \
    "\xE7\x18\x48\x3D\x0C\xE7\x69\x64\x4E\x2E\x42\xC7\xBC\x15\xB4\x63" \
    "\x8E\x1F\x98\xB1\x3B\x20\x44\x28\x56\x32\xA8\x03\xAF\xA9\x73\xEB" \
    "\xDE\x0F\xF2\x44\x87\x7E\xA6\x0A\x4C\xB0\x43\x2C\xE5\x77\xC3\x1B" \
    "\xEB\x00\x9C\x5C\x2C\x49\xAA\x2E\x4E\xAD\xB2\x17\xAD\x8C\xC0\x9B"
#define TEST4_RESULT_512 \
    "\x89\xD0\x5B\xA6\x32\xC6\x99\xC3\x12\x31\xDE\xD4\xFF\xC1\x27\xD5" \
    "\xA8\x94\xDA\xD4\x12\xC0\xE0\x24\xDB\x87\x2D\x1A\xBD\x2B\xA8\x14" \
    "\x1A\x0F\x85\x07\x2A\x9B\xE1\xE2\xAA\x04\xCF\x33\xC7\x65\xCB\x51" \
    "\x08\x13\xA3\x9C\xD5\xA8\x4C\x4A\xCA\xA6\x4D\x3F\x3F\xB7\xBA\xE9"
#define TEST5_RESULT_512 \
    "\xD4\xEE\x29\xA9\xE9\x09\x85\x44\x6B\x91\x3C\xF1\xD1\x37\x6C\x83" \
    "\x6F\x4B\xE2\xC1\xCF\x3C\xAD\xA0\x72\x0A\x6B\xF4\x85\x7D\x88\x6A" \
    "\x7E\xCB\x3C\x4E\x4C\x0F\xA8\xC7\xF9\x52\x14\xE4\x1D\xC1\xB0\xD2" \
    "\x1B\x22\xA8\x4C\xC0\x3B\xF8\xCE\x48\x45\xF3\x4D\xD5\xBD\xBA\xD4"
#define TEST6_512_RESULT \
    "\x99\x92\x20\x29\x38\xE8\x82\xE7\x3E\x20\xF6\xB6\x9E\x68\xA0\xA7" \
    "\x14\x90\x90\x42\x3D\x93\xC8\x1B\xAB\x3F\x21\x67\x8D\x4A\xCE\xEE" \
    "\xE5\x0E\x4E\x8C\xAF\xAD\xA4\xC8\x5A\x54\xEA\x83\x06\x82\x6C\x4A" \
    "\xD6\xE7\x4C\xEC\xE9\x63\x1B\xFA\x8A\x54\x9B\x4A\xB3\xFB\xBA\x15"
#define TEST7_512_RESULT \
    "\xED\x8D\xC7\x8E\x8B\x01\xB6\x97\x50\x05\x3D\xBB\x7A\x0A\x9E\xDA" \
    "\x0F\xB9\xE9\xD2\x92\xB1\xED\x71\x5E\x80\xA7\xFE\x29\x0A\x4E\x16" \
    "\x66\x4F\xD9\x13\xE8\x58\x54\x40\x0C\x5A\xF0\x5E\x6D\xAD\x31\x6B" \
    "\x73\x59\xB4\x3E\x64\xF8\xBE\xC3\xC1\xF2\x37\x11\x99\x86\xBB\xB6"
#define TEST8_512_RESULT \
    "\xCB\x0B\x67\xA4\xB8\x71\x2C\xD7\x3C\x9A\xAB\xC0\xB1\x99\xE9\x26" \
    "\x9B\x20\x84\x4A\xFB\x75\xAC\xBD\xD1\xC1\x53\xC9\x82\x89\x24\xC3" \
    "\xDD\xED\xAA\xFE\x66\x9C\x5F\xDD\x0B\xC6\x6F\x63\x0F\x67\x73\x98" \
    "\x82\x13\xEB\x1B\x16\xF5\x17\xAD\x0D\xE4\xB2\xF0\xC9\x5C\x90\xF8"
#define TEST9_512_RESULT \
    "\x32\xBA\x76\xFC\x30\xEA\xA0\x20\x8A\xEB\x50\xFF\xB5\xAF\x18\x64" \
    "\xFD\xBF\x17\x90\x2A\x4D\xC0\xA6\x82\xC6\x1F\xCE\xA6\xD9\x2B\x78" \
    "\x32\x67\xB2\x10\x80\x30\x18\x37\xF5\x9D\xE7\x9C\x6B\x33\x7D\xB2" \
    "\x52\x6F\x8A\x0A\x51\x0E\x5E\x53\xCA\xFE\xD4\x35\x5F\xE7\xC2\xF1"
#define TEST10_512_RESULT \
    "\xC6\x65\xBE\xFB\x36\xDA\x18\x9D\x78\x82\x2D\x10\x52\x8C\xBF\x3B" \
    "\x12\xB3\xEE\xF7\x26\x03\x99\x09\xC1\xA1\x6A\x27\x0D\x48\x71\x93" \
    "\x77\x96\x6B\x95\x7A\x87\x8E\x72\x05\x84\x77\x9A\x62\x82\x5C\x18" \
    "\xDA\x26\x41\x5E\x49\xA7\x17\x6A\x89\x4E\x75\x10\xFD\x14\x51\xF5"

#define TEST1_RESULT_512_LEN        sizeof(TEST1_RESULT_512) - 1
#define TEST2_2_RESULT_512_LEN      sizeof(TEST2_2_RESULT_512) - 1
#define TEST3_RESULT_512_LEN        sizeof(TEST3_RESULT_512) -1
#define TEST4_RESULT_512_LEN        sizeof(TEST4_RESULT_512) - 1
#define TEST5_RESULT_512_LEN        sizeof(TEST5_RESULT_512) - 1
#define TEST6_512_RESULT_LEN        sizeof(TEST6_512_RESULT) - 1
#define TEST7_512_RESULT_LEN        sizeof(TEST7_512_RESULT) - 1
#define TEST8_512_RESULT_LEN        sizeof(TEST8_512_RESULT) - 1
#define TEST9_512_RESULT_LEN        sizeof(TEST9_512_RESULT) - 1
#define TEST10_512_RESULT_LEN       sizeof(TEST10_512_RESULT) - 1
#endif /* TM_USE_SHA512 */
#endif /* TM_USE_SHA256 || TM_USE_SHA384 || TM_USE_SHA512 */
/* End SHA Algorithm tests */

/* End Hash Algorithm tests */

/* Encryption Algorithm tests */
/* End encryption tests */

#if ( defined(TM_USE_SHA256) || defined(TM_USE_SHA384) || \
      defined(TM_USE_SHA512) || \
     (defined(TM_USE_AESXCBC) && defined(TM_USE_AES))     )
typedef struct tsCryptoHash
{
    int             cryptoAlg;
    int             cryptoTestCount;
    int             requestType;
    struct          
    {
        tt8BitPtr       testKey;
        tt8BitPtr       testInput;
        tt8BitPtr       testResult;  
        tt32Bit         testKeyLen;
        tt32Bit         testInputLen;
        tt32Bit         testResultLen;        
    } cryptoTest[MAX_TEST_COUNT];
} ttCryptoHash;

ttCryptoHash cryptoHash[] =
{
#ifdef TM_USE_SHA256
/* First the HMAC tests */
    {
        SADB_AALG_SHA256HMAC,
        TM_HMAC_SHA256_TEST_COUNT,
        TM_CEREQUEST_CRYPTO_HMACHASH,
        {
            {
                (tt8BitPtr)HMAC_TEST1_KEY,
                (tt8BitPtr)HMAC_TEST1_DATA,
                (tt8BitPtr)HMAC_TEST1_RESULT_256,
                HMAC_TEST1_KEY_LEN,
                HMAC_TEST1_DATA_LEN,
                HMAC_TEST1_RESULT_256_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST2_KEY,
                (tt8BitPtr)HMAC_TEST2_DATA,
                (tt8BitPtr)HMAC_TEST2_RESULT_256,
                HMAC_TEST2_KEY_LEN,
                HMAC_TEST2_DATA_LEN,
                HMAC_TEST2_RESULT_256_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST3_KEY,
                (tt8BitPtr)HMAC_TEST3_DATA,
                (tt8BitPtr)HMAC_TEST3_RESULT_256,
                HMAC_TEST3_KEY_LEN,
                HMAC_TEST3_DATA_LEN,
                HMAC_TEST3_RESULT_256_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST4_KEY,
                (tt8BitPtr)HMAC_TEST4_DATA,
                (tt8BitPtr)HMAC_TEST4_RESULT_256,
                HMAC_TEST4_KEY_LEN,
                HMAC_TEST4_DATA_LEN,
                HMAC_TEST4_RESULT_256_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST5_KEY,
                (tt8BitPtr)HMAC_TEST5_DATA,
                (tt8BitPtr)HMAC_TEST5_RESULT_256,
                HMAC_TEST5_KEY_LEN,
                HMAC_TEST5_DATA_LEN,
                HMAC_TEST5_RESULT_256_LEN
            }
#ifdef TM_AALG_VAR_BLOCK_SIZE
            ,
            {
                (tt8BitPtr)HMAC_TEST6_KEY,
                (tt8BitPtr)HMAC_TEST6_DATA,
                (tt8BitPtr)HMAC_TEST6_RESULT_256,
                HMAC_TEST6_KEY_LEN,
                HMAC_TEST6_DATA_LEN,
                HMAC_TEST6_RESULT_256_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST7_KEY,
                (tt8BitPtr)HMAC_TEST7_DATA,
                (tt8BitPtr)HMAC_TEST7_RESULT_256,
                HMAC_TEST7_KEY_LEN,
                HMAC_TEST7_DATA_LEN,
                HMAC_TEST7_RESULT_256_LEN
            }
#endif /* TM_AALG_VAR_BLOCK_SIZE */
        }
    },
/* Next the normal hash tests */
    {
        SADB_AALG_SHA256HMAC,
        CRYPTO_SHA256_COUNT,
        TM_CEREQUEST_CRYPTO_HASH,
        {
            /* Passed TEST1 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST1,
                (tt8BitPtr)TEST1_RESULT_256,
                0,
                TEST1_LEN,
                TEST1_RESULT_256_LEN
            },
            
            /* Passed TEST2 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST2_1,
                (tt8BitPtr)TEST2_1_RESULT_256,
                0,
                TEST2_1_LEN,
                TEST2_1_RESULT_256_LEN
            },
            
            /* Passed TEST3 RFC */
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST3,
                (tt8BitPtr)TEST3_RESULT_256,
                0,
                TEST3_LEN,
                TEST3_RESULT_256_LEN
            },
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */

            /* Passed TEST4 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST4,
                (tt8BitPtr)TEST4_RESULT_256,
                0,
                TEST4_LEN,
                TEST4_RESULT_256_LEN
            },
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST5,
                (tt8BitPtr)TEST5_RESULT_256,
                0,
                TEST5_LEN,
                TEST5_RESULT_256_LEN
            }, 
#endif /* TM_USE_FAILED_SHA_TESTS */
            
            /* Passed \x19 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST6_256,
                (tt8BitPtr)TEST6_256_RESULT,
                0,
                TEST6_256_LEN,
                TEST6_256_RESULT_LEN
            },          
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST7_256,
                (tt8BitPtr)TEST7_256_RESULT,
                0,
                TEST7_256_LEN,
                TEST7_256_RESULT_LEN
            },
#endif /* TM_USE_FAILED_SHA_TESTS */
            
            /* Passed TEST8_256 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST8_256,
                (tt8BitPtr)TEST8_256_RESULT,
                0,
                TEST8_256_LEN,
                TEST8_256_RESULT_LEN
            },     
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST9_256,
                (tt8BitPtr)TEST9_256_RESULT,
                0,
                TEST9_256_LEN,
                TEST9_256_RESULT_LEN
            }, 
#endif /* TM_USE_FAILED_SHA_TESTS */
            
            /* Passed TEST10_256 */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST10_256,
                (tt8BitPtr)TEST10_256_RESULT,
                0,
                TEST10_256_LEN,
                TEST10_256_RESULT_LEN
            },

            /* Passed Febooti Test Vector */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST11_256,
                (tt8BitPtr)TEST11_256_RESULT,
                0,
                TEST11_256_LEN,
                TEST11_256_RESULT_LEN
            }
        }
    }
/* Only include a comma if there are more tests to come */
#if ( defined(TM_USE_SHA384) || defined(TM_USE_SHA512) || \
     (defined(TM_USE_AESXCBC) && defined(TM_USE_AES)) )
    ,
#endif /* TM_USE_SHA256 || TM_USE_SHA384 || TM_USE_SHA512 || TM_USE_AES */
#endif /* TM_USE_SHA256 */

#ifdef TM_USE_SHA384
/* First the HMAC tests */
    {
        SADB_AALG_SHA384HMAC,
        TM_HMAC_SHA384_TEST_COUNT,
        TM_CEREQUEST_CRYPTO_HMACHASH,
        {
            {
                (tt8BitPtr)HMAC_TEST1_KEY,
                (tt8BitPtr)HMAC_TEST1_DATA,
                (tt8BitPtr)HMAC_TEST1_RESULT_384,
                HMAC_TEST1_KEY_LEN,
                HMAC_TEST1_DATA_LEN,
                HMAC_TEST1_RESULT_384_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST2_KEY,
                (tt8BitPtr)HMAC_TEST2_DATA,
                (tt8BitPtr)HMAC_TEST2_RESULT_384,
                HMAC_TEST2_KEY_LEN,
                HMAC_TEST2_DATA_LEN,
                HMAC_TEST2_RESULT_384_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST3_KEY,
                (tt8BitPtr)HMAC_TEST3_DATA,
                (tt8BitPtr)HMAC_TEST3_RESULT_384,
                HMAC_TEST3_KEY_LEN,
                HMAC_TEST3_DATA_LEN,
                HMAC_TEST3_RESULT_384_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST4_KEY,
                (tt8BitPtr)HMAC_TEST4_DATA,
                (tt8BitPtr)HMAC_TEST4_RESULT_384,
                HMAC_TEST4_KEY_LEN,
                HMAC_TEST4_DATA_LEN,
                HMAC_TEST4_RESULT_384_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST5_KEY,
                (tt8BitPtr)HMAC_TEST5_DATA,
                (tt8BitPtr)HMAC_TEST5_RESULT_384,
                HMAC_TEST5_KEY_LEN,
                HMAC_TEST5_DATA_LEN,
                HMAC_TEST5_RESULT_384_LEN
            }
#ifdef TM_AALG_VAR_BLOCK_SIZE
            ,
            {
                (tt8BitPtr)HMAC_TEST6_KEY,
                (tt8BitPtr)HMAC_TEST6_DATA,
                (tt8BitPtr)HMAC_TEST6_RESULT_384,
                HMAC_TEST6_KEY_LEN,
                HMAC_TEST6_DATA_LEN,
                HMAC_TEST6_RESULT_384_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST7_KEY,
                (tt8BitPtr)HMAC_TEST7_DATA,
                (tt8BitPtr)HMAC_TEST7_RESULT_384,
                HMAC_TEST7_KEY_LEN,
                HMAC_TEST7_DATA_LEN,
                HMAC_TEST7_RESULT_384_LEN
            }
#endif /* TM_AALG_VAR_BLOCK_SIZE */
        }
    },
/* Next the normal hash tests */
    {
        SADB_AALG_SHA384HMAC,
        CRYPTO_SHA384_COUNT,
        TM_CEREQUEST_CRYPTO_HASH,
        {
            /* Passed TEST1 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST1,
                (tt8BitPtr)TEST1_RESULT_384,
                0,
                TEST1_LEN,
                TEST1_RESULT_384_LEN
            },
            
            /* Passed TEST2 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST2_2,
                (tt8BitPtr)TEST2_2_RESULT_384,
                0,
                TEST2_2_LEN,
                TEST2_2_RESULT_384_LEN
            },
            
            /* Passed TEST3 RFC */
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST3,
                (tt8BitPtr)TEST3_RESULT_384,
                0,
                TEST3_LEN,
                TEST3_RESULT_384_LEN
            },
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */

            /* Passed TEST4 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST4,
                (tt8BitPtr)TEST4_RESULT_384,
                0,
                TEST4_LEN,
                TEST4_RESULT_384_LEN
            },
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST5,
                (tt8BitPtr)TEST5_RESULT_384,
                0,
                TEST5_LEN,
                TEST5_RESULT_384_LEN
            }, 
#endif /* TM_USE_FAILED_SHA_TESTS */
            
            /* Passed \x19 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST6_384,
                (tt8BitPtr)TEST6_384_RESULT,
                0,
                TEST6_384_LEN,
                TEST6_384_RESULT_LEN
            },          
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST7_384,
                (tt8BitPtr)TEST7_384_RESULT,
                0,
                TEST7_384_LEN,
                TEST7_384_RESULT_LEN
            }, 
#endif /* TM_USE_FAILED_SHA_TESTS */
            
            /* Passed TEST8_384 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST8_384,
                (tt8BitPtr)TEST8_384_RESULT,
                0,
                TEST8_384_LEN,
                TEST8_384_RESULT_LEN
            },     
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST9_384,
                (tt8BitPtr)TEST9_384_RESULT,
                0,
                TEST9_384_LEN,
                TEST9_384_RESULT_LEN
            }, 
#endif /* TM_USE_FAILED_SHA_TESTS */
            
            /* Passed TEST10_384 */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST10_384,
                (tt8BitPtr)TEST10_384_RESULT,
                0,
                TEST10_384_LEN,
                TEST10_384_RESULT_LEN
            }
        }
    }
/* Only include a comma if there are more tests to come */
#if ( defined(TM_USE_SHA512) || \
     (defined(TM_USE_AESXCBC) && defined(TM_USE_AES)) )
    ,
#endif /* TM_USE_SHA512 || ( TM_USE_AESXCBC && TM_USE_AES ) */
#endif /* TM_USE_SHA384 */

#ifdef TM_USE_SHA512
/* First the HMAC tests */
    {
        SADB_AALG_SHA512HMAC,
        TM_HMAC_SHA512_TEST_COUNT,
        TM_CEREQUEST_CRYPTO_HMACHASH,
        {
            {
                (tt8BitPtr)HMAC_TEST1_KEY,
                (tt8BitPtr)HMAC_TEST1_DATA,
                (tt8BitPtr)HMAC_TEST1_RESULT_512,
                HMAC_TEST1_KEY_LEN,
                HMAC_TEST1_DATA_LEN,
                HMAC_TEST1_RESULT_512_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST2_KEY,
                (tt8BitPtr)HMAC_TEST2_DATA,
                (tt8BitPtr)HMAC_TEST2_RESULT_512,
                HMAC_TEST2_KEY_LEN,
                HMAC_TEST2_DATA_LEN,
                HMAC_TEST2_RESULT_512_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST3_KEY,
                (tt8BitPtr)HMAC_TEST3_DATA,
                (tt8BitPtr)HMAC_TEST3_RESULT_512,
                HMAC_TEST3_KEY_LEN,
                HMAC_TEST3_DATA_LEN,
                HMAC_TEST3_RESULT_512_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST4_KEY,
                (tt8BitPtr)HMAC_TEST4_DATA,
                (tt8BitPtr)HMAC_TEST4_RESULT_512,
                HMAC_TEST4_KEY_LEN,
                HMAC_TEST4_DATA_LEN,
                HMAC_TEST4_RESULT_512_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST5_KEY,
                (tt8BitPtr)HMAC_TEST5_DATA,
                (tt8BitPtr)HMAC_TEST5_RESULT_512,
                HMAC_TEST5_KEY_LEN,
                HMAC_TEST5_DATA_LEN,
                HMAC_TEST5_RESULT_512_LEN
            }
#ifdef TM_AALG_VAR_BLOCK_SIZE
            ,
            {
                (tt8BitPtr)HMAC_TEST6_KEY,
                (tt8BitPtr)HMAC_TEST6_DATA,
                (tt8BitPtr)HMAC_TEST6_RESULT_512,
                HMAC_TEST6_KEY_LEN,
                HMAC_TEST6_DATA_LEN,
                HMAC_TEST6_RESULT_512_LEN
            },
            {
                (tt8BitPtr)HMAC_TEST7_KEY,
                (tt8BitPtr)HMAC_TEST7_DATA,
                (tt8BitPtr)HMAC_TEST7_RESULT_512,
                HMAC_TEST7_KEY_LEN,
                HMAC_TEST7_DATA_LEN,
                HMAC_TEST7_RESULT_512_LEN
            }
#endif /* TM_AALG_VAR_BLOCK_SIZE */
        }
    },
/* Next the normal hash tests */
    {
        SADB_AALG_SHA512HMAC,
        CRYPTO_SHA512_COUNT,
        TM_CEREQUEST_CRYPTO_HASH,
        {
            /* Passed TEST1 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST1,
                (tt8BitPtr)TEST1_RESULT_512,
                0,
                TEST1_LEN,
                TEST1_RESULT_512_LEN
            },
            
            /* Passed TEST2 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST2_2,
                (tt8BitPtr)TEST2_2_RESULT_512,
                0,
                TEST2_2_LEN,
                TEST2_2_RESULT_512_LEN
            },
            
            /* Passed TEST3 RFC */
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST3,
                (tt8BitPtr)TEST3_RESULT_512,
                0,
                TEST3_LEN,
                TEST3_RESULT_512_LEN
            },
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */

            /* Passed TEST4 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST4,
                (tt8BitPtr)TEST4_RESULT_512,
                0,
                TEST4_LEN,
                TEST4_RESULT_512_LEN
            },
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST5,
                (tt8BitPtr)TEST5_RESULT_512,
                0,
                TEST5_LEN,
                TEST5_RESULT_512_LEN
            }, 
#endif TM_USE_FAILED_SHA_TESTS
            
            /* Passed \x19 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST6_512,
                (tt8BitPtr)TEST6_512_RESULT,
                0,
                TEST6_512_LEN,
                TEST6_512_RESULT_LEN
            },          
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST7_512,
                (tt8BitPtr)TEST7_512_RESULT,
                0,
                TEST7_512_LEN,
                TEST7_512_RESULT_LEN
            }, 
#endif TM_USE_FAILED_SHA_TESTS
            
            /* Passed TEST8_512 RFC */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST8_512,
                (tt8BitPtr)TEST8_512_RESULT,
                0,
                TEST8_512_LEN,
                TEST8_512_RESULT_LEN
            },     
            
#ifdef TM_USE_FAILED_SHA_TESTS
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST9_512,
                (tt8BitPtr)TEST9_512_RESULT,
                0,
                TEST9_512_LEN,
                TEST9_512_RESULT_LEN
            },
#endif TM_USE_FAILED_SHA_TESTS
            
            /* Passed TEST10_512 */
            {
                (tt8BitPtr)0,
                (tt8BitPtr)TEST10_512,
                (tt8BitPtr)TEST10_512_RESULT,
                0,
                TEST10_512_LEN,
                TEST10_512_RESULT_LEN
            }
        }
    }
/* Only include a comma if there are more tests to come */
#if ( (defined(TM_USE_AESXCBC) && defined(TM_USE_AES)) )
    ,
#endif /* (defined(TM_USE_AESXCBC) && defined(TM_USE_AES)) */
#endif /* TM_USE_SHA512 */
    
#if (defined(TM_USE_AESXCBC) && defined(TM_USE_AES))
    {
        SADB_AALG_AESXCBCMAC,
        CRYPTO_AES_XCBC_VECTOR_COUNT,
        TM_CEREQUEST_CRYPTO_CMACHASH,
        {
            {
                (tt8BitPtr)TEST1_AES_XCBC_KEY,
                (tt8BitPtr)TEST1_AES_XCBC_INPUT,
                (tt8BitPtr)TEST1_AES_XCBC_RESULT,
                TEST1_AES_XCBC_KEY_LEN,
                TEST1_AES_XCBC_INPUT_LEN,
                TEST1_AES_XCBC_RESULT_LEN
            },
            {
                (tt8BitPtr)TEST2_AES_XCBC_KEY,
                (tt8BitPtr)TEST2_AES_XCBC_INPUT,
                (tt8BitPtr)TEST2_AES_XCBC_RESULT,
                TEST2_AES_XCBC_KEY_LEN,
                TEST2_AES_XCBC_INPUT_LEN,
                TEST2_AES_XCBC_RESULT_LEN
            },
            {
                (tt8BitPtr)TEST3_AES_XCBC_KEY,
                (tt8BitPtr)TEST3_AES_XCBC_INPUT,
                (tt8BitPtr)TEST3_AES_XCBC_RESULT,
                TEST3_AES_XCBC_KEY_LEN,
                TEST3_AES_XCBC_INPUT_LEN,
                TEST3_AES_XCBC_RESULT_LEN
            },
            {
                (tt8BitPtr)TEST4_AES_XCBC_KEY,
                (tt8BitPtr)TEST4_AES_XCBC_INPUT,
                (tt8BitPtr)TEST4_AES_XCBC_RESULT,
                TEST4_AES_XCBC_KEY_LEN,
                TEST4_AES_XCBC_INPUT_LEN,
                TEST4_AES_XCBC_RESULT_LEN
            },
            {
                (tt8BitPtr)TEST5_AES_XCBC_KEY,
                (tt8BitPtr)TEST5_AES_XCBC_INPUT,
                (tt8BitPtr)TEST5_AES_XCBC_RESULT,
                TEST5_AES_XCBC_KEY_LEN,
                TEST5_AES_XCBC_INPUT_LEN,
                TEST5_AES_XCBC_RESULT_LEN
            },
            {
                (tt8BitPtr)TEST6_AES_XCBC_KEY,
                (tt8BitPtr)TEST6_AES_XCBC_INPUT,
                (tt8BitPtr)TEST6_AES_XCBC_RESULT,
                TEST6_AES_XCBC_KEY_LEN,
                TEST6_AES_XCBC_INPUT_LEN,
                TEST6_AES_XCBC_RESULT_LEN
            },
            {
                (tt8BitPtr)TEST7_AES_XCBC_KEY,
                (tt8BitPtr)TEST7_AES_XCBC_INPUT,
                (tt8BitPtr)TEST7_AES_XCBC_RESULT,
                TEST7_AES_XCBC_KEY_LEN,
                TEST7_AES_XCBC_INPUT_LEN,
                TEST7_AES_XCBC_RESULT_LEN
            }
        }
    }
#endif /* (defined(TM_USE_AESXCBC) && defined(TM_USE_AES)) */
};
#endif /* TM_USE_SHA256||TM_USE_SHA384||TM_USE_SHA512||(AESXCBC && AES) */


/*
 * Test variable length sequence of "\x01\x02\x03 . . ."
 */
typedef struct tsCryptoSeqHash
{
    int             seqLength;
    ttCharPtr       seqResult;
} ttCryptoSeqHash;
typedef const ttCryptoSeqHash TM_FAR * ttCryptoSeqHashConstPtr;
typedef struct tsCryptoSeqHashAlg
{
    int                     cryptoAlg;
    ttCryptoSeqHashConstPtr resultVector;
} ttCryptoSeqHashAlg;

#ifdef TM_USE_MD5
static const ttCryptoSeqHash TM_CONST_QLF cryptoSeqTestMD5[] =
{
 {0, "\xd4\x1d\x8c\xd9\x8f\x00\xb2\x04\xe9\x80\x09\x98\xec\xf8\x42\x7e"},
 {1, "\x55\xa5\x40\x08\xad\x1b\xa5\x89\xaa\x21\x0d\x26\x29\xc1\xdf\x41"},
 {2, "\x0c\xb9\x88\xd0\x42\xa7\xf2\x8d\xd5\xfe\x2b\x55\xb3\xf5\xac\x7a"},
 {3, "\x52\x89\xdf\x73\x7d\xf5\x73\x26\xfc\xdd\x22\x59\x7a\xfb\x1f\xac"},
 {4, "\x08\xd6\xc0\x5a\x21\x51\x2a\x79\xa1\xdf\xeb\x9d\x2a\x8f\x26\x2f"},
 {5, "\x7c\xfd\xd0\x78\x89\xb3\x29\x5d\x6a\x55\x09\x14\xab\x35\xe0\x68"},
 {7, "\x49\x80\x01\x21\x7b\xc6\x32\xcb\x15\x85\x88\x22\x4d\x7d\x23\xc4"},
 {8, "\x0e\xe0\x64\x6c\x1c\x77\xd8\x13\x1c\xc8\xf4\xee\x65\xc7\x67\x3b"},
 {9, "\x85\x96\xc1\xaf\x55\xb1\x4b\x7b\x32\x01\x12\x94\x4f\xcb\x85\x36"},
 {15, "\xab\x38\x25\xd5\xae\xae\xc5\x92\x5b\x05\xd4\x4b\xeb\x7d\xdc\x7d"},
 {16, "\x19\x0c\x4c\x10\x57\x86\xa2\x12\x1d\x85\x01\x89\x39\x10\x8a\x6c"},
 {17, "\x51\x87\xe4\x84\x29\x02\x50\xb2\x20\xc6\x7d\x5c\x40\xea\xc7\x60"},
 {31, "\xd3\x89\x6a\x49\x78\xb9\x0f\x9c\x47\x03\xc0\x9c\x01\x05\x0a\xe7"},
 {32, "\x59\x85\x33\x1b\xce\xc9\x71\xb2\x62\x12\x2a\xa1\xca\x5a\xd4\x11"},
 {33, "\xd4\x16\x08\xa6\xb3\x70\x22\xca\x41\xd7\xb7\xd3\x2d\x8a\x0e\xac"},
 {63, "\xd4\x16\x76\xf6\x50\x9e\x0a\xde\x85\x9c\x00\x40\xea\x4b\xfd\xc0"},
 {64, "\x50\xe8\x45\xbf\xe6\xc7\x0f\xee\x66\x3a\xb5\x50\x0f\xd0\xc4\x11"},
 {65, "\x04\x48\x94\x24\x7a\xe4\xb4\xe8\x06\x48\xe4\x46\xfe\x9a\xf9\xcd"},
 {127, "\xca\x92\x7a\x25\x22\xf2\xe2\xd0\xc1\x50\xc1\xe1\xa8\x91\x39\xce"},
 {128, "\xa7\x1a\xfa\xca\x79\x1d\xf5\xaa\xad\xb0\x24\x73\xdc\xff\x25\xdd"},
 {129, "\x5d\x8c\x4c\xea\x03\xb2\x5b\x11\x3f\x0c\x62\xd2\x47\x95\x5a\xc8"},
 {255, "\x2a\x43\xf7\x9c\xeb\x44\x83\x1d\x96\xf6\xe4\x56\x83\x97\x44\xe4"},
 {256, "\x27\x5c\xc8\x0a\x75\xc5\x8f\x23\xfe\x58\x18\x0d\x79\x4a\x9e\x88"},
 {257, "\xc8\x55\x94\x39\x38\x9f\xc8\xe8\x97\x77\x8e\x7c\x20\xd4\x11\x0f"},
 {511, "\xb7\x5b\xf1\x83\x06\xa4\xa0\x48\xff\x27\x9e\x67\x35\x12\xee\xcf"},
 {512, "\x8a\x11\xa5\x55\xd0\x01\x36\xd6\xa7\x82\x17\x67\x99\xe8\xa8\xe3"},
 {513, "\x2d\x76\x69\xa0\xe1\x93\x53\xe0\x53\x51\x94\xe6\x3b\xa5\x31\x8a"},
 {767, "\xa5\x98\x82\xd4\xe4\xfd\xba\xe4\x78\x46\x0e\x94\x57\xe4\x6e\x8a"},
 {768, "\x36\xfc\x1d\xb6\x02\xcd\x3a\xa3\x5c\x98\x20\x6d\x1a\xe7\x40\xdb"},
 {769, "\x8c\x9f\x6f\x65\x0e\x02\xd0\xd8\xd8\xe0\xea\xc5\x22\xd0\xba\x32"},
 {1023, "\xb2\x4b\x11\x32\x74\xa4\x7c\x40\xac\xa1\x47\xc2\xba\xd4\xec\xb9"},
 {1024, "\xd8\x77\x7b\x6e\xb4\xc7\x1b\x0d\xa4\x37\x3d\x11\xe8\x51\x2c\x73"},
 {1025, "\x14\x3d\x84\xc4\x73\x2b\xcf\x1d\x1e\x6e\xef\x56\xc0\x1e\x1c\xc7"},
 {1279, "\x55\xb3\xec\xd1\x68\xd1\xff\xe2\x07\x65\x1a\x45\x25\x0a\xe1\x2f"},
 {1280, "\x36\x1f\xaf\xbd\x48\x30\x30\xc6\x60\x6e\x9e\xcd\x54\xbd\xeb\x7e"},
 {1281, "\x73\xfe\x16\x1b\xc1\xa9\x3e\xec\xbe\x5d\x48\x2e\xb4\x63\xf9\x34"},
 {1535, "\xad\x4d\xa4\xba\xcf\x03\xc3\xfd\x4a\xca\x42\x56\x16\xee\xa6\x95"},
 {1536, "\x6f\xfb\x5a\x25\x69\xbf\x29\xf9\x4b\xe9\x6c\xd8\xf0\xc3\x0b\xae"},
 {1537, "\x69\xd3\x85\x1b\x0d\x36\x95\x5e\xf4\x84\xee\x00\x32\x4c\x5f\x89"},
 {1791, "\xe1\x08\x63\x2f\xb6\x13\xd8\x02\xc0\x9d\x5a\x12\xf6\xbc\x33\x42"},
 {1792, "\x71\xc5\x0f\xb9\x5e\x8b\xf6\x61\x61\xd6\x61\x4d\xc4\x22\xba\x16"},
 {1793, "\xad\x84\x36\xc9\x3c\x21\x9e\xbe\x71\xeb\x06\x73\x75\xbb\xd9\x41"},
 {2047, "\xf7\xd7\x04\x61\x5f\x17\x44\x4f\xe8\x04\xbf\xaf\xe7\xae\x52\x35"},
 {2048, "\x3b\xf8\xe1\x1a\x66\xac\x22\x4d\x1d\x9d\x86\x65\xac\xe4\x1a\xdf"},
 {2049, "\x1f\x0f\x87\x65\x46\xc3\xdf\x9d\x92\x47\x4e\xf3\xf4\x82\x96\xc9"},
 {-1, (ttCharPtr)0}
};
#endif /* TM_USE_MD5 */


#ifdef TM_USE_SHA256
static const ttCryptoSeqHash TM_CONST_QLF cryptoSeqTestSHA256[] =
{
 {0, "\xe3\xb0\xc4\x42\x98\xfc\x1c\x14\x9a\xfb\xf4\xc8\x99\x6f\xb9\x24"
     "\x27\xae\x41\xe4\x64\x9b\x93\x4c\xa4\x95\x99\x1b\x78\x52\xb8\x55"},
 {1, "\x4b\xf5\x12\x2f\x34\x45\x54\xc5\x3b\xde\x2e\xbb\x8c\xd2\xb7\xe3"
     "\xd1\x60\x0a\xd6\x31\xc3\x85\xa5\xd7\xcc\xe2\x3c\x77\x85\x45\x9a"},
 {2, "\xa1\x28\x71\xfe\xe2\x10\xfb\x86\x19\x29\x1e\xae\xa1\x94\x58\x1c"
     "\xbd\x25\x31\xe4\xb2\x37\x59\xd2\x25\xf6\x80\x69\x23\xf6\x32\x22"},
 {3, "\x03\x90\x58\xc6\xf2\xc0\xcb\x49\x2c\x53\x3b\x0a\x4d\x14\xef\x77"
     "\xcc\x0f\x78\xab\xcc\xce\xd5\x28\x7d\x84\xa1\xa2\x01\x1c\xfb\x81"},
 {4, "\x9f\x64\xa7\x47\xe1\xb9\x7f\x13\x1f\xab\xb6\xb4\x47\x29\x6c\x9b"
     "\x6f\x02\x01\xe7\x9f\xb3\xc5\x35\x6e\x6c\x77\xe8\x9b\x6a\x80\x6a"},
 {5, "\x74\xf8\x1f\xe1\x67\xd9\x9b\x4c\xb4\x1d\x6d\x0c\xcd\xa8\x22\x78"
     "\xca\xee\x9f\x3e\x2f\x25\xd5\xe5\xa3\x93\x6f\xf3\xdc\xec\x60\xd0"},
 {7, "\x32\xbb\xe3\x78\xa2\x50\x91\x50\x2b\x2b\xaf\x9f\x72\x58\xc1\x94"
     "\x44\xe7\xa4\x3e\xe4\x59\x3b\x08\x03\x0a\xcd\x79\x0b\xd6\x6e\x6a"},
 {8, "\x66\x84\x0d\xda\x15\x4e\x8a\x11\x3c\x31\xdd\x0a\xd3\x2f\x7f\x3a"
     "\x36\x6a\x80\xe8\x13\x69\x79\xd8\xf5\xa1\x01\xd3\xd2\x9d\x6f\x72"},
 {9, "\x47\xe4\xee\x7f\x21\x1f\x73\x26\x5d\xd1\x76\x58\xf6\xe2\x1c\x13"
     "\x18\xbd\x6c\x81\xf3\x75\x98\xe2\x0a\x27\x56\x29\x95\x42\xef\xcf"},
 {15, "\x36\x03\x0f\xcc\x7e\x56\x62\x94\x90\x5b\x49\xa7\x20\xeb\x45\xbf"
      "\x96\x22\x09\xd2\xee\x1c\x9b\x73\xe2\xb7\xbc\x7a\xe8\x83\x03\x76"},
 {16, "\x5d\xfb\xab\xee\xdf\x31\x8b\xf3\x3c\x09\x27\xc4\x3d\x76\x30\xf5"
      "\x1b\x82\xf3\x51\x74\x03\x01\x35\x4f\xa3\xd7\xfc\x51\xf0\x13\x2e"},
 {17, "\xa9\xce\x21\x3e\x0c\x75\xb4\x32\xa9\xe5\xc1\x66\xe4\xf3\x8e\x66"
      "\x1e\xc9\x5a\x33\x70\xcd\x11\x1c\x72\x68\x8e\xcb\x4b\x50\x32\x05"},
 {31, "\x78\x1a\x4a\x36\x77\xd3\x93\x6a\xf4\x88\xff\x68\x62\xd8\x4c\x60"
      "\xf5\x2f\x19\x67\xae\x30\x7c\x23\x15\x77\xe8\x91\xb8\xb4\x0b\x9d"},
 {32, "\xae\x21\x6c\x2e\xf5\x24\x7a\x37\x82\xc1\x35\xef\xa2\x79\xa3\xe4"
      "\xcd\xc6\x10\x94\x27\x0f\x5d\x2b\xe5\x8c\x62\x04\xb7\xa6\x12\xc9"},
 {33, "\x31\xb0\x3c\x6e\xae\xd4\x75\xdd\xe3\x45\xb1\xce\x82\x93\xb9\xae"
      "\x8b\xfc\x7b\xd9\x66\x65\x97\xdd\xf2\x8c\x18\xfa\x73\xd5\xc4\xf4"},
 {63, "\x70\x18\xeb\xfc\x63\xac\xba\x9d\x5c\x72\xa3\xd4\x68\x17\x6c\xf6"
      "\x94\x4d\x46\xc6\x0d\x15\xbf\xbc\x92\x5a\x5c\xec\xb2\xe3\x6c\xf8"},
 {64, "\x20\xa7\xec\x84\x68\x4f\x7f\xe1\x24\xcb\x37\x27\xd0\x49\x73\x4a"
      "\xb0\xb7\xda\x2f\x52\xfc\xaf\xbc\xef\x98\x9e\xcf\xd9\x1e\x87\x0b"},
 {65, "\x47\xe9\xb7\x36\xf9\x00\x1f\xab\x14\x64\x0d\x9c\x91\x55\x74\xc2"
      "\x06\x69\x62\x15\x20\x37\x1f\x05\xa4\xb5\x09\x06\x02\x0d\xc4\x9f"},
 {127, "\x22\x70\xf8\xb0\xf1\x7a\x3a\xce\x93\xd4\x42\xd5\x04\xb0\xab\xb3"
       "\x24\xe6\xe5\xe8\xc2\x6b\x3e\xe5\xd9\xbe\x69\xf3\x3b\x77\xe8\x63"},
 {128, "\xba\xfa\x52\xe3\xc9\xb5\xc8\x72\x42\xb6\xbb\x2e\xcb\x23\x97\x66"
       "\x25\xfe\x46\x9a\xc1\x56\x70\xd2\x2f\xb0\x9b\x5b\xe1\x11\x11\x15"},
 {129, "\xb5\xba\xf2\x9e\xa8\xef\x57\xad\xe3\x6b\x68\xa4\x6f\x56\x64\x23"
       "\x28\xe9\x71\x24\x77\xb5\x8c\x4b\x5b\xa4\x25\x21\x08\x16\x80\xd8"},
 {255, "\x92\x93\x51\xec\x9c\x27\x20\x28\xc6\xc7\x0f\x92\xa3\x3c\x69\x05"
       "\x96\x39\xc1\xef\x81\xd7\xba\xea\x06\x50\x55\x2d\x39\x73\x02\x66"},
 {256, "\x9b\xc0\x38\xd0\xa0\xfb\x39\x1f\x3b\x33\x61\x8d\xcf\x08\xb6\x55"
       "\x35\x60\xef\x0a\xe0\xf7\xad\x55\x78\x71\x59\x8f\x27\xb7\x19\x4b"},
 {257, "\x08\x70\x5b\x9b\x7d\x84\x45\x68\x01\xb0\x2d\xf0\xdc\x58\x0a\x19"
       "\x84\x1b\x4f\x71\xdd\x3c\xdb\xbf\x35\xe8\x82\xe8\xc6\x8d\xab\xf5"},
 {511, "\x11\x43\xf1\xc4\xe9\xef\x9b\xe3\x44\x2c\x80\xfb\xa0\x33\x6b\xb0"
       "\x8e\x9b\x11\x8f\xb7\x63\x58\x3d\xe6\xb0\x6e\x5b\x49\x77\x45\x4a"},
 {512, "\x28\x39\x8f\xf0\x46\xbc\x53\x5a\x23\x7d\xe1\x95\x15\x52\x97\xbe"
       "\xfb\x04\x82\x72\x9a\xe8\x10\xc6\x23\x85\x64\xf4\x40\xbe\x76\xa1"},
 {513, "\xf2\xf4\x81\xdf\x23\x23\xea\x86\x34\xcd\xa6\xb1\xfb\x33\x19\xcc"
       "\x22\x39\xb5\x4b\xc2\x55\x20\x10\x7f\xde\x10\x0b\x16\x4a\x78\x4b"},
 {767, "\x89\xe1\x7d\xba\xba\x17\xf8\x3a\x2c\x62\xb0\x6d\xfa\xbc\x9c\x92"
       "\xbc\xdd\x74\xe9\x77\xa9\x5f\xf8\xfb\x9f\x61\x35\xdd\xa2\xc4\xaf"},
 {768, "\xb5\x4f\x80\x42\xb2\x60\xbd\x8b\xa5\x6b\xd6\xd6\x46\xa7\x76\x71"
       "\xa5\x7d\x23\x52\x38\x1c\xab\x3b\xed\x17\x9e\x3c\xfc\xaa\x4a\x6c"},
 {769, "\xde\xc0\xdf\xb4\xb4\xab\xd9\xa9\x51\x3c\xfa\x0c\x3c\xaf\xc7\x1f"
       "\x3b\xe5\x3f\xf5\x65\xae\xbb\x17\xa2\x9f\x99\xe1\xa4\x69\xff\x3d"},
 {1023, "\x8c\xa9\x75\x2a\x9c\x43\x2b\x7e\x28\x3f\x43\xd8\xc0\x34\x88\x5c"
        "\xd4\xca\x9a\x1c\x87\x53\x90\x88\xf3\xa9\x2c\x5a\x29\x2a\x68\x29"},
 {1024, "\xd6\x86\x33\x1c\x14\xfd\x88\xba\xd9\x63\x26\x44\x6e\x9d\x75\xba"
        "\xde\x98\xf8\x9c\xe1\xd2\x7b\xd0\xe0\xeb\xbb\xf4\x01\xeb\x6f\x6d"},
 {1025, "\x43\xda\x26\x2c\x94\x5a\x88\xec\x7c\x73\x5f\xe5\x3b\x11\x30\xac"
        "\xbe\xe8\xd5\x41\x27\xad\x65\xd3\x4d\x1d\x71\x3d\x5c\xed\xe7\xd7"},
 {1279, "\xd9\xee\x48\xd7\x2d\x30\x6e\xe9\xd1\xc0\x2d\xd2\x4a\x07\xab\xd6"
        "\x01\xce\x99\x9d\x32\x86\xa7\x36\x41\x1f\xd1\x08\x8c\x68\xec\xc1"},
 {1280, "\x9c\xbe\xb5\x50\xe3\x23\x30\xf2\x9d\x4a\x71\x00\x79\x02\x29\x31"
        "\xa6\x57\xaf\x54\x37\x9e\x6a\x2e\x79\x90\xf6\x91\x73\xa5\x88\x3c"},
 {1281, "\x9c\x48\xa9\xe8\x98\xa6\xee\xd4\x84\xd6\xa8\xed\x9c\x98\x04\x83"
        "\x14\x2f\x2f\x7c\x7e\x01\xf4\x68\x6a\x99\x69\x52\xef\xf6\x57\x9d"},
 {1535, "\x54\x5f\x8d\x2b\xa1\x38\xa9\x27\xfa\x1b\xca\xad\xb4\x7c\xcf\x41"
        "\x75\xa7\xd0\x37\xe3\x8e\x42\xd5\xac\xad\x8e\x2c\xb0\x1c\xf8\x88"},
 {1536, "\x54\xe5\xcc\x74\x73\xf7\x42\x21\x40\x8d\xb6\x83\x3f\xaf\x3e\xf1"
        "\x2d\x95\x4d\xc2\x13\xd0\x8a\x4f\xaa\xab\x47\xc3\x7b\xfe\xcb\x5c"},
 {1537, "\x57\x28\x41\x5b\x57\xb7\x55\xd7\xb5\x71\x87\x2c\x5c\x52\x0c\xc5"
        "\x86\xe1\xec\x68\x2f\x08\xb3\xa1\x36\x92\xc3\x86\x04\x63\xf4\x70"},
 {1791, "\xb8\x3c\xc5\x26\x8a\x8b\xc4\x9f\xf9\x67\xcb\x85\xda\x92\xd6\x3c"
        "\x02\x43\x45\x29\xf8\x55\x61\x5a\x98\x96\x0b\xaa\xf6\xb8\x1b\xfc"},
 {1792, "\x59\x67\xc1\x17\x3c\xd3\x7c\x11\x87\x53\xe2\x11\xb1\xdd\x1e\x57"
        "\x37\xbe\x18\xe8\x24\x79\x45\x69\xeb\x73\xbe\x92\x3c\x5e\x63\xed"},
 {1793, "\x44\xa3\x70\x5c\x89\x2c\x84\xe1\x1d\x8e\xe8\xbf\xb8\x76\x02\xe9"
        "\xa3\x1e\x4a\x2e\x95\x5d\x5c\x24\xcb\xdb\xfa\x6d\x57\x55\xcc\x5f"},
 {2047, "\x60\xef\x92\xaf\xc7\xb0\x5e\x71\x7f\xd8\x73\x21\x78\x76\x87\x73"
        "\xee\xac\xf7\x97\x99\x45\xb9\x70\x8b\x29\x81\x7d\xb8\xe8\xb4\xde"},
 {2048, "\x8f\x78\x6a\x07\x17\xae\x6c\x4d\x70\xb7\x8d\x00\x33\x00\xb3\xd3"
        "\x40\xfe\x84\xfb\x13\x1c\x5b\xe1\xef\x02\x8d\x0f\xdc\xbe\x7f\x6d"},
 {2049, "\x63\x18\x8a\x2f\xf9\x27\xd7\xc5\xf1\xfc\x98\xc5\xc7\xc6\xed\xdd"
        "\x6c\xfa\xb3\xb6\x68\x09\xcf\x73\x66\x56\x48\xe3\x11\x2c\xc9\xa3"},
 {-1, (ttCharPtr)0}
};
#endif /* TM_USE_SHA256 */

static const ttCryptoSeqHashAlg TM_CONST_QLF cryptoSeqTest[] =
{
#ifdef TM_USE_MD5
    {SADB_AALG_MD5HMAC, cryptoSeqTestMD5},
#endif /* TM_USE_MD5 */
#ifdef TM_USE_SHA256
    {SADB_AALG_SHA256HMAC, cryptoSeqTestSHA256},
#endif /* TM_USE_SHA256 */
    {SADB_AALG_NULL, (ttCryptoSeqHashConstPtr)0}
};

#if ( defined(TM_USE_SHA256) || defined(TM_USE_SHA384)|| \
      defined(TM_USE_SHA512) || \
     (defined(TM_USE_AESXCBC) && defined(TM_USE_AES))    )
int tfxCryptoHashTest()
{
    ttCryptoRequest             request;
    ttCryptoRequestHmacHash     hhashRequest;
    ttCryptoChainData           chainData;
    ttCryptoSeqHashConstPtr     seqVectorPtr;
    int                         i;
    int                         n;    
    int                         updateSize;
    int                         errorCode;
    tt8BitPtr                   bufferPtr;
    ttVoidPtr                   ctxPtr;

#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)
    tt8BitPtr                   bigArrPtr;    
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */

    ttAhAlgorithmPtr            algPtr;
    ttCryptoEnginePtr           engPtr;
    tt8Bit                      bigTest;
    tt8Bit                      failed;

#ifdef TM_USE_PARSER2
    errorCode = tfIke2Initialize("treck.conf", TM_PARSER_FILE, (ttUserInterface)0);
#endif /* TM_USE_PARSER2 */
    failed = TM_8BIT_NO;
    errorCode = TM_ENOERROR;
    bigTest = TM_8BIT_NO;
    (void)tfCryptoEngineInit();
    engPtr = tfCryptoEngineGetFirst();
/* void all pointers so we know how to cleanup at the end */
    bufferPtr =  (tt8BitPtr)0;
    algPtr = (ttAhAlgorithmPtr)0;
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)    
    bigArrPtr = (tt8BitPtr)0;
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */
    if(!engPtr)
    {
        printf("Could not allocate engPtr\n");
        errorCode = TM_ENOMEM;
        goto CRYPTOTEST_RETURN;
    }
/* do the hash functions first */
    for(i = 0; i < CRYPTO_HASH_ALG_COUNT; i++)
    {
        algPtr = tfAhAlgorithmLookup(cryptoHash[i].cryptoAlg);
        if (!algPtr)
        {           
            printf("Crypto engine does not support alg %d\n",
                cryptoHash[i].cryptoAlg);
            errorCode = TM_EPERM;
            goto CRYPTOTEST_RETURN;           
        }
/* This buffer needs to hold the output of the hash for both HMAC and normal
 * hash functions.
 */
        bufferPtr = (tt8BitPtr)tm_get_raw_buffer((algPtr->aalgDigestOutBits>>3));
        
        if (bufferPtr == (tt8BitPtr)0)
        {
            printf("Could not allocate buffer\n");
            errorCode = TM_ENOMEM;
            goto CRYPTOTEST_RETURN;
        }
        
        for(n = 0; n < cryptoHash[i].cryptoTestCount; n++)
        {
            bigTest = TM_8BIT_NO;
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)            
            if(!(tm_strncmp(cryptoHash[i].cryptoTest[n].testInput,
                            TEST3,
                            strlen(TEST3)+1)))
            {
                bigArrPtr = malloc(1000000);
                if (!bigArrPtr)
                {
                    goto CRYPTOTEST_RETURN;
                }
                tm_memset(bigArrPtr,'a',1000000);
                cryptoHash[i].cryptoTest[n].testInput = bigArrPtr;
                bigTest = TM_8BIT_YES;
            }
            else
            {
                bigArrPtr = (tt8Bit)0;
            }
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */
/* zero crypto request */
            tm_bzero(&request, sizeof(ttCryptoRequest));
/* request HMAC */
            request.crType = (ttUser8Bit)cryptoHash[i].requestType;
/* assign pointer */
            request.crParamUnion.crHmacHashParamPtr = &hhashRequest;
/* give the hashAlg pointer its appropriate params */
            hhashRequest.crhhAlgPtr = algPtr;
            hhashRequest.crhhHashAlg = (tt16Bit)cryptoHash[i].cryptoAlg;
            hhashRequest.crhhInDataPtr = &chainData;
            hhashRequest.crhhKeyLen = (tt16Bit)cryptoHash[i].cryptoTest[n].testKeyLen;
            hhashRequest.crhhKeyPtr = cryptoHash[i].cryptoTest[n].testKey;
            hhashRequest.crhhOutPtr = bufferPtr;
            chainData.ccdDataLen = 
                            cryptoHash[i].cryptoTest[n].testInputLen;
            chainData.ccdDataPtr = cryptoHash[i].cryptoTest[n].testInput;
            chainData.ccdNextPtr = (ttCryptoChainDataPtr)0;
            hhashRequest.crhhDataLength = 
                            cryptoHash[i].cryptoTest[n].testInputLen;
            (void)engPtr->ceSessionProcessFuncPtr(&request);

            if(!bigTest)
            {
#ifdef TM_CRYPTO_TEST_DEBUG
                tfxCryptoPrint(cryptoHash[i].cryptoTest[n].testInput,
                               cryptoHash[i].cryptoTest[n].testResult,
                               bufferPtr,
                               cryptoHash[i].cryptoTest[n].testInputLen,
                               cryptoHash[i].cryptoTest[n].testResultLen,
                               (tt8BitPtr)0);
#endif /* TM_CRYPTO_TEST_DEBUG */
            }
/* compare results to plaintext buffer's associated results static var */
            if (tm_memcmp(bufferPtr,
                          cryptoHash[i].cryptoTest[n].testResult,
                          cryptoHash[i].cryptoTest[n].testResultLen))
            {
                (void)tfKernelPrint(
                        TM_PRINT_TYPE_WARN,
                        "tfKernelWarning!  HashTest: "   
                        "FAILED hash test (alg %d iter #%d)\n\n\n",
                        cryptoHash[i].cryptoAlg, n+1);
                failed = TM_8BIT_YES;
            }
#ifdef TM_CRYPTO_TEST_DEBUG
            else
            {
                printf("  PASSED hash test (alg %d iter #%d)\n\n", 
                       cryptoHash[i].cryptoAlg, n+1);
            }
#endif /* TM_CRYPTO_TEST_DEBUG */
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)
            if (bigTest && bigArrPtr)
            {
                cryptoHash[i].cryptoTest[n].testInput = (tt8BitPtr)TEST3;
                free(bigArrPtr);
            }
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */
        }
        if (bufferPtr != (tt8BitPtr)0)
        {
            tm_free_raw_buffer(bufferPtr);
            bufferPtr = (tt8BitPtr)0;
        }
    }

/*
 * Test variable length sequence of "\x01\x02\x03 . . ."
 */
/* Determine the largest buffer requirement */
    n = -1;
    for (i = 0; cryptoSeqTest[i].cryptoAlg != SADB_AALG_NULL; ++i)
    {
        for (seqVectorPtr = cryptoSeqTest[i].resultVector;
             seqVectorPtr->seqLength >= 0;
             ++seqVectorPtr)
        {
            if (seqVectorPtr->seqLength > n)
            {
                n = seqVectorPtr->seqLength;
            }
        }
    }
    if (n >= 0)
    {
/* Allocate and fill the data/result buffer */
        bufferPtr = (tt8BitPtr)tm_get_raw_buffer(n + TM_MAX_HASH_SIZE);
        tm_assert(tfxCryptoHashTest, bufferPtr != TM_8BIT_NULL_PTR);
        for (i = 1; i <= n; ++i)
        {
            bufferPtr[i + TM_MAX_HASH_SIZE - 1] = (tt8Bit)i;
        }
/* Run each algoritm test */
        for (i = 0; cryptoSeqTest[i].cryptoAlg != SADB_AALG_NULL; ++i)
        {
/* Algorithm specifics for this test */
            algPtr = tfAhAlgorithmLookup(cryptoSeqTest[i].cryptoAlg);
            tm_assert(tfxCryptoHashTest, algPtr != (ttAhAlgorithmPtr)0);
/* Buffer for algorithm context */
            ctxPtr = tm_get_raw_buffer((*algPtr->aalgContextLenFuncPtr)());
            tm_assert(tfxCryptoHashTest, ctxPtr != TM_VOID_NULL_PTR);
/* Test various size data buffers */
            for (seqVectorPtr = cryptoSeqTest[i].resultVector;
                 seqVectorPtr->seqLength >= 0;
                 ++seqVectorPtr)
            {
/* Digest the data in various sized pieces. Note: data length could be 0, in
 * which case we just want to initialize and finalize. */
                for (updateSize = seqVectorPtr->seqLength + 1;
                     updateSize > 0;
                     updateSize -= 37)
                {
/* Initialize the context */
                    (*algPtr->aalgInitFuncPtr)(ctxPtr);
                    for (n = 0; ; n += updateSize)
                    {
                        if (n + updateSize < seqVectorPtr->seqLength)
                        {
/* Next piece of data */
                            (*algPtr->aalgUpdateFuncPtr)(
                                            ctxPtr,
                                            bufferPtr + TM_MAX_HASH_SIZE + n,
                                            updateSize,
                                            0);
                        }
                        else
                        {
/* Last piece of data */
                            if (n < seqVectorPtr->seqLength)
                            {
                                (*algPtr->aalgUpdateFuncPtr)(
                                            ctxPtr,
                                            bufferPtr + TM_MAX_HASH_SIZE + n,
                                            seqVectorPtr->seqLength - n,
                                            0);
                            }
                            break;
                        }
                    }
/* Finalize the context */
                    (*algPtr->aalgFinalFuncPtr)(bufferPtr, ctxPtr);
/* Check the result */
                    n = (int)(algPtr->aalgDigestOutBits / 8);
                    n = tm_memcmp(bufferPtr, seqVectorPtr->seqResult, n);
                    tm_assert(tfxCryptoHashTest, n == 0);
                }
            }
/* Deallocate the context buffer */
            tm_free_raw_buffer(ctxPtr);
        }
/* Deallocate the data buffer */
        tm_free_raw_buffer(bufferPtr);
        bufferPtr = TM_8BIT_NULL_PTR;
    }

CRYPTOTEST_RETURN:
#if (defined(TM_KERNEL_VISUAL_X86) || defined(TM_KERNEL_WIN32_X86)) \
    && defined(CRYPTO_USE_BIG_TEST)
    if (bigTest && bigArrPtr)
    {        
        tm_free_raw_buffer(bigArrPtr);
    }
#endif /* (TM_KERNEL_VISUAL_X86||TM_KERNEL_WIN32_X86)&&CRYPTO_USE_BIG_TEST */
    if (bufferPtr != (tt8BitPtr)0)
    {
        tm_free_raw_buffer(bufferPtr);
    }
    if (failed)
    {
        errorCode = -1;
    }
    tm_assert(tfxCryptoHashTest, failed == TM_8BIT_NO);
    return errorCode;
}
#endif /* TM_USE_SHA256||TM_USE_SHA384||TM_USE_SHA512||(AESXCBC && AES) */


#ifdef TM_CRYPTO_TEST_DEBUG
void tfxCryptoPrint(tt8BitPtr inputPtr,
                    tt8BitPtr resultPtr,
                    tt8BitPtr hashResultPtr,
                    int       inputLen,
                    int       hashLen,
                    tt8BitPtr decryptBuffPtr)
{
    int i;

    printf("    Input:\n      ");
    for(i = 0; i < inputLen; i++)
    {
        if ((i) && ((i % 16) == 0))
        {
            printf(" \\\n      ");
        }
        printf("%02x ", inputPtr[i]);
    }

    printf("\n");

    printf("    Expected:\n      ");
    for(i = 0; i < hashLen; i++)
    {
        if ((i) && ((i % 16) == 0))
        {
            printf(" \\\n      ");
        }
        printf("%02x ", resultPtr[i]);
    }
    printf("\n");

    printf("    Actual:\n      ");
    for(i = 0; i < hashLen; i++)
    {
        if ((i) && ((i % 16) == 0))
        {
            printf(" \\\n      ");
        }
        printf("%02x ", hashResultPtr[i]);
    }
    printf("\n");
    
    if (decryptBuffPtr != (tt8BitPtr)0)
    {
        printf("    Decrypted:\n      ");
        for(i = 0; i < hashLen; i++)
        {
            if ((i) && ((i % 16) == 0))
            {
                printf(" \\\n      ");
            }
            printf("%02x ", decryptBuffPtr[i]);
        }
        printf("\n");
    }
}
#endif /* TM_CRYPTO_TEST_DEBUG */
#else /* !TM_USE_CRYPTO_TEST */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

int tlTxCryptoDummy = 0;
#endif /* TM_USE_CRYPTO_TEST */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tucrypto.c    File Revision: 6.0.2.4 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 19296    Output checksum: 19683
 */
