/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: testing the default gateway API
 * Filename: tudefgw.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002
 * $Source: test/unittest/tudefgw.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:24CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h> /* For LINT_UNUSED_HEADER && testing internal API */

#if defined(TM_MULTIPLE_CONTEXT) && defined(TM_USE_IPV4)
/* The following 3 include's are for testing the internal API */
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"

void tfxDefaultGw(void)
{
    int                         errorCode;
    ttUserContext               oldContext;
struct  sockaddr_storage        sock4Addr;

    oldContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleClient);

/* Add a reachable default gateway  */
    errorCode = tfAddDefaultGateway(
                    tvInterfaceHandleClient, 
                    tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr + 0x01000000
#ifdef TM_USE_STRONG_ESL
                  , 1
#endif /* TM_USE_STRONG_ESL */
                    );
    tm_assert(tfxDefaultGw, errorCode == TM_ENOERROR);

    errorCode = tfRtDestExists(
#ifdef TM_USE_STRONG_ESL
                    tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                    inet_addr("60.10.10.10"),
                    inet_addr("255.255.255.0")
                    );
    tm_assert(tfxDefaultGw, errorCode == TM_ENOERROR);
    sock4Addr.ss_family = AF_INET;
    sock4Addr.ss_len = sizeof(struct sockaddr_in);
    errorCode = inet_pton(AF_INET,
                          "60.10.10.10",
                          &sock4Addr.addr.ipv4.sin_addr
                             );
    tm_assert(tfxDefaultGw, errorCode == 1);

/* delete the reachable default gateway  */
    errorCode = tfDelDefaultGateway(
#ifdef TM_USE_STRONG_ESL
                    tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                    tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr + 0x01000000);
    tm_assert(tfxDefaultGw, errorCode == TM_ENOERROR);

    errorCode = tfRtDestExists(
#ifdef TM_USE_STRONG_ESL
                    tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                    inet_addr("60.10.10.10"),
                    inet_addr("255.255.255.0")
                    );
    tm_assert(tfxDefaultGw, errorCode != TM_ENOERROR);
    sock4Addr.ss_family = AF_INET;
    sock4Addr.ss_len = sizeof(struct sockaddr_in);
    errorCode = inet_pton(AF_INET,
                          "60.10.10.10",
                          &sock4Addr.addr.ipv4.sin_addr
                             );
    tm_assert(tfxDefaultGw, errorCode == 1);

/* delete the reachable default gateway again  */
    errorCode = tfDelDefaultGateway(
#ifdef TM_USE_STRONG_ESL
                    tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                    tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr + 0x01000000);
    tm_assert(tfxDefaultGw, errorCode == TM_ENOENT);


/* Add an unreachable default gateway  */
    errorCode = tfAddDefaultGateway(
                    tvInterfaceHandleClient, 
                    tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr + 0x1
#ifdef TM_USE_STRONG_ESL
                  , 1
#endif /* TM_USE_STRONG_ESL */
                    );
    tm_assert(tfxDefaultGw, errorCode == TM_EHOSTUNREACH);

/* delete the unreachable default gateway  */
    errorCode = tfDelDefaultGateway(
#ifdef TM_USE_STRONG_ESL
                    tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                    tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr + 0x1);
    tm_assert(tfxDefaultGw, errorCode == TM_ENOENT);

/* delete the unreachable default gateway  again */
    errorCode = tfDelDefaultGateway(
#ifdef TM_USE_STRONG_ESL
                    tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                    tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr + 0x1);
    tm_assert(tfxDefaultGw, errorCode == TM_ENOENT);

    tfSetCurrentContext(oldContext);

}

#else /* !defined(TM_MULTIPLE_CONTEXT) || !defined(TM_USE_IPV4) */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
;
#endif /* !defined(TM_MULTIPLE_CONTEXT) || !defined(TM_USE_IPV4) */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tudefgw.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 56168    Output checksum:  3814
 */
