/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Test device functions
 *
 * Filename: tudevice.c
 * Author: Ed Remmell
 * Date Created:
 * $Source: test/unittest/tudevice.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:25CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>

#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

/* for the automated testing */
#include "tutest.h"


LINT_UNUSED_HEADER

/* To allow link for builds when TM_USE_IPV6 is not defined */
int tv6TxDeviceDummy = 0;


#ifdef TM_MULTIPLE_CONTEXT

void tfx4ConfigUnconfig(void)
{
    int errorCode;
    ttUserContext OldContext;

    OldContext = tfGetCurrentContext(); 
    tfSetCurrentContext(tvContextHandleServer); 

/* test zero IP address */
    errorCode = tfOpenInterface(
                    tvInterfaceHandleServer,
                    inet_addr("0.0.0.0"),
                    inet_addr("255.255.255.0"),
                    TM_DEV_MCAST_ENB,   /* flag */
                    1);
    tm_assert(tfx4ConfigUnconfig, errorCode == TM_EINVAL);

#ifndef TM_SINGLE_INTERFACE_HOME
/* Test if proper error code is returned when calling for an interface 
   that was not previously configured */
    errorCode = tfUnConfigInterface (tvInterfaceHandleServer, 100);
    tm_assert(tfx4ConfigUnconfig, errorCode == TM_ENOENT);
#endif /* !TM_SINGLE_INTERFACE_HOME */

    tfSetCurrentContext(OldContext);
}


char configBootBuffer[100];
/* 
 * Test sending and receiving unicast packets with partially configured
 * interfaces.
 */
void tfx4ConfigBootUnconfig(void )
{
    ttUserContext       OldContext;
    ttUserInterface     cliInterfaceHandle;
    ttUserInterface     srvInterfaceHandle;
    struct sockaddr_in  cliAddr;
    struct sockaddr_in  srvAddr;
    struct sockaddr_in  rcliAddr;
    struct sockaddr_in  rsrvAddr;
    unsigned int        cliMhomeIndex;
    unsigned int        srvMhomeIndex;
    ttCharPtr           cliIpAddrStr;
    ttCharPtr           srvIpAddrStr;
#ifdef TM_IGMP
    ttCharPtr           msrvIpAddrStr;
#endif /* TM_IGMP */
    int                 addrLen;
    int                 phyAddrLen;
    int                 cliSd;
    int                 srvSd;
    int                 cliPort;
    int                 srvPort;
    int                 sendLen;
    int                 errorCode;
    int                 retCode;
    tt8Bit              cliPhyAddr[TM_ETHER_ADDRESS_LENGTH];
    tt8Bit              srvPhyAddr[TM_ETHER_ADDRESS_LENGTH];
    tt8Bit              rcliPhyAddr[TM_ETHER_ADDRESS_LENGTH];
#ifndef TM_TEST_NULL_LINK_LAYER
    tt8Bit              rsrvPhyAddr[TM_ETHER_ADDRESS_LENGTH];
#endif /* !TM_TEST_NULL_LINK_LAYER */

/* SAVE CONTEXT */
    OldContext = tfGetCurrentContext(); 

/* Configuration initialization */
#ifndef TM_SINGLE_INTERFACE_HOME
    cliMhomeIndex = 1;
    srvMhomeIndex = 2;
#else /* TM_SINGLE_INTERFACE_HOME */
    cliMhomeIndex = 0;
    srvMhomeIndex = 0;
#endif /* !TM_SINGLE_INTERFACE_HOME */
    cliPort = 10;
    srvPort = 20;

/* Test using unicast addresses */
    cliIpAddrStr = "192.168.0.2";
    srvIpAddrStr = "192.168.0.3";
#ifdef TM_IGMP
/* Test using multicast addresses */
    msrvIpAddrStr = "224.0.0.1";
#endif /* TM_IGMP */


/* SERVER */
    tfSetCurrentContext(tvContextHandleServer); 
    srvInterfaceHandle = tfxGetInterfaceByContext(tfGetCurrentContext());
/* zero IP address with TM_DEV_IP_USER_BOOT */
    errorCode = tfConfigInterface(
                    srvInterfaceHandle,
                    inet_addr("0.0.0.0"),
                    inet_addr("0.0.0.0"),
                    TM_DEV_MCAST_ENB | TM_DEV_IP_USER_BOOT,   /* flag */
                    1, (unsigned char)srvMhomeIndex);
    tm_assert(tfx4ConfigUnconfig, errorCode == TM_ENOERROR);

/* Get Server Ethernet address */
    phyAddrLen = sizeof(ttEnetAddress);
    (void)tfIntfMCtxDrvGetPhyAddr(srvInterfaceHandle, (ttCharPtr)srvPhyAddr);

/* Open a UDP server socket */
    srvSd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4ConfigBootUnconfig, srvSd != TM_SOCKET_ERROR);
/* Bind it to any addr, server port(20) */
    srvAddr.sin_family = AF_INET;
    srvAddr.sin_port = htons(srvPort); 
    srvAddr.sin_addr.s_addr = TM_UL(0);
    addrLen = sizeof(struct sockaddr_in);
    retCode = bind(srvSd, (const struct sockaddr TM_FAR *)&srvAddr, addrLen);
    tm_assert(tfx4ConfigBootUnconfig, retCode != TM_SOCKET_ERROR);

/* CLIENT */
    tfSetCurrentContext(tvContextHandleClient); 
    cliInterfaceHandle = tfxGetInterfaceByContext(tfGetCurrentContext());
/* zero IP address with TM_DEV_IP_USER_BOOT */
    errorCode = tfConfigInterface(
                    cliInterfaceHandle,
                    inet_addr("0.0.0.0"),
                    inet_addr("0.0.0.0"),
                    TM_DEV_MCAST_ENB | TM_DEV_IP_USER_BOOT,   /* flag */
                    1, (unsigned char)cliMhomeIndex);
    tm_assert(tfx4ConfigBootUnconfig, errorCode == TM_ENOERROR);

/* Get Client Ethernet address */
    phyAddrLen = sizeof(ttEnetAddress);
    (void)tfIntfMCtxDrvGetPhyAddr(cliInterfaceHandle, (ttCharPtr)cliPhyAddr);

/* Open a UDP client socket */
    cliSd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4ConfigBootUnconfig, cliSd != TM_SOCKET_ERROR);
/* Bind it to any addr, client port(10) */
    cliAddr.sin_family = AF_INET;
    cliAddr.sin_port = htons(cliPort); 
    cliAddr.sin_addr.s_addr = TM_UL(0);
    addrLen = sizeof(struct sockaddr_in);
    retCode = bind(cliSd, (const struct sockaddr TM_FAR *)&cliAddr, addrLen);
    tm_assert(tfx4ConfigBootUnconfig, retCode != TM_SOCKET_ERROR);

/*
 * Testing unicast sending and receiving with no IP address configured.
 */
/* CLIENT */
    tfSetCurrentContext(tvContextHandleClient); 
/* Send from client socket to server socket */
    sendLen = sizeof(configBootBuffer);
    cliAddr.sin_family = AF_INET;
    cliAddr.sin_port = htons(cliPort); 
    cliAddr.sin_addr.s_addr = inet_addr(cliIpAddrStr);
    srvAddr.sin_family = AF_INET;
    srvAddr.sin_port = htons(srvPort);
    srvAddr.sin_addr.s_addr = inet_addr(srvIpAddrStr);
    retCode = tfSendToFromInterface(cliSd,
                                    configBootBuffer,
                                    sendLen,
                                    0,
    (const struct sockaddr TM_FAR *)&srvAddr,
                                    sizeof(struct sockaddr_in),
    (const struct sockaddr TM_FAR *)&cliAddr,
                   (const tt8BitPtr)srvPhyAddr,
                                    phyAddrLen,
                                    cliInterfaceHandle, cliMhomeIndex);
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);

/* SERVER */
    tfSetCurrentContext(tvContextHandleServer); 
/* recv on the server */
    retCode = tfRecvFromToInterface(srvSd,
                                    configBootBuffer,
                                    sendLen,
                                    0,
          (struct sockaddr TM_FAR *)&rcliAddr,
                                    &addrLen,
          (struct sockaddr TM_FAR *)&rsrvAddr,
                                    rcliPhyAddr,
                                    &phyAddrLen,
                                    &srvInterfaceHandle );
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);
#ifndef TM_TEST_NULL_LINK_LAYER
    tm_assert(tfx4ConfigBootUnconfig, phyAddrLen == sizeof(ttEnetAddress));
    retCode = tm_memcmp(rcliPhyAddr, cliPhyAddr, phyAddrLen);
    tm_assert(tfx4ConfigBootUnconfig, retCode == 0);
#endif /* TM_TEST_NULL_LINK_LAYER */
    tm_assert(tfx4ConfigBootUnconfig, rcliAddr.sin_port == cliAddr.sin_port);
#ifndef TM_TEST_NULL_LINK_LAYER
    tm_assert(tfx4ConfigBootUnconfig,
              rcliAddr.sin_addr.s_addr == cliAddr.sin_addr.s_addr);
    tm_assert(tfx4ConfigBootUnconfig, rsrvAddr.sin_port == srvAddr.sin_port);
#endif /* TM_TEST_NULL_LINK_LAYER */
    tm_assert(tfx4ConfigBootUnconfig,
              rsrvAddr.sin_addr.s_addr == srvAddr.sin_addr.s_addr);

/* Send from server to client */
    retCode = tfSendToFromInterface(srvSd, configBootBuffer, sendLen, 0,
    (const struct sockaddr TM_FAR *)&rcliAddr,
                                    sizeof(struct sockaddr_in),
    (const struct sockaddr TM_FAR *)&rsrvAddr,
                   (const tt8BitPtr)rcliPhyAddr,
                                    phyAddrLen,
                                    srvInterfaceHandle, srvMhomeIndex);
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);
/*
 * Send a second small packet to test queueing a second small packet in the
 * UDP socket recv queue which causes the stack to copy the data into a new
 * packet. We want to make sure that all info that we need is being copied
 * into the new packet.
 */
    retCode = tfSendToFromInterface(srvSd, configBootBuffer, sendLen, 0,
    (const struct sockaddr TM_FAR *)&rcliAddr,
                                    sizeof(struct sockaddr_in),
    (const struct sockaddr TM_FAR *)&rsrvAddr,
#ifdef TM_TEST_NULL_LINK_LAYER
                   (const tt8BitPtr)0,
                                    0,
#else /* !TM_TEST_NULL_LINK_LAYER */
                   (const tt8BitPtr)rcliPhyAddr,
                                    phyAddrLen,
#endif /* !TM_TEST_NULL_LINK_LAYER */
                                    srvInterfaceHandle, srvMhomeIndex);
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);

/* CLIENT */
    tfSetCurrentContext(tvContextHandleClient); 
/* Recv first packet on the client */
    retCode = tfRecvFromToInterface(cliSd,
                                    configBootBuffer,
                                    sendLen,
                                    0,
          (struct sockaddr TM_FAR *)&rsrvAddr,
                                    &addrLen,
          (struct sockaddr TM_FAR *)&rcliAddr,
#ifdef TM_TEST_NULL_LINK_LAYER
                   (const tt8BitPtr)0,
                         (ttIntPtr)0,
#else /* !TM_TEST_NULL_LINK_LAYER */
                                    rsrvPhyAddr,
                                    &phyAddrLen,
#endif /* !TM_TEST_NULL_LINK_LAYER */
                                    &srvInterfaceHandle );
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);
#ifndef TM_TEST_NULL_LINK_LAYER
    tm_assert(tfx4ConfigBootUnconfig, phyAddrLen == sizeof(ttEnetAddress));
    retCode = tm_memcmp(rsrvPhyAddr, srvPhyAddr, phyAddrLen);
    tm_assert(tfx4ConfigBootUnconfig, retCode == 0);
#endif /* TM_TEST_NULL_LINK_LAYER */
    tm_assert(tfx4ConfigBootUnconfig, rcliAddr.sin_port == cliAddr.sin_port);
    tm_assert(tfx4ConfigBootUnconfig,
              rcliAddr.sin_addr.s_addr == cliAddr.sin_addr.s_addr);
    tm_assert(tfx4ConfigBootUnconfig, rsrvAddr.sin_port == srvAddr.sin_port);
    tm_assert(tfx4ConfigBootUnconfig,
              rsrvAddr.sin_addr.s_addr == srvAddr.sin_addr.s_addr);
/* Recv second packet on the client */
    retCode = tfRecvFromToInterface(cliSd,
                                    configBootBuffer,
                                    sendLen,
                                    0,
          (struct sockaddr TM_FAR *)&rsrvAddr,
                                    &addrLen,
          (struct sockaddr TM_FAR *)&rcliAddr,
#ifdef TM_TEST_NULL_LINK_LAYER
                   (const tt8BitPtr)0,
                         (ttIntPtr)0,
#else /* !TM_TEST_NULL_LINK_LAYER */
                                    rsrvPhyAddr,
                                    &phyAddrLen,
#endif /* TM_TEST_NULL_LINK_LAYER */
                                    &srvInterfaceHandle );
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);
#ifndef TM_TEST_NULL_LINK_LAYER
    tm_assert(tfx4ConfigBootUnconfig, phyAddrLen == sizeof(ttEnetAddress));
    retCode = tm_memcmp(rsrvPhyAddr, srvPhyAddr, phyAddrLen);
    tm_assert(tfx4ConfigBootUnconfig, retCode == 0);
#endif /* TM_TEST_NULL_LINK_LAYER */
    tm_assert(tfx4ConfigBootUnconfig, rcliAddr.sin_port == cliAddr.sin_port);
    tm_assert(tfx4ConfigBootUnconfig,
              rcliAddr.sin_addr.s_addr == cliAddr.sin_addr.s_addr);
    tm_assert(tfx4ConfigBootUnconfig, rsrvAddr.sin_port == srvAddr.sin_port);
    tm_assert(tfx4ConfigBootUnconfig,
              rsrvAddr.sin_addr.s_addr == srvAddr.sin_addr.s_addr);

#ifdef TM_IGMP
/*
 * Testing multicast sending and receiving with no IP address configured.
 */
/* CLIENT */
    tfSetCurrentContext(tvContextHandleClient); 
    sendLen = sizeof(configBootBuffer);
    cliAddr.sin_family = AF_INET;
    cliAddr.sin_port = htons(cliPort); 
    cliAddr.sin_addr.s_addr = inet_addr(cliIpAddrStr);
    srvAddr.sin_family = AF_INET;
    srvAddr.sin_port = htons(srvPort);
    srvAddr.sin_addr.s_addr = inet_addr(msrvIpAddrStr);
    retCode = tfSendToFromInterface(cliSd,
                                    configBootBuffer,
                                    sendLen,
                                    0,
    (const struct sockaddr TM_FAR *)&srvAddr,
                                    sizeof(struct sockaddr_in),
    (const struct sockaddr TM_FAR *)&cliAddr,
                   (const tt8BitPtr)srvPhyAddr,
                                    phyAddrLen,
                                    cliInterfaceHandle, cliMhomeIndex);
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);

/* SERVER */
    tfSetCurrentContext(tvContextHandleServer); 
/* recv on the server */
    retCode = tfRecvFromToInterface(srvSd,
                                    configBootBuffer,
                                    sendLen,
                                    0,
          (struct sockaddr TM_FAR *)&rcliAddr,
                                    &addrLen,
          (struct sockaddr TM_FAR *)&rsrvAddr,
#ifdef TM_TEST_NULL_LINK_LAYER
                   (const tt8BitPtr)0,
                          (ttIntPtr)0,
#else /* !TM_TEST_NULL_LINK_LAYER */
                                    rcliPhyAddr,
                                    &phyAddrLen,
#endif /* !TM_TEST_NULL_LINK_LAYER */
                                    &srvInterfaceHandle );
    tm_assert(tfx4ConfigBootUnconfig, retCode == sendLen);
#ifndef TM_TEST_NULL_LINK_LAYER
    tm_assert(tfx4ConfigBootUnconfig, phyAddrLen == sizeof(ttEnetAddress));
    retCode = tm_memcmp(rcliPhyAddr, cliPhyAddr, phyAddrLen);
    tm_assert(tfx4ConfigBootUnconfig, retCode == 0);
#endif /* TM_TEST_NULL_LINK_LAYER */
    tm_assert(tfx4ConfigBootUnconfig, rcliAddr.sin_port == cliAddr.sin_port);
    tm_assert(tfx4ConfigBootUnconfig,
              rcliAddr.sin_addr.s_addr == cliAddr.sin_addr.s_addr);
    tm_assert(tfx4ConfigBootUnconfig, rsrvAddr.sin_port == srvAddr.sin_port);
    tm_assert(tfx4ConfigBootUnconfig,
              rsrvAddr.sin_addr.s_addr == srvAddr.sin_addr.s_addr);
#endif /* TM_IGMP */

/* CLIENT */
    tfSetCurrentContext(tvContextHandleClient); 
/* Close client socket */
    tfClose(cliSd);
/* Unconfigure interface */
    errorCode = tfUnConfigInterface (tvInterfaceHandleClient,
                                    (unsigned char)cliMhomeIndex);
    tm_assert(tfx4ConfigUnconfig, errorCode == TM_ENOERROR);

/* SERVER */
    tfSetCurrentContext(tvContextHandleServer); 
/* Close server socket */
    tfClose(srvSd);
/* Unconfigure interface */
    errorCode = tfUnConfigInterface (tvInterfaceHandleServer,
                                    (unsigned char)srvMhomeIndex);
    tm_assert(tfx4ConfigUnconfig, errorCode == TM_ENOERROR);

/* RESTORE CONTEXT */
    tfSetCurrentContext(OldContext);
}

#endif /*TM_MULTIPLE_CONTEXT*/

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tudevice.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  5225    Output checksum:  9154
 */
