/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: TCP/UDP v4/V6 echo server and client for testing 
 * @REQUIREMENT TRACIBILTY [IPV6REQ] section2.18
 * Filename: tuecho.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002
 * $Source: test/unittest/tuecho.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:28CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>

#ifdef TM_MULTIPLE_CONTEXT

/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"

void tfxUdpEchoServer(struct sockaddr TM_FAR * sockAddrPtr)
{
    int           errorCode;
    int           serverSd;
    int           sentLen;
    int           recvLen;
    char          buffer[TM_TEST_BUFFER_SIZE];
    struct sockaddr_storage fromAddr;
    int           fromLength;

    tfSetCurrentContext(tvContextHandleServer);

    serverSd = socket(sockAddrPtr->sa_family, SOCK_DGRAM, IP_PROTOUDP);
    if(serverSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxUdpEchoServer, serverSd != TM_SOCKET_ERROR); 

    errorCode = tfBlockingState(serverSd, TM_BLOCKING_ON);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxUdpEchoServer, errorCode != TM_SOCKET_ERROR); 

    errorCode = bind(serverSd,
                     sockAddrPtr,
                     sockAddrPtr->sa_len);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxUdpEchoServer, errorCode == TM_ENOERROR); 

    for(;;)
    {
	    fromLength = (sizeof(fromAddr));
        recvLen = recvfrom(serverSd,
                           buffer,
                           TM_TEST_BUFFER_SIZE,
                           0, /* flag */
                           (struct sockaddr *)&fromAddr,
                            &fromLength);
        if(errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(serverSd);
        }
        tm_assert(tfxUdpEchoServer, recvLen != TM_SOCKET_ERROR); 

        sentLen = sendto(serverSd,
                         buffer,
                         recvLen,
                         0,
                         (const struct sockaddr*)&fromAddr,
                         fromLength
                       );
        if(errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(serverSd);
        }
        tm_assert(tfxUdpEchoServer, sentLen == recvLen); 
    }

}



void tfxTcpEchoServer(struct sockaddr TM_FAR * sockAddrPtr)
{
    int           errorCode;
    int           serverSd;
    int           acceptSd;
    int           sentLen;
    int           recvLen;
    char          buffer[TM_TEST_BUFFER_SIZE];

    struct linger   lingerStruct;
    lingerStruct.l_onoff = 1;
    lingerStruct.l_linger= 0;

    tfSetCurrentContext(tvContextHandleServer);

    serverSd = socket(sockAddrPtr->sa_family, SOCK_STREAM, IP_PROTOTCP);
    if(serverSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxTcpEchoServer, serverSd != TM_SOCKET_ERROR); 

    setsockopt(serverSd,
               SOL_SOCKET,
               SO_LINGER,
               (char*)&lingerStruct,
               sizeof(lingerStruct));

    errorCode = tfBlockingState(serverSd, TM_BLOCKING_ON);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxTcpEchoServer, errorCode != TM_SOCKET_ERROR); 

    errorCode = bind(serverSd,
                     sockAddrPtr,
                     sockAddrPtr->sa_len);
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxTcpEchoServer, errorCode == TM_ENOERROR); 

    errorCode = listen(serverSd, 1);
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxTcpEchoServer, errorCode != TM_SOCKET_ERROR);

    for(;;)
    {
        acceptSd = accept( serverSd,
                          (struct sockaddr TM_FAR *)0,
                          (int TM_FAR             *)0
                         );
        if(acceptSd == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(serverSd);
        }
        tm_assert(tfxTcpEchoServer, acceptSd != TM_SOCKET_ERROR);

        errorCode = tfBlockingState(acceptSd, TM_BLOCKING_ON);
        if (errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(serverSd);
        }
        tm_assert(tfxTcpEchoServer, errorCode != TM_SOCKET_ERROR); 

        for(;;)
        {
            recvLen = recv(acceptSd, buffer, TM_TEST_BUFFER_SIZE, 0);
            if (recvLen == TM_SOCKET_ERROR )
            {
                errorCode = tfGetSocketError(acceptSd);
            }
            tm_assert(tfxTcpEchoServer, recvLen != TM_SOCKET_ERROR);

            if (recvLen == 0)
            {
/* end of file */
                break;
            }
            else
            {
                sentLen = send(acceptSd,
                               buffer,
                               recvLen,
                               0);
                if(sentLen != recvLen)
                {
                    errorCode = tfGetSocketError(acceptSd);
                }
                tm_assert(tfxTcpEchoServer, sentLen == recvLen);
            }
        }
        tfClose(acceptSd);
    }
}



void tfxUdpEchoClient(struct sockaddr TM_FAR * sockAddrPtr)
{
    int           clientSd;
    int           errorCode;
    int           sentLen;
    int           recvLen;
    char          sendBuffer[TM_TEST_BUFFER_SIZE];
    char          recvBuffer[TM_TEST_BUFFER_SIZE];
    struct sockaddr_storage fromAddr;
    int           fromLength;

    tfSetCurrentContext(tvContextHandleClient);

/* create client socket */
    clientSd = socket(sockAddrPtr->sa_family, SOCK_DGRAM, IP_PROTOUDP);
    if(clientSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, clientSd != TM_SOCKET_ERROR); 

    errorCode = tfBlockingState(clientSd, TM_BLOCKING_ON);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, errorCode != TM_SOCKET_ERROR); 

/* unconnected */
    FillSendBuffer(sendBuffer,  TM_TEST_BUFFER_SIZE);
    sentLen = sendto(clientSd,
                     sendBuffer,
                     TM_TEST_BUFFER_SIZE,
                     0,
                     sockAddrPtr,
                     sockAddrPtr->sa_len);
    if(sentLen != TM_TEST_BUFFER_SIZE)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, sentLen != TM_SOCKET_ERROR); 
    tm_assert(tfxUdpEchoClient, sentLen == TM_TEST_BUFFER_SIZE); 

    fromLength = (sizeof(fromAddr));

    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = recvfrom( clientSd,
                        recvBuffer,
                        TM_TEST_BUFFER_SIZE,
                        0,
                        (struct sockaddr *)&fromAddr,
                        &fromLength);
    if(recvLen != sentLen)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, recvLen != TM_SOCKET_ERROR); 
    tm_assert(tfxUdpEchoClient, recvLen == TM_TEST_BUFFER_SIZE); 
    tm_assert(tfxUdpEchoClient, !tm_memcmp(sendBuffer, 
                                          recvBuffer, 
                                          TM_TEST_BUFFER_SIZE)); 
/* connected */
    errorCode = connect(clientSd,
                        sockAddrPtr,
                        sockAddrPtr->sa_len);
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, errorCode != TM_SOCKET_ERROR);

    FillSendBuffer(sendBuffer,  TM_TEST_BUFFER_SIZE);
    sentLen = send(clientSd, 
                   sendBuffer, 
                   TM_TEST_BUFFER_SIZE,
                   0);
    if(sentLen != TM_TEST_BUFFER_SIZE)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, sentLen != TM_SOCKET_ERROR); 
    tm_assert(tfxUdpEchoClient, sentLen == TM_TEST_BUFFER_SIZE); 

    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = recv(clientSd,
                   recvBuffer, 
                   TM_TEST_BUFFER_SIZE,
                   0);
    if(recvLen != sentLen)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxUdpEchoClient, recvLen != TM_SOCKET_ERROR); 
    tm_assert(tfxUdpEchoClient, recvLen == TM_TEST_BUFFER_SIZE); 
    tm_assert(tfxUdpEchoClient, !tm_memcmp(sendBuffer, 
                                          recvBuffer, 
                                          TM_TEST_BUFFER_SIZE)); 
    tfClose(clientSd);
}

void tfxTcpEchoClient(struct sockaddr TM_FAR * sockAddrPtr)
{
    int           clientSd;
    int           errorCode;
    int           sentLen;
    int           recvLen;
    char          sendBuffer[TM_TEST_BUFFER_SIZE];
    char          recvBuffer[TM_TEST_BUFFER_SIZE];
  
    tfSetCurrentContext(tvContextHandleClient);

/* create client socket */
    clientSd = socket(sockAddrPtr->sa_family, SOCK_STREAM, IP_PROTOTCP);
    if(clientSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxTcpEchoClient, clientSd != TM_SOCKET_ERROR); 


    errorCode = tfBlockingState(clientSd, TM_BLOCKING_ON);
    if(clientSd == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxTcpEchoClient, clientSd != TM_SOCKET_ERROR); 

    errorCode = connect(clientSd, 
                        sockAddrPtr, 
                        sockAddrPtr->sa_len
                        );
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxTcpEchoClient, errorCode == TM_ENOERROR); 
/*
    FillSendBuffer(sendBuffer,  TM_TEST_BUFFER_SIZE);
    sentLen = send(clientSd,
                   sendBuffer,
                   TM_TEST_BUFFER_SIZE,
                   0
                   );
    tm_assert(tfxTcpEchoClient, sentLen != TM_SOCKET_ERROR); 
    tm_assert(tfxTcpEchoClient, sentLen == TM_TEST_BUFFER_SIZE); 
    
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = recv( clientSd,
                    recvBuffer,
                    TM_TEST_BUFFER_SIZE,
                    0
                    );
    tm_assert(tfxTcpEchoClient, recvLen != TM_SOCKET_ERROR); 
*/
    FillSendBuffer(sendBuffer,  TM_TEST_BUFFER_SIZE);
    sentLen = send(clientSd,
                   sendBuffer,
                   TM_TEST_BUFFER_SIZE,
                   0);
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxTcpEchoClient, sentLen != TM_SOCKET_ERROR); 
    tm_assert(tfxTcpEchoClient, sentLen == TM_TEST_BUFFER_SIZE); 
    
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecv( clientSd,
                    recvBuffer,
                    TM_TEST_BUFFER_SIZE
                    );
    if(recvLen != sentLen)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxTcpEchoClient, recvLen != TM_SOCKET_ERROR); 
    tm_assert(tfxTcpEchoClient, recvLen == sentLen); 
    tm_assert(tfxTcpEchoClient, !tm_memcmp(sendBuffer, 
                                          recvBuffer, 
                                          TM_TEST_BUFFER_SIZE)); 
    tfClose(clientSd);
}

#else /* !TM_MULTIPLE_CONTEXT */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltx6echoDummy = 0;

#endif /* !TM_MULTIPLE_CONTEXT */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuecho.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 62436    Output checksum: 58810
 */
