/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Automated tests for FTP client and FTP server, mostly copied
 *  from Odile's txnat.c example code
 *
 * Filename: tuftp.c
 * Author: Ed Remmell
 * Date Created: January 7, 2005
 * $Source: test/unittest/tuftp.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2011/10/20 11:38:57CDT $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#ifdef TM_MULTIPLE_CONTEXT
#if (defined(TM_USE_FTP) && defined (TM_USE_FTPD))
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#include "tutest.h"

/* constant declarations and enumerations */

/* type definitions */
/* One configuration structure per interface */
typedef struct tsConfigure
{
    ttUserContext           confContext;
    ttUserFtpHandle         confFtpHandle;
} ttConfigure;
typedef  ttConfigure TM_FAR * ttConfigurePtr;

/* local function prototypes */
int tfxFtpExecuteLoop(ttConfigurePtr confClientPtr,
                      ttConfigurePtr confServerPtr);
int tfxFtpServerStart(ttConfigurePtr confPtr);
int tfxFtpServerStop(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr);
int tfxFtpClientStart(ttConfigurePtr confClientPtr,
                      ttConfigurePtr confServerPtr,
                      struct sockaddr_storage TM_FAR * publicServerSockAddrPtr,
                      int            ftpVerbose);
int tfxFtpClientStop(ttConfigurePtr confClientPtr);
int tfxFtpClientLogin(ttConfigurePtr confClientPtr,
                      ttConfigurePtr confServerPtr);
int tfxFtpClientPasv(ttConfigurePtr confClientPtr, 
                     ttConfigurePtr confServerPtr,
                     int            onFlag );
int tfxFtpClientDirList(ttConfigurePtr confClientPtr,
                        ttConfigurePtr confServerPtr,
                        ttCharPtr      pathName);
int tfxFtpClientPwd(ttConfigurePtr confClientPtr,
                    ttConfigurePtr confServerPtr,
                    ttCharPtr pwd,
                    int pwdLength);
int tfxFtpClientCdup(ttConfigurePtr confClientPtr,
                     ttConfigurePtr confServerPtr);
int tfxFtpClientCd(ttConfigurePtr confClientPtr,
                   ttConfigurePtr confServerPtr,
                   ttCharPtr      namePtr);
int tfxFtpClientMkdir(ttConfigurePtr confClientPtr,
                      ttConfigurePtr confServerPtr,
                      ttCharPtr      dirNamePtr,
                      ttCharPtr      remoteDir,
                      int            remoteDirLen);
int tfxFtpClientType(ttConfigurePtr confClientPtr,
                     ttConfigurePtr confServerPtr,
                     int            clientType );
int tfxFtpClientPut(ttConfigurePtr confClientPtr,
                    ttConfigurePtr confServerPtr,
                    ttCharPtr      localNamePtr,
                    ttCharPtr      remoteNamePtr);
int tfxFtpClientGet(ttConfigurePtr confClientPtr,
                    ttConfigurePtr confServerPtr,
                    ttCharPtr      remoteNamePtr,
                    ttCharPtr      localNamePtr);
int tfxFtpClientQuit(ttConfigurePtr confClientPtr,
                     ttConfigurePtr confServerPtr );
void tfxTimer(void);

/* local variable definitions */

/* static const tables */

/* macro definitions */
#define TM_MAX_LOOP_ITERATIONS TM_UL(100000)

/****************************************************************************
* FUNCTION: tfxFtpClient
*
* PURPOSE: verify the fix for bug ID 1470 "FTP client file transfer fails
*   using IPv4-mapped IPv6 address, unless passive-mode transfer is used"
*
* PARAMETERS:
*   FTP server IP address as a string
*
* RETURNS:
*   Nothing
*
* NOTES:
*
****************************************************************************/
void tfxFtpClient(struct sockaddr_storage TM_FAR * ftpServerSockAddrPtr)
{
    ttUserContext   origContext;
    ttConfigure     clientConfig;
    ttConfigure     serverConfig;
    ttConfigurePtr  confClientPtr;
    ttConfigurePtr  confServerPtr;
    char            pwd[128];
    int             ftpVerbose;
    int             pasvFlags;
    int             errorCode;

    ftpVerbose = 1;
    pasvFlags = TM_FTP_PASSIVE_MODE_OFF;

/* Switch context to the client */
    origContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleClient);

    confClientPtr = &clientConfig;
    tm_bzero(&clientConfig, sizeof(clientConfig));
    clientConfig.confContext = tvContextHandleClient;

    confServerPtr = &serverConfig;
    tm_bzero(&serverConfig, sizeof(serverConfig));
    serverConfig.confContext = tvContextHandleServer;

/* int tfxFtpClientServer(
                    ttConfigurePtr confClientPtr,
                    ttConfigurePtr confServerPtr,
                    struct sockaddr_storage TM_FAR * publicServerSockAddrPtr,
                    int            pasvFlags,
                    int            ftpVerbose) */

    errorCode = TM_ENOERROR;
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
            printf("    Starting FTP server\n");
        }
        errorCode = tfxFtpServerStart(confServerPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
            printf("    Starting FTP Client ");
        }
        errorCode = tfxFtpClientStart(confClientPtr,
                                      confServerPtr,
                                      ftpServerSockAddrPtr,
                                      ftpVerbose);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: login\n");
        }
        errorCode = tfxFtpClientLogin(confClientPtr, confServerPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (pasvFlags == TM_FTP_PASSIVE_MODE_ON)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: PASSIVE mode\n");
        }
        errorCode = tfxFtpClientPasv(confClientPtr, confServerPtr, pasvFlags);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: CD /\n");
        }
        errorCode = tfxFtpClientCd(confClientPtr, confServerPtr, "/");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        tm_bzero(pwd, sizeof(pwd));
        if (ftpVerbose)
        {
        printf("    FTP Client: PWD ");
        }
        errorCode = tfxFtpClientPwd(confClientPtr, confServerPtr,
                                     pwd, sizeof(pwd));
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
        if (ftpVerbose)
        {
            printf("%s'\n", pwd);
        }
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: DIR LIST "".""\n");
        }
        errorCode = tfxFtpClientDirList(confClientPtr, confServerPtr, "");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: TYPE BIN\n");
        }
        errorCode = tfxFtpClientType(confClientPtr, confServerPtr,
                                      TM_TYPE_BINARY );
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
            printf("    FTP Client: CD %s\n",
                   TM_FTP_AUTOTEST_DIR);
        }
        errorCode = tfxFtpClientCd(confClientPtr,
                                   confServerPtr,
                                   TM_FTP_AUTOTEST_DIR);
        tm_assert(tfxFtp, (   (errorCode == TM_ENOERROR)
                           || (errorCode == 8550) ) );
        if (errorCode == 8550)
        {
            if (ftpVerbose)
            {
                printf("    FTP Client: MKDIR %s\n",
                       TM_FTP_AUTOTEST_DIR);
            }
            errorCode = tfxFtpClientMkdir(confClientPtr,
                                          confServerPtr,
                                          TM_FTP_AUTOTEST_DIR,
                                          pwd,
                                          sizeof(pwd));
            tm_assert(tfxFtp, errorCode == TM_ENOERROR);
            if (ftpVerbose)
            {
                printf("    FTP Client: CD %s\n",
                       TM_FTP_AUTOTEST_DIR);
            }
            errorCode = tfxFtpClientCd(confClientPtr,
                                       confServerPtr,
                                       TM_FTP_AUTOTEST_DIR);
            tm_assert(tfxFtp, errorCode == TM_ENOERROR);
        }
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: PWD ");
        }
        tm_bzero(pwd, sizeof(pwd));
        errorCode = tfxFtpClientPwd(confClientPtr, confServerPtr,
                                     pwd, sizeof(pwd));
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
        if (ftpVerbose)
        {
            printf("'%s'\n", pwd);
        }
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: DIR LIST "".""\n");
        }
        errorCode = tfxFtpClientDirList(confClientPtr, confServerPtr, "");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: CD UP\n");
        }
        errorCode = tfxFtpClientCdup(confClientPtr, confServerPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: PWD ");
        }
        tm_bzero(pwd, sizeof(pwd));
        errorCode = tfxFtpClientPwd(confClientPtr, confServerPtr,
                                     pwd, sizeof(pwd));
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
        if (ftpVerbose)
        {
            printf("'%s'\n", pwd);
        }
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: DIR LIST "".""\n");
        }
        errorCode = tfxFtpClientDirList(confClientPtr, confServerPtr, "");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
            printf("    FTP Client: CD %s\n", TM_FTP_AUTOTEST_DIR);
        }
        errorCode = tfxFtpClientCd(confClientPtr,
                                   confServerPtr,
                                   TM_FTP_AUTOTEST_DIR);
        tm_assert(tfxFtp, (   (errorCode == TM_ENOERROR)
                           || (errorCode == 8550) ) );
        if (errorCode == 8550)
        {
            if (ftpVerbose)
            {
                printf("    FTP Client: MKDIR %s\n",
                       TM_FTP_AUTOTEST_DIR);
            }
            errorCode = tfxFtpClientMkdir(confClientPtr,
                                          confServerPtr,
                                          TM_FTP_AUTOTEST_DIR,
                                          pwd,
                                          sizeof(pwd));
            tm_assert(tfxFtp, errorCode == TM_ENOERROR);
            if (ftpVerbose)
            {
                printf("    FTP Client: CD %s\n",
                       TM_FTP_AUTOTEST_DIR);
            }
            errorCode = tfxFtpClientCd(confClientPtr,
                                       confServerPtr,
                                       TM_FTP_AUTOTEST_DIR);
            tm_assert(tfxFtp, errorCode == TM_ENOERROR);
        }
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: PWD ");
        }
        tm_bzero(pwd, sizeof(pwd));
        errorCode = tfxFtpClientPwd(confClientPtr, confServerPtr,
                                     pwd, sizeof(pwd));
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
        if (ftpVerbose)
        {
            printf("'%s'\n", pwd);
        }
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: DIR LIST "".""\n");
        }
        errorCode = tfxFtpClientDirList(confClientPtr, confServerPtr, "");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: TYPE BIN\n");
        }
        errorCode = tfxFtpClientType(confClientPtr, confServerPtr,
                                      TM_TYPE_BINARY );
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: PUT index.htm remoteIndex.htm\n");
        }
        errorCode = tfxFtpClientPut(confClientPtr, confServerPtr,
                                     "index.htm", "remoteIndex.htm");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: GET remoteIndex.htm localIndex.htm\n");
        }
        errorCode = tfxFtpClientGet(confClientPtr, confServerPtr,
                                     "remoteIndex.htm", "localIndex.htm");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: DIR LIST "".""\n");
        }
        errorCode = tfxFtpClientDirList(confClientPtr, confServerPtr, "");
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: CD UP\n");
        }
        errorCode = tfxFtpClientCdup(confClientPtr, confServerPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP Client: QUIT\n");
        }
        errorCode = tfxFtpClientQuit(confClientPtr, confServerPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP SERVER: STOP\n");
        }
        errorCode = tfxFtpServerStop(confClientPtr, confServerPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }
    if (errorCode == TM_ENOERROR)
    {
        if (ftpVerbose)
        {
        printf("    FTP CLIENT: STOP\n");
        }
        errorCode = tfxFtpClientStop(confClientPtr);
        tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    }

    tfSetCurrentContext(origContext);
    return;
}

int tfxFtpServerStart(ttConfigurePtr confPtr)
{
    int errorCode;
/* Start the FTP server */
    tm_set_current_context(confPtr->confContext);
    errorCode = tfNgFtpdUserStart(
        TM_FS_ALLCMND_MASK , 2, 2, 300, TM_BLOCKING_OFF, 0);

    tm_assert(tfxFtp, errorCode == TM_ENOERROR);
    return errorCode;
}

int tfxFtpServerStop(ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr)
{
    int errorCode;
/* Stop the FTP server */
    tm_set_current_context(confServerPtr->confContext);
    errorCode = tfFtpdUserStop();
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientStop(ttConfigurePtr confClientPtr)
{
    int errorCode;
/* Stop the FTP server */
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpFreeSession(confClientPtr->confFtpHandle);
    return errorCode;
}

/*
 * FTP client test
 */
#define FS_USER_NAME     "treck"
#define FS_PASSWORD      "treck"
#define FTP_USER_NAME    "treck"
#define FTP_PASSWORD     "treck"
#define FTP_ACCOUNT_NAME "account"

int ftpCBFunc(ttUserFtpHandle ftpHandle, char *bufferPtr, int bufferSize);

int ftpCBFunc(ttUserFtpHandle ftpHandle, char *bufferPtr, int bufferSize)
{
    TM_UNREF_IN_ARG(ftpHandle);
    TM_UNREF_IN_ARG(bufferSize);
    TM_UNREF_IN_ARG(bufferPtr);
/* printf("%s", bufferPtr); */
    return TM_ENOERROR;
}

int tfxFtpClientStart(
    ttConfigurePtr confClientPtr,
    ttConfigurePtr confServerPtr,
    struct sockaddr_storage TM_FAR * publicServerSockAddrPtr,
    int ftpVerbose)
{
    int errorCode;

    errorCode = TM_EPERM;
    tm_set_current_context(confClientPtr->confContext);
    confClientPtr->confFtpHandle = tfFtpNewSession(
        TM_FS_ALLCMND_MASK, TM_BLOCKING_OFF, FS_USER_NAME, FS_PASSWORD);
    if (ftpVerbose)
    {
        printf("NewSession == 0x%p\n", confClientPtr->confFtpHandle );
    }
    tm_assert(tfxFtp, confClientPtr->confFtpHandle != (ttVoidPtr)0);
    if (confClientPtr->confFtpHandle != (ttUserFtpHandle)0)
    {
        errorCode = tfNgFtpConnect( confClientPtr->confFtpHandle,
                                    publicServerSockAddrPtr );
        tm_assert(
            tfxFtp,
            (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    }
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

/* Execute FTP server, and FTP client until done */
int tfxFtpExecuteLoop(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr)
{
    tt32Bit  iterations;
    int      errorCode;

    iterations = TM_UL(0);
    do
    {
        tm_set_current_context(confServerPtr->confContext);
        tfFtpdUserExecute();
        tm_set_current_context(confClientPtr->confContext);
        if (iterations == 4496)
        {
/* debug stuff */
            errorCode = TM_ENOERROR;
        }
        errorCode = tfFtpUserExecute(confClientPtr->confFtpHandle);
        tfxTimer();
        iterations++;
    }
    while (   (errorCode == TM_EWOULDBLOCK)
            && (iterations <= TM_MAX_LOOP_ITERATIONS));
    tm_assert(tfxFtp, iterations < TM_MAX_LOOP_ITERATIONS);
    return errorCode;
}

int tfxFtpClientLogin(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr)
{
    int errorCode;
/*------ Login Name/password ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpLogin(
        confClientPtr->confFtpHandle, FTP_USER_NAME, FTP_PASSWORD,
        FTP_ACCOUNT_NAME );
    tm_assert(tfxFtp,
              (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientDirList(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr,
    ttCharPtr pathName)
{
    int errorCode;
/*------ Login Name/password ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpDirList(
        confClientPtr->confFtpHandle, pathName, TM_DIR_LONG, ftpCBFunc );
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientPwd(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr,
    ttCharPtr pwd, int pwdLength)
{
    int errorCode;
/*------ Login Name/password ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpPwd(confClientPtr->confFtpHandle, pwd, pwdLength);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK) );
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientCdup(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr)
{
    int errorCode;
/*------ Login Name/password ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpCdup(confClientPtr->confFtpHandle);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientCd(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr,
    ttCharPtr pathNamePtr)
{
    int errorCode;
/*------ Login Name/password ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpCwd(confClientPtr->confFtpHandle, pathNamePtr);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientMkdir(ttConfigurePtr confClientPtr,
                      ttConfigurePtr confServerPtr,
                      ttCharPtr      dirNamePtr,
                      ttCharPtr      remoteDir,
                      int            remoteDirLen)
{
    int errorCode;
/*------ Login Name/password ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpMkd(confClientPtr->confFtpHandle,
                         dirNamePtr,
                         remoteDir,
                         remoteDirLen);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientPasv(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr, int onFlag )
{
    int errorCode;
/*------ Passive Mode ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpTurnPasv(confClientPtr->confFtpHandle, onFlag);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientType(
    ttConfigurePtr confClientPtr, ttConfigurePtr confServerPtr, int clientType)
{
    int errorCode;
/*------ Passive Mode ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpType(confClientPtr->confFtpHandle, clientType);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientPut(ttConfigurePtr confClientPtr,
                    ttConfigurePtr confServerPtr,
                    ttCharPtr      localNamePtr,
                    ttCharPtr      remoteNamePtr)
{
    int errorCode;
/*------ Client PUT ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpStor(confClientPtr->confFtpHandle, (ttUserFtpHandle)0, 
                          localNamePtr, remoteNamePtr);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

int tfxFtpClientGet(ttConfigurePtr confClientPtr, 
                    ttConfigurePtr confServerPtr,
                    ttCharPtr      remoteNamePtr, 
                    ttCharPtr      localNamePtr)
{
    int errorCode;
/*------ Client GET ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpRetr(confClientPtr->confFtpHandle,  (ttUserFtpHandle)0,
                          remoteNamePtr, localNamePtr);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
        tm_assert(
            tfxFtp,
            (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    }
    return errorCode;
}

int tfxFtpClientQuit(ttConfigurePtr confClientPtr,
                      ttConfigurePtr confServerPtr )
{
    int errorCode;
/*------ Client QUIT ----------------------------*/
    tm_set_current_context(confClientPtr->confContext);
    errorCode = tfFtpQuit(confClientPtr->confFtpHandle);
    tm_assert(
        tfxFtp,
        (errorCode == TM_ENOERROR) || (errorCode == TM_EWOULDBLOCK));
    if (errorCode == TM_EWOULDBLOCK)
    {
        errorCode = tfxFtpExecuteLoop(confClientPtr, confServerPtr);
    }
    return errorCode;
}

void tfxTimer(void)
{
    ttUserContext   currentContext;

/* Save current context */
    tm_get_current_context(currentContext);
    tfTimerUpdate(); /* context independant call */
    tfTimerUpdate(); /* context independant call */
    tm_set_current_context(tvContextHandleClient);
    tfTimerExecute(); /* context dependant */
    tm_set_current_context(tvContextHandleServer);
    tfTimerExecute(); /* context dependant */
/* Restore current context */
    tm_set_current_context(currentContext);
}

#else /* !FTP Client and !FTP server */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

int tlxFtpDummy = 0;

#endif /* !FTP client and !FTP server */
#endif /* TM_MULTIPLE_CONTEXT */

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuftp.c       File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 28784    Output checksum:  2260
 */
