/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2009
 * Copyright Elmic Systems Japan 1997 - 2009
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Testing if_indextoname and if_nametoindex
 *
 * Filename: tuifidnm.c
 * Author: Bryan Wang
 * Date Created:
 * $Source: test/unittest/tuifidnm.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:33CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#ifdef TM_MULTIPLE_CONTEXT
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h> 
#include "tutest.h"

#ifdef TM_USE_IFNAMEINDEX
/*
 * List of expected server and client interface names
 */
static const char TM_FAR * tlxServerInterfaceNames[] =
{
#ifdef TM_SINGLE_INTERFACE_HOME
    "Server",
#else /* !TM_SINGLE_INTERFACE_HOME */
    "LOOPBACK",
    "offlinkS",
    "Server"
#ifdef TM_USE_PPP
    , "Ppp.Server"
#endif /* TM_USE_PPP */
#ifdef TM_USE_PPPOE_CLIENT_SERVER
    , "Pppoe.Server"
#endif /* TM_USE_PPPOE_CLIENT_SERVER */
#endif /* !TM_SINGLE_INTERFACE_HOME */
};
#define TM_SERVER_NAME_COUNT (  sizeof(tlxServerInterfaceNames) \
                              / sizeof(tlxServerInterfaceNames[0]) )

static const char TM_FAR * tlxClientInterfaceNames[] =
{
#ifdef TM_SINGLE_INTERFACE_HOME
    "Client",
#else /* !TM_SINGLE_INTERFACE_HOME */
    "LOOPBACK",
    "offlinkC",
    "Client",
    "extraC"
#ifdef TM_USE_PPP
    , "Ppp.Client"
#endif /* TM_USE_PPP */
#ifdef TM_USE_PPPOE_CLIENT_SERVER
    , "Pppoe.Client"
#endif /* TM_USE_PPPOE_CLIENT_SERVER */
#endif /* !TM_SINGLE_INTERFACE_HOME */
};
#define TM_CLIENT_NAME_COUNT (  sizeof(tlxClientInterfaceNames) \
                              / sizeof(tlxClientInterfaceNames[0]) )

static const struct
{
    ttUserContext TM_FAR *          contextPtr;
    const char TM_FAR * TM_FAR *    nameArray;
    unsigned int                    nameArraySize;
} tlxIfNameIndexData[] =
{
    { &tvContextHandleServer, tlxServerInterfaceNames, TM_SERVER_NAME_COUNT },
    { &tvContextHandleClient, tlxClientInterfaceNames, TM_CLIENT_NAME_COUNT }
};
#define TM_NAME_INDEX_DATA_COUNT (  sizeof(tlxIfNameIndexData) \
                                  / sizeof(tlxIfNameIndexData[0]) )
#endif /* TM_USE_IFNAMEINDEX */

/*
 * @REQUIREMENT TRACIBILITY 
 * [RFC2553].R4.2:10
 */
void tfxIfIndexToName(ttUserContext context)
{
#ifdef TM_ERROR_CHECKING
    char name[IF_NAMESIZE];
#endif /* TM_ERROR_CHECKING */
    ttUserContext   oldContext;
    int             clientServerIndex;

    oldContext = tfGetCurrentContext();

    tfSetCurrentContext(context);

    tm_assert(tfxIfIndexToName, 
              if_indextoname(0, name) == NULL 
              && tfGetSocketError(-1) == TM_ENXIO);
#ifndef TM_SINGLE_INTERFACE_HOME
    tm_assert(tfxIfIndexToName,
             !tm_strcmp(if_indextoname(1, name), "LOOPBACK"));
#endif /* TM_SINGLE_INTERFACE_HOME */

#ifdef TM_SINGLE_INTERFACE_HOME
    clientServerIndex = 1;
#else /* !TM_SINGLE_INTERFACE_HOME */
    clientServerIndex = 3;
#endif /* !TM_SINGLE_INTERFACE_HOME */
    if(context == tvContextHandleServer)
    {
#ifdef TM_USE_STRONG_ESL
        tm_assert(tfxIfIndexToName,
                     (    if_indextoname(4, name) == NULL 
                       && tfGetSocketError(-1) == TM_ENXIO)
                  || (!tm_strcmp(name, "Ppp.Server"))
                  || (!tm_strcmp(name, "Server2"))
                  || (!tm_strcmp(name, "Tmpv6GWServer")) );
#else /* TM_USE_STRONG_ESL */
        tm_assert(tfxIfIndexToName,
                     (    if_indextoname(4, name) == NULL 
                       && tfGetSocketError(-1) == TM_ENXIO)
                  || (!tm_strcmp(name, "Ppp.Server"))
                  || (!tm_strcmp(name, "Tmpv6GWServer")) );
#endif /* TM_USE_STRONG_ESL */
#ifndef TM_SINGLE_INTERFACE_HOME
        tm_assert(tfxIfIndexToName,
                 !tm_strcmp(if_indextoname(2, name), "offlinkS"));
#endif /* ! TM_SINGLE_INTERFACE_HOME */
        tm_assert(tfxIfIndexToName,
                 !tm_strcmp(if_indextoname(clientServerIndex, name), "Server"));
    }   
    else 
    {
        tm_assert(tfxIfIndexToName, 
                     (    if_indextoname(5, name) == NULL 
                       && tfGetSocketError(-1) == TM_ENXIO)
                  || (!tm_strcmp(name, "Ppp.Client")) );
        tm_assert(tfxIfNameToIndex, context == tvContextHandleClient)
#ifndef TM_SINGLE_INTERFACE_HOME
        tm_assert(tfxIfIndexToName,
                 !tm_strcmp(if_indextoname(2, name), "offlinkC"));
#endif /* ! TM_SINGLE_INTERFACE_HOME */
        tm_assert(tfxIfIndexToName,
                 !tm_strcmp(if_indextoname(clientServerIndex, name), "Client"));
    }

    tfSetCurrentContext(oldContext);
}


/*
 * @REQUIREMENT TRACIBILITY
 * [RFC2553].R4.1:10
 * to add the interface properly
 */
void tfxIfNameToIndex(ttUserContext context)
{
    ttUserContext   oldContext;
    unsigned int    clientServerIndex;

#ifdef TM_SINGLE_INTERFACE_HOME
    clientServerIndex = 1;
#else /* !TM_SINGLE_INTERFACE_HOME */
    clientServerIndex = 3;
#endif /* !TM_SINGLE_INTERFACE_HOME */
    oldContext = tfGetCurrentContext();
    tfSetCurrentContext(context);

    if(context == tvContextHandleServer)
    {
        tm_assert(tfxIfNameToIndex, if_nametoindex("Server") ==
                                    clientServerIndex );
    }
    else 
    {
        tm_assert(tfxIfNameToIndex, context == tvContextHandleClient)
        tm_assert(tfxIfNameToIndex, if_nametoindex("Client") ==
                                    clientServerIndex );
    }
/* loop back interface have index 1 always */
#ifndef TM_SINGLE_INTERFACE_HOME
    tm_assert(tfxIfNameToIndex, if_nametoindex("LOOPBACK") ==1 );
#endif /* TM_SINGLE_INTERFACE_HOME */
    
/* eth3 doesn't exist */
    tm_assert(tfxIfNameToIndex, 
              if_nametoindex("eth3") ==0  
              && tfGetSocketError(-1) == TM_ENXIO);

    tfSetCurrentContext(oldContext);
}


#ifdef TM_USE_IFNAMEINDEX
/*
 * @REQUIREMENT TRACIBILITY
 * [RFC3493].R4.3
 */
void tfxIfNameIndex(void)
{
    ttUserContext                   oldContext;
    struct if_nameindex TM_FAR *    niPtr;
    const char TM_FAR * TM_FAR *    nameArray;
    unsigned int                    nameArraySize;
    unsigned int                    testCount;
    unsigned int                    niCount;

    oldContext = tfGetCurrentContext();

    for (testCount = 0; testCount < TM_NAME_INDEX_DATA_COUNT; ++testCount)
    {
        tfSetCurrentContext(*tlxIfNameIndexData[testCount].contextPtr);
        nameArray = tlxIfNameIndexData[testCount].nameArray;
        nameArraySize = tlxIfNameIndexData[testCount].nameArraySize;

/* Get a list of all interface names */
        niPtr = if_nameindex();
        tm_assert(tfxIfNameIndex, niPtr != 0);

/* Check each entry in the list */
        for (   niCount = 0;
                niPtr[niCount].if_index != 0 && niPtr[niCount].if_name != 0;
                ++niCount )
        {
            tm_assert(tfxIfNameIndex, niPtr[niCount].if_index == niCount + 1);
            tm_assert( tfxIfNameIndex, niCount >= nameArraySize ||
                    !tm_strcmp(niPtr[niCount].if_name, nameArray[niCount]) );
        }
/* The last entry must be all zeros */
        tm_assert(tfxIfNameIndex,
                niPtr[niCount].if_index == 0 && niPtr[niCount].if_name == 0);
/* Free the list */
        if_freenameindex(niPtr);
    }

    tfSetCurrentContext(oldContext);
}
#endif /* TM_USE_IFNAMEINDEX */

#else /* !TM_MULTIPLE_CONTEXT */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltxIfIdNmDummy = 0;

#endif /* !TM_MULTIPLE_CONTEXT */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuifidnm.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 23278    Output checksum: 64094
 */
