/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Test the multicast
 * @REQUIREMENT TRACIBILTY [IPV6REQ] section2.18
 * Filename: tumcast.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002
 * $Source: test/unittest/tumcast.c $
 *
 * Modification History
 * $Revision: 6.0.2.7 $
 * $Date: 2011/11/14 14:01:17CST $
 * $Author: lfox $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>

#ifdef TM_MULTIPLE_CONTEXT

#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"

#ifdef TM_4_USE_SCOPE_ID
#define tm_4_addr_str(s) "::ffff:" s
#else /* !TM_4_USE_SCOPE_ID */
#define tm_4_addr_str(s) s
#endif /* !TM_4_USE_SCOPE_ID */


#if (defined(TM_USE_IPV4) && defined(TM_IGMP))
static void tfx4UdpMcastSendRecv(int                         clientSd,
                                 int                         serverSd,
                                 struct sockaddr_storage *   destAddrPtr,
                                 int                         expErrCode);
#endif /* TM_USE_IPV4 && TM_IGMP */


#ifdef TM_IGMP
void tfx4UdpMCast(void)
{
    struct sockaddr_in sockAddr;
    int     sd;
    int     sentLen;
    int     errorCode;
    char    buffer[TM_TEST_BUFFER_SIZE];
    int               setLen ;
    struct ip_mreq    v4SetValue;
    ttUserInterface   interfaceHandle;
    int               mcastTestSd;

    tfSetCurrentContext(tvContextHandleServer);

    tm_bzero(&sockAddr, sizeof(sockAddr));
    sockAddr.sin_family = AF_INET;
    sockAddr.sin_len    = sizeof(sockAddr);
    sockAddr.sin_port   = ntohs(100);
    sockAddr.sin_addr.s_addr = inet_addr("224.0.0.1");

    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4UdpMCast, sd != TM_SOCKET_ERROR);

    errorCode = setsockopt(
        sd,
        IP_PROTOIP,
        IPO_MULTICAST_IF,
        (const char*)&(tv4ServerSockAddr[0].addr.ipv4.sin_addr),
        sizeof(struct in_addr));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(sd);
    }
    tm_assert(tfx4UdpMCast, errorCode == TM_ENOERROR);

    sentLen = sendto(
        sd,
        buffer,
        TM_TEST_BUFFER_SIZE,
        0,
        (struct sockaddr *)&sockAddr,
        sizeof(sockAddr));
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
    }
    tm_assert(tfx4UdpMCast, sentLen != TM_SOCKET_ERROR);
    tm_assert(tfx4UdpMCast, sentLen == TM_TEST_BUFFER_SIZE);
    errorCode = tfClose(sd);
    tm_assert(tfx4UdpMCast, errorCode == TM_ENOERROR);

/* create socket to test IGMP multicast group join/leave */
    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4UdpMCast, sd != -1);
    interfaceHandle = tfxGetInterfaceByContext(tfGetCurrentContext());
    errorCode = tfSetMcastInterface(interfaceHandle, 0);
    tm_assert(
        tfx4UdpMCast,
        ((errorCode == TM_ENOERROR) || (errorCode == TM_EALREADY)));

/* give a interface index 0, let system choose*/
    v4SetValue.imr_interface.s_addr = 0;
    errorCode = inet_pton(
        AF_INET, "224.100.1.1", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

/* leave the multicast group */
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_DROP_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

/* try to leave the multicast group a 2nd time, when we are no longer a member */
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_DROP_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode == -1);

/* join some multicast groups but don't leave them, when the socket is closed
   we will leave them. */
    errorCode = inet_pton(
        AF_INET, "224.123.1.1", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

    errorCode = inet_pton(
        AF_INET, "224.222.1.2", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

    errorCode = inet_pton(
        AF_INET, "224.233.1.3", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

    errorCode = inet_pton(
        AF_INET, "224.244.1.4", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

    errorCode = inet_pton(
        AF_INET, "224.250.1.5", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(sd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

/* create 2nd socket to test multicast group socket ownership count */
    mcastTestSd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4UdpMCast, mcastTestSd != -1);

/* join some of the same multicast groups on 2nd socket, we will close this
   socket here */
    errorCode = inet_pton(
        AF_INET, "224.233.1.3", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(mcastTestSd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

    errorCode = inet_pton(
        AF_INET, "224.222.1.2", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(mcastTestSd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

    errorCode = inet_pton(
        AF_INET, "224.244.1.4", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(mcastTestSd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

/* join the same group again */
    errorCode = setsockopt(mcastTestSd, IPPROTO_IP, IPO_ADD_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != TM_EADDRINUSE);

/* leave one of the multicast groups we just joined on the 2nd socket */
    errorCode = inet_pton(
        AF_INET, "224.233.1.3", & v4SetValue.imr_multiaddr);
    tm_assert(tfx4UdpMCast, errorCode == 1);
    setLen = sizeof(v4SetValue);
    errorCode = setsockopt(mcastTestSd, IPPROTO_IP, IPO_DROP_MEMBERSHIP,
                           (char *) &v4SetValue, setLen);
    tm_assert(tfx4UdpMCast, errorCode != -1);

/* reinitialize the multicast address list in the EMAC device driver */
    errorCode = tfEtherReinitDriverMcast(interfaceHandle);
    tm_assert(tfx4UdpMcast, (errorCode == TM_ENOERROR));

/* close the sockets */
    errorCode = tfClose(sd);
    tm_assert(tfx4UdpMCast, errorCode == TM_ENOERROR);
    errorCode = tfClose(mcastTestSd);
    tm_assert(tfx4UdpMCast, errorCode == TM_ENOERROR);
}

/* test multicast TTL field */
void tfx4UdpMCastTTL(void)
{
    struct sockaddr_in sockAddr;
    struct sockaddr_in addr;
    struct ip_mreq    v4SetValue;
    int     sd;
    int     sentLen;
    int     errorCode;
    char    buffer[TM_TEST_BUFFER_SIZE];
    tt8Bit  ttl;
    int     i;

    tfSetCurrentContext(tvContextHandleClient);

    tm_bzero(&sockAddr, sizeof(sockAddr));
    sockAddr.sin_family = AF_INET;
    sockAddr.sin_len    = sizeof(sockAddr);
    sockAddr.sin_port   = ntohs(100);
    sockAddr.sin_addr.s_addr = inet_addr("224.0.0.1");

    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4UdpMCastTTL, sd != TM_SOCKET_ERROR);

/* try bind to test if it has effect on TTL*/
     addr.sin_family = PF_INET;
     addr.sin_port = htons(1025);
     addr.sin_addr.s_addr = 0;
     errorCode = bind ( sd,
                       (struct sockaddr TM_FAR *)&addr,
                        sizeof(struct sockaddr_in));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(sd);
    }
    tm_assert(tfx4UdpMCastTTL, errorCode == TM_ENOERROR);

/* try join multicast group to test if it has effect on TTL */
    v4SetValue.imr_multiaddr.s_addr = inet_addr ("224.0.0.1");
    v4SetValue.imr_interface.s_addr = 0;
    errorCode = setsockopt( sd,
                            IP_PROTOIP,
                            IPO_ADD_MEMBERSHIP,
                            (char TM_FAR *)&v4SetValue,
                             sizeof(struct ip_mreq));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(sd);
    }
    tm_assert(tfx4UdpMCastTTL, errorCode == TM_ENOERROR);

/* set multicast TTL value */
    ttl = 14;
    errorCode = setsockopt(
        sd,
        IP_PROTOIP,
        IPO_MULTICAST_TTL,
        (const char*)&ttl,
        sizeof(ttl));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(sd);
    }
    tm_assert(tfx4UdpMCastTTL, errorCode == TM_ENOERROR);

    for(i = 0; i< 5; i++)
    {
        sentLen = sendto(
            sd,
            buffer,
            TM_TEST_BUFFER_SIZE,
            0,
            (struct sockaddr *)&sockAddr,
            sizeof(sockAddr));
        if(sentLen == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(sd);
        }
        tm_assert(tfx4UdpMCastTTL, sentLen != TM_SOCKET_ERROR);
        tm_assert(tfx4UdpMCastTTL, sentLen == TM_TEST_BUFFER_SIZE);
    }
    errorCode = tfClose(sd);
    tm_assert(tfx4UdpMCastTTL, errorCode == TM_ENOERROR);
}

/*
 * tfx4UdpMcastSockDeliveryTest
 *
 * This function tests the ability for sockets to recieve multicast packets
 * depending on the destination multicast address and whether the socket
 * has joined the multicast group or not.
 *
 * Here is what this test verifies:
 * Sockets that join 224.0.0.3 recv the packets to 224.0.0.3
 * Sockets that join 224.0.0.3 recv the packets to 224.0.0.1
 * Sockets that don't join 224.0.0.3 don't recv the packets to 224.0.0.3
 * Sockets that don't join 224.0.0.3 recv the packets to 224.0.0.1
 */
void tfx4UdpMcastSockDeliveryTest(void)
{
    ttUserContext           oldContext;
    struct sockaddr_storage mcastSockAddr;
    struct ip_mreq          v4SetValue;
    int                     errorCode;
    int                     serverSd;
    int                     clientSd;

    oldContext = tfGetCurrentContext();

/* Sockets that join 224.0.0.3 recv the packets to 224.0.0.3 */
    tfxGetNonConnectedUdpPair(&serverSd, &clientSd, AF_INET, 0
#ifdef TM_USE_STRONG_ESL
                             , "Client", "Server"
#endif /* TM_USE_STRONG_ESL */
                             );

    tfSetCurrentContext(tvContextHandleServer);
/* join the group */
    v4SetValue.imr_multiaddr.s_addr = inet_addr("224.0.0.3");
    v4SetValue.imr_interface.s_addr = 0;
    errorCode = setsockopt(serverSd,
                           IP_PROTOIP,
                           IPO_ADD_MEMBERSHIP,
                           (char TM_FAR *)&v4SetValue,
                           sizeof(struct ip_mreq));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfx4UdpMcastSockDeliveryTest, errorCode == TM_ENOERROR);

    tfxInetAddr(tm_4_addr_str("224.0.0.3"), &mcastSockAddr);
    mcastSockAddr.ss_port = tv4ServerSockAddr[0].addr.ipv4.sin_port;

    tfSetCurrentContext(tvContextHandleClient);
    tfx4UdpMcastSendRecv(clientSd,
                         serverSd,
                         &mcastSockAddr,
                         TM_ENOERROR);

/* Sockets that don't join 224.0.0.3 don't recv the packets to 224.0.0.3 */
    tfSetCurrentContext(tvContextHandleServer);
/* leave the group */
    errorCode = setsockopt(serverSd,
                           IP_PROTOIP,
                           IPO_DROP_MEMBERSHIP,
                           (char TM_FAR *)&v4SetValue,
                           sizeof(struct ip_mreq));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfx4UdpMcastSockDeliveryTest, errorCode == TM_ENOERROR);

    tfSetCurrentContext(tvContextHandleClient);
    tfx4UdpMcastSendRecv(clientSd,
                         serverSd,
                         &mcastSockAddr,
                         TM_EWOULDBLOCK);

    tfxCloseUdpConnection(serverSd, clientSd);

/* Sockets that join 224.0.0.3 recv the packets to 224.0.0.1 */
    tfxGetNonConnectedUdpPair(&serverSd, &clientSd, AF_INET, 0
#ifdef TM_USE_STRONG_ESL
                             , "Client", "Server"
#endif /* TM_USE_STRONG_ESL */
                             );

    tfSetCurrentContext(tvContextHandleServer);
/* join the group */
    v4SetValue.imr_multiaddr.s_addr = inet_addr("224.0.0.3");
    v4SetValue.imr_interface.s_addr = 0;
    errorCode = setsockopt(serverSd,
                           IP_PROTOIP,
                           IPO_ADD_MEMBERSHIP,
                           (char TM_FAR *)&v4SetValue,
                           sizeof(struct ip_mreq));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfx4UdpMcastSockDeliveryTest, errorCode == TM_ENOERROR);

    errorCode = inet_pton(AF_INET,
                          "224.0.0.1",
                          &(mcastSockAddr.addr.ipv4.sin_addr));

    tfSetCurrentContext(tvContextHandleClient);
    tfx4UdpMcastSendRecv(clientSd,
                         serverSd,
                         &mcastSockAddr,
                         TM_ENOERROR);

/* Sockets that don't join 224.0.0.3 don't recv the packets to 224.0.0.3 */
    tfSetCurrentContext(tvContextHandleServer);
/* leave the group */
    errorCode = setsockopt(serverSd,
                           IP_PROTOIP,
                           IPO_DROP_MEMBERSHIP,
                           (char TM_FAR *)&v4SetValue,
                           sizeof(struct ip_mreq));
    if(errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfx4UdpMcastSockDeliveryTest, errorCode == TM_ENOERROR);

    tfSetCurrentContext(tvContextHandleClient);
    tfx4UdpMcastSendRecv(clientSd,
                         serverSd,
                         &mcastSockAddr,
                         TM_ENOERROR);

    tfxCloseUdpConnection(serverSd, clientSd);
    tfSetCurrentContext(oldContext);

    return;
}

/*
 * tfx4UdpMcastSendRecv
 *
 * This is a helper function to remove duplicate code from the some of the
 * test functions like tfx4UdpMcastSockDeliveryTest
 *
 * Send to destAddrPtr using the clientSd and receive using the serverSd
 */
static void tfx4UdpMcastSendRecv(int                         clientSd,
                                 int                         serverSd,
                                 struct sockaddr_storage *   destAddrPtr,
                                 int                         expErrCode)
{
    char    buffer[TM_TEST_BUFFER_SIZE];
    int     sentLen;
    int     recvLen;
    int     errorCode;

    sentLen = sendto(clientSd,
                     buffer,
                     TM_TEST_BUFFER_SIZE,
                     0,
                     (struct sockaddr*)destAddrPtr,
                     destAddrPtr->ss_len);
    tm_assert(tfx4UdpMcastSendRecv, sentLen != TM_SOCKET_ERROR);

    tfSwitchContext();
    recvLen = recvfrom(serverSd,
                       buffer,
                       TM_TEST_BUFFER_SIZE,
                       0,
                       NULL,
                       0);
    if (expErrCode == TM_ENOERROR)
    {
        tm_assert(tfx4UdpMcastSendRecv, recvLen != TM_SOCKET_ERROR);
    }
    else
    {
        tm_assert(tfx4UdpMcastSendRecv, recvLen == TM_SOCKET_ERROR);
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfx4UdpMcastSendRecv, (errorCode == expErrCode));
    }
    
    tfSwitchContext();

    return;
}
#endif /* TM_IGMP */






#else /* TM_MULTIPLE_CONTEXT */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltx6McastDummy = 0;
#endif /* TM_MULTIPLE_CONTEXT */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tumcast.c     File Revision: 6.0.2.7 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 16910    Output checksum: 64020
 */
