/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: testing inet_ntop
 *
 * Filename: tuntop.c
 * Author: Bryan Wang
 * Date Created:
 * $Source: test/unittest/tuntop.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:39CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#include "tutest.h"

#define TM_INET_ADDRSTRLEN INET_ADDRSTRLEN

void tfxInetNtoP()
{
    unsigned long       ipAddr[4];
    char                ipAddrStr[TM_INET_ADDRSTRLEN];
    const char TM_FAR  *ipAddrStrPtr;
    int                 errorCode;

/* unknow address family */
    ipAddrStrPtr = inet_ntop(-1,
                             ipAddr,
                             ipAddrStr,
                             16);
    tm_assert(tfxInetNtop, ipAddrStrPtr == NULL);
    errorCode = tfGetSocketError(-1);
    tm_assert(tfxInetNtop, errorCode == TM_EAFNOSUPPORT)

    ipAddrStrPtr = inet_ntop(1,
                             ipAddr,
                             ipAddrStr,
                             16);
    tm_assert(tfxInetNtop, ipAddrStrPtr == NULL);
    tm_assert(tfxInetNtop, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    ipAddrStrPtr = inet_ntop(3,
                             ipAddr,
                             ipAddrStr,
                             16);
    tm_assert(tfxInetNtop, ipAddrStrPtr == NULL);
    tm_assert(tfxInetNtop, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    ipAddrStrPtr = inet_ntop(27,
                             ipAddr,
                             ipAddrStr,
                             16);
    tm_assert(tfxInetNtop, ipAddrStrPtr == NULL);
    tm_assert(tfxInetNtop, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    ipAddrStrPtr = inet_ntop(29,
                             ipAddr,
                             ipAddrStr,
                             16);
    tm_assert(tfxInetNtop, ipAddrStrPtr == NULL);
    tm_assert(tfxInetNtop, tfGetSocketError(-1) ==TM_EAFNOSUPPORT);


/* AF_INET good */
    tm_memset(ipAddr, 0, TM_TEST_IP_ADDR_LEN);
    tm_memset(ipAddrStr, 0, TM_INET_ADDRSTRLEN); 
    ipAddrStrPtr = inet_ntop(AF_INET,
                             ipAddr,
                             ipAddrStr,
                             TM_INET_ADDRSTRLEN);
    tm_assert(tfxInetNtop, ipAddrStrPtr == ipAddrStr);
    tm_assert(tfxInetNtop, !tm_strcmp(ipAddrStr, "0.0.0.0"));

    tm_memset(ipAddr, 0, TM_TEST_IP_ADDR_LEN);
    ipAddr[0]=htonl(0x10204080);
    tm_memset(ipAddrStr, 0, TM_INET_ADDRSTRLEN); 
    ipAddrStrPtr = inet_ntop(AF_INET,
                             ipAddr,
                             ipAddrStr,
                             TM_INET_ADDRSTRLEN);
    tm_assert(tfxInetNtop, ipAddrStrPtr == ipAddrStr);
    tm_assert(tfxInetNtop, !tm_strcmp(ipAddrStr, "16.32.64.128"));

    tm_memset(ipAddr, 0, TM_TEST_IP_ADDR_LEN);
    ipAddr[0]=htonl(0xffffffff);
    tm_memset(ipAddrStr, 0, TM_INET_ADDRSTRLEN); 
    ipAddrStrPtr = inet_ntop(AF_INET,
                             ipAddr,
                             ipAddrStr,
                             TM_INET_ADDRSTRLEN);
    tm_assert(tfxInetNtop, ipAddrStrPtr == ipAddrStr);
    tm_assert(tfxInetNtop, !tm_strcmp(ipAddrStr, "255.255.255.255"));

/* AF_INET bad src, too big */


/* AF_INET bad, str buffer too small */
    tm_memset(ipAddr, 0, TM_TEST_IP_ADDR_LEN);
    ipAddr[0]=htonl(0x10204080L);
    tm_memset(ipAddrStr, 0, TM_INET_ADDRSTRLEN); 
    ipAddrStrPtr = inet_ntop(AF_INET,
                             ipAddr,
                             ipAddrStr,
                             2);
    tm_assert(tfxInetNtop, ipAddrStrPtr == NULL );
    tm_assert(tfxInetNtop, tfGetSocketError(-1) == TM_ENOSPC);


/* AF_INET6 good */
}



/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuntop.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 44662    Output checksum: 62368
 */
