/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: testing the netstat tool
 * Filename: tuntstat.c
 * Author: Bryan 
 * Date Created: 05/03/2002
 * $Source: test/unittest/tuntstat.c $
 *
 * Modification History
 * $Revision: 6.0.2.3 $
 * $Date: 2011/05/27 12:17:45CDT $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h>

#if defined(TM_USE_NETSTAT) && defined(TM_MULTIPLE_CONTEXT)

#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#include "tutest.h"

/*
 * since printf is not reentrant, it's not recommended for using in 
 * real-time systems, defining this macro would cause printf to be used,
 * should define it only in non-preemptive circumstances
 */
#define TM_USE_PRINTF
 
#ifdef TM_USE_PRINTF
#include <stdio.h>
#endif

#ifdef TM_TEST_NETSTAT_VERBOSE
#define tm_nt_printf printf
#else /* TM_TEST_NETSTAT_VERBOSE*/
#define tm_nt_printf 
#endif /* !TM_TEST_NETSTAT_VERBOSE */

#define TM_TEST_NT_PORT         10

#define TM_TEST_NT_MAX_SOCKET       20
#define TM_TEST_V4_SOCK_INDEX_MIN   0
#define TM_TEST_V4_SOCK_INDEX_MAX   3
#define TM_TEST_V6_SOCK_INDEX_MIN   4
#define TM_TEST_V6_SOCK_INDEX_MAX   7


/*
 * The following 4 functions is a example showing a set of user provided 
 * call back functions for tNetstat. It uses prinf to output the 
 * netstat information tot the STDOUT. 
 * NOTE: please take extra care when use printf, since it's not reentrant,
 *       Use it only for non-preemptive circumstances. 
 *       TM_USE_PRINTF needs to be defined to active prinf.
 */

int tfxNtRteEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{

    char    buffer[TM_NT_ENTRY_STR_LEN];
    int     entryStrLen;

    TM_UNREF_IN_ARG(genParam1);
    TM_UNREF_IN_ARG(genParam2);
/* could be either a ip address or ethernet addr */
    entryStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtRteEntryToStr(&ntEntryUPtr->ntRteEntry,
                      buffer,
                      &entryStrLen);
    tm_nt_printf("%s\n", buffer);

/* 
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries 
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}

int tfxNtArpEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{

    char    buffer[TM_NT_ENTRY_STR_LEN];
    int     entryStrLen;

    TM_UNREF_IN_ARG(genParam1);
    TM_UNREF_IN_ARG(genParam2);
    entryStrLen = TM_NT_ENTRY_STR_LEN;

    tfNtArpEntryToStr(&ntEntryUPtr->ntArpEntry,
                      buffer,
                      &entryStrLen);
    tm_nt_printf("%s\n", buffer);

/* 
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries 
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}



int tfxNtTcpEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{

    char    buffer[TM_NT_ENTRY_STR_LEN];
    int     entryStrLen;

    TM_UNREF_IN_ARG(genParam1);
    TM_UNREF_IN_ARG(genParam2);
    entryStrLen = TM_NT_ENTRY_STR_LEN;

    tfNtTcpEntryToStr(&ntEntryUPtr->ntTcpEntry,
                      buffer,
                      &entryStrLen);
    tm_nt_printf("%s\n", buffer);

/* 
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries 
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}


int tfxNtUdpEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{
    char    buffer[TM_NT_ENTRY_STR_LEN];
    int     entryStrLen;

    TM_UNREF_IN_ARG(genParam1);
    TM_UNREF_IN_ARG(genParam2);

    entryStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtUdpEntryToStr(&ntEntryUPtr->ntUdpEntry,
                      buffer,
                      &entryStrLen);
    tm_nt_printf("%s\n", buffer);

/* 
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries 
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}





int tfxNtDeviceEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{
    char			buffer[TM_NT_ENTRY_STR_LEN];
    ttNtDevEntryPtr ntDevPtr;
    int             i;
    int             size;

    TM_UNREF_IN_ARG(genParam1);
    TM_UNREF_IN_ARG(genParam2);


    ntDevPtr = &(ntEntryUPtr->ntDevEntry);

/* print the basic device information */
    size = TM_NT_ENTRY_STR_LEN;
    tfNtDeviceEntryToStr(ntDevPtr, buffer, &size);
    tm_nt_printf("%s\n", buffer);

    if (ntDevPtr->ntDevStatus != TM_NT_DEVICE_STATUS_CLOSED)
    {
/* print the address infor header */
        size = TM_NT_ENTRY_STR_LEN;
        tfNtGetAddrConfHeaderStr(buffer, &size);
        tm_nt_printf("%s\n", buffer);


/* print out each of the IPv4 address info in a row */
        for (i=0; i<TM_NT_MAX_IP_PER_IF; i++)
        {
            if (   ntDevPtr->ntDevAddrConfArray[i].ntAddrType
                != TM_NT_ADDR_CONF_TYPE_NONE)
            {
                size = TM_NT_ENTRY_STR_LEN;
                tfNtAddrConfEntryToStr(
                    &ntDevPtr->ntDevAddrConfArray[i], 
                    buffer, 
                    &size);
                tm_nt_printf("%s\n", buffer);
            }
        }
    }
    tm_nt_printf("\n");

/* 
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries 
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}






void tfxOpenManySocket()
{
    int                 testSd[TM_TEST_NT_MAX_SOCKET];
    int                 errorCode;
    int                 i;
    struct sockaddr_storage sockAddr;
    int                 sockIndex;
    struct linger       lingerStruct;
 
    lingerStruct.l_onoff = 1;
    lingerStruct.l_linger= 0;

    for (i=0; i< TM_TEST_NT_MAX_SOCKET; i++)
    {
        testSd[i] = -1;
    }
/* these are the sockets for testing only */
    sockIndex = TM_TEST_V4_SOCK_INDEX_MIN;

    testSd[sockIndex] = socket(AF_INET, SOCK_DGRAM, IP_PROTOUDP);
    tm_assert(tfxNetstat, testSd[sockIndex] != TM_SOCKET_ERROR);
    sockIndex++;

    testSd[sockIndex] = socket(AF_INET, SOCK_DGRAM, IP_PROTOUDP);
    tm_assert(tfxNetstat, testSd[sockIndex] != TM_SOCKET_ERROR);
    sockIndex++;

    testSd[sockIndex] = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    tm_assert(tfxNetstat, testSd[sockIndex] != TM_SOCKET_ERROR);
    sockIndex++;

    testSd[sockIndex] = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    tm_assert(tfxNetstat, testSd[sockIndex] != TM_SOCKET_ERROR);

    tm_memcpy(&sockAddr, &tv4ServerSockAddr[0], sizeof(sockAddr));
    for (i =  TM_TEST_V4_SOCK_INDEX_MIN; 
         i <= TM_TEST_V4_SOCK_INDEX_MAX; 
         i++)
    {
        sockAddr.ss_port = htons(TM_TEST_NT_PORT + i + 1);
        errorCode = bind(testSd[i], 
                         (struct sockaddr TM_FAR *)&sockAddr, 
                         sizeof(struct sockaddr));
        if (errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(testSd[i]);
        }
        tm_assert(tfxNetstat, errorCode == TM_ENOERROR);

        errorCode = setsockopt(testSd[i],
                               SOL_SOCKET,
                               SO_LINGER,
                               (char*)&lingerStruct,
                               sizeof(lingerStruct));
        tm_assert(tfxNetstat, errorCode == TM_ENOERROR);
    }
    errorCode = listen(testSd[TM_TEST_V4_SOCK_INDEX_MIN +2], 10);
    tm_assert(tfxNetstat, errorCode == TM_ENOERROR);



/* close all the open sockets */
    for (i = TM_TEST_V4_SOCK_INDEX_MIN;
         i<= TM_TEST_V6_SOCK_INDEX_MAX; 
         i++)
    {
        if(testSd[i] != -1)
        {
            errorCode = tfClose(testSd[i]);
            testSd[i] = -1;
            if (errorCode != TM_ENOERROR)
            {
                errorCode = tfGetSocketError(testSd[i]);
            }
            tm_assert(tfxNetstat, errorCode != TM_SOCKET_ERROR);
        }
    }
}

void tfxNetStat(ttUserContext context)
{
    int                 headerStrLen;
    char                buffer[TM_NT_ENTRY_STR_LEN];
    int                 errorCode;
    ttUserContext       oldConetxt;

    ttUserGenericUnion  genUnion;

    genUnion.gen8BitParm = TM_8BIT_ZERO;

    oldConetxt = tfGetCurrentContext();
    tfSetCurrentContext(context);

/* print the ARP table using netstat tools */
    tm_nt_printf("------------\nARP table\n------------\n");
    headerStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtGetArpHeaderStr(buffer, &headerStrLen);
    tm_assert(tfxNetstat, headerStrLen < TM_NT_ENTRY_STR_LEN );
    tm_nt_printf("%s\n", buffer);
    errorCode = tfNetStat(
                  TM_NT_TABLE_ARP, 
                  tfxNtArpEntryCBPrintf, 
                  genUnion,
                  genUnion);
    tm_assert(tfxNetstat, errorCode == TM_ENOERROR);
    tm_nt_printf("------------\n\n");

/* print the RTE table using netstat tools */
    tm_nt_printf("------------\nRouting table\n------------\n");
    headerStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtGetRteHeaderStr(buffer, &headerStrLen);
    tm_nt_printf("%s\n", buffer);
    errorCode = tfNetStat(
                  TM_NT_TABLE_RTE, 
                  tfxNtRteEntryCBPrintf, 
                  genUnion,
                  genUnion);
    tm_assert(tfxNetstat, errorCode == TM_ENOERROR);
    tm_nt_printf("------------\n\n");

/* Print the Device Table */
    tm_nt_printf("------------\nDevice Table\n------------\n");
    headerStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtGetDeviceHeaderStr(buffer, &headerStrLen);
    tm_nt_printf("%s\n", buffer);
    errorCode = tfNetStat(
                  TM_NT_TABLE_DEVICE, 
                  tfxNtDeviceEntryCBPrintf, 
                  genUnion,
                  genUnion);
    tm_assert(tfxNetstat, errorCode == TM_ENOERROR);
    tm_nt_printf("------------\n\n");

/* print the TCP socket table using netstat tools */
    tm_nt_printf("------------\nTCP sockets\n------------\n");
    headerStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtGetTcpHeaderStr(buffer, &headerStrLen);
    tm_nt_printf("%s------------\n", buffer);
    errorCode = tfNetStat(
                  TM_NT_TABLE_TCP, 
                  tfxNtTcpEntryCBPrintf, 
                  genUnion,
                  genUnion);
    tm_assert(tfxNetstat, errorCode == TM_ENOERROR);
    tm_nt_printf("------------\n\n");


/* print the UDP socket table using netstat tools */
    tm_nt_printf("------------\nUDP sockets\n------------\n");
    headerStrLen = TM_NT_ENTRY_STR_LEN;
    tfNtGetUdpHeaderStr(buffer, &headerStrLen);
    tm_nt_printf("%s\n", buffer);
    errorCode = tfNetStat(
                  TM_NT_TABLE_UDP, 
                  tfxNtUdpEntryCBPrintf, 
                  genUnion,
                  genUnion);
    tm_assert(tfxNetstat, errorCode == TM_ENOERROR);
    tm_nt_printf("------------\n\n");







    tfSetCurrentContext(oldConetxt);

}

#else /* !(TM_USE_NETSTAT && TM_MULTIPLE_CONTEXT) */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

int tlTxNetstatDummy = 0;
#endif /* !(TM_USE_NETSTAT && TM_MULTIPLE_CONTEXT) */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuntstat.c    File Revision: 6.0.2.3 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum:  6447    Output checksum: 27986
 */
