/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Automated test code for Ping application
 *
 * Filename: tuping.c
 * Author: Ed Remmell
 * Date Created:
 * $Source: test/unittest/tuping.c $
 *
 * Modification History
 * $Revision: 6.0.2.4 $
 * $Date: 2011/11/29 16:06:01CST $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include "tutest.h"

/* constant declarations and enumerations */

/* type definitions */

/* local function prototypes */

/* local variable definitions */
static ttUser32Bit tlLastPingTransmittedCount;
static ttUser32Bit tlLastPingReceivedCount;

/* static const tables */

/* macro definitions */

#ifndef TM_SINGLE_INTERFACE_HOME
static void tfxPingCB(int pingSd)
{
    ttPingInfo pingInfo;

    tfPingGetStatistics(pingSd, &pingInfo);

    if (pingInfo.pgiSendErrorCode == TM_ENOERROR)
    {
        if (tlLastPingTransmittedCount != pingInfo.pgiTransmitted)
        {
/* sent Echo Request to gateway */
            tlLastPingTransmittedCount = pingInfo.pgiTransmitted;
        }
    }

    if (pingInfo.pgiRecvErrorCode == TM_ENOERROR)
    {
        if (tlLastPingReceivedCount != pingInfo.pgiReceived)
        {
            tlLastPingReceivedCount = pingInfo.pgiReceived;

            printf("\nReceived reply to ping! sent: %u, received %u",
                tlLastPingTransmittedCount, tlLastPingReceivedCount);

            if (tlLastPingReceivedCount == 2)
            {
                tfPingClose(pingSd);
            }
        }
    }
}
#endif /* ! TM_SINGLE_INTERFACE_HOME */

#ifdef TM_MULTIPLE_CONTEXT

#ifndef TM_SINGLE_INTERFACE_HOME
static void tfxPing1 (ttCharPtr ipAddrStr);
#endif /* !TM_SINGLE_INTERFACE_HOME */
/****************************************************************************
* FUNCTION: tfxPing
*
* PURPOSE: reproduce bug ID 1209. When TM_LOOP_TO_DRIVER is not #define'd,
*	tfKernelPendCountSem should *NOT* be called.
*
* PARAMETERS:
*   None
*
* RETURNS:
*   Nothing
*
* NOTES:
*
****************************************************************************/
void tfxPing (int route)
{
    ttUserContext   oldContext;
#ifndef TM_SINGLE_INTERFACE_HOME
#ifndef TM_4_USE_SCOPE_ID
    struct sockaddr_storage TM_FAR * serverSockAddrPtr;
    struct sockaddr_storage TM_FAR * serverSockAddr1Ptr;
    char            serverAddrStringPtr[INET_ADDRSTRLEN];
    char            serverAddr1StringPtr[INET_ADDRSTRLEN];
    int             serverSockAddrLen;
#endif /* !TM_4_USE_SCOPE_ID */
#endif /* !TM_SINGLE_INTERFACE_HOME */
    int             errorCode;
    
    oldContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleServer);
    errorCode = tfSetTreckOptions(TM_OPTION_ICMP_ROUTE_REPLY, (tt32Bit)route);
    tm_assert(tfxPing, errorCode == TM_ENOERROR);
    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfSetTreckOptions(TM_OPTION_ICMP_ROUTE_REPLY, (tt32Bit)route);
    tm_assert(tfxPing, errorCode == TM_ENOERROR);
    
#ifndef TM_SINGLE_INTERFACE_HOME
/* try pinging the loopback address, and then in the ping callback we call
   tfPingClose. This used to result in a deadlock. */

    tfxPing1("127.0.0.1");
#ifndef TM_4_USE_SCOPE_ID
#ifndef TM_LOOP_TO_DRIVER
/* Ping to our own address */
/*
 * We do not run this test if TM_LOOP_TO_DRIVER is defined because the
 * autotest IntfMCtxDrv driver does not support looping back packets.
 */
    tfxGetClientSockAddr(2,
                         AF_INET,
                         (struct sockaddr**)&serverSockAddrPtr,
                         &serverSockAddrLen);
    inet_ntop(AF_INET,
              &(serverSockAddrPtr->addr.ipv4.sin_addr),
              serverAddrStringPtr,
              sizeof(serverAddrStringPtr));
    tfxPing1(serverAddrStringPtr);
#endif /* TM_LOOP_TO_DRIVER */

    tfxGetServerSockAddr(2,
                         AF_INET,
                         (struct sockaddr**)&serverSockAddrPtr,
                         &serverSockAddrLen);
    inet_ntop(AF_INET,
              &(serverSockAddrPtr->addr.ipv4.sin_addr),
              serverAddrStringPtr,
              sizeof(serverAddrStringPtr));
    tfxPing1(serverAddrStringPtr);
/* Add a default gateway that points to the other server */
    tfxGetServerSockAddr(3,
                         AF_INET,
                         (struct sockaddr**)&serverSockAddr1Ptr,
                         &serverSockAddrLen);
    inet_ntop(AF_INET,
              &(serverSockAddr1Ptr->addr.ipv4.sin_addr),
              serverAddr1StringPtr,
              sizeof(serverAddr1StringPtr));
    errorCode = tfAddDefaultGateway(
                    tvInterfaceHandleClient, 
                    serverSockAddr1Ptr->addr.ipv4.sin_addr.s_addr
#ifdef TM_USE_STRONG_ESL
                  , 1
#endif /* TM_USE_STRONG_ESL */
                    );
    tm_assert(tfxPing, errorCode == TM_ENOERROR);
    if (errorCode == TM_ENOERROR)
    {
/* Add a static route that will try and remove the ARP entry */
        errorCode = tfAddStaticRoute(tvInterfaceHandleClient,
                    serverSockAddrPtr->addr.ipv4.sin_addr.s_addr,
                    htonl(0xFFFFFFFF),
                    serverSockAddr1Ptr->addr.ipv4.sin_addr.s_addr,
                    1);
        tm_assert(tfxPing, errorCode == TM_ENOERROR);
        if (errorCode == TM_ENOERROR)
        {
            tfxPing1(serverAddrStringPtr);
            tfDelStaticRoute(
#ifdef TM_USE_STRONG_ESL
                        tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                        serverSockAddrPtr->addr.ipv4.sin_addr.s_addr,
                        htonl(0xFFFFFFFF)
                        );
        }
        tfDelDefaultGateway(
#ifdef TM_USE_STRONG_ESL
                tvInterfaceHandleClient,
#endif /* TM_USE_STRONG_ESL */
                serverSockAddr1Ptr->addr.ipv4.sin_addr.s_addr);
    }
#endif /* !TM_4_USE_SCOPE_ID */
#endif /* !TM_SINGLE_INTERFACE_HOME */
    tfSetCurrentContext(oldContext);
    return;
}

#ifndef TM_SINGLE_INTERFACE_HOME
static void tfxPing1 (ttCharPtr ipAddrStr)
{
	int             pingSd;

    tlLastPingReceivedCount = 0;
    tlLastPingTransmittedCount = 0;
    pingSd = tfPingOpenStart(
        ipAddrStr, 0, 0, (ttPingCBFuncPtr) tfxPingCB);
    tm_assert(tfxPing1, pingSd != TM_SOCKET_ERROR);
    if (pingSd != TM_SOCKET_ERROR)
    {
#ifdef TM_USE_ECHO_CONFIG
        if (tm_context(tvIcmpEchoEnable) != (TM_8BIT_ZERO))
#endif /* TM_USE_ECHO_CONFIG */
        {
            while(tlLastPingReceivedCount != 2)
            {
                tfTimerUpdate();
                tfTimerExecute();
            }
        }
    }
    tfPingClose(pingSd);
}
#endif /* !TM_SINGLE_INTERFACE_HOME */
#endif /* TM_MULTIPLE_CONTEXT */

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:53      For: Xilinx-Samsung
 * File Name: tuping.c      File Revision: 6.0.2.4 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 52692    Output checksum: 15257
 */
