/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: PPP automated test code.
 *
 * Filename: tuppp.c
 * Author: Jason Schmidlapp
 * Date Created:
 * $Source: test/unittest/tuppp.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:42CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#if defined(TM_MULTIPLE_CONTEXT) && defined(TM_USE_PPP)
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include "tutest.h"

ttUserInterface tvPPPInterfaceHandleClient;
ttUserInterface tvPPPInterfaceHandleServer;
/* constant declarations and enumerations */

/* type definitions */

/* Test functions */
static void tfxPppAuthTest(int serverAuthProto, int clientAuthProto);
static void tfxPppAuthMethodTest(int serverAuthMethod1, 
                                 int serverAuthMethod2,
                                 int serverAuthMethod3);
static void tfxPppChapAuthTest(void);
static void tfxPppEapAuthTest(void);

static void tfx4PppVjCompTest(void);
static void tfx4PppIphcTest(void);
static void tfx4PppBasicTest(void);
static void tfx6PppIphcTest(void);
static void tfx6PppBasicTest(void);

/* Utility functions */
static void tfxPppSetupPap(ttUserInterface authServerInterface,
                           ttUserContext   authServerContext,
                           ttUserInterface authClientInterface,
                           ttUserContext   authClientContext);
static void tfxPppSetupChap(ttUserInterface authServerInterface,
                            ttUserContext   authServerContext,
                            ttUserInterface authClientInterface,
                            ttUserContext   authClientContext);
static void tfxPppSetupEap(ttUserInterface authServerInterface,
                           ttUserContext   authServerContext,
                           ttUserInterface authClientInterface,
                           ttUserContext   authClientContext);
static void tfxPppServerNotify(
    ttUserInterface interfaceHandle, unsigned int flags);
static void tfxPppClientNotify(
    ttUserInterface interfaceHandle, unsigned int flags);   
static void tfx4PppSetupAddr(ttUserInterface interfaceHandle,
                             char TM_FAR *   ipAddrStr);
static void tfx4PppStart(void);
static void tfx6PppStart(void);
static void tfxPppStop(void);
static void tfx4PppEnableVj(ttUserInterface interfaceHandle);
static void tfxPppEnableIphc(ttUserInterface interfaceHandle, int family);

#ifdef TM_USE_TCP
static void tfxPppTestCompression(int family,
                                  int testCount,
                                  int maxBufSize,
                                  int timestampFlag);

static int tfxVerifyBuffer(tt8BitPtr srcBufPtr,
                           tt8BitPtr srcDestPtr,
                           int       bufLength);
static void tfxFillBufferRandom(tt8BitPtr bufPtr, int bufLength);
#endif /* TM_USE_TCP */


/* local variable definitions */
int tlServerPppFlags;
int tlClientPppFlags;
int tl6ClientAddrConfig;
int tl6ServerAddrConfig;

char tlPapUsername[]    = "pap_username";
char tlPapPassword[]    = "pap_password";

char tlChapUsername[]   = "chap_username";
char tlChapSecret[]     = "chap_secret";
char tlChapServername[] = "chap_servername";

char tlEapMd5Username[] = "eap_md5_username";
char tlEapMd5Secret[]   = "eap_md5_secret";
char tlEapMd5Servername[] = "eap_servername";

char tlEapLocalIdent[]  = "eap_identity";

/* static const tables */

/* macro definitions */
#define TM_PPP_CLIENT_ADDRESS "10.2.2.2"
#define TM_PPP_SERVER_ADDRESS "10.1.1.1"

#define TM_6_PPP_CLIENT_ADDRESS "3ffe::2"
#define TM_6_PPP_SERVER_ADDRESS "3ffe::1"

#define TM_MAX_IPHC_BUF 1200
#define TM_IPHC_TEST_COUNT 25

#define TM_MAX_VJ_BUF 1200
#define TM_VJ_TEST_COUNT 25

#define NONE 0

void tfxPppAdInterfaces(void)
{
    ttUserLinkLayer linkLayerHandleClient;
    ttUserLinkLayer linkLayerHandleServer;
    int             errorCode;
#ifdef TM_USE_VCHAN
    int             mhomeIndex;
#endif /* TM_USE_VCHAN */
    
/* Setup PPP client */
    tfSetCurrentContext(tvContextHandleClient);

    linkLayerHandleClient = tfUseAsyncPpp(tfxPppClientNotify);

    if (linkLayerHandleClient != (ttUserLinkLayer)0)
    {
        tvPPPInterfaceHandleClient = tfUseIntfMCtxDriver(
                                    "Ppp.Client", 
                                     linkLayerHandleClient, 
                                     &errorCode);
        tm_assert(tfxPppAdInterfaces, errorCode == TM_ENOERROR);

#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
        for (mhomeIndex = 0;
             mhomeIndex < TM_MAX_IPS_PER_IF;
             mhomeIndex++)
        {
            errorCode = tfNgInterfaceSetVirtualChannel(
                tvPPPInterfaceHandleClient,
                (ttUser32Bit) mhomeIndex,
                AF_INET,
                mhomeIndex );
            tm_assert(tfxPppAdInterfaces, errorCode == TM_ENOERROR);
        }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
    }
    else
    {
        tvPPPInterfaceHandleClient = (ttUserInterface)0;
    }
    
/* Setup PPP server */
    tfSetCurrentContext(tvContextHandleServer);

    linkLayerHandleServer = tfUseAsyncServerPpp(tfxPppServerNotify);

    if (linkLayerHandleServer != (ttUserLinkLayer)0)
    {
        tvPPPInterfaceHandleServer = tfUseIntfMCtxDriver(
                                    "Ppp.Server", 
                                     linkLayerHandleServer, 
                                     &errorCode);
        tm_assert(tfxPppAdInterfaces, errorCode == TM_ENOERROR);    

#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
        for (mhomeIndex = 0;
             mhomeIndex < TM_MAX_IPS_PER_IF;
             mhomeIndex++)
        {
            errorCode = tfNgInterfaceSetVirtualChannel(
                tvPPPInterfaceHandleServer,
                (ttUser32Bit) mhomeIndex,
                AF_INET,
                mhomeIndex );
            tm_assert(tfxPppAdInterfaces, errorCode == TM_ENOERROR);
        }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
    }
    else
    {
        tvPPPInterfaceHandleServer = (ttUserInterface)0;
    }

}

void tfxPppAutoTest(void)
{
    if (tvPPPInterfaceHandleClient != (ttUserInterface)0)
    {
/*
 * Authentication tests.
 */

/* PAP <--> No auth */
        tm_test_with_2_param(tfxPppAuthTest,
                                 TM_PPP_PAP_PROTOCOL,
                                 NONE);
/* PAP <--> PAP */
        tm_test_with_2_param(tfxPppAuthTest,
                                 TM_PPP_PAP_PROTOCOL,
                                 TM_PPP_PAP_PROTOCOL);

/* CHAP <--> No auth */
        tm_test_with_2_param(tfxPppAuthTest,
                                 TM_PPP_CHAP_PROTOCOL,
                                 NONE);

/* CHAP <--> CHAP */
        tm_test_with_2_param(tfxPppAuthTest,
                                 TM_PPP_CHAP_PROTOCOL,
                                 TM_PPP_CHAP_PROTOCOL);

/* PAP <--> CHAP */
        tm_test_with_2_param(tfxPppAuthTest,
                                 TM_PPP_PAP_PROTOCOL,
                                 TM_PPP_CHAP_PROTOCOL);

/* NOTE: You must support CHAP in order to support MSCHAP
 *
 * Server wants PAP, standard CHAP, MS-CHAPv1 and in this order
 * Client only allows MS-CHAPv1
 * Server sends PAP config first, Client NAKs it with MS-CHAPv1
 * Server sends CHAP config again, Client NAKs it with MS-CHAPv1
 * Server sends MS-CHAPv1 config, Client ACKs it.
 */

        


/* Basic IPv4 over PPP test. */
        tm_test(tfx4PppBasicTest);
        
/* IPv4 Van Jacobson header compression test */
        tm_test(tfx4PppVjCompTest);

/* IPv4 RFC-2507 header compression test. */
        
    }
    else
    {
        tfKernelPrint(TM_PRINT_TYPE_FAIL,
                     "\nNot running PPP tests - PPP code not available\n");
    }
}

/* Server wants PAP, standard CHAP, MS-CHAPv1 and in this order
 * Client only allows MS-CHAPv1
 * Server sends PAP config first, Client NAKs it with MS-CHAPv1
 * Server sends CHAP config again, Client NAKs it with MS-CHAPv1
 * Server sends MS-CHAPv1 config, Client ACKs it.
 */


static void tfxPppAuthTest(int serverAuthProto, int clientAuthProto)
{
    switch (serverAuthProto)
    {
        case TM_PPP_PAP_PROTOCOL:
            tfxPppSetupPap(tvPPPInterfaceHandleServer,
                           tvContextHandleServer,
                           tvPPPInterfaceHandleClient,
                           tvContextHandleClient);
            break;

        case TM_PPP_CHAP_PROTOCOL:
            tfxPppSetupChap(tvPPPInterfaceHandleServer,
                            tvContextHandleServer,
                            tvPPPInterfaceHandleClient,
                            tvContextHandleClient);
            break;

    }

    switch (clientAuthProto)
    {
        case TM_PPP_PAP_PROTOCOL:
            tfxPppSetupPap(tvPPPInterfaceHandleClient,
                           tvContextHandleClient,
                           tvPPPInterfaceHandleServer,
                           tvContextHandleServer);
            break;

        case TM_PPP_CHAP_PROTOCOL:
            tfxPppSetupChap(tvPPPInterfaceHandleClient,
                            tvContextHandleClient,
                            tvPPPInterfaceHandleServer,
                            tvContextHandleServer);
            break;

    }

    
    tfx4PppStart();

    tfxPppStop();

}


static int tfxPapAuthCallback(char TM_FAR * username,
                              char TM_FAR * password)
{
    int errorCode;
    int serverLength, clientLength;

    serverLength = (int)tm_strlen(tlPapUsername);
    clientLength = (int)tm_strlen(username);
    tm_assert(tfxPapAuthCallback, serverLength == clientLength);
    
    errorCode = tm_memcmp(username, tlPapUsername, clientLength);
    tm_assert(tfxPapAuthCallback, errorCode == 0);

    serverLength = (int)tm_strlen(tlPapPassword);
    clientLength = (int)tm_strlen(password);
    tm_assert(tfxPapAuthCallback, serverLength == clientLength);
    
    errorCode = tm_memcmp(password, tlPapPassword, clientLength);
    tm_assert(tfxPapAuthCallback, errorCode == 0);

    return 1;
}

static void tfxPppSetupPap(ttUserInterface authServerInterface,
                           ttUserContext   authServerContext,
                           ttUserInterface authClientInterface,
                           ttUserContext   authClientContext)
{
    int     optLength;
    int     errorCode;
    tt8Bit  opt8;
    tt16Bit opt16;
    
    if(authClientInterface)
    {
/* Enable PAP on the client side, setup username & password. */
    tfSetCurrentContext(authClientContext);

    opt16 = TM_PPP_PAP_PROTOCOL;
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_ALLOW,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppSetupPap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlPapUsername);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_PAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_PAP_USERNAME,
                                tlPapUsername,
                                optLength );
    tm_assert(tfxPppSetupPap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlPapPassword);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_PAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_PAP_PASSWORD,
                                tlPapPassword,
                                optLength );
    tm_assert(tfxPppSetupPap, errorCode == TM_ENOERROR);

    opt8 = 1;
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_PAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_PAP_TIMEOUT,
                                (const char TM_FAR *) &opt8,
                                sizeof(opt8) );
    tm_assert(tfxPppSetupPap, errorCode == TM_ENOERROR);

    }
/* Setup PAP on server side, and register PAP authentication routine. */
    tfSetCurrentContext(authServerContext);

    opt16 = TM_PPP_PAP_PROTOCOL;
    errorCode = tfPppSetOption( authServerInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppSetupPap, errorCode == TM_ENOERROR);

   
    errorCode = tfPapRegisterAuthenticate( authServerInterface,
                                           tfxPapAuthCallback );
    tm_assert(tfxPppSetupPap, errorCode == TM_ENOERROR);

}


static char TM_FAR *tfxChapAuthCallback(char TM_FAR * username)
{
    int errorCode;
    int serverLength, clientLength;

    serverLength = (int)tm_strlen(tlChapUsername);
    clientLength = (int)tm_strlen(username);
    tm_assert(tfxChapAuthCallback, serverLength == clientLength);
    
    errorCode = tm_memcmp(username, tlChapUsername, clientLength);
    tm_assert(tfxChapAuthCallback, errorCode == 0);

    return tlChapSecret;
}




static void tfxPppSetupChap(ttUserInterface authServerInterface,
                            ttUserContext   authServerContext,
                            ttUserInterface authClientInterface,
                            ttUserContext   authClientContext)
{
    int     optLength;
    int     errorCode;
    tt16Bit opt16;

    if(authClientInterface)
    {

/* Enable CHAP on the client side, setup username & password. */
    tfSetCurrentContext(authClientContext);

    opt16 = TM_PPP_CHAP_PROTOCOL;
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_ALLOW,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppSetupChap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlChapUsername);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_CHAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_CHAP_USERNAME,
                                tlChapUsername,
                                optLength );
    tm_assert(tfxPppSetupChap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlChapSecret);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_CHAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_CHAP_SECRET,
                                tlChapSecret,
                                optLength );
    tm_assert(tfxPppSetupChap, errorCode == TM_ENOERROR);

    }
/* Setup PAP on server side, and register PAP authentication routine. */
    tfSetCurrentContext(authServerContext);

    opt16 = TM_PPP_CHAP_PROTOCOL;
    errorCode = tfPppSetOption( authServerInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppSetupChap, errorCode == TM_ENOERROR);

    optLength = (int)tm_strlen(tlChapServername);
    errorCode = tfPppSetOption( authServerInterface,
                                TM_PPP_CHAP_PROTOCOL,
                                TM_PPP_OPT_ALLOW,
                                TM_CHAP_USERNAME,
                                tlChapServername,
                                optLength );
    tm_assert(tfxPppSetupChap, errorCode == TM_ENOERROR);

    errorCode = tfChapRegisterAuthenticate( authServerInterface,
                                            tfxChapAuthCallback );

    
    tm_assert(tfxPppSetupChap, errorCode == TM_ENOERROR);

}




static void tfx4PppBasicTest(void)
{
    tfSetCurrentContext(tvContextHandleServer);
    tfx4PppSetupAddr(tvPPPInterfaceHandleServer, TM_PPP_SERVER_ADDRESS);

    tfSetCurrentContext(tvContextHandleClient);
    tfx4PppSetupAddr(tvPPPInterfaceHandleClient, TM_PPP_CLIENT_ADDRESS);
    
#if (defined(TM_USE_IPV4) && !defined(TM_4_USE_SCOPE_ID))
    tfx4PppStart();
#else /* !TM_USE_IPV4 && TM_4_USE_SCOPE_ID */
    tfx6PppStart();
#endif /* TM_USE_IPV4 && !TM_4_USE_SCOPE_ID */

#ifdef TM_USE_TCP
/* Run the compression test, just with no compression enabled. */
    tfxPppTestCompression(
#ifdef TM_4_USE_SCOPE_ID
                          AF_INET6,
#else /* TM_4_USE_SCOPE_ID */
                          AF_INET,
#endif /* TM_4_USE_SCOPE_ID */
                          TM_IPHC_TEST_COUNT,                          
                          TM_MAX_IPHC_BUF,
                          1); /* enable TCP timestamps. */
#endif /* TM_USE_TCP */
    
    tfxPppStop();

    tfSetCurrentContext(tvContextHandleClient);
}


static void tfx4PppVjCompTest(void)
{

    tfSetCurrentContext(tvContextHandleServer);
    tfx4PppSetupAddr(tvPPPInterfaceHandleServer, TM_PPP_SERVER_ADDRESS);
    tfx4PppEnableVj(tvPPPInterfaceHandleServer);

    tfSetCurrentContext(tvContextHandleClient);
    tfx4PppSetupAddr(tvPPPInterfaceHandleClient, TM_PPP_CLIENT_ADDRESS);
    tfx4PppEnableVj(tvPPPInterfaceHandleClient);
    
#if (defined(TM_USE_IPV4) && !defined(TM_4_USE_SCOPE_ID))
    tfx4PppStart();
#else /* !TM_USE_IPV4 && TM_4_USE_SCOPE_ID */
    tfx6PppStart();
#endif /* TM_USE_IPV4 && !TM_4_USE_SCOPE_ID */

#ifdef TM_USE_TCP
    tfxPppTestCompression(
#ifdef TM_4_USE_SCOPE_ID
                          AF_INET6,
#else /* TM_4_USE_SCOPE_ID */
                          AF_INET,
#endif /* TM_4_USE_SCOPE_ID */
                          TM_VJ_TEST_COUNT,                          
                          TM_MAX_VJ_BUF,
                          0); /* disable TCP timestamps. */
#endif /* TM_USE_TCP */
    
    tfxPppStop();

}

static void tfx4PppEnableVj(ttUserInterface interfaceHandle)
{
    int     errorCode;
    tt16Bit opt16;

    opt16 = TM_PPP_COMP_TCP_PROTOCOL;
    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_WANT,
                               TM_IPCP_COMP_PROTOCOL,
                               (const char TM_FAR *) &opt16,
                               sizeof(opt16));

    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_ALLOW,
                               TM_IPCP_COMP_PROTOCOL,
                               (const char TM_FAR *) &opt16,
                               sizeof(opt16));
}

/*
 * Sets the IPv4 address to request from the peer, and allows the peer
 * to set their own IP address.
 */ 
static void tfx4PppSetupAddr(ttUserInterface interfaceHandle,
                             char TM_FAR *   ipAddrStr)
{
    int errorCode;
    tt4IpAddress ipAddr;

/* Set the IP address to request from the peer. */
    ipAddr = inet_addr(ipAddrStr);
    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_WANT,
                               TM_IPCP_IP_ADDRESS,
                               (const char TM_FAR *) &ipAddr,
                               sizeof(ipAddr));
    tm_assert(tfx4PppSetupAddr, errorCode == TM_ENOERROR);

/* Allow the peer to set their address. */
    ipAddr = TM_UL(0);
    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_ALLOW,
                               TM_IPCP_IP_ADDRESS,
                               (const char TM_FAR *) &ipAddr,
                               sizeof(ipAddr));
    tm_assert(tfx4PppSetupAddr, errorCode == TM_ENOERROR);    
}


static void tfx4PppStart(void)
{
    tt4IpAddress ipAddr;    
    int          flagsOK;
    int          errorCode;
    int          i;

    
/* Start PPP server. */
    tfSetCurrentContext(tvContextHandleServer);    
    tlServerPppFlags = 0;
    ipAddr = inet_addr(TM_PPP_SERVER_ADDRESS);
    errorCode = tfOpenInterface(tvPPPInterfaceHandleServer,
                                ipAddr,
                                TM_IP_HOST_MASK,
                                TM_DEV_SCATTER_SEND_ENB,
                                1500);
    tm_assert(tfx4PppStart, errorCode == TM_EINPROGRESS);

/* Start PPP client. */
    tlClientPppFlags = 0;
    tfSetCurrentContext(tvContextHandleClient);
    ipAddr = inet_addr(TM_PPP_CLIENT_ADDRESS);
    errorCode = tfOpenInterface(tvPPPInterfaceHandleClient,
                                ipAddr,
                                TM_IP_HOST_MASK,
                                TM_DEV_SCATTER_SEND_ENB,
                                1500);
    tm_assert(tfx4PppStart, errorCode == TM_EINPROGRESS);

    while (! (tm_16bit_all_bits_set(tlServerPppFlags,
                              (TM_LL_LCP_UP | TM_LL_IP4_OPEN_COMPLETE |
                               TM_LL_OPEN_COMPLETE))) ||
           ! (tm_16bit_all_bits_set(tlClientPppFlags,
                              (TM_LL_LCP_UP | TM_LL_IP4_OPEN_COMPLETE |
                               TM_LL_OPEN_COMPLETE)))
          )
    {
        tfSetCurrentContext(tvContextHandleClient);
        tfTimerUpdate();
        tfTimerExecute();

        tfSetCurrentContext(tvContextHandleServer);
        tfTimerUpdate();
        tfTimerExecute();

/* Make sure that the connection hasn't been closed (because of excess
   retransmissions, for instance).  This would indicate a failure of this
   test. */
        tm_assert(tfx4PppStart, !(tlClientPppFlags & TM_LL_CLOSE_COMPLETE));
        tm_assert(tfx4PppStart, !(tlServerPppFlags & TM_LL_CLOSE_COMPLETE));
    }
    
/* Verify that the proper notifications were sent to PPP server. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlServerPppFlags,
                              (TM_LL_LCP_UP | TM_LL_IP4_OPEN_COMPLETE |
                               TM_LL_OPEN_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfx4PppStart, flagsOK == 1);

/* Verify that the proper notifications were sent to the PPP client. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlClientPppFlags,
                              (TM_LL_LCP_UP | TM_LL_IP4_OPEN_COMPLETE |
                               TM_LL_OPEN_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfx4PppStart, flagsOK == 1);

    for (i=0;i<=1;i++)
    {
        tm_strcpy(tv4ClientIpAddrStr[i], TM_PPP_CLIENT_ADDRESS);
        tm_memset(&tv4ClientSockAddr[i], 0, sizeof(struct sockaddr_storage));
        tv4ClientSockAddr[i].ss_family = AF_INET;
        tv4ClientSockAddr[i].ss_len = sizeof(struct sockaddr_storage);
        errorCode = inet_pton(AF_INET,
                              TM_PPP_CLIENT_ADDRESS,
                              &tv4ClientSockAddr[i].addr.ipv4.sin_addr);
        tm_assert(tfxPppAdInterfaces, errorCode == 1);
    }

    for (i=0;i<=1;i++)
    {
        tm_strcpy(tv4ServerIpAddrStr[i], TM_PPP_SERVER_ADDRESS);
        tm_memset(&tv4ServerSockAddr[i], 0, sizeof(struct sockaddr_storage));
        tv4ServerSockAddr[i].ss_family = AF_INET;
        tv4ServerSockAddr[i].ss_len = sizeof(struct sockaddr_storage);
        tv4ServerSockAddr[i].addr.ipv4.sin_port = htons(4001);        
        errorCode = inet_pton(AF_INET,
                              TM_PPP_SERVER_ADDRESS,
                              &tv4ServerSockAddr[i].addr.ipv4.sin_addr);
        tm_assert(tfxPppAdInterfaces, errorCode == 1);
    }
    
}

#ifdef TM_USE_TCP
static void tfxPppTestCompression(int family,
                                  int testCount,
                                  int maxBufSize,
                                  int timestampFlag)
{
    int serverSd;
    int acceptSd;
    int clientSd;
    int sockAddrLength;
    int errorCode;
    struct sockaddr_storage *sockAddrPtr;
    tt8BitPtr outDataPtr, inDataPtr;
    struct sockaddr_storage fromAddr;
    int fromAddrLen;
    int i;
    int bufLength;
    
/* Allocate working buffers. */
    tfSetCurrentContext(tvContextHandleClient);    
    outDataPtr = (tt8BitPtr) tm_get_raw_buffer(TM_MAX_IPHC_BUF);
    tm_assert(tfxPppTestCompression, outDataPtr != TM_8BIT_NULL_PTR);

    inDataPtr = (tt8BitPtr) tm_get_raw_buffer(TM_MAX_IPHC_BUF);
    tm_assert(tfxPppTestCompression, inDataPtr != TM_8BIT_NULL_PTR);

    tfxGetTcpConnection(tvPPPInterfaceHandleClient,
                        0,
                        &serverSd,
                        &acceptSd,
                        &clientSd,
                        family,
                        timestampFlag,
#ifdef TM_USE_STRONG_ESL
                        "Ppp.Client", "Ppp.Server",
#endif /* TM_USE_STRONG_ESL */
                        0);

    tfxGetServerSockAddr(0,
                         family,
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);

/* Test #1: Random sized packets sent in both directions. */
    for (i=0;i<testCount;i++)
    {

/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = send(clientSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfxLoopRecv( acceptSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);

/* Send data from server to client. */
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);

        errorCode = send(acceptSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from server to client. */
        tfSetCurrentContext(tvContextHandleClient);
        errorCode = tfxLoopRecv( clientSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);
    }

/*
 * Test #2: Special case #1 (SWU): typing traffic (TCP echo)
 *          NOTE: it is important that the length sent is a full
 *          sized segment, in order to avoid triggering TCP's
 *          "Silly Window Syndrome" prevention, which would prevent
 *          the window size from being updating for every packet.  This
 *          will cause the window size to change with every packet, and
 *          prevent the use of the SWU special case.
 */
    bufLength = maxBufSize; /* cannot be bigger than size of allocated buffer */
    for (i=0;i<testCount;i++)
    {
/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = send(clientSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfxLoopRecv( acceptSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);

/* Echo data back from server back to client. */
        errorCode = send(acceptSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from server to client. */
        tfSetCurrentContext(tvContextHandleClient);
        errorCode = tfxLoopRecv( clientSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);
    }

/* Test #3: Special case #2 (SAWU): bulk data transfer (e.g. FTP) */
    bufLength = 512;
    for (i=0;i<TM_IPHC_TEST_COUNT;i++)
    {
/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = send(clientSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfxLoopRecv( acceptSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);
    }
    
    tfxCloseTcpConnection(serverSd, acceptSd, clientSd);
    
    tfSetCurrentContext(tvContextHandleServer);


/* Test #4: bi-directional UDP data, random sized. */
    tfxGetNonConnectedUdpPair(&serverSd, &clientSd, family, 1
#ifdef TM_USE_STRONG_ESL
                             , "Ppp.Client", "Ppp.Server"
#endif /* TM_USE_STRONG_ESL */
                             );
    tfxGetServerSockAddr(1,
                         family,
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);

    for (i=0;i<testCount;i++)
    {

/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = sendto(clientSd,
                           (ttCharPtr) outDataPtr,
                           bufLength,
                           0,
                           (struct sockaddr TM_FAR *) sockAddrPtr,
                           sockAddrLength);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        fromAddrLen = sizeof(fromAddr);
        errorCode = recvfrom(serverSd,
                             (ttCharPtr) inDataPtr,
                             bufLength,
                             0,
                             (struct sockaddr TM_FAR *) &fromAddr,
                             &fromAddrLen);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);

/* Send data from server to client. */
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);

        errorCode = sendto(serverSd,
                           (ttCharPtr) outDataPtr,
                           bufLength,
                           0,
                           (struct sockaddr TM_FAR *) &fromAddr,
                           fromAddrLen);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

/* Receive data from server to client. */
        tfSetCurrentContext(tvContextHandleClient);
        fromAddrLen = sizeof(fromAddr);
        errorCode = recvfrom(clientSd,
                             (ttCharPtr) inDataPtr,
                             bufLength,
                             0,
                             (struct sockaddr TM_FAR *) &fromAddr,
                             &fromAddrLen);
        tm_assert(tfxPppTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppTestCompression, errorCode == TM_ENOERROR);
    }
    
    tfxCloseUdpConnection(serverSd, clientSd);

/* Free working buffers. */
    tm_free_raw_buffer(outDataPtr);
    tm_free_raw_buffer(inDataPtr);
              
}
#endif /* TM_USE_TCP */


/*
 * Enables IP header compression on the given interface.
 */

static void tfxPppClientNotify(
    ttUserInterface interfaceHandle, unsigned int flags)
{
    interfaceHandle=interfaceHandle;
    tlClientPppFlags |= flags;
}

static void tfxPppServerNotify(
    ttUserInterface interfaceHandle, unsigned int flags)
{
    interfaceHandle=interfaceHandle;
    tlServerPppFlags |= flags;
}

static void tfxPppStop(void)
{
    int errorCode;
    int flagsOK;
    
    tlServerPppFlags = 0;
    tlClientPppFlags = 0;
    
    tfSetCurrentContext(tvContextHandleServer);    

    errorCode = tfCloseInterface(tvPPPInterfaceHandleServer);
    tm_assert(tfxPppStop, errorCode == TM_EINPROGRESS);

    tfSetCurrentContext(tvContextHandleClient);    
    errorCode = tfCloseInterface(tvPPPInterfaceHandleClient);

    tm_assert(tfxPppStop, errorCode == TM_ENOERROR);

    while ( !(tlClientPppFlags & TM_LL_CLOSE_COMPLETE))
    {
        tfSetCurrentContext(tvContextHandleClient);
        tfTimerUpdate();
        tfTimerExecute();

        tfSetCurrentContext(tvContextHandleServer);
        tfTimerUpdate();
        tfTimerExecute();
    }
    
/* Verify that the proper notifications were sent to PPP server. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlServerPppFlags,
                              (TM_LL_CLOSE_STARTED | TM_LL_CLOSE_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfxPppStop, flagsOK == 1);

/* Verify that the proper notifications were sent to the PPP client. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlClientPppFlags,
                              (TM_LL_CLOSE_STARTED | TM_LL_CLOSE_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfxPppStop, flagsOK == 1);
    
}

#ifdef TM_USE_TCP

static void tfxFillBufferRandom(tt8BitPtr bufPtr, int bufLength)
{
    int    i;
    tt8Bit randomChar;

    for (i=0;i<bufLength;i++)
    {
        randomChar = (tt8Bit) (tfGetRandom() % 255);
        bufPtr[i] = randomChar;
    }
}

static int tfxVerifyBuffer(tt8BitPtr srcBufPtr,
                           tt8BitPtr srcDestPtr,
                           int       bufLength)
{
    int i;
    int errorCode;

    errorCode = TM_ENOERROR;
    for (i=0;i<bufLength;i++)
    {
        if (srcBufPtr[i] != srcDestPtr[i])
        {
            errorCode = TM_EINVAL;
            break;
        }
    }

    return errorCode;
}
#endif /* TM_USE_TCP */
#else /* !TM_MULTIPLE_CONTEXT || !TM_USE_PPP */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

int tlTxPppDummy = 0;
#endif /* !TM_MULTIPLE_CONTEXT || !TM_USE_PPP */

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tuppp.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 41557    Output checksum:  5937
 */
