/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2005
 * Copyright Elmic Systems Japan 1997 - 2005
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: PPPoE automated test code.
 *
 * Filename: tupppoe.c
 * Author: Odile Lapidus
 * Date Created: 2005/03/09
 * $Source: test/unittest/tupppoe.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:43CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#if defined(TM_MULTIPLE_CONTEXT) && defined(TM_USE_PPPOE_CLIENT_SERVER)
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include "tutest.h"

static ttUserInterface tlPppoeInterfaceHandleClient;
static ttUserInterface tlPppoeInterfaceHandleServer;
/* constant declarations and enumerations */

/* type definitions */

/* Test functions */
static void tfxPppoeAuthTest(int serverAuthProto, int clientAuthProto);
static void tfxPppoeAuthMethodTest(int serverAuthMethod1, 
                                 int serverAuthMethod2,
                                 int serverAuthMethod3);
static void tfxPppoeChapAuthTest(void);

static void tfx4PppoeBasicTest(int delaySetup);
static void tfx6PppoeBasicTest(void);

/* Utility functions */
static void tfxPppoeSetupPap(ttUserInterface authServerInterface,
                           ttUserContext   authServerContext,
                           ttUserInterface authClientInterface,
                           ttUserContext   authClientContext);
static void tfxPppoeSetupChap(ttUserInterface authServerInterface,
                            ttUserContext   authServerContext,
                            ttUserInterface authClientInterface,
                            ttUserContext   authClientContext);
static void tfxPppoeSetupEap(ttUserInterface authServerInterface,
                           ttUserContext   authServerContext,
                           ttUserInterface authClientInterface,
                           ttUserContext   authClientContext);
static void tfxPppoeServerNotify(
    ttUserInterface interfaceHandle, unsigned int flags);
static void tfxPppoeClientNotify(
    ttUserInterface interfaceHandle, unsigned int flags);   
static void tfx4PppoeSetupAddr(ttUserInterface interfaceHandle,
                             char TM_FAR *   ipAddrStr);
static void tfx4PppoeStart(void);
static void tfx6PppoeStart(void);
static void tfxPppoeStop(void);
static void tfx4PppEnableVj(ttUserInterface interfaceHandle);
static void tfxPppoeEnableIphc(ttUserInterface interfaceHandle, int family);
static void tfx4PppoeVjCompTest(void);
static void tfxPppoeSetOptionPtr(ttUserInterface interfaceHandle, int optName);
static void tfxPppoeSetOptionU32(ttUserInterface interfaceHandle, int optName);
static void tfxPppoeSetOptionInt(ttUserInterface interfaceHandle, int optName);

#ifdef TM_USE_TCP
static int tfxVerifyBuffer(tt8BitPtr srcBufPtr,
                           tt8BitPtr srcDestPtr,
                           int       bufLength);
static void tfxFillBufferRandom(tt8BitPtr bufPtr, int bufLength);
static void tfxPppoeTestCompression(int family,
                                  int testCount,
                                  int maxBufSize,
                                  int timestampFlag);
#endif /* TM_USE_TCP */


/* local variable definitions */
static int tlServerPppoeFlags;
static int tlClientPppoeFlags;
static int tl6ClientAddrConfig;
static int tl6ServerAddrConfig;

static char tlPapUsername[]    = "pap_username";
static char tlPapPassword[]    = "pap_password";

static char tlChapUsername[]   = "chap_username";
static char tlChapSecret[]     = "chap_secret";
static char tlChapServername[] = "chap_servername";

static char tlEapMd5Username[] = "eap_md5_username";
static char tlEapMd5Secret[]   = "eap_md5_secret";
static char tlEapMd5Servername[] = "eap_servername";

static char tlEapLocalIdent[]  = "eap_identity";

/* static const tables */

/* macro definitions */
#define TM_PPPOE_CLIENT_ADDRESS "10.2.2.2"
#define TM_PPPOE_SERVER_ADDRESS "10.1.1.1"

#define TM_MAX_IPHC_BUF 1200
#define TM_IPHC_TEST_COUNT 25

#define NONE 0

void tfxPppoeAdInterfaces(void)
{
    ttUserLinkLayer linkLayerHandleClient;
    ttUserLinkLayer linkLayerHandleServer;
    int             errorCode;
#ifdef TM_USE_VCHAN
    int             mhomeIndex;
#endif /* TM_USE_VCHAN */
    
/* Setup PPPoE client */
    tfSetCurrentContext(tvContextHandleClient);

    linkLayerHandleClient = tfUseClientPppoe(tfxPppoeClientNotify);

    if (linkLayerHandleClient != (ttUserLinkLayer)0)
    {
        tlPppoeInterfaceHandleClient = tfUseIntfMCtxDriver(
                                    "Pppoe.Client", 
                                     linkLayerHandleClient, 
                                     &errorCode);
        tm_assert(tfxPppoeAdInterfaces, errorCode == TM_ENOERROR);

#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
        for (mhomeIndex = 0;
             mhomeIndex < TM_MAX_IPS_PER_IF;
             mhomeIndex++)
        {
            errorCode = tfNgInterfaceSetVirtualChannel(
                tlPppoeInterfaceHandleClient,
                (ttUser32Bit) mhomeIndex,
                AF_INET,
                mhomeIndex );
            tm_assert(tfxPppoeAdInterfaces, errorCode == TM_ENOERROR);
        }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
    }
    else
    {
        tlPppoeInterfaceHandleClient = (ttUserInterface)0;
    }
    
/* Setup PPPoE server */
    tfSetCurrentContext(tvContextHandleServer);

    linkLayerHandleServer = tfUseServerPppoe(tfxPppoeServerNotify);

    if (linkLayerHandleServer != (ttUserLinkLayer)0)
    {
        tlPppoeInterfaceHandleServer = tfUseIntfMCtxDriver(
                                    "Pppoe.Server", 
                                     linkLayerHandleServer, 
                                     &errorCode);
        tm_assert(tfxPppoeAdInterfaces, errorCode == TM_ENOERROR);    

#ifdef TM_USE_VCHAN
/* initialize virtual channel information for this interface */
#if defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME)
        for (mhomeIndex = 0;
             mhomeIndex < TM_MAX_IPS_PER_IF;
             mhomeIndex++)
        {
            errorCode = tfNgInterfaceSetVirtualChannel(
                tlPppoeInterfaceHandleServer,
                (ttUser32Bit) mhomeIndex,
                AF_INET,
                mhomeIndex );
            tm_assert(tfxPppoeAdInterfaces, errorCode == TM_ENOERROR);
        }
#endif /* defined(TM_USE_IPV4) && !defined(TM_SINGLE_INTERFACE_HOME) */
#endif /* TM_USE_VCHAN */
    }
    else
    {
        tlPppoeInterfaceHandleServer = (ttUserInterface)0;
    }

}

void tfxPppoeAutoTest(void)
{
    if (tlPppoeInterfaceHandleClient != (ttUserInterface)0)
    {
/*
 * Test starting PPPoE with no PPP options.
 */

/* Basic IPv4 over PPPoE test. Delay PPP address setup until PPPoE callback */
        tm_test_with_1_param(tfx4PppoeBasicTest, 1);
/*
 * Authentication tests.
 */

/* PAP <--> No auth */
        tm_test_with_2_param(tfxPppoeAuthTest,
                                 TM_PPP_PAP_PROTOCOL,
                                 NONE);
/* PAP <--> PAP */
        tm_test_with_2_param(tfxPppoeAuthTest,
                                 TM_PPP_PAP_PROTOCOL,
                                 TM_PPP_PAP_PROTOCOL);
/* CHAP <--> No auth */
        tm_test_with_2_param(tfxPppoeAuthTest,
                                 TM_PPP_CHAP_PROTOCOL,
                                 NONE);

/* CHAP <--> CHAP */
        tm_test_with_2_param(tfxPppoeAuthTest,
                                 TM_PPP_CHAP_PROTOCOL,
                                 TM_PPP_CHAP_PROTOCOL);

/* PAP <--> CHAP */
        tm_test_with_2_param(tfxPppoeAuthTest,
                                 TM_PPP_PAP_PROTOCOL,
                                 TM_PPP_CHAP_PROTOCOL);

/* NOTE: You must support CHAP in order to support MSCHAP
 *
 * Server wants PAP, standard CHAP, MS-CHAPv1 and in this order
 * Client only allows MS-CHAPv1
 * Server sends PAP config first, Client NAKs it with MS-CHAPv1
 * Server sends CHAP config again, Client NAKs it with MS-CHAPv1
 * Server sends MS-CHAPv1 config, Client ACKs it.
 */

        


/* Basic IPv4 over PPPoE test. */
        tm_test_with_1_param(tfx4PppoeBasicTest, 0);
        
/* IPv4 Van Jacobson header compression test */
        tm_test(tfx4PppoeVjCompTest);

        
        tfSetCurrentContext(tvContextHandleClient);
        tm_test_with_2_param(tfxPppoeSetOptionPtr, tlPppoeInterfaceHandleClient,
                             TM_PPPOE_OPT_SRVC_NAME_TAG);
        tm_test_with_2_param(tfxPppoeSetOptionPtr, tlPppoeInterfaceHandleClient,
                             TM_PPPOE_OPT_HOST_UNIQ_TAG);
        tm_test_with_2_param(tfxPppoeSetOptionU32, tlPppoeInterfaceHandleClient,
                             TM_PPPOE_OPT_INITIAL_RETRY_WAIT);
        tm_test_with_2_param(tfxPppoeSetOptionInt, tlPppoeInterfaceHandleClient,
                             TM_PPPOE_OPT_PADI_RETRIES_MAX);
        tm_test_with_2_param(tfxPppoeSetOptionInt, tlPppoeInterfaceHandleClient,
                             TM_PPPOE_OPT_PADR_RETRIES_MAX);
        tm_test_with_2_param(tfxPppoeSetOptionInt, tlPppoeInterfaceHandleClient,
                             TM_PPPOE_OPT_DISC_RETRIES_MAX);
        tfSetCurrentContext(tvContextHandleServer);
        tm_test_with_2_param(tfxPppoeSetOptionPtr, tlPppoeInterfaceHandleServer,
                             TM_PPPOE_OPT_SRVC_NAME_TAG);
        tm_test_with_2_param(tfxPppoeSetOptionPtr, tlPppoeInterfaceHandleServer,
                             TM_PPPOE_OPT_AC_NAME_TAG);
        tm_test_with_2_param(tfxPppoeSetOptionU32, tlPppoeInterfaceHandleServer,
                             TM_PPPOE_OPT_SRVR_TIMEOUT);
    }
    else
    {
        tfKernelPrint(TM_PRINT_TYPE_FAIL,
                     "\nNot running PPPoE tests - PPPoE code not available\n");
    }
}
/*
======================================================
CLIENT MODE

tfPppoeSetOption TM_PPPOE_OPT_SRVC_NAME_TAG
1.optionValuePtr=Null     <--- OK
2.optionLength=0      <--- OK
3.SRVC_NAME='0123456789'    <--- OK
4.SRVC_NAME='abcdefgVWXYZ'    <--- OK
5.SRVC_NAME='!#$%&'()=~|'    <--- OK
7.SRVC_NAME='0123456789...'(256bytes) <--- NOT OK
8.SRVC_NAME='0123456789...'(255bytes) <--- OK
------------------------------------------------------------------------
tfPppoeSetOption TM_PPPOE_OPT_HOST_UNIQ_TAG
1.optionValuePtr = Null     <--- OK
2.optionLength = 0      <--- OK
3.HOST_UNIQ='0123456789'    <--- OK
4.HOST_UNIQ='abcdefgVWXYZ'    <--- OK
5.HOST_UNIQ='!#$%&'()=~|'    <--- OK
7.HOST_UNIQ='0123456789...'(256bytes) <--- NOT OK
8.HOST_UNIQ='0123456789...'(255bytes) <--- OK

==============================================================
SERVER MODE

tfPppoeSetOption TM_PPPOE_OPT_SRVC_NAME_TAG
1.optionValuePtr = Null     <--- OK 
2.optionLength = 0      <--- OK 
3.SRVC_NAME='0123456789'    <--- OK
4.SRVC_NAME='abcdefgVWXYZ'    <--- OK
5.SRVC_NAME='!#$%&'()=~|'    <--- OK
7.SRVC_NAME='0123456789...'(256bytes) <--- OK
8.SRVC_NAME='0123456789...'(255bytes) <--- OK
-------------------------------------------------------------------------
tfPppoeSetOption TM_PPPOE_OPT_AC_NAME_TAG
1.optionValuePtr =Null     <--- OK
2.optionLength =0      <--- OK
3.AC_NAME='0123456789'     <--- OK
4.AC_NAME='abcdefgVWXYZ'    <--- OK
5.AC_NAME='!#$%&'()=~|'     <--- OK
7.AC_NAME='0123456789...'(256bytes)  <--- NOT OK
8.AC_NAME='0123456789...'(255bytes)  <--- OK
-------------------------------------------------------------------------
*/
static void tfxPppoeSetOptionPtr(ttUserInterface interfaceHandle, int optName)
{
    void TM_FAR   * optionValuePtr;
    int             optionLength;
    int             errorCode;

/*
1.1 tfPppoeSetOption TM_PPPOE_OPT_SRVC_NAME_TAG
1.optionValuePtr=Null     <--- OK / NG ?  Which is right?
2.optionLength=0      <--- OK / NG ?  Which is right?
*/
    optionLength = 128;
    optionValuePtr = (ttVoidPtr)0;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValuePtr = "abcdefgVWXYZ"; optionLength = 255;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValuePtr = "abcdefgVWXYZ"; optionLength = 256;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionValuePtr = "!#$%&'()=~|"; optionLength = (int)tm_strlen(optionValuePtr);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValuePtr = "0123456789..."; optionLength = 256;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionValuePtr = "0123456789..."; optionLength = 255;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValuePtr = "0123456789"; optionLength = (int)tm_strlen(optionValuePtr);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValuePtr = "abcdefgVWXYZ"; optionLength = (int)tm_strlen(optionValuePtr);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionLength = 0;
    optionValuePtr = "\0";
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValuePtr = "abcdefgVWXYZ"; optionLength = (int)tm_strlen(optionValuePtr);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    return;
}

/*
==============================================================
CLIENT MODE
-------------------------------------------------------------------------
tfPppoeSetOption TM_PPPOE_OPT_INITIAL_RETRY_WAIT
1.optionValuePtr = Null     <--- Invalid
2.optionLength = 0      <--- Invalid
3.INITIAL_RETRY_WAIT=0xffffffff   <--- OK
4.INITIAL_RETRY_WAIT=0     <--- OK
5.INITIAL_RETRY_WAIT=-1     <--- OK (unisgned value) 
6.INITIAL_RETRY_WAIT=10     <--- OK
7.INITIAL_RETRY_WAIT=10000    <--- OK
-------------------------------------------------------------------------
==============================================================
SERVER MODE
tfPppoeSetOption TM_PPPOE_OPT_SRVR_TIMEOUT
1.optionValuePtr =Null     <--- Invalid
2.optionLength =0      <--- Invalid
3.SRVR_TIMEOUT=0xffffffff    <--- OK 
4.SRVR_TIMEOUT=0      <--- OK 
5.SRVR_TIMEOUT=-1      <--- OK (unsigned value)
6.SRVR_TIMEOUT=10      <--- OK
7.SRVR_TIMEOUT=10000     <--- OK
-------------------------------------------------------------------------
*/
static void tfxPppoeSetOptionU32(ttUserInterface interfaceHandle, int optName)
{
    void TM_FAR   * optionValuePtr;
    int             optionLength;
    int             errorCode;
    tt32Bit         optionValue;

    optionLength = 128;
    optionValuePtr = (ttVoidPtr)0;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);

    optionLength = 0;
    optionValue = (tt32Bit)0;
    optionValuePtr = &optionValue;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionLength = sizeof(tt32Bit);
    optionValue = TM_UL(0xffffffff);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValue = TM_UL(0);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValue = TM_UL(0xffffffff);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValue = TM_UL(10);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValue = TM_UL(10000);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
}
/*
==============================================================
CLIENT MODE
-------------------------------------------------------------------------
tfPppoeSetOption TM_PPPOE_OPT_PADI_RETRIES_MAX
1.optionValuePtr = Null     <--- Invalid
2.optionLength = 0      <--- Invalid
3.PADI_RETRIES=0xffffffff    <--- Invalid (negative)
4.PADI_RETRIES=0      <---  OK
5.PADI_RETRIES=-1      <--- Invalid 
6.PADI_RETRIES=10      <--- OK
7.PADI_RETRIES=256      <--- Invalid
-------------------------------------------------------------------------
tfPppoeSetOption TM_PPPOE_OPT_PADR_RETRIES_MAX
1.optionValuePtr = Null     <--- Invalid
2.optionLength = 0      <--- Invalid
3.PADR_RETRIES=0xffffffff    <--- Invalid (negative)
4.PADR_RETRIES=0      <---  OK
5.PADR_RETRIES=-1      <--- Invalid
6.PADR_RETRIES=10      <--- OK
7.PADR_RETRIES=256      <--- Invalid
-------------------------------------------------------------------------
tfPppoeSetOption TM_PPPOE_OPT_DISC_RETRIES_MAX
1.optionValuePtr = Null     <--- Invalid
2.optionLength = 0      <--- Invalid
3.DISC_RETRIES=0xffffffff    <--- Invalid (negative)
4.DISC_RETRIES=0      <--- OK
5.DISC_RETRIES=-1      <--- Invalid
6.DISC_RETRIES=10      <--- OK
7.DISC_RETRIES=256      <--- Invalid
*/
static void tfxPppoeSetOptionInt(ttUserInterface interfaceHandle, int optName)
{
    void TM_FAR   * optionValuePtr;
    int             optionLength;
    int             errorCode;
    int             optionValue;

    optionLength = 128;
    optionValuePtr = (ttVoidPtr)0;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionLength = 0;
    optionValue = 0;
    optionValuePtr = &optionValue;
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionLength = sizeof(int);
    optionValue = 0xffffffff; /* negative integer */
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionValue = TM_UL(0);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValue = TM_UL(0xffffffff);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
    optionValue = TM_UL(10);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_ENOERROR);
    optionValue = TM_UL(256);
    errorCode = tfPppoeSetOption(interfaceHandle, optName, optionValuePtr, optionLength);
    tm_assert(tfxPppoeSetOption, errorCode == TM_EINVAL);
}

/* Server wants PAP, standard CHAP, MS-CHAPv1 and in this order
 * Client only allows MS-CHAPv1
 * Server sends PAP config first, Client NAKs it with MS-CHAPv1
 * Server sends CHAP config again, Client NAKs it with MS-CHAPv1
 * Server sends MS-CHAPv1 config, Client ACKs it.
 */


static void tfxPppoeAuthTest(int serverAuthProto, int clientAuthProto)
{
    switch (serverAuthProto)
    {
        case TM_PPP_PAP_PROTOCOL:
            tfxPppoeSetupPap(tlPppoeInterfaceHandleServer,
                           tvContextHandleServer,
                           tlPppoeInterfaceHandleClient,
                           tvContextHandleClient);
            break;

        case TM_PPP_CHAP_PROTOCOL:
            tfxPppoeSetupChap(tlPppoeInterfaceHandleServer,
                            tvContextHandleServer,
                            tlPppoeInterfaceHandleClient,
                            tvContextHandleClient);
            break;

    }

    switch (clientAuthProto)
    {
        case TM_PPP_PAP_PROTOCOL:
            tfxPppoeSetupPap(tlPppoeInterfaceHandleClient,
                           tvContextHandleClient,
                           tlPppoeInterfaceHandleServer,
                           tvContextHandleServer);
            break;

        case TM_PPP_CHAP_PROTOCOL:
            tfxPppoeSetupChap(tlPppoeInterfaceHandleClient,
                            tvContextHandleClient,
                            tlPppoeInterfaceHandleServer,
                            tvContextHandleServer);
            break;

    }

    
    tfx4PppoeStart();

    tfxPppoeStop();

}


static int tfxPapAuthCallback(char TM_FAR * username,
                              char TM_FAR * password)
{
    int errorCode;
    int serverLength, clientLength;

    serverLength = (int)tm_strlen(tlPapUsername);
    clientLength = (int)tm_strlen(username);
    tm_assert(tfxPapAuthCallback, serverLength == clientLength);
    
    errorCode = tm_memcmp(username, tlPapUsername, clientLength);
    tm_assert(tfxPapAuthCallback, errorCode == 0);

    serverLength = (int)tm_strlen(tlPapPassword);
    clientLength = (int)tm_strlen(password);
    tm_assert(tfxPapAuthCallback, serverLength == clientLength);
    
    errorCode = tm_memcmp(password, tlPapPassword, clientLength);
    tm_assert(tfxPapAuthCallback, errorCode == 0);

    return 1;
}

static void tfxPppoeSetupPap(ttUserInterface authServerInterface,
                           ttUserContext   authServerContext,
                           ttUserInterface authClientInterface,
                           ttUserContext   authClientContext)
{
    int     optLength;
    int     errorCode;
    tt8Bit  opt8;
    tt16Bit opt16;
    
    if(authClientInterface)
    {
/* Enable PAP on the client side, setup username & password. */
    tfSetCurrentContext(authClientContext);

    opt16 = TM_PPP_PAP_PROTOCOL;
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_ALLOW,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppoeSetupPap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlPapUsername);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_PAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_PAP_USERNAME,
                                tlPapUsername,
                                optLength );
    tm_assert(tfxPppoeSetupPap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlPapPassword);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_PAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_PAP_PASSWORD,
                                tlPapPassword,
                                optLength );
    tm_assert(tfxPppoeSetupPap, errorCode == TM_ENOERROR);

    opt8 = 1;
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_PAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_PAP_TIMEOUT,
                                (const char TM_FAR *) &opt8,
                                sizeof(opt8) );
    tm_assert(tfxPppoeSetupPap, errorCode == TM_ENOERROR);

    }
/* Setup PAP on server side, and register PAP authentication routine. */
    tfSetCurrentContext(authServerContext);

    opt16 = TM_PPP_PAP_PROTOCOL;
    errorCode = tfPppSetOption( authServerInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppoeSetupPap, errorCode == TM_ENOERROR);

   
    errorCode = tfPapRegisterAuthenticate( authServerInterface,
                                           tfxPapAuthCallback );
    tm_assert(tfxPppoeSetupPap, errorCode == TM_ENOERROR);

}


static char TM_FAR *tfxChapAuthCallback(char TM_FAR * username)
{
    int errorCode;
    int serverLength, clientLength;

    serverLength = (int)tm_strlen(tlChapUsername);
    clientLength = (int)tm_strlen(username);
    tm_assert(tfxChapAuthCallback, serverLength == clientLength);
    
    errorCode = tm_memcmp(username, tlChapUsername, clientLength);
    tm_assert(tfxChapAuthCallback, errorCode == 0);

    return tlChapSecret;
}




static void tfxPppoeSetupChap(ttUserInterface authServerInterface,
                            ttUserContext   authServerContext,
                            ttUserInterface authClientInterface,
                            ttUserContext   authClientContext)
{
    int     optLength;
    int     errorCode;
    tt16Bit opt16;

    if(authClientInterface)
    {

/* Enable CHAP on the client side, setup username & password. */
    tfSetCurrentContext(authClientContext);

    opt16 = TM_PPP_CHAP_PROTOCOL;
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_ALLOW,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppoeSetupChap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlChapUsername);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_CHAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_CHAP_USERNAME,
                                tlChapUsername,
                                optLength );
    tm_assert(tfxPppoeSetupChap, errorCode == TM_ENOERROR);
    
    optLength = (int)tm_strlen(tlChapSecret);
    errorCode = tfPppSetOption( authClientInterface,
                                TM_PPP_CHAP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_CHAP_SECRET,
                                tlChapSecret,
                                optLength );
    tm_assert(tfxPppoeSetupChap, errorCode == TM_ENOERROR);

    }
/* Setup PAP on server side, and register PAP authentication routine. */
    tfSetCurrentContext(authServerContext);

    opt16 = TM_PPP_CHAP_PROTOCOL;
    errorCode = tfPppSetOption( authServerInterface,
                                TM_PPP_LCP_PROTOCOL,
                                TM_PPP_OPT_WANT,
                                TM_LCP_AUTH_PROTOCOL,
                                (const char TM_FAR *) &opt16,
                                sizeof(opt16) );
    tm_assert(tfxPppoeSetupChap, errorCode == TM_ENOERROR);

    optLength = (int)tm_strlen(tlChapServername);
    errorCode = tfPppSetOption( authServerInterface,
                                TM_PPP_CHAP_PROTOCOL,
                                TM_PPP_OPT_ALLOW,
                                TM_CHAP_USERNAME,
                                tlChapServername,
                                optLength );
    tm_assert(tfxPppoeSetupChap, errorCode == TM_ENOERROR);

    errorCode = tfChapRegisterAuthenticate( authServerInterface,
                                            tfxChapAuthCallback );

    
    tm_assert(tfxPppoeSetupChap, errorCode == TM_ENOERROR);

}





static void tfx4PppoeBasicTest(int delaySetup)
{
    tfSetCurrentContext(tvContextHandleServer);
    if (!delaySetup)
    {
        tfx4PppoeSetupAddr(tlPppoeInterfaceHandleServer, TM_PPPOE_SERVER_ADDRESS);
    }

    tfSetCurrentContext(tvContextHandleClient);
    if (!delaySetup)
    {
        tfx4PppoeSetupAddr(tlPppoeInterfaceHandleClient, TM_PPPOE_CLIENT_ADDRESS);
    }
    
#if (defined(TM_USE_IPV4) && !defined(TM_4_USE_SCOPE_ID))
    tfx4PppoeStart();
#else /* !TM_USE_IPV4 && TM_4_USE_SCOPE_ID */
    tfx6PppoeStart();
#endif /* TM_USE_IPV4 && !TM_4_USE_SCOPE_ID */

#ifdef TM_USE_TCP
/* Run the compression test, just with no compression enabled. */
    tfxPppoeTestCompression(
#ifdef TM_4_USE_SCOPE_ID
                            AF_INET6,
#else /* TM_4_USE_SCOPE_ID */
                            AF_INET,
#endif /* TM_4_USE_SCOPE_ID */
                            TM_IPHC_TEST_COUNT,                          
                            TM_MAX_IPHC_BUF,
                            1); /* enable TCP timestamps. */
#endif /* TM_USE_TCP */
    
    tfxPppoeStop();

    tfSetCurrentContext(tvContextHandleClient);
}


static void tfx4PppoeVjCompTest(void)
{

    tfSetCurrentContext(tvContextHandleServer);
    tfx4PppEnableVj(tlPppoeInterfaceHandleServer);

    tfSetCurrentContext(tvContextHandleClient);
    tfx4PppEnableVj(tlPppoeInterfaceHandleClient);
    
    tfx4PppoeStart();
    
    tfxPppoeStop();

}

static void tfx4PppEnableVj(ttUserInterface interfaceHandle)
{
    int     errorCode;
    tt16Bit opt16;

    opt16 = TM_PPP_COMP_TCP_PROTOCOL;
    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_WANT,
                               TM_IPCP_COMP_PROTOCOL,
                               (const char TM_FAR *) &opt16,
                               sizeof(opt16));
    tm_assert(tfx4PppEnableVj, errorCode != TM_ENOERROR);

    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_ALLOW,
                               TM_IPCP_COMP_PROTOCOL,
                               (const char TM_FAR *) &opt16,
                               sizeof(opt16));
    tm_assert(tfx4PppEnableVj, errorCode != TM_ENOERROR);
}

/*
 * Sets the IPv4 address to request from the peer, and allows the peer
 * to set their own IP address.
 */ 
static void tfx4PppoeSetupAddr(ttUserInterface interfaceHandle,
                             char TM_FAR *   ipAddrStr)
{
    int errorCode;
    tt4IpAddress ipAddr;

/* Set the IP address to request from the peer. */
    ipAddr = inet_addr(ipAddrStr);
    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_WANT,
                               TM_IPCP_IP_ADDRESS,
                               (const char TM_FAR *) &ipAddr,
                               sizeof(ipAddr));
    tm_assert(tfx4PppoeSetupAddr, errorCode == TM_ENOERROR);

/* Allow the peer to set their address. */
    ipAddr = TM_UL(0);
    errorCode = tfPppSetOption(interfaceHandle,
                               TM_PPP_IPCP_PROTOCOL,
                               TM_PPP_OPT_ALLOW,
                               TM_IPCP_IP_ADDRESS,
                               (const char TM_FAR *) &ipAddr,
                               sizeof(ipAddr));
    tm_assert(tfx4PppoeSetupAddr, errorCode == TM_ENOERROR);    
}


static void tfx4PppoeStart(void)
{
    tt4IpAddress ipAddr;    
    int          flagsOK;
    int          errorCode;
    int          i;

    
/* Start PPPoE server. */
    tfSetCurrentContext(tvContextHandleServer);    
    tlServerPppoeFlags = 0;
    ipAddr = inet_addr(TM_PPPOE_SERVER_ADDRESS);
    errorCode = tfOpenInterface(tlPppoeInterfaceHandleServer,
                                ipAddr,
                                TM_IP_HOST_MASK,
                                TM_DEV_SCATTER_SEND_ENB,
                                1500);
    tm_assert(tfx4PppoeStart, errorCode == TM_EINPROGRESS);

/* Start PPPoE client. */
    tlClientPppoeFlags = 0;
    tfSetCurrentContext(tvContextHandleClient);
    ipAddr = inet_addr(TM_PPPOE_CLIENT_ADDRESS);
    errorCode = tfOpenInterface(tlPppoeInterfaceHandleClient,
                                ipAddr,
                                TM_IP_HOST_MASK,
                                TM_DEV_SCATTER_SEND_ENB,
                                1500);
    tm_assert(tfx4PppoeStart, errorCode == TM_EINPROGRESS);

    while ( !(tlClientPppoeFlags & TM_LL_OPEN_COMPLETE) ||
            !(tlServerPppoeFlags & TM_LL_OPEN_COMPLETE) )
    {
        tfSetCurrentContext(tvContextHandleClient);
        tfTimerUpdate();
        tfTimerExecute();

        tfSetCurrentContext(tvContextHandleServer);
        tfTimerUpdate();
        tfTimerExecute();

/* Make sure that the connection hasn't been closed (because of excess
   retransmissions, for instance).  This would indicate a failure of this
   test. */
        tm_assert(tfx4PppoeStart, !(tlClientPppoeFlags & TM_LL_CLOSE_COMPLETE));
        tm_assert(tfx4PppoeStart, !(tlServerPppoeFlags & TM_LL_CLOSE_COMPLETE));
    }
    
/* Verify that the proper notifications were sent to PPPoE server. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlServerPppoeFlags,
                              (TM_LL_LCP_UP | TM_LL_IP4_OPEN_COMPLETE |
                               TM_LL_OPEN_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfx4PppoeStart, flagsOK == 1);

/* Verify that the proper notifications were sent to the PPPoE client. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlClientPppoeFlags,
                              (TM_LL_LCP_UP | TM_LL_IP4_OPEN_COMPLETE |
                               TM_LL_OPEN_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfx4PppoeStart, flagsOK == 1);

    for (i=0;i<=1;i++)
    {
        tm_strcpy(tv4ClientIpAddrStr[i], TM_PPPOE_CLIENT_ADDRESS);
        tm_memset(&tv4ClientSockAddr[i], 0, sizeof(struct sockaddr_storage));
        tv4ClientSockAddr[i].ss_family = AF_INET;
        tv4ClientSockAddr[i].ss_len = sizeof(struct sockaddr_storage);
        errorCode = inet_pton(AF_INET,
                              TM_PPPOE_CLIENT_ADDRESS,
                              &tv4ClientSockAddr[i].addr.ipv4.sin_addr);
        tm_assert(tfxPppoeAdInterfaces, errorCode == 1);
    }

    for (i=0;i<=1;i++)
    {
        tm_strcpy(tv4ServerIpAddrStr[i], TM_PPPOE_SERVER_ADDRESS);
        tm_memset(&tv4ServerSockAddr[i], 0, sizeof(struct sockaddr_storage));
        tv4ServerSockAddr[i].ss_family = AF_INET;
        tv4ServerSockAddr[i].ss_len = sizeof(struct sockaddr_storage);
        tv4ServerSockAddr[i].addr.ipv4.sin_port = htons(4001);        
        errorCode = inet_pton(AF_INET,
                              TM_PPPOE_SERVER_ADDRESS,
                              &tv4ServerSockAddr[i].addr.ipv4.sin_addr);
        tm_assert(tfxPppoeAdInterfaces, errorCode == 1);
    }
    
}

#ifdef TM_USE_TCP
static void tfxPppoeTestCompression(int family,
                                  int testCount,
                                  int maxBufSize,
                                  int timestampFlag)
{
    int serverSd;
    int acceptSd;
    int clientSd;
    int sockAddrLength;
    int errorCode;
    struct sockaddr_storage *sockAddrPtr;
    tt8BitPtr outDataPtr, inDataPtr;
    struct sockaddr_storage fromAddr;
    int fromAddrLen;
    int i;
    int bufLength;
    
/* Allocate working buffers. */
    tfSetCurrentContext(tvContextHandleClient);    
    outDataPtr = (tt8BitPtr) tm_get_raw_buffer(TM_MAX_IPHC_BUF);
    tm_assert(tfxPppoeTestCompression, outDataPtr != TM_8BIT_NULL_PTR);

    inDataPtr = (tt8BitPtr) tm_get_raw_buffer(TM_MAX_IPHC_BUF);
    tm_assert(tfxPppoeTestCompression, inDataPtr != TM_8BIT_NULL_PTR);

    tfxGetTcpConnection(tlPppoeInterfaceHandleClient,
                        0,
                        &serverSd,
                        &acceptSd,
                        &clientSd,
                        family,
                        timestampFlag,
#ifdef TM_USE_STRONG_ESL
                        "Pppoe.Client", "Pppoe.Server",
#endif /* TM_USE_STRONG_ESL */
                        0);

    tfxGetServerSockAddr(0,
                         family,
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);

/* Test #1: Random sized packets sent in both directions. */
    for (i=0;i<testCount;i++)
    {

/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = send(clientSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfxLoopRecv( acceptSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);

/* Send data from server to client. */
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);

        errorCode = send(acceptSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

/* Receive data from server to client. */
        tfSetCurrentContext(tvContextHandleClient);
        errorCode = tfxLoopRecv( clientSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);
    }

/*
 * Test #2: Special case #1 (SWU): typing traffic (TCP echo)
 *          NOTE: it is important that the length sent is a full
 *          sized segment, in order to avoid triggering TCP's
 *          "Silly Window Syndrome" prevention, which would prevent
 *          the window size from being updating for every packet.  This
 *          will cause the window size to change with every packet, and
 *          prevent the use of the SWU special case.
 */
    bufLength = maxBufSize; /* cannot be bigger than size of allocated buffer */
    for (i=0;i<testCount;i++)
    {
/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = send(clientSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfxLoopRecv( acceptSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);

/* Echo data back from server back to client. */
        errorCode = send(acceptSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

/* Receive data from server to client. */
        tfSetCurrentContext(tvContextHandleClient);
        errorCode = tfxLoopRecv( clientSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);
    }

/* Test #3: Special case #2 (SAWU): bulk data transfer (e.g. FTP) */
    bufLength = 512;
    for (i=0;i<TM_IPHC_TEST_COUNT;i++)
    {
/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = send(clientSd, (ttCharPtr) outDataPtr, bufLength, 0);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfxLoopRecv( acceptSd, (ttCharPtr) inDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);
    }
    
    tfxCloseTcpConnection(serverSd, acceptSd, clientSd);
    
    tfSetCurrentContext(tvContextHandleServer);


/* Test #4: bi-directional UDP data, random sized. */
    tfxGetNonConnectedUdpPair(&serverSd, &clientSd, family, 1
#ifdef TM_USE_STRONG_ESL
                              , "Pppoe.Client", "Pppoe.Server"
#endif /* TM_USE_STRONG_ESL */
                             );
    tfxGetServerSockAddr(1,
                         family,
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);

    for (i=0;i<testCount;i++)
    {

/* Send data from client to server. */
        tfSetCurrentContext(tvContextHandleClient);
        
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);
        
        errorCode = sendto(clientSd,
                           (ttCharPtr) outDataPtr,
                           bufLength,
                           0,
                           (struct sockaddr TM_FAR *) sockAddrPtr,
                           sockAddrLength);
        if (errorCode == -1)
        {
            errorCode = tfGetSocketError(clientSd);
            tfKernelError("tfxPppoeTestCompression", "sendto error");
        }
        else
        {
            tm_assert(tfxPppoeTestCompression, errorCode == bufLength);
        }

/* Receive data from client to server. */
        tfSetCurrentContext(tvContextHandleServer);
        fromAddrLen = sizeof(fromAddr);
        errorCode = recvfrom(serverSd,
                             (ttCharPtr) inDataPtr,
                             bufLength,
                             0,
                             (struct sockaddr TM_FAR *) &fromAddr,
                             &fromAddrLen);
        if (errorCode == -1)
        {
            errorCode = tfGetSocketError(serverSd);
            tfKernelError("tfxPppoeTestCompression", "recvfrom error");
        }
        else
        {
            tm_assert(tfxPppoeTestCompression, errorCode == bufLength);
        }

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);

/* Send data from server to client. */
        bufLength = (tfGetRandom() % maxBufSize) + 1;
        tfxFillBufferRandom(outDataPtr, bufLength);

        errorCode = sendto(serverSd,
                           (ttCharPtr) outDataPtr,
                           bufLength,
                           0,
                           (struct sockaddr TM_FAR *) &fromAddr,
                           fromAddrLen);
        if (errorCode == -1)
        {
            errorCode = tfGetSocketError(serverSd);
            tfKernelError("tfxPppoeTestCompression", "sendto error");
        }
        else
        {
            tm_assert(tfxPppoeTestCompression, errorCode == bufLength);
        }
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

/* Receive data from server to client. */
        tfSetCurrentContext(tvContextHandleClient);
        fromAddrLen = sizeof(fromAddr);
        errorCode = recvfrom(clientSd,
                             (ttCharPtr) inDataPtr,
                             bufLength,
                             0,
                             (struct sockaddr TM_FAR *) &fromAddr,
                             &fromAddrLen);
        if (errorCode == -1)
        {
            errorCode = tfGetSocketError(clientSd);
            tfKernelError("tfxPppoeTestCompression", "recvfrom error");
        }
        else
        {
            tm_assert(tfxPppoeTestCompression, errorCode == bufLength);
        }
        tm_assert(tfxPppoeTestCompression, errorCode == bufLength);

        errorCode = tfxVerifyBuffer(inDataPtr, outDataPtr, bufLength);
        tm_assert(tfxPppoeTestCompression, errorCode == TM_ENOERROR);
    }
    
    tfxCloseUdpConnection(serverSd, clientSd);

/* Free working buffers. */
    tm_free_raw_buffer(outDataPtr);
    tm_free_raw_buffer(inDataPtr);
              
}
#endif /* TM_USE_TCP */


/*
 * Enables IP header compression on the given interface.
 */

static void tfxPppoeClientNotify(
    ttUserInterface interfaceHandle, unsigned int flags)
{
    interfaceHandle=interfaceHandle;
    tlClientPppoeFlags |= flags;
    if (flags & TM_LL_PPPOE_DISC_COMPLETE) 
    {
        tfx4PppoeSetupAddr(tlPppoeInterfaceHandleClient, TM_PPPOE_CLIENT_ADDRESS);
    }
}

static void tfxPppoeServerNotify(
    ttUserInterface interfaceHandle, unsigned int flags)
{
    interfaceHandle=interfaceHandle;
    tlServerPppoeFlags |= flags;
    if (flags & TM_LL_PPPOE_DISC_COMPLETE) 
    {
        tfx4PppoeSetupAddr(tlPppoeInterfaceHandleServer, TM_PPPOE_SERVER_ADDRESS);
    }
}

static void tfxPppoeStop(void)
{
    int errorCode;
    int flagsOK;
    
    tlServerPppoeFlags = 0;
    tlClientPppoeFlags = 0;
    
    tfSetCurrentContext(tvContextHandleServer);    

    errorCode = tfCloseInterface(tlPppoeInterfaceHandleServer);
    tm_assert(tfxPppoeStop, errorCode == TM_EINPROGRESS);

    tfSetCurrentContext(tvContextHandleClient);    
    errorCode = tfCloseInterface(tlPppoeInterfaceHandleClient);

    tm_assert(tfxPppoeStop, errorCode == TM_ENOERROR);

    while ( !(tlClientPppoeFlags & TM_LL_CLOSE_COMPLETE))
    {
        tfSetCurrentContext(tvContextHandleClient);
        tfTimerUpdate();
        tfTimerExecute();

        tfSetCurrentContext(tvContextHandleServer);
        tfTimerUpdate();
        tfTimerExecute();
    }
    
/* Verify that the proper notifications were sent to PPPoE server. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlServerPppoeFlags,
                              (TM_LL_CLOSE_STARTED | TM_LL_CLOSE_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfxPppoeStop, flagsOK == 1);

/* Verify that the proper notifications were sent to the PPPoE client. */
    flagsOK = 0;
    if (tm_16bit_all_bits_set(tlClientPppoeFlags,
                              (TM_LL_CLOSE_STARTED | TM_LL_CLOSE_COMPLETE)))
    {
        flagsOK = 1;
    }
    tm_assert(tfxPppoeStop, flagsOK == 1);
    
}

#ifdef TM_USE_TCP

static void tfxFillBufferRandom(tt8BitPtr bufPtr, int bufLength)
{
    int    i;
    tt8Bit randomChar;

    for (i=0;i<bufLength;i++)
    {
        randomChar = (tt8Bit) (tfGetRandom() % 255);
        bufPtr[i] = randomChar;
    }
}

static int tfxVerifyBuffer(tt8BitPtr srcBufPtr,
                           tt8BitPtr srcDestPtr,
                           int       bufLength)
{
    int i;
    int errorCode;

    errorCode = TM_ENOERROR;
    for (i=0;i<bufLength;i++)
    {
        if (srcBufPtr[i] != srcDestPtr[i])
        {
            errorCode = TM_EINVAL;
            break;
        }
    }

    return errorCode;
}
#endif /* TM_USE_TCP */
#else /* !TM_MULTIPLE_CONTEXT || !TM_USE_PPPOE_CLIENT_SERVER */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

int tlTxPppoeDummy = 0;
#endif /* !TM_MULTIPLE_CONTEXT || !TM_USE_PPPOE_CLIENT_SERVER */

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tupppoe.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 33226    Output checksum: 37808
 */
