/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: testing inet_pton
 *
 * Filename: tupton.c
 * Author: Bryan Wang
 * Date Created:
 * $Source: test/unittest/tupton.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:44CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#include "tutest.h"


/*
 * @REQUIREMENT TRACIBILITY
 * [RFC2553].R6.6:10
 */
void tfxInetPtoN()
{
    unsigned char  ipAddr[TM_TEST_IP_ADDR_LEN];
    unsigned char  ipAddrRef[TM_TEST_IP_ADDR_LEN];
    int            errorCode;

/* unknow address family */
    errorCode = inet_pton(-1, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == -1);
    tm_assert(tfxInetPtoN, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    errorCode = inet_pton(0, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == -1);
    tm_assert(tfxInetPtoN, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    errorCode = inet_pton(1, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == -1);
    tm_assert(tfxInetPtoN, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    errorCode = inet_pton(3, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == -1);
    tm_assert(tfxInetPtoN, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    errorCode = inet_pton(27, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == -1);
    tm_assert(tfxInetPtoN, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

    errorCode = inet_pton(29, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == -1);
    tm_assert(tfxInetPtoN, tfGetSocketError(-1) ==TM_EAFNOSUPPORT)

/* AF_INET -- good */
    errorCode = inet_pton(AF_INET, "0.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 1);
    tm_memset(ipAddrRef, 0, TM_IP_ADDRESS_LENGTH);
    tm_assert(tfxInetPtoN, !tm_memcmp(ipAddr, ipAddrRef,TM_IP_ADDRESS_LENGTH));

    errorCode = inet_pton(AF_INET, "100.101.102.103", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 1);
    tm_memset(ipAddrRef, 0, TM_IP_ADDRESS_LENGTH);
    ipAddrRef[0] = 100;
    ipAddrRef[1] = 101;
    ipAddrRef[2] = 102;
    ipAddrRef[3] = 103;
    tm_assert(tfxInetPtoN, !tm_memcmp(ipAddr, ipAddrRef,TM_IP_ADDRESS_LENGTH));

    errorCode = inet_pton(AF_INET, "100.0.0.100", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 1);
    tm_memset(ipAddrRef, 0, TM_IP_ADDRESS_LENGTH);
    ipAddrRef[0] = 100;
    ipAddrRef[3] = 100;
    tm_assert(tfxInetPtoN, !tm_memcmp(ipAddr, ipAddrRef,TM_IP_ADDRESS_LENGTH));

    errorCode = inet_pton(AF_INET, "200.0.0.0", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 1);
    tm_memset(ipAddrRef, 0, TM_IP_ADDRESS_LENGTH);
    ipAddrRef[0] = 200;
    tm_assert(tfxInetPtoN, !tm_memcmp(ipAddr, ipAddrRef,TM_IP_ADDRESS_LENGTH));

    errorCode = inet_pton(AF_INET, "255.255.255.255", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 1);
    tm_memset(ipAddrRef, 255, TM_IP_ADDRESS_LENGTH);
    tm_assert(tfxInetPtoN, !tm_memcmp(ipAddr, ipAddrRef,TM_IP_ADDRESS_LENGTH));

/* AF_INET -- bad */
    errorCode = inet_pton(AF_INET, "255.255.255.256", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 0);

    errorCode = inet_pton(AF_INET, "a.b.c.d", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 0);

    errorCode = inet_pton(AF_INET, "10.20.30.40.50", (void*)ipAddr);
    tm_assert(tfxInetPtoN, errorCode == 0);

}



/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tupton.c      File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 41061    Output checksum: 61627
 */
