/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Automated test code for IPv6 raw sockets (tf6RawSocket)
 *
 * Filename: tu6raw.c
 * Author: Jason Schmidlapp
 * Date Created: 12/5/2002
 * $Source: test/unittest/turaw.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:46CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */



/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#ifdef TM_MULTIPLE_CONTEXT
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"



/* constant declarations and enumerations */

/* type definitions */

/* local function prototypes */

/* local variable definitions */

/* static const tables */

/* macro definitions */

/* tf6RawSocket test types (see below for description) */
#define TM_6_RAW_TEST_UNICAST        0
#define TM_6_RAW_TEST_UNICAST_HDR    1
#define TM_6_RAW_TEST_MULTICAST      2
#define TM_6_RAW_TEST_MULTICAST_HDR  3
#define TM_6_RAW_TEST_LAST           TM_6_RAW_TEST_UNICAST_HDR + 1

/* Maximum length of test packets, for tests without included IPv6 header.  This
   should be more than 1240 (to test fragmentation) but not greater than 2480. */

/* Maximum length of test packets, for test with included IPv6 header.  Must not
   exceed MTU. */
#define TM_6_RAW_TEST_MAX_LEN_HDR 1280

#define TM_6_RAW_CS_OFFSET 2

/* Example test protocol */
#define TM_6_RAW_TEST_PROTO             77
#define TM_6_RAW_TEST_PROTO_NO_SERVER   78

void tfxRawSendToFrom(int serverSd, 
                      int clientSd, 
                      int family,
                      struct sockaddr_storage *clientSockAddrPtr,
                      struct sockaddr_storage *serverSockAddrPtr,
                      int addrLen,
                      int maxLength,
                      int hdrLength,
                      int rcvHdrLength);



void tfx6TestRawSockets(void)
{
    return;
}

void tfx6TestRawSocketsSendToFrom(void)
{
    return;
}


#ifdef TM_USE_RAW_SOCKET
#define TM_RAW_TEST_PROTO       20
void tfxTestRawSocketsSendToFromProto(int proto);
void tfxRawSockSendToFrom(void)
{
    tfxTestRawSocketsSendToFromProto(TM_RAW_TEST_PROTO);
    tfxTestRawSocketsSendToFromProto(IPPROTO_ICMP);
}

void tfxTestRawSocketsSendToFromProto(int proto)
{
    struct sockaddr_storage *serverSockAddrPtr;
    struct sockaddr_storage *clientSockAddrPtr;
    int                     clientSd;
    int                     serverSd;
    int                     sockAddrLength;
    int                     hdrLength;
    int                     maxLength;
    int                     rcvHdrLength;

#define TM_RAW_TEST_MAX_LEN_HDR 1480
    hdrLength = 0; /* Stack includes IP header */
    maxLength = TM_RAW_TEST_MAX_LEN_HDR; /* no fragmentation */
    tfSetCurrentContext(tvContextHandleClient);
    clientSd = tfRawSocket(tv4ClientSockAddr[0].addr.ipv4.sin_addr.s_addr,
                           proto);
    tm_assert(tfxRawSockSendToFrom, clientSd >= 0);
    tfBlockingState(clientSd, TM_BLOCKING_OFF);

    tfSetCurrentContext(tvContextHandleServer);
    serverSd = tfRawSocket(tv4ServerSockAddr[0].addr.ipv4.sin_addr.s_addr,
                            proto);
    tm_assert(tfxRawSockSendToFrom, serverSd >= 0);
    tfBlockingState(serverSd, TM_BLOCKING_OFF);

    tfxGetServerSockAddr(0,
                         AF_INET, 
                         (struct sockaddr**)&serverSockAddrPtr, 
                         &sockAddrLength);
    tfxGetClientSockAddr(0, 
                         AF_INET, 
                         (struct sockaddr**)&clientSockAddrPtr, 
                         &sockAddrLength);
    rcvHdrLength = 20;
    tfxRawSendToFrom(serverSd, 
                     clientSd, 
                     AF_INET,
                     clientSockAddrPtr,
                     serverSockAddrPtr,
                     sockAddrLength,
                     maxLength,
                     hdrLength,
                     20);
    tfSetCurrentContext(tvContextHandleClient);
    tfClose(clientSd);
    tfSetCurrentContext(tvContextHandleServer);
    tfClose(serverSd);
}

#else /* !TM_USE_RAW_SOCKET */
void tfxRawSockSendToFrom(void)
{
    return;
}
#endif /* TM_USE_RAW_SOCKET */

void tfxRawSendToFrom(int serverSd, 
                      int clientSd, 
                      int family,
                      struct sockaddr_storage *clientSockAddrPtr,
                      struct sockaddr_storage *serverSockAddrPtr,
                      int addrLen,
                      int maxLength,
                      int hdrLength,
                      int rcvHdrLength)
{
    int                     sentLen;
    int                     recvLen;
    ttCharPtr               sendBufPtr;
    ttCharPtr               recvBufPtr;
    ttUserContext           oldContext;
    struct sockaddr_storage clientAddr;
#ifndef TM_SINGLE_INTERFACE_HOME
    struct sockaddr_storage client2Addr;
    struct sockaddr_storage *clientSockAddr2Ptr;
#endif /* !TM_SINGLE_INTERFACE_HOME */
    struct sockaddr_storage serverAddr;
    int                     fromLength;
    int                     errorCode;

#ifdef TM_SINGLE_INTERFACE_HOME
    TM_UNREF_IN_ARG(family);
#endif /* TM_SINGLE_INTERFACE_HOME */
    oldContext = tfGetCurrentContext();

/* switch to the client node */
    tfSetCurrentContext(tvContextHandleClient);

/* send a buffer to the server*/
/* Allocate working buffer for sending */
    sendBufPtr = tm_get_raw_buffer(maxLength + hdrLength);
    tm_assert(tfxRawSendToFrom, sendBufPtr != 0);
/* Allocate working buffer for reception. */    
    recvBufPtr = tm_get_raw_buffer(maxLength + rcvHdrLength);
    FillSendBuffer(sendBufPtr+hdrLength,  maxLength);
    sentLen = tfSendToFrom(clientSd,
                           sendBufPtr,
                           maxLength + hdrLength,
                           0,
                           (struct sockaddr TM_FAR*) serverSockAddrPtr,
                           addrLen,
                           (struct sockaddr TM_FAR*) clientSockAddrPtr );

    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxRawSendToFrom, sentLen != TM_SOCKET_ERROR); 
    tm_assert(tfxRawSendToFrom, sentLen == maxLength + hdrLength); 
    
/* switch to the server node */
/* recvfrom at the server. */
    tfSetCurrentContext(tvContextHandleServer);
    fromLength = sizeof(struct sockaddr_in);

/* use recvfrom for non connected server socket */
    recvLen = tfxLoopRecvFrom(
        serverSd,
        recvBufPtr,
        maxLength+rcvHdrLength,
        (struct sockaddr *)&clientAddr,
        &fromLength);

    tm_assert(tfxRawSendToFrom, recvLen != TM_SOCKET_ERROR); 
    tm_assert(tfxRawSendToFrom, recvLen == maxLength + rcvHdrLength); 

/* server reply to the client */
    sentLen = tfSendToFrom(serverSd,
                           recvBufPtr + rcvHdrLength,
                           maxLength,
                           0,
                           (const struct sockaddr*)&clientAddr,
                           fromLength,
                           (struct sockaddr TM_FAR*) serverSockAddrPtr );

    tm_assert(tfxRawSendToFrom, sentLen != TM_SOCKET_ERROR); 
    tm_assert(tfxRawSendToFrom, sentLen == maxLength); 

/* switch to the client node */
/* recv at the client */
    tfSetCurrentContext(tvContextHandleClient);
    fromLength = sizeof(struct sockaddr_storage);
    recvLen = tfxLoopRecvFrom(
        clientSd,
        recvBufPtr,
        maxLength + rcvHdrLength,
        (struct sockaddr *)&serverAddr,
        &fromLength);
    tm_assert(tfxRawSendToFrom, recvLen != TM_SOCKET_ERROR); 
    tm_assert(tfxRawSendToFrom, recvLen == maxLength + rcvHdrLength); 
/* Add +4 because of checksum overwrite when socket is raw ICMPV6 */
    tm_assert(tfxRawSendToFrom, !tm_memcmp(sendBufPtr + hdrLength + 4, 
                                           recvBufPtr + rcvHdrLength + 4, 
                                           maxLength - 4)); 
#ifndef TM_SINGLE_INTERFACE_HOME
    {
/* 
 * BUG ID 815: Verify that the client source address has not been changed by
 * tfSendToFrom
 */
/* 
 * Verify that tfSendToFrom does not change client source address.
 */
/* server send to an IP address different from the client */
        tfxGetClientSockAddr(1, 
                             family, 
                             (struct sockaddr**)&clientSockAddr2Ptr, 
                             &fromLength);
/* Use a client address different from the bound one */
        sentLen = tfSendToFrom(clientSd,
                               sendBufPtr,
                               maxLength + hdrLength,
                               0,
                               (struct sockaddr TM_FAR*) serverSockAddrPtr,
                               addrLen,
                               (struct sockaddr TM_FAR*)clientSockAddr2Ptr );

        if(sentLen == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(clientSd);
        }
        tm_assert(tfxRawSendToFrom, sentLen != TM_SOCKET_ERROR); 
        tm_assert(tfxRawSendToFrom, sentLen == maxLength + hdrLength); 
/* switch to the server node */
        tfSetCurrentContext(tvContextHandleServer);
        fromLength = sizeof(struct sockaddr_in);
/* Verify that data was sent successfully */

/* use recvfrom for non connected server socket */
        recvLen = tfxLoopRecvFrom(
            serverSd,
            recvBufPtr,
            maxLength + rcvHdrLength,
            (struct sockaddr *)&client2Addr,
            &fromLength);

        tm_assert(tfxRawSendToFrom, recvLen != TM_SOCKET_ERROR); 
        tm_assert(tfxRawSendToFrom, recvLen == maxLength + rcvHdrLength); 

/* switch to the client node */
        tfSetCurrentContext(tvContextHandleClient);
/* Client socket source address should still match clientAddr */
        errorCode = getsockname(clientSd, 
                                (struct sockaddr *)&client2Addr,
                                &fromLength);
        if (errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(clientSd);
            tm_assert(tfxRawSendToFrom, errorCode == TM_ENOERROR);
        }
        if (family == AF_INET)
        {
            tm_assert(tfxRawSendToFrom, !tm_memcmp(
                                    &client2Addr.addr.ipv4.sin_addr, 
                                    &clientAddr.addr.ipv4.sin_addr, 
                                    sizeof(clientAddr.addr.ipv4.sin_addr)));
        }

/* End of BUG ID 815 */
    }
#endif /* !TM_SINGLE_INTERFACE_HOME */
    tfSetCurrentContext(tvContextHandleClient);
    tm_free_raw_buffer((ttRawBufferPtr)sendBufPtr);
    tm_free_raw_buffer((ttRawBufferPtr)recvBufPtr);
}
#endif /* TM_MULTIPLE_CONTEXT */

/***************** End Of File *****************/

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: turaw.c       File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 26185    Output checksum: 14491
 */
