/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Test of public socket API functions
 *              blocking mode sockets, two context, running in one task.
 * @REQUIREMENT TRACIBILTY [IPV6REQ] section2.18
 * Filename: tusocket.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002
 * $Source: test/unittest/tusocket.c $
 *
 * Modification History
 * $Revision: 6.0.2.10 $
 * $Date: 2011/12/14 00:54:56CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


#include <trsocket.h>
/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#ifdef TM_MULTIPLE_CONTEXT
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"
#else /* !TM_MULTIPLE_CONTEXT*/
#if defined(TM_ERROR_CHECKING) && defined(TM_USE_ANSI_LINE_FILE)
#include <trtype.h>   /* for trglobal.h */
#include <trglobal.h> /* for tm_assert */
#endif /* defined(TM_ERROR_CHECKING) && defined(TM_USE_ANSI_LINE_FILE) */
#endif /* !TM_MULTIPLE_CONTEXT*/

#ifdef TM_MULTIPLE_CONTEXT

#ifdef TM_USE_REUSEPORT
#ifdef TM_USE_TCP
static void tfxTcpSockReusePort(int af);
#endif /* TM_USE_TCP */
static void tfxUdpSockReusePort(int af);
static void tfxUdpSockReusePortLocalAddr (
    int                         af,
    struct sockaddr_storage     *mySockAddrPtr,
    int                         mySockAddrLength);
#if defined(TM_IGMP) || defined(TM_6_USE_MLD)
static void tfxUdpMCastSockReusePort(int af);
#endif /* defined(TM_IGMP) || defined(TM_6_USE_MLD) */
#endif /* TM_USE_REUSEPORT */

static void tfxTestOobData(int clientSd, int acceptSd);
static void tfx4UdpClientSendtoServer(
                                int serverSd,
                                struct sockaddr_storage * broadcastAddrPtr);


void tfx4SockReuseAddr(void)
{
#if (defined(TM_USE_REUSEADDR_LIST) && (!(defined(TM_SINGLE_INTERFACE_HOME))))
    ttUserContext   origContext;
    int reuse = 1;
    int sdTestReuse;
    struct sockaddr_storage * mySockaddrPtr;
    ttUserInterface   interfaceHandle;
    int sd;
    int errorCode;
    int mySockaddrLen;
    u_short tempAddr1Port;
    u_short tempAddr2Port;

    origContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleServer);
    interfaceHandle = tfxGetInterfaceByContext(tvContextHandleServer);
    sd = socket(
#ifdef TM_4_USE_SCOPE_ID
                AF_INET6,
#else /* TM_4_USE_SCOPE_ID */
                AF_INET,
#endif /* TM_4_USE_SCOPE_ID */
                SOCK_DGRAM,
                IPPROTO_UDP);
    tm_assert(tfx4SockReuseAddr, sd != -1);
    
    sdTestReuse = socket(
#ifdef TM_4_USE_SCOPE_ID
                         AF_INET6,
#else /* TM_4_USE_SCOPE_ID */
                         AF_INET,
#endif /* TM_4_USE_SCOPE_ID */
                         SOCK_DGRAM,
                         IPPROTO_UDP);
    tm_assert(tfx4SockReuseAddr, sdTestReuse != -1);
    
    tfxGetServerSockAddr(2,
                         AF_INET,
                         (struct sockaddr **)&mySockaddrPtr,
                         &mySockaddrLen);
    tempAddr1Port = mySockaddrPtr->ss_port;
    mySockaddrPtr->ss_port = htons(7);
    errorCode = bind(
        sd, (struct sockaddr *)mySockaddrPtr, mySockaddrLen);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
        tm_assert(tfx4SockReuseAddr, (errorCode == TM_ENOERROR));
    }

    tfxGetServerSockAddr(3,
                         AF_INET,
                         (struct sockaddr **)&mySockaddrPtr,
                         &mySockaddrLen);
    tempAddr2Port = mySockaddrPtr->ss_port;
    mySockaddrPtr->ss_port = htons(7);
    errorCode = bind(
        sdTestReuse,
        (struct sockaddr *)mySockaddrPtr, mySockaddrLen);
    tm_assert(tfx4SockReuseAddr, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdTestReuse);
    tm_assert(tfx4SockReuseAddr, (errorCode == TM_EADDRINUSE));

/* set SO_REUSEADDR on the 2nd socket, retry bind */
    errorCode = setsockopt(
        sdTestReuse, SOL_SOCKET, SO_REUSEADDR, (const char TM_FAR *)&reuse, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdTestReuse);
        tm_assert(tfx4SockReuseAddr, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdTestReuse,
        (struct sockaddr *)mySockaddrPtr, mySockaddrLen);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdTestReuse);
        tm_assert(tfx4SockReuseAddr, (errorCode == TM_ENOERROR));
    }

/* tear down the test setup */
    errorCode = tfClose(sd);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
        tm_assert(tfx4SockReuseAddr, (errorCode == TM_ENOERROR));
    }
    errorCode = tfClose(sdTestReuse);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdTestReuse);
        tm_assert(tfx4SockReuseAddr, (errorCode == TM_ENOERROR));
    }
    tfxGetServerSockAddr(2,
                         AF_INET,
                         (struct sockaddr **)&mySockaddrPtr,
                         &mySockaddrLen);
    mySockaddrPtr->ss_port = tempAddr1Port;
    tfxGetServerSockAddr(3,
                         AF_INET,
                         (struct sockaddr **)&mySockaddrPtr,
                         &mySockaddrLen);
    mySockaddrPtr->ss_port = tempAddr2Port;
    tfSetCurrentContext(origContext);
#endif /* TM_USE_REUSEADDR_LIST && !TM_SINGLE_INTERFACE_HOME */
    return;
}

#if (defined(TM_USE_IPV4) && defined(TM_USE_TCP) && defined(TM_USE_REUSEPORT))
void tfx4TcpSockReusePort(void)
{
#ifdef TM_4_USE_SCOPE_ID
    tfxTcpSockReusePort(AF_INET6);
#else /* TM_4_USE_SCOPE_ID */
    tfxTcpSockReusePort(AF_INET);
#endif /* TM_4_USE_SCOPE_ID */
    return;
}
#endif /* TM_USE_IPV4 && TM_USE_TCP && TM_USE_REUSEPORT */


#if (defined(TM_USE_TCP) && defined(TM_USE_REUSEPORT))
static void tfxTcpSockReusePort(int af)
{
    int                     optionOn = 1;
    int                     optionOff = 0;
    struct sockaddr_storage * sockAddrPtr;
    struct sockaddr_storage *clientSockAddrPtr;
    struct sockaddr_storage mySockaddr;
    int                     sockAddrLength;
    int                     sdA;
    int                     sdB;
    int                     serverSd;
    int                     acceptSd;
#ifndef TM_SINGLE_INTERFACE_HOME
    int                     serverSdB;
    int                     acceptSdB;
#endif /* TM_SINGLE_INTERFACE_HOME */
    int                     errorCode;

    tfSetCurrentContext(tvContextHandleClient);

    sdA = socket(af, SOCK_STREAM, IP_PROTOTCP);
    tm_assert(tfxTcpSockReusePort, sdA != -1);
    sdB = socket(af, SOCK_STREAM, IP_PROTOTCP);
    tm_assert(tfxTcpSockReusePort, sdB != -1);
    tfxGetClientSockAddr(0, 
                         af, 
                         (struct sockaddr**)&clientSockAddrPtr, 
                         &sockAddrLength);
    tm_bcopy(clientSockAddrPtr, &mySockaddr, sizeof(struct sockaddr_storage));

    mySockaddr.ss_port = htons(100);
/* 1.1 Bind both sockets without setting SO_REUSEPORT on either */
    errorCode = bind(
        sdA, (struct sockaddr *)&mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));

    errorCode = bind(
        sdB, (struct sockaddr *)&mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_EADDRINUSE));

/* 1.2 Bind both sockets with SO_REUSEPORT set on socket A only */
    errorCode = setsockopt(
        sdA, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *)&mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_EADDRINUSE));

/* 1.3 Bind both sockets with SO_REUSEPORT set on socket B only */
    errorCode = setsockopt(
        sdA, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOff, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = setsockopt(
        sdB, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *)&mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_EADDRINUSE));

/* 1.4 Bind both sockets with SO_REUSEPORT set on both sockets */
    errorCode = setsockopt(
        sdA, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *)&mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));

    errorCode = tfClose(sdA);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = tfClose(sdB);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }

/*
 * 2.1 With both sockets bound to the same local addr + port
 *     connect to the same remote addr + port.
 *     Note that the second bind does not fail even though SO_REUSEPORT is
 *     not set, because the first socket is already connected.
 *     The second socket connect should fail because it would violate
 *     the 4-tuple that uniquely identifies a TCP connection
 */
    tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd, &acceptSd,
                        &sdA, af, 1,
#ifdef TM_USE_STRONG_ESL
                        "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                        (struct sockaddr TM_FAR*)&mySockaddr);
    tfxGetServerSockAddr(0, af, (struct sockaddr **)&sockAddrPtr, &sockAddrLength);
    tfxExcerciseChannel(acceptSd, 
                        sdA, 
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        af,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
    sdB = socket(af, SOCK_STREAM, IP_PROTOTCP);
    tm_assert(tfxTcpSockReusePort, sdB != -1);
    errorCode = bind(
        sdB, (struct sockaddr *) &mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    errorCode = connect(sdB, (struct sockaddr *)sockAddrPtr, sockAddrLength);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_EADDRINUSE));

/* tear down the test setup */
    tfxCloseTcpConnection(serverSd, acceptSd, sdA);
    errorCode = tfClose(sdB);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
/*
 * 2.2 With both sockets bound to the same local addr + port
 *     connect to the same remote addr + port with both.
 *     open the second socket first so that we can set SO_REUSEPORT and bind
 *     it before we try and connect on the first one.
 *     The second socket connect should fail because it would violate
 *     the 4-tuple that uniquely identifies a TCP connection
 */
    sdB = socket(af, SOCK_STREAM, IP_PROTOTCP);
    tm_assert(tfxTcpSockReusePort, sdB != -1);
    errorCode = setsockopt(
        sdB, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *) &mySockaddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd, &acceptSd,
                        &sdA, af, 1,
#ifdef TM_USE_STRONG_ESL
                        "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                        (struct sockaddr TM_FAR*)&mySockaddr);
    tfxGetServerSockAddr(0, af, (struct sockaddr **)&sockAddrPtr,
                         &sockAddrLength);
    tfxExcerciseChannel(acceptSd, 
                        sdA, 
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        af,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
    errorCode = connect(sdB, (struct sockaddr *)sockAddrPtr, sockAddrLength);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_EADDRINUSE));

/* tear down the test setup */
    tfxCloseTcpConnection(serverSd, acceptSd, sdA);
    errorCode = tfClose(sdB);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    }
/*
 * 2.3 With both sockets bound to the same local addr + port
 *     connect to different remote addresses.
 *     The second socket connect should succeed because 
 *     the 4-tuple that uniquely identifies a TCP connection is different
 */
    tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd, &acceptSd,
                        &sdA, af, 1,
#ifdef TM_USE_STRONG_ESL
                        "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                        (struct sockaddr TM_FAR*)&mySockaddr);
    tfxGetServerSockAddr(0, af, (struct sockaddr **)&sockAddrPtr,
                         &sockAddrLength);
    tfxExcerciseChannel(acceptSd, 
                        sdA, 
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        af,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
#ifndef TM_SINGLE_INTERFACE_HOME
/* Connect to a different server */
    tfxGetTcpConnection(tvInterfaceHandleClient, 1, &serverSdB, &acceptSdB,
                        &sdB, af, 1,
#ifdef TM_USE_STRONG_ESL
                        "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                        (struct sockaddr TM_FAR*)&mySockaddr);
    tfxGetServerSockAddr(1, af, (struct sockaddr **)&sockAddrPtr,
                         &sockAddrLength);
    tfxExcerciseChannel(acceptSdB, 
                        sdB, 
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        af,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
    tfxCloseTcpConnection(serverSdB, acceptSdB, sdB);
#endif /* TM_SINGLE_INTERFACE_HOME */
/* tear down the test setup */
    tfxCloseTcpConnection(serverSd, acceptSd, sdA);
    tm_assert(tfxTcpSockReusePort, (errorCode == TM_ENOERROR));
    return;
}
#endif /* TM_USE_TCP && TM_USE_REUSEPORT */

/* test the v4 non connected UDP socket pair */
void tfx4NonConnUdp(void)
{
    int serverSd;
    int clientSd;
    int index;
    int sockAddrLength;
    struct sockaddr_storage *sockAddrPtr;

#ifdef TM_SINGLE_INTERFACE_HOME
    index = 0;
#else /* !TM_SINGLE_INTERFACE_HOME */
    index = 1;
#endif /* !TM_SINGLE_INTERFACE_HOME */
    tfxGetNonConnectedUdpPair(&serverSd, &clientSd, AF_INET, index
#ifdef TM_USE_STRONG_ESL
                             , "Client", "Server"
#endif /* TM_USE_STRONG_ESL */
                             );
    tfxGetServerSockAddr(index,
                         AF_INET, 
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);
    tfxExcerciseChannel(serverSd, 
                        clientSd, 
                        TM_TEST_SOCKET_TYPE_NON_CONNECTED,
                        TM_TEST_SOCKET_TYPE_NON_CONNECTED,
                        AF_INET,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
    tfxExcerciseChannel(serverSd, 
                        clientSd, 
                        TM_TEST_SOCKET_TYPE_NON_CONNECTED,
                        TM_TEST_SOCKET_TYPE_NON_CONNECTED,
                        AF_INET,
                        sockAddrPtr,
                        sockAddrLength,
                        1
                        );
    tfxCloseUdpConnection(serverSd, clientSd);
}

#if (defined(TM_USE_IPV4) && !defined(TM_4_USE_SCOPE_ID))
/* test the v4 non connected UDP socket pair using 255.255.255.255 */
void tfx4NonConnUdpBroadcast(void)
{
    struct sockaddr_storage zeroAddr;
    struct sockaddr_storage broadcastAddr;
    int                     serverSd;
    int                     errorCode;


/* Setup the zeroAddr to bind to on the server */
    tm_bzero(&zeroAddr, sizeof(struct sockaddr_storage));
    zeroAddr.addr.ipv4.sin_len = sizeof(struct sockaddr_in);
    zeroAddr.addr.ipv4.sin_family = AF_INET;
    zeroAddr.addr.ipv4.sin_port = htons(7);
    errorCode = inet_pton(AF_INET,
                          "0.0.0.0",
                          &zeroAddr.addr.ipv4.sin_addr);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == 1));

/* Test Limited Broadcast (255.255.255.255) */
    tm_bzero(&broadcastAddr, sizeof(struct sockaddr_storage));
    broadcastAddr.addr.ipv4.sin_len = sizeof(struct sockaddr_in);
    broadcastAddr.addr.ipv4.sin_family = AF_INET;
    broadcastAddr.addr.ipv4.sin_port = htons(7);
    errorCode = inet_pton(AF_INET,
                          "255.255.255.255",
                          &broadcastAddr.addr.ipv4.sin_addr);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == 1));

    tfSetCurrentContext(tvContextHandleServer);
    serverSd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    errorCode = bind(serverSd,
                     (struct sockaddr *)&zeroAddr,
                     sizeof(struct sockaddr_in));
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == TM_ENOERROR));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);

#ifndef TM_ALLOW_IP_SRC_CLASS_BROADCAST
/* Test Network Broadcast (10.255.255.255) */
    errorCode = inet_pton(AF_INET,
                          "10.255.255.255",
                          &broadcastAddr.addr.ipv4.sin_addr);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == 1));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);
#endif /* !TM_ALLOW_IP_SRC_CLASS_BROADCAST */

#ifndef TM_SINGLE_INTERFACE_HOME
#ifndef TM_ALLOW_IP_SRC_CLASS_BROADCAST
/* Test Network Broadcast (192.168.10.255) */
    tfSetCurrentContext(tvContextHandleClient);
    errorCode = inet_pton(AF_INET,
                          "192.168.10.255",
                          &broadcastAddr.addr.ipv4.sin_addr);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == 1));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);
#endif /* !TM_ALLOW_IP_SRC_CLASS_BROADCAST */

#if (TM_MAX_IPS_PER_IF >= 5)
#ifndef TM_ALLOW_IP_SRC_CLASS_BROADCAST
/* Test Network Broadcast (172.16.255.255) */
    tfSetCurrentContext(tvContextHandleClient);
    errorCode = inet_pton(AF_INET,
                          "172.16.255.255",
                          &broadcastAddr.addr.ipv4.sin_addr);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == 1));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);
#endif /* !TM_ALLOW_IP_SRC_CLASS_BROADCAST */

/* Test Subnet Broadcast (172.16.10.175/8 => 172.255.255.255) */
    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfGetBroadcastAddress(tvInterfaceHandleClient, 
                                      &broadcastAddr.addr.ipv4.sin_addr.s_addr,
                                      4);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == TM_ENOERROR));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);
#endif /* (TM_MAX_IPS_PER_IF >= 5) */

/* Test Subnet Broadcast (192.168.10.173/16 => 192.168.255.255) */
    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfGetBroadcastAddress(tvInterfaceHandleClient, 
                                      &broadcastAddr.addr.ipv4.sin_addr.s_addr,
                                      2);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == TM_ENOERROR));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);
#endif /* !TM_SINGLE_INTERFACE_HOME */

/* Test Subnet Broadcast (10.10.10.171/24 => 10.10.10.255) */
    tfSetCurrentContext(tvContextHandleClient);
    errorCode = tfGetBroadcastAddress(tvInterfaceHandleClient, 
                                      &broadcastAddr.addr.ipv4.sin_addr.s_addr,
                                      0);
    tm_assert(tfx4NonConnUdpBroadcast, (errorCode == TM_ENOERROR));

    tfx4UdpClientSendtoServer(serverSd, &broadcastAddr);


    tfSetCurrentContext(tvContextHandleServer);
    tfClose(serverSd);

    return;
}

static void tfx4UdpClientSendtoServer(
                                int serverSd,
                                struct sockaddr_storage * broadcastAddrPtr)
{
    char            buffer[TM_TEST_BUFFER_SIZE];
    ttUserContext   origContext;
    int             clientSd;
    int             sentLen;
    int             recvLen;


    origContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleClient);
    clientSd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    sentLen = sendto(clientSd,
                     buffer,
                     TM_TEST_BUFFER_SIZE,
                     0,
                     (struct sockaddr*)broadcastAddrPtr,
                     sizeof(struct sockaddr));
    tm_assert(tfx4NonConnUdpBroadcast, sentLen != TM_SOCKET_ERROR);

    tfSetCurrentContext(tvContextHandleServer);
    recvLen = recvfrom(serverSd,
                       buffer,
                       TM_TEST_BUFFER_SIZE,
                       0,
                       NULL,
                       0);
    tm_assert(tfx4NonConnUdpBroadcast, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfx4NonConnUdpBroadcast, recvLen == sentLen);

    tfSetCurrentContext(tvContextHandleClient);
    tfClose(clientSd);
    tfSetCurrentContext(origContext);
    return;
}
#endif /* TM_USE_IPV4 && !TM_4_USE_SCOPE_ID */

/* test v4 the connected UDP socket pair */
void tfx4ConnUdp(void)
{
    int serverSd;
    int clientSd;
    int sockAddrLength;
    struct sockaddr_storage *sockAddrPtr;

    tfxGetUdpConnection(tvInterfaceHandleClient, &serverSd, &clientSd,
                        AF_INET, 0
#ifdef TM_USE_STRONG_ESL
                        , "Client", "Server"
#endif /* TM_USE_STRONG_ESL */
                    );
    tfxGetServerSockAddr(0, 
                         AF_INET6, 
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);
    tfxExcerciseChannel(serverSd, 
                        clientSd, 
                        TM_TEST_SOCKET_TYPE_NON_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        AF_INET,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
    tfxExcerciseChannel(serverSd, 
                        clientSd, 
                        TM_TEST_SOCKET_TYPE_NON_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        AF_INET,
                        sockAddrPtr,
                        sockAddrLength,
                        1
                        );
    tfxCloseUdpConnection(serverSd, clientSd);

#ifdef TM_USE_FILTERING
    tfxIpFilteringTests(AF_INET, IPPROTO_UDP);
#endif /* TM_USE_FILTERING */

    return;
}

#if (   (defined(TM_USE_IPV4) && defined(TM_USE_REUSEPORT) \
     && !(defined(TM_4_USE_SCOPE_ID)) && !(defined(TM_SINGLE_INTERFACE_HOME))) )
void tfx4UdpSockReusePort(void)
{
    tfxUdpSockReusePort(AF_INET);
    return;
}
#endif /*    TM_USE_IPV4 && TM_USE_REUSEPORT
          && !TM_4_USE_SCOPE_ID && !TM_SINGLE_INTERFACE_HOME */


#ifdef TM_USE_REUSEPORT
void tfxUdpSockReusePort(int af)
{
    struct sockaddr_storage     *mySockAddrPtr;
    struct sockaddr_storage     mySockAddr;
    int                         mySockAddrLength;

/* Use a non zero local address, port 50 */
    tfxGetClientSockAddr(0, 
                         af, 
                         (struct sockaddr**)&mySockAddrPtr, 
                         &mySockAddrLength);
    mySockAddrPtr->ss_port = htons(50);
    tfxUdpSockReusePortLocalAddr(af, mySockAddrPtr, mySockAddrLength);

/* Use a zero local address, port 50 */
    tm_bzero(&mySockAddr, sizeof(mySockAddr));
    mySockAddrPtr = &mySockAddr;
    mySockAddrLength = sizeof(mySockAddr);
    mySockAddrPtr->ss_family = (tt8Bit)af;
    mySockAddrPtr->ss_port = htons(50);
    tfxUdpSockReusePortLocalAddr(af, mySockAddrPtr, mySockAddrLength);
}

static void tfxUdpSockReusePortLocalAddr (
    int                         af,
    struct sockaddr_storage     *mySockAddrPtr,
    int                         mySockAddrLength)
{
    struct sockaddr_storage     *sockAddrPtr;
    int                         sockAddrLength;
    int                         optionOn;
    int                         sd;
    int                         sd2;
    int                         errorCode;

    optionOn = 1;

    sd = socket(af, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4UdpSockReusePort, sd != -1);

    errorCode = setsockopt(
        sd, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
        tm_assert(tfx4UdpSockReusePort, (errorCode == TM_ENOERROR));
    }

    errorCode = bind(
        sd, (struct sockaddr *)mySockAddrPtr, mySockAddrLength);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
        tm_assert(tfx4UdpSockReusePort, (errorCode == TM_ENOERROR));
    }

    sd2 = socket(af, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfx4UdpSockReusePort, sd2 != -1);

    errorCode = setsockopt(
        sd2, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd2);
        tm_assert(tfx4UdpSockReusePort, (errorCode == TM_ENOERROR));
    }

    errorCode = bind(
        sd2, (struct sockaddr *)mySockAddrPtr, mySockAddrLength);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd2);
        tm_assert(tfx4UdpSockReusePort, (errorCode == TM_ENOERROR));
    }
    tfxGetServerSockAddr(0, 
                         af, 
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);
    errorCode = connect(sd,
                        (struct sockaddr *)sockAddrPtr,
                        sockAddrLength);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
        tm_assert(tfxUdpSockReusePort, (errorCode == TM_ENOERROR));
    }
    tfxGetServerSockAddr(1, 
                         af, 
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);
    errorCode = connect(sd2,
                        (struct sockaddr *)sockAddrPtr,
                        sockAddrLength);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd2);
        tm_assert(tfxUdpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = tfClose(sd);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd);
        tm_assert(tfxUdpSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = tfClose(sd2);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sd2);
        tm_assert(tfxUdpSockReusePort, (errorCode == TM_ENOERROR));
    }

    return;
}

#if (   (defined(TM_USE_IPV4) && defined(TM_IGMP) \
     && !(defined(TM_4_USE_SCOPE_ID))) )
void tfx4UdpMCastSockReusePort(void)
{
    tfxUdpMCastSockReusePort(AF_INET);
    return;
}
#endif /* TM_USE_IPV4 && TM_IGMP && !TM_4_USE_SCOPE_ID */


#if defined(TM_IGMP) || defined(TM_6_USE_MLD)
void tfxUdpMCastSockReusePort(int af)
{
    struct sockaddr_storage multicastAddr;
    struct sockaddr_storage *serverAddrPtr;
#ifndef TM_SINGLE_INTERFACE_HOME
    struct sockaddr_storage *serverAddr2Ptr;
#endif /* !TM_SINGLE_INTERFACE_HOME */
#ifdef TM_IGMP
    struct ip_mreq          v4SetValue;
#endif /* TM_IGMP */
    ttUserContext           oldContext;
    int                     serverAddrLength;
#ifndef TM_SINGLE_INTERFACE_HOME
    int                     serverAddr2Length;
#endif /* !TM_SINGLE_INTERFACE_HOME */
    int                     sdA;
    int                     sdB;
    int                     serverSd;
    int                     optionOn;
    int                     optionOff;
    int                     errorCode;
    int                     sentLen;
    int                     recvLen;
    char                    sendBuffer[TM_TEST_BUFFER_SIZE];
    char                    recvBuffer[TM_TEST_BUFFER_SIZE];

    optionOn = 1;
    optionOff = 0;

    oldContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleClient);

    sdA = socket(af, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfxUdpMCastSockReusePort, sdA != TM_SOCKET_ERROR);
    sdB = socket(af, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfxUdpMCastSockReusePort, sdB != TM_SOCKET_ERROR);

    if (af == AF_INET)
    {
        multicastAddr.addr.ipv4.sin_len = sizeof(struct sockaddr_in);
        multicastAddr.addr.ipv4.sin_family = AF_INET;
        multicastAddr.addr.ipv4.sin_port = htons(100);
        errorCode = inet_pton(AF_INET, "224.0.0.1", &multicastAddr.addr.ipv4.sin_addr);
        tm_assert(tfxUdpMCastSockReusePort, errorCode == 1);
    }

/* 1.1 Bind both sockets without setting SO_REUSEPORT on either */
    errorCode = bind(
        sdA, (struct sockaddr *)&multicastAddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));

    errorCode = bind(
        sdB, (struct sockaddr *)&multicastAddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_EADDRINUSE));

/* 1.2 Bind both sockets with SO_REUSEPORT set on socket A only */
    errorCode = setsockopt(
        sdA, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *)&multicastAddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_EADDRINUSE));

/* 1.3 Bind both sockets with SO_REUSEPORT set on socket B only */
    errorCode = setsockopt(
        sdA, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOff, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = setsockopt(
        sdB, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *)&multicastAddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_SOCKET_ERROR));
    errorCode = tfGetSocketError(sdB);
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_EADDRINUSE));

/* 1.4 Bind both sockets with SO_REUSEPORT set on both sockets */
    errorCode = setsockopt(
        sdA, SOL_SOCKET, SO_REUSEPORT, (const char TM_FAR *)&optionOn, 
        sizeof(int));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = bind(
        sdB, (struct sockaddr *)&multicastAddr, sizeof(struct sockaddr_storage));
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));


/* Join a multicast group */
#ifdef TM_IGMP
    if (af == AF_INET)
    {
        v4SetValue.imr_multiaddr.s_addr = inet_addr("224.0.0.1");
        v4SetValue.imr_interface.s_addr = 0;
        errorCode = setsockopt(sdA, IP_PROTOIP, IP_ADD_MEMBERSHIP,
                               (char *)&v4SetValue, sizeof(v4SetValue));
        if(errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(sdA);
            tm_assert(tfxUdpMCastSockReusePort, errorCode == TM_ENOERROR);
        }
        errorCode = setsockopt(sdB, IP_PROTOIP, IP_ADD_MEMBERSHIP,
                               (char *)&v4SetValue, sizeof(v4SetValue));
        if(errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(sdB);
            tm_assert(tfxUdpMCastSockReusePort, errorCode == TM_ENOERROR);
        }
    }
#endif /* TM_IGMP */

/* 2.1 Send data from the server to the multicast address */
    tfSetCurrentContext(tvContextHandleServer);

    serverSd = socket(af, SOCK_DGRAM, IPPROTO_UDP);
    tm_assert(tfxUdpMCastSockReusePort, serverSd != TM_SOCKET_ERROR);

    tfxGetServerSockAddr(0,
                         af,
                         (struct sockaddr **)&serverAddrPtr,
                         &serverAddrLength);
    errorCode = bind(serverSd,
                     (struct sockaddr *)serverAddrPtr,
                     serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));

    sentLen = sendto(serverSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0,
                     (struct sockaddr *)&multicastAddr,
                     sizeof(multicastAddr));
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfxUdpMCastSockReusePort, sentLen != TM_SOCKET_ERROR);
        tm_assert(tfxUdpMCastSockReusePort, sentLen == TM_TEST_BUFFER_SIZE);
    }

/* 2.2 Both sockets should receive the data */
    tfSetCurrentContext(tvContextHandleClient);
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        sdA,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxUdpMCastSockReusePort, !tm_memcmp(sendBuffer,
                                                   recvBuffer,
                                                   TM_TEST_BUFFER_SIZE));

    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        sdB,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxUdpMCastSockReusePort, !tm_memcmp(sendBuffer,
                                                   recvBuffer,
                                                   TM_TEST_BUFFER_SIZE));

#ifndef TM_SINGLE_INTERFACE_HOME
/* 3.1 Connect socket A to the server's second address */
    tfxGetServerSockAddr(1,
                         af,
                         (struct sockaddr **)&serverAddr2Ptr,
                         &serverAddr2Length);
    errorCode = connect(sdA,
                        (struct sockaddr *)serverAddr2Ptr,
                        serverAddr2Length);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }

/* 3.2 Send data from the server to the multicast address */
    tfSetCurrentContext(tvContextHandleServer);

    sentLen = sendto(serverSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0,
                     (struct sockaddr *)&multicastAddr,
                     sizeof(multicastAddr));
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfxUdpMCastSockReusePort, sentLen != TM_SOCKET_ERROR);
        tm_assert(tfxUdpMCastSockReusePort, sentLen == TM_TEST_BUFFER_SIZE);
    }

/* 3.3 Only socket B should receive the data */
    tfSetCurrentContext(tvContextHandleClient);
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        sdB,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxUdpMCastSockReusePort, !tm_memcmp(sendBuffer,
                                                   recvBuffer,
                                                   TM_TEST_BUFFER_SIZE));

    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = recvfrom(
        sdA,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        0,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_SOCKET_ERROR);
#endif /* !TM_SINGLE_INTERFACE_HOME */

/* 4.1 Connect socket B to the server's first address */
    errorCode = connect(sdB,
                        (struct sockaddr *)serverAddrPtr,
                        serverAddrLength);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }

/* 4.2 Send data from the server to the multicast address */
    tfSetCurrentContext(tvContextHandleServer);

    sentLen = sendto(serverSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0,
                     (struct sockaddr *)&multicastAddr,
                     sizeof(multicastAddr));
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfxUdpMCastSockReusePort, sentLen != TM_SOCKET_ERROR);
        tm_assert(tfxUdpMCastSockReusePort, sentLen == TM_TEST_BUFFER_SIZE);
    }

/* 4.3 Only the socket B should receive the data */
    tfSetCurrentContext(tvContextHandleClient);
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        sdB,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxUdpMCastSockReusePort, !tm_memcmp(sendBuffer,
                                                   recvBuffer,
                                                   TM_TEST_BUFFER_SIZE));

#ifndef TM_SINGLE_INTERFACE_HOME
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = recvfrom(
        sdA,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        0,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_SOCKET_ERROR);
#endif /* !TM_SINGLE_INTERFACE_HOME */

/* 5.1 Connect socket A to the server's first address */
    errorCode = connect(sdA,
                        (struct sockaddr *)serverAddrPtr,
                        serverAddrLength);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }

/* 5.2 Send data from the server to the multicast address */
    tfSetCurrentContext(tvContextHandleServer);

    sentLen = sendto(serverSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0,
                     (struct sockaddr *)&multicastAddr,
                     sizeof(multicastAddr));
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfxUdpMCastSockReusePort, sentLen != TM_SOCKET_ERROR);
        tm_assert(tfxUdpMCastSockReusePort, sentLen == TM_TEST_BUFFER_SIZE);
    }

/* 5.3 Both sockets should receive the data */
    tfSetCurrentContext(tvContextHandleClient);
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        sdA,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxUdpMCastSockReusePort, !tm_memcmp(sendBuffer,
                                                   recvBuffer,
                                                   TM_TEST_BUFFER_SIZE));

    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        sdB,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)serverAddrPtr,
        &serverAddrLength);
    tm_assert(tfxUdpMCastSockReusePort, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxUdpMCastSockReusePort, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxUdpMCastSockReusePort, !tm_memcmp(sendBuffer,
                                                   recvBuffer,
                                                   TM_TEST_BUFFER_SIZE));

/* Close the sockets */
    errorCode = tfClose(sdA);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdA);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }
    errorCode = tfClose(sdB);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(sdB);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }

    tfSetCurrentContext(tvContextHandleServer);
    
    tfClose(serverSd);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
        tm_assert(tfxUdpMCastSockReusePort, (errorCode == TM_ENOERROR));
    }

    tfSetCurrentContext(oldContext);

    return;
}
#endif /* defined(TM_IGMP) || defined(TM_6_USE_MLD) */
#endif /* TM_USE_REUSEPORT */

/* test the v4 TCP connection */
void tfx4TcpSocket(void)
{
    struct sockaddr_storage *   sockAddrPtr;
    int                         serverSd;
    int                         acceptSd;
    int                         clientSd;
    int                         sockAddrLength;

    tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd,
                        &acceptSd, &clientSd, AF_INET, 1,
#ifdef TM_USE_STRONG_ESL
                        "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                        0);
    tfxGetServerSockAddr(0, 
                         AF_INET, 
                         (struct sockaddr**)&sockAddrPtr, 
                         &sockAddrLength);
    tfxTestOobData(clientSd, acceptSd);
    tfxExcerciseChannel(acceptSd, 
                        clientSd, 
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        AF_INET,
                        sockAddrPtr,
                        sockAddrLength,
                        0
                        );
    tfxExcerciseChannel(acceptSd, 
                        clientSd, 
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        TM_TEST_SOCKET_TYPE_CONNECTED,
                        AF_INET,
                        sockAddrPtr,
                        sockAddrLength,
                        2
                        );
    tfxCloseTcpConnection(serverSd, acceptSd, clientSd);

#ifdef TM_USE_FILTERING
    tfxIpFilteringTests(AF_INET, IPPROTO_TCP);
#endif /* TM_USE_FILTERING */

    return;
}

static void tfxTestOobData(int clientSd, int acceptSd)
{
    ttUserContext       oldContext;
    int                 errorCode;
    int                 param;
    int                 sendDataLen;
    int                 sentLen;
    int                 recvDataLen;
    int                 recvdLen;
    char                recvBuffer[TM_TEST_BUFFER_SIZE];

    oldContext = tfGetCurrentContext();

    tfSetCurrentContext(tvContextHandleServer);
    param = 0;
    errorCode = setsockopt(acceptSd, SOL_SOCKET, SO_OOBINLINE,
                           (ttConstCharPtr)&param, sizeof(param));
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tfSetCurrentContext(tvContextHandleClient);
    param = 1;
    errorCode = setsockopt(clientSd, IPPROTO_TCP,
                           TCP_NODELAY, (ttConstCharPtr)&param,
                           sizeof(param));
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
/* TEST sending all the data and receiving all the data */
/* Client sends */
    tfSetCurrentContext(tvContextHandleClient);
    recvDataLen = 0;
    sendDataLen = 11;
    sentLen = send(clientSd, "OOB: BEFORE", sendDataLen, 0);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen += sentLen;
    sendDataLen = 1;
    sentLen = send(clientSd, "!", sendDataLen, MSG_OOB);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen += sentLen;
    sendDataLen = 5;
    sentLen = send(clientSd, "AFTER", sendDataLen, 0);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen += sentLen;
    tfSetCurrentContext(tvContextHandleServer);
/* Server recvs */
    recvdLen = recv(acceptSd,
                    recvBuffer,
                    recvDataLen,
                    0);

    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 11);
    tm_assert(tfxTestOobData,
              tm_strncmp("OOB: BEFORE", recvBuffer, 11) == 0);
    recvdLen = recv(acceptSd, recvBuffer, 1, MSG_OOB);
    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 1);
    tm_assert(tfxTestOobData,
              tm_strncmp("!", recvBuffer, recvdLen) == 0);
    recvdLen = recv(acceptSd,
                    recvBuffer,
                    recvDataLen,
                    0);

    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 5);
    tm_assert(tfxTestOobData,
              tm_strncmp("AFTER", recvBuffer, 5) == 0);

/* TEST sending and receiving alternatively */
/* Client sends */
    tfSetCurrentContext(tvContextHandleClient);
    recvDataLen = 0;
    sendDataLen = 11;
    sentLen = send(clientSd, "OOB: BEFORE", sendDataLen, 0);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen = sentLen;
    tfSetCurrentContext(tvContextHandleServer);
/* Server recvs */
    recvdLen = recv(acceptSd,
                    recvBuffer,
                    recvDataLen,
                    0);

    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 11);
    tm_assert(tfxTestOobData,
              tm_strncmp("OOB: BEFORE", recvBuffer, 11) == 0);
    recvdLen = recv(acceptSd, recvBuffer, 1, MSG_OOB);
/* Client sends */
    tfSetCurrentContext(tvContextHandleClient);
    sendDataLen = 1;
    sentLen = send(clientSd, "!", sendDataLen, MSG_OOB);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen = sentLen;
    tfSetCurrentContext(tvContextHandleServer);
/* Server recvs */
    recvdLen = recv(acceptSd, recvBuffer, recvDataLen, 0);
    tm_assert(tfxTestOobData, recvdLen == TM_SOCKET_ERROR);
    recvdLen = recv(acceptSd, recvBuffer, recvDataLen, MSG_OOB);
    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 1);
    tm_assert(tfxTestOobData,
              tm_strncmp("!", recvBuffer, recvdLen) == 0);

/* Client sends */
    tfSetCurrentContext(tvContextHandleClient);
    sendDataLen = 5;
    sentLen = send(clientSd, "AFTER", sendDataLen, 0);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen = sentLen;
    tfSetCurrentContext(tvContextHandleServer);
/* Server recvs */
    recvdLen = recv(acceptSd,
                    recvBuffer,
                    recvDataLen,
                    0);

    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 5);
    tm_assert(tfxTestOobData,
              tm_strncmp("AFTER", recvBuffer, 5) == 0);
/* TEST not receiving the OOB data */
/* Client sends */
    tfSetCurrentContext(tvContextHandleClient);
    recvDataLen = 0;
    sendDataLen = 11;
    sentLen = send(clientSd, "OOB: BEFORE", sendDataLen, 0);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen += sentLen;
    sendDataLen = 1;
    sentLen = send(clientSd, "!", sendDataLen, MSG_OOB);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen += sentLen;
    sendDataLen = 5;
    sentLen = send(clientSd, "AFTER", sendDataLen, 0);
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    recvDataLen += sentLen;
    tfSetCurrentContext(tvContextHandleServer);
/* Server recvs */
    recvdLen = recv(acceptSd,
                    recvBuffer,
                    recvDataLen,
                    0);

    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 11);
    tm_assert(tfxTestOobData,
              tm_strncmp("OOB: BEFORE", recvBuffer, 11) == 0);
    recvdLen = recv(acceptSd,
                    recvBuffer,
                    recvDataLen,
                    0);

    if (recvdLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(acceptSd);
        tm_assert(tfxTestOobData, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxTestOobData, recvdLen == 5);
    tm_assert(tfxTestOobData,
              tm_strncmp("AFTER", recvBuffer, 5) == 0);

    tfSetCurrentContext(oldContext);
}


/*
 * create a TCP socket in the server context
 * bind it to tv6ServerSockAddr[index]
 * establish a connection from the client and
 * carry back the socket pair
 */
void tfxGetTcpConnection(
                         ttUserInterface interfaceHandleClient,
                         int   index,
                         int * serverSdPtr, 
                         int * acceptSdPtr, 
                         int * clientSdPtr,
                         int family,
                         int enableTimestamps,
#ifdef TM_USE_STRONG_ESL
                         ttCharPtr clientNamePtr,
                         ttCharPtr serverNamePtr,
#endif /* TM_USE_STRONG_ESL */
                         struct sockaddr TM_FAR* clientSockAddrPtr
                         )
{
    int             errorCode;
    ttUserContext   oldContext;
    fd_set          readFdset;
    int             tcpWaitTime;
    struct linger   lingerStruct;
    tt32Bit         rcvBuf;
    int             rcvBufSize;
    int             param;

    struct sockaddr TM_FAR* sockAddrPtr;
    int             sockAddrLength;
    
    TM_UNREF_IN_ARG(interfaceHandleClient);

    tcpWaitTime = 0;
    lingerStruct.l_onoff = 1;
    lingerStruct.l_linger= 0;
    
    oldContext = tfGetCurrentContext();

/* let's come to the server node */
    tfSetCurrentContext(tvContextHandleServer);

/* create a server socket */
#ifdef TM_4_USE_SCOPE_ID
    *serverSdPtr = socket(AF_INET6, SOCK_STREAM, IP_PROTOTCP);
#else /* ! TM_4_USE_SCOPE_ID */
    *serverSdPtr = socket(family, SOCK_STREAM, IP_PROTOTCP);
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxGetTcpConnection, *serverSdPtr != TM_SOCKET_ERROR);
    tfBlockingState(*serverSdPtr, TM_BLOCKING_OFF);

    if (enableTimestamps == 0)
    {
        param = 0;
    }
    else
    {
        param = 1;
    }
    errorCode = setsockopt(*serverSdPtr,
                           IP_PROTOTCP,
                           TM_TCP_TS,
                           (const char *) &param,
                           sizeof(param));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }

/* bind the server socket to tv6ServerSockAddr[index]*/
    tfxGetServerSockAddr(index, family, &sockAddrPtr, &sockAddrLength);
    errorCode = bind(*serverSdPtr,
                     sockAddrPtr,
                     sockAddrLength
                     );
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
#ifdef TM_USE_STRONG_ESL
    errorCode = setsockopt(*serverSdPtr,
                           SOL_SOCKET,
                           SO_BINDTODEVICE,
                           serverNamePtr,
                           IFNAMSIZ);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
#endif /* TM_USE_STRONG_ESL */
    errorCode = setsockopt(*serverSdPtr, 
               IP_PROTOTCP, 
               TM_TCP_2MSLTIME, 
               (ttConstCharPtr)&tcpWaitTime, 
               sizeof(tcpWaitTime));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
    errorCode = setsockopt(*serverSdPtr,
               SOL_SOCKET,
               SO_LINGER,
               (char*)&lingerStruct,
               sizeof(lingerStruct));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }

/* listen at the bound socket */
    errorCode = listen(*serverSdPtr, 1);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
/* Get the recv queue size */
    rcvBufSize = sizeof(rcvBuf);
    errorCode = getsockopt(*serverSdPtr, SOL_SOCKET, SO_RCVBUF,
                           (ttCharPtr)&rcvBuf, &rcvBufSize);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
    rcvBuf = rcvBuf + 1;
/* Set the TCP pre accept recv window to be too big. Should get an error */
    errorCode = setsockopt(*serverSdPtr, IPPROTO_TCP,
                           TM_TCP_PEND_ACCEPT_RECV_WND, (ttConstCharPtr)&rcvBuf,
                           sizeof(rcvBuf));
    tm_assert(tfxGetTcpConnection, errorCode == TM_SOCKET_ERROR);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_EPERM);
    }
/*
 * Set the TCP pre accept recv window to be the same as the recv
 * window. Should succeed.
 */
    rcvBuf = rcvBuf - 1;
    errorCode = setsockopt(*serverSdPtr, IPPROTO_TCP,
                           TM_TCP_PEND_ACCEPT_RECV_WND, (ttConstCharPtr)&rcvBuf,
                           sizeof(rcvBuf));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
/*
 * Set the TCP pre accept recv window to be smaller than the recv
 * window. Should succeed.
 */
    rcvBuf = rcvBuf - 1;
    errorCode = setsockopt(*serverSdPtr, IPPROTO_TCP,
                           TM_TCP_PEND_ACCEPT_RECV_WND, (ttConstCharPtr)&rcvBuf,
                           sizeof(rcvBuf));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }

/* let's switch to the client node */    
    tfSetCurrentContext(tvContextHandleClient);
 
/* create a socket at the client */
#ifdef TM_4_USE_SCOPE_ID
    *clientSdPtr = socket(AF_INET6, SOCK_STREAM, IP_PROTOTCP);
#else /* ! TM_4_USE_SCOPE_ID */
    *clientSdPtr = socket(family, SOCK_STREAM, IP_PROTOTCP);
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxGetTcpConnection, *clientSdPtr != TM_SOCKET_ERROR);
    tfBlockingState(*clientSdPtr, TM_BLOCKING_OFF);
#ifdef TM_USE_STRONG_ESL
    errorCode = setsockopt(*clientSdPtr, SOL_SOCKET, SO_BINDTODEVICE, clientNamePtr, IFNAMSIZ);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
#endif /* TM_USE_STRONG_ESL */

    if(clientSockAddrPtr)
    {
        param = 1;
        errorCode = setsockopt(
            *clientSdPtr, SOL_SOCKET, SO_REUSEPORT,
            (const char TM_FAR *)&param, sizeof(int));
        if (errorCode == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(*clientSdPtr);
            tm_assert(tfxGetTcpConnection, (errorCode == TM_ENOERROR));
        }
        errorCode = bind(*clientSdPtr,
                         clientSockAddrPtr,
                         clientSockAddrPtr->sa_len);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
    errorCode = setsockopt(*clientSdPtr, 
               IP_PROTOTCP, 
               TM_TCP_2MSLTIME, 
               (ttCharPtr)&tcpWaitTime, 
               sizeof(tcpWaitTime));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
    errorCode = setsockopt(*clientSdPtr, 
               SOL_SOCKET,
               SO_LINGER,
               (char*)&lingerStruct,
               sizeof(lingerStruct));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
/* try to connect to the server*/
    errorCode = connect(*clientSdPtr,
                        sockAddrPtr,
                        sockAddrLength
                        );
/* pool for the connect operation to complete */
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_EINPROGRESS);
        while ((errorCode = tfGetSocketError(*clientSdPtr)) == TM_EINPROGRESS)
        {
#ifdef TM_KERNEL_VISUAL_X86
            tfTimerUpdate();
            tfTimerExecute();
#endif /* TM_KERNEL_VISUAL_X86 */
        }
    }
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }

/* switch to the server node */
    tfSetCurrentContext(tvContextHandleServer);


    FD_ZERO(&readFdset);
    FD_SET(*serverSdPtr, &readFdset);
    errorCode = select(*serverSdPtr + 1, 
                       &readFdset,
                       0,
                       0,
                       0);
    tm_assert(tfxGetTcpConnection,errorCode >0)
    tm_assert(tfxGetTcpConnection, FD_ISSET(*serverSdPtr, &readFdset) > 0);

/* accept the incoming connection from the client*/
    *acceptSdPtr = accept(*serverSdPtr,
                      (struct sockaddr TM_FAR *)0,
                      (int TM_FAR             *)0
                     );
    if (*acceptSdPtr == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
    errorCode = setsockopt(*acceptSdPtr, 
               IP_PROTOTCP, 
               TM_TCP_2MSLTIME, 
               (ttCharPtr)&tcpWaitTime, 
               sizeof(tcpWaitTime));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*acceptSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }
    tm_assert(tfxGetTcpConnection, errorCode != TM_SOCKET_ERROR);
    errorCode = setsockopt(*acceptSdPtr, 
               SOL_SOCKET,
               SO_LINGER,
               (char*)&lingerStruct,
               sizeof(lingerStruct));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(*acceptSdPtr);
        tm_assert(tfxGetTcpConnection, errorCode == TM_ENOERROR);
    }

    tfSetCurrentContext(oldContext);
}


/*
 * create a UDP socket in the server context
 * bind it to tv6ServerSockAddr[index]
 * establish a connection from the client and
 * carry back the connected UDP socket pair
 */
void tfxGetUdpConnection(ttUserInterface interfaceHandleClient,
                         int *serverSdPtr, int *clientSdPtr, int family,
                         int index
#ifdef TM_USE_STRONG_ESL
                       , ttCharPtr clientNamePtr
                       , ttCharPtr serverNamePtr
#endif /* TM_USE_STRONG_ESL */
                         )
{
    int           errorCode;
    ttUserContext oldContext;
    struct  sockaddr TM_FAR* sockAddrPtr;
    int           sockAddrLength;

    TM_UNREF_IN_ARG(interfaceHandleClient);

    oldContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleServer);

/* create server socket */
#ifdef TM_4_USE_SCOPE_ID
    *serverSdPtr = socket(AF_INET6, SOCK_DGRAM, IP_PROTOUDP);
#else /* ! TM_4_USE_SCOPE_ID */
    *serverSdPtr = socket(family, SOCK_DGRAM, IP_PROTOUDP);
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxGetUdpConnection, *serverSdPtr != TM_SOCKET_ERROR);
    tfBlockingState(*serverSdPtr, TM_BLOCKING_OFF);


/* server bind*/
    tfxGetServerSockAddr(index, family, &sockAddrPtr, &sockAddrLength);
    errorCode = bind(*serverSdPtr,
                     sockAddrPtr,
                     sockAddrLength);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetUdpConnection , errorCode == TM_ENOERROR);
    }
#ifdef TM_USE_STRONG_ESL
    errorCode = setsockopt(*serverSdPtr, SOL_SOCKET, SO_BINDTODEVICE, serverNamePtr, IFNAMSIZ);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetUdpConnection , errorCode == TM_ENOERROR);
    }
#endif /* TM_USE_STRONG_ESL */

    tfSetCurrentContext(tvContextHandleClient);

/* create client socket */
#ifdef TM_4_USE_SCOPE_ID
    *clientSdPtr = socket(AF_INET6, SOCK_DGRAM, IP_PROTOUDP);
#else /* ! TM_4_USE_SCOPE_ID */
    *clientSdPtr = socket(family, SOCK_DGRAM, IP_PROTOUDP);
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxGetUdpConnection, *clientSdPtr != TM_SOCKET_ERROR);
    tfBlockingState(*clientSdPtr, TM_BLOCKING_OFF);
#ifdef TM_USE_STRONG_ESL
    errorCode = setsockopt(*clientSdPtr, SOL_SOCKET, SO_BINDTODEVICE, clientNamePtr, IFNAMSIZ);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetUdpConnection , errorCode == TM_ENOERROR);
    }
#endif /* TM_USE_STRONG_ESL */

/* connect */
    errorCode = connect(*clientSdPtr,
                        sockAddrPtr,
                        sockAddrLength);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetUdpConnection , errorCode == TM_ENOERROR);
    }

    tfSetCurrentContext(oldContext);
}


/*
 * create a UDP socket in the server context
 * bind it to tv6ServerSockAddr[index]
 * carry back the non-connected UDP socket pair
 */
void tfxGetNonConnectedUdpPair(
    int *serverSdPtr, int *clientSdPtr, int family, int index
#ifdef TM_USE_STRONG_ESL
  , ttCharPtr clientNamePtr
  , ttCharPtr serverNamePtr
#endif /* TM_USE_STRONG_ESL */
    )
{
    ttUserContext oldContext;
    int           errorCode;
    struct  sockaddr TM_FAR* sockAddrPtr;
    int           sockAddrLength;

    
/* set up non connected UDP server  */
    oldContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleServer);

#ifdef TM_4_USE_SCOPE_ID
    *serverSdPtr = socket(AF_INET6, SOCK_DGRAM, IP_PROTOUDP);
#else /* ! TM_4_USE_SCOPE_ID */
    *serverSdPtr = socket(family, SOCK_DGRAM, IP_PROTOUDP);
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxGetNonConnectedUdpPair, *serverSdPtr != TM_SOCKET_ERROR);
    tfBlockingState(*serverSdPtr, TM_BLOCKING_OFF);

    tfxGetServerSockAddr(index, family, &sockAddrPtr, &sockAddrLength);

    errorCode = bind(*serverSdPtr,
                     sockAddrPtr,
                     sockAddrLength
                    );
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetNonConnectedUdpPair, errorCode == TM_ENOERROR);
    }
#ifdef TM_USE_STRONG_ESL
    errorCode = setsockopt(*serverSdPtr, SOL_SOCKET, SO_BINDTODEVICE, serverNamePtr, IFNAMSIZ);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*serverSdPtr);
        tm_assert(tfxGetNonConnectedUdpPair, errorCode == TM_ENOERROR);
    }
#endif /* TM_USE_STRONG_ESL */

/* set up UDP client  */
    tfSetCurrentContext(tvContextHandleClient);
#ifdef TM_4_USE_SCOPE_ID
    *clientSdPtr = socket(AF_INET6, SOCK_DGRAM, IP_PROTOUDP);
#else /* ! TM_4_USE_SCOPE_ID */
    *clientSdPtr = socket(family, SOCK_DGRAM, IP_PROTOUDP);
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxGetNonConnectedUdpPair, *clientSdPtr != TM_SOCKET_ERROR);
    tfBlockingState(*clientSdPtr, TM_BLOCKING_OFF);
#ifdef TM_USE_STRONG_ESL
    errorCode = setsockopt(*clientSdPtr, SOL_SOCKET, SO_BINDTODEVICE, clientNamePtr, IFNAMSIZ);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(*clientSdPtr);
        tm_assert(tfxGetNonConnectedUdpPair, errorCode == TM_ENOERROR);
    }
#endif /* TM_USE_STRONG_ESL */

/* restore the context */
    tfSetCurrentContext(oldContext);
}

/*
 * exercise the connected socket pair 
 * by exchange 2 buffer betweent the server and the client sockets using the
 * API tfSendToFrom
 *
 */
void tfxSendToFrom(int serverSd, 
                   int clientSd, 
                   int family,
                   struct sockaddr_storage *clientSockAddrPtr,
                   struct sockaddr_storage *serverSockAddrPtr,
                   int addrLen)
{
    int                     sentLen;
    int                     recvLen;
    char                    sendBuffer[TM_TEST_BUFFER_SIZE];
    char                    recvBuffer[TM_TEST_BUFFER_SIZE];
    char                    buffer[TM_TEST_BUFFER_SIZE];
    ttUserContext           oldContext;
    struct sockaddr_storage clientAddr;
    struct sockaddr_storage client2Addr;
    struct sockaddr_storage serverAddr;
    int                     fromLength;
    int                     errorCode;
    struct sockaddr_storage *clientSockAddr2Ptr;

    oldContext = tfGetCurrentContext();

/* switch to the client node */
    tfSetCurrentContext(tvContextHandleClient);

/* send a buffer to the server*/
    FillSendBuffer(sendBuffer,  TM_TEST_BUFFER_SIZE);
    sentLen = tfSendToFrom(clientSd,
                           sendBuffer,
                           TM_TEST_BUFFER_SIZE,
                           0,
                           (struct sockaddr TM_FAR*) serverSockAddrPtr,
                           addrLen,
                           (struct sockaddr TM_FAR*) clientSockAddrPtr );

    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxSendToFrom, sentLen != TM_SOCKET_ERROR);
    tm_assert(tfxSendToFrom, sentLen == TM_TEST_BUFFER_SIZE);
    
/* switch to the server node */
/* recvfrom at the server. */
    tfSetCurrentContext(tvContextHandleServer);
    fromLength = sizeof(struct sockaddr_in);

/* use recvfrom for non connected server socket */
    recvLen = tfxLoopRecvFrom(
        serverSd,
        buffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)&clientAddr,
        &fromLength);

    if (recvLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }

    tm_assert(tfxSendToFrom, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxSendToFrom, recvLen == TM_TEST_BUFFER_SIZE);

/* server reply to the client */
    sentLen = tfSendToFrom(serverSd,
                           buffer,
                           TM_TEST_BUFFER_SIZE,
                           0,
                           (const struct sockaddr*)&clientAddr,
                           fromLength,
                           (struct sockaddr TM_FAR*) serverSockAddrPtr );

    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }
    tm_assert(tfxSendToFrom, sentLen != TM_SOCKET_ERROR);
    tm_assert(tfxSendToFrom, sentLen == TM_TEST_BUFFER_SIZE);

/* switch to the client node */
/* recv at the client */
    tfSetCurrentContext(tvContextHandleClient);
    fromLength = sizeof(struct sockaddr_storage);
    tm_bzero(recvBuffer, TM_TEST_BUFFER_SIZE);
    recvLen = tfxLoopRecvFrom(
        clientSd,
        recvBuffer,
        TM_TEST_BUFFER_SIZE,
        (struct sockaddr *)&serverAddr,
        &fromLength);
    if (recvLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxSendToFrom, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxSendToFrom, recvLen == TM_TEST_BUFFER_SIZE);
    tm_assert(tfxSendToFrom, !tm_memcmp(sendBuffer, 
                                              recvBuffer, 
                                              TM_TEST_BUFFER_SIZE));
    {
/* 
 * BUG ID 815: Verify that the client source address has not been changed by
 * tfSendToFrom
 */
/* Added bind for BUG ID 815 */
        tm_bcopy(clientSockAddrPtr, &clientAddr, 
                 sizeof(struct sockaddr_storage));
/* Bind client socket to its port, and IP address 0 */
        if (family == AF_INET)
        {
            fromLength = sizeof(struct sockaddr_in);
        }
        clientAddr.ss_port = htons(10);
        errorCode = bind(clientSd, 
                         (const struct sockaddr *)&clientAddr, 
                         fromLength);
        if (errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(clientSd);
            tm_assert(tfxSendToFrom, errorCode == TM_ENOERROR);
        }
/* end of bind addition */
/* 
 * Verify that tfSendToFrom does not change client source address.
 */
/* server send to an IP address different from the client */
        tfxGetClientSockAddr(1, 
                             family, 
                             (struct sockaddr**)&clientSockAddr2Ptr, 
                             &fromLength);
/* Use a client address different from the bound one */
        sentLen = tfSendToFrom(clientSd,
                               sendBuffer,
                               TM_TEST_BUFFER_SIZE,
                               0,
                               (struct sockaddr TM_FAR*) serverSockAddrPtr,
                               addrLen,
                               (struct sockaddr TM_FAR*)clientSockAddr2Ptr );

        if(sentLen == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(clientSd);
        }
        tm_assert(tfxSendToFrom, sentLen != TM_SOCKET_ERROR);
        tm_assert(tfxSendToFrom, sentLen == TM_TEST_BUFFER_SIZE);
/* switch to the server node */
        tfSetCurrentContext(tvContextHandleServer);
        fromLength = sizeof(struct sockaddr_in);
/* Verify that data was sent successfully */

/* use recvfrom for non connected server socket */
        recvLen = tfxLoopRecvFrom(
            serverSd,
            buffer,
            TM_TEST_BUFFER_SIZE,
            (struct sockaddr *)&client2Addr,
            &fromLength);
        if (recvLen == TM_SOCKET_ERROR)
        {
            errorCode = tfGetSocketError(serverSd);
        }

        tm_assert(tfxSendToFrom, recvLen != TM_SOCKET_ERROR);
        tm_assert(tfxSendToFrom, recvLen == TM_TEST_BUFFER_SIZE);

/* switch to the client node */
        tfSetCurrentContext(tvContextHandleClient);
/* Client socket source address should still match clientAddr */
        errorCode = getsockname(clientSd, 
                                (struct sockaddr *)&client2Addr,
                                &fromLength);
        if (errorCode != TM_ENOERROR)
        {
            errorCode = tfGetSocketError(clientSd);
            tm_assert(tfxSendToFrom, errorCode == TM_ENOERROR);
        }
        tm_assert(tfxSendToFrom, !tm_memcmp(&client2Addr, 
                                            &clientAddr, 
                                            sizeof(struct sockaddr_storage)));

/* End of BUG ID 815 */
    }
}


/*
 * exercise the connected socket pair 
 * by exchange 2 buffer betweent the server and the client sockets
 *
 * serverSocketType:  TM_TEST_SOCKET_TYPE_CONNECTED for TCP
 *                    TM_TEST_SOCKET_TYPE_NON_CONNECTED for UDP
 * clientSocketType:  TM_TEST_SOCKET_TYPE_CONNECTED for TCP and connected UDP
 *                    TM_TEST_SOCKET_TYPE_NON_CONNECTED for non connected UDP
 */
void tfxExcerciseChannel(
    int serverSd, 
    int clientSd, 
    int serverSocketType,
    int clientSocketType,
    int family,
    struct sockaddr_storage *serverSockAddrPtr,
    int serverSockAddrLen,
    int useZeroCopy)
{

    tt32Bit randSize;

    tfxExcerciseChannelSize(
        serverSd, 
        clientSd, 
        serverSocketType,
        clientSocketType,
        family,
        serverSockAddrPtr,
        serverSockAddrLen,
        TM_TEST_BUFFER_SIZE,
        useZeroCopy);

    tfTimerUpdate();
    tfTimerExecute();

    randSize = tfGetRandom() % TM_TEST_BUFFER_SIZE;
    if (randSize ==0 )
    {
        randSize ++;
    }

    tfxExcerciseChannelSize(
        serverSd, 
        clientSd, 
        serverSocketType,
        clientSocketType,
        family,
        serverSockAddrPtr,
        serverSockAddrLen,
        randSize,
        useZeroCopy);
}

void tfxExcerciseChannelSize(
    int     serverSd, 
    int     clientSd, 
    int     serverSocketType,
    int     clientSocketType,
    int     family,
    struct sockaddr_storage *serverSockAddrPtr,
    int     serverSockAddrLen,
    tt32Bit dataLen,
    int     useZeroCopy)
{
    tt32Bit                 sentLen;
    tt32Bit                 sentLeni;
    tt32Bit                 dataLeni;
    tt32Bit                 recvLen;
    ttCharPtr               sendBufferPtr;
    ttCharPtr               recvBufferPtr;
    ttCharPtr               bufferPtr;
    ttUserBuffer            sendBufferHandlei;
    ttUserBuffer            sendBufferHandle;
    ttUserBuffer            bufferHandle;
    ttPacketPtr             newPacketPtr;
    char                    sendBuffer[TM_TEST_BUFFER_SIZE];
    char                    recvBuffer[TM_TEST_BUFFER_SIZE];
    char                    buffer[TM_TEST_BUFFER_SIZE];
    ttUserContext           oldContext;
    struct sockaddr_storage serverAddr;
    struct sockaddr_storage clientAddr;
    int                     fromLength;
    struct sockaddr TM_FAR* sockAddrPtr;
    int                     sockAddrLength;
    int                     i;
    int                     param;
    int                     errorCode;

    TM_UNREF_IN_ARG(family);
    oldContext = tfGetCurrentContext();

    //tfKernelPrint(TM_PRINT_TYPE_INFO, "data size = %d\n", dataLen);

/* switch to the client node */
    tfSetCurrentContext(tvContextHandleClient);

    sockAddrPtr = (struct sockaddr*)serverSockAddrPtr;
    sockAddrLength = serverSockAddrLen;
    recvBufferPtr = &recvBuffer[0];
    sendBufferHandle = (ttUserBuffer)0;
    bufferHandle = (ttUserBuffer)0;
    if (useZeroCopy != 0) /* TCP/UDP zero copy */
    {
        FillSendBuffer(sendBuffer,  dataLen);
/* Client buffer */
        sendBufferHandle = tfGetZeroCopyBuffer(dataLen, &sendBufferPtr);
        tm_assert(tfxExcerciseChannelSize, sendBufferHandle != (ttVoidPtr)0);
        tfSetCurrentContext(tvContextHandleServer);
/* Server buffer */
        bufferHandle = tfGetZeroCopyBuffer(dataLen, &bufferPtr);
        tm_assert(tfxExcerciseChannelSize, bufferHandle != (ttVoidPtr)0);
        tfSetCurrentContext(tvContextHandleClient);
    }
    else
    {
        sendBufferPtr = &sendBuffer[0];
        bufferPtr = &buffer[0];
    }

/* send a buffer to the server*/
    FillSendBuffer(sendBufferPtr, dataLen);
    if (clientSocketType == TM_TEST_SOCKET_TYPE_NON_CONNECTED)
    {
/* use sendto for non connected client socket */
        if (useZeroCopy != 0)
        {
            sentLen = tfZeroCopySendTo(clientSd,
                             sendBufferHandle,
                             dataLen,
                             0,
                             sockAddrPtr,
                             sockAddrLength
                             );
        }
        else
        {
            sentLen = sendto(clientSd,
                             sendBufferPtr,
                             dataLen,
                             0,
                             sockAddrPtr,
                             sockAddrLength
                             );
        }
    }
    else
    {
        tm_assert(tfxExcerciseChannelSize,
                  clientSocketType == TM_TEST_SOCKET_TYPE_CONNECTED);
/* use send for connected client socket */ 
        if (useZeroCopy)
        {
            sentLen = 0;
            if (useZeroCopy == 2) /* TCP */
            {
                param = 1;
                errorCode = setsockopt(clientSd, IPPROTO_TCP,
                           TCP_NODELAY, (ttConstCharPtr)&param,
                           sizeof(param));
                if (errorCode != TM_ENOERROR)
                {
                    errorCode = tfGetSocketError(clientSd);
                    tm_assert(tfxExcerciseChannelSize,
                                        errorCode == TM_ENOERROR);
                }
                dataLeni = 0;
                for (i = 0; i < 2; i++)
                {
                    if (i == 0)
                    {
                        dataLeni = dataLen/2 + 1;
                    }
                    else
                    {
                        dataLeni = dataLen - dataLeni;
                        if (dataLeni == 0)
                        {
                            break;
                        }
                    }
                    sendBufferHandlei =
                            tfDuplicateSharedBuffer(sendBufferHandle);
                    tm_assert(tfxExcerciseChannelSize,
                                    sendBufferHandlei != (ttVoidPtr)0);
                    newPacketPtr = (ttPacketPtr)sendBufferHandlei;
                    if (i != 0)
                    {
                        newPacketPtr->pktLinkDataPtr += dataLeni;
                    }
                    newPacketPtr->pktChainDataLength = (ttPktLen)dataLeni;
                    newPacketPtr->pktLinkDataLength = (ttPktLen)dataLeni;
                    sentLeni = tfZeroCopySend(clientSd,
                                   sendBufferHandlei,
                                   dataLeni,
                                   0
                                  );
                    if (sentLeni == TM_SOCKET_ERROR)
                    {
                        errorCode = tfGetSocketError(clientSd);
                    }
                    tm_assert(tfxExcerciseChannelSize,
                                    sentLeni != TM_SOCKET_ERROR);
                    tm_assert(tfxExcerciseChannelSize, sentLeni == dataLeni);
                    sentLen += sentLeni;
                }
            }
            else /* non TCP */
            {
                sentLen = tfZeroCopySend(clientSd,
                                   sendBufferHandle,
                                   dataLen,
                                   0
                                  );
            }
        }
        else
        {
            sentLen = send(clientSd,
                           sendBufferPtr,
                           dataLen,
                           0
                          );
        }
    }
    if(sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxExcerciseChannelSize, sentLen != TM_SOCKET_ERROR);
    tm_assert(tfxExcerciseChannelSize, sentLen == dataLen);
    
/* switch to the server node */
/* recvfrom at the server. */
    tfSetCurrentContext(tvContextHandleServer);
    fromLength = sizeof(struct sockaddr_in);

    if (serverSocketType == TM_TEST_SOCKET_TYPE_NON_CONNECTED)
    {
/* use recvfrom for non connected server socket */
        recvLen = tfxLoopRecvFrom(
                           serverSd,
                           bufferPtr,
                           dataLen,
                           (struct sockaddr *)&clientAddr,
                            &fromLength);
    }
    else
    {
        tm_assert(tfxExcerciseChannelSize, 
                  serverSocketType == TM_TEST_SOCKET_TYPE_CONNECTED);
/* connected server socket means TCP use recv for connected server socket */

        recvLen = tfxLoopRecv( serverSd,
                               bufferPtr,
                               dataLen
                               );
    }
    if (recvLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(serverSd);
    }

    tm_assert(tfxExcerciseChannelSize, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxExcerciseChannelSize, recvLen == dataLen);

/* server reply to the client */
    if (serverSocketType == TM_TEST_SOCKET_TYPE_NON_CONNECTED)
    {
/* use sendto for non connected server socket */
        if (useZeroCopy)
        {
            sentLen = tfZeroCopySendTo(serverSd,
                             bufferHandle,
                             dataLen,
                             0,
                             (const struct sockaddr*)&clientAddr,
                             fromLength
                           );
        }
        else
        {
            sentLen = sendto(serverSd,
                             bufferPtr,
                             dataLen,
                             0,
                             (const struct sockaddr*)&clientAddr,
                             fromLength
                           );
        }
    }
    else
    {
/* use send for connected server socket */ 
        tm_assert(tfxExcerciseChannelSize,
                  serverSocketType  == TM_TEST_SOCKET_TYPE_CONNECTED);
        if (useZeroCopy)
        {
            sentLen = tfZeroCopySend(serverSd,
                            bufferHandle,
                            dataLen,
                            0);
        }
        else
        {
            sentLen = send(serverSd,
                            bufferPtr,
                            dataLen,
                            0);
        }

    }
    if (sentLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }

    tm_assert(tfxExcerciseChannelSize, sentLen != TM_SOCKET_ERROR);
    tm_assert(tfxExcerciseChannelSize, sentLen == dataLen);

/* switch to the client node */
/* recv at the client */
    tfSetCurrentContext(tvContextHandleClient);
    fromLength = sizeof(struct sockaddr_storage);
    tm_bzero(recvBufferPtr, dataLen);
    if (clientSocketType == TM_TEST_SOCKET_TYPE_NON_CONNECTED)
    {
/* use recvfrom for non-connected client socket */
        recvLen = tfxLoopRecvFrom(
                            clientSd,
                            recvBufferPtr,
                            dataLen,
                            (struct sockaddr *)&serverAddr,
                            &fromLength);
    }
    else
    {
/* use recv for connected client socket */
        tm_assert(tfxExcerciseChannelSize, 
                  clientSocketType== TM_TEST_SOCKET_TYPE_CONNECTED);

        recvLen = tfxLoopRecv(clientSd,
                             recvBufferPtr,
                             dataLen
                             );

    }
    if (recvLen == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(clientSd);
    }
    tm_assert(tfxExcerciseChannelSize, recvLen != TM_SOCKET_ERROR);
    tm_assert(tfxExcerciseChannelSize, recvLen == dataLen);
    tm_assert(tfxExcerciseChannelSize, !tm_memcmp(sendBuffer, 
                                           recvBufferPtr,
                                           dataLen));
    if (useZeroCopy == 2)
    {
        tfFreeZeroCopyBuffer(sendBufferHandle);
    }
}


/* 
 * loop recving until the peer socket is closed
 * return bytes received 
 */
int tfxLoopRecv(int  socketDescriptor, 
                char TM_FAR * buffer, 
                int  bufferSize)
{

    int recvLength;
    int returnCode;
    int errorCode;
    int blockCount;

    recvLength = 0;
    errorCode  = TM_ENOERROR;
    blockCount = 0;

/* Loop until there is no more data to recv, or we failed */
    do
    {
        returnCode = recv(socketDescriptor,
                          buffer + recvLength,
                          bufferSize - recvLength,
                          0);
        if (returnCode > 0)
        {
/* received OK , let's keep on recv */
            blockCount = 0;
            recvLength += returnCode;
            if (recvLength == bufferSize)
            {
/* got all what we want */
                break;
            }

        }
        else if (returnCode == TM_SOCKET_ERROR)
        {
            blockCount++;
/* data tempraly not available TM_EWOULDBLOCK is OK, we can just keep on recv */
            errorCode = tfGetSocketError(socketDescriptor);
/* if not TM_EWOULDBLOCK, there is an real problem */
            tm_assert(tfxLoopRecv, errorCode == TM_EWOULDBLOCK);
        }
        else
        {
/* End of file */
            blockCount = 0;
            break;
        }

#ifdef TM_KERNEL_VISUAL_X86
        tfTimerUpdate();
        tfTimerExecute();
#endif /* TM_KERNEL_VISUAL_X86 */
        if (blockCount == 100000)
        {
            tfKernelError("tfxLoopRecv", "Waiting forever for incoming data");
        }

    }
    while (errorCode == TM_ENOERROR || errorCode == TM_EWOULDBLOCK);

    return recvLength;
}




/* 
 * loop recving until the peer socket is closed
 * return bytes received 
 */
int tfxLoopRecvFrom(int     socketDescriptor, 
                    char    TM_FAR * buffer, 
                    int     bufferSize,
                    struct  sockaddr * sockAddrPtr,
                    int     *sockAddrLengthPtr
                    )
{

    int recvLength;
    int returnCode;
    int errorCode;
    int blockCount;

    recvLength = 0;
    errorCode  = TM_ENOERROR;
    blockCount = 0;

/* Loop until there is no more data to recv, or we failed */
    do
    {
        returnCode = recvfrom(socketDescriptor,
                          buffer + recvLength,
                          bufferSize - recvLength,
                          0,
                          sockAddrPtr,
                          sockAddrLengthPtr
                          );
        if (returnCode > 0)
        {
/* received OK , let's keep on recv */
            blockCount = 0;
            recvLength += returnCode;
            if (recvLength == bufferSize)
            {
/* got all what we want , let go*/
                break;
            }

        }
        else if (returnCode == TM_SOCKET_ERROR)
        {
/* data tempraly not available TM_EWOULDBLOCK is OK, we can just keep on recv */
            errorCode = tfGetSocketError(socketDescriptor);
            blockCount++;
/* if not TM_EWOULDBLOCK, there is an real problem */
            tm_assert(tfxLoopRecvFrom, errorCode == TM_EWOULDBLOCK);
        }
        else
        {
/* End of file */
            blockCount = 0;
            break;
        }

#ifdef TM_KERNEL_VISUAL_X86
        tfTimerUpdate();
        tfTimerExecute();
#endif /* TM_KERNEL_VISUAL_X86 */

        if (blockCount == 100000)
        {
            tfKernelError("tfxLoopRecvFrom", 
                          "Waiting forever for incoming data");
        }
    }
    while (errorCode == TM_ENOERROR || errorCode == TM_EWOULDBLOCK);

    return recvLength;
}




void tfxCloseUdpConnection(int serverSd, int clientSd)
{
    ttUserContext oldContext;

    oldContext = tfGetCurrentContext();

    tfSetCurrentContext(tvContextHandleClient);
    tfClose(clientSd);
    tfTimerUpdate();
    tfTimerUpdate();
    tfTimerExecute();


    tfSetCurrentContext(tvContextHandleServer);
    tfClose(serverSd);
    tfTimerUpdate();
    tfTimerUpdate();
    tfTimerExecute();

    tfSetCurrentContext(oldContext);
}

void tfxCloseTcpConnection(int serverSd, int acceptSd, int clientSd)
{
    ttUserContext oldContext;

    oldContext = tfGetCurrentContext();

    tfSetCurrentContext(tvContextHandleClient);
    tfClose(clientSd);
    tfTimerUpdate();
    tfTimerUpdate();
    tfTimerExecute();

    tfSetCurrentContext(tvContextHandleServer);
    tfClose(serverSd);
    tfClose(acceptSd);
    tfTimerUpdate();
    tfTimerUpdate();
    tfTimerExecute();

    tfSetCurrentContext(oldContext);
}

#endif /*TM_MULTIPLE_CONTEXT*/

void tfxSocket(void)
{
    int                     sd;
    int                     sd2;
    int                     errorCode;
    char                    buffer[1];

    
/* normal case */
    sd = socket(AF_INET, SOCK_DGRAM, IP_PROTOUDP);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);

#ifdef TM_USE_TCP
    sd = socket(AF_INET, SOCK_STREAM, IP_PROTOTCP);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);
#endif /* TM_USE_TCP */

    sd = socket(AF_INET, SOCK_RAW, IP_PROTOICMP);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);

    sd = socket(AF_INET, SOCK_DGRAM, 0);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);

#ifdef TM_USE_TCP
    sd = socket(AF_INET, SOCK_STREAM, 0);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);
#endif /* TM_USE_TCP */

    sd = socket(AF_INET, SOCK_RAW, 0);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);

/* bad case */
    sd = socket(AF_INET6, SOCK_RAW, IP_PROTOICMP);
    tm_assert(tfxSocket, sd == TM_SOCKET_ERROR);
    errorCode = tfGetSocketError(sd);
    tm_assert(tfxSocket, tfGetSocketError(sd) == TM_EAFNOSUPPORT);
/* invalid protocol */
    sd = socket(AF_INET, SOCK_DGRAM, 48);
    tm_assert(tfxSocket, sd == TM_SOCKET_ERROR);
    tm_assert(tfxSocket, tfGetSocketError(sd) == TM_EPROTONOSUPPORT);
/* invalid family*/
#ifdef TM_USE_TCP
    sd = socket(14, SOCK_STREAM, IP_PROTOTCP);
    tm_assert(tfxSocket, sd == TM_SOCKET_ERROR);
    tm_assert(tfxSocket, tfGetSocketError(sd) == TM_EAFNOSUPPORT);
#endif /* TM_USE_TCP */


    
/* bad case */

    sd = socket(-1, SOCK_RAW, IP_PROTOICMP);
    tm_assert(tfxSocket, sd == TM_SOCKET_ERROR);
    tm_assert(tfxSocket, tfGetSocketError(sd) == TM_EAFNOSUPPORT);

    sd = socket(AF_INET6, SOCK_DGRAM, IP_PROTOUDP);
    tm_assert(tfxSocket, sd == TM_SOCKET_ERROR);
    tm_assert(tfxSocket, tfGetSocketError(sd) == TM_EAFNOSUPPORT);

    sd = socket(
                AF_INET,
                SOCK_DGRAM,
                IP_PROTOUDP);
    tm_assert(tfxSocket, sd != TM_SOCKET_ERROR);
    sd2 = socket(
                 AF_INET,
                 SOCK_DGRAM,
                 IP_PROTOUDP);
    tm_assert(tfxSocket, sd2 != TM_SOCKET_ERROR);

/* Kick a valid socket */
    errorCode = tfSocketKickOne(sd, 1234);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);

/* Make sure the kick worked */
    errorCode = recvfrom(sd, buffer, 1, 0, (struct sockaddr *)0, (int *)0);
    tm_assert(tfxSocket, errorCode == TM_SOCKET_ERROR);
    errorCode = tfGetSocketError(sd);
    tm_assert(tfxSocket, errorCode == 1234);

/* Kick all the sockets */
    tfSocketKickAll(4321);

/* Make sure the kick worked */
    errorCode = recvfrom(sd, buffer, 1, 0, (struct sockaddr *)0, (int *)0);
    tm_assert(tfxSocket, errorCode == TM_SOCKET_ERROR);
    errorCode = tfGetSocketError(sd);
    tm_assert(tfxSocket, errorCode == 4321);
    errorCode = recvfrom(sd2, buffer, 1, 0, (struct sockaddr *)0, (int *)0);
    tm_assert(tfxSocket, errorCode == TM_SOCKET_ERROR);
    errorCode = tfGetSocketError(sd2);
    tm_assert(tfxSocket, errorCode == 4321);

    errorCode = tfClose(sd);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);
    errorCode = tfClose(sd2);
    tm_assert(tfxSocket, errorCode == TM_ENOERROR);

/* Kick an invalid socket */
    errorCode = tfSocketKickOne(sd2, 0);
    tm_assert(tfxSocket, errorCode == TM_EBADF);

/* Kick an invalid socket */
    errorCode = tfSocketKickOne(sd2+1, 0);
    tm_assert(tfxSocket, errorCode == TM_EBADF);

/* Kick an invalid socket */
    errorCode = tfSocketKickOne(TM_SOCKET_ERROR, 0);
    tm_assert(tfxSocket, errorCode == TM_EBADF);
}


#if defined(TM_USE_FILTERING) || defined(TM_6_USE_FILTERING)
void tfxIpFilteringTests(int af, int protocol)
{
    ttUserContext   origContext;
    int             serverSd;
    int             acceptSd;
    int             clientSd;
    int             errorCode;
    int             sentLen;
    int             clientOptionName;
    int             serverOptionName;
    char            sendBuffer[TM_TEST_BUFFER_SIZE];
    char            optionVal;

    origContext = tfGetCurrentContext();

    serverSd = TM_SOCKET_ERROR;
    clientOptionName = 0;
    serverOptionName = 0;

    if (protocol == IPPROTO_TCP)
    {
        tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd,
                            &acceptSd, &clientSd, af, 1,
#ifdef TM_USE_STRONG_ESL
                            "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                            0);
    }
    else
    {
        tfxGetUdpConnection(tvInterfaceHandleClient, &acceptSd,
                            &clientSd, af, 0
#ifdef TM_USE_STRONG_ESL
                            , "Client", "Server"
#endif /* TM_USE_STRONG_ESL */
                            );
    }

/* Register the filter callbacks */
    if (af == AF_INET)
    {
        tfSetCurrentContext(tvContextHandleClient);
#ifdef TM_USE_FILTERING
        errorCode = tfUserRegisterFilter(tfxFilterCallback);
        tm_assert(tfxIpFilteringTests,
                  (errorCode == TM_ENOERROR) || (errorCode == TM_EALREADY));
        tfSetCurrentContext(tvContextHandleServer);
        errorCode = tfUserRegisterFilter(tfxFilterCallback);
        tm_assert(tfxIpFilteringTests,
                  (errorCode == TM_ENOERROR) || (errorCode == TM_EALREADY));
        clientOptionName = TM_DEV_OPTIONS_FILTER_OUTGOING;
        serverOptionName = TM_DEV_OPTIONS_FILTER_INCOMING;
#endif /* TM_USE_FILTERING */
    }
    else
    {
        tfKernelError("tfxIpFilteringTests", "Invalid address family!");
    }

/* Enable the filter callback for the client */
    tfSetCurrentContext(tvContextHandleClient);
    optionVal = 1;
    errorCode = tfInterfaceSetOptions(tvInterfaceHandleClient,
                                      clientOptionName,
                                      &optionVal,
                                      sizeof(optionVal));
    tm_assert(tfxIpFilteringTests, errorCode == TM_ENOERROR);

/* Send data and make sure it gets dropped */
    sentLen = send(clientSd,
                   sendBuffer,
                   TM_TEST_BUFFER_SIZE,
                   0);
    if (protocol == IPPROTO_TCP)
    {
/* send() will return successfully even if the data was filtered */
        tm_assert(tfxIpFilteringTests, sentLen == TM_TEST_BUFFER_SIZE);
        tfSetCurrentContext(tvContextHandleServer);
        sentLen = recv(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0);
    }
    else
    {
/* send() will return an error */
        tm_assert(tfxIpFilteringTests, sentLen == TM_SOCKET_ERROR);
        errorCode = tfGetSocketError(clientSd);
        tm_assert(tfxIpFilteringTests, errorCode == TM_EOPNOTSUPP);
        tfSetCurrentContext(tvContextHandleServer);
        sentLen = recvfrom(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0, 0, 0);
    }
    tm_assert(tfxIpFilteringTests, sentLen == TM_SOCKET_ERROR);
    errorCode = tfGetSocketError(acceptSd);
    tm_assert(tfxIpFilteringTests, errorCode == TM_EWOULDBLOCK);

/* Disable the filter callback for the client */
    tfSetCurrentContext(tvContextHandleClient);
    optionVal = 0;
    errorCode = tfInterfaceSetOptions(tvInterfaceHandleClient,
                                      clientOptionName,
                                      &optionVal,
                                      sizeof(optionVal));
    tm_assert(tfxIpFilteringTests, errorCode == TM_ENOERROR);

    if (protocol == IPPROTO_TCP)
    {
        tfxCloseTcpConnection(serverSd, acceptSd, clientSd);
        tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd,
                            &acceptSd, &clientSd, af, 1,
#ifdef TM_USE_STRONG_ESL
                            "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                            0);
    }

/* Send data and make sure it gets received */
    tfSetCurrentContext(tvContextHandleClient);
    sentLen = send(clientSd,
                   sendBuffer,
                   TM_TEST_BUFFER_SIZE,
                   0);
    tm_assert(tfxIpFilteringTests, sentLen == TM_TEST_BUFFER_SIZE);
    tfSetCurrentContext(tvContextHandleServer);
    if (protocol == IPPROTO_TCP)
    {
        sentLen = tfxLoopRecv(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE);
    }
    else
    {
        sentLen = tfxLoopRecvFrom(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0, 0);
    }
    tm_assert(tfxIpFilteringTests, sentLen == TM_TEST_BUFFER_SIZE);

    if (protocol == IPPROTO_TCP)
    {
        tfxCloseTcpConnection(serverSd, acceptSd, clientSd);
        tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd,
                            &acceptSd, &clientSd, af, 1,
#ifdef TM_USE_STRONG_ESL
                            "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                            0);
    }

/* Enable the filter callback for the server */
    tfSetCurrentContext(tvContextHandleServer);
    optionVal = 1;
    errorCode = tfInterfaceSetOptions(tvInterfaceHandleServer,
                                      serverOptionName,
                                      &optionVal,
                                      sizeof(optionVal));
    tm_assert(tfxIpFilteringTests, errorCode == TM_ENOERROR);

/* Send data and make sure it gets dropped */
    tfSetCurrentContext(tvContextHandleClient);
/* send() will return successfully even if the data was filtered */
    tm_assert(tfxIpFilteringTests, sentLen == TM_TEST_BUFFER_SIZE);
    tfSetCurrentContext(tvContextHandleServer);
    if (protocol == IPPROTO_TCP)
    {
        sentLen = recv(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0);
    }
    else
    {
        sentLen = recvfrom(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0, 0, 0);
    }
    tm_assert(tfxIpFilteringTests, sentLen == TM_SOCKET_ERROR);

/* Disable the filter callback for the server */
    optionVal = 0;
    errorCode = tfInterfaceSetOptions(tvInterfaceHandleServer,
                                      serverOptionName,
                                      &optionVal,
                                      sizeof(optionVal));
    tm_assert(tfxIpFilteringTests, errorCode == TM_ENOERROR);

    if (protocol == IPPROTO_TCP)
    {
        tfxCloseTcpConnection(serverSd, acceptSd, clientSd);
        tfxGetTcpConnection(tvInterfaceHandleClient, 0, &serverSd,
                            &acceptSd, &clientSd, af, 1,
#ifdef TM_USE_STRONG_ESL
                            "Client", "Server",
#endif /* TM_USE_STRONG_ESL */
                            0);
    }

/* Send data and make sure it gets received */
    tfSetCurrentContext(tvContextHandleClient);
    sentLen = send(clientSd,
                   sendBuffer,
                   TM_TEST_BUFFER_SIZE,
                   0);
    tm_assert(tfxIpFilteringTests, sentLen == TM_TEST_BUFFER_SIZE);
    tfSetCurrentContext(tvContextHandleServer);
    if (protocol == IPPROTO_TCP)
    {
        sentLen = tfxLoopRecv(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE);
    }
    else
    {
        sentLen = tfxLoopRecvFrom(acceptSd, sendBuffer, TM_TEST_BUFFER_SIZE, 0, 0);
    }
    tm_assert(tfxIpFilteringTests, sentLen == TM_TEST_BUFFER_SIZE);

    if (protocol == IPPROTO_TCP)
    {
        tfxCloseTcpConnection(serverSd, acceptSd, clientSd);
    }
    else
    {
        tfxCloseUdpConnection(acceptSd, clientSd);
    }

    tfSetCurrentContext(origContext);
    return;
}
#endif /* TM_USE_FILTERING || TM_6_USE_FILTERING */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tusocket.c    File Revision: 6.0.2.10 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 32304    Output checksum: 64666
 */
