/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Automated tests for Telnet server
 *
 * Filename: tutelntd.c
 * Author: Jason Schmidlapp
 * Date Created: 2/4/2003
 * $Source: test/unittest/tutelntd.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:49:51CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h>
#include <trmacro.h>
#if defined(TM_MULTIPLE_CONTEXT) && defined(TM_USE_TELNETD)
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
/* for the automated testing */
#include "tutest.h"
#include "trramfs.h"

/* constant declarations and enumerations */

/* type definitions */

/* macro definitions */
#define TM_SERVER_BUF_LEN 128
#define TM_CLIENT_BUF_LEN 128

/* local function prototypes */

/* local variable definitions */
int tlTelnetOpened;
int tlTelnetClosed;
ttUserTeldHandle tlTelnetHandle;
struct sockaddr_storage TM_FAR * tlTelnetSockAddrPtr;


char tlServerDataBuf[TM_SERVER_BUF_LEN];
int tlServerDataLength;


/* static const tables */
const char clientData[] = "Test data from the client... 1234567890\n";
const char serverData[] = "Test data from the server... ABCDEFGHIJ\n";




void tfxTestTelnetServer( int family )
{
    struct sockaddr_storage remoteAddr;
    struct sockaddr_storage localAddr;    
    ttSockAddrPtrUnion      tempSockAddr;    
    int                     errorCode;
    int                     clientSd;
    int                     match;
    int                     dataLength;
    int                     timeoutCount;
    char                    clientDataBuf[TM_CLIENT_BUF_LEN];

    tlTelnetHandle = (ttUserTeldHandle) 0;
    tlServerDataLength = 0;
    tlTelnetOpened = 0;
    tlTelnetClosed = 0;
    
/* Get address of Telnet server. */
    if (family == AF_INET)
    {
#ifdef TM_4_USE_SCOPE_ID
        tm_bcopy(&tv4MappedServerSockAddr[0],
                 &remoteAddr,
                 sizeof(tv4MappedServerSockAddr[0]));
        tm_bcopy(&tv4MappedClientSockAddr[0],
                 &localAddr,
                 sizeof(tv4MappedClientSockAddr[0]));
#else /* ! TM_4_USE_SCOPE_ID */
        tm_bcopy(&tv4ServerSockAddr[0],
                 &remoteAddr,
                 sizeof(tv4ServerSockAddr[0]));
        tm_bcopy(&tv4ClientSockAddr[0],
                 &localAddr,
                 sizeof(tv4ClientSockAddr[0]));
#endif /* ! TM_4_USE_SCOPE_ID */
    }
    else
    {
    }

    
/* SERVER: Start the Telnet server. */
    tfSetCurrentContext(tvContextHandleServer);

    errorCode = tfTeldUserStart( 0, /* No options allowed.  */
                                 1, /* maximum connections. */
                                 1, /* maximum backlog      */
                                 TM_BLOCKING_OFF );
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);

/* CLIENT: connect to the Telnet server. */
    tfSetCurrentContext(tvContextHandleClient);
#ifdef TM_4_USE_SCOPE_ID
    clientSd = socket(AF_INET6, SOCK_STREAM, IPPROTO_TCP );
#else /* ! TM_4_USE_SCOPE_ID */
    clientSd = socket( family, SOCK_STREAM, IPPROTO_TCP );
#endif /* ! TM_4_USE_SCOPE_ID */
    tm_assert(tfxTestTelnetServer, clientSd != TM_SOCKET_ERROR);

    tempSockAddr.sockNgPtr = &localAddr;
    localAddr.ss_port = 0;
    errorCode = bind( clientSd,
                      tempSockAddr.sockPtr,
                      sizeof(struct sockaddr_storage) );

    remoteAddr.ss_port = htons(23);
    tempSockAddr.sockNgPtr = &remoteAddr;
    errorCode = connect( clientSd,
                         tempSockAddr.sockPtr,
                         sizeof(struct sockaddr_storage) );
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);    
                         

/* SERVER: verify that tfTeldOpened/tfNgTeldOpened was called */
    tfSetCurrentContext(tvContextHandleServer);

    errorCode = tfTeldUserExecute();
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);

    tm_assert(tfxTestTelnetServer, tlTelnetOpened == 1);
    tm_assert(tfxTestTelnetServer, tlTelnetHandle != 0);
    tm_assert(tfxTestTelnetServer, tlTelnetSockAddrPtr != 0);

/* Verify that the address returned by the server is correct. */
    tm_assert(tfxTestTelnetServer,
              tlTelnetSockAddrPtr->ss_family == AF_INET);


    match = 0;
    if (family == AF_INET)
    {
#ifdef TM_4_USE_SCOPE_ID
        if (tm_ip_match(localAddr.addr.ipv6.sin6_addr.s6LAddr[3],
                       tlTelnetSockAddrPtr->addr.ipv4.sin_addr.s_addr))
#else /* ! TM_4_USE_SCOPE_ID */
        if ( tm_ip_match(tlTelnetSockAddrPtr->addr.ipv4.sin_addr.s_addr,
                         localAddr.addr.ipv4.sin_addr.s_addr) )
#endif /* ! TM_4_USE_SCOPE_ID */
        {
            match = 1;
        }
    }
    else
    {
    }

    tm_assert(tfxTestTelnetServer, match == 1);


/* CLIENT: send data to the Telnet server. */
    tfSetCurrentContext(tvContextHandleClient);

    dataLength = (int)tm_strlen(clientData);
    errorCode  = send( clientSd,
                       (char TM_FAR *) clientData,
                       dataLength,
                       0 );
    tm_assert(tfxTestTelnetServer, errorCode == dataLength);
    
/* SERVER: receive data from the Telnet client. (in tfTeldIncoming) */
    tfSetCurrentContext(tvContextHandleServer);

    errorCode = tfTeldUserExecute();
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);

/* Verify length, excluding \n */
    tm_assert(tfxTestTelnetServer, tlServerDataLength == dataLength);
    
/* SERVER: verify that the received data matches the expected data. */
    match = tm_memcmp(tlServerDataBuf,
                      clientData,
                      dataLength);
    tm_assert(tfxTestTelnetServer, match == 0);

/* SERVER: send data to the Telnet client. */
    dataLength = (int)tm_strlen(serverData);

    errorCode = tfTeldUserSend( tlTelnetHandle,
                                (char TM_FAR *) serverData,
                                dataLength,
                                0 );
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);
    
/* CLIENT: receive data from the Telnet server. */
    tfSetCurrentContext(tvContextHandleClient);

    errorCode = recv( clientSd,
                      clientDataBuf,
                      TM_SERVER_BUF_LEN,
                      0 );
    tm_assert(tfxTestTelnetServer, errorCode > 0);
    
/* CLIENT: verify that the received data matches the expected data. */
    dataLength = (int)tm_strlen(serverData);
    tm_assert(tfxTestTelnetServer,errorCode == dataLength);
    
    match = tm_memcmp(clientDataBuf,
                      serverData,
                      dataLength);
    tm_assert(tfxTestTelnetServer, match == 0);
    
/* CLIENT: close Telnet connection. */
    tfClose(clientSd);

/* SERVER: Verify that tfTeldClosed was called. */
    tfSetCurrentContext(tvContextHandleServer);

    errorCode = tfTeldUserExecute();
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);
    
    tm_assert(tfxTestTelnetServer, tlTelnetClosed == 1);

/* SERVER: Stop the Telnet server. */
    errorCode = tfTeldUserStop();
    tm_assert(tfxTestTelnetServer, errorCode == TM_ENOERROR);

/* Allow 2MSL time to expire so socket will be freed up, preventing an
   apparent "memory leak" in the automated test code. */
    tfSetCurrentContext(tvContextHandleClient);
    timeoutCount = tm_context(tvTcp2MSLTime) * TM_UL(1000) / tvTimerTickLength;
    while (timeoutCount-- > 0)
    {
        tfTimerUpdate();
        tfTimerExecute();
    }
}

void tfTeldOpened( ttUserTeldHandle            teldHandle,
                   struct sockaddr_in TM_FAR * sockAddrPtr )
{
    tlTelnetHandle      = teldHandle;
    tlTelnetOpened      = 1;
    tlTelnetSockAddrPtr = (struct sockaddr_storage TM_FAR *) sockAddrPtr;
    return;
}

int tfTeldIncoming( ttUserTeldHandle teldHandle,
                    char    TM_FAR * teldRecvBuf,
                    int              teldBytes,
                    int              eolFlag )
{

    TM_UNREF_IN_ARG(teldHandle);
    TM_UNREF_IN_ARG(eolFlag);
    
    tm_assert(tfTeldIncoming, teldHandle == tlTelnetHandle);

    tm_assert(tfTeldIncoming,
              tlServerDataLength+teldBytes <= TM_SERVER_BUF_LEN);
    
    tm_bcopy( teldRecvBuf,
              tlServerDataBuf + tlServerDataLength,
              teldBytes );

    tlServerDataLength += teldBytes;

    return TM_ENOERROR;
}

void tfTeldClosed(ttUserTeldHandle teldHandle)
{
    TM_UNREF_IN_ARG(teldHandle);
    tm_assert(tfTeldIncoming, teldHandle == tlTelnetHandle);

    tlTelnetClosed = 1;
    
    return;
}

/***************** End Of File *****************/
#else /* !TM_MULTIPLE_CONTEXT || !TM_USE_TELNETD */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

int tlTxTeldDummy = 0;

void tfTeldClosed(ttUserTeldHandle teldHandle)
{
    TM_UNREF_IN_ARG(teldHandle);
    return;
}

int tfTeldIncoming( ttUserTeldHandle teldHandle,
                    char    TM_FAR * teldRecvBuf,
                    int              teldBytes,
                    int              eolFlag )
{

    TM_UNREF_IN_ARG(teldHandle);
    TM_UNREF_IN_ARG(teldRecvBuf);
    TM_UNREF_IN_ARG(teldBytes);
    TM_UNREF_IN_ARG(eolFlag);

    return TM_ENOERROR;
}

void tfTeldOpened( ttUserTeldHandle            teldHandle,
                   struct sockaddr_in TM_FAR * sockAddrPtr )
{
    TM_UNREF_IN_ARG(teldHandle);
    TM_UNREF_IN_ARG(sockAddrPtr);
    return;
}
#endif /* !TM_MULTIPLE_CONTEXT || !TM_USE_TELNETD */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tutelntd.c    File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 34640    Output checksum: 51375
 */
