/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: entry function for automated testing.
 *
 *
 * Two invocating examples, tx6vc.c and tx6ucos.c are located in the 
 * example directory:
 *
 * Filename: tutest.c
 * Author: Bryan Wang
 * Date Created:
 * $Source: test/unittest/tutest.c $
 *
 * Modification History
 * $Revision: 6.0.2.17 $
 * $Date: 2011/11/29 11:05:19CST $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */


#include <trsocket.h>
#ifdef TM_MULTIPLE_CONTEXT

/* The following 4 include's are for testing the internal API */
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>

#include "tutest.h"

void tfxTestWrapAround(void);
void tfxTestWrapAround1(tt32Bit, tt32Bit, int, int, int, int);
void tfxTestStrChrNull(void);


void tfxPrintErrorCodes()
{
    printf("Treck Automated testing v1.0. Alright reserved. (c)\n\n");

    printf("Color for fail    ------------------------------------------------------ \n");
    tfKernelPrint(TM_PRINT_TYPE_FAIL, "FAIL\n");
    printf("Color for warning ------------------------------------------------------ ");
    tfKernelPrint(TM_PRINT_TYPE_WARN, "WARN\n");
    printf("Color for pass    ------------------------------------------------------ \n");
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n\n");

    tfxPrintErrorCodeTable();
}

void tfxConfigureIpAddr()
{

/* server ipv4 address 1 0x0A0A0AA1 on port 4001 */
    tfx4ConfigureIpAddr("10.10.10.161",
                        "255.255.255.0",
                        4001,
                        0,
                        (ttUser8Bit)1);
#ifndef TM_SINGLE_INTERFACE_HOME
/* server ipv4 address 2 0x0A0A0AA2 on port 4002 */
    tfx4ConfigureIpAddr("10.10.10.162",
                        "255.255.255.0",
                        4002,
                        1,
                        (ttUser8Bit)1);
/* server ipv4 address 3 0xC0A80AA3 on port 4003 */
    tfx4ConfigureIpAddr("192.168.10.163",
                        "255.255.0.0",
                        4003,
                        2,
                        (ttUser8Bit)1);
/* server ipv4 address 4 0xC0A80AA4 on port 4004 */
    tfx4ConfigureIpAddr("192.168.10.164",
                        "255.255.0.0",
                        4004,
                        3,
                        (ttUser8Bit)1);
#if (TM_MAX_IPS_PER_IF >= 5)
/* server ipv4 address 5 0xAC100AA5 on port 4005 */
    tfx4ConfigureIpAddr("172.16.10.165",
                        "255.0.0.0",
                        4005,
                        4,
                        (ttUser8Bit)1);
#endif /* (TM_MAX_IPS_PER_IF >= 5) */
#endif /* !TM_SINGLE_INTERFACE_HOME */
/* client ipv4 address 1 0x0A0A0AB1 */
    tfx4ConfigureIpAddr("10.10.10.171",
                        "255.255.255.0",
                        0,
                        0,
                        (ttUser8Bit)0);
#ifndef TM_SINGLE_INTERFACE_HOME
/* client ipv4 address 2 0x0A0A0AB2 */
    tfx4ConfigureIpAddr("10.10.10.172",
                        "255.255.255.0",
                        0,
                        1,
                        (ttUser8Bit)0);
/* client ipv4 address 3 0xC0A80AB3 */
    tfx4ConfigureIpAddr("192.168.10.173",
                        "255.255.0.0",
                        0,
                        2,
                        (ttUser8Bit)0);
/* client ipv4 address 4 0xC0A80AB4 */
    tfx4ConfigureIpAddr("192.168.10.174",
                        "255.255.0.0",
                        0,
                        3,
                        (ttUser8Bit)0);
#if (TM_MAX_IPS_PER_IF >= 5)
/* client ipv4 address 5 0xAC100AB5 */
    tfx4ConfigureIpAddr("172.16.10.175",
                        "255.0.0.0",
                        0,
                        4,
                        (ttUser8Bit)0);
#endif /* (TM_MAX_IPS_PER_IF >= 5) */
#endif /* !TM_SINGLE_INTERFACE_HOME */


}



void tfxAutoTest()
{
#ifndef TM_USE_RAM_FS
#if (defined(TM_USE_FTP) && defined(TM_USE_FTPD))
#if (   (defined(TM_USE_IPV4) && !defined(TM_4_USE_SCOPE_ID)) \
     || (defined(TM_USE_IPV6)) )
    struct sockaddr_storage TM_FAR *    sockAddrPtr;
    struct sockaddr_storage             ftpServerAddr;
    int                                 sockAddrLen;
#endif /* (TM_USE_IPV4 && !TM_4_USE_SCOPE_ID) || TM_USE_IPV6 */
#endif /* TM_USE_FTP && TM_USE_FTPD */
#endif /* !TM_USE_RAM_FS */
    int                                 errorCode;

/* Test 32-bit wraparound arithmetic on first pass */
    if (tvTestRound == 1)
    {
        tm_test(tfxTestWrapAround);
        tm_test(tfxTestStrChrNull);
    }
    if ((tvTestRound & 1) == 1)
/* Odd round, test the TM_OPTION_TIMER_MAX_EXECUTE */
    {
        errorCode = tfSetTreckOptions(TM_OPTION_TIMER_MAX_EXECUTE, 2); 
        tm_assert(tfxAutoTest, errorCode == TM_ENOERROR);
    }

#ifdef TM_BYPASS_ETHER_LL
    tm_test(tfxMalformedPacket);
#endif /* TM_BYPASS_ETHER_LL */





    tm_test(tfx4SockReuseAddr);
    tm_test(tfx4SockOpt);
    tm_test(tfx4ConfigUnconfig);
#ifdef TM_IGMP
    tm_test(tfx4UdpMCast);
    tm_test(tfx4UdpMCastTTL);
    tm_test(tfx4UdpMcastSockDeliveryTest);
#endif /* TM_IGMP */
    tm_test(tfx4NonConnUdp);
#ifndef TM_4_USE_SCOPE_ID
    tm_test(tfx4NonConnUdpBroadcast);
#endif /* !TM_4_USE_SCOPE_ID */
    tm_test(tfx4ConnUdp);
#if (   (defined(TM_USE_IPV4) && defined(TM_USE_REUSEPORT) \
     && !(defined(TM_4_USE_SCOPE_ID)) && !(defined(TM_SINGLE_INTERFACE_HOME))) )
    tm_test(tfx4UdpSockReusePort);
#endif /*    TM_USE_IPV4 && TM_USE_REUSEPORT
          && !TM_4_USE_SCOPE_ID && !TM_SINGLE_INTERFACE_HOME */
#if (defined(TM_USE_REUSEPORT) && !defined(TM_4_USE_SCOPE_ID))
#ifdef TM_IGMP
    tm_test(tfx4UdpMCastSockReusePort);
#endif /* TM_IGMP */
#endif /* TM_USE_REUSEPORT && !TM_4_USE_SCOPE_ID */
#ifdef TM_USE_TCP
    tm_test(tfx4TcpSocket);
#ifdef TM_USE_REUSEPORT
    tm_test(tfx4TcpSockReusePort);
#endif /* TM_USE_REUSEPORT */
#endif /* TM_USE_TCP */
    tm_test(tfxDefaultGw);
#if defined(TM_USE_RAW_SOCKET) && !defined(TM_4_USE_SCOPE_ID)
    tm_test(tfxRawSockSendToFrom);
#endif /* TM_USE_RAW_SOCKET and not TM_4_USE_SCOPE_ID */


#ifdef TM_6_USE_MLDV2
    tm_test_with_1_param(tfxUdpMCastSrcList, AF_INET6);
#endif /* TM_6_USE_MLDV2 */


/* common for V4 and V6 */
    tm_test(tfxInetNtoP);
    tm_test(tfxInetPtoN);
    tm_test_with_1_param(tfxIfIndexToName, tvContextHandleServer);
    tm_test_with_1_param(tfxIfIndexToName, tvContextHandleClient);
    tm_test_with_1_param(tfxIfNameToIndex, tvContextHandleServer);
    tm_test_with_1_param(tfxIfNameToIndex, tvContextHandleClient);
#ifdef TM_USE_IFNAMEINDEX
    tm_test(tfxIfNameIndex);
#endif /* TM_USE_IFNAMEINDEX */
    tm_test(tfxSocket);
    tm_test_with_1_param(tfxPing, 0); /* Do not route reply */
    tm_test_with_1_param(tfxPing, 1); /* route reply */
#ifdef TM_USE_NETSTAT
    tm_test_with_1_param(tfxNetStat, tvContextHandleServer);
    tm_test_with_1_param(tfxNetStat, tvContextHandleClient);
#endif /* TM_USE_NETSTAT */

#ifndef TM_USE_RAM_FS
#if (defined(TM_USE_FTP) && defined (TM_USE_FTPD))
#if defined(TM_USE_IPV4) && !defined(TM_4_USE_SCOPE_ID)
    tfxGetServerSockAddr(0,
                         AF_INET,
                         (struct sockaddr**)&sockAddrPtr,
                         &sockAddrLen);
    tm_memcpy(&ftpServerAddr, sockAddrPtr, sizeof(struct sockaddr_storage));
    ftpServerAddr.ss_port = htons(21);
    tm_test_with_1_param(tfxFtpClient, &ftpServerAddr);
#endif /* TM_USE_IPV4 and not TM_4_USE_SCOPE_ID */
#endif /* FTP client and server */
#endif /* !TM_USE_RAM_FS */

    tfxTestDnsResolver();

#ifndef TM_USE_RAM_FS
#if defined(TM_USE_TFTP) && defined(TM_USE_TFTPD)
/* Tests the TFTP client and server.  This test expects that some sort of
   file system is already enabled and initialized. */
    tfxTestTftp();
#endif /* TM_USE_TFTP && TM_USE_TFTPD */
#endif /* !TM_USE_RAM_FS */

#ifdef TM_USE_TELNETD
    tm_test_with_1_param(tfxTestTelnetServer, AF_INET);

#endif /* TM_USE_TELNETD */

#ifdef TM_USE_CRYPTO_TEST
#if ( defined(TM_USE_SHA256) || defined(TM_USE_SHA384)|| \
      defined(TM_USE_SHA512) || \
     (defined(TM_USE_AESXCBC) && defined(TM_USE_AES))    )
    tm_test(tfxCryptoHashTest);
#endif /* TM_USE_SHA256||TM_USE_SHA384||TM_USE_SHA512||(AESXCBC && AES) */

#endif /* TM_USE_CRYPTO_TEST */


#ifdef TM_USE_TCP
#endif /* TM_USE_TCP */


#if (TM_SNMP_VERSION > 1) && !defined(TM_4_USE_SCOPE_ID)
     tfxSnmpAgent(AF_INET);
#endif /* (TM_SNMP_VERSION > 1) && !defined(TM_4_USE_SCOPE_ID) */

#ifndef TM_SINGLE_INTERFACE_HOME
#endif /* TM_SINGLE_INTERFACE_HOME */
}

/*
 * Test 32-bit wraparound arithmetic
 */
void tfxTestWrapAround(void)
{
    tfxTestWrapAround1(TM_UL(1),          TM_UL(0x7FFFFFFF), 1, 1, 0, 0);
    tfxTestWrapAround1(TM_UL(0x80000000), TM_UL(0x7FFFFFFF), 0, 0, 1, 1);
    tfxTestWrapAround1(TM_UL(0x80000000), TM_UL(0x80000001), 1, 1, 0, 0);
    tfxTestWrapAround1(TM_UL(0xFFFFFFFF), TM_UL(0x80000001), 0, 0, 1, 1);
    tfxTestWrapAround1(TM_UL(0xFFFFFFFF), TM_UL(0),          1, 1, 0, 0);
    tfxTestWrapAround1(TM_UL(0),          TM_UL(0),          0, 1, 0, 1);
}

void tfxTestWrapAround1(tt32Bit v1, tt32Bit v2,
                        int isLt, int isLeq, int isGt, int isGeq)
{
    if (tm_32bit_lt(v1, v2))
    {
        tm_assert(tfxTestWrapAround1, isLt);
    }
    else
    {
        tm_assert(tfxTestWrapAround1, !isLt);
    }

    if (tm_32bit_leq(v1, v2))
    {
        tm_assert(tfxTestWrapAround1, isLeq);
    }
    else
    {
        tm_assert(tfxTestWrapAround1, !isLeq);
    }
    
    if (tm_32bit_gt(v1, v2))
    {
        tm_assert(tfxTestWrapAround1, isGt);
    }
    else
    {
        tm_assert(tfxTestWrapAround1, !isGt);
    }
    
    if (tm_32bit_geq(v1, v2))
    {
        tm_assert(tfxTestWrapAround1, isGeq);
    }
    else
    {
        tm_assert(tfxTestWrapAround1, !isGeq);
    }
}

void tfxTestStrChrNull(void)
{
    char *  resultStr;
    char    testStringBeginning[5];
    char    testStringMiddle[5];
    char    testStringEnd[5];

/* Find the character at the beginning */
    testStringBeginning[0] = '\0';
    testStringBeginning[1] = 't';
    testStringBeginning[2] = 'e';
    testStringBeginning[3] = 's';
    testStringBeginning[4] = 't';
    resultStr = (char *)0;
    resultStr = tfStrChr(testStringBeginning, '\0');
    tm_assert(tfxTestStrChrNull, resultStr == &(testStringBeginning[0]));

/* Find the character in the middle */
    testStringMiddle[0] = 't';
    testStringMiddle[1] = 'e';
    testStringMiddle[2] = '\0';
    testStringMiddle[3] = 's';
    testStringMiddle[4] = 't';
    resultStr = (char *)0;
    resultStr = tfStrChr(testStringMiddle, '\0');
    tm_assert(tfxTestStrChrNull, resultStr == &(testStringMiddle[2]));

/* Find the character at the end */
    testStringEnd[0] = 't';
    testStringEnd[1] = 'e';
    testStringEnd[2] = 's';
    testStringEnd[3] = 't';
    testStringEnd[4] = '\0';
    resultStr = (char *)0;
    resultStr = tfStrChr(testStringEnd, '\0');
    tm_assert(tfxTestStrChrNull, resultStr == &(testStringEnd[4]));

    return;
}


#ifdef TM_BYPASS_ETHER_LL
void tfxMalformedPacket(void)
{
    ttUserContext   origContext;
    ttPacketPtr     packetPtr;
    tt8Bit          malformedPacket1515[] = {
        0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x00, 0x0f,
        0x4d, 0x00, 0x48, 0xab, 0x15, 0x15, 0x00, 0x01,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xff, 0xe0,
        0xf5, 0x05, 0x64, 0x36, 0x00, 0x00, 0x50, 0xec,
        0x8c, 0x05, 0xf8, 0x56, 0x09, 0x00, 0x98, 0xab,
        0x1d, 0x00, 0x06, 0xd4, 0x7f, 0x62, 0x05, 0x00,
        0x01, 0x50, 0xf5, 0x47, 0x25, 0x00, 0xf0, 0x55,
        0x9d, 0x1a, 0xe8, 0xe5, 0x04, 0x00, 0xbe, 0x17,
        0x00, 0x00, 0x05, 0x09, 0xf7, 0x3c, 0x08, 0xa5,
        0x07, 0x00, 0x9e, 0x50, 0x00, 0x00, 0xcf, 0x4f,
        0xaa, 0xf5, 0x81, 0xf0, 0x06, 0x1a, 0xff, 0xff,
        0xff, 0xff, 0x01, 0x0a, 0x00, 0x00, 0x00, 0xff,
        0x00, 0x00, 0x88, 0x83, 0x89, 0x13, 0xfd, 0x7e,
        0x89, 0x13, 0x80, 0x84, 0x89, 0x13, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0xf0, 0xa6, 0x43, 0x16, 0x27, 0xf3,
        0x49, 0x13, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
        0x00, 0x00 };
    int             errorCode;
    int             i;

    origContext = tfGetCurrentContext();
    tfSetCurrentContext(tvContextHandleClient);

    for (i=0; i<1000; i++)
    {
        packetPtr = tfGetSharedBuffer(TM_LL_MAX_HDR_LEN,
                                      sizeof(malformedPacket1515),
                                      0);
        tm_assert("tfxMalformedPacket", (packetPtr != (ttPacketPtr)0));
        tm_bcopy(malformedPacket1515, packetPtr->pktLinkDataPtr, 
                 sizeof(malformedPacket1515));
        packetPtr->pktDeviceEntryPtr = tvInterfaceHandleClient;
        packetPtr->pktFlags =   TM_LL_BROADCAST_FLAG
                              | TM_ETHERNET_ADDRESS_FLAG
                              | TM_OUTPUT_DEVICE_FLAG;
        errorCode = tfDeviceSend(packetPtr);
        tm_assert("tfxMalformedPacket", (errorCode == TM_ENOERROR));
    }

    tfSetCurrentContext(origContext);
    return;
}
#endif /* TM_BYPASS_ETHER_LL */

#else /* !TM_MULTIPLE_CONTEXT */

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */
static int tltx6testDummy = 0;

#endif /* !TM_MULTIPLE_CONTEXT */


/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tutest.c      File Revision: 6.0.2.17 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 28491    Output checksum: 42659
 */
