/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: function prototypes, constant definition, macro definition
 *              for the testing code
 * Filename: tutest.c
 * Author: Bryan Wang
 * Date Created: 04/23/2002 
 * $Source: test/unittest/tutest.h $
 *
 * Modification History
 * $Revision: 6.0.2.16 $
 * $Date: 2011/12/06 12:34:53CST $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */
#include <trsocket.h>
#include <trtest.h>

#define TM_TEST_IP_ADDR_LEN TM_IP_ADDRESS_LENGTH

#define TM_TEST_PORT                htons(7000)

#define TM_TEST_SERVER_PORT_V4_1    htons(4001)
#define TM_TEST_SERVER_PORT_V4_2    htons(4002)
#define TM_TEST_SERVER_PORT_V6_1    htons(6001)
#define TM_TEST_SERVER_PORT_V6_2    htons(6002)

#define TM_TEST_CLIENT_PORT_V4_1    htons(4011)
#define TM_TEST_CLIENT_PORT_V4_2    htons(4022)
#define TM_TEST_CLIENT_PORT_V6_1    htons(6011)
#define TM_TEST_CLIENT_PORT_V6_2    htons(6022)



#ifndef TM_TEST_BUFFER_SIZE
#ifndef TM_IP_FRAGMENT
#define TM_TEST_BUFFER_SIZE     1024
#endif /* TM_IP_FRAGMENT */
#endif /* TM_TEST_BUFFER_SIZE */
#ifndef TM_TEST_BUFFER_SIZE
#ifndef TM_IP_REASSEMBLY
#define TM_TEST_BUFFER_SIZE     1024
#endif /* TM_IP_REASSEMBLY */
#endif /* TM_TEST_BUFFER_SIZE */

#ifndef TM_TEST_BUFFER_SIZE
#define TM_TEST_BUFFER_SIZE     1600
#endif /* TM_TEST_BUFFER_SIZE */
#define TM_TEST_PREFIX_LEN      64
#define TM_NUMBER_OF_INTERFACE  3
#define IOVEC_COUNT             5

#ifndef NULL
#define NULL 0
#endif

#define TM_TEST_SOCKET_TYPE_CONNECTED     0 
#define TM_TEST_SOCKET_TYPE_NON_CONNECTED 1

/* #define TM_TEST_NULL_LINK_LAYER */

#if (defined(TM_SINGLE_INTERFACE_HOME) && defined(MY_SSL_TEST_MULTIPLECONN))
#error "You cannot define both TM_SINGLE_INTERFACE_HOME and MY_SSL_TEST_MULTIPLECONN!"
#endif /* TM_SINGLE_INTERFACE_HOME && MY_SSL_TEST_MULTIPLECONN */

void tfxPrintMemUse(void);
void tfxPrintRoundStats(void);


extern int tvTestRound;
extern int tvLastRoundUsedBytes;
extern int tvCurrUsedBytes;
extern int tvUsedBytesIncrement;


#define tm_test(funcName)       \
{                               \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s\t", tvTestRound, #funcName); \
    funcName();                 \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n");  \
}


#define tm_test_with_1_param(funcName, param1)  \
{                                               \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s)\t", \
                  tvTestRound, #funcName, #param1); \
    funcName(param1);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n"); \
}

#define tm_test_with_2_param(funcName, param1, param2)      \
{                                                           \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s, %s)\t", \
                  tvTestRound, #funcName, #param1, #param2); \
    funcName(param1, param2);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n"); \
}

#define tm_test_with_2_param_str(funcName, param1, param2)  \
{                                                           \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s, %s)\t", \
                  tvTestRound, #funcName, #param1, #param2); \
    funcName(param1, param2);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n"); \
}


#define tm_test_with_2_param_int(funcName, param1, param2)  \
{                                                           \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s, %d)\t", \
                  tvTestRound, #funcName, #param1, #param2); \
    funcName(param1, param2);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n"); \
}

#define tm_test_with_3_param(funcName, param1, param2, param3)  \
{                                                               \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s, %s, %s)\t", \
                  tvTestRound, #funcName, #param1, #param2, #param3); \
    funcName(param1, param2, param3);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n");                 \
}

#define tm_test_with_4_param(funcName, param1, param2, param3, param4)  \
{                                                               \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s, %s, %s, %s)\t", \
                  tvTestRound, #funcName, #param1, #param2, #param3, #param4); \
    funcName(param1, param2, param3, param4);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n");                 \
}

#define tm_test_with_5_param(funcName, param1, param2, param3, param4, param5)  \
{                                                               \
    tfKernelPrint(TM_PRINT_TYPE_INFO, "Rnd %d %s(%s, %s, %s, %s, %s)\t", \
                  tvTestRound, #funcName, #param1, #param2, #param3, #param4, #param5); \
    funcName(param1, param2, param3, param4, param5);                           \
    tfxPrintMemUse();           \
    tfKernelPrint(TM_PRINT_TYPE_PASS, "PASS\n");                 \
}

typedef enum teScopeCat
{
    TM_ADDR_SCOPE_LINKLOCAL_UNICAST,
    TM_ADDR_SCOPE_SITELOCAL_UNICAST,
    TM_ADDR_SCOPE_GLOBAL_UNICAST
} ttScopeCat;

/* global varibales for testing */
extern char* tvRaPrefixStr;
extern ttUserInterface tvInterfaceHandle[TM_NUMBER_OF_INTERFACE];
extern ttUserInterface tvInterfaceHandleServer;
extern ttUserInterface tvOffLinkInterfaceServer;
extern ttUserInterface tvInterfaceHandleClient;
extern ttUserInterface tvOffLinkInterfaceClient;

#ifdef TM_MULTIPLE_CONTEXT
extern ttUserContext   tvContextHandleServer;
extern ttUserContext   tvContextHandleClient;
#endif /* TM_MULTIPLE_CONTEXT */



/* both server and client have two set of V4 and V6 address */

extern char    tv4ServerIpAddrStr[TM_MAX_IPS_PER_IF][INET_ADDRSTRLEN];
extern char    tv4ClientIpAddrStr[TM_MAX_IPS_PER_IF][INET_ADDRSTRLEN];
extern struct  sockaddr_storage tv4ServerSockAddr[TM_MAX_IPS_PER_IF];
extern struct  sockaddr_storage tv4ClientSockAddr[TM_MAX_IPS_PER_IF]; 

extern ttUser16Bit  tv4ServerPort[TM_MAX_IPS_PER_IF];



/* supporing functions for the testing */
void tfxPrintErrorCodes(void);
void tfxConfigureIpAddr(void);
void tfxCreateStartContexts(void);
void tfxCreateStartServerContext(void);
void tfxCreateStartClientContext(void);

void tfxAutoTest(void);
void tfxUdpEchoServer(struct sockaddr TM_FAR * sockAddrPtr);
void tfxTcpEchoServer(struct sockaddr TM_FAR * sockAddrPtr);
void tfxUdpEchoClient(struct sockaddr TM_FAR * sockAddrPtr);
void tfxTcpEchoClient(struct sockaddr TM_FAR * sockAddrPtr);

void tfxAddOnlinkInterfaces(void);
void tfxAddOfflinkInterfaces(void);
void tfxAddExtraClientInterfaces(void);

int tfIntfMCtxDrvGetPhyAddr( 
    ttUserInterface     interfaceHandle,
    char TM_FAR *       physicalAddress );

#if (defined(TM_USE_FILTERING) || defined(TM_6_USE_FILTERING))
int tfxFilterCallback(ttUserInterface interfaceHandle,
                      void TM_FAR *   iphPtr,
                      void TM_FAR *   ulpPtr,
                      int             totalLength,
                      int             direction);
#endif /* TM_USE_FILTERING || TM_6_USE_FILTERING */

void tfx6ConfigureIpAddr
       (char * serverIpAddrStr1,
        char * serverIpAddrStr2,
        char * serverOffLinkAddrStr,
        char * clientIpAddrStr1,
        char * clientIpAddrStr2,
        char * clientOffLinkAddrStr,
        ttUser16Bit serverPort1,
        ttUser16Bit serverPort2,
        ttUser16Bit serverOffLinkPort
       );
void tfx4ConfigureIpAddr(char * ipaddrStr,
                         char * netmaskStr,
                         ttUser16Bit port,
                         unsigned char mhomeIndex,
                         ttUser8Bit isServer);
void tfxSetupMCastInterface(void);

int tfxGetLocalAddressWithScope
        (int        interfaceIndex,
         ttScopeCat scope, 
         struct sockaddr_storage TM_FAR * ipAddrPtr);

int tfxVerifyAddress(struct sockaddr_storage TM_FAR * serverIpAddrPtr);

int  tfxGetSrcAddrIndex(void);
void tfxGetTcpConnection(ttUserInterface clientInterfaceHandle,
                         int   index,
                         int * serverSdPtr, 
                         int * acceptSdPtr, 
                         int * clientSdPtr,
                         int   family,
                         int   enableTimeStamps,
#ifdef TM_USE_STRONG_ESL
                         ttCharPtr clientNamePtr,
                         ttCharPtr serverNamePtr,
#endif /* TM_USE_STRONG_ESL */
                         struct sockaddr TM_FAR* clientSockAddrPtr
                         );
void tfxGetUdpConnection(ttUserInterface clientInterfaceHandle,
                         int * serverSdPtr, 
                         int *acceptSdPtr,
                         int family,
                         int index
#ifdef TM_USE_STRONG_ESL
                       , ttCharPtr clientNamePtr
                       , ttCharPtr serverNamePtr
#endif /* TM_USE_STRONG_ESL */
                         );
void tfxGetNonConnectedUdpPair(int *serverSdPtr, 
                               int *clientSdPtr,
                               int family,
                               int index
#ifdef TM_USE_STRONG_ESL
                             , ttCharPtr clientNamePtr
                             , ttCharPtr serverNamePtr
#endif /* TM_USE_STRONG_ESL */
                               );
void tfxSendToFrom(int serverSd, 
                   int clientSd, 
                   int family,
                   struct sockaddr_storage *clientSockAddrPtr,
                   struct sockaddr_storage *serverSockAddrPtr,
                   int addrLen);
void tfxExcerciseChannel(
        int serverSd, 
        int clientSd, 
        int serverSocketType,
        int clientSocketType,
        int family,
        struct sockaddr_storage *serverSockAddrPtr,
        int serverSockAddrLen,
        int useZeroCopy); 
void tfxExcerciseChannelSize(
        int             serverSd, 
        int             clientSd, 
        int             serverSocketType,
        int             clientSocketType,
        int             family,
        struct sockaddr_storage *serverSockAddrPtr,
        int             serverSockAddrLen,
        tt32Bit         dataLen,
        int             useZeroCopy);
void tfxCloseUdpConnection(int serverSd, int clientSd);
void tfxCloseTcpConnection(int serverSd, int acceptSd, int clientSd);
void tfxNotJoinReservedMulticastAddr(char *ipAddrStr);
void tfxNotSendtoReservedMulticastAddr(char *ipAddrStr);

/* calls the functions below*/
void tfxAddressing(void);
/* individule testing entries -- addr */
void tfxAddrList(void);
void tfxSrcAddrManualSelection(void);
void tfxSrcAddrAutoSelection(void);
void tfxChkLinkLocalAddr(void);
void tfxIdAddr(void);
void tfxPref2Depr(void);
void tfxUseDeprAddrInExistingComm(void);
void tfxNotUseDeprAddrInNewComm(void);
void tfxUseDeprAddrInNewComm(void);
void tfxAcceptDataToDepr(void);
void tfxNotUseInvalidAddrForSrc(void);
void tfxNotUseInvalidAddrForDst(void);
void tfxNotAssignUnspec(void);
void tfxSendToUnSpec(void);
void tfxNotUseLoopbackAsSrc(void);
void tfxNotUseIpOnlyAddr(void);
void tfxNotUseMulticastAddrAsSrc(void);
void tfxReservedMulticastAddr(void);


/* individule testing entries -- api */
void tfxSockFuncCompatible(void);

void tfxSockOpt(void);
void tfxInetPtoN(void);
void tfxInetNtoP(void);
void tfxIn6Addr(void);
void tfxSockaddrIn6(void);
void tfxAddrTestMacro(void);

void tfxSockOptHops(int sd, int option);
void tfxSockOptMulticastIf(int sd);
void tfxSockOptJoinGroup(int sd);
void tfxSockOptLeaveGroup(int sd);
void tfxMulticast(char *ipAddrStr);

void tfx4SockOpt(void);
void tfx4SockReuseAddr(void);
#ifdef TM_IGMP
void tfx4UdpMCast(void);
void tfx4UdpMCastTTL(void);
void tfx4UdpMcastSockDeliveryTest(void);
#endif /* TM_IGMP */
void tfx4NonConnUdp(void);
#ifndef TM_4_USE_SCOPE_ID
void tfx4NonConnUdpBroadcast(void);
#endif /* !TM_4_USE_SCOPE_ID */
void tfx4ConnUdp(void);
#if (   (defined(TM_USE_IPV4) && defined(TM_USE_REUSEPORT) \
     && !(defined(TM_4_USE_SCOPE_ID)) && !(defined(TM_SINGLE_INTERFACE_HOME))) )
void tfx4UdpSockReusePort(void);
#endif /*    TM_USE_IPV4 && TM_USE_REUSEPORT
          && !TM_4_USE_SCOPE_ID && !TM_SINGLE_INTERFACE_HOME */
#if (defined(TM_USE_REUSEPORT) && !defined(TM_4_USE_SCOPE_ID))
#ifdef TM_IGMP
void tfx4UdpMCastSockReusePort(void);
#endif /* TM_IGMP */
#endif /* TM_USE_REUSEPORT && !TM_4_USE_SCOPE_ID */
#ifdef TM_USE_TCP
void tfx4TcpSocket(void);
#if (defined(TM_USE_REUSEPORT) && !defined(fTM_4_USE_SCOPE_ID))
void tfx4TcpSockReusePort(void);
#endif /* TM_USE_REUSEPORT && !TM_4_USE_SCOPE_ID */
#endif /* TM_USE_TCP */
void tfxDefaultGw(void);
#ifdef TM_USE_RAW_SOCKET
void tfxRawSockSendToFrom(void);
#endif /* TM_USE_RAW_SOCKET */




void tfxDeprecateAddr(struct sockaddr_storage TM_FAR * ipAddrPtr
#ifdef TM_MULTIPLE_CONTEXT
                      ,
                      ttUserContext context
#endif
                      );
void tfxInvalidateAddr(struct sockaddr_storage TM_FAR * ipAddrPtr
#ifdef TM_MULTIPLE_CONTEXT
                       ,
                      ttUserContext context
#endif
                      );

int tfxWaitConfigueInterface(
            ttUserInterface interfaceHandle,
            int protocolFamily);

int tfxWaitConfigureIndex(
    ttUserInterface interfaceHandle,
    int             protocolFamily,
    unsigned int    mHomeIndex,
    tt8Bit          dualContext);


void tfxGetClientSockAddr(
    int                         index, /* choose a address slot */
    int                         family,
    struct sockaddr TM_FAR **   sockAddrPtrPtr,
    int TM_FAR *                sockAddrLengthPtr);

void tfxGetServerSockAddr( 
    int                         index,/* choose a address slot */
    int                         family,
    struct sockaddr TM_FAR **   sockAddrPtrPtr,
    int TM_FAR *                sockAddrLengthPtr);

int tfxLoopRecv(int  socketDescriptor, 
                char TM_FAR * buffer, 
                int  bufferSize);
int tfxLoopRecvFrom(int     socketDescriptor, 
                    char    TM_FAR * buffer, 
                    int     bufferSize,
                    struct  sockaddr * sockAddrPtr,
                    int     *sockAddrLengthPtr
                    );
void FillSendBuffer(char * buffer, int size);

void tfxFtpServer(void);
void tfxFtpClient(struct sockaddr_storage TM_FAR * serverSockAddrPtr);
void tfxPrint(int msgType, char *msgStr);


#ifdef TM_MULTIPLE_CONTEXT
void tfxArp(ttUserContext context);
ttUserInterface tfxGetInterfaceByContext(ttUserContext context);
void tfSwitchContext(void);
void tfxIfIndexToName(ttUserContext context);
void tfxIfNameToIndex(ttUserContext context);
void tfxIfNameIndex(void);
void tfxSetReachable(ttUserContext context);
void tfxSocket(void);
#if defined(TM_USE_FILTERING) || defined(TM_6_USE_FILTERING)
void tfxIpFilteringTests(int af, int protocol);
#endif /* TM_USE_FILTERING || TM_6_USE_FILTERING */
void tfxPing(int route);
int tfxSendRa(
    ttUserInterface     interfaceHandle, 
    tt8Bit              onLinkFlag,
    char *              prefixStr);
void tfxRa(tt8Bit onLinkFlag, char* prefixStr);
#ifdef TM_USE_NETSTAT

void tfxNetStat(ttUserContext context);
int tfxNtRteEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2);

int tfxNtArpEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2);

int tfxNtUdpEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2);

int tfxNtTcpEntryCBPrintf(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2);




#endif /* TM_USE_NETSTAT */

void tfxNud(void);
void tfxPrintErrorCodeTable(void);
void tfxCleanUp(void);
void tfxInterfaceSetOptions(void);
void tfx6ConfigUnconfig(void);
void tfx6ConfigDadUnconfig(void);
void tfx4ConfigUnconfig(void);
void tfx4ConfigBootUnconfig(void);


void tfxTestDnsResolver(void);
void tfxTestTftp(void);
void tfxTestTelnetServer( int family );





void tfxPppAdInterfaces(void);
void tfxPppAutoTest(void);

#ifdef TM_USE_PPPOE_CLIENT_SERVER
void tfxPppoeAdInterfaces(void);
void tfxPppoeAutoTest(void);
#endif /* TM_USE_PPPOE_CLIENT_SERVER */

#if (TM_SNMP_VERSION > 1) && !defined(TM_4_USE_SCOPE_ID)
void tfxSnmpAgent(int);
#endif /* (TM_SNMP_VERSION > 1) && !defined(TM_4_USE_SCOPE_ID) */





ttSockAddrStoragePtr tfxInetAddr(
    ttConstCharPtr          addrStr,
    ttSockAddrStoragePtr    addrPtr );

void tfxMalformedPacket(void);

#ifdef TM_USE_FTP
#define TM_FTP_AUTOTEST_DIR "ftp_autotest_dir"
#endif /* TM_USE_FTP */

#endif /* TM_MULTIPLE_CONTEXT */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:54      For: Xilinx-Samsung
 * File Name: tutest.h      File Revision: 6.0.2.16 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 40415    Output checksum: 49867
 */
