/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2003 - 2005 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file menus.c
*
* Implements menu utility driver.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver Who  Date     Changes
* --- ---- -------- -----------------------------------------------
*  1  drg  03/32/08 First release
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <unistd.h>
#include <stdlib.h>
#include "menus.h"
#include "util_userio.h"

#include "trsocket.h"

/************************** Constant Definitions *****************************/
#define EXIT_VALUE    99

#define CMD_LINE_MAX_LEN    16

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

/************************** Variable Definitions *****************************/

char CmdLine[CMD_LINE_MAX_LEN];

/*****************************************************************************/
/**
*
* Display a banner.
*
* @param    DriverName - A pointer to the name of the diver used.
*
* @return   None.
*
* @note     None.
*
 *****************************************************************************/
void DisplayBanner(char * DriverName)
{
    /*
     * Print banner.
     */
    xil_printf("\n\n\r%s DRIVER INTEGRATION TEST\n\r", DriverName);
#ifdef TM_TRECK_NO_KERNEL
    xil_printf("Treck stack running standalone w/o OS\n\r");
#else /* TM_TRECK_NO_KERNEL */
    xil_printf("Treck stack running on Xilkernel\n\r");
#endif /* TM_TRECK_NO_KERNEL */
    xil_printf("Created on %s\n\n\r", __DATE__);
    xil_printf("================================\n\r");
}

/*****************************************************************************/
/**
*
* Display a MenuScreen structure and prompt the user for input.
*
* @param    Screen - A pointer to a MenuScreen structure to put on the screen.
*
* @return   None.
*
* @note     None.
*
******************************************************************************/
void DisplayMenu(MenuScreen* Screen)
{
    char* Token;
    char* tmp;
    unsigned Index;
    unsigned   TestLoops;

    while(1)
    {
        /*
         * Print the menu title
         */
        xil_printf("\n\r%s Menu\n\r", Screen->Title);
        xil_printf("================================\n\r");

        /*
         * Print each menu item, plus an idex for the user to select.
         */
        for (Index = 0; Index < Screen->NumMenuItems; Index++)
        {
            if(MENUTYPE_CALL_VALUE == Screen->MenuItems[Index].MenuType){
                xil_printf("%2-d - %s (%d {New Value})\n\r", Index, Screen->MenuItems[Index].MenuString, Index);
            }
            else{
                xil_printf("%2-d - %s\n\r", Index, Screen->MenuItems[Index].MenuString);
            }
        }
        /*
         * Put up an exit item.
         */
        xil_printf("%d - Exit %s Menu\n\r", EXIT_VALUE, Screen->Title);
        /*
         * Get the selection from the user. The existing CT calls are used.
         */
        while(!GetUserInput("Enter selection: ", CmdLine, sizeof(CmdLine)-1))
        {
#ifdef TM_TRECK_NO_KERNEL
/* Without a receive or timer thread, we need to check for incoming packets and
 * expiring timers here.
 */
            PollIO();
#endif /* TM_TRECK_NO_KERNEL */

        }

        Index = atoi(CmdLine);

        if(EXIT_VALUE == Index)
        {
            return;
        }

        if (Index > Screen->NumMenuItems)
        {
            xil_printf("Invalid selection\n\r");
            continue;
        }


        /*
         * Depending on the type of the selected menu item, do the right thing.
         */
        switch (Screen->MenuItems[Index].MenuType){

            case MENUTYPE_SUB_MENU:
                if (XNULL == Screen->MenuItems[Index].Menu)
                {
                    xil_printf("Invalid menu data\n\r");
                    continue;
                }
                DisplayMenu((MenuScreen *)Screen->MenuItems[Index].Menu);
                break;
            case MENUTYPE_CALL_TEST:
                if (XNULL == Screen->MenuItems[Index].TestFunc)
                {
                    xil_printf("Invalid function call data\n\r");
                    continue;
                }
                /*
                 * Parse input line.
                 */
                Token = strtok_r(CmdLine, " ", &tmp);
                TestLoops = atoi(tmp);

                if (0 == TestLoops)
                {
                    TestLoops = 1;
                }

                xil_printf("Running %s\n\r", Screen->MenuItems[Index].MenuString);
                /*
                 * Call the test
                 */
                (int)(*Screen->MenuItems[Index].TestFunc)(Index, TestLoops);
                /*
                 * Count the errors
                 */
                break;
            case MENUTYPE_CALL_FUNC:
                if (XNULL == Screen->MenuItems[Index].TestFunc)
                {
                    xil_printf("Invalid function call data\n\r");
                    continue;
                }
                xil_printf("Calling %s\n\r________________________________\n\r", Screen->MenuItems[Index].MenuString);
                /*
                 * Call the function, and don't worry about the value.
                 */
                (int)(*Screen->MenuItems[Index].TestFunc)(Index, 0);
                break;
            case MENUTYPE_CALL_VALUE:
                if (XNULL == Screen->MenuItems[Index].TestFunc)
                {
                    xil_printf("Invalid function call data\n\r");
                    continue;
                }

                /*
                 * Parse input line.
                 */
                Token = strtok_r(CmdLine, " ", &tmp);
                TestLoops = atoi(tmp);

                /*
                 * Call the function, and pass in a value.
                 */
                xil_printf("Calling %s\n\r", Screen->MenuItems[Index].MenuString);
                if(0 != (int)(*Screen->MenuItems[Index].TestFunc)(Index, TestLoops)){
                    xil_printf("ERROR: Failed in %s\n\r",Screen->MenuItems[Index].MenuString);
                }
                break;
            default:
                xil_printf("Invalid menu type\n\r");
                break;
        }

    }
}
