/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2008 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file menu.h
*
* Code test menu utility driver.
*
* This package is intented to help designers test their Xilinx device drivers
* in unit and integration testing environments. The macros and functions
* provide the following services:
*
* - Menu display and input
*
* The main benefit of this package is to simplify test code and make it easier
* for the tester to get more coverage in their integration and unit testing
* without having to replicate common testing constructs.
*
* <b>Integrating Source code</b>
*
* This package consists of this header file and an implementation file. The
* implementation can be operating system specific. When including this package
* in your test, use this file and one of the implementation files such as
* vxworks.c.
*
* There are GNU specific "C" extensions used in this package and as a result
* make it non-ANSI complient. The tester must compile this package and their
* other test code with the GNU compiler suite.
*
* The CT package requires that there be available standard header files
* stdio.h, string.h, and stdarg.h.
*
* <b>Message Logging</b>
*
* This package uses the printf() library for message logging.
*
* Ideally, the tester's environment should include some sort of UART. By
* default CT utilizes a preconfigured console UART.
*
* If your system does not contain a UART, then another method of providing
* results includes printf'ing messages to memory. To enable this method,
* enable the definition of IO_USE_BUFFER in this file. To further tailor this
* method of logging, change IO_BUF_SIZE and IO_BUF_CUSHION. All output will
* be written to the TextBuf array. Use your debugger or your own test
* utilities to examine this buffer for generated output.
*
*
* <b>Limitations</b>
*
*  - This package must be compiled under the GNU compiler suite.
*  - Some sort of BSP is required to support implementation of string.h,
*    stdio.h, and stdarg.h at a minimum.
*  - Testing asserts requires that NDEBUG not be defined in your driver under
*    test code.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver Who  Date     Changes
* --- ---- -------- -----------------------------------------------
*  1  drg  03/32/08 First release
* </pre>
*
******************************************************************************/

#ifndef MENUS_H
#define MENUS_H

#ifdef __cplusplus

extern "C" {

#endif
/***************************** Include Files *********************************/

#include "xbasic_types.h"  /* Needed for assert testing */

/************************** Constant Definitions *****************************/
#define MENUTYPE_SUB_MENU       0
#define MENUTYPE_CALL_FUNC      1
#define MENUTYPE_CALL_TEST      2
#define MENUTYPE_CALL_VALUE     3


/**************************** Type Definitions *******************************/
typedef struct
{
    unsigned MenuType;
    char *MenuString;
    void *Menu;
    int (*TestFunc)(unsigned MenuIndex, unsigned Value);
}MenuItem;

typedef struct
{
    char *Title;
    MenuItem *MenuItems;
    unsigned NumMenuItems;
} MenuScreen;

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

void DisplayBanner(char * DriverName);

void DisplayMenu(MenuScreen * Screen);

void PollIO (void);

#ifdef __cplusplus

}

#endif

#endif  /* MENUS_H */

