/* $Id: */
/********************************************************************
 * Copyright(C) 2008 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times. (c) Copyright 2008 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/*****************************************************************************
*
* app.c - Executes the various applicaiton programs
*
* Running under the standalone OS requires all TCP/IP IO to be polled. A
* remote machine interacting with the target will not know the difference
* however.
*
* The applications defined here do the following:
*
*   - TCP Tx client. This app sends as much data as possible to a
*     compatible server running on a remote machine.
*
*   - TCP Rx server. This is the server version of the Tx client app. It
*     requies a compatible client running on a remote machine.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1     rmm  03/17/06 First release
* 2     rmm  06/02/06 Added Rx app, moved stack startup code outside this
*                     module
* 3     drg  04/08/08 Changed apps to use new menu function calls
* 4     acb  06/08/09 Added UDP Tx client
* 5     pdmc 03/10/11 Modified for AXI bus
* 6     pdmc 04/11/11 Fixed cache coherency issues when using
*                     Treck driver buffers
*
*****************************************************************************/

/***************************** Include Files ********************************/
#include "trsocket.h"
#ifdef TM_SNMP_VERSION
#include "trapi.h"
#endif /* TM_SNMP_VERSION */

#ifdef TM_TRECK_NO_KERNEL

#include "util_time.h"
#include "util_userio.h"
#include "defaults.h"
#include "debug.h"
#include "app.h"

#ifdef TM_USE_DCACHE_MANAGEMENT
#include "cache.h"
#endif /* TM_USE_DCACHE_MANAGEMENT */

#include "marvell_88e1111.h"
#ifdef TM_XILINX_USE_AXI4
#include "xtr_axiethernet.h"
#include "xtr_axidma.h"
#else /* TM_XILINX_USE_AXI4 */
#include "xtr_lltemac.h"
#include "xtr_lldma.h"
#endif /* TM_XILINX_USE_AXI4 */

#ifdef XPAR_XUARTNS550_NUM_INSTANCES
#include "xuartns550_l.h"
#endif /* XPAR_XUARTNS550_NUM_INSTANCES */
#ifdef XPAR_XUARTLITE_NUM_INSTANCES
#include "xuartlite.h"
#endif /* XPAR_XUARTLITE_NUM_INSTANCES */
#include "xparameters.h"

#include <stdlib.h>
#include <stdio.h>

/**************************** Type Definitions ******************************/


/***************** Macros (Inline Functions) Definitions ********************/

/* The ratio of the socket's send buffer to the ChunkSize. Lowering this value
 * causes larger chunks to be sent in each call to a socket send function
 * The actual size of the chunk may vary as it is rounded to the nearest TCP
 * MSS.
 */
#define CHUNK_SNDBUF_RATIO  4

#define BYTES_PER_MB    (1024 * 1024)
#define BYTES_PER_GB    (1024 * 1024 * 1024)
#define BITS_PER_BYTE   8
#define MS_PER_SECOND   1000

#define BYTES_PER_MB_MBPS_CALC    (1000 * 1000)

/* Define to enable debugging code in this module */
#undef DEBUG

#define APP_TX_ADV_SOCK_CONNECTED   1
#define APP_TX_ADV_SOCK_CLOSING     2
#define APP_TX_ADV_SOCK_CLOSED      4

typedef unsigned char   BYTE;

/************************** Variable Definitions ****************************/
#ifdef TM_USE_HTTPD
ttHttpdUserSrvHandle httpSrvHandle;
#endif /* TM_USE_HTTPD */

static int httpServerEn;
static int ftpServerEn;
static int telnetServerEn;
#ifdef TM_SNMP_VERSION
static int snmpServerEn;
#endif /* TM_SNMP_VERSION */

/* vars that keep track of our time in tests */
XTime StartTime;
unsigned TimeLimitMs;
int TimeExpired;
int TimingTest;

/* Locals */
volatile static int AppError;
volatile static int AppTxAdvSockStatus;
volatile static int AppRxSimpleConnected;
volatile static int AppRxSimpleError;
static ttUserInterface interfaceHandle;
static char AppBuf[APP_BUF_SIZE]
#ifdef __GNUC__
        __attribute__((aligned(32)))
#endif
        ;
static int ChunkSize;/* How many bytes we send in each socket call */
static int SndBufSize;
static int TcpMss;
static int TestFileSizeMB = DEFAULT_TEST_FILE_SIZE_MB;
static char Ipv4Addr[IP_ADDR_LEN] = DEFAULT_IP_ADDR;
static char Ipv4NetMask[IP_ADDR_LEN] = DEFAULT_NETMASK;
static char HostIp[IP_ADDR_LEN] = DEFAULT_SERVER;           /* Host IP address */
static int ServerTcpPort = DEFAULT_TCP_PORT;
static int bytesReceived;
static int gigaByteCount;

static XTime startReceive;
static XTime endReceive;
static XTime lastGigaByteReceived;


unsigned sentMBytes;
unsigned sentBytes;
int IfaceMtu = DEFAULT_MTU;

/************************** Function Prototypes *****************************/
#ifdef TM_USE_HTTPD
extern void tfxHttpdEventHandler
        (ttHttpdUserConHandle conHandle, ttHttpdUserEvent event);
#endif /* TM_USE_HTTPD */
/* Client related functions */
static int  AppTxTcpScatSendFree(char *BufPtr);
static void AppTxTcpAdvCB (int Sock, int Flag);
/* Server related functions */
static void AppRxTcpSimpleCB(int Sock, int Flag);

/* Common */
extern TemacDevData XpsTemac;    /* The Temac instance */

/* Debug */
#ifdef DEBUG
static void DebugSocket(int Sock);
#endif

/******************************************************************************
 * Setup the network and open an Interface
 *****************************************************************************/
int AppInitTreck()
{
    int errorCode;

/* Set global options and start the Treck stack */
    IfaceMtu = DEFAULT_MTU;
/* The first Treck API called should be tfInitTreck() */
    tfInitTreck();
    tfInitTreckOptions(TM_OPTION_TICK_LENGTH, TICK_INTERVAL_MS);
    errorCode = tfStartTreck();
    if (errorCode != TM_ENOERROR)
    {
        DEBUG_LOG(DEBUG_ERROR,"tfStartTreck error %d\r\n", errorCode);
        return(errorCode);
    }

#ifdef TM_USE_TIMER_CB
/* Enable Treck's timer callback mechanism */
    errorCode = tfRegisterTimerCB(TimerCBFunc, (ttUserGenericUnion)0);
    if (errorCode != TM_ENOERROR)
    {
        DEBUG_LOG(DEBUG_ERROR,"TimerInit: Failed to register Trecks' timer callback\r\n");
        return (errorCode);
    }
#endif /* TM_USE_TIMER_CB */

/* Setup the timer counter to signal the Treck stack */
    TreckTimerInit();

    return TM_ENOERROR;
}

/******************************************************************************
 * Create 1 of three types of interfaces: 1) Scatter gather with enhanced
 * scat-send capability; 2) Scatter gather; 3) FIFO direct.
 *
 * Parameters:
 * None
 *
 * Returns:
 *   0 on success; -1 on failure
 *****************************************************************************/
int AppAddInterface()
{
    ttUserLinkLayer LinkHandle;
    int errorCode;

    LinkHandle = tfUseEthernet();

#ifdef XLNX_USE_DRV_BUF_MEM
    interfaceHandle = tfAddInterface("TEMAC.0", LinkHandle, AxiDma_ScatOpen,
                            AxiDma_Close, NULL, AxiDma_Recv, AxiDma_BufFree, AxiDma_Ioctl,
                            AxiTemac_GetDeviceAddr, &errorCode);


#else /* XLNX_USE_DRV_BUF_MEM */
    interfaceHandle = tfAddInterface("TEMAC.0", LinkHandle, AxiDma_ScatOpen,
                            AxiDma_Close, NULL, AxiDma_Recv, NULL, AxiDma_Ioctl,
                            AxiTemac_GetDeviceAddr, &errorCode);
#endif /* XLNX_USE_DRV_BUF_MEM */

    if (interfaceHandle == (ttUserInterface)0 )
    {
        xil_printf("AppAddInterface: error adding interface (%s)\r\n",
                            tfStrError(errorCode));
        errorCode = -1;
        goto APP_ADD_INTF_END;
    }

/* The One Scat Send interface must be used to support checksum offload */
    tfUseInterfaceOneScatSend(interfaceHandle, AxiDma_ScatSend);

/* Store the Temac information in the Treck User Interface Structure */
    tfDeviceStorePointer(interfaceHandle, &XpsTemac);

#ifndef XLNX_USE_DRV_BUF_MEM
/* Setup the cache functions */
    errorCode = tfUseInterfaceCacheInvalFunc(interfaceHandle,
                                             tfDevCacheInvalFuncCb,
                                             (ttUserGenericUnion)0);
    if (errorCode != TM_ENOERROR)
    {
        DEBUG_LOG(DEBUG_ERROR,"tfUseInterfaceCacheInvalFunc error %d\r\n", errorCode);
        return(errorCode);
    }
    errorCode = tfUseInterfaceCacheFlushFunc(interfaceHandle,
                                             tfDevCacheFlushFuncCb,
                                             (ttUserGenericUnion)0);
    if (errorCode != TM_ENOERROR)
    {
        DEBUG_LOG(DEBUG_ERROR,"tfUseInterfaceCacheFlushFunc error %d\r\n", errorCode);
        return(errorCode);
    }
#endif /* !XLNX_USE_DRV_BUF_MEM */

/* Wait for device to settle after it was opened */
#ifdef __PPC__
    sleep(INIT_DELAY_MSEC/1000);
#else
    usleep(INIT_DELAY_MSEC);
#endif
APP_ADD_INTF_END:
    return errorCode;
}


int AppOpenInterface()
{
    int errorCode;

/* Set the Default MTU for the interface. */
    tfSetIfMtu(interfaceHandle, IfaceMtu);


/* Open the interface so the appliactions can use it */
    errorCode = tfOpenInterface(interfaceHandle,
                         inet_addr(Ipv4Addr),
                         inet_addr(Ipv4NetMask),
                         TM_DEV_SCATTER_SEND_ENB,
                         MAX_FRAG_BUFS);
    if (errorCode != TM_DEV_OKAY)
    {
        xil_printf("AppOpenInterface: Failed opening interface (%s)\r\n",
                                tfStrError(errorCode));
        errorCode = -1;
        goto APP_OPEN_INTF_END;
    }

/* Add a default gateway */
    //errorCode = tfAddDefaultGateway(interfaceHandle, inet_addr(DEFAULT_GATEWAY));
    // STRONG_ESL
    errorCode = tfAddDefaultGateway(interfaceHandle, inet_addr(DEFAULT_GATEWAY), 1);
    if (errorCode != TM_DEV_OKAY)
    {
        xil_printf("AppOpenInterface: Failed adding default gateway (%s)\r\n",
                                tfStrError(errorCode));
        errorCode = -1;
        goto APP_OPEN_INTF_END;
    }
/* Set the global variables for MSS and socket send queue size */
    SndBufSize = sizeof(AppBuf);
    TcpMss = IfaceMtu;

APP_OPEN_INTF_END:
    return errorCode;
}


/****************************************************************************/
/****************************************************************************/
/*                          TX CLIENT CODE                                  */
/****************************************************************************/
/****************************************************************************/


/******************************************************************************
 * Run Tx application that sends TCP data as fast as possible to a server
 * running on a host machine
 *****************************************************************************/
int AppTxTcpMainSimple(unsigned MenuIndex, unsigned TestLoops)
{
    struct sockaddr_in serv_addr;
    int sock;
    int errorCode;
    int optVal;
    int bytes;

    errorCode = 0;

/* Calculate how many bytes to send per call to the socket's send()
 *
 * - For best performance, ChunkSize will be set to a multiple of
 *   the MSS except when...
 *
 * - If the socket's send buffer is smaller than the MSS, then we
 *   have to limit the chunk to the size of the send buffer size
 *
 */
    if (TcpMss >= SndBufSize)
    {
        ChunkSize = SndBufSize;
    }
    else
    {
        ChunkSize = (SndBufSize / TcpMss) / CHUNK_SNDBUF_RATIO;
        if (ChunkSize <= 0)
        {
            ChunkSize = TcpMss;
        }
        else
        {
            ChunkSize *= TcpMss;
        }
    }
/* Create a socket with an ephemeral port number */
    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock == TM_SOCKET_ERROR)
    {
        xil_printf("AppTxTcpMainSimple: failed to create a socket\r\n");
        errorCode = -1;
        goto APP_TX_SIMPLE_EXIT;
    }

/* Make this socket non-blocking since we don't have an OS and cannot block */
    optVal = 1;
    errorCode = tfIoctl(sock, FIONBIO, &optVal);
    if (errorCode < 0)
    {
        xil_printf("AppTxTcpMainSimple: ioctl failed: %s\r\n", tfStrError(errorCode));
        goto APP_TX_SIMPLE_EXIT;
    }

/* disable TCP timestamps - this gives us an extra 12 bytes per frame for data */
    optVal = 0;
    errorCode = setsockopt(sock, IP_PROTOTCP, TM_TCP_TS, (char *)&optVal, sizeof(optVal));
    if (errorCode < 0)
    {
        xil_printf("AppTxTcpMainSimple: TM_TCP_TS failed: %s\r\n", tfStrError(errorCode));
        goto APP_TX_SIMPLE_EXIT;
    }

/* set the socket send buffer size */
    optVal = SndBufSize;
    errorCode = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&optVal, sizeof(optVal));
    if (errorCode < 0)
    {
        xil_printf("AppTxTcpMainSimple: SO_SNDBUF failed: %s\r\n", tfStrError(errorCode));
        goto APP_TX_SIMPLE_EXIT;
    }


/* Fill in the structure "serv_addr" with the address of the
 * server that we want to connect with.
 */
    tfMemSet(&serv_addr, 0, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = inet_addr(HostIp);
    serv_addr.sin_port = htons(ServerTcpPort);

/* connect to the server */
    errorCode = connect(sock,(struct sockaddr*)&serv_addr, sizeof(serv_addr));
    if (errorCode < 0)
    {
        errorCode = tfGetSocketError(sock);
        if (errorCode != TM_EINPROGRESS)
        {
            xil_printf("AppTxTcpMainSimple: call to connect failed(%s)\r\n",
                        tfStrError(errorCode));
            errorCode = -1;
            goto APP_TX_SIMPLE_EXIT;
        }
    }
/*
 * Sit here and process incoming packets until the connect completes.
 * We could also register a callback for this.
 */
    while (errorCode == TM_EINPROGRESS)
    {
        PollIO();
        errorCode = tfGetSocketError(sock);
        if (errorCode == TM_ENOERROR)
        {
            xil_printf("AppTxTcpMainSimple: connect succeeded\r\n");
            break;
        }
        if (errorCode != TM_EINPROGRESS)
        {
            xil_printf("AppTxTcpMainSimple: failed to connect(%s)\r\n",
                        tfStrError(errorCode));
            errorCode = -1;
            goto APP_TX_SIMPLE_EXIT;
        }
    }
/* send TestFileSizeMB Megabytes to the server */
    sentBytes = 0;
    sentMBytes = 0;
    while(1)
    {
        PollIO();
        bytes = tfZeroCopyUserBufferSend(sock,
                                    AppBuf,
                                    AppBuf,
                                    ChunkSize,
                                    AppTxTcpScatSendFree,
                                    0);

        if (bytes < 0)
        {
            errorCode = tfGetSocketError(sock);
            if (errorCode != TM_EWOULDBLOCK)
            {
               xil_printf("AppTxTcpMainSimple: send failed(%s)\r\n",
                                        tfStrError(errorCode));
                errorCode = -1;
                goto APP_TX_SIMPLE_EXIT;
            }
        }
        else
        {
            sentBytes += bytes;
            while (sentBytes > BYTES_PER_MB)
            {
                sentMBytes++;
                sentBytes -= BYTES_PER_MB;
            }

            if ((ChunkSize + sentBytes) > (BYTES_PER_MB)
                && ((sentMBytes + 1) == TestFileSizeMB))
            {
/*
* Another call to send ChunkSize will be too much.
* Just send what's left from TestFileSizeMB.
*/
                ChunkSize = (BYTES_PER_MB) - sentBytes;
                if (ChunkSize == 0)
                {
                    xil_printf("AppTxTcpMainSimple: file sent\r\n", ChunkSize);
                    goto APP_TX_SIMPLE_EXIT;
                }
                xil_printf("AppTxTcpMainSimple: Adjusting ChunkSize to %d bytes to finish the file\r\n", ChunkSize);
            }
            else if (sentMBytes >= TestFileSizeMB)
            {
/* This case should never be hit */
                xil_printf("AppTxTcpMainSimple: How did we get here? sentMBytes = %d ChunkSize = %d\r\n",
                            sentMBytes,
                            ChunkSize);
                goto APP_TX_SIMPLE_EXIT;
            }
        }
    }/* while(1) */

APP_TX_SIMPLE_EXIT:
    tfClose(sock);
    return errorCode;

}
int AppTxTcpMainCallbacks(unsigned MenuIndex, unsigned TestLoops)
{

    struct sockaddr_in serv_addr;
    int val;
    int opt;
    int opt_len;
    int err;
    int retCode;
    int clientSocket;

    retCode = 0;
    sentBytes = 0;
    sentMBytes = 0;
    AppError = 0;
    AppTxAdvSockStatus = 0;
/* Calculate how many bytes to send per call to the socket's send()
 *
 * - For best performance, ChunkSize will be set to a multiple of
 *   the MSS except when...
 *
 * - If the socket's send buffer is smaller than the MSS, then we
 *   have to limit the chunk to the size of the send buffer size
 *
 */
    if (TcpMss >= SndBufSize)
    {
        ChunkSize = SndBufSize;
    }
    else
    {
        ChunkSize = (SndBufSize / TcpMss) / CHUNK_SNDBUF_RATIO;
        if (ChunkSize <= 0)
        {
            ChunkSize = TcpMss;
        }
        else
        {
            ChunkSize *= TcpMss;
        }
    }

/* Idle for a few seconds */
#ifdef __PPC__
    sleep(3);
#else
    usleep(3000);
#endif
    xil_printf("AppTxTcpMainCallbacks: Sending %d bytes at a time\r\n", ChunkSize);

/* Create a TCP connection to the host */


/* Fill in the structure "serv_addr" with the address of the
 * server that we want to connect with.
 */
    tfMemSet(&serv_addr, 0, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = inet_addr(HostIp);
    serv_addr.sin_port = htons(ServerTcpPort);

    clientSocket = socket(AF_INET, SOCK_STREAM, 0);
    if (clientSocket < 0)
    {
        xil_printf("AppTxTcpMainCallbacks: unable to create a socket\r\n");
        retCode = -1;
        goto APP_TX_TCP_ADV_EXIT;
    }

/* disable TCP timestamps */
    opt = 0;
    opt_len = sizeof(opt);
    err = setsockopt(clientSocket, IP_PROTOTCP, TM_TCP_TS, (char *)&opt, opt_len);
    if (err < 0)
    {
        xil_printf("AppTxTcpMainCallbacks: TM_TCP_TS failed: %s\r\n", tfStrError(err));
        retCode = -1;
        goto APP_TX_TCP_ADV_EXIT;

    }

/* set send buffer sizes */
    opt = SndBufSize;
    opt_len = sizeof(opt);
    err = setsockopt(clientSocket, SOL_SOCKET, SO_SNDBUF, (char *)&opt, opt_len);
    if (err < 0)
    {
        xil_printf("AppTxTcpMainCallbacks: SO_SNDBUF failed: %s\r\n", tfStrError(err));
        retCode = -1;
        goto APP_TX_TCP_ADV_EXIT;

    }

    opt = (3*SndBufSize)/4;
    opt_len = sizeof(opt);
    err = setsockopt(clientSocket, SOL_SOCKET, SO_SNDLOWAT, (char *)&opt, opt_len);
    if (err < 0)
    {
        xil_printf("AppTxTcpMainCallbacks: SO_SNDLOWAT failed: %s\r\n", tfStrError(err));
        retCode = -1;
        goto APP_TX_TCP_ADV_EXIT;

    }

/* Make this socket non-blocking (needed for polled I/O) */
    val = 1;
    err = tfIoctl(clientSocket, FIONBIO, &val);
    if (err < 0)
    {
        xil_printf("AppTxTcpMainCallbacks: ioctl failed: %s\r\n", tfStrError(err));
        retCode = -1;
        goto APP_TX_TCP_ADV_EXIT;

    }

/* Setup a callback routine. This will be called once
 * the connection is completed or an error occurs
 */
    err = tfRegisterSocketCB(clientSocket, AppTxTcpAdvCB,
                             TM_CB_CONNECT_COMPLT |
                             TM_CB_SOCKET_ERROR |
                             TM_CB_CLOSE_COMPLT |
                             TM_CB_WRITE_READY |
                             TM_CB_RESET |
                             TM_CB_REMOTE_CLOSE);
    if (err < 0)
    {
        xil_printf("AppTxTcpMainCallbacks: failed to register callback (%s)\r\n", tfStrError(err));
        retCode = -1;
        goto APP_TX_TCP_ADV_EXIT;

    }

/* Begin connection process, check for errors in the initial call to
 * connect. From this point, the stack will invoke the callback on the
 * outcome of the connection.
 */
    err = connect(clientSocket, (struct sockaddr *) &serv_addr, sizeof(serv_addr));
    if (err < 0)
    {
        err = tfGetSocketError(clientSocket);
        if (err != TM_EINPROGRESS)
        {
            xil_printf("AppTxTcpMainCallbacks: failed to connect (%s)\r\n", tfStrError(err));
            retCode = -1;
            goto APP_TX_TCP_ADV_EXIT;

        }
    }
    while ((AppTxAdvSockStatus == 0) && (AppError == 0))
    {
        PollIO();
    }

/* Main loop sends data and receives acks */
    while(AppError == 0)
    {
        PollIO();

        if ((ChunkSize + sentBytes) > (BYTES_PER_MB)
            && ((sentMBytes + 1) == TestFileSizeMB))
        {
/*
 * Another call to send ChunkSize will be too much.
 * Just send what's left from TestFileSizeMB.
 */
            ChunkSize = (BYTES_PER_MB) - sentBytes;
            if (ChunkSize == 0)
            {
                break;
            }
            xil_printf("AppTxTcpMainCallbacks: Adjusting ChunkSize to %d bytes to finish the file\r\n", ChunkSize);
        }
        else if (sentMBytes >= TestFileSizeMB)
        {
/* This case could get hit if ChunkSize is large enough */
            break;
        }

    }
APP_TX_TCP_ADV_EXIT:

/*
 * Close the socket and wait for the close to finish.
 */
    if (clientSocket >= 0)
    {
        AppTxAdvSockStatus |= APP_TX_ADV_SOCK_CLOSING;
        if (AppError != 0)
        {
/*
 * We got here because there was a problem in the app. We have no choice
 * but to reset the connection
 */
            xil_printf("AppTxTcpMainCallbacks: failed to register close callback (%s)\r\n", tfStrError(err));
            (void)tfResetConnection(clientSocket);
            (void)tfClose(clientSocket);
        }
        else
        {
/* Close the socket gracefully */
            xil_printf("AppTxTcpMainCallbacks: closing the socket\r\n", tfStrError(err));
            tfClose(clientSocket);
/* Wait for the close to complete */
            while((AppTxAdvSockStatus & APP_TX_ADV_SOCK_CLOSED) == 0)
            {
                PollIO();
            }
        }
    }
    return retCode;
}


/******************************************************************************
 * This function is called upon completion of a TCP connect or a socket error
 *
 * Parameters:
 *   Sock - Socket the calback was invoked for
 *   Flag - Which event has occurred
 *****************************************************************************/
static void AppTxTcpAdvCB(int Sock, int Flag)
{
    int errorCode;
    int len;
    int i;
    int bytes;

    bytes = 0;

/* Was it an error? */
    if (Flag & TM_CB_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(Sock);
        if (errorCode == TM_ENOERROR)
        {
            len = sizeof(errorCode);
            getsockopt(Sock,
                        SOL_SOCKET,
                        SO_ERROR,
                        (char*)&errorCode,
                        &len);
        }
        xil_printf("AppTxTcpAdvCB: error (%s)\r\n", tfStrError(errorCode));
        AppError = 1;
        return;
    }

/*
 * Connection is completed. Now reset the callback to invoke the send
 * function when there is room in the socket buffer
 */
    if (Flag & TM_CB_CONNECT_COMPLT)
    {
        xil_printf("AppTxTcpAdvCB: Connection established\r\n");
        AppTxAdvSockStatus |= APP_TX_ADV_SOCK_CONNECTED;

/* Place a pattern in the send buffer */
        for (i=0; i<sizeof(AppBuf); i++)
        {
            AppBuf[i] = (char)i;
        }
        bytes = tfZeroCopyUserBufferSend(Sock,
                                        AppBuf,
                                        AppBuf,
                                        sizeof(AppBuf),
                                        AppTxTcpScatSendFree,
                                        0);
        if (bytes > 0)
        {
            sentBytes += bytes;
        }
    }
/*
 * Connection is completed. Now reset the callback to invoke the send
 * function when there is room in the socket buffer
 */
    if (Flag & TM_CB_CLOSE_COMPLT)
    {
        xil_printf("AppTxTcpAdvCB: Connection closed\r\n");
        AppTxAdvSockStatus |= APP_TX_ADV_SOCK_CLOSED;
    }
/*
 * If we get a WRITE-READY callback, we need to make sure the socket is not
 * closing or closed.
 */
    if ((Flag & TM_CB_WRITE_READY) &&
        ((AppTxAdvSockStatus ^ APP_TX_ADV_SOCK_CONNECTED) == 0))
    {
        bytes = tfZeroCopyUserBufferSend(Sock,
                                        AppBuf,
                                        AppBuf,
                                        ChunkSize,
                                        AppTxTcpScatSendFree,
                                        0);
        if (bytes < 0)
        {
           errorCode = tfGetSocketError(Sock);
           if (errorCode != TM_EWOULDBLOCK)
           {
               xil_printf("AppTxTcpAdvCB: failed to send in WRITE_READY (%s)\r\n", tfStrError(errorCode));
               AppError = 1;
           }
        }
        else
        {
            sentBytes += bytes;
            while (sentBytes > BYTES_PER_MB)
            {
                sentMBytes++;
                sentBytes -= BYTES_PER_MB;
            }
        }
    }

    if (Flag & TM_CB_RESET)
    {
        xil_printf("AppTxTcpAdvCB: Connection reset by peer\r\n");
        AppError = 1;
    }

    if (Flag & TM_CB_REMOTE_CLOSE)
    {
        xil_printf("AppTxTcpAdvCB: Connection closed by peer\r\n");
        AppError = 1;
    }
}

/******************************************************************************
 * Stack calls this function whenever our AppBuf is free after a scatter
 * send. Do nothing because we don't ever change the AppBuf.
 *
 * Returns:
 *   TM_ENOERROR
 *****************************************************************************/
static int AppTxTcpScatSendFree(char *BufPtr)
{
    return(TM_ENOERROR);
}

/******************************************************************************
 * Run Tx application that sends UDP data as fast as possible to a server
 * running on a host machine
 *****************************************************************************/
int AppTxUdpMainWithoutConnection(unsigned MenuIndex, unsigned TestLoops)
{
    struct sockaddr_in serv_addr;
    int sock;
    int errorCode;
    int optVal;
    int bytes;

    errorCode = 0;

/* Calculate how many bytes to send per call to the socket's send()
 *
 * - For best performance, ChunkSize will be set to MTU less 20
 *   bytes for IP header and 8 bytes for UDP header except when...
 *
 * - If the socket's send buffer is smaller than the MTU, then we
 *   have to limit the chunk to the size of the send buffer size
 *
 */
    if (IfaceMtu >= SndBufSize)
    {
        ChunkSize = SndBufSize;
    }
    else
    {
/*ChunkSize = MTU - 20 bytes for the IP header and 8 bytes for UDP header*/
            ChunkSize = IfaceMtu - 28;
    }
/* Create a socket with an ephemeral port number */
    sock = socket(AF_INET, SOCK_DGRAM, IP_PROTOUDP);
    if (sock == TM_SOCKET_ERROR)
    {
        xil_printf("AppTxTcpMainSimple: failed to create a socket\r\n");
        errorCode = -1;
        goto APP_TX_SIMPLE_EXIT;
    }

/* Make this socket non-blocking since we don't have an OS and cannot block */
    optVal = 1;
    errorCode = tfIoctl(sock, FIONBIO, &optVal);
    if (errorCode < 0)
    {
        xil_printf("AppTxTcpMainSimple: ioctl failed: %s\r\n", tfStrError(errorCode));
        goto APP_TX_SIMPLE_EXIT;
    }

/* set the socket send buffer size */
    optVal = SndBufSize;
    errorCode = setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (char *)&optVal, sizeof(optVal));
    if (errorCode < 0)
    {
        xil_printf("AppTxTcpMainSimple: SO_SNDBUF failed: %s\r\n", tfStrError(errorCode));
        goto APP_TX_SIMPLE_EXIT;
    }

/*
 *  Fill in the structure "serv_addr" with the address of the
 * server that we want to connect with.
 */
    tfMemSet(&serv_addr, 0, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = inet_addr(HostIp);
    serv_addr.sin_port = htons(ServerTcpPort);

/*
 * Sit here and process incoming packets until the connect completes.
 * We could also register a callback for this.
 */
    while (errorCode == TM_EINPROGRESS)
    {
        PollIO();
        errorCode = tfGetSocketError(sock);
        if (errorCode == TM_ENOERROR)
        {
            xil_printf("AppTxTcpMainSimple: connect succeeded\r\n");
            break;
        }
        if (errorCode != TM_EINPROGRESS)
        {
            xil_printf("AppTxTcpMainSimple: failed to connect(%s)\r\n",
                        tfStrError(errorCode));
            errorCode = -1;
            goto APP_TX_SIMPLE_EXIT;
        }
    }
/* send TestFileSizeMB Megabytes to the server */
    sentBytes = 0;
    sentMBytes = 0;
    while(1)
    {
        PollIO();
        bytes = tfZeroCopyUserBufferSendTo(sock,
                                    AppBuf,
                                    AppBuf,
                                    ChunkSize,
                                    AppTxTcpScatSendFree,
                                    0,
                                    (struct sockaddr TM_FAR *)&serv_addr,
                                    sizeof(struct sockaddr_in));

        if (bytes < 0)
        {
            errorCode = tfGetSocketError(sock);
            if (errorCode != TM_EWOULDBLOCK)
            {
               xil_printf("AppTxTcpMainSimple: send failed(%s)\r\n",
                                        tfStrError(errorCode));
               errorCode = -1;
               goto APP_TX_SIMPLE_EXIT;
            }
            else
            {
/*Polls hardware to check if send has completed*/
                tfIoctlInterface(interfaceHandle,
                TM_TEMAC_IOCTL_SEND_COMPLETE, (void *) 0, 0);
            }
        }
        else
        {
            sentBytes += bytes;
            while (sentBytes > BYTES_PER_MB)
            {
                sentMBytes++;
                sentBytes -= BYTES_PER_MB;
            }

            if ((ChunkSize + sentBytes) > (BYTES_PER_MB)
                && ((sentMBytes + 1) == TestFileSizeMB))
            {
/*
* Another call to send ChunkSize will be too much.
* Just send what's left from TestFileSizeMB.
*/
                ChunkSize = (BYTES_PER_MB) - sentBytes;
                if (ChunkSize == 0)
                {
                    xil_printf("AppTxTcpMainSimple: %dMB sent\r\n", sentMBytes);
                    goto APP_TX_SIMPLE_EXIT;
                }
                xil_printf("AppTxTcpMainSimple: Adjusting ChunkSize to %d bytes to finish the file\r\n", ChunkSize);
            }
            else if (sentMBytes >= TestFileSizeMB)
            {
/* This case could get hit if ChunkSize is large enough */
                xil_printf("AppTxTcpMainSimple: %dMB sent\r\n", sentMBytes);
                goto APP_TX_SIMPLE_EXIT;
            }
        }
    }/* while(1) */

APP_TX_SIMPLE_EXIT:
    tfClose(sock);
    return errorCode;

}

/****************************************************************************/
/****************************************************************************/
/*                          RX SERVER CODE                                  */
/****************************************************************************/
/****************************************************************************/

/******************************************************************************
 * Entry point for Rx app. Using polled mode I/O, setup a socket to listen
 * on a known port. When a new connection is accepted, receive any bulk data
 * the peer sends (received data is ignored) until the connection is closed
 * by the peer.
 *****************************************************************************/
int AppRxTcpMain(unsigned MenuIndex, unsigned TestLoops)
{
    struct sockaddr clientAddr;
    struct sockaddr_in serverAddr;
    int listenSocket;
    int acceptSocket;
    int errorCode;
    int opt;
    int addrLen;

    acceptSocket = 0;
    bytesReceived = 0;
    gigaByteCount = 0;
    AppRxSimpleConnected = 0;
    xil_printf("AppRxTcpMain started\r\n");

    errorCode = TM_ENOERROR;

/* Specify the address family for the server */
    serverAddr.sin_family = AF_INET;
/* Specify the destination port for the server */
    serverAddr.sin_port = htons(ServerTcpPort );
/*
 * Specify the destination IP address (our IP address). Setting
 * this value to 0 tells the stack that we don?t care what IP
 * address we use - it should pick one. For systems with one IP
 * address, this is the easiest approach.
 */
    serverAddr.sin_addr.s_addr = INADDR_ANY;
    serverAddr.sin_len = sizeof(struct sockaddr_in);

/* Create an IPv4 TCP socket */
    listenSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

/* Verify the socket was created correctly. If not, return immediately */
    if (listenSocket == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: socket error: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }

    opt = 1;
    errorCode = setsockopt(listenSocket, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(opt));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: SO_REUSEADDR failed: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }

    opt = SndBufSize;
    xil_printf("AppRxTcpMain: setting recv sock size to %d bytes\r\n", opt);
    errorCode = setsockopt(listenSocket, SOL_SOCKET, SO_RCVBUF, (char *)&opt, sizeof(opt));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: SO_RCVBUF failed: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }

/* Make this socket non-blocking (needed for polled I/O) */
    opt = 1;
    errorCode = tfIoctl(listenSocket, FIONBIO, &opt);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: ioctl failed: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }

/* Bind the socket to the specified port and address */
    errorCode = bind(listenSocket, (struct sockaddr *)&serverAddr, sizeof(serverAddr));

    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: bind error: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }

/* Listen on the newly created and bound socket */
    errorCode = listen(listenSocket, 1);

    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: listen error: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }


/* Register a callback to notify the application when a connection is accepted */
    errorCode = tfRegisterSocketCB(listenSocket, AppRxTcpSimpleCB,
                             TM_CB_ACCEPT | TM_CB_SOCKET_ERROR | TM_CB_RESET);

    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxTcpMain: error registering listening socket callback: %s\r\n",
                   tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }

/* Wait for the client to connect */
    while (!AppRxSimpleConnected)
    {
        PollIO();
    }

/* Accept incoming connections on the listening socket */
    addrLen = sizeof(clientAddr);
    acceptSocket = accept(listenSocket, &clientAddr, &addrLen);
    if (acceptSocket == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        if (errorCode != TM_EWOULDBLOCK)
        {
            xil_printf("AppRxTcpMain: accept error: %s\r\n", tfStrError(errorCode));
            goto SIMPLE_TCP_RX_EXIT;
        }
    }
/* Register a simple callback to notify the application when events occur on
 * the accepted socket.
 */
    errorCode = tfRegisterSocketCB(acceptSocket, AppRxTcpSimpleCB,
                             TM_CB_REMOTE_CLOSE | TM_CB_SOCKET_ERROR | TM_CB_RESET | TM_CB_RECV);
    if (errorCode != TM_ENOERROR)
    {
        errorCode = tfGetSocketError(acceptSocket);
        xil_printf("AppRxTcpMain: error registering accepted socket callback: %s\r\n",
                   tfStrError(errorCode));
        goto SIMPLE_TCP_RX_EXIT;
    }
/*
 * Since we receive data on the new socket in the callbacks. All that we have to
 * do here is sit and process incoming packets until the peer closes the
 * connection or an error occurs.
 */
    startReceive = 0;
    while (AppRxSimpleConnected)
    {
        PollIO();
    }
     endReceive = TimeStampGet();
     xil_printf("AppRxTcpMain: In total, %dMB received\n\r",
         gigaByteCount * 1024 + bytesReceived / BYTES_PER_MB);

    xil_printf("AppRxTcpMain: Time Elapsed - %d ms\n\r", TimeStampDiffMs(startReceive, endReceive));



SIMPLE_TCP_RX_EXIT:
/* Make sure we have a socket before closing it */
    if (listenSocket != -1)
    {
        xil_printf("AppRxTcpMain: closing the listening socket\r\n");
/* Close the socket */
        tfClose(listenSocket);
    }
    if (acceptSocket != -1)
    {
        xil_printf("AppRxTcpMain: closing the accepted socket\r\n");
/* Close the socket */
        tfClose(acceptSocket);
    }

    return(0);
}
/* A list of all the event flags
 * #define TM_CB_CONNECT_COMPLT            0x0001
 * #define TM_CB_ACCEPT                    0x0002
 * #define TM_CB_RECV                      0x0004
 * #define TM_CB_RECV_OOB                  0x0008
 * #define TM_CB_SEND_COMPLT               0x0010
 * #define TM_CB_REMOTE_CLOSE              0x0020
 * #define TM_CB_SOCKET_ERROR              0x0040
 * #define TM_CB_RESET                     0x0080
 * #define TM_CB_CLOSE_COMPLT              0x0100
 * #define TM_CB_WRITE_READY               0x0200
 */

static void AppRxTcpSimpleCB(int Sock, int Flag)
{
    ttUserMessage   RxBufHandle;
    char*           RxBuf;
    int             errorCode;
    float           seconds;
    float           mbps;

    if (Flag & TM_CB_ACCEPT)
    {
        xil_printf("AppRxTcpSimpleCB: accepted a connection\r\n");
        AppRxSimpleConnected = 1;
    }
    if (Flag & TM_CB_REMOTE_CLOSE)
    {
        xil_printf("AppRxTcpSimpleCB: connection closed by peer\r\n");
        AppRxSimpleConnected = 0;
    }
    if (Flag & TM_CB_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(Sock);
        xil_printf("AppRxTcpSimpleCB: socket error(%s)\r\n", tfStrError(errorCode));
        AppRxSimpleConnected = 0;
    }
    if (Flag & TM_CB_RESET)
    {
        xil_printf("AppRxTcpSimpleCB: connection reset by peer\r\n");
        AppRxSimpleConnected = 0;
    }
    if (Flag & TM_CB_RECV)
    {
        errorCode = tfZeroCopyRecv(Sock, &RxBufHandle, &RxBuf, SndBufSize, 0);

        if (errorCode > 0)
        {
            bytesReceived = bytesReceived + errorCode;
            errorCode = tfFreeZeroCopyBuffer(RxBufHandle);

            if(errorCode != TM_ENOERROR)
            {
                xil_printf("AppRxTcpSimpleCB: failed to free zero copy buffer\r\n");
                     AppRxSimpleError = 1;
            }
            if(bytesReceived > BYTES_PER_GB)
            {
                 endReceive = TimeStampGet();
                 seconds = TimeStampDiffMs(lastGigaByteReceived, endReceive);
                 seconds /= 1000;

                 gigaByteCount++;

/* Calculate megabits per second*/
                 mbps = bytesReceived;
                 mbps /= BYTES_PER_MB_MBPS_CALC;
                 mbps *= BITS_PER_BYTE;
                 mbps /= seconds;

/* xil_printf doesn't support floating point numbers */
                 xil_printf("AppRxTcpSimpleCB: %dGB received @ %dMbits/s\n\r", gigaByteCount, (int)mbps);
                 bytesReceived = 0;
                 lastGigaByteReceived = endReceive;
            }
/*
 * if DEFAULT_RX_GIGABYTE_CUTOFF is reached, exit while loop in AppRxTcpMain, stop
 * polling for IO
 */
            if(gigaByteCount > (DEFAULT_RX_GIGABYTE_CUTOFF - 1))
            {
/* unregister the callback */
                tfRegisterSocketCB(Sock, (ttUserSocketCBFuncPtr)0,
                                   TM_CB_SOCKET_ERROR
                                   | TM_CB_RESET
                                   | TM_CB_RECV);
/* Set AppRxSimpleConnect to 0 to exit the while loop*/
                AppRxSimpleConnected = 0;
            }
/* if first time data is received, get time stamp */
            if(startReceive == 0)
            {
                startReceive = TimeStampGet();
                lastGigaByteReceived = startReceive;
            }
        }
        else
        {
            errorCode = tfGetSocketError(Sock);
            if ((errorCode != TM_EWOULDBLOCK)
                    && (errorCode != TM_ENOERROR))
            {
                xil_printf("AppRxTcpSimpleCB: recv error: %s\r\n", tfStrError(errorCode));
                AppRxSimpleConnected = 0;
            }
        }
    }
}

static void AppRxUdpSimpleCB(int Sock, int Flag)
{
    ttUserMessage   RxBufHandle;
    char*           RxBuf;
    int             errorCode;
     float           seconds;
     float            mbps;

    if (Flag & TM_CB_REMOTE_CLOSE)
    {
        xil_printf("AppRxTcpSimpleCB: connection closed by peer\r\n");
        AppRxSimpleError = 1;
    }
    if (Flag & TM_CB_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(Sock);
        xil_printf("AppRxTcpSimpleCB: socket error(%s)\r\n", tfStrError(errorCode));
        AppRxSimpleError = 1;
    }
    if (Flag & TM_CB_RESET)
    {
        xil_printf("AppRxTcpSimpleCB: connection reset by peer\r\n");
        AppRxSimpleError = 1;
    }
    if (Flag & TM_CB_RECV)
    {
        errorCode = tfZeroCopyRecvFrom(Sock, &RxBufHandle, &RxBuf,
              SndBufSize, 0, (struct sockaddr TM_FAR *)0, 0);

        if (errorCode > 0)
        {
            bytesReceived = bytesReceived + errorCode;
                errorCode = tfFreeZeroCopyBuffer(RxBufHandle);

                if(errorCode != TM_ENOERROR)
                {
                    xil_printf("AppRxUdpSimpleCB: failed to free zero copy buffer\r\n");
                         AppRxSimpleError = 1;
                }
                if(bytesReceived > BYTES_PER_GB)
                {
                     endReceive = TimeStampGet();
                     seconds = TimeStampDiffMs(lastGigaByteReceived, endReceive) / MS_PER_SECOND;
                     lastGigaByteReceived = endReceive;
                     gigaByteCount++;

/* Calculate megabits per second*/
                     mbps = bytesReceived;
                     mbps /= BYTES_PER_MB_MBPS_CALC;
                     mbps *= BITS_PER_BYTE;
                     mbps /= seconds;

/* xil_printf doesn't support floating point numbers */
                     xil_printf("AppRxUdpSimpleCB: %dGB received @ %dMbits/s\n\r", gigaByteCount, (int)mbps);
                     bytesReceived = 0;
                }
/*
 * if DEFAULT_RX_GIGABYTE_CUTOFF is reached, exit while loop in AppRxUdpMain, stop
 * polling for IO
 */
                if(gigaByteCount > (DEFAULT_RX_GIGABYTE_CUTOFF - 1))
                {
                    AppRxSimpleConnected = 1;
                }

/* if first time data is received, get time stamp */
                if(startReceive == 0)
                {
                    xil_printf("AppRxUdpSimpleCB: Receiving Data...\n\r");
                    startReceive = TimeStampGet();
                    lastGigaByteReceived = startReceive;
                }
        }
        else
        {
            errorCode = tfGetSocketError(Sock);
            if ((errorCode != TM_EWOULDBLOCK)
                    && (errorCode != TM_ENOERROR))
            {
                xil_printf("AppRxTcpSimpleCB: recv error: %s\r\n", tfStrError(errorCode));
                AppRxSimpleError = 1;
            }
        }
    }
}
/******************************************************************************
 * Entry point for UDP Rx app. Using polled mode I/O, setup a socket to listen
 * on a known port. When data is sent, receive any bulk data
 * the peer sends (received data is ignored) until a certain amount of data is
 * received.
 *****************************************************************************/
int AppRxUdpMain(unsigned MenuIndex, unsigned TestLoops)
{
    struct sockaddr_in serverAddr;
    int listenSocket;
    int errorCode;
    int opt;

    AppRxSimpleConnected = 0;
     AppRxSimpleError = 0;
     bytesReceived = 0;
     gigaByteCount = 0;
    xil_printf("AppRxUdpMain started\r\n");

    errorCode = TM_ENOERROR;

/* Specify the address family for the server */
    serverAddr.sin_family = AF_INET;
/* Specify the destination port for the server */
    serverAddr.sin_port = htons(ServerTcpPort);
/*
 * Specify the destination IP address (our IP address). Setting
 * this value to 0 tells the stack that we don?t care what IP
 * address we use - it should pick one. For systems with one IP
 * address, this is the easiest approach.
 */
    serverAddr.sin_addr.s_addr = INADDR_ANY;
    serverAddr.sin_len = sizeof(struct sockaddr_in);

/* Create an IPv4 UDP socket */
    listenSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

/* Verify the socket was created correctly. If not, return immediately */
    if (listenSocket == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxUdpMain: socket error: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_UDP_RX_EXIT;
    }

    opt = 1;
    errorCode = setsockopt(listenSocket, SOL_SOCKET, SO_REUSEADDR, (char *)&opt, sizeof(opt));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxUdpMain: SO_REUSEADDR failed: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_UDP_RX_EXIT;
    }

    opt = SndBufSize;
    xil_printf("AppRxUdpMain: setting recv sock size to %d bytes\r\n", opt);
    errorCode = setsockopt(listenSocket, SOL_SOCKET, SO_RCVBUF, (char *)&opt, sizeof(opt));
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxUdpMain: SO_RCVBUF failed: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_UDP_RX_EXIT;
    }

/* Make this socket non-blocking (needed for polled I/O) */
    opt = 1;
    errorCode = tfIoctl(listenSocket, FIONBIO, &opt);
    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxUdpMain: ioctl failed: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_UDP_RX_EXIT;
    }

/* Bind the socket to the specified port and address */
    errorCode = bind(listenSocket, (struct sockaddr *)&serverAddr, sizeof(serverAddr));

    if (errorCode == TM_SOCKET_ERROR)
    {
        errorCode = tfGetSocketError(listenSocket);
        xil_printf("AppRxUdpMain: bind error: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_UDP_RX_EXIT;
    }

/* Register a callback to notify the application when data is received or errors occur */
    errorCode = tfRegisterSocketCB(listenSocket, AppRxUdpSimpleCB,
                             TM_CB_REMOTE_CLOSE | TM_CB_SOCKET_ERROR | TM_CB_RESET | TM_CB_RECV);

    if (errorCode != TM_ENOERROR)
    {
        xil_printf("AppRxUdpMain: error registering listening socket callback: %s\r\n", tfStrError(errorCode));
        goto SIMPLE_UDP_RX_EXIT;
    }

/*
 * Continue to poll for new data until certain amount of data received
 * or an error occurs.  The amount of data is specified in gigabytes
 * in the DEFAULT_RX_GIGABYTE_CUTOFF macro in defaults.h
 */
    startReceive = 0;
     while (!AppRxSimpleConnected && !AppRxSimpleError)
    {
        PollIO();
    }
     endReceive = TimeStampGet();
     xil_printf("AppRxUdpMain: In total, %dMB received\n\r",
         gigaByteCount * 1024 + bytesReceived / BYTES_PER_MB);

    xil_printf("AppRxUdpMain: Time Elapsed - %d ms\n\r", TimeStampDiffMs(startReceive, endReceive));


SIMPLE_UDP_RX_EXIT:
/* Make sure we have a socket before closing it */
    if (listenSocket != -1)
    {
        xil_printf("AppRxUdpMain: closing the socket\r\n");
/* Close the socket */
        tfClose(listenSocket);
    }

    return(0);
}

#ifdef TM_USE_HTTPD
int AppHttpServerToggle(unsigned MenuIndex, unsigned TestLoops)
{
    int err;

     if(httpServerEn == 1)
     {
         httpServerEn = 0;
         err = tfHttpdUserStop(httpSrvHandle);
         if(err != TM_ENOERROR)
         {
             xil_printf("AppHttpServerToggle: http server error: %s\n\r", tfStrError(err));
         }
         else
         {
             xil_printf("Http Server Started Successfully\n\r");
         }
    }
    else
    {
         httpServerEn = 1;
         int err;
         httpSrvHandle = tfHttpdUserStart(1,
                                          1,
                                          60,
                                          TM_BLOCKING_OFF,
                                          "/",
                                          "/index.htm",
                                          "/cgi-bin",
                                          NULL,
                                          &err);
         if(httpSrvHandle == TM_HTTPD_INVALID_SERVER_HANDLE)
         {
              xil_printf("Problem starting the webserver: %d, %d\n\r", tfStrError(err));
         }
         else
         {
             xil_printf("AppHttpServerToggle: Webserver Enabled\n\r");
         }
         tfHttpdUserSetEventHandler(httpSrvHandle, tfxHttpdEventHandler);

    }
     return(0);
}
#endif /* TM_USE_HTTPD */

#ifdef TM_USE_FTPD
int AppFtpServerToggle(unsigned MenuIndex, unsigned TestLoops)
{
    int err;
    if(ftpServerEn == 1)
    {
        ftpServerEn = 0;
        err = tfFtpdUserStop();
        if(err != 0)
        {
            xil_printf("AppFtpServerToggle: ERROR - %s\n\r", tfStrError(err));
        }
        else
        {
            xil_printf("FTP Server Stopped Successfully\n\r");
        }
    }
    else
    {
        ftpServerEn = 1;
        err = tfFtpdUserStart(TM_FS_ALLCMND_MASK,
                                  4,
                                  3,
                                  301,
                                  TM_BLOCKING_OFF);
        if(err != TM_ENOERROR)
        {
            xil_printf("AppFtpServerToggle: ERROR %s\n\r", tfStrError(err));
        }
        else
        {
            xil_printf("FTP Server Started Successfully\n\r");
        }
    }
   return(0);
}
#endif /* TM_USE_FTPD */

#ifdef TM_USE_TELNETD
int AppTelnetServerToggle(unsigned MenuIndex, unsigned TestLoops)
{
    int err;
    if(telnetServerEn == 1)
    {
        telnetServerEn = 0;
        err = tfTeldUserStop();
        if(err != 0)
        {
            xil_printf("AppTelnetServerToggle: ERROR - %s\n\r", tfStrError(err));
        }
        else
        {
            xil_printf("Telnet Server Stopped Successfully\n\r");
        }
    }
    else
    {
        telnetServerEn = 1;
        err = tfTeldUserStart(TM_TELD_BINARY_ON | TM_TELD_ECHO_ON,
                                  4,
                                  3,
                                  TM_BLOCKING_OFF);
        if(err != TM_ENOERROR)
        {
            xil_printf("AppTelnetServerToggle: ERROR %s\n\r", tfStrError(err));
        }
        else
        {
            xil_printf("Telnet Server Started Successfully\n\r");
        }

    }
   return(0);
}
#endif /* TM_USE_TELNETD */

#ifdef TM_SNMP_VERSION
int AppSnmpServerToggle(unsigned MenuIndex, unsigned TestLoops)
{
    int err;
    if(snmpServerEn == 1)
    {
        snmpServerEn = 0;
        err = tfSnmpdStop();
        if(err != 0)
        {
            xil_printf("AppSnmpServerToggle: ERROR - %s\n\r", tfStrError(err));
        }
        else
        {
            xil_printf("Snmp Agent Stopped Successfully\n\r");
        }
    }
    else
    {
        snmpServerEn = 1;
        err = tfSnmpdMain(TM_BLOCKING_OFF, 0);
        if(err != TM_ENOERROR)
        {
            xil_printf("AppSnmpServerToggle: ERROR %s\n\r", tfStrError(err));
        }
        else
        {
            xil_printf("Snmp Agent Started Successfully\n\r");
        }

    }
   return(0);
}
#endif /* TM_SNMP_VERSION */

int AppLoopbackTests(unsigned MenuIndex, unsigned TestLoops)
{
    xil_printf("AppLoopbackTests: starting ping loopback test...\n\r");
    treck_test_ping_loopback(interfaceHandle);
    xil_printf("\n\rAppLoopbackTests: starting tcp loopback test...\n\r");
    treck_test_tcp_loopback();
    xil_printf("\n\rAppLoopbackTests: starting udp loopback test...\n\r");
    treck_test_udp_loopback();

    xil_printf("\n\rAppLoopbackTests: tests complete\n\r");

    return 0;
}
/****************************************************************************/
/****************************************************************************
 ****************************************************************************
 *
 * The following code is common to all apps:
 *
 ****************************************************************************/
/****************************************************************************/
/****************************************************************************/

/*****************************************************************************
 * Poll the driver and the stack for frames sent, frames received.
 *
 * Parameters:
 *   Action - if POLL_TILL_EXPIRE then the polling option is performed as
 *     long as TimeExpired is not asserted, when TimeExpired is asserted
 *     then polling is not performed. If POLL_FOREVER then the polling
 *     operation is performed irregardless of the state of TimeExpired.
 *****************************************************************************/
void PollIO()
{
    int err;

#ifdef TM_USE_TIMER_CB
    /* Check for expired timers */
    if (nextTimerExecuteTs == 0)
    {
        tfTimerExecute();
    }
#else /* TM_USE_TIMER_CB */
    tfTimerExecute();
#endif /* TM_USE_TIMER_CB */

    /* There have been events which an endless loop is created between
     * this function and one of the send callbacks. This check should
     * prevent it.
     */

    /* Check for received frames */
    while(tfCheckReceiveInterface(interfaceHandle) == 0)
    {
        /* Calls the driver's recv function */
        tfRecvInterface(interfaceHandle);
    }

    /* No check required for transmitted frames. The driver should handle that
     * internally.
     * Call the Ioctl function to force any outstanding sends.
    tfIoctlInterface(interfaceHandle, TM_DEV_IOCTL_EMPTY_XMIT_FLAG, (void*)0, 0);
     */
#ifdef TM_USE_HTTPD
      if(httpServerEn == 1)
      {
          err = tfHttpdUserExecute(httpSrvHandle);
          if (err != TM_ENOERROR)
          {
                xil_printf("PollIO: tfHttpdUserExecute failed.\n\r", err);
          }
      }
#endif /* TM_USE_HTTPD*/

#ifdef TM_USE_FTPD
      if(ftpServerEn == 1)
      {
          err = tfFtpdUserExecute();
          if (err == TM_EPERM)
          {
                xil_printf("PollIO: tfFtpdUserExecute failed: %s\n\r", tfStrError(err));
          }
      }
#endif /* TM_USE_FTPD */

#ifdef TM_USE_TELNETD
      if(telnetServerEn == 1)
      {
          err = tfTeldUserExecute();
          if (err == TM_EPERM)
          {
                xil_printf("PollIO: tfTeldUserExecute failed: %s\n\r", tfStrError(err));
          }
      }
#endif /*TM_USE_TELNETD */
}

/*****************************************************************************/
/******************************************************************************
 *                    Utility Menu Functions
 *****************************************************************************/
/*****************************************************************************/
static void AppUtil_DumpCsum(ttDevIoctlOffloadStruct * CsumOff){
        /* Print the Checksum offload structure */
        xil_printf("Offload Flags: 0x%08x\n\r",CsumOff->offloadFlags);
}

static void AppUtil_DumpCoalesc(TemacCoalesce * CoalescePtr){

    xil_printf("TxThreshold %d\n\r",CoalescePtr->TxThreshold);
    xil_printf("TxWaitbound %d\n\r",CoalescePtr->TxWaitbound);
    xil_printf("RxThreshold %d\n\r",CoalescePtr->RxThreshold);
    xil_printf("RxWaitbound %d\n\r",CoalescePtr->RxWaitbound);
}

int Util_Csum(unsigned MenuIndex, unsigned TestLoops)
{
    TemacDevData *          Mac;
    ttDevIoctlOffloadStruct CsumOff;

    Mac = &XpsTemac;

    tfIoctlInterface(interfaceHandle, TM_DEV_IOCTL_OFFLOAD_GET, &CsumOff, sizeof(ttDevIoctlOffloadStruct));
    AppUtil_DumpCsum(&CsumOff);

    if (1==AskYesNo("TCP TX Csum Enable ",1))
    {
        if (XAxiEthernet_IsTxFullCsum(&Mac->TemacInst))
        {
            CsumOff.offloadFlags |=
                (  TM_DEVO_IP_CHKSUM_OFFLOAD
                 | TM_DEVO_TCP_CHKSUM_OFFLOAD
                 | TM_DEVO_UDP_CHKSUM_OFFLOAD );
        }
        if (XAxiEthernet_IsTxPartialCsum(&Mac->TemacInst))
        {
            CsumOff.offloadFlags |=
                (  TM_DEVO_TCP_CHKSUM_OFFLOAD
                 | TM_DEVO_UDP_CHKSUM_OFFLOAD );
        }
    }
    else
    {
        CsumOff.offloadFlags &=
               ~(  TM_DEVO_IP_CHKSUM_OFFLOAD
                 | TM_DEVO_TCP_CHKSUM_OFFLOAD
                 | TM_DEVO_UDP_CHKSUM_OFFLOAD );
    }
    if (1==AskYesNo("TCP RX Csum Enable ",1))
    {
        if (XAxiEthernet_IsRxFullCsum(&Mac->TemacInst))
        {
            CsumOff.offloadFlags |=
                (  TM_DEVO_IP_RECV_CHKSUM_VALIDATE
                 | TM_DEVO_TCP_RECV_CHKSUM_VALIDATE
                 | TM_DEVO_UDP_RECV_CHKSUM_VALIDATE );
        }
        if (XAxiEthernet_IsRxPartialCsum(&Mac->TemacInst))
        {
            CsumOff.offloadFlags |= TM_DEVO_PACKET_RECV_CHKSUM_COMPUTE;
        }
    }
    else
    {
        CsumOff.offloadFlags &=
               ~(  TM_DEVO_PACKET_RECV_CHKSUM_COMPUTE
                 | TM_DEVO_IP_RECV_CHKSUM_VALIDATE
                 | TM_DEVO_TCP_RECV_CHKSUM_VALIDATE
                 | TM_DEVO_UDP_RECV_CHKSUM_VALIDATE );
    }

    tfIoctlInterface(interfaceHandle, TM_DEV_IOCTL_OFFLOAD_SET, &CsumOff, sizeof(ttDevIoctlOffloadStruct));
    AppUtil_DumpCsum(&CsumOff);
    return(0);
}

int AppUtil_DumpStats()
{
    struct in_addr Addr;

    tfGetIpAddress(interfaceHandle, &Addr.s_addr, 0);
    xil_printf("IP Address: %s\r\n",inet_ntoa(Addr));

    tfGetNetMask(interfaceHandle, &Addr.s_addr, 0);
    xil_printf("Netmask: %s\r\n",inet_ntoa(Addr));

    //tfGetDefaultGateway(&Addr.s_addr);
    // STRONG_ESL
    tfGetDefaultGateway(interfaceHandle, &Addr.s_addr);
    xil_printf("Default Gateway: %s\r\n",inet_ntoa(Addr));

    xil_printf("Network MTU: %d\r\n",(TcpMss));

    marvell_phy_print_status();

    return(0);
}

unsigned long GetUserAddr(char* Prompt)
{
    unsigned long UserAddress;
    int InputLen;
    char UserAddr[IP_ADDR_LEN];

/* Get the IP address from the user */
    xil_printf("%s - ", Prompt);
    InputLen = GetUserInput("Enter address:", UserAddr, IP_ADDR_LEN);

/* Make sure it is valid, just a basic check */
    UserAddress = inet_addr(UserAddr);
    if(UserAddress == TM_SOCKET_ERROR)
    {
        xil_printf("Error Invalid Address!!\n\r");
        DEBUG_LOG(DEBUG_ERROR,"Address: %s",UserAddr);
        DEBUG_LOG(DEBUG_ERROR,"Hex %08x Address", UserAddress);
    }
    return(UserAddress);
}

int Util_Addr(unsigned MenuIndex, unsigned TestLoops)
{
    unsigned long UserIPAddr, UserNetMask;

    /* Get the IP address from the user */
    UserIPAddr = GetUserAddr("IP Address");
    if(UserIPAddr == TM_SOCKET_ERROR)
    {
        return(UserIPAddr);
    }

    UserNetMask = GetUserAddr("NetMask");
    if(UserNetMask == TM_SOCKET_ERROR)
    {
        return(UserIPAddr);
    }

/*
 * Unconfigure the old address.
 */
    tfUnConfigInterface(interfaceHandle, (unsigned char)0);

/*
 * Set it for the interface
 * Not doing multihome so the address index is 0.
 */
    tfConfigInterface(interfaceHandle,
                    UserIPAddr,
                    UserNetMask,
                    0, 1, (unsigned char)0);

    return(AppUtil_DumpStats());
}

int Util_DestAddr(unsigned MenuIndex, unsigned TestLoops)
{
    unsigned long UserAddress;
    int InputLen;
    char UserAddr[IP_ADDR_LEN];

/* Get the IP address from the user */
    xil_printf("iPerf PC - ");
    InputLen = GetUserInput("Enter address:", UserAddr, IP_ADDR_LEN);

/* Make sure it is valid, just a basic check */
    UserAddress = inet_addr(UserAddr);
    if(UserAddress == TM_SOCKET_ERROR)
    {
        xil_printf("Error Invalid Address!!\n\r");
        return(UserAddress);
    }

    strncpy(HostIp, UserAddr, IP_ADDR_LEN);

    xil_printf("iPerf PC Address set to %s\n\r", UserAddr);

    return (0);
}

int Util_Mtu(unsigned MenuIndex, unsigned TestLoops)
{

    if((9000 != TestLoops) &&
        (1500 != TestLoops) )
    {
        xil_printf("MTU must be either 1500 or 9000\n\r");
        return(-1);
    }

    IfaceMtu = TestLoops;
    tfSetIfMtu(interfaceHandle, IfaceMtu);
    xil_printf("MTU set to %d.\n\r", TestLoops);
    return Util_CloseOpen(MenuIndex, TestLoops);
}


int Util_Coalesc(unsigned MenuIndex, unsigned TestLoops)
{
    TemacCoalesce Coalesce;
    char UserResponse[5];

    tfIoctlInterface(interfaceHandle, TM_TEMAC_IOCTL_COAL_GET, &Coalesce, sizeof(TemacCoalesce));

    /* Update PktRxThreshold */
    xil_printf("Rx packet threshold count [%d]: ", Coalesce.RxThreshold);
    if (GetUserInput(NULL, UserResponse, IN_BUF_LEN - 1) != 0)
    {
        Coalesce.RxThreshold = atoi(UserResponse);
    }

    /* Update PktRxWaitbound */
    xil_printf("Rx packet waitbound timer [%d]: ", Coalesce.RxWaitbound);
    if (GetUserInput(NULL, UserResponse, IN_BUF_LEN - 1) != 0)
    {
        Coalesce.RxWaitbound = atoi(UserResponse);
    }

    tfIoctlInterface(interfaceHandle, TM_TEMAC_IOCTL_COAL_SET, &Coalesce, sizeof(TemacCoalesce));
    AppUtil_DumpCoalesc(&Coalesce);

    return(0);
}

int Util_TxTestFileSize(unsigned MenuIndex, unsigned TestLoops)
{
    xil_printf("Transmit test file size changed from %d MB, to %d MB\n\r", TestFileSizeMB,
                    TestLoops);
    TestFileSizeMB = TestLoops;
    return(0);
}

int Util_Port(unsigned MenuIndex, unsigned TestLoops)
{
    xil_printf("Socket port changed from %d, to %d\n\r", ServerTcpPort,
                    TestLoops);
    ServerTcpPort = TestLoops;
    return(0);
}

int Util_SockSize(unsigned MenuIndex, unsigned TestLoops)
{
    xil_printf("Socket size changed from %d, to %d\n\r", SndBufSize,
                    TestLoops);
    SndBufSize = TestLoops;
    return(0);
}

int Util_CloseOpen(unsigned MenuIndex, unsigned TestLoops)
{
    int Status;

    tfCloseInterface(interfaceHandle);

    Status = AppOpenInterface();
    return(Status);
}

int Util_SetSpeed(unsigned MenuIndex, unsigned TestLoops)
{
    if((1000 == TestLoops) ||
        (100 == TestLoops) ||
          ( 10 == TestLoops) ||
        (  0 == TestLoops)){
        tfIoctlInterface(interfaceHandle, TM_TEMAC_SET_LINK_SPEED, (void*)&TestLoops,
                        sizeof(TestLoops));
        return(0);
    }
    return(-1);
}

/****************************************************************************/
/****************************************************************************
 ****************************************************************************
 *
 * The following code is used to get internal TCP socket info
 *
 ****************************************************************************/
/****************************************************************************/
/****************************************************************************/
#ifdef DEBUG_SOCK

/******************************************************************************
 * If the socket descriptor ID matches the Param1 then dump all known info
 * about that socket. The Treck mechanism will call this function for each
 * TCP socket defined in the system.
 *
 * Parameters:
 *   EntryPtr - TCP control structure for current socket ID
 *   Param1 - This is the socket ID to print info for
 *   Param2 - Not used
 *****************************************************************************/
static int TcpDump(ttNtEntryUPtr EntryPtr, ttUserGenericUnion Param1,
                   ttUserGenericUnion Param2)
{
    ttNtTcpEntry *Entry = (ttNtTcpEntry *)EntryPtr;

    if (Entry->ntTcpSockDesc == Param1.genIntParm)
    {
        /* must use printf since xil_printf doesn't seem to know %u */
        printf("ntTcpBytesInRecvQ    = %u\r\n", Entry->ntTcpBytesInRecvQ);
        printf("ntTcpBytesInSendQ    = %u\r\n", Entry->ntTcpBytesInSendQ);
        printf("ntTcpRecvQSize       = %u\r\n", Entry->ntTcpRecvQSize);
        printf("ntTcpSendQSize       = %u\r\n", Entry->ntTcpSendQSize);
        printf("ntTcpOwnerCount      = %u\r\n", Entry->ntTcpOwnerCount);
        printf("ntTcpFlags           = %u\r\n", Entry->ntTcpFlags);
        printf("ntTcpRto             = %u\r\n", Entry->ntTcpRto);
        printf("ntTcpReXmitCnt       = %u\r\n", Entry->ntTcpReXmitCnt);
        printf("ntTcpSndUna          = %u\r\n", Entry->ntTcpSndUna);
        printf("ntTcpIss             = %u\r\n", Entry->ntTcpIss);
        printf("ntTcpSndNxt          = %u\r\n", Entry->ntTcpSndNxt);
        printf("ntTcpMaxSndNxt       = %u\r\n", Entry->ntTcpMaxSndNxt);
        printf("ntTcpIrs             = %u\r\n", Entry->ntTcpIrs);
        printf("ntTcpSndWL1          = %u\r\n", Entry->ntTcpSndWL1);
        printf("ntTcpSndWL2          = %u\r\n", Entry->ntTcpSndWL2);
        printf("ntTcpMaxSndWnd       = %u\r\n", Entry->ntTcpMaxSndWnd);
        printf("ntTcpRcvNxt          = %u\r\n", Entry->ntTcpRcvNxt);
        printf("ntTcpRcvWnd          = %u\r\n", Entry->ntTcpRcvWnd);
        printf("ntTcpRcvAdv          = %u\r\n", Entry->ntTcpRcvAdv);
        printf("ntTcpCwnd            = %u\r\n", Entry->ntTcpCwnd);
        printf("ntTcpSsthresh        = %u\r\n", Entry->ntTcpSsthresh);
        printf("ntTcpBackLog         = %u\r\n", Entry->ntTcpBackLog);
        printf("ntTcpDupAck          = %u\r\n", Entry->ntTcpDupAck);
        printf("ntTcpAcksAfterRexmit = %u\r\n", Entry->ntTcpAcksAfterRexmit);
        printf("ntTcpState           = %u\r\n", Entry->ntTcpState);
        printf("ntTcpFiller[3]       = %u %u %u\r\n", Entry->ntTcpFiller[0],
               Entry->ntTcpFiller[1], Entry->ntTcpFiller[2]);
    }
    return(TM_ENOERROR);
}

/******************************************************************************
 * Print TCP data structure info for the given TCP socket
 *
 * Parameters:
 *   Sock - Socket ID to print info for
 *****************************************************************************/
static void DebugSocket(int Sock)
{
    ttUserGenericUnion Param;

    Param.genIntParm = Sock;
    tfNetStat(TM_NT_TABLE_TCP, TcpDump, Param, Param);
}

#endif  /* DEBUG */

#endif /* TM_TRECK_NO_KERNEL */

