/* $Id: */
/********************************************************************
 * Copyright(C) 2008 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times.
 * (c) Copyright 2008 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/******************************************************************************
*
* main.c - Entry point for performance test suite
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1     drg  03/31/08 First release
* 2     pdmc 04/11/11 Implement cache invalidation on initialization
*                     as per Xilinx documentation recommendations
*                     Also reduced RAM FS size when using Treck
*                     driver buffers
*
*****************************************************************************/

/***************************** Include Files ********************************/
#include <stdio.h>

#include "trsocket.h"
#ifdef TM_XILINX_USE_AXI4
#include "xtr_axiethernet.h"
#else /* TM_XILINX_USE_AXI4 */
#include "xtr_lltemac.h"
#endif /* TM_XILINX_USE_AXI4 */
#include "menus.h"
#include "debug.h"
#include "cache.h"
#include "xparameters.h"
#include "xintc.h"
#include "app.h"
#include "util_time.h"
//#include "xgpio.h"
#ifdef __PPC__
#include "xexception_l.h"
#else
#include "mb_interface.h"
#endif

#ifdef XPAR_XUARTNS550_NUM_INSTANCES
#include "xuartns550_l.h"
#endif /* XPAR_XUARTNS550_NUM_INSTANCES */
#include "xparameters.h"
#include <string.h>

#ifdef XPAR_XUARTNS550_NUM_INSTANCES
#include "xuartns550_l.h"
#endif

#include <trsocket.h>
#include <trmacro.h>

#ifdef TM_USE_RAM_FS
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <trsocket.h>
#include <trramfs.h>
#endif
/************************** Constant Definitions ****************************/

/**************************** Type Definitions ******************************/


/***************** Macros (Inline Functions) Definitions ********************/

/************************** Function Prototypes *****************************/
static void SetupExceptionSystem(void);
static int Util_DumpStats(unsigned MenuIndex, unsigned TestLoops);
#ifdef TDEBUG
static int Util_DbgTx(unsigned MenuIndex, unsigned TestLoops);
static int Util_DbgRx(unsigned MenuIndex, unsigned TestLoops);
static int Util_DbgApp(unsigned MenuIndex, unsigned TestLoops);
static int Util_DbgMsc(unsigned MenuIndex, unsigned TestLoops);
static int Util_DbgMem(unsigned MenuIndex, unsigned TestLoops);
static int Util_DbgAll(unsigned MenuIndex, unsigned TestLoops);
#endif

/************************** Variable Definitions ****************************/

static XIntc Intc;                   /* Interrupt controller instance */

/* All globals are defined here */

int tdebug = (DEBUG_ERROR);  /* Use the flag in debug.h to define which debugs
                              outputs to send*/

/* Menu display structures */
MenuItem UtilityMenuItems[] = {
    {MENUTYPE_CALL_FUNC, "Setup status dump", NULL, Util_DumpStats},
    {MENUTYPE_CALL_FUNC, "Csum enable/disable", NULL, Util_Csum},
    {MENUTYPE_CALL_FUNC, "Set IP Address", NULL, Util_Addr},
    {MENUTYPE_CALL_FUNC, "Set iPerf IP Address", NULL, Util_DestAddr},
    {MENUTYPE_CALL_VALUE, "Set PHY address (0-31)", NULL, Util_PhyAddr},
    {MENUTYPE_CALL_VALUE, "Set Network MTU (1500 or 9000)", NULL, Util_Mtu},
    {MENUTYPE_CALL_FUNC, "Set Interrupt Coalescing", NULL, Util_Coalesc},
    {MENUTYPE_CALL_VALUE, "Set TX File Size (in MB)", NULL, Util_TxTestFileSize},
    {MENUTYPE_CALL_VALUE, "Set TCP Port", NULL, Util_Port},
    {MENUTYPE_CALL_VALUE, "Set Socket buffer size", NULL, Util_SockSize},
    {MENUTYPE_CALL_FUNC, "Close and open the interface", NULL, Util_CloseOpen},
    {MENUTYPE_CALL_VALUE, "Set speed to 10, 100, 1000Mbps (0 for Auto-Negotiate)", NULL, Util_SetSpeed},
#ifdef TDEBUG
    {MENUTYPE_CALL_FUNC, "Toggle Debug TX", NULL, Util_DbgTx},
    {MENUTYPE_CALL_FUNC, "Toggle Debug RX", NULL, Util_DbgRx},
    {MENUTYPE_CALL_FUNC, "Toggle Debug APP", NULL, Util_DbgApp},
    {MENUTYPE_CALL_FUNC, "Toggle Debug MISC", NULL, Util_DbgMsc},
    {MENUTYPE_CALL_FUNC, "Toggle Debug Memory", NULL, Util_DbgMem},
    {MENUTYPE_CALL_FUNC, "Toggle Debug ALL", NULL, Util_DbgAll},
#endif
};

MenuScreen UtilityMenu = {
    "Settings",
    UtilityMenuItems,
    (sizeof(UtilityMenuItems) / sizeof(MenuItem)),

};
MenuItem MainMenuItems[] = {
        {MENUTYPE_SUB_MENU, "Settings Menu", &UtilityMenu, NULL},
        {MENUTYPE_CALL_TEST, "Treck Loopback Tests", NULL, AppLoopbackTests},
        {MENUTYPE_CALL_TEST, "Simple TCP TX Client", NULL, AppTxTcpMainSimple},
        {MENUTYPE_CALL_TEST, "TCP TX Client with callbacks", NULL, AppTxTcpMainCallbacks},
        {MENUTYPE_CALL_TEST, "TCP RX Server", NULL, AppRxTcpMain},
        {MENUTYPE_CALL_TEST, "UDP TX Client w/o Connection", NULL, AppTxUdpMainWithoutConnection},
        {MENUTYPE_CALL_TEST, "UDP RX Server", NULL, AppRxUdpMain},
#ifdef TM_USE_HTTPD
        {MENUTYPE_CALL_TEST, "Toggle Http Server", NULL, AppHttpServerToggle},
#endif /* TM_USE_HTTPD */
#ifdef TM_USE_FTPD
        {MENUTYPE_CALL_TEST, "Toggle Ftp Server", NULL, AppFtpServerToggle},
#endif /* TM_USE_FTPD */
#ifdef TM_USE_TELNETD
        {MENUTYPE_CALL_TEST, "Toggle Telnet Server", NULL, AppTelnetServerToggle},
#endif /* TM_USE_FTPD */
#ifdef TM_SNMP_VERSION
        {MENUTYPE_CALL_TEST, "Toggle SNMP Agent", NULL, AppSnmpServerToggle},
#endif /* TM_SNMP_VERSION */
};

MenuScreen MainMenu = {
    "Main",
    MainMenuItems,
    (sizeof(MainMenuItems) / sizeof(MenuItem)),

};

//XGpio GpioOutput;
//XGpio GpioInput;

/******************************************************************************
 * Entry point for the performance measurement suite. Runs the main menu
 *****************************************************************************/
int main(void)
{
    int Status;
#ifdef XPAR_XUARTNS550_NUM_INSTANCES
    /* Initialize RS232_Uart_1 - Set baudrate and number of stop bits */
    XUartNs550_SetBaud(XPAR_RS232_UART_1_BASEADDR, XPAR_XUARTNS550_CLOCK_HZ, 115200);
    XUartNs550_SetLineControlReg(XPAR_RS232_UART_1_BASEADDR, XUN_LCR_8_DATA_BITS);
#endif /* XPAR_XUARTNS550_NUM_INSTANCES */
    /* print banner */
    DisplayBanner("L1 TRECK");
#ifdef __PPC__
    xil_printf("PPC440 core         : %d MHz\r\n", XPAR_CPU_PPC440_CORE_CLOCK_FREQ_HZ/1000000);
#ifndef TM_XILINX_USE_AXI4
    xil_printf("PLB bus             : %d MHz\r\n\r\n", XPAR_CPU_PPC440_MPLB_FREQ_HZ/1000000);
#endif /* !TM_XILINX_USE_AXI4 */
    /* Make sure this program is linked to run in external memory */
    if (((Xuint32)main >= XPAR_XPS_BRAM_IF_CNTLR_1_BASEADDR) && ((Xuint32)main <= XPAR_XPS_BRAM_IF_CNTLR_1_HIGHADDR))
    {
        xil_printf("Fatal error: This program may not be run from BRAM\r\n");
        return(0);
    }
#else /* __PPC__ */
    xil_printf("MicroBlaze core     : %d MHz\r\n", XPAR_CPU_CORE_CLOCK_FREQ_HZ/1000000);
#ifndef TM_XILINX_USE_AXI4
    xil_printf("PLB bus             : %d MHz\r\n\r\n", XPAR_CPU_IPLB_FREQ_HZ/1000000);
#endif /* !TM_XILINX_USE_AXI4 */
#endif /* __PPC__ */


#ifdef TM_ERROR_CHECKING
    xil_printf("Treck Error Check is: on\r\n");
#endif
#ifdef TDEBUG
    xil_printf("DEBUG_LOG         is: on\r\n");
#endif
#ifdef tm_kernel_set_critical
    xil_printf("set critical defined\r\n");
#endif
    xil_printf("Simple Heap Size    : %d Bytes\r\n\r\n", TM_SHEAP_SIZE);

    /* Perform rest of init and start the menu */
    SetupCache();            /* Data/Instruction cache setup */
    SetupExceptionSystem();  /* Setup exception table and interrupt controller */

    /* Initialize the timers so calls to sleep will work. */
    TimerInit();
    xil_printf("Initializing MAC and waiting for link ...\r\n");

/* Setup Ethernet and DMA when the device opens */

    Status = AppInitTreck();
    if (Status != XST_SUCCESS)
    {
        /* The Network initialization failed. There is no way to continue */
        xil_printf("main: failed to initialize Treck. Status %d", Status);
        exit(-1);
    }

    Status = AppAddInterface();
    if (Status != XST_SUCCESS)
    {
        xil_printf("main: failed to add an interface. Status %d", Status);
        exit(-1);
    }
    Status = AppOpenInterface();
    if (Status != XST_SUCCESS)
    {
        xil_printf("main: failed to open an interface. Status %d", Status);
        exit(-1);
    }
#ifdef TM_USE_RAM_FS
/* Init ramfs file system */
#ifndef XLNX_USE_DRV_BUF_MEM
/* We don't have room for as big of a RAM FS */
     Status = tfRamfsInit(32768);
#else /* !XLNX_USE_DRV_BUF_MEM */
     Status = tfRamfsInit(65535);
#endif /* !XLNX_USE_DRV_BUF_MEM */
     if (Status != TM_ENOERROR){
         xil_printf("main: failed to initialize RAMFS %s\n\r", tfStrError(tfRamfsErrNo()));
     }
#endif /* TM_USE_RAM_FS */

/* Init GPIO */
#ifdef XPAR_LEDS_4BITS_DEVICE_ID
     if(XGpio_Initialize(&GpioOutput, XPAR_LEDS_4BITS_DEVICE_ID) != XST_SUCCESS)
     {
         xil_printf("main: XGpio_Initialize failed for output\n\r");
     }
#endif /* XPAR_LEDS_4BITS_DEVICE_ID */
#ifdef XPAR_LEDS_8BIT_DEVICE_ID
     if(XGpio_Initialize(&GpioOutput, XPAR_LEDS_8BIT_DEVICE_ID) != XST_SUCCESS)
     {
         xil_printf("main: XGpio_Initialize failed for output\n\r");
     }
#endif /*XPAR_LEDS_8BIT_DEVICE_ID */
#ifdef XPAR_DIP_SWITCHES_8BIT_DEVICE_ID
     if(XGpio_Initialize(&GpioInput, XPAR_DIP_SWITCHES_8BIT_DEVICE_ID) != XST_SUCCESS)
     {
         xil_printf("main: XGpio_Initialize failed for input\n\r");
     }
#endif /* XPAR_DIP_SWITCHES_8BIT_DEVICE_ID */

#ifdef TM_USE_HTTPD
/* Init Http Server */
     Status = tfHttpdUserInit(5);
     if(Status != TM_ENOERROR){
        xil_printf("main: http webserver: %s\n\r", tfStrError(Status));
     }
#endif /* TM_USE_HTTPD */

/* Invoke main menu */
    DisplayMenu(&MainMenu);
    return(0);
}

#ifdef TDEBUG
/*
 * Dynamically change the debug outputs while the app is running.
 */
static int Util_Dbg(unsigned DbgOutput){
    if(DbgOutput == (tdebug & DbgOutput)){
        printf("Debug Disabled\n");
        tdebug &= ~DbgOutput;
    }
    else{
        printf("Debug Enabled\n");
        tdebug |= DbgOutput;
    }

    return(0);
}

static int Util_DbgTx(unsigned MenuIndex, unsigned TestLoops){
    return(Util_Dbg(DEBUG_TX));
}

static int Util_DbgRx(unsigned MenuIndex, unsigned TestLoops){
    return(Util_Dbg(DEBUG_RX));
}

static int Util_DbgApp(unsigned MenuIndex, unsigned TestLoops){
    return(Util_Dbg(DEBUG_APP));
}

static int Util_DbgMsc(unsigned MenuIndex, unsigned TestLoops){
    return(Util_Dbg(DEBUG_MISC));
}

static int Util_DbgMem(unsigned MenuIndex, unsigned TestLoops){
    return(Util_Dbg(DEBUG_MEM));
}

static int Util_DbgAll(unsigned MenuIndex, unsigned TestLoops){
    Util_Dbg(DEBUG_ALL);

    /* Always output ERROR debugs! */
    tdebug |= DEBUG_ERROR;
    return(0);
}
#endif /* TDEBUG */


static int Util_DumpStats(unsigned MenuIndex, unsigned TestLoops){
    int Status;

    Status = AppUtil_DumpStats();
    Status |= TemacUtil_DumpStats();
    return(Status);
}


/******************************************************************************
 * Perform any necessary setups to the exception system including the intc
 *****************************************************************************/
#ifdef XPAR_INTERRUPT_CNTLR_DEVICE_ID
#ifndef XPAR_XPS_INTC_0_DEVICE_ID
#define XPAR_XPS_INTC_0_DEVICE_ID XPAR_INTERRUPT_CNTLR_DEVICE_ID
#endif /* XPAR_XPS_INTC_0_DEVICE_ID */
#endif /* XPAR_INTERRUPT_CNTLR_DEVICE_ID  */

#ifdef XPAR_INTC_0_DEVICE_ID
#ifndef XPAR_XPS_INTC_0_DEVICE_ID
#define XPAR_XPS_INTC_0_DEVICE_ID XPAR_INTC_0_DEVICE_ID
#endif /* XPAR_XPS_INTC_0_DEVICE_ID */
#endif /* XPAR_INTC_0_DEVICE_ID */

static void SetupExceptionSystem(void)
{
    XStatus Status;

    /* Initialize and start the interrupt controller instance */
    Status = XIntc_Initialize(&Intc, XPAR_XPS_INTC_0_DEVICE_ID);
    if (Status != XST_SUCCESS)
    {
        xil_printf("Failed to initialize interrupt controller, code %d\n", Status);
        return;
    }

    Status = XIntc_Start(&Intc, XIN_REAL_MODE);
    if (Status != XST_SUCCESS)
    {
        xil_printf("Failed to start interrupt controller, code %d\n", Status);
        return;
    }

#ifdef __PPC__
    /* Setup exception table and register the interrupt controller handler */
    XExc_Init();
    XExc_RegisterHandler(XEXC_ID_NON_CRITICAL_INT,
                         (XExceptionHandler)XIntc_InterruptHandler, &Intc);

    /* Enable interrupts from interrupt controller to processor */
    XExc_mEnableExceptions(XEXC_NON_CRITICAL);

    /* Disable decrementer interrupts */
    XTime_DECDisableInterrupt();
#else

    /*
     * Connect the interrupt controller interrupt handler to the hardware
     * interrupt handling logic in the microblaze processor.
     */
    microblaze_register_handler((XInterruptHandler)
                XIntc_InterruptHandler,
                &Intc);

    /*
     * Enable interrupts in the Microblaze
     */

    microblaze_enable_interrupts();
#endif

}

/* This is a lame attempt to keep the Intc Instance pointer from being a global.
 */
int Setup_Interrupt(u8 Id, XInterruptHandler Handler, void * CallBackRef)
{
    int Status;

    /* Install callbacks */
    Status = XIntc_Connect(&Intc, Id, (XInterruptHandler) Handler, CallBackRef);
    if (Status != XST_SUCCESS) {
        return(Status);
    }

    XIntc_Enable(&Intc, Id);
    return(XST_SUCCESS);
}


