/* $Id: */
/********************************************************************
 * Copyright(C) 2003 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times. (c) Copyright 2008 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/******************************************************************************
*
* tcp_loopback_test.c
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1
*
*****************************************************************************/

/***************************** Include Files ********************************/
#include <stdio.h>
#include "loopback.h"

/************************** Constant Definitions ****************************/

/**************************** Type Definitions ******************************/

/***************** Macros (Inline Functions) Definitions ********************/

/************************** Variable Definitions ****************************/
char bufferArray[TM_BUF_ARRAY_SIZE];

/************************** Function Prototypes *****************************/


void treck_test_tcp_loopback(void)
{
    int                 errorCode;
    int                 failed;
    struct sockaddr_in  addr;
    int                 lsd; /* listening socket */
    int                 csd;  /* connecting socket */
    int                 asd; /* accept socket */
    int                 len;
    int                 recvdLength;
    int                 sentLength;

    failed = 0;
    lsd = TM_SOCKET_ERROR;
    csd = TM_SOCKET_ERROR;
    asd = TM_SOCKET_ERROR;
    recvdLength = TM_SOCKET_ERROR;
    sentLength = TM_SOCKET_ERROR;
/* Amount of data to send, and receive */
    len = sizeof(bufferArray);

/* Open the listening TCP socket */
    lsd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);

    if ( (lsd == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed socket call */
        errorCode = tfGetSocketError(lsd);
        xil_printf( "socket failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
/* Bind the TCP listening socket to a well known TCP port */
        addr.sin_family = PF_INET;
        addr.sin_port = TM_PORT_LOOPBACK_TEST;
        addr.sin_addr.s_addr = 0;
        errorCode = bind( lsd,
                          (struct sockaddr TM_FAR *)&addr,
                          sizeof(struct sockaddr_in));
    }
    if ( (errorCode == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed bind call */
        errorCode = tfGetSocketError(lsd);
        xil_printf( "bind failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
/* Listen for incoming connections */
        errorCode = listen(lsd, 1);
    }
    if ( (errorCode == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed listen call */
        errorCode = tfGetSocketError(lsd);
        xil_printf( "listen failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
/* Open the connecting TCP socket */
        csd = socket(PF_INET, SOCK_STREAM, IP_PROTOTCP);
    }
    if ( (csd == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed socket call */
        errorCode = tfGetSocketError(csd);
        xil_printf( "socket failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
/* connect to our own listening socket */
        addr.sin_addr.s_addr = inet_addr("127.0.0.1");
        errorCode = connect ( csd,
                              (struct sockaddr TM_FAR *)&addr,
                              sizeof(struct sockaddr_in));
    }
    if ( (errorCode == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed bind call */
        errorCode = tfGetSocketError(csd);
        xil_printf( "connect failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
        xil_printf("Connected to server socket\r\n");
        asd = accept(lsd,
                     (struct sockaddr TM_FAR *)0,
                     (int TM_FAR             *)0);
    }
    if ( (asd == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed bind call */
        errorCode = tfGetSocketError(lsd);
        xil_printf( "accept failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
/* Send some loop back data to the accept socket */
        sentLength = send( csd,
                           bufferArray,
                           len,
                           0);
    }
    if ( (sentLength == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed send call */
        errorCode = tfGetSocketError(csd);
        xil_printf( "send failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
        xil_printf("%d sent successfully\r\n", sentLength);
/* Receive loop back data from our accept socket */
        recvdLength = recv( asd,
                            bufferArray,
                            len,
                            0);
    }
    if ( (recvdLength == TM_SOCKET_ERROR) && (failed == 0) )
    {
/* Retrieve the socket error for failed recv call */
        errorCode = tfGetSocketError(asd);
        xil_printf( "recv failed, '%s'\r\n",
                 tfStrError(errorCode));
        failed = 1;
    }
    if (failed == 0)
    {
        xil_printf("%d received succesfully\r\n", recvdLength);
    }
    if (csd != TM_SOCKET_ERROR)
    {
/* All done. Close the connecting socket */
        (void)tfClose(csd);
    }
    if (asd != TM_SOCKET_ERROR)
    {
        (void)tfClose(asd);
    }
    if (lsd != TM_SOCKET_ERROR)
    {
        (void)tfClose(lsd);
    }
    if (failed == 0)
    {
        xil_printf("TCP LOOP BACK Test success\r\n");
    }
}
