/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: CGI sample
 * Filename: txcgi.c
 * Author: Bryan Wang
 * Date Created: 9/30/2003
 *
 * Modification History
 * $Revision: 1.4 $
 * $Date: 2009/09/02 12:51:07EDT $
 * $Author: tcarney $
 * $ProjectName: /home/mks/proj/trksave.pj $
 * $ProjectRevision: Last Checkpoint: 4.1 $
 */

#ifdef _WIN32
/* compile and run on Win32 platform */
/* to avoid an warning in MS VC++ header file */
struct _RPC_ASYNC_STATE;
#include <windows.h>
#endif /* _WIN32 */

#include <stdio.h>
#include <trsocket.h>
#include <trmacro.h>
#include <string.h>
#include "xparameters.h"

#if  defined(XPAR_LEDS_4BITS_DEVICE_ID) || defined(XPAR_LEDS_8BIT_DEVICE_ID) || defined(XPAR_DIP_SWITCHES_8BIT_DEVICE_ID)
#include "xgpio.h"
#endif

#include "xstatus.h"

#ifdef _WIN32
#define  TM_BODY_BUFFER_LEN  1460
#else /* !_WIN32 */
#define  TM_BODY_BUFFER_LEN  128
#endif /* _WIN32 */

#define  TM_HEADER_CNT       3
#define  TM_RESPONSE_BODY    "Low-Level API works!"

#define DATA_DIRECTION_OUTPUT 0x0
#define DATA_DIRECTION_INPUT  0xFFFFFFFF
#define CHANNEL 1
#define LED_0 0x1
#define LED_1 0x2
#define LED_2 0x4
#define LED_3 0x8
#define LED_4 0x10
#define LED_5 0x20
#define LED_6 0x40
#define LED_7 0x80

#ifdef TM_USE_HTTPD_CGI

#ifdef _WIN32
TM_PROTO_EXTERN DWORD WINAPI  tfxCgiHandlingTask(void * pParam);
#else /* ! _WIN32 */
TM_PROTO_EXTERN int tfxCgiHandlingTask(void * pParam);
#endif /* ! _WIN32 */

void myCgiHandler(
    ttHttpdUserConHandle conHandle,
    char TM_FAR *pathPtr,
    char TM_FAR *argNames[],
    char TM_FAR *argValues[],
    int          argCnt);

char * tfxStrToHtmlTableHeader(char * string, char * row);

#ifdef TM_USE_NETSTAT
int tfxNetstatCgi(
    ttHttpdUserConHandle    conHandle,
    ttNtTableId             tableId);

int tfxNtEntryCBCgi(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2);

static char * tlNtTableName[] =
{
    "ARP Table",
    "Routing Table",
    "TCP Socket Table",
    "UDP Socket Table",
#ifdef TM_6_USE_MIP_CN
    "Mipv6 bindings",
#endif /* TM_6_USE_MIP_CN */
#ifdef TM_USE_IKE
    "IKE SADB",
#endif /* TM_USE_IKE */
#ifdef TM_USE_IKE_POLICIES
    "IKE SPD",
#endif /* TM_USE_IKE_POLICIES */
#ifdef TM_USE_IPSEC
    "IPsec SADB",
    "IPsec SPD",
#endif /* TM_USE_IPSEC */
    "Device Table"
};
#endif /* TM_USE_NETSTAT */

#if (defined (XPAR_LEDS_8BIT_DEVICE_ID) || defined (XPAR_LEDS_4BITS_DEVICE_ID))
extern XGpio GpioOutput;
#endif

#ifdef XPAR_DIP_SWITCHES_8BIT_DEVICE_ID
extern XGpio GpioInput;
#endif

char * tfxStrToHtmlRow(char * string, char * row);
void tfxCgiIOReport(ttHttpdUserConHandle conHandle, int argCnt, char TM_FAR *argNames[]);

int tfxCgiSendNetstatTable(ttHttpdUserConHandle conHandle, int tableId);
int tfxCgiSendHello(ttHttpdUserConHandle conHandle, const char* name);
int tfxCgLowLevelApi(ttHttpdUserConHandle conHandle);
int tfxCgiSendNotFound(ttHttpdUserConHandle conHandle, const char* path);
int tfxCgiAuthConfig(
    ttHttpdUserConHandle    conHandle,
    ttHttpdUserSrvHandle    srvHandle,
    char *                  objectNamePtr,
    char *                  realmPtr,
    char *                  usernamePtr,
    char *                  passwordPtr,
    char *                  actionPtr);

/* Insist on sending a buffer through a loop that keeps calling
 * tfxHttpdUserSendLoop until sent or error
 */
TM_PROTO_EXTERN int tfxHttpdUserSendLoop(
    ttHttpdUserConHandle   conHandle,
    const char TM_FAR *     bufferPtr,
    int                     bufferLen,
    int TM_FAR *            errorCodePtr);

/* Interface function to be implemented by the user,
 * Web server calls this function when the following events happens
 */
TM_PROTO_EXTERN void tfxHttpdEventHandler
        (ttHttpdUserConHandle conHandle, ttHttpdUserEvent event)
{
#ifdef _WIN32
    unsigned long           tid;
    FILE *                  filePtr;
    ttUserGenericUnionPtr   genUnionPtr;
#else /* !_WIN32 */
    ttUserGenericUnion      genUnion;
#endif /* !_WIN32 */
    int                     bufferLen;
    ttUserCharPtr           buffer;

/*  We could get the connection information and log it */
    ttHttpdUserConEntryPtr  conEntryPtr;

    conEntryPtr = tfHttpdUserGetConInfo(conHandle);
    if (conEntryPtr != (ttHttpdUserConEntryPtr)0)
    {
/* log the information in the con entry*/
    }

    switch(event)
    {
        case TM_HTTPD_EVENT_CON_OPENED:
/* log the information in the con entry*/
            break;
        case TM_HTTPD_EVENT_CON_CLOSE:
/* log the information in the con entry*/
            break;
        case TM_HTTPD_EVENT_METHOD_LINE_RECV:
/* log the information in the con entry*/
            break;
        case TM_HTTPD_EVENT_HEADERS_RECV:
/* log the information in the con entry*/
            break;
        case TM_HTTPD_EVENT_BODY_RECV:
            buffer = tm_malloc(TM_BODY_BUFFER_LEN);
            if (buffer != (ttUserCharPtr)0)
            {
                do
                {
                    bufferLen = TM_BODY_BUFFER_LEN;
#ifdef _WIN32
/* Try to save the message body in a file */
                    tfHttpdUserGetRequestBody(conHandle, buffer,
                                              &bufferLen , 0);
                    if (bufferLen)
                    {
                        filePtr = fopen("request_body.html", "a+");
                        if (filePtr)
                        {
                            fwrite(buffer, bufferLen, 1, filePtr);
                            fclose(filePtr);
                        }
                    }
#else /* !_WIN32 */
/* Do not copy the body message. Let the HTTPD server process CGI Args in body */
                tfHttpdUserGetRequestBody(conHandle, buffer, &bufferLen ,
                                          TM_HTTPD_NO_COPY);
#endif /* _WIN32 */
                }
                while (bufferLen);
                tm_free(buffer);
            }
/* log the information in the con entry*/
            break;

        case TM_HTTPD_EVENT_ALL_RECV:
            break;
#ifdef TM_USE_HTTPD_CGI
        case TM_HTTPD_EVENT_SEND_READY:
#ifdef _WIN32
            genUnionPtr = tm_malloc(sizeof(ttUserGenericUnion));
            if (genUnionPtr != (ttUserGenericUnionPtr)0)
            {
                genUnionPtr->gen32bitParm = conHandle;
/* Create a task to handle this CGI request */
                CreateThread(NULL, 0, tfxCgiHandlingTask, (void *)genUnionPtr, 0, &tid);
            }
#else /* ! _WIN32 */
            genUnion.gen32bitParm = conHandle;
            (void) tfxCgiHandlingTask((void *)&genUnion);
#endif /* ! _WIN32 */
            break;
#endif /* TM_USE_HTTPD_CGI */
        default:
            break;
    }
}

#ifdef TM_USE_HTTPD_CGI
#ifdef _WIN32
DWORD WINAPI  tfxCgiHandlingTask(void * pParam)
#else /* ! _WIN32 */
int tfxCgiHandlingTask(void * pParam)
#endif /* ! _WIN32 */
{
    ttHttpdUserConHandle    conHandle;
    ttHttpdUserConEntryPtr  conEntryPtr;

    conHandle = ((ttUserGenericUnionPtr)pParam)->gen32bitParm;
#ifdef _WIN32
    tm_free(pParam);
#endif /* _WIN32 */
    conEntryPtr = tfHttpdUserGetConInfo(conHandle);

    if (conEntryPtr != (ttHttpdUserConEntryPtr)0)
    {
        myCgiHandler(conHandle,
                     conEntryPtr->uconCgiPathPtr,
                     conEntryPtr->uconCgiArgNames,
                     conEntryPtr->uconCgiArgValues,
                     conEntryPtr->uconCgiArgCnt);
    }

    return 0;
}


void myCgiHandler(
    ttHttpdUserConHandle conHandle,
    char TM_FAR *pathPtr,
    char TM_FAR *argNames[],
    char TM_FAR *argValues[],
    int          argCnt)
{
    int errorCode;

    ttHttpdUserConEntryPtr  conEntryPtr;

    conEntryPtr = tfHttpdUserGetConInfo(conHandle);

/* Just to avoid compiler warnings of unreferenced arguments */
    errorCode = sizeof(argNames) * 0;

    if (strcmp(pathPtr, (const char *)"/cgi-bin/low") == 0 )
    {
        tfxCgLowLevelApi(conHandle);
    }

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/hello") == 0 )
    {
        if (argCnt > 0 && (strlen(argValues[0]) != 0))
        {
            tfxCgiSendHello(conHandle, argValues[0]);
        }
        else
        {
            tfxCgiSendHello(conHandle, "Friend");
        }

    }

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/authconfig") == 0 )
    {
        tfxCgiAuthConfig(conHandle,
                        conEntryPtr->uconSrvHandle,
                        argValues[0],
                        argValues[1],
                        argValues[2],
                        argValues[3],
                        argValues[4]);
    }
#ifdef TM_USE_NETSTAT
    else if (strcmp(pathPtr, (const char *)"/cgi-bin/tcp") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_TCP);
    }

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/udp") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_UDP);
    }

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/arp") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_ARP);
    }

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/route") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_RTE);
    }

#ifdef TM_6_USE_MIP_CN
    else if (strcmp(pathPtr, (const char *)"/cgi-bin/binding") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_BINDING);
    }
#endif /* TM_6_USE_MIP_CN */
    else if (strcmp(pathPtr, (const char *)"/cgi-bin/device") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_DEVICE);
    }
#endif /* TM_USE_NETSTAT */

#ifdef TM_USE_IPSEC
    else if (strcmp(pathPtr, (const char *)"/cgi-bin/sadb") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_SADB);
    }

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/spd") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_IPSEC_SPD);
    }
#endif /* TM_USE_IPSEC */
#ifdef TM_USE_IKE
    else if (strcmp(pathPtr, (const char *)"/cgi-bin/ikesadb") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_ISASTATE_ENTRY);
    }
#ifdef TM_USE_IKE_POLICIES

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/ikespd") == 0 )
    {
        tfxCgiSendNetstatTable(conHandle, TM_NT_TABLE_ISASTATE_SPD);
    }
#endif /* TM_USE_IKE_POLICIES */
#endif /* TM_USE_IKE */
#ifdef TM_USE_HTTPD_SSI
    else if (strcmp(pathPtr, (const char *)"/cgi-bin/ssi") == 0 )
    {
        if (conEntryPtr != (ttHttpdUserConEntryPtr)0)
        {
/* Store the pointer to a const string in the user connection entry */
            conEntryPtr->uconUserVariable.genVoidParmPtr = "SSI is Cool!";
        }

        errorCode = tfHttpdUserSendFile(conHandle, "testssi.shtm", 0);
        if (errorCode != TM_ENOERROR)
        {
            errorCode = errorCode;/* debugging trap */
        }
    }
#endif /* TM_USE_HTTPD_SSI */

    else if (strcmp(pathPtr, (const char *)"/cgi-bin/io") == 0 )
    {
        tfxCgiIOReport(conHandle, argCnt, argNames);
    }
    else
    {
        tfxCgiSendNotFound(conHandle, pathPtr);
    }
}
void tfxCgiIOReport(ttHttpdUserConHandle conHandle, int argCnt, char TM_FAR *argNames[])
{
#if (   defined(XPAR_LEDS_8BIT_DEVICE_ID)  \
     || defined(XPAR_LEDS_4BITS_DEVICE_ID) \
     || defined(XPAR_DIP_SWITCHES_8BIT_DEVICE_ID) )
    int i;
    int bitIsolator;
#endif /* XPAR_LEDS_8BIT_DEVICE_ID ... */
#if (   defined(XPAR_LEDS_8BIT_DEVICE_ID)  \
     || defined(XPAR_LEDS_4BITS_DEVICE_ID) )
    int leds;
#endif /* XPAR_LEDS_8BIT_DEVICE_ID ... */
    int errorCode;
#ifdef XPAR_DIP_SWITCHES_8BIT_DEVICE_ID
    Xuint32 dipSwitches;
#endif /* XPAR_DIP_SWITCHES_8BIT_DEVICE_ID */

#if (   !(defined(XPAR_LEDS_8BIT_DEVICE_ID)) \
     && !(defined(XPAR_LEDS_4BITS_DEVICE_ID)) )
    TM_UNREF_IN_ARG(argCnt);
    TM_UNREF_IN_ARG(argNames);
#endif /* XPAR_LEDS_8BIT_DEVICE_ID && XPAR_LEDS_4BITS_DEVICE_ID */

#if (   defined(XPAR_LEDS_8BIT_DEVICE_ID)  \
     || defined(XPAR_LEDS_4BITS_DEVICE_ID) )
    leds = 0;
#endif /* XPAR_LEDS_8BIT_DEVICE_ID ... */
#ifdef XPAR_DIP_SWITCHES_8BIT_DEVICE_ID
    dipSwitches = 0;
#endif /* XPAR_DIP_SWITCHES_8BIT_DEVICE_ID */

    errorCode = TM_ENOERROR;

/* set data direction.  Notice initialization was done in shell_main*/
#if (defined(XPAR_LEDS_8BIT_DEVICE_ID) || defined(XPAR_LEDS_4BITS_DEVICE_ID))
    XGpio_SetDataDirection(&GpioOutput, CHANNEL, DATA_DIRECTION_OUTPUT);
#endif /* XPAR_LEDS_8BIT_DEVICE_ID */
#ifdef XPAR_DIP_SWITCHES_8BIT_DEVICE_ID
    XGpio_SetDataDirection(&GpioInput, CHANNEL, DATA_DIRECTION_INPUT);
#endif /* XPAR_DIP_SWITCHES_8BIT_DEVICE_ID */
/* Send beginning of the page */
    tfxHttpdUserSendLoop(
                conHandle,
                "<html>\n"
                "<head>\n"
                "<title>TreckDemo: IO Report</title>\n"
                "</head>\n"
                "<body>\n",
                -1,
                &errorCode);
#ifdef XPAR_LEDS_8BIT_DEVICE_ID
    tfxHttpdUserSendLoop(
                conHandle,
                "<font face=\"Arial, Arial, Helvetica\">\n"
                "<font color=\"#2F9F89\" size=\"4\">\n"
                "<center>LED Report</center></font>\n",
                -1,
                &errorCode);
    if (argCnt > 0)
    {
/* Loop through the argNames, setting the bit for each LED that comes in*/
        for(i = 0; i < argCnt; i++)
        {
            if(strcmp(argNames[i], (const char *)"led0") == 0 )
            {
                leds |= LED_0;
            }
            else if(strcmp(argNames[i], (const char *)"led1") == 0 )
            {
                leds |= LED_1;
            }
            else if(strcmp(argNames[i], (const char *)"led2") == 0 )
            {
                leds |= LED_2;
            }
            else if(strcmp(argNames[i], (const char *)"led3") == 0 )
            {
                leds |= LED_3;
            }
            else if(strcmp(argNames[i], (const char *)"led4") == 0 )
            {
                leds |= LED_4;
            }
            else if(strcmp(argNames[i], (const char *)"led5") == 0 )
            {
                leds |= LED_5;
            }
            else if(strcmp(argNames[i], (const char *)"led6") == 0 )
            {
                leds |= LED_6;
            }
            else if(strcmp(argNames[i], (const char *)"led7") == 0 )
            {
                leds |= LED_7;
            }
        }
/* write leds to the physical led reg */
        XGpio_DiscreteWrite(&GpioOutput, CHANNEL, leds);
    }
/* Label the led images in the table */
    tfxHttpdUserSendLoop(conHandle,
                         "<table cellspacing=3 align=\"center\">\n"
                         "<tr>\n"
                         "<td align=\"center\">led0</td>\n"
                         "<td align=\"center\">led1</td>\n"
                         "<td align=\"center\">led2</td>\n"
                         "<td align=\"center\">led3</td>\n"
                         "<td align=\"center\">led4</td>\n"
                         "<td align=\"center\">led5</td>\n"
                         "<td align=\"center\">led6</td>\n"
                         "<td align=\"center\">led7</td>\n"
                         "</tr>\n"
                         "<tr>\n",
                         -1,
                         &errorCode);
/* use leds to create an image entry for each led in the table */
    for(i = 0; i < 8; i++)
    {
        bitIsolator = (1 << i);
        if((leds & bitIsolator) != 0)
        {
            tfxHttpdUserSendLoop(conHandle,
                             "<td>\n"
                             "<img border=\"0\" src=\"/images/green.jpg\"\n"
                             "</td>\n",
                             -1,
                             &errorCode);
        }
        else
        {
            tfxHttpdUserSendLoop(conHandle,
                             "<td>\n"
                             "<img border=\"0\" src=\"/images/grey.png\"\n"
                             "</td>\n",
                             -1,
                             &errorCode);
        }
    }
/* close the table and start the dip switch table in html */
    tfxHttpdUserSendLoop(conHandle,
                         "</tr>\n"
                         "</table>\n",
                          -1,
                         &errorCode);
#endif /* XPAR_LEDS_8BIT_DEVICE_ID */
#ifdef XPAR_LEDS_4BITS_DEVICE_ID

    tfxHttpdUserSendLoop(
                conHandle,
                "<font face=\"Arial, Arial, Helvetica\">\n"
                "<font color=\"#2F9F89\" size=\"4\">\n"
                "<center>LED Report</center></font>\n",
                -1,
                &errorCode);
    if (argCnt > 0)
    {
/* Loop through the argNames, setting the bit for each LED that comes in*/
        for(i = 0; i < argCnt; i++)
        {
            if(strcmp(argNames[i], (const char *)"led0") == 0 )
            {
                leds |= LED_0;
            }
            else if(strcmp(argNames[i], (const char *)"led1") == 0 )
            {
                leds |= LED_1;
            }
            else if(strcmp(argNames[i], (const char *)"led2") == 0 )
            {
                leds |= LED_2;
            }
            else if(strcmp(argNames[i], (const char *)"led3") == 0 )
            {
                leds |= LED_3;
            }
        }
/* write leds to the physical led reg */
        XGpio_DiscreteWrite(&GpioOutput, CHANNEL, leds);
    }
/* Label the led images in the table */
    tfxHttpdUserSendLoop(conHandle,
                         "<table cellspacing=3 align=\"center\">\n"
                         "<tr>\n"
                         "<td align=\"center\">led0</td>\n"
                         "<td align=\"center\">led1</td>\n"
                         "<td align=\"center\">led2</td>\n"
                         "<td align=\"center\">led3</td>\n"
                         "</tr>\n"
                         "<tr>\n",
                         -1,
                         &errorCode);
/* use leds to create an image entry for each led in the table */
    for(i = 0; i < 4; i++)
    {
        bitIsolator = (1 << i);
        if((leds & bitIsolator) != 0)
        {
            tfxHttpdUserSendLoop(conHandle,
                             "<td>\n"
                             "<img border=\"0\" src=\"/images/green.jpg\"\n"
                             "</td>\n",
                             -1,
                             &errorCode);
        }
        else
        {
            tfxHttpdUserSendLoop(conHandle,
                             "<td>\n"
                             "<img border=\"0\" src=\"/images/grey.png\"\n"
                             "</td>\n",
                             -1,
                             &errorCode);
        }
    }
/* close the table and start the dip switch table in html */
    tfxHttpdUserSendLoop(conHandle,
                         "</tr>\n"
                         "</table>\n",
                          -1,
                         &errorCode);


#endif /* XPAR_LEDS_4BITS_DEVICE_ID */
#ifdef XPAR_DIP_SWITCHES_8BIT_DEVICE_ID
    tfxHttpdUserSendLoop(conHandle,
                         "<font color=\"#2F9F89\" size=\"4\"><br>\n"
                         "<center>DIP Switch Report</center></font>\n"
                         "<table cellspacing=3 align=\"center\">\n"
                         "<tr>\n"
                         "<td align=\"center\">sw1</td>\n"
                         "<td align=\"center\">sw2</td>\n"
                         "<td align=\"center\">sw3</td>\n"
                         "<td align=\"center\">sw4</td>\n"
                         "<td align=\"center\">sw5</td>\n"
                         "<td align=\"center\">sw6</td>\n"
                         "<td align=\"center\">sw7</td>\n"
                         "<td align=\"center\">sw8</td>\n"
                         "</tr>\n"
                         "<tr>\n",
                         -1,
                         &errorCode);
/* read the value of the dip switches from the board*/
    dipSwitches = XGpio_DiscreteRead(&GpioInput, CHANNEL);
/*
 * use dipSwitches to create an image entry for each switch in the table.
 * notice i starts at 7, this is so the images and lables match
 * the way the switches look on the board
 */
    for(i = 7; i >= 0; i--)
    {
        bitIsolator = (1 << i);
        if((dipSwitches & bitIsolator) != 0)
        {
            tfxHttpdUserSendLoop(conHandle,
                             "<td>\n"
                             "<img border=\"0\" src=\"/images/on.png\"\n"
                             "</td>\n",
                             -1,
                             &errorCode);
        }
        else
        {
            tfxHttpdUserSendLoop(conHandle,
                             "<td>\n"
                             "<img border=\"0\" src=\"/images/off.png\"\n"
                             "</td>\n",
                             -1,
                             &errorCode);
        }
    }
/* close the table and dangling html tags */
    tfxHttpdUserSendLoop(conHandle,
                         "</tr>\n"
                         "</table>\n",
                         -1,
                         &errorCode);
#endif /* XPAR_DIP_SWITCHES_8BIT_DEVICE_ID */

    tfxHttpdUserSendLoop(conHandle,
                         "</font>\n"
                         "</body>\n"
                         "</html>\n",
                         -1,
                         &errorCode);
/* signal the end of the transmission */
    tfHttpdUserSendBuffer(conHandle,
                          0,
                          0,
                          (TM_HTTPD_END | TM_BLOCKING_ON),
                          &errorCode);
    return;
}
#ifdef TM_USE_NETSTAT
int tfxNetstatCgi(
    ttHttpdUserConHandle   conHandle,
    ttNtTableId         tableId)
{
    int                 headerStrLen;
    ttUserCharPtr       buffer;
    ttUserCharPtr       row;
    ttUserGenericUnion  genUnion1;
    ttUserGenericUnion  genUnion2;
    int                 errorCode;

    buffer = tm_malloc(4*(TM_NT_ENTRY_STR_LEN + 2));
    if (buffer == (char *)0)
    {
        errorCode = TM_ENOBUFS;
    }
    else
    {
        row = buffer + TM_NT_ENTRY_STR_LEN + 2;
        errorCode = TM_ENOERROR;
        headerStrLen = TM_NT_ENTRY_STR_LEN;
        switch(tableId)
        {
        case TM_NT_TABLE_TCP:
            tfNtGetTcpHeaderStr(buffer, &headerStrLen);
            break;
        case TM_NT_TABLE_UDP:
            tfNtGetUdpHeaderStr(buffer, &headerStrLen);
            break;
        case TM_NT_TABLE_RTE:
            tfNtGetRteHeaderStr(buffer, &headerStrLen);
            break;
        case TM_NT_TABLE_ARP:
            tfNtGetArpHeaderStr(buffer, &headerStrLen);
            break;
        case TM_NT_TABLE_DEVICE:
            tfNtGetDeviceHeaderStr(buffer, &headerStrLen);
            break;
#ifdef TM_USE_IKE
        case TM_NT_TABLE_ISASTATE_ENTRY:
            tfNtGetIsaStateEntryHeaderStr(buffer, &headerStrLen);
            break;
#endif /* TM_USE_IKE */
#ifdef TM_USE_IKE_POLICIES
        case TM_NT_TABLE_ISASTATE_SPD:
            tfNtGetIsaStateSpdHeaderStr(buffer, &headerStrLen);
            break;
#endif /* TM_USE_IKE_POLICIES */
#ifdef TM_USE_IPSEC
        case TM_NT_TABLE_SADB:
            tfNtGetIpsecSadbHeaderStr(buffer, &headerStrLen);
            break;
        case TM_NT_TABLE_IPSEC_SPD:
            tfNtGetIpsecSpdHeaderStr(buffer, &headerStrLen);
            break;
#endif /* TM_USE_IPSEC */
#ifdef TM_6_USE_MIP_CN
        case TM_NT_TABLE_BINDING:
            tfNtGetBindingHeaderStr(buffer, &headerStrLen);
            break;
#endif /* TM_6_USE_MIP_CN */
        default:
            break;
        }

        strcat(buffer, "\r\n");

/* Table tag */
        tfxHttpdUserSendLoop(
            conHandle,
            "<table align=\"center\" border=\"1\" width=\"100%\">",
            -1,
            &errorCode);

/* Table header */
        if ((errorCode == TM_ENOERROR) && (tableId != TM_NT_TABLE_DEVICE))
        {
            tfxHttpdUserSendLoop(
                conHandle, tfxStrToHtmlTableHeader(buffer, row), -1, &errorCode );
        }

/* the table contents */
        genUnion1.gen8BitParm = (ttUser8Bit)tableId;
        genUnion2.gen32bitParm = conHandle;

        errorCode = tfNetStat(
                      tableId,
                      tfxNtEntryCBCgi,
                      genUnion1,
                      genUnion2);

/* Table end */
        if (errorCode == TM_ENOERROR)
        {
            tfxHttpdUserSendLoop(conHandle, "</table>", -1, &errorCode );
        }
        tm_free(buffer);
    }

    return errorCode;
}
#endif /* TM_USE_NETSTAT */


#ifdef TM_USE_NETSTAT
int tfxNtEntryCBCgi(
    ttNtEntryUPtr       ntEntryUPtr,
    ttUserGenericUnion  genParam1,
    ttUserGenericUnion  genParam2)
{
    ttUserCharPtr       buffer;
    ttUserCharPtr       row;
    int                 entryStrLen;
    ttNtDevEntryPtr     ntDevPtr;
    int                 errorCode;
    int                 i;
    int                 size;
    ttHttpdUserConHandle    conHandle;

    buffer = tm_malloc(4*(TM_NT_ENTRY_STR_LEN + 2));
    if (buffer == (char *)0)
    {
        errorCode = TM_ENOBUFS;
    }
    else
    {
        row = buffer + TM_NT_ENTRY_STR_LEN + 2;
        errorCode = TM_ENOERROR;

        conHandle = (ttHttpdUserConHandle)genParam2.gen32bitParm;
        entryStrLen = TM_NT_ENTRY_STR_LEN;

        if (genParam1.gen8BitParm == TM_NT_TABLE_DEVICE)
        {
/* print device header */
            size = TM_NT_ENTRY_STR_LEN;
            tfNtGetDeviceHeaderStr(buffer, &size);
            if (errorCode == TM_ENOERROR)
            {
                tfxHttpdUserSendLoop(
                    conHandle, tfxStrToHtmlTableHeader(buffer, row), -1, &errorCode );
            }

            ntDevPtr = &(ntEntryUPtr->ntDevEntry);
            size = TM_NT_ENTRY_STR_LEN;
/* print the basic device information */
            tfNtDeviceEntryToStr(ntDevPtr, buffer, &size);
            if (errorCode == TM_ENOERROR)
            {
                tfxHttpdUserSendLoop(
                    conHandle, tfxStrToHtmlRow(buffer, row), -1, &errorCode );
            }

            if (ntDevPtr->ntDevStatus != TM_NT_DEVICE_STATUS_CLOSED)
            {
/* print the address infor header */
                size = TM_NT_ENTRY_STR_LEN;
                tfNtGetAddrConfHeaderStr(buffer, &size);

                if (errorCode == TM_ENOERROR)
                {
                    tfxHttpdUserSendLoop(
                        conHandle, tfxStrToHtmlTableHeader(buffer, row), -1, &errorCode );
                }

#ifdef TM_USE_IPV6
/* print out each of the IPv6 address info in a row */
                for (i=0; i<TM_NT_MAX_IP_PER_IF; i++)
                {
                    if (   ntDevPtr->ntDev6AddrConfArray[i].nt6AddrType
                        != TM_NT_ADDR_CONF_TYPE_NONE)
                    {
                        size = TM_NT_ENTRY_STR_LEN;
                        tf6NtAddrConfEntryToStr(
                            &ntDevPtr->ntDev6AddrConfArray[i],
                            buffer,
                            &size);
                        strcat(buffer, "\r\n");

                        if (errorCode == TM_ENOERROR)
                        {
                            tfxHttpdUserSendLoop(
                                conHandle,
                                tfxStrToHtmlRow(buffer, row),
                                -1,
                                &errorCode );
                        }

                    }
                }
#endif /* TM_USE_IPV6 */

#ifdef TM_USE_IPV4

/* print out each of the IPv4 address info in a row */
                for (i=0; i<TM_NT_MAX_IP_PER_IF; i++)
                {
                    if (   ntDevPtr->ntDevAddrConfArray[i].ntAddrType
                        != TM_NT_ADDR_CONF_TYPE_NONE)
                    {
                        size = TM_NT_ENTRY_STR_LEN;
                        tfNtAddrConfEntryToStr(
                            &ntDevPtr->ntDevAddrConfArray[i],
                            buffer,
                            &size);
                        strcat(buffer, "\r\n");

                        if (errorCode == TM_ENOERROR)
                        {
                            tfxHttpdUserSendLoop(
                                conHandle,
                                tfxStrToHtmlRow(buffer, row),
                                -1,
                                &errorCode );
                        }
                    }
                }

#endif /* TM_USE_IPV4 */
            }
        }
        else
        {
            switch(genParam1.gen8BitParm)
            {
            case TM_NT_TABLE_ARP:
                tfNtArpEntryToStr(&ntEntryUPtr->ntArpEntry,
                                  buffer,
                                  &entryStrLen);
                break;
            case TM_NT_TABLE_RTE:
                tfNtRteEntryToStr(&ntEntryUPtr->ntRteEntry,
                                  buffer,
                                  &entryStrLen);
                break;
            case TM_NT_TABLE_TCP:
                tfNtTcpEntryToStr(&ntEntryUPtr->ntTcpEntry,
                                  buffer,
                                  &entryStrLen);
                break;
            case TM_NT_TABLE_UDP:
                tfNtUdpEntryToStr(&ntEntryUPtr->ntUdpEntry,
                                  buffer,
                                  &entryStrLen);
                break;
#ifdef TM_USE_IPSEC
            case TM_NT_TABLE_SADB:
                tfNtSadbEntryToStr(&ntEntryUPtr->ntSadbEntry,
                                  buffer,
                                  &entryStrLen);
                break;
            case TM_NT_TABLE_IPSEC_SPD:
                tfNtIpsecSpdEntryToStr(&ntEntryUPtr->ntIpsecSpdEntry,
                                  buffer,
                                  &entryStrLen);
                break;
#endif /* TM_USE_IPSEC */
#ifdef TM_6_USE_MIP_CN
        case TM_NT_TABLE_BINDING:
                tf6NtBindingEntryToStr(
                                  &ntEntryUPtr->nt6BindingEntry,
                                  buffer,
                                  &entryStrLen);
                break;
#endif /* TM_6_USE_MIP_CN */

            default:
                break;
            }


            if (errorCode == TM_ENOERROR)
            {
                tfxHttpdUserSendLoop(
                    conHandle,
                    tfxStrToHtmlRow(buffer, row),
                    -1,
                    &errorCode );
            }
        }
/* start new table for next device if this is the device table*/
        if ((errorCode == TM_ENOERROR)
            && (genParam1.gen8BitParm == TM_NT_TABLE_DEVICE))
        {
            tfxHttpdUserSendLoop(conHandle,
                                 "</table><br>"
                                 "<table align=\"center\" border=\"1\" width=\"100%\">",
                                 -1,
                                 &errorCode );
        }
        tm_free(buffer);
    }
/*
 * return TM_ENOERROR   to ask tfNetStat to call back with more entries
 * return other codes to ask tfNetStat to stop calling back
 */
    return TM_ENOERROR;
}
#endif /* TM_USE_NETSTAT */

char * tfxStrToHtmlRow(char * string, char * row)
{
    char*   outPtr;
    char*   columnHeader;

    outPtr = row;

    outPtr += tm_sprintf(outPtr, "\n<tr style=\"text-align:center\"><td>");
    columnHeader = strtok(string, " \n\r");

    while(columnHeader != NULL)
    {
        outPtr += tm_sprintf(outPtr, "%s", columnHeader);
        columnHeader = strtok(NULL, " \n\r");
        if(columnHeader != NULL)
        {
            outPtr += tm_sprintf(outPtr,"</td>\n");
            outPtr += tm_sprintf(outPtr,"    <td>");
        }
    }

    outPtr += tm_sprintf(outPtr, "</td>\n</tr>\n");
    *outPtr = 0;

    return row;
}

char * tfxStrToHtmlTableHeader(char * string, char * row)
{
    char*   outPtr;
    char*   columnHeader;

    outPtr = row;

    outPtr += tm_sprintf(outPtr, "\n<tr><th><b>");
    columnHeader = strtok(string, " \n\r");

    while(columnHeader != NULL)
    {
        outPtr += tm_sprintf(outPtr, "%s", columnHeader);
        columnHeader = strtok(NULL, " \n\r");
        if(columnHeader != NULL)
        {
            outPtr += tm_sprintf(outPtr,"</b></th>\n");
            if (strcmp(columnHeader, (const char *)"flags") == 0 )
            {
                outPtr += tm_sprintf(outPtr,"    <th colspan=\"6\"><b>");
            }
            else
            {
                outPtr += tm_sprintf(outPtr,"    <th><b>");
            }
        }
    }

    outPtr += tm_sprintf(outPtr, "</b></th>\n</tr>\n");
    *outPtr = 0;

    return row;
}

/* Test the low leverl APIs that allow user to build their own
 * status line, headers and body
 */
int tfxCgLowLevelApi(ttHttpdUserConHandle   conHandle)
{
    ttHttpUserHeaderType    headerTypes[TM_HEADER_CNT]; /*types  */
    ttUserCharPtr           headerValues[TM_HEADER_CNT];/*header values */
    int                     headerCount;
    int         bufferLen;

    tfHttpdUserSendResponseStatusLine(
        conHandle,
        TM_HTTP_VERSION_1_1,
        200,
        "OK",
        0);

    headerTypes[0] = TM_HTTP_HEADER_DATE;
    headerTypes[1] = TM_HTTP_HEADER_SERVER;
    headerTypes[2] = TM_HTTP_HEADER_CONTENT_LENGTH;

    headerValues[0]= "Fri, 19 Nov 2004 21:53:50 GMT";
    headerValues[1]= "Treck Web Server 4.1";
    headerValues[2]= "20";

/* testing with too many headers */
/*
    for (i= 0; i< TM_HEADER_CNT; i++)
    {
        headerTypes[i] = TM_HTTP_HEADER_DATE;
        headerValues[i]= "Fri, 19 Nov 2004 21:53:50 GMT";
    }
*/
    headerCount = TM_HEADER_CNT;

    tfHttpdUserSendResponseHeaders(
        conHandle,
        &headerCount,
        headerTypes,
        (ttUserConstCharPtrPtr)headerValues,
        0);

    bufferLen = (int)strlen(TM_RESPONSE_BODY);

    tfHttpdUserSendResponseBody(
        conHandle,
        TM_RESPONSE_BODY,
        &bufferLen,
        TM_HTTPD_END);

    return TM_ENOERROR;
}


#ifdef TM_USE_NETSTAT
int tfxCgiSendNetstatTable(
    ttHttpdUserConHandle   conHandle,
    int                    tableId)
{
    int errorCode;

    errorCode = TM_ENOERROR;


/* Try to send the page */
    tfxHttpdUserSendLoop(
                conHandle,
                "<html>\n"
                "<head>\n"
                "<title>\n"
                "TreckDemo - ",
                -1,
                &errorCode);

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                tlNtTableName[tableId],
                -1,
                &errorCode);
    }

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                "</title>\n"
                "</head>\n"
                "<body>\n"
                "<font size=\"6\">\n"
                "<p align=\"center\"><b>",
                -1,
                &errorCode);
    }

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                tlNtTableName[tableId],
                -1,
                &errorCode);
    }

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                "</b></p>"
                "<font size=\"2\">\n",
                -1,
                &errorCode);
    }

/* Try to send the netstat table as part of the page body */
    if (errorCode == TM_ENOERROR)
    {
        errorCode = tfxNetstatCgi(
            conHandle, (ttNtTableId) tableId);
    }

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                "</body>\n"
                "</html>\n",
                -1,
                &errorCode);
    }

    tfHttpdUserSendBuffer( conHandle,
                        0,
                        0,
                        (TM_HTTPD_END | TM_BLOCKING_ON),
                        &errorCode);
    return TM_ENOERROR;
}
#endif /* TM_USE_NETSTAT */


int tfxCgiSendHello(ttHttpdUserConHandle conHandle, const char* name)
{
    int errorCode;

    errorCode = TM_ENOERROR;

/* Try to send the page */
    tfxHttpdUserSendLoop(
                conHandle,
                "<html>\n"
                "<head>\n"
                "<title>TreckDemo - Hello</title>\n"
                "</head>\n"
                "<body>\n"
                "<font size=\"6\">\n"
                "Hello \n",
                -1,
                &errorCode);

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                name,
                -1,
                &errorCode);
    }

/* Try to send the rest of the page */
    if (errorCode == TM_ENOERROR)
    {
        tfxHttpdUserSendLoop(
                conHandle,
                "</body>\n"
                "</html>\n",
                -1,
                &errorCode);
    }


/* Whether or not we've sucessfully send the data,
 * we'd like to try to indicate the end of the response
 */
    tfHttpdUserSendBuffer( conHandle,
                        0,
                        0,
                        (TM_HTTPD_END | TM_BLOCKING_ON),
                        &errorCode);

    return errorCode;
}


int tfxCgiSendNotFound(ttHttpdUserConHandle conHandle, const char* path)
{
    int errorCode;


    errorCode = TM_ENOERROR;

/* Try to send the page */
    tfxHttpdUserSendLoop(
                conHandle,
                "<html>\n"
                "<head>\n"
                "<title>TreckDemo - CGI Not Found</title>\n"
                "</head>\n"
                "<body>\n"
                "<font size=\"6\">\n"
                "CGI Not Found\n"
                "</body>\n"
                "</html>\n",
                -1,
                &errorCode);


    tfHttpdUserSendBuffer(
                conHandle,
                path,
                -1,
                TM_BLOCKING_ON,
                &errorCode);

    tfHttpdUserSendBuffer(
                conHandle,
                "</body>\n"
                "</html>\n",
                -1,
                TM_BLOCKING_ON,
                &errorCode);


/* Whether or not we've sucessfully send the data,
 * we'd like to try to indicate the end of the response
 */
    tfHttpdUserSendBuffer(
                conHandle,
                0,
                0,
                (TM_HTTPD_END | TM_BLOCKING_ON),
                &errorCode);

    return errorCode;
}

/* Insist on sending a buffer through a loop that keeps calling
 * tfxHttpdUserSendLoop till sent or error
 */
TM_PROTO_EXTERN int tfxHttpdUserSendLoop(
    ttHttpdUserConHandle   conHandle,
    const char TM_FAR *     bufferPtr,
    int                     bufferLen,
    int TM_FAR *            errorCodePtr)
{
    int retCode;
    int sentLen = 0;

    if(bufferLen == -1)
    {
        bufferLen = (int)strlen(bufferPtr);
    }

    while ( (bufferLen > sentLen) && (*errorCodePtr == TM_ENOERROR) )
    {
        retCode = tfHttpdUserSendBuffer(
                    conHandle,
                    bufferPtr + sentLen,
                    bufferLen - sentLen,
                    TM_BLOCKING_ON,
                    errorCodePtr);

        if (retCode != TM_SOCKET_ERROR)
        {
            sentLen += retCode;
        }
        else
        {
            break;
        }
    }

    return sentLen;
}

int tfxCgiAuthConfig(
    ttHttpdUserConHandle    conHandle,
    ttHttpdUserSrvHandle    srvHandle,
    char *                  objectNamePtr,
    char *                  realmPtr,
    char *                  usernamePtr,
    char *                  passwordPtr,
    char *                  actionPtr)
{
    char response[256];
    int  errorCode;

    if (strstr(actionPtr, "addchange"))
    {
        errorCode = tfHttpdUserAuthConfig(
                            srvHandle,
                            objectNamePtr,
                            realmPtr,
                            usernamePtr,
                            passwordPtr,
                            TM_HTTPD_AUTH_BAA);

        if (errorCode == TM_ENOERROR)
        {
            strcpy(response, "You've sucessfully added/changed");
        }
        else
        {
            strcpy(response, "Failed add/change");
        }
    }
    else    if (strstr(actionPtr, "remove"))
    {
        errorCode = tfHttpdUserAuthConfig(
                            srvHandle,
                            objectNamePtr,
                            realmPtr,
                            usernamePtr,
                            passwordPtr,
                            TM_HTTPD_AUTH_NONE);

        if (errorCode == TM_ENOERROR)
        {
            strcpy(response, "You've sucessfully removed");
        }
        else
        {
            strcpy(response, "Failed remove");
        }
    }

    strcat(response, " the authentication entry for ");
    strcat(response, objectNamePtr);

    tfHttpdUserSendBuffer(
                    conHandle,
                    response,
                    -1,
                    (TM_HTTPD_END | TM_BLOCKING_ON),
                    &errorCode);

    return errorCode;
}

#endif /* TM_USE_HTTPD_CGI */

#else /* ! TM_USE_HTTPD*/

#ifdef TM_LINT
LINT_UNUSED_HEADER
#endif /* TM_LINT */

/* To allow link for builds when TM_USE_HTTPD_CGI is not defined */
int tvCgi = 0;

#endif /* TM_USE_HTTPD*/
