/* $Id: */
/********************************************************************
 * Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times. (c) Copyright 1995-2003 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/******************************************************************************
*
* util_time.c - Implements interface defined in util_time.h
*
* This implementation is for the EDK Standalone BSP.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1     rmm  03/08/05 First release
* 2     drg  09/20/07 Port to work on MicroBlaze
* 3     drg  04/04/08 Remove junk and clean up for Treck.
* 4     pdmc 03/10/11 Modified for AXI bus
*
*****************************************************************************/

/***************************** Include Files ********************************/
#include "trsocket.h"
#ifdef TM_TRECK_PREEMPTIVE_KERNEL
#include "xmk.h"
#endif /* TM_TRECK_PREEMPTIVE_KERNEL */
#include "xparameters.h"
#include "util_time.h"
#include "defaults.h"
#ifdef __PPC__
#    include "xreg440.h"
#    include "xpseudo_asm.h"
#    include "xtime_l.h"
#    include "xexception_l.h"
#else
#  include "mb_interface.h"
#  include "xtmrctr.h"
#endif

/**************************** Type Definitions ******************************/
typedef unsigned long long Uint64;

typedef struct {
    unsigned Samples;
    XTime SampleStart;
    XTime SampleStop;
    XTime Sum;
    XTime High;
    XTime Low;
} TimeStat;

/***************** Macros (Inline Functions) Definitions ********************/
#ifdef __PPC__
#  define XPAR_TIMER_CLK    XPAR_CPU_PPC440_CORE_CLOCK_FREQ_HZ
#    define TRECK_TICK_INTERVAL ((XPAR_CPU_PPC440_CORE_CLOCK_FREQ_HZ / 1000) * TICK_INTERVAL_MS)
#else
#  define TIMER_DEVID       XPAR_TMRCTR_0_DEVICE_ID
#  define TIMER_CTRID_0     0
#  define TIMER_CTRID_1     1
#  define TIMER_RSTVAL      1
    /* This is the AXI clock frequency. It matches the Core Clock. */
#  define XPAR_TIMER_CLK    (XPAR_CPU_CORE_CLOCK_FREQ_HZ / 2)
#  define TRECK_TICK_INTERVAL ((XPAR_TIMER_CLK / 1000) * TICK_INTERVAL_MS)
#endif

/************************** Variable Definitions ****************************/

static int ServiceTreck = FALSE;

extern XTime StartTime;             /* vars that keep track of our time in tests */
extern unsigned TimeLimitMs;
extern int TimeExpired;
extern int TimingTest;

#ifndef __PPC__

static XTmrCtr TmrCtrInst;

#ifdef TM_XILINX_USE_AXI4
typedef struct {
    u32 timerval;
    u32 overflows;
}time_big;
#else /* TM_XILINX_USE_AXI4 */
typedef struct {
    u32 overflows;
    u32 timerval;
}time_big;
#endif /* TM_XILINX_USE_AXI4 */

static union {
    XTime    Time64bit;
    time_big big_time;
} time_base;


/************************** Function Prototypes *****************************/

int Setup_Interrupt(u8 Id, XInterruptHandler Handler, void * CallBackRef);

static void TimerIsr(void * CallBackRef, u8 TmrCtrNumber);
#else
static void PitTickISR(void);
#endif


/****************************************************************************
 * TimeStampGet
 *
 * Description:
 *   Retrieve the system free running counter value
 *
 * Parameters:
 *   none
 *
 * Returns:
 *   Current value of the system free running clock
 *
 ****************************************************************************/
XTime TimeStampGet(void)
{
    XTime Rc;

#ifdef __PPC__
    XTime_GetTime(&Rc);
#else

    /* Not Protected from Rollovers. */

    time_base.big_time.timerval = XTmrCtr_GetValue(&TmrCtrInst, TIMER_CTRID_1);

    Rc = time_base.Time64bit;

#endif

    return(Rc);
}

/****************************************************************************
 * TimeStampFromSec
 *
 * Description:
 *   Convert the given number of seconds into the equivalent time stamp
 *
 * Parameters:
 *   Seconds - the number of seconds to convert from
 *
 * Returns:
 *   The value of a timestamp that corresponds to the number of given seconds
 *
 ****************************************************************************/
XTime TimeStampFromSec(int Seconds)
{
    XTime Calc = XPAR_TIMER_CLK;

    Calc = Calc * Seconds;
    return(Calc);
}

/****************************************************************************
 * TimeStampFromMs
 *
 * Description:
 *   Convert the given number of milliseconds into the equivalent time stamp
 *
 * Parameters:
 *   Seconds - the number of Ms to convert from
 *
 * Returns:
 *   The value of a timestamp that corresponds to the number of given Ms
 *
 ****************************************************************************/
XTime TimeStampFromMs(int Milliseconds)
{
    XTime Calc = XPAR_TIMER_CLK;

    Calc = Calc / 1000.0;
    Calc = (Calc * Milliseconds);
    return(Calc);
}

/****************************************************************************
 * TimeStampFromUs
 *
 * Description:
 *   Convert the given number of microseconds into the equivalent time stamp
 *
 * Parameters:
 *   Seconds - the number of Us to convert from
 *
 * Returns:
 *   The value of a timestamp that corresponds to the number of given Us
 *
 ****************************************************************************/
XTime TimeStampFromUs(int Microseconds)
{
    XTime Calc = XPAR_TIMER_CLK;

    Calc = Calc / 1000000.0;
    Calc = (Calc * Microseconds);
    return(Calc);
}

/****************************************************************************
 * TimeStampDiffUs
 *
 * Description:
 *   Compute the absolute difference between two time stamps and return that
 *   value in units of microseconds
 *
 * Parameters:
 *   Ts1 - the first time stamp retrieved with TimeStampGet()
 *   Ts2 - the second time stamp retrieved with TimeStampGet()
 *
 * Returns:
 *   Time difference in microseconds
 *
 ****************************************************************************/
unsigned TimeStampDiffUs(XTime Ts1, XTime Ts2)
{
    double Calc;

    Calc = (double)Ts2 - (double)Ts1;

    if (Calc < 0)
    {
        Calc = -Calc;
    }

    /* Make calculation */
    Calc = Calc / ((double)XPAR_TIMER_CLK / 1000000);

    /* Round off to nearest whole number */
    Calc = Calc + 0.5;

    /* Return an int type (decimal part is truncated) */
    return((unsigned)Calc);
}

/****************************************************************************
 * TimeStampDiffUs
 *
 * Description:
 *   Compute the absolute difference between two time stamps and return that
 *   value in units of milliseconds
 *
 * Parameters:
 *   Ts1 - the first time stamp retrieved with TimeStampGet()
 *   Ts2 - the second time stamp retrieved with TimeStampGet()
 *
 * Returns:
 *   Time difference in milliseconds
 *
 ****************************************************************************/
unsigned TimeStampDiffMs(XTime Ts1, XTime Ts2)
{
    double Calc;

    Calc = (double)Ts2 - (double)Ts1;

    if (Calc < 0)
    {
        Calc = -Calc;
    }

    /* Make calculation */
    Calc = Calc / ((double)XPAR_TIMER_CLK / 1000);

    /* Round off to nearest whole number */
    Calc = Calc + 0.5;

    /* Return an int type (decimal part is truncated) */
    return((unsigned)Calc);
}

/****************************************************************************
 * TimeStampDiff
 *
 * Description:
 *   Compute the absolute difference between two time stamps and return that
 *   value in units of seconds
 *
 * Parameters:
 *   Ts1 - the first time stamp retrieved with TimeStampGet()
 *   Ts2 - the second time stamp retrieved with TimeStampGet()
 *
 * Returns:
 *   Time difference in seconds (floating point)
 *
 ****************************************************************************/
float TimeStampDiff(XTime Ts1, XTime Ts2)
{
    double Calc;

    Calc = (double)Ts2 - (double)Ts1;

    if (Calc < 0)
    {
        Calc = -Calc;
    }

    /* Make calculation */
    Calc = Calc / (double)XPAR_TIMER_CLK;

    /* Return an int type (decimal part is truncated) */
    return((float)Calc);
}

/* Only needed for standalone, xilkernel demo has a timer task */
#ifdef TM_TRECK_NO_KERNEL
void TreckTimerInit(void)
{
    ServiceTreck = TRUE;
}
#endif /* TM_TRECK_NO_KERNEL */

#ifdef __PPC__
/******************************************************************************
 * Initialize the timers
 *****************************************************************************/
void TimerInit(){

    /* Setup and enable PPC decrementer to be our tick timer */
    XExc_RegisterHandler(XEXC_ID_DEC_INT, (XExceptionHandler)PitTickISR, NULL);

    XTime_TSRClearStatusBits(XREG_TSR_CLEAR_ALL);
    XTime_DECEnableAutoReload();

    XTime_DECSetInterval(TRECK_TICK_INTERVAL);

    /* Enable timer */
    XTime_DECEnableInterrupt();
}

/******************************************************************************
 * Notify the stack that a tick has occurred
 *****************************************************************************/
static void PitTickISR(void)
{
    XTime       currentTime;
    unsigned    elapsedTimeMs;

    if(ServiceTreck)
    {
        tfTimerUpdateIsr();
    }
    XTime_DECEnableInterrupt();
    XTime_DECClearInterrupt();
}

#else /* MicroBlaze */
void usleep(unsigned long usec) {
    volatile XTime CurTime, EndTime;

    /* Get the current time and calculate when timeout occurs */
    CurTime = TimeStampGet();
    EndTime = CurTime + TimeStampFromUs(usec);

    /* Spin */
    while(1)
    {
        /* Timeout? */
        CurTime = TimeStampGet();

        if (CurTime >= EndTime)
        {
            return;
        }
    }
}

void usleepWithRecvInterface(unsigned long usec,
                             ttUserInterface interfaceHandle)
{
    volatile XTime CurTime, EndTime;

    /* Get the current time and calculate when timeout occurs */
    CurTime = TimeStampGet();
    EndTime = CurTime + TimeStampFromUs(usec);

    /* Spin */
    while(1)
    {
        while(tfCheckReceiveInterface(interfaceHandle) == 0)
        {
            /* Calls the driver's recv function */
            tfRecvInterface(interfaceHandle);
        }

        /* Timeout? */
        CurTime = TimeStampGet();

        if (CurTime >= EndTime)
        {
            return;
        }
    }
}

#if 0
void msleep(unsigned long msec) {
    volatile XTime CurTime, EndTime;

    /* Get the current time and calculate when timeout occurs */
    CurTime = TimeStampGet();
    EndTime = CurTime + TimeStampFromMs(msec);

    /* Spin */
    while(1)
    {
        /* Timeout? */
        CurTime = TimeStampGet();

        if (CurTime >= EndTime)
        {
            return;
        }
    }
}
#endif


/******************************************************************************
 * Initialize the timers
 *****************************************************************************/
void TimerInit(void){
    XStatus Status;

    /* Clear the time_base before using it */
    time_base.Time64bit = 0;
    time_base.big_time.overflows = 0;
    time_base.big_time.timerval = 0;

    Status = XTmrCtr_Initialize(&TmrCtrInst, TIMER_DEVID);
    if (Status == XST_DEVICE_NOT_FOUND)
    {
        xil_printf("TimerInit: Device not found\r\n");
        return;
    }
    else if (Status == XST_DEVICE_IS_STARTED)
    {
        xil_printf("TimerInit: Device already started\r\n");
    }

    Status = XTmrCtr_SelfTest(&TmrCtrInst, TIMER_CTRID_0);
    if (Status != XST_SUCCESS)
    {
        xil_printf("TimerInit: Failed timer self test\r\n");
        return;
    }

    /* Tie the timer counter into the interrupt controller */
#ifdef __PPC__
    Status = Setup_Interrupt( TMRCTR_INTR_ID, XTmrCtr_InterruptHandler,
                    &TmrCtrInst);
#else
    Status = Setup_Interrupt( XPAR_INTC_0_TMRCTR_0_VEC_ID, XTmrCtr_InterruptHandler,
                    &TmrCtrInst);
#endif
    if (Status != XST_SUCCESS)
    {
        xil_printf("TimerInit: Failed to connect timer interrupt\r\n");
        return;
    }

    XTmrCtr_SetHandler(&TmrCtrInst, (XTmrCtr_Handler)TimerIsr,
            &TmrCtrInst);

    /* Setup timer 0 counter to use interrupts, count down */
    XTmrCtr_SetOptions(&TmrCtrInst, TIMER_CTRID_0,
                    XTC_INT_MODE_OPTION | XTC_AUTO_RELOAD_OPTION |
                    XTC_EXT_COMPARE_OPTION | XTC_DOWN_COUNT_OPTION);
    XTmrCtr_SetResetValue(&TmrCtrInst, TIMER_CTRID_0, TRECK_TICK_INTERVAL);

    XTmrCtr_Start(&TmrCtrInst, TIMER_CTRID_0);


    XTmrCtr_SetOptions(&TmrCtrInst, TIMER_CTRID_1,
                    XTC_INT_MODE_OPTION | XTC_AUTO_RELOAD_OPTION |
                    XTC_EXT_COMPARE_OPTION);
    XTmrCtr_SetResetValue(&TmrCtrInst, TIMER_CTRID_1, TIMER_RSTVAL);

    XTmrCtr_Start(&TmrCtrInst, TIMER_CTRID_1);

    TimingTest = FALSE;
}

#ifdef TM_USE_TIMER_CB
void TimerCBFunc(ttUser32Bit        timerInterval,
                 ttUserGenericUnion userParam)
{
    TM_UNREF_IN_ARG(userParam);
    nextTimerExecuteTs = (timerInterval + (TM_TICK_LENGTH -1)) / TM_TICK_LENGTH;
    return;
}
#endif /* TM_USE_TIMER_CB */

/******************************************************************************
 * Notify the stack that a tick has occurred
 * For Microblaze Timer 0 is used to do the periodic tick interval for Treck.
 * Timer 1 is used as a free running counter. The count roll overs a used to
 * simulate the 64 bit RC counter in the PPC.
 *****************************************************************************/
static void TimerIsr(void * CallBackRef, u8 TmrCtrNumber){
    /* Call the CpPitIsr if it is counter 0. This adds a layer of stacking, but
     * keeps the code simple for PPC and MB
     */
    if(TIMER_CTRID_0 == TmrCtrNumber){
        if(ServiceTreck){
            tfTimerUpdateIsr();
            if (nextTimerExecuteTs != 0)
            {
                nextTimerExecuteTs--;
            }
        }
        XTmrCtr_Start(&TmrCtrInst, TIMER_CTRID_0);
    }
    else{
        /* Update the overflow count */
        time_base.big_time.overflows++;
        XTmrCtr_Start(&TmrCtrInst, TIMER_CTRID_1);
    }
    if ((TimingTest) && (TimeStampDiffMs(StartTime, TimeStampGet()) > TimeLimitMs)){
        TimeExpired = TRUE;
    }
    return;
}

#endif
