/* $Id: */
/********************************************************************
 * Copyright(C) 2008 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times.
 * (c) Copyright 2008 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/******************************************************************************
*
* xtr_lldma.c - The treck stack to DMA interface.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1     drg  03/31/08 First release
* 2     pdmc 03/10/11 Modified for AXI bus and added full checksum
*                     offloading support
*
*****************************************************************************/

/***************************** Include Files ********************************/
#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>

#include "xaxidma.h"
#include "xbuffermem.h"
#include "xtr_axiethernet.h"
#include "util_time.h"
#include "xtr_axidma.h"
#include "cache.h"
#include "marvell_88e1111.h"
#include "debug.h"
#include "defaults.h"


/************************** Constant Definitions ****************************/
#define BD_ALIGNMENT     XAXIDMA_BD_MINIMUM_ALIGNMENT    /* Alignment of buffer descriptors */
#define RX_BD_FREE_THRESHOLD    (RX_BD_COUNT / 4)
    /* Threshold to disable the interrupts, so buffers can get freed */
#define RX_BD_IRPTDIS_THRESHOLD  (RX_BD_FREE_THRESHOLD/2)

#define CSUM_ENABLE_PARTIAL     0x00000001
#define CSUM_ENABLE_FULL        0x00000002

#define SINGLE_PKT_RX_COMPLETE_FLAGS (XAXIDMA_BD_STS_RXSOF_MASK | \
                XAXIDMA_BD_STS_COMPLETE_MASK | XAXIDMA_BD_STS_RXEOF_MASK)
/**************************** Type Definitions ******************************/

/***************** Macros (Inline Functions) Definitions ********************/

/************************** Function Prototypes *****************************/
static void SgTxISR(void *Callback);

static int AxiDma_FreeTxBuffer(ttUserInterface Handle);

/************************** Variable Definitions ****************************/

extern int IfaceMtu;

static char RxBdSpace[XAxiDma_BdRingMemCalc(RX_BD_COUNT, BD_ALIGNMENT)
        + BD_ALIGNMENT];
static char TxBdSpace[XAxiDma_BdRingMemCalc(TX_BD_COUNT, BD_ALIGNMENT)
        + BD_ALIGNMENT];

int DmaInitialized = FALSE;

static XAxiDma_Bd *RxBdCurPtr;
static int RxBdCurCnt, RxBdCnt;

#ifndef XLNX_USE_DRV_BUF_MEM
static unsigned RxTreckBufferIndex = 0;
static ttUserBuffer RxTreckBuffers[RX_BD_COUNT];

/* manipulate the buffer handles */
#define RX_BUFHANDLE_PTR()    (&RxTreckBuffers[RxTreckBufferIndex])
#define RX_BUFHANDLE_NEXT()    ((++RxTreckBufferIndex >= (RX_BD_COUNT))? \
                (RxTreckBufferIndex = 0) : (RxTreckBufferIndex))

#endif
/************************** Function Definitions ****************************/
int InitializeDmaAndEthernet(TemacDevData * Mac)
{
    XAxiEthernet_Config *   TemacCfgPtr;
    XAxiDma_Config *        DmaCfgPtr;
    int                     Status;

    Mac->TemacInst.Config.DeviceId = XPAR_AXIETHERNET_0_DEVICE_ID;

    DmaCfgPtr = XAxiDma_LookupConfig(Mac->TemacInst.Config.DeviceId);
    Status = XAxiDma_CfgInitialize(&Mac->DmaInst, DmaCfgPtr);
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "InitializeDmaAndEthernet: Failed to initialize DMA");
    }

    if (Status == XST_SUCCESS)
    {
        TemacCfgPtr = XAxiEthernet_LookupConfig(Mac->TemacInst.Config.DeviceId);
        Status = XAxiEthernet_CfgInitialize(&Mac->TemacInst,
                                            TemacCfgPtr,
                                            TemacCfgPtr->BaseAddress);
        if (Status != XST_SUCCESS)
        {
            DEBUG_LOG(DEBUG_ERROR, "InitializeDmaAndEthernet: Failed to initialize Temac");
        }
    }

    if (Status == XST_SUCCESS)
    {
        Status = AxiTemac_Setup(Mac);
        if (Status != XST_SUCCESS)
        {
            DEBUG_LOG(DEBUG_ERROR, "InitializeDmaAndEthernet: Failed to setup DMA");
        }
    }

    if (Status == XST_SUCCESS)
    {
        Status = AxiDma_Setup(Mac);
        if (Status != XST_SUCCESS)
        {
            DEBUG_LOG(DEBUG_ERROR, "InitializeDmaAndEthernet: Failed to setup DMA");
        }
    }

    return Status;
}

/*
 * Setup the DMA
 */
int AxiDma_Setup(TemacDevData * Mac)
{
    XAxiDma_BdRing * RxRingPtr;
    XAxiDma_BdRing * TxRingPtr;
    XAxiDma_Bd BdTemplate;
    Xuint32 BdSpacePtr;
    int Status;


    /* SGDMA channels:
     *   - Setup descriptor rings
     *   - Setup interrupt coalescing parameters
     */
    RxRingPtr = XAxiDma_GetRxRing(&(Mac->DmaInst));

    /* Start with Rx channel
     */
    XAxiDma_BdClear(&BdTemplate);

    /*
     * Create the RxBD ring
     */
    /*
     * Size check, is there enough space for this number of BD?
     */
    if(sizeof(RxBdSpace) < XAxiDma_BdRingMemCalc(BD_ALIGNMENT, RX_BD_COUNT)){
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Error not enough RxBD space, %d",
                        XAxiDma_BdRingMemCalc(BD_ALIGNMENT, RX_BD_COUNT));
        return(XST_FAILURE);
    }

    /*
     * Locating the BD space is tricky. It must be aligned with the BD
     * Alignment. It can be placed arbitrarily at the end of memory, but that is
     * dangerous.
     * Using the array, bigger than needed by BD_ALIGNMENT, then find the first
     * aligned address.
     */
    BdSpacePtr = ROUND_UP((Xuint32)&RxBdSpace, BD_ALIGNMENT);
    Status = XAxiDma_BdRingCreate(RxRingPtr, (Xuint32)BdSpacePtr,
                    (Xuint32)BdSpacePtr, BD_ALIGNMENT, RX_BD_COUNT);
    if (Status != XST_SUCCESS) {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Error setting up RxBD space, code %d", Status);
        return(Status);
    }

    Status = XAxiDma_BdRingClone(RxRingPtr, &BdTemplate);
    if (Status != XST_SUCCESS) {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Error initializing RxBD space, code %d", Status);
        return(Status);
    }

    Status |= XAxiDma_BdRingSetCoalesce(RxRingPtr,
                    DEFAULT_RX_THRESHOLD, DEFAULT_RX_TIMEOUT);
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Failed to set RxBD coalescing, code %d\n",
                  Status);
        return(Status);
    }

    TxRingPtr = XAxiDma_GetTxRing(&(Mac->DmaInst));

    /* Setup Tx channel */
    XAxiDma_BdClear(&BdTemplate);

    /*
     * Size check, is there enough space for this number of BD?
     */
    if(sizeof(TxBdSpace) < XAxiDma_BdRingMemCalc(BD_ALIGNMENT, TX_BD_COUNT)){
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Error not enough TxBD space, %d",
                        XAxiDma_BdRingMemCalc(BD_ALIGNMENT, TX_BD_COUNT));
        return(XST_FAILURE);
    }
    /*
     * Create the TxBD ring
     */
    BdSpacePtr = ROUND_UP((Xuint32)&TxBdSpace, BD_ALIGNMENT);
    Status = XAxiDma_BdRingCreate(TxRingPtr, (Xuint32)BdSpacePtr,
                    (Xuint32)BdSpacePtr, BD_ALIGNMENT, TX_BD_COUNT);

    if (Status != XST_SUCCESS) {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Error setting up TxBD space, code %d", Status);
        return(Status);
    }

    Status = XAxiDma_BdRingClone(RxRingPtr, &BdTemplate);
    if (Status != XST_SUCCESS) {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Error initializing TxBD space, code %d", Status);
        return(Status);
    }

    Status |= XAxiDma_BdRingSetCoalesce(TxRingPtr,
                    DEFAULT_TX_THRESHOLD, DEFAULT_TX_TIMEOUT);
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Setup: Failed to setup TxBD coalescing, code %d\n",
                  Status);
        return(Status);
    }

    /* Initialize variables use to communicate status between the interrupt
     * handler and the receive routine.
     */
    RxBdCurPtr = NULL;
    RxBdCurCnt = 0;
    RxBdCnt = 0;

#ifdef XLNX_USE_DRV_BUF_MEM
    xbuf_init();
#endif

    DmaInitialized = TRUE;

    return(Status);

}

/******************************************************************************
 * Initialize and setup the device with DMA capability
 *
 * Parameters:
 *   Handle - User interface object
 *
 * Returns:
 *   TM_DEV_ERROR if open fails, TM_DEV_OKAY otherwise
 *****************************************************************************/
int AxiDma_Open(ttUserInterface Handle)
{
    int Status;
    XAxiDma_BdRing * RxRingPtr;
    XAxiDma_BdRing * TxRingPtr;
    TemacDevData *Mac = (TemacDevData*)tfDeviceGetPointer(Handle);

    Status = InitializeDmaAndEthernet(Mac);
    if (Status != XST_SUCCESS)
    {
        return Status;
    }

    /*
     * Setup the memory area for RX Buffers. Allow the Treck stack to manage the
     * memory. The driver just allocates and uses buffers.
     * For the RX space, get all available BD, and tie them to buffers, then
     * let the hardware have the BD ring to use for RX.
     */
    if (AxiDma_RefillRxRing(Handle) < 0) {
        DEBUG_LOG(DEBUG_ERROR, "Open: AxiDma_RefillRxRing failed");
        return(TM_DEV_ERROR);
    }

    if (IfaceMtu > XAE_MTU)
    {
        /* Enable Jumbo frames */
        xil_printf("AxiDma_Open: enabling Jumbo frames\r\n");
        XAxiEthernet_SetOptions(&Mac->TemacInst, XAE_JUMBO_OPTION);
    }

    /* Setup the interrupt handlers and enable the interrupts here. The
     * interrupt handler needs the Treck UI Handle.
     */
    Status = Setup_Interrupt(Mac->TemacInst.Config.AxiDmaTxIntr,
                (XInterruptHandler) SgTxISR,
                Handle);
    Status |= Setup_Interrupt(Mac->TemacInst.Config.AxiDmaRxIntr,
                (XInterruptHandler) SgRxISR,
                Handle);

    /* Install master interrupt handler */
    if (Status != XST_SUCCESS) {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Open: Failed to install interrupt handler");
        return(Status);
    }

    /* For TX, do nothing, the buffer will be handed to the driver in the
     * AxiDma_Send()
     */

    /*
     * Enable transmit and receive related interrupts
     */
    TxRingPtr = XAxiDma_GetTxRing(&(Mac->DmaInst));
    RxRingPtr = XAxiDma_GetRxRing(&(Mac->DmaInst));

    //XAxiDma_BdRingIntEnable(TxRingPtr, XAXIDMA_IRQ_ALL_MASK);
    XAxiDma_BdRingIntEnable(RxRingPtr, XAXIDMA_IRQ_ALL_MASK);

    /* Start the DMA engines */
    Status = XAxiDma_BdRingStart(TxRingPtr);
    Status |= XAxiDma_BdRingStart(RxRingPtr);
    if (Status != XST_SUCCESS) {
        DEBUG_LOG(DEBUG_ERROR, "Open: Error starting BdRing, code %d\n", Status);
        return(TM_DEV_ERROR);
    }

    return(TM_DEV_OKAY);
}

/******************************************************************************
 * Disable and free up buffer pool associated with this device
 *
 * Parameters:
 *   Handle - User interface object
 *
 * Returns:
 *   TM_DEV_OKAY
 *****************************************************************************/
int AxiDma_Close(ttUserInterface Handle)
{
    TemacDevData *Mac = (TemacDevData*)tfDeviceGetPointer(Handle);

    DEBUG_LOG(DEBUG_MISC, "Temac: Close()");
    XAxiDma_BdRingIntDisable(&Mac->DmaInst.RxBdRing, XAXIDMA_IRQ_ALL_MASK);
    XAxiDma_BdRingIntDisable(&Mac->DmaInst.TxBdRing, XAXIDMA_IRQ_ALL_MASK);
    XAxiEthernet_Stop(&Mac->TemacInst);

    return(TM_DEV_OKAY);
}

/******************************************************************************
 * Return transmit buffer(s) from the driver back to the stack. This is done by
 * querying HW as to whether it has any processed descriptors for us to go
 * through. If there are some, then free up the associated buffer.
 *
 * Parameters:
 *   Handle - User interface object
 *
 * Returns:
 *   TM_DEV_ERROR if error accessing the DMA engine
 *   TM_DEV_OKAY if no errors
 *****************************************************************************/
static int AxiDma_FreeTxBuffer(ttUserInterface Handle)
{
    TemacDevData *Mac = (TemacDevData*)tfDeviceGetPointer(Handle);
    XAxiDma_BdRing * TxRingPtr;
    XAxiDma_Bd *BdPtr, *BdCurPtr;
    XStatus Status;
    register int NumBds, i;


    TxRingPtr = XAxiDma_GetTxRing(&(Mac->DmaInst));

    if ((NumBds = XAxiDma_BdRingFromHw(TxRingPtr, XAXIDMA_ALL_BDS, &BdPtr)) > 0)
    {
        for (i=0, BdCurPtr=BdPtr; i<NumBds; i++)
        {
            /*Find the last Bd for a packet, then tell the Treck stack to free
             * the buffers */
            if (XAxiDma_BdGetCtrl(BdCurPtr) & XAXIDMA_BD_CTRL_TXEOF_MASK)
            {
                DEBUG_LOG(DEBUG_TX, "AxiDma_FreeTxBuffer(): SendComplete interface signalled");
                tfSendCompleteInterface(Handle,
                                  TM_DEV_SEND_COMPLETE_DRIVER);
                Mac->EthStats.TxFrames++;
            }
            if (XAxiDma_BdGetSts(BdCurPtr) & XAXIDMA_BD_STS_ALL_ERR_MASK)
            {
                DEBUG_LOG(DEBUG_ERROR,"DMA ERROR\r\n");
            }
            BdCurPtr = XAxiDma_BdRingNext(TxRingPtr, BdCurPtr);
        }

        Status = XAxiDma_BdRingFree(TxRingPtr, NumBds, BdPtr);
        if (Status != XST_SUCCESS)
        {
            DEBUG_LOG(DEBUG_ERROR,"AxiDma_FreeTxBuffer(): Error freeing TxBDs, code %d",
                      Status);
            return(TM_DEV_ERROR);
        }
    }

    DEBUG_LOG(DEBUG_TX, "AxiDma_FreeTxBuffer(): Freed %d buffers", NumBds);

    return(TM_DEV_OKAY);
}

/******************************************************************************
 * Send exactly one packet which is contained entirely within a single
 * buffer.
 *
 * Parameters:
 *   Handle - User interface object
 *   DataPtr - Address of packet
 *   DataLen - Length of packet
 *   Flag - Flags from the stack
 *
 * Returns:
 *   TM_DEV_ERROR if an error occured setting up the transmission of the frame
 *   TM_DEV_OKAY if frame transmission was kicked off without problems
 *
 *****************************************************************************/
int AxiDma_Send(ttUserInterface Handle, ttUserPacketPtr PktPtr){
    XStatus                     Status;
    XAxiDma_Bd *                BdPtr;
    XAxiDma_BdRing *            TxRingPtr;
    TemacDevData *              Mac;
    ttDevSendOffloadStruct *    CsumOff;
    char                        csumEnable;

    Mac = (TemacDevData*)tfDeviceGetPointer(Handle);

/*
 * Free any Tx Buffers still held by the hardware. Do this so no TX
 * Interrupt is used.
 */
    AxiDma_FreeTxBuffer(Handle);

/* For scatter send, we do it only through ScatSend() */
    if ((PktPtr->pktuLinkExtraCount > 1) || (PktPtr->pktuLinkNextPtr != NULL))
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Send(): invalid Extra Count = %d",
                        PktPtr->pktuLinkExtraCount);
        return(TM_DEV_ERROR);
    }

    TxRingPtr = XAxiDma_GetTxRing(&(Mac->DmaInst));

/* Allocate a single BD, prepare it */
    Status = XAxiDma_BdRingAlloc(TxRingPtr, 1, &BdPtr);

    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Send(): error allocating TxBD, code %d", Status);
        return(TM_DEV_ERROR);
    }

/*
 * Set the BD data and length pointers, be sure to flush the buffer from the
 * cache. The BD will be flushed in the ToHw call, so foggetaboutit.
 */
    cacheFlush(PktPtr->pktuLinkDataPtr, PktPtr->pktuLinkDataLength);
    XAxiDma_BdSetBufAddr(BdPtr, (Xuint32)PktPtr->pktuLinkDataPtr);
    XAxiDma_BdSetLength(BdPtr, PktPtr->pktuLinkDataLength);

/* Store the packet identifier */
    XAxiDma_BdSetId(BdPtr, PktPtr);

/* Set the start and end of packet flag one buffer per BD */
    XAxiDma_BdSetCtrl(BdPtr, (XAXIDMA_BD_CTRL_TXSOF_MASK | XAXIDMA_BD_CTRL_TXEOF_MASK));

/* If checksum offloading, then setup the 1st descriptor */
    CsumOff = PktPtr->pktuDevOffloadPtr;
    csumEnable = 0;
    if (   (CsumOff)
        && (CsumOff->devoFlags & (  TM_DEVO_IP_CHKSUM_OFFLOAD
                                  | TM_DEVO_TCP_CHKSUM_OFFLOAD
                                  | TM_DEVO_UDP_CHKSUM_OFFLOAD )) )
    {
/* Enable hardware checksum computation for the buffer descriptor */
        if (XAxiEthernet_IsTxFullCsum(&Mac->TemacInst))
        {
            csumEnable = CSUM_ENABLE_FULL;
        }
        else if (XAxiEthernet_IsTxPartialCsum(&Mac->TemacInst))
        {
            csumEnable = CSUM_ENABLE_PARTIAL;
        }
    }

    if (csumEnable != 0)
    {
        XAxiDma_BdWrite((BdPtr),
                        XAXIDMA_BD_USR0_OFFSET,
                        (  XAxiDma_BdRead((BdPtr), XAXIDMA_BD_USR0_OFFSET)
                         | csumEnable) );

/* Write Start Offset and Insert Offset into BD */
        XAxiDma_BdWrite(BdPtr,
                        XAXIDMA_BD_USR1_OFFSET,
                        (CsumOff->devoTucss << 16) | CsumOff->devoTucso);

/* Write 0, as the seed value, to the BD */
        XAxiDma_BdWrite(BdPtr, XAXIDMA_BD_USR2_OFFSET, 0);
    }
    else
    {
        XAxiDma_BdWrite((BdPtr),
                        XAXIDMA_BD_USR0_OFFSET,
                        (  XAxiDma_BdRead((BdPtr), XAXIDMA_BD_USR0_OFFSET)
                         & ~(CSUM_ENABLE_FULL | CSUM_ENABLE_PARTIAL)));
        DEBUG_LOG(DEBUG_TX, "Checksum offload is disabled on this Tx packet\r\n");
    }


/* Enqueue the descriptor */
    Status = XAxiDma_BdRingToHw(TxRingPtr, 1, BdPtr);

    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Send(): error committing TxBD, code %d", Status);
        Status = XAxiDma_BdRingUnAlloc(TxRingPtr, 1, BdPtr);
        return(TM_DEV_ERROR);
    }

    return(TM_DEV_OKAY);
}


/******************************************************************************
 * Refill the receive BD ring and give the buffers to the DMA hardware.
 *
 * Parameters:
 *   Handle - User interface object
 *
 * Returns:
 *   TM_DEV_ERROR if an error occured setting up the transmission of the frame
 *   TM_DEV_OKAY if frame transmission was kicked off without problems
 *
 *****************************************************************************/
int AxiDma_RefillRxRing(ttUserInterface Handle){
    XStatus Status;
    XAxiDma_Bd *Bd1stPtr, *BdCurPtr;
    XAxiDma_BdRing * RxRingPtr;
#ifndef XLNX_USE_DRV_BUF_MEM
    ttUserBufferPtr BufHandlePtr;
#endif
    int BdsProcessed, BdsLeft, RxBufLen;
    char TM_FAR *Buf;
    TemacDevData * Mac;

    Mac = (TemacDevData*)tfDeviceGetPointer(Handle);

    RxRingPtr = XAxiDma_GetRxRing(&(Mac->DmaInst));

    /* Allocate all available free RxBDs */
    BdsLeft = XAxiDma_BdRingGetFreeCnt(RxRingPtr);
    DEBUG_LOG(DEBUG_RX, "AxiDma_RefillRxRing() BdsLeft=%d", BdsLeft);
    if (BdsLeft == 0)
    {
        DEBUG_LOG(DEBUG_RX, "AxiDma_RefillRxRing(): No more RxBDs");
        return(TM_DEV_OKAY);
    }

    Status = XAxiDma_BdRingAlloc(RxRingPtr, BdsLeft, &Bd1stPtr);
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_RefillRxRing(): Error allocating, code %d",
                  Status);
        return(TM_DEV_ERROR);
    }

    /* Get the buffer length based on the interface MTU. MTU is the maximum
     * packet size
     */
#ifdef USE_SMALL_DRV_BUFFER
    RxBufLen = RX_SCAT_BUF_LEN;
#else
    RxBufLen = IfaceMtu;
    RxBufLen += ETHER_HEADER_SIZE;
#endif

    for (BdsProcessed = 0, BdCurPtr = Bd1stPtr; BdsProcessed < BdsLeft; )
    {
#ifndef XLNX_USE_DRV_BUF_MEM
        /* Get a buffer handle from the array
         * The drive maintains the buffer handles. GetEthernetBuffer will fill
         * in the BufHandle structure.
         */
        BufHandlePtr = RX_BUFHANDLE_PTR();

        /*
         * Changed for scatter recieve, don't get the maxumum sized buffer
         * Buf = tfGetEthernetBuffer(BufHandlePtr); only supports 1518 byte
         * buffers. To do jumbo frames, we must use tfGetDriverBuffer.
         *
         * The RxBufLen will be the MTU, or the SCAT_BUF_LEN.
         *
         * Align the buffer on a 4 byte boundary.
         */
        Buf = tfGetCacheSafeDrvBuf(Handle,
                                   BufHandlePtr,
                                   RxBufLen,
                                   4,
                                   0);
#else /* !XLNX_USE_DRV_BUF_MEM */
        Buf = xbuf_alloc(RxBufLen);
#endif /* !XLNX_USE_DRV_BUF_MEM */

        if(NULL == Buf){
            DEBUG_LOG(DEBUG_MISC, "AxiDma_RefillRxRing(): Get Buffer failed %d Bd Processed out of %d", BdsProcessed, BdsLeft);
            Mac->EthStats.OutOfMem++;
            break;
        }

        //DEBUG_LOG(DEBUG_MEM, "AxiDma_RefillRxRing(): Buffer pointer 0x%08x, Length %d", Buf,RxBufLen);
        /* Add 2 to align the IP header, take 2 from the length. */
        Buf +=2;

        XAxiDma_BdSetBufAddr(BdCurPtr, (u32)Buf);
        XAxiDma_BdSetLength(BdCurPtr, RxBufLen-2);

#ifndef XLNX_USE_DRV_BUF_MEM
        /* Keep the handle to use later for free and RX */
        XAxiDma_BdSetId(BdCurPtr, BufHandlePtr);

        /* Move to the next buffer handle in the array */
        RX_BUFHANDLE_NEXT();
        DEBUG_LOG(DEBUG_MEM, "AxiDma_RefillRxRing(): Buffer Index %d", RxTreckBufferIndex);
#endif /* !XLNX_USE_DRV_BUF_MEM */

        /*
         * TEST FOR SCATTER RX BUG
         * Set the SOP and EOP flag in every buffer, not just the first and
         * last!
        XAxiDma_BdSetCtrl(BdCurPtr,
                    (XAXIDMA_BD_STS_RXSOF_MASK | XAXIDMA_BD_STS_RXEOF_MASK));
         */
        XAxiDma_BdSetCtrl(BdCurPtr, 0);

#ifdef XLNX_USE_DRV_BUF_MEM
        cacheInvalidate(Buf, RxBufLen-2);
#endif /* XLNX_USE_DRV_BUF_MEM */

        BdCurPtr = XAxiDma_BdRingNext(RxRingPtr, BdCurPtr);
        BdsProcessed++;

    }

    /* Set the End of Packet flag in the Last BD */
    if (BdsProcessed == 1){
        XAxiDma_BdSetCtrl(Bd1stPtr,
                    (XAXIDMA_BD_STS_RXSOF_MASK | XAXIDMA_BD_STS_RXEOF_MASK));
    }
    else{
        /* Set the Start of Packet flag in the first BD */
        XAxiDma_BdSetCtrl(Bd1stPtr, XAXIDMA_BD_STS_RXSOF_MASK);

        XAxiDma_BdSetCtrl(XAxiDma_BdRingPrev(RxRingPtr, BdCurPtr), XAXIDMA_BD_STS_RXEOF_MASK);
    }

    BdsLeft -= BdsProcessed;
    DEBUG_LOG(DEBUG_RX, "AxiDma_RefillRxRing(): Enqueued %d RxBDs", BdsProcessed);

    if (BdsProcessed)
    {
        Status = XAxiDma_BdRingToHw(RxRingPtr, BdsProcessed, Bd1stPtr);
        if (Status != XST_SUCCESS)
        {
            DEBUG_LOG(DEBUG_ERROR, "AxiDma_RefillRxRing(): Error handing to HW, code %d",
                      Status);
            return(TM_DEV_ERROR);
        }

    }
    else
    {
        DEBUG_LOG(DEBUG_RX, "AxiDma_RefillRxRing(): Out of buffers, suspended Rx channel");
        XAxiDma_BdRingIntDisable(&Mac->DmaInst.RxBdRing, XAXIDMA_IRQ_ALL_MASK);
    }

    if (BdsLeft)
    {
        Status = XAxiDma_BdRingUnAlloc(RxRingPtr, BdsLeft, BdCurPtr);
        if (Status != XST_SUCCESS)
        {
            DEBUG_LOG(DEBUG_ERROR, "AxiDma_RefillRxRing(): Error unalloc, code %d",
                      Status);
            return(TM_DEV_ERROR);
        }
    }

    return(TM_DEV_OKAY);

}

/******************************************************************************
 * Free the buffer owned by the driver. The Treck stack is done with it.
 *
 * Parameters:
 *   Handle - User interface object
 *   dataPtr - A pointer to the buffer to be freed
 *
 * Returns:
 *   TM_DEV_ERROR if an error occured setting up the transmission of the frame
 *   TM_DEV_OKAY if frame transmission was kicked off without problems
 *
 *****************************************************************************/
int AxiDma_BufFree(ttUserInterface Handle, char TM_FAR *dataPtr){
    /* Just free the buffer and be done
     * Subtract the two bytes we added to get the IP header aligned. */

    DEBUG_LOG(DEBUG_RX, "AxiDma_BufFreee(): Buffer: 0x%08x", dataPtr-2);
    xbuf_free(dataPtr-2);

    return(TM_DEV_OKAY);
}

/******************************************************************************
 * Transfer one packet from the driver to the stack. The SgRxISR function
 * should have saved off a list of BDs for this function to work on. Once that
 * list has been processed, free the BD resources.
 *
 * Parameters:
 *   Handle - User interface object
 *   DataPtr - Pointer to next packet received
 *   DataLen - Length of next packet received
 *   BufHandle - Not used, set to 0
 *   Csum - Checksum offload information from HW
 *
 * Returns:
 *   TM_DEV_ERROR if an error occured processing the received packet
 *   TM_DEV_OKAY if frame info successfully provided to the stack
 *
 *****************************************************************************/
int AxiDma_Recv(ttUserInterface Handle, char TM_FAR** DataPtr,
                  int TM_FAR* DataLen, ttUserBufferPtr BufHandle,
                  ttDevRecvOffloadPtr Csum) {
    TemacDevData *Mac = (TemacDevData*)tfDeviceGetPointer(Handle);
    XAxiDma_BdRing * RxRingPtr;
    XStatus Status;
    Xuint32 BufAddr, BufLen;
    Xuint32 IntStatus, StsCtl;
    int RxError = FALSE;
    Xuint16 rxCsumStatus;

    DEBUG_LOG(DEBUG_RX, "AxiDma_Recv()");

    /* This is called once for each packet received. Recv always works with the
     * head of the Bd queue.
     * There is only one buffer per packet, and one Bd per buffer.
     */

    if(NULL == RxBdCurPtr){
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv Current Pointer is NULL!!!");
        Mac->EthStats.RxErrors++;
        return(TM_DEV_ERROR);
    }

    /* Make sure the DMA did not have any error before pulling data from the BD
     */
    StsCtl = XAxiDma_BdGetSts(RxBdCurPtr);

    if((SINGLE_PKT_RX_COMPLETE_FLAGS != (StsCtl & SINGLE_PKT_RX_COMPLETE_FLAGS)) ||
                    (StsCtl & XAXIDMA_BD_STS_ALL_ERR_MASK)){
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv Status Control Not complete or scattered 0x%08x, Count %d", StsCtl, RxBdCurCnt);
        /*
         * Need to recover the buffer here and clean up gracefully.
         * Free the buffer below, move to the next descriptor and all should be
         * OK.
         */

        RxError = TRUE;
    }

    DEBUG_LOG(DEBUG_RX,"AxiDma_recv: Frames=%d, RxBDCurPtr 0x%08x", RxBdCurCnt, RxBdCurPtr);
    /* Retrieve buffer info from descriptor */
    BufAddr = XAxiDma_BdGetBufAddr(RxBdCurPtr);

    /*
     * The packet length returned from XAxiDma_BdRead is very large.
     * For a 60 byte packet 0x04500003c was being returned instead of 0x3c
     * Mask off the 16 MSbs - tjc
     */

    BufLen = (0xffff & (XAxiDma_BdRead(RxBdCurPtr, XAXIDMA_BD_USR4_OFFSET)));

    *DataPtr = (char TM_FAR *)BufAddr;
    *DataLen = (int)BufLen;
    //DEBUG_LOG(DEBUG_MEM,"AxiDma_Recv: DataPtr 0x%08x", *DataPtr);

#ifdef XLNX_USE_DRV_BUF_MEM
    *BufHandle = NULL;
    if((RxError) && (NULL != *DataPtr)){
        /* Free the buffer */
        xbuf_free(*DataPtr-2);
    }
#else
    /* Tell stack Treck owns this buffer
     * Copy the User buffer information into the user buffer provided by the
     * stack
     */
    *BufHandle = *(ttUserBufferPtr)XAxiDma_BdGetId(RxBdCurPtr);
    if((RxError) && (NULL != *BufHandle)){
        tfFreeDriverBuffer(*BufHandle);
    }
#endif

    if (BufLen <= 60) {
        DEBUG_LOG(DEBUG_RX, "AxiDma_Recv small packet (%d bytes)\r\n", BufLen);
    }

    if (BufLen > 9020) {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv: Data Length huge %d", BufLen);
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv Status 0x%08x",
                        XAxiDma_BdGetSts(RxBdCurPtr));
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv Control 0x%08x",
                        XAxiDma_BdGetCtrl(RxBdCurPtr));
    }

    /* Extract checksum data from descriptor if supported */
    if (XAxiEthernet_IsRxFullCsum(&Mac->TemacInst))
    {
/*
 * rxCsumStatus values:
 * 000: IP/TCP/UDP=unverified
 * 001: IP=good, TCP/UDP=unverified
 * 010: IP=good, TCP=good
 * 011: IP=good, UDP=good
 * 100: Reserved
 * 101: IP=BAD, TCP/UDP=unverified
 * 110: IP=good, TCP=BAD
 * 111: IP=good, UDP=BAD
 */
        rxCsumStatus = (XAxiDma_BdRead((RxBdCurPtr), XAXIDMA_BD_USR2_OFFSET) >> 3) & 0x7;
        switch (rxCsumStatus)
        {
/*
 *      case 0x0:
 *          break;
 */
        case 0x1:
            Csum->devoRFlags =   TM_DEVOR_IP_RECV_CHKSUM_VALIDATED;
            break;
        case 0x2:
            Csum->devoRFlags =   TM_DEVOR_IP_RECV_CHKSUM_VALIDATED
                               | TM_DEVOR_TCP_RECV_CHKSUM_VALIDATED;
            break;
        case 0x3:
            Csum->devoRFlags =   TM_DEVOR_IP_RECV_CHKSUM_VALIDATED
                               | TM_DEVOR_UDP_RECV_CHKSUM_VALIDATED;
            break;
/*
 * Reserved
 *      case 0x4:
 *          break;
 */
        case 0x5:
            Csum->devoRFlags =   TM_DEVOR_IP_RECV_CHKSUM_FAILED;
            break;
        case 0x6:
            Csum->devoRFlags =   TM_DEVOR_IP_RECV_CHKSUM_VALIDATED
                               | TM_DEVOR_TCP_RECV_CHKSUM_FAILED;
            break;
        case 0x7:
            Csum->devoRFlags =   TM_DEVOR_IP_RECV_CHKSUM_VALIDATED
                               | TM_DEVOR_UDP_RECV_CHKSUM_FAILED;
            break;
        default:
            break;
        }
    }
    else if (BufLen > 64 && (XAxiEthernet_IsRxPartialCsum(&Mac->TemacInst)))
    {
        Csum->devoRFlags = TM_DEVOR_PACKET_RECV_CHKSUM_COMPUTED;
        Csum->devoRecvXsum = ~(XAxiDma_BdRead((RxBdCurPtr), XAXIDMA_BD_USR3_OFFSET));
        Csum->devoRecvXsum = Xil_EndianSwap16(Csum->devoRecvXsum);
        Csum->devoRecvXsumOffset = 14;
        Csum->devoRecvXsumLength = BufLen - 14;
    }
    else
    {
        DEBUG_LOG(DEBUG_RX, "AxiDma_Recv: Csum Disabled on packet");
        Csum->devoRFlags = 0;
    }

    /* Disable receive channel interrupts while the counts and pointers are
     * moved to the next Bd to process
     */

    RxRingPtr = XAxiDma_GetRxRing(&(Mac->DmaInst));

    IntStatus = XAxiDma_BdRingIntGetEnabled(RxRingPtr);
    XAxiDma_BdRingIntDisable(RxRingPtr, XAXIDMA_IRQ_ALL_MASK);

    /* Free the BD to the Ring list, and move to next descriptor */


    DEBUG_LOG(DEBUG_RX, "AxiDma_Recv: Freeing 1 of %d Bd's", RxBdCurCnt);
    Status = XAxiDma_BdRingFree(RxRingPtr, 1, RxBdCurPtr);
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv: Error freeing BDs, code %d", Status);
        Mac->EthStats.RxErrors++;
/* Restore the interrupt state */
        XAxiDma_BdRingIntEnable(&Mac->DmaInst.RxBdRing, IntStatus);
        return(TM_DEV_ERROR);
    }

    RxBdCurPtr = XAxiDma_BdRingNext(RxRingPtr, RxBdCurPtr);
    RxBdCurCnt--;

    /* Restore the interrupt state */
    XAxiDma_BdRingIntEnable(&Mac->DmaInst.RxBdRing, IntStatus);

    RxBdCnt++;

    /* Was this the last descriptor in the current processing list */
    if ((RxBdCnt >= RX_BD_FREE_THRESHOLD) || (RxBdCurCnt == 0) ){

        RxBdCnt = 0;

        /* Refill buffers in the DMA engine */
        if (AxiDma_RefillRxRing(Handle) < 0)
        {
            DEBUG_LOG(DEBUG_ERROR, "AxiDma_Recv: AxiDma_RefillRxRing failed");
        }

        /* Re-enable receive channel interrupts */
        IntStatus = XAXIDMA_IRQ_ALL_MASK;
    }

    /* Restore the interrupt state */
    XAxiDma_BdRingIntEnable(&Mac->DmaInst.RxBdRing, IntStatus);


    if(RxError){
        return(TM_DEV_ERROR);
        Mac->EthStats.RxErrors++;
    }

    Mac->EthStats.RxFrames++;
    return(TM_DEV_OKAY);
}

/******************************************************************************
 * Handle SGDMA transmit interrupts
 *
 * Parameters:
 *   Callback - Device instance
 *****************************************************************************/
static void SgTxISR(void *Callback) {
    /* This function should not be called */
    DEBUG_LOG(DEBUG_ERROR,"SgTxISR: was invoked");
}

/******************************************************************************
 * Handle SGDMA receive interrupts
 *
 * Parameters:
 *   Callback - Device instance
 *****************************************************************************/

void SgRxISR(void *Callback) {
    ttUserInterface *TreckUI = (ttUserInterface*)Callback;
    TemacDevData * Mac;
    XAxiDma_Bd *BdPtr;
    XAxiDma_BdRing *RxRingPtr;
    int NumFrames;
    u32 IrqStatus;

    Mac = (TemacDevData*)tfDeviceGetPointer(TreckUI);

    RxRingPtr = XAxiDma_GetRxRing(&(Mac->DmaInst));

    /* Read pending interrupts */
    IrqStatus = XAxiDma_BdRingGetIrq(RxRingPtr);

    /* Acknowledge pending interrupts */
    XAxiDma_BdRingAckIrq(RxRingPtr, IrqStatus);

    /* If error interrupt is asserted, raise error flag, reset the
     * hardware to recover from the error, and return with no further
     * processing.
     */
    if ((IrqStatus & XAXIDMA_ERR_ALL_MASK)) {
        DEBUG_LOG(DEBUG_ERROR, "SgRxISR: Error: IrqStatus & XAXIDMA_ERR_ALL_MASK");
        xil_printf("SgRxISR error: Resetting DMA, Status: 0x%08x\n\r",
        XAxiDma_ReadReg(Mac->DmaInst.RegBase, XAXIDMA_SR_OFFSET + XAXIDMA_RX_OFFSET));
        XAxiDma_Reset(&Mac->DmaInst);
        return;
    }

    /*
     * If Reception done interrupt is asserted, call RX call back function
     * to handle the processed BDs and then raise the according flag.
     */
    if (!(IrqStatus & (XAXIDMA_IRQ_DELAY_MASK | XAXIDMA_COALESCE_MASK))) {
        return;
    }

    DEBUG_LOG(DEBUG_RX, "Temac: SgRxISR()");

    /* Retrieve the completed BDs from HW */
    NumFrames = XAxiDma_BdRingFromHw(RxRingPtr, XAXIDMA_ALL_BDS, &BdPtr);

    /* If frames were received, save the descriptor list and disable
     * receive interrupts. The task level receive function will consume
     * the descriptors. When it has gone through them all, then they will
     * all be released.
     */
    if (NumFrames > 0)
    {
        DEBUG_LOG(DEBUG_RX,"SgRxISR: Frames=%d, First BDPtr 0x%08x", NumFrames, BdPtr);
        if (NULL == RxBdCurPtr){
            /* If the Bd list is empty, make it the BdPtr. If it is not, the
             * Bd's are appended thought the count mechanism.
             */
            DEBUG_LOG(DEBUG_RX,"SgRxISR: Set RxBdCurPtr");
            RxBdCurPtr = BdPtr;
        }

        RxBdCurCnt += NumFrames;
        Mac->EthStats.RxISRFrames += NumFrames;

        tfNotifyInterfaceIsr(TreckUI, NumFrames, 0, 0, 0);

        /*
        * Disable the receive related interrupts
        */
        if (RxBdCurCnt > RX_BD_IRPTDIS_THRESHOLD){
            XAxiDma_BdRingIntDisable(&Mac->DmaInst.RxBdRing, XAXIDMA_IRQ_ALL_MASK);
        }
    }

}

/******************************************************************************
 * Ioctl command for this device
 *
 * Parameters:
 *   Handle - User interface object
 *   Flag - Which ioctl command to execute
 *   OptionPtr (input/output) - Data associated with Flag
 *   OptionLen - Length of datum pointed to by OptionPtr
 *
 * Returns:
 *   TM_DEV_ERROR if error executing command or command doesn't exist
 *   TM_DEV_OKAY if command executed without error
 *****************************************************************************/
int AxiDma_Ioctl(ttUserInterface    Handle,
                 int                Flag,
                 void TM_FAR *      OptionPtr,
                 int                OptionLen)
{
    TemacDevData *              Mac;
    TemacCoalesce *             CoalescePtr;
    ttDevIoctlOffloadStruct *   CsumOff;
    XStatus                     Status;
    int                         retCode;


    Mac = (TemacDevData *)tfDeviceGetPointer(Handle);

    DEBUG_LOG(DEBUG_MISC, "Temac: Ioctl() Flag=%d", Flag);

    retCode = TM_DEV_OKAY;

    switch(Flag)
    {
        case TM_DEV_IOCTL_OFFLOAD_GET:
            if (OptionPtr == NULL)
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Null OptionPtr");
                retCode = TM_DEV_ERROR;
                break;
            }

            CsumOff = (ttDevIoctlOffloadStruct*)OptionPtr;
            CsumOff->offloadFlags = 0;

            if (XAxiEthernet_IsTxFullCsum(&Mac->TemacInst))
            {
                CsumOff->offloadFlags |=
                    (  TM_DEVO_IP_CHKSUM_OFFLOAD
                     | TM_DEVO_TCP_CHKSUM_OFFLOAD
                     | TM_DEVO_UDP_CHKSUM_OFFLOAD );
            }
            if (XAxiEthernet_IsTxPartialCsum(&Mac->TemacInst))
            {
                CsumOff->offloadFlags |=
                    (  TM_DEVO_TCP_CHKSUM_OFFLOAD
                     | TM_DEVO_UDP_CHKSUM_OFFLOAD );
            }
            if (XAxiEthernet_IsRxFullCsum(&Mac->TemacInst))
            {
                CsumOff->offloadFlags |=
                    (  TM_DEVO_IP_RECV_CHKSUM_VALIDATE
                     | TM_DEVO_TCP_RECV_CHKSUM_VALIDATE
                     | TM_DEVO_UDP_RECV_CHKSUM_VALIDATE );
            }
            if (XAxiEthernet_IsRxPartialCsum(&Mac->TemacInst))
            {
                CsumOff->offloadFlags |= TM_DEVO_PACKET_RECV_CHKSUM_COMPUTE;
            }
            break;

        case TM_DEV_IOCTL_OFFLOAD_SET:
            if (OptionPtr == NULL)
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Null OptionPtr");
                retCode = TM_DEV_ERROR;
                break;
            }

            CsumOff = (ttDevIoctlOffloadStruct*)OptionPtr;

            if (   (XAxiEthernet_IsTxFullCsum(&Mac->TemacInst) == 0)
                && (XAxiEthernet_IsTxPartialCsum(&Mac->TemacInst) == 0)
                && (CsumOff->offloadFlags &
                        (  TM_DEVO_IP_CHKSUM_OFFLOAD
                         | TM_DEVO_TCP_CHKSUM_OFFLOAD
                         | TM_DEVO_UDP_CHKSUM_OFFLOAD )) )
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): TX Checksum unavailable");
                retCode = TM_DEV_ERROR;
                break;
            }
            else if (   (XAxiEthernet_IsTxFullCsum(&Mac->TemacInst) == 0)
                     && (CsumOff->offloadFlags & TM_DEVO_IP_CHKSUM_OFFLOAD) )
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Full TX Checksum unavailable");
                retCode = TM_DEV_ERROR;
                break;
            }
            else if (   (XAxiEthernet_IsRxFullCsum(&Mac->TemacInst) == 0)
                     && (XAxiEthernet_IsRxPartialCsum(&Mac->TemacInst) == 0)
                     && (CsumOff->offloadFlags &
                            (  TM_DEVO_PACKET_RECV_CHKSUM_COMPUTE
                             | TM_DEVO_IP_RECV_CHKSUM_VALIDATE
                             | TM_DEVO_TCP_RECV_CHKSUM_VALIDATE
                             | TM_DEVO_UDP_RECV_CHKSUM_VALIDATE )) )
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): RX Checksum unavailable");
                retCode = TM_DEV_ERROR;
                break;
            }
            else if (   (XAxiEthernet_IsRxFullCsum(&Mac->TemacInst) == 0)
                    && (CsumOff->offloadFlags &
                            (  TM_DEVO_IP_RECV_CHKSUM_VALIDATE
                             | TM_DEVO_TCP_RECV_CHKSUM_VALIDATE
                             | TM_DEVO_UDP_RECV_CHKSUM_VALIDATE )) )
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Full RX Checksum unavailable");
                retCode = TM_DEV_ERROR;
                break;
            }
            else if (   (XAxiEthernet_IsRxPartialCsum(&Mac->TemacInst) == 0)
                     && (CsumOff->offloadFlags & TM_DEVO_PACKET_RECV_CHKSUM_COMPUTE) )
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Partial RX Checksum unavailable");
                retCode = TM_DEV_ERROR;
                break;
            }
            break;

        case TM_TEMAC_IOCTL_COAL_SET:
            if (!XAxiEthernet_IsDma(&Mac->TemacInst))
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Asked to set coalescing, don't have capability");
                retCode = TM_DEV_ERROR;
                break;
            }

            CoalescePtr = (TemacCoalesce*)OptionPtr;
            XAxiEthernet_Stop(&Mac->TemacInst);
            Status = XAxiDma_BdRingSetCoalesce(&Mac->DmaInst.TxBdRing,
                                          CoalescePtr->TxThreshold,
                                          CoalescePtr->TxWaitbound);
            Status |= XAxiDma_BdRingSetCoalesce(&Mac->DmaInst.RxBdRing,
                                           CoalescePtr->RxThreshold,
                                           CoalescePtr->RxWaitbound);
            //XAxiEthernet_Start(&Mac->TemacInst);
            if (Status != XST_SUCCESS)
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl() COAL_SET error %d", Status);
                retCode = TM_DEV_ERROR;
            }
            break;

        case TM_TEMAC_IOCTL_COAL_GET:
            if (!XAxiEthernet_IsDma(&Mac->TemacInst))
            {
                DEBUG_LOG(DEBUG_ERROR, "Ioctl(): Asked to get coalescing, don't have capability");
                retCode = TM_DEV_ERROR;
                break;
            }

            CoalescePtr = (TemacCoalesce*)OptionPtr;
            XAxiDma_BdRingGetCoalesce(&Mac->DmaInst.TxBdRing,
                                          &CoalescePtr->TxThreshold,
                                          &CoalescePtr->TxWaitbound);
            XAxiDma_BdRingGetCoalesce(&Mac->DmaInst.RxBdRing,
                                           &CoalescePtr->RxThreshold,
                                           &CoalescePtr->RxWaitbound);
            break;

        case TM_TEMAC_SET_LINK_SPEED:
            marvell_phy_set_link(*(int*)OptionPtr, DUPLEX_FULL);
            break;

        case TM_TEMAC_IOCTL_SEND_COMPLETE:
            AxiDma_FreeTxBuffer(Handle);
            break;

        default:
            DEBUG_LOG(DEBUG_ERROR, "Ioctl(): unknown cmd %d", Flag);
            retCode = TM_DEV_ERROR;
            break;
    }

    return retCode;
}

/*
 * Utility Functions
 */

int DmaUtil_DumpStats(XAxiDma* DmaInst){
    XAxiDma_BdRing * RingPtr;

    RingPtr = XAxiDma_GetTxRing(DmaInst);

    xil_printf("Tx DMA Stats\n\r");
    xil_printf("Tx DMA Bd Free Count %d\n\r",XAxiDma_BdRingGetFreeCnt(RingPtr));

    RingPtr = XAxiDma_GetRxRing(DmaInst);

    xil_printf("Rx DMA Stats\n\r");
    xil_printf("Rx DMA Bd Free Count %d\n\r",XAxiDma_BdRingGetFreeCnt(RingPtr));

    xil_printf("Rx DMA Bd CurCount %d, RxCount %d\n\r",RxBdCurCnt,RxBdCnt);
    return(0);
}
