/* $Id: */
/********************************************************************
 * Copyright(C) 2008 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times.
 * (c) Copyright 2008 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/******************************************************************************
*
* xtr_lltemac.c - Temac specific initialization and setup functions.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1     drg  04/04/08 First release
* 2     pdmc 03/10/11 Modified for AXI bus
*
*****************************************************************************/

/***************************** Include Files ********************************/
#include "xtr_axiethernet.h"
/* Included in /\
#include <trsocket.h>
#include "xlltemac.h"
 */
#include "xtr_axidma.h"
#include "util_time.h"
#include "debug.h"
#include "defaults.h"
#include "marvell_88e1111.h"

#ifndef XLNX_USE_DRV_BUF_MEM
#include "cache.h"
#endif /* !XLNX_USE_DRV_BUF_MEM */

/************************** Constant Definitions ****************************/

/**************************** Type Definitions ******************************/

/***************** Macros (Inline Functions) Definitions ********************/

/************************** Function Prototypes *****************************/
static void ErrISR(void *Callback);
int    AxiTemac_GetDeviceAddr(ttUserInterface Handle, char TM_FAR * PhysAddr);

/************************** Variable Definitions ****************************/

TemacDevData XpsTemac;    /* The Temac instance structures and setup data */
char MacAddr[] = {DEFAULT_MACADDR};
int PhyAddr;
int Err;

/************************** Function Definitions ****************************/

/****************************************************************************
 * Error handler used for all interrupt driven modes.
 *
 * Parameters:
 *   Callback - Device instance
 *   ErrorClass - Info from TEMAC L1 driver denoting the type of error
 *   Word1 - ErrorClass specific
 *   Word2 - ErrorClass specific
 ****************************************************************************/
static void ErrISR(void *Callback)
{
    TemacDevData *Mac = (TemacDevData*)Callback;
    Xuint32 Pending = XAxiEthernet_IntPending(&Mac->TemacInst);

    DEBUG_LOG(DEBUG_ERROR, "Temac: ErrISR()");
    Mac->EthStats.TemacErr++;
    XAxiEthernet_IntClear(&Mac->TemacInst, Pending);
}


/*
 * Setup the TEMAC for use. This is done after initialization of Ethernet.
 */
int AxiTemac_Setup(TemacDevData * Mac)
{
    int     Status;
    Xuint32 Rdy;

    /* Make sure the hard TEMAC is ready */
    do
    {
        Rdy = XAxiEthernet_ReadReg(Mac->TemacInst.Config.BaseAddress, XAE_IS_OFFSET);
    }
    while ((Rdy & XAE_INT_MGTRDY_MASK) == 0);

    /*
     * Make sure Tx and Rx are enabled
     */
    Status = XAxiEthernet_SetOptions(&Mac->TemacInst,
                                     (  XAE_RECEIVER_ENABLE_OPTION
                                      | XAE_TRANSMITTER_ENABLE_OPTION
                                      | XAE_BROADCAST_OPTION) );
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "Open: Error setting options, code %d\n", Status);
        return(TM_DEV_ERROR);
    }

    /* Assign our address and default link speed */
    /* Set the MAC Address of the device */
    XAxiEthernet_SetMacAddress(&Mac->TemacInst, MacAddr);


    /* Allow the PHY to auto-negotiate the link speed */
    PhyAddr = DEFAULT_PHY_ADDR;
    marvell_phy_autonegotiate();

    /* Install callbacks */
    Status = Setup_Interrupt(Mac->TemacInst.Config.TemacIntr,
                             (XInterruptHandler)ErrISR,
                             Mac);
    if (Status != XST_SUCCESS)
    {
        DEBUG_LOG(DEBUG_ERROR, "Open: Failed to install interrupt handler");
        return(Status);
    }

    XAxiEthernet_IntEnable(&Mac->TemacInst, XAE_INT_RECV_ERROR_MASK);

    memset(&XpsTemac.EthStats, 0, sizeof(XpsTemac.EthStats));

    return(Status);
}

/****************************************************************************/
/**
 * Get the Physical MAC address
 *
 * @param ttUserInterface Treck User Interface Handle
 * @param Physical Address a pointer to a char buffer for the PHY address
 *
 * @return
 *   0 if successful
 *   TM_DEV_ERROR if failed
 ****************************************************************************/
int    AxiTemac_GetDeviceAddr(ttUserInterface Handle, char TM_FAR * PhysAddr){
    TemacDevData *Mac = (TemacDevData *)tfDeviceGetPointer(Handle);

    DEBUG_LOG(DEBUG_MISC, "Temac: PhysAddr()");

    XAxiEthernet_GetMacAddress(&Mac->TemacInst, PhysAddr);
    return(TM_DEV_OKAY);
}

/****************************************************************************/
/**
 * Read the given PHY register
 *
 * @param Reg PHY register to read
 * @param Data (output) Value read from register
 *
 * @return
 *   0 if successful
 *  -1 if init failed
 ****************************************************************************/
int Mac_PhyRead(Xuint16 Reg, Xuint16 *Data)
{
    XAxiEthernet_PhyRead(&XpsTemac.TemacInst, PhyAddr,
                         Reg, Data);
    return(0);
}

/****************************************************************************/
/**
 * Write the given PHY register
 *
 * @param Reg PHY register to write
 * @param Data Value to write to register
 *
 * @return
 *   0 if successful
 *  -1 if init failed
 *****************************************************************************/
int Mac_PhyWrite(Xuint16 Reg, Xuint16 Data)
{
    XAxiEthernet_PhyWrite(&XpsTemac.TemacInst, PhyAddr,
                          Reg, Data);
    return(0);
}

/******************************************************************************
 * TECK RELATED FUNCTIONS
 *****************************************************************************/

/******************************************************************************
 * Create 1 of three types of interfaces: 1) Scatter gather with enhanced
 * scat-send capability; 2) Scatter gather; 3) FIFO direct.
 *
 * Parameters:
 *   CfgPtr - TEMAC configuration table entry. It is used to determine
 *     capabilities of the device and to select which type of interface is
 *     created
 *
 *   DrvDataPtr - contains load-time information needed when the device
 *     is initialized and setup. Fill in LoadParam attributes prior to
 *     calling tfOpenInterface().
 *
 * Returns:
 *   User interface object if successful, NULL otherwise
 *****************************************************************************/
ttUserInterface Treck_CreateInterface(int Dma_Type)
{
    ttUserInterface TreckUI;
    ttUserLinkLayer LinkHandle;
#ifndef XLNX_USE_DRV_BUF_MEM
    int             errorCode;
#endif /* !XLNX_USE_DRV_BUF_MEM */

    LinkHandle = tfUseEthernet();

    TreckUI = NULL;

#ifndef TM_XILINX_USE_AXI4
    /* Based on the type of device, create a user interface object */
    if (XpsTemac.TemacInst.Config.LLDevType != XPAR_LL_DMA)
    {
        DEBUG_LOG(DEBUG_ERROR, "Temac_CreateInterface(): No Interface specified");
        return(NULL);
    }
#endif /* !TM_XILINX_USE_AXI4 */

    if (TRECK_USE_SIMPLE_DMA == Dma_Type){

        DEBUG_LOG(DEBUG_MISC, "Temac_CreateInterface(): Simple DMA");
        TreckUI = tfAddInterface("TEMAC.0", LinkHandle, AxiDma_Open,
                    AxiDma_Close, NULL, AxiDma_Recv, NULL, AxiDma_Ioctl,
                    AxiTemac_GetDeviceAddr, &Err);

        if (NULL == TreckUI ){
            DEBUG_LOG(DEBUG_ERROR, "Temac_CreateInterface(): error %d, %s", Err, tfStrError(Err));
            return(TreckUI);
        }

        /* The One Scat Send interface must be used to support checksum
         * offload */
        tfUseInterfaceOneScatSend(TreckUI, AxiDma_Send);
    }
    else
    {
        DEBUG_LOG(DEBUG_MISC, "Temac_CreateInterface(): Scatter Gather DMA");

#ifdef XLNX_USE_DRV_BUF_MEM
        TreckUI = tfAddInterface("TEMAC.0", LinkHandle, AxiDma_ScatOpen,
                    AxiDma_Close, NULL, AxiDma_Recv, AxiDma_BufFree, AxiDma_Ioctl,
                    AxiTemac_GetDeviceAddr, &Err);

#else /* XLNX_USE_DRV_BUF_MEM */
        TreckUI = tfAddInterface("TEMAC.0", LinkHandle, AxiDma_ScatOpen,
                    AxiDma_Close, NULL, AxiDma_Recv, NULL, AxiDma_Ioctl,
                    AxiTemac_GetDeviceAddr, &Err);
#endif /* XLNX_USE_DRV_BUF_MEM */

        if (NULL == TreckUI ){
            DEBUG_LOG(DEBUG_ERROR, "Temac_CreateInterface(): error %d, %s", Err, tfStrError(Err));
            return(TreckUI);
        }

        /* The One Scat Send interface must be used to support checksum
         * offload */
        tfUseInterfaceOneScatSend(TreckUI, AxiDma_ScatSend);

    } /* Dma_Type */

    /* Store the Temac information in the Treck User Interface Structure */
    tfDeviceStorePointer(TreckUI, &XpsTemac);

#ifndef XLNX_USE_DRV_BUF_MEM
/* Setup the cache functions */
    errorCode = tfUseInterfaceCacheInvalFunc(TreckUI,
                                             tfDevCacheInvalFuncCb,
                                             (ttUserGenericUnion)0);
    if (errorCode != TM_ENOERROR)
    {
        DEBUG_LOG(DEBUG_ERROR,"tfUseInterfaceCacheInvalFunc error %d\r\n", errorCode);
        return(errorCode);
    }
    errorCode = tfUseInterfaceCacheFlushFunc(TreckUI,
                                             tfDevCacheFlushFuncCb,
                                             (ttUserGenericUnion)0);
    if (errorCode != TM_ENOERROR)
    {
        DEBUG_LOG(DEBUG_ERROR,"tfUseInterfaceCacheFlushFunc error %d\r\n", errorCode);
        return(errorCode);
    }
#endif /* !XLNX_USE_DRV_BUF_MEM */


    return (TreckUI);
}

/*
 * Utility Functions
 */
int Util_PhyAddr(unsigned MenuIndex, unsigned TestLoops){

        if((TestLoops > 31)){
            xil_printf("ERROR: Current PHY address is %d, %d out of range.\n\r", PhyAddr, TestLoops);
            return(-1);
        }
        xil_printf("Current PHY address is %d, Changing to %d.\n\r", PhyAddr, TestLoops);
        PhyAddr = TestLoops;
        return(0);
}


int TemacUtil_DumpStats(){

    xil_printf("Temac dropped packets %d\n\r",XpsTemac.EthStats.TemacErr);
    xil_printf("Received Frames %d\n\r",XpsTemac.EthStats.RxFrames);
    xil_printf("Received ISR Frames %d\n\r",XpsTemac.EthStats.RxISRFrames);
    xil_printf("Received Error Frames %d\n\r",XpsTemac.EthStats.RxErrors);
    xil_printf("Transmitted Frames %d\n\r",XpsTemac.EthStats.TxFrames);
    xil_printf("Out of buffer memory %d\n\r",XpsTemac.EthStats.OutOfMem);
    xil_printf("Current PHY Address = %d\n\r",PhyAddr);
#ifdef TM_XILINX_USE_AXI4
    if ((XAxiEthernet_IsDma(&(XpsTemac.TemacInst))))
#else /* TM_XILINX_USE_AXI4 */
    if (XpsTemac.TemacInst.Config.LLDevType == XPAR_LL_DMA)
#endif /* TM_XILINX_USE_AXI4 */
    {
            return(DmaUtil_DumpStats(&XpsTemac.DmaInst));
    }
    xil_printf("\n\r\n\r");
    return(0);
}

