/* $Id: */
/********************************************************************
 * Copyright(C) 2008 by Xilinx, Inc. All rights reserved.
 * This text/file contains proprietary, confidential
 * information of Xilinx, Inc., is distributed under license
 * from Xilinx, Inc., and may be used, copied and/or
 * disclosed only pursuant to the terms of a valid license
 * agreement with Xilinx, Inc.  Xilinx hereby grants you a
 * license to use this text/file solely for design, simulation,
 * implementation and creation of design files limited
 * to Xilinx devices or technologies. Use with non-Xilinx
 * devices or technologies is expressly prohibited and
 * immediately terminates your license unless covered by
 * a separate agreement.
 *
 * Xilinx is providing this design, code, or information
 * as-is solely for use in developing programs and
 * solutions for Xilinx devices, with no obligation on the
 * part of Xilinx to provide support.  By providing this design,
 * code, or information as one possible implementation of
 * this feature, application or standard, Xilinx is making no
 * representation that this implementation is free from any
 * claims of infringement.  You are responsible for
 * obtaining any rights you may require for your implementation.
 * Xilinx expressly disclaims any warranty whatsoever with
 * respect to the adequacy of the implementation, including
 * but not limited to any warranties or representations that this
 * implementation is free from claims of infringement, implied
 * warranties of merchantability or fitness for a particular
 * purpose.
 *
 * Xilinx products are not intended for use in life support
 * appliances, devices, or systems. Use in such applications is
 * expressly prohibited.
 *
 * Any modifications that are made to the Source Code are
 * done at the user's sole risk and will be unsupported.
 * The Xilinx Support Hotline does not have access to source
 * code and therefore cannot answer specific questions related
 * to source HDL.
 *
 * This copyright and support notice must be retained as part
 * of this text at all times.
 * (c) Copyright 2008 Xilinx, Inc.
 * All rights reserved.
 ********************************************************************/

/******************************************************************************
*
* xps_lltemac.c - Temac specific initialization and setup functions.
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1     drg  04/04/08 First release
* 2     pdmc 03/10/11 Modified for AXI bus
*
*****************************************************************************/

#ifndef _XPS_LLTEMAC_H_
#define _XPS_LLTEMAC_H_

/***************************** Include Files ********************************/
#include "xaxiethernet.h"
#include "xaxidma.h"
#include "xllfifo.h"
#include <trsocket.h>

/************************** Constant Definitions ****************************/

#define XLNX_USE_DRV_BUF_MEM
#ifndef XLNX_USE_DRV_BUF_MEM
#ifndef TM_USE_DCACHE_MANAGEMENT
#error "You MUST define TM_USE_DCACHE_MANAGEMENT in order to use Treck's driver buffers!"
#endif /* !TM_USE_DCACHE_MANAGEMENT */
#endif /* !XLNX_USE_DRV_BUF_MEM */

/**************************** Type Definitions ******************************/
typedef struct {
    XAxiEthernet TemacInst;
    XAxiDma DmaInst;
    XLlFifo  FifoInst;
    struct {
        u32 RxFrames;
        u32 RxISRFrames;
        u32 RxErrors;
        u32 TxFrames;
        u32 TemacErr;
        u32 OutOfMem;
    }EthStats;
} TemacDevData;


/* This ioctl structure is used to set and retrieve interrupt coalescing
 * settings when the device is configured with scatter gather DMA
 */
typedef struct {
    u32 TxThreshold;
    u32 RxThreshold;
    u32 TxWaitbound;
    u32 RxWaitbound;
} TemacCoalesce;

/***************** Macros (Inline Functions) Definitions ********************/

#define TM_TEMAC_IOCTL_COAL_SET      1 /* Ioctl command for setting interrupt
                                        coalescing parameters */
#define TM_TEMAC_IOCTL_COAL_GET      2 /* Ioctl command for getting interrupt
                                        coalescing parameters */
#define TM_TEMAC_SET_LINK_SPEED      3 /* Ioctl command for setting MAC link
                                        speed (the speed must match the speed
                                        the PHY currently is operating at) */
#define TM_TEMAC_IOCTL_SEND_COMPLETE 4 /* Ioctl command for periodically
                                          polling for sent packets*/

/************************** Function Prototypes *****************************/

int AxiTemac_Setup(TemacDevData * Mac);

int Mac_SetLink(int Speed, int FullDuplex);

int Mac_PhyRead(Xuint16 Reg, Xuint16 *Data);

int Mac_PhyWrite(Xuint16 Reg, Xuint16 Data);

void AxiTemac_Start();

void AxiTemac_Stop();

int AxiTemac_GetDeviceAddr(ttUserInterface Handle, char TM_FAR * PhysAddr);
/******************************************************************************
 * TRECK RELATED FUNCTIONS
 *****************************************************************************/
ttUserInterface Treck_CreateInterface(int Dma_Type);

int Setup_Interrupt(u8 Id, XInterruptHandler Handler, void * CallBackRef);

int Util_PhyAddr(unsigned MenuIndex, unsigned TestLoops);

int TemacUtil_DumpStats();

/************************** Variable Definitions ****************************/

#endif /* _XPS_LLTEMAC_H_ */
