/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Intel 80314 Gigabit Ethernet Device Driver
 * Filename: tr80314.c
 * Author: Jin Zhang and Ed Remmell
 * Date Created:
 * $Source: source/drivers/80314/tr80314.c $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:35:47CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#include <trsocket.h>
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>
#include <tr80314.h>

/*
 * Our receive pool consists of two sized buffers: small and large.
 * These macros determine the size and number of these buffers in our
 * pre-allocated receive pool.
 */
#define TM_80314_LARGE_RECV_POOL     40
#define TM_80314_SMALL_RECV_POOL     0 /* not used */
#define TM_80314_LARGE_RECV_BUF_SIZE 1522
#define TM_80314_SMALL_RECV_BUF_SIZE 128 /* not used */

#define TM_XSCALE_CACHE_LINE_LEN     32

static void tf80314SendComplete(ttUserInterface interfaceHandle);


/*
 * tuntypes.h 
 */

#ifndef __INCvxTypesOldh

typedef char    INT8;
typedef short   INT16;
typedef int      INT32;

typedef unsigned char    UINT8;
typedef unsigned short  UINT16;
typedef unsigned int      UINT32;

typedef unsigned char    UCHAR;
typedef unsigned short  USHORT;
typedef unsigned int      UINT;
typedef unsigned long    ULONG;

#ifndef TRUE
  typedef enum {
    FALSE = 0,
    TRUE = 1
  } BOOL;
#endif
 
typedef int      STATUS;

typedef void        VOID;

#ifdef __cplusplus
typedef int         (*FUNCPTR) (...);     /* ptr to function returning int */
typedef void        (*VOIDFUNCPTR) (...); /* ptr to function returning void */
typedef double      (*DBLFUNCPTR) (...);  /* ptr to function returning double*/
typedef float       (*FLTFUNCPTR) (...);  /* ptr to function returning float */
#else
typedef int         (*FUNCPTR) ();     /* ptr to function returning int */
typedef void        (*VOIDFUNCPTR) (); /* ptr to function returning void */
typedef double      (*DBLFUNCPTR) ();  /* ptr to function returning double*/
typedef float       (*FLTFUNCPTR) ();  /* ptr to function returning float */
#endif          /* _cplusplus */

#endif

#ifndef __tunvolatile

#define __tunvolatile

typedef volatile UCHAR VCHAR;   /* shorthand for volatile UCHAR */
typedef volatile INT32 VINT32; /* volatile unsigned word */
typedef volatile INT16 VINT16; /* volatile unsigned halfword */
typedef volatile INT8 VINT8;   /* volatile unsigned byte */
typedef volatile UINT32 VUINT32; /* volatile unsigned word */
typedef volatile UINT16 VUINT16; /* volatile unsigned halfword */
typedef volatile UINT8 VUINT8;   /* volatile unsigned byte */

#endif

/*
 * endof tuntypes.h 
 */


/*
 * fl_board.h
 */

/**************************************************************
 * Memory-mapped CIU resources (CPU view)
 * The memory map is set by initialization program (RedBoot).
 **************************************************************
 */
#define XS_CSR_BASE_ADDR    0x50100000  /* Remapped FL CSR Base Address */
#define XS_SRAM_BASE_ADDR   0x50000000  /* FL SRAM Base Address */
#define XS_SDRAM_BASE_ADDR  0x00000000  /* SDRAM base address */
#define BOARD_SDRAM_SIZE    0x20000000  /* Default value: 512MB */

/* Size of SDRAM space reserved for user space. This limits space available
for dynamic allocation. */
#define USER_RESERVED_MEM   (BOARD_SDRAM_SIZE - 0x02000000)
#define USER_RESERVED_BASE  \
    (XS_SDRAM_BASE_ADDR + (BOARD_SDRAM_SIZE - USER_RESERVED_MEM)

/*----------------------------------------------------------
 * HOST_PCI1 initiator (outbound) window to PCI1 bus
 *----------------------------------------------------------
 */

#define XS_PCI1_BASE_ADDR   0x80000000  /* Base Address for the PCI1 window */

/*----------------------------------------------------------
 * HOST_PCI2 initiator (outbound) window to PCI2 bus
 *----------------------------------------------------------
 */

#define XS_PCI2_BASE_ADDR  0xC0000000   /* Base Address for the PCI2 window */

/*
 * endof fl_board.h
 */

/*
 * 80200.h
 */

#if defined(__cplusplus)
#define REGISTER
#else
#define REGISTER register
#endif

/***************************************************************
 * Coprocessor 15 
 **************************************************************/
/* Cache functions */
#define _Invalidate_I_BTB \
asm("mcr\tp15, 0, r0, c7, c5, 0")

#define _Invalidate_D_Cache \
asm("mcr\tp15, 0, r0, c7, c6, 0" : : : "memory")

#define _Invalidate_D_Cache_Line(MVA) \
asm("mcr\tp15, 0, %0, c7, c6, 1" : : "r" (MVA) : "memory")

#define _Clean_D_Cache_Line(MVA) \
asm("mcr\tp15, 0, %0, c7, c10, 1" : : "r" (MVA))

#define _Drain_Write_Buffer \
asm("mcr\tp15, 0, r0, c7, c10, 4")

#define _Allocate_D_Cache_Line(MVA) \
asm("mcr\tp15, 0, %0, c7, c2, 5" : : "r" (MVA) : "memory")


/***************************************************************
 * Coprocessor 13 
 **************************************************************/
#define _Read_INT_Control \
({ REGISTER unsigned _val_; \
   asm("mrc\tp13, 0, %0, c0, c0, 0" : "=r" (_val_)); \
   _val_ & 0xF; \
})

#define _Write_INT_Control(VAL) \
 asm("mcr\tp13, 0, %0, c0, c0, 0" : : "r" (VAL))

#define _INTCTL_BCU_ENABLE (1 << 3)
#define _INTCTL_PMU_ENABLE (1 << 2)
#define _INTCTL_IRQ_ENABLE (1 << 1)
#define _INTCTL_FIQ_ENABLE (1 << 0)

#define _Read_INT_Source \
({ REGISTER unsigned _val_; \
   asm("mrc\tp13, 0, %0, c4, c0, 0" : "=r" (_val_)); \
   _val_; \
})

#define _INTSRC_FIQ_ACTIVE (1 << 31)
#define _INTSRC_IRQ_ACTIVE (1 << 30)
#define _INTSRC_BCU_ACTIVE (1 << 29)
#define _INTSRC_PMU_ACTIVE (1 << 28)

#define _Read_INT_Steering \
({ REGISTER unsigned _val_; \
   asm("mrc\tp13, 0, %0, c8, c0, 0" : "=r" (_val_)); \
   _val_ & 0x3; \
})

#define _Write_INT_Steering(VAL) \
 asm("mcr\tp13, 0, %0, c8, c0, 0" : : "r" (VAL))

#define _INTSTR_BCU_FIQ (1 << 1)
#define _INTSTR_BCU_IRQ (0)
#define _INTSTR_PMU_FIQ (1 << 0)
#define _INTSTR_PMU_IRQ (0)


/* D. Kranak addition (inserted by avb) */
/* Get high bits into flags */
#define _WaitForNoIRQ() \
 asm("1:\tmrc\tp13, 0, r15, c4, c0, 0\n\tnop\n\tbeq\t1b")

#define _WaitForNoFIQ() \
 asm("1:\tmrc\tp13, 0, r15, c4, c0, 0\n\tnop\n\tbmi\t1b")

#define _WaitForNoBCU() \
 asm("1:\tmrc\tp13, 0, r15, c4, c0, 0\n\tnop\n\tbcs\t1b")

#define _WaitForNoPMU() \
 asm("1:\tmrc\tp13, 0, r15, c4, c0, 0\n\tnop\n\tbvs\t1b")


/***************************************************************
 * Misc
 **************************************************************/

#define _CPWAIT \
asm("mrc\tp15, 0, r0, c2, c0, 0\n\t" \
    "mov\tr0, r0\n\t" \
    "sub\tpc, pc, #4" : : : "r0", "memory")

#define _Write_CPSR(VAL) \
asm("msr\tcpsr, %0" : : "rI" (VAL) : "cc", "memory")

#define _Write_MODE(VAL) \
asm("msr\tcpsr_c, %0" : : "rI" (VAL) : "memory")

#define _Write_FLAGS(VAL) \
asm("msr\tcpsr_f, %0" : : "rI" (VAL) : "cc")

/*
 * endof 80200.h
 */


/*
 * sys_utils.h
 */
/* SF port numbers */
typedef enum {
    PORT_HLP, PORT_PCI1, PORT_PCI2, PORT_XSCALE, PORT_SDRAM /*PORT_GIGE*/,
    PORT_DMA, PORT_GIGE/*PORT_SDRAM*/
    } SF_PORT_NUM;

#define PORT_SRAM   PORT_XSCALE

#ifndef DATA_CACHE_SIZE
    #define DATA_CACHE_SIZE 0x8000
#endif

#ifndef DATA_CACHE_FLUSH_ADDR
    #define DATA_CACHE_FLUSH_ADDR 0x50300000
#endif


static unsigned sysVirtualToOcn(
    void* xs_addr, unsigned* ocn_addr_m, unsigned* ocn_addr_l, unsigned* ocn_port
    );
static void* sysOcnToVirtual(
    unsigned ocn_addr_m, unsigned ocn_addr_l, unsigned ocn_port
    );
static void cleanDCache(void);

/*
 * end of sys_utils.h
 */


/*
 * SpacEthPublic.h
 */

#define ETH_BASE_RTN_CODE (0)

/* Return codes. */
typedef enum {
  ETH_OK = 0, /* Operation succeded. */   
  ETH_ERR = -1, /* Operation failed. */   
  ETH_PARAM_ERR = ETH_BASE_RTN_CODE -2, /* Invalid parameter. */    
  ETH_BASE_ADDR_ERR = ETH_BASE_RTN_CODE -3, /* Invalid chip base address. */
  ETH_MEM_ALLOC_ERR = ETH_BASE_RTN_CODE -4, /* Memory allocation error. */ 
  ETH_MEM_ALIGN_ERR = ETH_BASE_RTN_CODE -5, /* Memory alignment error. */ 
  ETH_GET_BUFF_ERR = ETH_BASE_RTN_CODE -6, /* Out of buffers condition. */ 
  ETH_Q_FULL_ERR = ETH_BASE_RTN_CODE -7, /* Queue full condition. */ 
  ETH_Q_EMPTY_ERR = ETH_BASE_RTN_CODE -8, /* Queue empty condition. */ 
  ETH_SEMCREATE_ERR = ETH_BASE_RTN_CODE -9, /* Semaphore creation error. */
  ETH_SEMDELETE_ERR = ETH_BASE_RTN_CODE -10, /* Semaphore deletion error. */
  ETH_NOT_INIT_ERR = ETH_BASE_RTN_CODE -11, /* Attempt to operate on a non 
                                          initialized object. */
  ETH_ALREADY_INIT_ERR = ETH_BASE_RTN_CODE -12, /* Attempt to operate on an 
                                          already object. */
  ETH_SYNC_OBJ_ERR = ETH_BASE_RTN_CODE -13, /* Semaphore operation error. */
  ETH_Q_INTEGRITY_ERR = ETH_BASE_RTN_CODE -14, /* Queue integrity error. */
  ETH_FREE_BUFF_ERR = ETH_BASE_RTN_CODE -15, /* Buffer release error. */ 
  ETH_RX_DEMUX_ERR = ETH_BASE_RTN_CODE -16, /* RX demux error. */ 
  ETH_MII_IF_STATUS_ERR = ETH_BASE_RTN_CODE -17, /* MII interface status 
                                                    error. */ 
  ETH_TIMEOUT_ERR = ETH_BASE_RTN_CODE -18, /* Time-out error. */ 
  ETH_INVAL_CFG_ERR = ETH_BASE_RTN_CODE -19, /* Invalid configuration error.*/
} SpacEthRtnCode_t;

/*
 * end of SpacEthPublic.h
 */


/* 
 * SpacEthHardwareDefs.h 
 */

/* helper macros */
#define REG32_PTR(a)                      ((VUINT32 *)(a))
#define REG32_VAL(a)                      ((UINT32)(a))

/* Xscale speed (in Hz). */
#define ETH_XSCALE_CLK                    (400000000)

/* Number of nanoseconds to wait before checking the MII interface status. */
#define ETH_MII_IF_STATUS_WAIT            (1000000)

/* Number of nanoseconds to wait before checking the MII status when doing 
   "scan" cycle. */
#define ETH_MII_SCAN_STATUS_WAIT          (1000000000)

/* Number of nanoseconds to wait before checking the link status. Refer to
   the IEEE specification for Auto-neg in clause 36 and clause 28. */
#define ETH_MII_LINK_WAIT                 (5000000000)

/* GigE low base address, as an offset from the chip base address. */
#define ETH_BASE_ADDR_LO                  (0x6000)

/* GigE high base address, as an offset from the chip base address. */
#define ETH_BASE_ADDR_HI                  (0x6fff)

/* Descriptor HI address  mask. Only 12 bits are significant. */
#define ETH_DESCR_ADDR_HI_MASK            (0xfff) 

/* frame descriptor size. */
#define ETH_DESCR_SIZE                     (32)

/* Number of GigE ports. */
#define ETH_NUM_PORTS                     (1)

/* Port1 offset from Port0 base address. */
#define ETH_PORT_OFFSET                   (0x400)

/* Number of TX queues. */
#define ETH_NUM_TX_Q                      (1)

/* TX Queue (0, 1, 2, 3) Registers are offset by 0x10 from the precedent
 * queue register set. */
#define ETH_TX_Q_REG_OFFSET               (0x10)

/* RX Queue (0, 1, 2, 3) Registers are offset by 0x10 from the precedent
 * queue register set. */
#define ETH_RX_Q_REG_OFFSET               (0x10)

/* Number of RX queues. */
#define ETH_NUM_RX_Q                      (1)

/* Number of RX 32-bit words in the unicast+multicast hash tables. */
#define ETH_NUM_RX_HASH_WORDS             (32)

/* Up to 32 frames can be oustanding in the TX FIFO. */
#define ETH_TX_NUM_FIFO_FRAMES            (32)

/* TBI PHY address, as seen by the MII interface. */
#define ETH_TBI_PHY_ADDR                  (0x1E)

/* Start of BCM5421-specific code */ 

/* PHY addresses, as seen by the MII interface. */
#define ETH_PHY0_ADDR                     (0x10)
#define ETH_PHY1_ADDR                     (0x01)

/* End of BCM5421-specific code */ 

/* Descriptor "last" bit field. */
#define ETH_DESCR_LAST                    (1 << 31)

/* Up to 32 interrupt sources can be signaled through the GigE 
   ETH_PORT_INT_STATUS_REG register. */
#define ETH_MAX_NUM_INTR_SOURCES          (32)

/* CLZ returns 32 if no bits are set in the source register. Refer to  
   ARM Architecture Reference Manual. */
#define ETH_ARM_CLZ_BIT_NOT_SET           (32)

/* RX/TX queue interrupt source bit limits. */
#define ETH_RX_Q_INTR_STATUS_LO           (16)
#define ETH_RX_Q_INTR_STATUS_HI           (19)
#define ETH_TX_Q_INTR_STATUS_LO           (0)
#define ETH_TX_Q_INTR_STATUS_HI           (3)

/* Byte width. */
#define ETH_BYTE_WIDTH                    (8)

/* OCN port ids. */
#define ETH_OCN_PCI1_ID                   (1)
#define ETH_OCN_PCI2_ID                   (2)
#define ETH_OCN_XSCALE_ID                 (3) /* SRAM id, also. */
#define ETH_OCN_SDRAM_ID                  (4)
#define ETH_OCN_GIGE_ID                   (6)

/* memory alignment. */
#define ETH_MEM_ALIGN (8)

/* MAC header length. */
#define ETH_MAC_HEADER_LEN (14)
/* MAC address length. */
#define ETH_MAC_ADDR_LEN (6)
/* type/length length. */
#define ETH_MAC_TYPE_LEN (2)
/* FCS address length. */
#define ETH_MAC_FCS_LEN (4)

/******************************************************************************
 * MAC Registers. 
 */
#define ETH_MAC_CFG1_REG                  (ETH_BASE_ADDR_LO + 0x000) 
#define ETH_MAC_CFG2_REG                  (ETH_BASE_ADDR_LO + 0x004) 
#define ETH_MAC_IPG_IFG_REG               (ETH_BASE_ADDR_LO + 0x008) 
#define ETH_MAC_HALF_DUPLEX_REG           (ETH_BASE_ADDR_LO + 0x00C) 
#define ETH_MAC_MAX_FRAME_REG             (ETH_BASE_ADDR_LO + 0x010) 
#define ETH_MAC_TEST_REG                  (ETH_BASE_ADDR_LO + 0x01C) 
#define ETH_MAC_MII_CFG_REG               (ETH_BASE_ADDR_LO + 0x020) 
#define ETH_MAC_MII_CMD_REG               (ETH_BASE_ADDR_LO + 0x024) 
#define ETH_MAC_MII_ADDR_REG              (ETH_BASE_ADDR_LO + 0x028) 
#define ETH_MAC_MII_CTRL_REG              (ETH_BASE_ADDR_LO + 0x02C) 
#define ETH_MAC_MII_STATUS_REG            (ETH_BASE_ADDR_LO + 0x030) 
#define ETH_MAC_MII_INDIC_REG             (ETH_BASE_ADDR_LO + 0x034) 
#define ETH_MAC_IF_CTRL_REG               (ETH_BASE_ADDR_LO + 0x038) 
#define ETH_MAC_IF_STATUS_REG             (ETH_BASE_ADDR_LO + 0x03C) 
#define ETH_MAC_ADDR_PART1_REG            (ETH_BASE_ADDR_LO + 0x040) 
#define ETH_MAC_ADDR_PART2_REG            (ETH_BASE_ADDR_LO + 0x044) 

/******************************************************************************
 * MSTAT Registers. 
 */
#define ETH_MSTAT_TX64_REG                (ETH_BASE_ADDR_LO + 0x050) 
#define ETH_MSTAT_TX127_REG               (ETH_BASE_ADDR_LO + 0x054) 
#define ETH_MSTAT_TX255_REG               (ETH_BASE_ADDR_LO + 0x058) 
#define ETH_MSTAT_TX511_REG               (ETH_BASE_ADDR_LO + 0x05C) 
#define ETH_MSTAT_TX1K_REG                (ETH_BASE_ADDR_LO + 0x060) 
#define ETH_MSTAT_TXMAX_REG               (ETH_BASE_ADDR_LO + 0x064) 
#define ETH_MSTAT_TXMGV_REG               (ETH_BASE_ADDR_LO + 0x068) 
#define ETH_MSTAT_RXBYTE_REG              (ETH_BASE_ADDR_LO + 0x06C) 
#define ETH_MSTAT_RXPKT_REG               (ETH_BASE_ADDR_LO + 0x070) 
#define ETH_MSTAT_RXFCS_REG               (ETH_BASE_ADDR_LO + 0x074) 
#define ETH_MSTAT_RXMCA_REG               (ETH_BASE_ADDR_LO + 0x078) 
#define ETH_MSTAT_RXBCA_REG               (ETH_BASE_ADDR_LO + 0x07C) 
#define ETH_MSTAT_RXCF_REG                (ETH_BASE_ADDR_LO + 0x080) 
#define ETH_MSTAT_RXPF_REG                (ETH_BASE_ADDR_LO + 0x084) 
#define ETH_MSTAT_RXUO_REG                (ETH_BASE_ADDR_LO + 0x088) 
#define ETH_MSTAT_RXALN_REG               (ETH_BASE_ADDR_LO + 0x08C) 
#define ETH_MSTAT_RXFLR_REG               (ETH_BASE_ADDR_LO + 0x090) 
#define ETH_MSTAT_RXCDE_REG               (ETH_BASE_ADDR_LO + 0x094) 
#define ETH_MSTAT_RXCSE_REG               (ETH_BASE_ADDR_LO + 0x098) 
#define ETH_MSTAT_RXUND_REG               (ETH_BASE_ADDR_LO + 0x09C) 
#define ETH_MSTAT_RXOVR_REG               (ETH_BASE_ADDR_LO + 0x0A0) 
#define ETH_MSTAT_RXFRG_REG               (ETH_BASE_ADDR_LO + 0x0A4) 
#define ETH_MSTAT_RXJBR_REG               (ETH_BASE_ADDR_LO + 0x0A8) 
#define ETH_MSTAT_RXDRP_REG               (ETH_BASE_ADDR_LO + 0x0AC) 
#define ETH_MSTAT_TXBYTE_REG              (ETH_BASE_ADDR_LO + 0x0B0) 
#define ETH_MSTAT_TXPKT_REG               (ETH_BASE_ADDR_LO + 0x0B4) 
#define ETH_MSTAT_TXMCA_REG               (ETH_BASE_ADDR_LO + 0x0B8) 
#define ETH_MSTAT_TXBCA_REG               (ETH_BASE_ADDR_LO + 0x0BC) 
#define ETH_MSTAT_TXPF_REG                (ETH_BASE_ADDR_LO + 0x0C0) 
#define ETH_MSTAT_TXDFR_REG               (ETH_BASE_ADDR_LO + 0x0C4) 
#define ETH_MSTAT_TXEDF_REG               (ETH_BASE_ADDR_LO + 0x0C8) 
#define ETH_MSTAT_TXSCL_REG               (ETH_BASE_ADDR_LO + 0x0CC) 
#define ETH_MSTAT_TXMCL_REG               (ETH_BASE_ADDR_LO + 0x0D0) 
#define ETH_MSTAT_TXLCL_REG               (ETH_BASE_ADDR_LO + 0x0D4) 
#define ETH_MSTAT_TXXCL_REG               (ETH_BASE_ADDR_LO + 0x0D8) 
#define ETH_MSTAT_TXNCL_REG               (ETH_BASE_ADDR_LO + 0x0DC) 
#define ETH_MSTAT_TXPFH_REG               (ETH_BASE_ADDR_LO + 0x0E0) 
#define ETH_MSTAT_TXDRP_REG               (ETH_BASE_ADDR_LO + 0x0E4) 
#define ETH_MSTAT_TXJBR_REG               (ETH_BASE_ADDR_LO + 0x0E8) 
#define ETH_MSTAT_TXFCS_REG               (ETH_BASE_ADDR_LO + 0x0EC) 
#define ETH_MSTAT_TXCF_REG                (ETH_BASE_ADDR_LO + 0x0F0) 
#define ETH_MSTAT_TXOVR_REG               (ETH_BASE_ADDR_LO + 0x0F4) 
#define ETH_MSTAT_TXUND_REG               (ETH_BASE_ADDR_LO + 0x0F8) 
#define ETH_MSTAT_TXFRG_REG               (ETH_BASE_ADDR_LO + 0x0FC) 
#define ETH_MSTAT_CAR1_REG                (ETH_BASE_ADDR_LO + 0x100) 
#define ETH_MSTAT_CAR2_REG                (ETH_BASE_ADDR_LO + 0x104) 
#define ETH_MSTAT_CARM1_REG               (ETH_BASE_ADDR_LO + 0x108) 
#define ETH_MSTAT_CARM2_REG               (ETH_BASE_ADDR_LO + 0x10C) 

/******************************************************************************
 * PORT Registers. 
 */
#define ETH_PORT_CTRL_REG                 (ETH_BASE_ADDR_LO + 0x200) 
#define ETH_PORT_INT_STATUS_REG           (ETH_BASE_ADDR_LO + 0x204) 
#define ETH_PORT_INT_MASK_REG             (ETH_BASE_ADDR_LO + 0x208) 
#define ETH_PORT_OCN_STATUS_REG           (ETH_BASE_ADDR_LO + 0x20C)   
  
/******************************************************************************
 * TX Registers. 
 */  
#define ETH_TX_CFG_REG                    (ETH_BASE_ADDR_LO + 0x220) 
#define ETH_TX_CTRL_REG                   (ETH_BASE_ADDR_LO + 0x224) 
#define ETH_TX_STATUS_REG                 (ETH_BASE_ADDR_LO + 0x228) 
#define ETH_TX_XSTATUS_REG                (ETH_BASE_ADDR_LO + 0x22C) 
#define ETH_TX_THRESH_REG                 (ETH_BASE_ADDR_LO + 0x230) 
#define ETH_TX_PRI_THRESH_REG             (ETH_BASE_ADDR_LO + 0x234) 
#define ETH_TX_Q_PRI_MAP_REG              (ETH_BASE_ADDR_LO + 0x240) 
#define ETH_TX_PEND_COMP_REG              (ETH_BASE_ADDR_LO + 0x248)
#define ETH_TX_PAUSE_CNT_REG              (ETH_BASE_ADDR_LO + 0x250)
#define ETH_TX_GVLAN_TAG_REG              (ETH_BASE_ADDR_LO + 0x254)
#define ETH_TX_DIAG_ADDR_REG              (ETH_BASE_ADDR_LO + 0x270)
#define ETH_TX_DIAG_DATA_REG              (ETH_BASE_ADDR_LO + 0x274)
#define ETH_TX_ERR_STATUS_REG             (ETH_BASE_ADDR_LO + 0x278)
/******************************************************************************
 * TX Queue (0, 1, 2, 3) Registers are offset by 0x10 from the precedent
 * queue register set.
 */
#define ETH_TX_Q0_CFG_REG                 (ETH_BASE_ADDR_LO + 0x280)
#define ETH_TX_Q0_BUFF_CFG_REG            (ETH_BASE_ADDR_LO + 0x284)
#define ETH_TX_Q0_PTR_LO_REG              (ETH_BASE_ADDR_LO + 0x288)
#define ETH_TX_Q0_PTR_HI_REG              (ETH_BASE_ADDR_LO + 0x28C)

/******************************************************************************
 * RX Registers. 
 */
#define ETH_RX_CFG_REG                    (ETH_BASE_ADDR_LO + 0x320) 
#define ETH_RX_CTRL_REG                   (ETH_BASE_ADDR_LO + 0x324) 
#define ETH_RX_STATUS_REG                 (ETH_BASE_ADDR_LO + 0x328) 
#define ETH_RX_XSTATUS_REG                (ETH_BASE_ADDR_LO + 0x32C) 
#define ETH_RX_PAUSE_THRESH_REG           (ETH_BASE_ADDR_LO + 0x330) 
#define ETH_RX_THRESH_REG                 (ETH_BASE_ADDR_LO + 0x334) 
#define ETH_RX_VLAN_TAG_MAP_REG           (ETH_BASE_ADDR_LO + 0x340)
#define ETH_RX_HASH_ADDR_REG              (ETH_BASE_ADDR_LO + 0x360)
#define ETH_RX_HASH_DATA_REG              (ETH_BASE_ADDR_LO + 0x364)
#define ETH_RX_DIAG_ADDR_REG              (ETH_BASE_ADDR_LO + 0x370)
#define ETH_RX_DIAG_DATA_REG              (ETH_BASE_ADDR_LO + 0x374)
#define ETH_RX_ERR_STATUS_REG             (ETH_BASE_ADDR_LO + 0x378)

/******************************************************************************
 * RX Queue (0, 1, 2, 3) Registers are offset by 0x10 from the precedent
 * queue register set.
 */
#define ETH_RX_Q0_CFG_REG                 (ETH_BASE_ADDR_LO + 0x380)
#define ETH_RX_Q0_BUFF_CFG_REG            (ETH_BASE_ADDR_LO + 0x384)
#define ETH_RX_Q0_PTR_LO_REG              (ETH_BASE_ADDR_LO + 0x388)
#define ETH_RX_Q0_PTR_HI_REG              (ETH_BASE_ADDR_LO + 0x38C)


/******************************************************************************
 * TX/RX Diagnostic (Indirect Access) register Indexes.
 */
#define ETH_BUFF_ADDR0_REG_IDX            (0x0)
#define ETH_BUFF_ADDR1_REG_IDX            (0x1)
#define ETH_NEXT_ADDR0_REG_IDX            (0x2)
#define ETH_NEXT_ADDR1_REG_IDX            (0x3)
#define ETH_VLAN_BCOUNT_REG_IDX           (0x4)
#define ETH_STATUS_CFG_REG_IDX            (0x5)

/******************************************************************************
 * TX Cache Word0 and Word1 (Indirect Access) register Indexes. 
 * Even indexes (0x40-0x7E) are for Word0. Odd indexes (0x41-0x7F)
 * are for Word1.
 */
#define ETH_TX_CACHE_WORD0_REG_IDX        (0x40)
#define ETH_TX_CACHE_WORD1_REG_IDX        (0x41)

/******************************************************************************
 * RX Start Frame (0, 1, 2) (Indirect Access) register Indexes. 
 * These registers hold information about starting descriptor from the current
 * frame being received.
 */
#define ETH_RX_START_FRM0_REG_IDX         (0x8)
#define ETH_RX_START_FRM1_REG_IDX         (0x9)
#define ETH_RX_START_FRM2_REG_IDX         (0xA)

/******************************************************************************
 * TBI (Indirect Access) register Indexes.
 */
#define ETH_TBI_CTRL1_REG_IDX             (0x00)
#define ETH_TBI_STATUS_REG_IDX            (0x01)
#define ETH_TBI_AN_ADV_REG_IDX            (0x04)
#define ETH_TBI_AN_LP_BP_REG_IDX          (0x05)
#define ETH_TBI_AN_EXP_REG_IDX            (0x06)
#define ETH_TBI_AN_NP_TX_REG_IDX          (0x07)
#define ETH_TBI_AN_LP_ANP_REG_IDX         (0x08)
#define ETH_TBI_EXT_STATUS_REG_IDX        (0x0F)
#define ETH_TBI_JITT_DIAG_REG_IDX         (0x10)
#define ETH_TBI_CTRL2_REG_IDX             (0x11)

/* Start of BCM5421-specific code */ 
/******************************************************************************
 * PHY (Indirect Access) register Indexes.
 */
#define ETH_PHY_CTRL1_REG_IDX             (0x00)
#define ETH_PHY_STATUS_REG_IDX            (0x01)
#define ETH_PHY_AN_ADV_REG_IDX            (0x04)
#define ETH_PHY_1000BASE_T_REG_IDX        (0x09)
#define ETH_PHY_EXT_STATUS_REG_IDX        (0x0F)
#define ETH_PHY_CTRL2_REG_IDX             (0x10)
#define ETH_PHY_SUM_STATUS_REG_IDX        (0x19)
#define ETH_PHY_SPR_CTRL2_REG_IDX         (0x1C)

/* End of BCM5421-specific code */ 

/******************************************************************************
 * MAC Register bit masks. 
 */
#define ETH_MAC_IF_CTRL_PHY_MASK          (1 << 24) 
#define ETH_MAC_RESET_MASK                (1 << 31)
#define ETH_MAC_MII_RESET_MASK            (1 << 31)                              

/******************************************************************************
 * MSTAT Register bit masks. 
 */
#define ETH_MSTAT_CAR1_RPKT_MASK          (1 << 15) 
#define ETH_MSTAT_CAR2_TPKT_MASK          (1 << 12)  

/******************************************************************************
 * PORT Register bit masks. 
 */
#define ETH_PORT_INT_STATUS_MASK          (1 << 31)
#define ETH_PORT_CTRL_RESET_MASK          (1 << 31)
 
/******************************************************************************
 * TX Register bit masks. 
 */
#define ETH_TX_Q_X_INTR_MASK              (0x00010101)
#define ETH_TX_DESCR_Q_X_INTR_MASK        (1 << 16)
#define ETH_TX_EOF_Q_X_INTR_MASK          (1 << 8)
#define ETH_TX_EOQ_Q_X_INTR_MASK          (1)
#define ETH_TX_Q0_INTR_MASK               (1)
#define ETH_TX_Q_WAIT_INTR_MASK           (1 << 8)
#define ETH_TX_Q0_ERR_MASK                (1 << 24)
#define ETH_TX_Q0_DESC_ERR_MASK           (1 << 7)
#define ETH_TX_Q0_TIMEOUT_ERR_MASK        (1 << 6)
#define ETH_TX_Q0_READ_ERR_MASK           (1 << 5)
#define ETH_TX_Q0_TEA_ERR_MASK            (1 << 4)
#define ETH_TX_Q0_TRESP_ERR_MASK          (0xF)
#define ETH_TX_ABORT_INTR_MASK            (1 << 30)
#define ETH_TX_ABORT_EN_INTR_MASK         (1 << 29)
#define ETH_TX_IDLE_EN_INTR_MASK          (1 << 31)
#define ETH_TX_BUFF_W_SWAP_MASK           (1 << 11)
#define ETH_TX_BUFF_B_SWAP_MASK           (1 << 10)
#define ETH_TX_Q_ENABLE_ALL_MASK          (0xF)
#define ETH_TX_Q_CFG_GVI_MASK             (1 << 15)
#define ETH_TX_RESET_MASK                 (1 << 31)
#define ETH_TX_CTRL_GO_MASK               (1 << 15)
#define ETH_TX_STATUS_ACTIVE_MASK         (1 << 15)
#define ETH_TX_DESCR_Q_INTR_EN_MASK       (1 << 20)
#define ETH_TX_EOF_Q_INTR_EN_MASK         (1 << 16)
#define ETH_TX_EOQ_Q_INTR_EN_MASK         (7 << 17)

/******************************************************************************
 * TX Queue (0, 1, 2, 3) Register bit masks.
 */

/******************************************************************************
 * RX Register bit masks. 
 */
#define ETH_RX_Q_X_INTR_MASK              (0x01010101)
#define ETH_RX_DESCR_Q_X_INTR_MASK        (1 << 16)
#define ETH_RX_EOF_Q_X_INTR_MASK          (1 << 8)
#define ETH_RX_EOQ_Q_X_INTR_MASK          (1)
#define ETH_RX_Q0_INTR_MASK               (1 << 16)
#define ETH_RX_Q_WAIT_INTR_MASK           (1 << 24)
#define ETH_RX_Q0_ERR_MASK                (1 << 24)
#define ETH_RX_Q0_DESC_ERR_MASK           (1 << 7)
#define ETH_RX_Q0_TIMEOUT_ERR_MASK        (1 << 6)
#define ETH_RX_Q0_READ_ERR_MASK           (1 << 5)
#define ETH_RX_Q0_TEA_ERR_MASK            (1 << 4)
#define ETH_RX_Q0_TRESP_ERR_MASK          (0xF)
#define ETH_RX_ABORT_INTR_MASK            (1 << 30)
#define ETH_RX_ABORT_EN_INTR_MASK         (1 << 29)
#define ETH_RX_IDLE_EN_INTR_MASK          (1 << 31)
#define ETH_RX_BUFF_W_SWAP_MASK           (1 << 11)
#define ETH_RX_BUFF_B_SWAP_MASK           (1 << 10)
#define ETH_RX_Q_ENABLE_ALL_MASK          (0xF)
#define ETH_RX_RESET_MASK                 (1 << 31)
#define ETH_RX_STATUS_ACTIVE_MASK         (1 << 15)
#define ETH_RX_DESCR_Q_INTR_EN_MASK       (1 << 20)
#define ETH_RX_EOF_Q_INTR_EN_MASK         (1 << 16)
#define ETH_RX_EOQ_Q_INTR_EN_MASK         (7 << 17)


/******************************************************************************
 * RX Queue (0, 1, 2, 3) Register bit masks.
 */


/******************************************************************************
 * TX descriptor bit masks. 
 */
 
/* Last descriptor. */
#define ETH_TX_DESCR_LAST                 (1 << 31)


/******************************************************************************
 * RX descriptor bit masks. 
 */
 
/* Last descriptor. */
#define ETH_RX_DESCR_LAST                 (1 << 31)


/******************************************************************************
 * TBI Register bit masks. 
 */
 
#define ETH_TBI_PHY_RESET_MASK            (1 << 15)
#define ETH_TBI_RESET_MASK                (1 << 15)

/* Start of BCM5421-specific code */ 
/******************************************************************************
 * PHY Register bit masks. 
 */
 
#define ETH_PHY_RESET_MASK                (1 << 15)
#define ETH_PHY_SPR_CTRL2_REG_VAL         (9040)

/* End of BCM5421-specific code */ 

/*
 * End of SpacEthHardwareDefs.h
 */

/* SpacEthDefs.h */

/* Port0 and Port1 ids. */
#define ETH_PORT0_ID (0)
#define ETH_PORT1_ID (1)

/* CPU0 and CPU1 ids. */
#define ETH_CPU0_ID (0)
#define ETH_CPU1_ID (1)

/* Enabled/Disabled type. */
typedef enum {
  ETH_BIT_CLEARED = 0,   
  ETH_BIT_SET = 1,   
} EthBitField_t;

/* TX/RX direction flow type. */
typedef enum {
  ETH_TX = 0,   
  ETH_RX = 1,   
} EthDirFlow_t;

/* TX/RX descriptor owner type. */
typedef enum {
  ETH_DESCR_SYS = 0,    /* descriptor owned by the system. */  
  ETH_DESCR_DMA = 1,    /* descriptor owned by the DMA. */  
} EthDescrOwner_t;

/* TX/RX buffer descriptor scheme type. */
typedef enum {
  ETH_SINGLE_DESCR = 0,         /* Single descriptor frames. */    
  ETH_MULTI_DESCR = 1,          /* Multiple descriptor frames. */    
  ETH_RING_DESCR = 2,           /* Ring descriptor frames. */    
  ETH_MFRAME_LIST_DESCR = 3,    /* Multi-frame descriptor list. */    
} EthDescr_t;

/* TX/RX buffer descriptor terminator type. */
typedef enum {
  ETH_DESCR_TERM_LAST = 0,      /* "Last" bit set in NEXT_CPP. */    
  ETH_DESCR_TERM_NULL = 1,      /* NEXT_CPP is NULL. */    
  ETH_DESCR_TERM_OWN = 2,       /* "Owner" bit set in DCSW. */    
} EthDescrTerm_t;

/* TX/RX buffer size type. We recommend to use MTU dividers. */
typedef enum {
  ETH_BUFF_SIZE_64 = 64, 
  ETH_BUFF_SIZE_96 = 96, 
  ETH_BUFF_SIZE_128 = 128,     
  ETH_BUFF_SIZE_256 = 256,    
  ETH_BUFF_SIZE_512 = 512,    
  ETH_BUFF_SIZE_MTU = 1536,     /* Ethernet frame size, 64-bit aligned. */ 
  ETH_BUFF_SIZE_1600 = 1600, 
  ETH_BUFF_SIZE_2048 = 2048, 
  ETH_BUFF_SIZE_4096 = 4096, 
  ETH_BUFF_SIZE_8192 = 8192, 
  ETH_BUFF_SIZE_16384 = 16384, 
} EthBuffSize_t;

/* TX/RX OCN data burst type. */
typedef enum {
  ETH_OCN_BURST_8 = 0,          /* 8-bytes burst. */    
  ETH_OCN_BURST_32 = 1,         /* 32-bytes burst. */    
  ETH_OCN_BURST_128 = 2,        /* 128-bytes burst. */    
  ETH_OCN_BURST_256 = 3,        /* 256-bytes burst. */    
} EthOcnBurst_t;

/* Interface(to which MAC is connected) mode type. */
typedef enum {
  ETH_MAC_IF_RESERVED1 = 0,     /* Reserved. */     
  ETH_MAC_IF_NIBLE = 1,         /* 10/100Mb/s. */     
  ETH_MAC_IF_BYTE = 2,          /* 1000Mb/s. */    
  ETH_MAC_IF_RESERVED2 = 3,     
} EthMacIfMode_t;

/* MII management cycle type. */
typedef enum {
  ETH_MAC_MII_SCAN = 0,         /* Scan cycle. */     
  ETH_MAC_MII_READ = 1,         /* Read cycle. */   
  ETH_MAC_MII_WRITE = 2,        /* Write cycle. */   
} EthMacMiiCycle_t;

/* TX/RX VLAN/Byte count type. */
typedef struct {
  VUINT32 byteCount:16;         /* TX/RX byte count. */
  VUINT32 vlanTag:16;           /* TX/RX VLAN info. */
} EthVlanBCnt_t, * EthVlanBCntPtr_t;

/* TX/RX buffer descriptor type. MUST be 32-byte aligned in memory. */
typedef struct {
    VUINT32 startAddr0;    /* buffer address, least significant bytes. */
    VUINT32 startAddr1;    /* buffer address, most significant bytes. */
    VUINT32 nextDescrAddr0;/* next descriptor address, least significant bytes
                              Must be 64-bit aligned. */
    VUINT32 nextDescrAddr1;/* next descriptor address, most significant bytes*/
    VUINT32 vlanBCnt;      /* VLAN tag(MS2B)/byte count(LS2B). */
    VUINT32 cfgStatus;     /* Configuration/Status. */
    VUINT32 padding[2];    /* must be aligned on 32-byte boundary for 80200 */
} EthBuffDescr_t, * EthBuffDescrPtr_t;

/* TX/RX descriptor info type. */
typedef struct {
  volatile EthDescr_t descrMode;
  volatile EthDescrTerm_t descrTermMode; 
  VUINT16 numDescr; /* Limit the number of buffers/descriptors per queue 
                       to ETH_NUM_DESCR. */
  VUINT16 buffSize; /* Assign EthBuffSize_t to this field. Used only for RX. */
  volatile  BOOL descrIntr; /* Allows per-descriptor interrupts. */
} EthDescrInfo_t, * EthDescrInfoPtr_t;

/* TX frame status/configuration type. */
typedef struct {
/* Beginning of configuration fields. */    
  UINT32 eof:1;                 /* End of frame field. */
  UINT32 sof:1;                 /* Start of frame field. */
  UINT32 pfVlan:1;              /* Per frame VLAN field. */
  UINT32 hugeFrame:1;           /* Huge frame field. */
  UINT32 padFrame:1;            /* Pad frame field. */
  UINT32 crc:1;                 /* Generate CRC field. */
  UINT32 reserved1:8;           /* reserved field. */
  UINT32 descrIntr:1;           /* Descriptor interrupt field. */
  UINT32 reserved2:1;           /* reserved field. */
/* End of configuration fields. */  
/* Beginning of status fields. */   
  UINT32 retryCount:4;          /* Retry count field. */ 
  UINT32 collision:1;           /* Collision field. */
  UINT32 reserved3:3;           /* reserved field. */  
  UINT32 lateCollision:1;       /* Late collision field. */
  UINT32 underrunAbort:1;       /* Underrun abort field. */
  UINT32 retryLimReached:1;     /* Retry limit reached field. */  
  UINT32 reserved4:3;           /* reserved field. */
  UINT32 frmTxOk:1;             /* Frame transmit ok field. */
/* End of status fields. */  
  UINT32 owner:1;               /* Owner(0 = system, 1 = user) field. */  
} EthTxFrmStatusCfg_t, * EthTxFrmStatusCfgPtr_t;

/* RX frame status/configuration type. */
typedef struct {
/* Beginning of status fields. */     
  UINT32 eof:1;                 /* End of frame field. */
  UINT32 sof:1;                 /* Start of frame field. */
  UINT32 vtf:1;                 /* In frame VLAN Tag field. */
  UINT32 frmIsType:1;           /* Ethernet frame type field. */
  UINT32 shortFrame:1;          /* Short frame field. */
  UINT32 reserved1:2;           /* reserved field. */
  UINT32 hashTableMatch:1;      /* Hash table match found field. */
  UINT32 badFrame:1;            /* frame collision occured field. */
  UINT32 overRun:1;             /* RX FIFO overrun. */
  UINT32 reserved2:1;           /* reserved field. */
  UINT32 maxFrmLeng:1;          /* Frame truncated to max length received 
                                   from MAC. */ 
  UINT32 crcError:1;            /* CRC error field. */
/* End of status fields. */    
  UINT32 descrIntr:1;           /* Descriptor interrupt field. */
  UINT32 reserved3:1;           /* reserved field. */  
  UINT32 owner:1;               /* Owner(0 = system, 1 = user) field. */  
  UINT32 byteCount:16;          /* RX buffer size. */  
} EthRxFrmStatusCfg_t, * EthRxFrmStatusCfgPtr_t;

/* TX/RX memory pool type. */
typedef struct {
  EthBuffDescrPtr_t descrPoolPtr;/* Descriptor pool handle for deallocation 
                                     time. */ 
  VINT32 putInIdx;              /* TX/RX put-in index. Used to add buffers to
                                   descriptor rings. */     
  VINT32 takeOutIdx;            /* TX/RX take-out index. Used to remove
                                   buffers from descriptor rings. */  
  VINT32 freeEntries;           /* number of free TX/RX entries in ring. */
/* This accomodate for maintaining a list of system-specific buffer headers.
   The ring descriptor store only reference to buffers. The(safe) assumption
   is that the header struct has a "next" type pointer. */  
} EthMemPool_t, * EthMemPoolPtr_t;

/* Ethernet header type. */
typedef struct {
  UINT8 destAddr[ETH_MAC_ADDR_LEN];     /* destination address. */
  UINT8 sourceAddr[ETH_MAC_ADDR_LEN];   /* source address. */
  UINT8 typeLen[ETH_MAC_TYPE_LEN];      /* type/length. */
} EthHeader_t, * EthHeaderPtr_t;

/* TX "PUT" frame configuration info type. */
typedef struct {
  EthTxFrmStatusCfg_t txFrmStatusCfg;   /* Frame-specific configuration info*/
  VUINT16             vlanData;         /* Frame-specific VLAN tag. */
} EthTxFrmCfgInfo_t, * EthTxFrmCfgInfoPtr_t;

/******************************************************************************
 * Info structures. They are used to convey initialization info to the 
 * driver init routines. 
 */

/* TX/RX queue info types. */
/* ETH_TX_Q#_CFG_REG info type. */
typedef struct {
  UINT32 ocnPort:6; 
  UINT32 reserved1:4; 
  UINT32 byteSwap:1; 
  UINT32 wordSwap:1; 
  UINT32 reserved2:2; 
  UINT32 autoMode:1; 
  UINT32 gVlanInsert:1; 
  UINT32 enableEofIntr:1; 
  UINT32 enableLastIntr:1; 
  UINT32 enableNullIntr:1; 
  UINT32 enableOwnerIntr:1; 
  UINT32 enableDescrIntr:1; 
  UINT32 reserved3:11; 
} EthTxQCfg_t, * EthTxQCfgPtr_t;

/* ETH_TX_Q#_BUFF_CFG_REG info type. */
typedef struct {
  UINT32 ocnPort:6; 
  UINT32 reserved1:2; 
  UINT32 burst:2; 
  UINT32 byteSwap:1; 
  UINT32 wordSwap:1; 
  UINT32 reserved2:20; 
} EthTxQBuffCfg_t, * EthTxQBuffCfgPtr_t;

/* ETH_TX/RX_Q#_PTR_HI_REG type. */
typedef struct {
  UINT32 descrAddr:12; 
  UINT32 reserved1:19; 
  UINT32 valid:1; 
} EthQPtrHi_t, * EthQPtrHiPtr_t;

typedef struct {
  EthDescrInfo_t descrInfo;
  EthTxQCfg_t txQCfgInfo;
  EthTxQBuffCfg_t txQBuffCfgInfo;
} EthTxQInfo_t, * EthTxQInfoPtr_t;

/* ETH_RX_Q#_CFG_REG info type. */
typedef struct {
  UINT32 ocnPort:6; 
  UINT32 reserved1:4; 
  UINT32 byteSwap:1; 
  UINT32 wordSwap:1; 
  UINT32 reserved2:2; 
  UINT32 autoMode:1; 
  UINT32 reserved3:1; 
  UINT32 enableEofIntr:1; 
  UINT32 enableLastIntr:1; 
  UINT32 enableNullIntr:1; 
  UINT32 enableOwnerIntr:1; 
  UINT32 enableDescrIntr:1; 
  UINT32 reserved4:11; 
} EthRxQCfg_t, * EthRxQCfgPtr_t;

/* ETH_RX_Q#_BUFF_CFG_REG info type. */
typedef struct {
  UINT32 ocnPort:6; 
  UINT32 reserved1:2; 
  UINT32 burst:2; 
  UINT32 byteSwap:1; 
  UINT32 wordSwap:1; 
  UINT32 reserved2:20; 
} EthRxQBuffCfg_t, * EthRxQBuffCfgPtr_t;

typedef struct {
  EthDescrInfo_t descrInfo;
  EthRxQCfg_t rxQCfgInfo;
  EthRxQBuffCfg_t rxQBuffCfgInfo;
} EthRxQInfo_t, * EthRxQInfoPtr_t;

/* ETH_TX_CFG_REG info type. */
typedef struct {
  UINT32 startQNum:2; 
  UINT32 reserved1:20; 
  UINT32 enableHigherPriority:1; 
  UINT32 changePriority:1; 
  UINT32 reserved2:7;
  UINT32 reset:1; 
} EthTxCfg_t, * EthTxCfgPtr_t;

/* ETH_TX_CTRL_REG info type. */
typedef struct {
  UINT32 qEnables:4; 
  UINT32 reserved1:11;
  UINT32 go:1; 
  UINT32 reserved2:8;
  UINT32 manualPause:1; 
  UINT32 reserved3:4;
  UINT32 enableAbortIntr:1;
  UINT32 abort:1;
  UINT32 enableIdleIntr:1;
} EthTxCtrl_t, * EthTxCtrlPtr_t;

/* ETH_TX_STATUS_REG info type. */
typedef struct {
  UINT32 qUsable:4; 
  UINT32 reserved1:4;
  UINT32 currentQ:2; 
  UINT32 reserved2:5;
  UINT32 active:1;
  UINT32 qIdle:4; 
  UINT32 reserved3:4;
  UINT32 eoqPending:4; 
  UINT32 reserved4:4;
} EthTxStatus_t, * EthTxStatusPtr_t;

/* ETH_TX_THRESH_REG info type. */
typedef struct {
  UINT32 startSending:8; 
  UINT32 reserved1:8;
  UINT32 stopFilling:8; 
  UINT32 reserved2:8;
} EthTxThresh_t, * EthTxThreshPtr_t;

/* ETH_TX_PRI_THRESH_REG info type. */
typedef struct {
  UINT32 underrunPriThresh:8; 
  UINT32 reserved1:24;
} EthTxPriThresh_t, * EthTxPriThreshPtr_t;

/* ETH_TX_Q_PRI_MAP_REG info type. */
typedef struct {
  UINT32 nextQAfter0:2; 
  UINT32 reserved1:1;
  UINT32 revertAfter0:1; 
  UINT32 frameLimitQ0:4; 
  UINT32 nextQAfter1:2; 
  UINT32 reserved2:1;
  UINT32 revertAfter1:1; 
  UINT32 frameLimitQ1:4; 
  UINT32 nextQAfter2:2; 
  UINT32 reserved3:1;
  UINT32 revertAfter2:1; 
  UINT32 frameLimitQ2:4; 
  UINT32 nextQAfter3:2; 
  UINT32 reserved4:1;
  UINT32 revertAfter3:1; 
  UINT32 frameLimitQ3:4; 
} EthTxPriMap_t, * EthTxPriMapPtr_t;

/* ETH_TX_PAUSE_CNT_REG info type. */
typedef struct {
  UINT32 pauseCount:16; 
  UINT32 reserved1:16;
} EthTxPauseCnt_t, * EthTxPauseCntPtr_t;

/* ETH_TX_GVLAN_TAG_REG info type. */
typedef struct {
  UINT32 gVlanTag:32; 
} EthTxGVlan_t, * EthTxGVlanPtr_t;

/* TX/RX block info types. */
typedef struct {
  EthTxCfg_t            txCfgInfo;
  EthTxCtrl_t           txCtrlInfo;
  EthTxThresh_t         txThreshInfo;
  EthTxPriThresh_t      txPriThreshInfo;
  EthTxPriMap_t         txPriMapInfo;
  EthTxPauseCnt_t       txPauseCntInfo;
  EthTxGVlan_t          txGVlanInfo;
  EthTxQInfo_t          txQInfo[ETH_NUM_TX_Q];
} EthTxInfo_t, * EthTxInfoPtr_t;

/* ETH_RX_CFG_REG info type. */
typedef struct {
  UINT32 defaultQNum:2; 
  UINT32 reserved1:6; 
  UINT32 enableMcastHashFilter:1; 
  UINT32 enableUcastHashFilter:1; 
  UINT32 bcastFrmEnable:1; 
  UINT32 mcastFrmEnable:1; 
  UINT32 ucastFrmEnable:1; 
  UINT32 stationEnable:1; 
  UINT32 reserved2:7; 
  UINT32 alowBadFrm:1; 
  UINT32 autoPauseEnable:1; 
  UINT32 changePriority:1; 
  UINT32 reserved3:7;
  UINT32 reset:1; 
} EthRxCfg_t, * EthRxCfgPtr_t;

/* ETH_RX_CTRL_REG info type. */
typedef struct {
  UINT32 qEnables:4; 
  UINT32 reserved1:11;
  UINT32 go:1; 
  UINT32 reserved2:13;
  UINT32 enableAbortIntr:1;
  UINT32 abort:1;
  UINT32 enableIdleIntr:1;
} EthRxCtrl_t, * EthRxCtrlPtr_t;

/* ETH_RX_STATUS_REG info type. */
typedef struct {
  UINT32 qUsable:4; 
  UINT32 reserved1:4;
  UINT32 currentQ:2; 
  UINT32 reserved2:4;
  UINT32 prbs:1;
  UINT32 active:1;
  UINT32 qIdle:4; 
  UINT32 reserved3:12;
} EthRxStatus_t, * EthRxStatusPtr_t;

/* ETH_RX_XSTATUS_REG info type. */
typedef struct {
  UINT32 eoq:4; 
  UINT32 reserved1:4;
  UINT32 eof:4; 
  UINT32 reserved2:4;
  UINT32 descrIntr:4;
  UINT32 reserved3:4;
  UINT32 errFlag:4;
  UINT32 reserved4:4;
} EthRxXStatus_t, * EthRxXStatusPtr_t;

/* ETH_RX_PAUSE_THRESH_REG info type. */
typedef struct {
  UINT32 unpauseThresh:9; 
  UINT32 reserved1:7;
  UINT32 pauseThresh:9; 
  UINT32 reserved2:7;
} EthRxPauseThresh_t, * EthRxPauseThreshPtr_t;

/* ETH_RX_THRESH_REG info type. */
typedef struct {
  UINT32 overrunWarningThresh:9; 
  UINT32 reserved1:23;
} EthRxThresh_t, * EthRxThreshPtr_t;

/* ETH_RX_VLAN_TAG_MAP_REG info type. */
typedef struct {
  UINT32 qNumForTag0:2; 
  UINT32 reserved1:2; 
  UINT32 qNumForTag1:2; 
  UINT32 reserved2:2; 
  UINT32 qNumForTag2:2; 
  UINT32 reserved3:2; 
  UINT32 qNumForTag3:2; 
  UINT32 reserved4:2; 
  UINT32 qNumForTag4:2; 
  UINT32 reserved5:2; 
  UINT32 qNumForTag5:2; 
  UINT32 reserved6:2; 
  UINT32 qNumForTag6:2; 
  UINT32 reserved7:2; 
  UINT32 qNumForTag7:2; 
  UINT32 reserved8:2; 
} EthRxVlanMap_t, * EthRxVlanMapPtr_t;

typedef struct {
  EthRxCfg_t            rxCfgInfo;
  EthRxCtrl_t           rxCtrlInfo;
  EthRxPauseThresh_t    rxPauseThreshInfo;
  EthRxThresh_t         rxThreshInfo;
  EthRxVlanMap_t        rxVlanMapInfo;
  EthRxQInfo_t          rxQInfo[ETH_NUM_RX_Q];
} EthRxInfo_t, * EthRxInfoPtr_t;

/* ETH_MAC_CFG1_REG info type. */
typedef struct {
  UINT32 txEnable:1; 
  UINT32 synchTxEnable:1; 
  UINT32 rxEnable:1; 
  UINT32 synchRxEnable:1; 
  UINT32 txFlowCtrl:1; 
  UINT32 rxFlowCtrl:1; 
  UINT32 reserved1:2; 
  UINT32 loopBack:1; 
  UINT32 reserved2:7; 
  UINT32 resetTxFnct:1; 
  UINT32 resetRxFnct:1; 
  UINT32 resetTxMac:1; 
  UINT32 resetRxMac:1; 
  UINT32 reserved3:10; 
  UINT32 simReset:1; 
  UINT32 softReset:1; 
} EthMacCfg1_t, * EthMacCfg1Ptr_t;

/* ETH_MAC_CFG2_REG info type. */
typedef struct {
  UINT32 fullDuplex:1; 
  UINT32 crcEnable:1; 
  UINT32 padCrc:1; 
  UINT32 reserved1:1; 
  UINT32 lenCheck:1; 
  UINT32 hugeFrame:1; 
  UINT32 reserved2:2; 
  UINT32 ifMode:2; 
  UINT32 reserved3:2; 
  UINT32 preambleLen:4; 
  UINT32 reserved4:16; 
} EthMacCfg2_t, * EthMacCfg2Ptr_t;

/* ETH_MAC_MAX_FRAME_REG info type. */
typedef struct {
  UINT32 maxFrmLen:16; 
  UINT32 reserved1:16; 
} EthMacMaxFrm_t, * EthMacMaxFrmPtr_t;

/* ETH_MAC_TEST_REG info type. */
typedef struct {
  UINT32 slotTime:1; 
  UINT32 pause:1; 
  UINT32 txHdFlowEnable:1; 
  UINT32 maxBackOff:1; 
  UINT32 reserved1:28; 
} EthMacTest_t, * EthMacTestPtr_t;

/* ETH_MAC_ADDR_PART1_REG info type. */
typedef struct {
  UINT32 octet4:8; 
  UINT32 octet3:8; 
  UINT32 octet2:8; 
  UINT32 octet1:8; 
} EthMacAddr1_t, * EthMacAddr1Ptr_t;

/* ETH_MAC_ADDR_PART2_REG info type. */
typedef struct {
  UINT32 reserved1:16; 
  UINT32 octet6:8; 
  UINT32 octet5:8; 
} EthMacAddr2_t, * EthMacAddr2Ptr_t;

/* ETH_MAC_MII_CFG_REG info type. */
typedef struct {
  UINT32 mngmClkSel:3; 
  UINT32 reserved1:1; 
  UINT32 noPreamble:1; 
  UINT32 scanIncr:1; 
  UINT32 reserved2:25; 
  UINT32 resetMngm:1; 
} EthMacMiiCfg_t, * EthMacMiiCfgPtr_t;

/* ETH_MAC_MII_CMD_REG info type. */
typedef struct {
  UINT32 readCycle:1; 
  UINT32 scanCycle:1;
  UINT32 reserved1:30; 
} EthMacMiiCmd_t, * EthMacMiiCmdPtr_t;

/* ETH_MAC_MII_ADDR_REG info type. */
typedef struct {
  UINT32 regAddress:5; 
  UINT32 reserved1:3; 
  UINT32 phyAddress:5; 
  UINT32 reserved2:19; 
} EthMacMiiAddr_t, * EthMacMiiAddrPtr_t;

/* ETH_MAC_MII_CTRL_REG info type. */
typedef struct {
  UINT32 phyCtrl:16; 
  UINT32 reserved1:16; 
} EthMacMiiCtrl_t, * EthMacMiiCtrlPtr_t;

/* ETH_MAC_MII_STATUS_REG info type. */
typedef struct {
  UINT32 phyStatus:16; 
  UINT32 reserved1:16; 
} EthMacMiiStatus_t, * EthMacMiiStatusPtr_t;

/* ETH_MAC_MII_INDIC_REG info type. */
typedef struct {
  UINT32 busy:1; 
  UINT32 scanning:1;
  UINT32 notValid:1;
  UINT32 reserved1:29; 
} EthMacMiiIndic_t, * EthMacMiiIndicPtr_t;

/* ETH_MAC_IF_CTRL_REG info type. */
typedef struct {
  UINT32 jabberProtectEnable:1; 
  UINT32 reserved1:6;
  UINT32 resetGpsi:1;
  UINT32 linkFailDisable:1;
  UINT32 noCipher:1;
  UINT32 forceQuiet:1;
  UINT32 reserved2:4;
  UINT32 resetPmd:1;
  UINT32 speed:1;
  UINT32 reserved3:6;
  UINT32 resetRmii:1;
  UINT32 phyMode:1;
  UINT32 lhdMode:1;
  UINT32 ghdMode:1;
  UINT32 ifMode:1;
  UINT32 reserved4:3;
  UINT32 resetIf:1;
} EthMacIfCtrl_t, * EthMacIfCtrlPtr_t;

/* ETH_MAC_IF_STATUS_REG info type. */
typedef struct {
  UINT32 jabberPe10t:1; 
  UINT32 sqeErr:1; 
  UINT32 lossCarrier:1; 
  UINT32 linkFail:1; 
  UINT32 speed:1; 
  UINT32 fullDuplex:1; 
  UINT32 linkOk:1; 
  UINT32 jabberMiiPhy:1; 
  UINT32 clash:1; 
  UINT32 excessDefer:1; 
  UINT32 reserved1:22; 
} EthMacIfStatus_t, * EthMacIfStatusPtr_t;

/* MII info type. */
/*typedef struct {*/
/*} EthMiiInfo_t, * EthMiiInfoPtr_t;*/

/* ETH_TBI_CTRL1_REG info type. */
typedef struct {
  UINT32 reserved1:6; 
  UINT32 speed1:1; 
  UINT32 reserved2:1; 
  UINT32 fullDuplex:1; 
  UINT32 resetAn:1; 
  UINT32 reserved3:2; 
  UINT32 anEnable:1; 
  UINT32 speed0:1; 
  UINT32 loopBack:1; 
  UINT32 phyReset:1; 
} EthTbiCtrl1_t, * EthTbiCtrl1Ptr_t;

/* ETH_TBI_STATUS_REG info type. */
typedef struct {
  UINT32 extdCapability:1; 
  UINT32 reserved1:1; 
  UINT32 linkStatus:1; 
  UINT32 anAbility:1; 
  UINT32 remoteFault:1; 
  UINT32 anComplete:1; 
  UINT32 noPreamble:1; 
  UINT32 reserved2:1; 
  UINT32 extdStatus:1; 
  UINT32 reserved3:7; 
} EthTbiStatus_t, * EthTbiStatusPtr_t;

/* ETH_TBI_AN_ADV_REG info type. */
typedef struct {
  UINT32 reserved1:5; 
  UINT32 fullDuplex:1; 
  UINT32 halfDuplex:1; 
  UINT32 pause:2; 
  UINT32 reserved2:3; 
  UINT32 remoteFault:2; 
  UINT32 reserved3:1; 
  UINT32 nextPage:1; 
} EthTbiAnAdv_t, * EthTbiAnAdvPtr_t;

/* ETH_TBI_AN_LP_BP_REG info type. */
typedef struct {
  UINT32 reserved1:5; 
  UINT32 fullDuplex:1; 
  UINT32 halfDuplex:1; 
  UINT32 pause:2; 
  UINT32 reserved2:3; 
  UINT32 remoteFault:2; 
  UINT32 reserved3:1; 
  UINT32 nextPage:1; 
} EthTbiAnLpBp_t, * EthTbiAnLpBpPtr_t;

/* ETH_TBI_AN_EXP_REG info type. */
typedef struct {
  UINT32 reserved1:1; 
  UINT32 pageReceived:1; 
  UINT32 nextPageAble:1; 
  UINT32 reserved2:13; 
} EthTbiAnExp_t, * EthTbiAnExpPtr_t;

/* ETH_TBI_AN_NP_TX_REG info type. */
typedef struct {
  UINT32 message:11; 
  UINT32 toggle:1; 
  UINT32 ack2:1; 
  UINT32 messagePage:1; 
  UINT32 reserved1:1; 
  UINT32 nextPage:1; 
} EthTbiAnNpTx_t, * EthTbiAnNpTxPtr_t;

/* ETH_TBI_AN_LP_ANP_REG info type. */
typedef struct {
  UINT32 message:11; 
  UINT32 toggle:1; 
  UINT32 ack2:1; 
  UINT32 messagePage:1; 
  UINT32 reserved1:1; 
  UINT32 nextPage:1; 
} EthTbiAnLpAnp_t, * EthTbiAnLpAnpPtr_t;

/* ETH_TBI_EXT_STATUS_REG info type. */
typedef struct {
  UINT32 reserved1:12; 
  UINT32 t1000Half:1; 
  UINT32 t1000Full:1; 
  UINT32 x1000Half:1; 
  UINT32 x1000Full:1; 
} EthTbiExtStatus_t, * EthTbiExtStatusPtr_t;

/* ETH_TBI_JITT_DIAG_REG info type. */
typedef struct {
  UINT32 jitterPattern:10; 
  UINT32 reserved1:2; 
  UINT32 jitterSelect:3; 
  UINT32 jitterEnable:1; 
} EthTbiJittDiag_t, * EthTbiJittDiagPtr_t;

/* ETH_TBI_CTRL2_REG info type. */
typedef struct {
  UINT32 commaDetect:1; 
  UINT32 wrapEnable:1; 
  UINT32 reserved1:2; 
  UINT32 ifMode:1; 
  UINT32 rxClkSelect:1; 
  UINT32 reserved2:2; 
  UINT32 anSense:1; 
  UINT32 reserved3:3; 
  UINT32 txDisparDisable:1; 
  UINT32 rxDisparDisable:1; 
  UINT32 shortCutLinkTmr:1; 
  UINT32 softReset:1; 
} EthTbiCtrl2_t, * EthTbiCtrl2Ptr_t;

/* Start of BCM5421-specific code */ 

/* ETH_PHY_CTRL1_REG info type. */
typedef struct {
  UINT32 reserved1:6; 
  UINT32 speed1:1; 
  UINT32 cte:1; 
  UINT32 fullDuplex:1; 
  UINT32 resetAn:1; 
  UINT32 isolate:1; 
  UINT32 powerDown:1; 
  UINT32 anEnable:1; 
  UINT32 speed0:1; 
  UINT32 loopBack:1; 
  UINT32 phyReset:1; 
} EthPhyCtrl1_t, * EthPhyCtrl1Ptr_t;

/* ETH_PHY_STATUS_REG info type. */
typedef struct {
  UINT32 extdCapability:1; 
  UINT32 jabberDetect:1; 
  UINT32 linkStatus:1; 
  UINT32 anAbility:1; 
  UINT32 remoteFault:1; 
  UINT32 anComplete:1; 
  UINT32 noPreamble:1; 
  UINT32 reserved2:1; 
  UINT32 extdStatus:1; 
  UINT32 t2_100Half:1; 
  UINT32 t2_100Full:1; 
  UINT32 t10Half:1; 
  UINT32 t10Full:1; 
  UINT32 tx100Half:1; 
  UINT32 tx100Full:1; 
  UINT32 t4_100:1; 
} EthPhyStatus_t, * EthPhyStatusPtr_t;

/* ETH_PHY_AN_ADV_REG info type. */
typedef struct {
  UINT32 selectorField:5; 
  UINT32 t10Half:1; 
  UINT32 t10Full:1; 
  UINT32 tx100Half:1; 
  UINT32 tx100Full:1; 
  UINT32 t4_100:1; 
  UINT32 pause:1; 
  UINT32 asymPause:1; 
  UINT32 reserved1:1; 
  UINT32 remoteFault:1; 
  UINT32 reserved2:1; 
  UINT32 nextPage:1; 
} EthPhyAnAdv_t, * EthPhyAnAdvPtr_t;

/* ETH_PHY_1000BASE_T_REG info type. */
typedef struct {
  UINT32 reserved1:8; 
  UINT32 t1000Half:1; 
  UINT32 t1000Full:1;   
  UINT32 hubDte:1; 
  UINT32 masterSlaveVal:1; 
  UINT32 masterSlaveEnable:1; 
  UINT32 testMode:3; 
} EthPhy1000BaseT_t, * EthPhy1000BaseTPtr_t;

/* ETH_PHY_EXT_STATUS_REG info type. */
typedef struct {
  UINT32 reserved1:12; 
  UINT32 t1000Half:1; 
  UINT32 t1000Full:1; 
  UINT32 x1000Half:1; 
  UINT32 x1000Full:1; 
} EthPhyExtStatus_t, * EthPhyExtStatusPtr_t;

/* ETH_PHY_CTRL2_REG info type. */
typedef struct {
  UINT32 gmiiFifoElasticity:1; 
  UINT32 reserved1:1; 
  UINT32 reserved2:1; 
  UINT32 ledOff:1; 
  UINT32 ledOn:1; 
  UINT32 ledTrafficModeEnable:1; 
  UINT32 resetScrambler:1; 
  UINT32 bypassRsa:1; 
  UINT32 bypassMlt3:1; 
  UINT32 bypassScrambler:1; 
  UINT32 bypass4b5b:1; 
  UINT32 forceInterrupt:1; 
  UINT32 interruptDisable:1; 
  UINT32 txDisable:1; 
  UINT32 autoMdiCrossOverDisable:1; 
  UINT32 ifMode:1; 
} EthPhyCtrl2_t, * EthPhyCtrl2Ptr_t;

/* ETH_PHY_SUM_STATUS_REG info type. */
typedef struct {
  UINT32 txPauseEnable:1; 
  UINT32 rxPauseEnable:1; 
  UINT32 linkStatus:1; 
  UINT32 linkPartnerNPAbility:1; 
  UINT32 linkPartnerANAbility:1; 
  UINT32 anPageReceived:1; 
  UINT32 remoteFault:1; 
  UINT32 paralelDirFault:1; 
  UINT32 reportedStatus:3; 
  UINT32 anNPWait:1; 
  UINT32 anAbilityDetect:1; 
  UINT32 anAckDetect:1; 
  UINT32 anCompleteAck:1; 
  UINT32 anComplete:1; 
} EthPhySumStatus_t, * EthPhySumStatusPtr_t;

/* BCM5421 summary status type. */
typedef enum {
  ETH_PHY_NOT_OK = 0,           /* AN not completed or no common denominator*/
  ETH_PHY_10_HALF = 1,          /* 10BASE-T half-duplex. */     
  ETH_PHY_10_FULL = 2,          /* 10BASE-T full-duplex. */     
  ETH_PHY_100TX_HALF = 3,       /* 10BASE-TX half-duplex. */   
  ETH_PHY_100T4 = 4,            /* 10BASE-T4. */   
  ETH_PHY_100TX_FULL = 5,       /* 10BASE-T full-duplex. */     
  ETH_PHY_1000T_HALF = 6,       /* 1000BASE-T half-duplex. */     
  ETH_PHY_1000T_FULL = 7,       /* 1000BASE-T full-duplex. */     
} EthPhyStatusMode_t;

/* End of BCM5421-specific code */ 

/* PHY info type. */
typedef struct {
  EthTbiCtrl1_t tbiCtrl1Info; /* TBI Control1 register info. */
  EthTbiAnAdv_t tbiAnAdvInfo; /* TBI AN Advertisement register info. */
  EthTbiCtrl2_t tbiCtrl2Info; /* TBI Control2 register info. */
  
/* Start of BCM5421-specific code */ 
  EthPhyCtrl1_t phyCtrl1Info; /* PHY Control1 register info. */
  EthPhyAnAdv_t phyAnAdvInfo; /* PHY AN Advertisement register info. */
  EthPhy1000BaseT_t phy1000BaseTInfo; /* PHY 1000BASE-T register info. */
  EthPhyCtrl2_t phyCtrl2Info; /* PHY Control2 register info. */
/* End of BCM5421-specific code */ 

} EthLinkInfo_t, * EthLinkInfoPtr_t;

/* MAC info type. */  
typedef struct {
  EthMacCfg1_t   cfg1Info;      /* config1 register info. */
  EthMacCfg2_t   cfg2Info;      /* config2 register info. */
  EthMacMaxFrm_t maxFrmInfo;    /* max frame register info. */
  EthMacMiiCfg_t miiCfgInfo;    /* MII config register info. */
  EthMacIfCtrl_t ifCtrlInfo;    /* interface control register info. */
  EthMacAddr1_t  addrPart1Info; /* address part1 register info. */
  EthMacAddr2_t  addrPart2Info; /* address part2 register info. */
  EthLinkInfo_t  linkInfo;      /* Link partner info. */
} EthMacInfo_t, * EthMacInfoPtr_t;

/* ETH_PORT_CTRL_REG info type. */
typedef struct {
  UINT32 portPriority:1; 
  UINT32 reserved1:15; 
  UINT32 backPressureType:1; 
  UINT32 reserved2:1; 
  UINT32 speed:1; 
  UINT32 rbcMode:1; 
  UINT32 reserved3:1; 
  UINT32 prbsEnable:1; 
  UINT32 disableTxCgo:1; 
  UINT32 tbiOutputEnable:1; 
  UINT32 reserved4:4; 
  UINT32 statisticsEnable:1; 
  UINT32 zeroOnRead:1; 
  UINT32 clearStatistics:1; 
  UINT32 statusReset:1; 
} EthPortCtrl_t, * EthPortCtrlPtr_t;

/* ETH_PORT_INT_MASK_REG info type. */
typedef struct {
  UINT32 txQueues:4; 
  UINT32 reserved1:4; 
  UINT32 txWait:1; 
  UINT32 txThreshold:1; 
  UINT32 txUnderrun:1; 
  UINT32 txError:1; 
  UINT32 txAbort:1; 
  UINT32 txIdle:1; 
  UINT32 reserved2:1; 
  UINT32 statsCarry:1; 
  UINT32 rxQueues:4; 
  UINT32 reserved3:4; 
  UINT32 rxWait:1; 
  UINT32 rxThreshold:1; 
  UINT32 rxOverrun:1; 
  UINT32 rxError:1; 
  UINT32 rxAbort:1; 
  UINT32 rxIdle:1; 
  UINT32 ocnError:1;
  UINT32 reserved4:1; 
} EthPortIntrMask_t, * EthPortIntrMaskPtr_t;

/* ETH_PORT_INT_STATUS_REG info type. */
typedef struct {
  UINT32 txQueues:4; 
  UINT32 reserved1:4; 
  UINT32 txWait:1; 
  UINT32 txThreshold:1; 
  UINT32 txUnderrun:1; 
  UINT32 txError:1; 
  UINT32 txAbort:1; 
  UINT32 txIdle:1; 
  UINT32 reserved2:1; 
  UINT32 statsCarry:1; 
  UINT32 rxQueues:4; 
  UINT32 reserved3:4; 
  UINT32 rxWait:1; 
  UINT32 rxThreshold:1; 
  UINT32 rxOverrun:1; 
  UINT32 rxError:1; 
  UINT32 rxAbort:1; 
  UINT32 rxIdle:1; 
  UINT32 ocnError:1;
  UINT32 intrStatus:1; 
} EthPortIntrStatus_t, * EthPortIntrStatusPtr_t;

/* ETH_PORT_OCN_STATUS_REG info type. */
typedef struct {
  UINT32 ocnResponseHdr:24; 
  UINT32 ocnErrCount:8; 
} EthPortOcnStatus_t, * EthPortOcnStatusPtr_t;

/* ETH_MSTAT_CAR1_REG info type. */
typedef struct {
  UINT32 rdrp:1; 
  UINT32 rjbr:1; 
  UINT32 rfrg:1; 
  UINT32 rovr:1; 
  UINT32 rund:1; 
  UINT32 rcse:1; 
  UINT32 rcde:1; 
  UINT32 rflr:1; 
  UINT32 raln:1; 
  UINT32 rxuo:1; 
  UINT32 rxpf:1; 
  UINT32 rxcf:1; 
  UINT32 rbca:1; 
  UINT32 rmca:1; 
  UINT32 rfcs:1; 
  UINT32 rpkt:1; 
  UINT32 rbyt:1; 
  UINT32 reserved:8; 
  UINT32 trmgv:1; 
  UINT32 trmax:1; 
  UINT32 tr1k:1; 
  UINT32 tr511:1; 
  UINT32 tr255:1; 
  UINT32 tr127:1; 
  UINT32 tr64:1; 
} EthMstatCar1_t, * EthMstatCar1Ptr_t;

/* ETH_MSTAT_CARM1_REG info type. */
typedef struct {
  UINT32 rdrp:1; 
  UINT32 rjbr:1; 
  UINT32 rfrg:1; 
  UINT32 rovr:1; 
  UINT32 rund:1; 
  UINT32 rcse:1; 
  UINT32 rcde:1; 
  UINT32 rflr:1; 
  UINT32 raln:1; 
  UINT32 rxuo:1; 
  UINT32 rxpf:1; 
  UINT32 rxcf:1; 
  UINT32 rbca:1; 
  UINT32 rmca:1; 
  UINT32 rfcs:1; 
  UINT32 rpkt:1; 
  UINT32 rbyt:1; 
  UINT32 reserved:8; 
  UINT32 trmgv:1; 
  UINT32 trmax:1; 
  UINT32 tr1k:1; 
  UINT32 tr511:1; 
  UINT32 tr255:1; 
  UINT32 tr127:1; 
  UINT32 tr64:1; 
} EthMstatCarm1_t, * EthMstatCarm1Ptr_t;

/* ETH_MSTAT_CAR2_REG info type. */
typedef struct {
  UINT32 tdrp:1; 
  UINT32 tpfh:1; 
  UINT32 tncl:1; 
  UINT32 txcl:1; 
  UINT32 tlcl:1; 
  UINT32 tmcl:1; 
  UINT32 tscl:1; 
  UINT32 tedf:1; 
  UINT32 tdfr:1; 
  UINT32 txpf:1; 
  UINT32 tbca:1; 
  UINT32 tmca:1; 
  UINT32 tpkt:1; 
  UINT32 tbyt:1; 
  UINT32 tfrg:1; 
  UINT32 tund:1;   
  UINT32 tovr:1; 
  UINT32 txcf:1; 
  UINT32 tfcs:1; 
  UINT32 tjbr:1; 
  UINT32 reserved:12; 
} EthMstatCar2_t, * EthMstatCar2Ptr_t;

/* ETH_MSTAT_CARM2_REG info type. */
typedef struct {
  UINT32 tdrp:1; 
  UINT32 tpfh:1; 
  UINT32 tncl:1; 
  UINT32 txcl:1; 
  UINT32 tlcl:1; 
  UINT32 tmcl:1; 
  UINT32 tscl:1; 
  UINT32 tedf:1; 
  UINT32 tdfr:1; 
  UINT32 txpf:1; 
  UINT32 tbca:1; 
  UINT32 tmca:1; 
  UINT32 tpkt:1; 
  UINT32 tbyt:1; 
  UINT32 tfrg:1; 
  UINT32 tund:1;   
  UINT32 tovr:1; 
  UINT32 txcf:1; 
  UINT32 tfcs:1; 
  UINT32 tjbr:1; 
  UINT32 reserved:12; 
} EthMstatCarm2_t, * EthMstatCarm2Ptr_t;

/* MSTAT info type. */
typedef struct {
  EthMstatCarm1_t       carm1Info;
  EthMstatCarm2_t       carm2Info;
} EthMstatInfo_t, * EthMstatInfoPtr_t;

/* GigE port info type. */  
typedef struct {
  EthPortCtrl_t         portCtrlInfo;
  EthPortIntrMask_t     portIntrMaskInfo;
  EthMacInfo_t          macInfo;        /* MAC block info. */
  EthMstatInfo_t        mstatInfo;      /* MSTAT block info. */
  EthTxInfo_t           txInfo;         /* TX block info. */
  EthRxInfo_t           rxInfo;         /* RX block info. */
} EthPortInfo_t, * EthPortInfoPtr_t;

/******************************************************************************
 * Driver structures. They are used to object-oriented model the device. 
 */

/* TX/RX queue types. */
typedef struct {
  EthMemPool_t memPool; 
} EthTxQ_t, * EthTxQPtr_t;

typedef struct {
  EthMemPool_t  memPool; 
  volatile BOOL invalidFrm;     /* Frame status info. */ 
  VUINT16       vLanTag;        /* VLAN tag info of the last SOF. */    
} EthRxQ_t, * EthRxQPtr_t;

/* TX/RX block types. */
typedef struct {
  EthTxQ_t txQ[ETH_NUM_TX_Q];
} EthTx_t, * EthTxPtr_t;

typedef struct {
  EthRxQ_t rxQ[ETH_NUM_RX_Q];
} EthRx_t, * EthRxPtr_t;

/* MAC type. */  
typedef struct {
  EthTx_t tx; /* TX block. */
  EthRx_t rx; /* RX block. */
} EthMac_t, * EthMacPtr_t;

/* GigE port type. */  
typedef struct {
  EthMac_t      mac;            /* MAC block. */
  volatile BOOL portEnabled;    /* Port status flag. */  
} EthPort_t, * EthPortPtr_t;

/* GigE block type. */
typedef struct {
  EthPort_t             port[ETH_NUM_PORTS];    /* port(0, 1). */
  EthPortInfoPtr_t      portInfo[ETH_NUM_PORTS];/* Port(0, 1) info, passed
                                                   on at init time. */  
  volatile BOOL         drvEnabled;             /* Driver status flag. */
} Eth_t, * EthPtr_t;


/* Number of port directions. */
#define ETH_NUM_PORT_DIRECTIONS (2)

/* Begins user-configurable section. 
*************************************/

#define ETH_MII_MNGM_WAIT_COUNT (100)  /* for now. */ 

/* Max number of buffer bytes to be dumped. */
#define ETH_NUM_DUMPED_BYTES (100)

/* User modifiable configuration information id. */
#define ETH_CFG_INFO_IDX (0)

/* User modifiable memory space id. */
#define ETH_MEM_ALLOC_ID (ETH_OCN_SDRAM_ID)

/* Max number of PCI1/2 and SDRAM descriptors per queue. */
#define ETH_MAX_NUM_PCI_DESCR_PER_Q (16)

/* Max number of PCI1/2 and SDRAM buffers per descriptor. */
#define ETH_NUM_PCI_BUFF_PER_DESCR (3)

/* memory buffer size on PCI1/2 and SDRAM. */
#define ETH_MAX_PCI_BUFF_SIZE TM_80314_LARGE_RECV_BUF_SIZE

/* memory buffer size on SRAM. */
#define ETH_MAX_SRAM_BUFF_SIZE (ETH_BUFF_SIZE_96)

/* Number of descriptors per queue. */
#define ETH_NUM_DESCR_PER_Q (ETH_MAX_NUM_PCI_DESCR_PER_Q)

/* Number of buffers per descriptor. */
#define ETH_NUM_BUFF_PER_DESCR (ETH_NUM_PCI_BUFF_PER_DESCR)

/* Buffer size. */
#define ETH_MAX_BUFF_SIZE (ETH_MAX_PCI_BUFF_SIZE)

/* more for buffer allocation on PCI1/2 and SDRAM. 
   less for buffer allocation on SRAM. */
#define ETH_BUFF_MEM_SIZE (ETH_NUM_PORTS * (ETH_NUM_TX_Q + ETH_NUM_RX_Q) * \
                           ETH_NUM_DESCR_PER_Q * ETH_NUM_BUFF_PER_DESCR * \
                           ETH_MAX_BUFF_SIZE)
                           
/* Number of alocated buffers. */
#define ETH_NUM_BUFFS (ETH_BUFF_MEM_SIZE / ETH_MAX_BUFF_SIZE)
                           
/* more for descriptor allocation on PCI1/2 and SDRAM.
   less for descriptor allocation on SRAM. */
#define ETH_DESCR_MEM_SIZE (ETH_NUM_PORTS * (ETH_NUM_TX_Q + ETH_NUM_RX_Q) * \
                           ETH_NUM_DESCR_PER_Q * sizeof(EthBuffDescr_t))

/* User modifiable OCN TX/RX burst size. */
#define ETH_OCN_BURST_SIZE (ETH_OCN_BURST_256)

/* PCI1/2 and SRAM descriptor memory types. */ 

 
typedef struct {
  EthBuffDescr_t descrTable[ETH_NUM_DESCR_PER_Q]; 
} EthDescrQ_t, * EthDescrQPtr_t;

typedef struct {
  EthDescrQ_t descrQ[ETH_NUM_TX_Q]; 
} EthDirectDescr_t, * EthDirectDescrPtr_t;

typedef struct {
  EthDirectDescr_t directDescrQ[ETH_NUM_PORT_DIRECTIONS]; 
} EthPortDescr_t, * EthPortDescrPtr_t;

typedef struct {
  EthPortDescr_t portDescr[ETH_NUM_PORTS]; 
} EthMemDescr_t, * EthMemDescrPtr_t;

/* define own allocator in the SDRAM high memeory. */
/* NOTE: this must be 32-byte aligned, because the 80200 cache line size is
   32-bytes and each frame descriptor must have its own cache line. Frame
   descriptors start here. */
#define SDRAM_HIGH_MEM (0x0C000000)

/* Ends user-configurable section. 
*************************************/

/*
 * 
 * End of SpacEthDefs.h 
 */


/*
 * SpacEthRegOps.h
 */

/* Operation codes. */
typedef enum {
    REG_WRITE = 0,   
    REG_READ = 1,   
    REG_AND = 2,   
    REG_OR = 3,   
    REG_XOR = 4,   
} SpacEthRegOps_t;


/*=========================================================================
 *  Function    : EthRegAccess
 *
 *  Arguments   : portId - port ID (0, 1)
 *              : regName - register offset from the Port's base address.
 *              : regValue - value to be read/written from/to.
 *              : operation - operation to be completed.
 * 
 *  Description : Accesses the chip GigE block's registers.
 * 
 *  Return      : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthRegAccess(
  UINT32 portId, 
  UINT32 regName, 
  UINT32 * regValue,
  SpacEthRegOps_t operation
);
 
/*=========================================================================
 *  Function    : EthIndirectRegAccess
 *
 *  Arguments   : portId - port ID (0, 1)
 *              : addrRegName - register offset from the Port's base address.
 *              : dataRegName - register offset from the Port's base address.
 *              : regValue1 - value containing the index to be written. Input.
 *              : regValue2 - value to be read/written from/to. Input/output.
 *              : operation - operation to be completed.
 * 
 *  Description : Indirect accesses the chip GigE block's registers.
 * 
 *  Return      : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : See the Indirect Register Access mechanism description.
 *              : This routine is used to initiate the first access. 
 *              : Subsequent accesses use EthRegAccess routine.
 *              : Supported operations are REG_WRITE and REG_READ.
 =========================================================================*/
SpacEthRtnCode_t EthIndirectRegAccess(
  UINT32 portId, 
  UINT32 addrRegName, 
  UINT32 dataRegName, 
  UINT32 * regValue1,
  UINT32 * regValue2,
  SpacEthRegOps_t operation
);
 
/*
 * End of SpacEthRegOps.h
 */


/*
 * SpacEthInterrupts.h
 */


/* IRQ vector @0x18 calls 0x38 */
#define IRQVECTOR ((volatile unsigned int *)0x00000038)
/* FIQ vector @0x1C calls 0x3C */
#define FIQVECTOR ((volatile unsigned int *)0x0000003c )
  
/* Maximal number of user-defined interrupt vectors */  
#define MAX_USER_VECTORS (64) 
/* Temporary define for the GigE ISR vector. (the IRQ line) */
#define ETH_MPIC_GIGE_VECTOR (16)

/* timer0 interrupt vector  uses 32 please */
#define ETH_MPIC_TIMER0_VECTOR (32)
/* MPIC low base address, as an offset from the chip base address. */
#define ETH_MPIC_BASE_ADDR_LO (0x7400)
/* MPIC high base address, as an offset from the chip base address. */
#define ETH_MPIC_BASE_ADDR_HI (0x77ff) 
/* MPIC Registers. */
/* Timer 0: 10ms is equivallent to 0x145855 count * 7.5ns*/
#define ETH_MPIC_TIMER_COUNT    (0x145855)
#define ETH_MPIC_GTCCR0_REG (ETH_MPIC_BASE_ADDR_LO + 0x30)
#define ETH_MPIC_GTBCR0_REG (ETH_MPIC_BASE_ADDR_LO + 0x34)
#define ETH_MPIC_GTVPR0_REG (ETH_MPIC_BASE_ADDR_LO + 0x38)
#define ETH_MPIC_GTDR0_REG  (ETH_MPIC_BASE_ADDR_LO + 0x3c)
#define ETH_MPIC_TIMER_OFFSET                 (0x10)
#define ETH_MPIC_IVPR16_REG (ETH_MPIC_BASE_ADDR_LO + 0x180)
#define ETH_MPIC_IDR16_REG (ETH_MPIC_BASE_ADDR_LO + 0x184)
#define ETH_MPIC_TASK0_REG (ETH_MPIC_BASE_ADDR_LO + 0x300)
#define ETH_MPIC_TASK1_REG (ETH_MPIC_BASE_ADDR_LO + 0x340)
#define ETH_MPIC_TASK2_REG (ETH_MPIC_BASE_ADDR_LO + 0x380)
#define ETH_MPIC_TASK3_REG (ETH_MPIC_BASE_ADDR_LO + 0x3C0)
#define ETH_MPIC_IACK0_REG (ETH_MPIC_BASE_ADDR_LO + 0x304) 
#define ETH_MPIC_IACK1_REG (ETH_MPIC_BASE_ADDR_LO + 0x344) 
#define ETH_MPIC_IACK2_REG (ETH_MPIC_BASE_ADDR_LO + 0x384) 
#define ETH_MPIC_IACK3_REG (ETH_MPIC_BASE_ADDR_LO + 0x3C4) 
#define ETH_MPIC_EOI0_REG (ETH_MPIC_BASE_ADDR_LO + 0x308) 
#define ETH_MPIC_EOI1_REG (ETH_MPIC_BASE_ADDR_LO + 0x348) 
#define ETH_MPIC_EOI2_REG (ETH_MPIC_BASE_ADDR_LO + 0x388) 
#define ETH_MPIC_EOI3_REG (ETH_MPIC_BASE_ADDR_LO + 0x3C8) 
#define ETH_MPIC_CSR0_REG (ETH_MPIC_BASE_ADDR_LO + 0x30C)
#define ETH_MPIC_CSR1_REG (ETH_MPIC_BASE_ADDR_LO + 0x34C)
#define ETH_MPIC_CSR2_REG (ETH_MPIC_BASE_ADDR_LO + 0x38C)
#define ETH_MPIC_CSR3_REG (ETH_MPIC_BASE_ADDR_LO + 0x3CC)
/* MPIC register masks. */
#define ETH_MPIC_IACK_MASK (0x000000FF)
#define ETH_MPIC_EOI_MASK (0x000000FF)
#define ETH_MPIC_IVPR_M_MASK (1 << 31)
#define ETH_MPIC_IVPR_PRIO_MASK (0xF << 16)
#define ETH_MPIC_IVPR_S_MASK (1 << 25)
#define ETH_MPIC_IVPR_P_MASK (1 << 24)
#define ETH_MPIC_IDR_OUT1_MASK (1 << 0)
#define ETH_MPIC_IDR_OUT2_MASK (1 << 1)
#define ETH_MPIC_IDR_OUT3_MASK (1 << 2)
#define ETH_MPIC_IDR_OUT4_MASK (1 << 3)
#define ETH_MPIC_TASK_MASK (0x00000000) /* task priority 0 - lowest */
/* CIU low base address, as an offset from the chip base address. */
#define ETH_CIU_BASE_ADDR_LO (0x3000)
/* CIU high base address, as an offset from the chip base address. */
#define ETH_CIU_BASE_ADDR_HI (0x3fff) 
/* CIU Registers. */
#define ETH_CIU_CFG_REG (ETH_CIU_BASE_ADDR_LO + 0x038)
/* CIU register masks. */
#define ETH_CIU_PID_MASK (1 << 8)
/* 80200 register masks. */
#define ETH_XSCALE_INTCTL_MASK (1 << 1)  

typedef void (*ISR_SERVICE)(void);
  
/*----------------------------------------------------------------------------
 * Macro: Save and restore registers inside exception handler. 
   NOTE :
   The IRQ_OVI_EPILOG macro is intended as a workaround for a GNU compiler
   stack issue. The code generated by the compiler does not properly
   restore the stack pointer, in case of a routine flagged as being 
   an interrupt handler.
   It is recommended that the IRQ_OVI_EPILOG macro should be re-inspected
   after each interrupt handler code modification, and updated if needed.
 *--------------------------------------------------------------------------*/
#define ISR_PROLOG  asm volatile("stmfd sp!, {r0-r11,ip,lr}")
#define ISR_EPILOG  asm volatile("ldmfd sp!,{r0-r11,ip,lr}\n\t" \
                                      "subs pc, lr, #4")
#define IRQ_OVI_EPILOG \
        asm volatile("ldmdb r11, {r0,r1,r2,r3,r11,sp,lr}\n\t" \
        "ldmia sp!, {r12}\n\t" \
        "subs pc, lr, #4")
  

/*----------------------------------------------------------------------------
 * Macro: Restore registers and jump to original vector address 
 *--------------------------------------------------------------------------*/
  #define EXCEPTION_CHAIN(oldVec) \
                asm volatile("stmfd   sp!, {%0};               \n\t" \
                "ldmfd   sp!, {r0, r1, r2, r3, r4, r5, r6, ip, lr, pc}; \n\t" \
                      :               /* Output  list */                \
                      : "r" (oldVec)  /* Input   list */                \
                                      /* Clobber list */);
  
/*----------------------------------------------------------------------------
 * Macro: Load new stack pointer value 
 *--------------------------------------------------------------------------*/
#define INIT_STACK_PTR(address) asm volatile("mov r0, #0; \n\t" \
                                    "mov sp, %0; \n\t" \
                                    :  \
                                    :"r" (address)  \
                                      )
  
/*=========================================================================
 *  Function    : intMpicInit
 *
 *  Arguments   : N/A.
 * 
 *  Description : Initializes the MPIC block.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_INVAL_CFG_ERR - Invalid configuration error. 
 *
 *  Note        : For now GigE-oriented. 
 *           
 =========================================================================*/
SpacEthRtnCode_t intMpicInit(void);
  
/*=========================================================================
 *  Function    : intMpicReset
 *
 *  Arguments   : N/A.
 * 
 *  Description : Resets the MPIC block.
 * 
 *  return      : ETH_OK - Operation succeded.
 *
 *  Note        : For now GigE-oriented. 
 *           
 =========================================================================*/
SpacEthRtnCode_t intMpicReset(void);
  
/*=========================================================================
 *  Function    : SpacIrqIsr
 *
 *  Arguments   : N/A.
 * 
 *  Description : SPac's ISR.
 * 
 *  return      : N/A.
 *
 *  Note        : For now servicing only GigE interrupts. They are expected
 *              : at output 1 only.
 *              
 =========================================================================*/
void SpacIrqIsr(void) __attribute__ ((interrupt ("IRQ")));

/*=========================================================================
 *  Function    : SpacFiqIsr
 *
 *  Arguments   : N/A.
 * 
 *  Description : SPac's ISR.
 * 
 *  return      : N/A.
 *
 *  Note        : For now servicing only FIQ GigE interrupts. They are expected
 *              : at output 0 only.
 *              
 =========================================================================*/
void SpacFiqIsr(void) __attribute__ ((interrupt ("FIQ")));

/*=========================================================================
 *  Function    : gigeHandlerAttachIRQ
 *
 *  Arguments   : irq_service - ISR.
 * 
 *  Description : SPac's ISR attach routine.
 * 
 *  return      : N/A.
 *
 =========================================================================*/
void gigeHandlerAttachIRQ(ISR_SERVICE irq_service);

/*=========================================================================
 *  Function    : gigeHandlerAttachFIQ
 *
 *  Arguments   : irq_service - ISR.
 * 
 *  Description : SPac's ISR attach routine.
 * 
 *  return      : N/A.
 *
 =========================================================================*/
void gigeHandlerAttachFIQ(ISR_SERVICE fiq_service);

/*=========================================================================
 *  Function    : gigeHandlerDetachIRQ
 *
 *  Arguments   : N/A.
 * 
 *  Description : SPac's ISR dettach routine.
 * 
 *  return      : N/A.
 *
 =========================================================================*/
void gigeHandlerDetachIRQ(void);

/*=========================================================================
 *  Function    : gigeHandlerDetachFIQ
 *
 *  Arguments   : N/A.
 * 
 *  Description : SPac's ISR dettach routine.
 * 
 *  return      : N/A.
 *
 =========================================================================*/
void gigeHandlerDetachFIQ(void);

/*=========================================================================
 *  Function    : gigeHandlerAttachVector
 *
 *  Arguments   : req_type - IRQ/FIQ (not used for now).
 *              : vector - index into the vector table.
 *              : int_routine - vector handler to attach. 
 * 
 *  Description : Attach service routine for interrupt vector.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *              
 =========================================================================*/
SpacEthRtnCode_t gigeHandlerAttachVector(
  int req_type,  
  unsigned vector, 
  SpacEthRtnCode_t (*int_routine)(void *));

/*=========================================================================
 *  Function    : EthIsr
 *
 *  Arguments   : infoPtr - input, user info passed in.
 * 
 *  Description : GigE's ISR.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : Use of "PRINTF" macro to allow for non-blocking logging.
 *              : Port0 will get first chance to service its interrupts.
 *              
 =========================================================================*/
SpacEthRtnCode_t EthIsr(
  void * infoPtr
);

/*=========================================================================
 *  Function    : EthPortIsr
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : intrStatus - Port interrupt status.
 *              : infoPtr - input, user info passed in. 
 * 
 *  Description : GigE's ISR.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : Use of "PRINTF" macro to allow for non-blocking logging.
 *              : Prior to returning, this routine, attempts to clear the 
 *              : interrupt sources.
 =========================================================================*/
SpacEthRtnCode_t EthPortIsr(
  UINT32 portId, 
  register UINT32 intrStatus,   
  void * infoPtr
);

/*=========================================================================
 *  Function    : EthRxQueueIsr
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : info - input, used to retrieve the qeue id. 
 * 
 *  Description : GigE's RX QUEUE ISR.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *
 *  Note        : Use of "PRINTF" macro to allow for non-blocking logging.
 *              : An EOQ condition is expected to happen if a frame was 
 *              : received, and DMA has encountered a LAST/NULL EOQ 
 *              : condition on the last processed descriptor. Update the 
 *              : queue pointer, clean-up and restart the queue. 
 =========================================================================*/
SpacEthRtnCode_t EthRxQueueIsr(
  UINT32 portId, 
  UINT32 info
);
/*
 * End of SpacEthInterrupts.h
 */


/*
 * SpacEthMac.h
 */


/* Interface(to which MAC is connected) mode type. */
typedef enum {
  ETH_MAC_IF_SPEED_10 =         1, /* 10Mb/s. */     
  ETH_MAC_IF_SPEED_100 =        2, /* 100Mb/s. */    
  ETH_MAC_IF_SPEED_1000 =       3, /* 1000Mb/s. */     
} EthMacIfSpeed_t;

/* Duplex mode type. */
typedef enum {
  ETH_MAC_DPLX_HALF =           1, /* Half-duplex. */     
  ETH_MAC_DPLX_FULL =           2, /* Full-duplex. */    
} EthMacDuplex_t;

/*=========================================================================
 *  Function    : EthConfigMac
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : macInfoPtr - Configuration info pointer.
 * 
 *  Description : Configures the chip's MAC block of a specified GigE port.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_MII_IF_STATUS_ERR - MII interface status error.
 *              : ETH_TIMEOUT_ERR - Time-out error.
 *              : ETH_INVAL_CFG_ERR - Invalid configuration error.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthConfigMac(
  UINT32 portId, 
  EthMacInfoPtr_t macInfoPtr
);

/*
 * End of SpacEthMac.h
 */


typedef struct tsForestLakeInfo
{
    tt8Bit            flPortId;
}
ttForestLakeInfo;
typedef ttForestLakeInfo TM_FAR * ttForestLakeInfoPtr;


typedef SpacEthRtnCode_t (* EthSourceIsrPtr_t)(UINT32 portId, UINT32 info);
const EthSourceIsrPtr_t EthIsrTable[ETH_MAX_NUM_INTR_SOURCES] = 
{
/* interrupt status bit 31. */ 0/*NULL By Jimmy*/,
/* interrupt status bit 30. */ 0,/* EthOcnErrIsr, */
/* interrupt status bit 29. */ 0,/* EthRxIdleIsr, */
/* interrupt status bit 28. */ 0,/* EthRxAbortIsr, */
/* interrupt status bit 27. */ 0,/* EthRxErrorIsr, */
/* interrupt status bit 26. */ 0,/* EthRxOverrunIsr, */
/* interrupt status bit 25. */ 0,/* EthRxThresholdIsr, */
/* interrupt status bit 24. */ 0,/* EthRxWaitIsr, */
/* interrupt status bit 23. */ 0/*NULL*/,
/* interrupt status bit 22. */ 0/*NULL*/,
/* interrupt status bit 21. */ 0/*NULL*/,
/* interrupt status bit 20. */ 0/*NULL*/,
/* interrupt status bit 19. */ EthRxQueueIsr, /* RX queue 3 */
/* interrupt status bit 18. */ EthRxQueueIsr, /* RX queue 2 */
/* interrupt status bit 17. */ EthRxQueueIsr, /* RX queue 1 */
/* interrupt status bit 16. */ EthRxQueueIsr, /* RX queue 0 */
/* interrupt status bit 15. */ /* EthStatsCarryIsr, */
/* interrupt status bit 14. */ 0/*NULL*/,
/* interrupt status bit 13. */ 0,/* EthTxIdleIsr, */
/* interrupt status bit 12. */ 0,/* EthTxAbortIsr, */
/* interrupt status bit 11. */ 0,/* EthTxErrorIsr, */
/* interrupt status bit 10. */ 0,/* EthTxUnderrunIsr, */
/* interrupt status bit 9. */  0,/* EthTxThresholdIsr, */
/* interrupt status bit 8. */  0,/* EthTxWaitIsr, */
/* interrupt status bit 7. */  0/*NULL*/,
/* interrupt status bit 6. */  0/*NULL*/,
/* interrupt status bit 5. */  0/*NULL*/,
/* interrupt status bit 4. */  0/*NULL*/,
/* interrupt status bit 3. */  0,//EthTxQueueIsr,
/* interrupt status bit 2. */  0,//EthTxQueueIsr,
/* interrupt status bit 1. */  0,//EthTxQueueIsr,
/* interrupt status bit 0. */  0,//EthTxQueueIsr,
};
  
/* SPac GigE driver's global config info object. */
EthPortInfo_t EthDriverInfo[] = 
{
/* Begins Config0 info. */
    {
        {
            ETH_BIT_CLEARED,    /* portPriority:1 */ 
            0x00000000,         /* reserved1:15 */ 
            ETH_BIT_CLEARED,    /* backPressureType:1 */ 
            0x00000000,         /* reserved2:1 */ 
            ETH_BIT_CLEARED,    /* speed:1 */ 
            ETH_BIT_SET,        /* rbcMode:1 */ 
            0x00000000,         /* reserved3:1 */ 
            ETH_BIT_CLEARED,    /* prbsEnable:1 */ 
            ETH_BIT_CLEARED,    /* disableTxCgo:1 */ 
            ETH_BIT_CLEARED,    /* tbiOutputEnable:1 */ 
            0x00000000,         /* reserved4:4 */ 
            ETH_BIT_SET,        /* statisticsEnable:1 */ 
            ETH_BIT_CLEARED,    /* zeroOnRead:1 */ 
            ETH_BIT_CLEARED,    /* clearStatistics:1 */ 
            ETH_BIT_CLEARED     /* statusReset:1 */ 
        },                      /* portCtrlInfo */
/* Interrupt mask */
        {
            0xF,                /* txQueues:4 */ 
            0x0,                /* reserved1:4 */ 
            ETH_BIT_SET,        /* txWait:1 */ 
            ETH_BIT_SET,        /* txThreshold:1 */ 
            ETH_BIT_SET,        /* txUnderrun:1 */ 
            ETH_BIT_SET,        /* txError:1 */ 
            ETH_BIT_SET,        /* txAbort:1 */ 
            ETH_BIT_SET,        /* txIdle:1 */ 
            0x0,                /* reserved2:1 */ 
            ETH_BIT_SET,        /* statsCarry:1 */ 
            0x0,                /* rxQueues:4 */ 
            0x0,                /* reserved3:4 */ 
            ETH_BIT_SET,        /* rxWait:1 */ 
            ETH_BIT_SET,        /* rxThreshold:1 */ 
            ETH_BIT_SET,        /* rxOverrun:1 */ 
            ETH_BIT_SET,        /* rxError:1 */ 
            ETH_BIT_SET,        /* rxAbort:1 */ 
            ETH_BIT_SET,        /* rxIdle:1 */ 
            ETH_BIT_SET,        /* ocnError:1 */
            0x0                 /* reserved4:1 */ 
        },                      /* portIntrMaskInfo */
/* Begins MAC info. */
        {
            {
                ETH_BIT_SET,    /* txEnable:1 */ 
                0x00000000,     /* synchTxEnable:1 */ 
                ETH_BIT_SET,    /* rxEnable:1 */ 
                0x00000000,     /* synchRxEnable:1 */ 
                ETH_BIT_SET,    /* txFlowCtrl:1 */ 
                ETH_BIT_SET,    /* rxFlowCtrl:1 */ 
                0x00000000,     /* reserved1:2 */ 
                ETH_BIT_CLEARED,/* loopBack:1 */ 
                0x00000000,     /* reserved2:7 */ 
                ETH_BIT_CLEARED,/* resetTxFnct:1 */ 
                ETH_BIT_CLEARED,/* resetRxFnct:1 */ 
                ETH_BIT_CLEARED,/* resetTxMac:1 */ 
                ETH_BIT_CLEARED,/* resetRxMac:1 */ 
                0x00000000,     /* reserved3:10 */ 
                ETH_BIT_CLEARED,/* simReset:1 */ 
                ETH_BIT_CLEARED /* softReset:1 */ 
            },                  /* cfg1Info */
            {
                0x00000000,     /* fullDuplex:1 */ 
                ETH_BIT_SET,    /* crcEnable:1 */ 
                ETH_BIT_CLEARED,/* padCrc:1 */ 
                0x00000000,     /* reserved1:1 */ 
                ETH_BIT_CLEARED,/* lenCheck:1 */ 
                ETH_BIT_SET,    /* hugeFrame:1 */ 
                0x00000000,     /* reserved2:2 */ 
                0x00000000,     /* ifMode:2 */ 
                0x00000000,     /* reserved3:2 */ 
                0x07,           /* preambleLen:4 */ 
                0x00000000      /* reserved4:16 */ 
            },                  /* cfg2Info */
            {
                0x0600,         /* maxFrmLen:16 */ 
                0x00000000      /* reserved1:16 */ 
            },                  /* maxFrmInfo */
            {
                0x00000000,     /* mngmClkSel:3 */ 
                0x00000000,     /* reserved1:1 */ 
                ETH_BIT_SET,    /* noPreamble:1 */ 
                ETH_BIT_CLEARED,/* scanIncr:1 */ 
                0x00000000,     /* reserved2:25 */ 
                ETH_BIT_CLEARED /* resetMngm:1 */ 
            },                  /* miiCfgInfo */
            {
                0x00000000,     /* jabberProtectEnable:1 */ 
                0x00000000,     /* reserved1:6 */
                0x00000000,     /* resetGpsi:1 */
                0x00000000,     /* linkFailDisable:1 */
                0x00000000,     /* noCipher:1 */
                0x00000000,     /* forceQuiet:1 */
                0x00000000,     /* reserved2:4 */
                0x00000000,     /* resetPmd:1 */
                0x00000000,     /* speed:1 */
                0x00000000,     /* reserved3:6 */
                0x00000000,     /* resetRmii:1 */
                0x00000000,     /* phyMode:1 */
                0x00000000,     /* lhdMode:1 */
                0x00000000,     /* ghdMode:1 */
                0x00000000,     /* ifMode:1 */
                0x00000000,     /* reserved4:3 */
                ETH_BIT_CLEARED /* resetIf:1 */
            },                  /* ifCtrlInfo */
/* MAC physical address By Jimmy*/
            {
                0x03,           /* octet4:8 */ 
                0x02,           /* octet3:8 */ 
                0x01,           /* octet2:8 */ 
                0x00            /* octet1:8 */ 
            },                  /* addrPart1Info */
            {
                0x0000,         /* reserved1:16 */ 
                0x05,           /* octet6:8 */ 
                0x04            /* octet5:8 */ 
            },                  /* addrPart2Info */
/* Begins Link info. */
            {
                {
                    0x00000000,         /* reserved1:6 */ 
                    0x00000000,         /* speed1:1 */ 
                    0x00000000,         /* reserved2:1 */ 
                    0x00000000,         /* fullDuplex:1 */ 
                    ETH_BIT_CLEARED,    /* resetAn:1 */ 
                    0x00000000,         /* reserved3:2 */ 
                    ETH_BIT_CLEARED,    /* anEnable:1 */ 
                    0x00000000,         /* speed0:1 */ 
                    ETH_BIT_CLEARED,    /* loopBack:1 */ 
                    ETH_BIT_CLEARED     /* phyReset:1 */ 
                },                      /* tbiCtrl1Info */
                {
                    0x00000000,         /* reserved1:5 */ 
                    ETH_BIT_SET,        /* fullDuplex:1 */ 
                    ETH_BIT_SET,        /* halfDuplex:1 */ 
                    0x00000000,         /* pause:2 */ 
                    0x00000000,         /* reserved2:3 */ 
                    0x00000000,         /* remoteFault:2 */ 
                    0x00000000,         /* reserved3:1 */ 
                    ETH_BIT_SET         /* nextPage:1 */ 
                },                      /* tbiAnAdvInfo */
                {
                    ETH_BIT_CLEARED,    /* commaDetect:1 */ 
                    ETH_BIT_CLEARED,    /* wrapEnable:1 */ 
                    0x00000000,         /* reserved1:2 */ 
                    0x00000000,         /* ifMode:1 */ 
                    ETH_BIT_SET,        /* rxClkSelect:1 */ 
                    0x00000000,         /* reserved2:2 */ 
                    ETH_BIT_CLEARED,    /* anSense:1 */ 
                    0x00000000,         /* reserved3:3 */ 
                    ETH_BIT_CLEARED,    /* txDisparDisable:1 */ 
                    ETH_BIT_CLEARED,    /* rxDisparDisable:1 */ 
                    ETH_BIT_CLEARED,    /* shortCutLinkTmr:1 */ 
                    ETH_BIT_CLEARED     /* softReset:1 */ 
                },                      /* tbiCtrl2Info */
/* Start of BCM5421-specific code */ 
                {
                    0x00000000,         /* reserved1:6 */ 
                    ETH_BIT_CLEARED,    /* speed1:1 */ 
                    ETH_BIT_CLEARED,    /* cte:1 */ 
                    ETH_BIT_SET,        /* fullDuplex:1 */ 
                    0x00000000,         /* resetAn:1 */ 
                    ETH_BIT_CLEARED,    /* isolate:1 */ 
                    ETH_BIT_CLEARED,    /* powerDown:1 */ 
                    ETH_BIT_SET,        /* anEnable:1 */ 
                    ETH_BIT_CLEARED,    /* speed0:1 */ 
                    ETH_BIT_CLEARED,    /* loopBack:1 */ 
                    ETH_BIT_CLEARED     /* phyReset:1 */ 
                },                      /* phyCtrl1Info */
                {
                    0x01,               /* selectorField:5 */ 
                    ETH_BIT_SET,        /* t10Half:1 */ 
                    ETH_BIT_SET,        /* t10Full:1 */ 
                    ETH_BIT_SET,        /* tx100Half:1 */ 
                    ETH_BIT_SET,        /* tx100Full:1 */ 
                    ETH_BIT_CLEARED,    /* t4_100:1 */ 
                    ETH_BIT_CLEARED,    /* pause:1 */ 
                    ETH_BIT_CLEARED,    /* asymPause:1 */ 
                    0x00000000,         /* reserved1:1 */ 
                    ETH_BIT_CLEARED,    /* remoteFault:1 */ 
                    0x00000000,         /* reserved2:1 */ 
                    ETH_BIT_CLEARED     /* nextPage:1 */ 
                },                      /* phyAnAdvInfo */
                {
                    0x00000000,         /* reserved1:8 */ 
                    ETH_BIT_SET,        /* t1000Half:1 */ 
                    ETH_BIT_SET,        /* t1000Full:1 */   
                    ETH_BIT_CLEARED,    /* hubDte:1 */ 
                    ETH_BIT_CLEARED,    /* masterSlaveVal:1 */ 
                    ETH_BIT_CLEARED,    /* masterSlaveEnable:1 */ 
                    0x00000000          /* testMode:3 */ 
                },                      /* phy1000BaseTInfo */
                {
                    ETH_BIT_CLEARED,    /* gmiiFifoElasticity:1 */ 
                    0x00000000,         /* reserved1:1 */ 
                    0x00000000,         /* reserved2:1 */ 
                    ETH_BIT_CLEARED,    /* ledOff:1 */ 
                    ETH_BIT_CLEARED,    /* ledOn:1 */ 
                    ETH_BIT_SET,        /* ledTrafficModeEnable:1 */ 
                    ETH_BIT_CLEARED,    /* resetScrambler:1 */ 
                    ETH_BIT_CLEARED,    /* bypassRsa:1 */ 
                    ETH_BIT_CLEARED,    /* bypassMlt3:1 */ 
                    ETH_BIT_CLEARED,    /* bypassScrambler:1 */ 
                    ETH_BIT_CLEARED,    /* bypass4b5b:1 */ 
                    ETH_BIT_CLEARED,    /* forceInterrupt:1 */ 
                    ETH_BIT_SET,        /* interruptDisable:1 */ 
                    ETH_BIT_CLEARED,    /* txDisable:1 */ 
                    ETH_BIT_CLEARED,    /* autoMdiCrossOverDisable:1 */ 
                    ETH_BIT_CLEARED     /* ifMode:1 */ 
                }                       /* phyCtrl2Info */
/* End of BCM5421-specific code */ 
            }
/* Ends Link info. */
        },
/* Ends MAC info. */
/* Begins MSTAT info. */
        {
            {
                ETH_BIT_CLEARED,        /* rdrp:1 */ 
                ETH_BIT_CLEARED,        /* rjbr:1 */ 
                ETH_BIT_CLEARED,        /* rfrg:1 */ 
                ETH_BIT_CLEARED,        /* rovr:1 */ 
                ETH_BIT_CLEARED,        /* rund:1 */ 
                ETH_BIT_CLEARED,        /* rcse:1 */ 
                ETH_BIT_CLEARED,        /* rcde:1 */ 
                ETH_BIT_CLEARED,        /* rflr:1 */ 
                ETH_BIT_CLEARED,        /* raln:1 */ 
                ETH_BIT_CLEARED,        /* rxuo:1 */ 
                ETH_BIT_CLEARED,        /* rxpf:1 */ 
                ETH_BIT_CLEARED,        /* rxcf:1 */ 
                ETH_BIT_CLEARED,        /* rbca:1 */ 
                ETH_BIT_CLEARED,        /* rmca:1 */ 
                ETH_BIT_CLEARED,        /* rfcs:1 */ 
                ETH_BIT_CLEARED,        /* rpkt:1 */ 
                ETH_BIT_CLEARED,        /* rbyt:1 */ 
                ETH_BIT_CLEARED,        /* reserved:8 */ 
                ETH_BIT_CLEARED,        /* trmgv:1 */ 
                ETH_BIT_CLEARED,        /* trmax:1 */ 
                ETH_BIT_CLEARED,        /* tr1k:1 */ 
                ETH_BIT_CLEARED,        /* tr511:1 */ 
                ETH_BIT_CLEARED,        /* tr255:1 */ 
                ETH_BIT_CLEARED,        /* tr127:1 */ 
                ETH_BIT_CLEARED         /* tr64:1 */ 
            },                          /* carm1Info */
            {
                ETH_BIT_CLEARED,        /* tdrp:1 */ 
                ETH_BIT_CLEARED,        /* tpfh:1 */ 
                ETH_BIT_CLEARED,        /* tncl:1 */ 
                ETH_BIT_CLEARED,        /* txcl:1 */ 
                ETH_BIT_CLEARED,        /* tlcl:1 */ 
                ETH_BIT_CLEARED,        /* tmcl:1 */ 
                ETH_BIT_CLEARED,        /* tscl:1 */ 
                ETH_BIT_CLEARED,        /* tedf:1 */ 
                ETH_BIT_CLEARED,        /* tdfr:1 */ 
                ETH_BIT_CLEARED,        /* txpf:1 */ 
                ETH_BIT_CLEARED,        /* tbca:1 */ 
                ETH_BIT_CLEARED,        /* tmca:1 */ 
                ETH_BIT_CLEARED,        /* tpkt:1 */ 
                ETH_BIT_CLEARED,        /* tbyt:1 */ 
                ETH_BIT_CLEARED,        /* tfrg:1 */ 
                ETH_BIT_CLEARED,        /* tund:1 */   
                ETH_BIT_CLEARED,        /* tovr:1 */ 
                ETH_BIT_CLEARED,        /* txcf:1 */ 
                ETH_BIT_CLEARED,        /* tfcs:1 */ 
                ETH_BIT_CLEARED,        /* tjbr:1 */ 
                ETH_BIT_CLEARED         /* reserved:12 */ 
            }                           /* carm2Info */
        },
/* Ends MSTAT info. */
/* Begins TX info. */
        {
            {
                0,                      /* startQNum:2 */ 
                0x00000000,             /* reserved1:20 */ 
                ETH_BIT_CLEARED,        /* enableHigherPriority:1 */ 
                ETH_BIT_CLEARED,        /* changePriority:1 */ 
                0x00000000,             /* reserved2:7 */
                ETH_BIT_CLEARED         /* reset:1 */ 
            },                          /* txCfgInfo */
            {
                1,                      /* qEnables:4 */ 
                0x00000000,             /* reserved1:11 */
                ETH_BIT_SET,            /* go:1 */ 
                0x00000000,             /* reserved2:8 */
                ETH_BIT_CLEARED,        /* manualPause:1 */ 
                0x00000000,             /* reserved3:4 */
                ETH_BIT_SET,            /* enableAbortIntr:1 */
                ETH_BIT_CLEARED,        /* abort:1 */
                ETH_BIT_CLEARED         /* enableIdleIntr:1 */
            },                          /* txCtrlInfo */
            {
                0x60,                   /* 0x20 startSending:8 */ 
                0x00000000,             /* reserved1:8 */
                0xE0,                   /* 0xDF stopFilling:8 */ 
                0x00000000              /* reserved2:8 */
            },                          /* txThreshInfo */
            {
                0x00,                   /* underrunPriThresh:8 */ 
                0x00000000              /* reserved1:24 */
            },                          /* txPriThreshInfo */
            {
                0x3,                    /* nextQAfter0:2 */ 
                0x00000000,             /* reserved1:1 */
                ETH_BIT_SET,            /* revertAfter0:1 */ 
                0x1,                    /* frameLimitQ0:4 */ 
                0x0,                    /* nextQAfter1:2 */ 
                0x00000000,             /* reserved2:1 */
                ETH_BIT_SET,            /* revertAfter1:1 */ 
                0x2,                    /* frameLimitQ1:4 */ 
                0x1,                    /* nextQAfter2:2 */ 
                0x00000000,             /* reserved3:1 */
                ETH_BIT_SET,            /* revertAfter2:1 */ 
                0x4,                    /* frameLimitQ2:4 */ 
                0x2,                    /* nextQAfter3:2 */ 
                0x00000000,             /* reserved4:1 */
                ETH_BIT_SET,            /* revertAfter3:1 */ 
                0x8                     /* frameLimitQ3:4 */ 
            },                          /* txPriMapInfo */
            {
                0x00000000,             /* pauseCount:16 */ 
                0x00000000              /* reserved1:16 */
            },                          /* txPauseCntInfo */
            {
                0x8100abcd              /* gVlanTag:32 */ 
            },                          /* txGVlanInfo */
/* Begins txQInfo info. */
            {
/* Begins txQInfo[0] info. */
                {
/* Begins descrInfo info. */
                    {
                        ETH_RING_DESCR,         /* descrMode */
                        ETH_DESCR_TERM_OWN,     /* descrTermMode */
                        ETH_NUM_DESCR_PER_Q,    /* numDescr */
                        ETH_MAX_BUFF_SIZE,      /* buffSize */
                        FALSE                   /* descrIntr */
                    },
/* Ends descrInfo info. */
                    {
                        ETH_MEM_ALLOC_ID,       /* ocnPort:6 */ 
                        0x00000000,             /* reserved1:4 */ 
                        ETH_BIT_CLEARED,        /* byteSwap:1 */ 
                        ETH_BIT_CLEARED,        /* wordSwap:1 */ 
                        0x00000000,             /* reserved2:2 */ 
                        ETH_BIT_CLEARED,        /* autoMode:1 */ 
                        ETH_BIT_CLEARED,        /* gVlanInsert:1 */ 
                        ETH_BIT_SET,            /* enableEofIntr:1 */ 
                        ETH_BIT_CLEARED,        /* enableLastIntr:1 */ 
                        ETH_BIT_CLEARED,        /* enableNullIntr:1 */ 
                        ETH_BIT_SET,            /* enableOwnerIntr:1 */ 
                        ETH_BIT_CLEARED,        /* enableDescrIntr:1 */ 
                        0x00000000              /* reserved3:11 */ 
                    },                          /* txQCfgInfo */
                    {
                        ETH_MEM_ALLOC_ID,       /* ocnPort:6 */ 
                        0x00000000,             /* reserved1:2 */ 
                        ETH_OCN_BURST_SIZE,     /* burst:2 */ 
                        ETH_BIT_CLEARED,        /* byteSwap:1 */ 
                        ETH_BIT_CLEARED,        /* wordSwap:1 */ 
                        0x00000000              /* reserved2:20 */ 
                    }                           /* txQBuffCfgInfo */
                },
/* Ends txQInfo[0] info. */
            }
/* Ends txQInfo info. */
        },
/* Ends TX info. */
/* Begins RX info. */
        {
/* Rx configuration register by Jimmy */
            {
                0,                      /* defaultQNum:2 */ 
                0x00000000,             /* reserved1:6 */ 
                ETH_BIT_CLEARED,        /* enableMcastHashFilter:1 */ 
                ETH_BIT_CLEARED,        /* enableUcastHashFilter:1 */ 
                ETH_BIT_SET,            /* bcastFrmEnable:1 */ 
                ETH_BIT_SET,            /* mcastFrmEnable:1 */ 
                ETH_BIT_SET,            /* ucastFrmEnable:1 */ 
                ETH_BIT_SET,            /* stationEnable:1 */ 
                0x00000000,             /* reserved2:7 */ 
                ETH_BIT_CLEARED,        /* alowBadFrm:1 */ 
                ETH_BIT_SET,            /* autoPauseEnable:1 */ 
                ETH_BIT_CLEARED,        /* changePriority:1 */ 
                0x00000000,             /* reserved3:7 */
                ETH_BIT_CLEARED         /* reset:1 */ 
            },                          /* rxCfgInfo */
            {
                1,                      /* qEnables:4 */ 
                0x00000000,             /* reserved1:11 */
                ETH_BIT_SET,            /* go:1 */ 
                0x00000000,             /* reserved2:13 */
                ETH_BIT_SET,            /* enableAbortIntr:1 */
                ETH_BIT_CLEARED,        /* abort:1 */
                ETH_BIT_CLEARED         /* enableIdleIntr:1 */
            },                          /* rxCtrlInfo */
            {
                0x180,                  /* unpauseThresh:9 */ 
                0x00000000,             /* reserved1:7 */
                0x1C0,                  /* pauseThresh:9 */ 
                0x00000000              /* reserved2:7 */
            },                          /* rxPauseThreshInfo */
            {
                0x1E0,                  /* overrunWarningThresh:9 */ 
                0x00000000              /* reserved1:23 */
            },                          /* rxThreshInfo */
            {
                0x0,                    /* qNumForTag0:2 */ 
                0x00000000,             /* reserved1:2 */ 
                0x0,                    /* qNumForTag1:2 */ 
                0x00000000,             /* reserved2:2 */ 
                0x1,                    /* qNumForTag2:2 */ 
                0x00000000,             /* reserved3:2 */ 
                0x1,                    /* qNumForTag3:2 */ 
                0x00000000,             /* reserved4:2 */ 
                0x2,                    /* qNumForTag4:2 */ 
                0x00000000,             /* reserved5:2 */ 
                0x2,                    /* qNumForTag5:2 */ 
                0x00000000,             /* reserved6:2 */ 
                0x3,                    /* qNumForTag6:2 */ 
                0x00000000,             /* reserved7:2 */ 
                0x3,                    /* qNumForTag7:2 */ 
                0x00000000              /* reserved8:2 */ 
            },                          /* rxVlanMapInfo */
/* Begins rxQInfo info. */
            {
/* Begins rxQInfo[0] info. */
                {
/* Begins descrInfo info. */
                    {
                        ETH_RING_DESCR,         /* descrMode */
                        ETH_DESCR_TERM_OWN,     /* descrTermMode */
                        ETH_NUM_DESCR_PER_Q,    /* numDescr */
                        ETH_MAX_BUFF_SIZE,      /* buffSize */
                        FALSE                   /* descrIntr */
                    },
/* Ends descrInfo info. */
                    {
                        ETH_MEM_ALLOC_ID,       /* ocnPort:6 */ 
                        0x00000000,             /* reserved1:4 */ 
                        ETH_BIT_CLEARED,        /* byteSwap:1 */ 
                        ETH_BIT_CLEARED,        /* wordSwap:1 */ 
                        0x00000000,             /* reserved2:2 */ 
                        ETH_BIT_CLEARED,        /* autoMode:1 */ 
                        0x00000000,             /* reserved3:1 */ 
                        ETH_BIT_SET,            /* enableEofIntr:1 */ 
                        ETH_BIT_CLEARED,        /* enableLastIntr:1 */ 
                        ETH_BIT_CLEARED,        /* enableNullIntr:1 */ 
                        ETH_BIT_SET,            /* enableOwnerIntr:1 */ 
                        ETH_BIT_CLEARED,        /* enableDescrIntr:1 */ 
                        0x00000000              /* reserved4:11 */ 
                    },                          /* rxQCfgInfo */
                    {
                        ETH_MEM_ALLOC_ID,       /* ocnPort:6 */ 
                        0x00000000,             /* reserved1:2 */ 
                        ETH_OCN_BURST_SIZE,     /* burst:2 */ 
                        ETH_BIT_CLEARED,        /* byteSwap:1 */ 
                        ETH_BIT_CLEARED,        /* wordSwap:1 */ 
                        0x00000000              /* reserved2:20 */ 
                    }                           /* rxQBuffCfgInfo */
                },
/* Ends rxQInfo[0] info. */
            }
/* Ends rxQInfo info. */
        }
/* Ends RX info. */
    }
/* Ends Config0 info. */
/* Begins Configx info. */
/* Ends Configx info. */
}; 

UINT32 SpacBaseAddr = XS_CSR_BASE_ADDR;
SpacEthRtnCode_t (*fl_vector_table[MAX_USER_VECTORS])(void *); 
char gigeIrqStack[4096]__attribute__ ((section("STACK")))= {0};
  ISR_SERVICE gige_next_irq_service = 0;

/* SPac GigE driver global object. The C run-time system should zero-out it. 
   Please verify anyway!  */
Eth_t EthDriver; 
/* CPU id. */
UINT32 GigeCpuId = 1;
/* IACK and EOI register. Discriminated at init time. */
/*static UINT32 GigeFiqIack = XS_CSR_BASE_ADDR + ETH_MPIC_IACK1_REG;*/
/*static UINT32 GigeFiqEoi = XS_CSR_BASE_ADDR + ETH_MPIC_EOI1_REG;*/
static UINT32 GigeIrqIack = XS_CSR_BASE_ADDR + ETH_MPIC_IACK3_REG;
static UINT32 GigeIrqEoi = XS_CSR_BASE_ADDR + ETH_MPIC_EOI3_REG;

ttUserInterface tl80314InterfaceHandle; /* only one, currently */

/*=========================================================================
 *  Function    : EthRegAccess
 *
 *  Arguments   : portId - port ID (0, 1)
 *              : regName - register offset from the Port's base address.
 *              : regValue - value to be read/written from/to.
 *              : operation - operation to be completed.
 * 
 *  Description : Accesses the chip GigE block's registers.
 * 
 *  Return      : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthRegAccess(
  UINT32 portId, 
  UINT32 regName, 
  UINT32 * regValue,
  SpacEthRegOps_t operation
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
    
  if(SpacBaseAddr == 0x00000000){
    return ETH_BASE_ADDR_ERR;
  } 
    
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if((regName < ETH_BASE_ADDR_LO) || (regName > ETH_BASE_ADDR_HI)){
    return ETH_PARAM_ERR;
  } 
  
  {
    VUINT32 * regPtr = REG32_PTR(SpacBaseAddr + regName + (portId * ETH_PORT_OFFSET));
      
    if(operation == REG_WRITE){
      *regPtr = *regValue;
    }else if(operation == REG_READ){
      *regValue = *regPtr;
    }else if (operation == REG_AND){
      *regPtr &= *regValue;
    }else if (operation == REG_OR){
      *regPtr |= *regValue;
    }else if (operation == REG_XOR){
      *regPtr ^= *regValue;
    }else{
      return ETH_PARAM_ERR;
    }  
  }
  
  return rtnCode;
}

/*******************************************************************************
* tf80314Nanosleep
* DESCRIPTION:
* Call this routine in order to relinquish the CPU for the parameter-specified
* number of nanoseconds.
*
*
* RETURNS:
*    N/A
*
* Note:
*     This guaranties to sleep for at least but not exact the parameter's value
*     time. To avoid unsigned overflowing, we split the integer division, thus
*     forcing the parameter value to at least 100. 
******************************************************************************/
void tf80314Nanosleep(unsigned nanoseconds)
{
  register int counter = 0;
  unsigned threshold = (ETH_XSCALE_CLK / 10000000) * (nanoseconds / 100);
  
  do{
    counter++;
  }while(counter < threshold);

}


/*=========================================================================
 *  Function    : EthSynchCache
 *
 *  Arguments   : address - input, address of DMA buffer.
 *              : direction - input, TX/RX direction flag.
 *              : len - input, length of DMA buffer. 
 * 
 *  Description : Synchronize precessor's cache with the DMA buffers.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : 
 =========================================================================*/
int EthSynchCache(void * address, unsigned len, 
  unsigned direction)
{

    SpacEthRtnCode_t retCode = ETH_OK;
    unsigned lineAddr, endAddr;
  
#if 0
    if(address == 0){
        return ETH_PARAM_ERR;
    } 

    if(len == 0x00000000){
        return ETH_PARAM_ERR;
    } 
#endif 

    lineAddr = 
        (unsigned)address - (((unsigned)address) % TM_XSCALE_CACHE_LINE_LEN);
    endAddr = (unsigned)address + len;
    
    if(direction == ETH_TX)
    {
/* Clean/flush cache. */    
        do{
            _Clean_D_Cache_Line(lineAddr);
            lineAddr += TM_XSCALE_CACHE_LINE_LEN;
        }while(lineAddr < endAddr);
        _Drain_Write_Buffer;
        
        _WaitForNoBCU();
    }
    else if(direction == ETH_RX)
    {
/* Invalidate cache.    
    _Invalidate_D_Cache;
*/ 
        do{
            _Invalidate_D_Cache_Line(lineAddr);
            lineAddr += TM_XSCALE_CACHE_LINE_LEN;
        }while(lineAddr < endAddr);
        _WaitForNoBCU();
    }
    else
    {
        return ETH_PARAM_ERR;
    }
    
    return retCode;
  
}


/*=========================================================================
 *  Function    : GigEDoLinkMngm
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : phyAddress - PHY address (0x1E for PE-TBI).
 *              : regAddress - Register index.
 *              : miiCycle - Cycle (scan/read/wite) type.
 *              : miiValPtr - Input/Output.
 * 
 *  Description : Runs a management (scan/read/write) cycle on the MII/TBI 
 *              : interface.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_TIMEOUT_ERR - Time-out error.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t GigEDoLinkMngm(
  UINT32 portId,
  UINT32 phyAddress,
  UINT32 regAddress,
  EthMacMiiCycle_t miiCycle,
  UINT32 * miiValPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  EthMacMiiCmd_t miiCmd; 
  EthMacMiiAddr_t miiAdr; 
  EthMacMiiIndic_t miiIndic;
/*  int counter = 0;*/

  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if((phyAddress <= 0) || 
     (phyAddress > ETH_TBI_PHY_ADDR)){
    return ETH_PARAM_ERR;
  } 
  
  if((regAddress < ETH_TBI_CTRL1_REG_IDX) || 
     (regAddress > ETH_PHY_SPR_CTRL2_REG_IDX)){
    return ETH_PARAM_ERR;
  } 
     
  if(miiValPtr == 0){
    return ETH_PARAM_ERR;
  }      
  
/* Begins the critical section. In a multitasking OS shal be protected by a per-port mutex. */  

/* Wait for a non-busy interface condition. */  
  do{
    rtnCode = EthRegAccess(portId, ETH_MAC_MII_INDIC_REG, 
      (UINT32 *)&miiIndic, REG_READ);
    if(rtnCode != ETH_OK){
      return rtnCode;
    }
/*    
    counter++;
    if(counter > ETH_MII_MNGM_WAIT_COUNT){
      return ETH_TIMEOUT_ERR;
    }
*/    
  }while(miiIndic.busy == ETH_BIT_SET);
/*
  counter = 0;
*/  
/* Start a cycle. */  
  *((UINT32 *)&miiCmd) = 0x00000000;
  *((UINT32 *)&miiAdr) = 0x00000000;
  miiAdr.phyAddress = phyAddress;    
  miiAdr.regAddress = regAddress;
  
  rtnCode = EthRegAccess(portId, ETH_MAC_MII_ADDR_REG, 
    (UINT32 *)&miiAdr, REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
/* It seems that we have to clear the command register in between MII read operations. */    
  rtnCode = EthRegAccess(portId, ETH_MAC_MII_CMD_REG, 
    (UINT32 *)&miiCmd, REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  if(miiCycle == ETH_MAC_MII_WRITE){
    rtnCode = EthRegAccess(portId, ETH_MAC_MII_CTRL_REG, 
      miiValPtr, REG_WRITE);
    if(rtnCode != ETH_OK){
      return rtnCode;
    }
  }else{ 
    if(miiCycle == ETH_MAC_MII_SCAN){
      miiCmd.scanCycle = ETH_BIT_SET;
    }else{
      miiCmd.readCycle = ETH_BIT_SET;
    }
    rtnCode = EthRegAccess(portId, ETH_MAC_MII_CMD_REG, 
      (UINT32 *)&miiCmd, REG_WRITE);
    if(rtnCode != ETH_OK){
      return rtnCode;
    } 
  
    if(miiCycle == ETH_MAC_MII_SCAN){
      tf80314Nanosleep(ETH_MII_SCAN_STATUS_WAIT);
    }else{
      do{
        rtnCode = EthRegAccess(portId, ETH_MAC_MII_INDIC_REG, 
          (UINT32 *)&miiIndic, REG_READ);
        if(rtnCode != ETH_OK){
          return rtnCode;
        }
/*        
        counter++;
        if(counter > ETH_MII_MNGM_WAIT_COUNT){
          return ETH_TIMEOUT_ERR;
        }
*/        
      }while(miiIndic.notValid == ETH_BIT_SET);
    }
    
    rtnCode = EthRegAccess(portId, ETH_MAC_MII_STATUS_REG, 
      miiValPtr, REG_READ);
    if(rtnCode != ETH_OK){
      return rtnCode;
    }
    
    if(miiCycle == ETH_MAC_MII_SCAN){
      *((UINT32 *)&miiCmd) = 0x00000000;
      
    /* We have to clear the command register after MII scan operations. */    
      rtnCode = EthRegAccess(portId, ETH_MAC_MII_CMD_REG, 
        (UINT32 *)&miiCmd, REG_WRITE);
      if(rtnCode != ETH_OK){
        return rtnCode;
      } 
    }    
  }
  
/* Ends the critical section. */  
  
  return rtnCode;
    
}



/*=========================================================================
 *  Function    : GigEConfigLink
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : linkInfoPtr - Link configuration info pointer.
 * 
 *  Description : Configures the link partners through the MII/TBI register  
 *              : interfaces.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_TIMEOUT_ERR - Time-out error.
 *
 *  Note        : This routine is BCM5421-specific. It might need to be  
 *              : modified if using a different PHY device.
 *              : The current implementation assumes the AN is performed
 *              : by the PHY. The final configuration for both link partners
 *              : is controlled by the PHY's config info.
 =========================================================================*/
SpacEthRtnCode_t GigEConfigLink(
  UINT32 portId,
  EthLinkInfoPtr_t linkInfoPtr,
  EthPhySumStatusPtr_t phySumStatusPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;  
  
  
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if(linkInfoPtr == 0){
    return ETH_PARAM_ERR;
  }      
  
  if(phySumStatusPtr == 0){
    return ETH_PARAM_ERR;
  }      
  

  {
    EthPhyStatus_t phyStatus; 
    EthPhyCtrl2_t phyCtrl2Temp;
    UINT32 phySpareCtrl2 = ETH_PHY_SPR_CTRL2_REG_VAL;
    UINT32  phyAddress = 
      (portId == ETH_PORT0_ID ? ETH_PHY0_ADDR : ETH_PHY1_ADDR);
      
/* If phy addres is 0, take the phy out of "isolate" mode. */ 
    if(phyAddress == 0){
      UINT32 phyCtrl1Val = 0x00000000;
      rtnCode = GigEDoLinkMngm(ETH_PORT0_ID, phyAddress, ETH_PHY_CTRL1_REG_IDX, 
        ETH_MAC_MII_WRITE, &phyCtrl1Val);
      if(rtnCode != ETH_OK){
        return rtnCode;
      }
    }
      
/* Get the defaults for ETH_PHY_CTRL2_REG. */ 
    rtnCode = GigEDoLinkMngm(ETH_PORT0_ID, phyAddress, ETH_PHY_CTRL2_REG_IDX, 
      ETH_MAC_MII_READ, (UINT32 *)&phyCtrl2Temp);
    if(rtnCode != ETH_OK){
      return rtnCode;
    }   
    
    if(phyCtrl2Temp.ifMode != linkInfoPtr->phyCtrl2Info.ifMode){
      rtnCode = GigEDoLinkMngm(
          ETH_PORT0_ID, phyAddress, ETH_PHY_SPR_CTRL2_REG_IDX, 
          ETH_MAC_MII_WRITE, &phySpareCtrl2);
      if(rtnCode != ETH_OK){
        return rtnCode;
      }   
    }
    linkInfoPtr->tbiCtrl2Info.ifMode = ~linkInfoPtr->phyCtrl2Info.ifMode;
    linkInfoPtr->tbiCtrl1Info.speed1 = linkInfoPtr->phyCtrl1Info.speed1;
    linkInfoPtr->tbiCtrl1Info.speed0 = linkInfoPtr->phyCtrl1Info.speed0;
    linkInfoPtr->tbiCtrl1Info.fullDuplex = linkInfoPtr->phyCtrl1Info.fullDuplex;


/* TBI Configuration. */  
    rtnCode = GigEDoLinkMngm(portId, ETH_TBI_PHY_ADDR, ETH_TBI_CTRL2_REG_IDX, 
      ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->tbiCtrl2Info));
    if(rtnCode != ETH_OK){
      return rtnCode;
    }   
    rtnCode = GigEDoLinkMngm(portId, ETH_TBI_PHY_ADDR, ETH_TBI_CTRL1_REG_IDX, 
      ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->tbiCtrl1Info));
    if(rtnCode != ETH_OK){
      return rtnCode;
    }
    
/* PHY Configuration. Always use Port0 MII Management register set. */  
    rtnCode = GigEDoLinkMngm(
        ETH_PORT0_ID, phyAddress, ETH_PHY_1000BASE_T_REG_IDX, 
        ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->phy1000BaseTInfo));
    if(rtnCode != ETH_OK){
      return rtnCode;
    }   
    rtnCode = GigEDoLinkMngm(ETH_PORT0_ID, phyAddress, ETH_PHY_CTRL2_REG_IDX, 
      ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->phyCtrl2Info));
    if(rtnCode != ETH_OK){
      return rtnCode;
    }   
    if(linkInfoPtr->phyCtrl1Info.anEnable == ETH_BIT_SET){
      rtnCode = GigEDoLinkMngm(
          portId, ETH_TBI_PHY_ADDR, ETH_TBI_AN_ADV_REG_IDX, 
          ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->tbiAnAdvInfo));
      if(rtnCode != ETH_OK){
        return rtnCode;
      }
      rtnCode = GigEDoLinkMngm(
          ETH_PORT0_ID, phyAddress, ETH_PHY_AN_ADV_REG_IDX, 
          ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->phyAnAdvInfo));
      if(rtnCode != ETH_OK){
        return rtnCode;
      }      
      linkInfoPtr->phyCtrl1Info.resetAn = ETH_BIT_SET;
    }
    rtnCode = GigEDoLinkMngm(ETH_PORT0_ID, phyAddress, ETH_PHY_CTRL1_REG_IDX, 
      ETH_MAC_MII_WRITE, (UINT32 *)&(linkInfoPtr->phyCtrl1Info));
    if(rtnCode != ETH_OK){
      return rtnCode;
    }
        
/* Wait for the link to be established.    
    tf80314Nanosleep(ETH_MII_LINK_WAIT);
*/ 

    do{
/* Read ETH_PHY_SUM_STATUS_REG returned in ETH_MAC_MII_STATUS_REG register.*/
      rtnCode = GigEDoLinkMngm(
          ETH_PORT0_ID, phyAddress, ETH_PHY_SUM_STATUS_REG_IDX, 
          ETH_MAC_MII_READ, (UINT32 *)phySumStatusPtr);
      if(rtnCode != ETH_OK){
        return rtnCode;
      }
/* Here we can insert a counter threshold conditional to avoid a system lock*/
    }while(((linkInfoPtr->phyCtrl1Info.anEnable == ETH_BIT_SET) && 
            (phySumStatusPtr->anComplete != ETH_BIT_SET) && 
            (phySumStatusPtr->linkStatus != ETH_BIT_SET)) || 
           ((linkInfoPtr->phyCtrl1Info.anEnable != ETH_BIT_SET) && 
            (phySumStatusPtr->linkStatus != ETH_BIT_SET)));

    do{
/* Read ETH_PHY_STATUS_REG returned in ETH_MAC_MII_STATUS_REG register. */   
      rtnCode = GigEDoLinkMngm(
          ETH_PORT0_ID, phyAddress, ETH_PHY_STATUS_REG_IDX, 
          ETH_MAC_MII_READ, (UINT32 *)&phyStatus);
      if(rtnCode != ETH_OK){
        return rtnCode;
      }
/* Here we can insert a counter threshold conditional to avoid a system lock*/
    }while(phyStatus.linkStatus != ETH_BIT_SET);
    
/* Check the LINK STATUS field value.      
    if((linkInfoPtr->phyCtrl1Info.anEnable == ETH_BIT_SET) && 
      (phyStatus.anComplete != ETH_BIT_SET)){
      return rtnCode;
    }
    
    if(phyStatus.linkStatus != ETH_BIT_SET){
      return rtnCode;
    }
*/
    
  }    
  
  return rtnCode;
    
}



/*=========================================================================
 *  Function    : EthConfigMac
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : macInfoPtr - Configuration info pointer.
 * 
 *  Description : Configures the chip's MAC block of a specified GigE port.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_MII_IF_STATUS_ERR - MII interface status error.
 *              : ETH_TIMEOUT_ERR - Time-out error.
 *              : ETH_INVAL_CFG_ERR - Invalid configuration error.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthConfigMac(
  UINT32 portId, 
  EthMacInfoPtr_t macInfoPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  BOOL serialMii = FALSE;
  EthPhySumStatus_t phySumStatus; 
    
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if(macInfoPtr == 0){
    return ETH_PARAM_ERR;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_MAC_CFG1_REG, 
    (UINT32 *)&(macInfoPtr->cfg1Info), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
    
  rtnCode = EthRegAccess(portId, ETH_MAC_MII_CFG_REG, 
    (UINT32 *)&(macInfoPtr->miiCfgInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_MAC_MAX_FRAME_REG, 
    (UINT32 *)&(macInfoPtr->maxFrmInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  rtnCode = EthRegAccess(portId, ETH_MAC_ADDR_PART1_REG, 
    (UINT32 *)&(macInfoPtr->addrPart1Info), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  rtnCode = EthRegAccess(portId, ETH_MAC_ADDR_PART2_REG, 
    (UINT32 *)&(macInfoPtr->addrPart2Info), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
    
/* FHY Configurtion. */  
  rtnCode = GigEConfigLink(portId, &(macInfoPtr->linkInfo), &phySumStatus);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  if((phySumStatus.reportedStatus == ETH_PHY_10_HALF) || 
    (phySumStatus.reportedStatus == ETH_PHY_10_FULL)){
    macInfoPtr->ifCtrlInfo.phyMode = ETH_BIT_SET;
    serialMii = TRUE;
  }else if((phySumStatus.reportedStatus == ETH_PHY_100TX_HALF) || 
    (phySumStatus.reportedStatus == ETH_PHY_100TX_FULL)){
    macInfoPtr->ifCtrlInfo.phyMode = ETH_BIT_CLEARED;
    serialMii = TRUE;    
  }
    
  if(serialMii == TRUE){
    rtnCode = EthRegAccess(portId, ETH_MAC_IF_CTRL_REG, 
      (UINT32 *)&(macInfoPtr->ifCtrlInfo), REG_WRITE);
    if(rtnCode != ETH_OK){
      return rtnCode;
    } 
/* Sleep. 
    tf80314Nanosleep(ETH_MII_IF_STATUS_WAIT); 
*/
    {
      EthMacIfStatus_t ifStatusInfo;
      int counter = 0;
      do{
/* Read ETH_MAC_IF_STATUS_REG register. */   
/* Check the interface status. */
        rtnCode = EthRegAccess(portId, ETH_MAC_IF_STATUS_REG, 
          (UINT32 *)&ifStatusInfo, REG_READ);
        if(rtnCode != ETH_OK){
          return rtnCode;
        }
/*        else{ 
          if(ifStatusInfo.linkFail == ETH_BIT_SET){
            return ETH_MII_IF_STATUS_ERR;
          }
        }
*/        
        counter++;if(counter == 100)break;
/* Here we can insert a counter threshold conditional to avoid a system lock*/
      }while(ifStatusInfo.linkFail == ETH_BIT_SET);
    }
  }  
  
  if((phySumStatus.reportedStatus == ETH_PHY_1000T_HALF) || 
    (phySumStatus.reportedStatus == ETH_PHY_1000T_FULL)){
    macInfoPtr->cfg2Info.ifMode = ETH_MAC_IF_BYTE;
  }else if(serialMii == TRUE){
    macInfoPtr->cfg2Info.ifMode = ETH_MAC_IF_NIBLE;
  }else{
    return ETH_INVAL_CFG_ERR;
  }        
    
  if((phySumStatus.reportedStatus == ETH_PHY_1000T_HALF) || 
    (phySumStatus.reportedStatus == ETH_PHY_100TX_HALF) || 
    (phySumStatus.reportedStatus == ETH_PHY_10_HALF)){
    macInfoPtr->cfg2Info.fullDuplex = ETH_BIT_CLEARED;
  }else if((phySumStatus.reportedStatus == ETH_PHY_1000T_FULL) || 
    (phySumStatus.reportedStatus == ETH_PHY_100TX_FULL) || 
    (phySumStatus.reportedStatus == ETH_PHY_10_FULL)){
    macInfoPtr->cfg2Info.fullDuplex = ETH_BIT_SET;
  }else{
    return ETH_INVAL_CFG_ERR;
  }
               
  rtnCode = EthRegAccess(portId, ETH_MAC_CFG2_REG, 
    (UINT32 *)&(macInfoPtr->cfg2Info), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  return rtnCode;
    
}


/*=========================================================================
 *  Function    : EthDrvInit
 *
 *  Arguments   : N/A.
 * 
 *  Description : Configures the chip's GigE block.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_MEM_ALLOC_ERR - Memory allocation error.
 * 
 *
 *  Note        : This is a good place to initialize data structure used 
 *              : by the driver.
 =========================================================================*/
SpacEthRtnCode_t EthDrvInit(
    ttUserInterface     interfaceHandle)
{
    SpacEthRtnCode_t rtnCode = ETH_OK;
  
    (void)tfPoolCreate(
        interfaceHandle,
        TM_80314_LARGE_RECV_POOL,
        TM_80314_SMALL_RECV_POOL,
        TM_80314_LARGE_RECV_BUF_SIZE,
        TM_80314_SMALL_RECV_BUF_SIZE,
        TM_XSCALE_CACHE_LINE_LEN,
        TM_POOL_REFILL_IN_LINE );

    EthDriver.drvEnabled = TRUE;

    return rtnCode;
}

/*=========================================================================
 *  Function    : EthGetRxFrames
 *
 *  Arguments   : portId - Port ID (0, 1).
 *              : qId - Queue ID (0, 3).
 * 
 *  Description : Gets received frames from a specified RX queue.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_Q_INTEGRITY_ERR - Queue integrity error. 
 *
 *  Note        : This would be a good place to gather statistics about 
 *              : received frames. 
 *              : Do not use local static variables, as this routine needs
 *              : to be reentrant.
 *              : We chose to allow frame processing despite an error at
 *              : release invalid frame and demux time.
 =========================================================================*/
SpacEthRtnCode_t EthGetRxFrames(
                             ttUserInterface    interfaceHandle,
                             UINT32 portId, 
                             UINT32 qId)
{
/* this function is called at interrupt time */
    SpacEthRtnCode_t rtnCode = ETH_OK;
    EthRxQPtr_t rxQPtr = 0;
    int         receiveCount;
    VUINT16 buffLen;  /* data length. */
    VUINT8 *dataPtr;
    VINT32 takeOutIdx; /* TX/RX take-out index. */
    EthBuffDescrPtr_t descrPoolPtr;
    UINT32 ocnPortId = 0xFFFFFFFF;    
    
    receiveCount = 0;
    
    if(portId >= ETH_NUM_PORTS)
    {
        return ETH_PARAM_ERR;
    } 
    
    if(qId >= ETH_NUM_RX_Q)
    {
        return ETH_PARAM_ERR;
    } 
    
    rxQPtr = &(EthDriver.port[portId].mac.rx.rxQ[qId]);
    
    takeOutIdx = rxQPtr->memPool.takeOutIdx;
    descrPoolPtr = rxQPtr->memPool.descrPoolPtr;
    
    /* Point to the descriptor's Config dword. */    
    EthRxFrmStatusCfgPtr_t rxFrmStatusCfgPtr = 
        (EthRxFrmStatusCfgPtr_t)&(descrPoolPtr[takeOutIdx].cfgStatus);
    EthVlanBCntPtr_t vlanBCntPtr = 0;
    UINT16 numDescr =    ETH_MAX_NUM_PCI_DESCR_PER_Q;
    
/* synchronize the cache (descriptors only) */
    EthSynchCache(&(descrPoolPtr[takeOutIdx]), ETH_DESCR_SIZE, ETH_RX);
    
/* If the "owner" bit of the pointed descriptor is not own by DMA and the 
   data buffer address is not NULL, the ring has entries to be reassembled. */
    while((rxFrmStatusCfgPtr->owner != ETH_DESCR_DMA) && 
          ((descrPoolPtr[rxQPtr->memPool.
                         takeOutIdx].startAddr0 != 0x00000000) ||
           ((descrPoolPtr[rxQPtr->memPool.
                          takeOutIdx].startAddr1
             & ETH_DESCR_ADDR_HI_MASK) != 0x000)))
    {
        
        /* Point to the descriptor's VlanBcount dword. */    
        vlanBCntPtr = (EthVlanBCntPtr_t)&(descrPoolPtr[takeOutIdx].vlanBCnt);
        
        /* SOF or SOFEOF */      
/* By Jimmy, now no scattered data, much simplified */
        if((rxFrmStatusCfgPtr->sof == ETH_BIT_SET)
           && (rxFrmStatusCfgPtr->eof == ETH_BIT_SET))
        {
            if(rxFrmStatusCfgPtr->badFrame == ETH_BIT_SET)
            {
                rxQPtr->invalidFrm = TRUE;    
            }
            else
            {
                if(rxFrmStatusCfgPtr->vtf == ETH_BIT_SET)
                {
                    rxQPtr->vLanTag = vlanBCntPtr->vlanTag;
                }
/* by jimmy */
                receiveCount ++;
                rxQPtr->invalidFrm = FALSE;
            }
        }
        else
        {
            rxQPtr->invalidFrm = TRUE;    
        }
        
        buffLen = vlanBCntPtr->byteCount;
        
/* get a pointer to the rx packet buffer from the frame desccriptor.
   NOTE: to be more robust, we should call sysOcnToVirtual here */
        dataPtr = (VUINT8 *) descrPoolPtr[takeOutIdx].startAddr0;
        
/* Cache synchronization of rx packet data buffer. */    
        EthSynchCache((void*)dataPtr, buffLen, ETH_RX);
        
/* Get a Treck packet buffer, link it into the rx frame descriptor */
        dataPtr = (VUINT8 *) tfPoolIsrGetBuffer(
            tl80314InterfaceHandle,
            TM_80314_LARGE_RECV_BUF_SIZE);
        tm_assert(EthGetRxFrames, (dataPtr != ((VUINT8 *) 0)));
/* link new packet data buffer into the rx frame descriptor */
        rtnCode = sysVirtualToOcn(
            (void*)dataPtr,
            (UINT32 *)(&descrPoolPtr[takeOutIdx].startAddr1), 
            (UINT32 *)(&descrPoolPtr[takeOutIdx].startAddr0), &ocnPortId);
        tm_assert(EthGetRxFrames, (rtnCode == 0));
        
        /* Clear frame configuration. */      
        *((UINT32 *)rxFrmStatusCfgPtr) = 0x00000000;      
        rxFrmStatusCfgPtr->byteCount = TM_80314_LARGE_RECV_BUF_SIZE;
        rxFrmStatusCfgPtr->owner = ETH_DESCR_DMA;
        
        /* Clear VLAN/Byte count configuration. */      
        descrPoolPtr[takeOutIdx].vlanBCnt = 0x00000000;
        /* This should be the last clean-up step for each descriptor. */      
        /* Cache synchronization of rx frame descriptor. */    
        EthSynchCache(&(descrPoolPtr[takeOutIdx]), ETH_DESCR_SIZE, ETH_TX);
        /* This should be atomic. */
        rxQPtr->memPool.freeEntries++;
        
        /* Update "TAKE-OUT" index. */
        takeOutIdx++;
        takeOutIdx = takeOutIdx % numDescr;
        rxQPtr->memPool.takeOutIdx = takeOutIdx;
        
        /* Cache synchronization. */    
        EthSynchCache(&(descrPoolPtr[takeOutIdx]), ETH_DESCR_SIZE, ETH_RX);
        /* Point to the descriptor's Config dword. */    
        rxFrmStatusCfgPtr =
            (EthRxFrmStatusCfgPtr_t)&(descrPoolPtr[takeOutIdx].cfgStatus);
        rxQPtr->vLanTag = 0x0000;
    }
    
    if(receiveCount)
    {
        tfNotifyInterfaceIsr(
            interfaceHandle, receiveCount, 0, 0UL, 0UL );
    }
    
    rxQPtr->invalidFrm = FALSE;
    rxQPtr->vLanTag = 0x0000;
    
    return rtnCode;
}


/*=========================================================================
 *  Function    : EthRxQueueIsr
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : info - input, used to retrieve the qeue id. 
 * 
 *  Description : GigE's RX QUEUE ISR.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *
 *  Note        : Use of "flPrintf" macro to allow for non-blocking logging.
 *              : An EOQ condition is expected to happen if a frame was 
 *              : received, and DMA has encountered a LAST/NULL EOQ 
 *              : condition on the last processed descriptor. Update the 
 *              : queue pointer, clean-up and restart the queue. 
 =========================================================================*/
SpacEthRtnCode_t EthRxQueueIsr(
  UINT32 portId, 
  UINT32 info
){
    SpacEthRtnCode_t errorCode = ETH_OK;
    SpacEthRtnCode_t rtnCode = ETH_OK;
/*  SpacEthRtnCode_t rtnTestCode = ETH_OK;*/
    UINT32 extIntrStatus, rxQCfg, qId;
  
    tm_assert(EthRxQueueIsr, (portId < ETH_NUM_PORTS));
  
    tm_assert(
        EthRxQueueIsr,
        ((info <= ETH_RX_Q_INTR_STATUS_HI)
         && (info >= ETH_RX_Q_INTR_STATUS_LO)));
    
    qId = info - ETH_RX_Q_INTR_STATUS_LO;
    
    EthQPtrHi_t qPtrHi;
    EthBuffDescrPtr_t descrPoolPtr = 
        EthDriver.port[portId].mac.rx.rxQ[qId].memPool.descrPoolPtr;
    UINT16 numDescr =    ETH_MAX_NUM_PCI_DESCR_PER_Q;
    INT32 lastBuffDescrIdx = 0;
    INT32 lastBuffDescrIdxIncr = 0;
    UINT32 descrAddrLo = 0x00000000;
    UINT32 ocnPortId = 0xFFFFFFFF;    
    SpacEthRegOps_t regOp = REG_OR;    
    *((UINT32 *)&qPtrHi) = 0x00000000;
    UINT32 loopExitCounter = 0;      
    
/* Get the extended status bits. */      
    rtnCode = EthRegAccess(
        portId, ETH_RX_XSTATUS_REG, &extIntrStatus, REG_READ);
    if(rtnCode != ETH_OK){
        errorCode = rtnCode;
        goto exitRxIsr;
    }
/* Keep only the extended status bits specific to a particular queue. */  
    extIntrStatus &= (ETH_RX_Q_X_INTR_MASK << qId);
    
/* Get the config enable bits. */      
    rtnCode = EthRegAccess(
        portId, 
        ETH_RX_Q0_CFG_REG + (qId * ETH_RX_Q_REG_OFFSET), &rxQCfg, REG_READ);
    if(rtnCode != ETH_OK){
        errorCode = rtnCode;
        goto exitRxIsr;
    } 
    
/* Descriptor interrupt. */  
    if((extIntrStatus & (ETH_RX_DESCR_Q_X_INTR_MASK << qId)) && 
       (rxQCfg & ETH_RX_DESCR_Q_INTR_EN_MASK)){
    }
/* EOF interrupt. */  
/*    if((extIntrStatus & (ETH_RX_EOF_Q_X_INTR_MASK << qId)) &&
      (rxQCfg & ETH_RX_EOF_Q_INTR_EN_MASK)){ */                
    
/* Get new received frames. */      
    rtnCode = EthGetRxFrames(tl80314InterfaceHandle, portId, qId);
    if(rtnCode != ETH_OK){
/* Give a chance to re-validate queue.       
   errorCode = rtnCode;
   goto exitRxIsr; */
/*        } */
    }
/* EOQ interrupt. */
/* ecr: is the following code needed? */
    if((extIntrStatus & (ETH_RX_EOQ_Q_X_INTR_MASK << qId)) &&
       (rxQCfg & ETH_RX_EOQ_Q_INTR_EN_MASK)){    
        
        /* Find the last descriptor (processed by DMA). */  
        lastBuffDescrIdx = 
            EthDriver.port[portId].mac.rx.rxQ[qId].memPool.takeOutIdx;
        /* Cache synchronization. */    
        EthSynchCache(
            &(descrPoolPtr[lastBuffDescrIdx]), ETH_DESCR_SIZE, ETH_RX);
        
        if(descrPoolPtr[lastBuffDescrIdx].vlanBCnt == 0x00000000){
            lastBuffDescrIdxIncr = lastBuffDescrIdx;
            lastBuffDescrIdx--;
            if(lastBuffDescrIdx < 0){
                lastBuffDescrIdx = numDescr - 1;
            }
        }else{
            loopExitCounter = 1;            
            lastBuffDescrIdxIncr = (lastBuffDescrIdx + 1) % numDescr;
            /* Cache synchronization. */    
            EthSynchCache(
                &(descrPoolPtr[lastBuffDescrIdxIncr]), ETH_DESCR_SIZE, ETH_RX);
            while((descrPoolPtr[lastBuffDescrIdxIncr].vlanBCnt != 0x00000000)
                  && (loopExitCounter < numDescr))
            {
                lastBuffDescrIdx = lastBuffDescrIdxIncr;
                lastBuffDescrIdxIncr = (lastBuffDescrIdxIncr + 1) % numDescr;
                /* Cache synchronization. */    
                EthSynchCache(
                    &(descrPoolPtr[lastBuffDescrIdxIncr]),
                    ETH_DESCR_SIZE, ETH_RX);
                loopExitCounter++;
            }
            if(loopExitCounter == numDescr)
            {
                lastBuffDescrIdxIncr = EthDriver.port[portId].mac.rx.rxQ[
                    qId].memPool.takeOutIdx;        
                lastBuffDescrIdx--;
                if(lastBuffDescrIdx < 0){
                    lastBuffDescrIdx = numDescr - 1;
                }
            }
        }        
        
/* Manually update the descriptor pointer, if NULL terminated. */         
        if((descrPoolPtr[lastBuffDescrIdx].nextDescrAddr0 == 0x00000000)
           && ((descrPoolPtr[lastBuffDescrIdx].nextDescrAddr1
                & ETH_DESCR_ADDR_HI_MASK) == 0x000))
        {
            rtnCode = sysVirtualToOcn(
                descrPoolPtr + lastBuffDescrIdxIncr,
                (UINT32 *)&qPtrHi, (UINT32 *)&descrAddrLo, &ocnPortId);
            if(rtnCode != 0){
                errorCode = rtnCode;
                goto exitRxIsr;
            } 
            rtnCode = EthRegAccess(
                portId, 
                ETH_RX_Q0_PTR_LO_REG + (qId * ETH_RX_Q_REG_OFFSET), 
                (UINT32 *)&descrAddrLo, REG_WRITE);
            if(rtnCode != ETH_OK){
                errorCode = rtnCode;
                goto exitRxIsr;
            } 
            regOp = REG_WRITE;
        }
        
/* Re-validate the queue. If NULL terminated, qPtrHi contains the HI bits. */
        qPtrHi.valid = ETH_BIT_SET;   
        rtnCode = EthRegAccess(
            portId, 
            ETH_RX_Q0_PTR_HI_REG + (qId * ETH_RX_Q_REG_OFFSET), 
            (UINT32 *)&qPtrHi, regOp);
        if(rtnCode != ETH_OK){
            errorCode = rtnCode;
            goto exitRxIsr;
        }
    }
    
 exitRxIsr:
/* Clear all pending interrupts in the gigabit ethernet controller */
    rtnCode = EthRegAccess(
        portId, ETH_RX_XSTATUS_REG, &extIntrStatus, REG_WRITE);
    if(rtnCode != ETH_OK){
        errorCode = rtnCode;
    }
    
    return errorCode;   
}

/*=========================================================================
 *  Function    : EthPortIsr
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : intrStatus - Port interrupt status.
 *              : infoPtr - input, user info passed in. 
 * 
 *  Description : GigE's ISR.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *
 *  Note        : Use of "flPrintf" macro to allow for non-blocking logging.
 *              : Prior to returning, this routine, attempts to clear the 
 *              : interrupt sources.
 =========================================================================*/
SpacEthRtnCode_t EthPortIsr(
  UINT32 portId, 
  UINT32 intrStatus,   
  void * infoPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  UINT32 intrMask;
  
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if(infoPtr == 0){
    return ETH_PARAM_ERR;
  } 
    
  rtnCode = EthRegAccess(portId, ETH_PORT_INT_MASK_REG, &intrMask, REG_READ);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  {
    register unsigned idx, statusBit, isrParam, loopCounter = 0;
/* If there are any interrupts, dispatch them. */      
    do{
/* Get the index into the EthIsrTable. */ 
      __asm volatile ("CLZ %0, %1": "=r" (idx) : "r" (intrStatus));
        
      if(idx == ETH_ARM_CLZ_BIT_NOT_SET){
        break;
      }
      statusBit = ETH_MAX_NUM_INTR_SOURCES - (idx + 1);
/* If the interrupt is enabled, call its handler. */      
      if((intrMask & (1 << statusBit)) == 
        ETH_BIT_CLEARED){
        if(EthIsrTable[idx] != 0){
          if((intrStatus & ETH_TX_Q_WAIT_INTR_MASK) ||
            (intrStatus & ETH_RX_Q_WAIT_INTR_MASK)){
            isrParam = (UINT32)(&intrStatus);
          }else{
            isrParam = statusBit;
          }
          rtnCode = EthIsrTable[idx](portId, isrParam);
          if(rtnCode != ETH_OK){
/* Give a chance to service next interrupt source.       
            return rtnCode; */  
          }
        }else{
/* Give a chance to service next interrupt source.       
          return rtnCode; */  
        }
      }
/* Clear the serviced interrupt source bit. */      
      intrStatus &= ~(1 << statusBit);
/* Paranoid redundancy mechanism, in case something is VERY wrong with the 
   CLZ instruction. */      
      loopCounter++;
    }while(loopCounter < ETH_MAX_NUM_INTR_SOURCES);
  }

  return rtnCode;
    
}

SpacEthRtnCode_t FLTimerIsr( void * infoPtr)
{
    
    TM_UNREF_IN_ARG(infoPtr);
    tfTimerUpdateIsr();
    return ETH_OK;
}

/*=========================================================================
 *  Function    : EthIsr
 *
 *  Arguments   : infoPtr - input, user info passed in.
 * 
 *  Description : GigE's ISR.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *
 *  Note        : Use of "flPrintf" macro to allow for non-blocking logging.
 *              : Port0 will get first chance to service its interrupts.
 *              
 =========================================================================*/
SpacEthRtnCode_t EthIsr(
  void * infoPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  int idx;
  EthPortIntrStatus_t gigeIntrStatus[ETH_NUM_PORTS];
  static int portIdx = ETH_PORT0_ID;
  
  
  if(infoPtr == 0){
    return ETH_PARAM_ERR;
  }
  
  for(idx = 0; idx < ETH_NUM_PORTS; idx++){  
/* Read both ports' interrupt status register to avoid spurious interrupts*/
    rtnCode = EthRegAccess(idx, ETH_PORT_INT_STATUS_REG, 
      (UINT32 *)&(gigeIntrStatus[idx]), REG_READ);
    if(rtnCode != ETH_OK){
      return rtnCode;
    }
    if(gigeIntrStatus[idx].intrStatus == ETH_BIT_SET){
/* Dilemma which to clear first : extended status registers bits, or 
 ETH_PORT_INT_STATUS_REG register's bits. */      
      rtnCode = EthRegAccess(idx, ETH_PORT_INT_STATUS_REG, 
        (UINT32 *)&(gigeIntrStatus[idx]), REG_WRITE);
      if(rtnCode != ETH_OK){
        return rtnCode;   
      } 
    }    
  } 
      
  for(idx = 0; idx < ETH_NUM_PORTS; idx++){
    if(gigeIntrStatus[portIdx].intrStatus == ETH_BIT_SET){
      rtnCode = EthPortIsr(portIdx, 
        *((UINT32 *)&(gigeIntrStatus[portIdx])) & ~ETH_PORT_INT_STATUS_MASK, 
        infoPtr);
      if(rtnCode != ETH_OK){
/* Give a chance to service next port.       
        return rtnCode; */  
      }   
    } 
    portIdx = (portIdx == ETH_PORT0_ID ? ETH_PORT1_ID : ETH_PORT0_ID);
  }

  portIdx = (portIdx == ETH_PORT0_ID ? ETH_PORT1_ID : ETH_PORT0_ID);

  return rtnCode;
    
}

/*=========================================================================
 *  Function    : gigeHandlerAttachVector
 *
 *  Arguments   : req_type - IRQ/FIQ (not used for now).
 *              : vector - index into the vector table.
 *              : int_routine - vector handler to attach. 
 * 
 *  Description : Attach service routine for interrupt vector.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *
 *              
 =========================================================================*/
SpacEthRtnCode_t gigeHandlerAttachVector(
  int req_type,  
  unsigned vector, 
  SpacEthRtnCode_t (*int_routine)(void *))
{
  
  SpacEthRtnCode_t rtnCode = ETH_OK;

  if(vector >= MAX_USER_VECTORS)
  {
    return ETH_PARAM_ERR;
  }

  *(fl_vector_table + vector) = int_routine;
  
  return rtnCode;
  
}

/*=========================================================================
 *  Function    : gigeHandlerAttachIRQ
 *
 *  Arguments   : irq_service - ISR.
 * 
 *  Description : SPac's ISR attach routine.
 * 
 *  return      : N/A.
 *
 =========================================================================*/
void gigeHandlerAttachIRQ(ISR_SERVICE irq_service)
{
  
  static unsigned int cpsr;
  
/* Setup stack for the IRQ mode. */
  cpsr = _Read_CPSR;
  _Write_MODE(0xd2);  /* Set IRQ Mode (I and F disabled) */
  INIT_STACK_PTR(gigeIrqStack + sizeof(gigeIrqStack)); /* Set SP_irq */
/* Restore CPSR */
  _Write_CPSR(cpsr);
  
  gige_next_irq_service=*(ISR_SERVICE*)IRQVECTOR;  /* Save current IRQ vector*/
  *(ISR_SERVICE*)IRQVECTOR = irq_service;          /* Set new vector */
    
}

/*=========================================================================
 *  Function    : SpacIrqIsr
 *
 *  Arguments   : N/A.
 * 
 *  Description : SPac's ISR.
 * 
 *  return      : N/A.
 *
 *  Note        : for now servicing only IRQ GigE interrupts. They are expected
 *              : at output 1 only.
 *              : See note in SpacEthInterrupts.h file about the usage of the
 *              : IRQ_OVI_EPILOG macro.
 *              
 =========================================================================*/
void SpacIrqIsr(void)
{
    
    SpacEthRtnCode_t rtnCode = ETH_OK; 
    UINT32 irqVal = 0x00000000;
    UINT32 iAckVal = 0x00000000;  
  
/* Identify the source of interrupt. */
    irqVal = _Read_INT_Source;
    if(irqVal & _INTSRC_IRQ_ACTIVE) /* Check IRQ# pin. */
    {
/* IACK cycle. */
        iAckVal = *(REG32_PTR(GigeIrqIack));  
        irqVal = iAckVal & ETH_MPIC_TIMER0_VECTOR;  
        if(!irqVal)
        {
            irqVal = iAckVal & ETH_MPIC_GIGE_VECTOR;  
        }
        if((irqVal < MAX_USER_VECTORS) && (0 != fl_vector_table[irqVal]))
        {
/* Call a service function. */
/* *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IVPR16_REG)) |= ETH_MPIC_IVPR_M_MASK;*/
            rtnCode = fl_vector_table[irqVal]((void *)&EthDriver);
/* *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IVPR16_REG)) &= ~ETH_MPIC_IVPR_M_MASK;*/
/*      if(rtnCode != ETH_OK){ } */
        }
/* EOI cycle. */
        *(REG32_PTR(GigeIrqEoi)) = ETH_MPIC_EOI_MASK;  
/*        _WaitForNoIRQ(); */          
    }
    else
    {
/* Call the redboot interupt handler */
/* GoTo "OldVectorValue" stored in intHandlerAttach() */
        EXCEPTION_CHAIN(gige_next_irq_service);
    }

    IRQ_OVI_EPILOG;
  
}



/*=========================================================================
 *  Function    : intMpicInit
 *
 *  Arguments   : N/A.
 * 
 *  Description : Initializes the MPIC block.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_INVAL_CFG_ERR - Invalid configuration error. 
 *
 *  Note        : for now GigE-oriented. 
 *           
 =========================================================================*/
SpacEthRtnCode_t intMpicInit(void)
{
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  UINT32 cpsr;
  
  GigeCpuId = 
    (*(REG32_PTR(SpacBaseAddr + ETH_CIU_CFG_REG)) & ETH_CIU_PID_MASK)
        >> ETH_BYTE_WIDTH;
  
  if((GigeCpuId != ETH_CPU0_ID) && (GigeCpuId != ETH_CPU1_ID)){
    return ETH_INVAL_CFG_ERR;
  } 
  
/* Make sure that interrupts are disabled */
  cpsr = _Read_CPSR;
  _Write_CPSR(cpsr | _IRQ_DISABLE | _FIQ_DISABLE);

  rtnCode = gigeHandlerAttachVector(0, ETH_MPIC_GIGE_VECTOR, EthIsr);
/*  if(rtnCode != ETH_OK){  } */

/* Jimmy add Timer IRQ here */  
  rtnCode = gigeHandlerAttachVector(0, ETH_MPIC_TIMER0_VECTOR, FLTimerIsr);

  gigeHandlerAttachIRQ(SpacIrqIsr);
  
#ifdef __GIGE_SDRAM_FIQ_DEBUG__     
  gigeHandlerAttachFIQ(SpacFiqIsr);
#endif

    /* Make sure that interrupt vectors will be updated correctly */

    // Force new interrupt handler settings out of the Data Cache
    // and Write Buffer.
    _Clean_D_Cache_Line(IRQVECTOR/*FIQVECTOR*/);
    _CPWAIT;
    _Invalidate_D_Cache_Line(IRQVECTOR/*FIQVECTOR*/);
    _CPWAIT;
    _Drain_Write_Buffer;
    _CPWAIT;
    // Invalidate Instruction Cache and BTB
    _Invalidate_I_BTB;
    _CPWAIT;

/* Set IRQ to be enabled later */  
  cpsr &= ~_IRQ_DISABLE;     
  
#ifdef __GIGE_SDRAM_FIQ_DEBUG__     
/* Set FIQ to be enabled later */  
  cpsr &= ~_FIQ_DISABLE;     
#endif    

/* Enable IRQs. */  
  _Write_INT_Control(ETH_XSCALE_INTCTL_MASK);
  
#ifdef __GIGE_SDRAM_FIQ_DEBUG__        
/* Enable FIQs and IRQs. */  
//  _Write_INT_Control(3);  
/* Enable FIQs. */  
  _Write_INT_Control(1);  
#endif    
  
  if(GigeCpuId == ETH_CPU0_ID){
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_CSR2_REG)) = (1 << 1);  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_TASK2_REG)) = ETH_MPIC_TASK_MASK;  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IDR16_REG)) = ETH_MPIC_IDR_OUT3_MASK;
    GigeIrqIack = SpacBaseAddr + ETH_MPIC_IACK2_REG;
    GigeIrqEoi = SpacBaseAddr + ETH_MPIC_EOI2_REG;
    
#ifdef __GIGE_SDRAM_FIQ_DEBUG__              
    
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_CSR0_REG)) = (1 << 1);  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_TASK0_REG)) = ETH_MPIC_TASK_MASK;  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IDR16_REG)) = ETH_MPIC_IDR_OUT1_MASK;
/* *(REG32_PTR(SpacBaseAddr + (ETH_MPIC_BASE_ADDR_LO + 0x19C)))
   = ETH_MPIC_IDR_OUT1_MASK; */
    GigeFiqIack = SpacBaseAddr + ETH_MPIC_IACK0_REG;
    GigeFiqEoi = SpacBaseAddr + ETH_MPIC_EOI0_REG;
    
#endif    

  }else{
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_CSR3_REG)) = (1 << 1);  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_TASK3_REG)) = ETH_MPIC_TASK_MASK;  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IDR16_REG)) = ETH_MPIC_IDR_OUT4_MASK;
    GigeIrqIack = SpacBaseAddr + ETH_MPIC_IACK3_REG;
    GigeIrqEoi = SpacBaseAddr + ETH_MPIC_EOI3_REG;
    
#ifdef __GIGE_SDRAM_FIQ_DEBUG__              
    
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_CSR1_REG)) = (1 << 1);  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_TASK1_REG)) = ETH_MPIC_TASK_MASK;  
    *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IDR16_REG)) = ETH_MPIC_IDR_OUT2_MASK;
/* *(REG32_PTR(SpacBaseAddr + (ETH_MPIC_BASE_ADDR_LO + 0x19C)))
   = ETH_MPIC_IDR_OUT2_MASK; */
    GigeFiqIack = SpacBaseAddr + ETH_MPIC_IACK1_REG;
    GigeFiqEoi = SpacBaseAddr + ETH_MPIC_EOI1_REG;

#endif     
    
  }
  
  *(REG32_PTR(SpacBaseAddr + ETH_MPIC_IVPR16_REG)) = ETH_MPIC_IVPR_PRIO_MASK | 
    ETH_MPIC_IVPR_S_MASK | ETH_MPIC_IVPR_P_MASK | ETH_MPIC_GIGE_VECTOR;  
   
#ifdef __GIGE_SDRAM_FIQ_DEBUG__          
/* *(REG32_PTR(SpacBaseAddr + (ETH_MPIC_BASE_ADDR_LO + 0x198)))
   = ETH_MPIC_IVPR_PRIO_MASK | ETH_MPIC_IVPR_S_MASK | ETH_MPIC_IVPR_P_MASK
   | 0x13; */
    
#endif     

#ifdef TEST_TIMER
/* enable timer 0 interrupt, CI first to 1, then 0, in order to load timer */
  *(REG32_PTR(SpacBaseAddr + ETH_MPIC_GTBCR0_REG)) = 0x80000000;
  *(REG32_PTR(SpacBaseAddr + ETH_MPIC_GTBCR0_REG)) = ETH_MPIC_TIMER_COUNT;
/* SEL_OUT to 0x04 to route the timer interrupt to IRQ0 */
  *(REG32_PTR(SpacBaseAddr + ETH_MPIC_GTDR0_REG)) = 0x04;

/* clear the interrupt mask & increase the priority level, and set vector*/
  *(REG32_PTR(SpacBaseAddr + ETH_MPIC_GTVPR0_REG)) = 0x000e0020;
#endif /* TEST_TIMER */


/* Enable interrupts */
  _Write_CPSR(cpsr);

  return rtnCode;
  
}
  
/*=========================================================================
 *  Function    : gigeHandlerDetachIRQ
 *
 *  Arguments   : N/A.
 * 
 *  Description : SPac's ISR dettach routine.
 * 
 *  return      : N/A.
 *
 =========================================================================*/
void gigeHandlerDetachIRQ(void)
{
  *IRQVECTOR = (unsigned int)gige_next_irq_service;
}

/*=========================================================================
 *  Function    : intMpicReset
 *
 *  Arguments   : N/A.
 * 
 *  Description : Resets the MPIC block.
 * 
 *  return      : ETH_OK - Operation succeded.
 *
 *  Note        : for now GigE-oriented. 
 *           
 =========================================================================*/
SpacEthRtnCode_t intMpicReset(void)
{
  
  SpacEthRtnCode_t rtnCode = ETH_OK;

  gigeHandlerDetachIRQ();
  
#ifdef __GIGE_SDRAM_FIQ_DEBUG__     
  gigeHandlerDetachFIQ();
#endif
  
/*  *(REG32_PTR(SpacBaseAddr + ETH_MPIC_GCR_REG)) = TS_MPIC_GCR_R; */
    
  return rtnCode;
  
}
  

/*=========================================================================
 *  Function    : EthSetupDescriptors
 *
 *  Arguments   : portId - Port ID (0, 1).
 *              : qId - Queue ID (0, 3).
 *              : direction - input, TX/RX.
 *              : memSpaceId - input, PCI/SDRAM/SRAM. 
 * 
 *  Description : Sets up a ring descriptor scheme.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_MEM_ALLOC_ERR - Memory allocation error.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : The "putInIdx/takeOutIdx" points to the location in the 
 *              : descriptor ring where the next "PUT"/"GET" operation
 *              : shall take place at TX/RX time. It is maintained by
 *              : the  software.
 *              : This routine would be called before enabling a queue.
 *              : Do not use local static variables, as this routine needs
 *              : to be reentrant.
 =========================================================================*/
SpacEthRtnCode_t EthSetupDescriptors(
  UINT32 portId, 
  UINT32 qId,
  EthDirFlow_t direction,
  UINT32 memSpaceId
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  EthDescrInfoPtr_t descrInfoPtr = 0; 
  EthMemPoolPtr_t memPoolPtr = 0;
  UINT32 ocnPortId = 0xFFFFFFFF;  
  EthMemDescrPtr_t memDescrPtr = 0;  
  UINT32 descrMemSize = 0;
  VUINT8 *dataPtr;
  EthRxFrmStatusCfgPtr_t rxFrmCfgPtr = 0;

  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 

  if(qId >= ETH_NUM_TX_Q){
    return ETH_PARAM_ERR;
  } 
  
  descrInfoPtr = (direction == ETH_TX ? 
    &(EthDriver.portInfo[portId]->txInfo.txQInfo[qId].descrInfo) : 
    &(EthDriver.portInfo[portId]->rxInfo.rxQInfo[qId].descrInfo));
    
  memPoolPtr = (direction == ETH_TX ? 
    &(EthDriver.port[portId].mac.tx.txQ[qId].memPool) : 
    &(EthDriver.port[portId].mac.rx.rxQ[qId].memPool));
      
  if((descrInfoPtr->numDescr == 0) || 
     (descrInfoPtr->numDescr > ETH_NUM_DESCR_PER_Q)){
    return ETH_PARAM_ERR;
  } 

  descrMemSize = descrInfoPtr->numDescr * ETH_DESCR_SIZE;
  
/* Use platform-specific means to get aligned memory. */
  if((memSpaceId == ETH_OCN_PCI1_ID) || (memSpaceId == ETH_OCN_PCI2_ID)){
    memDescrPtr = 
      (EthMemDescrPtr_t)((memSpaceId == ETH_OCN_PCI1_ID
                          ? XS_PCI1_BASE_ADDR : XS_PCI2_BASE_ADDR));
    memPoolPtr->descrPoolPtr = 
      memDescrPtr->portDescr[portId].directDescrQ[direction].descrQ[
          qId].descrTable;
  }else if(memSpaceId == ETH_OCN_SDRAM_ID){
    memDescrPtr = (EthMemDescrPtr_t)SDRAM_HIGH_MEM;
    memPoolPtr->descrPoolPtr = 
      memDescrPtr->portDescr[portId].directDescrQ[direction].descrQ[
          qId].descrTable;
  }else if(memSpaceId == ETH_OCN_XSCALE_ID){
    memDescrPtr = (EthMemDescrPtr_t)XS_SRAM_BASE_ADDR;
    memPoolPtr->descrPoolPtr = 
      memDescrPtr->portDescr[portId].directDescrQ[direction].descrQ[
          qId].descrTable;
  }else{
    return ETH_PARAM_ERR;
  }
      
  if(memPoolPtr->descrPoolPtr == 0){
    return ETH_MEM_ALLOC_ERR;
  }else{
    memPoolPtr->putInIdx = memPoolPtr->takeOutIdx = 0;
  }
    
/* MUST clear the descriptor pool. */
  tm_bzero((char *)memPoolPtr->descrPoolPtr, descrMemSize); 
  
  {    
  int idx;      
/* Loop until the last descriptor/buffer pair, and set the descriptor 
   fields appropriately. Set the next descriptor address */
    for(idx = 0; idx < descrInfoPtr->numDescr; idx++){
      rtnCode = sysVirtualToOcn(
          memPoolPtr->descrPoolPtr
          + ((idx +1) % descrInfoPtr->numDescr), 
          (UINT32*)(&(memPoolPtr->descrPoolPtr[idx].nextDescrAddr1)), 
          (UINT32*)(&(memPoolPtr->descrPoolPtr[idx].nextDescrAddr0)),
          &ocnPortId);
      if(rtnCode != 0){
        return rtnCode;
      }

/* frame descriptors MUST be 32-byte aligned, because that is the size of
   a cache line on the 80200 */
      tm_assert(
          EthSetupDescriptors,
          ((((UINT32) &(memPoolPtr->descrPoolPtr[idx]))
            & (TM_XSCALE_CACHE_LINE_LEN - 1)) == 0));

      if(direction == ETH_TX){
/* This needed in WAIT/QUEUE ISRs!!! */          
        ((EthTxFrmStatusCfgPtr_t)&(memPoolPtr->descrPoolPtr[
                                       idx].cfgStatus))->frmTxOk = ETH_BIT_SET;
      }else{
          tm_assert(EthSetupDesriptors, (direction == ETH_RX));

/* Get a Treck packet buffer, link it into the descriptor */
          dataPtr = (VUINT8 *) tfPoolIsrGetBuffer(
              tl80314InterfaceHandle,
              TM_80314_LARGE_RECV_BUF_SIZE);
          tm_assert(EthSetupDescriptors, (dataPtr != ((VUINT8 *) 0)));

          rtnCode = sysVirtualToOcn(
              (void*)dataPtr,
              (UINT32 *)(&memPoolPtr->descrPoolPtr[idx].startAddr1), 
              (UINT32 *)(&memPoolPtr->descrPoolPtr[idx].startAddr0),
              &ocnPortId);
          tm_assert(EthSetupDesriptors, (rtnCode == 0));

          rxFrmCfgPtr =
              (EthRxFrmStatusCfgPtr_t)&(memPoolPtr->descrPoolPtr[
                                            idx].cfgStatus);  
          rxFrmCfgPtr->byteCount = TM_80314_LARGE_RECV_BUF_SIZE;
          rxFrmCfgPtr->owner = ETH_DESCR_DMA; 
      }
    }

/*    if(descrInfoPtr->descrTermMode == ETH_DESCR_TERM_LAST){
      memPoolPtr->descrPoolPtr[idx - 1].nextDescrAddr1 |= ETH_DESCR_LAST; 
    }else if(descrInfoPtr->descrTermMode == ETH_DESCR_TERM_NULL){
      memPoolPtr->descrPoolPtr[idx - 1].nextDescrAddr1 &=
      ~ETH_DESCR_ADDR_HI_MASK; 
      memPoolPtr->descrPoolPtr[idx - 1].nextDescrAddr0 = 0x00000000; 
      } */

/* sync the cache for the frame descriptors only */
    EthSynchCache(memPoolPtr->descrPoolPtr, descrMemSize, ETH_TX);
    
  }
    
  memPoolPtr->freeEntries = descrInfoPtr->numDescr;
    
  return rtnCode;
    
}

/*=========================================================================
 *  Function    : EthFreeTxFrame
 *
 *  Arguments   : portId - Port ID (0, 1).
 *              : qId - Queue ID (0, 3). 
 *              : memPoolPtr - input, used to retrieve the "TAKE-OUT" pointer.
 * 
 *  Description : Releases transmitted frames from a specified TX queue.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_FREE_BUFF_ERR - Buffer release error.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : This would be a good place to gather statistics about 
 *              : transmitted frames.
 *              : Do not use local static variables, as this routine needs
 *              : to be reentrant.
 =========================================================================*/
SpacEthRtnCode_t EthFreeTxFrame(
    ttUserInterface     interfaceHandle,
    UINT32              portId, 
    UINT32              qId,  
    EthMemPoolPtr_t     memPoolPtr)
{
    SpacEthRtnCode_t rtnCode = ETH_OK;
  
/* Point to the descriptor's Config/Status dword. */    
    EthTxFrmStatusCfgPtr_t txFrmStatusCfgPtr = 
      (EthTxFrmStatusCfgPtr_t)&(memPoolPtr->descrPoolPtr[
                                    memPoolPtr->takeOutIdx].cfgStatus);
    UINT16 numDescr = 
      EthDriver.portInfo[portId]->txInfo.txQInfo[qId].descrInfo.numDescr;
    
/* Cache synchronization. */    
    EthSynchCache(
        &(memPoolPtr->descrPoolPtr[memPoolPtr->takeOutIdx]),
        ETH_DESCR_SIZE, ETH_RX);          
/* If the "owner" bit of the pointed descriptor is not own by DMA and the 
   data buffer address is not NULL, the ring has entries to be freed. */
    while((txFrmStatusCfgPtr->owner != ETH_DESCR_DMA) && 
       ((memPoolPtr->descrPoolPtr[memPoolPtr->takeOutIdx].startAddr0
         != 0x00000000)
        || ((memPoolPtr->descrPoolPtr[
                 memPoolPtr->takeOutIdx].startAddr1 & ETH_DESCR_ADDR_HI_MASK)
            != 0x000)))
    {
        if(txFrmStatusCfgPtr->eof == ETH_BIT_SET)
        {
/* notify Treck TCP/IP that a packet has been transmitted */
            tfSendCompleteInterface(
                interfaceHandle, TM_DEV_SEND_COMPLETE_DRIVER);
        }

/* Clear descriptor configuration/status. */      
        *((UINT32 *)txFrmStatusCfgPtr) = 0x00000000;       
/* Clear VLAN/Byte count configuration. */      
        memPoolPtr->descrPoolPtr[
            memPoolPtr->takeOutIdx].vlanBCnt = 0x00000000;
/* This should be the last clean-up step for each descriptor. */      
        memPoolPtr->descrPoolPtr[
            memPoolPtr->takeOutIdx].startAddr0 = 0x00000000;   
        memPoolPtr->descrPoolPtr[
            memPoolPtr->takeOutIdx].startAddr1 &= ~ETH_DESCR_ADDR_HI_MASK;   
/* Cache synchronization. */    
        EthSynchCache(
            &(memPoolPtr->descrPoolPtr[memPoolPtr->takeOutIdx]),
            ETH_DESCR_SIZE, ETH_TX);           
/* This should be atomic. */
        memPoolPtr->freeEntries++;
            
/* Update "TAKE-OUT" index. */
        memPoolPtr->takeOutIdx = (++memPoolPtr->takeOutIdx) % numDescr;
/* Point to the next descriptor's Config dword. */    
        txFrmStatusCfgPtr = 
            (EthTxFrmStatusCfgPtr_t)
            &(memPoolPtr->descrPoolPtr[memPoolPtr->takeOutIdx].cfgStatus); 
/* Cache synchronization. */    
        EthSynchCache(
            &(memPoolPtr->descrPoolPtr[memPoolPtr->takeOutIdx]),
            ETH_DESCR_SIZE, ETH_RX);           
    } 

    return rtnCode;
}


/*=========================================================================
 *  Function    : EthConfigTx
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : txInfoPtr - TX Configuration info pointer.
 * 
 *  Description : Configures the chip's DMA TX engine of a specified GigE 
 *              : port.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_MEM_ALLOC_ERR - Memory allocation error.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthConfigTx(
  UINT32 portId, 
  EthTxInfoPtr_t txInfoPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
    
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if(txInfoPtr == 0){
    return ETH_PARAM_ERR;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_TX_CFG_REG, 
    (UINT32 *)&(txInfoPtr->txCfgInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_TX_THRESH_REG, 
    (UINT32 *)&(txInfoPtr->txThreshInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_TX_PRI_THRESH_REG, 
    (UINT32 *)&(txInfoPtr->txPriThreshInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_TX_Q_PRI_MAP_REG, 
    (UINT32 *)&(txInfoPtr->txPriMapInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  rtnCode = EthRegAccess(portId, ETH_TX_PAUSE_CNT_REG, 
    (UINT32 *)&(txInfoPtr->txPauseCntInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  rtnCode = EthRegAccess(portId, ETH_TX_GVLAN_TAG_REG, 
    (UINT32 *)&(txInfoPtr->txGVlanInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  {
    int idx;
    EthMemPoolPtr_t memPoolPtr = 0; 
    UINT32 addrLo = 0x00000000; 
    UINT32 ocnPortId = 0xFFFFFFFF; 
    EthQPtrHi_t qPtrHi;
    *((UINT32 *)&qPtrHi) = 0x00000000;
    for(idx = 0; idx < ETH_NUM_TX_Q; idx++){
      if(txInfoPtr->txCtrlInfo.qEnables & (1 << idx)){
        rtnCode = EthRegAccess(portId, 
          ETH_TX_Q0_CFG_REG + (idx * ETH_TX_Q_REG_OFFSET), 
          (UINT32 *)&(txInfoPtr->txQInfo[idx].txQCfgInfo), REG_WRITE);
        if(rtnCode != ETH_OK){
          return rtnCode;
        } 
       
        rtnCode = EthRegAccess(portId, 
          ETH_TX_Q0_BUFF_CFG_REG + (idx * ETH_TX_Q_REG_OFFSET), 
          (UINT32 *)&(txInfoPtr->txQInfo[idx].txQBuffCfgInfo), REG_WRITE);
        if(rtnCode != ETH_OK){
          return rtnCode;
        } 
        
        rtnCode = EthSetupDescriptors(portId, idx, ETH_TX, ETH_MEM_ALLOC_ID);
        if(rtnCode != ETH_OK){
/* Do not return. Try to setup other queues. 
          return rtnCode; */ 
        }else{
          memPoolPtr = &(EthDriver.port[portId].mac.tx.txQ[idx].memPool);
/* Virtual-to-OCN address translation. */    
          rtnCode = sysVirtualToOcn(
              memPoolPtr->descrPoolPtr, (UINT32 *)&qPtrHi, 
              &addrLo, &ocnPortId);
          if(rtnCode != 0){
            return rtnCode;
          } 
/* Write LOW pointer registers before HIGH ones. */      
          rtnCode = EthRegAccess(
              portId, ETH_TX_Q0_PTR_LO_REG + (idx * ETH_TX_Q_REG_OFFSET), 
              &addrLo, REG_WRITE);
          if(rtnCode != ETH_OK){
            return rtnCode;
          } 
/* Set the valid bit only when you put frames in the TX queues.       
          qPtrHi.valid = ETH_BIT_SET;
*/
          rtnCode = EthRegAccess(
              portId, ETH_TX_Q0_PTR_HI_REG + (idx * ETH_TX_Q_REG_OFFSET), 
              (UINT32 *)&qPtrHi, REG_WRITE);
          if(rtnCode != ETH_OK){
            return rtnCode;
          }
          *((UINT32 *)&qPtrHi) = 0x00000000;
        }       
      }
    }
  }
    
  rtnCode = EthRegAccess(
      portId, ETH_TX_CTRL_REG, (UINT32 *)&(txInfoPtr->txCtrlInfo), 
      REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
    
  return rtnCode;
    
}



/*=========================================================================
 *  Function    : EthConfigRx
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : rxInfoPtr - RX Configuration info pointer.
 * 
 *  Description : Configures the chip's DMA RX engine of a specified GigE 
 *              : port.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_MEM_ALLOC_ERR - Memory allocation error.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthConfigRx(
  UINT32 portId, 
  EthRxInfoPtr_t rxInfoPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
    
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if(rxInfoPtr == 0){
    return ETH_PARAM_ERR;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_RX_CFG_REG, 
    (UINT32 *)&(rxInfoPtr->rxCfgInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_RX_PAUSE_THRESH_REG, 
    (UINT32 *)&(rxInfoPtr->rxPauseThreshInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  rtnCode = EthRegAccess(portId, ETH_RX_THRESH_REG, 
    (UINT32 *)&(rxInfoPtr->rxThreshInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  rtnCode = EthRegAccess(portId, ETH_RX_VLAN_TAG_MAP_REG, 
    (UINT32 *)&(rxInfoPtr->rxVlanMapInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
  {
    int idx;
    EthMemPoolPtr_t memPoolPtr = 0; 
    UINT32 addrLo = 0x00000000; 
    UINT32 ocnPortId = 0xFFFFFFFF; 
    EthQPtrHi_t qPtrHi;
    *((UINT32 *)&qPtrHi) = 0x00000000;
    for(idx = 0; idx < ETH_NUM_RX_Q; idx++){
      if(rxInfoPtr->rxCtrlInfo.qEnables & (1 << idx)){
        rtnCode = EthRegAccess(portId, 
          ETH_RX_Q0_CFG_REG + (idx * ETH_RX_Q_REG_OFFSET), 
          (UINT32 *)&(rxInfoPtr->rxQInfo[idx].rxQCfgInfo), REG_WRITE);
        if(rtnCode != ETH_OK){
          return rtnCode;
        } 
        rtnCode = EthRegAccess(portId, 
          ETH_RX_Q0_BUFF_CFG_REG + (idx * ETH_RX_Q_REG_OFFSET), 
          (UINT32 *)&(rxInfoPtr->rxQInfo[idx].rxQBuffCfgInfo), REG_WRITE);
        if(rtnCode != ETH_OK){
          return rtnCode;
        } 
        rtnCode = EthSetupDescriptors(portId, idx, ETH_RX, ETH_MEM_ALLOC_ID);
        if(rtnCode != ETH_OK){
/* Do not return. Try to setup other queues. 
          return rtnCode; */ 
        }else{
          memPoolPtr = &(EthDriver.port[portId].mac.rx.rxQ[idx].memPool);
/* Virtual-to-OCN address translation. */    
          rtnCode = sysVirtualToOcn(
              memPoolPtr->descrPoolPtr, (UINT32 *)&qPtrHi, 
              &addrLo, &ocnPortId);
          if(rtnCode != 0){
            return rtnCode;
          }           
/* Write LOW pointer registers before HIGH ones. */      
          rtnCode = EthRegAccess(
              portId, ETH_RX_Q0_PTR_LO_REG + (idx * ETH_RX_Q_REG_OFFSET), 
              &addrLo, REG_WRITE);
          if(rtnCode != ETH_OK){
            return rtnCode;
          } 
/* Set the valid bit.  */     
          qPtrHi.valid = ETH_BIT_SET;
          rtnCode = EthRegAccess(
              portId, ETH_RX_Q0_PTR_HI_REG + (idx * ETH_RX_Q_REG_OFFSET), 
              (UINT32 *)&qPtrHi, REG_WRITE);
          if(rtnCode != ETH_OK){
            return rtnCode;
          }
          *((UINT32 *)&qPtrHi) = 0x00000000;
        } 
      }
    }
  }
  
  rtnCode = EthRegAccess(
      portId, ETH_RX_CTRL_REG, (UINT32 *)&(rxInfoPtr->rxCtrlInfo),
      REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  return rtnCode;
}




/*=========================================================================
 *  Function    : EthConfigPort
 *
 *  Arguments   : portId - Port ID (0, 1)
 *              : portInfoPtr - Configuration info pointer.
 * 
 *  Description : Configures the chip's specified GigE port.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_MII_IF_STATUS_ERR - MII interface status error.
 *              : ETH_TIMEOUT_ERR - Time-out error.
 *              : ETH_INVAL_CFG_ERR - Invalid configuration error.
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthConfigPort(
  UINT32 portId, 
  EthPortInfoPtr_t portInfoPtr
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  UINT32 resetVal = 0x00000000;
    
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
  
  if(portInfoPtr == (EthPortInfoPtr_t)0){
    return ETH_PARAM_ERR;
  } 
  
/* Store a reference into the driver's structure. */
  EthDriver.portInfo[portId] = portInfoPtr;
  
/* Bring DMA/FIFO out of reset. */ 
  rtnCode = EthRegAccess(portId, ETH_TX_CFG_REG, &resetVal, REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  rtnCode = EthRegAccess(portId, ETH_RX_CFG_REG, &resetVal, REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
/* Bring MAC out of reset. */ 
  rtnCode = EthRegAccess(portId, ETH_MAC_CFG1_REG, &resetVal, REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

/* DMA MAC configuration. */
  rtnCode = EthConfigMac(portId, &portInfoPtr->macInfo);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

/* ETH_PORT_CTRL_REG.SPEED and BPT bits field must be set after auto-neg */
  if(portInfoPtr->macInfo.cfg2Info.ifMode == ETH_MAC_IF_NIBLE){
    portInfoPtr->portCtrlInfo.speed = ETH_BIT_SET;
  }else if(portInfoPtr->macInfo.cfg2Info.ifMode == ETH_MAC_IF_BYTE){
    portInfoPtr->portCtrlInfo.speed = ETH_BIT_CLEARED;
  }
  portInfoPtr->portCtrlInfo.backPressureType = 
    ~portInfoPtr->macInfo.cfg2Info.fullDuplex;
  rtnCode = EthRegAccess(portId, ETH_PORT_CTRL_REG, 
    (UINT32 *)&(portInfoPtr->portCtrlInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

/* Enable GigE interrupts. */
  rtnCode = EthRegAccess(portId, ETH_PORT_INT_MASK_REG, 
    (UINT32 *)&(portInfoPtr->portIntrMaskInfo), REG_WRITE);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

/* DMA RX/TX configuration. */
  rtnCode = EthConfigRx(portId, &portInfoPtr->rxInfo);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  rtnCode = EthConfigTx(portId, &portInfoPtr->txInfo);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

/* Let the queues go! 
  {
  UINT32 qGoVal = 1 << 15;
  
    rtnCode = EthRegAccess(portId, ETH_TX_CTRL_REG, &qGoVal, REG_WRITE);
    if(rtnCode != ETH_OK){
      return rtnCode;
    } 
    rtnCode = EthRegAccess(portId, ETH_RX_CTRL_REG, &qGoVal, REG_WRITE);
    if(rtnCode != ETH_OK){
      return rtnCode;
    } 
  }
*/

  EthDriver.port[portId].portEnabled = TRUE;
  
  return rtnCode;
    
}


/*=========================================================================
 *  Function    : EthResetPort
 *
 *  Arguments   : portId - Port ID (0, 1)
 * 
 *  Description : Resets the chip's specified GigE port.
 * 
 *  Return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_OK - Operation succeded.
 *              : ETH_BASE_ADDR_ERR - Invalid chip base address.
 *              : ETH_TIMEOUT_ERR - Time-out error. 
 *
 *  Note        : N/A.
 =========================================================================*/
SpacEthRtnCode_t EthResetPort(
  UINT32 portId
){
  
  SpacEthRtnCode_t rtnCode = ETH_OK;
  UINT32 resetVal = 0x00000000;
  UINT32  phyAddress = 
    (portId == ETH_PORT0_ID ? ETH_PHY0_ADDR : ETH_PHY1_ADDR);  
    
  if(portId >= ETH_NUM_PORTS){
    return ETH_PARAM_ERR;
  } 
    
/* Reset the STATS. */ 
  resetVal = ETH_PORT_CTRL_RESET_MASK;
  rtnCode = EthRegAccess(portId, ETH_PORT_CTRL_REG, &resetVal, REG_OR);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
/* Reset the MAC. */ 
  resetVal = ETH_MAC_RESET_MASK;
  rtnCode = EthRegAccess(portId, ETH_MAC_CFG1_REG, &resetVal, REG_OR);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
/* Reset DMA/FIFO. */   
  resetVal = ETH_RX_RESET_MASK;
  rtnCode = EthRegAccess(portId, ETH_RX_CFG_REG, &resetVal, REG_OR);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  resetVal = ETH_TX_RESET_MASK;
  rtnCode = EthRegAccess(portId, ETH_TX_CFG_REG, &resetVal, REG_OR);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 
  
/* Start of BCM5421-specific code. There is a 2us for the reset process
   to complete. Refer to BCM5421 datasheet. */ 
  resetVal = ETH_PHY_RESET_MASK;
  rtnCode = GigEDoLinkMngm(ETH_PORT0_ID, phyAddress, ETH_PHY_CTRL1_REG_IDX, 
    ETH_MAC_MII_WRITE, &resetVal);
  if(rtnCode != ETH_OK){
    return rtnCode;
  }   
/* End of BCM5421-specific code */ 
 
  resetVal = ETH_TBI_PHY_RESET_MASK;
  rtnCode = GigEDoLinkMngm(portId, ETH_TBI_PHY_ADDR, ETH_TBI_CTRL1_REG_IDX, 
    ETH_MAC_MII_WRITE, &resetVal);
  if(rtnCode != ETH_OK){
    return rtnCode;
  }   
  resetVal = ETH_TBI_RESET_MASK;
  rtnCode = GigEDoLinkMngm(portId, ETH_TBI_PHY_ADDR, ETH_TBI_CTRL2_REG_IDX, 
    ETH_MAC_MII_WRITE, &resetVal);
  if(rtnCode != ETH_OK){
    return rtnCode;
  }   
  resetVal = ETH_MAC_MII_RESET_MASK;
  rtnCode = EthRegAccess(portId, ETH_MAC_MII_CFG_REG, &resetVal, REG_OR);
  if(rtnCode != ETH_OK){
    return rtnCode;
  } 

  EthDriver.portInfo[portId] = 0;
  EthDriver.port[portId].portEnabled = FALSE;
  
  return rtnCode;
    
}


/*=========================================================================
 *  Function    : EthDrvReset
 *
 *  Arguments   : N/A.
 * 
 *  Description : Resets the chip's GigE block.
 * 
 *  Return      : ETH_OK - Operation succeded.
 * 
 *
 *  Note        : This is a good place to clear data structure used 
 *              : by the driver.
 =========================================================================*/
SpacEthRtnCode_t EthDrvReset(
    ttUserInterface     interfaceHandle)
{
    SpacEthRtnCode_t rtnCode = ETH_OK;    

/* 
 * Free memory from pre-allocated buffers in the receive ring 
 */
    (void)tfPoolDelete(interfaceHandle);
  
/* Free all buffers in the transmit ring */
    tf80314SendComplete(interfaceHandle);

    EthDriver.drvEnabled = FALSE;

    return rtnCode;
}


/*=========================================================================
 *  Function    : EthPutFrame
 *
 *  Arguments   : portId - Port ID (0, 1).
 *              : qId - Queue ID (0, 3).
 *              : framePtr - input, frame to transmit.
 *              : txFrmCfgInfoPtr - per-frame config info.
 * 
 *  Description : Puts a frame in a specified TX queue.
 * 
 *  return      : ETH_PARAM_ERR - Invalid parameter.
 *              : ETH_Q_FULL_ERR - Queue(TX ring) full condition.
 *              : ETH_OK - Operation succeded.
 *
 *  Note        : It is assumed that prior to enqueue a frame, a check for the
 *              : ring available space("freeEntries" value), has been done .
 *              : Do not use local static variables, as this routine needs
 *              : to be reentrant.
 =========================================================================*/
SpacEthRtnCode_t EthPutFrame(
  UINT32                portId,
  UINT32                qId,
  ttUserPacketPtr       uPacketPtr,
  EthTxFrmCfgInfoPtr_t  txFrmCfgInfoPtr)
{
    SpacEthRtnCode_t rtnCode = ETH_OK;
    EthTxFrmStatusCfgPtr_t txFrmCfgPtr = 0;
    EthVlanBCntPtr_t txVlanBCntPtr = 0;
    ttUserPacketPtr walkingUPacketPtr = uPacketPtr;
    EthMemPoolPtr_t memPoolPtr = 
      &(EthDriver.port[portId].mac.tx.txQ[qId].memPool);
    UINT16 numDescr = ETH_MAX_NUM_PCI_DESCR_PER_Q;
    UINT32 ocnPortId = 0xFFFFFFFF;      
    EthQPtrHi_t qPtrHi;
    int              scatterIndex;
    
    if ((uPacketPtr->pktuLinkDataPtr == 0)
        || (uPacketPtr->pktuLinkDataLength == 0))
    {
        return ETH_PARAM_ERR;
    }   

    scatterIndex = uPacketPtr->pktuLinkExtraCount;
    *((UINT32 *)&qPtrHi) = 0x00000000;
    
    while ( scatterIndex-- >= 0 )
    {
/* Cache synchronization. */    
    EthSynchCache(
        &(memPoolPtr->descrPoolPtr[memPoolPtr->putInIdx]),
        ETH_DESCR_SIZE, ETH_RX);      
/* Point to the descriptor's Config dword. */    
    txFrmCfgPtr = 
        (EthTxFrmStatusCfgPtr_t)
        &(memPoolPtr->descrPoolPtr[memPoolPtr->putInIdx].cfgStatus);        
/* If the "owner" bit of the pointed descriptor is own by DMA, or the 
   data buffer address is not NULL, the ring is full. It shouldn't happen if 
   checking for available space has ben done prior to call this routine. */
    if((txFrmCfgPtr->owner == ETH_DESCR_DMA)
       || (memPoolPtr->descrPoolPtr[memPoolPtr->putInIdx].startAddr0
           != 0x00000000)
       || ((memPoolPtr->descrPoolPtr[memPoolPtr->putInIdx].startAddr1
            & ETH_DESCR_ADDR_HI_MASK) != 0x000))
    {
        return ETH_Q_FULL_ERR;    
    }
    else
    {
/* Cache synchronization. This may be done before reaching this point. */    
        EthSynchCache(
            (void *)walkingUPacketPtr->pktuLinkDataPtr,
            walkingUPacketPtr->pktuLinkDataLength,
            ETH_TX); 
/* Virtual-to-OCN address translation. */    
        rtnCode = sysVirtualToOcn(
            (void*)walkingUPacketPtr->pktuLinkDataPtr,
            (UINT32*)(&memPoolPtr->descrPoolPtr[
                          memPoolPtr->putInIdx].startAddr1), 
            (UINT32*)(&memPoolPtr->descrPoolPtr[
                          memPoolPtr->putInIdx].startAddr0), &ocnPortId);
        if(rtnCode != 0)
        {
            return rtnCode;
        } 
   
/* This should be atomic. */
        memPoolPtr->freeEntries--;
                
/* Frame configuration. */    
        txFrmCfgPtr->descrIntr = txFrmCfgInfoPtr->txFrmStatusCfg.descrIntr;
        txVlanBCntPtr = 
          (EthVlanBCntPtr_t)&(memPoolPtr->descrPoolPtr[
                                  memPoolPtr->putInIdx].vlanBCnt);
        txVlanBCntPtr->byteCount = walkingUPacketPtr->pktuLinkDataLength;     

        if(walkingUPacketPtr == uPacketPtr)
        { /* sof. */
            txFrmCfgPtr->padFrame = txFrmCfgInfoPtr->txFrmStatusCfg.padFrame;
            txFrmCfgPtr->hugeFrame = txFrmCfgInfoPtr->txFrmStatusCfg.hugeFrame;
            txFrmCfgPtr->crc = ETH_BIT_SET; 
            txFrmCfgPtr->pfVlan = ETH_BIT_CLEARED;
            txFrmCfgPtr->sof = ETH_BIT_SET;   
            if(txFrmCfgPtr->pfVlan)
            {
                txVlanBCntPtr->vlanTag = txFrmCfgInfoPtr->vlanData;
            }   
        }         

        if (scatterIndex < 0 )
        {
/* Set EOF bit on last packet */
            txFrmCfgPtr->eof = ETH_BIT_SET; 
        }
/* This needed in WAIT/QUEUE ISRs!!! */
        txFrmCfgPtr->frmTxOk = ETH_BIT_SET;
/* Update "walking" pointer. */      
        walkingUPacketPtr = walkingUPacketPtr->pktuLinkNextPtr;
/* This should be the last step for each descriptor. */      
        txFrmCfgPtr->owner = ETH_DESCR_DMA;       
/* Cache synchronization. */    
        EthSynchCache(
            &(memPoolPtr->descrPoolPtr[memPoolPtr->putInIdx]),
            ETH_DESCR_SIZE, ETH_TX);      
/* Update "PUT-IN" index. */
        memPoolPtr->putInIdx = (++memPoolPtr->putInIdx) % numDescr;
       }
    }
/* Re-validate the queue. */  
    qPtrHi.valid = ETH_BIT_SET;   
    rtnCode = EthRegAccess(portId, 
        ETH_TX_Q0_PTR_HI_REG + (qId * ETH_TX_Q_REG_OFFSET), 
        (UINT32 *)&qPtrHi, REG_OR);
    if(rtnCode != ETH_OK)
    {
        return rtnCode;
    }
  
    return rtnCode;
}

int tf80314Initialize(
    ttUserInterface     interfaceHandle,
    int                 portId)
{
    ttForestLakeInfoPtr   infoPtr;
    int                   errorCode;
    
    errorCode = TM_ENOERROR;
    infoPtr = tm_get_raw_buffer(sizeof(ttForestLakeInfo));
    if(!infoPtr)
    {
        errorCode = TM_ENOBUFS;
        goto INIT_RETURN;
    }

    infoPtr->flPortId = portId;

    ((ttDeviceEntryPtr)interfaceHandle)->devDriverPtr = infoPtr;

INIT_RETURN:
    return errorCode;
}

/* 
 * tf80314Open()
 *  
 *      This routine initializes the 80314 Gigabit Ethernet block.  
 *   allocating and initializing the data structures (info, transmit and 
 *   receive rings,etc) and installing the ISR handler.
 *
 *
 */

int tf80314Open(ttUserInterface interfaceHandle)
{ 
  SpacEthRtnCode_t rtnCode = ETH_OK;
  int              idx;
  
  tm_bzero(&EthDriver, sizeof(EthDriver));

  tl80314InterfaceHandle = interfaceHandle;

/* we support only a single interface, currently */
  tf80314Initialize(interfaceHandle, 0);

  rtnCode = EthDrvInit(interfaceHandle);
  if(rtnCode != ETH_OK)
  {
      goto SPACOPEN_RETURN;
  } 

/* Attach GigE ISR here. */
  rtnCode = intMpicInit();
  if(rtnCode != ETH_OK)
  {
      goto SPACOPEN_RETURN;
  }

/* Start with GigE block configuration. */
  for(idx = 0; idx < ETH_NUM_PORTS; idx++)
  {
      rtnCode = EthConfigPort(idx, &EthDriverInfo[ETH_CFG_INFO_IDX]);
      if(rtnCode != ETH_OK)
      {
          goto SPACOPEN_RETURN;
      } 
  }    

SPACOPEN_RETURN:
  return rtnCode;
}


/*
 * tf80314Close()
 *
 *   Closes the 80314.  This routine should stop all 80314 operations and
 *   free all associated memory.
 *
 *   1. Disable interrupts from the 80314
 *   2. Stop the 80314 receive process.
 *   3. Free all transmit buffers.
 *   4. Free all receive buffers.
 */
int tf80314Close(ttUserInterface interfaceHandle)
{
    SpacEthRtnCode_t rtnCode = ETH_OK;
    ttForestLakeInfoPtr   infoPtr ;

    {
        int idx;
/* Start with GigE block reset. */
        for(idx = 0; idx < ETH_NUM_PORTS; idx++){
            rtnCode = EthResetPort(idx);
            if(rtnCode != ETH_OK){
                return rtnCode;
            } 
        }
    }
  
/* Remove GigE ISR here. */
    rtnCode = intMpicReset();
    if(rtnCode != ETH_OK){
    } 

    rtnCode = EthDrvReset(interfaceHandle);
    if(rtnCode != ETH_OK){
        return rtnCode;
    } 
  
    infoPtr = ((ttDeviceEntryPtr)interfaceHandle)->devDriverPtr;

    if(infoPtr)
    {
        tm_free_raw_buffer((ttRawBufferPtr)infoPtr);
        ((ttDeviceEntryPtr)interfaceHandle)->devDriverPtr = 
            (ttForestLakeInfoPtr)0;
    }

    return rtnCode;  
}


/* 
 * tf80314Ioctl()
 *
 *   This routine performs four primary functions:  refilling the receive ring,
 *   and maintaining multicast addresses.  The refill portion simply walks
 *   through the recieve ring and allocates a new buffer for each buffer that
 *   is empty.  While doing this, it also moves the suspend bit, so that the
 *   chip will always stop receiving upon filling the next to last packet,
 *   thus preventing overruns.
 *   for send/recv offload functions, this function will return the offload
 *   functions of the chip to stack (GET), and set the offload functions by
 *   the command from stack (SET)
 */
int tf80314Ioctl(ttUserInterface interfaceHandle,
                 int             flag,
                 void TM_FAR *   optionPtr,
                 int             optionLen)
{
#if 0
/*
 * Refill our receive buffers.
 */
    if (flag & TM_80314_IOCTL_REFILL)
    {
/* not used, currently we use the Treck receive buffer pool */
    }
#endif

/*
 *  Perform send completes on any necessary buffers in the transmit ring.
 */
    if (flag & TM_80314_IOCTL_SEND_COMPLETE)
    {
#ifndef TM_TASK_SEND
        tf80314SendComplete(interfaceHandle);
#endif /* TM_TASK_SEND */
    }
    return (TM_DEV_OKAY);
}

static void tf80314SendComplete(ttUserInterface interfaceHandle)
{
    int              portId, qId;
    EthMemPoolPtr_t  memPoolPtr;

    portId = 0;
    qId = 0;
    memPoolPtr = &(EthDriver.port[portId].mac.tx.txQ[qId].memPool);    

    (void) EthFreeTxFrame(interfaceHandle, portId, qId, memPoolPtr);
}

#ifdef TM_USE_DRV_ONE_SCAT_SEND
int tf80314ScatSend(ttUserInterface interfaceHandle,
                    ttUserPacketPtr  uPacketPtr )
{
    int                 errorCode;
    int                 scatterIndex;
    int                 portId;
    int                 qId;
    SpacEthRtnCode_t    rtnCode;
    EthTxFrmCfgInfo_t   txFrmCfgInfo;
    EthMemPoolPtr_t     memPoolPtr;

    scatterIndex = uPacketPtr->pktuLinkExtraCount;
    errorCode = TM_DEV_OKAY;
    portId = 0;
    qId = 0;
    *((UINT32 *)&(txFrmCfgInfo.txFrmStatusCfg)) = 0x00000000;
    txFrmCfgInfo.vlanData = 0x0000;
    memPoolPtr = &(EthDriver.port[portId].mac.tx.txQ[qId].memPool);    
        
/* reclaim frame descriptors for packets that have been transmitted */
    tf80314SendComplete(interfaceHandle);

    while (memPoolPtr->freeEntries <= scatterIndex)
    {
/* spin lock */
        tfInterfaceSpinLock(interfaceHandle);
/* reclaim frame descriptors for packets that have been transmitted, until
   we have enough frame descriptors to satisfy this request */
        tf80314SendComplete(interfaceHandle);
    }

/* send the packet */
    rtnCode = EthPutFrame(portId, qId, uPacketPtr, &txFrmCfgInfo);
    tm_assert(tf80314ScatSend, (rtnCode == ETH_OK));
  
    return errorCode;
}
#endif /* TM_USE_DRV_ONE_SCAT_SEND */

/*
 * tf80314Send()
 *
 *   This is a general sending packet function. 
 *   1. Check to see if a transmit command descriptor is free, spin-locking
 *      if necessary.
 *   2. Attempt to send-complete any packets that have not yet been freed:
 *     2.1 Start off in the ring where we left off last time (dirty index).
 *     2.2 For each descriptor that is a Transmit command, call 
 *         tfSendCompleteInterface.
 *   3. Add the data pointer for this buffer in the appropriate spot in the 
 *      current descripor.
 *   4. If this is the last buffer in this packet (TM_USER_BUFFER_LAST flag 
 *      set), trigger the command
 *   
 */
int tf80314Send(ttUserInterface interfaceHandle,
                char TM_FAR *   dataPtr,
                int             dataLength,
                int             flag)
{
    int                 errorCode;
    int                 portId;
    int                 qId;
    SpacEthRtnCode_t    rtnCode;
    EthTxFrmCfgInfo_t   txFrmCfgInfo;
    EthMemPoolPtr_t     memPoolPtr;
    ttUserPacket        uPacket;

/* Flags not used since we don't support scattered sends */
    flag=flag;
    tm_assert(tf80314Send, (flag & TM_USER_BUFFER_LAST));

/* build the structure that we pass to EthPutFrame */
    uPacket.pktuLinkExtraCount = 0; /* only one buffer to send */
    uPacket.pktuLinkNextPtr = (void TM_FAR *) 0;
    uPacket.pktuChainDataLength = dataLength;
    uPacket.pktuLinkDataLength = dataLength;
    uPacket.pktuLinkDataPtr = dataPtr;

    errorCode = TM_DEV_OKAY;
    portId = 0;
    qId = 0;
    *((UINT32 *)&(txFrmCfgInfo.txFrmStatusCfg)) = 0x00000000;
    txFrmCfgInfo.vlanData = 0x0000;
    memPoolPtr = &(EthDriver.port[portId].mac.tx.txQ[qId].memPool);    
        
/* reclaim frame descriptors for packets that have been transmitted */
    tf80314SendComplete(interfaceHandle);

    while (memPoolPtr->freeEntries < 1)
    {
/* spin lock */
        tfInterfaceSpinLock(interfaceHandle);
/* reclaim frame descriptors for packets that have been transmitted, until
   we have enough frame descriptors to satisfy this request */
        tf80314SendComplete(interfaceHandle);
    }

/* send the packet */
    rtnCode = EthPutFrame(portId, qId, &uPacket, &txFrmCfgInfo);
    tm_assert(tf80314Send, (rtnCode == ETH_OK));
  
    return errorCode;
}

/*
 * tf80314Receive()
 *
 *   1. Check current packet descripor (status) is ready 
 *   2. Set buffer/data pointer, and checksum offload pointer.
 *   3. Mark the frame receive datapointer as 0 (unavailable)
 */
int tf80314Receive(ttUserInterface        interfaceHandle,
                   char TM_FAR * TM_FAR * dataPtrPtr,
                   int  TM_FAR *          dataLengthPtr,
                   ttUserBufferPtr        bufHandlePtr)
{
    return tfPoolReceive( interfaceHandle,
                          dataPtrPtr,
                          dataLengthPtr,
                          bufHandlePtr );
}

/*
 * tf80314GetPhysAddr()
 *
 *   Returns the physical address of this Ethernet interface.  This is just
 *   sample data and this routine should be modified to return real data.
 *
 */
int tf80314GetPhysAddr(ttUserInterface interfaceHandle,
                       char TM_FAR *   physicalAddress)
{    
    ttForestLakeInfoPtr   infoPtr;
    tt32Bit               macWord1;
    tt32Bit               macWord2;
    int                   portId;
    int                   errorCode;

    errorCode = TM_DEV_ERROR;

    infoPtr = ((ttDeviceEntryPtr)interfaceHandle)->devDriverPtr;
    if(!infoPtr)
    {
        goto GETPHY_RETURN;
    }

    portId = infoPtr->flPortId;
    macWord1 = *(REG32_PTR(SpacBaseAddr + ETH_MAC_ADDR_PART1_REG + 
                           (portId * ETH_PORT_OFFSET)));

    macWord2 = *(REG32_PTR(SpacBaseAddr + ETH_MAC_ADDR_PART2_REG + 
                           (portId * ETH_PORT_OFFSET)));

    physicalAddress[0] = (macWord1 >> 24) & 0xff;
    physicalAddress[1] = (macWord1 >> 16) & 0xff;
    physicalAddress[2] = (macWord1 >>  8) & 0xff;
    physicalAddress[3] = (macWord1 >>  0) & 0xff;
    physicalAddress[4] = (macWord2 >> 24) & 0xff;
    physicalAddress[5] = (macWord2 >> 16) & 0xff;

GETPHY_RETURN:
    return errorCode;
}

/* sys_utils.c stuff */
typedef struct _MEM_MAP_ENTRY
{
    UINT32 proc_va_start; // procesor virtual address
    UINT32 proc_va_end;   // procesor virtual address

    UINT32 ocn_sa_m;    // most significant part of OCN start address
    UINT32 ocn_sa_l;    // least significant part of OCN start address

    UINT32 ocn_ea_m;    // most significant part of OCN end address
    UINT32 ocn_ea_l;    // least significant part of OCN end address

    UINT32 ocn_port;      // OCN port number
} MEM_MAP_ENTRY;


MEM_MAP_ENTRY eb_memory_map[] = {
// SDRAM (normally mapped cacheable)
{
XS_SDRAM_BASE_ADDR, (XS_SDRAM_BASE_ADDR + BOARD_SDRAM_SIZE - 1), // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x1FFFFFFF, // ocn end address
PORT_SDRAM              // ocn port number
},
// Non-cacheable SDRAM alias
{
0x20000000, 0x3FFFFFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x1FFFFFFF, // ocn end address
PORT_SDRAM              // ocn port number
},
// FLASH
{
0x40000000, 0x4007FFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x0007FFFF, // ocn end address
PORT_HLP                // ocn port number
},
// SRAM
/*{
0x50000000, 0x5000FFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x0000FFFF, // ocn end address
PORT_XSCALE             // ocn port number
},*/
// Non-cacheable SRAM alias
{
0x50200000, 0x5020FFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x0000FFFF, // ocn end address
PORT_XSCALE             // ocn port number
},
// Switching Fabric SDRAM alias
{
0x60000000, 0x7FFFFFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x1FFFFFFF, // ocn end address
PORT_SDRAM              // ocn port number
},
// PCI1 MEM32
{
0x80000000, 0x9FFFFFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x1FFFFFFF, // ocn end address
PORT_PCI1               // ocn port number
},
// PCI1 PFM1
{
0xA0000000, 0xAFFFFFFF, // xs start/end address
0x00000000, 0x40000000, // ocn start address
0x00000000, 0x4FFFFFFF, // ocn end address
PORT_PCI1               // ocn port number
},
// PCI1 PFM2
{
0xB0000000, 0xBFFFFFFF, // xs start/end address
0x00000000, 0x80000000, // ocn start address
0x00000000, 0x8FFFFFFF, // ocn end address
PORT_PCI1               // ocn port number
},

// PCI2 MEM32
{
0xC0000000, 0xDFFFFFFF, // xs start/end address
0x00000000, 0x00000000, // ocn start address
0x00000000, 0x1FFFFFFF, // ocn end address
PORT_PCI2               // ocn port number
},

// PCI2 PFM1
{
0xE0000000, 0xEFFFFFFF, // xs start/end address
0x00000000, 0x40000000, // ocn start address
0x00000000, 0x4FFFFFFF, // ocn end address
PORT_PCI2               // ocn port number
},

// PCI2
{
0xF0000000, 0xFFFFFFFF, // xs start/end address
0x00000000, 0x80000000, // ocn start address
0x00000000, 0x8FFFFFFF, // ocn end address
PORT_PCI2               // ocn port number
}

};


/*============================================================================
 *  Function    : sysVirtualToOcn()
 *
 *  Arguments   : xs_addr - processor address for which the corresponding OCN
 *                          coordinates (address:port) have to be defined.
 *                ocn_addr_m - pointer to location (processor address) where
 *                             the most significant part of the OCN address will
 *                             returned.
 *                ocn_addr_l - pointer to location (processor address) where
 *                             the least significant part of the OCN address will
 *                             returned.
 *                ocn_port - pointer to location (processor address) where
 *                           the number of the corresponding OCN port will
 *                           returned.
 *
 *  Description : This function converts an address in the processor address
 *                space into an OCN adrress:port combination that can be
 *                used by independent masters on other OCN ports (DMA, GIGE)
 *                
 *  Return      : 0 if the processor address was mapped, othrwise 0xFFFF_FFFF
 *
 ===========================================================================*/
UINT32 sysVirtualToOcn(
    void* xs_addr,
    UINT32* ocn_addr_m,
    UINT32* ocn_addr_l,
    UINT32* ocn_port
    )
{
UINT32 i;
UINT32 ret_val = 0xFFFFFFFF;

    // Walk through the memory map to find matching entry
    for (i = 0; i < (sizeof(eb_memory_map)/sizeof(MEM_MAP_ENTRY)); i++)
    {
        if (
            (UINT32)xs_addr >= eb_memory_map[i].proc_va_start &&
            (UINT32)xs_addr <= eb_memory_map[i].proc_va_end
            )
        {
            *ocn_addr_m = eb_memory_map[i].ocn_sa_m;
            *ocn_addr_l = eb_memory_map[i].ocn_sa_l + 
                ((UINT32)xs_addr - eb_memory_map[i].proc_va_start);
            *ocn_port =   eb_memory_map[i].ocn_port;
            ret_val = 0;
            break;
        }
    }

    return ret_val;
}

/*============================================================================
 *  Function    : sysOcnToVirtual()
 *
 *  Arguments   : ocn_addr_m - most significant part of the OCN address.
 *                ocn_addr_l - least significant part of the OCN address.
 *                ocn_port   - number of the corresponding OCN port.
 *
 *  Description : This function converts an OCN address:port combination into
 *                an address in the processor address space.
 *                
 *  Return      : An address in the processor space or 0xFFFFFFFF if unable
 *                to map specified OCN address.
 *
 ===========================================================================*/
void* sysOcnToVirtual(
    unsigned ocn_addr_m,
    unsigned ocn_addr_l,
    unsigned ocn_port
    )
{
UINT32 ret_val = 0xFFFFFFFF;
UINT32 xs_addr;
UINT32 i;

    // Walk through the memory map to find matching entry
    for (i = 0; i < (sizeof(eb_memory_map)/sizeof(MEM_MAP_ENTRY)); i++)
    {
        if (
            ocn_port == eb_memory_map[i].ocn_port &&
            ocn_addr_m == eb_memory_map[i].ocn_sa_m &&
            ocn_addr_l >= eb_memory_map[i].ocn_sa_l &&
            ocn_addr_l <= eb_memory_map[i].ocn_ea_l
           )
        {
            xs_addr = eb_memory_map[i].proc_va_start + 
                        (ocn_addr_l - eb_memory_map[i].ocn_sa_l);

            if (xs_addr <= eb_memory_map[i].proc_va_end)
            {
                ret_val = xs_addr;
            }

            break;
        }
    }

    return (void*)ret_val;
}


/*============================================================================
 *  Function    : cleanDCache()
 *
 *  Arguments   : none
 *
 *  Description : Cleans and invalidates Data Cache.
 *                
 *  Return      : none
 *
 ===========================================================================*/
void cleanDCache(void)
{   
int i;
UINT32 address = DATA_CACHE_FLUSH_ADDR;
UINT32 cpsr;

    /* Make sure that interrupts are disabled */
    cpsr = _Read_CPSR;
    _Write_CPSR(cpsr | _IRQ_DISABLE | _FIQ_DISABLE);

    for(i = 0; i < DATA_CACHE_SIZE/TM_XSCALE_CACHE_LINE_LEN; i++)
    {
        _Allocate_D_Cache_Line(address);
        address += TM_XSCALE_CACHE_LINE_LEN;
    }
    
    _Invalidate_D_Cache;
    _Drain_Write_Buffer;

    /* Restore CPSR (interrupt enables) */
    _Write_CPSR(cpsr);
}




/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:39      For: Xilinx-Samsung
 * File Name: tr80314.c     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 41805    Output checksum: 36074
 */
