/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2004
 * Copyright Elmic Systems Japan 1997 - 2004
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: Header file for Freescale Coldfire MCF5485 FEC driver
 *              This driver is based off of the example driver provided by
 *              Freescale.  It functionality for the Fase Ethernet Controller
 *              (FEC), the multi-channel DMA, and the programmable serial port
 *              (PSC).  Depending on what phy you are using, you will need to
 *              change the file bcm5222.h that is provided with this driver
 *              to operate with the phy that you are currently using.
 *
 *              At this time, the second ethernet port has not been shown to
 *              work.  The same problem occurs with the example drivers.  All
 *              of the code is present for the second ethernet port, and when
 *              it is shown to work this paragraph will be removed.
 *
 * $Source: source/drivers/mcf5485/trm5485.h $
 *
 * Modification History
 * $Revision: 6.0.2.2 $
 * $Date: 2010/01/18 06:36:08CST $
 * $Author: odile $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

#ifndef _TRM5485_H_
#define _TRM5485_H_

/* NOTE: TM_MCF5485_RX_BUF_LEN must be evenly divisible by 16 */
#ifdef TM_USE_DRV_SCAT_RECV
/* Not recommended to use smaller than 256. The last buffer will also
 include the CRC. */
#define TM_MCF5485_RX_BUF_LEN           256
/* ethernet receive buffer descriptors */
#define TM_MCF5485_ETHER_MAX_RECV_BUFD  46
#else /* ! TM_USE_DRV_SCAT_RECV */
/* Enough room to guarantee a complete packet per buffer including CRC */
#define TM_MCF5485_RX_BUF_LEN           1520
/* ethernet receive buffer descriptors */
#define TM_MCF5485_ETHER_MAX_RECV_BUFD  8
#endif /* ! TM_USE_DRV_SCAT_RECV */

/* ethernet transmit buffer descriptors */
#define TM_MCF5485_ETHER_MAX_XMIT_BUFD  32

/* device driver function prototypes */
void tfMcf5485ReceiveISRFec0(void);
void tfMcf5485ReceiveISRFec1(void);
int tfMcf5485Open(ttUserInterface interfaceHandle);
int tfMcf5485Close(ttUserInterface interfaceHandle);
int tfMcf5485Send(
    ttUserInterface  interfaceHandle,
    char TM_FAR    * bufferPtr,
    int              length,
    int              flag);
int tfMcf5485Receive(
    ttUserInterface interfaceHandle,
    char TM_FAR    ** dataPtrPtr,
    int  TM_FAR     * lengthPtr,
    ttUserBufferPtr   userBufferHandlePtr);
/* IOCTL function type flags */    
#define TM_MCF5485_IOCTL_REFILL_RX 0x0001
#define TM_MCF5485_SEND_COMPLETE   0x0002
int tfMcf5485Ioctl(
    ttUserInterface interfaceHandle, int flag,
    void TM_FAR * optionPtr, int optionLen);
int tfMcf5485GetPhyAddr ( ttUserInterface userInterface,
                         char   TM_FAR * address );

extern unsigned char __MBAR[];

/********************************************************************/
/* MII Speed Settings */
#define FEC_MII_10BASE_T        0
#define FEC_MII_100BASE_TX      1

/* MII Duplex Settings */
#define FEC_MII_HALF_DUPLEX     0
#define FEC_MII_FULL_DUPLEX     1

/* Timeout for MII communications */
#define FEC_MII_TIMEOUT         0x10000

/* External Interface Modes */
#define FEC_MODE_7WIRE          0
#define FEC_MODE_MII            1
#define FEC_MODE_LOOPBACK       2   /* Internal Loopback */

#define NUM_ETH_PORTS           2

/* Ethernet standard lengths in bytes*/
#define ETH_ADDR_LEN    (6)
#define ETH_TYPE_LEN    (2)
#define ETH_CRC_LEN     (4)
#define ETH_MAX_DATA    (1500)
#define ETH_MIN_DATA    (46)
#define ETH_HDR_LEN     (ETH_ADDR_LEN * 2 + ETH_TYPE_LEN)

/* Defined Ethernet Frame Types */
#define ETH_FRM_IP      (0x0800)
#define ETH_FRM_ARP     (0x0806)
#define ETH_FRM_RARP    (0x8035)
#define ETH_FRM_TEST    (0xA5A5)

/* Maximum and Minimum Ethernet Frame Sizes */
#define ETH_MAX_FRM     (ETH_HDR_LEN + ETH_MAX_DATA + ETH_CRC_LEN)
#define ETH_MIN_FRM     (ETH_HDR_LEN + ETH_MIN_DATA + ETH_CRC_LEN)
#define ETH_MTU         (ETH_HDR_LEN + ETH_MAX_DATA)

/* Ethernet Addresses */
typedef tt8Bit ETH_ADDR[ETH_ADDR_LEN];

/* 16-bit Ethernet Frame Type, ie. Protocol */
typedef tt16Bit ETH_FRM_TYPE;

/* Ethernet Frame Header definition */
typedef struct
{
    ETH_ADDR     dest;
    ETH_ADDR     src;
    ETH_FRM_TYPE type;
} ETH_HDR;

/* Ethernet Frame definition */
typedef struct
{
    ETH_HDR head;
    tt8Bit*  data;
} ETH_FRAME;

/*****************************************************************************
 *
 *  DMA Section
 *
 */

/*
 * Turn Execution Unit tasks ON (#define) or OFF (#undef)
 */
#undef MCD_INCLUDE_EU

/*
 * Number of DMA channels
 */
#define NCHANNELS 16

/*
 * Total number of variants
 */
#ifdef MCD_INCLUDE_EU
#define NUMOFVARIANTS   6
#else
#define NUMOFVARIANTS   4
#endif

/*
 * Define sizes of the various tables
 */
#define TASK_TABLE_SIZE     (NCHANNELS*32)
#define VAR_TAB_SIZE        (128)
#define CONTEXT_SAVE_SIZE   (128)
#define FUNCDESC_TAB_SIZE   (256)

#ifdef MCD_INCLUDE_EU
#define FUNCDESC_TAB_NUM    16
#else
#define FUNCDESC_TAB_NUM    1
#endif

/*
 * PTD contrl reg bits
 */
#define PTD_CTL_TSK_PRI         0x8000
#define PTD_CTL_COMM_PREFETCH   0x0001

/*
 * Task Control reg bits and field masks
 */
#define TASK_CTL_EN             0x8000
#define TASK_CTL_VALID          0x4000
#define TASK_CTL_ALWAYS         0x2000
#define TASK_CTL_INIT_MASK      0x1f00
#define TASK_CTL_ASTRT          0x0080
#define TASK_CTL_HIPRITSKEN     0x0040
#define TASK_CTL_HLDINITNUM     0x0020
#define TASK_CTL_ASTSKNUM_MASK  0x000f

/*
 * Priority reg bits and field masks
 */
#define PRIORITY_HLD            0x80
#define PRIORITY_PRI_MASK       0x07

/*
 * Debug Control reg bits and field masks
 */
#define DBG_CTL_BLOCK_TASKS_MASK    0xffff0000
#define DBG_CTL_AUTO_ARM            0x00008000
#define DBG_CTL_BREAK               0x00004000
#define DBG_CTL_COMP1_TYP_MASK      0x00003800
#define DBG_CTL_COMP2_TYP_MASK      0x00000070
#define DBG_CTL_EXT_BREAK           0x00000004
#define DBG_CTL_INT_BREAK           0x00000002

/*
 * PTD Debug reg selector addresses
 * This reg must be written with a value to show the contents of
 * one of the desired internal register.
 */
#define PTD_DBG_REQ             0x00 /* shows the state of 31 initiators */
#define PTD_DBG_TSK_VLD_INIT    0x01 /* shows which 16 tasks are valid and
                                        have initiators asserted */


/*
 * General return values
 */
#define MCD_OK                   0
#define MCD_ERROR               -1
#define MCD_TABLE_UNALIGNED     -2
#define MCD_CHANNEL_INVALID     -3

/*
 * tfMcdInitDma input flags
 */
#define MCD_RELOC_TASKS         0x00000001
#define MCD_NO_RELOC_TASKS      0x00000000
#define MCD_COMM_PREFETCH_EN    0x00000002  /* Commbus Prefetching -
                                               MCF547x/548x ONLY */

/*
 * tfMcdDmaStatus Status Values for each channel
 */
#define MCD_NO_DMA  1 /* No DMA has been requested since reset */
#define MCD_IDLE    2 /* DMA active, but the initiator is currently
                         inactive */
#define MCD_RUNNING 3 /* DMA active, and the initiator is currently active */
#define MCD_PAUSED  4 /* DMA active but it is currently paused */
#define MCD_HALTED  5 /* the most recent DMA has been killed with
                         MCD_killTask() */
#define MCD_DONE    6 /* the most recent DMA has completed. */


/*
 * tfMcdStartDma parameter defines
 */

/*
 * Constants for the funcDesc parameter
 */
/* Byte swapping: */
#define MCD_NO_BYTE_SWAP    0x00045670  /* to disable byte swapping. */
#define MCD_BYTE_REVERSE    0x00076540  /* to reverse the bytes of each
                                           tt32Bit of the DMAed data. */
#define MCD_U16_REVERSE     0x00067450  /* to reverse the 16-bit halves of
                                           each 32-bit data value being
                                           DMAed.*/
#define MCD_U16_BYTE_REVERSE    0x00054760 /* to reverse the byte halves of
                                              each 16-bit half of each
                                              32-bit data value DMAed */
#define MCD_NO_BIT_REV  0x00000000  /* do not reverse the bits of each byte
                                       DMAed. */
#define MCD_BIT_REV     0x00088880  /* reverse the bits of each byte DMAed */
/* CRCing: */
#define MCD_CRC16       0xc0100000  /* to perform CRC-16 on DMAed data. */
#define MCD_CRCCCITT    0xc0200000  /* to perform CRC-CCITT on DMAed data. */
#define MCD_CRC32       0xc0300000  /* to perform CRC-32 on DMAed data. */
#define MCD_CSUMINET    0xc0400000  /* to perform internet checksums on
                                       DMAed data.*/
#define MCD_NO_CSUM     0xa0000000  /* to perform no checksumming. */

#define MCD_FUNC_NOEU1 (MCD_NO_BYTE_SWAP | MCD_NO_BIT_REV | MCD_NO_CSUM)
#define MCD_FUNC_NOEU2 (MCD_NO_BYTE_SWAP | MCD_NO_CSUM)

/*
 * Constants for the flags parameter
 */
#define MCD_TT_FLAGS_RL   0x00000001 /* Read line */
#define MCD_TT_FLAGS_CW   0x00000002 /* Combine Writes */
#define MCD_TT_FLAGS_SP   0x00000004 /* Speculative prefetch(XLB)
                                        MCF547x/548x ONLY  */
#define MCD_TT_FLAGS_MASK 0x000000ff
#define MCD_TT_FLAGS_DEF  (MCD_TT_FLAGS_RL | MCD_TT_FLAGS_CW)

#define MCD_SINGLE_DMA  0x00000100 /* Unchained DMA */
#define MCD_CHAIN_DMA              /* TBD */
#define MCD_EU_DMA                 /* TBD */
#define MCD_FECTX_DMA   0x00001000 /* FEC TX ring DMA */
#define MCD_FECRX_DMA   0x00002000 /* FEC RX ring DMA */


/* 
 * these flags are valid for tfMcdStartDma and the chained buffer
 * descriptors
 */
#define MCD_BUF_READY   0x80000000 /* indicates that this buffer is now under
                                      the DMA's control */
#define MCD_WRAP        0x20000000 /* to tell the FEC Dmas to wrap to the
                                      first BD */
#define MCD_INTERRUPT   0x10000000 /* to generate an interrupt after
                                      completion of the DMA. */
#define MCD_END_FRAME   0x08000000 /* tell the DMA to end the frame when
                                      transferring last byte of data in
                                      buffer */
#define MCD_CRC_RESTART 0x40000000 /* to empty out the accumulated checksum
                                      prior to performing the DMA. */

/* Defines for the FEC buffer descriptor control/status word*/
#define MCD_FEC_BUF_READY   0x8000
#define MCD_FEC_WRAP        0x2000
#define MCD_FEC_INTERRUPT   0x1000
#define MCD_FEC_END_FRAME   0x0800


/*
 * Defines for general intuitiveness
 */

#define MCD_TRUE  1
#define MCD_FALSE 0

/*
 * Three different cases for destination and source.
 */
#define MINUS1          -1
#define ZERO            0
#define PLUS1           1

/*
 * Defines for the debug control register's functions
 */
#define DBG_CTL_COMP1_TASK  (0x00002000) /* have comparator 1 look for a
                                            task # */
#define DBG_CTL_ENABLE      (DBG_CTL_AUTO_ARM    | \
                             DBG_CTL_BREAK       | \
                             DBG_CTL_INT_BREAK   | \
                             DBG_CTL_COMP1_TASK)
#define DBG_CTL_DISABLE     (DBG_CTL_AUTO_ARM    | \
                             DBG_CTL_INT_BREAK   | \
                             DBG_CTL_COMP1_TASK)
#define DBG_KILL_ALL_STAT   (0xFFFFFFFF)

/*
 * Offset to context save area where progress info is stored
 */
#define CSAVE_OFFSET        10

/*
 * Defines for Byte Swapping
 */
#define MCD_BYTE_SWAP_KILLER    0xFFF8888F
#define MCD_NO_BYTE_SWAP_ATALL  0x00040000

/*
 * Execution Unit Identifiers
 */
#define MAC  0  /* legacy - not used */
#define LUAC 1  /* legacy - not used */
#define CRC  2  /* legacy - not used */
#define LURC 3  /* Logic Unit with CRC */

/*
 * Task Identifiers
 */
#define TASK_CHAINNOEU  0
#define TASK_SINGLENOEU 1
#ifdef MCD_INCLUDE_EU
#define TASK_CHAINEU    2
#define TASK_SINGLEEU   3
#define TASK_FECRX      4
#define TASK_FECTX      5
#else
#define TASK_CHAINEU    0
#define TASK_SINGLEEU   1
#define TASK_FECRX      2
#define TASK_FECTX      3
#endif

/* General DMA defines */
#define CURRBD 4
#define DCOUNT 6
#define DESTPTR 5
#define SRCPTR 7

/*
 * These structures represent the internal registers of the
 * multi-channel DMA
 */
struct dmaRegs_s {
   tt32Bit taskbar;         /* task table base address register */
   tt32Bit currPtr;
   tt32Bit endPtr;
   tt32Bit varTablePtr;
   tt16Bit dma_rsvd0;
   tt16Bit ptdControl;      /* ptd control */
   tt32Bit intPending;      /* interrupt pending register */
   tt32Bit intMask;         /* interrupt mask register */
   tt16Bit taskControl[16]; /* task control registers */
   tt8Bit  priority[32];    /* priority registers */
   tt32Bit initiatorMux;    /* initiator mux control */
   tt32Bit taskSize0;       /* task size control register 0. */
   tt32Bit taskSize1;       /* task size control register 1. */
   tt32Bit dma_rsvd1;       /* reserved */
   tt32Bit dma_rsvd2;       /* reserved */
   tt32Bit debugComp1;      /* debug comparator 1 */
   tt32Bit debugComp2;      /* debug comparator 2 */
   tt32Bit debugControl;    /* debug control */
   tt32Bit debugStatus;     /* debug status */
   tt32Bit ptdDebug;        /* priority task decode debug */
   tt32Bit dma_rsvd3[31];   /* reserved */
};
typedef volatile struct dmaRegs_s dmaRegs;

/* Task Table Entry struct*/
typedef struct {
    tt32Bit TDTstart;   /* task descriptor table start */
    tt32Bit TDTend;     /* task descriptor table end */
    tt32Bit varTab;     /* variable table start */
    tt32Bit FDTandFlags;    /* function descriptor table start and flags */
    volatile tt32Bit descAddrAndStatus;
    volatile tt32Bit modifiedVarTab;
    tt32Bit contextSaveSpace;   /* context save space start */
    tt32Bit literalBases;
} TaskTableEntry;


/* Chained buffer descriptor */
typedef volatile struct MCD_bufDesc_struct MCD_bufDesc;
struct MCD_bufDesc_struct {
   tt32Bit flags;         /* flags describing the DMA */
   tt32Bit csumResult;    /* checksum from checksumming performed since
                             last checksum reset */
   char  *srcAddr;      /* the address to move data from */
   char  *destAddr;     /* the address to move data to */
   char  *lastDestAddr; /* the last address written to */
   tt32Bit dmaSize;       /* the number of bytes to transfer independent
                             of the transfer size */
   MCD_bufDesc *next; /* next buffer descriptor in chain */
   tt32Bit info;          /* private information about this descriptor;
                             DMA does not affect it */
};

/* Progress Query struct */
typedef volatile struct MCD_XferProg_struct {
   char *lastSrcAddr;         /* the most-recent or last, post-increment
                               source address */
   char *lastDestAddr;        /* the most-recent or last, post-increment
                               destination address */
   tt32Bit  dmaSize;            /* the amount of data transferred for the
                                    current buffer */
   MCD_bufDesc *currBufDesc;/* pointer to the current buffer descriptor
                               being DMAed */
} MCD_XferProg;

/*
 * Create identifiers for each initiator/requestor
 */
#define DMA_ALWAYS      (0)
#define DMA_DSPI_RX     (1)
#define DMA_DSPI_TX     (2)
#define DMA_DREQ0       (3)
#define DMA_PSC0_RX     (4)
#define DMA_PSC0_TX     (5)
#define DMA_USBEP0      (6)
#define DMA_USBEP1      (7)
#define DMA_USBEP2      (8)
#define DMA_USBEP3      (9)
#define DMA_PCI_TX      (10)
#define DMA_PCI_RX      (11)
#define DMA_PSC1_RX     (12)
#define DMA_PSC1_TX     (13)
#define DMA_I2C_RX      (14)
#define DMA_I2C_TX      (15)
#define DMA_FEC0_RX     (16)
#define DMA_FEC0_TX     (17)
#define DMA_FEC1_RX     (18)
#define DMA_FEC1_TX     (19)
#define DMA_DREQ1       (20)
#define DMA_CTM0        (21)
#define DMA_CTM1        (22)
#define DMA_CTM2        (23)
#define DMA_CTM3        (24)
#define DMA_CTM4        (25)
#define DMA_CTM5        (26)
#define DMA_CTM6        (27)
#define DMA_CTM7        (28)
#define DMA_USBEP4      (29)
#define DMA_USBEP5      (30)
#define DMA_USBEP6      (31)
#define DMA_PSC2_RX     (32)
#define DMA_PSC2_TX     (33)
#define DMA_PSC3_RX     (34)
#define DMA_PSC3_TX     (35)
#define DMA_FEC_RX(x)   ((x == 0) ? DMA_FEC0_RX : DMA_FEC1_RX)
#define DMA_FEC_TX(x)   ((x == 0) ? DMA_FEC0_TX : DMA_FEC1_TX)

/*
 * Bit level definitions for status field of buffer descriptors
 */
#define TX_BD_R             0x8000
#define TX_BD_TO1           0x4000
#define TX_BD_SW_COMPLETE   0x4000
#define TX_BD_W             0x2000
#define TX_BD_TO2           0x1000
#define TX_BD_INTERRUPT     0x1000  /* MCF547x/8x Only */
#define TX_BD_L             0x0800
#define TX_BD_TC            0x0400
#define TX_BD_DEF           0x0200  /* MCF5272 Only */
#define TX_BD_ABC           0x0200
#define TX_BD_HB            0x0100  /* MCF5272 Only */
#define TX_BD_LC            0x0080  /* MCF5272 Only */
#define TX_BD_RL            0x0040  /* MCF5272 Only */
#define TX_BD_UN            0x0002  /* MCF5272 Only */
#define TX_BD_CSL           0x0001  /* MCF5272 Only */

#define RX_BD_E             0x8000
#define RX_BD_R01           0x4000
#define RX_BD_SW_ISR        0x4000
#define RX_BD_W             0x2000
#define RX_BD_R02           0x1000
#define RX_BD_INTERRUPT     0x1000  /* MCF547x/8x Only */
#define RX_BD_L             0x0800
#define RX_BD_SW_REFILL     0x0400
#define RX_BD_M             0x0100
#define RX_BD_BC            0x0080
#define RX_BD_MC            0x0040
#define RX_BD_LG            0x0020
#define RX_BD_NO            0x0010
#define RX_BD_CR            0x0004
#define RX_BD_OV            0x0002
#define RX_BD_TR            0x0001
#define RX_BD_ERROR         (RX_BD_NO | RX_BD_CR | RX_BD_OV | RX_BD_TR)

/* 
 * The following set of defines are included in this file in case the user 
 * does not have the include files that are provided by Freescale.  If the
 * user does have these files, they may remove these defines and use those
 * files instead.
 */

/****************************************************************************
 *
 * DMA Defines
 *
 */

/* Register read/write macros */
#define MCF_DMA_DIPR    (*(vuint32*)(void*)(&__MBAR[0x008014]))
#define MCF_DMA_DIMR    (*(vuint32*)(void*)(&__MBAR[0x008018]))
#define MCF_DMA_IMCR    (*(vuint32*)(void*)(&__MBAR[0x00805C]))

/* Bit definitions and macros for MCF_DMA_DIPR */
#define MCF_DMA_DIPR_TASK0           (0x00000001UL)
#define MCF_DMA_DIPR_TASK1           (0x00000002UL)
#define MCF_DMA_DIPR_TASK2           (0x00000004UL)
#define MCF_DMA_DIPR_TASK3           (0x00000008UL)
#define MCF_DMA_DIPR_TASK4           (0x00000010UL)
#define MCF_DMA_DIPR_TASK5           (0x00000020UL)
#define MCF_DMA_DIPR_TASK6           (0x00000040UL)
#define MCF_DMA_DIPR_TASK7           (0x00000080UL)
#define MCF_DMA_DIPR_TASK8           (0x00000100UL)
#define MCF_DMA_DIPR_TASK9           (0x00000200UL)
#define MCF_DMA_DIPR_TASK10          (0x00000400UL)
#define MCF_DMA_DIPR_TASK11          (0x00000800UL)
#define MCF_DMA_DIPR_TASK12          (0x00001000UL)
#define MCF_DMA_DIPR_TASK13          (0x00002000UL)
#define MCF_DMA_DIPR_TASK14          (0x00004000UL)
#define MCF_DMA_DIPR_TASK15          (0x00008000UL)

/* Bit definitions and macros for MCF_DMA_DIMR */
#define MCF_DMA_DIMR_TASK0           (0x00000001UL)
#define MCF_DMA_DIMR_TASK1           (0x00000002UL)
#define MCF_DMA_DIMR_TASK2           (0x00000004UL)
#define MCF_DMA_DIMR_TASK3           (0x00000008UL)
#define MCF_DMA_DIMR_TASK4           (0x00000010UL)
#define MCF_DMA_DIMR_TASK5           (0x00000020UL)
#define MCF_DMA_DIMR_TASK6           (0x00000040UL)
#define MCF_DMA_DIMR_TASK7           (0x00000080UL)
#define MCF_DMA_DIMR_TASK8           (0x00000100UL)
#define MCF_DMA_DIMR_TASK9           (0x00000200UL)
#define MCF_DMA_DIMR_TASK10          (0x00000400UL)
#define MCF_DMA_DIMR_TASK11          (0x00000800UL)
#define MCF_DMA_DIMR_TASK12          (0x00001000UL)
#define MCF_DMA_DIMR_TASK13          (0x00002000UL)
#define MCF_DMA_DIMR_TASK14          (0x00004000UL)
#define MCF_DMA_DIMR_TASK15          (0x00008000UL)

/* Bit definitions and macros for MCF_DMA_IMCR */
#define MCF_DMA_IMCR_SRC16(x)        (((x)&0x00000003UL)<<0)
#define MCF_DMA_IMCR_SRC17(x)        (((x)&0x00000003UL)<<2)
#define MCF_DMA_IMCR_SRC18(x)        (((x)&0x00000003UL)<<4)
#define MCF_DMA_IMCR_SRC19(x)        (((x)&0x00000003UL)<<6)
#define MCF_DMA_IMCR_SRC20(x)        (((x)&0x00000003UL)<<8)
#define MCF_DMA_IMCR_SRC21(x)        (((x)&0x00000003UL)<<10)
#define MCF_DMA_IMCR_SRC22(x)        (((x)&0x00000003UL)<<12)
#define MCF_DMA_IMCR_SRC23(x)        (((x)&0x00000003UL)<<14)
#define MCF_DMA_IMCR_SRC24(x)        (((x)&0x00000003UL)<<16)
#define MCF_DMA_IMCR_SRC25(x)        (((x)&0x00000003UL)<<18)
#define MCF_DMA_IMCR_SRC26(x)        (((x)&0x00000003UL)<<20)
#define MCF_DMA_IMCR_SRC27(x)        (((x)&0x00000003UL)<<22)
#define MCF_DMA_IMCR_SRC28(x)        (((x)&0x00000003UL)<<24)
#define MCF_DMA_IMCR_SRC29(x)        (((x)&0x00000003UL)<<26)
#define MCF_DMA_IMCR_SRC30(x)        (((x)&0x00000003UL)<<28)
#define MCF_DMA_IMCR_SRC31(x)        (((x)&0x00000003UL)<<30)
#define MCF_DMA_IMCR_SRC16_FEC0RX    (0x00000000UL)
#define MCF_DMA_IMCR_SRC17_FEC0TX    (0x00000000UL)
#define MCF_DMA_IMCR_SRC18_FEC0RX    (0x00000020UL)
#define MCF_DMA_IMCR_SRC19_FEC0TX    (0x00000080UL)
#define MCF_DMA_IMCR_SRC20_FEC1RX    (0x00000100UL)
#define MCF_DMA_IMCR_SRC21_DREQ1     (0x00000000UL)
#define MCF_DMA_IMCR_SRC21_FEC1TX    (0x00000400UL)
#define MCF_DMA_IMCR_SRC22_FEC0RX    (0x00001000UL)
#define MCF_DMA_IMCR_SRC23_FEC0TX    (0x00004000UL)
#define MCF_DMA_IMCR_SRC24_CTM0      (0x00010000UL)
#define MCF_DMA_IMCR_SRC24_FEC1RX    (0x00020000UL)
#define MCF_DMA_IMCR_SRC25_CTM1      (0x00040000UL)
#define MCF_DMA_IMCR_SRC25_FEC1TX    (0x00080000UL)
#define MCF_DMA_IMCR_SRC26_USBEP4    (0x00000000UL)
#define MCF_DMA_IMCR_SRC26_CTM2      (0x00200000UL)
#define MCF_DMA_IMCR_SRC27_USBEP5    (0x00000000UL)
#define MCF_DMA_IMCR_SRC27_CTM3      (0x00800000UL)
#define MCF_DMA_IMCR_SRC28_USBEP6    (0x00000000UL)
#define MCF_DMA_IMCR_SRC28_CTM4      (0x01000000UL)
#define MCF_DMA_IMCR_SRC28_DREQ1     (0x02000000UL)
#define MCF_DMA_IMCR_SRC28_PSC2RX    (0x03000000UL)
#define MCF_DMA_IMCR_SRC29_DREQ1     (0x04000000UL)
#define MCF_DMA_IMCR_SRC29_CTM5      (0x08000000UL)
#define MCF_DMA_IMCR_SRC29_PSC2TX    (0x0C000000UL)
#define MCF_DMA_IMCR_SRC30_FEC1RX    (0x00000000UL)
#define MCF_DMA_IMCR_SRC30_CTM6      (0x10000000UL)
#define MCF_DMA_IMCR_SRC30_PSC3RX    (0x30000000UL)
#define MCF_DMA_IMCR_SRC31_FEC1TX    (0x00000000UL)
#define MCF_DMA_IMCR_SRC31_CTM7      (0x80000000UL)
#define MCF_DMA_IMCR_SRC31_PSC3TX    (0xC0000000UL)
                                                
                                                
/****************************************************************************
 *                                              
 * Edge Port Module Defines                     
 *
 */

/* Register read/write macros */
#define MCF_EPORT_EPPAR    (*(vuint16*)(void*)(&__MBAR[0x000F00]))
#define MCF_EPORT_EPDDR    (*(vuint8 *)(void*)(&__MBAR[0x000F04]))
#define MCF_EPORT_EPIER    (*(vuint8 *)(void*)(&__MBAR[0x000F05]))
#define MCF_EPORT_EPDR     (*(vuint8 *)(void*)(&__MBAR[0x000F08]))
#define MCF_EPORT_EPPDR    (*(vuint8 *)(void*)(&__MBAR[0x000F09]))
#define MCF_EPORT_EPFR     (*(vuint8 *)(void*)(&__MBAR[0x000F0C]))

/* Bit definitions and macros for MCF_EPORT_EPPAR */
#define MCF_EPORT_EPPAR_EPPA1(x)         (((x)&0x0003)<<2)
#define MCF_EPORT_EPPAR_EPPA2(x)         (((x)&0x0003)<<4)
#define MCF_EPORT_EPPAR_EPPA3(x)         (((x)&0x0003)<<6)
#define MCF_EPORT_EPPAR_EPPA4(x)         (((x)&0x0003)<<8)
#define MCF_EPORT_EPPAR_EPPA5(x)         (((x)&0x0003)<<10)
#define MCF_EPORT_EPPAR_EPPA6(x)         (((x)&0x0003)<<12)
#define MCF_EPORT_EPPAR_EPPA7(x)         (((x)&0x0003)<<14)
#define MCF_EPORT_EPPAR_EPPAx_LEVEL      (0)
#define MCF_EPORT_EPPAR_EPPAx_RISING     (1)
#define MCF_EPORT_EPPAR_EPPAx_FALLING    (2)
#define MCF_EPORT_EPPAR_EPPAx_BOTH       (3)

/* Bit definitions and macros for MCF_EPORT_EPDDR */
#define MCF_EPORT_EPDDR_EPDD1            (0x02)
#define MCF_EPORT_EPDDR_EPDD2            (0x04)
#define MCF_EPORT_EPDDR_EPDD3            (0x08)
#define MCF_EPORT_EPDDR_EPDD4            (0x10)
#define MCF_EPORT_EPDDR_EPDD5            (0x20)
#define MCF_EPORT_EPDDR_EPDD6            (0x40)
#define MCF_EPORT_EPDDR_EPDD7            (0x80)

/* Bit definitions and macros for MCF_EPORT_EPIER */
#define MCF_EPORT_EPIER_EPIE1            (0x02)
#define MCF_EPORT_EPIER_EPIE2            (0x04)
#define MCF_EPORT_EPIER_EPIE3            (0x08)
#define MCF_EPORT_EPIER_EPIE4            (0x10)
#define MCF_EPORT_EPIER_EPIE5            (0x20)
#define MCF_EPORT_EPIER_EPIE6            (0x40)
#define MCF_EPORT_EPIER_EPIE7            (0x80)

/* Bit definitions and macros for MCF_EPORT_EPDR */
#define MCF_EPORT_EPDR_EPD1              (0x02)
#define MCF_EPORT_EPDR_EPD2              (0x04)
#define MCF_EPORT_EPDR_EPD3              (0x08)
#define MCF_EPORT_EPDR_EPD4              (0x10)
#define MCF_EPORT_EPDR_EPD5              (0x20)
#define MCF_EPORT_EPDR_EPD6              (0x40)
#define MCF_EPORT_EPDR_EPD7              (0x80)

/* Bit definitions and macros for MCF_EPORT_EPPDR */
#define MCF_EPORT_EPPDR_EPPD1            (0x02)
#define MCF_EPORT_EPPDR_EPPD2            (0x04)
#define MCF_EPORT_EPPDR_EPPD3            (0x08)
#define MCF_EPORT_EPPDR_EPPD4            (0x10)
#define MCF_EPORT_EPPDR_EPPD5            (0x20)
#define MCF_EPORT_EPPDR_EPPD6            (0x40)
#define MCF_EPORT_EPPDR_EPPD7            (0x80)

/* Bit definitions and macros for MCF_EPORT_EPFR */
#define MCF_EPORT_EPFR_EPF1              (0x02)
#define MCF_EPORT_EPFR_EPF2              (0x04)
#define MCF_EPORT_EPFR_EPF3              (0x08)
#define MCF_EPORT_EPFR_EPF4              (0x10)
#define MCF_EPORT_EPFR_EPF5              (0x20)
#define MCF_EPORT_EPFR_EPF6              (0x40)
#define MCF_EPORT_EPFR_EPF7              (0x80)


/****************************************************************************
 *
 * Flexbus Chip Select (FBCS) Defines
 *
 */

/* Register read/write macros */
#define MCF_FBCS_CSAR0      (*(vuint32*)(void*)(&__MBAR[0x000500]))
#define MCF_FBCS_CSMR0      (*(vuint32*)(void*)(&__MBAR[0x000504]))
#define MCF_FBCS_CSCR0      (*(vuint32*)(void*)(&__MBAR[0x000508]))
#define MCF_FBCS_CSAR1      (*(vuint32*)(void*)(&__MBAR[0x00050C]))
#define MCF_FBCS_CSMR1      (*(vuint32*)(void*)(&__MBAR[0x000510]))
#define MCF_FBCS_CSCR1      (*(vuint32*)(void*)(&__MBAR[0x000514]))
#define MCF_FBCS_CSAR2      (*(vuint32*)(void*)(&__MBAR[0x000518]))
#define MCF_FBCS_CSMR2      (*(vuint32*)(void*)(&__MBAR[0x00051C]))
#define MCF_FBCS_CSCR2      (*(vuint32*)(void*)(&__MBAR[0x000520]))
#define MCF_FBCS_CSAR3      (*(vuint32*)(void*)(&__MBAR[0x000524]))
#define MCF_FBCS_CSMR3      (*(vuint32*)(void*)(&__MBAR[0x000528]))
#define MCF_FBCS_CSCR3      (*(vuint32*)(void*)(&__MBAR[0x00052C]))
#define MCF_FBCS_CSAR4      (*(vuint32*)(void*)(&__MBAR[0x000530]))
#define MCF_FBCS_CSMR4      (*(vuint32*)(void*)(&__MBAR[0x000534]))
#define MCF_FBCS_CSCR4      (*(vuint32*)(void*)(&__MBAR[0x000538]))
#define MCF_FBCS_CSAR5      (*(vuint32*)(void*)(&__MBAR[0x00053C]))
#define MCF_FBCS_CSMR5      (*(vuint32*)(void*)(&__MBAR[0x000540]))
#define MCF_FBCS_CSCR5      (*(vuint32*)(void*)(&__MBAR[0x000544]))
#define MCF_FBCS_CSAR(x)    (*(vuint32*)(void*)(&__MBAR[0x000500 \
                                                        +((x)*0x00C)]))

#define MCF_FBCS_CSMR(x)    (*(vuint32*)(void*)(&__MBAR[0x000504 \
                                                    +((x)*0x00C)]))

#define MCF_FBCS_CSCR(x)    (*(vuint32*)(void*)(&__MBAR[0x000508 \
                                                    +((x)*0x00C)]))

/* Bit definitions and macros for MCF_FBCS_CSAR */
#define MCF_FBCS_CSAR_BA(x)        ((x)&0xFFFF0000)

/* Bit definitions and macros for MCF_FBCS_CSMR */
#define MCF_FBCS_CSMR_V            (0x00000001UL)
#define MCF_FBCS_CSMR_WP           (0x00000100UL)
#define MCF_FBCS_CSMR_BAM(x)       (((x)&0x0000FFFFUL)<<16)
#define MCF_FBCS_CSMR_BAM_4G       (0xFFFF0000UL)
#define MCF_FBCS_CSMR_BAM_2G       (0x7FFF0000UL)
#define MCF_FBCS_CSMR_BAM_1G       (0x3FFF0000UL)
#define MCF_FBCS_CSMR_BAM_1024M    (0x3FFF0000UL)
#define MCF_FBCS_CSMR_BAM_512M     (0x1FFF0000UL)
#define MCF_FBCS_CSMR_BAM_256M     (0x0FFF0000UL)
#define MCF_FBCS_CSMR_BAM_128M     (0x07FF0000UL)
#define MCF_FBCS_CSMR_BAM_64M      (0x03FF0000UL)
#define MCF_FBCS_CSMR_BAM_32M      (0x01FF0000UL)
#define MCF_FBCS_CSMR_BAM_16M      (0x00FF0000UL)
#define MCF_FBCS_CSMR_BAM_8M       (0x007F0000UL)
#define MCF_FBCS_CSMR_BAM_4M       (0x003F0000UL)
#define MCF_FBCS_CSMR_BAM_2M       (0x001F0000UL)
#define MCF_FBCS_CSMR_BAM_1M       (0x000F0000UL)
#define MCF_FBCS_CSMR_BAM_1024K    (0x000F0000UL)
#define MCF_FBCS_CSMR_BAM_512K     (0x00070000UL)
#define MCF_FBCS_CSMR_BAM_256K     (0x00030000UL)
#define MCF_FBCS_CSMR_BAM_128K     (0x00010000UL)
#define MCF_FBCS_CSMR_BAM_64K      (0x00000000UL)
                                              
/* Bit definitions and macros for MCF_FBCS_CSCR */
#define MCF_FBCS_CSCR_BSTW         (0x00000008UL)
#define MCF_FBCS_CSCR_BSTR         (0x00000010UL)
#define MCF_FBCS_CSCR_PS(x)        (((x)&0x00000003UL)<<6)
#define MCF_FBCS_CSCR_AA           (0x00000100UL)
#define MCF_FBCS_CSCR_WS(x)        (((x)&0x0000003FUL)<<10)
#define MCF_FBCS_CSCR_WRAH(x)      (((x)&0x00000003UL)<<16)
#define MCF_FBCS_CSCR_RDAH(x)      (((x)&0x00000003UL)<<18)
#define MCF_FBCS_CSCR_ASET(x)      (((x)&0x00000003UL)<<20)
#define MCF_FBCS_CSCR_SWSEN        (0x00800000UL)    
#define MCF_FBCS_CSCR_SWS(x)       (((x)&0x0000003FUL)<<26)
#define MCF_FBCS_CSCR_PS_8         (0x00000040UL)    
#define MCF_FBCS_CSCR_PS_16        (0x00000080UL)
#define MCF_FBCS_CSCR_PS_32        (0x00000000UL)

/****************************************************************************
 *
 * Fast Ethernet Controller (FEC) Defines
 *
 */

/* Register read/write macros */
#define MCF_FEC_EIR0                (*(vuint32*)(void*)(&__MBAR[0x009004]))
#define MCF_FEC_EIMR0               (*(vuint32*)(void*)(&__MBAR[0x009008]))
#define MCF_FEC_ECR0                (*(vuint32*)(void*)(&__MBAR[0x009024]))
#define MCF_FEC_MMFR0               (*(vuint32*)(void*)(&__MBAR[0x009040]))
#define MCF_FEC_MSCR0               (*(vuint32*)(void*)(&__MBAR[0x009044]))
#define MCF_FEC_MIBC0               (*(vuint32*)(void*)(&__MBAR[0x009064]))
#define MCF_FEC_RCR0                (*(vuint32*)(void*)(&__MBAR[0x009084]))
#define MCF_FEC_R_HASH0             (*(vuint32*)(void*)(&__MBAR[0x009088]))
#define MCF_FEC_TCR0                (*(vuint32*)(void*)(&__MBAR[0x0090C4]))
#define MCF_FEC_PALR0               (*(vuint32*)(void*)(&__MBAR[0x0090E4]))
#define MCF_FEC_PAUR0               (*(vuint32*)(void*)(&__MBAR[0x0090E8]))
#define MCF_FEC_OPD0                (*(vuint32*)(void*)(&__MBAR[0x0090EC]))
#define MCF_FEC_IAUR0               (*(vuint32*)(void*)(&__MBAR[0x009118]))
#define MCF_FEC_IALR0               (*(vuint32*)(void*)(&__MBAR[0x00911C]))
#define MCF_FEC_GAUR0               (*(vuint32*)(void*)(&__MBAR[0x009120]))
#define MCF_FEC_GALR0               (*(vuint32*)(void*)(&__MBAR[0x009124]))
#define MCF_FEC_FECTFWR0            (*(vuint32*)(void*)(&__MBAR[0x009144]))
#define MCF_FEC_FECRFDR0            (*(vuint32*)(void*)(&__MBAR[0x009184]))
#define MCF_FEC_FECRFSR0            (*(vuint32*)(void*)(&__MBAR[0x009188]))
#define MCF_FEC_FECRFCR0            (*(vuint32*)(void*)(&__MBAR[0x00918C]))
#define MCF_FEC_FECRLRFP0           (*(vuint32*)(void*)(&__MBAR[0x009190]))
#define MCF_FEC_FECRLWFP0           (*(vuint32*)(void*)(&__MBAR[0x009194]))
#define MCF_FEC_FECRFAR0            (*(vuint32*)(void*)(&__MBAR[0x009198]))
#define MCF_FEC_FECRFRP0            (*(vuint32*)(void*)(&__MBAR[0x00919C]))
#define MCF_FEC_FECRFWP0            (*(vuint32*)(void*)(&__MBAR[0x0091A0]))
#define MCF_FEC_FECTFDR0            (*(vuint32*)(void*)(&__MBAR[0x0091A4]))
#define MCF_FEC_FECTFSR0            (*(vuint32*)(void*)(&__MBAR[0x0091A8]))
#define MCF_FEC_FECTFCR0            (*(vuint32*)(void*)(&__MBAR[0x0091AC]))
#define MCF_FEC_FECTLRFP0           (*(vuint32*)(void*)(&__MBAR[0x0091B0]))
#define MCF_FEC_FECTLWFP0           (*(vuint32*)(void*)(&__MBAR[0x0091B4]))
#define MCF_FEC_FECTFAR0            (*(vuint32*)(void*)(&__MBAR[0x0091B8]))
#define MCF_FEC_FECTFRP0            (*(vuint32*)(void*)(&__MBAR[0x0091BC]))
#define MCF_FEC_FECTFWP0            (*(vuint32*)(void*)(&__MBAR[0x0091C0]))
#define MCF_FEC_FRST0               (*(vuint32*)(void*)(&__MBAR[0x0091C4]))
#define MCF_FEC_CTCWR0              (*(vuint32*)(void*)(&__MBAR[0x0091C8]))
#define MCF_FEC_RMON_T_DROP0        (*(vuint32*)(void*)(&__MBAR[0x009200]))
#define MCF_FEC_RMON_T_PACKETS0     (*(vuint32*)(void*)(&__MBAR[0x009204]))
#define MCF_FEC_RMON_T_BC_PKT0      (*(vuint32*)(void*)(&__MBAR[0x009208]))
#define MCF_FEC_RMON_T_MC_PKT0      (*(vuint32*)(void*)(&__MBAR[0x00920C]))
#define MCF_FEC_RMON_T_CRC_ALIGN0   (*(vuint32*)(void*)(&__MBAR[0x009210]))
#define MCF_FEC_RMON_T_UNDERSIZE0   (*(vuint32*)(void*)(&__MBAR[0x009214]))
#define MCF_FEC_RMON_T_OVERSIZE0    (*(vuint32*)(void*)(&__MBAR[0x009218]))
#define MCF_FEC_RMON_T_FRAG0        (*(vuint32*)(void*)(&__MBAR[0x00921C]))
#define MCF_FEC_RMON_T_JAB0         (*(vuint32*)(void*)(&__MBAR[0x009220]))
#define MCF_FEC_RMON_T_COL0         (*(vuint32*)(void*)(&__MBAR[0x009224]))
#define MCF_FEC_RMON_T_P640         (*(vuint32*)(void*)(&__MBAR[0x009228]))
#define MCF_FEC_RMON_T_P65TO1270    (*(vuint32*)(void*)(&__MBAR[0x00922C]))
#define MCF_FEC_RMON_T_P128TO2550   (*(vuint32*)(void*)(&__MBAR[0x009230]))
#define MCF_FEC_RMON_T_P256TO5110   (*(vuint32*)(void*)(&__MBAR[0x009234]))
#define MCF_FEC_RMON_T_P512TO10230  (*(vuint32*)(void*)(&__MBAR[0x009238]))
#define MCF_FEC_RMON_T_P1024TO20470 (*(vuint32*)(void*)(&__MBAR[0x00923C]))
#define MCF_FEC_RMON_T_P_GTE20480   (*(vuint32*)(void*)(&__MBAR[0x009240]))
#define MCF_FEC_RMON_T_OCTETS0      (*(vuint32*)(void*)(&__MBAR[0x009244]))
#define MCF_FEC_IEEE_T_DROP0        (*(vuint32*)(void*)(&__MBAR[0x009248]))
#define MCF_FEC_IEEE_T_FRAME_OK0    (*(vuint32*)(void*)(&__MBAR[0x00924C]))
#define MCF_FEC_IEEE_T_1COL0        (*(vuint32*)(void*)(&__MBAR[0x009250]))
#define MCF_FEC_IEEE_T_MCOL0        (*(vuint32*)(void*)(&__MBAR[0x009254]))
#define MCF_FEC_IEEE_T_DEF0         (*(vuint32*)(void*)(&__MBAR[0x009258]))
#define MCF_FEC_IEEE_T_LCOL0        (*(vuint32*)(void*)(&__MBAR[0x00925C]))
#define MCF_FEC_IEEE_T_EXCOL0       (*(vuint32*)(void*)(&__MBAR[0x009260]))
#define MCF_FEC_IEEE_T_MACERR0      (*(vuint32*)(void*)(&__MBAR[0x009264]))
#define MCF_FEC_IEEE_T_CSERR0       (*(vuint32*)(void*)(&__MBAR[0x009268]))
#define MCF_FEC_IEEE_T_SQE0         (*(vuint32*)(void*)(&__MBAR[0x00926C]))
#define MCF_FEC_IEEE_T_FDXFC0       (*(vuint32*)(void*)(&__MBAR[0x009270]))
#define MCF_FEC_IEEE_T_OCTETS_OK0   (*(vuint32*)(void*)(&__MBAR[0x009274]))
#define MCF_FEC_RMON_R_PACKETS0     (*(vuint32*)(void*)(&__MBAR[0x009284]))
#define MCF_FEC_RMON_R_BC_PKT0      (*(vuint32*)(void*)(&__MBAR[0x009288]))
#define MCF_FEC_RMON_R_MC_PKT0      (*(vuint32*)(void*)(&__MBAR[0x00928C]))
#define MCF_FEC_RMON_R_CRC_ALIGN0   (*(vuint32*)(void*)(&__MBAR[0x009290]))
#define MCF_FEC_RMON_R_UNDERSIZE0   (*(vuint32*)(void*)(&__MBAR[0x009294]))
#define MCF_FEC_RMON_R_OVERSIZE0    (*(vuint32*)(void*)(&__MBAR[0x009298]))
#define MCF_FEC_RMON_R_FRAG0        (*(vuint32*)(void*)(&__MBAR[0x00929C]))
#define MCF_FEC_RMON_R_JAB0         (*(vuint32*)(void*)(&__MBAR[0x0092A0]))
#define MCF_FEC_RMON_R_RESVD_00     (*(vuint32*)(void*)(&__MBAR[0x0092A4]))
#define MCF_FEC_RMON_R_P640         (*(vuint32*)(void*)(&__MBAR[0x0092A8]))
#define MCF_FEC_RMON_R_P65TO1270    (*(vuint32*)(void*)(&__MBAR[0x0092AC]))
#define MCF_FEC_RMON_R_P128TO2550   (*(vuint32*)(void*)(&__MBAR[0x0092B0]))
#define MCF_FEC_RMON_R_P256TO5110   (*(vuint32*)(void*)(&__MBAR[0x0092B4]))
#define MCF_FEC_RMON_R_512TO10230   (*(vuint32*)(void*)(&__MBAR[0x0092B8]))
#define MCF_FEC_RMON_R_1024TO20470  (*(vuint32*)(void*)(&__MBAR[0x0092BC]))
#define MCF_FEC_RMON_R_P_GTE20480   (*(vuint32*)(void*)(&__MBAR[0x0092C0]))
#define MCF_FEC_RMON_R_OCTETS0      (*(vuint32*)(void*)(&__MBAR[0x0092C4]))
#define MCF_FEC_IEEE_R_DROP0        (*(vuint32*)(void*)(&__MBAR[0x0092C8]))
#define MCF_FEC_IEEE_R_FRAME_OK0    (*(vuint32*)(void*)(&__MBAR[0x0092CC]))
#define MCF_FEC_IEEE_R_CRC0         (*(vuint32*)(void*)(&__MBAR[0x0092D0]))
#define MCF_FEC_IEEE_R_ALIGN0       (*(vuint32*)(void*)(&__MBAR[0x0092D4]))
#define MCF_FEC_IEEE_R_MACERR0      (*(vuint32*)(void*)(&__MBAR[0x0092D8]))
#define MCF_FEC_IEEE_R_FDXFC0       (*(vuint32*)(void*)(&__MBAR[0x0092DC]))
#define MCF_FEC_IEEE_R_OCTETS_OK0   (*(vuint32*)(void*)(&__MBAR[0x0092E0]))
#define MCF_FEC_EIR1                (*(vuint32*)(void*)(&__MBAR[0x009804]))
#define MCF_FEC_EIMR1               (*(vuint32*)(void*)(&__MBAR[0x009808]))
#define MCF_FEC_ECR1                (*(vuint32*)(void*)(&__MBAR[0x009824]))
#define MCF_FEC_MMFR1               (*(vuint32*)(void*)(&__MBAR[0x009840]))
#define MCF_FEC_MSCR1               (*(vuint32*)(void*)(&__MBAR[0x009844]))
#define MCF_FEC_MIBC1               (*(vuint32*)(void*)(&__MBAR[0x009864]))
#define MCF_FEC_RCR1                (*(vuint32*)(void*)(&__MBAR[0x009884]))
#define MCF_FEC_R_HASH1             (*(vuint32*)(void*)(&__MBAR[0x009888]))
#define MCF_FEC_TCR1                (*(vuint32*)(void*)(&__MBAR[0x0098C4]))
#define MCF_FEC_PALR1               (*(vuint32*)(void*)(&__MBAR[0x0098E4]))
#define MCF_FEC_PAUR1               (*(vuint32*)(void*)(&__MBAR[0x0098E8]))
#define MCF_FEC_OPD1                (*(vuint32*)(void*)(&__MBAR[0x0098EC]))
#define MCF_FEC_IAUR1               (*(vuint32*)(void*)(&__MBAR[0x009918]))
#define MCF_FEC_IALR1               (*(vuint32*)(void*)(&__MBAR[0x00991C]))
#define MCF_FEC_GAUR1               (*(vuint32*)(void*)(&__MBAR[0x009920]))
#define MCF_FEC_GALR1               (*(vuint32*)(void*)(&__MBAR[0x009924]))
#define MCF_FEC_FECTFWR1            (*(vuint32*)(void*)(&__MBAR[0x009944]))
#define MCF_FEC_FECRFDR1            (*(vuint32*)(void*)(&__MBAR[0x009984]))
#define MCF_FEC_FECRFSR1            (*(vuint32*)(void*)(&__MBAR[0x009988]))
#define MCF_FEC_FECRFCR1            (*(vuint32*)(void*)(&__MBAR[0x00998C]))
#define MCF_FEC_FECRLRFP1           (*(vuint32*)(void*)(&__MBAR[0x009990]))
#define MCF_FEC_FECRLWFP1           (*(vuint32*)(void*)(&__MBAR[0x009994]))
#define MCF_FEC_FECRFAR1            (*(vuint32*)(void*)(&__MBAR[0x009998]))
#define MCF_FEC_FECRFRP1            (*(vuint32*)(void*)(&__MBAR[0x00999C]))
#define MCF_FEC_FECRFWP1            (*(vuint32*)(void*)(&__MBAR[0x0099A0]))
#define MCF_FEC_FECTFDR1            (*(vuint32*)(void*)(&__MBAR[0x0099A4]))
#define MCF_FEC_FECTFSR1            (*(vuint32*)(void*)(&__MBAR[0x0099A8]))
#define MCF_FEC_FECTFCR1            (*(vuint32*)(void*)(&__MBAR[0x0099AC]))
#define MCF_FEC_FECTLRFP1           (*(vuint32*)(void*)(&__MBAR[0x0099B0]))
#define MCF_FEC_FECTLWFP1           (*(vuint32*)(void*)(&__MBAR[0x0099B4]))
#define MCF_FEC_FECTFAR1            (*(vuint32*)(void*)(&__MBAR[0x0099B8]))
#define MCF_FEC_FECTFRP1            (*(vuint32*)(void*)(&__MBAR[0x0099BC]))
#define MCF_FEC_FECTFWP1            (*(vuint32*)(void*)(&__MBAR[0x0099C0]))
#define MCF_FEC_FRST1               (*(vuint32*)(void*)(&__MBAR[0x0099C4]))
#define MCF_FEC_CTCWR1              (*(vuint32*)(void*)(&__MBAR[0x0099C8]))
#define MCF_FEC_RMON_T_DROP1        (*(vuint32*)(void*)(&__MBAR[0x009A00]))
#define MCF_FEC_RMON_T_PACKETS1     (*(vuint32*)(void*)(&__MBAR[0x009A04]))
#define MCF_FEC_RMON_T_BC_PKT1      (*(vuint32*)(void*)(&__MBAR[0x009A08]))
#define MCF_FEC_RMON_T_MC_PKT1      (*(vuint32*)(void*)(&__MBAR[0x009A0C]))
#define MCF_FEC_RMON_T_CRC_ALIGN1   (*(vuint32*)(void*)(&__MBAR[0x009A10]))
#define MCF_FEC_RMON_T_UNDERSIZE1   (*(vuint32*)(void*)(&__MBAR[0x009A14]))
#define MCF_FEC_RMON_T_OVERSIZE1    (*(vuint32*)(void*)(&__MBAR[0x009A18]))
#define MCF_FEC_RMON_T_FRAG1        (*(vuint32*)(void*)(&__MBAR[0x009A1C]))
#define MCF_FEC_RMON_T_JAB1         (*(vuint32*)(void*)(&__MBAR[0x009A20]))
#define MCF_FEC_RMON_T_COL1         (*(vuint32*)(void*)(&__MBAR[0x009A24]))
#define MCF_FEC_RMON_T_P641         (*(vuint32*)(void*)(&__MBAR[0x009A28]))
#define MCF_FEC_RMON_T_P65TO1271    (*(vuint32*)(void*)(&__MBAR[0x009A2C]))
#define MCF_FEC_RMON_T_P128TO2551   (*(vuint32*)(void*)(&__MBAR[0x009A30]))
#define MCF_FEC_RMON_T_P256TO5111   (*(vuint32*)(void*)(&__MBAR[0x009A34]))
#define MCF_FEC_RMON_T_P512TO10231  (*(vuint32*)(void*)(&__MBAR[0x009A38]))
#define MCF_FEC_RMON_T_P1024TO20471 (*(vuint32*)(void*)(&__MBAR[0x009A3C]))
#define MCF_FEC_RMON_T_P_GTE20481   (*(vuint32*)(void*)(&__MBAR[0x009A40]))
#define MCF_FEC_RMON_T_OCTETS1      (*(vuint32*)(void*)(&__MBAR[0x009A44]))
#define MCF_FEC_IEEE_T_DROP1        (*(vuint32*)(void*)(&__MBAR[0x009A48]))
#define MCF_FEC_IEEE_T_FRAME_OK1    (*(vuint32*)(void*)(&__MBAR[0x009A4C]))
#define MCF_FEC_IEEE_T_1COL1        (*(vuint32*)(void*)(&__MBAR[0x009A50]))
#define MCF_FEC_IEEE_T_MCOL1        (*(vuint32*)(void*)(&__MBAR[0x009A54]))
#define MCF_FEC_IEEE_T_DEF1         (*(vuint32*)(void*)(&__MBAR[0x009A58]))
#define MCF_FEC_IEEE_T_LCOL1        (*(vuint32*)(void*)(&__MBAR[0x009A5C]))
#define MCF_FEC_IEEE_T_EXCOL1       (*(vuint32*)(void*)(&__MBAR[0x009A60]))
#define MCF_FEC_IEEE_T_MACERR1      (*(vuint32*)(void*)(&__MBAR[0x009A64]))
#define MCF_FEC_IEEE_T_CSERR1       (*(vuint32*)(void*)(&__MBAR[0x009A68]))
#define MCF_FEC_IEEE_T_SQE1         (*(vuint32*)(void*)(&__MBAR[0x009A6C]))
#define MCF_FEC_IEEE_T_FDXFC1       (*(vuint32*)(void*)(&__MBAR[0x009A70]))
#define MCF_FEC_IEEE_T_OCTETS_OK1   (*(vuint32*)(void*)(&__MBAR[0x009A74]))
#define MCF_FEC_RMON_R_PACKETS1     (*(vuint32*)(void*)(&__MBAR[0x009A84]))
#define MCF_FEC_RMON_R_BC_PKT1      (*(vuint32*)(void*)(&__MBAR[0x009A88]))
#define MCF_FEC_RMON_R_MC_PKT1      (*(vuint32*)(void*)(&__MBAR[0x009A8C]))
#define MCF_FEC_RMON_R_CRC_ALIGN1   (*(vuint32*)(void*)(&__MBAR[0x009A90]))
#define MCF_FEC_RMON_R_UNDERSIZE1   (*(vuint32*)(void*)(&__MBAR[0x009A94]))
#define MCF_FEC_RMON_R_OVERSIZE1    (*(vuint32*)(void*)(&__MBAR[0x009A98]))
#define MCF_FEC_RMON_R_FRAG1        (*(vuint32*)(void*)(&__MBAR[0x009A9C]))
#define MCF_FEC_RMON_R_JAB1         (*(vuint32*)(void*)(&__MBAR[0x009AA0]))
#define MCF_FEC_RMON_R_RESVD_01     (*(vuint32*)(void*)(&__MBAR[0x009AA4]))
#define MCF_FEC_RMON_R_P641         (*(vuint32*)(void*)(&__MBAR[0x009AA8]))
#define MCF_FEC_RMON_R_P65TO1271    (*(vuint32*)(void*)(&__MBAR[0x009AAC]))
#define MCF_FEC_RMON_R_P128TO2551   (*(vuint32*)(void*)(&__MBAR[0x009AB0]))
#define MCF_FEC_RMON_R_P256TO5111   (*(vuint32*)(void*)(&__MBAR[0x009AB4]))
#define MCF_FEC_RMON_R_512TO10231   (*(vuint32*)(void*)(&__MBAR[0x009AB8]))
#define MCF_FEC_RMON_R_1024TO20471  (*(vuint32*)(void*)(&__MBAR[0x009ABC]))
#define MCF_FEC_RMON_R_P_GTE20481   (*(vuint32*)(void*)(&__MBAR[0x009AC0]))
#define MCF_FEC_RMON_R_OCTETS1      (*(vuint32*)(void*)(&__MBAR[0x009AC4]))
#define MCF_FEC_IEEE_R_DROP1        (*(vuint32*)(void*)(&__MBAR[0x009AC8]))
#define MCF_FEC_IEEE_R_FRAME_OK1    (*(vuint32*)(void*)(&__MBAR[0x009ACC]))
#define MCF_FEC_IEEE_R_CRC1         (*(vuint32*)(void*)(&__MBAR[0x009AD0]))
#define MCF_FEC_IEEE_R_ALIGN1       (*(vuint32*)(void*)(&__MBAR[0x009AD4]))
#define MCF_FEC_IEEE_R_MACERR1      (*(vuint32*)(void*)(&__MBAR[0x009AD8]))
#define MCF_FEC_IEEE_R_FDXFC1       (*(vuint32*)(void*)(&__MBAR[0x009ADC]))
#define MCF_FEC_IEEE_R_OCTETS_OK1   (*(vuint32*)(void*)(&__MBAR[0x009AE0]))
#define MCF_FEC_EIR(x)                  (*(vuint32*)(void*)(&__MBAR[0x009004 \
                                                            +((x)*0x800)]))
#define MCF_FEC_EIMR(x)                 (*(vuint32*)(void*)(&__MBAR[0x009008 \
                                                            +((x)*0x800)]))
#define MCF_FEC_ECR(x)                  (*(vuint32*)(void*)(&__MBAR[0x009024 \
                                                            +((x)*0x800)]))
#define MCF_FEC_MMFR(x)                 (*(vuint32*)(void*)(&__MBAR[0x009040 \
                                                            +((x)*0x800)]))
#define MCF_FEC_MSCR(x)                 (*(vuint32*)(void*)(&__MBAR[0x009044 \
                                                            +((x)*0x800)]))
#define MCF_FEC_MIBC(x)                 (*(vuint32*)(void*)(&__MBAR[0x009064 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RCR(x)                  (*(vuint32*)(void*)(&__MBAR[0x009084 \
                                                            +((x)*0x800)]))
#define MCF_FEC_R_HASH(x)               (*(vuint32*)(void*)(&__MBAR[0x009088 \
                                                            +((x)*0x800)]))
#define MCF_FEC_TCR(x)                  (*(vuint32*)(void*)(&__MBAR[0x0090C4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_PALR(x)                 (*(vuint32*)(void*)(&__MBAR[0x0090E4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_PAUR(x)                 (*(vuint32*)(void*)(&__MBAR[0x0090E8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_OPD(x)                  (*(vuint32*)(void*)(&__MBAR[0x0090EC \
                                                            +((x)*0x800)]))
#define MCF_FEC_IAUR(x)                 (*(vuint32*)(void*)(&__MBAR[0x009118 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IALR(x)                 (*(vuint32*)(void*)(&__MBAR[0x00911C \
                                                            +((x)*0x800)]))
#define MCF_FEC_GAUR(x)                 (*(vuint32*)(void*)(&__MBAR[0x009120 \
                                                            +((x)*0x800)]))
#define MCF_FEC_GALR(x)                 (*(vuint32*)(void*)(&__MBAR[0x009124 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFWR(x)              (*(vuint32*)(void*)(&__MBAR[0x009144 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRFDR(x)              (*(vuint32*)(void*)(&__MBAR[0x009184 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRFSR(x)              (*(vuint32*)(void*)(&__MBAR[0x009188 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRFCR(x)              (*(vuint32*)(void*)(&__MBAR[0x00918C \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRLRFP(x)             (*(vuint32*)(void*)(&__MBAR[0x009190 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRLWFP(x)             (*(vuint32*)(void*)(&__MBAR[0x009194 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRFAR(x)              (*(vuint32*)(void*)(&__MBAR[0x009198 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRFRP(x)              (*(vuint32*)(void*)(&__MBAR[0x00919C \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECRFWP(x)              (*(vuint32*)(void*)(&__MBAR[0x0091A0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFDR(x)              (*(vuint32*)(void*)(&__MBAR[0x0091A4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFSR(x)              (*(vuint32*)(void*)(&__MBAR[0x0091A8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFCR(x)              (*(vuint32*)(void*)(&__MBAR[0x0091AC \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTLRFP(x)             (*(vuint32*)(void*)(&__MBAR[0x0091B0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTLWFP(x)             (*(vuint32*)(void*)(&__MBAR[0x0091B4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFAR(x)              (*(vuint32*)(void*)(&__MBAR[0x0091B8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFRP(x)              (*(vuint32*)(void*)(&__MBAR[0x0091BC \
                                                            +((x)*0x800)]))
#define MCF_FEC_FECTFWP(x)              (*(vuint32*)(void*)(&__MBAR[0x0091C0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_FRST(x)                 (*(vuint32*)(void*)(&__MBAR[0x0091C4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_CTCWR(x)                (*(vuint32*)(void*)(&__MBAR[0x0091C8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_DROP(x)          (*(vuint32*)(void*)(&__MBAR[0x009200 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_PACKETS(x)       (*(vuint32*)(void*)(&__MBAR[0x009204 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_BC_PKT(x)        (*(vuint32*)(void*)(&__MBAR[0x009208 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_MC_PKT(x)        (*(vuint32*)(void*)(&__MBAR[0x00920C \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_CRC_ALIGN(x)     (*(vuint32*)(void*)(&__MBAR[0x009210 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_UNDERSIZE(x)     (*(vuint32*)(void*)(&__MBAR[0x009214 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_OVERSIZE(x)      (*(vuint32*)(void*)(&__MBAR[0x009218 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_FRAG(x)          (*(vuint32*)(void*)(&__MBAR[0x00921C \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_JAB(x)           (*(vuint32*)(void*)(&__MBAR[0x009220 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_COL(x)           (*(vuint32*)(void*)(&__MBAR[0x009224 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P64(x)           (*(vuint32*)(void*)(&__MBAR[0x009228 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P65TO127(x)      (*(vuint32*)(void*)(&__MBAR[0x00922C \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P128TO255(x)     (*(vuint32*)(void*)(&__MBAR[0x009230 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P256TO511(x)     (*(vuint32*)(void*)(&__MBAR[0x009234 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P512TO1023(x)    (*(vuint32*)(void*)(&__MBAR[0x009238 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P1024TO2047(x)   (*(vuint32*)(void*)(&__MBAR[0x00923C \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_P_GTE2048(x)     (*(vuint32*)(void*)(&__MBAR[0x009240 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_T_OCTETS(x)        (*(vuint32*)(void*)(&__MBAR[0x009244 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_DROP(x)          (*(vuint32*)(void*)(&__MBAR[0x009248 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_FRAME_OK(x)      (*(vuint32*)(void*)(&__MBAR[0x00924C \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_1COL(x)          (*(vuint32*)(void*)(&__MBAR[0x009250 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_MCOL(x)          (*(vuint32*)(void*)(&__MBAR[0x009254 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_DEF(x)           (*(vuint32*)(void*)(&__MBAR[0x009258 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_LCOL(x)          (*(vuint32*)(void*)(&__MBAR[0x00925C \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_EXCOL(x)         (*(vuint32*)(void*)(&__MBAR[0x009260 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_MACERR(x)        (*(vuint32*)(void*)(&__MBAR[0x009264 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_CSERR(x)         (*(vuint32*)(void*)(&__MBAR[0x009268 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_SQE(x)           (*(vuint32*)(void*)(&__MBAR[0x00926C \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_FDXFC(x)         (*(vuint32*)(void*)(&__MBAR[0x009270 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_T_OCTETS_OK(x)     (*(vuint32*)(void*)(&__MBAR[0x009274 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_PACKETS(x)       (*(vuint32*)(void*)(&__MBAR[0x009284 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_BC_PKT(x)        (*(vuint32*)(void*)(&__MBAR[0x009288 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_MC_PKT(x)        (*(vuint32*)(void*)(&__MBAR[0x00928C \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_CRC_ALIGN(x)     (*(vuint32*)(void*)(&__MBAR[0x009290 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_UNDERSIZE(x)     (*(vuint32*)(void*)(&__MBAR[0x009294 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_OVERSIZE(x)      (*(vuint32*)(void*)(&__MBAR[0x009298 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_FRAG(x)          (*(vuint32*)(void*)(&__MBAR[0x00929C \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_JAB(x)           (*(vuint32*)(void*)(&__MBAR[0x0092A0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_RESVD_0(x)       (*(vuint32*)(void*)(&__MBAR[0x0092A4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_P64(x)           (*(vuint32*)(void*)(&__MBAR[0x0092A8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_P65TO127(x)      (*(vuint32*)(void*)(&__MBAR[0x0092AC \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_P128TO255(x)     (*(vuint32*)(void*)(&__MBAR[0x0092B0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_P256TO511(x)     (*(vuint32*)(void*)(&__MBAR[0x0092B4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_512TO1023(x)     (*(vuint32*)(void*)(&__MBAR[0x0092B8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_1024TO2047(x)    (*(vuint32*)(void*)(&__MBAR[0x0092BC \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_P_GTE2048(x)     (*(vuint32*)(void*)(&__MBAR[0x0092C0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_RMON_R_OCTETS(x)        (*(vuint32*)(void*)(&__MBAR[0x0092C4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_DROP(x)          (*(vuint32*)(void*)(&__MBAR[0x0092C8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_FRAME_OK(x)      (*(vuint32*)(void*)(&__MBAR[0x0092CC \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_CRC(x)           (*(vuint32*)(void*)(&__MBAR[0x0092D0 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_ALIGN(x)         (*(vuint32*)(void*)(&__MBAR[0x0092D4 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_MACERR(x)        (*(vuint32*)(void*)(&__MBAR[0x0092D8 \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_FDXFC(x)         (*(vuint32*)(void*)(&__MBAR[0x0092DC \
                                                            +((x)*0x800)]))
#define MCF_FEC_IEEE_R_OCTETS_OK(x)     (*(vuint32*)(void*)(&__MBAR[0x0092E0 \
                                                            +((x)*0x800)]))
                                                                             
/* Bit definitions and macros for MCF_FEC_EIR */
#define MCF_FEC_EIR_RFERR              (0x00020000UL)
#define MCF_FEC_EIR_XFERR              (0x00040000UL)
#define MCF_FEC_EIR_XFUN               (0x00080000UL)
#define MCF_FEC_EIR_RL                 (0x00100000UL)
#define MCF_FEC_EIR_LC                 (0x00200000UL)
#define MCF_FEC_EIR_MII                (0x00800000UL)
#define MCF_FEC_EIR_TXF                (0x08000000UL)
#define MCF_FEC_EIR_GRA                (0x10000000UL)
#define MCF_FEC_EIR_BABT               (0x20000000UL)
#define MCF_FEC_EIR_BABR               (0x40000000UL)
#define MCF_FEC_EIR_HBERR              (0x80000000UL)
#define MCF_FEC_EIR_CLEAR_ALL          (0xFFFFFFFFUL)

/* Bit definitions and macros for MCF_FEC_EIMR */
#define MCF_FEC_EIMR_RFERR             (0x00020000UL)
#define MCF_FEC_EIMR_XFERR             (0x00040000UL)
#define MCF_FEC_EIMR_XFUN              (0x00080000UL)
#define MCF_FEC_EIMR_RL                (0x00100000UL)
#define MCF_FEC_EIMR_LC                (0x00200000UL)
#define MCF_FEC_EIMR_MII               (0x00800000UL)
#define MCF_FEC_EIMR_TXF               (0x08000000UL)
#define MCF_FEC_EIMR_GRA               (0x10000000UL)
#define MCF_FEC_EIMR_BABT              (0x20000000UL)
#define MCF_FEC_EIMR_BABR              (0x40000000UL)
#define MCF_FEC_EIMR_HBERR             (0x80000000UL)
#define MCF_FEC_EIMR_MASK_ALL          (0x00000000UL)
#define MCF_FEC_EIMR_UNMASK_ALL        (0xFFFFFFFFUL)

/* Bit definitions and macros for MCF_FEC_ECR */
#define MCF_FEC_ECR_RESET              (0x00000001UL)
#define MCF_FEC_ECR_ETHER_EN           (0x00000002UL)

/* Bit definitions and macros for MCF_FEC_MMFR */
#define MCF_FEC_MMFR_DATA(x)           (((x)&0x0000FFFFUL)<<0)
#define MCF_FEC_MMFR_TA(x)             (((x)&0x00000003UL)<<16)
#define MCF_FEC_MMFR_RA(x)             (((x)&0x0000001FUL)<<18)
#define MCF_FEC_MMFR_PA(x)             (((x)&0x0000001FUL)<<23)
#define MCF_FEC_MMFR_OP(x)             (((x)&0x00000003UL)<<28)
#define MCF_FEC_MMFR_ST(x)             (((x)&0x00000003UL)<<30)
#define MCF_FEC_MMFR_ST_01             (0x40000000UL)    
#define MCF_FEC_MMFR_OP_READ           (0x20000000UL)    
#define MCF_FEC_MMFR_OP_WRITE          (0x10000000UL)
#define MCF_FEC_MMFR_TA_10             (0x00020000UL)
                                                  
/* Bit definitions and macros for MCF_FEC_MSCR */ 
#define MCF_FEC_MSCR_MII_SPEED(x)      (((x)&0x0000003FUL)<<1)
#define MCF_FEC_MSCR_DIS_PREAMBLE      (0x00000080UL)
#define MCF_FEC_MSCR_MII_SPEED_133     (0x1B<<1)
#define MCF_FEC_MSCR_MII_SPEED_120     (0x18<<1)
#define MCF_FEC_MSCR_MII_SPEED_66      (0xE<<1)
#define MCF_FEC_MSCR_MII_SPEED_60      (0xC<<1)

/* Bit definitions and macros for MCF_FEC_MIBC */
#define MCF_FEC_MIBC_MIB_IDLE          (0x40000000UL)
#define MCF_FEC_MIBC_MIB_DISABLE       (0x80000000UL)

/* Bit definitions and macros for MCF_FEC_RCR */
#define MCF_FEC_RCR_LOOP               (0x00000001UL)
#define MCF_FEC_RCR_DRT                (0x00000002UL)
#define MCF_FEC_RCR_MII_MODE           (0x00000004UL)
#define MCF_FEC_RCR_PROM               (0x00000008UL)
#define MCF_FEC_RCR_BC_REJ             (0x00000010UL)
#define MCF_FEC_RCR_FCE                (0x00000020UL)
#define MCF_FEC_RCR_MAX_FL(x)          (((x)&0x000007FF)<<16)

/* Bit definitions and macros for MCF_FEC_R_HASH */
#define MCF_FEC_R_HASH_HASH(x)         (((x)&0x0000003FUL)<<24)
#define MCF_FEC_R_HASH_MULTCAST        (0x40000000UL)
#define MCF_FEC_R_HASH_FCE_DC          (0x80000000UL)

/* Bit definitions and macros for MCF_FEC_TCR */
#define MCF_FEC_TCR_GTS                (0x00000001UL)
#define MCF_FEC_TCR_HBC                (0x00000002UL)
#define MCF_FEC_TCR_FDEN               (0x00000004UL)
#define MCF_FEC_TCR_TFC_PAUSE          (0x00000008UL)
#define MCF_FEC_TCR_RFC_PAUSE          (0x00000010UL)
                                                  
/* Bit definitions and macros for MCF_FEC_PAUR */ 
#define MCF_FEC_PAUR_TYPE(x)           (((x)&0x0000FFFFUL)<<0)
#define MCF_FEC_PAUR_PADDR2(x)         (((x)&0x0000FFFFUL)<<16)
                                                       
/* Bit definitions and macros for MCF_FEC_OPD */       
#define MCF_FEC_OPD_OP_PAUSE(x)        (((x)&0x0000FFFFUL)<<0)
#define MCF_FEC_OPD_OPCODE(x)          (((x)&0x0000FFFFUL)<<16)
                                                       
/* Bit definitions and macros for MCF_FEC_FECTFWR */
#define MCF_FEC_FECTFWR_X_WMRK(x)      (((x)&0x0000000FUL)<<0)
#define MCF_FEC_FECTFWR_X_WMRK_64      (0x00000000UL)
#define MCF_FEC_FECTFWR_X_WMRK_128     (0x00000001UL)
#define MCF_FEC_FECTFWR_X_WMRK_192     (0x00000002UL)
#define MCF_FEC_FECTFWR_X_WMRK_256     (0x00000003UL)
#define MCF_FEC_FECTFWR_X_WMRK_320     (0x00000004UL)
#define MCF_FEC_FECTFWR_X_WMRK_384     (0x00000005UL)
#define MCF_FEC_FECTFWR_X_WMRK_448     (0x00000006UL)
#define MCF_FEC_FECTFWR_X_WMRK_512     (0x00000007UL)
#define MCF_FEC_FECTFWR_X_WMRK_576     (0x00000008UL)
#define MCF_FEC_FECTFWR_X_WMRK_640     (0x00000009UL)
#define MCF_FEC_FECTFWR_X_WMRK_704     (0x0000000AUL)
#define MCF_FEC_FECTFWR_X_WMRK_768     (0x0000000BUL)
#define MCF_FEC_FECTFWR_X_WMRK_832     (0x0000000CUL)
#define MCF_FEC_FECTFWR_X_WMRK_896     (0x0000000DUL)
#define MCF_FEC_FECTFWR_X_WMRK_960     (0x0000000EUL)
#define MCF_FEC_FECTFWR_X_WMRK_1024    (0x0000000FUL)
                                                  
/* Bit definitions and macros for MCF_FEC_FECRFDR */
#define MCF_FEC_FECRFDR_ADDR0            ((void*)(&__MBAR[0x009184]))
#define MCF_FEC_FECRFDR_ADDR1            ((void*)(&__MBAR[0x009984]))
#define MCF_FEC_FECRFDR_ADDR(x)          ((void*)(&__MBAR[0x009184 \
                                                    +(0x800*ch)]))

/* Bit definitions and macros for MCF_FEC_FECRFSR */
#define MCF_FEC_FECRFSR_EMT            (0x00010000UL)
#define MCF_FEC_FECRFSR_ALARM          (0x00020000UL)
#define MCF_FEC_FECRFSR_FU             (0x00040000UL)
#define MCF_FEC_FECRFSR_FR             (0x00080000UL)
#define MCF_FEC_FECRFSR_OF             (0x00100000UL)
#define MCF_FEC_FECRFSR_UF             (0x00200000UL)
#define MCF_FEC_FECRFSR_RXW            (0x00400000UL)
#define MCF_FEC_FECRFSR_FAE            (0x00800000UL)
#define MCF_FEC_FECRFSR_FRM(x)         (((x)&0x0000000FUL)<<24)
#define MCF_FEC_FECRFSR_IP             (0x80000000UL)

/* Bit definitions and macros for MCF_FEC_FECRFCR */
#define MCF_FEC_FECRFCR_COUNTER(x)     (((x)&0x0000FFFFUL)<<0)
#define MCF_FEC_FECRFCR_OF_MSK         (0x00080000UL)
#define MCF_FEC_FECRFCR_UF_MSK         (0x00100000UL)
#define MCF_FEC_FECRFCR_RXW_MSK        (0x00200000UL)
#define MCF_FEC_FECRFCR_FAE_MSK        (0x00400000UL)
#define MCF_FEC_FECRFCR_IP_MSK         (0x00800000UL)
#define MCF_FEC_FECRFCR_GR(x)          (((x)&0x00000007UL)<<24)
#define MCF_FEC_FECRFCR_FRM            (0x08000000UL)
#define MCF_FEC_FECRFCR_TIMER          (0x10000000UL)
#define MCF_FEC_FECRFCR_WFR            (0x20000000UL)
#define MCF_FEC_FECRFCR_WCTL           (0x40000000UL)
                                                  
/* Bit definitions and macros for MCF_FEC_FECRLRFPUL */
#define MCF_FEC_FECRLRFP_LRFP(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECRLWFP */
#define MCF_FEC_FECRLWFP_LWFP(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECRFAR */
#define MCF_FEC_FECRFAR_ALARM(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECRFRP */
#define MCF_FEC_FECRFRP_READ(x)        (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECRFWP */
#define MCF_FEC_FECRFWP_WRITE(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECTFDR */
#define MCF_FEC_FECTFDR_TFCW_TC        (0x04000000UL)
#define MCF_FEC_FECTFDR_TFCW_ABC       (0x02000000UL)
#define MCF_FEC_FECTFDR_ADDR0            ((void*)(&__MBAR[0x0091A4]))
#define MCF_FEC_FECTFDR_ADDR1            ((void*)(&__MBAR[0x0099A4]))
#define MCF_FEC_FECTFDR_ADDR(x)          ((void*)(&__MBAR[0x0091A4 \
                                                    +(0x800*ch)]))

/* Bit definitions and macros for MCF_FEC_FECTFSR */
#define MCF_FEC_FECTFSR_EMT            (0x00010000UL)
#define MCF_FEC_FECTFSR_ALARM          (0x00020000UL)
#define MCF_FEC_FECTFSR_FU             (0x00040000UL)
#define MCF_FEC_FECTFSR_FR             (0x00080000UL)
#define MCF_FEC_FECTFSR_OF             (0x00100000UL)
#define MCF_FEC_FECTFSR_UP             (0x00200000UL)
#define MCF_FEC_FECTFSR_FAE            (0x00800000UL)
#define MCF_FEC_FECTFSR_FRM(x)         (((x)&0x0000000FUL)<<24)
#define MCF_FEC_FECTFSR_TXW            (0x40000000UL)
#define MCF_FEC_FECTFSR_IP             (0x80000000UL)

/* Bit definitions and macros for MCF_FEC_FECTFCR */
#define MCF_FEC_FECTFCR_RESERVED       (0x00200000UL)
#define MCF_FEC_FECTFCR_COUNTER(x)     (((x)&0x0000FFFFUL)<<0|0x00200000)
#define MCF_FEC_FECTFCR_TXW_MSK        (0x00240000UL)
#define MCF_FEC_FECTFCR_OF_MSK         (0x00280000UL)
#define MCF_FEC_FECTFCR_UF_MSK         (0x00300000UL)
#define MCF_FEC_FECTFCR_FAE_MSK        (0x00600000UL)
#define MCF_FEC_FECTFCR_IP_MSK         (0x00A00000UL)
#define MCF_FEC_FECTFCR_GR(x)          (((x)&0x00000007)<<24|0x00200000)
#define MCF_FEC_FECTFCR_FRM            (0x08200000UL)
#define MCF_FEC_FECTFCR_TIMER          (0x10200000UL)
#define MCF_FEC_FECTFCR_WFR            (0x20200000UL)
#define MCF_FEC_FECTFCR_WCTL           (0x40200000UL)
                                                  
/* Bit definitions and macros for MCF_FEC_FECTLRFP */
#define MCF_FEC_FECTLRFP_LRFP(x)       (((x)&0x00000FFFUL)<<0)
                                                  
/* Bit definitions and macros for MCF_FEC_FECTLWFP */
#define MCF_FEC_FECTLWFP_LWFP(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECTFAR */
#define MCF_FEC_FECTFAR_ALARM(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECTFRP */
#define MCF_FEC_FECTFRP_READ(x)        (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FECTFWP */
#define MCF_FEC_FECTFWP_WRITE(x)       (((x)&0x00000FFFUL)<<0)

/* Bit definitions and macros for MCF_FEC_FRST */
#define MCF_FEC_FRST_RST_CTL           (0x01000000UL)
#define MCF_FEC_FRST_SW_RST            (0x02000000UL)

/* Bit definitions and macros for MCF_FEC_CTCWR */
#define MCF_FEC_CTCWR_TFCW             (0x01000000UL)
#define MCF_FEC_CTCWR_CRC              (0x02000000UL)

/****************************************************************************
 *
 * GPIO Defines
 *
 */

/* Register read/write macros */
#define MCF_GPIO_PODR_FBCTL         (*(vuint8 *)(void*)(&__MBAR[0x000A00]))
#define MCF_GPIO_PODR_FBCS          (*(vuint8 *)(void*)(&__MBAR[0x000A01]))
#define MCF_GPIO_PODR_DMA           (*(vuint8 *)(void*)(&__MBAR[0x000A02]))
#define MCF_GPIO_PODR_FEC0H         (*(vuint8 *)(void*)(&__MBAR[0x000A04]))
#define MCF_GPIO_PODR_FEC0L         (*(vuint8 *)(void*)(&__MBAR[0x000A05]))
#define MCF_GPIO_PODR_FEC1H         (*(vuint8 *)(void*)(&__MBAR[0x000A06]))
#define MCF_GPIO_PODR_FEC1L         (*(vuint8 *)(void*)(&__MBAR[0x000A07]))
#define MCF_GPIO_PODR_FECI2C        (*(vuint8 *)(void*)(&__MBAR[0x000A08]))
#define MCF_GPIO_PODR_PCIBG         (*(vuint8 *)(void*)(&__MBAR[0x000A09]))
#define MCF_GPIO_PODR_PCIBR         (*(vuint8 *)(void*)(&__MBAR[0x000A0A]))
#define MCF_GPIO_PODR_PSC3PSC2      (*(vuint8 *)(void*)(&__MBAR[0x000A0C]))
#define MCF_GPIO_PODR_PSC1PSC0      (*(vuint8 *)(void*)(&__MBAR[0x000A0D]))
#define MCF_GPIO_PODR_DSPI          (*(vuint8 *)(void*)(&__MBAR[0x000A0E]))
#define MCF_GPIO_PDDR_FBCTL         (*(vuint8 *)(void*)(&__MBAR[0x000A10]))
#define MCF_GPIO_PDDR_FBCS          (*(vuint8 *)(void*)(&__MBAR[0x000A11]))
#define MCF_GPIO_PDDR_DMA           (*(vuint8 *)(void*)(&__MBAR[0x000A12]))
#define MCF_GPIO_PDDR_FEC0H         (*(vuint8 *)(void*)(&__MBAR[0x000A14]))
#define MCF_GPIO_PDDR_FEC0L         (*(vuint8 *)(void*)(&__MBAR[0x000A15]))
#define MCF_GPIO_PDDR_FEC1H         (*(vuint8 *)(void*)(&__MBAR[0x000A16]))
#define MCF_GPIO_PDDR_FEC1L         (*(vuint8 *)(void*)(&__MBAR[0x000A17]))
#define MCF_GPIO_PDDR_FECI2C        (*(vuint8 *)(void*)(&__MBAR[0x000A18]))
#define MCF_GPIO_PDDR_PCIBG         (*(vuint8 *)(void*)(&__MBAR[0x000A19]))
#define MCF_GPIO_PDDR_PCIBR         (*(vuint8 *)(void*)(&__MBAR[0x000A1A]))
#define MCF_GPIO_PDDR_PSC3PSC2      (*(vuint8 *)(void*)(&__MBAR[0x000A1C]))
#define MCF_GPIO_PDDR_PSC1PSC0      (*(vuint8 *)(void*)(&__MBAR[0x000A1D]))
#define MCF_GPIO_PDDR_DSPI          (*(vuint8 *)(void*)(&__MBAR[0x000A1E]))
#define MCF_GPIO_PPDSDR_FBCTL       (*(vuint8 *)(void*)(&__MBAR[0x000A20]))
#define MCF_GPIO_PPDSDR_FBCS        (*(vuint8 *)(void*)(&__MBAR[0x000A21]))
#define MCF_GPIO_PPDSDR_DMA         (*(vuint8 *)(void*)(&__MBAR[0x000A22]))
#define MCF_GPIO_PPDSDR_FEC0H       (*(vuint8 *)(void*)(&__MBAR[0x000A24]))
#define MCF_GPIO_PPDSDR_FEC0L       (*(vuint8 *)(void*)(&__MBAR[0x000A25]))
#define MCF_GPIO_PPDSDR_FEC1H       (*(vuint8 *)(void*)(&__MBAR[0x000A26]))
#define MCF_GPIO_PPDSDR_FEC1L       (*(vuint8 *)(void*)(&__MBAR[0x000A27]))
#define MCF_GPIO_PPDSDR_FECI2C      (*(vuint8 *)(void*)(&__MBAR[0x000A28]))
#define MCF_GPIO_PPDSDR_PCIBG       (*(vuint8 *)(void*)(&__MBAR[0x000A29]))
#define MCF_GPIO_PPDSDR_PCIBR       (*(vuint8 *)(void*)(&__MBAR[0x000A2A]))
#define MCF_GPIO_PPDSDR_PSC3PSC2    (*(vuint8 *)(void*)(&__MBAR[0x000A2C]))
#define MCF_GPIO_PPDSDR_PSC1PSC0    (*(vuint8 *)(void*)(&__MBAR[0x000A2D]))
#define MCF_GPIO_PPDSDR_DSPI        (*(vuint8 *)(void*)(&__MBAR[0x000A2E]))
#define MCF_GPIO_PCLRR_FBCTL        (*(vuint8 *)(void*)(&__MBAR[0x000A30]))
#define MCF_GPIO_PCLRR_FBCS         (*(vuint8 *)(void*)(&__MBAR[0x000A31]))
#define MCF_GPIO_PCLRR_DMA          (*(vuint8 *)(void*)(&__MBAR[0x000A32]))
#define MCF_GPIO_PCLRR_FEC0H        (*(vuint8 *)(void*)(&__MBAR[0x000A34]))
#define MCF_GPIO_PCLRR_FEC0L        (*(vuint8 *)(void*)(&__MBAR[0x000A35]))
#define MCF_GPIO_PCLRR_FEC1H        (*(vuint8 *)(void*)(&__MBAR[0x000A36]))
#define MCF_GPIO_PCLRR_FEC1L        (*(vuint8 *)(void*)(&__MBAR[0x000A37]))
#define MCF_GPIO_PCLRR_FECI2C       (*(vuint8 *)(void*)(&__MBAR[0x000A38]))
#define MCF_GPIO_PCLRR_PCIBG        (*(vuint8 *)(void*)(&__MBAR[0x000A39]))
#define MCF_GPIO_PCLRR_PCIBR        (*(vuint8 *)(void*)(&__MBAR[0x000A3A]))
#define MCF_GPIO_PCLRR_PSC3PSC2     (*(vuint8 *)(void*)(&__MBAR[0x000A3C]))
#define MCF_GPIO_PCLRR_PSC1PSC0     (*(vuint8 *)(void*)(&__MBAR[0x000A3D]))
#define MCF_GPIO_PCLRR_DSPI         (*(vuint8 *)(void*)(&__MBAR[0x000A3E]))
#define MCF_GPIO_PAR_FBCTL          (*(vuint16*)(void*)(&__MBAR[0x000A40]))
#define MCF_GPIO_PAR_FBCS           (*(vuint8 *)(void*)(&__MBAR[0x000A42]))
#define MCF_GPIO_PAR_DMA            (*(vuint8 *)(void*)(&__MBAR[0x000A43]))
#define MCF_GPIO_PAR_FECI2CIRQ      (*(vuint16*)(void*)(&__MBAR[0x000A44]))
#define MCF_GPIO_PAR_PCIBG          (*(vuint16*)(void*)(&__MBAR[0x000A48]))
#define MCF_GPIO_PAR_PCIBR          (*(vuint16*)(void*)(&__MBAR[0x000A4A]))
#define MCF_GPIO_PAR_PSC3           (*(vuint8 *)(void*)(&__MBAR[0x000A4C]))
#define MCF_GPIO_PAR_PSC2           (*(vuint8 *)(void*)(&__MBAR[0x000A4D]))
#define MCF_GPIO_PAR_PSC1           (*(vuint8 *)(void*)(&__MBAR[0x000A4E]))
#define MCF_GPIO_PAR_PSC0           (*(vuint8 *)(void*)(&__MBAR[0x000A4F]))
#define MCF_GPIO_PAR_DSPI           (*(vuint16*)(void*)(&__MBAR[0x000A50]))
#define MCF_GPIO_PAR_TIMER          (*(vuint8 *)(void*)(&__MBAR[0x000A52]))

/* Bit definitions and macros for MCF_GPIO_PODR_FBCTL */
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL0              (0x01)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL1              (0x02)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL2              (0x04)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL3              (0x08)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL4              (0x10)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL5              (0x20)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL6              (0x40)
#define MCF_GPIO_PODR_FBCTL_PODR_FBCTL7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_FBCS */
#define MCF_GPIO_PODR_FBCS_PODR_FBCS1                (0x02)
#define MCF_GPIO_PODR_FBCS_PODR_FBCS2                (0x04)
#define MCF_GPIO_PODR_FBCS_PODR_FBCS3                (0x08)
#define MCF_GPIO_PODR_FBCS_PODR_FBCS4                (0x10)
#define MCF_GPIO_PODR_FBCS_PODR_FBCS5                (0x20)

/* Bit definitions and macros for MCF_GPIO_PODR_DMA */
#define MCF_GPIO_PODR_DMA_PODR_DMA0                  (0x01)
#define MCF_GPIO_PODR_DMA_PODR_DMA1                  (0x02)
#define MCF_GPIO_PODR_DMA_PODR_DMA2                  (0x04)
#define MCF_GPIO_PODR_DMA_PODR_DMA3                  (0x08)

/* Bit definitions and macros for MCF_GPIO_PODR_FEC0H */
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H0              (0x01)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H1              (0x02)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H2              (0x04)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H3              (0x08)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H4              (0x10)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H5              (0x20)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H6              (0x40)
#define MCF_GPIO_PODR_FEC0H_PODR_FEC0H7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_FEC0L */
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L0              (0x01)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L1              (0x02)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L2              (0x04)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L3              (0x08)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L4              (0x10)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L5              (0x20)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L6              (0x40)
#define MCF_GPIO_PODR_FEC0L_PODR_FEC0L7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_FEC1H */
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H0              (0x01)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H1              (0x02)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H2              (0x04)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H3              (0x08)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H4              (0x10)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H5              (0x20)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H6              (0x40)
#define MCF_GPIO_PODR_FEC1H_PODR_FEC1H7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_FEC1L */
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L0              (0x01)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L1              (0x02)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L2              (0x04)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L3              (0x08)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L4              (0x10)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L5              (0x20)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L6              (0x40)
#define MCF_GPIO_PODR_FEC1L_PODR_FEC1L7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_FECI2C */
#define MCF_GPIO_PODR_FECI2C_PODR_FECI2C0            (0x01)
#define MCF_GPIO_PODR_FECI2C_PODR_FECI2C1            (0x02)
#define MCF_GPIO_PODR_FECI2C_PODR_FECI2C2            (0x04)
#define MCF_GPIO_PODR_FECI2C_PODR_FECI2C3            (0x08)

/* Bit definitions and macros for MCF_GPIO_PODR_PCIBG */
#define MCF_GPIO_PODR_PCIBG_PODR_PCIBG0              (0x01)
#define MCF_GPIO_PODR_PCIBG_PODR_PCIBG1              (0x02)
#define MCF_GPIO_PODR_PCIBG_PODR_PCIBG2              (0x04)
#define MCF_GPIO_PODR_PCIBG_PODR_PCIBG3              (0x08)
#define MCF_GPIO_PODR_PCIBG_PODR_PCIBG4              (0x10)

/* Bit definitions and macros for MCF_GPIO_PODR_PCIBR */
#define MCF_GPIO_PODR_PCIBR_PODR_PCIBR0              (0x01)
#define MCF_GPIO_PODR_PCIBR_PODR_PCIBR1              (0x02)
#define MCF_GPIO_PODR_PCIBR_PODR_PCIBR2              (0x04)
#define MCF_GPIO_PODR_PCIBR_PODR_PCIBR3              (0x08)
#define MCF_GPIO_PODR_PCIBR_PODR_PCIBR4              (0x10)

/* Bit definitions and macros for MCF_GPIO_PODR_PSC3PSC2 */
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC20        (0x01)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC21        (0x02)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC22        (0x04)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC23        (0x08)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC24        (0x10)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC25        (0x20)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC26        (0x40)
#define MCF_GPIO_PODR_PSC3PSC2_PODR_PSC3PSC27        (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_PSC1PSC0 */
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC00        (0x01)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC01        (0x02)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC02        (0x04)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC03        (0x08)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC04        (0x10)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC05        (0x20)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC06        (0x40)
#define MCF_GPIO_PODR_PSC1PSC0_PODR_PSC1PSC07        (0x80)

/* Bit definitions and macros for MCF_GPIO_PODR_DSPI */
#define MCF_GPIO_PODR_DSPI_PODR_DSPI0                (0x01)
#define MCF_GPIO_PODR_DSPI_PODR_DSPI1                (0x02)
#define MCF_GPIO_PODR_DSPI_PODR_DSPI2                (0x04)
#define MCF_GPIO_PODR_DSPI_PODR_DSPI3                (0x08)
#define MCF_GPIO_PODR_DSPI_PODR_DSPI4                (0x10)
#define MCF_GPIO_PODR_DSPI_PODR_DSPI5                (0x20)
#define MCF_GPIO_PODR_DSPI_PODR_DSPI6                (0x40)

/* Bit definitions and macros for MCF_GPIO_PDDR_FBCTL */
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL0              (0x01)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL1              (0x02)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL2              (0x04)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL3              (0x08)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL4              (0x10)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL5              (0x20)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL6              (0x40)
#define MCF_GPIO_PDDR_FBCTL_PDDR_FBCTL7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_FBCS */
#define MCF_GPIO_PDDR_FBCS_PDDR_FBCS1                (0x02)
#define MCF_GPIO_PDDR_FBCS_PDDR_FBCS2                (0x04)
#define MCF_GPIO_PDDR_FBCS_PDDR_FBCS3                (0x08)
#define MCF_GPIO_PDDR_FBCS_PDDR_FBCS4                (0x10)
#define MCF_GPIO_PDDR_FBCS_PDDR_FBCS5                (0x20)

/* Bit definitions and macros for MCF_GPIO_PDDR_DMA */
#define MCF_GPIO_PDDR_DMA_PDDR_DMA0                  (0x01)
#define MCF_GPIO_PDDR_DMA_PDDR_DMA1                  (0x02)
#define MCF_GPIO_PDDR_DMA_PDDR_DMA2                  (0x04)
#define MCF_GPIO_PDDR_DMA_PDDR_DMA3                  (0x08)

/* Bit definitions and macros for MCF_GPIO_PDDR_FEC0H */
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H0              (0x01)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H1              (0x02)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H2              (0x04)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H3              (0x08)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H4              (0x10)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H5              (0x20)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H6              (0x40)
#define MCF_GPIO_PDDR_FEC0H_PDDR_FEC0H7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_FEC0L */
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L0              (0x01)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L1              (0x02)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L2              (0x04)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L3              (0x08)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L4              (0x10)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L5              (0x20)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L6              (0x40)
#define MCF_GPIO_PDDR_FEC0L_PDDR_FEC0L7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_FEC1H */
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H0              (0x01)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H1              (0x02)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H2              (0x04)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H3              (0x08)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H4              (0x10)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H5              (0x20)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H6              (0x40)
#define MCF_GPIO_PDDR_FEC1H_PDDR_FEC1H7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_FEC1L */
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L0              (0x01)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L1              (0x02)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L2              (0x04)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L3              (0x08)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L4              (0x10)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L5              (0x20)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L6              (0x40)
#define MCF_GPIO_PDDR_FEC1L_PDDR_FEC1L7              (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_FECI2C */
#define MCF_GPIO_PDDR_FECI2C_PDDR_FECI2C0            (0x01)
#define MCF_GPIO_PDDR_FECI2C_PDDR_FECI2C1            (0x02)
#define MCF_GPIO_PDDR_FECI2C_PDDR_FECI2C2            (0x04)
#define MCF_GPIO_PDDR_FECI2C_PDDR_FECI2C3            (0x08)

/* Bit definitions and macros for MCF_GPIO_PDDR_PCIBG */
#define MCF_GPIO_PDDR_PCIBG_PDDR_PCIBG0              (0x01)
#define MCF_GPIO_PDDR_PCIBG_PDDR_PCIBG1              (0x02)
#define MCF_GPIO_PDDR_PCIBG_PDDR_PCIBG2              (0x04)
#define MCF_GPIO_PDDR_PCIBG_PDDR_PCIBG3              (0x08)
#define MCF_GPIO_PDDR_PCIBG_PDDR_PCIBG4              (0x10)

/* Bit definitions and macros for MCF_GPIO_PDDR_PCIBR */
#define MCF_GPIO_PDDR_PCIBR_PDDR_PCIBR0              (0x01)
#define MCF_GPIO_PDDR_PCIBR_PDDR_PCIBR1              (0x02)
#define MCF_GPIO_PDDR_PCIBR_PDDR_PCIBR2              (0x04)
#define MCF_GPIO_PDDR_PCIBR_PDDR_PCIBR3              (0x08)
#define MCF_GPIO_PDDR_PCIBR_PDDR_PCIBR4              (0x10)

/* Bit definitions and macros for MCF_GPIO_PDDR_PSC3PSC2 */
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC20        (0x01)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC21        (0x02)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC22        (0x04)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC23        (0x08)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC24        (0x10)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC25        (0x20)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC26        (0x40)
#define MCF_GPIO_PDDR_PSC3PSC2_PDDR_PSC3PSC27        (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_PSC1PSC0 */
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC00        (0x01)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC01        (0x02)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC02        (0x04)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC03        (0x08)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC04        (0x10)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC05        (0x20)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC06        (0x40)
#define MCF_GPIO_PDDR_PSC1PSC0_PDDR_PSC1PSC07        (0x80)

/* Bit definitions and macros for MCF_GPIO_PDDR_DSPI */
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI0                (0x01)
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI1                (0x02)
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI2                (0x04)
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI3                (0x08)
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI4                (0x10)
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI5                (0x20)
#define MCF_GPIO_PDDR_DSPI_PDDR_DSPI6                (0x40)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FBCTL */
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL0          (0x01)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL1          (0x02)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL2          (0x04)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL3          (0x08)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL4          (0x10)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL5          (0x20)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL6          (0x40)
#define MCF_GPIO_PPDSDR_FBCTL_PPDSDR_FBCTL7          (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FBCS */
#define MCF_GPIO_PPDSDR_FBCS_PPDSDR_FBCS1            (0x02)
#define MCF_GPIO_PPDSDR_FBCS_PPDSDR_FBCS2            (0x04)
#define MCF_GPIO_PPDSDR_FBCS_PPDSDR_FBCS3            (0x08)
#define MCF_GPIO_PPDSDR_FBCS_PPDSDR_FBCS4            (0x10)
#define MCF_GPIO_PPDSDR_FBCS_PPDSDR_FBCS5            (0x20)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_DMA */
#define MCF_GPIO_PPDSDR_DMA_PPDSDR_DMA0              (0x01)
#define MCF_GPIO_PPDSDR_DMA_PPDSDR_DMA1              (0x02)
#define MCF_GPIO_PPDSDR_DMA_PPDSDR_DMA2              (0x04)
#define MCF_GPIO_PPDSDR_DMA_PPDSDR_DMA3              (0x08)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FEC0H */
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H0          (0x01)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H1          (0x02)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H2          (0x04)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H3          (0x08)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H4          (0x10)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H5          (0x20)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H6          (0x40)
#define MCF_GPIO_PPDSDR_FEC0H_PPDSDR_FEC0H7          (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FEC0L */
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L0          (0x01)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L1          (0x02)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L2          (0x04)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L3          (0x08)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L4          (0x10)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L5          (0x20)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L6          (0x40)
#define MCF_GPIO_PPDSDR_FEC0L_PPDSDR_FEC0L7          (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FEC1H */
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H0          (0x01)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H1          (0x02)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H2          (0x04)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H3          (0x08)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H4          (0x10)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H5          (0x20)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H6          (0x40)
#define MCF_GPIO_PPDSDR_FEC1H_PPDSDR_FEC1H7          (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FEC1L */
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L0          (0x01)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L1          (0x02)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L2          (0x04)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L3          (0x08)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L4          (0x10)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L5          (0x20)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L6          (0x40)
#define MCF_GPIO_PPDSDR_FEC1L_PPDSDR_FEC1L7          (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_FECI2C */
#define MCF_GPIO_PPDSDR_FECI2C_PPDSDR_FECI2C0        (0x01)
#define MCF_GPIO_PPDSDR_FECI2C_PPDSDR_FECI2C1        (0x02)
#define MCF_GPIO_PPDSDR_FECI2C_PPDSDR_FECI2C2        (0x04)
#define MCF_GPIO_PPDSDR_FECI2C_PPDSDR_FECI2C3        (0x08)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_PCIBG */
#define MCF_GPIO_PPDSDR_PCIBG_PPDSDR_PCIBG0          (0x01)
#define MCF_GPIO_PPDSDR_PCIBG_PPDSDR_PCIBG1          (0x02)
#define MCF_GPIO_PPDSDR_PCIBG_PPDSDR_PCIBG2          (0x04)
#define MCF_GPIO_PPDSDR_PCIBG_PPDSDR_PCIBG3          (0x08)
#define MCF_GPIO_PPDSDR_PCIBG_PPDSDR_PCIBG4          (0x10)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_PCIBR */
#define MCF_GPIO_PPDSDR_PCIBR_PPDSDR_PCIBR0          (0x01)
#define MCF_GPIO_PPDSDR_PCIBR_PPDSDR_PCIBR1          (0x02)
#define MCF_GPIO_PPDSDR_PCIBR_PPDSDR_PCIBR2          (0x04)
#define MCF_GPIO_PPDSDR_PCIBR_PPDSDR_PCIBR3          (0x08)
#define MCF_GPIO_PPDSDR_PCIBR_PPDSDR_PCIBR4          (0x10)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_PSC3PSC2 */
#define MCF_GPIO_PPDSDR_PSC3PSC2_PPDSDR_PSC3PSC20    (0x01)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PPDSDR_PSC3PSC21    (0x02)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PPDSDR_PSC3PSC22    (0x04)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PPDSDR_PSC3PSC23    (0x08)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PDDR_PSC3PSC24      (0x10)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PDDR_PSC3PSC25      (0x20)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PPDSDR_PSC3PSC26    (0x40)
#define MCF_GPIO_PPDSDR_PSC3PSC2_PPDSDR_PSC3PSC27    (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_PSC1PSC0 */
#define MCF_GPIO_PPDSDR_PSC1PSC0_PPDSDR_PSC1PSC00    (0x01)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PDDR_PSC1PSC01      (0x02)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PPDSDR_PSC1PSC02    (0x04)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PDDR_PSC1PSC03      (0x08)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PPDSDR_PSC1PSC04    (0x10)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PPDSDR_PSC1PSC05    (0x20)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PPDSDR_PSC1PSC06    (0x40)
#define MCF_GPIO_PPDSDR_PSC1PSC0_PPDSDR_PSC1PSC07    (0x80)

/* Bit definitions and macros for MCF_GPIO_PPDSDR_DSPI */
#define MCF_GPIO_PPDSDR_DSPI_PPDSDR_DSPI0            (0x01)
#define MCF_GPIO_PPDSDR_DSPI_PPDSDR_DSPI1            (0x02)
#define MCF_GPIO_PPDSDR_DSPI_PPDSDR_DSPI2            (0x04)
#define MCF_GPIO_PPDSDR_DSPI_PPDSDR_DSPI3            (0x08)
#define MCF_GPIO_PPDSDR_DSPI_PDDR_DSPI4              (0x10)
#define MCF_GPIO_PPDSDR_DSPI_PPDSDR_DSPI5            (0x20)
#define MCF_GPIO_PPDSDR_DSPI_PPDSDR_DSPI6            (0x40)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FBCTL */
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL0            (0x01)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL1            (0x02)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL2            (0x04)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL3            (0x08)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL4            (0x10)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL5            (0x20)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL6            (0x40)
#define MCF_GPIO_PCLRR_FBCTL_PCLRR_FBCTL7            (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FBCS */
#define MCF_GPIO_PCLRR_FBCS_PCLRR_FBCS1              (0x02)
#define MCF_GPIO_PCLRR_FBCS_PCLRR_FBCS2              (0x04)
#define MCF_GPIO_PCLRR_FBCS_PCLRR_FBCS3              (0x08)
#define MCF_GPIO_PCLRR_FBCS_PCLRR_FBCS4              (0x10)
#define MCF_GPIO_PCLRR_FBCS_PCLRR_FBCS5              (0x20)

/* Bit definitions and macros for MCF_GPIO_PCLRR_DMA */
#define MCF_GPIO_PCLRR_DMA_PCLRR_DMA0                (0x01)
#define MCF_GPIO_PCLRR_DMA_PCLRR_DMA1                (0x02)
#define MCF_GPIO_PCLRR_DMA_PCLRR_DMA2                (0x04)
#define MCF_GPIO_PCLRR_DMA_PCLRR_DMA3                (0x08)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FEC0H */
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H0            (0x01)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H1            (0x02)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H2            (0x04)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H3            (0x08)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H4            (0x10)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H5            (0x20)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H6            (0x40)
#define MCF_GPIO_PCLRR_FEC0H_PCLRR_FEC0H7            (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FEC0L */
#define MCF_GPIO_PCLRR_FEC0L_PCLRR_FEC0L0            (0x01)
#define MCF_GPIO_PCLRR_FEC0L_PODR_FEC0L1             (0x02)
#define MCF_GPIO_PCLRR_FEC0L_PCLRR_FEC0L2            (0x04)
#define MCF_GPIO_PCLRR_FEC0L_PCLRR_FEC0L3            (0x08)
#define MCF_GPIO_PCLRR_FEC0L_PODR_FEC0L4             (0x10)
#define MCF_GPIO_PCLRR_FEC0L_PODR_FEC0L5             (0x20)
#define MCF_GPIO_PCLRR_FEC0L_PODR_FEC0L6             (0x40)
#define MCF_GPIO_PCLRR_FEC0L_PCLRR_FEC0L7            (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FEC1H */
#define MCF_GPIO_PCLRR_FEC1H_PCLRR_FEC1H0            (0x01)
#define MCF_GPIO_PCLRR_FEC1H_PCLRR_FEC1H1            (0x02)
#define MCF_GPIO_PCLRR_FEC1H_PCLRR_FEC1H2            (0x04)
#define MCF_GPIO_PCLRR_FEC1H_PODR_FEC1H3             (0x08)
#define MCF_GPIO_PCLRR_FEC1H_PODR_FEC1H4             (0x10)
#define MCF_GPIO_PCLRR_FEC1H_PCLRR_FEC1H5            (0x20)
#define MCF_GPIO_PCLRR_FEC1H_PCLRR_FEC1H6            (0x40)
#define MCF_GPIO_PCLRR_FEC1H_PCLRR_FEC1H7            (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FEC1L */
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L0            (0x01)
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L1            (0x02)
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L2            (0x04)
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L3            (0x08)
#define MCF_GPIO_PCLRR_FEC1L_PODR_FEC1L4             (0x10)
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L5            (0x20)
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L6            (0x40)
#define MCF_GPIO_PCLRR_FEC1L_PCLRR_FEC1L7            (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_FECI2C */
#define MCF_GPIO_PCLRR_FECI2C_PCLRR_FECI2C0          (0x01)
#define MCF_GPIO_PCLRR_FECI2C_PCLRR_FECI2C1          (0x02)
#define MCF_GPIO_PCLRR_FECI2C_PODR_FECI2C2           (0x04)
#define MCF_GPIO_PCLRR_FECI2C_PCLRR_FECI2C3          (0x08)

/* Bit definitions and macros for MCF_GPIO_PCLRR_PCIBG */
#define MCF_GPIO_PCLRR_PCIBG_PODR_PCIBG0             (0x01)
#define MCF_GPIO_PCLRR_PCIBG_PODR_PCIBG1             (0x02)
#define MCF_GPIO_PCLRR_PCIBG_PODR_PCIBG2             (0x04)
#define MCF_GPIO_PCLRR_PCIBG_PCLRR_PCIBG3            (0x08)
#define MCF_GPIO_PCLRR_PCIBG_PCLRR_PCIBG4            (0x10)

/* Bit definitions and macros for MCF_GPIO_PCLRR_PCIBR */
#define MCF_GPIO_PCLRR_PCIBR_PCLRR_PCIBR0            (0x01)
#define MCF_GPIO_PCLRR_PCIBR_PCLRR_PCIBR1            (0x02)
#define MCF_GPIO_PCLRR_PCIBR_PCLRR_PCIBR2            (0x04)
#define MCF_GPIO_PCLRR_PCIBR_PODR_PCIBR3             (0x08)
#define MCF_GPIO_PCLRR_PCIBR_PODR_PCIBR4             (0x10)

/* Bit definitions and macros for MCF_GPIO_PCLRR_PSC3PSC2 */
#define MCF_GPIO_PCLRR_PSC3PSC2_PODR_PSC3PSC20       (0x01)
#define MCF_GPIO_PCLRR_PSC3PSC2_PODR_PSC3PSC21       (0x02)
#define MCF_GPIO_PCLRR_PSC3PSC2_PCLRR_PSC3PSC22      (0x04)
#define MCF_GPIO_PCLRR_PSC3PSC2_PCLRR_PSC3PSC23      (0x08)
#define MCF_GPIO_PCLRR_PSC3PSC2_PCLRR_PSC3PSC24      (0x10)
#define MCF_GPIO_PCLRR_PSC3PSC2_PODR_PSC3PSC25       (0x20)
#define MCF_GPIO_PCLRR_PSC3PSC2_PODR_PSC3PSC26       (0x40)
#define MCF_GPIO_PCLRR_PSC3PSC2_PCLRR_PSC3PSC27      (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_PSC1PSC0 */
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC00      (0x01)
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC01      (0x02)
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC02      (0x04)
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC03      (0x08)
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC04      (0x10)
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC05      (0x20)
#define MCF_GPIO_PCLRR_PSC1PSC0_PODR_PSC1PSC06       (0x40)
#define MCF_GPIO_PCLRR_PSC1PSC0_PCLRR_PSC1PSC07      (0x80)

/* Bit definitions and macros for MCF_GPIO_PCLRR_DSPI */
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI0              (0x01)
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI1              (0x02)
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI2              (0x04)
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI3              (0x08)
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI4              (0x10)
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI5              (0x20)
#define MCF_GPIO_PCLRR_DSPI_PCLRR_DSPI6              (0x40)

/* Bit definitions and macros for MCF_GPIO_PAR_FBCTL */
#define MCF_GPIO_PAR_FBCTL_PAR_TS(x)                 (((x)&0x0003)<<0)
#define MCF_GPIO_PAR_FBCTL_PAR_TA                    (0x0004)
#define MCF_GPIO_PAR_FBCTL_PAR_RWB(x)                (((x)&0x0003)<<4)
#define MCF_GPIO_PAR_FBCTL_PAR_OE                    (0x0040)
#define MCF_GPIO_PAR_FBCTL_PAR_BWE0                  (0x0100)
#define MCF_GPIO_PAR_FBCTL_PAR_BWE1                  (0x0400)
#define MCF_GPIO_PAR_FBCTL_PAR_BWE2                  (0x1000)
#define MCF_GPIO_PAR_FBCTL_PAR_BWE3                  (0x4000)
#define MCF_GPIO_PAR_FBCTL_PAR_TS_GPIO               (0)
#define MCF_GPIO_PAR_FBCTL_PAR_TS_TBST               (2)
#define MCF_GPIO_PAR_FBCTL_PAR_TS_TS                 (3)
#define MCF_GPIO_PAR_FBCTL_PAR_RWB_GPIO              (0x0000)
#define MCF_GPIO_PAR_FBCTL_PAR_RWB_TBST              (0x0020)
#define MCF_GPIO_PAR_FBCTL_PAR_RWB_RWB               (0x0030)

/* Bit definitions and macros for MCF_GPIO_PAR_FBCS */
#define MCF_GPIO_PAR_FBCS_PAR_CS1                    (0x02)
#define MCF_GPIO_PAR_FBCS_PAR_CS2                    (0x04)
#define MCF_GPIO_PAR_FBCS_PAR_CS3                    (0x08)
#define MCF_GPIO_PAR_FBCS_PAR_CS4                    (0x10)
#define MCF_GPIO_PAR_FBCS_PAR_CS5                    (0x20)

/* Bit definitions and macros for MCF_GPIO_PAR_DMA */
#define MCF_GPIO_PAR_DMA_PAR_DREQ0(x)                (((x)&0x03)<<0)
#define MCF_GPIO_PAR_DMA_PAR_DREQ1(x)                (((x)&0x03)<<2)
#define MCF_GPIO_PAR_DMA_PAR_DACK0(x)                (((x)&0x03)<<4)
#define MCF_GPIO_PAR_DMA_PAR_DACK1(x)                (((x)&0x03)<<6)
#define MCF_GPIO_PAR_DMA_PAR_DACKx_GPIO              (0)
#define MCF_GPIO_PAR_DMA_PAR_DACKx_TOUT              (2)
#define MCF_GPIO_PAR_DMA_PAR_DACKx_DACK              (3)
#define MCF_GPIO_PAR_DMA_PAR_DREQx_GPIO              (0)
#define MCF_GPIO_PAR_DMA_PAR_DREQx_TIN               (2)
#define MCF_GPIO_PAR_DMA_PAR_DREQx_DREQ              (3)

/* Bit definitions and macros for MCF_GPIO_PAR_FECI2CIRQ */
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_IRQ5              (0x0001)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_IRQ6              (0x0002)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_SCL               (0x0004)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_SDA               (0x0008)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDC(x)          (((x)&0x0003)<<6)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDIO(x)         (((x)&0x0003)<<8)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MII             (0x0400)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E17               (0x0800)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E0MDC             (0x1000)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E0MDIO            (0x2000)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E0MII             (0x4000)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E07               (0x8000)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDIO_CANRX      (0x0000)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDIO_SDA        (0x0200)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDIO_EMDIO      (0x0300)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDC_CANTX       (0x0000)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDC_SCL         (0x0080)
#define MCF_GPIO_PAR_FECI2CIRQ_PAR_E1MDC_EMDC        (0x00C0)

/* Bit definitions and macros for MCF_GPIO_PAR_PCIBG */
#define MCF_GPIO_PAR_PCIBG_PAR_PCIBG0(x)             (((x)&0x0003)<<0)
#define MCF_GPIO_PAR_PCIBG_PAR_PCIBG1(x)             (((x)&0x0003)<<2)
#define MCF_GPIO_PAR_PCIBG_PAR_PCIBG2(x)             (((x)&0x0003)<<4)
#define MCF_GPIO_PAR_PCIBG_PAR_PCIBG3(x)             (((x)&0x0003)<<6)
#define MCF_GPIO_PAR_PCIBG_PAR_PCIBG4(x)             (((x)&0x0003)<<8)

/* Bit definitions and macros for MCF_GPIO_PAR_PCIBR */
#define MCF_GPIO_PAR_PCIBR_PAR_PCIBG0(x)             (((x)&0x0003)<<0)
#define MCF_GPIO_PAR_PCIBR_PAR_PCIBG1(x)             (((x)&0x0003)<<2)
#define MCF_GPIO_PAR_PCIBR_PAR_PCIBG2(x)             (((x)&0x0003)<<4)
#define MCF_GPIO_PAR_PCIBR_PAR_PCIBG3(x)             (((x)&0x0003)<<6)
#define MCF_GPIO_PAR_PCIBR_PAR_PCIBR4(x)             (((x)&0x0003)<<8)

/* Bit definitions and macros for MCF_GPIO_PAR_PSC3 */
#define MCF_GPIO_PAR_PSC3_PAR_TXD3                   (0x04)
#define MCF_GPIO_PAR_PSC3_PAR_RXD3                   (0x08)
#define MCF_GPIO_PAR_PSC3_PAR_RTS3(x)                (((x)&0x03)<<4)
#define MCF_GPIO_PAR_PSC3_PAR_CTS3(x)                (((x)&0x03)<<6)
#define MCF_GPIO_PAR_PSC3_PAR_CTS3_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC3_PAR_CTS3_BCLK              (0x80)
#define MCF_GPIO_PAR_PSC3_PAR_CTS3_CTS               (0xC0)
#define MCF_GPIO_PAR_PSC3_PAR_RTS3_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC3_PAR_RTS3_FSYNC             (0x20)
#define MCF_GPIO_PAR_PSC3_PAR_RTS3_RTS               (0x30)
#define MCF_GPIO_PAR_PSC3_PAR_CTS2_CANRX             (0x40)

/* Bit definitions and macros for MCF_GPIO_PAR_PSC2 */
#define MCF_GPIO_PAR_PSC2_PAR_TXD2                   (0x04)
#define MCF_GPIO_PAR_PSC2_PAR_RXD2                   (0x08)
#define MCF_GPIO_PAR_PSC2_PAR_RTS2(x)                (((x)&0x03)<<4)
#define MCF_GPIO_PAR_PSC2_PAR_CTS2(x)                (((x)&0x03)<<6)
#define MCF_GPIO_PAR_PSC2_PAR_CTS2_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC2_PAR_CTS2_BCLK              (0x80)
#define MCF_GPIO_PAR_PSC2_PAR_CTS2_CTS               (0xC0)
#define MCF_GPIO_PAR_PSC2_PAR_RTS2_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC2_PAR_RTS2_CANTX             (0x10)
#define MCF_GPIO_PAR_PSC2_PAR_RTS2_FSYNC             (0x20)
#define MCF_GPIO_PAR_PSC2_PAR_RTS2_RTS               (0x30)

/* Bit definitions and macros for MCF_GPIO_PAR_PSC1 */
#define MCF_GPIO_PAR_PSC1_PAR_TXD1                   (0x04)
#define MCF_GPIO_PAR_PSC1_PAR_RXD1                   (0x08)
#define MCF_GPIO_PAR_PSC1_PAR_RTS1(x)                (((x)&0x03)<<4)
#define MCF_GPIO_PAR_PSC1_PAR_CTS1(x)                (((x)&0x03)<<6)
#define MCF_GPIO_PAR_PSC1_PAR_CTS1_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC1_PAR_CTS1_BCLK              (0x80)
#define MCF_GPIO_PAR_PSC1_PAR_CTS1_CTS               (0xC0)
#define MCF_GPIO_PAR_PSC1_PAR_RTS1_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC1_PAR_RTS1_FSYNC             (0x20)
#define MCF_GPIO_PAR_PSC1_PAR_RTS1_RTS               (0x30)

/* Bit definitions and macros for MCF_GPIO_PAR_PSC0 */
#define MCF_GPIO_PAR_PSC0_PAR_TXD0                   (0x04)
#define MCF_GPIO_PAR_PSC0_PAR_RXD0                   (0x08)
#define MCF_GPIO_PAR_PSC0_PAR_RTS0(x)                (((x)&0x03)<<4)
#define MCF_GPIO_PAR_PSC0_PAR_CTS0(x)                (((x)&0x03)<<6)
#define MCF_GPIO_PAR_PSC0_PAR_CTS0_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC0_PAR_CTS0_BCLK              (0x80)
#define MCF_GPIO_PAR_PSC0_PAR_CTS0_CTS               (0xC0)
#define MCF_GPIO_PAR_PSC0_PAR_RTS0_GPIO              (0x00)
#define MCF_GPIO_PAR_PSC0_PAR_RTS0_FSYNC             (0x20)
#define MCF_GPIO_PAR_PSC0_PAR_RTS0_RTS               (0x30)

/* Bit definitions and macros for MCF_GPIO_PAR_DSPI */
#define MCF_GPIO_PAR_DSPI_PAR_SOUT(x)                (((x)&0x0003)<<0)
#define MCF_GPIO_PAR_DSPI_PAR_SIN(x)                 (((x)&0x0003)<<2)
#define MCF_GPIO_PAR_DSPI_PAR_SCK(x)                 (((x)&0x0003)<<4)
#define MCF_GPIO_PAR_DSPI_PAR_CS0(x)                 (((x)&0x0003)<<6)
#define MCF_GPIO_PAR_DSPI_PAR_CS2(x)                 (((x)&0x0003)<<8)
#define MCF_GPIO_PAR_DSPI_PAR_CS3(x)                 (((x)&0x0003)<<10)
#define MCF_GPIO_PAR_DSPI_PAR_CS5                    (0x1000)
#define MCF_GPIO_PAR_DSPI_PAR_CS3_GPIO               (0x0000)
#define MCF_GPIO_PAR_DSPI_PAR_CS3_CANTX              (0x0400)
#define MCF_GPIO_PAR_DSPI_PAR_CS3_TOUT               (0x0800)
#define MCF_GPIO_PAR_DSPI_PAR_CS3_DSPICS             (0x0C00)
#define MCF_GPIO_PAR_DSPI_PAR_CS2_GPIO               (0x0000)
#define MCF_GPIO_PAR_DSPI_PAR_CS2_CANTX              (0x0100)
#define MCF_GPIO_PAR_DSPI_PAR_CS2_TOUT               (0x0200)
#define MCF_GPIO_PAR_DSPI_PAR_CS2_DSPICS             (0x0300)
#define MCF_GPIO_PAR_DSPI_PAR_CS0_GPIO               (0x0000)
#define MCF_GPIO_PAR_DSPI_PAR_CS0_FSYNC              (0x0040)
#define MCF_GPIO_PAR_DSPI_PAR_CS0_RTS                (0x0080)
#define MCF_GPIO_PAR_DSPI_PAR_CS0_DSPICS             (0x00C0)
#define MCF_GPIO_PAR_DSPI_PAR_SCK_GPIO               (0x0000)
#define MCF_GPIO_PAR_DSPI_PAR_SCK_BCLK               (0x0010)
#define MCF_GPIO_PAR_DSPI_PAR_SCK_CTS                (0x0020)
#define MCF_GPIO_PAR_DSPI_PAR_SCK_SCK                (0x0030)
#define MCF_GPIO_PAR_DSPI_PAR_SIN_GPIO               (0x0000)
#define MCF_GPIO_PAR_DSPI_PAR_SIN_RXD                (0x0008)
#define MCF_GPIO_PAR_DSPI_PAR_SIN_SIN                (0x000C)
#define MCF_GPIO_PAR_DSPI_PAR_SOUT_GPIO              (0x0000)
#define MCF_GPIO_PAR_DSPI_PAR_SOUT_TXD               (0x0002)
#define MCF_GPIO_PAR_DSPI_PAR_SOUT_SOUT              (0x0003)

/* Bit definitions and macros for MCF_GPIO_PAR_TIMER */
#define MCF_GPIO_PAR_TIMER_PAR_TOUT2                 (0x01)
#define MCF_GPIO_PAR_TIMER_PAR_TIN2(x)               (((x)&0x03)<<1)
#define MCF_GPIO_PAR_TIMER_PAR_TOUT3                 (0x08)
#define MCF_GPIO_PAR_TIMER_PAR_TIN3(x)               (((x)&0x03)<<4)
#define MCF_GPIO_PAR_TIMER_PAR_TIN3_CANRX            (0x00)
#define MCF_GPIO_PAR_TIMER_PAR_TIN3_IRQ              (0x20)
#define MCF_GPIO_PAR_TIMER_PAR_TIN3_TIN              (0x30)
#define MCF_GPIO_PAR_TIMER_PAR_TIN2_CANRX            (0x00)
#define MCF_GPIO_PAR_TIMER_PAR_TIN2_IRQ              (0x04)
#define MCF_GPIO_PAR_TIMER_PAR_TIN2_TIN              (0x06)

/****************************************************************************
 *
 * General Purpose Timer (GPT) Defines
 *
 */

/* Register read/write macros */
#define MCF_GPT_GMS0       (*(vuint32*)(void*)(&__MBAR[0x000800]))
#define MCF_GPT_GCIR0      (*(vuint32*)(void*)(&__MBAR[0x000804]))
#define MCF_GPT_GPWM0      (*(vuint32*)(void*)(&__MBAR[0x000808]))
#define MCF_GPT_GSR0       (*(vuint32*)(void*)(&__MBAR[0x00080C]))
#define MCF_GPT_GMS1       (*(vuint32*)(void*)(&__MBAR[0x000810]))
#define MCF_GPT_GCIR1      (*(vuint32*)(void*)(&__MBAR[0x000814]))
#define MCF_GPT_GPWM1      (*(vuint32*)(void*)(&__MBAR[0x000818]))
#define MCF_GPT_GSR1       (*(vuint32*)(void*)(&__MBAR[0x00081C]))
#define MCF_GPT_GMS2       (*(vuint32*)(void*)(&__MBAR[0x000820]))
#define MCF_GPT_GCIR2      (*(vuint32*)(void*)(&__MBAR[0x000824]))
#define MCF_GPT_GPWM2      (*(vuint32*)(void*)(&__MBAR[0x000828]))
#define MCF_GPT_GSR2       (*(vuint32*)(void*)(&__MBAR[0x00082C]))
#define MCF_GPT_GMS3       (*(vuint32*)(void*)(&__MBAR[0x000830]))
#define MCF_GPT_GCIR3      (*(vuint32*)(void*)(&__MBAR[0x000834]))
#define MCF_GPT_GPWM3      (*(vuint32*)(void*)(&__MBAR[0x000838]))
#define MCF_GPT_GSR3       (*(vuint32*)(void*)(&__MBAR[0x00083C]))
#define MCF_GPT_GMS(x)     (*(vuint32*)(void*)(&__MBAR[0x000800+((x)*0x010)]))
#define MCF_GPT_GCIR(x)    (*(vuint32*)(void*)(&__MBAR[0x000804+((x)*0x010)]))
#define MCF_GPT_GPWM(x)    (*(vuint32*)(void*)(&__MBAR[0x000808+((x)*0x010)]))
#define MCF_GPT_GSR(x)     (*(vuint32*)(void*)(&__MBAR[0x00080C+((x)*0x010)]))

/* Bit definitions and macros for MCF_GPT_GMS */
#define MCF_GPT_GMS_TMS(x)         (((x)&0x00000007UL)<<0)
#define MCF_GPT_GMS_GPIO(x)        (((x)&0x00000003UL)<<4)
#define MCF_GPT_GMS_IEN            (0x00000100UL)
#define MCF_GPT_GMS_OD             (0x00000200UL)
#define MCF_GPT_GMS_SC             (0x00000400UL)
#define MCF_GPT_GMS_CE             (0x00001000UL)
#define MCF_GPT_GMS_WDEN           (0x00008000UL)
#define MCF_GPT_GMS_ICT(x)         (((x)&0x00000003UL)<<16)
#define MCF_GPT_GMS_OCT(x)         (((x)&0x00000003UL)<<20)
#define MCF_GPT_GMS_OCPW(x)        (((x)&0x000000FFUL)<<24)
#define MCF_GPT_GMS_OCT_FRCLOW     (0x00000000UL)
#define MCF_GPT_GMS_OCT_PULSEHI    (0x00100000UL)
#define MCF_GPT_GMS_OCT_PULSELO    (0x00200000UL)
#define MCF_GPT_GMS_OCT_TOGGLE     (0x00300000UL)
#define MCF_GPT_GMS_ICT_ANY        (0x00000000UL)
#define MCF_GPT_GMS_ICT_RISE       (0x00010000UL)
#define MCF_GPT_GMS_ICT_FALL       (0x00020000UL)
#define MCF_GPT_GMS_ICT_PULSE      (0x00030000UL)
#define MCF_GPT_GMS_GPIO_INPUT     (0x00000000UL)
#define MCF_GPT_GMS_GPIO_OUTLO     (0x00000020UL)
#define MCF_GPT_GMS_GPIO_OUTHI     (0x00000030UL)
#define MCF_GPT_GMS_TMS_DISABLE    (0x00000000UL)
#define MCF_GPT_GMS_TMS_INCAPT     (0x00000001UL)
#define MCF_GPT_GMS_TMS_OUTCAPT    (0x00000002UL)
#define MCF_GPT_GMS_TMS_PWM        (0x00000003UL)
#define MCF_GPT_GMS_TMS_GPIO       (0x00000004UL)

/* Bit definitions and macros for MCF_GPT_GCIR */
#define MCF_GPT_GCIR_CNT(x)        (((x)&0x0000FFFFUL)<<0)
#define MCF_GPT_GCIR_PRE(x)        (((x)&0x0000FFFFUL)<<16)

/* Bit definitions and macros for MCF_GPT_GPWM */
#define MCF_GPT_GPWM_LOAD          (0x00000001UL)
#define MCF_GPT_GPWM_PWMOP         (0x00000100UL)
#define MCF_GPT_GPWM_WIDTH(x)      (((x)&0x0000FFFFUL)<<16)

/* Bit definitions and macros for MCF_GPT_GSR */
#define MCF_GPT_GSR_CAPT           (0x00000001UL)
#define MCF_GPT_GSR_COMP           (0x00000002UL)
#define MCF_GPT_GSR_PWMP           (0x00000004UL)
#define MCF_GPT_GSR_TEXP           (0x00000008UL)
#define MCF_GPT_GSR_PIN            (0x00000100UL)
#define MCF_GPT_GSR_OVF(x)         (((x)&0x00000007UL)<<12)
#define MCF_GPT_GSR_CAPTURE(x)     (((x)&0x0000FFFFUL)<<16)

/****************************************************************************
 *
 * Interrupt Controller Defines
 *
 */
 
/* Register read/write macros */
#define MCF_INTC_IPRH         (*(vuint32*)(void*)(&__MBAR[0x000700]))
#define MCF_INTC_IPRL         (*(vuint32*)(void*)(&__MBAR[0x000704]))
#define MCF_INTC_IMRH         (*(vuint32*)(void*)(&__MBAR[0x000708]))
#define MCF_INTC_IMRL         (*(vuint32*)(void*)(&__MBAR[0x00070C]))
#define MCF_INTC_INTFRCH      (*(vuint32*)(void*)(&__MBAR[0x000710]))
#define MCF_INTC_INTFRCL      (*(vuint32*)(void*)(&__MBAR[0x000714]))
#define MCF_INTC_IRLR         (*(vuint8 *)(void*)(&__MBAR[0x000718]))
#define MCF_INTC_IACKLPR      (*(vuint8 *)(void*)(&__MBAR[0x000719]))
#define MCF_INTC_ICR0         (*(vuint8 *)(void*)(&__MBAR[0x000740]))
#define MCF_INTC_ICR1         (*(vuint8 *)(void*)(&__MBAR[0x000741]))
#define MCF_INTC_ICR2         (*(vuint8 *)(void*)(&__MBAR[0x000742]))
#define MCF_INTC_ICR3         (*(vuint8 *)(void*)(&__MBAR[0x000743]))
#define MCF_INTC_ICR4         (*(vuint8 *)(void*)(&__MBAR[0x000744]))
#define MCF_INTC_ICR5         (*(vuint8 *)(void*)(&__MBAR[0x000745]))
#define MCF_INTC_ICR6         (*(vuint8 *)(void*)(&__MBAR[0x000746]))
#define MCF_INTC_ICR7         (*(vuint8 *)(void*)(&__MBAR[0x000747]))
#define MCF_INTC_ICR8         (*(vuint8 *)(void*)(&__MBAR[0x000748]))
#define MCF_INTC_ICR9         (*(vuint8 *)(void*)(&__MBAR[0x000749]))
#define MCF_INTC_ICR10        (*(vuint8 *)(void*)(&__MBAR[0x00074A]))
#define MCF_INTC_ICR11        (*(vuint8 *)(void*)(&__MBAR[0x00074B]))
#define MCF_INTC_ICR12        (*(vuint8 *)(void*)(&__MBAR[0x00074C]))
#define MCF_INTC_ICR13        (*(vuint8 *)(void*)(&__MBAR[0x00074D]))
#define MCF_INTC_ICR14        (*(vuint8 *)(void*)(&__MBAR[0x00074E]))
#define MCF_INTC_ICR15        (*(vuint8 *)(void*)(&__MBAR[0x00074F]))
#define MCF_INTC_ICR16        (*(vuint8 *)(void*)(&__MBAR[0x000750]))
#define MCF_INTC_ICR17        (*(vuint8 *)(void*)(&__MBAR[0x000751]))
#define MCF_INTC_ICR18        (*(vuint8 *)(void*)(&__MBAR[0x000752]))
#define MCF_INTC_ICR19        (*(vuint8 *)(void*)(&__MBAR[0x000753]))
#define MCF_INTC_ICR20        (*(vuint8 *)(void*)(&__MBAR[0x000754]))
#define MCF_INTC_ICR21        (*(vuint8 *)(void*)(&__MBAR[0x000755]))
#define MCF_INTC_ICR22        (*(vuint8 *)(void*)(&__MBAR[0x000756]))
#define MCF_INTC_ICR23        (*(vuint8 *)(void*)(&__MBAR[0x000757]))
#define MCF_INTC_ICR24        (*(vuint8 *)(void*)(&__MBAR[0x000758]))
#define MCF_INTC_ICR25        (*(vuint8 *)(void*)(&__MBAR[0x000759]))
#define MCF_INTC_ICR26        (*(vuint8 *)(void*)(&__MBAR[0x00075A]))
#define MCF_INTC_ICR27        (*(vuint8 *)(void*)(&__MBAR[0x00075B]))
#define MCF_INTC_ICR28        (*(vuint8 *)(void*)(&__MBAR[0x00075C]))
#define MCF_INTC_ICR29        (*(vuint8 *)(void*)(&__MBAR[0x00075D]))
#define MCF_INTC_ICR30        (*(vuint8 *)(void*)(&__MBAR[0x00075E]))
#define MCF_INTC_ICR31        (*(vuint8 *)(void*)(&__MBAR[0x00075F]))
#define MCF_INTC_ICR32        (*(vuint8 *)(void*)(&__MBAR[0x000760]))
#define MCF_INTC_ICR33        (*(vuint8 *)(void*)(&__MBAR[0x000761]))
#define MCF_INTC_ICR34        (*(vuint8 *)(void*)(&__MBAR[0x000762]))
#define MCF_INTC_ICR35        (*(vuint8 *)(void*)(&__MBAR[0x000763]))
#define MCF_INTC_ICR36        (*(vuint8 *)(void*)(&__MBAR[0x000764]))
#define MCF_INTC_ICR37        (*(vuint8 *)(void*)(&__MBAR[0x000765]))
#define MCF_INTC_ICR38        (*(vuint8 *)(void*)(&__MBAR[0x000766]))
#define MCF_INTC_ICR39        (*(vuint8 *)(void*)(&__MBAR[0x000767]))
#define MCF_INTC_ICR40        (*(vuint8 *)(void*)(&__MBAR[0x000768]))
#define MCF_INTC_ICR41        (*(vuint8 *)(void*)(&__MBAR[0x000769]))
#define MCF_INTC_ICR42        (*(vuint8 *)(void*)(&__MBAR[0x00076A]))
#define MCF_INTC_ICR43        (*(vuint8 *)(void*)(&__MBAR[0x00076B]))
#define MCF_INTC_ICR44        (*(vuint8 *)(void*)(&__MBAR[0x00076C]))
#define MCF_INTC_ICR45        (*(vuint8 *)(void*)(&__MBAR[0x00076D]))
#define MCF_INTC_ICR46        (*(vuint8 *)(void*)(&__MBAR[0x00076E]))
#define MCF_INTC_ICR47        (*(vuint8 *)(void*)(&__MBAR[0x00076F]))
#define MCF_INTC_ICR48        (*(vuint8 *)(void*)(&__MBAR[0x000770]))
#define MCF_INTC_ICR49        (*(vuint8 *)(void*)(&__MBAR[0x000771]))
#define MCF_INTC_ICR50        (*(vuint8 *)(void*)(&__MBAR[0x000772]))
#define MCF_INTC_ICR51        (*(vuint8 *)(void*)(&__MBAR[0x000773]))
#define MCF_INTC_ICR52        (*(vuint8 *)(void*)(&__MBAR[0x000774]))
#define MCF_INTC_ICR53        (*(vuint8 *)(void*)(&__MBAR[0x000775]))
#define MCF_INTC_ICR54        (*(vuint8 *)(void*)(&__MBAR[0x000776]))
#define MCF_INTC_ICR55        (*(vuint8 *)(void*)(&__MBAR[0x000777]))
#define MCF_INTC_ICR56        (*(vuint8 *)(void*)(&__MBAR[0x000778]))
#define MCF_INTC_ICR57        (*(vuint8 *)(void*)(&__MBAR[0x000779]))
#define MCF_INTC_ICR58        (*(vuint8 *)(void*)(&__MBAR[0x00077A]))
#define MCF_INTC_ICR59        (*(vuint8 *)(void*)(&__MBAR[0x00077B]))
#define MCF_INTC_ICR60        (*(vuint8 *)(void*)(&__MBAR[0x00077C]))
#define MCF_INTC_ICR61        (*(vuint8 *)(void*)(&__MBAR[0x00077D]))
#define MCF_INTC_ICR62        (*(vuint8 *)(void*)(&__MBAR[0x00077E]))
#define MCF_INTC_ICR63        (*(vuint8 *)(void*)(&__MBAR[0x00077F]))
#define MCF_INTC_ICRn(x)      (*(vuint8 *)(void*)(&__MBAR[0x000740 \
                                                    +((x)*0x001)]))
#define MCF_INTC_SWIACK       (*(vuint8 *)(void*)(&__MBAR[0x0007E0]))
#define MCF_INTC_L1IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007E4]))
#define MCF_INTC_L2IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007E8]))
#define MCF_INTC_L3IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007EC]))
#define MCF_INTC_L4IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007F0]))
#define MCF_INTC_L5IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007F4]))
#define MCF_INTC_L6IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007F8]))
#define MCF_INTC_L7IACK       (*(vuint8 *)(void*)(&__MBAR[0x0007FC]))
#define MCF_INTC_LnIACK(x)    (*(vuint8 *)(void*)(&__MBAR[0x0007E4 \
                                                    +((x)*0x004)]))

/* Bit definitions and macros for MCF_INTC_IPRH */
#define MCF_INTC_IPRH_INT32          (0x00000001UL)
#define MCF_INTC_IPRH_INT33          (0x00000002UL)
#define MCF_INTC_IPRH_INT34          (0x00000004UL)
#define MCF_INTC_IPRH_INT35          (0x00000008UL)
#define MCF_INTC_IPRH_INT36          (0x00000010UL)
#define MCF_INTC_IPRH_INT37          (0x00000020UL)
#define MCF_INTC_IPRH_INT38          (0x00000040UL)
#define MCF_INTC_IPRH_INT39          (0x00000080UL)
#define MCF_INTC_IPRH_INT40          (0x00000100UL)
#define MCF_INTC_IPRH_INT41          (0x00000200UL)
#define MCF_INTC_IPRH_INT42          (0x00000400UL)
#define MCF_INTC_IPRH_INT43          (0x00000800UL)
#define MCF_INTC_IPRH_INT44          (0x00001000UL)
#define MCF_INTC_IPRH_INT45          (0x00002000UL)
#define MCF_INTC_IPRH_INT46          (0x00004000UL)
#define MCF_INTC_IPRH_INT47          (0x00008000UL)
#define MCF_INTC_IPRH_INT48          (0x00010000UL)
#define MCF_INTC_IPRH_INT49          (0x00020000UL)
#define MCF_INTC_IPRH_INT50          (0x00040000UL)
#define MCF_INTC_IPRH_INT51          (0x00080000UL)
#define MCF_INTC_IPRH_INT52          (0x00100000UL)
#define MCF_INTC_IPRH_INT53          (0x00200000UL)
#define MCF_INTC_IPRH_INT54          (0x00400000UL)
#define MCF_INTC_IPRH_INT55          (0x00800000UL)
#define MCF_INTC_IPRH_INT56          (0x01000000UL)
#define MCF_INTC_IPRH_INT57          (0x02000000UL)
#define MCF_INTC_IPRH_INT58          (0x04000000UL)
#define MCF_INTC_IPRH_INT59          (0x08000000UL)
#define MCF_INTC_IPRH_INT60          (0x10000000UL)
#define MCF_INTC_IPRH_INT61          (0x20000000UL)
#define MCF_INTC_IPRH_INT62          (0x40000000UL)
#define MCF_INTC_IPRH_INT63          (0x80000000UL)
                                                
/* Bit definitions and macros for MCF_INTC_IPRL */
#define MCF_INTC_IPRL_INT1           (0x00000002UL)
#define MCF_INTC_IPRL_INT2           (0x00000004UL)
#define MCF_INTC_IPRL_INT3           (0x00000008UL)
#define MCF_INTC_IPRL_INT4           (0x00000010UL)
#define MCF_INTC_IPRL_INT5           (0x00000020UL)
#define MCF_INTC_IPRL_INT6           (0x00000040UL)
#define MCF_INTC_IPRL_INT7           (0x00000080UL)
#define MCF_INTC_IPRL_INT8           (0x00000100UL)
#define MCF_INTC_IPRL_INT9           (0x00000200UL)
#define MCF_INTC_IPRL_INT10          (0x00000400UL)
#define MCF_INTC_IPRL_INT11          (0x00000800UL)
#define MCF_INTC_IPRL_INT12          (0x00001000UL)
#define MCF_INTC_IPRL_INT13          (0x00002000UL)
#define MCF_INTC_IPRL_INT14          (0x00004000UL)
#define MCF_INTC_IPRL_INT15          (0x00008000UL)
#define MCF_INTC_IPRL_INT16          (0x00010000UL)
#define MCF_INTC_IPRL_INT17          (0x00020000UL)
#define MCF_INTC_IPRL_INT18          (0x00040000UL)
#define MCF_INTC_IPRL_INT19          (0x00080000UL)
#define MCF_INTC_IPRL_INT20          (0x00100000UL)
#define MCF_INTC_IPRL_INT21          (0x00200000UL)
#define MCF_INTC_IPRL_INT22          (0x00400000UL)
#define MCF_INTC_IPRL_INT23          (0x00800000UL)
#define MCF_INTC_IPRL_INT24          (0x01000000UL)
#define MCF_INTC_IPRL_INT25          (0x02000000UL)
#define MCF_INTC_IPRL_INT26          (0x04000000UL)
#define MCF_INTC_IPRL_INT27          (0x08000000UL)
#define MCF_INTC_IPRL_INT28          (0x10000000UL)
#define MCF_INTC_IPRL_INT29          (0x20000000UL)
#define MCF_INTC_IPRL_INT30          (0x40000000UL)
#define MCF_INTC_IPRL_INT31          (0x80000000UL)

/* Bit definitions and macros for MCF_INTC_IMRH */
#define MCF_INTC_IMRH_INT_MASK32     (0x00000001UL)
#define MCF_INTC_IMRH_INT_MASK33     (0x00000002UL)
#define MCF_INTC_IMRH_INT_MASK34     (0x00000004UL)
#define MCF_INTC_IMRH_INT_MASK35     (0x00000008UL)
#define MCF_INTC_IMRH_INT_MASK36     (0x00000010UL)
#define MCF_INTC_IMRH_INT_MASK37     (0x00000020UL)
#define MCF_INTC_IMRH_INT_MASK38     (0x00000040UL)
#define MCF_INTC_IMRH_INT_MASK39     (0x00000080UL)
#define MCF_INTC_IMRH_INT_MASK40     (0x00000100UL)
#define MCF_INTC_IMRH_INT_MASK41     (0x00000200UL)
#define MCF_INTC_IMRH_INT_MASK42     (0x00000400UL)
#define MCF_INTC_IMRH_INT_MASK43     (0x00000800UL)
#define MCF_INTC_IMRH_INT_MASK44     (0x00001000UL)
#define MCF_INTC_IMRH_INT_MASK45     (0x00002000UL)
#define MCF_INTC_IMRH_INT_MASK46     (0x00004000UL)
#define MCF_INTC_IMRH_INT_MASK47     (0x00008000UL)
#define MCF_INTC_IMRH_INT_MASK48     (0x00010000UL)
#define MCF_INTC_IMRH_INT_MASK49     (0x00020000UL)
#define MCF_INTC_IMRH_INT_MASK50     (0x00040000UL)
#define MCF_INTC_IMRH_INT_MASK51     (0x00080000UL)
#define MCF_INTC_IMRH_INT_MASK52     (0x00100000UL)
#define MCF_INTC_IMRH_INT_MASK53     (0x00200000UL)
#define MCF_INTC_IMRH_INT_MASK54     (0x00400000UL)
#define MCF_INTC_IMRH_INT_MASK55     (0x00800000UL)
#define MCF_INTC_IMRH_INT_MASK56     (0x01000000UL)
#define MCF_INTC_IMRH_INT_MASK57     (0x02000000UL)
#define MCF_INTC_IMRH_INT_MASK58     (0x04000000UL)
#define MCF_INTC_IMRH_INT_MASK59     (0x08000000UL)
#define MCF_INTC_IMRH_INT_MASK60     (0x10000000UL)
#define MCF_INTC_IMRH_INT_MASK61     (0x20000000UL)
#define MCF_INTC_IMRH_INT_MASK62     (0x40000000UL)
#define MCF_INTC_IMRH_INT_MASK63     (0x80000000UL)

/* Bit definitions and macros for MCF_INTC_IMRL */
#define MCF_INTC_IMRL_MASKALL        (0x00000001UL)
#define MCF_INTC_IMRL_INT_MASK1      (0x00000002UL)
#define MCF_INTC_IMRL_INT_MASK2      (0x00000004UL)
#define MCF_INTC_IMRL_INT_MASK3      (0x00000008UL)
#define MCF_INTC_IMRL_INT_MASK4      (0x00000010UL)
#define MCF_INTC_IMRL_INT_MASK5      (0x00000020UL)
#define MCF_INTC_IMRL_INT_MASK6      (0x00000040UL)
#define MCF_INTC_IMRL_INT_MASK7      (0x00000080UL)
#define MCF_INTC_IMRL_INT_MASK8      (0x00000100UL)
#define MCF_INTC_IMRL_INT_MASK9      (0x00000200UL)
#define MCF_INTC_IMRL_INT_MASK10     (0x00000400UL)
#define MCF_INTC_IMRL_INT_MASK11     (0x00000800UL)
#define MCF_INTC_IMRL_INT_MASK12     (0x00001000UL)
#define MCF_INTC_IMRL_INT_MASK13     (0x00002000UL)
#define MCF_INTC_IMRL_INT_MASK14     (0x00004000UL)
#define MCF_INTC_IMRL_INT_MASK15     (0x00008000UL)
#define MCF_INTC_IMRL_INT_MASK16     (0x00010000UL)
#define MCF_INTC_IMRL_INT_MASK17     (0x00020000UL)
#define MCF_INTC_IMRL_INT_MASK18     (0x00040000UL)
#define MCF_INTC_IMRL_INT_MASK19     (0x00080000UL)
#define MCF_INTC_IMRL_INT_MASK20     (0x00100000UL)
#define MCF_INTC_IMRL_INT_MASK21     (0x00200000UL)
#define MCF_INTC_IMRL_INT_MASK22     (0x00400000UL)
#define MCF_INTC_IMRL_INT_MASK23     (0x00800000UL)
#define MCF_INTC_IMRL_INT_MASK24     (0x01000000UL)
#define MCF_INTC_IMRL_INT_MASK25     (0x02000000UL)
#define MCF_INTC_IMRL_INT_MASK26     (0x04000000UL)
#define MCF_INTC_IMRL_INT_MASK27     (0x08000000UL)
#define MCF_INTC_IMRL_INT_MASK28     (0x10000000UL)
#define MCF_INTC_IMRL_INT_MASK29     (0x20000000UL)
#define MCF_INTC_IMRL_INT_MASK30     (0x40000000UL)
#define MCF_INTC_IMRL_INT_MASK31     (0x80000000UL)

/* Bit definitions and macros for MCF_INTC_INTFRCH */
#define MCF_INTC_INTFRCH_INTFRC32    (0x00000001UL)
#define MCF_INTC_INTFRCH_INTFRC33    (0x00000002UL)
#define MCF_INTC_INTFRCH_INTFRC34    (0x00000004UL)
#define MCF_INTC_INTFRCH_INTFRC35    (0x00000008UL)
#define MCF_INTC_INTFRCH_INTFRC36    (0x00000010UL)
#define MCF_INTC_INTFRCH_INTFRC37    (0x00000020UL)
#define MCF_INTC_INTFRCH_INTFRC38    (0x00000040UL)
#define MCF_INTC_INTFRCH_INTFRC39    (0x00000080UL)
#define MCF_INTC_INTFRCH_INTFRC40    (0x00000100UL)
#define MCF_INTC_INTFRCH_INTFRC41    (0x00000200UL)
#define MCF_INTC_INTFRCH_INTFRC42    (0x00000400UL)
#define MCF_INTC_INTFRCH_INTFRC43    (0x00000800UL)
#define MCF_INTC_INTFRCH_INTFRC44    (0x00001000UL)
#define MCF_INTC_INTFRCH_INTFRC45    (0x00002000UL)
#define MCF_INTC_INTFRCH_INTFRC46    (0x00004000UL)
#define MCF_INTC_INTFRCH_INTFRC47    (0x00008000UL)
#define MCF_INTC_INTFRCH_INTFRC48    (0x00010000UL)
#define MCF_INTC_INTFRCH_INTFRC49    (0x00020000UL)
#define MCF_INTC_INTFRCH_INTFRC50    (0x00040000UL)
#define MCF_INTC_INTFRCH_INTFRC51    (0x00080000UL)
#define MCF_INTC_INTFRCH_INTFRC52    (0x00100000UL)
#define MCF_INTC_INTFRCH_INTFRC53    (0x00200000UL)
#define MCF_INTC_INTFRCH_INTFRC54    (0x00400000UL)
#define MCF_INTC_INTFRCH_INTFRC55    (0x00800000UL)
#define MCF_INTC_INTFRCH_INTFRC56    (0x01000000UL)
#define MCF_INTC_INTFRCH_INTFRC57    (0x02000000UL)
#define MCF_INTC_INTFRCH_INTFRC58    (0x04000000UL)
#define MCF_INTC_INTFRCH_INTFRC59    (0x08000000UL)
#define MCF_INTC_INTFRCH_INTFRC60    (0x10000000UL)
#define MCF_INTC_INTFRCH_INTFRC61    (0x20000000UL)
#define MCF_INTC_INTFRCH_INTFRC62    (0x40000000UL)
#define MCF_INTC_INTFRCH_INTFRC63    (0x80000000UL)

/* Bit definitions and macros for MCF_INTC_INTFRCL */
#define MCF_INTC_INTFRCL_INTFRC1     (0x00000002UL)
#define MCF_INTC_INTFRCL_INTFRC2     (0x00000004UL)
#define MCF_INTC_INTFRCL_INTFRC3     (0x00000008UL)
#define MCF_INTC_INTFRCL_INTFRC4     (0x00000010UL)
#define MCF_INTC_INTFRCL_INTFRC5     (0x00000020UL)
#define MCF_INTC_INTFRCL_INT6        (0x00000040UL)
#define MCF_INTC_INTFRCL_INT7        (0x00000080UL)
#define MCF_INTC_INTFRCL_INT8        (0x00000100UL)
#define MCF_INTC_INTFRCL_INT9        (0x00000200UL)
#define MCF_INTC_INTFRCL_INT10       (0x00000400UL)
#define MCF_INTC_INTFRCL_INTFRC11    (0x00000800UL)
#define MCF_INTC_INTFRCL_INTFRC12    (0x00001000UL)
#define MCF_INTC_INTFRCL_INTFRC13    (0x00002000UL)
#define MCF_INTC_INTFRCL_INTFRC14    (0x00004000UL)
#define MCF_INTC_INTFRCL_INT15       (0x00008000UL)
#define MCF_INTC_INTFRCL_INTFRC16    (0x00010000UL)
#define MCF_INTC_INTFRCL_INTFRC17    (0x00020000UL)
#define MCF_INTC_INTFRCL_INTFRC18    (0x00040000UL)
#define MCF_INTC_INTFRCL_INTFRC19    (0x00080000UL)
#define MCF_INTC_INTFRCL_INTFRC20    (0x00100000UL)
#define MCF_INTC_INTFRCL_INTFRC21    (0x00200000UL)
#define MCF_INTC_INTFRCL_INTFRC22    (0x00400000UL)
#define MCF_INTC_INTFRCL_INTFRC23    (0x00800000UL)
#define MCF_INTC_INTFRCL_INTFRC24    (0x01000000UL)
#define MCF_INTC_INTFRCL_INTFRC25    (0x02000000UL)
#define MCF_INTC_INTFRCL_INTFRC26    (0x04000000UL)
#define MCF_INTC_INTFRCL_INTFRC27    (0x08000000UL)
#define MCF_INTC_INTFRCL_INTFRC28    (0x10000000UL)
#define MCF_INTC_INTFRCL_INTFRC29    (0x20000000UL)
#define MCF_INTC_INTFRCL_INTFRC30    (0x40000000UL)
#define MCF_INTC_INTFRCL_INTFRC31    (0x80000000UL)

/* Bit definitions and macros for MCF_INTC_IRLR */
#define MCF_INTC_IRLR_IRQ(x)         (((x)&0x7F)<<1)

/* Bit definitions and macros for MCF_INTC_IACKLPR */
#define MCF_INTC_IACKLPR_PRI(x)      (((x)&0x0F)<<0)
#define MCF_INTC_IACKLPR_LEVEL(x)    (((x)&0x07)<<4)

/* Bit definitions and macros for MCF_INTC_ICRn */
#define MCF_INTC_ICRn_IP(x)          (((x)&0x07)<<0)
#define MCF_INTC_ICRn_IL(x)          (((x)&0x07)<<3)

/****************************************************************************
 *
 * Programmable Serial Controller (PSC) Defines
 *
 */

/* Register read/write macros */
#define MCF_PSC_MR0            (*(vuint8 *)(void*)(&__MBAR[0x008600]))
#define MCF_PSC_SR0            (*(vuint16*)(void*)(&__MBAR[0x008604]))
#define MCF_PSC_CSR0           (*(vuint8 *)(void*)(&__MBAR[0x008604]))
#define MCF_PSC_CR0            (*(vuint8 *)(void*)(&__MBAR[0x008608]))
#define MCF_PSC_RB0            (*(vuint32*)(void*)(&__MBAR[0x00860C]))
#define MCF_PSC_TB0            (*(vuint32*)(void*)(&__MBAR[0x00860C]))
#define MCF_PSC_TB_8BIT0       (*(vuint32*)(void*)(&__MBAR[0x00860C]))
#define MCF_PSC_TB_16BIT0      (*(vuint32*)(void*)(&__MBAR[0x00860C]))
#define MCF_PSC_TB_AC970       (*(vuint32*)(void*)(&__MBAR[0x00860C]))
#define MCF_PSC_IPCR0          (*(vuint8 *)(void*)(&__MBAR[0x008610]))
#define MCF_PSC_ACR0           (*(vuint8 *)(void*)(&__MBAR[0x008610]))
#define MCF_PSC_ISR0           (*(vuint16*)(void*)(&__MBAR[0x008614]))
#define MCF_PSC_IMR0           (*(vuint16*)(void*)(&__MBAR[0x008614]))
#define MCF_PSC_CTUR0          (*(vuint8 *)(void*)(&__MBAR[0x008618]))
#define MCF_PSC_CTLR0          (*(vuint8 *)(void*)(&__MBAR[0x00861C]))
#define MCF_PSC_IP0            (*(vuint8 *)(void*)(&__MBAR[0x008634]))
#define MCF_PSC_OPSET0         (*(vuint8 *)(void*)(&__MBAR[0x008638]))
#define MCF_PSC_OPRESET0       (*(vuint8 *)(void*)(&__MBAR[0x00863C]))
#define MCF_PSC_SICR0          (*(vuint8 *)(void*)(&__MBAR[0x008640]))
#define MCF_PSC_IRCR10         (*(vuint8 *)(void*)(&__MBAR[0x008644]))
#define MCF_PSC_IRCR20         (*(vuint8 *)(void*)(&__MBAR[0x008648]))
#define MCF_PSC_IRSDR0         (*(vuint8 *)(void*)(&__MBAR[0x00864C]))
#define MCF_PSC_IRMDR0         (*(vuint8 *)(void*)(&__MBAR[0x008650]))
#define MCF_PSC_IRFDR0         (*(vuint8 *)(void*)(&__MBAR[0x008654]))
#define MCF_PSC_RFCNT0         (*(vuint16*)(void*)(&__MBAR[0x008658]))
#define MCF_PSC_TFCNT0         (*(vuint16*)(void*)(&__MBAR[0x00865C]))
#define MCF_PSC_RFSR0          (*(vuint16*)(void*)(&__MBAR[0x008664]))
#define MCF_PSC_TFSR0          (*(vuint16*)(void*)(&__MBAR[0x008684]))
#define MCF_PSC_RFCR0          (*(vuint32*)(void*)(&__MBAR[0x008668]))
#define MCF_PSC_TFCR0          (*(vuint32*)(void*)(&__MBAR[0x008688]))
#define MCF_PSC_RFAR0          (*(vuint16*)(void*)(&__MBAR[0x00866E]))
#define MCF_PSC_TFAR0          (*(vuint16*)(void*)(&__MBAR[0x00868E]))
#define MCF_PSC_RFRP0          (*(vuint16*)(void*)(&__MBAR[0x008672]))
#define MCF_PSC_TFRP0          (*(vuint16*)(void*)(&__MBAR[0x008692]))
#define MCF_PSC_RFWP0          (*(vuint16*)(void*)(&__MBAR[0x008676]))
#define MCF_PSC_TFWP0          (*(vuint16*)(void*)(&__MBAR[0x008696]))
#define MCF_PSC_RLRFP0         (*(vuint16*)(void*)(&__MBAR[0x00867A]))
#define MCF_PSC_TLRFP0         (*(vuint16*)(void*)(&__MBAR[0x00869A]))
#define MCF_PSC_RLWFP0         (*(vuint16*)(void*)(&__MBAR[0x00867E]))
#define MCF_PSC_TLWFP0         (*(vuint16*)(void*)(&__MBAR[0x00869E]))
#define MCF_PSC_MR1            (*(vuint8 *)(void*)(&__MBAR[0x008700]))
#define MCF_PSC_SR1            (*(vuint16*)(void*)(&__MBAR[0x008704]))
#define MCF_PSC_CSR1           (*(vuint8 *)(void*)(&__MBAR[0x008704]))
#define MCF_PSC_CR1            (*(vuint8 *)(void*)(&__MBAR[0x008708]))
#define MCF_PSC_RB1            (*(vuint32*)(void*)(&__MBAR[0x00870C]))
#define MCF_PSC_TB1            (*(vuint32*)(void*)(&__MBAR[0x00870C]))
#define MCF_PSC_TB_8BIT1       (*(vuint32*)(void*)(&__MBAR[0x00870C]))
#define MCF_PSC_TB_16BIT1      (*(vuint32*)(void*)(&__MBAR[0x00870C]))
#define MCF_PSC_TB_AC971       (*(vuint32*)(void*)(&__MBAR[0x00870C]))
#define MCF_PSC_IPCR1          (*(vuint8 *)(void*)(&__MBAR[0x008710]))
#define MCF_PSC_ACR1           (*(vuint8 *)(void*)(&__MBAR[0x008710]))
#define MCF_PSC_ISR1           (*(vuint16*)(void*)(&__MBAR[0x008714]))
#define MCF_PSC_IMR1           (*(vuint16*)(void*)(&__MBAR[0x008714]))
#define MCF_PSC_CTUR1          (*(vuint8 *)(void*)(&__MBAR[0x008718]))
#define MCF_PSC_CTLR1          (*(vuint8 *)(void*)(&__MBAR[0x00871C]))
#define MCF_PSC_IP1            (*(vuint8 *)(void*)(&__MBAR[0x008734]))
#define MCF_PSC_OPSET1         (*(vuint8 *)(void*)(&__MBAR[0x008738]))
#define MCF_PSC_OPRESET1       (*(vuint8 *)(void*)(&__MBAR[0x00873C]))
#define MCF_PSC_SICR1          (*(vuint8 *)(void*)(&__MBAR[0x008740]))
#define MCF_PSC_IRCR11         (*(vuint8 *)(void*)(&__MBAR[0x008744]))
#define MCF_PSC_IRCR21         (*(vuint8 *)(void*)(&__MBAR[0x008748]))
#define MCF_PSC_IRSDR1         (*(vuint8 *)(void*)(&__MBAR[0x00874C]))
#define MCF_PSC_IRMDR1         (*(vuint8 *)(void*)(&__MBAR[0x008750]))
#define MCF_PSC_IRFDR1         (*(vuint8 *)(void*)(&__MBAR[0x008754]))
#define MCF_PSC_RFCNT1         (*(vuint16*)(void*)(&__MBAR[0x008758]))
#define MCF_PSC_TFCNT1         (*(vuint16*)(void*)(&__MBAR[0x00875C]))
#define MCF_PSC_RFSR1          (*(vuint16*)(void*)(&__MBAR[0x008764]))
#define MCF_PSC_TFSR1          (*(vuint16*)(void*)(&__MBAR[0x008784]))
#define MCF_PSC_RFCR1          (*(vuint32*)(void*)(&__MBAR[0x008768]))
#define MCF_PSC_TFCR1          (*(vuint32*)(void*)(&__MBAR[0x008788]))
#define MCF_PSC_RFAR1          (*(vuint16*)(void*)(&__MBAR[0x00876E]))
#define MCF_PSC_TFAR1          (*(vuint16*)(void*)(&__MBAR[0x00878E]))
#define MCF_PSC_RFRP1          (*(vuint16*)(void*)(&__MBAR[0x008772]))
#define MCF_PSC_TFRP1          (*(vuint16*)(void*)(&__MBAR[0x008792]))
#define MCF_PSC_RFWP1          (*(vuint16*)(void*)(&__MBAR[0x008776]))
#define MCF_PSC_TFWP1          (*(vuint16*)(void*)(&__MBAR[0x008796]))
#define MCF_PSC_RLRFP1         (*(vuint16*)(void*)(&__MBAR[0x00877A]))
#define MCF_PSC_TLRFP1         (*(vuint16*)(void*)(&__MBAR[0x00879A]))
#define MCF_PSC_RLWFP1         (*(vuint16*)(void*)(&__MBAR[0x00877E]))
#define MCF_PSC_TLWFP1         (*(vuint16*)(void*)(&__MBAR[0x00879E]))
#define MCF_PSC_MR2            (*(vuint8 *)(void*)(&__MBAR[0x008800]))
#define MCF_PSC_SR2            (*(vuint16*)(void*)(&__MBAR[0x008804]))
#define MCF_PSC_CSR2           (*(vuint8 *)(void*)(&__MBAR[0x008804]))
#define MCF_PSC_CR2            (*(vuint8 *)(void*)(&__MBAR[0x008808]))
#define MCF_PSC_RB2            (*(vuint32*)(void*)(&__MBAR[0x00880C]))
#define MCF_PSC_TB2            (*(vuint32*)(void*)(&__MBAR[0x00880C]))
#define MCF_PSC_TB_8BIT2       (*(vuint32*)(void*)(&__MBAR[0x00880C]))
#define MCF_PSC_TB_16BIT2      (*(vuint32*)(void*)(&__MBAR[0x00880C]))
#define MCF_PSC_TB_AC972       (*(vuint32*)(void*)(&__MBAR[0x00880C]))
#define MCF_PSC_IPCR2          (*(vuint8 *)(void*)(&__MBAR[0x008810]))
#define MCF_PSC_ACR2           (*(vuint8 *)(void*)(&__MBAR[0x008810]))
#define MCF_PSC_ISR2           (*(vuint16*)(void*)(&__MBAR[0x008814]))
#define MCF_PSC_IMR2           (*(vuint16*)(void*)(&__MBAR[0x008814]))
#define MCF_PSC_CTUR2          (*(vuint8 *)(void*)(&__MBAR[0x008818]))
#define MCF_PSC_CTLR2          (*(vuint8 *)(void*)(&__MBAR[0x00881C]))
#define MCF_PSC_IP2            (*(vuint8 *)(void*)(&__MBAR[0x008834]))
#define MCF_PSC_OPSET2         (*(vuint8 *)(void*)(&__MBAR[0x008838]))
#define MCF_PSC_OPRESET2       (*(vuint8 *)(void*)(&__MBAR[0x00883C]))
#define MCF_PSC_SICR2          (*(vuint8 *)(void*)(&__MBAR[0x008840]))
#define MCF_PSC_IRCR12         (*(vuint8 *)(void*)(&__MBAR[0x008844]))
#define MCF_PSC_IRCR22         (*(vuint8 *)(void*)(&__MBAR[0x008848]))
#define MCF_PSC_IRSDR2         (*(vuint8 *)(void*)(&__MBAR[0x00884C]))
#define MCF_PSC_IRMDR2         (*(vuint8 *)(void*)(&__MBAR[0x008850]))
#define MCF_PSC_IRFDR2         (*(vuint8 *)(void*)(&__MBAR[0x008854]))
#define MCF_PSC_RFCNT2         (*(vuint16*)(void*)(&__MBAR[0x008858]))
#define MCF_PSC_TFCNT2         (*(vuint16*)(void*)(&__MBAR[0x00885C]))
#define MCF_PSC_RFSR2          (*(vuint16*)(void*)(&__MBAR[0x008864]))
#define MCF_PSC_TFSR2          (*(vuint16*)(void*)(&__MBAR[0x008884]))
#define MCF_PSC_RFCR2          (*(vuint32*)(void*)(&__MBAR[0x008868]))
#define MCF_PSC_TFCR2          (*(vuint32*)(void*)(&__MBAR[0x008888]))
#define MCF_PSC_RFAR2          (*(vuint16*)(void*)(&__MBAR[0x00886E]))
#define MCF_PSC_TFAR2          (*(vuint16*)(void*)(&__MBAR[0x00888E]))
#define MCF_PSC_RFRP2          (*(vuint16*)(void*)(&__MBAR[0x008872]))
#define MCF_PSC_TFRP2          (*(vuint16*)(void*)(&__MBAR[0x008892]))
#define MCF_PSC_RFWP2          (*(vuint16*)(void*)(&__MBAR[0x008876]))
#define MCF_PSC_TFWP2          (*(vuint16*)(void*)(&__MBAR[0x008896]))
#define MCF_PSC_RLRFP2         (*(vuint16*)(void*)(&__MBAR[0x00887A]))
#define MCF_PSC_TLRFP2         (*(vuint16*)(void*)(&__MBAR[0x00889A]))
#define MCF_PSC_RLWFP2         (*(vuint16*)(void*)(&__MBAR[0x00887E]))
#define MCF_PSC_TLWFP2         (*(vuint16*)(void*)(&__MBAR[0x00889E]))
#define MCF_PSC_MR3            (*(vuint8 *)(void*)(&__MBAR[0x008900]))
#define MCF_PSC_SR3            (*(vuint16*)(void*)(&__MBAR[0x008904]))
#define MCF_PSC_CSR3           (*(vuint8 *)(void*)(&__MBAR[0x008904]))
#define MCF_PSC_CR3            (*(vuint8 *)(void*)(&__MBAR[0x008908]))
#define MCF_PSC_RB3            (*(vuint32*)(void*)(&__MBAR[0x00890C]))
#define MCF_PSC_TB3            (*(vuint32*)(void*)(&__MBAR[0x00890C]))
#define MCF_PSC_TB_8BIT3       (*(vuint32*)(void*)(&__MBAR[0x00890C]))
#define MCF_PSC_TB_16BIT3      (*(vuint32*)(void*)(&__MBAR[0x00890C]))
#define MCF_PSC_TB_AC973       (*(vuint32*)(void*)(&__MBAR[0x00890C]))
#define MCF_PSC_IPCR3          (*(vuint8 *)(void*)(&__MBAR[0x008910]))
#define MCF_PSC_ACR3           (*(vuint8 *)(void*)(&__MBAR[0x008910]))
#define MCF_PSC_ISR3           (*(vuint16*)(void*)(&__MBAR[0x008914]))
#define MCF_PSC_IMR3           (*(vuint16*)(void*)(&__MBAR[0x008914]))
#define MCF_PSC_CTUR3          (*(vuint8 *)(void*)(&__MBAR[0x008918]))
#define MCF_PSC_CTLR3          (*(vuint8 *)(void*)(&__MBAR[0x00891C]))
#define MCF_PSC_IP3            (*(vuint8 *)(void*)(&__MBAR[0x008934]))
#define MCF_PSC_OPSET3         (*(vuint8 *)(void*)(&__MBAR[0x008938]))
#define MCF_PSC_OPRESET3       (*(vuint8 *)(void*)(&__MBAR[0x00893C]))
#define MCF_PSC_SICR3          (*(vuint8 *)(void*)(&__MBAR[0x008940]))
#define MCF_PSC_IRCR13         (*(vuint8 *)(void*)(&__MBAR[0x008944]))
#define MCF_PSC_IRCR23         (*(vuint8 *)(void*)(&__MBAR[0x008948]))
#define MCF_PSC_IRSDR3         (*(vuint8 *)(void*)(&__MBAR[0x00894C]))
#define MCF_PSC_IRMDR3         (*(vuint8 *)(void*)(&__MBAR[0x008950]))
#define MCF_PSC_IRFDR3         (*(vuint8 *)(void*)(&__MBAR[0x008954]))
#define MCF_PSC_RFCNT3         (*(vuint16*)(void*)(&__MBAR[0x008958]))
#define MCF_PSC_TFCNT3         (*(vuint16*)(void*)(&__MBAR[0x00895C]))
#define MCF_PSC_RFSR3          (*(vuint16*)(void*)(&__MBAR[0x008964]))
#define MCF_PSC_TFSR3          (*(vuint16*)(void*)(&__MBAR[0x008984]))
#define MCF_PSC_RFCR3          (*(vuint32*)(void*)(&__MBAR[0x008968]))
#define MCF_PSC_TFCR3          (*(vuint32*)(void*)(&__MBAR[0x008988]))
#define MCF_PSC_RFAR3          (*(vuint16*)(void*)(&__MBAR[0x00896E]))
#define MCF_PSC_TFAR3          (*(vuint16*)(void*)(&__MBAR[0x00898E]))
#define MCF_PSC_RFRP3          (*(vuint16*)(void*)(&__MBAR[0x008972]))
#define MCF_PSC_TFRP3          (*(vuint16*)(void*)(&__MBAR[0x008992]))
#define MCF_PSC_RFWP3          (*(vuint16*)(void*)(&__MBAR[0x008976]))
#define MCF_PSC_TFWP3          (*(vuint16*)(void*)(&__MBAR[0x008996]))
#define MCF_PSC_RLRFP3         (*(vuint16*)(void*)(&__MBAR[0x00897A]))
#define MCF_PSC_TLRFP3         (*(vuint16*)(void*)(&__MBAR[0x00899A]))
#define MCF_PSC_RLWFP3         (*(vuint16*)(void*)(&__MBAR[0x00897E]))
#define MCF_PSC_TLWFP3         (*(vuint16*)(void*)(&__MBAR[0x00899E]))
#define MCF_PSC_MR(x)          (*(vuint8 *)(void*)(&__MBAR[0x008600 \
                                                    +((x)*0x100)]))
#define MCF_PSC_SR(x)          (*(vuint16*)(void*)(&__MBAR[0x008604 \
                                                    +((x)*0x100)]))
#define MCF_PSC_CSR(x)         (*(vuint8 *)(void*)(&__MBAR[0x008604 \
                                                    +((x)*0x100)]))
#define MCF_PSC_CR(x)          (*(vuint8 *)(void*)(&__MBAR[0x008608 \
                                                    +((x)*0x100)]))
#define MCF_PSC_RB(x)          (*(vuint32*)(void*)(&__MBAR[0x00860C \
                                                    +((x)*0x100)]))
#define MCF_PSC_TB(x)          (*(vuint32*)(void*)(&__MBAR[0x00860C \
                                                    +((x)*0x100)]))
#define MCF_PSC_TB_8BIT(x)     (*(vuint32*)(void*)(&__MBAR[0x00860C \
                                                    +((x)*0x100)]))
#define MCF_PSC_TB_16BIT(x)    (*(vuint32*)(void*)(&__MBAR[0x00860C \
                                                    +((x)*0x100)]))
#define MCF_PSC_TB_AC97(x)     (*(vuint32*)(void*)(&__MBAR[0x00860C \
                                                    +((x)*0x100)]))
#define MCF_PSC_IPCR(x)        (*(vuint8 *)(void*)(&__MBAR[0x008610 \
                                                    +((x)*0x100)]))
#define MCF_PSC_ACR(x)         (*(vuint8 *)(void*)(&__MBAR[0x008610 \
                                                    +((x)*0x100)]))
#define MCF_PSC_ISR(x)         (*(vuint16*)(void*)(&__MBAR[0x008614 \
                                                    +((x)*0x100)]))
#define MCF_PSC_IMR(x)         (*(vuint16*)(void*)(&__MBAR[0x008614 \
                                                    +((x)*0x100)]))
#define MCF_PSC_CTUR(x)        (*(vuint8 *)(void*)(&__MBAR[0x008618 \
                                                    +((x)*0x100)]))
#define MCF_PSC_CTLR(x)        (*(vuint8 *)(void*)(&__MBAR[0x00861C \
                                                    +((x)*0x100)]))
#define MCF_PSC_IP(x)          (*(vuint8 *)(void*)(&__MBAR[0x008634 \
                                                    +((x)*0x100)]))
#define MCF_PSC_OPSET(x)       (*(vuint8 *)(void*)(&__MBAR[0x008638 \
                                                    +((x)*0x100)]))
#define MCF_PSC_OPRESET(x)     (*(vuint8 *)(void*)(&__MBAR[0x00863C \
                                                    +((x)*0x100)]))
#define MCF_PSC_SICR(x)        (*(vuint8 *)(void*)(&__MBAR[0x008640 \
                                                    +((x)*0x100)]))
#define MCF_PSC_IRCR1(x)       (*(vuint8 *)(void*)(&__MBAR[0x008644 \
                                                    +((x)*0x100)]))
#define MCF_PSC_IRCR2(x)       (*(vuint8 *)(void*)(&__MBAR[0x008648 \
                                                    +((x)*0x100)]))
#define MCF_PSC_IRSDR(x)       (*(vuint8 *)(void*)(&__MBAR[0x00864C \
                                                    +((x)*0x100)]))
#define MCF_PSC_IRMDR(x)       (*(vuint8 *)(void*)(&__MBAR[0x008650 \
                                                    +((x)*0x100)]))
#define MCF_PSC_IRFDR(x)       (*(vuint8 *)(void*)(&__MBAR[0x008654 \
                                                    +((x)*0x100)]))
#define MCF_PSC_RFCNT(x)       (*(vuint16*)(void*)(&__MBAR[0x008658 \
                                                    +((x)*0x100)]))
#define MCF_PSC_TFCNT(x)       (*(vuint16*)(void*)(&__MBAR[0x00865C \
                                                    +((x)*0x100)]))
#define MCF_PSC_RFSR(x)        (*(vuint16*)(void*)(&__MBAR[0x008664 \
                                                    +((x)*0x100)]))
#define MCF_PSC_TFSR(x)        (*(vuint16*)(void*)(&__MBAR[0x008684 \
                                                    +((x)*0x100)]))
#define MCF_PSC_RFCR(x)        (*(vuint32*)(void*)(&__MBAR[0x008668 \
                                                    +((x)*0x100)]))
#define MCF_PSC_TFCR(x)        (*(vuint32*)(void*)(&__MBAR[0x008688 \
                                                    +((x)*0x100)]))
#define MCF_PSC_RFAR(x)        (*(vuint16*)(void*)(&__MBAR[0x00866E \
                                                    +((x)*0x100)]))
#define MCF_PSC_TFAR(x)        (*(vuint16*)(void*)(&__MBAR[0x00868E \
                                                    +((x)*0x100)]))
#define MCF_PSC_RFRP(x)        (*(vuint16*)(void*)(&__MBAR[0x008672 \
                                                    +((x)*0x100)]))
#define MCF_PSC_TFRP(x)        (*(vuint16*)(void*)(&__MBAR[0x008692 \
                                                    +((x)*0x100)]))
#define MCF_PSC_RFWP(x)        (*(vuint16*)(void*)(&__MBAR[0x008676 \
                                                    +((x)*0x100)]))
#define MCF_PSC_TFWP(x)        (*(vuint16*)(void*)(&__MBAR[0x008696 \
                                                    +((x)*0x100)]))
#define MCF_PSC_RLRFP(x)       (*(vuint16*)(void*)(&__MBAR[0x00867A \
                                                    +((x)*0x100)]))
#define MCF_PSC_TLRFP(x)       (*(vuint16*)(void*)(&__MBAR[0x00869A \
                                                    +((x)*0x100)]))
#define MCF_PSC_RLWFP(x)       (*(vuint16*)(void*)(&__MBAR[0x00867E \
                                                    +((x)*0x100)]))
#define MCF_PSC_TLWFP(x)       (*(vuint16*)(void*)(&__MBAR[0x00869E \
                                                    +((x)*0x100)]))

/* Bit definitions and macros for MCF_PSC_MR */
#define MCF_PSC_MR_BC(x)              (((x)&0x03)<<0)
#define MCF_PSC_MR_PT                 (0x04)
#define MCF_PSC_MR_PM(x)              (((x)&0x03)<<3)
#define MCF_PSC_MR_ERR                (0x20)
#define MCF_PSC_MR_RXIRQ              (0x40)
#define MCF_PSC_MR_RXRTS              (0x80)
#define MCF_PSC_MR_SB(x)              (((x)&0x0F)<<0)
#define MCF_PSC_MR_TXCTS              (0x10)
#define MCF_PSC_MR_TXRTS              (0x20)
#define MCF_PSC_MR_CM(x)              (((x)&0x03)<<6)
#define MCF_PSC_MR_PM_MULTI_ADDR      (0x1C)
#define MCF_PSC_MR_PM_MULTI_DATA      (0x18)
#define MCF_PSC_MR_PM_NONE            (0x10)
#define MCF_PSC_MR_PM_FORCE_HI        (0x0C)
#define MCF_PSC_MR_PM_FORCE_LO        (0x08)
#define MCF_PSC_MR_PM_ODD             (0x04)
#define MCF_PSC_MR_PM_EVEN            (0x00)
#define MCF_PSC_MR_BC_5               (0x00)
#define MCF_PSC_MR_BC_6               (0x01)
#define MCF_PSC_MR_BC_7               (0x02)
#define MCF_PSC_MR_BC_8               (0x03)
#define MCF_PSC_MR_CM_NORMAL          (0x00)
#define MCF_PSC_MR_CM_ECHO            (0x40)
#define MCF_PSC_MR_CM_LOCAL_LOOP      (0x80)
#define MCF_PSC_MR_CM_REMOTE_LOOP     (0xC0)
#define MCF_PSC_MR_SB_STOP_BITS_1     (0x07)
#define MCF_PSC_MR_SB_STOP_BITS_15    (0x08)
#define MCF_PSC_MR_SB_STOP_BITS_2     (0x0F)

/* Bit definitions and macros for MCF_PSC_SR */
#define MCF_PSC_SR_ERR                (0x0040)
#define MCF_PSC_SR_CDE_DEOF           (0x0080)
#define MCF_PSC_SR_RXRDY              (0x0100)
#define MCF_PSC_SR_FU                 (0x0200)
#define MCF_PSC_SR_TXRDY              (0x0400)
#define MCF_PSC_SR_TXEMP_URERR        (0x0800)
#define MCF_PSC_SR_OE                 (0x1000)
#define MCF_PSC_SR_PE_CRCERR          (0x2000)
#define MCF_PSC_SR_FE_PHYERR          (0x4000)
#define MCF_PSC_SR_RB_NEOF            (0x8000)

/* Bit definitions and macros for MCF_PSC_CSR */
#define MCF_PSC_CSR_TCSEL(x)          (((x)&0x0F)<<0)
#define MCF_PSC_CSR_RCSEL(x)          (((x)&0x0F)<<4)
#define MCF_PSC_CSR_RCSEL_SYS_CLK     (0xD0)
#define MCF_PSC_CSR_RCSEL_CTM16       (0xE0)
#define MCF_PSC_CSR_RCSEL_CTM         (0xF0)
#define MCF_PSC_CSR_TCSEL_SYS_CLK     (0x0D)
#define MCF_PSC_CSR_TCSEL_CTM16       (0x0E)
#define MCF_PSC_CSR_TCSEL_CTM         (0x0F)

/* Bit definitions and macros for MCF_PSC_CR */
#define MCF_PSC_CR_RXC(x)             (((x)&0x03)<<0)
#define MCF_PSC_CR_TXC(x)             (((x)&0x03)<<2)
#define MCF_PSC_CR_MISC(x)            (((x)&0x07)<<4)
#define MCF_PSC_CR_NONE               (0x00)
#define MCF_PSC_CR_STOP_BREAK         (0x70)
#define MCF_PSC_CR_START_BREAK        (0x60)
#define MCF_PSC_CR_BKCHGINT           (0x50)
#define MCF_PSC_CR_RESET_ERROR        (0x40)
#define MCF_PSC_CR_RESET_TX           (0x30)
#define MCF_PSC_CR_RESET_RX           (0x20)
#define MCF_PSC_CR_RESET_MR           (0x10)
#define MCF_PSC_CR_TX_DISABLED        (0x08)
#define MCF_PSC_CR_TX_ENABLED         (0x04)
#define MCF_PSC_CR_RX_DISABLED        (0x02)
#define MCF_PSC_CR_RX_ENABLED         (0x01)

/* Bit definitions and macros for MCF_PSC_TB_8BIT */
#define MCF_PSC_TB_8BIT_TB3(x)        (((x)&0x000000FFUL)<<0)
#define MCF_PSC_TB_8BIT_TB2(x)        (((x)&0x000000FFUL)<<8)
#define MCF_PSC_TB_8BIT_TB1(x)        (((x)&0x000000FFUL)<<16)
#define MCF_PSC_TB_8BIT_TB0(x)        (((x)&0x000000FFUL)<<24)
                                                       
/* Bit definitions and macros for MCF_PSC_TB_16BIT */
#define MCF_PSC_TB_16BIT_TB1(x)       (((x)&0x0000FFFFUL)<<0)
#define MCF_PSC_TB_16BIT_TB0(x)       (((x)&0x0000FFFFUL)<<16)

/* Bit definitions and macros for MCF_PSC_TB_AC97 */
#define MCF_PSC_TB_AC97_SOF           (0x00000800UL)
#define MCF_PSC_TB_AC97_TB(x)         (((x)&0x000FFFFFUL)<<12)

/* Bit definitions and macros for MCF_PSC_IPCR */
#define MCF_PSC_IPCR_RESERVED         (0x0C)
#define MCF_PSC_IPCR_CTS              (0x0D)
#define MCF_PSC_IPCR_D_CTS            (0x1C)
#define MCF_PSC_IPCR_SYNC             (0x8C)

/* Bit definitions and macros for MCF_PSC_ACR */
#define MCF_PSC_ACR_IEC0              (0x01)
#define MCF_PSC_ACR_CTMS(x)           (((x)&0x07)<<4)
#define MCF_PSC_ACR_BRG               (0x80)

/* Bit definitions and macros for MCF_PSC_ISR */
#define MCF_PSC_ISR_ERR               (0x0040)
#define MCF_PSC_ISR_DEOF              (0x0080)
#define MCF_PSC_ISR_TXRDY             (0x0100)
#define MCF_PSC_ISR_RXRDY_FU          (0x0200)
#define MCF_PSC_ISR_DB                (0x0400)
#define MCF_PSC_ISR_IPC               (0x8000)

/* Bit definitions and macros for MCF_PSC_IMR */
#define MCF_PSC_IMR_ERR               (0x0040)
#define MCF_PSC_IMR_DEOF              (0x0080)
#define MCF_PSC_IMR_TXRDY             (0x0100)
#define MCF_PSC_IMR_RXRDY_FU          (0x0200)
#define MCF_PSC_IMR_DB                (0x0400)
#define MCF_PSC_IMR_IPC               (0x8000)

/* Bit definitions and macros for MCF_PSC_IP */
#define MCF_PSC_IP_CTS                (0x01)
#define MCF_PSC_IP_TGL                (0x40)
#define MCF_PSC_IP_LWPR_B             (0x80)

/* Bit definitions and macros for MCF_PSC_OPSET */
#define MCF_PSC_OPSET_RTS             (0x01)

/* Bit definitions and macros for MCF_PSC_OPRESET */
#define MCF_PSC_OPRESET_RTS           (0x01)

/* Bit definitions and macros for MCF_PSC_SICR */
#define MCF_PSC_SICR_SIM(x)           (((x)&0x07)<<0)
#define MCF_PSC_SICR_SHDIR            (0x10)
#define MCF_PSC_SICR_DTS              (0x20)
#define MCF_PSC_SICR_AWR              (0x40)
#define MCF_PSC_SICR_ACRB             (0x80)
#define MCF_PSC_SICR_SIM_UART         (0x00)
#define MCF_PSC_SICR_SIM_MODEM8       (0x01)
#define MCF_PSC_SICR_SIM_MODEM16      (0x02)
#define MCF_PSC_SICR_SIM_AC97         (0x03)
#define MCF_PSC_SICR_SIM_SIR          (0x04)
#define MCF_PSC_SICR_SIM_MIR          (0x05)
#define MCF_PSC_SICR_SIM_FIR          (0x06)

/* Bit definitions and macros for MCF_PSC_IRCR1 */
#define MCF_PSC_IRCR1_SPUL            (0x01)
#define MCF_PSC_IRCR1_SIPEN           (0x02)
#define MCF_PSC_IRCR1_FD              (0x04)

/* Bit definitions and macros for MCF_PSC_IRCR2 */
#define MCF_PSC_IRCR2_NXTEOF          (0x01)
#define MCF_PSC_IRCR2_ABORT           (0x02)
#define MCF_PSC_IRCR2_SIPREQ          (0x04)

/* Bit definitions and macros for MCF_PSC_IRMDR */
#define MCF_PSC_IRMDR_M_FDIV(x)       (((x)&0x7F)<<0)
#define MCF_PSC_IRMDR_FREQ            (0x80)

/* Bit definitions and macros for MCF_PSC_IRFDR */
#define MCF_PSC_IRFDR_F_FDIV(x)       (((x)&0x0F)<<0)

/* Bit definitions and macros for MCF_PSC_RFCNT */
#define MCF_PSC_RFCNT_CNT(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_TFCNT */
#define MCF_PSC_TFCNT_CNT(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_RFSR */
#define MCF_PSC_RFSR_EMT              (0x0001)
#define MCF_PSC_RFSR_ALARM            (0x0002)
#define MCF_PSC_RFSR_FU               (0x0004)
#define MCF_PSC_RFSR_FRMRY            (0x0008)
#define MCF_PSC_RFSR_OF               (0x0010)
#define MCF_PSC_RFSR_UF               (0x0020)
#define MCF_PSC_RFSR_RXW              (0x0040)
#define MCF_PSC_RFSR_FAE              (0x0080)
#define MCF_PSC_RFSR_FRM(x)           (((x)&0x000F)<<8)
#define MCF_PSC_RFSR_TAG              (0x1000)
#define MCF_PSC_RFSR_TXW              (0x4000)
#define MCF_PSC_RFSR_IP               (0x8000)
#define MCF_PSC_RFSR_FRM_BYTE0        (0x0800)
#define MCF_PSC_RFSR_FRM_BYTE1        (0x0400)
#define MCF_PSC_RFSR_FRM_BYTE2        (0x0200)
#define MCF_PSC_RFSR_FRM_BYTE3        (0x0100)

/* Bit definitions and macros for MCF_PSC_TFSR */
#define MCF_PSC_TFSR_EMT              (0x0001)
#define MCF_PSC_TFSR_ALARM            (0x0002)
#define MCF_PSC_TFSR_FU               (0x0004)
#define MCF_PSC_TFSR_FRMRY            (0x0008)
#define MCF_PSC_TFSR_OF               (0x0010)
#define MCF_PSC_TFSR_UF               (0x0020)
#define MCF_PSC_TFSR_RXW              (0x0040)
#define MCF_PSC_TFSR_FAE              (0x0080)
#define MCF_PSC_TFSR_FRM(x)           (((x)&0x000F)<<8)
#define MCF_PSC_TFSR_TAG              (0x1000)
#define MCF_PSC_TFSR_TXW              (0x4000)
#define MCF_PSC_TFSR_IP               (0x8000)
#define MCF_PSC_TFSR_FRM_BYTE0        (0x0800)
#define MCF_PSC_TFSR_FRM_BYTE1        (0x0400)
#define MCF_PSC_TFSR_FRM_BYTE2        (0x0200)
#define MCF_PSC_TFSR_FRM_BYTE3        (0x0100)

/* Bit definitions and macros for MCF_PSC_RFCR */
#define MCF_PSC_RFCR_CNTR(x)          (((x)&0x0000FFFFUL)<<0)
#define MCF_PSC_RFCR_TXW_MSK          (0x00040000UL)
#define MCF_PSC_RFCR_OF_MSK           (0x00080000UL)
#define MCF_PSC_RFCR_UF_MSK           (0x00100000UL)
#define MCF_PSC_RFCR_RXW_MSK          (0x00200000UL)
#define MCF_PSC_RFCR_FAE_MSK          (0x00400000UL)
#define MCF_PSC_RFCR_IP_MSK           (0x00800000UL)
#define MCF_PSC_RFCR_GR(x)            (((x)&0x00000007UL)<<24)
#define MCF_PSC_RFCR_FRMEN            (0x08000000UL)
#define MCF_PSC_RFCR_TIMER            (0x10000000UL)
#define MCF_PSC_RFCR_WRITETAG         (0x20000000UL)
#define MCF_PSC_RFCR_SHADOW           (0x80000000UL)
                                                 
/* Bit definitions and macros for MCF_PSC_TFCR */
#define MCF_PSC_TFCR_CNTR(x)          (((x)&0x0000FFFFUL)<<0)
#define MCF_PSC_TFCR_TXW_MSK          (0x00040000UL)
#define MCF_PSC_TFCR_OF_MSK           (0x00080000UL)
#define MCF_PSC_TFCR_UF_MSK           (0x00100000UL)
#define MCF_PSC_TFCR_RXW_MSK          (0x00200000UL)
#define MCF_PSC_TFCR_FAE_MSK          (0x00400000UL)
#define MCF_PSC_TFCR_IP_MSK           (0x00800000UL)
#define MCF_PSC_TFCR_GR(x)            (((x)&0x00000007UL)<<24)
#define MCF_PSC_TFCR_FRMEN            (0x08000000UL)
#define MCF_PSC_TFCR_TIMER            (0x10000000UL)
#define MCF_PSC_TFCR_WRITETAG         (0x20000000UL)
#define MCF_PSC_TFCR_SHADOW           (0x80000000UL)
                                                 
/* Bit definitions and macros for MCF_PSC_RFAR */
#define MCF_PSC_RFAR_ALARM(x)         (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_TFAR */
#define MCF_PSC_TFAR_ALARM(x)         (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_RFRP */
#define MCF_PSC_RFRP_READ(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_TFRP */
#define MCF_PSC_TFRP_READ(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_RFWP */
#define MCF_PSC_RFWP_WRITE(x)         (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_TFWP */
#define MCF_PSC_TFWP_WRITE(x)         (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_RLRFP */
#define MCF_PSC_RLRFP_LFP(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_TLRFP */
#define MCF_PSC_TLRFP_LFP(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_RLWFP */
#define MCF_PSC_RLWFP_LFP(x)          (((x)&0x01FF)<<0)

/* Bit definitions and macros for MCF_PSC_TLWFP */
#define MCF_PSC_TLWFP_LFP(x)          (((x)&0x01FF)<<0)

/****************************************************************************
 *
 * SDRAM Controller (SDRAMC) Defines
 *
 */

/* Register read/write macros */
#define MCF_SDRAMC_SDRAMDS      (*(vuint32*)(void*)(&__MBAR[0x000004]))
#define MCF_SDRAMC_CS0CFG       (*(vuint32*)(void*)(&__MBAR[0x000020]))
#define MCF_SDRAMC_CS1CFG       (*(vuint32*)(void*)(&__MBAR[0x000024]))
#define MCF_SDRAMC_CS2CFG       (*(vuint32*)(void*)(&__MBAR[0x000028]))
#define MCF_SDRAMC_CS3CFG       (*(vuint32*)(void*)(&__MBAR[0x00002C]))
#define MCF_SDRAMC_CSnCFG(x)    (*(vuint32*)(void*)(&__MBAR[0x000020 \
                                                        +((x)*0x004)]))
#define MCF_SDRAMC_SDMR         (*(vuint32*)(void*)(&__MBAR[0x000100]))
#define MCF_SDRAMC_SDCR         (*(vuint32*)(void*)(&__MBAR[0x000104]))
#define MCF_SDRAMC_SDCFG1       (*(vuint32*)(void*)(&__MBAR[0x000108]))
#define MCF_SDRAMC_SDCFG2       (*(vuint32*)(void*)(&__MBAR[0x00010C]))

/* Bit definitions and macros for MCF_SDRAMC_SDRAMDS */
#define MCF_SDRAMC_SDRAMDS_SB_D(x)         (((x)&0x00000003UL)<<0)
#define MCF_SDRAMC_SDRAMDS_SB_S(x)         (((x)&0x00000003UL)<<2)
#define MCF_SDRAMC_SDRAMDS_SB_A(x)         (((x)&0x00000003UL)<<4)
#define MCF_SDRAMC_SDRAMDS_SB_C(x)         (((x)&0x00000003UL)<<6)
#define MCF_SDRAMC_SDRAMDS_SB_E(x)         (((x)&0x00000003UL)<<8)
#define MCF_SDRAMC_SDRAMDS_DRIVE_8MA       (0x02)
#define MCF_SDRAMC_SDRAMDS_DRIVE_16MA      (0x01)
#define MCF_SDRAMC_SDRAMDS_DRIVE_24MA      (0x00)
#define MCF_SDRAMC_SDRAMDS_DRIVE_NONE      (0x03)

/* Bit definitions and macros for MCF_SDRAMC_CSnCFG */
#define MCF_SDRAMC_CSnCFG_CSSZ(x)          (((x)&0x0000001FUL)<<0)
#define MCF_SDRAMC_CSnCFG_CSBA(x)          (((x)&0x00000FFFUL)<<20)
#define MCF_SDRAMC_CSnCFG_CSSZ_DIABLE      (0x00000000UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_1MBYTE      (0x00000013UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_2MBYTE      (0x00000014UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_4MBYTE      (0x00000015UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_8MBYTE      (0x00000016UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_16MBYTE     (0x00000017UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_32MBYTE     (0x00000018UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_64MBYTE     (0x00000019UL)
#define MCF_SDRAMC_CSnCFG_CSSZ_128MBYTE    (0x0000001AUL)
#define MCF_SDRAMC_CSnCFG_CSSZ_256MBYTE    (0x0000001BUL)
#define MCF_SDRAMC_CSnCFG_CSSZ_512MBYTE    (0x0000001CUL)
#define MCF_SDRAMC_CSnCFG_CSSZ_1GBYTE      (0x0000001DUL)
#define MCF_SDRAMC_CSnCFG_CSSZ_2GBYTE      (0x0000001EUL)
#define MCF_SDRAMC_CSnCFG_CSSZ_4GBYTE      (0x0000001FUL)
                                                      
/* Bit definitions and macros for MCF_SDRAMC_SDMR */
#define MCF_SDRAMC_SDMR_CMD                (0x00010000UL)
#define MCF_SDRAMC_SDMR_AD(x)              (((x)&0x00000FFFUL)<<18)
#define MCF_SDRAMC_SDMR_BNKAD(x)           (((x)&0x00000003UL)<<30)
#define MCF_SDRAMC_SDMR_BNKAD_LMR          (0x00000000UL)
#define MCF_SDRAMC_SDMR_BNKAD_LEMR         (0x40000000UL)

/* Bit definitions and macros for MCF_SDRAMC_SDCR */
#define MCF_SDRAMC_SDCR_IPALL              (0x00000002UL)
#define MCF_SDRAMC_SDCR_IREF               (0x00000004UL)
#define MCF_SDRAMC_SDCR_BUFF               (0x00000010UL)
#define MCF_SDRAMC_SDCR_DQS_OE(x)          (((x)&0x0000000FUL)<<8)
#define MCF_SDRAMC_SDCR_RCNT(x)            (((x)&0x0000003FUL)<<16)
#define MCF_SDRAMC_SDCR_DRIVE              (0x00400000UL)
#define MCF_SDRAMC_SDCR_AP                 (0x00800000UL)
#define MCF_SDRAMC_SDCR_MUX(x)             (((x)&0x00000003UL)<<24)
#define MCF_SDRAMC_SDCR_REF                (0x10000000UL)
#define MCF_SDRAMC_SDCR_DDR                (0x20000000UL)
#define MCF_SDRAMC_SDCR_CKE                (0x40000000UL)
#define MCF_SDRAMC_SDCR_MODE_EN            (0x80000000UL)
                                                      
/* Bit definitions and macros for MCF_SDRAMC_SDCFG1 */
#define MCF_SDRAMC_SDCFG1_WTLAT(x)         (((x)&0x00000007UL)<<4)
#define MCF_SDRAMC_SDCFG1_REF2ACT(x)       (((x)&0x0000000FUL)<<8)
#define MCF_SDRAMC_SDCFG1_PRE2ACT(x)       (((x)&0x00000007UL)<<12)
#define MCF_SDRAMC_SDCFG1_ACT2RW(x)        (((x)&0x00000007UL)<<16)
#define MCF_SDRAMC_SDCFG1_RDLAT(x)         (((x)&0x0000000FUL)<<20)
#define MCF_SDRAMC_SDCFG1_SWT2RD(x)        (((x)&0x00000007UL)<<24)
#define MCF_SDRAMC_SDCFG1_SRD2RW(x)        (((x)&0x0000000FUL)<<28)

/* Bit definitions and macros for MCF_SDRAMC_SDCFG2 */
#define MCF_SDRAMC_SDCFG2_BL(x)            (((x)&0x0000000FUL)<<16)
#define MCF_SDRAMC_SDCFG2_BRD2WT(x)        (((x)&0x0000000FUL)<<20)
#define MCF_SDRAMC_SDCFG2_BWT2RW(x)        (((x)&0x0000000FUL)<<24)
#define MCF_SDRAMC_SDCFG2_BRD2PRE(x)       (((x)&0x0000000FUL)<<28)
                                                           
/****************************************************************************
 *
 * Slice Timer (SLT) Defines
 *
 */

/* Register read/write macros */
#define MCF_SLT_SLTCNT0      (*(vuint32*)(void*)(&__MBAR[0x000900]))
#define MCF_SLT_SCR0         (*(vuint32*)(void*)(&__MBAR[0x000904]))
#define MCF_SLT_SCNT0        (*(vuint32*)(void*)(&__MBAR[0x000908]))
#define MCF_SLT_SSR0         (*(vuint32*)(void*)(&__MBAR[0x00090C]))
#define MCF_SLT_SLTCNT1      (*(vuint32*)(void*)(&__MBAR[0x000910]))
#define MCF_SLT_SCR1         (*(vuint32*)(void*)(&__MBAR[0x000914]))
#define MCF_SLT_SCNT1        (*(vuint32*)(void*)(&__MBAR[0x000918]))
#define MCF_SLT_SSR1         (*(vuint32*)(void*)(&__MBAR[0x00091C]))
#define MCF_SLT_SLTCNT(x)    (*(vuint32*)(void*)(&__MBAR[0x000900 \
                                                    +((x)*0x010)]))
#define MCF_SLT_SCR(x)       (*(vuint32*)(void*)(&__MBAR[0x000904 \
                                                    +((x)*0x010)]))
#define MCF_SLT_SCNT(x)      (*(vuint32*)(void*)(&__MBAR[0x000908 \
                                                    +((x)*0x010)]))
#define MCF_SLT_SSR(x)       (*(vuint32*)(void*)(&__MBAR[0x00090C \
                                                    +((x)*0x010)]))

/* Bit definitions and macros for MCF_SLT_SCR */
#define MCF_SLT_SCR_TEN    (0x01000000UL)
#define MCF_SLT_SCR_IEN    (0x02000000UL)
#define MCF_SLT_SCR_RUN    (0x04000000UL)
                                      
/* Bit definitions and macros for MCF_SLT_SSR */
#define MCF_SLT_SSR_ST     (0x01000000UL)
#define MCF_SLT_SSR_BE     (0x02000000UL)

/****************************************************************************
 *
 * BCM5222 (PHY for the MCF5485EVB) Defines
 *
 */

/* MII Register Addresses */
#define BCM5222_CTRL                (0x00)
#define BCM5222_STAT                (0x01)
#define BCM5222_PHY_ID1             (0x02)
#define BCM5222_PHY_ID2             (0x03)
#define BCM5222_AN_ADV              (0x04)
#define BCM5222_AN_LINK_PAR         (0x05)
#define BCM5222_AN_EXP              (0x06)
#define BCM5222_AN_NPR              (0x07)
#define BCM5222_LINK_NPA            (0x08)
#define BCM5222_ACSR                (0x18)

/* Bit definitions and macros for BCM5222_CTRL */
#define BCM5222_CTRL_RESET          (0x8000)
#define BCM5222_CTRL_LOOP           (0x4000)
#define BCM5222_CTRL_SPEED          (0x2000)
#define BCM5222_CTRL_ANE            (0x1000)
#define BCM5222_CTRL_PD             (0x0800)
#define BCM5222_CTRL_ISOLATE        (0x0400)
#define BCM5222_CTRL_RESTART_AN     (0x0200)
#define BCM5222_CTRL_FDX            (0x0100)
#define BCM5222_CTRL_COL_TEST       (0x0080)


/* Bit definitions and macros for BCM5222_STAT */
#define BCM5222_STAT_100BT4         (0x8000)
#define BCM5222_STAT_100BTX_FDX     (0x4000)
#define BCM5222_STAT_100BTX         (0x2000)
#define BCM5222_STAT_10BT_FDX       (0x1000)
#define BCM5222_STAT_10BT           (0x0800)
#define BCM5222_STAT_NO_PREAMBLE    (0x0040)
#define BCM5222_STAT_AN_COMPLETE    (0x0020)
#define BCM5222_STAT_REMOTE_FAULT   (0x0010)
#define BCM5222_STAT_AN_ABILITY     (0x0008)
#define BCM5222_STAT_LINK           (0x0004)
#define BCM5222_STAT_JABBER         (0x0002)
#define BCM5222_STAT_EXTENDED       (0x0001)

/* Bit definitions and macros for BCM5222_AN_ADV */
#define BCM5222_AN_ADV_NEXT_PAGE    (0x8001)
#define BCM5222_AN_ADV_REM_FAULT    (0x2001)
#define BCM5222_AN_ADV_PAUSE        (0x0401)
#define BCM5222_AN_ADV_100BT4       (0x0201)
#define BCM5222_AN_ADV_100BTX_FDX   (0x0101)
#define BCM5222_AN_ADV_100BTX       (0x0081)
#define BCM5222_AN_ADV_10BT_FDX     (0x0041)
#define BCM5222_AN_ADV_10BT         (0x0021)
#define BCM5222_AN_ADV_802_3        (0x0001)

/* Bit definitions and macros for BCM5222_ACSR */
#define BCM5222_ACSR_100BTX         (0x0002)
#define BCM5222_ACSR_FDX            (0x0001)

#endif /* _MCF5485_H_ */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:40      For: Xilinx-Samsung
 * File Name: trm5485.h     File Revision: 6.0.2.2 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 23887    Output checksum: 60677
 */
