/*
 * Copyright Notice:
 * Copyright Treck Incorporated  1997 - 2011
 * Copyright Elmic Systems Japan 1997 - 2011
 * No portions or the complete contents of this file may be copied or
 * distributed in any form (including but not limited to printed or electronic
 * forms) without the expressed written consent of Treck Incorporated OR
 * Elmic Systems.  Copyright laws and International Treaties protect the
 * contents of this file.  Unauthorized use is prohibited.
 * All rights reserved.
 *
 * Description: IP router functions 
 * Filename: trroute.c
 * Author: Odile
 * Date Created: 11/10/97
 * $Source: source/trroute.c $
 *
 * Modification History
 * $Revision: 6.0.2.31 $
 * $Date: 2011/12/06 12:34:59CST $
 * $Author: pcarney $
 * $ProjectName: /home/mks/proj/tcpip.pj $
 * $ProjectRevision: 6.0.1.23 $
 */

/*
 * Include
 */
#include <trsocket.h> 
#ifdef TM_RT_OUTPUT
#include <stdio.h>
#endif /* TM_RT_OUTPUT */
#include <trmacro.h>
#include <trtype.h>
#include <trproto.h>
#include <trglobal.h>


/*
 * Local Macros constant definitions
 */

/* Supress unreferenced macro error for PC-LINT */
#ifdef TM_LINT
LINT_UNREF_MACRO(TM_RT_BITS_PER_BYTE)
#endif /* TM_LINT */
#ifndef TM_RT_OUTPUT
#ifdef TM_LINT
LINT_UNREF_MACRO(tm_rt_rte_to_leaf)
#endif /* TM_LINT */
#endif /* !TM_RT_OUTPUT */

/*
 * TRY_BIT_OFFSET should only be set if
 * There are a lot of entries in the routing table
 */
/*#define TM_TRY_BIT_OFFSET */

/* Null node */
#define TM_RADIX_NULL_PTR (ttRtRadixNodePtr)0

#define TM_RT_BITS_PER_BYTE       8

/*
 * Given a leaf, corresponding routing entry
 */
#define tm_rt_leaf_to_rte(leafPtr) ((ttRteEntryPtr)((ttVoidPtr)leafPtr))

/* Patricia Tree flags */
/* Node (not leaf) in Patricia tree */
#define TM_RTN_NODE         (tt8Bit)0x01
/* Root leaf/node in Patricia tree */
#define TM_RTN_ROOT         (tt8Bit)0x02
/* Node or leaf is in the tree */
#define TM_RTN_IN           (tt8Bit)0x04
 /* Mask field is not all ones */
#define TM_RTN_NETWORK      (tt8Bit)0x08
/* Leaf is part of a network list */
#define TM_RTN_LIST_NETWORK (tt8Bit)0x10

/* rtnNSBitMask macros */
#ifdef TM_LITTLE_ENDIAN
#define TM_RTN_HIGH_BIT      0x80
#define TM_RTN_LOW_BIT       0x01
#else /* !TM_LITTLE_ENDIAN */
#define TM_RTN_HIGH_BIT      (tt4IpAddress)TM_UL(0x80000000)
#define TM_RTN_LOW_BIT       (tt4IpAddress)TM_UL(0x00000001)
#endif /* TM_LITTLE_ENDIAN */

/*
 * Path MTU plateau values as defined in RFC 1191
 */
/* TM_PMTU_MAX defined in trmacro.h */
/*#define TM_PMTU_MAX              (tt16Bit)65535 */
#define TM_PMTU_SECOND_PLATEAU   (tt16Bit)32000
#define TM_PMTU_16MB_TKN_RING    (tt16Bit)17914
#define TM_PMTU_802_4            (tt16Bit)8166
#define TM_PMTU_FDDI             (tt16Bit)4352
#define TM_PMTU_802_5            (tt16Bit)2002
#define TM_PMTU_802_3            (tt16Bit)1492
#define TM_PMTU_SLIP             (tt16Bit)1006
#define TM_PMTU_ARCNET           (tt16Bit)508
#define TM_PMTU_LOW_DELAY        (tt16Bit)296
#define TM_PMTU_NULL             (tt16Bit)0

/* Minimum Path MTU that we will accept */
#ifndef TM_PMTU_MINIMUM
#define TM_PMTU_MINIMUM          TM_PMTU_LOW_DELAY
#endif /* TM_PMTU_MINIMUM */

/* Try next lower Path MTU estimate */
#define TM_PMTU_LOWER            (tt8Bit)0
/* Try next higher Path MTU estimate */
#define TM_PMTU_HIGHER           (tt8Bit)1

/*
 * Number of entries in the RFC 1192 MTU plateau table.
 * From TM_PMTU_MAX to TM_PMTU_LOW_DELAY, not including TM_PMTU_NUL
 */
#define TM_PMTU_ENTRIES          10

/* type definitions */
typedef struct ts6RtDevMtuUpdate
{
    ttDeviceEntryPtr dmuDevEntryPtr;
    tt16Bit          dmuNewMtu;
    tt16Bit          dmuAf;
} tt6RtDevMtuUpdate;

typedef tt6RtDevMtuUpdate TM_FAR * tt6RtDevMtuUpdatePtr;

#ifdef TM_USE_STRONG_ESL
#endif /* TM_USE_STRONG_ESL */


/*
 * static functions declarations
 */


/*
 * Reset all routing caches pointing to either an indirect route (if
 * flag is TM_RTE_INDIRECT), or an lan ARP/Mcast/Bcast mapping
 * entries (if flag is TM_RTE_CLONED).
 */
#ifdef TM_LOCK_NEEDED
static void tfRtResetCaches(tt16Bit           flag, 
                            int               addressFamilyFlag,
                            tt8Bit            lockFlag);
#define tm_rt_reset_caches(rteFlag, afFlag, lockFlag) \
    tfRtResetCaches(rteFlag, afFlag, lockFlag) 
#else /* !TM_LOCK_NEEDED */
static void tfRtResetCaches(tt16Bit rteFlag, int addressFamilyFlag);
#define tm_rt_reset_caches(rteFlag, afFlag, lockFlag) \
    tfRtResetCaches(rteFlag, afFlag)
#endif /* TM_LOCK_NEEDED */

#ifdef TM_USE_ARP_FLUSH
#ifndef TM_SNMP_CACHE
static int tfRtRemArpEntriesCB(ttRteEntryPtr rtePtr,  ttVoidPtr parmPtr);
#else /* TM_SNMP_CACHE */
static int tf4RtArpNodeDeleteCB(ttNodePtr nodePtr, ttGenericUnion  genParam);
#endif /* TM_SNMP_CACHE */
#endif /* TM_USE_ARP_FLUSH */

/*
 * Delete an IPv4 ARP entry pointed to by rtePtr.
 */
static void tf4RtArpDelByRte(ttRteEntryPtr rtePtr, int flag);


/*
 * ARP entry timer
 */
TM_NEARCALL static void tfRtArpTimer (ttVoidPtr      timerBlockPtr,
                          ttGenericUnion userParm1,
                          ttGenericUnion userParm2);

/*
 * RIP and indirect routes timer
 */
TM_NEARCALL static void tfRtTimer (ttVoidPtr      timerBlockPtr,
                       ttGenericUnion userParm1,
                       ttGenericUnion userParm2);

/*
 * Routing Entry processing function
 * Remove any obsolete indirect entry (pointing to a non up local entry)
 * Also remove any multicast entry which have same device/multihome index
 * as voidPtr (local routing entry pointer)
 */
TM_NEARCALL static int tfRtRemObsLocalCB(ttRteEntryPtr rtePtr, ttVoidPtr voidPtr);

/* 
 * Remove a host entry, and remove all gateways (except default routers)
 * pointing to it
 */
static void tfRtRemoveHostEntry(ttRteEntryPtr rtePtr,
                                unsigned int  ownerCount,
                                tt16Bit       af);
/*
 * Remove any gateway pointing to the passed cloned entry (voidPtr).
 */
TM_NEARCALL static int tfRtRemoveGwToClonedCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr);


static int tfRtUpdateDevMtuCB(ttRteEntryPtr rtePtr, ttVoidPtr newMtuPtr);

/*
 * Routing Entry processing function
 * Age any RIP/REDIRECT entry and remove it when expired
 */
static int tfRtAgeEntryCB(ttRteEntryPtr rtePtr, ttVoidPtr voidPtr);

/*
 * Routing Entry processing function
 * Age any ARP entry and remove it when expired
 */
static int tfRtArpAgeEntryCB(ttRteEntryPtr rtePtr, ttVoidPtr voidPtr);

/*
 * returns with no error if the routing entry is an ARP entry corresponding
 * to address (as specifiend in ARP mapping command) contained in
 * arpMapping structure.
 */
static int tfRtArpMappingCB(ttRteEntryPtr rtePtr, ttVoidPtr arpMappingPtr);

/*
 * If rtePtr is an ARP entry not being resolved and not being referenced
 * by another pointer, check whether its TTL is less than the current
 * least used ARP entry if any (stored in leastUsedRtePtrPtr). It if is
 * less, or there was none stored in leastUsedRtePtrPtr, store it in
 * leastUsedRtePtrPtr.
 */
TM_NEARCALL static int tfRtArpGetLeastUsedCB( ttRteEntryPtr rtePtr,
                                  ttVoidPtr leastUsedRtePtrPtr );

/* 
 * Add an ARP entry. Common code between IPv4 and IPv6.
 * IPv4: called from tf4RtArpAddEntry which is itself called when a user
 * adds an ARP entry, or in ARP incoming processing.
 * IPv6: Called from tf6RtArpAddEntry which is itself called when a user
 * adds an ARP entry.
 */
static int tfRtArpAddEntry ( 
    ttRtCacheEntryPtr  rtcCachePtr,
    tt8BitPtr          physAddrPtr,
    tt32Bit            ttl,
    tt8Bit             physAddrLen 
                           );
/*Delete a route. Called by tf6RtDelRoute() or tf4RtDelRoute() */
static int tfRtDelRoute(ttRteEntryPtr rtePtr, tt16Bit rteCreateFlag);

#ifdef TM_RT_OUTPUT
/*
 * Routing Entry processing function
 * Print a routing entry
 */
static int tfRtPrintEntryCB(ttRteEntryPtr rtePtr, ttVoidPtr voidPtr);

/* Print a leaf node */
static void tfRtPrintLeaf(ttCharPtr msg, ttRtRadixNodePtr leafPtr);

#ifdef TM_RT_INT_OUTPUT
/* Print an internal node */
static void tfRtPrintNode(ttCharPtr msg, ttRtRadixNodePtr nodePtr);
#endif /* TM_RT_INT_OUTPUT */

#endif /* TM_RT_OUTPUT */

/*
 * Find an entry in the routing table for the destination IP address
 * This function is called by tfRtFindNClone() after exclusive access to
 * the routing table has been obtained.
 */
static ttRteEntryPtr tfRtFind(
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr     devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress        destIpAddr
    );

/* Get router entry for the Destination IP address
 * If REJECT flag is set, return null pointer.
 * If we got a LAN network entry (TM_RTN_CLONE set), clone a ARP router entry.
 */
static ttRteEntryPtr tfRtFindNClone(
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr     devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress        destIpAddr
    );
#ifdef TM_USE_STOP_TRECK
static int tfRtRemEntriesCB(ttRteEntryPtr rtePtr,  ttVoidPtr parmPtr);
#endif /* TM_USE_STOP_TRECK */

/*
 * tfRtFindIpAddrIpNet() is called by tf4RtDelRoute(), tf6RtDelRoute(),
 * tf4RtGetDefault(), tf4RtModifyDefGatewayMhome(), tfRtArpCommand(),
 * tfRtRip(), and by tfRtAddEntry() to
 * try and find the unique route entry for a given destination IP address
 * destination network mask.
 * Function is called with routing table locked. So we don't need to
 * lock the table.
 */
ttRteEntryPtr tfRtFindIpAddrIpNet( 
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress   ipAddress,
    tt4IpAddress   netMask
    );

/*
 * tfRtFindSockAddr() is called by tfNgRtDestExists(), tfRtInterface().
 * Try and find a route entry for a given sock addr destination IP address
 * destination network mask/prefix.
 * Function is called with routing table locked. So we don't need to
 * lock the table.
 */
static ttRteEntryPtr tfRtFindSockAddr(
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr                        devPtr,
#endif /* TM_USE_STRONG_ESL */
    const struct sockaddr_storage TM_FAR *  sockAddrPtr,
    tt4IpAddress                            netMask
    );

/*
 * tfRtRtRemoveEntry is called by tf4RtDelRoute(), and tf6RtDelRoute().
 * Try and remove the routing table entry pointed to by rtEntryPtr from
 * the routing table.
 */
static void tfRtRemoveEntry(ttRteEntryPtr rtePtr);

/*
 *  Called by tfRtAddRoute() to insert a new entry in the routing table
 *  Also called by tfRtGet()/tfRtClone() when we need to clone an
 *  entry for ARP. Also called by tfRtInit(), tf4RtAddDefGw(), 
 *  tf4RtInsertEmptyDefaultGateway() for IPV4 default gateway.
 */
static int  tfRtAddEntry(
    ttRtRadixNodePtr        nodePtr, /* Node to start the search at */
    ttRteEntryPtr           rtePtr, /* Newly allocated routing entry */
    ttDeviceEntryPtr        devPtr, /* device for the route */
/* destination network address */
    tt4IpAddress            ipAddr,
    tt4IpAddress            netMask,
/* multihome index for local routes */
    tt16Bit                 mhomeIndex,
    int                     hops, /* metrics for non local routes */
    tt32Bit                 ttl, /* Time to live for this entry */
/* flag to indicate creator local/static/redirect/RIP */
    tt16Bit                 flags );

/*
 * Clone a routing entry. Return 0 on success.
 */
static ttRteEntryPtr tfRtClone( 
    ttRteEntryPtr           rtePtrPtr,
    tt4IpAddress            destIpAddr
    );

/*
 * Cache routing entry (and corresponding ARP entry if routing entry
 * is indirect).
 */
static void tfRtCacheGet (ttRtCacheEntryPtr rtcPtr, ttRteEntryPtr rtePtr);

/*
 * copy oldNode to newNode, reset old node's in flag
 */
static void tfRtCopyNode( ttRtRadixNodePtr oldNodePtr,
                          ttRtRadixNodePtr newNodePtr );

static void tfRtInsertNet( ttRtRadixNodePtr     nodePtr,
                           ttRtRadixNodePtr     leafPtr,
                           int                  maskLen );

/* 
 * Called by the tf{4,6}RtAddxxx() functions. 
 * Allocate a routing entry.
 * Lock the tree if needed, Initializing the owner count to 1.
 * Map IPV4 address to IPv4-mapped IPv6 address if needed
 * Copy IP address in ttRteEntry structure.
 */
static ttRteEntryPtr tfRtAllocNLockNCopy(tt8Bit          needLock);

/*
 * Called by the tf{4,6}RtAddxxx() functions. 
 * Insert entry in SNMP routing cache, if insertion to Patricia tree succeeded.
 * UNLOCK the tree to release access to the routing table, if needed.
 * Free routing entry, if insertion failed. 
 * Ignore error for local route if errorCode is TM_EALREADY, since local
 * routes is for the same interface.
 */
static int tfRtCacheUnlockNClean( ttRteEntryPtr rtePtr,
                                  tt8Bit needUnLock, int errorCode,
                                  tt16Bit flags );

/* 
 * Common function to remove entries to be removed after a tree walk.
 */
static void tfRtTreeRem(void);

/* 
 * Common function to unlock the tree.
 */
static void tfRtTreeUnlock(void);

/* 
 * Common function to unlock the tree, and reset
 * the caches, when entries have been added/deleted 
 */
static void tfRtTreeUnlockNResetCaches(void);




#ifdef TM_PMTU_DISC
static tt16Bit tfRtPickNextPlateauMtu(tt16Bit currentMtu, tt8Bit flag);
#endif /* TM_PMTU_DISC */

#if (defined(TM_PMTU_DISC) || defined(TM_6_PMTU_DISC))
static int tfRtAddPathMtuEntry(
#ifdef TM_USE_STRONG_ESL
                               ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                                ttRteEntryPtr   rtePtr,
                                ttIpAddress     destIpAddress,
                                tt32Bit         timeout,
                                tt16Bit         flags,
                                tt16Bit         nextMtu );
#endif /* TM_PMTU_DISC || TM_6_PMTU_DISC */


#ifdef TM_PMTU_DISC
/*
 * RFC1191 MTU plateau table
 */
static const tt16Bit  TM_CONST_QLF tlMtuPlateauValues[TM_PMTU_ENTRIES + 1] =
{
    TM_PMTU_MAX,         TM_PMTU_SECOND_PLATEAU, TM_PMTU_16MB_TKN_RING,
    TM_PMTU_802_4,       TM_PMTU_FDDI,           TM_PMTU_802_5,
    TM_PMTU_802_3,       TM_PMTU_SLIP,           TM_PMTU_ARCNET,
    TM_PMTU_LOW_DELAY,   TM_PMTU_NULL
};
#endif /* TM_PMTU_DISC */

/*
 * NOTE about locking multiple resources and deadlocks: Care must be taken
 * when locking more than one resource, that if a task obtains the lock to at
 * least 2 different resources, it always happens in the same order to avoid
 * deadlock. For example, if TASK1 locks res1, TASK2 locks res2, TASK1
 * tries and locks res2 and Pend, TASK2 tries and locks res1 and Pend, we get
 * into a deadlock. Both TASK1 and TASK2 will pend forever waiting for the
 * other task to release the resource it locked.
 * This is easily avoided by not locking one of the 2 resources while
 * the other one is locked, or if we must obtain the lock to 2 or more
 * resources that the order in which it happens is always the same. In our
 * case we never lock the tree after having obtained the lock to the recycle
 * queue. Note that we do sometimes lock the recycle queue after we obtained
 * the lock for the tree, but this will not result in a deadlock since we
 * never obtain the lock to the tree after having obtained the lock to the
 * recycle queue.
 */


/*
 * Local macros function definitions
 */
#ifndef TM_LITTLE_ENDIAN
 /* bitmask of bit to be tested */
#define tm_rtn_bit_mask(bitnumber)        (TM_RTN_HIGH_BIT >> bitnumber)
#endif /* TM_LITTLE_ENDIAN */

#define tm_rtn_bit_set(nodePtr, key)    \
                 (tm_ip_and(nodePtr->rtnNSBitMask, key) != TM_IP_ZERO)

/*
 * Left child of an internal node
 */
#define tm_rt_left_child(nodePtr)  nodePtr->rtnNSLeftPtr

/*
 * Right child of an internal node
 */
#define tm_rt_right_child(nodePtr) nodePtr->rtnNSRightPtr

/*
 * Parent of a leaf/node
 */
#define tm_rt_parent(nodePtr)      nodePtr->rtnParentPtr

/*
 * Previous leaf in a chain of duplicated leaves
 */
#define tm_rt_prev_dup(leafPtr)    tm_rt_parent(leafPtr)

/*
 * Next leaf in a chain of duplicated leaves
 */
#define tm_rt_next_dup(leafPtr)    leafPtr->rtnLSDupKeyPtr

/*
 * Previous entry in a chain of network entries (for network checks)
 */
#define tm_rt_prev_net(leafPtr)   leafPtr->rtnLSPrevPtr

/*
 * Next entry in a chain of network entries (for network checks)
 */
#define tm_rt_next_net(nodePtr)   nodePtr->rtnNextPtr

/*
 * Number of matching bits so far down in the tree
 */
#define tm_rt_matching_bits(nodePtr) nodePtr->rtnBitOffset

/*
 * Mask length of a leaf entry
 */
#define tm_rt_mask_len(leafPtr) tm_rt_matching_bits(leafPtr)

/*
 * Given a routing entry, corresponding leaf node
 */
#define tm_rt_rte_to_leaf(rtePtr) (&(rtePtr->rteLeafNode))

/*
 * Given a routing entry, corresponding internal node
 */
#define tm_rt_rte_to_int_node(rtePtr) (&(rtePtr->rteInternalNode))

/*
 * Given a leaf, internal node from the corresponding routing entry
 */
#define tm_rt_leaf_to_int_node(leafPtr) \
    tm_rt_rte_to_int_node(tm_rt_leaf_to_rte(leafPtr))


/*
 * For a given key, find out the child (left or right) of a node (nodePtr)
 *
 * nodePtr, key value args 
 */
#define tm_rt_get_child(nodePtr, key) \
    ((tm_rtn_bit_set(nodePtr, key)) ? \
            tm_rt_right_child(nodePtr) : tm_rt_left_child(nodePtr))

/*
 * given a child and a parent, find the other child of the parent
 *
 * parentPtr value arg, childPtr value arg
 */
#define tm_rt_get_other_child(parentPtr, childPtr) \
    ( (tm_rt_left_child(parentPtr) == childPtr) ? \
        tm_rt_right_child(parentPtr) : \
        tm_rt_left_child(parentPtr) )


/*
 * Given a key, search down the tree starting at nodePtr until we reach a
 * leaf.
 *
 * nodePtr var arg, key value arg
 */
#define tm_rt_tree_search(nodePtr, key) \
{ \
    while (tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_NODE)) \
    { \
        nodePtr = tm_rt_get_child(nodePtr, key); \
    } \
}

/*
 * fetch the left most leaf of the tree
 */
#define tm_rt_tree_leftmost_leaf(nodePtr) \
{ \
    while (tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_NODE)) \
    { \
        nodePtr = tm_rt_left_child(nodePtr); \
    } \
}

#ifdef TM_ERROR_CHECKING
/*
 * Check whether we reached a non valid leaf (null)
 *
 * nodePtr value arg.
 */
#define tm_rt_check_null_leaf(nodePtr) \
{ \
    if (nodePtr == TM_RADIX_NULL_PTR) \
    { \
        tfKernelError("tm_rt_tree_search", \
                      "Null leaf in the tree"); \
        tfRtTreeUnlock(); \
        tm_thread_stop; \
    } \
}
#endif /* TM_ERROR_CHECKING */


/*
 * Given a child's key, set the left or right child of its parent (parentPtr)
 * to its child node (childPtr).
 *
 * parentPtr, childPtr, key value args.
 */
#define tm_rt_set_child_key(parentPtr, childPtr, key) \
{ \
    if (tm_rtn_bit_set(parentPtr, key))  \
    { \
        tm_rt_right_child(parentPtr) = childPtr; \
    } \
    else \
    { \
        tm_rt_left_child(parentPtr) = childPtr; \
    } \
}

/*
 * Given a child's key, set the left and right child of its parent (parentPtr)
 * to its children leaf and node (leafPtr, nodePtr). We know leafPtr is
 * a leaf, but nodePtr could be either a leaf or a node.
 *
 * parentPtr, leafPtr, leafKey, nodePtr value args
 */
#define tm_rt_set_children_key(parentPtr, leafPtr, leafKey, nodePtr) \
{ \
    if (tm_rtn_bit_set(parentPtr, leafKey))  \
    { \
        tm_rt_right_child(parentPtr) = leafPtr; \
        tm_rt_left_child(parentPtr) = nodePtr; \
    } \
    else \
    { \
        tm_rt_left_child(parentPtr) = leafPtr; \
        tm_rt_right_child(parentPtr) = nodePtr; \
    } \
}

#define tm_rt_set_parent(parentPtr, childPtr) \
{ \
    tm_rt_parent(childPtr) = parentPtr; \
}

/*
 * Given a child's key, set the left or right child of its parent (parentPtr)
 * to its child node (childPtr). Set the child node (childPtr)'s parent to
 * parentPtr (childPtr's current grand-parent).
 *
 * parentPtr, childPtr, key value args.
 */
#define tm_rt_set_child_parent_key(parentPtr, childPtr, key) \
{ \
    tm_rt_set_child_key(parentPtr, childPtr, key); \
    tm_rt_set_parent(parentPtr, childPtr); \
}

/*
 * Given a child's key, set the left and right child of its parent (parentPtr)
 * to its children leaf and node (leafPtr, nodePtr). We know leafPtr is
 * a leaf, but nodePtr could be either a leaf or a node. Set both children's
 * parent to the parent (parentPtr).
 *
 * parentPtr, leafPtr, leafKey, nodePtr value args
 */
#define tm_rt_set_children_parent(parentPtr, leafPtr, leafKey, nodePtr) \
{ \
    tm_rt_set_children_key(parentPtr, leafPtr, leafKey, nodePtr); \
    tm_rt_set_parent(parentPtr, leafPtr); \
    tm_rt_set_parent(parentPtr, nodePtr); \
}


/*
 * Given a parent, and one of its children, set the left or right child
 * to its new child node (childPtr).
 *
 * parentPtr, childPtr, key value args.
 */
#define tm_rt_set_new_child(parentPtr, oldChildPtr, newChildPtr) \
{ \
    if (tm_rt_left_child(parentPtr) == oldChildPtr) \
    { \
        tm_rt_left_child(parentPtr) = newChildPtr; \
    } \
    else \
    { \
        tm_rt_right_child(parentPtr) = newChildPtr; \
    } \
}

/*
 * Find insertion point in the tree. We walk up the tree from a reached
 * leaf until we find the parent and (left or right) child for the
 * node that we want to insert (the one that has bitOffset). Our new
 * parent should be the first node that we find on our way up that has a
 * smaller bit offset than the entry that we want to insert.
 * This piece of code is also used to find the node in the tree where
 * we want to insert the non host leaf (for network matches). The match
 * is the child of the node that has a rtnBitOffset (matching bits) < mask
 * length.
 * Note that this piece of code is called with a non zero bitOffset. So
 * a match is guaranteed since the root has a rtnBitoffset of zero.
 *
 * childPtr var arg, parentPtr non initialized var arg, bitOffset value arg.
 */
#define tm_rt_tree_find_insert(childPtr, parentPtr, bitOffset) \
{ \
    parentPtr = tm_rt_parent(childPtr); \
    while (tm_rt_matching_bits(parentPtr) >= bitOffset) \
    { \
        childPtr = parentPtr; \
        parentPtr = tm_rt_parent(childPtr); \
    } \
}

#ifdef TM_ERROR_CHECKING
#define tm_rt_check_dup_nodes(childPtr, bitOffset) \
{ \
    if (   tm_16bit_one_bit_set(childPtr->rtnFlags, TM_RTN_NODE) \
        && (tm_rt_matching_bits(childPtr) == bitOffset) ) \
    { \
        tfKernelError("tfRtAddEntry", " inserting a duplicate node"); \
        tfRtTreeUnlock(); \
        tm_thread_stop; \
    } \
}
#endif /* TM_ERROR_CHECKING */

/*
 * Given a mask length find the node in which to insert the network leaf.
 *
 * nodePtr var arg, tempPtr temp var, bitOffset value arg.
 */
#define tm_rt_tree_find_net_list(nodePtr, tempPtr, bitOffset) \
{ \
    if (bitOffset != TM_16BIT_ZERO) \
    { \
        tm_rt_tree_find_insert(nodePtr, tempPtr, bitOffset); \
    } \
    else \
    { \
        nodePtr = tm_context(tvRtTreeHeadPtr); \
    } \
}

/*
 * Check whether a leaf has the same mask as netMask whose length is maskBits.
 *
 * leafPtr, maskBits, value args.
 */
#define tm_rt_same_mask(leafPtr, maskBits) \
    (tm_rt_mask_len(leafPtr) == maskBits)

/*
 * Remove a leaf from a doubly linked list of network entries. It has
 * already been verified that leafPtr is part of a list of network entries.
 *
 * leafPtr value arg
 */
#define tm_rt_remove_net(leafPtr) \
{ \
    tm_rt_next_net(tm_rt_prev_net(leafPtr)) = tm_rt_next_net(leafPtr); \
    if (tm_rt_next_net(leafPtr) != TM_RADIX_NULL_PTR) \
    { \
        tm_rt_prev_net(tm_rt_next_net(leafPtr)) = tm_rt_prev_net(leafPtr); \
    } \
}

/*
 * Remove a leaf from a doubly linked list of duplicates.
 *
 * leafPtr value arg, prevDupPtr value arg, nxtDupPtr value arg
 */
#define tm_rt_remove_dup_member(leafPtr, prevDupPtr, nxtDupPtr) \
{ \
    tm_rt_next_dup(prevDupPtr) = tm_rt_next_dup(leafPtr); \
    if (nxtDupPtr != TM_RADIX_NULL_PTR) \
    { \
        tm_rt_prev_dup(nxtDupPtr) = prevDupPtr; \
    } \
}


/*
 * Global Functions
 */

/* Called by the Treck initialization routine
 * 1. We initialize the lock entry for the routing table (Patricia Tree)
 * 1. We initialize the pointer to the head of the Patricia tree:
 *    tvRtTreeHeadPtr points to the  address of tvRtRadixTreeHead.rhHeadNode.
 * 3. We then initialize an empty Patricia tree. It is a structure containing
 *    three nodes. The middle node is the head of the tree, and the left
 *    node is the leftmost leaf of the tree (all bits in the key set to
 *    zero), and the right node is the rightmost leaf of the tree (all bits
 *    in the key set to 1). We zero this Patricia tree head structure
 *    tvRtRadixTreeHead, and then we make the  parent pointer of the head
 *    node of the tree point to itself. We make the left leaf parent pointer,
 *    and the right leaf parent pointer point to the head node.
 *    The flags of the left and right leaves indicate that the nodes are
 *    part of the tree, they are at the root structure, and they are
 *    leaves. The flags of the head node indicate that it is root,
 *    part of the tree, and an internal node. The left pointer
 *    of the head node point to the left leaf, and the right pointer of
 *    the head node point to the right leaf. The bit mask of the head
 *    node (used to make the left/right decision) is set to 0x80000000UL for
 *    big endian and 0x00000080UL for little endian CPUS,
 *    which means that we start testing the first bit
 *    (in network byte order). The right leaf key is 0xFFFFFFFFUL. All
 *    other fields are zeroes.
 * 4. We then initialize the router entries recycle queue and its lock
 */

void tfRtInit ( void  )
{

/* Initialize global pointer to head of Patricia tree */
    tm_context(tvRtTreeHeadPtr) = &tm_context(tvRtRadixTreeHead).rthHeadNode;
/* Initialize Patricia tree head structure, which contains the head node,
 * the leftmost and rightmost leaf nodes.
 */
/* parent of head is head itself */
    tm_context(tvRtTreeHeadPtr)->rtnParentPtr = tm_context(tvRtTreeHeadPtr);
/* parent of leftmost leaf is head of tree */
    tm_context(tvRtRadixTreeHead).rthLeftEndNode.rtnParentPtr =
                                                tm_context(tvRtTreeHeadPtr);
/* parent of righmost leaf is head of tree */
    tm_context(tvRtRadixTreeHead).rthRightEndNode.rtnParentPtr =
                                                tm_context(tvRtTreeHeadPtr);
/* leaf in the tree, in root structure */
    tm_context(tvRtRadixTreeHead).rthLeftEndNode.rtnFlags =
                                                TM_RTN_IN|TM_RTN_ROOT;
/* leaf in the tree, in root structure */
    tm_context(tvRtRadixTreeHead).rthRightEndNode.rtnFlags =
                                                TM_RTN_IN|TM_RTN_ROOT;
/* node in the tree, in root structure */
    tm_context(tvRtTreeHeadPtr)->rtnFlags = TM_RTN_IN|TM_RTN_ROOT|TM_RTN_NODE;
/* left child of root node */
    tm_rt_left_child(tm_context(tvRtTreeHeadPtr)) =
                               &tm_context(tvRtRadixTreeHead).rthLeftEndNode;
/* right child of root node */
    tm_rt_right_child(tm_context(tvRtTreeHeadPtr)) =
                               &tm_context(tvRtRadixTreeHead).rthRightEndNode;
/* bit mask for left/right decision in root node */
    tm_ip_copy(TM_IP_ONE_HIGH_BIT, tm_context(tvRtTreeHeadPtr)->rtnNSBitMask);
/* rtnBitOffset in root node initialized to zero */
/* IPV6 rtn6LongIndex, and rtn6LongBitOffset in root node initialized to zero */
/* Right most leaf has invalid destination IP address (or key)*/
/* IPv4 mode only */
    tm_ip_copy( TM_IP_ALL_ONES,
                tm_context(tvRtRadixTreeHead).rthRightEndNode.rtnLSKey );
    tm_ip_copy( TM_IP_ALL_ONES,
                tm_context(tvRtRadixTreeHead).rthRightEndNode.rtnLSMask);
/* 
 * To support longest prefix match on a routing table entry having duplicate
 *  keys of 0, the left end of the tree must have a netmask of all one's. 
 */
    tm_ip_copy( TM_IP_ALL_ONES,
                tm_context(tvRtRadixTreeHead).rthLeftEndNode.rtnLSMask);


}


/*
 * set physical address mapping to a routing table ARP Entry
 * Called by the ARP interface, or user interface to set a physical address
 * mapping
 * Parameters: rtePtr pointer to ARP entry in routing table
 *             physAddrPtr pointer to link layer address
 */
int tfRtArpAddByRte ( ttRteEntryPtr     rtePtr,
                      tt8BitPtr         physAddrPtr,
                      tt32Bit           ttl,
                      tt8Bit            physAddrLen )
{
    int errorCode;
    
#ifdef TM_ERROR_CHECKING
    if (   (physAddrPtr == TM_8BIT_NULL_PTR)
        || (rtePtr == TM_RTE_NULL_PTR))
    {
        tfKernelError("tfRtArpAddByRte", "bad parameters");
        tm_thread_stop;
    }
#endif

/* Check that the entry is an ARP entry, and that it is up */
    if (    tm_16bit_all_bits_set( rtePtr->rteFlags,
                                   TM_RTE_ARP | TM_RTE_CLONED )
         && tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                   TM_RTE2_HOST | TM_RTE2_UP ) )
    {
 
        if (rtePtr->rteFlags & TM_RTE_LINK_LAYER)
        {
/* reset previous mapping */
            rtePtr->rteFlags &= ~TM_RTE_LINK_LAYER;
        }
        tm_bcopy(physAddrPtr, rtePtr->rteHSPhysAddr, physAddrLen);
        rtePtr->rteFlags |= TM_RTE_LINK_LAYER;
        rtePtr->rteRefresh = TM_8BIT_ZERO;
        rtePtr->rteTtl = ttl;

        TM_UNREF_IN_ARG(physAddrLen); /* avoid compiler warning */
        errorCode = TM_ENOERROR;
    }
    else
    {
        errorCode = TM_EINVAL;
    }
    return errorCode;
}



/*
 * Called by the socket API interface.
 * Walk the tree, to find an ARP entry corresponding to the valid
 * field in arpMapping structure, and execute the command in arpmCommand
 * field of ttArpMapping structure as follows:
 *
 *   TM_ARP_GET_BY_PHYS:
 *      Get an ARP Entry mapping in routing table using Physical address to
 *      locate.
 *   TM_ARP_GET_BY_IP:
 *      Get an ARP Entry mapping in routing table using IP address to locate 
 *   TM_ARP_DEL_BY_IP:
 *      Erase an ARP Entry mapping in routing table using IP address to locate
 *   TM_ARP_DEL_BY_PHYS:
 *      Erase an ARP Entry mapping in routing table using Physical address to
 *      locate.
 *   TM_ARP_SET_TTL
 *      Change ttl of ARP entry.
 *   TM_ARP_SET_PHYS
 *      Change physical address of ARP entry.
 * Parameters:
 *   arpMappingPtr
 *      Pointer to structure containing ARP table command to execute, 
 *      and its parameters.
 * Return values: 
 *    TM_ENOERROR
 *      success
 *    TM_ENOENT 
 *      no such Arp entry
 *    TM_EINVAL 
 *      unrecognized ARP command, or
 *      provided physical address buffer is too short
 */
int tfRtArpCommand (ttArpMappingPtr arpMappingPtr)
{
    ttRteEntryPtr rtePtr;
    int           length;
    int           match;
    int           errorCode;
    tt16Bit       af;
    tt8Bit        arpCommand;

    arpCommand = arpMappingPtr->arpmCommand;
    errorCode = TM_ENOERROR;
    if (arpCommand >  TM_ARP_LAST_COMMAND)
    {
        errorCode = TM_EINVAL;
    }
    if (errorCode == TM_ENOERROR)
    {
/* LOCK the tree to obtain exclusive access to the routing table */
        tm_call_lock_wait(&tm_context(tvRtTreeLock));
        if (tm_8bit_one_bit_set( arpCommand,
                             (TM_ARP_GET_BY_PHYS | TM_ARP_DEL_BY_PHYS)))
        {
/* No key. No choice but to walk the tree */
            errorCode = tfRtTreeWalk( tfRtArpMappingCB,
                                      (ttVoidPtr)arpMappingPtr,
                                      TM_8BIT_ZERO );
        }
        else
        {
/* Locate ARP entry in the tree, using the key, and host mask */
            errorCode = TM_ENOENT;
            rtePtr = tfRtFindIpAddrIpNet( 
#ifdef TM_USE_STRONG_ESL
                                          arpMappingPtr->arpmDevPtr,
#endif /* TM_USE_STRONG_ESL */
                                          arpMappingPtr->arpm4IpAddress,
                                          TM_IP_HOST_MASK);
            if (    (rtePtr != (ttRteEntryPtr)0)
                 && tm_16bit_one_bit_set( rtePtr->rteFlags, TM_RTE_ARP )
                 && tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                           TM_RTE2_HOST | TM_RTE2_UP ) )
            {
                if (tm_8bit_one_bit_set(arpCommand, TM_ARP_DEL_BY_IP))
/* Delete even non resolved ARP entry */
                {
                    errorCode = TM_ENOERROR;
                    af = (tt16Bit)AF_INET;
                    tfRtRemoveHostEntry(rtePtr, 1, af);
                }
                else if (tm_8bit_one_bit_set(arpCommand, TM_ARP_CHK_BY_IP))
/* indicate whether the entry exists or not */
                {
                    errorCode = TM_ENOERROR;
                }
                else
/*
 * All other commands valid on resolved ARP entries only, i.e. if
 * TM_RTE_LINK_LAYER set
 */
                {
                    if ( tm_16bit_one_bit_set(rtePtr->rteFlags, 
                                              TM_RTE_LINK_LAYER ) )
                    {
                        length = (unsigned)rtePtr->rteHSPhysAddrLen;
/*
 * Since code is common between IPv6, and IPv4, reset the IPV6 flag,
 * to keep the code smaller.
 */
                        arpCommand = (tt8Bit)(arpCommand & (TM_ARP_CMD_SET));
                        switch(arpCommand)
                        {
                        case TM_ARP_GET_BY_IP:
                            if (arpMappingPtr->arpmPhysAddrLen < length)
                            {
                                errorCode = TM_EINVAL; /* not enough space */
                                arpMappingPtr->arpmPhysAddrLen 
                                                        = (tt8Bit)length;
                            }
                            else
                            {
                                errorCode = TM_ENOERROR; /* ARP entry found */
                                tm_bcopy( rtePtr->rteHSPhysAddr,
                                          arpMappingPtr->arpmPhysAddr,
                                          length);
                                arpMappingPtr->arpmPhysAddrLen 
                                                        = (tt8Bit)length;
/* ecr: get the devIndex and TTL for the SNMP Agent */
                                arpMappingPtr->arpmDevIndex =
                                        rtePtr->rteOrigDevPtr->devIndex;
                                arpMappingPtr->arpmTtl = rtePtr->rteTtl;
                            }
                            break;
                        case TM_ARP_SET_TTL:
                            errorCode = TM_ENOERROR; /* ARP entry found */
                            tm_phys_addr_match(arpMappingPtr->arpmPhysAddr,
                                               rtePtr->rteHSPhysAddr,
                                               length,
                                               match);
                            if (match == length)
                            {
                                errorCode = TM_ENOERROR; /* ARP entry found */
                                rtePtr->rteTtl = arpMappingPtr->arpmTtl;
                                if (rtePtr->rteTtl == TM_RTE_INF)
                                {
/* static ARP cache entry (one created by the user) has an infinite lifetime */
                                    rtePtr->rteFlags |= TM_RTE_STATIC;
                                }
                                else
                                {
/* dynamic ARP cache entry (one resolved by ARP) has a finite lifetime */
                                    rtePtr->rteFlags &= ~TM_RTE_STATIC;
                                }
                            }
                            break;
                        case TM_ARP_SET_PHYS:
                            errorCode = TM_ENOERROR; /* ARP entry found */
                            tm_bcopy( arpMappingPtr->arpmPhysAddr,
                                      rtePtr->rteHSPhysAddr,
                                      arpMappingPtr->arpmPhysAddrLen);
                            rtePtr->rteHSPhysAddrLen =
                                              arpMappingPtr->arpmPhysAddrLen;
                            break;
                        default:
/* someone passed us an unrecognized command code, return an error */
                            errorCode = TM_EINVAL;
                            break;
                        }
                    }
                }
            }
        }
/* UNLOCK to release access to the routing table */
        tfRtTreeUnlockNResetCaches();
    }
    return errorCode;
}

/*
 * Tree walk process routine. Called for each entry in the
 * routing tree, to execute the user command passed to tfRtArpCommand()
 * (TM_ARP_GET_BY_PHYS, or TM_ARP_DEL_BY_PHYS).
 * Returns with no error if the routing entry is an ARP entry corresponding
 * to the ttArpMapping structure entry.
 */
static int tfRtArpMappingCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr)
{
    ttArpMappingPtr arpMappingPtr;
    int length;
    int match;
    int errorCode;

    arpMappingPtr = (ttArpMappingPtr)voidPtr;
    errorCode = TM_ENOENT;
/* Check that the routing entry is an ARP entry, and that it is up */
    if (    tm_16bit_all_bits_set( rtePtr->rteFlags,
                                   TM_RTE_LINK_LAYER | TM_RTE_ARP )
         && tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                   TM_RTE2_HOST | TM_RTE2_UP ) 
#ifdef TM_USE_STRONG_ESL
         && (rtePtr->rteOrigDevPtr == arpMappingPtr->arpmDevPtr) 
#endif /* TM_USE_STRONG_ESL */
       )
    {
        length = arpMappingPtr->arpmPhysAddrLen;
        if (length == TM_ETHER_ADDRESS_LENGTH)
        {
            match = tm_ether_match( arpMappingPtr->arpmEnetAddress,
                                    rtePtr->rteHSEnetAdd );
        }
        else
        {
            tm_phys_addr_match(arpMappingPtr->arpmPhysAddr,
                               rtePtr->rteHSPhysAddr, length, match);
            if (match != length)
            {
                match = 0;
            }
        }
        if (match)
        {
            {
                switch(arpMappingPtr->arpmCommand)
                {
                    case TM_4_ARP_GET_BY_PHYS:
                        tm_ip_copy(rtePtr->rteDest,
                                   arpMappingPtr->arpm4IpAddress);
                        arpMappingPtr->arpmDevIndex =
                                                rtePtr->rteOrigDevPtr->devIndex;
                        break;
                    case TM_4_ARP_DEL_BY_PHYS:
                        tfRtRemoveHostEntry(rtePtr, 1, (tt16Bit)AF_INET);
                        break;
                    default:
                        break;
                }
                errorCode = TM_ENOERROR; /* done */
            }
        }
    }
    return errorCode; /* ask for next entry, if errorCode is TM_ENOENT */
}



/*
 * Given a destination IP address in the cache, obtain a pointer
 * to the corresponding routing table and store it in the cache.
 * 1. Obtain exclusive access to the routing table.
 * 2. Get a routing entry corresponding to the given destination address
 * 3. If a match has been found we store it in the second field of the cache 
 * entry, increase the ownership count of the routing entry. error is 0.
 * 3. If a match has not been found, we store a null pointer in the second 
 * field. error is TM_EHOSTUNREACH
 * 4. We release exclusive access to the routing table.
 * 5. We return the error.
 * Parameter   Description
 * ttRtCachePtr    Pointer to a cache entry where the destination IP address
 *                 is stored. On successful return, the route entry pointer 
 *                 inside the cache entry is initialized
 * 
 * Returns
 * 0               no error
 * TM_EHOSTUNREACH no match
 * TM_EINVAL       parameter error
 * TM_EWOULDBLOCK  routing table was locked and the user did not want to wait
 */
int tfRtGet (ttRtCacheEntryPtr rtcCachePtr)
{
    ttRteEntryPtr               rtePtr;
#ifdef TM_6_ENABLE_ONLINK_ASSUMPTION
    tt32Bit                     scopeId;
    unsigned int                devCount;
    unsigned int                indx;
#endif /* TM_6_ENABLE_ONLINK_ASSUMPTION */
    int                         errorCode;

#ifdef TM_ERROR_CHECKING
#ifdef TM_USE_NETSTAT
#ifdef TM_DEBUG_LOGGING
    tfxLogRteTable("tfRtGet");
    tfxLogArpTable("tfRtGet");
    tm_debug_log1("tfRtGet: %lx",
                  ntohl(rtcCachePtr->rtcDestIpAddr));
#endif /* TM_DEBUG_LOGGING */
#endif /*TM_USE_NETSTAT*/

    if (rtcCachePtr == TM_RTC_NULL_PTR)
    {
        errorCode = TM_EINVAL;
        goto exitRtGet;
    }
#endif /* TM_ERROR_CHECKING */



/* LOCK the tree to obtain exclusive access to the routing table */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
/* Find an entry in the routing table for the destination IP address */
/* If entry is a LAN network destination, clone for specific host */
    rtePtr = tfRtFindNClone(
#ifdef TM_USE_STRONG_ESL
                             rtcCachePtr->rtcDevPtr,
#endif /* TM_USE_STRONG_ESL */
                             rtcCachePtr->rtcDestIpAddr);
    if (rtePtr != TM_RTE_NULL_PTR)
    {
/*
 * If one found, store in cache and increase owner count, also store
 * corresponding ARP entry if route is indirect
 */
        tfRtCacheGet(rtcCachePtr, rtePtr);
        errorCode = TM_ENOERROR;
    }
    else
    {
        errorCode = TM_EHOSTUNREACH;
    }


/* UNLOCK the tree to release access to the routing table */
/* No need to reset caches */
    tfRtTreeUnlock(); 

#ifdef TM_6_ENABLE_ONLINK_ASSUMPTION
    if ((errorCode != TM_ENOERROR) && !IN6_IS_ADDR_MULTICAST(destIpAddrPtr)
        && !tm_6_ip_zero(destIpAddrPtr)
        && !(IN6_IS_ADDR_V4MAPPED(destIpAddrPtr))
        )
    {
/* No route was found. Per [RFC2461].R5.2:30, if there is no default router,
   we should now assume the destination is on-link, and send the packet out
   any interface that has IPv6 enabled. This entails creating a special
   unresolved ARP cache entry when we don't have a clonable local route to
   clone it from. */
#ifdef TM_USE_STRONG_ESL
        def6HeadPtr = (tt6RtDefRouterHeadPtr)0;
        if (rtcCachePtr->rtcDevPtr != (ttDeviceEntryPtr)0)
        {
            def6HeadPtr =
                    &(rtcCachePtr->rtcDevPtr->dev6DefaultRouterHead);
        }
        else
        {
/* Find an IPv6 enabled device with a default router */
            devPtr = tf6RtGetNextDefaultGatewayList(0);
            if (devPtr != (ttDeviceEntryPtr)0)
            {
                def6HeadPtr = &(devPtr->dev6DefaultRouterHead);
            }
        }
#else /* !TM_USE_STRONG_ESL */
        def6HeadPtr =
                    &(tm_context(tvRtRadixTreeHead).rth6DefaultRouterHead);
#endif /* TM_USE_STRONG_ESL */
        if (
#ifdef TM_USE_STRONG_ESL
             (def6HeadPtr == (tt6RtDefRouterHeadPtr)0) ||
#endif /* TM_USE_STRONG_ESL */
             (def6HeadPtr->drh6RouterCount == 0)
           )
        {
#ifdef TM_USE_STRONG_ESL
            if (rtcCachePtr->rtcDevPtr != (ttDeviceEntryPtr)0)
            {
                devPtr = rtcCachePtr->rtcDevPtr;
                if (    !(tm_6_link_local_cached(devPtr))
                     || (    !(tm_6_ll_is_lan(devPtr))
                          && !(tm_ll_uses_ppp(devPtr))) )
                {
                    errorCode = TM_EHOSTUNREACH;
                    goto exitRtGet;
                }
            }
            else
#endif /* TM_USE_STRONG_ESL */
            {
                devPtr = TM_DEV_NULL_PTR;
                tm_6_get_embedded_scope_id(destIpAddrPtr, scopeId);
/* search the list of IPv6-enabled devices until we either find one
   that supports link-layer addresses or which is a PPP interface */
                tm_lock_wait(&tm_context(tvDeviceListLock));
                devNodePtr = (ttDeviceEntryPtr)(ttVoidPtr)
                    tm_context(tvDeviceList).listHeadNode.nodeNextPtr;
                devCount = tm_context(tvDeviceList).listCount;
                for ( indx = 0;
                     (indx < devCount);
                      indx++ )
                {
                    if (    (tm_6_link_local_cached(devNodePtr))
                        && (    tm_6_ll_is_lan(devNodePtr)
                             || tm_ll_uses_ppp(devNodePtr))
                        && (    !IN6_IS_ADDR_LINKLOCAL(destIpAddrPtr)
                             || (scopeId == (tt32Bit)devNodePtr->devIndex) )
                       )
                    {
                        devPtr = devNodePtr;
                        break;
                    }
                    devNodePtr = (ttDeviceEntryPtr)(ttVoidPtr)
                        devNodePtr->devNodeEntry.nodeNextPtr; /* next */
                }
                tm_unlock(&tm_context(tvDeviceListLock));
            }
            if ((devPtr == TM_DEV_NULL_PTR) || tm_dev_is_loopback(devPtr))
            {
/*  make sure that it isn't the loopback interface. If it is, we cannot
    continue. */
                errorCode = TM_EHOSTUNREACH;
                goto exitRtGet;
            }
/* LOCK the tree to obtain exclusive access to the routing table */
            tm_call_lock_wait(&tm_context(tvRtTreeLock));
            if (tm_ll_uses_ppp(devPtr))
            {
/* use the peer PPP address as the destination address */
                destIpAddrPtr = &(devPtr->dev6Pt2PtPeerIpAddr);
            }
            else
            {
/* if the interface supports link-layer addresses, then create a host
   entry for the destination in the INCOMPLETE state (i.e. without the
   TM_RTE_LINK_LAYER bit set) and return this as the routing entry. */
                tf6RtAddHost(
                    devPtr,
                    destIpAddrPtr,
                    TM_8BIT_NULL_PTR, /* physical address ptr */
                    (tt8Bit) 0,       /* physical address len */
                    TM_6_RT_RESERVED_MHOME_INDEX,
                    TM_RTE_INF,
                    TM_RTE_ARP | TM_RTE_CLONED,
                    TM_8BIT_NO);     /* tree already locked */
            }

/* Find an entry in the routing table for the destination IP address */
/* If entry is a LAN network destination, clone for specific host */
            rtePtr = tfRtFindNClone(
#ifdef TM_USE_STRONG_ESL
                                     devPtr,
#endif /* TM_USE_STRONG_ESL */
                                     destIpAddrPtr);
            if (rtePtr != TM_RTE_NULL_PTR)
            {
/*
 * If one found, store in cache and increase owner count, also store
 * corresponding ARP entry if route is indirect
 */
                tfRtCacheGet(rtcCachePtr, rtePtr);
                errorCode = TM_ENOERROR;
            }
/* UNLOCK the tree to release access to the routing table */
/* No need to reset caches */
            tfRtTreeUnlock(); 
        }
    }
#endif /* TM_6_ENABLE_ONLINK_ASSUMPTION */

#if (defined(TM_ERROR_CHECKING) || defined(TM_6_ENABLE_ONLINK_ASSUMPTION))
exitRtGet:
#endif /* TM_ERROR_CHECKING or TM_6_ENABLE_ONLINK_ASSUMPTION */

    return errorCode;
}

/*
 * Cache routing entry (and corresponding ARP entry if routing entry
 * is indirect).
 * 1. Store passed routing entry in the cache and increase its owner count.
 * 2. If routing entry is indirect, store corresponding ARP cache entry in
 *    the cache and increase its owner count.
 *
 * Parameter   Description
 * rtcPtr    Pointer to a cache entry where we store the route entry pointer
 *           and possibly corresponding ARP entry if route entry is indirect.
 * rtePtr    Pointer to routing entry to be cached.
 *
 * Returns
 * none.
 */
static void tfRtCacheGet (ttRtCacheEntryPtr rtcPtr, ttRteEntryPtr rtePtr)
{
    tm_incr_owner_count(rtePtr->rteOwnerCount);
    rtcPtr->rtcRtePtr = rtePtr;
    if (    (tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_INDIRECT))
         && (rtePtr->rteClonedPtr != TM_RTE_NULL_PTR) )
    {
/*
 * If route is indirect, point to ARP entry if there is one, and increase its
 * owner count.
 */
        rtcPtr->rtcRteClonedPtr = rtePtr->rteClonedPtr;
        tm_incr_owner_count(rtePtr->rteClonedPtr->rteOwnerCount);
    }
    return;
}


/*
 * tfRtUnGet Function Description:
 * Release ownership to a routing entry. If ownership is zero, recycle the
 * entry.
 * 1. We decrease the ownership count of the routing entry.
 * 2. If the ownership count is zero
 *    a. If entry is a gateway entry and points to a local/Arp entry,
 *       decrease ownership of the entry the gateway points to.
 *    b. recycle the routing entry.
 * Note about owner count and UP/IN flag: We have to use ownership count
 * to keep track of number of people pointing to a routing entry. We
 * have to use the UP/IN flag to figure out whether a route is still valid
 * (in the tree). We can only recycle the entry when owner count reaches 0.
 *
 * Parameter    Description
 * ttRtEntryPtr Pointer to a routing entry
 *
 * Return values none:
 */

void tfRtUnGet (ttRteEntryPtr rtePtr)
{
    unsigned int ownerCount;


#ifdef TM_ERROR_CHECKING
#ifdef TM_USE_NETSTAT
#ifdef TM_DEBUG_LOGGING
    tfxLogRteTable("tfRtUnGet");
    tfxLogArpTable("tfRtUnGet");
    tfxLogRteEntry("tfRtUnGet", rtePtr);
#endif /* TM_DEBUG_LOGGING */
#endif /*TM_USE_NETSTAT*/
/* Ownership count on a routing entry should be != 0 before an unget */
    if (rtePtr->rteOwnerCount == 0)
    {
        tfKernelError("tfRtUnGet", "ownership count of routing entry < 0\n");
    }
#endif /* TM_ERROR_CHECKING */
    tm_decr_owner_count(ownerCount, rtePtr->rteOwnerCount);
/*
 * if owner count is 0, entry no longer in the tree, and nobody is
 * pointing to it
 */
    if (ownerCount == 0)
    {
#ifdef TM_ERROR_CHECKING
/* The leaf node/internal node better not be root, or in the tree */
        if (    tm_16bit_one_bit_set(rtePtr->rteFlags2, TM_RTE2_UP)
             || tm_16bit_one_bit_set( (   rtePtr->rteLeafNode.rtnFlags
                                        | rtePtr->rteInternalNode.rtnFlags),
                                      ( TM_RTN_IN | TM_RTN_ROOT ) ) )
        {
            rtePtr->rteOwnerCount = 1; /* Do not recycle the entry */
            tfKernelError("tfRtUnGet",
        "Attempting to free a routing entry still in the tree, or a root!\n");
        }
        else
#endif /* TM_ERROR_CHECKING */
        {
            if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_INDIRECT) )
/* If gateway entry */
            {
                if (rtePtr->rteClonedPtr != TM_RTE_NULL_PTR)
                {
/* If gateway entry points to cloned route, dereference cloned entry. */
                    tfRtUnGet(rtePtr->rteClonedPtr);
                    rtePtr->rteClonedPtr = TM_RTE_NULL_PTR;
                }
            }
/* Put back the entry in the recycle queue */
            tm_recycle_free(rtePtr, TM_RECY_RTE);
#ifdef TM_ERROR_CHECKING
/* Keep track of out of tree non recycled entries */
            tm_context(tvRteLimboCount)--;
#endif /* TM_ERROR_CHECKING */
        }
    }
}



/*
 * Static functions
 */

/*
 * Route Arp timer. Fired every tvArpResolution (60 seconds) to check on a
 * route Arp entry. Need to lock the tree to prevent insertion of a
 * new ARP entry, while we are walking down the list.
 * If timer expires, reset the mapping
 */
static void tfRtArpTimer (ttVoidPtr      timerBlockPtr,
                          ttGenericUnion userParm1,
                          ttGenericUnion userParm2)
{
/* Avoid compiler warnings about unused parameters */
    TM_UNREF_IN_ARG(userParm2);
    TM_UNREF_IN_ARG(timerBlockPtr);
    
    (void)tfRtTreeWalk(tfRtArpAgeEntryCB,
                       userParm1.genVoidParmPtr,
                       TM_8BIT_YES);
    return;
}

/*
 * Walk down the tree and age all RIP and Indirect routes.
 *
 * We call the walk down tree routing, passing the function
 * tfRtAgeEntryCB() as a parameter, which will be called for each
 * leaf, with the corresponding routing entry as a parameter.
 */
static void tfRtTimer (ttVoidPtr      timerBlockPtr,
                       ttGenericUnion userParm1,
                       ttGenericUnion userParm2)
{

/* Avoid compiler warnings about unused parameters */
    TM_UNREF_IN_ARG(userParm2);
    TM_UNREF_IN_ARG(timerBlockPtr);
    
    (void)tfRtTreeWalk(tfRtAgeEntryCB, userParm1.genVoidParmPtr, TM_8BIT_YES);
    return;
}

/*
 * tfRtTreeWalk function description:
 * Walk down the tree, and for each reached leaf call the routing entry
 * processing function. We stop when we have walked down the whole tree,
 * or when the processing function returns with no error.
 *
 * Lock the Tree if not already locked.
 * 0) Indicate that we are walking the tree by increasing the tree walk count
 * 1) We try and find the leftmost leaf first
 * 2) Then in a loop, we get the next leaf from the tree (see
 *    tm_rt_get_next_leaf() for details, and process the
 *    current leaf and its duplicates while the next leaf is not the
 *    rightmost root leaf.
 * 3) decrease the tree walk count indicating that we finished one tree walk.
 *    If the walk count is 0, i.e we are on the outer part of nested tree walks,
 *    then really removed routing entries queued to be removed.
 * Unlock the tree if locked here (cleaning up caches if needed)
 *
 * Algorithm to process the current leaf and its duplicates:
 * In a loop, we get the next duplicate, and call the processing function
 * passing the routing entry corresponding to the current leaf,
 * while there is a next duplicate.
 *
 *
 * In both loops, care is taken to compute the next leaf, before processing
 * the current leaf since current leaf might be removed from the tree.
 *
 */
int tfRtTreeWalk ( ttIntRtFuncPtr processRtePtr,
                   ttVoidPtr      voidPtr,
                   tt8Bit         needLock )
{
    ttRtRadixNodePtr    nodePtr;
    ttRtRadixNodePtr    nextPtr;
    ttRtRadixNodePtr    nextDupPtr;
    int                 errorCode;

#ifdef TM_LOCK_NEEDED
    if (needLock != TM_8BIT_ZERO)
    {
/* LOCK the tree to obtain exclusive access to the routing table */
        tm_call_lock_wait(&tm_context(tvRtTreeLock));
    }
#else /* !TM_LOCK_NEEDED */
    TM_UNREF_IN_ARG(needLock);
#endif /* !TM_LOCK_NEEDED */
    tm_context(tvRtRadixTreeHead).rthTreeWalkCount++;
/* start at root of the tree */
    nodePtr = tm_context(tvRtTreeHeadPtr);
/* Go all the way to the left. Starting leaf to check */
    tm_rt_tree_leftmost_leaf(nodePtr);
#ifdef TM_ERROR_CHECKING
    tm_rt_check_null_leaf(nodePtr); /* check that the leaf is non null */
#endif /* TM_ERROR_CHECKING */
    nextPtr = nodePtr; /* starting point to search next leaf */
    errorCode = TM_ENOENT;
    do
    {
/*
 * Get next leaf to check:
 * While we are a right child and not a root, we go up the tree.
 * Now we are either a left child or root; we take the leftmost leaf of the
 * parent's right child.
 * (note that the root's parent is the parent itself).
*/
        while (    (tm_rt_right_child(tm_rt_parent(nextPtr)) == nextPtr)
                && tm_16bit_bits_not_set(nextPtr->rtnFlags, TM_RTN_ROOT) )
        {
            nextPtr = tm_rt_parent(nextPtr);
        }
        nextPtr = tm_rt_right_child(tm_rt_parent(nextPtr));
        tm_rt_tree_leftmost_leaf(nextPtr);
/* End of next leaf retrieval */
#ifdef TM_ERROR_CHECKING
        tm_rt_check_null_leaf(nextPtr); /* check that the leaf is non null */
#endif /* TM_ERROR_CHECKING */
/* process the current leaf nodePtr and its duplicates */
        nextDupPtr = nodePtr; /* starting point of duplicate search */
        do
        {
/* compute the next duplicate since currrent leaf could be removed */
            nextDupPtr = tm_rt_next_dup(nextDupPtr);
/* process current leaf/duplicate nodePtr (root nodes are not leaves) */
            if ( tm_16bit_bits_not_set(nodePtr->rtnFlags, TM_RTN_ROOT) )
            {
#ifdef TM_ERROR_CHECKING
                if ( (tm_rt_leaf_to_rte(nodePtr))->rteOwnerCount == 0 )
                {
                    tfKernelError("tfRtTreeWalk", "Corrupted Patricia tree");
                }
#endif /* TM_ERROR_CHECKING */
                errorCode = (*processRtePtr)( tm_rt_leaf_to_rte(nodePtr),
                                              voidPtr );
                if (errorCode == TM_ENOERROR)
                {
                    goto treeWalkExit;
                }
            }
/* process next duplicate */
            nodePtr = nextDupPtr;
        }
        while (nodePtr != TM_RADIX_NULL_PTR);
/* end of processing the current leaf and its duplicates */
/* process the next leaf. Save pointer to reached leaf */
        nodePtr = nextPtr;
/* We are done when we have reached the rightmost root leaf */
    }
    while ( tm_16bit_bits_not_set(nodePtr->rtnFlags, TM_RTN_ROOT) );
treeWalkExit:
    tm_context(tvRtRadixTreeHead).rthTreeWalkCount--;
    if (tm_context(tvRtRadixTreeHead).rthTreeWalkCount == TM_16BIT_ZERO)
/*
 * No longer in a tree walk. Safe to remove the routing entries queued to
 * be removed while we were walking the tree(s).
 */
    {
        tfRtTreeRem();
    }
#ifdef TM_LOCK_NEEDED
    if (needLock != TM_8BIT_ZERO)
    {
/* UNLOCK the tree to release access to the routing table */
        tfRtTreeUnlockNResetCaches();
    }
#endif /* TM_LOCK_NEEDED */
    return errorCode;
}


/*
 * Algorithm to age the passed routing entry:
 * It routing entry is a REDIRECT or RIP routing entry, or SNMP routing
 * entry, or router discovery, path MTU discovery entry we
 * decrease the routing entry TTL by 1, and if it reaches zero for a
 * non path MTU discovery entry, we remove the entry. If the entry is a
 * Path MTU discovery entry (TM_RTE_PMTU_DISC set, and TM_RTE_NO_PMTU_DISC
 * not set, and it reaches zero, try and pick the next plateau MTU value.
 * If there is a next plateau MTU, and it is not bigger than the device
 * MTU, then add a new host entry with a 2 minutes timeout, otherwise
 * remove the entry.
 */
static int tfRtAgeEntryCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr)
{
#ifdef TM_PMTU_DISC
   tt16Bit     nextMtu;
   tt8Bit      needRemoveEntry;
#endif /* TM_PMTU_DISC */

/* Unused Parameter */
    voidPtr=voidPtr;
    if ( tm_16bit_one_bit_set( rtePtr->rteFlags, TM_RTE_DYNAMIC) )
/* RIP, Router Discovery, Path MTU discovery, or REDIRECT gateway entry */
    {
        if ( rtePtr->rteTtl != TM_RTE_INF )
        {
/* decrease time to live */
            if ( rtePtr->rteTtl > tm_context(tvRtTimerResolution) )
            {
                rtePtr->rteTtl -= tm_context(tvRtTimerResolution);
            }
            else
            {
/*
 * If TTL is zero, remove entry.
 *
 * For Path MTU discovery entries:
 * IPv6 PMTU entries should be allowed to timeout and be removed as usual.
 * This will cause anything using the previous PMTU to reset to the link MTU,
 * and perform PMTU discovery again if necessary. ([RFC1981].R4:20,
 * [RFC1981].R5.3:20)
 * IPv4 PMTU entries will perform the algorithm below, which raises the PMTU
 * to the next plateau, etc
 */
#if (defined(TM_USE_IPV4) && defined(TM_PMTU_DISC))
                needRemoveEntry = TM_8BIT_YES;                
                if (    tm_16bit_one_bit_set(rtePtr->rteFlags,
                                             TM_RTE_PMTU_DISC)
                     && tm_16bit_bits_not_set(rtePtr->rteFlags,
                                             TM_RTE_NO_PMTU_DISC)
                    )
                {
/*
 * If entry is path MTU discovery, and Path MTU discovery is allowed.
 */
/* OK to use rteMtu unprotected here, because IPv4 entries do not have
   their MTU updated dynamically (as IPv6 ones do). */
                    nextMtu = tfRtPickNextPlateauMtu( rtePtr->rteMtu,
                                                      TM_PMTU_HIGHER );
                    if (    (nextMtu != 0)
/* PATH MTU Maximum not reached */
                         && (nextMtu <= rtePtr->rteDevPtr->devMtu) )
/* and Next MTU does not exceed our interface MTU */
                    {
/* Enter new entry with bigger MTU */
                        (void)tfRtAddPathMtuEntry(
#ifdef TM_USE_STRONG_ESL
                                            rtePtr->rteOrigDevPtr,
#endif /* TM_USE_STRONG_ESL */
                                            rtePtr,
                                            rtePtr->rteDest,
/*
 * Our Path MTU discovery timed out. Attempt a larger MTU with a 2 minutes
 * timeout
 */
                                            tm_context(tvRtLargerMtuTimeout),
                                              TM_RTE_INDIRECT
                                            | TM_RTE_PMTU_DISC,
                                            nextMtu);
/* Entry has been replaced, do not remove it */
                        needRemoveEntry = TM_8BIT_ZERO;
                    }
                }
                if (needRemoveEntry != TM_8BIT_ZERO)
#endif /* TM_PMTU_DISC && TM_USE_IPV4 */
                {
/* if zero remove entry */
                    tfRtRemoveEntry(rtePtr);
                }
            }
        }
    }
    return TM_ENOENT; /* ask for next entry */
}

/*
 * Delete an IPv4 ARP entry pointed to by rtePtr.
 */
static void tf4RtArpDelByRte (ttRteEntryPtr rtePtr, int flag)
{
/*
 * If a cache entry or a gateway is pointing to the entry, or if the
 * entry is used for PATH MTU:
 *   . if the  entry is an ARP entry, invalidate the mapping.
 *   . also set the ttl to 0, so that the Arp timer, at the next tick,
 *     will check again on the entry ownership to try and remove the entry.
 * else, remove the entry.
 */
    if ((rtePtr->rteOwnerCount > 1) || flag)
    {
        if ( tm_16bit_all_bits_set( rtePtr->rteFlags,
                                    TM_RTE_LINK_LAYER | TM_RTE_ARP ) )
        {
            rtePtr->rteFlags &= ~TM_RTE_LINK_LAYER;
        }
        rtePtr->rteTtl = TM_UL(0);
    }
/* otherwise remove it */
    else
    {
        tfRtRemoveEntry(rtePtr);
    }
    return;
}


/*
 * Function called from tfRtTreeWalk.
 * If the passed routing entry pointer parameter is an ARP entry, or
 * cloned MULTICAST mapping entry, age it.
 * If timeout has expired remove the entry from the tree.
 */
static int tfRtArpAgeEntryCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr)
{

    int     af;
#ifdef TM_PMTU_DISC
    tt16Bit currentMtu;
    tt16Bit nextMtu;
    tt16Bit deviceMtu;
    tt16Bit rteFlags;
#endif /* TM_PMTU_DISC */
    tt8Bit  flag;
    
    TM_UNREF_IN_ARG(voidPtr); /* avoid compiler warning */

/* If valid host cloned entry, ( ARP/multicast/broadcast mapping entry): */
    if (    tm_16bit_one_bit_set( rtePtr->rteFlags, TM_RTE_CLONED )
         && tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                   TM_RTE2_HOST | TM_RTE2_UP )
       )
    {
        af = 0; 
        {
#ifdef TM_PMTU_DISC
/* Update route MTU according if our previous MTU has expired */
            if (    tm_16bit_one_bit_set(rtePtr->rteFlags,
                                         TM_RTE_PMTU_DISC)
                 && tm_16bit_bits_not_set(rtePtr->rteFlags,
                                         TM_RTE_NO_PMTU_DISC)
                )
            {
                tm_kernel_set_critical;
                currentMtu = rtePtr->rteMtu;
                deviceMtu = rtePtr->rteDevPtr->devMtu;
                if ( rtePtr->rteHSPmtuTimeout > tm_context(tvArpResolution) )
/* Decrease PMTU time to live by ARP resolution (60000 milliseconds) */
                {
                    rtePtr->rteHSPmtuTimeout -= tm_context(tvArpResolution);
                    flag = TM_8BIT_NO;
                }
                else
                {
/* PMTU timeout, MTU expired */
                    flag = TM_8BIT_YES;
                    rtePtr->rteHSPmtuTimeout = 0;
                }
                tm_kernel_release_critical;
                if (flag != TM_8BIT_NO)
/* Expired */
                {
/*
 * If entry is path MTU discovery, and Path MTU discovery is allowed.
 */
                    nextMtu = tfRtPickNextPlateauMtu( currentMtu,
                                                      TM_PMTU_HIGHER );
                    if (    (nextMtu != 0)
/* PATH MTU Maximum not reached */
                         && (nextMtu <= deviceMtu) )
/* and Next MTU does not exceed our interface MTU */
                    {
                        tm_kernel_set_critical;
                        af = AF_INET;
                        rtePtr->rteMtu = nextMtu;
                        rtePtr->rteHSPmtuTimeout =
                                         tm_context(tvRtLargerMtuTimeout);
                        tm_kernel_release_critical;
                    }
                }
            }
#endif /* TM_PMTU_DISC */
            if (rtePtr->rteTtl != TM_RTE_INF)
            {
                if ( rtePtr->rteTtl > tm_context(tvArpResolution) )
/* Decrease time to live by ARP resolution (60000 milliseconds) */
                {
                    rtePtr->rteTtl -= tm_context(tvArpResolution);
                    if (tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_ARP))
                    {
                        if (rtePtr->rteTtl <= tm_context(tvArpResolution))
/* If ARP entry time to live was <= 2 * tvArpResolution */
                        {
/* Indicate that ARP entry should be refreshed */
                            rtePtr->rteRefresh = TM_8BIT_YES;
                        }
                    }
                }
                else
                {
                    flag = 0; /* Arp entry can be deleted */
#ifdef TM_PMTU_DISC
                    rteFlags = (tt16Bit)
                     (rtePtr->rteFlags & (TM_RTE_NO_PMTU_DISC | TM_RTE_PMTU_DISC));
                    if (    (rteFlags == TM_RTE_NO_PMTU_DISC)
                         || (rteFlags == TM_RTE_PMTU_DISC) )
/* User disabled path MTU entry, or active path MTU entry */
                    {
/* Keep the ARP entry. Only mark the mapping as invalid */
                        flag = 1;
                    }
#endif /* TM_PMTU_DISC */
                    tf4RtArpDelByRte(rtePtr, flag);
                }
            }
        }
#if defined(TM_PMTU_DISC) || defined(TM_6_PMTU_DISC)
        if (af)
        {
/* Reset the caches, so that the upper layer can use the changed MTU */
            tm_context(tvRtRadixTreeHead).rthResetCachesFlags |= TM_RTE_CLONED;
            tm_context(tvRtRadixTreeHead).rthResetCachesAf |= af;
        }
#endif /* TM_PMTU_DISC || TM_6_PMTU_DISC */
    }
    return TM_ENOENT;
}

/*
 * If rtePtr is an ARP entry not being resolved and not being referenced
 * by another pointer, or a multicast mapping entry, check whether its TTL
 * is less than the current used ARP entry if any (stored in
 * leastUsedRtePtrPtr). It if is less, or there was none stored in
 * leastUsedRtePtrPtr, store it in leastUsedRtePtrPtr.
 */
static int tfRtArpGetLeastUsedCB ( ttRteEntryPtr rtePtr,
                                   ttVoidPtr leastUsedRtePtrPtr )
{
    ttRteEntryPtr leastUsedRtePtr;

    leastUsedRtePtr = *((ttRteEntryPtr TM_FAR *)leastUsedRtePtrPtr);
    if (   tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                  TM_RTE2_HOST | TM_RTE2_UP )
/* If ARP/MCAST/BCAST mapping entry */
        && tm_16bit_one_bit_set( rtePtr->rteFlags, TM_RTE_CLONED )
/* and not being static */
        && tm_16bit_bits_not_set(rtePtr->rteFlags, TM_RTE_STATIC)
/* and nobody points to it (i.e. no cache, no ARP resolution) */
        && ( rtePtr->rteOwnerCount == 1 )
#ifdef TM_PMTU_DISC
/* And user did not call tfDisablePathMtu() */
        && (    (tt16Bit)(   rtePtr->rteFlags
                           & (TM_RTE_NO_PMTU_DISC | TM_RTE_PMTU_DISC) )
             != TM_RTE_NO_PMTU_DISC
           )
#endif /* TM_PMTU_DISC */
       )
    {
        if ( leastUsedRtePtr == TM_RTE_NULL_PTR )
        {
            *((ttRteEntryPtr TM_FAR *)leastUsedRtePtrPtr) = rtePtr;
        }
        else
        {
            if (rtePtr->rteTtl <= leastUsedRtePtr->rteTtl)
            {
#ifdef TM_PMTU_DISC
                if (rtePtr->rteHSPmtuTimeout <= 
                                leastUsedRtePtr->rteHSPmtuTimeout)
#endif /* TM_PMTU_DISC */
                {
                    *((ttRteEntryPtr TM_FAR *)leastUsedRtePtrPtr) = rtePtr;
                }
            }
        }
    }
    return TM_ENOENT;
}

/*
 * Remove any entry using device/multihome index of routing entry voidPtr
 * (except for the reject default gateway.
 */
static int tfRtRemObsLocalCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr)
{
    ttRteEntryPtr    localRtePtr;

    localRtePtr = (ttRteEntryPtr)voidPtr;
    if (rtePtr != localRtePtr )
    {
        if (    (rtePtr->rteOrigDevPtr == localRtePtr->rteOrigDevPtr)
             && (rtePtr->rteMhomeIndex == localRtePtr->rteMhomeIndex)
             && (tm_16bit_bits_not_set(rtePtr->rteFlags, TM_RTE_LOCAL))
          )
        {
            {
                tfRtRemoveEntry(rtePtr);
            }
        }
    }
/* ask for the next entry. We need to check the whole tree */
    return TM_ENOENT;
}

/*
 * Remove any gateway pointing to the passed cloned entry (voidPtr), but
 * not a default router.
 */
static int tfRtRemoveGwToClonedCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr)
{
    ttRteEntryPtr clonedRtePtr;

    clonedRtePtr = (ttRteEntryPtr)voidPtr;
    if (       ( rtePtr != clonedRtePtr )
            && ( rtePtr->rteClonedPtr == clonedRtePtr) )
    {
        if ( tm_16bit_all_bits_set( rtePtr->rteFlags, 
                                    TM_RTE_CLONE | TM_RTE_LOCAL) )
/* 
 * If this is a local entry, then the cloned entry is loop back for 
 * the interface, just reset the back pointer to it.
 */
        {
            rtePtr->rteClonedPtr = TM_RTE_NULL_PTR;
        }
        else
        {
            if (      tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_STATIC)
               )
            {
/*
 * Mark the clone entry that the static route/default router points to as
 * invalid
 */
                clonedRtePtr->rteFlags2 |= TM_RTE2_INVALID;
            }
            else
/*
 * Remove the entry only if it is not a static route and not a default
 * router
 */
            {
                tfRtRemoveEntry(rtePtr);
            }
        }
    }
/* Ask for the next entry. We need to check the whole tree */
    return TM_ENOENT;
}

/*
 * Find an entry in the routing table for the destination IP address
 * This function is called after exclusive access to the routing
 * table has been obtained.
 * Based on the passed destination IP address passed, we scan the routing
 * table to find the best match.
 * 1) We first go down the tree, and when we reach a leaf check  for a
 * host/network match.
 * 2) If this fails we check for a network match on the duplicate keys of
 * the leaf
 * 3) If this fails too, we backtrack up the tree until we find a network
 * match.
 * 4) We return a pointer to a tree leaf on success, null pointer
 * otherwise. Note that since a tree leaf is the first structure of a
 * routing entry, the tree leaf pointer is also a routing entry pointer.
 * Parameter:
 *   Destination Ip address for which we want to obtain a routing table
 *   entry pointer.
 * Return values:
 *   Routing entry pointer on success
 *   Null routing entry pointer otherwise.
 */
static ttRteEntryPtr tfRtFind (
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr     devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress        destIpAddr
    )
{
    ttRtRadixNodePtr      nodePtr;
    ttRtRadixNodePtr      leafPtr;
    ttRteEntryPtr         foundRtePtr;
#ifdef TM_USE_STRONG_ESL
    ttRteEntryPtr         tempRtePtr;
#endif /* TM_USE_STRONG_ESL */
#ifdef TM_TRY_BIT_OFFSET

#ifndef TM_LITTLE_ENDIAN
    tt4IpAddress          bittest;
#endif /* TM_LITTLE_ENDIAN */
    tt16Bit               bits;

#endif /* TM_TRY_BIT_OFFSET */
 
    foundRtePtr = (ttRteEntryPtr)0; /* assume not found */
/* Start at head of tree */
    nodePtr = tm_context(tvRtTreeHeadPtr);
/* Go down the tree until we reach a leaf. nodePtr changed to a leaf */
    tm_rt_tree_search(nodePtr, destIpAddr);
#ifdef TM_ERROR_CHECKING
    tm_rt_check_null_leaf(nodePtr); /* check that the leaf is non null */
#endif /* TM_ERROR_CHECKING */
/*
 * We have now reached a leaf. Check for host or network match. Note that
 * we check for network match on the first leaf of the chain of duplicate
 * keys, but this would yield the host match as well since the most
 * specific route (host) would be first and would match that way (mask
 * all one's).
 */
    if (tm_ip_net_match(nodePtr->rtnLSKey, destIpAddr, nodePtr->rtnLSMask))
    {
        if (tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_ROOT))
        {
/*
 * Were we asked for the default entry? If so, make sure we point to it.
 * (If there is no default entry, next duplicate is NULL, and we will
 * return a null pointer (no match).) Note that this check will also
 * return a null pointer, in case the user had requested a route to
 * limited broadcast address.
 */
             nodePtr = tm_rt_next_dup(nodePtr);
#ifdef TM_USE_STRONG_ESL
/*
 * This is an end point of the tree (either left or right).
 */
            if (devPtr != (ttDeviceEntryPtr)0)
/* 
 * If we need to match on the device too (non null device):
 * We need to go through all the leaves right here, and not below as
 * backtracking done below will lead us nowhere.
 */
            {
                while (nodePtr != (ttRtRadixNodePtr)0)
                {
                    foundRtePtr = tm_rt_leaf_to_rte(nodePtr);
                    if (foundRtePtr->rteOrigDevPtr == devPtr)
                    {
                        goto rtFindExit; /* exact match */
                    }
                    nodePtr = tm_rt_next_dup(nodePtr);
                }
            }
#endif /* TM_USE_STRONG_ESL */
        }
        foundRtePtr = tm_rt_leaf_to_rte(nodePtr);
#ifdef TM_USE_STRONG_ESL
/*
 * If we have found an exact match, need to make sure that we match
 * on the device
 */
        if (nodePtr != (ttRtRadixNodePtr)0)
        {
            if (    (devPtr == (ttDeviceEntryPtr)0)
                 || (foundRtePtr->rteOrigDevPtr == devPtr) )
            {
                goto rtFindExit; /* exact match */
            }
            else
            {
/* We will need to go through the leaves below */
                foundRtePtr = (ttRteEntryPtr)0; /* reset */
            }
        }
        else
        {
            goto rtFindExit; /* null default gateway or brodcast addr match */
        }
#else /* TM_USE_STRONG_ESL */
        goto rtFindExit; /* exact match */
#endif /* TM_USE_STRONG_ESL */
    }
/*
 * No host match. Let us check for other network matches. First check through
 * all duplicate keys of the leaf we are at (network routes).
 * STRONG_ESL: also check for host match for the device in duplicate keys.
 */
#ifdef TM_TRY_BIT_OFFSET
/* a bit more work here.
 *
 * bittest temp var, bits var arg. Other parameters unchanged. Result
 * stored in bits.
 */
#ifdef TM_LITTLE_ENDIAN
    tm_ip_bit_offset(nodePtr->rtnLSkey, destIpAddr, bits);
#else /* TM_BIG_ENDIAN */
    tm_ip_bit_offset(nodePtr->rtnLSkey, destIpAddr, bittest, bits);
#endif /* TM_BIG_ENDIAN */
#endif /* TM_TRY_BIT_OFFSET */
/* save nodePtr, use leafPtr to go down the list of duplicates */
    leafPtr = tm_rt_next_dup(nodePtr);
    while (leafPtr != TM_RADIX_NULL_PTR)
    {
#ifdef TM_USE_STRONG_ESL
        tempRtePtr = tm_rt_leaf_to_rte(leafPtr);
        if (    (devPtr == (ttDeviceEntryPtr)0)
             || (tempRtePtr->rteOrigDevPtr == devPtr) )
#endif /* TM_USE_STRONG_ESL */
        {
/* Network portion of addresses match ? */
#ifdef TM_TRY_BIT_OFFSET
/* less work here */
/* matching bits cover netmask */
            if (bits >= tm_rt_mask_len(leafPtr))
#else /* TM_TRY_BIT_OFFSET */
            if ( tm_ip_net_match( leafPtr->rtnLSKey, destIpAddr,
                                  leafPtr->rtnLSMask ) )
#endif /* TM_TRY_BIT_OFFSET */
            {
                foundRtePtr = tm_rt_leaf_to_rte(leafPtr);
                goto rtFindExit;
            }
        }
        leafPtr = tm_rt_next_dup(leafPtr); /* get duplicate key leaf */
    }
/*
 * No match yet. Backup up the tree for network matches until we
 * find a network match or we reach the root
 */
    do
    {
/* Back up the tree. nodePtr moves up the tree */
       nodePtr = tm_rt_parent(nodePtr);
/* Check list of non host route entries for the subtree */
/* leafPtr is used to go down the linked list of network leaves */
       leafPtr = tm_rt_next_net(nodePtr);
       while (leafPtr != TM_RADIX_NULL_PTR)
       {
#ifdef TM_USE_STRONG_ESL
            tempRtePtr = tm_rt_leaf_to_rte(leafPtr);
            if (    (devPtr == (ttDeviceEntryPtr)0)
                 || (tempRtePtr->rteOrigDevPtr == devPtr) )
#endif /* TM_USE_STRONG_ESL */
            {
/* Check for a network match */
#ifdef TM_TRY_BIT_OFFSET
/* matching bits cover netmask (less work here) */
                if (bits >= tm_rt_mask_len(leafPtr))
#else /* TM_TRY_BIT_OFFSET */
                if ( tm_ip_net_match( destIpAddr, leafPtr->rtnLSKey,
                                      leafPtr->rtnLSMask ) )
#endif /* TM_TRY_BIT_OFFSET */
                {
                    foundRtePtr = tm_rt_leaf_to_rte(leafPtr);
                    goto rtFindExit;
                }
            }
/* next non host leaf to check for this subtree */
            leafPtr = tm_rt_next_net(leafPtr);
        }
    }
    while (nodePtr != tm_context(tvRtTreeHeadPtr));
rtFindExit:
    return foundRtePtr;
}

/*
 * Called by tf4RtAddRoute(), tf6RtAddRoute(), tfRtGet()/tfRtClone(),
 * tfRtInit(), tf4RtReplaceRoute to insert a routing entry into the
 * Patricia tree.
 * 1) Initialize the routing entry with passed parameters
 * 2) If there is already an entry for ipAddr/Netmask, if local remove it
 *    otherwise return error.
 * 3) Add the entry to the tree
 * 4) If indirect entry, add ARP router entry for the gateway and point to it.
 *
 * Parameters
 * rtePtr: new routing entry to be initialized and inserted.
 * devPtr: device for the route 
 * ipAddr: Destination Ip address 
 * netMask: destination network address 
 * mhomeIndex: multihome index for local routes
 * gateway: Gateway IP address for non local routes, interface IP address
 *          for local routes.
 * hops: number of hops to destination (0 for local route)
 * ttl: Time to live for the route entry
 * flags: flag to indicate creator local/static/redirect/RIP, host route etc.
 *
 * Return value:
 * TM_ENOERROR          success
 * TM_EALREADY          route already in the tree
 * TM_EHOSTUNREACH      destination unreachable
 */

static int tfRtAddEntry (
    ttRtRadixNodePtr        nodePtr, /* Node to start the search at */
    ttRteEntryPtr           rtePtr, /* new routing entry */
    ttDeviceEntryPtr        devPtr, /* device for the route */
    tt4IpAddress            ipAddress,
    tt4IpAddress            netMask,
/* multihome index for local routes */
    tt16Bit                 mhomeIndex,
    int                     hops, /* metrics for non local routes */
    tt32Bit                 ttl,
/* flag to indicate creator local/static/redirect/RIP */
    tt16Bit                 flags )
{
        ttRtRadixNodePtr    leafPtr; /* leaf node to be inserted */
        ttRtRadixNodePtr    intNodePtr; /* internal node to be inserted */
        ttRtRadixNodePtr    parentPtr; /* temporary */
        ttRtRadixNodePtr    nextPtr; /* temporary */
        ttRtRadixNodePtr    nextTempPtr; /* temporary */
        ttRteEntryPtr       oRtePtr; /* temporary */
        ttGenericUnion      timerParm1;
auto    tt4IpAddress        bittest; /* temporary */
auto    int                 errorCode;
        tt16Bit             af;
        tt16Bit             keyBits; /* bitoffset for internal node */
        tt16Bit             maskBits; /* bitoffset for leaf */
        tt16Bit             maskLength; /* temporary */
#if (!defined(TM_LITTLE_ENDIAN)) || defined(TM_USE_IPV6)
        tt16Bit             longBitOffset; /* bitoffset for internal node */
#endif /* (!defined(TM_LITTLE_ENDIAN)) || defined(TM_USE_IPV6) */
        tt8Bit              removeEntry;
        tt8Bit              arpClone; /* adding an ARP clone with dup key? */

#ifdef TM_ERROR_CHECKING
#ifdef TM_USE_NETSTAT
#ifdef TM_DEBUG_LOGGING
    tfxLogRteTable("tfRtAddEntry");
    tfxLogArpTable("tfRtAddEntry");
#endif /* TM_DEBUG_LOGGING */
#endif /*TM_USE_NETSTAT*/
#endif /* TM_ERROR_CHECKING */

    errorCode = TM_ENOERROR;
    arpClone = TM_8BIT_NO;
    tm_ip_copy(TM_IP_ZERO, bittest);
/* store the device pointer in the routing entry early, just in case we fail
   to add the routing entry, so that tfRtCacheUnlockNClean can reference it */
    rtePtr->rteDevPtr = devPtr;
    rtePtr->rteOrigDevPtr = devPtr;

/*
 * Compute the key IP address. We need to mask the given IP address
 * with the network mask for local addition (tfConfigInterface), and also
 * for static route, in case the user did not do it properly.
 */
    tm_ip_copy(tm_ip_net(ipAddress, netMask), ipAddress);
    oRtePtr = tfRtFindIpAddrIpNet(
#ifdef TM_USE_STRONG_ESL
                                   devPtr,
#endif /* TM_USE_STRONG_ESL */
                                   ipAddress, netMask);
    if (oRtePtr != TM_RTE_NULL_PTR)
    {
/* Remove any existing routing entry for our local route */
        if (tm_16bit_one_bit_set(flags, TM_RTE_LOCAL))
        {
            removeEntry = TM_8BIT_NO;
            if (    (oRtePtr->rteDevPtr != devPtr)
                 || tm_16bit_bits_not_set(oRtePtr->rteFlags, TM_RTE_LOCAL) )
/* non identical devices, or non local route */
            {
                removeEntry = TM_8BIT_YES;
            }
            if (removeEntry)
            {
                tfRtRemoveEntry(oRtePtr);
            }
            else
            {
/*
 * If the new local route entry is on the same interface as the old local
 * route entry, and the old local route entry is not temporary, or both
 * local routes are temporary, then don't remove the old one
 */
                errorCode = TM_EALREADY;
                goto rtAddEntryExit;
            }
        }
        else
        {
/* If creator is same, just update the ttl */
            if ((tt16Bit)(oRtePtr->rteFlags & TM_RTE_CREATE_MASK) == flags)
            {
                oRtePtr->rteTtl = ttl;
            }
            if (    tm_16bit_all_bits_set(oRtePtr->rteFlags,
                                         TM_RTE_INDIRECT | TM_RTE_CLONE)
                 && tm_16bit_all_bits_set(flags,
                                          TM_RTE_CLONED | TM_RTE_ARP))
            {
/* Adding an ARP clone with same key and netmask as static route */
                arpClone = TM_8BIT_YES;
            }
            else
            {
                if (    tm_16bit_all_bits_set(flags,
                                              TM_RTE_INDIRECT | TM_RTE_STATIC)
                     && tm_16bit_all_bits_set(oRtePtr->rteFlags,
                                              TM_RTE_CLONED | TM_RTE_ARP))
                {
                    af = AF_INET;
                    tfRtRemoveHostEntry(oRtePtr, 1, af);
                }
                else
                {
/* Do not allow the user to add a static route without removing it first */
                    errorCode = TM_EALREADY;
                    goto rtAddEntryExit;
                }
            }
/* Allow cloning an ARP entry at the same IP address */
        }
    }
    if (    tm_16bit_one_bit_set( flags, TM_RTE_DYNAMIC)
/* RIP, Router Discovery, or REDIRECT gateway entry */
         && (tm_context(tvRtEntriesCount) >= tm_context(tvRtMaxEntries)) )
    {
#ifdef TM_SNMP_MIB
/*
 * Number of routing entries which were chosen to be
 * discarded even though they are valid.
 */
        tm_context(tvIpData).ipRoutingDiscards++;
#endif /* TM_SNMP_MIB */
        errorCode = TM_ENOBUFS;
        goto rtAddEntryExit;
    }
/* Initialize the routing entry with passed parameters */
    rtePtr->rteMhomeIndex = mhomeIndex;
    rtePtr->rteOrigMhomeIndex = mhomeIndex;
    tm_ip_copy(ipAddress, rtePtr->rteDest);
/*
 * No need to add to device owner count, since device will delete
 * routing entry first.
 */
    rtePtr->rteHops = (tt32Bit)hops;
    rtePtr->rteTtl = ttl;
/* for the time being, we only use clones with local routes */
    if (tm_16bit_one_bit_set(flags, TM_RTE_CLONE))
    {
        rtePtr->rteCloneMask = TM_IP_HOST_MASK;
    }
/* Pointers to leaf node and internal node of routing entry */
    leafPtr = &rtePtr->rteLeafNode;
    intNodePtr = &rtePtr->rteInternalNode;
    tm_ip_copy(ipAddress, leafPtr->rtnLSKey);
    tm_ip_copy(netMask, leafPtr->rtnLSMask);
/* Length of netmask: Number of leading contiguous 1's.  */
    maskBits = (tt16Bit)tf4NetmaskToPrefixLen(netMask);
    if (tm_ip_not_match(tvRt32ContiguousBitsPtr[maskBits], netMask))
    {
        errorCode = TM_EINVAL;
        goto rtAddEntryExit;
    }
/* Leaf node initialization. We store the key and network mask */
    tm_rt_mask_len(leafPtr) = maskBits; /* length of netmask */
/* End of routing entry Initialization */
/* Add a brand new entry */
    {
/* start the search at nodePtr. Key is ipAddress. Network mask is netMask */
/* search down the tree until we find a leaf */
        tm_rt_tree_search(nodePtr, ipAddress);
#ifdef TM_ERROR_CHECKING
        tm_rt_check_null_leaf(nodePtr);
#endif /* TM_ERROR_CHECKING */
/* check whether we have duplicate key */
        if (tm_ip_match(nodePtr->rtnLSKey, ipAddress))
        {
#ifdef TM_USE_STRONG_ESL
            oRtePtr = tm_rt_leaf_to_rte(nodePtr);
#endif /* TM_USE_STRONG_ESL */
/*
 * Duplicate key. insert LeafPtr in list of duplicate keys, sorted
 * by decreasing length of masks. If entry is already in, return error.
 * Strong End System Model adition:
 * For same mask, if device index is different, insert by increasing number
 * of hops or increasing device index if number of hops same.
 * Insert ARP clone entry BEFORE routing entry at same IP address, so that it
 * can be found first by tfRtGet() since rteClonedPtr points to the local
 * route not the ARP entry.
 */
            if (    (tm_rt_mask_len(nodePtr) > maskBits)
                 || (tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_ROOT))
#ifdef TM_USE_STRONG_ESL
/* check for different device index, if mask len is same */
                 || (    (tm_rt_same_mask(nodePtr, maskBits))
                      && (devPtr->devIndex != oRtePtr->rteOrigDevPtr->devIndex)
                      && (    ((tt32Bit)hops > oRtePtr->rteHops)
                           || (    ((tt32Bit)hops == oRtePtr->rteHops)
                                && (devPtr->devIndex
                                        > oRtePtr->rteOrigDevPtr->devIndex) )
                         )
                    )
#endif /* TM_USE_STRONG_ESL */
               )
            {
/*
 * Entry has a shorter mask or is default
 * Strong End System Model adition:
 * or Entry has same mask, and has bigger number of hops, or bigger
 * device index.
 * insert inside the list of duplicates after the first one.
 */
/*
 * keep nodePtr as pointer to head of duplicate list. Use parentPtr, and
 * nextPtr to move pointers around.
 */
                parentPtr = nodePtr;
                nextPtr = tm_rt_next_dup(parentPtr);
                while (nextPtr != TM_RADIX_NULL_PTR)
                {
/* Insert leaf with the longest masks first */
                    if ( tm_rt_mask_len(nextPtr) < maskBits )
                    {
/* insert leafPtr before nextPtr  (next previous is leafPtr) */
                        tm_rt_prev_dup(nextPtr) = leafPtr;
                        break;
                    }
                    if ( tm_rt_same_mask(nextPtr, maskBits) )
                    {
#ifdef TM_USE_STRONG_ESL
                        oRtePtr = tm_rt_leaf_to_rte(nextPtr);
/* Check for different device index, if mask len is same */
                        if (devPtr->devIndex !=
                                        oRtePtr->rteOrigDevPtr->devIndex)
                        {
                            if (   ((tt32Bit)hops < oRtePtr->rteHops)
                                || (    ((tt32Bit)hops == oRtePtr->rteHops)
                                     && (devPtr->devIndex
                                          < oRtePtr->rteOrigDevPtr->devIndex) )
                               )
                            {
/* insert leafPtr before nextPtr (next previous is leafPtr) */
                                tm_rt_prev_dup(nextPtr) = leafPtr;
                                break;
                           }
                        }
                        else
/* 
 * If routing entry already in (duplicate key with same mask), and device
 * index is the same return error, unless we have an ARP entry at the
 * same IP address as the route.
 */
#endif /* TM_USE_STRONG_ESL */
                        {
                            if (arpClone)
                            {
                                tm_rt_prev_dup(nextPtr) = leafPtr;
                                break;
                            }
                            else
                            {
/* 
 * If routing entry already in (duplicate key with same mask), return error.
 */
                                errorCode = TM_EALREADY;
                                goto rtAddEntryExit;
                            }
                        }
                    }
/* Since this is not a circular list we need to keep track of the parent */
                    parentPtr = nextPtr;
                    nextPtr = tm_rt_next_dup(parentPtr);
                }
/*
 * insert leafPtr between parentPtr and nextPtr (next three macros)
 */
                tm_rt_next_dup(leafPtr) = nextPtr; /* leaf next is nextPtr */
/* leaf previous is parentPtr */
                tm_rt_prev_dup(leafPtr) = parentPtr;
/* parent next is leafPtr */
                tm_rt_next_dup(parentPtr) = leafPtr;
/* nodePtr point to head of list of duplicates */
            }
            else
            { 
/*
 * Leaf has a longer mask than or same mask as the current head and is not
 * root
 */
                if (tm_rt_same_mask(nodePtr, maskBits))
/* If same mask, and not an ARP clone and same index, fail */
                {
                    if (    (!arpClone)
#ifdef TM_USE_STRONG_ESL
                         && (devPtr->devIndex 
                                 == oRtePtr->rteOrigDevPtr->devIndex)
#endif /* TM_USE_STRONG_ESL */
                       )
                    {
                        errorCode = TM_EALREADY;
                        goto rtAddEntryExit;
                    }
                }
/*
 * insert leafPtr at head of list of duplicates, before the current head
 * (nodePtr)
 */
                parentPtr = tm_rt_parent(nodePtr);
                tm_rt_set_child_parent_key(parentPtr, leafPtr, ipAddress);
                tm_rt_prev_dup(nodePtr) = leafPtr;
                tm_rt_next_dup(leafPtr) = nodePtr;
/* nodePtr points to current head's internal node */
                nodePtr = tm_rt_leaf_to_int_node(nodePtr);
#ifdef TM_ERROR_CHECKING
/* Make sure there is a grandparent */
                if ( tm_16bit_one_bit_set(parentPtr->rtnFlags, TM_RTN_ROOT) )
                {
                    tfKernelError("tfRtAddEntry",
                                  "Duplicate head and no grandfather");
                    tfRtTreeUnlock();
                    tm_thread_stop;
                }
/* Verify that current head internal node is in the tree */
                if ( tm_16bit_bits_not_set( nodePtr->rtnFlags,
                                            TM_RTN_IN) )
                {
                    tfKernelError("tfRtAddEntry",
                                  "Duplicate head internal node not in tree");
                    tfRtTreeUnlock();
                    tm_thread_stop;
                }
#endif /* TM_ERROR_CHECKING */
/*
 * tfRtCopyNode will copy internal node of current head to internal node
 * of new head, and will reset the TM_RTN_IN flag in internal node of
 * current head. (That way only the head of a list of duplicates will
 * have its internal node in the tree. This is necessary so that we
 * are able to swap node, if we need to, when we remove the duplicate head
 * routing entry from the tree (see tfRtRemoveEntry() for algorithm).)
 */
                tfRtCopyNode(nodePtr, intNodePtr);
                nodePtr = leafPtr; /* point to head of list of duplicates */
            }
        }
        else
        {
/*
 * Not a duplicate key.
 * Find the number of leading key bits that match
 */
#ifdef TM_LITTLE_ENDIAN
            keyBits = tfRtBitOffset(nodePtr->rtnLSKey, ipAddress, &bittest);
#else /* !TM_LITTLE_ENDIAN */
/* rtnLSkey value arg, ipAddress value arg, bittest temp var, keyBits var arg */
            tm_ip_bit_offset(nodePtr->rtnLSKey, ipAddress, bittest, keyBits);
            longBitOffset = keyBits; /* Bit offset in one 32-bit word */
#endif /* TM_LITTLE_ENDIAN */
#ifdef TM_ERROR_CHECKING
            if (    (keyBits > 
                        tm_bytes_to_bits(tm_byte_count(sizeof(ttIpAddress))))
                 || (keyBits == 0))
            {
                tfKernelError("tfRtAddEntry", "Corrupted Patricia Tree");
                tfRtTreeUnlock();
                tm_thread_stop;
            }
#endif /* TM_ERROR_CHECKING */
/*
 * IntNodePtr internal node to insert. Set its bitOffset (used for addition
 * decisions) and BitMask (used for left/right decision when we search the
 * tree down).
 */
            /* Offset to first bit that differs */
            tm_rt_matching_bits(intNodePtr) = keyBits;
            /* bitmask of bit to be tested */
#ifdef TM_LITTLE_ENDIAN
            tm_ip_copy(bittest, intNodePtr->rtnNSBitMask);
#else /* !TM_LITTLE_ENDIAN */
            tm_ip_copy(tm_rtn_bit_mask(longBitOffset), 
                       intNodePtr->rtnNSBitMask);
#endif /* TM_LITTLE_ENDIAN */
/*
 * Find insertion point in the tree for intNodePtr. We know that one
 * of its children will be leafPtr. We need to find intNodePtr's parent
 * and its other child. We walk up the tree from the reached leaf (nodePtr),
 * using nodePtr and parentPtr to climb up the tree  until we find a parent/
 * child pair that will be intNodePtr's new parent (parentPtr)
 * and intNodePtr's new other child (nodePtr)). Our new parent should be the
 * first node that we find on  our way up that has a smaller bit offset than
 * the entry that we want to insert. The new child is the current child of
 * the new parent in the branch we climbed up.
 * nodePtr var arg, parentPtr non initialized var arg, keyBits value arg
 */
            tm_rt_tree_find_insert(nodePtr, parentPtr, keyBits);
/* Check whether we are not inserting a duplicate node */
#ifdef TM_ERROR_CHECKING
            tm_rt_check_dup_nodes(nodePtr, keyBits);
#endif /* TM_ERROR_CHECKING */

/*
 * Insertion point: parentPtr is parent, nodePtr will be one of the children
 * of IntNodePtr. Make intNodePtr the child of this parent 
 */
            tm_rt_set_child_parent_key(parentPtr, intNodePtr, ipAddress);
/* set children of intNodePtr (new entry leaf leafPtr, and nodePtr) */
            tm_rt_set_children_parent(intNodePtr, leafPtr, ipAddress,
                                      nodePtr);
            intNodePtr->rtnFlags = TM_RTN_IN|TM_RTN_NODE;
            if ( tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_NODE) )
            {
/*
 * If intNodePtr's other child (nodePtr) is a node,
 * move network entries from nodePtr network list whose mask lengths are <=
 * intNodePtr->rtnBitOffset (keyBits) to intNodePtr network list.
 * nodePtr walks the list of network entries of nodePtr.
 */
                nodePtr = tm_rt_next_net(nodePtr);
/*
 * Insert mask in parent node. parentPtr walks the list of network entries
 * for intNodePtr
 */
                parentPtr = intNodePtr;
                while (nodePtr != TM_RADIX_NULL_PTR)
                {
                    nextTempPtr = tm_rt_next_net(nodePtr);
                    maskLength = tm_rt_mask_len(nodePtr);
                    if ( maskLength <= keyBits )
                    {
/* remove network entry from current node. nodePtr value arg */
                        tm_rt_remove_net(nodePtr);
                        tfRtInsertNet(parentPtr, nodePtr, (int) maskLength);
/*
 * nodePtr has been inserted in intNodePtr list, start next insertion at
 * current nodePtr.
 */
                        parentPtr = nodePtr;
                    }
                    nodePtr = nextTempPtr;
                }
            }
            else
            {
                do
                {
                    if (   tm_16bit_one_bit_set(nodePtr->rtnFlags,
                                                TM_RTN_NETWORK)
                        && tm_16bit_bits_not_set( nodePtr->rtnFlags,
                                                  TM_RTN_LIST_NETWORK) )
                    {
/*
 * If intNodePtr's other child (nodePtr) is a leaf routing network entry
 * and if it is not already inserted in a tree node's network list,
 * insert it in intNodePtr network list, if its mask length is smaller
 * or equal to the bit offset for intNodePtr.
 */
                        maskLength = tm_rt_mask_len(nodePtr); /* mask length */
                        if (maskLength <= keyBits)
                        {
                            parentPtr = intNodePtr;
                            tfRtInsertNet(parentPtr, nodePtr, (int) maskLength);
                            nodePtr->rtnFlags |= TM_RTN_LIST_NETWORK;
                        }
                    }
                    nodePtr = tm_rt_next_dup(nodePtr);
                } while (nodePtr != (ttRtRadixNodePtr)0);
            }
            nodePtr = leafPtr; /* starting point of searching up the tree */
        }
/*
 * Node(s) inserted in tree. Insert leaf network entry in network list.
 * Set leaf and internal node flags
 */
        if ( maskBits != tm_bytes_to_bits(tm_byte_count(sizeof(ttIpAddress))) )
/* non host route (leaf network entry) */
        {
/*
 * If leaf network entry insert leaf node in appropriate network list.
 * Go up the tree (from the new leaf/head of duplicates) to add new node
 * in network list of the last node from leaf (first node from top)
 * that has its rtnBitOffset >= mask length.
 *
 * nodePtr points to leaf/head of duplicates. parentPtr is don't care
 * nodePtr var arg, parentPtr temp var, maskBits value arg.
 */
            tm_rt_tree_find_net_list(nodePtr, parentPtr, maskBits);
            if (tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_NODE))
            {
/*
 * nodePtr points to non leaf node where we want to insert network entry.
 * Insert leafPtr into network list of nodePtr. parentPtr is don't
 * care.
 * nodePtr value arg/temp var, parentPtr temp var, leafPtr, maskBits value
 * args
 */
                tfRtInsertNet(nodePtr, leafPtr, (int) maskBits);
                leafPtr->rtnFlags |= TM_RTN_LIST_NETWORK;
            }
/* Set route entry and leaf flags */
            rtePtr->rteFlags2 = TM_RTE2_UP;
            leafPtr->rtnFlags |= (tt16Bit)(TM_RTN_IN | TM_RTN_NETWORK);
        }
        else
        {
/* host route */
/* Set route entry and leaf flags */
            rtePtr->rteFlags2 = (tt16Bit)(TM_RTE2_UP | TM_RTE2_HOST);
            leafPtr->rtnFlags = TM_RTN_IN;
        }
        rtePtr->rteFlags = flags;
    }
/* If ARP/MCAST/BCAST mapping entry */
    if ( tm_16bit_one_bit_set(flags, TM_RTE_CLONED) )
    {
/* initialize the NS transmission timer to null */
        
/*
 * Add a timer for cloned entries - this timer should only be started if the
 * timer hasn't already been started and if this is the first IPv4 ARP entry
 * or if this is the first IPv6 ARP entry and Path MTU discovery is
 * enabled.
 */
        if (tm_context(tvRtArpTimerPtr) == TM_TMR_NULL_PTR)
        {
            timerParm1.genVoidParmPtr = (ttVoidPtr)0;
            tm_context(tvRtArpTimerPtr) = tfTimerAdd(
                                                  tfRtArpTimer,
                                                  timerParm1, /* unused */
                                                  timerParm1, /* unused */
                                                  tm_context(tvArpResolution),
                                                  TM_TIM_AUTO );
/* Do not check on errors from timer, since timer stops thread on failure */
        }

/* Counter for ALL ARP entries (both IPv4 and IPv6) */
        tm_context(tvRtArpEntriesCount)++;

/* Counter for just IPv4 ARP entries (only in dual mode). */
        {
            tm_context(tv4RtArpEntriesCount)++;
        }
        
    }
/* End of adding entry in tree */
    if (tm_16bit_one_bit_set(flags, TM_RTE_INDIRECT))
    {
        {
            if (
/* Either LAN or Not a zero address */
               (    tm_4_ll_is_lan(devPtr)
                 || !tm_ip_zero(rtePtr->rteGSGateway))
               )
            {
/*
 * Get router entry for the gateway. If we got a network routing entry
 * and network is LAN, the following code will clone to the ARP router
 * entry
 */
                oRtePtr = tfRtFindNClone(
#ifdef TM_USE_STRONG_ESL
                                           devPtr,
#endif /* TM_USE_STRONG_ESL */
                                           rtePtr->rteGSGateway
                                        );
                if (    (oRtePtr != TM_RTE_NULL_PTR)
                     && tm_16bit_bits_not_set( oRtePtr->rteFlags,
                                               TM_RTE_INDIRECT) )
                {
/* If one found in the tree, increase the ownership count */
                    tm_incr_owner_count(oRtePtr->rteOwnerCount);
                    if (    tm_16bit_one_bit_set(oRtePtr->rteFlags, TM_RTE_ARP)
/* ARP entry */
                         && tm_16bit_bits_not_set(oRtePtr->rteFlags, TM_RTE_STATIC)
/* ARP entry not entered by the user */
                         && (oRtePtr->rteTtl == TM_RTE_INF) )
/* Newly created ARP entry without a mapping */
                    {
/*
 * A newly created unmapped ARP entry has its rteTtl initialized to
 * TM_RTE_INF. Make sure that the ARP entry can be removed if the route is
 * removed without ever having its ARP entry mapped.
 */
                        oRtePtr->rteTtl = tm_context(tvRtTimeout);
                    }
/* Point to it */
                    rtePtr->rteClonedPtr = oRtePtr;
#ifndef TM_USE_STRONG_ESL
                    if (oRtePtr->rteOrigDevPtr != devPtr)
                    {
/* Copy device. For strong ESL, devPtr already match. */
                        rtePtr->rteDevPtr = oRtePtr->rteOrigDevPtr;
                        rtePtr->rteOrigDevPtr = oRtePtr->rteOrigDevPtr;
                    }
#endif /* TM_USE_STRONG_ESL */
/* Copy mhome */
                    rtePtr->rteMhomeIndex = oRtePtr->rteMhomeIndex;
                    rtePtr->rteOrigMhomeIndex = oRtePtr->rteOrigMhomeIndex;
/* 
 * Copy the MTU. Need to protect the route MTU, since it could be changed
 * dynamically due to an IPv6 router advertisement.
 */
                    tm_kernel_set_critical;
                    rtePtr->rteMtu = oRtePtr->rteMtu;
                    tm_kernel_release_critical;
                    if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_DYNAMIC) )
/* RIP, Router Discovery, or REDIRECT gateway entry */
                    {
                        if (tm_context(tvRtTimerPtr) == TM_TMR_NULL_PTR)
                        {
                            timerParm1.genVoidParmPtr = (ttVoidPtr)0;
                            tm_context(tvRtTimerPtr) =
                                      tfTimerAdd( tfRtTimer,
                                                  timerParm1, /* unused */
                                                  timerParm1, /* unused */
                                                  tm_context(tvRtTimerResolution),
                                                  TM_TIM_AUTO);
                        }
/*
 * Keep track of RIP, redirect entries, Path MTu and router discovery
 * entries
 */
                        tm_context(tvRtEntriesCount)++;
                    }
                    else
                    {
/* 
 * If the gateway is to one of our interface IP address, then
 * make the gateway entry clonable, so that ARP entry will be
 * created on the fly for any IP address that does not get found in
 * the tree. Note that in that case the gateway rteClonedPtr points to the
 * local route not the ARP entry, but that is OK because the clone
 * logic will create the ARP entry when tfRtGet() is called, and later
 * on tfRtGet() will return the ARP entry directly.
 */
                        if (tm_4_ll_is_lan(oRtePtr->rteOrigDevPtr))
                        {
                            if (
                                (
                                     tm_ip_match(
                                        tm_4_ip_addr(rtePtr->rteGSGateway),
                                        tm_ip_dev_addr(oRtePtr->rteOrigDevPtr, 
                                                       oRtePtr->rteOrigMhomeIndex))
                                )
                               )
                            {
/* Make the entry clonable */
                                rtePtr->rteFlags |= TM_RTE_CLONE;
                                tm_ip_copy(TM_IP_ALL_ONES, rtePtr->rteCloneMask);
                            }
                        }
                    }
                }
                else
                {
/* tfRtRemoveEntry will decrease the owner count */
                    rtePtr->rteOwnerCount++;
                    tfRtRemoveEntry(rtePtr);
                    errorCode = TM_EHOSTUNREACH;
                    goto rtAddEntryExit;
                }
            }
        }
    }
/* Reset all routing entry caches pointing to indirect/arp/local routes */
    tm_context(tvRtRadixTreeHead).rthResetCachesFlags |= (  TM_RTE_INDIRECT
                                                          | TM_RTE_CLONED
                                                          | TM_RTE_LOCAL );
    tm_context(tvRtRadixTreeHead).rthResetCachesAf |= (AF_INET | AF_INET6);
#ifdef TM_ERROR_CHECKING
#ifdef TM_USE_NETSTAT
#ifdef TM_DEBUG_LOGGING
/* Route entry inserted in the tree */
    tfxLogRteEntry("tfRtAddEntry", rtePtr);
#endif /* TM_DEBUG_LOGGING */
#endif /*TM_USE_NETSTAT*/
#endif /* TM_ERROR_CHECKING */
rtAddEntryExit:
    return errorCode;
}

/*
 * Given a destination address, return output device from route.
 */
ttUserInterface tfRtInterface(
                struct sockaddr_storage TM_FAR * sockAddrPtr
                )
{
    ttUserInterface  interfaceHandle;
    ttRteEntryPtr    rtePtr; 

    interfaceHandle = (ttUserInterface)0;

    tm_call_lock_wait(&tm_context(tvRtTreeLock));
    rtePtr = tfRtFindSockAddr(
#ifdef TM_USE_STRONG_ESL
                                (ttDeviceEntryPtr)0,
#endif /* TM_USE_STRONG_ESL */
                                sockAddrPtr,
                                TM_IP_ALL_ONES
                                  );
    if (rtePtr != (ttRteEntryPtr)0)
    {
        interfaceHandle = (ttUserInterface)rtePtr->rteDevPtr;
    }
    tfRtTreeUnlock(); 
    return interfaceHandle;
}

/*
 * tfRtFindSockAddr() is called by tfNgRtDestExists(), tfRtInterface().
 * Try and find a route entry for a given sock addr destination IP address
 * destination network mask/prefix length.
 * Function is called with routing table locked. So we don't need to
 * lock the table.
 */
static ttRteEntryPtr tfRtFindSockAddr(
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr                        devPtr,
#endif /* TM_USE_STRONG_ESL */
    const struct sockaddr_storage TM_FAR *  sockAddrPtr,
    tt4IpAddress                            netMask
    )
{
    ttRteEntryPtr    rtePtr; 
    tt4IpAddress     ipAddress;

    {
        tm_ip_copy( (tt4IpAddress)(sockAddrPtr->addr.ipv4.sin_addr.s_addr),
                    ipAddress);
    }
    tm_ip_copy(tm_ip_net(ipAddress, netMask), ipAddress);
    rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                       devPtr,
#endif /* TM_USE_STRONG_ESL */
                       ipAddress);

    return rtePtr;
}

#if (defined(TM_SNMP_MIB) && defined(TM_USE_IPV4))
/****************************************************************************
* FUNCTION: tfSnmpRtFindIpRouteDest
*
* PURPOSE: Called by the SNMP Agent to get a snapshot of a routing entry
*
* PARAMETERS:
*   ipRouteEntryPtr:
*       Points to a buffer allocated by the SNMP Agent which, on return,
*       has been set to a copy of the routing entry that was found.
*   ipRouteDest:
*       The key of the routing entry in the routing tree that we want to
*       find. This already has the mask applied. This is set to the value of
*       the ipRouteDest field of the ipRouteEntry by the SNMP Agent.
*
* RETURNS:
*   TM_ENOERROR:
*       Successfully found routing entry.
*   TM_ENOENT:
*       Could not find routing entry.
*
* NOTES:
*
****************************************************************************/
int tfSnmpRtFindIpRouteDest(
#ifdef TM_USE_STRONG_ESL
                            ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                            ttRteEntryPtr    ipRouteEntryPtr,
                            tt4IpAddress     ipRouteDest)
{
    ttRteEntryPtr rtePtr;
    int           retVal;

/* initialize default values */
    retVal = TM_ENOENT; /* assume row not found */

/* lock the routing tree for tfRtFind */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));

/* attempt to find the routing entry */
    rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                       devPtr,
#endif /* TM_USE_STRONG_ESL */
                       ipRouteDest);
    if (rtePtr != TM_RTE_NULL_PTR)
    {
/* we found the routing entry having the specified key */
        retVal = TM_ENOERROR;

        if (ipRouteEntryPtr != TM_RTE_NULL_PTR)
        {
/* grab a snapshot of the routing entry for the SNMP Agent */
            tm_bcopy(rtePtr, ipRouteEntryPtr, sizeof(ttRteEntry));
        }
    }

/* unlock the routing tree */
    tfRtTreeUnlock();

    return retVal;
}
#endif /* (defined(TM_SNMP_MIB) && defined(TM_USE_IPV4)) */



#if (defined(TM_SNMP_MIB) && defined(TM_USE_NG_MIB2))
/****************************************************************************
* FUNCTION: tfSnmpNgRtFindIpRouteDest
*
* PURPOSE: Called by the SNMP Agent to get a snapshot of a routing entry
*
* PARAMETERS:
*   ipRouteEntryPtr:
*       Points to a buffer allocated by the SNMP Agent which, on return,
*       has been set to a copy of the routing entry that was found.
*   ipRouteDest:
*       The key of the routing entry in the routing tree that we want to
*       find. This already has the mask applied. This is set to the value of
*       the ipRouteDest field of the ipRouteEntry by the SNMP Agent.
*
* RETURNS:
*   TM_ENOERROR:
*       Successfully found routing entry.
*   TM_ENOENT:
*       Could not find routing entry.
*
* NOTES:
*
****************************************************************************/
int tfSnmpNgRtFindIpRouteDest(
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
    ttRteEntryPtr    ipRouteEntryPtr,
    ttIpAddressPtr   ipRouteDest)
{
    ttRteEntryPtr rtePtr;
    int           retVal;

/* initialize default values */
    retVal = TM_ENOENT; /* assume row not found */

/* validate input parameters */
    tm_assert(tfSnmpNgRtFindIpRouteDest,
              (ipRouteEntryPtr != TM_RTE_NULL_PTR));

/* lock the routing tree for tfRtFind */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));

/* attempt to find the routing entry */
    rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                      devPtr,
#endif /* TM_USE_STRONG_ESL */
                     *ipRouteDest);

    if (rtePtr != TM_RTE_NULL_PTR)
    {
/* we found the routing entry having the specified key */
        retVal = TM_ENOERROR;
/* grab a snapshot of the routing entry for the SNMP Agent */
        tm_bcopy(rtePtr, ipRouteEntryPtr, sizeof(ttRteEntry));
    }

/* unlock the routing tree */
    tfRtTreeUnlock();

    return retVal;
}
#endif /* (defined(TM_SNMP_MIB) && defined(TM_USE_IPV4)) */




/*
 * tfRtFindIpAddrIpNet() is called by tf4RtDelRoute(), tf6RtDelRoute(),
 * tfRtArpCommand(), and by tfRtAddEntry() to
 * try and find the unique route entry for a given destination IP address
 * destination network mask.
 * Function is called with routing table locked. So we don't need to
 * lock the table.
 * IP address and network mask already anded before this call, so we
 * do not have to do it here.
 * Parameters:
 * ipAddress: destination Ip address
 * netMask: destination network mask
 * Return value:
 * Pointer to a routing entry if match is found
 * Null pointer otherwise
 */
ttRteEntryPtr tfRtFindIpAddrIpNet (
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress    ipAddress,
    tt4IpAddress    netMask 
    )
{
    ttRtRadixNodePtr    nodePtr;
    ttRteEntryPtr       rtePtr;
#ifdef TM_USE_STRONG_ESL
    ttRteEntryPtr       tempRtePtr;
#endif /* TM_USE_STRONG_ESL */


    rtePtr = TM_RTE_NULL_PTR;
/* Start at head of tree */
    nodePtr = tm_context(tvRtTreeHeadPtr);
/* Go down the tree until we reach a leaf */
    tm_ip_copy(tm_ip_net(ipAddress, netMask), ipAddress);
    tm_rt_tree_search(nodePtr, ipAddress);
#ifdef TM_ERROR_CHECKING
    tm_rt_check_null_leaf(nodePtr);
#endif /* TM_ERROR_CHECKING */
/* We have now reached a leaf. Do keys match exactly? */
    if ( tm_ip_match(nodePtr->rtnLSKey, ipAddress) )
    {
/* Keys match exactly */
        if ( tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_ROOT) )
/*
 * Check for default entry.
 * Were we asked for the default entry? If so, make sure we point to it.
 * (If there is no default entry, next duplicate is NULL, and we will
 * return a null pointer (no match).) Note that this check will also
 * return a null pointer, in case the user had requested a route to
 * limited broadcast address.
 */
        {
            nodePtr = tm_rt_next_dup(nodePtr);
#ifdef TM_USE_STRONG_ESL
/*
 * This is an end point of the tree (either left or right). We need to go
 * through all the leaves to find a match on the device if it is non null.
 */
            if (devPtr != (ttDeviceEntryPtr)0)
            {
                while (nodePtr != (ttRtRadixNodePtr)0)
                {
                    tempRtePtr = tm_rt_leaf_to_rte(nodePtr);
                    if (tempRtePtr->rteOrigDevPtr == devPtr)
                    {
                        break; /* exact match */
                    }
                    nodePtr = tm_rt_next_dup(nodePtr);
                 }
            }
#endif /* TM_USE_STRONG_ESL */
        }
/*
 * Check the list of duplicate (same key) for an exact network mask match.
 */
        while (   (nodePtr != TM_RADIX_NULL_PTR )
               && (tm_ip_not_match(nodePtr->rtnLSMask, netMask))
              )
        {
            nodePtr = tm_rt_next_dup(nodePtr);
        }
#ifdef TM_USE_STRONG_ESL
        if ( (devPtr != (ttDeviceEntryPtr)0) && (nodePtr != TM_RADIX_NULL_PTR) )
        {
            do
            {
                tempRtePtr = tm_rt_leaf_to_rte(nodePtr);
                if (devPtr == tempRtePtr->rteOrigDevPtr)
                {
                    break;
                }
                nodePtr = tm_rt_next_dup(nodePtr);
            }
            while (   (nodePtr != TM_RADIX_NULL_PTR)
                   && (tm_ip_match(nodePtr->rtnLSMask, netMask))
              );
        }
#endif /* TM_USE_STRONG_ESL */
        rtePtr = tm_rt_leaf_to_rte(nodePtr);
    }
    return rtePtr;
}


/*
 * tfRtRemoveEntry function description:
 * This function is called with the routing table locked.
 * Try and remove the router table entry pointed to by rtEntryPtr, from
 * the routing tree.
 * 0) If function called in the context of a routing tree walk, postpone
 *    removal when we exit from the tree walk, by queueing the entry
 *    in the routing tree remove list, if not already queued.
 *    If function is not called in the context of the routing tree walk, make
 *    sure to dequeue the entry if it had been queued for later removal in
 *    a previous tree walk.
 * 1) Derefence leafPtr, and remove the leaf from the tree
 *   Note that since we are removing a routing entry, and a routing entry
 *   contains a leaf node and an internal node, we have to make sure that
 *   its internal node is not in the tree. If the routing entry internal node
 *   is in the tree, we need to copy its contents in a free internal node.
 *     a. If Leaf is part of a network list, remove from the list
 *     b. If duplicate member (parent is a leaf), remove from duplicate
 *        chain. (We know that routing entry internal node is free (see
 *        tfRtAddEntry.)
 *     c. else If duplicate head (next duplicate non nil), make the
 *        next duplicate in the chain of duplicates, the head of the chain,
 *        by making the current duplicate head's parent the parent of the
 *        next duplicate. If routing entry internal node is in the tree,
 *        swap it with the next duplicate internal node which we know is
 *        not in the tree (see tfRtAddEntry()).
 *    d. else (not a duplicate), remove the node (parent of the leaf) from
 *       the tree, and make the grandparent of the leaf (parent of the node)
 *       the parent of the node's other child. If the node has a network list
 *       attached to it, move it to the node's other child if it is a node.
 *       If the other child is not a node, just remove all the leaves from
 *       the node's network list. If routing entry internal node is in the
 *       tree, swap it with node (parent of the leaf) which is free since
 *       we were about to remove it.
 * 2) If entry is cloneable, remove loop back routing entry.
 * 3) Deal with local route removal:
 *       If we are removing a local entry, Remove any entry sharing same
 *       device/multihome index as the local entry we are removing,
 *       (i.e ARP/MCAST/BCAST mapping entry, or gateway
 *        entry pointing to the removed (non up) local entry or gateway
 *        entry pointing to an ARP entry for the local entry to be removed)
 * 4) Free the entry:
 *    a. We reset the TM_RTE2_UP and TM_RTN_IN flags.
 *    b. We call tfRtUnGet() which will decrease the owner count and
 *       free the entry if the owner count is 0.
 * Parameters:
 * rtePtr: pointer to routing entry we want to remove
 *
 */
static void tfRtRemoveEntry ( ttRteEntryPtr rtePtr )
{
    ttRtRadixNodePtr    leafPtr; /* leaf and temporary */
    ttRtRadixNodePtr    nodePtr; /* internal node to be removed */
    ttRtRadixNodePtr    otherNodePtr; /* temporary */
    ttRtRadixNodePtr    nextPtr; /* temporary */
    ttRtRadixNodePtr    nextTempPtr; /* temporary */
    ttRteEntryPtr       curRtePtr;
    ttRteEntryPtrPtr    prevNextRtePtrPtr;
    ttIpAddress         key;
    tt16Bit             maskLength;
    tt16Bit             leafFlags;

#ifdef TM_ERROR_CHECKING
#ifdef TM_USE_NETSTAT
#ifdef TM_DEBUG_LOGGING
    tfxLogRteTable("tfRtRemoveEntry");
    tfxLogArpTable("tfRtRemoveEntry");
    tfxLogRteEntry("tfRtRemoveEntry", rtePtr);
#endif /* TM_DEBUG_LOGGING */
#endif /*TM_USE_NETSTAT*/
    if (rtePtr == TM_RTE_NULL_PTR)
    {
        tfKernelError("tfRtRemoveEntry", "null pointer removal");
                            /* Severe Error */
        tfRtTreeUnlock();
        tm_thread_stop;
    }
#endif /* TM_ERROR_CHECKING */
    if (tm_context(tvRtRadixTreeHead).rthTreeWalkCount != TM_16BIT_ZERO)
    {
/*
 * In a tree walk. Postpone removing the routing entry until we return from
 * Tree walk routine
 */
        if (tm_16bit_bits_not_set(rtePtr->rteFlags2, TM_RTE2_IN_REM_LIST))
        {
/* Insert in remove list if not already in */
            rtePtr->rteNextRemPtr = 
                        tm_context(tvRtRadixTreeHead).rthRteRemListPtr;
            tm_context(tvRtRadixTreeHead).rthRteRemListPtr = rtePtr;
            rtePtr->rteFlags2 |= TM_RTE2_IN_REM_LIST;
            if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_INDIRECT) )
            {
                if (rtePtr->rteClonedPtr != TM_RTE_NULL_PTR)
                {
/* If gateway entry points to cloned route, dereference cloned entry. */
                    tfRtUnGet(rtePtr->rteClonedPtr);
                    rtePtr->rteClonedPtr = TM_RTE_NULL_PTR;
                }
            }
        }
    }
    else
    {
        if (tm_16bit_one_bit_set(rtePtr->rteFlags2, TM_RTE2_IN_REM_LIST))
        {
/* Remove from remove list if already in */
            curRtePtr = tm_context(tvRtRadixTreeHead).rthRteRemListPtr;
            prevNextRtePtrPtr =
                             &(tm_context(tvRtRadixTreeHead).rthRteRemListPtr);
            while ((curRtePtr != (ttRteEntryPtr)0) && (curRtePtr != rtePtr))
            {
                prevNextRtePtrPtr = &(curRtePtr->rteNextRemPtr);
                curRtePtr  = curRtePtr->rteNextRemPtr;
            }
            if (curRtePtr == rtePtr)
/* Should always be true */
            {
                *prevNextRtePtrPtr = rtePtr->rteNextRemPtr;
                rtePtr->rteFlags2 &= ~TM_RTE2_IN_REM_LIST;
                rtePtr->rteNextRemPtr = (ttRteEntryPtr)0;
            }
        }
        otherNodePtr = (ttRtRadixNodePtr)0; /* to avoid compiler warning */
/*
 * Remove ARP timer if removing the last alive ARP/MCAST/BCAST
 * mapping entry.
 */
        if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_CLONED ) )
        {

/* Counter for ALL ARP entries (both IPv4 and IPv6) */        
            tm_context(tvRtArpEntriesCount)--;

/* Counter for just IPv4 ARP entries. */
            {
                tm_context(tv4RtArpEntriesCount)--;
            }

            if (tm_context(tv4RtArpEntriesCount) == 0)
            {
                if (tm_context(tvRtArpTimerPtr) != TM_TMR_NULL_PTR)
                {
                    tm_timer_remove(tm_context(tvRtArpTimerPtr));
                    tm_context(tvRtArpTimerPtr) = TM_TMR_NULL_PTR;
                }
            }

        }
/*
 * If we are removing the last Router Discovery, REDIRECT or RIP entry,
 * remove the dynamic routing entry aging timer.
 */
        if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_DYNAMIC) )
        {
/* decrease the timer count */
            tm_context(tvRtEntriesCount)--;
            if (tm_context(tvRtEntriesCount) == 0)
            {
                if (tm_context(tvRtTimerPtr) != TM_TMR_NULL_PTR)
                {
/* This could be called from the timer thread. */
                    tm_timer_remove(tm_context(tvRtTimerPtr));
                    tm_context(tvRtTimerPtr) = TM_TMR_NULL_PTR;
                }
            }
        }
/* End of timer removals */
/* point to leaf node */
        leafPtr = &rtePtr->rteLeafNode;
        leafFlags = leafPtr->rtnFlags;
        key = leafPtr->rtnLSKey;
#ifdef TM_ERROR_CHECKING
/*
 * Make sure routing entry is not corrupted. leaf node better not be a
 * root or an internal node. Also check that the entry has
 * not been removed before.
 */
        if (   tm_16bit_one_bit_set(leafFlags, TM_RTN_ROOT|TM_RTN_NODE)
            || tm_16bit_bits_not_set(leafFlags, TM_RTN_IN) )
        {
            tfKernelError("tfRtRemoveEntry",
                          "Routing entry root or corrupted or removed twice");
                          /* Severe Error */
            tfRtTreeUnlock();
            tm_thread_stop;
        }
#endif /* TM_ERROR_CHECKING */
/* Remove entry from the tree */
        if ( tm_16bit_one_bit_set(leafFlags, TM_RTN_LIST_NETWORK) )
        {
/* If Leaf is a network route, remove from its network list */
#ifdef TM_ERROR_CHECKING
            if (tm_rt_prev_net(leafPtr) == TM_RADIX_NULL_PTR)
            {
                tfKernelError("tfRtRemoveEntry",
                              "Network routing entry not on a network list");
                              /* Severe Error */
                tfRtTreeUnlock();
                tm_thread_stop;
            }
#endif /* TM_ERROR_CHECKING */
/* remove from the list. leafPtr value arg */
            tm_rt_remove_net(leafPtr);
            leafPtr->rtnFlags &= ~TM_RTN_LIST_NETWORK;
        }
        nodePtr = tm_rt_parent(leafPtr);
        nextPtr = tm_rt_next_dup(leafPtr);
/*
 * check whether parent is a leaf.
 */
        if ( tm_16bit_bits_not_set(nodePtr->rtnFlags, TM_RTN_NODE) )
        {
/* Duplicate member (parent is a leaf), remove from duplicate chain */
#ifdef TM_ERROR_CHECKING
            if (tm_ip_not_match(nodePtr->rtnLSKey, key))
            {
                tfKernelError("tfRtRemoveEntry",
                              "duplicate chain with non duplicate keys!");
                                /* Severe Error */
                tfRtTreeUnlock();
                tm_thread_stop;
            }
/* The duplicate member internal node should not be in the tree */
            if ( tm_16bit_one_bit_set( rtePtr->rteInternalNode.rtnFlags,
                                       TM_RTN_IN) )
            {
                tfKernelError("tfRtRemoveEntry",
                              "duplicate member with internal node in tree!");
                            /* Severe Error */
                tfRtTreeUnlock();
                tm_thread_stop;
            }
#endif /* TM_ERROR_CHECKING */
            tm_rt_remove_dup_member(leafPtr, nodePtr, nextPtr); /* value args */
        }
        else
        {
            if (nextPtr != TM_RADIX_NULL_PTR)
            {
/* duplicate head (next duplicate non nil, and parent is node) */
#ifdef TM_ERROR_CHECKING
                if (   tm_16bit_one_bit_set(nextPtr->rtnFlags, TM_RTN_NODE)
                    || tm_ip_not_match(nextPtr->rtnLSKey, key)
                    )
                {
                    tfKernelError("tfRtRemoveEntry",
                                  "duplicate chain with non duplicate!");
                                /* Severe Error */
                    tfRtTreeUnlock();
                    tm_thread_stop;
                }
#endif /* TM_ERROR_CHECKING */
/*
 * make the next duplicate in the chain of duplicates, the head of the chain.
 * otherNodePtr is parent, nextPtr (next duplicate) is child.
 */
                otherNodePtr = nodePtr;
                tm_rt_set_child_parent_key(otherNodePtr, nextPtr, key);
/*
 * Get internal node of nextPtr. This is the free internal node we can use to
 * copy internal node of the routing entry that we are removing. See below.
 */
                nodePtr = tm_rt_leaf_to_int_node(nextPtr);
#ifdef TM_ERROR_CHECKING
/* The next duplicate internal node should not be in the tree */
                if (tm_16bit_one_bit_set(nodePtr->rtnFlags, TM_RTN_IN))
                {
                    tfKernelError("tfRtRemoveEntry",
                                  "next duplicate internal node is not free!");
                                /* Severe Error */
                    tfRtTreeUnlock();
                    tm_thread_stop;
                }
#endif /* TM_ERROR_CHECKING */
            }
            else
/*
 * Not a duplicate: remove nodePtr (parent of the leaf) from the tree, and
 *   make the grandparent the parent of nodePtr's other child.
 *   If nodePtr has a network list attached to it, move it to
 *   nodePtr 's other child if it is a node. If the other child
 *   is not a node, just remove all the leaves from nodePtr's network list.
 *   Since we are removing the routing entry, and the routing entry contains
 *   a leaf node and an internal node, we have to make sure that its internal
 *   node is not in the tree. If routing entry internal node is in the tree,
 *   copy its contents in nodePtr and keep nodePtr around.
 */
            {
                nextPtr = tm_rt_parent(nodePtr); /* grandparent */
                otherNodePtr = tm_rt_get_other_child(nodePtr, leafPtr);
/*
 * make grandparent (nextPtr) the parent of the parent (nodePtr)'s
 * other child (otherNodePtr).
 */
                tm_rt_set_child_parent_key(nextPtr, otherNodePtr, key);
                if ( tm_16bit_one_bit_set(otherNodePtr->rtnFlags, TM_RTN_NODE) )
                {
                    nextPtr = tm_rt_next_net(nodePtr);
                    while (nextPtr != TM_RADIX_NULL_PTR)
                    {
                        maskLength = tm_rt_mask_len(nextPtr);
                        nextTempPtr = tm_rt_next_net(nextPtr);
/* remove network entry from nodePtr' network list */
                        tm_rt_remove_net(nextPtr); /* nextPtr value arg */
                        tfRtInsertNet(otherNodePtr, nextPtr, (int) maskLength);
/*
 * nextPtr has been inserted in other child's list, start next insertion at
 * current node (nextPtr).
 */
                        otherNodePtr = nextPtr;
                        nextPtr = nextTempPtr;
                    }
                }
                else
/*
 * Other child is a leaf. Just reset network list flag from all the leaves
 * from nodePtr's network list.
 */
                {
                    nextPtr = tm_rt_next_net(nodePtr);
                    tm_rt_next_net(nodePtr) = TM_RADIX_NULL_PTR;
                    while (nextPtr != TM_RADIX_NULL_PTR)
                    {
                        nextPtr->rtnFlags &= ~TM_RTN_LIST_NETWORK;
                        nextPtr = tm_rt_next_net(nextPtr);
/* remove mask from node */
                    }
                }
            }

/* nodePtr no longer in the tree */
            nodePtr->rtnFlags &= ~TM_RTN_IN;
            otherNodePtr = &rtePtr->rteInternalNode; /* internal node */
            if (tm_16bit_one_bit_set(otherNodePtr->rtnFlags, TM_RTN_IN))
            {
/*
 * internal node (otherNodePtr) is in the tree. We need to save
 * its contents in free nodePtr. That way otherNodePtr can be freed
 * along with the leafPtr. (They are both members of the same
 * routing entry.) tfRtCopyNode will reset the TM_RTN_IN flag in
 * otherNodePtr. After the copy, nodePtr is in the tree.
 */
                tfRtCopyNode(otherNodePtr, nodePtr);
            }
        }
/* End of removal from the tree */
/* If entry is a local entry */
        if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_LOCAL) )
        {
/*
 * remove any entry sharing same device/multihome index as the local
 * entry we are removing (i.e ARP/MCAST/BCAST mapping entry,
 * or gateway entry pointing to the removed (non up) local entry or
 * associated ARP entry). Note that we could not find a matching
 * IPv6 default router because the default routes uses the reserved
 * mhome index not used to configure local routes.
 */
            (void)tfRtTreeWalk(tfRtRemObsLocalCB, rtePtr, TM_8BIT_ZERO);
        }
#ifdef TM_SNMP_CACHE
        if (rtePtr->rteSnmpCacheEntry.sceNode.nodeNextPtr != (ttVoidPtr)0)
        {
            tfSnmpdCacheDeleteRoute(rtePtr);
        }
#endif /* TM_SNMP_CACHE */
/* Free the entry */
        rtePtr->rteLeafNode.rtnFlags &= ~TM_RTN_IN;
        rtePtr->rteFlags2 &= ~TM_RTE2_UP;
/* Remove default router entry if it points to this entry */

#ifdef TM_ERROR_CHECKING
/* Keep track of out of tree non recycled entries */
        {
            tm_context(tvRteLimboCount)++;
        }
#endif /* TM_ERROR_CHECKING */
        tfRtUnGet(rtePtr);
/* End of freeing the entry */
    }
}


/*
 * Clone a routing entry for ARP entry, our own IP address, directed
 * broadcast or multicast entry. Called with routing tree lock set. Called
 * by tfRtGet() via (tfRtFindNClone) (itself called from the layers 3-4
 * (including ARP), or from user add ARP entry) or tfRtAddEntry() (for an
 * indirect route for the gateway ARP router entry).
 * 0) Check for permanent cloned entries (our own IP address,
 *    multicast/broadcast)
 * 1) We allocate a new routing entry
 * 2) We use the routing fields from the routing entry we want to clone
 *    (for the mask field we use the clone mask field)
 * 3) Insert the cloned routing entry in the tree.
 * 4) return cloned routing entry.
 *
 * rtePtr       pointer to local lan link layer network routing entry,
 * destIpAddr   destination IP address
 */

static ttRteEntryPtr tfRtClone (
    ttRteEntryPtr rtePtr, 
    tt4IpAddress   destIpAddr
    )
{
    ttRteEntryPtr       cloneRtePtr;
    ttDeviceEntryPtr    devPtr;
    tt32Bit             ttlTimeout;
    int                 linkBroadMcast;
    int                 errorCode;
#ifdef TM_SNMP_CACHE
    int                 index;
#endif /* TM_SNMP_CACHE */
    tt16Bit             flags;
/* link layer address */
    tt8Bit              linkPhysAddr[TM_MAX_PHYS_ADDR];
    tt8BitPtr           destLinkPhysAddrPtr;
    int                 isLan;
    tt16Bit             mhomeIndex;
    int                 failed;

/*
 * Default timeout for an ARP entry is TM_RTE_INF before a link
 * layer mapping is found.
 * ARP timeout initialized when the ARP code/User store a mapping.
 * If this ARP entry is created because a route is being inserted, then
 * ARP timeout will be overwritten with tvRtTimeout in tfRtAddEntry(),
 * to make sure that the ARP cache entry will be eventually removed,
 * if we fail to get an ARP reply from the gateway.
 */
    cloneRtePtr = TM_RTE_NULL_PTR;
    linkBroadMcast = TM_ETHER_UCAST;
    ttlTimeout = tm_context(tvArpTimeout);
    devPtr = rtePtr->rteOrigDevPtr;
    mhomeIndex = rtePtr->rteOrigMhomeIndex;
    isLan = tm_4_ll_is_lan(devPtr);
/*
 * First check for permanent cloned routing entries (our own IP address,
 * or multicast/broadcast).
 */
    if (tm_ip_match(destIpAddr, rtePtr->rteHSDevIpAdd))
/* If destination is to our own interface, interface is loopback */
    {
      /* Mapped entry */
#if ( (defined(TM_SINGLE_INTERFACE_HOME) || defined(TM_LOOP_TO_DRIVER)) )

/* Make the entry non permanent, since it points to the local device */
        flags = TM_RTE_CLONED | TM_RTE_LINK_LAYER;
        linkBroadMcast = -1; /* So that the physical address is copied below */

#else /* !defined(TM_SINGLE_INTERFACE_HOME) && !defined(TM_LOOP_TO_DRIVER) */
        if (tm_context(tvLoopbackDevPtr) == TM_DEV_NULL_PTR)
        {
#ifdef TM_ERROR_CHECKING
            tfKernelWarning("tfRtClone",
                            "No loop back driver configured");
#endif /* TM_ERROR_CHECKING */
            goto rtCloneExit;
        }
        flags = TM_RTE_LINK_LAYER; /* permanent entry */
        ttlTimeout = TM_RTE_INF; /* permanent entry */

#endif /* !defined(TM_SINGLE_INTERFACE_HOME) && !defined(TM_LOOP_TO_DRIVER) */
    }
    else
    {
            linkBroadMcast = tfIpBroadMcastToLan(
                destIpAddr,
                devPtr,
                mhomeIndex,
                linkPhysAddr);
            if ( linkBroadMcast == TM_ETHER_UCAST )
            {
/* Arp entry for LAN interface */
                flags = TM_RTE_CLONED | TM_RTE_ARP;
                if (!isLan)
                {
/* Do not add/Get an ARP entry on non LAN interfaces */
                    cloneRtePtr = rtePtr;
                }
            }
            else
/*
 * If we mapped to Broadcast or multicast link address
 */
            {
                if (linkBroadMcast == TM_ETHER_MULTI)
                {
/*
 * Multicast entry.
 */
                    flags = (tt16Bit)(   TM_RTE_CLONED
                                         | TM_RTE_LINK_LAYER
                                         | TM_RTE_MCAST );
                }
                else
                {
/*
 * Broadcast entry.
 */
                    flags = (tt16Bit)(   TM_RTE_CLONED
                                         | TM_RTE_LINK_LAYER
                                         | TM_RTE_BCAST );
                }
            }
    }
    if (cloneRtePtr == TM_RTE_NULL_PTR)
    {
/* If not permanent entry, check on ARP cache limit */
        if (    (flags != TM_RTE_LINK_LAYER)
             && (    tm_context(tvRtArpEntriesCount) 
                  >= tm_context(tvArpMaxEntries) ) )
        {
            failed = 0;
            while (failed < 2)
            {
/* We reached the limit. Try and find the least used entry. */
                (void)tfRtTreeWalk( tfRtArpGetLeastUsedCB, &cloneRtePtr,
                                    TM_8BIT_ZERO );
                if (cloneRtePtr != TM_RTE_NULL_PTR)
                {
/* Remove the least used ARP entry */
                    tfRtRemoveEntry(cloneRtePtr);
                    break;
                }
                else
                {
                    failed++;
                    if (failed == 1)
                    {
/* Try and free as many ARP/MCAST/BCAST mapping entries as we can */
                        tm_rt_reset_caches( TM_RTE_CLONED, 
                                            (AF_INET | AF_INET6) , 
                                            TM_LOC_NOWAIT );
                    }
                    else
                    {
#ifdef TM_ERROR_CHECKING
                        tfKernelWarning("tfRtClone", "ARP cache full");
#endif /* TM_ERROR_CHECKING */
                        goto rtCloneExit;
                    }
                }
            }
        }
/* Add it to the Patricia tree */
/* Allocate a new routing entry */
        cloneRtePtr = (ttRteEntryPtr)tm_recycle_alloc(
                                                (unsigned)sizeof(ttRteEntry),
                                                 TM_RECY_RTE );
        if (cloneRtePtr != TM_RTE_NULL_PTR)
        { 
/* Brand new entry. Initialize owner count */
            cloneRtePtr->rteOwnerCount = 1; /* no need to be critical here */
#ifdef TM_SNMP_CACHE
/* Initialize SNMPD/NSTAT cache index */
            cloneRtePtr->rteSnmpCacheEntry.sceTableIndex = TM_SNMPC_NO_INDEX;
#endif /* TM_SNMP_CACHE */
            tm_ip_copy(rtePtr->rteHSDevIpAdd, cloneRtePtr->rteHSDevIpAdd);
            cloneRtePtr->rteHSPhysAddrLen = rtePtr->rteHSPhysAddrLen;
/* Need to protect the route MTU, since it could be changed dynamically
   due to an IPv6 router advertisement. */
            tm_kernel_set_critical;
            cloneRtePtr->rteMtu = rtePtr->rteMtu;
            tm_kernel_release_critical;
            
/* NOTE: we do not set the rteTtl for an IPv6 entry to TM_RTE_INF here,
   instead we wait until it transitions out of the CLONED/INCOMPLETE state */
            errorCode = tfRtAddEntry( tm_context(tvRtTreeHeadPtr),
                                      cloneRtePtr,
                                      devPtr,
                                      destIpAddr,
                                      rtePtr->rteCloneMask,
                                      mhomeIndex,
                                      (int)rtePtr->rteHops,
                                      ttlTimeout,
                                      flags );
            if (errorCode == TM_ENOERROR)
            {
                if (linkBroadMcast != TM_ETHER_UCAST)
                {
/* Broadcast/multicast/driver link address, store the mapping */
                    if (isLan)
/* Note rteHSPhysAddrLen is zero on non LAN interfaces. */
                    {
                        destLinkPhysAddrPtr = &(cloneRtePtr->rteHSPhysAddr[0]);
                        if (linkBroadMcast == -1)
                        {
/* Sending to our own IP address. Store link layer address. */
                            tm_bcopy(devPtr->devPhysAddrArr,
                                     destLinkPhysAddrPtr,
                                     rtePtr->rteHSPhysAddrLen);
                        }
                        else
                        {
/* LAN broadcast/multicast mapping as stored by tfIpBroadMcastToLan() */
#ifdef TM_LINT
LINT_UNINIT_AUTO_BEGIN(linkPhysAddr)
#endif /* TM_LINT */
                            tm_bcopy(linkPhysAddr, destLinkPhysAddrPtr,
                                     rtePtr->rteHSPhysAddrLen);
#ifdef TM_LINT
LINT_UNINIT_AUTO_END(linkPhysAddr)
#endif /* TM_LINT */
                        }
                    }
                }
#ifdef TM_SNMP_CACHE
                if (flags == (TM_RTE_CLONED | TM_RTE_ARP))
                {
                    index = TM_SNMPC_ARP_INDEX;
                }
                else
                {
                    index = TM_SNMPC_ROUTE_INDEX;
                }
                tfSnmpdCacheInsertRoute(cloneRtePtr, index);
#endif /* TM_SNMP_CACHE */
#if !defined(TM_SINGLE_INTERFACE_HOME) && !defined(TM_LOOP_TO_DRIVER)
                if (flags == TM_RTE_LINK_LAYER) /* permanent loop back entry */
                {
                    rtePtr->rteClonedPtr = cloneRtePtr;
                    cloneRtePtr->rteDevPtr = tm_context(tvLoopbackDevPtr);
                }
#endif /* !defined(TM_SINGLE_INTERFACE_HOME) && !defined(TM_LOOP_TO_DRIVER) */
            }
            else
            {
/* routing entry could not be inserted. Recycle it */
#ifdef TM_ERROR_CHECKING
/* Keep track of out of tree non recycled entries */
                tfKernelWarning(
                    "tfRtClone", 
                    "Could not insert ARP entry in routing tree");
                tm_context(tvRteLimboCount)++;
#endif /* TM_ERROR_CHECKING */
                tfRtUnGet(cloneRtePtr);
                cloneRtePtr = TM_RTE_NULL_PTR;
            }
        }
#ifdef TM_ERROR_CHECKING
        else
        {
            tfKernelError("tfRtClone", "No buffer to allocate clone entry");
        }
#endif /* TM_ERROR_CHECKING */
    }
rtCloneExit:
/* return cloned entry */
    return cloneRtePtr;
}

/*
 * Get router entry for the Destination IP address 
 * If REJECT flag is set, return null pointer.
 * If we got a LAN network entry (TM_RTE_CLONE set), clone a ARP router entry.
 *
 * destIpAddr value arg
 */
static ttRteEntryPtr tfRtFindNClone (
#ifdef TM_USE_STRONG_ESL
   ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress   destIpAddr
    )
{
    ttRteEntryPtr rtePtr;

    rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                      devPtr,
#endif /* TM_USE_STRONG_ESL */
                      destIpAddr);
    if (rtePtr != TM_RTE_NULL_PTR) 
    {
        if ( tm_16bit_bits_not_set(rtePtr->rteFlags, TM_RTE_REJECT) )
        {
            if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_CLONE) )
/* If entry is for a LAN network destination, clone for specific host */
            {
                rtePtr = tfRtClone(rtePtr, destIpAddr);
            }
        }
        else
        {
            rtePtr = TM_RTE_NULL_PTR;
        }
    }
    return rtePtr;
}

/*
 * tfRtResetCaches() Function Description
 * Called by the router code with TM_RTE_INDIRECT, or TM_RTE_CLONED
 * when all the socket and IP send and forward routing cache
 * entries need to be reset, because a new route has been added
 * (TM_RTE_INDIRECT flag), or because there is no more room in the ARP
 * cache (TM_RTE_CLONED).
 * Also called by the user ARP cache flush routine.
 *
 * Parameter       Description
 * rteFlag         TM_RTE_INDIRECT, or TM_RTE_ARP
 *
 * No Return
 */
static void tfRtResetCaches (  tt16Bit           rteFlag, 
                               int               addressFamilyFlag
#ifdef TM_LOCK_NEEDED
                             , tt8Bit            lockFlag
#endif /* !TM_LOCK_NEEDED */
                            )
{
    tfIpResetCaches( rteFlag, addressFamilyFlag
#ifdef TM_LOCK_NEEDED
                   , lockFlag
#endif /* !TM_LOCK_NEEDED */
                    );
    tfSocketResetCaches( rteFlag, addressFamilyFlag
#ifdef TM_LOCK_NEEDED
                       , lockFlag
#endif /* !TM_LOCK_NEEDED */
                    );
}

/*
 * tfRtCacheUnGet() Function Description
 * Called to release routing entry acquired in the cache,
 * and also arp entry (if any) concurrently acquired in the cache.
 *
 * Parameters
 * Parameter       Description
 * rtcPtr         Pointer to the cache entry
 *
 * No Return
 */

void tfRtCacheUnGet (ttRtCacheEntryPtr rtcPtr)
{
    tfRtCacheReset(rtcPtr, (tt16Bit)0xFFFF);
}

/*
 * tfRtCacheReset() Function Description
 * Called by the socket interface when the user closes the socket
 * to release the socket routing cache entry with rteFlag == 0xFFFF, or
 * by tfRtResetCaches() to reset an individual cache
 * pointing to an ARP/MCAST/BCAST entry (rteFlag == TM_RTE_CLONED),
 * or pointing to an indirect route (rteFlag == TM_RTE_INDIRECT).
 * We release the routing cache entry, if it corresponds to the type
 * specified in the rteFlag parameter. In that case, we also release the ARP
 * cache entry if it had been concurrently acquired (if routing entry was
 * indirect.)
 *
 * Parameters
 * Parameter       Description
 * rtcPtr         Pointer to the cache entry
 * rteFlag        either 0xFFFF (any type), or TM_RTE_CLONED
 *                or TM_RTE_INDIRECT (indirect route).
 *
 * No Return
 */

void tfRtCacheReset (ttRtCacheEntryPtr rtcPtr, tt16Bit rteFlag)
{
    ttRteEntryPtr rtePtr;

    if (rtcPtr->rtcRtePtr != TM_RTE_NULL_PTR)
    {
        rtePtr = rtcPtr->rtcRtePtr;
        if (tm_16bit_one_bit_set(rtePtr->rteFlags, rteFlag))
        {
            tfRtUnGet(rtcPtr->rtcRtePtr);
            rtcPtr->rtcRtePtr = TM_RTE_NULL_PTR;
            if (rtcPtr->rtcRteClonedPtr != TM_RTE_NULL_PTR)
/* Corresponding ARP entry */
            {
                tfRtUnGet(rtcPtr->rtcRteClonedPtr);
                rtcPtr->rtcRteClonedPtr = TM_RTE_NULL_PTR;
            }
        }
    }
}


/*
 * copy oldNode to newNode, reset old node's in flag
 *
 * oldNodePtr value arg, newNodePtr value arg, tempPtr temp var
 */
static void tfRtCopyNode ( ttRtRadixNodePtr oldNodePtr,
                           ttRtRadixNodePtr newNodePtr )
{
    ttRtRadixNodePtr tempPtr;

/* copy old node to new node */
    tm_bcopy(oldNodePtr, newNodePtr, sizeof(ttRtRadixNode));

/*
 * If there is a network list attached to old node, make it point to
 * new node
 */
    tempPtr = tm_rt_next_net(oldNodePtr);
    if (tempPtr != TM_RADIX_NULL_PTR)
    {
        tm_rt_prev_net(tempPtr) = newNodePtr;
    }
/* Make the old node's children point to the new node */
    tm_rt_parent(tm_rt_left_child(oldNodePtr)) = newNodePtr;
    tm_rt_parent(tm_rt_right_child(oldNodePtr)) = newNodePtr;
    tempPtr = tm_rt_parent(oldNodePtr); /* parent */
/* Make the old node's parent  point to new node */
    tm_rt_set_new_child(tempPtr, oldNodePtr, newNodePtr);
    oldNodePtr->rtnFlags &= ~TM_RTN_IN; /* no longer in the tree */
}

/*
 * Insert network entry (leafPtr) in sorted list of network entries whose
 * head is nodePtr. On return nodePtr will be leafPtr previous, and nextPtr
 * will be leafPtr next.
 *
 * nodePtr value arg/temp var, nextPtr temp var, leafPtr, maskLen value args
 */
static void tfRtInsertNet ( ttRtRadixNodePtr nodePtr,
                            ttRtRadixNodePtr leafPtr,
                            int              maskLen )
{
    ttRtRadixNodePtr nextPtr;
#ifdef TM_USE_STRONG_ESL
    ttRteEntryPtr    rtePtr;
    ttDeviceEntryPtr devPtr;
    tt32Bit          hops;
#endif /* TM_USE_STRONG_ESL */

#ifdef TM_USE_STRONG_ESL
    rtePtr = tm_rt_leaf_to_rte(leafPtr);
    hops = rtePtr->rteHops;
    devPtr = rtePtr->rteOrigDevPtr;
#endif /* TM_USE_STRONG_ESL */
    nextPtr = tm_rt_next_net(nodePtr);
    while (nextPtr != TM_RADIX_NULL_PTR)
    {
/* Insert leaf with with the longest masks first */
        if (maskLen > (int)(tm_rt_mask_len(nextPtr)))
        {
            tm_rt_prev_net(nextPtr) = leafPtr;
            break;
        }
#ifdef TM_USE_STRONG_ESL
        else
        {
            if (maskLen == (int)(tm_rt_mask_len(nextPtr))) 
/*
 * For same mask sort in order of increasing hops.
 * For same mask, and same hops, sort in order of increasing devIndex
 */
            {
                 rtePtr = tm_rt_leaf_to_rte(nextPtr);
                 if (    (hops < rtePtr->rteHops)
                      || (    (hops == rtePtr->rteHops)
                           && (devPtr->devIndex
                                   < rtePtr->rteOrigDevPtr->devIndex ) ) )
                 {
                    tm_rt_prev_net(nextPtr) = leafPtr;
                    break;
                 }
            }
        }
#endif /* TM_USE_STRONG_ESL */
/* need to move nodePtr around since list is not cicular */
        nodePtr = nextPtr;
        nextPtr = tm_rt_next_net(nextPtr);
    }
    tm_rt_next_net(leafPtr) = nextPtr;
    tm_rt_prev_net(leafPtr) = nodePtr;
    tm_rt_next_net(nodePtr) = leafPtr;
}

#ifdef TM_LITTLE_ENDIAN

tt16Bit tfRtBitOffset( tt4IpAddress    key,
                       tt4IpAddress    ipAddr,
                       tt4IpAddressPtr bitMaskPtr )
{ 
auto    tt4IpAddress    bittest;
        tt8BitPtr       ucp;
        tt8BitPtr       ucp2;
        tt8Bit          uc;
        tt16Bit         bits;
        tt16Bit         bytes;

    bittest = tm_ip_xor(key, ipAddr); /* keep only the bits that differ */
    ucp = (tt8BitPtr)&bittest;
    bytes = (tt16Bit)0;
    while ((*ucp == TM_8BIT_ZERO) && (bytes < sizeof(tt4IpAddress)))
    { 
        ucp++;
        bytes++;
    } 
    if (bytes != sizeof(tt4IpAddress))
    {
        bits = TM_RT_BITS_PER_BYTE; /* assume all bits match */
        uc = *ucp;
/* decrease by the number of non matching bits in the failing byte */
        while (uc != TM_8BIT_ZERO)
        {
            bits--;
            uc >>= 1;
        }
    }
    else
    {
        bits = (tt16Bit)0;
    }
    if (bitMaskPtr != (tt4IpAddressPtr)0)
    {
        tm_ip_copy(TM_IP_ZERO, *bitMaskPtr);
        if (bytes != sizeof(tt4IpAddress))
        {
            ucp2 = (tt8BitPtr)bitMaskPtr + bytes;
            *ucp2 = (tt8Bit)( (TM_RTN_HIGH_BIT >> bits) & 0xFF );
        }
    }
    bits = (tt16Bit)(bits + tm_bytes_to_bits(bytes));
    return bits;
}

#endif /* TM_LITTLE_ENDIAN */

#ifdef TM_PMTU_DISC
/*
 * tfRtPickNextPlateauMtu function description
 * Find next Path MTU to try.
 * Called with flag parameter set to TM_PMTU_LOWER, if we just have received
 * an ICMP error message with a message size error because we had set the
 * DF bit in the original message, and the IP datagram was too big, and the
 * router sending the ICMP message did not set the next hop MTU field.
 * We need to pick as the next PMTU estimate the greatest plateau value
 * that is less than the current MTU.
 * Called with the flag parameter set to TM_PMTU_HIGHER, if we have
 * successfully sent IP datagrams without occuring fragmentation for
 * the timeout period, and we want to try the next higher Path MTU
 * estimate at the end of the timeout period.
 * Parameters
 * Parameter       Description
 * currentMtu      Current IP Path MTU estimate used
 * flag            either TM_PMTU_LOWER or TM_PMTU_HIGHER
 * Return value
 * Next IP MTU to try
 */
static tt16Bit
tfRtPickNextPlateauMtu(tt16Bit currentMtu, tt8Bit flag)
{
    int     plateauIndex;

    plateauIndex = 0;
/*
 * Find the index to the greatest plateau value that is less or equal
 * than the current MTU.
 */
    while (plateauIndex < TM_PMTU_ENTRIES)
    {
        if (currentMtu >= tlMtuPlateauValues[plateauIndex])
        {
            break;
        }
        plateauIndex++;
    }
/* Pick next plateau MTU to try (lower or higher) */
    if (flag == TM_PMTU_LOWER)
/*
 * Pick as the next PMTU estimate the greatest plateau value that is less
 * than the current MTU.
 */
    {
/*
 * If in the while loop above, we did not find a lower MTU plateau value
 * (plateauIndex == TM_PMTU_ENTRIES), do nothing. We will just return
 * the null entry at the end of the table.
 */
        if (plateauIndex < TM_PMTU_ENTRIES)
/*
 * If in the while loop above, we found the entry that is less or equal than
 * the current MTU.
 */
        {
            if (currentMtu == tlMtuPlateauValues[plateauIndex])
/*
 * If the entry is the same as the current MTU, pick the
 * next value down by increasing  the index. No need to check
 * for table overflow since plateauIndex is less than TM_PMTU_ENTRIES
 * and the table goes from 0 to TM_PMTU_ENTRIES.
 */
            {
                plateauIndex++;
            }
/* 
 * If it is less, do nothing, we will just return that lesser value at the
 * current index.
 */
        }
    }
    else
/* Pick next higher MTU */
    {
        if (plateauIndex != 0)
        {
            plateauIndex--;
        }
        else
        {
/*
 * If we are currently at the biggest possible MTU. There is no new MTU
 * to try. Just return 0, so that we know we do not need try a new value.
 * (Table index goes from 0, to TM_PMTU_ENTRIES, and value at index
 *  TM_PMTU_ENTRIES is 0.)
 */
            plateauIndex = TM_PMTU_ENTRIES;
        }
    }
    return tlMtuPlateauValues[plateauIndex];
}
#endif /* TM_PMTU_DISC */

#if (defined(TM_PMTU_DISC) || defined(TM_6_PMTU_DISC))
/*
 * tfRtAddPathMtuEntry function description
 * Add a new host path MTU routing entry.
 * . If routing entry is host routing entry, delete it.
 * . Add new host routing with timeout value, and flags parameters
 * . Get new routing entry, and check that it is a host entry
 * . If nextMtu is non zero, set the routing entry MTU with it.
 * Parameters
 * Parameter    Description
 * rtePtr       Pointer to current routing entry to destination IP address.
 * timeout      Timeout value of new routing entry to insert.
 * flags        Flags value of new routing entry to insert.
 * nextMtu   New Path MTU to use
 *
 * Return values 
 * TM_ENOERROR      New host path MTU routing entry was added
 * TM_EHOSTUNREACH  Destination IP address unreachable
 * TM_ENOBUFS       Not enough memory to allocate new routing entry
 * TM_ENOENT        Could not insert host entry
 */
static int tfRtAddPathMtuEntry( 
#ifdef TM_USE_STRONG_ESL
                                ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                                ttRteEntryPtr    rtePtr,
                                ttIpAddress      destIpAddress,
                                tt32Bit          timeout,
                                tt16Bit          flags,
                                tt16Bit          nextMtu )
{
    ttDeviceEntryPtr    rteDevPtr;
    ttIpAddress         gwayAddress;
    int                 errorCode;
    tt16Bit             rteMhomeIndex;

    rteDevPtr = rtePtr->rteDevPtr;
    rteMhomeIndex = rtePtr->rteMhomeIndex;
    tm_ip_copy(rtePtr->rteGSGateway, gwayAddress);
    if ( tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                TM_RTE2_HOST | TM_RTE2_UP) )
/*
 * If the current entry is a host entry, remove it.
 */
    {
        tfRtRemoveEntry(rtePtr);
    }
/* Entry added with device MTU */
    errorCode = tf4RtAddRoute( rteDevPtr,
                               destIpAddress,
                               TM_IP_HOST_MASK,
                               gwayAddress,
                               TM_16BIT_ZERO, /* tag */
                               rteMhomeIndex, 1,
                               timeout,
                               flags,
/* no need to lock */
                               TM_8BIT_ZERO);
    if (errorCode == TM_ENOERROR)
    {
        rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                          devPtr,
#endif /* TM_USE_STRONG_ESL */
                          destIpAddress);
        errorCode = TM_EPERM;
        if (rtePtr != TM_RTE_NULL_PTR)
        {
            if ( tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                        TM_RTE2_HOST | TM_RTE2_UP ) )
            {
                if (nextMtu != 0)
/*
 * If User did not set the MTU, use default MTU (device MTU), otherwise
 * set the MTU to the parameter value.
 */
                {
                    rtePtr->rteMtu = nextMtu;
                }
                errorCode = TM_ENOERROR;
            }
        }
    }
    return errorCode;
}
#endif /* TM_PMTU_DISC || TM_6_PMTU_DISC */


/*
 * tfIpResetCaches function description:
 * Called by the router code with TM_RTE_INDIRECT, or TM_RTE_CLONED rteFlag, 
 * when the IP send and IP forward routing cache entries need to be reset, 
 * because a new route has been added (TM_RTE_INDIRECT), or because there is 
 * no more room in the ARP cache (TM_RTE_CLONED).
 * lockFlag can have two possible values TM_LOC_WAIT or TM_LOC_NOWAIT.
 * TM_LOC_WAIT
 *   If lockFlag is TM_LOCK_WAIT, then this function is called with the 
 *   routing tree unlocked, and not in the send/forward path. It is safe
 *   to lock the IP routing caches.
 * TM_LOC_NOWAIT
 *   . If lockFlag is TM_LOC_NOWAIT, then this function is called with the
 *     routing tree locked. If the routing tree is locked it is unsafe to 
 *     lock an IP forward or IP route cache. (example: send task locking
 *     an IP routing cache, and locking the tree; app task adding a route
 *     entry which calls this function with the tree locked)
 *   . If lockFlag is TM_LOC_WAIT, then this function could be called
 *     in the send/forward path with an IP routing cache already locked,
 *     resulting in a deadlock if we attempt to lock the same lock again.
 *   . To prevent deadlocks, if lockFlag is set to TM_LOC_NOWAIT,
 *     we will not attempt to reset any IP routing cache that is already 
 *     locked. 
 */
void tfIpResetCaches (  tt16Bit           rteFlag
                      , int               addressFamilyFlag
#ifdef TM_LOCK_NEEDED
                      , tt8Bit            lockFlag
#endif /* TM_LOCK_NEEDED */
                      )
{
    ttRtCacheEntryPtr rtcPtr;
    tt8Bit            cacheEntries;
#ifdef TM_LOCK_NEEDED
    ttLockEntryPtr    lockEntryPtr;
#endif /* TM_LOCK_NEEDED */

    if (addressFamilyFlag & AF_INET)
    {
/* First reset IPv4 send cache */
#ifdef TM_LOCK_NEEDED
        lockEntryPtr = &tm_context(tvRtIpSendCacheLock);
#endif /* TM_LOCK_NEEDED */
        rtcPtr = &tm_context(tvRtIpSendCache);
        cacheEntries = TM_8BIT_ZERO;
        do
        {
            if (rtcPtr->rtcRtePtr != TM_RTE_NULL_PTR)
            {
#ifdef TM_LOCK_NEEDED
                if (   (tvLockEnabled == TM_8BIT_ZERO)
                    || (tfLock(lockEntryPtr, lockFlag) == TM_LOC_OKAY) )
#endif /* TM_LOCK_NEEDED */
                {
                    tfRtCacheReset(rtcPtr, rteFlag);
                    tm_call_unlock(lockEntryPtr);
                }
            }
/* Second reset IP forward cache */
            rtcPtr = &tm_context(tvRtIpForwCache);
#ifdef TM_LOCK_NEEDED
            lockEntryPtr = &tm_context(tvRtIpForwCacheLock);
#endif /* TM_LOCK_NEEDED */
            cacheEntries++;
        } while (cacheEntries == (tt8Bit)1);
    }


    return;
}

/* 
 * Called by the tf{4,6}RtAddxxx() functions. 
 * Lock the tree if needed an allocate a routing entry, Initializing the
 * owner count to 1. 
 */
static ttRteEntryPtr tfRtAllocNLockNCopy(tt8Bit          needLock)
{
    ttRteEntryPtr       rtePtr;

#ifdef TM_ERROR_CHECKING
/* Check that routing tree has been initialized, and is not corrupted */
    if (tm_context(tvRtTreeHeadPtr) == TM_RADIX_NULL_PTR)
    {
        tfKernelError("tfRtAllocNLockNCopy",
                      "Routing table corrupted or not initialized");
        tm_thread_stop;
    }
#endif /* TM_ERROR_CHECKING */
/* Allocate a new routing entry */
    rtePtr = (ttRteEntryPtr)tm_recycle_alloc( (unsigned)sizeof(ttRteEntry),
                                              TM_RECY_RTE );

    if (rtePtr != TM_RTE_NULL_PTR)
    {
/* Brand new entry. Initialize owner count */
        rtePtr->rteOwnerCount = 1; /* no need to be critical here */
#ifdef TM_SNMP_CACHE
/* Initialize SNMPD/NSTAT cache index */
        rtePtr->rteSnmpCacheEntry.sceTableIndex = TM_SNMPC_NO_INDEX;
#endif /* TM_SNMP_CACHE */
/* Copy destination IP address */
/* LOCK the tree to obtain exclusive access to the routing table */
        if (needLock)
        {
            tm_call_lock_wait(&tm_context(tvRtTreeLock));
        }
    }
    return rtePtr;
}

/*
 * Called by the tf{4,6}RtAddxxx() functions. 
 * Insert entry in SNMP routing cache, if insertion to Patricia tree succeeded.
 * UNLOCK the tree to release access to the routing table, if needed.
 * Free routing entry, if insertion failed. 
 * Ignore error for local route if errorCode is TM_EALREADY, since local
 * routes is for the same interface.
 */
static int tfRtCacheUnlockNClean ( ttRteEntryPtr rtePtr,
                                   tt8Bit needUnLock, int errorCode,
                                   tt16Bit flags )
{
    tt8BitPtr        destLinkPhysAddrPtr;
    ttIpAddress      ipAddress;
    int              retCode;
    
    retCode = errorCode;
#ifdef TM_SNMP_CACHE
    if (errorCode == TM_ENOERROR)
    {
        tfSnmpdCacheInsertRoute(rtePtr, TM_SNMPC_ROUTE_INDEX);
    }
#endif /* TM_SNMP_CACHE */

/*
 * special case for local route entry, to handle multiple addresses configured
 * on the same interface with the same prefix (subnet):  The original
 * local route entry has already been added, but we need to add a loopback host
 * route entry for this new address, since the normal clone logic (tfRtClone)
 * cannot handle this case.  This entry will be removed when the address is
 * unconfigured.
 */
    if (    tm_16bit_one_bit_set(flags, TM_RTE_LOCAL)
         && (errorCode == TM_EALREADY) )
    {
        tm_ip_copy(rtePtr->rteHSDevIpAdd, ipAddress);
        retCode = TM_ENOERROR;
        errorCode = tfRtAddEntry( tm_context(tvRtTreeHeadPtr),
                                  rtePtr,
                                  rtePtr->rteOrigDevPtr,
                                  ipAddress,
                                  TM_IP_HOST_MASK,
                                  rtePtr->rteOrigMhomeIndex,
                                  (int) rtePtr->rteHops,
                                  TM_RTE_INF,
                                  TM_RTE_STATIC | TM_RTE_LINK_LAYER);
        if (    (errorCode == TM_ENOERROR)
             && tm_16bit_one_bit_set(flags, TM_RTE_CLONE) )
        {
/* 
 * So that we do get a NO ARP entry kernel error in tfIpSendPacket(). Also
 * done after tfRtAddEntry() so that we do not add an ARP timer.
 */
            rtePtr->rteFlags = (tt16Bit)(rtePtr->rteFlags | TM_RTE_CLONED);
            destLinkPhysAddrPtr = &(rtePtr->rteHSPhysAddr[0]);
/* Sending to our own IP address. Store link layer address. */
            tm_bcopy(rtePtr->rteOrigDevPtr->devPhysAddrArr,
                     destLinkPhysAddrPtr,
                     rtePtr->rteHSPhysAddrLen);
        }
#if ( !(defined(TM_SINGLE_INTERFACE_HOME)) && !(defined(TM_LOOP_TO_DRIVER)) )
        rtePtr->rteDevPtr = tm_context(tvLoopbackDevPtr);
#endif /* !defined(TM_SINGLE_INTERFACE_HOME) && !defined(TM_LOOP_TO_DRIVER) */
#ifdef TM_SNMP_CACHE
        if (errorCode == TM_ENOERROR)
        {
            tfSnmpdCacheInsertRoute(rtePtr, TM_SNMPC_ROUTE_INDEX);
        }
#endif /* TM_SNMP_CACHE */
    }
    
    if (needUnLock)
    {
        tfRtTreeUnlockNResetCaches();
    }
    if (errorCode != TM_ENOERROR)
    {
/* routing entry could not be inserted. Recycle it */
#ifdef TM_ERROR_CHECKING
/* Keep track of out of tree non recycled entries */
        tm_context(tvRteLimboCount)++;
#endif /* TM_ERROR_CHECKING */
        tfRtUnGet(rtePtr);
    }
    return retCode;
}

/* 
 * Add an ARP entry. Common code between IPv4 and IPv6.
 * IPv4: called from tf4RtArpAddEntry which is itself called when a user
 * adds an ARP entry, or in ARP incoming processing.
 * IPv6: Called from tf6RtArpAddEntry which is itself called when a user
 * adds an ARP entry.
 */
static int tfRtArpAddEntry ( 
    ttRtCacheEntryPtr  rtcCachePtr,
    tt8BitPtr          physAddrPtr,
    tt32Bit            ttl,
    tt8Bit             physAddrLen 
                )
{
    ttRteEntryPtr   rtePtr;
    int             errorCode;

    errorCode = tfRtGet(rtcCachePtr);
    if (errorCode == TM_ENOERROR)
    {
        rtePtr = rtcCachePtr->rtcRtePtr;
        errorCode = tfRtArpAddByRte(rtePtr, physAddrPtr, ttl, physAddrLen);
        if (errorCode == TM_ENOERROR)
        {
            if (ttl == TM_RTE_INF)
/* Permanent ARP entry */
            {
                rtePtr->rteFlags |= TM_RTE_STATIC;
            }
        }
/*
 * Release ownership, since we are not keeping the cache around
 */
        tfRtCacheUnGet(rtcCachePtr);
    }
    return errorCode;
}

/*
 * Delete a route. Called by tf6RtDelRoute() or tf4RtDelRoute()
 */
static int tfRtDelRoute(ttRteEntryPtr rtePtr, tt16Bit rteCreateFlag)
{
    ttRteEntryPtr       prevRtePtr;
    int                 errorCode;

    errorCode = TM_EPERM;
    prevRtePtr = TM_RTE_NULL_PTR;
    do
    {
        if (    tm_16bit_bits_not_set( rtePtr->rteFlags, TM_RTE_ARP )
             && tm_16bit_one_bit_set(rtePtr->rteFlags, rteCreateFlag) )
/*
 * If entry is not ARP entry, and deletor is identical to creator,
 * delete the entry.
 */
        {
            errorCode = TM_ENOERROR;
            if (prevRtePtr != TM_RTE_NULL_PTR)
            {
                if (tm_16bit_one_bit_set(prevRtePtr->rteFlags, TM_RTE_ARP))
/* Previous entry at same key is ARP clone, remove it */
                {
                    tfRtRemoveEntry(prevRtePtr);
                }
            }
/* Delete entry */
            tfRtRemoveEntry(rtePtr);
            break;
        }
        else
        {
            prevRtePtr = rtePtr;
            rtePtr = tm_rt_leaf_to_rte(
                                tm_rt_next_dup(tm_rt_rte_to_leaf(prevRtePtr)));
        }
    } while (rtePtr != (ttRteEntryPtr)0);
    return errorCode;
}

/* 
 * IPv6 functions
 */

/* 
 * IPv4 only functions
 */

/*
 * tfRtDestExists Function Description:
 * Called by the user to find out if there exists a route to a given
 * destination (given by destination IP address/destination netmask).
 * Parameters
 * Parameter        Description
 * ipAddress        Destination IP address
 * netMask          Destination netmask
 * Returns
 * 0                Route to destination IP address found
 * TM_EHOSTUNREACH  no Route to the destination IP address.
 */
int tfRtDestExists(
#ifdef TM_USE_STRONG_ESL
    ttUserInterface interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
    ttUserIpAddress ipAddress,
    ttUserIpAddress netMask )
{
    ttRteEntryPtr   rtePtr;
    int             errorCode;

#ifdef TM_USE_STRONG_ESL
    if (    (interfaceHandle != (ttUserInterface)0)
         && (tfValidInterface(interfaceHandle) != TM_ENOERROR))
    {
        errorCode = TM_EINVAL;
    }
    else
#endif /* TM_USE_STRONG_ESL */
    {
        errorCode = TM_EHOSTUNREACH;
/* Destination network */
        tm_ip_copy(tm_ip_net(ipAddress, netMask), ipAddress);
/* LOCK the tree to obtain exclusive access to the routing table */
        tm_call_lock_wait(&tm_context(tvRtTreeLock));
        rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                          interfaceHandle,
#endif /* TM_USE_STRONG_ESL */
                          ipAddress);
        if (rtePtr != TM_RTE_NULL_PTR)
        {
            errorCode = TM_ENOERROR;
        }
/* UNLOCK the tree to release access to the routing table */
        tfRtTreeUnlock();
    }
    return errorCode;
}

/*
 * Function called from the socket interface to delete a static route from the
 * routing table and by tfRemoveInterface() to delete a local route from the
 * routing table.
 * Parameters:
 * DestIpAddress: Destination IP address (key) of the route
 * myNetMask:     IP network mask of the route
 * rteCreateFlag: Deletor ID, one of TM_RTE_STATIC, TM_RTE_LOCAL
 * Return values:
 *   0 on success
 *   TM_ENOENT if routing enty cannot be found
 *
 */
int tf4RtDelRoute ( 
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress     destIpAddress,
    tt4IpAddress     destNetMask,
    tt16Bit          rteCreateFlag )
{
    ttRteEntryPtr      rtePtr;
    int                errorCode;

    errorCode = TM_ENOERROR;
#ifdef TM_ERROR_CHECKING
    if (tm_context(tvRtTreeHeadPtr) == TM_RADIX_NULL_PTR)
    {
        tfKernelError("tf4RtDelRoute",
                      "Routing table corrupted or not initialized");
/* Severe Error */
        tm_thread_stop;
    }
#endif /* TM_ERROR_CHECKING */
/* LOCK the tree to obtain exclusive access to the routing table */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
/*
 * Check whether this IP address/network mask is in the routing table.
 */
    rtePtr = tfRtFindIpAddrIpNet(
#ifdef TM_USE_STRONG_ESL
                                 devPtr,
#endif /* TM_USE_STRONG_ESL */
                                 destIpAddress, destNetMask);
    if (rtePtr != TM_RTE_NULL_PTR)
/* If entry already exists in the routing table */
    {
/* 
 * Note that if TM_USE_IPV6 is defined then IPV4 tfDelDefaultGateway calls
 * tf4RtDelDefGw, and does not call tf4RtDelRoute 
 */
        if (    tm_ip_zero(destNetMask) /* called by tfDelDefaultGateway */
/* user specified gateway (non zero) */
             && tm_ip_not_zero(destIpAddress)
/* user specified gateway, different from current route gateway */
             && tm_ip_not_match(destIpAddress, rtePtr->rteGSGateway) )
        {
            errorCode = TM_ENOENT;
        }
        else
        {
            errorCode = tfRtDelRoute(rtePtr, rteCreateFlag);
        }
    }
    else
    {
        errorCode = TM_ENOENT;
    }
/* UNLOCK the tree to release access to the routing table */
    tfRtTreeUnlockNResetCaches();
    return errorCode;
}

/*
 * Function called to add a local route in the routing table.
 * Parameters:
 * devPtr: pointer to device entry for the local route
 * Destination IP address: IP address of the network or of the remote
 * side for Point to point network.
 * myNetMask: IP network mask to be used on this device
 * myIpAddress: IP address to be used on this device
 * myMhomeIndex: Multihome Index on this device for IP address
 * Return values:
 * 0 on success
 * TM_EINVAL for parameter problems.
 */
int tf4RtAddLocal ( ttDeviceEntryPtr devPtr,
                    tt4IpAddress     destIpAddress,
                    tt4IpAddress     myNetMask,
                    tt4IpAddress     myIpAddress,
                    tt16Bit          myMhomeIndex )
{
    int                 errorCode;
    tt16Bit             flags;

    if (    tm_ip_is_loop_back(myIpAddress)
         && (devPtr != tm_context(tvLoopbackDevPtr)) )
    {
        errorCode = TM_EADDRNOTAVAIL;
    }
    else
    {
/* Allow cloning for local LAN link layer address. */
        flags = TM_RTE_LOCAL; /* default case */
        if (tm_4_ll_is_broadcast(devPtr))
        {
          if (tm_ip_match(myNetMask, TM_IP_HOST_MASK))
          {
                if (tm_4_ll_is_lan(devPtr))
                {
                    flags = TM_RTE_LOCAL | TM_RTE_CLONED | TM_RTE_LINK_LAYER;
                }
                /* else default TM_RTE_LOCAL */
          }
          else
          {
                flags = TM_RTE_CLONE | TM_RTE_LOCAL;
          }
        } /* else default TM_RTE_LOCAL */
        errorCode = tf4RtAddRoute( devPtr,
                                   destIpAddress,
                                   myNetMask,
                                   myIpAddress,
                                   TM_16BIT_ZERO,
                                   myMhomeIndex, 0, TM_RTE_INF,
                                   flags, TM_8BIT_YES );
    }
    return errorCode;
}

/*
 * Called by tfAddStaticRoute(), tf4RtAddLocal, tf4RtRedirect, tfRtRip,
 * BOOTP/DHCP, MOBILIP, ICMP, SNMP to add a new routing entry
 * to the routing table (Patricia tree).
 * 1) Call tfRtAllocNLockNCopy to allocate a new routing entry, lock
 *    the Patricia tree, and for IPV6 only copy the IPv4-mapped IPV6 
 *    address in the routing entry destination field.
 * 2) Copy gateway address in routing entry
 * 3) call tfRtAddEntry() to insert the entry in the Patricia tree.
 * 4) Call tfRtCacheUnlockNClean to cache the entry in the SNMP routing
 *    cache, unlock the tree, and unget the routing entry if we failed
 *    to insert in the tree.
 * Parameters
 * devPtr: device for the route 
 * ipAddr: Destination Ip address 
 * netMask: destination network address 
 * mHomeIndex: multihome index for local routes
 * gateway: Gateway IP address for non local routes, interface IP address
 *          for local routes.
 * tag: Used by tfRtRip.
 * hops: number of hops to destination (0 for local route)
 * ttl: Time to live for the route entry
 * flags: flag to indicate creator local/static/redirect/RIP, host route etc.
 *
 * Return value:
 * O on success
 * TM_ENOBUFS, when routing entry could not be allocated
 */
int tf4RtAddRoute (
    ttDeviceEntryPtr        devPtr, /* device for the route */
/* Destination Ip address, key */
    tt4IpAddress            ipAddress,
/* destination network address */
    tt4IpAddress            netMask,
/* Gateway for non local routes, interface IP address for local routes, etc. */
    tt4IpAddress            gateway,
    tt16Bit                 gwayTag,
/* multihome index for local routes */
    tt16Bit                 mhomeIndex,
    int                     hops, /* metrics for non local routes */
    tt32Bit                 ttl,
/* flag to indicate creator local/static/redirect/RIP */
    tt16Bit                 flags,
    tt8Bit                  needLock )
{
    ttRteEntryPtr       rtePtr;
    int                 errorCode;

    rtePtr = tfRtAllocNLockNCopy(needLock);
    if (rtePtr != TM_RTE_NULL_PTR)
    {
        rtePtr->rteHSPhysAddrLen = devPtr->devPhysAddrLength;
        if (tm_16bit_one_bit_set(flags, TM_RTE_LOCAL))
        {
            if (tm_16bit_all_bits_set(flags, TM_RTE_CLONED | TM_RTE_LINK_LAYER))
            {
/* Sending to our own IP address. Store link layer address. */
                tm_bcopy(devPtr->devPhysAddrArr,
                         &(rtePtr->rteHSPhysAddr[0]),
                         devPtr->devPhysAddrLength);
            }
            else
            {
                tm_ip_copy(gateway, rtePtr->rteHSDevIpAdd);
            }
        }
        else
        {
            tm_ip_copy(gateway, rtePtr->rteGSGateway);
        }
        rtePtr->rteGSTag = gwayTag;
        rtePtr->rteMtu = devPtr->devMtu;
        errorCode = tfRtAddEntry(
                            tm_context(tvRtTreeHeadPtr), rtePtr, devPtr,
                            ipAddress, netMask,
                            mhomeIndex,
                            hops, ttl, flags );
/*
 * Unlock the tree, cache entry if successful or free routing entry otherwise.
 * Ignore error for local route if errorCode is TM_EALREADY.
 */
        errorCode = tfRtCacheUnlockNClean(rtePtr, needLock, errorCode, flags);
    }
    else
    {
        errorCode =  TM_ENOBUFS;
    }
    return errorCode;
}

/*
 * IPV4 functions only
 */
/*
 * tfRtArpAddEntry function description:
 * Add/Set an ARP Entry to routing table for a given destination IP address.
 * Called by the ARP interface, or User interface to set a physical
 * address mapping
 * Parameters: ipAddress LAN Ip address to map
 *             physAddrPtr pointer to link layer address
 * return values:
 * 0                success
 * TM_EINVAL        bad parameter
 * TM_EHOSTUNREACH  No network for the ip address parameter
 */
int tf4RtArpAddEntry ( 
#ifdef TM_USE_STRONG_ESL
    ttDeviceEntryPtr  devPtr,
#endif /* TM_USE_STRONG_ESL */
    tt4IpAddress      ipAddress,
    tt8BitPtr         physAddrPtr,
    tt32Bit           ttl,
    tt8Bit            physAddrLen 
    )
{
auto    ttRtCacheEntry  rtcCache;

    tm_bzero(&rtcCache, sizeof(ttRtCacheEntry));
#ifdef TM_USE_STRONG_ESL
    rtcCache.rtcDevPtr = devPtr;
#endif /* TM_USE_STRONG_ESL */
    tm_ip_copy(ipAddress, rtcCache.rtcDestIpAddr);
    return tfRtArpAddEntry(&rtcCache, physAddrPtr, ttl, physAddrLen); 
}

/*
 * Function called from the socket interface to get the IPv4 default gateway
 * from the routing table
 * Parameters:
 * gatewayIpAddrPtr: pointer to gateway IP address to store result into.
 * Return values:
 *   0 on success
 *   TM_ENOENT if routing enty cannot be found
 *
 */
int tf4RtGetDefault (
#ifdef TM_USE_STRONG_ESL
                     ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                     tt4IpAddressPtr  gatewayIpAddrPtr,
                     ttIntPtr         hopsPtr
                    )
{
    ttRteEntryPtr      rtePtr;
    int                errorCode;

    errorCode = TM_ENOENT;
/* LOCK the tree to obtain exclusive access to the routing table */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
/*
 * Check whether this IP address/network mask is in the routing table.
 */
    rtePtr = tfRtFindIpAddrIpNet(
#ifdef TM_USE_STRONG_ESL
                                  devPtr,
#endif /* TM_USE_STRONG_ESL */
                                  TM_IP_ZERO, TM_IP_ZERO);
    if (rtePtr != TM_RTE_NULL_PTR)
/* If entry already exists in the routing table */
    {
        tm_ip_copy(rtePtr->rteGSGateway, *gatewayIpAddrPtr);
        if (hopsPtr != (ttIntPtr)0)
        {
            *hopsPtr = (int)rtePtr->rteHops;
        }
        errorCode = TM_ENOERROR;
    }
/* UNLOCK the tree to release access to the routing table */
    tfRtTreeUnlock();
    return errorCode;
}

/*
 * tfRtEchoReply Function Description:
 * Called by ICMP when an echo reply has been received 
 * If ipAddress is in routing table and entry is routing discovery,
 * RIP or Redirect, increase time to live
 */
void tfRtEchoReply (
#ifdef TM_USE_STRONG_ESL
                    ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                    tt4IpAddress     ipAddress)
{
   ttRteEntryPtr   rtePtr;

/* LOCK the tree to obtain exclusive access to the routing table */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
    rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                      devPtr,
#endif /* TM_USE_STRONG_ESL */
                      ipAddress);
    if (rtePtr != TM_RTE_NULL_PTR)
    {
        if ( tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_ROUTE_DYNAMIC) )
/* RIP, Router Discovery, or REDIRECT gateway entry */
        {
            rtePtr->rteTtl = tm_context(tvRtTimeout); /* 3 minutes */
        }
    }
/* UNLOCK the tree to release access to the routing table */
    tfRtTreeUnlock();
}

#ifdef TM_PMTU_DISC
/*
 * tfRtNextHopMtu Function Description.
 * 1. Called by ICMP when a UNREACH for fragment arrives (iphPtr non null)
 * 2. Or called by the user tfDisablePathMtuDisc() when the user
 *    wants to disable PATH MTU discovery on a host route (iphPtr null).
 * Algorithm.
 * Lock the tree
 * Find route entry in routing table.
 * If routing entry is found:
 * . If function called by the user, set timeout value to infinity,
 *   and set new routing entry flags to static, and to disable
 *   path MTU.
 * . If function is called from ICMP, set timeout to default path MTU
 *   discovery timeout (10 minutes), and set new routing entry flags to
 *   dynamic path MTU discovery. If routing entry is a host entry,
 *   return an error if path MTU is not allowed (by setting errorCode).
 *   ErrorCode is checked in all next statements. Next check the next Hop MTU
 *   found in the ICMP header. If none was set (router
 *   which does not implement RFC1191), then find the next plateau MTU as
 *   described in RFC 1191. If we were already at the lowest possible MTU,
 *   set the flags to disable path MTU. If the next hop MTU is bigger than
 *   the previously cached MTU, do not allow a change in the MTU.
 *   If we are not going to change the MTU, and we were not changing the
 *   routing entry flags either, then set the errorCode to disallow
 *   the routing entry change/addition.
 * . If errorCode not set,
 *     . if route is indirect, call tfRtAddPathMtuEntry() to delete the
 *       current routing entry if it is a host entry, and to add new host
 *       routing with timeout value, flags, and MTU as found earlier.
 *     . else if route is an ARP entry, just update the MTU/flags,
 *       and make sure we reset the caches.
 * . UnLock the tree
 * . Return the errorCode.
 * Parameters
 * Parameter    Description
 * ipAddress    destination IP address of the path MTU discovery
 * nextHopMtu   New Path MTU to try
 * iphPtr       Pointer to IP header when called from ICMP fragment
 *              error.
 *
 * Return values 
 * TM_ENOERROR      New host path MTU routing entry was added
 * TM_EHOSTUNREACH  Destination IP address unreachable
 * TM_ENOBUFS       Not enough memory to allocate new routing entry
 * TM_ENOENT        Could not insert host entry
 * TM_EPERM         Path MTU discovery not allowed.
 */
int tfRtNextHopMtu (
#ifdef TM_USE_STRONG_ESL
                     ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                     tt4IpAddress   ipAddress,
                     tt16Bit        nextHopMtu,
                     ttIpHeaderPtr  iphPtr )
{
    ttRteEntryPtr    rtePtr;
    tt32Bit          timeout;
    int              errorCode;
    tt16Bit          oldIpHdrLength;
    tt16Bit          flags;
    tt16Bit          indirectFlag;
    tt16Bit          oldMtu;
    tt16Bit          rteMtu;

    errorCode = TM_ENOERROR;
/* LOCK the tree to obtain exclusive access to the routing table */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
    rtePtr = tfRtFindNClone(
#ifdef TM_USE_STRONG_ESL
                      devPtr,
#endif /* TM_USE_STRONG_ESL */
                      ipAddress);
    if (rtePtr != TM_RTE_NULL_PTR)
    {
/* Extract indirect flag */
        indirectFlag = (tt16Bit)(rtePtr->rteFlags & TM_RTE_INDIRECT);
        {
            rteMtu = rtePtr->rteMtu;
            if (iphPtr == (ttIpHeaderPtr)0)
            {
/*
 * Called from the user, Make the host entry static. Disable Path MTU
 * discovery
 */
                timeout = TM_RTE_INF;
                flags = (tt16Bit)(  indirectFlag | TM_RTE_STATIC
                                  | TM_RTE_NO_PMTU_DISC);
/* Note: MTU will Default to device MTU if not set by the user */
            }
            else
            {
/*
 * Called from ICMP. Make the host entry a dynamic path MTU discovery entry.
 * Do not disable Path MTU discovery
 */
                flags = (tt16Bit)(indirectFlag | TM_RTE_PMTU_DISC);
/* 10 minutes */
                timeout = tm_context(tvRtDecreasedMtuTimeout);
                if (    tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                               TM_RTE2_HOST | TM_RTE2_UP) )
/*
 * If there was a host entry to the remote, we cannot change the routing
 * entry if path MTU discovery was not allowed.
 */
                {
                    if ( tm_16bit_one_bit_set( rtePtr->rteFlags,
                                               TM_RTE_NO_PMTU_DISC) )
                    {
/* Disallow modification of the routing table */
                        errorCode = TM_EPERM;
                    }
                }
                if (errorCode == TM_ENOERROR)
/* Change allowed so far */
                {
                    if (nextHopMtu == 0)
/*
 * Router not implementing RFC 1191 (not setting the next hop MTU field).
 * We need to estimate the next lower Path MTU to try.
 */
                    {
/*
 * From RFC 1191:
 * The ICMP Destination Unreachable message, contains the IP header
 * of the original datagram, which contains the Total length of the
 * datagram that was too big to be forwarded without fragmentation.
 * Since this Total Length may be less than the current
 * PMTU estimate, but is nonetheless larger than the actual PMTU, it may
 * be a good input to the method for choosing the next PMTU estimate.
 */
                        tm_ntohs(iphPtr->iphTotalLength, oldMtu);
#ifdef TM_LINT
LINT_UNINIT_SYM_BEGIN(oldMtu)
#endif /* TM_LINT */
                        if (oldMtu > rteMtu)
#ifdef TM_LINT
LINT_UNINIT_SYM_END(oldMtu)
#endif /* TM_LINT */
                        {
/*
 * RFC 1191:
 * Note: routers based on implementations derived from 4.2BSD
 * Unix send an incorrect value for the Total Length of the
 * original IP datagram.  The value sent by these routers is the
 * sum of the original Total Length and the original Header
 * Length (expressed in octets).  Since it is impossible for the
 * host receiving such a Datagram Too Big message to know if it
 * sent by one of these routers, the host must be conservative
 * and assume that it is.  If the Total Length field returned is
 * not less than the current PMTU estimate, it must be reduced by
 * 4 times the value of the returned Header Length field.
 */
                            oldIpHdrLength =
                                            tm_ip_hdr_len(iphPtr->iphVersLen);
                            oldIpHdrLength = (tt16Bit)(4 * oldIpHdrLength);
                            if (oldMtu > oldIpHdrLength)
                            {
                                oldMtu = (tt16Bit)(oldMtu - oldIpHdrLength);
                            }
                            else
                            {
/*
 * Information from the ICMP error message cannot be trusted. Use the
 * cache value.
 */
                                oldMtu = rteMtu;
                            }
                        }
                        nextHopMtu = tfRtPickNextPlateauMtu( oldMtu,
                                                             TM_PMTU_LOWER );
                    }
                    if (nextHopMtu < TM_PMTU_MINIMUM)
/* Our minimum PMTU is set at 296 bytes */
                    {
                        nextHopMtu = TM_PMTU_MINIMUM;
/*
 * Do not try to increase the Path MTU at the route timeout.
 * Packets are fragmented even with our smallest possible MTU.
 * When the route timeout, we will switch to the non host route, and
 * retry the path MTU discovery starting at the device IP MTU.
 */
                        flags = (tt16Bit)(  indirectFlag | TM_RTE_PMTU_DISC
                                          | TM_RTE_NO_PMTU_DISC);
                    }
                    if (nextHopMtu >= rteMtu)
/*
 * RFC 1191
 * Do not increase MTU beyond our previously cached entry,
 * as a result of a don't fragment error
 */
                    {
/* No MTU change */
                        nextHopMtu = rteMtu;
                        if ( tm_16bit_bits_not_set(flags, TM_RTE_NO_PMTU_DISC) )
/*
 * If we are not changing the flags either, then there is no need to
 * add/replace the routing entry.
 */
                        {
                            errorCode = TM_EPERM;
                        }
                    }
                }
            }
            if (errorCode == TM_ENOERROR)
/*
 * No error if command was from the user, or there was no host entry with
 * PATH MTU disabled.
 */
            {
                if (indirectFlag)
                {
                    errorCode = tfRtAddPathMtuEntry(
#ifdef TM_USE_STRONG_ESL
                                                devPtr,
#endif /* TM_USE_STRONG_ESL */
                                                rtePtr,
                                                ipAddress,
                                                timeout,
                                                flags, nextHopMtu);
                }
                else if (tm_16bit_one_bit_set(rtePtr->rteFlags,TM_RTE_CLONED))
                {
/*
 * Cloned (ARP) entry is found:
 *     Update the Path MTU, and flags. Reset the caches.
 */
/* Do not make the ARP entry static, if the user disables PMTU */
                    flags &= ~TM_RTE_STATIC;
                    tm_kernel_set_critical;
                    rtePtr->rteHSPmtuTimeout = timeout; /* PMTU timeout */
                    rtePtr->rteMtu = nextHopMtu;
                    rtePtr->rteFlags |= flags;
                    tm_kernel_release_critical;
                    tm_context(tvRtRadixTreeHead).rthResetCachesFlags 
                                                |= TM_RTE_CLONED;
                    tm_context(tvRtRadixTreeHead).rthResetCachesAf 
                                                |= (AF_INET);
                }
                else
                {
                    errorCode = TM_EPERM;
                }
            }
        }
    }
    else
    {
/* No routing entry found for destination IP address */
        errorCode = TM_EHOSTUNREACH;
    }
/* UNLOCK the tree to release access to the routing table */
    tfRtTreeUnlockNResetCaches();
    return errorCode;
}
#endif /* TM_PMTU_DISC */

/*
 * tf4RtRedirect() Function description
 * Called by ICMP when a redirect has been received 
 * 1. Check that 'gateway' is on the same connected (sub-net) through
 *    which the redirect arrived (netMatch on 'devPtr'/'mhomeIndex'),
 *    and that it is not our own IP address.
 *    (otherwise we silently ignore the redirect) (RFC 1122)
 * 2. Lock the tree, and get current routing entry for 'destIpAddr'.
 * 3. Check that 'srcIpAddr' is valid source for the Redirect (it should be
 *    the current first-hop gateway for 'destIpAddr', information that we
 *    get from the current routing entry)) ( otherwise we silently ignore
 *    the redirect) (RFC 1122)
 * 4. Delete current routing entry only if it is a host route (only
 *    delete host route, i.e DO NOT DELETE NETWORK ROUTE!)
 *    (RFC 1122).
 * 5. Add new entry with new 'gateway' using the host netmask
 *    (all ones). (Also set parameter so that tf4RtAddRoute does not 
 *    lock/unlock the tree.)
 * 6. Unlock the tree.
 *
 * Parameters:
 * devPtr      Interface through which the redirect came
 * mhomeIndex  Interface multi home index through which the redirect came
 * destIpAddr  Original destination IP address that caused the redirect
 * gateway     New gateway to use for the redirect
 * srcIpAddr   source of the redirect.
 *
 * No return value
 */
void tf4RtRedirect ( ttDeviceEntryPtr devPtr,
                     tt16Bit          mhomeIndex,
                     tt4IpAddress     destIpAddress,
                     tt4IpAddress     gateway,
                     tt4IpAddress     srcIpAddr )
{
    tt4IpAddress        intfIpAddr;
    tt4IpAddress        intfNetMask;
    ttRteEntryPtr       rtePtr;
#ifndef TM_SINGLE_INTERFACE_HOME
    ttDeviceEntryPtr    anyDevPtr;
    tt16Bit             anyMhomeIndex;
#endif /* TM_SINGLE_INTERFACE_HOME */
/*
 * 1. Check that 'gateway' is on the same connected (sub-net) through
 *    which the redirect arrived (netMatch on 'devPtr'/'mhomeIndex'),
 *    and that it is not our own IP address.
 *    (otherwise we silently ignore the redirect) (RFC 1122)
 */
    tm_ip_copy(tm_ip_dev_addr(devPtr, mhomeIndex), intfIpAddr);
    tm_ip_copy(tm_ip_dev_mask(devPtr, mhomeIndex), intfNetMask);
    if (    tm_ip_net_match(gateway, intfIpAddr, intfNetMask)
         && !tm_ip_match(gateway, intfIpAddr) )
    {
/* 2. LOCK the tree, and get current routing entry for 'destIpAddress'. */
        tm_call_lock_wait(&tm_context(tvRtTreeLock));
        rtePtr = tfRtFind(
#ifdef TM_USE_STRONG_ESL
                        devPtr,
#endif /* TM_USE_STRONG_ESL */
                        destIpAddress);
        if (rtePtr != TM_RTE_NULL_PTR)
        {
/*
 * 3. Check that 'srcIpAddr' is valid source for the Redirect (it should be
 *    the current first-hop gateway for 'destIpAddress', information that we
 *    get from the current routing entry)) ( otherwise we silently ignore
 *    the redirect) (RFC 1122)
 */
            if (    tm_16bit_one_bit_set(rtePtr->rteFlags, TM_RTE_INDIRECT)
                 && tm_ip_match(rtePtr->rteGSGateway, srcIpAddr)
#ifndef TM_SINGLE_INTERFACE_HOME
                 && (rtePtr->rteOrigMhomeIndex == mhomeIndex)
#endif /* TM_SINGLE_INTERFACE_HOME */
                 && (rtePtr->rteDevPtr == devPtr) )
            {
#ifndef TM_SINGLE_INTERFACE_HOME
/* Check that gateway is not one of our own IP address on another interface */
#ifdef TM_4_USE_SCOPE_ID
                if (IN4_IS_ADDR_LINKLOCAL(gateway))
                {
                    anyDevPtr = (ttDeviceEntryPtr)0;
                }
                else
#endif /* TM_4_USE_SCOPE_ID */
                {
                    anyDevPtr = tfIfaceMatch( gateway,
                                              tfMhomeAddrMatch,
                                              &anyMhomeIndex );
                }
                if ( anyDevPtr == (ttDeviceEntryPtr)0 )
#endif /* TM_SINGLE_INTERFACE_HOME */
                {
                    if ( tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                                TM_RTE2_HOST | TM_RTE2_UP ) )
/*
 * 4. Delete current routing entry only if it is a host route (only
 *    delete host route, i.e DO NOT DELETE NETWORK ROUTE!)
 *    (RFC 1122). Note that we delete, and do not modify the existing routing
 *    entry, since individual routing entries are never locked.
 */
                    {
                        tfRtRemoveEntry(rtePtr);
                    }
/*
 * 5. Add new entry with new 'gateway' using the host netmask
 *    (all ones). Set last paramter to zero, so that tf4RtAddRoute() 
 *    does not lock/unlock the tree.
 */
                    (void)tf4RtAddRoute(devPtr, destIpAddress,
                                        TM_IP_HOST_MASK, gateway,
                                        TM_16BIT_ZERO, mhomeIndex, 1,
                                        tm_context(tvRtTimeout),
                                        TM_RTE_INDIRECT|TM_RTE_REDIRECT,
/* no need to lock */
                                        TM_8BIT_ZERO);
                }
            } /* source IP address is valid for redirect */
        } /* Destination in routing table */
/* UNLOCK the tree to release access to the routing table */
        tfRtTreeUnlockNResetCaches();
    } /* netMatch of gateway */
}

/*
 * tfRtRip description
 * Called by tfRipRecvCB when an incoming RIP response arrives.
 * 1. LOCK the tree to obtain exclusive access to the routing table
 * 2. Check wheter we have already a route for this destination/mask
 * 3. Do not overwrite a route created by local user.
 * 4. If the existing route has same gateway, update the TTL, metrics,
 *    and if metrics is Infinity, remove entry from the routing tree.
 * 5. If the existing route does not have the same gateway, replace the
 *    route only if metrics is lower. If metrics is the same, replace only
 *    if timeout of the current route is at least halfway to the expiration
 *    point.
 * To remove, add, or replace an entry, the local variables needRemoveEntry,
 * and needAddEntry are used; replace is achieved by setting both
 * needRemoveEntry, and needAddEntry.
 * If needRemoveEntry is set, the current routing entry is removed.
 * If needAddEntry is set, the new route is added. 
 *
 * Parameters      description
 * devPtr:         device for the route
 * mHomeIndex:     multihome index for the route
 * destIpAddr:     destination Ip address
 * destNetMask:    destination network address
 * nextHop:        gateway IP address for the route
 * metrics:        number of hops to destination
 * routeTag:       Rip route tag
 *
 * Return value:
 * no return
 */
void tfRtRip ( ttDeviceEntryPtr devPtr, tt16Bit mhomeIndex,
               tt4IpAddress destIpAddress, tt4IpAddress destNetMask,
               tt4IpAddress nextHop, int metrics, tt16Bit routeTag )
{
    ttRteEntryPtr       rtePtr;
    tt16Bit             creator;
    tt8Bit              needAddEntry;
    tt8Bit              needRemoveEntry;

/* LOCK the tree to obtain exclusive access to the routing table */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
/* Do we have an entry for this destination/mask */
    rtePtr = tfRtFindIpAddrIpNet(
#ifdef TM_USE_STRONG_ESL
                       devPtr,
#endif /* TM_USE_STRONG_ESL */
                       destIpAddress, destNetMask);
    needAddEntry = TM_8BIT_ZERO;
    needRemoveEntry = TM_8BIT_ZERO;
    if (rtePtr != TM_RTE_NULL_PTR)
/* If route already exists */
    {
/* Get route creator */
        creator = (tt16Bit)(rtePtr->rteFlags & TM_RTE_CREATE_MASK);
        if (creator >= TM_RTE_RIP)
/* If route creator is not local or static */
        {
/*
 * RFC 1058:
 * If there is an existing route, first compare gateways.
 */
            if ( tm_ip_match(nextHop, rtePtr->rteGSGateway) )
            {
/*
 * RFC 1058:
 * If this datagram is from the same gateway as the existing route,
 * reinitialize the timeout.
 */
                rtePtr->rteTtl = tm_context(tvRtTimeout);
/*
 * RFC 1058:
 * same gateway and different metrics: update timeout. update metrics
 */
                rtePtr->rteHops = (tt32Bit)metrics;
/*
 * RFC 1058:
 * If the new metric is 16 (infinity), this starts the process for
 * deleting the route.  The route is no longer used for routing packets.
 * REVNOTE: we just remove the routing entry, we will not start a deletion
 * process since we are just a listener.
 */
                if ( metrics == TM_RIP_INF_METRIC )
                {
                    needRemoveEntry = TM_8BIT_YES;
                }
            }
            else
            {
/*
 * RFC1058:
 * Route is from a different gateway. Only replace the route if metrics
 * is lower
 */
                if ( (tt32Bit)metrics < rtePtr->rteHops )
                {
                    needRemoveEntry = TM_8BIT_YES;
                    needAddEntry = TM_8BIT_YES;
                }
/* RFC 1058:
 * If the new metric is the same as the old one, it is simplest to do
 * nothing further (beyond reinitializing the timeout, as specified
 * above).  However, the 4BSD routed uses an additional heuristic here.
 * Normally, it is senseless to change to a route with the same metric
 * as the existing route but a different gateway.  If the existing route
 * is showing signs of timing out, though, it may be better to switch to
 * an equally-good alternative route immediately, rather than waiting
 * for the timeout to happen.  (See section 3.3 for a discussion of
 * timeouts.)  Therefore, if the new metric is the same as the old one,
 * routed looks at the timeout for the existing route.  If it is at
 * least halfway to the expiration point, routed switches to the new
 * route.  That is, the gateway is changed to the source of the current
 * message.  This heuristic is optional.
 */
                else
                {
                    if ( (tt32Bit)metrics == rtePtr->rteHops )
/* If same metrics */
                    {
                        if ( rtePtr->rteTtl < (tm_context(tvRtTimeout)/2) )
/* If the timeout is at least halfway to the expiration point */
                        {
                            needRemoveEntry = TM_8BIT_YES;
                            needAddEntry = TM_8BIT_YES;
                        }
                    }
                }
            }
        } /* if (creator >= TM_RTE_RIP) */
    }
    else
    {
        if ( metrics != TM_RIP_INF_METRIC )
        {
            needAddEntry = TM_8BIT_YES;
        }
    }
    if ( needRemoveEntry != TM_8BIT_ZERO )
    {
        tfRtRemoveEntry(rtePtr);
    }
    if ( needAddEntry != TM_8BIT_ZERO )
    {
        (void)tf4RtAddRoute(devPtr, destIpAddress, destNetMask,
                            nextHop, routeTag, mhomeIndex, metrics,
                            tm_context(tvRtTimeout),
                            TM_RTE_INDIRECT|TM_RTE_RIP, TM_8BIT_ZERO);
    }
    tfRtTreeUnlockNResetCaches();
}

/*
 * tf4RtModifyDefGatewayMhome function description
 * Update the multihome index for the default gateway routing entry.
 *
 * Parameters
 * Parameter    Description
 * mhomeIndex   new multihome index for the default gateway
 *
 * Return values 
 * TM_ENOERROR  Multihome index successfully updated
 * TM_ENOENT    Could not modify multihome index (entry not found)
 */
int tf4RtModifyDefGatewayMhome(
#ifdef TM_USE_STRONG_ESL
                                ttDeviceEntryPtr devPtr,
#endif /* TM_USE_STRONG_ESL */
                                tt16Bit mhomeIndex )
{
    ttRteEntryPtr      rtePtr;
    int                errorCode;

    errorCode = TM_ENOENT;

/* 
 * Lock the routing table since tfRtFindIpAddrIpNet requires the tree to be
 * locked, and so we can modify the default gateway entry.
 */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));

/* Find routing table entry for the default gateway */
    rtePtr = tfRtFindIpAddrIpNet(
#ifdef TM_USE_STRONG_ESL
                                 devPtr,
#endif /* TM_USE_STRONG_ESL */
                                 TM_IP_ZERO, TM_IP_ZERO);

/* If default GW exists in the routing table, update it's multihome index */
    if (rtePtr != TM_RTE_NULL_PTR)
    {
        rtePtr->rteMhomeIndex = mhomeIndex;
        rtePtr->rteOrigMhomeIndex = mhomeIndex;
        errorCode = TM_ENOERROR;
    }
    tfRtTreeUnlock();
    return errorCode;
}

/*
 * tfRtAddHost function description:
 * Called by tfAddHostRoute()
 * to add a host entry to the routing table (Patricia tree) to set
 * a particular destination interface for a given multicast address.
 * 1) Call tfRtAllocNLockNCopy to allocate a new routing entry, lock
 *    the Patricia tree, and for IPV6 only copy the IPv4-mapped IPV6 
 *    address in the routing entry destination field.
 * 2) Copy physical address in routing entry
 * 3) call tfRtAddEntry() to insert the entry in the Patricia tree.
 * 4) Call tfRtCacheUnlockNClean to cache the entry in the SNMP routing
 *    cache, unlock the tree, and unget the routing entry if we failed
 *    to insert in the tree.
 * Parameters
 * devPtr: device for the route 
 * ipAddr: Destination Ip address 
 * physaddrPtr: pointer to physical address
 * physAddrLength: length of the physical address
 * mHomeIndex: multihome index 
 * ttl: Time to live for the route entry
 * flags: flag to indicate creator local/static/redirect/RIP, host route etc.
 *
 * Return value:
 * O on success
 * TM_ENOBUFS, when routing entry could not be allocated
 */
int tf4RtAddHost(
/* route device */
                  ttDeviceEntryPtr       devPtr,
/* Destination Ip address */
                  tt4IpAddress           ipAddress,
/* physical address */
                  tt8BitPtr              physAddrPtr,
/* physical address len */
                  tt8Bit                 physAddrLen,
/* multihome index */
                  tt16Bit                mhomeIndex,
                  tt32Bit                ttl,
/* flag to indicate cloned/static/link-layer/mcast */
                  tt16Bit                flags )
{
    ttRteEntryPtr       rtePtr;
    int                 errorCode;

    rtePtr = tfRtAllocNLockNCopy(TM_8BIT_YES);
    if (rtePtr != TM_RTE_NULL_PTR)
    {
        tm_bcopy(physAddrPtr, rtePtr->rteHSPhysAddr ,(unsigned)physAddrLen);
        rtePtr->rteHSPhysAddrLen = physAddrLen;
        rtePtr->rteMtu = devPtr->devMtu;
        errorCode = tfRtAddEntry(
                            tm_context(tvRtTreeHeadPtr), rtePtr, devPtr,
                            ipAddress, TM_IP_HOST_MASK,
                            mhomeIndex,
                            0, ttl, flags );
/*
 * Unlock the tree, cache entry if successful or free routing entry otherwise.
 */
        errorCode = 
                tfRtCacheUnlockNClean(rtePtr, TM_8BIT_YES, errorCode, flags);
    }
    else
    {
        errorCode =  TM_ENOBUFS;
    }
    return errorCode;
}




#ifdef TM_RT_OUTPUT

void tfRtTreePrint (void)
{
    (void)tfRtTreeWalk(tfRtPrintEntryCB, TM_VOID_NULL_PTR, TM_8BIT_YES);
}


static int tfRtPrintEntryCB (ttRteEntryPtr rtePtr, ttVoidPtr voidPtr)
{
    ttRtRadixNodePtr    leafPtr;
#ifdef TM_RT_INT_OUTPUT
    ttRtRadixNodePtr    parentPtr;
    ttRtRadixNodePtr    intNodePtr;
    ttRtRadixNodePtr    otherPtr;
    int                 validIntNode;
#endif /* TM_RT_INT_OUTPUT */

    TM_UNREF_IN_ARG(voidPtr); /* compiler */
    leafPtr = tm_rt_rte_to_leaf(rtePtr);
    printf(
        "RTE FLAGS 0x%x 0x%x gway 0x%lx TTL 0x%lx Owner %d clonedPtr 0x%x\n",
        (unsigned)rtePtr->rteFlags,
        (unsigned)rtePtr->rteFlags2,
        ntohl(rtePtr->rteGSGateway),
        rtePtr->rteTtl,
        (int)rtePtr->rteOwnerCount,
        rtePtr->rteClonedPtr);
    tfRtPrintLeaf("LEAF", leafPtr);
#ifdef TM_RT_INT_OUTPUT
    parentPtr = tm_rt_parent(leafPtr);
    if (tm_16bit_one_bit_set(parentPtr->rtnFlags, TM_RTN_NODE))
    {
        tfRtPrintNode("PARENT NODE", parentPtr);
        otherPtr = tm_rt_left_child(parentPtr);
        if (otherPtr != leafPtr)
        {
            if (tm_16bit_one_bit_set(otherPtr->rtnFlags, TM_RTN_NODE))
            {
                tfRtPrintNode("PARENT LEFT CHILD NODE", otherPtr);
            }
            else
            {
                tfRtPrintLeaf("PARENT LEFT CHILD LEAF", otherPtr);
            }
            printf("LEAF IS RIGHT CHILD OF PARENT\n");
        }
        else
        {
            printf("LEAF IS LEFT CHILD OF PARENT\n");
            otherPtr = tm_rt_right_child(parentPtr);
            if (tm_16bit_one_bit_set(otherPtr->rtnFlags, TM_RTN_NODE))
            {
                tfRtPrintNode("PARENT RIGHT CHILD NODE", otherPtr);
            }
            else
            {
                tfRtPrintLeaf("PARENT RIGHT CHILD LEAF", otherPtr);
            }
        }
    }
    else
    {
        tfRtPrintLeaf("LEAF DUP PREV", parentPtr);
        otherPtr = tm_rt_next_dup(leafPtr);
        if (otherPtr != TM_RADIX_NULL_PTR)
        {
            tfRtPrintLeaf("LEAF DUP NEXT", otherPtr);
        }
    }
    intNodePtr = tm_rt_rte_to_int_node(rtePtr);
    validIntNode = tm_16bit_one_bit_set(intNodePtr->rtnFlags, TM_RTN_IN);
    if (validIntNode)
    {
        tfRtPrintNode("INTNODE", intNodePtr);
        otherPtr = tm_rt_parent(intNodePtr);
        tfRtPrintNode("INTNODE PARENT", otherPtr);
        otherPtr = tm_rt_left_child(intNodePtr);
        if (tm_16bit_one_bit_set(otherPtr->rtnFlags, TM_RTN_NODE))
        {
            tfRtPrintNode("INTNODE LEFT CHILD NODE", otherPtr);
        }
        else
        {
            tfRtPrintLeaf("INTNODE LEFT CHILD LEAF", otherPtr);
        }
        otherPtr = tm_rt_right_child(intNodePtr);
        if (tm_16bit_one_bit_set(otherPtr->rtnFlags, TM_RTN_NODE))
        {
            tfRtPrintNode("INTNODE RIGHT CHILD NODE", otherPtr);
        }
        else
        {
            tfRtPrintLeaf("INTNODE RIGHT CHILD LEAF", otherPtr);
        }
        otherPtr = tm_rt_next_net(intNodePtr);
        while (otherPtr != TM_RADIX_NULL_PTR)
        {
            tfRtPrintLeaf("INTNODE NETWORK LIST LEAF", otherPtr);
            otherPtr = tm_rt_next_net(otherPtr);
        }
    }
#endif /* TM_RT_INT_OUTPUT */
    printf("**************************************************\n");
    return TM_ENOENT; /* ask for another entry */
}

static void tfRtPrintLeaf (ttCharPtr msg, ttRtRadixNodePtr leafPtr)
{
    printf( "%s: 0x%x", msg, leafPtr);
    printf( " flags 0x%x destIPAddr 0x%lx destMask 0x%lx\n",
             (unsigned)leafPtr->rtnFlags, ntohl(leafPtr->rtnLSKey),
             ntohl(leafPtr->rtnLSMask) );
}

#ifdef TM_RT_INT_OUTPUT
static void tfRtPrintNode (ttCharPtr msg, ttRtRadixNodePtr nodePtr)
{
    printf( "%s: 0x%x", msg, nodePtr);
    printf( " flags 0x%x bitOffset %d bitMask 0x%lx\n",
            (unsigned)nodePtr->rtnFlags,
            (int)tm_rt_matching_bits(nodePtr),
            ntohl(nodePtr->rtnNSBitMask) );
}
#endif /* TM_RT_INT_OUTPUT */

#endif /* TM_RT_OUTPUT */

/* Neighbor Unreachability Detection stuff */
/****************************************************************************
* FUNCTION: tfNudSetReachable
*
* PURPOSE: 
*   Calling this function indicates that a connection is making forward
*   progress.
*   ([RFC2461].R7.3.1:10, [RFC2461].R7.3.1:20, [RFC2461].R7.3.1:30,
*   [RFC2461].R7.3.1:40, [RFC2461].R7.3.3:20, [RFC2461].R7.3.3:30)
*
* PARAMETERS:
*   rteEntryPtr
*       Pointer to the routing entry.
*
* RETURNS:
*   Nothing
*
* NOTES:
*   This function needs to be moved to trroute.c
*
****************************************************************************/
void tfNudSetReachable(
    ttRteEntryPtr rteEntryPtr )
{


/* If the routing entry is not in the INCOMPLETE state (this is always the
   case, we don't store INCOMPLETE entries in the routing tree),
   [RFC2461].R7.3.3:20, then: */

/* Store the current time as the most recent time that this neighbor was
   confirmed reachable:
   ([RFC2461].R7.3.3:30) */
    /* done in tf6RtArpUpdateByRte */
    {
        tm_kernel_set_critical;
        rteEntryPtr->rteTtl = tm_context(tvArpTimeout);
        tm_kernel_release_critical;      
    }

    return;
}




/*
 * tfRtRemoveHostEntry function description
 * Function called with the tree locked.
 * Remove an IPv4/v6 host entry from the Patricia tree
 * 1. Check the owner count
 * 1.1 If the owner count is bigger than the ownerCount we expect 
 * 1.1.1 Try and reset all caches pointing to this entry
 * 1.1.2 If owner count is still bigger than the ownerCount we expect
 *       try and remove all gateways pointing to this entry.
 * 1.2 Remove entry from the tree.
 *
 * Parameters
 * rtePtr     Pointer to host routing entry
 * af         AF_INET (IPv4), or AF_INET6 (IPv6)
 * ownerCount Expected owner count of the entry.
 */
static void tfRtRemoveHostEntry (ttRteEntryPtr rtePtr,
                                unsigned int  ownerCount,
                                tt16Bit       af)
{
    if (rtePtr->rteOwnerCount > ownerCount)
/* 1.1 If the owner count is bigger than the ownerCount we expect  */
    {
/* 1.1.1 Try and reset all caches pointing to this entry */
        tm_rt_reset_caches(TM_RTE_CLONED, af, TM_LOC_NOWAIT);
        if (rtePtr->rteOwnerCount > ownerCount)
/* 
 * 1.1.2 If owner count is still bigger than the ownerCount check for
 *       gateways/local routes pointing to the entry.
 */
        {
            tfRtTreeWalk(tfRtRemoveGwToClonedCB, rtePtr, TM_8BIT_ZERO);
        }
    }
    if (tm_16bit_bits_not_set(rtePtr->rteFlags2, TM_RTE2_INVALID))
    {
/* 
 * 1.2 Remove entry from the tree. Note that it is safe to remove this
 * entry from the tree, even if it is being resolved, because we have
 * removed all entries pointing to this entry via the rteClonedPtr.
 */
/* make sure we are still in the tree before we try to remove from the tree */
        if ( tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                    TM_RTE2_HOST | TM_RTE2_UP ) )
        {
            tfRtRemoveEntry(rtePtr);
        }
    }
    else
    {
/* 
 * 1.2 There is a static route/default router pointing to this ARP entry. We cannot
 * remove it, and we will change the ARP mapping to stale so that it is usable.
 */
       rtePtr->rteFlags2 &= ~TM_RTE2_INVALID;
        {
            tf4RtArpDelByRte(rtePtr, 0);
        }
    }
}


/*
 * tfRtUpdateDevMtu function description
 * Update routing entries for a given device with a new MTU received in a
 * router advertisement or set by the user.  Called with the tree unlocked.
 * 
 * Parameters
 * devPtr     Pointer to the device that the RA was received on.
 * newMtu     New IPv6 MTU received in the RA.
 * af         address family flag (AF_INET, AF_INET6)
 */
void tfRtUpdateDevMtu(ttDeviceEntryPtr devPtr, tt16Bit newMtu, tt16Bit af)
{
    tt6RtDevMtuUpdate  mtuUpdate; 

    mtuUpdate.dmuDevEntryPtr = devPtr;
    mtuUpdate.dmuNewMtu = newMtu;
    mtuUpdate.dmuAf = af;
    
/* Lock the tree before walking through entries. */
    tfRtTreeWalk( tfRtUpdateDevMtuCB,
                  (ttVoidPtr) &mtuUpdate,
                  TM_8BIT_YES );

/* Reset all routing caches, since the MTU may have changed. */ 
    tm_rt_reset_caches(TM_RTE_CLONED | TM_RTE_INDIRECT, af, TM_LOC_WAIT);
}

/*
 * tfRtUpdateDevMtuCB function description
 * Routing tree walk callback function to update routing entries with
 * new MTU received from router advertisement, or set by the user.
 *
 * 1. Update matching IPv6 entries if AF_INET6 is set, IPv4 entries if AF_INET
 *    is set.
 * 2. If IPv6/IPv4 Path MTU discovery is enabled:
 *     2.1. If the user has explicitly set an MTU for this route, do not
 *          modify it.
 *     2.2. If this entry has had the MTU set be Path MTU discovery, only
 *          update the MTU if the new MTU is less than the current Path MTU.
 *          The current Path MTU may be smaller than the link MTU, but
 *          should never be larger.
 * 3. If this is not a Path MTU entry (or IPv6 Path MTU discovery and
 *    IPv4 Path MTU discovery are disabled), update the MTU.
 * 4. Upate the MTU
 * 
 * Parameters
 * rtePtr     Pointer to current routing entry.
 * newMtuPtr  Pointer to new IPv6 MTU received in the RA.
 */

static int tfRtUpdateDevMtuCB(ttRteEntryPtr rtePtr, ttVoidPtr newMtuPtr)
{
    tt6RtDevMtuUpdatePtr  mtuUpdatePtr;

    mtuUpdatePtr = (tt6RtDevMtuUpdatePtr) newMtuPtr;
/* 1. Only update matching entries. */
    if (rtePtr->rteDevPtr == mtuUpdatePtr->dmuDevEntryPtr)
    {
        {
#if ((defined(TM_USE_IPV4) && defined(TM_PMTU_DISC)) || (defined(TM_6_PMTU_DISC)))
/* 2. If Path MTU discovery is enabled: */
/*
 *     2.1. If the user has explicitly set an MTU for this route, do not
 *          modify it.
 *     2.2. If this entry has had the MTU set be Path MTU discovery, only
 *          update the MTU if the new MTU is less than the current Path MTU.
 *          The current Path MTU may be smaller than the link MTU, but
 *          should never be larger.
 */
            if (    tm_16bit_bits_not_set(rtePtr->rteFlags,
                                          TM_RTE_NO_PMTU_DISC)
                 && (    tm_16bit_bits_not_set (rtePtr->rteFlags,
                                                TM_RTE_PMTU_DISC)
                      || (mtuUpdatePtr->dmuNewMtu < rtePtr->rteMtu)))
#endif /* (TM_USE_IPV4 && TM_PMTU_DISC) || TM_6_PMTU_DISC */
            {
/*            
 *   Just update the MTU directly in the existing routing entry. 
 *   We will invalidate all routing caches upon return.
 */            
                tm_kernel_set_critical;
                rtePtr->rteMtu = mtuUpdatePtr->dmuNewMtu;
                tm_kernel_release_critical;
            }
        }
    }
/*  Return TM_ENOENT to ask for the next entry. */
    return TM_ENOENT;
}







/*
 * Common function to unlock the tree, when entries have been added/deleted.
 * Unlock the tree, and call tfRtResetCaches() if needed with the tree
 * unlocked.
 */
static void tfRtTreeUnlockNResetCaches (void)
{
    int               resetCachesAf;
    tt16Bit           resetCachesFlags;

    resetCachesFlags = tm_context(tvRtRadixTreeHead).rthResetCachesFlags;
/* Initialize resetCachesAf for compiler warning */
    resetCachesAf = (int)tm_context(tvRtRadixTreeHead).rthResetCachesAf;
    if (resetCachesFlags != TM_16BIT_ZERO)
    {
        tm_context(tvRtRadixTreeHead).rthResetCachesFlags = TM_16BIT_ZERO;
        tm_context(tvRtRadixTreeHead).rthResetCachesAf = TM_16BIT_ZERO;
    }
    tfRtTreeUnlock();
/* Call tfRtResetCaches() with tree lock unlocked */
    if (resetCachesFlags != TM_16BIT_ZERO)
    {
        tm_rt_reset_caches(resetCachesFlags, resetCachesAf, TM_LOC_WAIT);
    }
}

/*
 * Unlock the routing tree.
 */
static void tfRtTreeUnlock (void)
{
    tm_unlock(&tm_context(tvRtTreeLock));
}

/*
 * Remove routing entries queued to the tree remove list (while we were in a
 * tree walk) from the routing tree.
 */
static void tfRtTreeRem (void)
{
    ttRteEntryPtr rtePtr;

    rtePtr = tm_context(tvRtRadixTreeHead).rthRteRemListPtr;
    while (rtePtr !=(ttRteEntryPtr)0)
    {
/* Save Next on the list */
/* Dequeue top of the list */
        tm_context(tvRtRadixTreeHead).rthRteRemListPtr = rtePtr->rteNextRemPtr;
        rtePtr->rteFlags2 &= ~TM_RTE2_IN_REM_LIST;
        rtePtr->rteNextRemPtr = (ttRteEntryPtr)0;
/* Process removal of current routing entry */
        tfRtRemoveEntry(rtePtr);
/*
 * Next. We have to refresh from the top of the list, in case of recursive
 * tfRtTreeWalk()
 */
        rtePtr = tm_context(tvRtRadixTreeHead).rthRteRemListPtr;
    }
}

#ifdef TM_USE_STRONG_ESL
#endif /* TM_USE_STRONG_ESL */

#ifdef TM_USE_ARP_FLUSH
static int tf4RtArpNodeDeleteCB (
    ttNodePtr       nodePtr,
    ttGenericUnion  genParam)
{
    TM_UNREF_IN_ARG(genParam);
    tf4RtArpDelByRte((ttRteEntryPtr)
        (((ttSnmpCacheEntryPtr)(ttVoidPtr)nodePtr)->sceTableEntryPtr), 0);
    return 0;
}
/*
 * Flush the ARP table entries corresponding to the passed address family(ies)
 */
void tfRtArpFlush ( int af
#ifdef TM_LOCK_NEEDED
                  , tt8Bit lockFlag
#endif /* TM_LOCK_NEEDED */
                  )
{
#ifdef TM_SNMP_CACHE
    ttSnmpdCacheHeadPtr snmpdCacheHeadPtr;
    ttListCBFuncPtr     listCBFuncPtr;
    ttGenericUnion      genParam;
    int                 cacheIndex;
#endif /* TM_SNMP_CACHE */

/* Reset all caches pointing to ARP entries */
    tm_rt_reset_caches(TM_RTE_CLONED, af, lockFlag);
/* lock the tree */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
#ifdef TM_LOCK_NEEDED
/* Reset all caches pointing to ARP entries after having acquired the lock */
    tm_rt_reset_caches(TM_RTE_CLONED, af, TM_LOC_NOWAIT);
#endif /* TM_LOCK_NEEDED */
#ifndef TM_SNMP_CACHE
/* walk the whole tree */
    (void)tfRtTreeWalk(tfRtRemArpEntriesCB, &af, TM_8BIT_NO);
#else /* TM_SNMP_CACHE */
/* Walk the ARP list */
/*
 * Prevent removal on entries while we walk the list, otherwise we would have
 * a deadlock when the tfRtRemoveEntry() tries and locks the SNMP cache that
 * is already locked here
 */
    tm_context(tvRtRadixTreeHead).rthTreeWalkCount++;
    {
/* Assume IPv4 ARP cache */
        cacheIndex = TM_SNMPC_ARP_INDEX;
        {
            af &= ~AF_INET;
        }
/* start from the first route entry */
        snmpdCacheHeadPtr = &(tm_context(tvSnmpdCaches)[cacheIndex]);
        if (snmpdCacheHeadPtr->snmpcList.listCount != 0)
        {
            listCBFuncPtr = (ttListCBFuncPtr)0; 
            {
                listCBFuncPtr = tf4RtArpNodeDeleteCB;
            }
            if (listCBFuncPtr != (ttListCBFuncPtr)0)
            {
                genParam.genIntParm = 0;
/* List walk protected by the routing tree lock */
                tfListWalk(&(snmpdCacheHeadPtr->snmpcList),
                           listCBFuncPtr, genParam);
            }
        }
    }
/* Done with the walk */
    tm_context(tvRtRadixTreeHead).rthTreeWalkCount--;
#endif /* TM_SNMP_CACHE */
/* delete entries removed during the tree or ARP cache walk */
    tfRtTreeRem();
/* Unlock the tree */
    tfRtTreeUnlock();
    return;
}

#ifndef TM_SNMP_CACHE
/*
 * Tree walk call back function.
 */
static int tfRtRemArpEntriesCB (ttRteEntryPtr rtePtr,  ttVoidPtr parmPtr)
{
    int af;

    if (    tm_16bit_all_bits_set( rtePtr->rteFlags,
                                   (TM_RTE_CLONED | TM_RTE_ARP) )
/* ARP */
         && tm_16bit_all_bits_set( rtePtr->rteFlags2,
                                   TM_RTE2_HOST | TM_RTE2_UP )
       )
/* host entry in the tree */
    {
/* ARP entry */
        af = *((ttIntPtr)parmPtr);
        {
/* IPv4 ARP entry */
            if (af & AF_INET)
            {
                tf4RtArpDelByRte(rtePtr, 0);
            }
        }
    }
    return TM_ENOENT;
}
#endif /* !TM_SNMP_CACHE */
#endif /* TM_USE_ARP_FLUSH */

#ifdef TM_USE_STOP_TRECK
struct tsRtFlushEntry
{
    int     rtfAf;
    tt16Bit rtfFlags;
};

typedef struct tsRtFlushEntry TM_FAR * ttRtFlushEntryPtr;


/* Free all router/arp entries + route tables */
TM_PROTO_EXTERN void tfRtDeInit (
#ifdef TM_MULTIPLE_CONTEXT
                ttUserContext contextHandle
#else /* !TM_MULTIPLE_CONTEXT */
                void
#endif /* !TM_MULTIPLE_CONTEXT */
                )
{
    if (tm_context_var(tvRtArpTimerPtr) != TM_TMR_NULL_PTR)
    {
        tm_timer_remove(tm_context_var(tvRtArpTimerPtr));
        tm_context_var(tvRtArpTimerPtr) = TM_TMR_NULL_PTR;
    }
    if (tm_context_var(tvRtTimerPtr) != TM_TMR_NULL_PTR)
    {
        tm_timer_remove(tm_context_var(tvRtTimerPtr));
        tm_context_var(tvRtTimerPtr) = TM_TMR_NULL_PTR;
    }
    tfRtFlush(
                AF_INET | AF_INET6, 0xFFFF
#ifdef TM_LOCK_NEEDED
              , TM_8BIT_NO
#endif /* TM_LOCK_NEEDED */
             );
}
        
/*
 * Flush the route/ARP table entries corresponding to the passed address
 * family(ies)
 */
void tfRtFlush (
                 int           af,
                 tt16Bit       flags
#ifdef TM_LOCK_NEEDED
               , tt8Bit       lockFlag
#endif /* TM_LOCK_NEEDED */
                  )
{
    struct tsRtFlushEntry   rtFlushEntry;

/* Reset all caches pointing to entries as designated by the flags parameter */
    tm_rt_reset_caches((tt16Bit)flags, af, lockFlag);
/* lock the tree */
    tm_call_lock_wait(&tm_context(tvRtTreeLock));
#ifdef TM_LOCK_NEEDED
/* Reset all caches again after having acquired the lock */
    tm_rt_reset_caches((tt16Bit)flags, af, TM_LOC_NOWAIT);
#endif /* TM_LOCK_NEEDED */
/* walk the whole tree */
    rtFlushEntry.rtfAf = af;
    rtFlushEntry.rtfFlags = (tt16Bit)flags;
    (void)tfRtTreeWalk(tfRtRemEntriesCB, &rtFlushEntry, TM_8BIT_NO);
/* delete entries removed during the tree walk */
    tfRtTreeRem();
/* Unlock the tree */
    tfRtTreeUnlock();
    return;
}

/*
 * Tree walk call back function.
 */
static int tfRtRemEntriesCB (ttRteEntryPtr rtePtr,  ttVoidPtr parmPtr)
{
    tt16Bit flags;

    flags = ((ttRtFlushEntryPtr)parmPtr)->rtfFlags;
    if (    tm_16bit_one_bit_set(rtePtr->rteFlags, flags)
         && tm_16bit_one_bit_set(rtePtr->rteFlags2, TM_RTE2_UP) )
    {
        tfRtRemoveEntry(rtePtr);
    }
    return TM_ENOENT;
}
#endif /* TM_USE_STOP_TRECK */

/* Processed by Treck Shipping Tool v2.60 
 * 2012/02/06 10:46:52      For: Xilinx-Samsung
 * File Name: trroute.c     File Revision: 6.0.2.31 
 * Project Name: tcpip.pj   Project Release: 6.0.1.23
 * Input checksum: 41911    Output checksum: 14880
 */
